# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import json
import pprint

from django.utils.translation import ugettext as _

from sanet_common.timeutils import timestamp_to_isostr

from core.models import Event

from cli.clilib.ioslike import *
from cli.textformat import *

from cli.decorators import *

from entables.models import Alarm, AlarmArchive
from entables.models import TargetData

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def _show_alarm(cli, alarm, show_raw=False):

	user   = cli.env['CURRENT_USER']
	current_tenant = cli.env.get('CURRENT_TENANT', None)

	tenant = tenant_get(current_tenant)
	tenant.db_set_current_schema()




	cols = cli.env.get('COLS') or 80

	data = alarm.get_data()

	verbstatus = alarm.get_verbstatus()

	#cli.prnt(
	#	generic_details(
	#		alarm,
	#		suppress=[
	#			'tenant',
	#			'data',
	#			'stages',
	#		],
	#	)
	#)

	display_fields = [
		[_('uuid'           ) , alarm.uuid ], 
		[_('priority'       ) , alarm.priority ], 
		[_('status'         ) , alarm.status ], 
	]

	if isinstance(alarm, Alarm): 
		display_fields += [
			[_('rulestack'      ) , alarm.rulestack ], 
			[_('hold_time'      ) , "%s %s" % (timestamp_to_isostr(alarm.hold_time)      , alarm.hold_time) ], 
			[_('hold_wakeuptime') , "%s %s" % (timestamp_to_isostr(alarm.hold_wakeuptime), alarm.hold_wakeuptime) ], 
			[_('hold_handler'   ) , alarm.hold_handler ], 
			[_('merge_group_key') , alarm.merge_group_key ], 
			[_('ctime'          ) , alarm.ctime ], 
			[_('mtime'          ) , alarm.mtime ], 
		]

	cli.prnt(text_details(display_fields))

	cli.prnt("STAGES")
	cli.prnt("------")
	n = 0
	for l in verbstatus:
		cli.prnt("%s) %s" % (n, l))
		n+=1

	cli.prnt("")

	if show_raw:
		# Normal user can view "sensibile data" only with the capability. #545
		if user is not None:
			if 'gui.view_sensible_data' not in user.get_capabilities(tenant):
				# FIX: ugly, but works
				data_str = json.dumps(data)
				data_str = Event.hide_sensible_data_in_msg(data_str)
				data = json.loads(data_str)
		
		cli.prnt("RAW ALARM")
		cli.prnt("---------")
		raw_alarm =  pprint.pformat(data, indent=8, width=cols)
		cli.prnt(raw_alarm)
		cli.prnt("")
		
	if isinstance(alarm, Alarm) and alarm.merge_group_key:

		cli.prnt("MERGED ALARMS")
		cli.prnt("-------------")
		
		tdata = TargetData.objects.get(tenant=alarm.tenant, key=alarm.merge_group_key)
		merge_data = tdata.get_data()
		
		merged_alarms = AlarmArchive.objects.filter(uuid__in=merge_data['alarms'])
		if merged_alarms:
			for a in merged_alarms:
				_show_alarms(cli, merged_alarms)
		else:
			cli.prnt("There are no merged alarms.")

def _show_alarms(cli, alarms):

	cols = cli.env['COLS']

	r = [[
		['N.'  , 5],
		['UUID', -4],
		['CREATE', 10],
		['STATUS', 5],
		['PRIORITY', 5],
		['DATA', 60],
	]]

	pos = 0
	for a in alarms:

		r.append([
			pos,
			a.uuid,
			a.ctime,
			a.status,
			a.priority,
			a.data[0:60],
		])
		pos += 1

	cli.prnt(text_table(r, cols))

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def _show_pending_alarms(cli, alarms):

	cols = cli.env['COLS']

	r = [[
		['N.'  , 5],
		['UUID', -4],
		['CREATE', 10],
		['MODIFIED', 10],
		['STATUS', 5],
		['PRIORITY', 5],
		['HOLD', 19],
		['WAKE', 19],
		['DATA', 60],
	]]

	pos = 0
	for a in Alarm.objects.order_by('ctime').all():

		r.append([
			pos,
			a.uuid,
			a.ctime,
			a.mtime,
			a.status,
			a.priority,
			timestamp_to_isostr(a.hold_time) if a.hold_time else '',
			timestamp_to_isostr(a.hold_wakeuptime) if a.hold_wakeuptime else '',
			a.data[0:60],
		])
		pos += 1

	cli.prnt(text_table(r, cols))

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@cli_action_tenant_dependent
def cmd_entables_show_alarms(cli, tenant):

	alarms = AlarmArchive.objects.filter(tenant=tenant).order_by('ctime').all()

	_show_alarms(cli, alarms)

#------------------------------------------------------------------------------#

@cli_action_tenant_dependent
def cmd_entables_show_alarm_by_pos(cli, tenant):

	# show entables alarms id <id> raw
	pos = int(cli.canonical_words[4])

	show_raw = False
	if len(cli.canonical_words) > 5:
		show_raw = (cli.canonical_words[5] == 'raw')

	alarm = AlarmArchive.objects.filter(tenant=tenant).order_by('ctime')[pos][0]

	_show_alarm(cli, alarm, show_raw=show_raw)

#------------------------------------------------------------------------------#

@cli_action_tenant_dependent
def cmd_entables_show_alarm_by_id(cli, tenant):

	id = cli.canonical_words[4]

	show_raw = False
	if len(cli.canonical_words) > 5:
		show_raw = (cli.canonical_words[5] == 'raw')

	try:
		alarm = AlarmArchive.objects.get(tenant=tenant, uuid=id)

		_show_alarm(cli, alarm, show_raw=show_raw)

	except AlarmArchive.DoesNotExist, e:
		print "Not found (check in pending alarms)"
		return



@cli_action_tenant_dependent
def cmd_entables_show_last_alarm(cli, tenant):

	last = int( cli.canonical_words[-1] )

	alarms = AlarmArchive.objects.filter(tenant=tenant).order_by('-ctime')[0:last]

	alarms = list(alarms)

	alarms.reverse()

	_show_alarms(cli, alarms)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@cli_action_tenant_dependent
def cmd_entables_show_pending_alarms(cli, tenant):

	alarms = Alarm.objects.order_by('ctime').all()

	_show_pending_alarms(cli, alarms)

#------------------------------------------------------------------------------#

@cli_action_tenant_dependent
def cmd_entables_show_pending_by_id(cli, tenant):

	id = cli.canonical_words[4]

	show_raw = False
	if len(cli.canonical_words) > 5:
		show_raw = (cli.canonical_words[5] == 'raw')

	try:
		alarm = Alarm.objects.get(tenant=tenant, uuid=id)
		_show_alarm(cli, alarm, show_raw=show_raw)
	except Alarm.DoesNotExist, e:
		print "Not found"


#------------------------------------------------------------------------------#

@cli_action_tenant_dependent
def cmd_entables_show_pending_by_pos(cli, tenant):

	# show entables alarms id <id>
	pos = int(cli.canonical_words[4])

	show_raw = False
	if len(cli.canonical_words) > 5:
		show_raw = (cli.canonical_words[5] == 'raw')

	alarm = Alarm.objects.filter(tenant=tenant).order_by('ctime')[pos:pos+1][0]

	_show_alarm(cli, alarm, show_raw=show_raw)

#------------------------------------------------------------------------------#

@cli_action_tenant_dependent
def cmd_entables_show_last_pending_alarm(cli, tenant):

	last = int( cli.canonical_words[-1] )

	alarms = Alarm.objects.filter(tenant=tenant).order_by('-ctime')[0:last]

	alarms = list(alarms)

	alarms.reverse()

	_show_pending_alarms(cli, alarms)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

from django.conf import settings
from entables.server.threads import entablesd_ctl

def cmd_entables_info(cli):

	broker = '%s@%s' % ('/'.join(settings.BROKER_AUTH), ':'.join(settings.BROKER_ADDRESS))

	response = entablesd_ctl(broker, "info", [])

	print response

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

menu_show_entables = [

	T_Keyword('entables' , help='Show entables data', subtree=[

		T_Keyword('server' , help='Show entablesd informations', subtree=[
			T_Keyword('info' , help='Show entablesd internal status', callback=cmd_entables_info),
		]),

		T_Keyword('alarms' , help='Show processed alarms', callback=cmd_entables_show_alarms, subtree=[

			T_Keyword('id' , help='', subtree=[
				T_Anyword('<id>', help='', callback=cmd_entables_show_alarm_by_id, subtree=[
					T_Keyword('raw' , help='', callback=cmd_entables_show_alarm_by_id)
				])
			]),

			T_Keyword('pos' , help='',  subtree=[
				T_Number('<id>', help='', subtree=[
					T_Keyword('raw' , help='', callback=cmd_entables_show_alarm_by_pos)
				])
			]),

			T_Keyword('last' , help='', subtree=[
				T_Number('<id>', help='', callback=cmd_entables_show_last_alarm, subtree=[
				])
			]),

		]),

		T_Keyword('pending' , help='Show running alarms', callback=cmd_entables_show_pending_alarms, subtree=[

			T_Keyword('id' , help='',  subtree=[
				T_Anyword('<id>', help='',  callback=cmd_entables_show_pending_by_id, subtree=[
					T_Keyword('raw' , help='', callback=cmd_entables_show_pending_by_id)
				])
			]),

			T_Keyword('pos' , help='',  subtree=[
				T_Number('<id>', help='', subtree=[
					T_Keyword('raw' , help='', callback=cmd_entables_show_pending_by_pos)
				])
			]),

			T_Keyword('last' , help='', subtree=[
				T_Number('<id>', help='', callback=cmd_entables_show_last_pending_alarm, subtree=[
				])
			]),
		]),
	]),

]
