# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

.. _entables-lib:

########################################################################
Libreria utils
########################################################################

Codifica allarmi
========================================================================

.. autofunction:: encode_alarm

.. autofunction:: decode_alarm

"""

import json

import cPickle as pickle
import base64

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def encode_alarm(alarm):
	"""
	Serializza l'oggetto *alarm* in formato stringa *ascii*.
	
	Se possibile l'oggetto viene serializzato in formato JSON. Se non e' possibile, l'oggetto viene serializzato in una stringa di byte formato Pickle e poi converito in stringa ascii BASE64.
	
	Se l'oggetto non puo' essere serializzato, viene sollevata una eccezione.
	"""
	#
	# IMPORTANT: "alarm" data could contain binary strings (Es: SNMP OID with MAC 
	#            values or binary data).
	#            If we stringify this data with json.dumps(), it tries to handle 
	#            them with UTF8 and it does not work.
	#            Try binary encoding pickle(+base64) encoding as fallback.
	#
	try:
		return json.dumps(alarm)
	except Exception, e:
		return base64.b64encode(pickle.dumps(alarm, pickle.HIGHEST_PROTOCOL))

def decode_alarm(encoded_alarm):
	"""
	Converte la stringa *encoded_alarm* in un oggetto in memoria.
	
	I formati ammessi sono:
		* JSON
		* Stringa BASE64 di dati in formato Pickle.
		
	Se la stringa non puo' essere convertita viene sollevata una eccezione.
	"""
	try:
		return json.loads(encoded_alarm)
	except Exception, e:
		return pickle.loads(base64.b64decode(encoded_alarm))
