# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import json
import pprint

from sanet_common.ipc import IPCConnection
from sanet_common.ipc import IPCConnectionError, IPCException

from server.sanetd.sanetdlogic import AbstractAlarmProcessor
from server.sanetd.sanetdlogic import load_additional_alarm_informations
from server.sanetd.sanetdlogic import create_ipc_connection_from_config

from entables.lib import encode_alarm

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

"""
              +-------------------------------------------------------------------+
              | Sanetd                                                            |
              |                                                                   |
    QUEUE ----------- processing ---> alarms_queue <---- *EntablesAlarmProcessor ------> IPC ----> ENTABLES
              |                                                                   |
              |                                                                   |
              +-------------------------------------------------------------------+

"""

class EntablesAlarmProcessor(AbstractAlarmProcessor):
	
	def __init__(self, logger):

		super(EntablesAlarmProcessor, self).__init__(logger)

		self._alarm_connection = None

	def process_alarm(self, server, alarm_data):

		logger = self.logger

		logger.debug("Processing alarm")
		
		alarm_connection = self._setup_ipc_connection()

		load_additional_alarm_informations(logger, alarm_data)

		logger.log(9, "[event] Sending data to notify manager: %s", pprint.pformat(alarm_data))

		#
		# Encode data
		#
		msg = encode_alarm(alarm_data)

		#
		# Send data 
		#
		entables_err = None
		try:
			alarm_connection.send_msg( 'entables-alarms' , msg, blocking=False)
			
			logger.info("[event] sent")
			return True
		except IPCConnectionError, e:
			entables_err = e
		except IPCException, e:
			entables_err = e
		except Exception, e:
			entables_err = e

		if entables_err:
			logger.error("Unknown error: %s", e)
			self._close_ipc_connection()
		
		return 

	def _setup_ipc_connection(self):

		if self._alarm_connection is not None:
			return self._alarm_connection

		try:
			self.logger.info("[alarm] Creating connection...")
			#alarm_connection = self.create_ipc_connection()
			alarm_connection = create_ipc_connection_from_config(self.logger)
			alarm_connection.open()
			
			self._alarm_connection = alarm_connection
			
			return alarm_connection

		except IPCConnectionError, e:
			msg = traceback.format_exc()
			self.logger.error("[alarm] IPC Connection Error on event: %s", msg)

		except IPCException, e:
			msg = traceback.format_exc()
			self.logger.error("[alarm] IPC Error on event connection: %s", msg)

		except Exception, e:
			msg = traceback.format_exc()
			self.logger.fatal("[alarm] Generic error on event connection: %s", msg)


	def _close_ipc_connection(self):
		
		try:
			self.logger.error("[alarm] Trying to close connection")
			self._alarm_connection.close()
			
		except Exception, e:
			self.logger.error("[alarm] Error when closing connection: %s", e)
			
		self._alarm_connection = None


