# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os
import time
import uuid
import copy
import traceback

from sanet_common.ipc import *
USE_REDIS_IPC = os.environ.get('SANET_USE_REDISIPC', None)
if USE_REDIS_IPC:
	from sanet_common.ipc.redisipc import RedisIPCConnection as IPCConnection
else:
	from sanet_common.ipc          import IPCConnection

from sanet_common.sanetdefs.constants import ALARM_NORMAL

from entables.listeners import BaseListener 

from entables.lib import decode_alarm
from entables.constants import ALARM_STATUS_NEW
from entables.datastructures import *


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

POLL_TIMEOUT = 1

ENTABLES_QUEUE_NAME = "entables-alarms"

DEFAULT_CONFIGURATION = {
	"queue_name"     : ENTABLES_QUEUE_NAME,
	"qpid_queue_name": ENTABLES_QUEUE_NAME,
	"username"       : 'sanet',
	"password"       : 'sanet',
	"brokerip"       : '127.0.0.1',
}

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class QpidListenThread(BaseListener):
	
	def __init__(self, configuration, context):
		
		self.name = "qpid"
		self.connection = None
				
		super(QpidListenThread, self).__init__(configuration, context)

		self.init(configuration)

		return

	def init(self, configuration):
		
		self.context.log.info("Init listener")
		
		# FIX: validate configuration
		self.configuration = DEFAULT_CONFIGURATION
		self.configuration.update(configuration)


	def run(self):
		self.log.info("Starting %s listener", self.name)

		while True:
			
			try:
				self.creat_connection()
				self.listen_loop()
			except Exception, e:
				self.context.log.error("Error on input message: %s", str(e))
				
			try:
				if self.connection:
					self.connection.close()
			except Exception, e:
				self.context.log.error("Error while closing connection: %s", e)
				
				
				
	def listen_loop(self):
		
		while True:

			try:
				json_message = self.get_message(POLL_TIMEOUT)
				if json_message is None:
					self.ack_message()
					continue
			except Exception, e:
				self.context.log.critical("Error while reading from connection: %s", str(e))
				return
				
			try:
				#json_message = json.loads(json_message)

				# Gli allarmi generati Sanet -> Entables sono "normal"
				# Gli allarmi generati con push_alarm sono "custom"
				if not json_message.has_key('alarmtype'):
					json_message['alarmtype'] = ALARM_NORMAL
				#
				# Verifica campi minimi
				#
				
				# FIX: Verificare che il tenant esista!
				tenant_uuid = json_message["tenantid"]
				uuid        = json_message['uuid']
				alarmtype   = json_message['alarmtype']
				
					

				self.context.log.info("Received alarm %s:%s:%s", tenant_uuid, uuid, alarmtype)

				stages = []
				alarmqueue_alarm = AlarmMemoryObject(tenant_uuid=tenant_uuid, auuid=uuid, status=ALARM_STATUS_NEW, data=json_message, stages=stages )
				alarmqueue_alarm.set_verbstatus("Received at %s bt %s" % (time.time(), self.name))
				self.context.permaqueue.put(alarmqueue_alarm)

				self.context.log.info("Notifying main thread for new message")
				self.context.serverthread.start_process_new_alarm(uuid)
				
				self.context.log.debug("calling ack ")
			except Exception, e:
				self.context.log.error("Error on input message: %s", str(e))
				self.context.log.error("    -> %s", traceback.format_exc())

			try:
				self.ack_message()
			except Exception, e:
				self.context.log.error("Error on input message: %s", str(e))
				return

	def creat_connection(self):

		self.context.log.info("Opening connection...")

		queue_name      = self.configuration.get("queue_name", None) or self.configuration.get("qpid_queue_name")
		brokerip        = self.configuration["brokerip"]
		username        = self.configuration["username"]
		password        = self.configuration["password"]

		self.connection = IPCConnection(brokerip, username=username, password=password)
		self.connection.open()
		self.connection.bind(queue_name, persistent=True)

		self.context.log.info("Connection created")


	def get_message(self, max_timeout):
		self.context.log.debug("Receiving alarms (timeout=%s, available=%s)", max_timeout, str(self.connection.available()) )

		(sender, data) = self.connection.receive_msg(timeout=max_timeout)
		if data is None:
			return None

		json_alarm = decode_alarm(data)

		self.context.log.log(9, "listener:qpid received %r from %s ", str(json_alarm), str (sender) )

		return json_alarm

	def ack_message(self):
		self.context.log.debug("ACK MESSAGE")
		self.connection.ack_pending()


LISTEN_THREAD_CLASS = QpidListenThread
































