# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.
"""

.. _entables-entables_ctl:

entables_ctl
===========================================================================

Send commands to Entables's daemon.

.. warning:: This is an internal Sanet command. It is not recommended to use this command.

"""

import os
import sys
import time
import argparse

from django.utils.translation import ugettext as _

from core.management.commands import SanetInternalCommand

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(SanetInternalCommand):

	help = _("Send commands to Entables's daemon.")

	def add_arguments(self, parser):
		super(Command, self).add_arguments(parser)

		parser.add_argument('cmd'    , nargs='?', default=None)
		parser.add_argument('cargs'   , nargs=argparse.REMAINDER, default=None)

	def handle(self, *args, **options):

		cmd  = options['cmd']
		args = options['cargs']

		if not cmd:
			cmd = 'info'

		response = entables_ctl('sanet/sanet@localhost', cmd, list(args))

		print response

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

from sanet_common.ipc import *

def entables_ctl(broker,  cmd, params, timeout=3, blocking=False):

	host = None
	port = None
	user = None
	password = None

	auth, addr = broker.split('@')
	user,password = auth.split('/')
	addr = addr.split(':')
	host = addr[0]
	if len(addr) > 1:
		port = int(addr[1])

	connect_options = {}
	if port     != None: connect_options['port']     = port
	if user     != None: connect_options['username'] = user
	if password != None: connect_options['password'] = password

	c = IPCConnection( host, **connect_options)
	c.open()
	replyid = 'temp-entablesctl-%s' % (os.getpid())
	c.bind(replyid)

	request = encode_request( cmd, *(params+[ time.time() ]) )

	try:
		c.send_msg( 'entables-ctl', request, blocking=blocking)
	except IPCNotFound, e:
		raise Exception("Unable to send message to agent %s: %s" % (agentid, e))
	except IPCException, e:
		raise Exception("Generic error while sending data to agent %s: %s" % (agentid, e))

	try:
		response = None
		(msg_addr, msg) = c.receive_msg(ack_msg=True, timeout=timeout)
	except IPCException, e:
		raise Exception("Generic error on receive from agent %s: %s" % (agentid, e))

	c.close()

	if msg == None:
		raise Exception("Timeout")

	if msg != None:
		response = decode_response(msg)
		#if response['status'] == RESPONSE_SUCCESS:
		#	return response['content']
		return response['content']

	return None
