# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.
"""

entables_test_tags
========================================================================

Syntax:

	::
		
		sanet-manage entables_test_tags share/files/sanet_alarm_example.json  ' ${alarmid} '

Questo comando serve per testare manualmente i tags di entables a partire da un allarme di sanet in formato JSON:

.. warning:: Questo e' un comando di debug. Non utilizzare.

"""

import json
import logging
import time

from optparse import make_option

from django.conf import settings

#from django.core.management.base import BaseCommand
from django.utils.translation import ugettext as _

from core.models import Tenant

from core.management.commands import SanetInternalCommand

from entables.templatemanager import *
from entables.datastructures import TagproviderDefinitionList
from entables.modulesmanager import load_tagprovider_modules

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(SanetInternalCommand):
	
	help = _("Test entables's tags expansion.")
	
	def add_arguments(self, parser):
		super(Command, self).add_arguments(parser)

		parser.add_argument('-l'      , action='store'     , dest='loglevel' , default=20   , help='Set log level (<20 DEBUG, <30 INFO, <40 WARNING, <50 ERROR.'),

		parser.add_argument('alarm_filename', nargs='?', default=None)
		parser.add_argument('test_string' , nargs='?', default=None)

	def handle(self, *args, **options):

		configuration = settings.ENTABLESD_SETTINGS
		
		argv = [ p for p in args ]

		alarm_filename = options['alarm_filename'] #argv[0]
		test_string    = options['test_string'   ] #argv[1]
		
		f = open(alarm_filename, "r+")
		alarm_data = f.read()
		f.close()
		
		alarm_data = alarm_data.replace("\n", " ").replace('\t', '').replace("'", '"').replace("None", "null")
		alarm_data = json.loads(alarm_data)
		
		alarm_data['tenantid'] = Tenant.objects.get(is_primary=True).uuid
		alarm_data['ts'] = time.time()
		
		log = self.init_log(int(options['loglevel']))
		
		options = {
			'data' : test_string
		}
		defaults = {}

		templates = {}
		
		tag_providers = TagproviderDefinitionList()
		load_tagprovider_modules(log, configuration['tagproviders'], tag_providers)		
		
		options = calculate_options_value(log, alarm_data, options, defaults, tag_providers, templates)
		
		print ">" * 100
		print ">" * 100
		print ">" * 100
		print "BEFORE"
		print ">" * 100
		print ">" * 100
		print ">" * 100
		print test_string
		print ">" * 100
		print ">" * 100
		print ">" * 100
		print "AFTER"
		print ">" * 100
		print ">" * 100
		print ">" * 100
		print options['data']


	def init_log(self, log_level):

		format_str = '%(asctime)s %(levelname)s %(threadName)s > %(message)s'
		logger = logging.getLogger('test')
		logger.setLevel( log_level )

		self.logger = logger

		hdlr = logging.StreamHandler()
		hdlr.setFormatter( logging.Formatter(format_str) )
		hdlr.setLevel( log_level )
		logger.addHandler(hdlr)

		self.log_hdlr = hdlr

		return logger
