# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

"""

import time
import json
import logging

from optparse import make_option

from django.core.management.base import BaseCommand
from django.utils.translation import ugettext as _

from sanet_common.timeutils import timestamp_to_isostr

from core.timing import init_holidays_converter

from entables.lib.timing import calculate_until_ts

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

def str_to_timestamp(s):
	return time.mktime(time.strptime(s, "%Y-%m-%d %H:%M"))

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(BaseCommand):

	help = _("Test entables's tags expansion.")

	#option_list = BaseCommand.option_list + (
	#	make_option('-c', '--context'  , action='store'     , dest='context'   , default=None   , help='Holiday context'),
	#	make_option('-n', '--now'      , action='store'     , dest='now'       , default=None   , help=''),
	#	make_option('-l', '--loglevel ', action='store'     , dest='log_level' , default=20     , help=''),
	#)

	def add_arguments(self, parser):
		parser.add_argument('-c', '--context'  , action='store'     , dest='context'   , default=None   , help='Holiday context'),
		parser.add_argument('-n', '--now'      , action='store'     , dest='now'       , default=None   , help=''),
		parser.add_argument('-l', '--loglevel ', action='store'     , dest='log_level' , default=20     , help=''),

	def handle(self, timespec, *args, **options):

		context = options['context']
		now     = options['now']

		if now is None:
			now = time.time()
		else:
			now = str_to_timestamp(now)

		logger = logging.getLogger('sanet.global')

		if options['log_level']:
			l = int(options['log_level'])
			logger.setLevel( l )
			for handler in logger.handlers:
				handler.setLevel( l )

		holidays_converter = init_holidays_converter()

		holidays_converter.set_default_time_context(context)

		ts = calculate_until_ts(now, timespec, calendar_handler=holidays_converter)

		print "---- RESULT ----"
		print timestamp_to_isostr(ts)

