# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

ALL_TABLES = True

class BaseMatch(object):

	def __init__(self):
		self.name    = None
		self.options = None
		self.tables  = None

	def get_name(self):
		return self.name

	def get_valid_tables(self):
		return self.tables

	def get_options(self):
		return dict([ (o.name, o) for o in self.options])
		
	def get_options_default(self):
		effective_options = {}
		for n,o in self.get_options().items():
			if o.has_default():
				effective_options[n] = o.default
		return effective_options

	def validate_match_options(self, options_dict):

		# Base validation: check only required options
		for optiondef in self.options:
			if optiondef.required:
				if options_dict.get(optiondef.name, None) is None:
					raise Exception("Missing required option: %s", optiondef.name)
					
		return True

	def match(self, context, alarmobj, options):
		raise Exception("Not Implemented")
		


	#
	# Parsing
	#
	def build_from_parsed_args(self, args):
		options = {}
		prefix = self.name + "_"
		for optionname, optionvalue in args.items():
			if optionname.startswith(prefix):
				optionname = optionname[len(prefix):]
			options[optionname] = optionvalue
		return options

	def add_args_to_parser(self, argument_parser):

		prefix = self.name

		for option in self.options:#.keys():
			option.add_parser_options(prefix, argument_parser)
		return
