# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

.. _entables_match_counters:

COUNTERS
==============

Questo match permette di accedere alle informazioni memorizzate dal target :ref:`entables_target_updatecounters`.

	::
		
		-m counters --counters_<parametro> <valore>

Questi sono i parametri su cui e' possibile fare match:

	+---------------+---------------+------------------+---------------------------------------------------------------------------------------------------+
	| Parametro     | Type          | Default          | Descrizione                                                                                       |
	+===============+===============+==================+===================================================================================================+
	| status        | str           |                  | stato della condition dell'ultimo allarme processato da :ref:`entables_target_updatecounters`     |
	+---------------+---------------+------------------+---------------------------------------------------------------------------------------------------+
	
Esempi
------

Manda all'allarme per email solo se l'ultimo allarme processato da UPDATECOUNTERS era un UP.

	::
		
		...
		...
		entables -t notify -A DEFAULT                                        -J UPDATECOUNTERS 
		...
		...
		entabels -t notify -A DEFAULT    -m counters --counters_status UP    -j SMTP

"""

import time
import datetime
import json

from sanet_common.sanetdefs.constants import *

from core.models import Tenant

from entables.models import TargetData
from entables.constants import TABLE_NOTIFY

from entables.targets import Option
from entables.matches import BaseMatch

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class Match(BaseMatch):

	def __init__(self):
		super(Match, self).__init__()

		self.name    = "counters"

		self.tables  = [ TABLE_NOTIFY ]

		self.options = [
			Option("status"    , type=str, required=False),
		]

	def check_match_arguments(self, options_dict):

		super(BaseMatch, self).check_match_arguments(options_dict)

		status      = options_dict.get('status', None)
		if status not in CSTATE_CODENAMES: raise Exception("Invalid state code: %s" % (curr))

		return True

	def match(self, context, alarmobj, options):

		NOT_DEFINED = ''

		log = context.log

		alarmobj.set_verbstatus("Match %s. Options %s" % (self.name, options))

		tenant = Tenant.objects.get(uuid=alarmobj.tenant_uuid)

		alarmdata = alarmobj.data

		now = time.time()

		try:
			required_status      = CSTATE_FROM_CODENAME(options.get('status',None)) or NOT_DEFINED

			condition_path = alarmdata.get('condition',{}).get('path',None)

			#
			# Load counters from cache
			#
			key = 'entables.COUNTERS.%s.data' % (condition_path)
			target_data = TargetData.objects.get(tenant=tenant, key=key)

			info_data = target_data.get_data()
			if info_data is None:
				target_data = {}

			log.info("CURRENT DATA: %s", info_data)

			if info_data.get('status', None) != required_status:
				log.info("NOT MACHED")
				return False

			log.info("MATCHED")
			return True

		except KeyError, e:
			msg = "Unable to get info from alarm: %s" % (str(e))
			log.error(msg)

		except Exception, e:
			msg = "Unable to perform match: %s" % (str(e))
			log.error(msg)

		return False

