# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

tests
=====

NOTA BENE: questo match non va utilizzato. E' disponibile solo per test di debug.

Gli attributi su cui fare match vengono specificati con la sintassi:  ::

	-m tests --tests_<attributo> <valore>

Gli attributi su cui e' possibile fare match sono:

	+-----------+---------------+------------------+---------------------------------------------------------------------------------------------------+
	| Parametro | Type          | Default          | Descrizione                                                                                       |
	+===========+===============+==================+===================================================================================================+
	| newerthan | numero        |                  | Fa match se il timestamp dell'allarme risale a meno di <num> secondi fa'                          |
	+-----------+---------------+------------------+---------------------------------------------------------------------------------------------------+
	
Esempi
------

Cattura tutti gli allarmi generati negli ultimo 20 secondi e li ferma per 30 secondi: ::

	entables  -A DEFAULT  -m tests --tests_newerthan 20   -j DELAY --delay_seconds 30

"""

import time
import datetime

from entables.constants import TABLE_NOTIFY

from entables.targets import Option
from entables.matches import BaseMatch


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

PARAM_NEWERTHAN = "newerthan"

class Match(BaseMatch):

	def __init__(self):
		super(Match, self).__init__()
		
		self.name    = "tests"
		
		self.tables  = [ TABLE_NOTIFY ]
		
		self.options = [
			Option(PARAM_NEWERTHAN, type=int, default=1)
		]

	def check_match_arguments(self, options_dict):
		
		super(BaseMatch, self).check_match_arguments(options_dict)

	def match(self, context, alarmobj, options):
		
		log = context.log

		alarm = alarmobj.data

		now = time.time()

		param_min = int(options.get(PARAM_NEWERTHAN, 0))

		alarm_ts = int(alarm['ts'])

		if ((now - alarm_ts) < param_min):
			return True

		return False


