# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import uuid
import datetime

import json
import cPickle as pickle
import base64

from django.db import models
from django.db.models import Max

from django.contrib.auth.models import Group

from django.conf import settings

from core.models import TenantDependentModel, UUIDKeyModel
from core.models import Tenant
from core.api.base import db_set_current_tenant_schema

from entables.constants import *
from entables.datastructures import *

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def debug(string):
	if settings.DEBUG:
		print "DEBUG:", string

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class Template(TenantDependentModel, UUIDKeyModel):
	
	# tenant = 
	# uuid   = 
	mtime = models.DateTimeField(null=False, auto_now=True)
	name  = models.TextField(null=False, unique=True)
	data  = models.TextField(null=False, default='')              
	
	def save(self, *args, **kwargs):
		self.validate_data()
		super(Template, self).save(*args, **kwargs)

	def validate_data(self):
		from entables.templatemanager import validate_template
		validate_template(self.data)
		
#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def ENCODE_DATA(data):
	try:
		return json.dumps(data)
	except Exception, e:
		return base64.b64encode(pickle.dumps(data, pickle.HIGHEST_PROTOCOL))

def DECODE_DATA(data):
	try:
		return json.loads(data)
	except Exception, e:
		return pickle.loads(base64.b64decode(data))

#------------------------------------------------------------------------------#

class BaseAlarm(TenantDependentModel, UUIDKeyModel):

	# tenant
	# uuid
	priority = models.IntegerField(null=False, default=0) # Campo di comodo (dovrebbe avere lo stesso valore salvato da qualche parte in "data") 
	status   = models.TextField(default=ALARM_STATUS_NEW) 

	data     = models.TextField(null=False)  # Json allarme
	stages   = models.TextField(null=False) # Json lista passaggi dell'allarme dal listener fino all'archivio DISABILITARE QUANDO TRASFORMATO IN TABELLA

	#
	# Raw data
	#
	def set_data(self, data):
		self.data = ENCODE_DATA(data)

	def get_data(self):
		return DECODE_DATA(self.data)

	#
	# Stages
	#
	def set_stages(self, stages):
		# FIX: Validate?
		self.stages = json.dumps(stages)
		
	def get_stages(self):
		try:
			return json.loads(self.stages)
		except Exception, e:
			print e
			return []
			
	def get_verbstatus(self):
		return self.get_stages()

	class Meta:
		abstract = True
	
class Alarm(BaseAlarm):

	# tenant
	# uuid
	# priority
	# status

	# data
	# stages

	ctime = models.DateTimeField(null=False, auto_now_add=True, db_index=True)
	mtime = models.DateTimeField(null=False, auto_now=True, db_index=True)
	
	rulestack = models.TextField(null=True, default=None)
	
	hold_time       = models.PositiveIntegerField(default=0, null=False, db_index=True) # Orario di quando e' avvenuto l'HOLD
	hold_wakeuptime = models.PositiveIntegerField(default=0, null=False, db_index=True) # Orario di fine HOLD (un-hold)
	hold_handler    = models.TextField(null=True, default=None)                         # Target Handler per l'hold. Settato solo quando bisogna fare operazioni in fase di un-hold
	
	merge_group_key = models.TextField(null=True, default=None, db_index=True)  # Identificativo univoco per raggruppare gli allarmi "MERGED"


	#
	# Raw data
	#
	def set_rulestack(self, stack):
		# FIX: Validate?
		self.rulestack   = json.dumps(stack)

	def get_rulestack(self):
		if self.rulestack is None:
			return None
			
		try:
			return json.loads(self.rulestack)
		except Exception, e:
			print e
			return None



	@staticmethod
	def build_and_save(tenant, alarmobj):
		alarm = Alarm()
		alarm.build(tenant, alarmobj)
		return alarm
	    

	def build(self, tenant, alarmobj):
		self.tenant   = tenant
		self.uuid     = alarmobj.uuid
		self.priority = alarmobj.priority
		self.status   = alarmobj.status
		
		#self.data     = json.dumps(alarmobj.data)
		self.data     = ENCODE_DATA(alarmobj.data)
		self.stages   = json.dumps(alarmobj.stages)

		self.rulestack = json.dumps(alarmobj.get_rulestack())

		self.hold_time       = alarmobj.hold_time
		self.hold_wakeuptime = alarmobj.hold_wakeuptime
		self.hold_handler    = alarmobj.hold_handler
		
		self.merge_group_key = alarmobj.merge_group_key

		self.save()

	def load(self):
		stages    = json.loads(self.stages)
		#data      = json.loads(self.data)
		data      = DECODE_DATA(self.data)
		rulestack = json.loads(self.rulestack)

		alarmobj = AlarmMemoryObject(
			tenant_uuid = self.tenant.uuid,
			auuid       = self.uuid, 
			priority    = self.priority, 
			status      = self.status, 

			stages      = stages, 
			data        = data, 

			rulestack   = rulestack,

			hold_time       = self.hold_time,
			hold_wakeuptime = self.hold_wakeuptime,
			hold_handler    = self.hold_handler,
			
			merge_group_key = self.merge_group_key,
		)

		return alarmobj

	def destroy(self):
		self.delete()

	def change_status(self, status):
		self.status = status
		self.save()

	def move_to_archive(self):

		archivedalarm = AlarmArchive()
		archivedalarm.copy_from_alarm(self)
		archivedalarm.save()

		self.delete()


        
	def objdump(self):
		return { 
			"data"          : json.loads(self.data),
			"priority"      : self.priority,
			"uuid"          : self.uuid,
			"stages"	: json.loads(self.stages),
			"status"	: self.status,
		}

            
class AlarmArchive(BaseAlarm):

	# tenant
	# uuid
	#priority
	#data    
	#stages  
    
	ctime = models.DateTimeField(null=False)

	def copy_from_alarm(self, alarm):

		self.tenant   = alarm.tenant
		self.uuid     = alarm.uuid
		self.ctime    = alarm.ctime
		self.priority = alarm.priority
		self.status   = alarm.status
		self.data     = alarm.data
		self.stages   = alarm.stages

	def objdump(self):

		return { 
			"data"        : json.loads(self.data),
			"stages"      : json.loads(self.stages),
			"uuid"        : self.uuid,
		}             

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class Table(models.Model):
	
	name     = models.TextField(unique=True, null=False)
	priority = models.IntegerField(unique=True, null=False)

	def __str__(self):
		return "TABLE(%s)" % (self.name,)

	def destroy(self):

		for t in Tenant.objects.all():
			db_set_current_tenant_schema(t.name)
			
			for chain in self.chains.all():
				chain.destroy()
				
		self.delete()
		
		return
   
	#Table
	def add_chain(self,tenant, name, group, policy):
		try:
			new_chain = Chain()
			new_chain.build(tenant, name, policy, group, self)
		except Exception, e:
			print type(e), str(e)
		

	def delete_chain(self, name):
		chain = self.chains.get(name=name)
		chain.destroy()
        
	def load(self):

		tableobj = TableMemoryObject()
		tableobj.name = self.name
		tableobj.priority = self.priority
        
		for chain in self.chains.all():
			chainobj = chain.load()
			tableobj.chains[chainobj.name] = chainobj
            
		return tableobj
        
	def objdump(self):

		chaindump = {}
		for chain in self.chains.all():
			chaindump[chain.name] = chain.objdump()
            
		return {
			"name"    : self.name,
			"priority": self.priority,
			"chains"  : chaindump, 
		}
        
#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

RULE_POSITION_FIRST = 1

class Chain(TenantDependentModel, UUIDKeyModel):
	
	# tenant =
	# uuid   = 

	mtime = models.DateTimeField(null=False, auto_now=True)

	table = models.ForeignKey(Table, null=False, related_name="chains")    
	name = models.TextField(null=False)
	policy = models.ForeignKey('Target', null=False, related_name="chain")

	group = models.ForeignKey(Group, null=True, default=None)

	class Meta:
		unique_together = ("table", "name")

	@property
	def is_default(self):
		return self.name == DEFAULT_CHAIN

	def __str__(self):
		return self.table.name + " - " +self.name 

	def build(self, tenant, name, policy, group, table):
		self.name = name
		self.table = table 
		self.group = group

		target = Target()
		target.build(tenant, policy)

		self.policy = target
		self.tenant = tenant
		self.save()

		debug ("added chain " + self.table.name + " " + self.name + " " + policy.name)

	@property
	def ordered_rules(self):
		return self.rules.order_by('position')
    
	def get_max_position(self):
		return self.rules.aggregate(Max('position'))['position__max']
		
	def append_rule(self, tenant, ruleobj):

		rule_position = self.get_max_position() 
		if rule_position != None:
			rule_position = rule_position + 1
		else:
			rule_position = RULE_POSITION_FIRST

		new_rule = Rule()
		new_rule.build(tenant, ruleobj,rule_position,self)
           
	def insert_rule(self,tenant, ruleobj, rule_position):
	
		if rule_position < 1:
			raise Exception("Invalid rule position: %s" % (rule_position))
	
		max_position = self.get_max_position()
		if max_position != None:
			if rule_position > max_position:
				rule_position = max_position + 1
		else:
			rule_position = RULE_POSITION_FIRST
	    
		# insert rule
		for rule in self.rules.filter(chain=self).order_by('-position'):
			if rule.position >= rule_position: # FIX: controllare se rule != new_rule. Come confrontare se due regole sono semanticamente uguali?
				rule.shift_position(+1)

		new_rule = Rule()
		new_rule.build(tenant, ruleobj, rule_position, self)
   
	def replace_rule(self, tenant, ruleobj, rule_position):

		rule = self.rules.get(position=rule_position)
		rule.delete()

		new_rule = Rule()
		new_rule.build(tenant, ruleobj, rule_position, self)
             
	# Chain
	def find_rule(self,table,chain,ruledef):
		rule_position = 0
		for rule in self.rules:
			if ruledef == rule.ruledef:
				rule_position = rule.rule_position

		return rule_position
    
	# Chain
	def delete_rule_by_pos(self, rule_position):

		return self.delete_rule( rule_position, ruledef=None)

	def delete_rule(self,rule_position,ruledef=None):
	    
		if ruledef != None:
			rule_position = self.find_rule(ruledef)
        
		if rule_position==0:
			# Delete last rule, useful for flushing
			rule_position = self.rules.count()
            
		rule = self.rules.get(position = rule_position)

		rule.destroy()
        
		# Shift other rules only if isn't the last (optimization)
		if rule_position != self.rules.count() + 1:
			for rule in self.rules.filter(position__gte = rule_position).order_by('position'):
				rule.shift_position(-1)

	def rename(self,new_chain_name):
		self.name = new_chain_name
		self.save()

	def change_policy(self,chain_policy):
		#oldpolicy = self.policy
		#policy = Target()
		#policy.build(chain_policy)
		#self.policy = policy
		#self.save()
		#oldpolicy.destroy()
		self.policy.replace(self.tenant,chain_policy)

	def flush(self):
		for rule in self.rules.all():
			rule.destroy()
		self.mtime = datetime.datetime.utcnow()
		self.save()
    
	def zero(self, **kwargs):
		if "rulenum" in kwargs and kwargs["rulenum"] != 0:
			rule = self.rules.get(position = self.rulenum)
			rule.zerocounter()
		else:
			for rule in self.rules.all():
				rule.zerocounter()
    
	# Chain 
	def check_rule(self,table,chain,ruledef):
		return self.find_rule(ruledef)

	def list_rules(self):
		for rule in self.chain:
			rule.list

	def destroy(self):
		for rule in self.rules.all():
			rule.destroy()
		policy = self.policy
		policy.destroy()
		self.delete()     

	def load(self):
		chainobj = ChainMemoryObject()
		chainobj.name = self.name
        
		#print "RULES"
		rule_array = []
		for rule in self.rules.order_by('position'):
			ruleobj = rule.load()
			rule_array.append(ruleobj)
            
		chainobj.rules = rule_array


		#print "POLICY"
		target = self.policy
		#print self.policy.uuid, self.policy
		targetobj = target.load()
		#print "POLICY DUMP", targetobj.objectdump()
		chainobj.policy = targetobj

		return chainobj

	def objdump(self):
		rulesdump = []
		for rule in self.rules.order_by("position"):
			rulesdump.append(rule.objdump())
			
		return {
			"policy": self.policy.objdump(),
			"rules": rulesdump,
			"name": self.name,
			"mtime": str(self.mtime),
		}

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class Rule(TenantDependentModel, UUIDKeyModel):
	# tenant =
	# uuid   = 

	mtime = models.DateTimeField(null=False, auto_now=True)

	chain = models.ForeignKey(Chain, null=False, related_name="rules")

	# table = models.ForeignKey(Table, related_name="rules", default =1 )
	#definition = models.TextField(blank = True, null=True)    #XXXXXXXXXXXXXXXXX MAI USATO
	position = models.IntegerField()

	hitcounter = models.IntegerField(null=False, default=0)

	target = models.ForeignKey('Target', null=False, related_name="rule")

	stop_if_match = models.BooleanField(null=False, default=False)

	disabled = models.IntegerField(null=False, default=RULE_STATE_ACTIVE) # 0 = not disabled, 1 admin disabled , >2 disabled by error #XXXXXXXXXXXX NULLDEFAULT? 0

	class Meta:
		unique_together = ("chain", "position")


	def __str__(self):
		return self.chain.name +':'+ str(self.position)

	def zerocounter(self):  #XXXXXXXXXX RICONTROLLARE CHI CHIAMA QUESTO METODO E SE BISOGNA SALVARE ADESSO SUL DB
		self.hitcounter = 0
		self.save()

	def list(self): #XXXXXXXXXXXX TOGLIERE SE NON USATO
		# return string in entables cli format
		pass
    


	def is_jump_rule(self):
		
		target_name = self.target.handler_name
		
		return self.chain.table.chains.filter(name=target_name).count() == 1
    
	def disable(self):
		self.disabled = RULE_STATE_DISABLED
		# FIX: c'e' niente da fare per avvisare entables di questa modifica?
		self.save()
        
	def activate(self):
		self.disabled = RULE_STATE_ACTIVE
		# FIX: c'e' niente da fare per avvisare entables di questa modifica?
		self.save()
    
	def build(self, tenant, ruleobj, position, chain):
	    
		self.position = position
		self.disabled = ruleobj.disabled
		self.stop_if_match = ruleobj.stop_if_match
		self.chain = chain

		target = Target() 
		targetobj = ruleobj.target
		target.build(tenant, targetobj)

		self.target = target
		self.tenant = tenant

		self.save()

		for matchobj in ruleobj.matchset:
			match = Match()
			match.build(tenant,matchobj,self)
        
	def shift_position(self, position):
		self.position += position
		self.save()   
        
	def destroy(self):

		for match in self.matches.all():
			match.destroy() 
       
		target = self.target

		target.destroy()
		self.delete()
 
	def load(self):
		targetobj = self.target.load()
		
		#print "RULE"
		ruleobj = RuleMemoryObject(targetobj, uuid=self.uuid, position=self.position, stop_if_match=self.stop_if_match)
        
		for match in self.matches.all():
			matchobj = match.load()
			ruleobj.matchset.append(matchobj)
            
		ruleobj.disabled = self.disabled
		ruleobj.stop_if_match = self.stop_if_match
		
		return ruleobj

	@property
	def active_str(self):
		return str( not self.disabled).lower()


	def objdump(self):
	    
		matchesdump = []
		try:
			for match in self.matches.all():
				matchesdump.append(match.objdump())
		except Exception, e:
			pass
            
		target = None
		try:
			target = self.target.objdump()
		except Exception, e:
			pass
            
		return {
			"position"     : self.position,
			"hitcounter"   : self.hitcounter,
			"stop_if_match": self.stop_if_match,
			"mtime"        : str(self.mtime),
			"disabled"     : self.disabled,
			"target"       : target,
			"matches"      : matchesdump
		}       

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#
    
class Match(TenantDependentModel, UUIDKeyModel):
	
	mtime = models.DateTimeField(auto_now=True)

	handler_name = models.TextField(null=False)  

	rule = models.ForeignKey(Rule, related_name="matches")   
    
	class Meta:
		unique_together = ("rule", "handler_name")  # accorpare situazioni di questo tipo  "-m attr -m net -m attr"
    
	def __str__(self):
		return str(self.rule) + ':' + self.handler_name

	def build(self,tenant,matchobj,rule):
		self.handler_name = matchobj.name
		self.rule = rule
		self.tenant = tenant
		self.save()

		for matchobj_option_name in matchobj.options.keys():
			option = MatchOptions()
			option.build(tenant, matchobj_option_name,matchobj.options[matchobj_option_name],self)    

	def destroy(self):
		for option in self.match_options.all():
			option.destroy()
		self.delete()
    
	def load(self):
		matchobj = MatchMemoryObject()
		matchobj.name = self.handler_name
		for option in self.match_options.all():
			matchobj.options.update(option.load())
		return matchobj

	def objdump(self):
		matchoptionsdump = {}
		for option in self.match_options.all():
			matchoptionsdump[option.name] = option.value

		return {
			"handler_name": self.handler_name,
			"mtime"       : str(self.mtime),
			"options"     : matchoptionsdump,
		}


class MatchOptions(TenantDependentModel, UUIDKeyModel):

	match = models.ForeignKey('Match', null=False, related_name="match_options")
	mtime = models.DateTimeField(auto_now=True)

	name = models.TextField(null=False) 
	value = models.TextField(null=True, default=None)

	class Meta:
		unique_together = ("match", "name")  # accorpare situazioni di questo tipo  "-m attr --attr_name site --attr_name pippo

	def __str__(self):
		return str(self.match) + "{" + self.name +":"+self.value+ "}"
    
	def build(self,tenant,name, value,match):
		self.name = name
		self.value = value
		self.match = match
		self.tenant = tenant
		self.save()

	def destroy(self):
		self.delete()
        
	def load(self): 
		pair = {}
		pair[self.name] = self.value
		return pair

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class Target(TenantDependentModel, UUIDKeyModel):

	# tenant =
	# uuid   = 
	mtime = models.DateTimeField(auto_now=True)
	
	handler_name = models.TextField(null=False)
    
	def __str__(self):
		return str(self.rule) + ':' + self.handler_name
    
	
	def build(self,tenant,targetobj):
		self.handler_name = targetobj.name
		self.tenant = tenant
		self.save()

		for targetobj_option_name in targetobj.options.keys():
			option = TargetOptions()
			option.build(tenant , targetobj_option_name , targetobj.options[targetobj_option_name] , self)

	def destroy(self):
		for option in self.target_options.all():
			option.destroy()
		self.delete()

	def load(self):
		#print "loading", self.handler_name, self.uuid, self.target_options.count()

		targetobj = TargetMemoryObject()
		targetobj.name = self.handler_name

		for option in self.target_options.all():
			#print "\t", option
			o = option.load()
			#print "\t", o
			targetobj.options.update(o)
			
		#print "\t", targetobj.objectdump()
		return targetobj

	def replace(self , tenant , targetobj):

		self.handler_name = targetobj.name

		for option in self.target_options.all():
			option.destroy()

		for targetobj_option in targetobj.options.keys():
			option = TargetOptions()
			option.build(tenant,targetobj_option,targetobj.options[targetobj_option],self)

		self.save()

	def objdump(self):
		targetoptionsdump = {}
		for option in self.target_options.all():
			targetoptionsdump[option.name] = option.value
        
		return {
			"handler_name": self.handler_name,
			"mtime": str(self.mtime),
			"options": targetoptionsdump,
		}

class TargetOptions(TenantDependentModel, UUIDKeyModel):

	# tenant = 
	# uuid   =
	mtime    = models.DateTimeField(null=False, auto_now=True)
	
	target   = models.ForeignKey('Target', null=False, related_name="target_options")

	name     = models.TextField(null=False)  
	value    = models.TextField(null=True, default=None)

	class Meta:
		unique_together = ("target", "name")  # accorpare situazioni di questo tipo  "-m attr --attr_name site --attr_name pippo

	def __str__(self):
		return str(self.target) + "{" + self.name +":"+self.value+ "}"

	def build(self, tenant, name, value, target):
		self.tenant = tenant
		self.target = target
		self.name   = name
		self.value  = value
		self.save()

	def destroy(self):
		self.delete()

	def load(self):
		pair = {}
		pair[self.name] = self.value
		return pair

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class TargetData(TenantDependentModel):
	
	key      = models.TextField(null=False, primary_key=True)  	

	mtime    = models.DateTimeField(null=False, auto_now=True)
	data     = models.TextField(null=True, default=None)

	def get_data(self):
		try:
			return json.loads(self.data)
		except Exception, e:
			return None
		
	def set_data(self, data):
		self.data = json.dumps(data)



#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
# SANET SIGNALS                                                                #
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#

import logging

from django.dispatch import receiver

from core.models import sanet_init_subapp
from core.models import sanet_check_tenant
from core.models import sanet_cleanup_tenant
from core.models import sanet_post_check
from core.models import sanet_tenant_create

from core.models import Tenant

@receiver(sanet_init_subapp)
def _sanet_init_subapp_entables(sender, **kwargs):

	logger = logging.getLogger('sanet.global')

	logger.info(  "[entables] Init" )

	from entables.tablesstore.djangostore import init_entables_tables

	init_entables_tables(logger)


@receiver(sanet_check_tenant, sender=Tenant)
def _sanet_check_tenant_entables(sender, **kwargs):

	logger = logging.getLogger('sanet.global')

	t = kwargs['instance']

	logger.info(  "[entables] Check tenant %s" % (t) )

	from entables.tablesstore.djangostore import init_entables_chains, set_default_tenant_rules, fix_names

	init_entables_chains(logger, t)

	set_default_tenant_rules(logger, t)

	fix_names(logger, t)


@receiver(sanet_cleanup_tenant)
def _sanet_cleanup_tenant_entables(sender, **kwargs):

	logger = logging.getLogger('sanet.global')

	t = kwargs['instance']

	logger.info(  "[entables] Cleanup tenant %s" % (t) )

	return

@receiver(sanet_post_check)
def _sanet_post_check_entables(sender, **kwargs):

	logger = logging.getLogger('sanet.global')

	logger.info(  "[entables] Post check" )

@receiver(sanet_tenant_create)
def _sanet_tenant_create_entables(sender, **kwargs):

	tenant = kwargs['instance']

	from entables.tablesstore.djangostore import init_entables_chains, set_default_tenant_rules, fix_names

	init_entables_chains(None, tenant)

	set_default_tenant_rules(None, tenant)

	fix_names(None, tenant)
