# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import pkgutil

from sanet_common.pythonutils import load_symbol_from_module

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

"""
def load_class_from_path( path, symbol_name, base_name=None ):
	
	loaded_classes = {}
	
	for importer, modname, ispkg in pkgutil.iter_modules([path]):
		try:

			if base_name:
				module_fullname = '%s.%s' % (base_name, modname)
			else:
				module_fullname = modname
				
			symbol = load_symbol_from_module( module_fullname, symbol_name)
			if symbol == None:
				continue
				
			loaded_classes[module_fullname] = symbol
			
		except Exception, e:
			raise Exception("Error while importing '%s': %s" % (module_fullname, e))
		
	return loaded_classes
"""

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def load_match_module(log, module_name, matchlist):
	
	try:
		match_class = load_symbol_from_module( module_name, 'Match')
		if match_class == None:
			raise Exception("Match not found in module %s" % (module_name))

		matchmodule = match_class()

		matchlist.add_match(matchmodule)
		
		log.debug("Loaded match module '%s' with name '%s'", module_name, matchmodule.name) 

	except Exception, e:
		raise Exception("Error while importing '%s': %s" % (module_name, e))
		

def load_match_modules(log, moduled_name_list, matchlist):

    for match_module in moduled_name_list:
        load_match_module(log, match_module, matchlist)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#


from entables.targets import load_system_targets

def load_target_module(log, module_name, targetlist):

	try:
		target_class = load_symbol_from_module( module_name, 'Target')
		if target_class == None:
			raise Exception("Target not found in module %s" % (module_name))

		module = target_class()    

		log.debug("Loading target module %s", module_name)
		targetlist.add_target(module)

	except Exception, e:
		raise Exception("Error while importing '%s': %s" % (module_name, e))


def load_target_modules(log, module_name_list, targetlist):

	load_system_targets(targetlist)
	
	for module_name in module_name_list:
		load_target_module(log, module_name, targetlist)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def load_listener_module(log, module_name, listeners_list):

	try:
		symbol = load_symbol_from_module( module_name, 'LISTEN_THREAD_CLASS')
		if symbol == None:
			raise Exception("LISTEN_THREAD_CLASS not found in module %s" % (module_name))

		log.debug("Loaded listener class %s = %s", module_name, symbol)
		listeners_list.add_listener( module_name, symbol )

	except Exception, e:
		raise Exception("Error while importing '%s': %s" % (module_name, e))
		
def load_listener_modules(log, module_name_list, listeners_list):

	for listerner_module in module_name_list:
		load_listener_module(log, listerner_module, listeners_list)
		
#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def load_tagprovider_module(log, module_name, tagproviderslist):

	try:
		tagprovider_class = load_symbol_from_module( module_name, 'TagProvider')
		if tagprovider_class == None:
			raise Exception("TagProvider not found in module %s" % (module_name))

		tagprovider = tagprovider_class()

		tagproviderslist.add_tagprovider(tagprovider)  

		log.debug("Loaded tagprovider module %s: %s", module_name, tagprovider.name) 


	except Exception, e:
		msg = "Error while importing '%s': %s" % (module_name, e)
		log.warning(msg)
		raise Exception(msg)



def load_tagprovider_modules(log, module_name_list, tagproviderslist):

    for tagprovider_module in module_name_list:
        load_tagprovider_module(log, tagprovider_module, tagproviderslist)



