# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import time

import json
import datetime
import collections
import copy
import pprint
from heapq import *    # FIX: SERVE IN QUESTO FILE?


#from django.db import transaction

from core.models import Tenant
from core.api.base import db_set_current_tenant_schema

from core.db_utils import transaction_commit_on_success

from entables.constants import *

from entables.datastructures import TenantMemoryObject 
from entables.datastructures import RuleMemoryObject, TargetMemoryObject

from entables.models import Table, Chain, Template, Alarm

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#


@transaction_commit_on_success
def init(logger):
        init_entables_tables(logger)
        
        set_default_rule(logger)



def init_entables_tables(logger):
	
	for priority, table_name in DEFAULT_TABLES:

		# Create table
		table, created = Table.objects.get_or_create(name=table_name, priority=priority)
		if created:
			logger.info( "Created table %s" % (table_name) )
		else:
			logger.info( "Table %s exists" % ( table.name ) )
		table.save()


def init_entables_chains(logger, tenant):
	
	for table in Table.objects.all():
	
			db_set_current_tenant_schema(tenant.name)
			
			if logger: logger.info( "Checking table %s for tenant %s", table.name, tenant.name)
			
			if Chain.objects.filter(table=table).count() > 0:
				continue

			if logger: logger.info( "Adding default chain: %s", DEFAULT_CHAIN)

			# Add polity
			targetobj = TargetMemoryObject()
			targetobj.name = TABLE_DEFAULT_POLITY_TARGET.lower()
			
			
			table.add_chain(tenant, DEFAULT_CHAIN, DEFAULT_ADMIN_GROUP, targetobj)
                
#def set_default_rules():
#	
#	for tenant in Tenant.objects.all():


def set_default_tenant_rules(logger, tenant):

	db_set_current_tenant_schema(tenant.name)

	notify_table = Table.objects.get(name=TABLE_NOTIFY)

	# Se ci sono altre chain il sistema e' stato configurato in qualche modo
	if notify_table.chains.count() != 1:
		return

	default_chain = notify_table.chains.get(name=DEFAULT_CHAIN)

	# se ci sono regole non dobbiamo impostare niente di default
	if default_chain.rules.count() != 0:
		return

	# FIX: dobbiamo controllare che anche la policy di default si a quella
	#      attesa

	if logger: logger.info( "Adding default entables rule..." )

	rule = create_default_rule(logger)

	default_chain.append_rule(tenant, rule)

	return

def create_default_rule(logger):
	
	#
	# FIX: matches? forse no
	#
	if logger: logger.info( "Setting default rule for %s", TABLE_NOTIFY,DEFAULT_CHAIN )

	# Default target
	from entables.targets.smtp import create_default_smtp_target	

	smtp_default_target = create_default_smtp_target()

	rule = RuleMemoryObject(
		smtp_default_target,
		table  = TABLE_NOTIFY,
		chain  = DEFAULT_CHAIN,
	)

	return rule

def fix_names(logger, tenant):

	if logger: logger.info( "Fixing entables names" )

	db_set_current_tenant_schema(tenant.name)

	for c in Chain.objects.all():
		l = c.name.lower()
		c.name = l
		c.save()

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#
            
def dbdump():
	
    dump = {}
    
    for tenant in Tenant.objects.all():
	    
        db_set_current_tenant_schema(tenant.name)
        
        tenantdump = {}
        tenantdump['tables'] = collections.OrderedDict()
        tenantdump['templates'] = {}
        tenantdump['name'] = tenant.name
        
        for template in Template.objects.filter(tenant=tenant).all():
            tenantdump['templates'][template.name] = template.data
            
        for table in Table.objects.all().order_by('priority').all():
            tenantdump["tables"][table.name] = table.objdump()
            
        dump[tenant.name] = tenantdump
        
        
    return dump    

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def find_chain(tenant, table_name, chain_name):
	
	db_set_current_tenant_schema(tenant.name)

	table = Table.objects.get(name=table_name)
	try:
		chain = table.chains.get(tenant=tenant, name=chain_name)
		return True
	except Exception,e:
		return False


@transaction_commit_on_success
def set_chain_policy(tenant,table_name,chain_name,policy_targetobj):

	db_set_current_tenant_schema(tenant.name)

	table = Table.objects.get(name=table_name)   

	chain = table.chains.get(name=chain_name.lower())

	chain.change_policy(policy_targetobj)


@transaction_commit_on_success
def add_chain(tenant, table_name, chain_name, policy_targetobj):

	# FIX: verificare che il nome della chain non sia di una chain riservata????

	db_set_current_tenant_schema(tenant.name)

	table = Table.objects.get(name = table_name)

	table.add_chain(tenant, chain_name.lower(), DEFAULT_ADMIN_GROUP, policy_targetobj)


@transaction_commit_on_success
def delete_chain(tenant, table_name, chain_name):

	if chain_name in RESERVED_CHAIN_NAMES:
		raise Exception("Forbidden name '%s'" % (chain_name))

	db_set_current_tenant_schema(tenant.name)

	table = Table.objects.get(name = table_name)

	table.delete_chain(chain_name.lower())


@transaction_commit_on_success
def rename_chain(tenant, table_name, chain_name, new_name):

	db_set_current_tenant_schema(tenant.name)

	if chain_name in RESERVED_CHAIN_NAMES:
		raise Exception("Forbidden name '%s'" % (chain_name))

	if new_name in RESERVED_CHAIN_NAMES:
		raise Exception("Forbidden name '%s'" % (new_name))

	table = Table.objects.get(name=table_name)

	chain = table.chains.get(name=chain_name.lower())
	chain.rename(new_name)


@transaction_commit_on_success
def flush_chain(tenant,table_name,chain_name):
	
	db_set_current_tenant_schema(tenant.name)

	table = Table.objects.get(name=table_name)

	chain = table.chains.get(name=chain_name.lower())
	chain.flush()


@transaction_commit_on_success
def zero_chain(tenant, table_name, chain_name,rulenum=0):
	
	db_set_current_tenant_schema(tenant.name)

	table = Table.objects.get(name=table_name)

	chain = Chain.objects.get(name=chain_name.lower(), table=table)

	chain.zero()

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

   
@transaction_commit_on_success
def append_rule(tenant, table_name, chain_name, ruleobj):

	db_set_current_tenant_schema(tenant.name)

	table = Table.objects.get(name=table_name)

	name = chain_name.lower()
	chain = table.chains.get(name=name)
	chain.append_rule(tenant, ruleobj)


@transaction_commit_on_success
def insert_rule(tenant, table_name, chain_name, ruleobj, rulepos):

	db_set_current_tenant_schema(tenant.name)

	table = Table.objects.get(name=table_name)
	chain = table.chains.get(name=chain_name.lower())

	chain.insert_rule(tenant, ruleobj, rulepos)


@transaction_commit_on_success
def replace_rule(tenant, table_name, chain_name, ruleobj, rulepos):
	
	db_set_current_tenant_schema(tenant.name)

	table = Table.objects.get(name=table_name)
	chain = table.chains.get(name=chain_name.lower())

	chain.replace_rule(tenant, ruleobj, rulepos)


@transaction_commit_on_success
def delete_rule(tenant, table_name, chain_name, ruleobj, rule_position):

	db_set_current_tenant_schema(tenant.name)

	table = Table.objects.get(name=table_name)
	chain = table.chains.get(name=chain_name.lower())

	chain.delete_rule(rule_position, ruleobj)


@transaction_commit_on_success
def delete_rule_by_pos(tenant, table_name, chain_name, rule_position):
	db_set_current_tenant_schema(tenant.name)

	table = Table.objects.get(name=table_name)
	chain = table.chains.get(name=chain_name.lower())

	chain.delete_rule_by_pos(rule_position)


@transaction_commit_on_success
def disable_rule(tenant, table_name, chain_name, rule_position):
	
	db_set_current_tenant_schema(tenant.name)

	table = Table.objects.get(name=table_name)

	chain = Chain.objects.get(name=chain_name.lower(), tenant=tenant, table=table)

	rule = chain.rules.get(position=rule_position, tenant=tenant)

	rule.disable()


@transaction_commit_on_success
def activate_rule(tenant, table_name, chain_name, rule_position):
	
	db_set_current_tenant_schema(tenant.name)

	table = Table.objects.get(name=table_name)

	chain = Chain.objects.get(name=chain_name.lower(), tenant=tenant, table=table)

	rule = chain.rules.get(position=rule_position, tenant=tenant)

	rule.activate()

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class RuleDB(object):
    
	def __init__(self):
		self.tenants = {}
		self.load_time = 0
    
	def get_tenant(self, uuid):
		return self.tenants[uuid]
        
	def get_tenant_tables(self, tenant_uuid):
		return self.tenants[tenant_uuid].tables
        
	def load_rules(self):

		for dbtenant in Tenant.objects.all():
			db_set_current_tenant_schema(dbtenant.name)
			
			tenant = TenantMemoryObject(uuid=dbtenant.uuid, name=dbtenant.name)
			
			for table in Table.objects.all().order_by('priority'): 
				tenant.tables[table.name] = table.load()
				
			for template in Template.objects.filter(tenant=dbtenant).all(): 
				tenant.templates[template.name] = template.data

			self.tenants[tenant.uuid] = tenant
		
		self.load_time = time.time()
		
		#pprint.pprint( self.objectdump() )
    
	# Add method reload_rules to Ruledb datastructure    
	def reload_rules(self):
		# FIX: TODO
		pass

	def objectdump(self):
		objdump = {}
		for tenantid, tenant in self.tenants.items():
			objdump[tenant.name] = tenant.objectdump()
	
		return objdump
    
#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#
    
# Alarmcache is a priorityqueued dictionary ....
class AlarmCache(dict):

    def __init__(self,*args, **kwargs):
        super(AlarmCache, self).__init__(*args, **kwargs)
        self._rebuild_heap()

    def _rebuild_heap(self):
        self._heap = [(self['priority'], uuid) for uuid, alarm in self.iteritems()]
        heapify(self._heap)

    def __setitem__(self, uuid, alarmobj):
        # We are not going to remove the previous value from the heap,
        # since this would have a cost O(n).
        
        super(AlarmCache, self).__setitem__(uuid, alarmobj)
        
        if len(self._heap) < 2 * len(self):
            heappush(self._heap, (alarmobj.priority, uuid))
        else:
            # When the heap grows larger than 2 * len(self), we rebuild it
            # from scratch to avoid wasting too much memory.
            self._rebuild_heap()

    def __delitem__(self, uuid):
        #pprint.pprint(str(self))
        self._heap.remove((self[uuid].priority,uuid))
        super (AlarmCache, self).__delitem__(uuid)


    def iteralarms(self):
        for piority,uuid in self._heap:
            yield self[uuid]

    def update(self, *args, **kwargs):
        # Reimplementing dict.update is tricky -- see e.g.
        # http://mail.python.org/pipermail/python-ideas/2007-May/000744.html
        # We just rebuild the heap from scratch after passing to super.
        
        super(AlarmCache, self).update(*args, **kwargs)
        self._rebuild_heap()
    
#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class AlarmQueue(object):

    def __init__(self):
        self.alarmcache = AlarmCache()
        
        self.load_time = 0
        
        
    def load_alarms(self):

	for tenant in Tenant.objects.all():

		db_set_current_tenant_schema(tenant.name)
		pending_alarms = Alarm.objects.filter(tenant=tenant).all()

		for alarm in pending_alarms:
			self.alarmcache[alarm.uuid] = alarm.load()

	self.load_time = time.time()


    # XXX: Permanent queue caching ?
    @transaction_commit_on_success
    def put(self, alarmobj):	    
        tenant = Tenant.objects.get(uuid=alarmobj.tenant_uuid)
        db_set_current_tenant_schema(tenant.name)
        
        dbalarm = Alarm.build_and_save(tenant, alarmobj)
        alarmobj = dbalarm.load()

        self.alarmcache[alarmobj.uuid] = alarmobj
        
        return alarmobj
        

    def get(self,uuid):
        try:
            return self.alarmcache[uuid]
        except:
            return None

    def items(self):
        return self.alarmcache.iteralarms()

    def getall(self):
        #alarmlist = []
        #for uuid, in self.alarmcache.keys():
        #    alarmlist.append(self.alarmcache[uuid])
        #alarmlist = [ a for a in self.alarmcache.values() ]    
        return [ a for a in self.alarmcache.values() ]

    def dump(self):
        dumpall = []
        for uuid in self.alarmcache.keys():
            dumpall += self.alarmcache[uuid].dump()
        return dumpall

    @transaction_commit_on_success
    def change_status(self, alarmid, status):

        alarm = self.alarmcache[alarmid]
        
        # Update DB
        # FIX: fa schifo questa gestione. Tanto vale tenere in memoria direttamente gli oggetti django
        tenant = Tenant.objects.get(uuid=alarm.tenant_uuid)
        db_set_current_tenant_schema(tenant.name)                
        dbalarm = Alarm.objects.get(tenant=tenant, uuid = alarmid)
        dbalarm.set_rulestack( alarm.rulestack )
        dbalarm.set_data( alarm. data )
        dbalarm.set_stages( alarm.stages )
        dbalarm.status = status

        dbalarm.hold_time       = alarm.hold_time
        dbalarm.hold_wakeuptime = alarm.hold_wakeuptime
        dbalarm.hold_handler    = alarm.hold_handler

        dbalarm.merge_group_key = alarm.merge_group_key

        dbalarm.save()
        
        alarm.status = status

   
    @transaction_commit_on_success
    def save(self, alarmid):
        
        alarm = self.alarmcache[alarmid]
       
        tenant = Tenant.objects.get(uuid=alarm.tenant_uuid)
        db_set_current_tenant_schema(tenant.name)        
        
        dbalarm = Alarm.objects.get(tenant=tenant, uuid = alarmid)
        dbalarm.set_rulestack( alarm.rulestack )
        dbalarm.set_data     ( alarm. data )
        dbalarm.set_stages   ( alarm.stages )
        dbalarm.status          = alarm.status

        dbalarm.hold_time       = alarm.hold_time
        dbalarm.hold_wakeuptime = alarm.hold_wakeuptime
        dbalarm.hold_handler    = alarm.hold_handler


        dbalarm.merge_group_key = alarm.merge_group_key
        dbalarm.save()

    @transaction_commit_on_success
    def archive(self, tenant_uuid, alarmid):

        tenant = Tenant.objects.get(uuid=tenant_uuid)
        db_set_current_tenant_schema(tenant.name)        

        alarm = Alarm.objects.get(tenant=tenant, uuid = alarmid)
        alarm.move_to_archive()
        
        del self.alarmcache[alarmid]

