# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

Alarmpath
=========

Questo modulo permette di accedere alla struttura interna di un allarme e di estrapolare un qualunque campo. 

La sintassi e':

::
	
	{{ alarmpath.<key1> [ . <key N> ] }}

Ad esempio, ipotizzando che la struttura dell'allarme da processare sai la seguente: 

::
	
	{
		'condition': {
			'name': 'check',
			'path': 'localhost;;debug;check',
			'priority': -1,
			'uuid': '7b87d0fc87a340bba6a5e1d9cf03a76d'
		},
		'datasources': [],
		'element': {
			'name': 'localhost',
			'parent_id': None,
			'parent_name': None,
			'type': 1,
			'uuid': 'c3af80abb3104a2cad031b90e30efbdb'
		},
		'event': {
			'uuid': '37f99ce859534a959f82a07eeed2c728'
		},
		'flow-information': [],
		'mail_data': None,
		'result': {
			'laststatus': 100, 
			'status': 20, 
			'verbastatus': ''
		},
		'snmp_values': [],
		'tenantid': '10a279a6ce5c4438a98f9c33c0b77cde',
		'uuid': '2a0a23eaeab74b96a85b932877a77dfb'
	}

Sara' possibile estrarre i dati in questa maniera: 

::
	
	{{ alarmpath.condition.name }}			-> "check"
	
	{{ alarmpath.condition.priority }}		-> -1
	
	{{ alarmpath.event.uuid }}			-> "37f99ce859534a959f82a07eeed2c728"


NOTA: Poiche' la struttura dati di un allarme potrebbe cambiare nel tempo in base alla versione di SANET, questo tag e' pensato 
      quasi esclusivamente per gestire allarmi particolari generati/inseriti manualmente nel sistema in condizioni particolari.
      
      Per accedere in maniera standard ai dati di un allaerme si consiglia di utilizzare i tag forniti dal modulo "builtin".


Sintassi deprecata
---------------------

.. danger:: La seguente sintassi e' disponibile per retro-compatibilita' e verra' rimossa. Si consiglia di aggiornare i template utilizzati e utilizzare la sintassi descritta nel paragrafo precedente.

::
	
	${tags['alarmpath:condition:name']}
	
	${tags['alarmpath:condition:priority']}
	
	${tags['alarmpath:event:uuid']}
	...

"""

from entables.tagproviders import BaseTagProvider

from server.sanetd.sanetdlogic     import fix_object_for_jsondump

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class TagProvider(BaseTagProvider):

	def __init__(self):
		super(TagProvider, self).__init__()
		self.name = 'alarmpath'

	def get_provider_symbol(self, alarm, log=None):
		# Dobbiamo restituire una struttura dati che permetta di accedere
		# a tutte le chiavi (e sotto-chiavi) dell'allarme.
		#
		# L'allarme e' gia' un dizionario di chiavi quindi basta
		# restituire l'allarme stesso.

		return fix_object_for_jsondump(alarm)


	# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	# DEPRECATO >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

	def __gettags(self, alarm):
		try:
			ret = flatten(alarm)
			return ret
		except Exception, e:
			print "Error while calculating tags for %s: %s" % (self.name, e)
			return {}

	def has_symbol(self, alarm, symbol):
		tags = self.__gettags(alarm)
		return symbol in tags

	def get_symbol(self, alarm, symbol):
		tags = self.__gettags(alarm)
		return tags[symbol]

	def has_shortcut_symbol(self, alarm, symbol):
		# Non ci sono shortcut
		return False

	def get_shortcut_symbol(self, alarm, symbol):
		# Non ci sono shortcut
		return None

	# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
	# DEPRECATO <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
	# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<


def flatten(alarmobject, prefix=''):
	    
	res = {}
	for key, val in alarmobject.items():
		if isinstance(val, dict):
			flatted_values = flatten(val, prefix='%s:'%(key))
			res.update(flatted_values)
		else:
			res[prefix+key] = val
	return res
