# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

.. _tags_builtin:

Builtin
=========

I simboli definiti da questo tag sono definiti nella seguente tabella.

Poiche' questo e' un tag fornito dal sistema tutti i tag definiti sono accessibili anche tramite degli shortcut: 

Dati generici sull'allarme:
----------------------------

	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| TAG                              | Shortcut                  | Descrizione                                                                                                         |
	+==================================+===========================+=====================================================================================================================+
	| {{builtin.alarm.uuid}}           | {{alarm.uuid}}            | UUID Allarme                                                                                                        |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.alarm.ts}}             | {{alarm.ts}}              | Alarm creation timestamp                                                                                            |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.alarm.dateiso}}        | {{alarm.dateiso}}         | Alarm creation in ISO format                                                                                        |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.rawalarm}}             | {{rawalarm}}              | Testo contenente allarme in formato json                                                                            |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+

Dati sul tenant
----------------------------

	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| TAG                              | Shortcut                  | Descrizione                                                                                                         |
	+==================================+===========================+=====================================================================================================================+
	| {{builtin.tenant.name}}          | {{tenant.name}}           | Nome del tenant                                                                                                     |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.tenant.uuid}}          | {{tenant.uuid}}           | UUID del tenantl                                                                                                    |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+


Dati sull'elemento di monitoraggio
------------------------------------

	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| TAG                              | Shortcut                  | Descrizione                                                                                                         |
	+==================================+===========================+=====================================================================================================================+
	| {{builtin.element.kind}}         | {{element.kind}}          | Tipo di elemento del monitoraggio                                                                                   |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.element.type}}         | {{element.type}}          | Tipo di elemento del monitoraggio (numerico)                                                                        |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.node.name}}            | {{node.name}}             | Nome del nodo                                                                                                       |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.node.description}}     | {{node.description}}      | {{nodedescr}}             | Descrizione del nodo                                                                    |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.iface.name}}           | {{iface.name}}            | Nome interfaccia                                                                                                    | 
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.storage.name}}         | {{storage.name}}          | Nome storage                                                                                                        |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.service.name}}         | {{service.name}}          | Nome servizio                                                                                                       |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.device.name}}          | {{device.name}}           | Nome device                                                                                                         |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.node.sysdescr}}        | {{node.sysdescr}}         | SMNP SysDescr nodo                                                                                                  | 
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.linked_node.name}}     | {{linked_node.name}}      | Nome nodo collegato                                                                                                 |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.linked_iface.name}}    | {{linked_iface.name}}     | Nome interfaccia collegata                                                                                          | 
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+



Dati sulla condition legata all'allarme
------------------------------------------

	+----------------------------------------+------------------------------------+---------------------------------------------------------------------------------------------------------------------+
	| TAG                                    | Shortcut                           | Descrizione                                                                                                         |
	+========================================+====================================+=====================================================================================================================+
	| {{builtin.condition.status}}           | {{condition.status}}               | Stato della condition                                                                                               |
	+----------------------------------------+------------------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.condition.laststatus}}       | {{condition.laststatus}}           | Stato precedente della condition                                                                                    |
	+----------------------------------------+------------------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.condition.statuslastchange}} | {{condition.statuslastchange}}     | Ultima variaizone di stato della condition                                                                          |
	+----------------------------------------+------------------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.condition.verbstatus}}       | {{condition.verbstatus}}           | Verbstatus del check                                                                                                |
	+----------------------------------------+------------------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.condition.priority}}         | {{condition.priority}}             | Priorita' dell'allarme                                                                                              |
	+----------------------------------------+------------------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.condition.path}}             | {{condition.path}}                 | Path della condition                                                                                                |
	+----------------------------------------+------------------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.condition.maxtries}}         | {{condition.maxtries}}             | Max tries della condition                                                                                           |
	+----------------------------------------+------------------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.condition.classification}}   | {{condition.classification}}       | Classificazione della condition                                                                                     |
	+----------------------------------------+------------------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.snmpval}}                    | {{snmpval}}                        | Valore variabile SNMP letta dalla condition                                                                         |
	+----------------------------------------+------------------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.snmpvalues}}                 | {{snmpvalues}}                     | Valori SNMP letti per effettuare il controllo                                                                       |
	+----------------------------------------+------------------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.ip4s}}                       | {{ip4s}}                           | Indirizzi IPv4 del nodo separati da ','                                                                             |
	+----------------------------------------+------------------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.now}}                        | {{now}}                            | Timestamp orario elaborazione dell'allarme                                                                          |
	+----------------------------------------+------------------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.now_iso}}                    | {{now_iso}}                        | Orario elaborazione allarme in formato ISO                                                                          |
	+----------------------------------------+------------------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.element.url}}                | {{url}}                            | URL della risorsa monitorata                                                                                        |
	+----------------------------------------+------------------------------------+---------------------------------------------------------------------------------------------------------------------+

Tag associati alla condition:

	+----------------------------------------+------------------------------------+---------------------------------------------------------------------------------------------------------------------+
	| TAG                                    | Shortcut                           | Descrizione                                                                                                         |
	+========================================+====================================+=====================================================================================================================+
	| {{builtin.tags.all}}                   | {{tags._all_}}                     | Tutti i tag associati all'elemento di monitoraggio (separati da ',')                                                |
	+----------------------------------------+------------------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.tags.<tree_name>}}           | {{tags.<tree_name>}}               | Tutti i tag associati all'elemento di monitoraggio (separati da ',') nel tagtree con nome *tree_name*.              |
	+----------------------------------------+------------------------------------+---------------------------------------------------------------------------------------------------------------------+

	Esempi:

		::
			
			{{builtin.tags.geo}}        ->    geo:/Global/sede1
			
			{{builtin.tags.servizi}}    ->    servizi:/Posta,servizi:/Database
			
			{{builtin.tags.all}}        ->    geo:/Global/sede1,servizi:/Posta,servizi:/Database


Parametri email associati alla condition:

	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| TAG                                | Shortcut                  | Descrizione                                                                                                         |
	+====================================+===========================+=====================================================================================================================+
	| {{builtin.mail.to}}                | {{mail.to}}               | email, upemail                                                                                                      |
	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.mail.subject}}           | {{mail.subject}}          | msg-upsubj, msg-downsubj                                                                                            |
	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.mail.body}}              | {{mail.body}}             | msg-upbody, msg-downbody                                                                                            |
	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| {{builtin.mail.xheaders}}          | {{mail.xheaders}}         |                                                                                                                     |
	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+


Esempio: 

	::
		
		entables -A DEFAULT -j SYSLOG --syslog_msg "Allarme in arrivo ID {{alarmid}} generato in data {{alarmdateiso}}"



Sintassi deprecata
--------------------------

.. danger:: La seguente sintassi e' disponibile per retro-compatibilita' e verra' rimossa. Si consiglia di aggiornare i template utilizzati e utilizzare la sintassi descritta nel paragrafo precedente.


Questi sono i tag nel vecchio formato:

	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| TAG                              | Shortcut                  | Descrizione                                                                                                         |
	+==================================+===========================+=====================================================================================================================+
	| ${tags['builtin:alarmid']}       | ${alarmid}                | UUID Allarme                                                                                                        |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:alarmts']}       | ${alarmts}                | Alarm creation timestamp                                                                                            |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:alarmdateiso']}  | ${alarmdateiso}           | Alarm creation in ISO format                                                                                        |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:rawalarm']}      | ${rawalarm}               | Testo contenente allarme in formato json                                                                            |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+

Dati sul tenant:

	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| TAG                              | Shortcut                  | Descrizione                                                                                                         |
	+==================================+===========================+=====================================================================================================================+
	| ${tags['builtin:tenantname']}    | ${tenantname}             | Nome del tenant                                                                                                     |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:tenantid']}      | ${tenantid}               | UUID del tenantl                                                                                                    |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+

Dati sull'elemento di monitoraggio:

	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| TAG                              | Shortcut                  | Descrizione                                                                                                         |
	+==================================+===========================+=====================================================================================================================+
	| ${tags['builtin:kind']}          | ${kind}                   | Tipo di elemento del monitoraggio                                                                                   |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:type']}          | ${type}                   | Tipo di elemento del monitoraggio (numerico)                                                                        |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:nodename']}      | ${nodename}               | Nome del nodo                                                                                                       |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:nodedescr']}     | ${nodedescr}              | Descrizione del nodo                                                                                                |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:ifacename']}     | ${ifacename}              | Nome interfaccia                                                                                                    | 
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:storagename']}   | ${storagename}            | Nome storage                                                                                                        |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:servicename']}   | ${servicename}            | Nome servizio                                                                                                       |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:devicename']}    | ${devicename}             | Nome device                                                                                                         |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:sysdescr']}      | ${sysdescr}               | SMNP SysDescr nodo                                                                                                  | 
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:linked_node']}   | ${linked_node}            | Nome nodo collegato                                                                                                 |
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:linked_iface']}  | ${linked_iface}           | Nome interfaccia collegata                                                                                          | 
	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	
Dati sulla condition legata all'allarme:
	
	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| TAG                                | Shortcut                  | Descrizione                                                                                                         |
	+====================================+===========================+=====================================================================================================================+
	| ${tags['builtin:status']}          | ${status}                 | Stato della condition                                                                                               |
	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:laststatus']}      | ${laststatus}             | Stato precedente della condition                                                                                    |
	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:statuslastchange']}| ${statuslastchange}       | Ultima variaizone di stato della condition                                                                          |
	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:verbstatus']}      | ${verbstatus}             | Verbstatus del check                                                                                                |
	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:priority']}        | ${priority}               | Priorita' dell'allarme                                                                                              |
	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:path']}            | ${path}                   | Path della condition                                                                                                |
	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:maxtries']}        | ${maxtries}               | Max tries della condition                                                                                           |
	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:classification']}  | ${classification}         | Classificazione della condition                                                                                     |
	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:snmpval']}         | ${snmpval}                | Valore variabile SNMP letta dalla condition                                                                         |
	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:snmpvalues']}      | ${snmpvalues}             | Valori SNMP letti per effettuare il controllo                                                                       |
	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:ip4s']}            | ${ip4s}                   | Indirizzi IPv4 del nodo separati da ','                                                                             |
	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:geotag']}          | ${geotag}                 | Tag del tagtree "geo"                                                                                               |
	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:alltags']}         | ${alltags}                | Tutti i tag associati all'elemento di monitoraggio (separati da ',')                                                |
	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:now']}             | ${now}                    | Timestamp orario elaborazione dell'allarme                                                                          |
	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:now_iso']}         | ${now_iso}                | Orario elaborazione allarme in formato ISO                                                                          |
	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:url']}             | ${url}                    | URL della risorsa monitorata                                                                                        |
	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+

Parametri email associati alla condition:

	+------------------------------------+-----------------+---------------------------------------------------------------------------------------------------------------------+
	| TAG                                | Shortcut        | Descrizione                                                                                                         |
	+====================================+=================+=====================================================================================================================+
	| ${tags['builtin:mail_to']}         | ${mail_to}      | email, upemail                                                                                                      |
	+------------------------------------+-----------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:mail_subject']}    | ${mail_subject} | msg-upsubj, msg-downsubj                                                                                            |
	+------------------------------------+-----------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:mail_body']}       | ${mail_body}    | msg-upbody, msg-downbody                                                                                            |
	+------------------------------------+-----------------+---------------------------------------------------------------------------------------------------------------------+
	| ${tags['builtin:mail_headers']}    | ${mail_headers} |                                                                                                                     |
	+------------------------------------+-----------------+---------------------------------------------------------------------------------------------------------------------+

Esempio: 

	::
		
		entables -A DEFAULT -j SYSLOG --syslog_msg "Allarme in arrivo ID ${alarmid} generato in data ${alarmdateiso}"


"""

import time
import pprint
import traceback
import copy
import json
import types

from sanet_common.sanetdefs.constants import CSTATE_TO_CODENAME
from sanet_common.sanetdefs.constants import ELEMENT_KIND_IFACE
from sanet_common.sanetdefs.constants import ELEMENT_KIND_STORAGE
from sanet_common.sanetdefs.constants import ELEMENT_KIND_SERVICE
from sanet_common.sanetdefs.constants import ELEMENT_KIND_DEVICE

from sanet_common.timeutils import timestamp_to_isostr

from server.sanetd.sanetdlogic import fix_object_for_jsondump

from entables.tagproviders import BaseTagProvider

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

NOT_DEFINED = '*NOTDEFINED*'

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class TagProvider(BaseTagProvider):

	def __init__(self):
		super(TagProvider, self).__init__()
		self.name = 'builtin'

	def get_provider_symbol(self, alarm, log=None):

		_dict = {}

		def GET_OR_UNDEFINED(a, *keys):
			try:
				v = a
				for k in keys:
					v = v[k]
				return v
			except KeyError, e:
				return BaseTagProvider.SYMBOL_NOTDEFINED

		try:
			a = copy.deepcopy(alarm)
			a.update({
				'dateiso': timestamp_to_isostr(GET_OR_UNDEFINED(alarm,'ts'))
			})

			# NOTE: these wildcards are alread defined in {{system.}}
			_dict['now'    ] = time.time()
			_dict['now_iso'] = timestamp_to_isostr(time.time())

			_dict['alarm'] = a

			_dict['tenant'] = {
				'name': GET_OR_UNDEFINED(alarm,'tenantname'),
				'uuid': GET_OR_UNDEFINED(alarm,'tenantid'),
			}

			_dict['element'] = {
				'kind': GET_OR_UNDEFINED(alarm,'element','kind'),
				'type': GET_OR_UNDEFINED(alarm,'element','type'),

				'url' : GET_OR_UNDEFINED(alarm, 'url')
			}

			_dict[ 'node' ] = {
				'name'       : GET_OR_UNDEFINED(alarm,'element','parent_name') if alarm.get('element',{}).get('parent_name',None) != None else GET_OR_UNDEFINED(alarm,'element','name'),

				'description': GET_OR_UNDEFINED(alarm,'element', 'nodedescr'),

				'ip4s'       : ','.join( alarm.get('info',{}).get('ip4s',[])  ),

				'sysdescr'   : GET_OR_UNDEFINED(alarm, 'info', 'sysdescr')
			}
			

			_dict['iface']   = { 'name': GET_OR_UNDEFINED(alarm,'element','name') if (GET_OR_UNDEFINED(alarm,'element','kind') == ELEMENT_KIND_IFACE)   else BaseTagProvider.SYMBOL_NOTDEFINED }
			_dict['storage'] = { 'name': GET_OR_UNDEFINED(alarm,'element','name') if (GET_OR_UNDEFINED(alarm,'element','kind') == ELEMENT_KIND_STORAGE) else BaseTagProvider.SYMBOL_NOTDEFINED }
			_dict['service'] = { 'name': GET_OR_UNDEFINED(alarm,'element','name') if (GET_OR_UNDEFINED(alarm,'element','kind') == ELEMENT_KIND_SERVICE) else BaseTagProvider.SYMBOL_NOTDEFINED }
			_dict['device']  = { 'name': GET_OR_UNDEFINED(alarm,'element','name') if (GET_OR_UNDEFINED(alarm,'element','kind') == ELEMENT_KIND_DEVICE)  else BaseTagProvider.SYMBOL_NOTDEFINED }


			_dict[ 'linked_node' ] = {
				'name': alarm.get('linked',{}).get('node', BaseTagProvider.SYMBOL_NOTDEFINED)
			} 

			_dict[ 'linked_iface'] = {
				'name': alarm.get('linked',{}).get('iface', BaseTagProvider.SYMBOL_NOTDEFINED)
			} 

			_dict[ 'condition'] = {
				 'status'          : CSTATE_TO_CODENAME(alarm['result']['status'    ]) if (alarm.get('result', {}).get('status'    , None) != None ) else  BaseTagProvider.SYMBOL_NOTDEFINED
				,'laststatus'      : CSTATE_TO_CODENAME(alarm['result']['laststatus']) if (alarm.get('result', {}).get('laststatus', None) != None ) else  BaseTagProvider.SYMBOL_NOTDEFINED
				,'expr'            : GET_OR_UNDEFINED(alarm,'condition', 'expr'            )
				,'priority'        : GET_OR_UNDEFINED(alarm,'condition', 'priority'        )
				,'path'            : GET_OR_UNDEFINED(alarm,'condition', 'path'            )
				,'maxtries'        : GET_OR_UNDEFINED(alarm,'condition', 'maxtries'        )
				,'statuslastchange': GET_OR_UNDEFINED(alarm,'condition', 'statuslastchange')
				,'classification'  : GET_OR_UNDEFINED(alarm,'condition', 'classification'  )
				,'verbstatus'      : GET_OR_UNDEFINED(alarm,'result'   , 'verbstatus'      )
				,'path'            : GET_OR_UNDEFINED(alarm,'condition', 'path'            )
			}

			_dict[ 'rawalarm' ] = json.dumps( fix_object_for_jsondump(alarm) ) # Avoid problems with raw strings of bytes

			_dict[ 'params'] = alarm.get('params',{})

			_dict['ip4s'] = ','.join(alarm.get('info',{}).get('ip4s',[]))

			#
			# TAGS
			#
			_dict[ 'tags' ] = {
				'all': ','.join(alarm.get('tags', [])),
			}

			tags_by_tree = {}
			for full_path in alarm.get('tags',[]):
				tree_name, tag_path = full_path.split(':', 1)
				l = tags_by_tree.get(tree_name, tags_by_tree.setdefault(tree_name, []))
				l.append(full_path)
			for tree_name, tag_paths in tags_by_tree.items():
				_dict[ 'tags' ][tree_name] = ','.join(tag_paths)

			#
			# SNMP VALUES
			#
			snmp_values = alarm.get('saved_variables',{})
			if snmp_values is not None:
				snmp_values = snmp_values.get('snmp_variables',{})

			if snmp_values:
				safe_snmp_values = fix_object_for_jsondump(snmp_values)

				_dict['snmpval'   ] = sorted(safe_snmp_values.items())[0][1]
				_dict['snmpvalues'] = '\n'.join( ['%s = %s' % (oid, val) for oid,val in sorted(safe_snmp_values.items())] )

			mail_data = alarm.get('mail', None)
			if mail_data is not None:
				d = {
					  'to'       : mail_data['to']
					, 'subject'  : mail_data['subject']
					, 'body'     : mail_data['body']
					, 'xheaders' : '\n'.join([ "%s: %s" % (f,v) for f,v in mail_data.get('headers',{}).items()])
				}
				_dict[ 'mail'] = d


			#
			# Custom fields
			#
			try:
				from core.models import Tenant, MonitoredElement
				tenantid = GET_OR_UNDEFINED(alarm,'tenantid')
				uuid     = GET_OR_UNDEFINED(alarm,'element','uuid')

				tenant = Tenant.objects.get(uuid=tenantid)
				tenant.db_set_current_schema()

				me       = MonitoredElement.objects.get(uuid=uuid)

				_dict[ 'node' ]['customfields'] = me.customfields

			except Tenant.DoesNotExist, e:
				pass
			except MonitoredElement.DoesNotExist, e:
				pass


		except Exception, e:
			log.error("builtin: Error while producing tags: %s %s -> %s", type(e), str(e), traceback.format_exc())

		return _dict






	# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	# DEPRECATO >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>


	VALID_SYMBOLS = [
		  'alarmid'     
		, 'alarmts'     
		, 'alarmdateiso'     
		, 'tenantname'  
		, 'tenantid'    
		, 'kind'        
		, 'type'        
		, 'nodename'    
		, 'nodedescr'   
		, 'ifacename'   
		, 'storagename' 
		, 'servicename' 
		, 'devicename'  
		, 'sysdescr'    
		, 'linked_node' 
		, 'linked_iface'
		, 'status'          
		, 'laststatus'      
		, 'expr'            
		, 'priority'        
		, 'path'            
		, 'maxtries'        
		, 'statuslastchange'
		, 'classification'  
		, 'verbstatus'      
		, 'snmpval'   
		, 'snmpvalues'
		, 'snmpval'   
		, 'snmpvalues'
		, 'ip4s'          
		, 'geotag'        
		, 'alltags'       
		, 'now'           
		, 'now_iso'       
		, 'url'           
		, 'rawalarm'      
		, 'mail_to'       
		, 'mail_subject'  
		, 'mail_body'     
		, 'mail_xheaders' 
		
		
		# Simboli gestione template 2
		, 'alarm'
		, 'tenant'
		, 'element'
		, 'node'
		, 'iface'
		, 'storage'
		, 'service'
		, 'device'
		, 'linked_node'
		, 'linked_iface'
		, 'condition'
		
		, 'params'
		
		, 'tags'
		
		, 'mail'
		
		, 'notes'
		
	]
	
	VALID_SHORTCUTS = [
		  'alarmid'
		, 'alarmts'
		, 'alarmdateiso'     
		, 'tenantname'
		, 'tenantid'
		, 'kind'
		, 'type'
		, 'nodename'
		, 'nodedescr'   
		, 'ifacename'   
		, 'storagename' 
		, 'servicename' 
		, 'devicename'  
		, 'sysdescr'    
		, 'linked_node' 
		, 'linked_iface'
		, 'status'          
		, 'laststatus'      
		, 'expr'            
		, 'priority'        
		, 'path'            
		, 'maxtries'        
		, 'statuslastchange'
		, 'classification'  
		, 'verbstatus'      
		, 'snmpval'   
		, 'snmpvalues'
		, 'snmpval'   
		, 'snmpvalues'
		, 'ip4s'          
		, 'geotag'        
		, 'alltags'       
		, 'now'           
		, 'now_iso'       
		, 'url'           
		, 'rawalarm'      
		, 'mail_to'       
		, 'mail_subject'  
		, 'mail_body'     
		, 'mail_xheaders' 
	]	

	
	def __init__(self):
		
		super(TagProvider, self).__init__()
		
		self.name = 'builtin'


	def get_symbol(self, alarm, symbol):
		return self.__get(alarm, symbol)

	def get_shortcut_symbol(self, alarm, symbol):
		return self.__get(alarm, symbol)



	def __get(self, alarm, key):

		try:
			#
			# Dati evento
			#

			#
			# Dati allarme
			#
			if key == 'alarmid': return alarm['uuid']

			if key == 'alarmts': return alarm['ts']
			
			if key == 'alarmdateiso': return timestamp_to_isostr(alarm['ts'])

			#
			# Dati del tenant
			#

			#       tenant name                         -> ${tenantname}
			if key == 'tenantname': return alarm['tenantname']
			#       tenant id                           -> ${tenantid}
			if key == 'tenantid'  : return alarm['tenantid']

			#
			# Dati elementi del monitoraggio
			#

			# Dovrebbero esfsere identici
			if key == 'kind': return alarm['element']['kind']
			if key == 'type': return alarm['element']['type']

			#%H     node.name                           -> ${nodename}
			if key == 'nodename': return alarm['element']['parent_name'] if alarm['element']['parent_name'] != None else alarm['element']['name']

			#%D     descrizione del nodo                -> ${nodedescr}
			if key == 'nodedescr': return alarm['element']['nodedescr']

			#%I     iface.name                          -> $(ifacename}
			if key == 'ifacename'  : return alarm['element']['name'] if (alarm['element']['kind'] == ELEMENT_KIND_IFACE)   else NOT_DEFINED
			if key == 'storagename': return alarm['element']['name'] if (alarm['element']['kind'] == ELEMENT_KIND_STORAGE) else NOT_DEFINED
			if key == 'servicename': return alarm['element']['name'] if (alarm['element']['kind'] == ELEMENT_KIND_SERVICE) else NOT_DEFINED
			if key == 'devicename' : return alarm['element']['name'] if (alarm['element']['kind'] == ELEMENT_KIND_DEVICE)  else NOT_DEFINED

			#%ID    SNMP DESCRIPTION                    -> ${sysdescr}
			if key == 'sysdescr'   : return alarm.get('info',{}).get('sysdescr', NOT_DEFINED)

			#
			# Linked element data
			#

			#%L     <linked_node>:<linked_iface>        -> ${linked}
			if key == 'linked_node' : return alarm.get('linked',{}).get('node', NOT_DEFINED)
			if key == 'linked_iface': return alarm.get('linked',{}).get('iface', NOT_DEFINED)

			#
			# Dati del datagroup
			#

			# Datagroup parameters
			if alarm.has_key('params'):
				for p, v in alarm['params'].items():
					if key == ('$%s' % (p)): return v
			
			#
			# Dati della condition
			#

			if key == 'status'    : return CSTATE_TO_CODENAME(alarm['result']['status'])
			if key == 'laststatus': return CSTATE_TO_CODENAME(alarm['result']['laststatus'])

			#%E     target expr                         -> ${expr}
			if key == 'expr'            : return alarm['condition']['expr']

			#%P     target.priority                     -> ${priority}
			if key == 'priority'        : return alarm['condition']['priority']

			#%T     target.path                         -> ${path}
			if key == 'path'            : return alarm['condition']['path']
			
			#%M     max_tries                           -> ${maxtries}
			if key == 'maxtries'        : return alarm['condition']['maxtries']
			if key == 'statuslastchange': return alarm['condition']['statuslastchange']
			if key == 'classification'  : return alarm['condition']['classification']
			if key == 'verbstatus'      : return alarm['result']['verbstatus']

			#
			# Dati varie
			#

			#%U     RESOURCE URL                        -> ${url}

			#%V     SNMP value                          -> ${snmp_val}
			snmp_values = alarm.get('saved_variables',{})
			if snmp_values is not None:
				snmp_values = snmp_values.get('snmp_variables', {})

			if snmp_values:
				if key == 'snmpval'   : return fix_object_for_jsondump(sorted(snmp_values.items())[0][1])
				if key == 'snmpvalues': return '\n'.join( ['%s = %s' % (oid, fix_object_for_jsondump(val)) for oid,val in sorted(snmp_values.items())] )
			else:
				if key == 'snmpval'   : return NOT_DEFINED
				if key == 'snmpvalues': return NOT_DEFINED

			# %A   	inidirizzi IPv4 separati da spazi
			if key == 'ip4s': return ','.join( alarm.get('info',{}).get('ip4s',[])  )

			#%C     Container Tree GEO                  -> ${geotag}
			if key == 'geotag' : return ','.join([ t for t in alarm.get('tags',[]) if t.startswith('geo:') ])
			if key == 'alltags': return ','.join(alarm.get('tags', []))


			#%t     now (timestamp)                     -> ${now_ts}
			if key == 'now'    : return time.time()

			#%s     now (isostring)                     -> ${now_iso}
			if key == 'now_iso': return timestamp_to_isostr(time.time())

			#
			# Refs
			#
			if key == 'url': return alarm.get('url', NOT_DEFINED)

			#
			# RAW
			#
			#if key == 'rawalarm': return pprint.pformat(alarm)
			if key == 'rawalarm': return json.dumps(fix_object_for_jsondump(alarm))
			#if key == 'alarm'   : return _ppalarm(alarm)
			
			mail_data = alarm.get('mail', None)
			if mail_data is not None:
				if key == 'mail_to'       : return mail_data['to']
				if key == 'mail_subject'  : return mail_data['subject']
				if key == 'mail_body'     : return mail_data['body']
				if key == 'mail_xheaders' : return '\n'.join([ "%s: %s" % (f,v) for f,v in mail_data.get('headers',{}).items()])
			else:
				print "Mail data is empty"
			
		except Exception, e:
			print __file__, type(e), str(e)

		return NOT_DEFINED


	# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
	# DEPRECATO <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
	# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






#"""
#
#.. _tags_builtin:
#
#Builtin
#=========
#
#I simboli definiti da questo tag sono definiti nella seguente tabella.
#
#Poiche' questo e' un tag fornito dal sistema tutti i tag definiti sono accessibili anche tramite degli shortcut: 
#
#
#Dati generici sull'allarme:
#
#	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| TAG                              | Shortcut                  | Descrizione                                                                                                         |
#	+==================================+===========================+=====================================================================================================================+
#	| ${tags['builtin:alarmid']}       | ${alarmid}                | UUID Allarme                                                                                                        |
#	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:alarmts']}       | ${alarmts}                | Alarm creation timestamp                                                                                            |
#	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:alarmdateiso']}  | ${alarmdateiso}           | Alarm creation in ISO format                                                                                        |
#	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:rawalarm']}      | ${rawalarm}               | Testo contenente allarme in formato json                                                                            |
#	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#
#Dati sul tenant:
#
#	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| TAG                              | Shortcut                  | Descrizione                                                                                                         |
#	+==================================+===========================+=====================================================================================================================+
#	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:tenantname']}    | ${tenantname}             | Nome del tenant                                                                                                     |
#	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:tenantid']}      | ${tenantid}               | UUID del tenantl                                                                                                    |
#	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#
#Dati sull'elemento di monitoraggio:
#
#	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| TAG                              | Shortcut                  | Descrizione                                                                                                         |
#	+==================================+===========================+=====================================================================================================================+
#	| ${tags['builtin:kind']}          | ${kind}                   | Tipo di elemento del monitoraggio                                                                                   |
#	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:type']}          | ${type}                   | Tipo di elemento del monitoraggio (numerico)                                                                        |
#	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:nodename']}      | ${nodename}               | Nome del nodo                                                                                                       |
#	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:nodedescr']}     | ${nodedescr}              | Descrizione del nodo                                                                                                |
#	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:ifacename']}     | ${ifacename}              | Nome interfaccia                                                                                                    | 
#	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:storagename']}   | ${storagename}            | Nome storage                                                                                                        |
#	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:servicename']}   | ${servicename}            | Nome servizio                                                                                                       |
#	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:devicename']}    | ${devicename}             | Nome device                                                                                                         |
#	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:sysdescr']}      | ${sysdescr}               | SMNP SysDescr nodo                                                                                                  | 
#	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:linked_node']}   | ${linked_node}            | Nome nodo collegato                                                                                                 |
#	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:linked_iface']}  | ${linked_iface}           | Nome interfaccia collegata                                                                                          | 
#	+----------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	
#Dati sulla condition legata all'allarme:
#	
#	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| TAG                                | Shortcut                  | Descrizione                                                                                                         |
#	+====================================+===========================+=====================================================================================================================+
#	| ${tags['builtin:status']}          | ${status}                 | Stato della condition                                                                                               |
#	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:laststatus']}      | ${laststatus}             | Stato precedente della condition                                                                                    |
#	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:statuslastchange']}| ${statuslastchange}       | Ultima variaizone di stato della condition                                                                          |
#	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:verbstatus']}      | ${verbstatus}             | Verbstatus del check                                                                                                |
#	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:priority']}        | ${priority}               | Priorita' dell'allarme                                                                                              |
#	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:path']}            | ${path}                   | Path della condition                                                                                                |
#	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:maxtries']}        | ${maxtries}               | Max tries della condition                                                                                           |
#	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:classification']}  | ${classification}         | Classificazione della condition                                                                                     |
#	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:snmpval']}         | ${snmpval}                | Valore variabile SNMP letta dalla condition                                                                         |
#	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:snmpvalues']}      | ${snmpvalues}             | Valori SNMP letti per effettuare il controllo                                                                       |
#	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:ip4s']}            | ${ip4s}                   | Indirizzi IPv4 del nodo separati da ','                                                                             |
#	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:geotag']}          | ${geotag}                 | Tag del tagtree "geo"                                                                                               |
#	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:alltags']}         | ${alltags}                | Tutti i tag associati all'elemento di monitoraggio (separati da ',')                                                |
#	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:now']}             | ${now}                    | Timestamp orario elaborazione dell'allarme                                                                          |
#	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:now_iso']}         | ${now_iso}                | Orario elaborazione allarme in formato ISO                                                                          |
#	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:url']}             | ${url}                    | URL della risorsa monitorata                                                                                        |
#	+------------------------------------+---------------------------+---------------------------------------------------------------------------------------------------------------------+
#
#Parametri email associati alla condition:
#
#	+------------------------------------+-----------------+---------------------------------------------------------------------------------------------------------------------+
#	| TAG                                | Shortcut        | Descrizione                                                                                                         |
#	+====================================+=================+=====================================================================================================================+
#	| ${tags['builtin:mail_to']}         | ${mail_to}      | email, upemail                                                                                                      |
#	+------------------------------------+-----------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:mail_subject']}    | ${mail_subject} | msg-upsubj, msg-downsubj                                                                                            |
#	+------------------------------------+-----------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:mail_body']}       | ${mail_body}    | msg-upbody, msg-downbody                                                                                            |
#	+------------------------------------+-----------------+---------------------------------------------------------------------------------------------------------------------+
#	| ${tags['builtin:mail_headers']}    | ${mail_headers} |                                                                                                                     |
#	+------------------------------------+-----------------+---------------------------------------------------------------------------------------------------------------------+
#
#Esempio: 
#
#::
#	
#	entables -A DEFAULT -j SYSLOG --syslog_msg "Allarme in arrivo ID ${alarmid} generato in data ${alarmdateiso}"
#
#
#"""
