# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
Notes
=========

I simboli definiti da questo tag sono i seguenti: 

	+-------------------------+-----------------------------------------------------------------------------------------------------------------+
	| Tag                     | Descrizione                                                                                                     |
	+=========================+=================================================================================================================+
	| {{notes.totalcount}}    | Numero totale di note associate al nodo (e tutti i suoi sotto elementi) relativo all'allarme                    |
	+-------------------------+-----------------------------------------------------------------------------------------------------------------+
	| {{notes.totallist}}     | Elenco iterabile di tutte le note associate al nodo (e sotto elementi). Vedi :ref:`expand-notes-in-templates`.  |
	+-------------------------+-----------------------------------------------------------------------------------------------------------------+
	| {{notes.totaltext}}     | Elenco di tutte le note associate al nodo (e sotto elementi) in formato testuale.                               |
	+-------------------------+-----------------------------------------------------------------------------------------------------------------+

Il formato del {{notes.totaltext}} ( o {{notes.totaltext}} prevedere righe multiple ( una per nota ) con la seguente sintassi:

::
	
	- <data> - <utente - <testo nota>

Esempio:

::
	
	Ci sono {{notes.totalcount}}  note associate all'elemento di questo allarme.



.. _expand-notes-in-templates:

Produrre un dettaglio personalizzato delle note
------------------------------------------------

E' possibile usare il simbolo {{notes.totallist}} o {{notes.totallist}} all'interno dei template per formattare in maniera personalizzata le note.

Per iterare sulle note bisogna usare la seguente sintassi:

::
	
	{% for note in notes.totallist %}
		{{ note.<campo> }}
	{% endfo %}

..
	**IMPORTANTE**: per i dettagli sulla sintassi utilizzabile e' necessario fare riferimento alla documentaione del progetto Mako ( http://www.makotemplates.org/ ).


I campi delle note disponibili sono:

	* user_name: username dell'utente associato alla nota
	* isodate  : Data della nota in formato ISO (YYYY-MM-GG HH:MM)
	* text     : testo della nota


Esempio
~~~~~~~~~~~~~~~~~~~~~~~

Testo del messaggio:

::
	
	Ci sono {{notes.totalcount}}  note. Questo e' l'elenco delle note associato all'allarme:
	
	{% for note in notes.totallist %}
		* {{ note.isodate }} - {{ note.user_name }} - {{ note.text }}
	{% endfor  %}

Output visibile nel messaggio:

::
	
	Questo e' l'elenco delle note associato all'allarme:
		
		* 2015-04-10 11:33 - admin - Questa e' una nota
		* 2015-04-10 11:58 - admin - nota su interfaccia
		



Sintassi deprecata
---------------------

.. danger:: La seguente sintassi e' disponibile per retro-compatibilita' e verra' rimossa. Si consiglia di aggiornare i template utilizzati e utilizzare la sintassi descritta nel paragrafo precedente.


Questi sono i tag disponibili:

	+--------------------------------+---------------------+-----------------------------------------------------------------------------------------------------------------+
	| Tag                            | Shortcut            | Descrizione                                                                                                     |
	+================================+=====================+=================================================================================================================+
	| ${tags['notes:totalcount']}    | ${notestotalcount}  | Numero totale di note associate al nodo (e tutti i suoi sotto elementi) relativo all'allarme                    |
	+--------------------------------+---------------------+-----------------------------------------------------------------------------------------------------------------+
	| ${tags['notes:totallist']}     | ${notestotallist}   | Elenco iterabile di tutte le note associate al nodo (e sotto elementi). Vedi :ref:`expand-notes-in-templates`.  |
	+--------------------------------+---------------------+-----------------------------------------------------------------------------------------------------------------+
	| ${tags['notes:totaltext']}     | ${notestotaltext}   | Elenco di tutte le note associate al nodo (e sotto elementi) in formato testuale.                               |
	+--------------------------------+---------------------+-----------------------------------------------------------------------------------------------------------------+

Per iterare sulle note bisogna usare la seguente sintassi:

::
	
	% for note in notestotallist:
		${ note.<campo> }
	% endfor
	
Esempio
~~~~~~~~~~~~~~~~~~~~~~~

Testo del messaggio:

::
	
	Ci sono ${notestotalcount}  note. Questo e' l'elenco delle note associato all'allarme:
	
	% for note in notestotallist:
		* ${ note.isodate } - ${ note.user_name } - ${ note.text }
	% endfor	

"""

from core.models import MonitoredElement
from core.api.base import db_set_current_tenant_schema

# Required in order to load monkey-patches from sub-app "notes"
from notes.models import *

from entables.tagproviders import BaseTagProvider

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class TagProvider(BaseTagProvider):

	def __init__(self):
		super(TagProvider, self).__init__()
		self.name = "notes"

	def get_provider_symbol(self, alarm, log=None):

		class _SymbolExpander(object):

			TOTALTEXT_TEPLATE = """{% for note in allnotes %}- {{ note.isodate }} - {{ note.user_name }} - {{ note.text }}
{% endfor %}
"""

			def __init__(self, alarm, element):
				
				#print __file__, "INIT"
				self.alarm   = alarm
				self.element = element

			def totalcount(self):

				#print __file__, "TOTALCOUNT"

				t = Tenant.objects.get(uuid=self.element.tenant_id)
				t.db_set_current_schema()
				
				count = self.element.get_node().allnotes().count()
				
				return count

			def totallist(self):

				#print __file__, "TOTALLIST"

				t = Tenant.objects.get(uuid=self.element.tenant_id)
				t.db_set_current_schema()
				
				notes = list(self.element.get_node().allnotes())
				
				return notes

			def totaltext(self):

				#print __file__, "TOTALTEXT"

				try:
					t = Tenant.objects.get(uuid=self.element.tenant_id)
					t.db_set_current_schema()

					from django.template import Template, Context

					s = _SymbolExpander.TOTALTEXT_TEPLATE

					t = Template(s)
					c = { 'allnotes': list(self.element.get_node().allnotes().all()) }
					text = t.render(Context(c))

					return text
				except Exception, ex:
					return "*** error expanding digest text: %s:%s ***" % (type(e), str(e))


		e_uuid = alarm.get('element', {}).get('uuid', None)
		if e_uuid is None:
			return {}

		try:

			t = Tenant.objects.get(uuid=alarm['tenantid'])
			t.db_set_current_schema()

			me = MonitoredElement.objects.get(tenant=t, uuid=alarm['element']['uuid'])

			return _SymbolExpander(alarm, me)

		except Tenant.DoesNotExist, e:
			print __file__, e
			return {}

		except MonitoredElement.DoesNotExist, e:
			print __file__, e
			return {}





	# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	# DEPRECATO >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

	TOTALTEXT_TEPLATE = """% for note in allnotes:
- ${ note.isodate } - ${ note.user_name } - ${ note.text }
% endfor

"""

	VALID_SYMBOLS = [
		'totalcount',
		'totallist',
		'totaltext',
	]

	VALID_SHORTCUTS = [
		'notestotalcount',
		'notestotallist',
		'notestotaltext',
	]


	def get_symbol(self, alarm, symbol, version=1):
		return self.__get(alarm, symbol, version=version)

	def get_shortcut_symbol(self, alarm, symbol, version=1):
		return self.__get(alarm, symbol, version=version)

	def __get(self, alarm, symbol, version=1):

		try:
			
			e_uuid = alarm.get('element', {}).get('uuid', None)
			if e_uuid is None:
				return None

			t = Tenant.objects.get(uuid=alarm['tenantid'])
			db_set_current_tenant_schema(t.name)
			e = MonitoredElement.objects.get(uuid=alarm['element']['uuid'])
			
			e = e.get_node()

			if symbol in ['allnotes']: 
				return list(e.allnotes())

			if symbol in ['totalcount', 'notestotalcount']: 
				return str(e.allnotes().count())

			if symbol in ['totallist', 'notestotallist']: 
				return list(e.allnotes().all())

			if symbol in ['totaltext', 'notestotaltext']: 

				try:
					allnotes = list(e.allnotes().all())
					
					from mako.template import Template
					mytemplate = Template(self.TOTALTEXT_TEPLATE)
					text = mytemplate.render(allnotes=allnotes)
					return text
				except Exception, e:
					return "*** error expanding totaltext: %s:%s ***" % (type(e), str(e))

		except Tenant.DoesNotExist, e:
			pass
		except MonitoredElement.DoesNotExist, e:
			pass

		return None


	# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
	# DEPRECATO <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
	# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<








