# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
System
=========

I simboli definiti da questo tag sono i seguenti: 

	+---------------------------+----------------+-------------------------------------------------------------------------------+
	| Tag                       | Shortcut       | Descrizione                                                                   |
	+===========================+================+===============================================================================+
	| {{system.hostname}}       | {{hostname}}   | (Full Qualified) Hostname della macchina su cui e' in esecuzione Sanet.       |
	+---------------------------+----------------+-------------------------------------------------------------------------------+
	| {{system.now}}            | {{now}}        | Timestamp del momento di elaborazione dell'allarme.                           |
	+---------------------------+----------------+-------------------------------------------------------------------------------+
	| {{system.now_iso}}        | {{now_iso}}    | MOmento di elaborazione dell'allarme  in formato ISO (YYYY-DD-MM HH:MM:SS).   |
	+---------------------------+----------------+-------------------------------------------------------------------------------+

Esempio:

::
	
	Questo e' un messaggio da parte di SANET installato su {{hostname}}.
	
	Questo e' un messaggio da parte di SANET installato su {{system.hostname}}.


Sintassi deprecata
---------------------

.. danger:: La seguente sintassi e' disponibile per retro-compatibilita' e verra' rimossa. Si consiglia di aggiornare i template utilizzati e utilizzare la sintassi descritta nel paragrafo precedente.


I tag:
	
	+---------------------------+----------------+-------------------------------------------------------------------------------+
	| Tag                       | Shortcut       | Descrizione                                                                   |
	+===========================+================+===============================================================================+
	| $tags['system:hostname']  | ${hostname}    | (Full Qualified) Hostname della macchina su cui e' in esecuzione Sanet.       |
	+---------------------------+----------------+-------------------------------------------------------------------------------+
	
Esempio:
	
::
	
	Questo e' un messaggio da parte di SANET installato su ${hostname}
	
	Questo e' un messaggio da parte di SANET installato su $tags['system:hostname']

"""

import time
import socket

from sanet_common.timeutils import timestamp_to_isostr

from entables.tagproviders import BaseTagProvider

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class TagProvider(BaseTagProvider):

	def __init__(self):
		super(TagProvider, self).__init__()
		self.name = "system"

	def get_provider_symbol(self, alarm, log=None):

		d = {
			'hostname': socket.getfqdn(),
			'now'     : time.time(),
			'now_iso' : timestamp_to_isostr(time.time()),
		}

		return d


	# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	# DEPRECATO >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
	# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

	VALID_SYMBOLS = [
		'hostname'
	]

	VALID_SHORTCUTS = [
		'hostname'
	]

	def get_symbol(self, alarm, symbol):
		return self.__get(alarm, symbol)

	def get_shortcut_symbol(self, alarm, symbol):
		return self.__get(alarm, symbol)

	def __get(self, alarm, symbol):

		if symbol == 'hostname': 
			return socket.getfqdn()

		if symbol == 'now': 
			return time.time()

		if key == 'now_iso': 
			return time.time()

		return None

	# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
	# DEPRECATO <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
	# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<






