# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
SYSLOG
==========

Questo target permette loggare su syslog.

I parametri questo target vanno specificati secondo la sintassi: ::

	-J SYSLOG --syslog_<parametro> <valore>

Parametri:

	+---------------+---------------+-------------------------------+--------------------------------------------------------------------------+
	| Parametro     | Type          | Default                       | Descrizione                                                              |
	+===============+===============+===============================+==========================================================================+
	| msg           | stringa       | ${rawalarm}                   | Testo del messaggio syslog                                               |
	+---------------+---------------+-------------------------------+--------------------------------------------------------------------------+

Esempi
------

Produce un messaggio::

	entables -A DEFAULT -j SYSLOG --syslog_msg "[entables] Processed alarm ${alarmid}"

"""

import syslog

from entables.constants import TABLE_NOTIFY

from entables.constants import ROUTING_OUTCOME_CONTINUE

from entables.targets import BaseTargetHandler
from entables.targets import Option


class Target(BaseTargetHandler):

	def __init__(self):
		super(Target, self).__init__()

		self.name = 'syslog'

		self.tables = [
			TABLE_NOTIFY,
		]

		self.options    = [
			Option("msg", default="${rawalarm}"),
		]

	def init(self):
		return
		
	def validate_target_options(self, options_dict):
		
		super(Target, self).validate_target_options(options_dict)
		
		return True


	def dispatch(self, context, alarm, ruleobj, target_options):
		
		options = self.expand_options(context, alarm, target_options)

		context.log.debug("SYSLOG DISPATCH")
		
		alarmdata = alarm.data

		try:

			msg = options['msg']
			
			syslog.syslog(syslog.LOG_USER, msg)

		except Exception, e:
			context.log.error("SYSLOG error: %s", e)

		return ROUTING_OUTCOME_CONTINUE	

