# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

.. _entables_target_updatecounters:

UPDATECOUNTERS
=======================

Memorizza informazioni sull'allarme che viene processato dalla regola (e della regola associata all'allarme).

Queste informazioni possono essere utilizzate dal match :ref:`entables_match_counters`.

Utilizzo:

	::
		
		-J UPDATECOUNTERS

.. warning:: le informazioni vengono associate alla condition dell'allarme, identificata dal PATH. Questo significa che se condition viene rinominata, le informazioni gia' memorizzate vengon perse.


Informazioni/contatori aggiornati
-----------------------------------

Queste sono le informazioni aggiornate dal target:

	* stato dell'ultimo allarme intercettato


Esempi
------

Esempio regola che aggiorna i contatori e poi verifica se lo stato dell'ultimo allarme "passato' e' UP

::
	
	...
	...
	... regole di JUMP o DROP o ecc. ...
	...
	entables -t notify -A DEFAULT                                        -J UPDATECOUNTERS 
	...
	...
	... regole di JUMP o DROP o ecc. ...
	...
	...
	entabels -t notify -A DEFAULT    -m counters --counters_status UP    -j SMTP

"""

import time
import datetime
import json

from core.models import Tenant

from entables.constants import ROUTING_OUTCOME_CONTINUE
from entables.constants import TABLE_NOTIFY

from entables.models import TargetData
from entables.targets import BaseTargetHandler

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class Target(BaseTargetHandler):

	def __init__(self):
		super(Target, self).__init__()

		self.name = "updatecounters"

		self.tables = [
			TABLE_NOTIFY
		]

		self.options = [
		]

	def init(self):
		return

	def validate_target_options(self, options_dict):

		super(Target, self).validate_target_options(options_dict)

		return True

	def dispatch(self, context, alarm, ruleobj, target_options):

		tenant = Tenant.objects.get(uuid=alarm.tenant_uuid)
		tenant.db_set_current_schema()

		options = self.expand_options(context, alarm, target_options)

		log = context.log

		log.debug("%s DISPATCH", self.name.upper())

		alarm.set_verbstatus("Target %s: %s" % ( self.name.upper(), options))

		alarmdata = alarm.data

		try:
			condition_path = alarmdata.get('condition',{}).get('path',None)
			condition_uuid = alarmdata.get('condition',{}).get('uuid',None)

			status = alarmdata.get('result',{}).get('status',None)

			#
			# LOAD
			#
			key = 'entables.COUNTERS.%s.data' % (condition_path)

			target_data, created = TargetData.objects.get_or_create(tenant=tenant, key=key)
			
			info_data = target_data.get_data()
			if info_data is None:
				info_data = {}
			log.info("CURRENT DATA: %s", info_data)

			#
			# UPDATE
			#

			info_data['status'] = status

			#
			# SAVE
			#
			target_data.set_data(info_data)
			target_data.save()

			return ROUTING_OUTCOME_CONTINUE

		except Exception, e:
			log.debug("%s", str(e))

			alarm.set_verbstatus("Unable to execute target %s: %s:%s" % ( self.name, type(e), str(e)))

			return ROUTING_OUTCOME_CONTINUE
