# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os
import StringIO
import traceback

from entables.tagproviders import BaseTagProvider

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#


#BUILTIN_SHORTCUTS_TAGS_PROVIDERS = [       DEPRECATO
#	"builtin", 
#	"system",
#	"digest", 
#]

TAG_PROVIDERS_ORDER = [ 
	"builtin", 
	"system",
	"digest", 
]

TEMPLATE_PREFIX = "#template:"

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class TemplateException(Exception):
	pass

class TemplateSyntaxException(Exception):
	pass

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#


# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
# DEPRECATO >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

from mako.template import Template as MakoTemplate
from mako.template import Template as MakoTemplate
# NOTE: This is an hack. Ignore missing symbols when rendering templates! 
#       Check out for better solutions.
import mako.runtime
from mako.runtime import Context as MakoContext

mako.runtime.UNDEFINED = '**none**'


class Tagtemplate(MakoTemplate):
	"""
	Wrapper per i template della libreria "mako"
	"""
	pass

class SafeTagsDict2(dict):
	"""
	Dizionario modificato per non dare errore in caso di acceda
	ad una chiave inesistente.
	
	
	if key in tags:
	
		${tags[<key>]}   -> <val>
	else:
		${tags[<key>]}   -> *stringa vuota*
	"""
	
	def __init__(self, alarm, tag_providers):
		super(SafeTagsDict2, self).__init__()
		
		self.alarm = alarm
		self.tag_providers = tag_providers

	def get(self, *params):  
		rv = super(SafeTagsDict2, self).get(*params)
		return rv

	def __contains__(self, key):
		return True
		
	def __getitem__(self, key):

		try:
			#print self.__class__, "__getitem__", str(key)

			#
			# FIX: prevedere la possibilita di avere dei simboli di sistema 
			#      espansi automaticamente ogni volta che si accede ad uno di essi
			#
			if super(SafeTagsDict2, self).__contains__(key):
				return super(SafeTagsDict2, self).__getitem__(key)

			#print "\t", "NOT FOUND"

			# Stiamo cercando un simbolo con "provider:symbol"?
			if key.find(":") > -1:
				provider_name, symbol = key.split(":",1)

				for pname, provider in self.tag_providers.items():
					#print __file__, "PROVIDER?", pname, symbol, provider.has_symbol(self.alarm, symbol)
					if provider.has_symbol(self.alarm, symbol):
						v = provider.get_symbol(self.alarm, symbol)

						self[key] = v

						return v

			#print __file__, self.__class__, "__getitem__", "NOT FOUND!", key

			# Safe fallback per non far andare in crash la libreria di rendering del template
			# se il dizionario non contiene la chiave	
		except TypeError, e:
			#print __file__, self.__class__, "__getitem__", "ERROR", type(e), e
			pass
		except Exception, e:
			#print __file__, self.__class__, "__getitem__", "ERROR",  type(e), e
			pass

		return "** entables tag '%s' not defined **" % (key)
		#return None

class EntablesTemplateContext(MakoContext):

	def __init__(self, alarm, tag_providers, *args, **kwargs):
		super(EntablesTemplateContext, self).__init__(*args, **kwargs)

		self.alarm         = alarm
		self.tag_providers = tag_providers

	def __getitem__(self, key):
		return super(EntablesTemplateContext, self).__getitem__(key)

	def get(self, key, default=None):
		
		if self._data.has_key(key):
			return super(EntablesTemplateContext, self).get(key, default=default)
		
		for name, provider in self.tag_providers.items():

			if provider.has_shortcut_symbol(self.alarm, key):
				return provider.get_shortcut_symbol(self.alarm, key)
		
		return super(EntablesTemplateContext, self).get(key, default=default)

def __expand_string_with_template2(log, alarm, s, templates, registered_tagproviders):

	tags    = SafeTagsDict2(alarm, registered_tagproviders)

	#
	# Rendering
	#
	output_buffer = StringIO.StringIO()

	context = EntablesTemplateContext(alarm, registered_tagproviders,   output_buffer, **{ 'tags':tags })

	template = Tagtemplate(s)
	template.render_context(context)
	text = output_buffer.getvalue()
	
	output_buffer.close()

	return text

def validate_template2(text):

	try:
		text = unicode(text)
		template = Tagtemplate(text)
		template.render()
	except Exception, e:
		raise TemplateSyntaxException("Template non valido (2): %s" % (str(e)))
	return True

# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
# DEPRECATO <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<
# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<











class SafeTagsDict3(dict):
	"""
	Dizionario modificato per non dare errore in caso di acceda
	ad una chiave inesistente.
	
	
	if key in tags:
	
		${tags[<key>]}   -> <val>
	else:
		${tags[<key>]}   -> *stringa vuota*
	"""

	def __init__(self, log, alarm, tag_providers):
		super(SafeTagsDict3, self).__init__()

		self.log = log

		self.alarm = alarm

		self.tag_providers_dict = {}

		#
		# Generate tags
		#
		for name, tp in tag_providers.items():
			self.tag_providers_dict[name] = tp.get_provider_symbol(alarm, log=self.log)

	def get(self, *params):  
		rv = super(SafeTagsDict3, self).get(*params)
		return rv

	def __len__(self):
		return 1

	def __contains__(self, key):
		return True

	def __getitem__(self, key):

		try:
			if self.tag_providers_dict.has_key(key):
				return self.tag_providers_dict[key]

			others = set(self.tag_providers_dict.keys()) - set(TAG_PROVIDERS_ORDER) 
			provider_names  = TAG_PROVIDERS_ORDER + list(others)

			for provider_name in provider_names:

				obj = self.tag_providers_dict[provider_name]

				self.log.log(1, "tag proivder: %s -> %s", provider_name, obj)

				if hasattr(obj, key):
					return getattr(obj, key)

				if hasattr(obj, 'has_key') and obj.has_key(key):
					return obj[key]

		except TypeError, e:
			self.log.error("SafeTagsDict3.__getitem__ error %s %s", type(e), str(e))

		return "** template symbol '%s' not defined by any provider **" % (key)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

from django.template import Template, Context
from django.template.base import Variable
from django.template.base import VariableDoesNotExist

_ORIGINAL_DJANGO_VARIABLE_RESOLVE_LOOKUP = Variable._resolve_lookup

def __expand_string_with_template3(log, alarm, s, templates, registered_tagproviders):

	# Disable HTML escape. It's not a web page.
	s = '{% autoescape off %}' + s + '{% endautoescape %}'

	#
	#
	#
	_resolve_lookup = Variable._resolve_lookup

	def _entables_new_resolve_lookup(self, *args, **kwargs):
		try:
			o = _ORIGINAL_DJANGO_VARIABLE_RESOLVE_LOOKUP(self, *args, **kwargs)
			if o is not None:
				return o
			return BaseTagProvider.SYMBOL_NULL_VALUE
		except VariableDoesNotExist, e:
			return BaseTagProvider.SYMBOL_NOTDEFINED
		except Exception, e:
			return BaseTagProvider.SYMBOL_ERROR % (str(e))

	Variable._resolve_lookup = _entables_new_resolve_lookup

	#
	t = Template(s)
	c = SafeTagsDict3(log, alarm, registered_tagproviders)
	text = t.render(Context(c))

	return text


def validate_template3(text):
	try:
		from django.template import Template, Context
		t = Template(text)
		t.render(Context({}))
	except Exception, e:
		raise TemplateSyntaxException("Template invalid. Error: %s" % (str(e)))
	return True

def validate_template(text):

	#validate_template2(text)
	validate_template3(text)

	return True

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def calculate_options_value(log, alarm_data, defined_options, options_defaults, registered_tagproviders, templates):

	options = {}
	options.update(options_defaults)
	options.update(defined_options)

	for option_name, option_value in options.items():

		if option_value == None:
			option_value = ""

		log.debug("Option: %s : %s", option_name, option_value)

		option_value = str(option_value)
		text = option_value

		if text.startswith(TEMPLATE_PREFIX):
			template_name = text[len(TEMPLATE_PREFIX):]
			if not templates.has_key(template_name):
				raise Exception("Template %s not found" % (template_name))
			text = templates[template_name]

		#
		# Django templates {{ .... }}
		#
		try:
			text = __expand_string_with_template3(log, alarm_data, text, templates, registered_tagproviders)
		except Exception, e:
			log.error("ERROR WHILE PROCESSING OPTION VALUE 3 '%s'= %s:%s -> error %s:%s." % (option_name, type(option_value), option_value, type(e), str(e) ))
			log.debug("Fulltrace: %s" % (traceback.format_exc()))

			text += '\nErrors while expading wildcards (3)'

		#
		# DEPRACATED: Mako templates {% %}
		#
		try:
			text = __expand_string_with_template2(log, alarm_data, text, templates, registered_tagproviders)
		except Exception, e:
			log.error("ERROR WHILE PROCESSING OPTION VALUE 2 '%s'= %s:%s -> error %s:%s." % (option_name, type(option_value), option_value, type(e), str(e) ))
			log.debug("Fulltrace: %s" % (traceback.format_exc()))

			text += '\nErrors while expading wildcards (2)'

		log.debug("--> %s", text)
		options[ option_name ] = text

	return options

#def __expand_string_with_template(log, s, templates, tags, tags_shortcuts):
#	
#	ret = s
#	
#	if s.startswith(TEMPLATE_PREFIX):
#		
#		template_name = s[len(TEMPLATE_PREFIX):]
#		if not templates.has_key(template_name):
#			raise Exception("Template %s not found" % (template_name))
#			
#		s = templates[template_name]
#	
#	template = Tagtemplate(s)
#	
#	return template.render(tags=tags, **tags_shortcuts)

#def __compile_template_tags(providerlist, alarm):
#	
#	compiledtags = SafeTagsDict()
#	shortcuts = {}
#	
#	for provider_name, provider in providerlist.items(): 
#		
#		#
#		# Espansione normale dei tags
#		#
#		tags = provider.gettags(alarm)
#		
#		for tag_name, tag_val in tags.items():
#			complete_tag_key = "%s:%s" % ( provider.get_name() , tag_name )            
#			compiledtags[complete_tag_key] = tag_val
#		
#		#
#		# SHORTCUTS SOLO PER ALCUNI TAG PROVIDERS
#		#
#		if provider_name in BUILTIN_SHORTCUTS_TAGS_PROVIDERS  :
#			shortcuts.update( tags )
#	
#	return compiledtags, shortcuts

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

#class SafeTagsDict(dict):
#	"""
#	Dizionario modificato per non dare errore in caso di acceda
#	ad una chiave inesistente.
#	
#	
#	if key in tags:
#	
#		${tags[<key>]}   -> <val>
#	else:
#		${tags[<key>]}   -> *stringa vuota*
#	"""
#	
#	def __init__(self):
#		super(SafeTagsDict, self).__init__()
#
#	def get(self, *params):  
#		rv = super(SafeTagsDict, self).get(*params)
#		return rv
#
#	def __contains__(self, key):
#		return True
#
#	def __getitem__(self, key):
#
#		#
#		# FIX: prevedere la possibilita di avere dei simboli di sistema 
#		#      espansi automaticamente ogni volta che si accede ad uno di essi
#		#
#		#if key == 'now':
#		#	return str(time.time())
#		
#		if super(SafeTagsDict, self).__contains__(key):
#			return super(SafeTagsDict, self).__getitem__(key)
#		
#		# Safe fallback per non far andare in crash la libreria di rendering del template
#		# se il dizionario non contiene la chiave	
#		return "*** NOT FOUND ***"
		
#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

#def get_all_tags(providerlist):
#
#	alltags = {}
#
#	for provider_name, provider in providerlist.items(): 
#
#		partialtags = {}
#
#		tags = provider.gettags(None)
#
#		partialtags.update(tags)
#
#		for tag, value in partialtags.items():
#			alltags.update[ "%s:%s" % (provider.get_name(), tag) ] = value
#
#	return alltags

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

#import pprint
#
#def calculate_options_value(log, alarm, defined_options, options_defaults, registered_tagproviders, templates):
#
#	log.debug("Calculating tags' value...")
#	tags, tags_shortcuts = __compile_template_tags(registered_tagproviders, alarm.data)
#
#	log.log(9, "Tags     : %s", pprint.pformat(tags))
#	log.log(9, "Shortcuts: %s", pprint.pformat(tags_shortcuts))
#	
#	
#	options = {}
#	options.update(options_defaults)
#	options.update(defined_options)
#
#	log.debug("Expanding options:")
#	
#	for option_name, option_value in options.items():
#
#		if option_value == None:
#			option_value = ""
#
#		log.log(9, "\t%s = %s", option_name, option_value)
#
#		try:
#			option_value = str(option_value)
#
#			options[ option_name ] = __expand_string_with_template(log, option_value, templates, tags, tags_shortcuts)
#
#		except Exception, e:
#			log.error("ERROR WHILE PROCESSING OPTION VALUE '%s'= %s:%s :%s" % (option_name, type(option_value), option_value, e))
#			
#			raise e
#
#	log.debug("Expanded options:")
#	for o,v in sorted(options.items()):
#		log.debug("\t%s = %s...", o, v[0:30])
#
#	return options
