# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

# Cli per la configurazione del flusso
# imposta sopratutto catene, match e target
# queue show

import logging
import StringIO
import argparse
import sys
import tempfile
import subprocess
import os
import pprint

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

from django.conf import settings 

from core.api.base import tenant_get, tenant_get_primary

from entables.datastructures import *
from entables.tablesstore.djangostore import * #as db
from entables.modulesmanager import *

from entables.templatemanager import TemplateSyntaxException

from entables.ui.format import dumpprint
from entables.ui.format import dump_chain_rules
from entables.ui.format import dump_chain_policy_options

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

DIR = os.path.dirname(os.path.abspath(__file__)) + "/../"

#COMMAND_FIFO_PATH = "/tmp/entablesd-commandqueue"

EDITOR = "sensible-editor"

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#


def debug(string):
    #if settings.DEBUG:
    #    print string
    pass

def error(string):
	sys.stderr.write(string+"\n")


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

line_pos = 0

def load_configuration(input_stream):

	#
	#  tenant <name>
	#
	#    template <name>
	#    *body*
	#    .
	#
	#    rule ........
	#    rule ........
	#    rule ........
	#    rule ........
	#    rule ........
	#
	#  exit
	#
	#  tenant <name>
	#    ...
	#  exit

	global line_pos

	line_pos = -1
	while 1:
		if input_stream.closed:
			return

		line = input_stream.readline()
		line_pos += 1

		if not line:
			return

		line = line.strip()
		if len(line) == 0:
			continue
		if line.startswith("#"):
			continue

		if line.startswith('tenant'):
			try:
				tenant_name = line.split(' ',1)[1].strip()

				tenant = tenant_get(tenant_name)

				if tenant != None:
					load_tenant_conf(tenant, input_stream)
				else:
					print "Errore line %s: Invalid tenant name" % (line_pos)
			except Exception, e:
				print "Error while processing tenant: %s" % (e)
		else:
			print "Error line %s: invalid command: %s"  % (line_pos, line)

def load_tenant_conf(tenant, input_stream):

	global line_pos

	debug(tenant.name)

	while 1:

		line = input_stream.readline()
		line_pos += 1
		if not line:
			input_stream.close()
			break

		line = line.strip()
		if len(line) == 0:
			continue

		if line.startswith("#"):
			continue

		if line == 'exit':
			break

		elif line.startswith('template'):

			template_name = line.split(' ',1)[1].strip()

			read_and_save_template(tenant, template_name, input_stream)

		elif line.startswith('rule'):

			rule_line = line.split(' ',1)[1]

			argv = parse_rule_line(tenant, rule_line)

			process_rule_args(argv)
		else:
			print "Error line %s: Invalid command" % (line_pos, line)



def parse_rule_line(tenant, rule_line):
	
	global line_pos
	
	#
	# FIX: converrebbe usare le stesse funzioni di parsing che gia' usiamo nella CLI?
	#
	
	import shlex

	argv = []

	lexer = shlex.shlex(rule_line)
	lexer.wordchars += '-'
	
	for tok in lexer:
		# Togliamo gli apici/doppi apici dai parametri perche' il lexer non lo fa
		if tok[0] == tok[-1] in ['"', "'"]:
			tok = tok[1:-1]
		argv.append(tok)

	return argv



def read_and_save_template(tenant, template_name, input_stream):


	db_set_current_tenant_schema(tenant.name)
	
	template, created = Template.objects.get_or_create(tenant=tenant, name=template_name)

	data = __read_dot_terminated_text(input_stream)

	template.data = data

	template.save()
	
	"""
	try:

		if find_template(tenant, template_name):
			new = False
			templateobj = read_template(tenant, template_name)
		else:
			new = True
			templateobj = TemplateMemoryObject(template_name, "")

		
		
		templateobj.data = data

		if new:
			add_template(tenant, templateobj)
		else:
			change_template(tenant, template_name,  templateobj)

		print "saved %s" % (template_name)

	except Exception, e:
		print e
	"""

def __read_dot_terminated_text(input_stream):
	
	#
	# Inverse of dot_stuff
	#
	buf=''

	if input_stream.isatty():
		sys.stdout.write("Enter lines of input. Terminate with a single dot on a line by itself.\n")
		sys.stdout.write("To insert a line with a leading dot, use two dots.\n")

	while True:
		if input_stream.isatty():
			sys.stdout.write("(dot-terminated text)#")

		line = input_stream.readline()
		if line == ".\n":
			return buf

		if line[0:2] == '..':
			line = line[1:]

		#
		# FIX: qsingle?
		#	
		#add = qsingle(line)
		add = line

		if len(add)>1 and add[-2:] == '\n\n':
			add = add[:-1]

		buf = buf + add

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

registered_matches      = None
registered_targets      = None 
registered_tagproviders = None

def init_log():

	log = logging.getLogger("entablesd")

	level = 1
	formatter = logging.Formatter('%(message)s')

	log.setLevel( level )

	hdlr = logging.StreamHandler()
	hdlr.setFormatter( formatter )
	hdlr.setLevel(100)
	log.addHandler(hdlr)

	return log

def init_globals():

	global registered_matches 
	global registered_targets 
	global registered_tagproviders

	
	from django.conf import settings 
	
	configuration = settings.ENTABLESD_SETTINGS

	registered_matches = MatchDefinitionList()
	registered_targets = TargetDefinitionList()
	registered_tagproviders = TagproviderDefinitionList()

	log = init_log()

	load_match_modules(log, configuration['matches'], registered_matches)

	load_target_modules(log, configuration['targets'], registered_targets)    

	load_tagprovider_modules(log, configuration['tagproviders'], registered_tagproviders)


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def PARSE_ACTION( action_parser, args):
	try:
		parse = action_parser.parse_known_args(args)
		return parse[0], parse[1]
	except Exception, e:
		print action_parser.print_help()

def parse_rule_matchess_and_target_parameters(valid_chains, action_parser, rem_args):

	action_parser.add_argument('-m','--match'   , dest='match_modules',type=str,action='append')
	action_parser.add_argument('-j','--target'  , dest='target_module',type=str)
	action_parser.add_argument('--stop-if-match', action='store_true', default=False)
	action_parser.add_argument('--disabled'     , action='store_true', default=False)
	
	action_args, rem_args = PARSE_ACTION( action_parser, rem_args)
	
	if action_args.target_module is None:
		raise Exception("Target not specified")

	target_name = action_args.target_module.lower()

	targetobj = TargetMemoryObject()
	targetobj.name = target_name
	
	target_handler = registered_targets.get(target_name, None)
	if target_handler:
		target_parser = argparse.ArgumentParser(description='Target ')

		target_handler.add_args_to_parser(target_parser)

		#print "ARGS: ", rem_args

		parse = target_parser.parse_known_args(rem_args)   
		target_args = parse[0]
		rem_args    = parse[1]

		#print "TARGET_ARGS :", sorted(target_args.__dict__.items())

		# Validate options
		defined_options = target_handler.build_from_parsed_args(target_args.__dict__)

		#print "USER DIFINED:", sorted(defined_options)

		target_handler.validate_target_options(defined_options)

		# Set options
		targetobj.set_options(defined_options)
	else:
		if target_name not in valid_chains:
			raise Exception("Target '%s' is not a valid target or valid chain name" % (target_name))

	disable_status = RULE_STATE_ACTIVE if not action_args.disabled else RULE_STATE_DISABLED

	ruleobj = RuleMemoryObject(targetobj, stop_if_match=action_args.stop_if_match, disabled=disable_status)


	# match parsing
	match_parser = argparse.ArgumentParser(description='Match ')

	if action_args.match_modules != None:

		for match_module_name in set(action_args.match_modules):

			if match_module_name not in registered_matches:
				raise Exception("Invalid match '%s'. Valid modules are: %s." % (match_module_name, ','.join(registered_matches.keys())))

			match_handler = registered_matches[match_module_name]
			match_handler.add_args_to_parser(match_parser)

			parse = match_parser.parse_known_args(rem_args)

			match_args = parse[0]
			rem_args = parse[1]
			
			#print "MATCH ARGS", match_args
			#print "REM   ARGS", rem_args

			# Validate
			defined_options = match_handler.build_from_parsed_args(match_args.__dict__)
			match_handler.validate_match_options(defined_options)

			# Options
			matchobj = MatchMemoryObject() 
			matchobj.name = match_module_name
			matchobj.set_options(defined_options)
			
			#print defined_options

			#matchobj.build_from_args(match_args)
			#match_handler.check_match_arguments(matchobj.options) #FIX: GESTIRE MEGLIO LA MANCATA VALIDAZIONE DEI PARAMETRI. Adesso solleva un'eccezione che ferma lo script.

			ruleobj.add_match(matchobj)

	if target_handler:
		target_handler.check_rule(ruleobj)

	if rem_args:
		raise Exception("Invalid arguments: %s" % (' '.join(rem_args)))

	return action_args, ruleobj


def process_rule_args(argv):

	main_parser = argparse.ArgumentParser(description='Manage EventNotify Tables', prog="entables", add_help=False)

	# l'approccio con i subparser e' utilizzabile solo con python3 visto che subparsers per pyhton2.7 non accetta gli alias.
	#subparsers = parser.add_subparsers(help='sub-command help')
	#action_subparsers = parser.add_subparsers(help='sub-command help')
	#parser_insert = subparsers.add_parser('a', help='a help')

	#main_parser.add_argument("progname")


	# Opzioni discriminatori principali
	main_parser_mutex = main_parser.add_mutually_exclusive_group(required=True)

	main_parser_mutex.add_argument('-L','--list', action='store_true'   )

	main_parser_mutex.add_argument('-I','--insert-rule'  , action='store_true', help="Insert a rule")
	main_parser_mutex.add_argument('-A','--append-rule'  , action='store_true')
	main_parser_mutex.add_argument('-D','--delete-rule'  , action='store_true')
	main_parser_mutex.add_argument('-R','--replace-rule' , action='store_true')

	main_parser_mutex.add_argument('-x','--disable-rule' , action='store_true')
	main_parser_mutex.add_argument('-a','--activate-rule', action='store_true')

	main_parser_mutex.add_argument('-N','--new-chain'    , action='store_true')
	main_parser_mutex.add_argument('-X','--delete-chain' , action='store_true')
	main_parser_mutex.add_argument('-E','--rename-chain' , action='store_true')
	main_parser_mutex.add_argument('-F','--flush-chain'  , action='store_true')
	main_parser_mutex.add_argument('-P','--policy-chain' , action='store_true')

	main_parser_mutex.add_argument('-Z','--zero-chain'   , action='store_true')

	main_parser_mutex.add_argument('--save','--dump'     , action='store_true')

	main_parser_mutex.add_argument('--load'              , action='store_true')


	main_parser_mutex.add_argument('--show-template'     , action='store_true')
	main_parser_mutex.add_argument('--list-template'     , action='store_true')
	main_parser_mutex.add_argument('--edit-template'     , action='store_true')
	main_parser_mutex.add_argument('--load-template'     , action='store_true')
	main_parser_mutex.add_argument('--delete-template'   , action='store_true')

	main_parser_mutex.add_argument('--help'              , action="store_true")
	main_parser_mutex.add_argument('--version'           , action='version', version='%(prog)s 1.0')

	# FIX: questi parametri non sono validi per opzioni come 'help', 'version', 'load'
	main_parser.add_argument('-t','--table', default='notify', help="Table")
	main_parser.add_argument('--tenant', dest="tenantname", default=None)

	# Fake parameter for iptables-like commands -nL
	main_parser.add_argument('-n', action='store_true', default=True, help="Numeric output")


	#
	# Parse command type
	#
	parse = main_parser.parse_known_args(argv)

	main_args = parse[0]
	rem_args = parse[1]


	if main_args.help:
		main_parser.print_help()
		sys.exit(0)

	#
	# Set working tenant
	#
	if main_args.tenantname == None:
		main_args.tenantname = tenant_get_primary().name    
	main_args.tenantobj = tenant_get(main_args.tenantname)
	if main_args.tenantobj == None:
		raise Exception("Invalid tenant name '%s'" % (main_args.tenantname))

	# --save
	if main_args.save:

		if rem_args:
			raise Exception("Invalid options: %s" % (rem_args))
		
		tenant = main_args.tenantobj
		db_set_current_tenant_schema(tenant.name)
		
		dbdumpobj = dbdump()
		output_stream = StringIO.StringIO()
		dumpprint(output_stream, dbdumpobj)
		text = output_stream.getvalue()
		output_stream.close()
		print text
		sys.exit(0)
    
	# --load
	if main_args.load:
		
		if rem_args:
			raise Exception("Invalid options: %s" % (rem_args))
		
		load_configuration(sys.stdin)
		sys.exit(0)
        

	action_parser = argparse.ArgumentParser(description='Manage EventNotify Tables')

	#
	# Templates commands
	#
	if main_args.list_template or \
	   main_args.edit_template or \
	   main_args.delete_template or \
	   main_args.show_template or \
	   main_args.load_template:

		try:

			tenant = main_args.tenantobj
			db_set_current_tenant_schema(tenant.name)

			def PARSE_TEMPLATE_NAME(action_parser, args):
				action_parser.add_argument('templatename')
				parse = action_parser.parse_known_args(args)
				return parse[0], parse[1]

			# --list-template
			if main_args.list_template:

				if rem_args:
					raise Exception("Invalid options: %s" % (rem_args))

				for t in Template.objects.all():
					print t.name

			# --show-template
			if main_args.show_template:

				action_args, rem_args = PARSE_TEMPLATE_NAME(action_parser, rem_args)

				if rem_args:
					raise Exception("Invalid options: %s" % (rem_args))

				template = Template.objects.get(tenant=tenant, name=action_args.templatename)

				print template.data


			# --delete-template
			if main_args.delete_template:

				action_args, rem_args = PARSE_TEMPLATE_NAME(action_parser, rem_args)

				if rem_args:
					raise Exception("Invalid options: %s" % (rem_args))

				template = Template.objects.get(tenant=tenant, name=action_args.templatename)
				template.delete()

			# --load-template
			if main_args.load_template:

				action_args, rem_args = PARSE_TEMPLATE_NAME(action_parser, rem_args)

				if rem_args:
					raise Exception("Invalid options: %s" % (rem_args))

				data = sys.stdin.read()
				
				template, created = Template.objects.get_or_create(tenant=tenant, name=action_args.templatename)
				template.data = data
				template.save()

			# --edit-template
			if main_args.edit_template:

				action_args, rem_args = PARSE_TEMPLATE_NAME(action_parser, rem_args)
				

				if rem_args:
					raise Exception("Invalid options: %s" % (rem_args))

				template, created = Template.objects.get_or_create(tenant=tenant, name=action_args.templatename)
	    
				tmpfile = tempfile.NamedTemporaryFile(prefix='entables-')
				if not created:
					tmpfile.write(template.data)
				else:
					#tmpfile.write("<%doc>\n")
					#tmpfile.write(" template \'" + action_args.templatename + "\'\n")
					#tmpfile.write(" You can edit this template using <tags> \n")
					#tmpfile.write(" For more information pleas read mako template documentation \n")
					#tmpfile.write(" Every <%doc> comment will be delete from the saved template \n")
					#tmpfile.write(" ## comments will be retained instead \n")
					#tmpfile.write("</%doc>\n")
					tmpfile.write("**write you text here**\n")
	    
				tmpfile.flush()

				mtime0 = os.stat(tmpfile.name)[9]
				subprocess.call([EDITOR, tmpfile.name])
				mtime1 = os.stat(tmpfile.name)[9]
				if mtime0 != mtime1:			
					tmpfile.seek(0)
					data = tmpfile.read()
				else:
					data = None
				tmpfile.close()
					
				if data != None:
					template.data = data
					template.save()

		except TemplateSyntaxException, e:
			print e

		sys.exit(0)

	#
	# Rule commands
	#
	ruledb = RuleDB()
	ruledb.load_rules()
	
	action_args = None

	# -L --list
	if main_args.list:
		action_parser.add_argument('chain', action='store'     , nargs='?', default=None,)

		action_args, rem_args = PARSE_ACTION( action_parser, rem_args)

		if rem_args:
			raise Exception("Invalid options: %s" % (rem_args))

		chain_name = action_args.chain.lower() if action_args.chain != None else None

		output_stream = StringIO.StringIO()        
		dbdumpobj = dbdump()

		chains = dbdumpobj[ main_args.tenantobj.name ]['tables'][main_args.table]['chains']

		if chain_name is None:
			chains = sorted(chains.items())
		else:
			if chain_name not in chains:
				raise Exception("Chain %s not found" % (chain_name))

			chains = [ (chain_name, chains[chain_name]) ]

		for name, chain in chains:
			print >>output_stream, name.upper(), ": policy", chain["policy"]["handler_name"].upper(), ' '.join(dump_chain_policy_options(chain))
			dump_chain_rules(output_stream, main_args.table, name, chain, level=2, prefix='%(position)s)')
			print >>output_stream, ""

		text = output_stream.getvalue()
		output_stream.close()

		print text

		sys.exit(0)


	DEFAULT_POLICY_TARGET_HANDLER = 'continue'

	if main_args.new_chain:

		action_parser.add_argument('chain')
		action_parser.add_argument('-j', '--target', dest='target_module', type=str, default=DEFAULT_POLICY_TARGET_HANDLER)

		action_args, rem_args = PARSE_ACTION( action_parser, rem_args)

		chain_name         = action_args.chain.lower()

		target_module_name = action_args.target_module.lower() 

		#
		# Parse target part
		#
		if not registered_targets.has_key(target_module_name):
			raise Exception("Target non valido '%s'" % (target_module_name) )

		target_parser = argparse.ArgumentParser(description='Target')
		target_handler = registered_targets[target_module_name]
		target_handler.add_args_to_parser(target_parser)

		target_args, rem_args = PARSE_ACTION( target_parser, rem_args)

		if rem_args:
			raise Exception("Invalid options: %s" % (rem_args))

		# Validate
		defined_options = target_handler.build_from_parsed_args(target_args.__dict__)
		target_handler.validate_target_options(defined_options)

		targetobj = TargetMemoryObject()
		targetobj.name = action_args.target_module.lower()
		targetobj.set_options(defined_options)

		if find_chain(main_args.tenantobj, main_args.table, action_args.chain):
			error("Chain " + action_args.chain  + " already exists")
			#sys.exit(1)
			return

		add_chain(main_args.tenantobj, main_args.table, action_args.chain, targetobj)


	if main_args.delete_chain:
		action_parser.add_argument('chain')

		action_args, rem_args = PARSE_ACTION( action_parser, rem_args)

		if rem_args:
			raise Exception("Invalid options: %s" % (rem_args))

		chain_name     = action_args.chain.lower()

		delete_chain(main_args.tenantobj, main_args.table, chain_name)

		return


	if main_args.policy_chain:
		action_parser.add_argument('chain')
		action_parser.add_argument('-j', '--target', dest='target_module', type=str, default=DEFAULT_POLICY_TARGET_HANDLER)

		action_args, rem_args = PARSE_ACTION( action_parser, rem_args)

		chain_name         = action_args.chain.lower()

		target_module_name = action_args.target_module.lower() 

		if not registered_targets.has_key(target_module_name):
			raise Exception("Invalid target '%s'. Be sure entables is configured properly!" % (target_module_name) )

		target_parser = argparse.ArgumentParser(description='Target ')			
		target_handler = registered_targets[target_module_name]
		target_handler.add_args_to_parser(target_parser)

		target_args, rem_args = PARSE_ACTION( target_parser, rem_args)

		if rem_args:
			raise Exception("Invalid options: %s" % (rem_args))

		# Validate
		defined_options = target_handler.build_from_parsed_args(target_args.__dict__)
		target_handler.validate_target_options(defined_options)

		# Set
		targetobj = TargetMemoryObject()
		targetobj.name = action_args.target_module.lower()
		targetobj.set_options(defined_options)

		if rem_args:
			raise Exception("Invalid sintax. %s" % (rem_args))

		set_chain_policy(main_args.tenantobj, main_args.table, chain_name, targetobj)


	if main_args.rename_chain:
		action_parser.add_argument('chain')
		action_parser.add_argument('new_chain_name')

		action_args, rem_args = PARSE_ACTION( action_parser, rem_args)

		if rem_args:
			raise Exception("Invalid options: %s" % (rem_args))

		chain_name     = action_args.chain.lower()
		new_chain_name = action_args.new_chain_name.lower()

		rename_chain(main_args.tenantobj, main_args.table, chain_name, new_chain_name)


	if main_args.flush_chain:
		action_parser.add_argument('chain')

		action_args, rem_args = PARSE_ACTION( action_parser, rem_args)

		if rem_args:
			raise Exception("Invalid options: %s" % (rem_args))

		chain_name     = action_args.chain.lower()
        
		flush_chain(main_args.tenantobj, main_args.table, chain_name)
    
	if main_args.zero_chain:
		action_parser.add_argument('chain')

		action_args, rem_args = PARSE_ACTION( action_parser, rem_args)

		if rem_args:
			raise Exception("Invalid options: %s" % (rem_args))

		chain_name     = action_args.chain.lower()
        
		zero_chain(main_args.tenantobj, main_args.table, chain_name, rulenum=0)


	if main_args.disable_rule:
		action_parser.add_argument('chain')
		action_parser.add_argument('rule_position', nargs='?', type=int, default=1, help="rule position")

		action_args, rem_args = PARSE_ACTION( action_parser, rem_args)

		chain_name     = action_args.chain.lower()
		rule_pos       = action_args.rule_position

		if rem_args:
			raise Exception("Invalid options: %s" % (rem_args))

		disable_rule(main_args.tenantobj, main_args.table, chain_name, rule_pos)


	if main_args.activate_rule:
		action_parser.add_argument('chain')
		action_parser.add_argument('rule_position', nargs='?', type=int, default=1, help="rule position")

		action_args, rem_args = PARSE_ACTION( action_parser, rem_args)

		chain_name     = action_args.chain.lower()
		rule_pos       = action_args.rule_position

		if rem_args:
			raise Exception("Invalid options: %s" % (rem_args))

		activate_rule(main_args.tenantobj, main_args.table, chain_name, rule_pos)


	if main_args.delete_rule:
		action_parser.add_argument('chain')
		action_parser.add_argument('rule_position', nargs='?', type=int, default=FIRST_RULE_POS, help="rule position")
        
		action_args, rem_args = PARSE_ACTION( action_parser, rem_args)

		chain_name     = action_args.chain.lower()
		rule_pos       = action_args.rule_position

		if rem_args:
			raise Exception("Invalid options: %s" % (rem_args))

		delete_rule_by_pos(main_args.tenantobj, main_args.table, chain_name, rule_pos)

	if main_args.append_rule:

		action_parser.add_argument('chain')

		# FA SCHIFO: fare funzione per trovare le chain valide per un tenant
		dbdumpobj = dbdump()
		valid_chains = dbdumpobj[ main_args.tenantobj.name ]['tables'][main_args.table]['chains']

		action_args, ruleobj = parse_rule_matchess_and_target_parameters(valid_chains , action_parser, rem_args)

		append_rule(main_args.tenantobj, main_args.table, action_args.chain, ruleobj)


	if main_args.insert_rule:
		action_parser.add_argument('chain')
		action_parser.add_argument('rule_position', nargs='?', type=int, default=1, help="rule position")        

		# FA SCHIFO: fare funzione per trovare le chain valide per un tenant
		dbdumpobj = dbdump()
		valid_chains = dbdumpobj[ main_args.tenantobj.name ]['tables'][main_args.table]['chains']

		    
		action_args, ruleobj = parse_rule_matchess_and_target_parameters(valid_chains, action_parser, rem_args)
    
		insert_rule(main_args.tenantobj, main_args.table, action_args.chain, ruleobj, int(action_args.rule_position) )

	#if main_args.disable_rule:
	#	action_parser.add_argument('chain')
	#	action_parser.add_argument('rule_position', type=int, help="rule position")        

	#	action_args, rem_args = PARSE_ACTION( action_parser, rem_args)
		
	#	chain_name     = action_args.chain.lower()
	#	rule_pos       = action_args.rule_position

	#	disable_rule(main_args.tenantobj, main_args.table, action_args.chain, action_args.rule_position)


def entables_launch_cmdline(argv):

	init_globals()

	process_rule_args(argv)

