# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os

import pipes  # quoting

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def QUOTE(s):
	return pipes.quote(s)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def dump_line(l, output_stream, *args):
	print >>output_stream, ('  ' * l) + ' '.join(args)

def dumpprint(output_stream, dump, level=0):

	l = level

	for tenant_name, tenantobj in dump.items():

		dump_line(l, output_stream, "#" )
		dump_line(l, output_stream, "#" + tenant_name)
		dump_line(l, output_stream, "#")
		dump_line(l, output_stream, "tenant", tenant_name)

		dump_templates(output_stream,  tenantobj["templates"], level=l+1 )

		dump_tables(output_stream,  tenantobj["tables"], level=l+1 )

		dump_line(l, output_stream, "exit")

def dump_tables(output_stream,  tables_dict, level=0 ):

	#tables = tenantobj["tables"]
	l = level
	for table_name, table in tables_dict.items():

		dump_line(l, output_stream, "#" )
		dump_line(l, output_stream, "# TABLE", table_name)
		dump_line(l, output_stream, "#")

		chains = table["chains"]
		

		dump_line(l, output_stream, "#" )
		dump_line(l, output_stream, "# CHAINS DEFINITION")
		dump_line(l, output_stream, "#")

		dump_chains_definition(output_stream, table_name, chains, level=l+1)

		dump_line(l, output_stream, "#" )
		dump_line(l, output_stream, "# CHAINS RULES")
		dump_line(l, output_stream, "#")
		
		dump_chains_rules(output_stream, table_name, chains, level=l+1)

		dump_line(l, output_stream, "")

def dump_chains_definition(output_stream, table_name, chains_dict, level=0):

	l=level

	for chain_name, chain in chains_dict.items():
		dump_single_chain_definition(output_stream, table_name, chain_name, chain, level=l)

def dump_chain_policy_options(chain):

		args = []
		
		policy = chain["policy"]
		policyoptions = policy["options"]

		for option_name, option_value in policyoptions.items():

			if option_value == None:
				continue

			args += [ " --" + policy["handler_name"] + "_" + option_name , QUOTE(str(option_value)) ]

		return args
		
def dump_single_chain_definition(output_stream, table_name, chain_name, chain, level=0):
	
	l = level

	dump_line(l, output_stream, "rule", "-t", table_name, "-N", chain_name )

	args = [ "rule", "-t", table_name, "-P" , chain_name, "-j", chain["policy"]["handler_name"].upper() ]
	
	args += dump_chain_policy_options(chain)

	dump_line(l, output_stream, *args)


def dump_chains_rules(output_stream, table_name, chains_dict, level=0):

	l=level

	for chain_name, chain in chains_dict.items():
		dump_chain_rules(output_stream, table_name, chain_name, chain, level=l)
			
def dump_chain_rules(output_stream, table_name, chain_name, chain, level=0, prefix='rule'):
	
	l = level

	for rule in chain["rules"]:
		args = []
		
		matches = rule["matches"]
		target = rule["target"]
		targetopts = target["options"]
		
		args += [ prefix % rule ]
		
		args += [ "-t" , table_name ]
		
		args += [ "-A" , chain_name.upper() ]
	    
		for match in matches:
			
			matchopts = match["options"]
			
			args += [ "-m" , match["handler_name"] ]
			
			for option_name in matchopts.keys():
				
				if matchopts[option_name] is not None:
					
					args += [ "--" + match["handler_name"] + "_" + option_name , QUOTE(str(matchopts[option_name])) ]

		args += [ "-j" , target["handler_name"].upper() ]

		for option_name in targetopts.keys():
			if targetopts[option_name] is not None:
				args += [ "--" + target["handler_name"] + "_" + option_name ,  QUOTE(str(targetopts[option_name])) ]
	     
		if rule["stop_if_match"] == True:
			args += [ " --stop-if-match" ]

		if rule['disabled']:

			args += [ " --disabled" ]
			
		dump_line(l, output_stream, *args)
		
		#if rule['disabled']:
		#	args = []
		#	args += [ prefix % rule ]
		#	args += [ "-t" , table_name ]
		#	args += [ "-x" , chain_name.upper() ]
		#	args += [ str(rule["position"]) ]
		#	dump_line(l, output_stream, *args)

	return

def dump_templates(output_stream, templates_dict, level=0):
	l=level
	
	dump_line(l, output_stream, "#" )
	dump_line(l, output_stream, "# templates" )
	dump_line(l, output_stream, "#" )
        
	for name, body in templates_dict.items():
		dump_single_template(output_stream,  name, body, level=0 )

def dump_single_template(output_stream, name, body, level=0):
	
	l = level
	
	dump_line(l, output_stream, "template", name )
	
	dotted_body = __dot_stuff(body)
	
	print >>output_stream, dotted_body

	dump_line(l, output_stream, "# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< end template '%s' " % (name))

def __dot_stuff(text):
	# FIX: comune alla CLI. Usare funzioni uniche
	
	#
	# Inverse of dot_terminated_text
	#
	out=[]
	for line in text.split('\n'):
		if line != '' and line[0] == '.':
			out.append('.'+line)
		else:
			out.append(line)
	out.append(".")
	return '\n'.join(out)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

"""
def getTerminalSize():

    env = os.environ
    def ioctl_GWINSZ(fd):
        try:
            import fcntl, termios, struct, os
            cr = struct.unpack('hh', fcntl.ioctl(fd, termios.TIOCGWINSZ,'1234'))
        except:
            return None
        return cr
    cr = ioctl_GWINSZ(0) or ioctl_GWINSZ(1) or ioctl_GWINSZ(2)
    if not cr:
        try:
            fd = os.open(os.ctermid(), os.O_RDONLY)
            cr = ioctl_GWINSZ(fd)
            os.close(fd)
        except:
            pass
    if not cr:
        try:
            cr = (env['LINES'], env['COLUMNS'])
        except:
            cr = (25, 80)
    return int(cr[1]), int(cr[0])

"""

'''     
        tabprint example

/- Status D=disabled, X=Stopping
|
mangle:DEFAULT (policy DROP)
=======================================================
  | 1 | log        |             | attributes |
  |   | - log_file | file.log    | - status   | DOWN
  |   |            |             | - site     | AUSLRA
-------------------------------------------------------
D | 2 | smtp       |             | attributes |
  |   | - host     | 192.168.0.1 | - status   | UP
  |   |            |             | - site     | AUSLBO
=======================================================
notify:GRA (policy LOG log-file=file.log)
=======================================================
D | 1 | log        |             | attributes |
  |   | - log_file | file.log    | - status   | DOWN
  |   |            |             | - site     | AUSLRA
-------------------------------------------------------
X | 2 | smtp       |             | attributes |
  |   | - host     | 192.168.0.1 | - status   | UP
  |   |            |             | - site     | AUSLBO
=======================================================

'''
'''
def tabprint2(dump, printtenant, printtable, printchain):
    h_sep_width = 3
    listing = {
	'tenantname': {
                '__numchains': 0,
		'chainname': {
			    'header': '',
			    'total_width':0
			    'columns_width' : {
				'status':1,
				'id':0,
				'targets':0,
				'targets_optvalues':0,
				'matches':0,
				'matches_optvalues':0,
			    }
                            'numrules': 0,   
			    'rulerows': [
				 {
                                   'numrows': 0,
				   'textrows': [
				       {
					       'status': '',
					       'id': '',
					       'target': '',
					       'targetopts':'',
					       'match': '',
					       'matchopts':''
				       }
				   ]
				 }
			    ]
		}
	}
    }
    if tenant == 'all':
        tenants = dump.keys()
    else:
        tenants = printtenant
        
    for tenantname in tenants:
        tenant = dump[tenantname]
	listing[tenantname]={}
	listing[tenantname]['__numchains']=0

        if printtable == 'all':
            tables = tenant["tables"]
        else:
            tables = printtable
            
        for tablename in tables:
            table = tables[tablename]
            if printchain == 'all':
                chains = table["chains"]
            else:
                chains = printchain 
                
            for chainname in chains:
		listing[tenantname]['__numchains'] += 1
                listing[tenantname][tablename+":"+chainname] = {}
		listing_chain = listing[tenantname][tablename+":"+chainname]
                listing_chain['numrules'] = 0
		listing_chain['column_width'] = {}
		listing_chain['column_width']['target'] = 0

                for rule in chain['rules']:
                    numrule = listing_chain['numrules']

                    listing_chain['rulerow'][numrule] = {}
                    listing_rule = listing_chain['rulerow'][numrule]


                    listing_chain['numrules'] += 1
                    listing_rule[]
                    target = rule.target
                    target.name
		    numrowstargets = 1
		    numrowstargets += len(target.options)
		    numrowsmatches = len(rule.matches)
                    for match in rule.matches:
                    	numrowsmatches += len(match.options)

		    numrows = numrowstargets
                    if numrowsmatches > numrowstargets:
	                numrows = numrowsmatches

                    for rownumber in range(numrows):
                  
                    for opt in target.options.keys():
			if len(opt) > listing_chain['colum_width']['target']
				= len(opt)
			if len(target.options[opt]) > 
				= len(target.options[opt])
			target_rows[rownumber] = (opt, target.options[opt])
                    for remnantrow in range(numrows - numrowstargets):
			target_rows[rownumber+remnantrow] = ('','')
			
                    for match in rule.matches:
                        for opt in match.options.keys():
			if len(opt) > listing_chain['colum_width']['target']
				= len(opt)
			if len(target.options[opt]) > 
				= len(target.options[opt])
			target_rows[rownumber] = (opt, target.options[opt])
                    for remnantrow in range(numrows - numrowstargets):
			target_rows[rownumber+remnantrow] = ('','')
                            match_column['values'].append("- " + opt)
                            if len(opt) +2 > match_column['max_width']:
                                match_column['max_width'] = len(opt) +2
                            match_column['rows']+=1
                            
                            match_opts_column['values'].append(match.options[opt])
                            if len(match.options[opt]) > match_opts_column['max_width']:
                                match_opts_column['max_width'] = len(match.options[opt])
                            match_opts_column['rows']+=1        
'''
    
"""
def tabprint(dump,whichtenant, whichtable,whichchain):
    text = ""
    if whichtenant == 'all':
        tenants = dump.keys()
    else:
        tenants = whichtenant
        
    for tenantname in tenants:
        tenant = dump[tenantname]
        if whichtable == 'all':
            tables = tenant["tables"]
        else:
            tables = whichtable
            
        for tablename in tables:
            table = tables[tablename]
            if whichchain == 'all':
                chains = table["chains"]
            else:
                chains = whichchain
                
            for chainname in chains:
                chain = chains[chainname]
                status_column = {'values' : [],'max_width' : 0, 'actual_width' : 0,'rows' : 0}
                position_column = {'values' : [],'max_width' : 0, 'actual_width' : 0,'rows' : 0}
                target_column= {'values' : [],'max_width' : 0, 'actual_width' : 0,'rows' : 0}
                target_opts_column= {'values' : [],'max_width' : 0, 'actual_width' : 0,'rows' : 0}
                match_column= {'values' : [],'max_width' : 0, 'actual_width' : 0,'rows' : 0}
                match_opts_column= {'values' : [],'max_width' : 0, 'actual_width' : 0,'rows' : 0}
                
                needed_rows = 0
                status_column['max_width']=1
                            
                position = 0
                n_rules = len(chain["rules"])
                
                for rule in chain['rules']:            
                # first row
                    position +=1
                    status = " "
                    if rule.disabled:
                        status = "D"
                    if rule.stopping:
                        status = "X"
                    
                    #status
                    status_column['values'].append(status)
                    status_column['rows']+=1
                    
                    #position
                    position_column['values'].append(str(position))
                    if len(str(position)) > position_column['max_width']:
                        position_column['max_width'] = len(str(position))
                    position_column['rows']+=1
                    
                    
                    # target row
                    target = rule.target
                    target_column['values'].append(target.name)
                    if len(target.name) > target_column['max_width']:
                        target_column['max_width'] = len(target.name)
                    target_column['rows']+=1
                                
                    target_opts_column['values'].append("")
                    target_opts_column['rows']+=1
                    
                    # target opts rows
                    for opt in target.options.keys(): 
                        target_column['values'].append("- " + opt)
                        if len(opt) +2 > target_column['max_width']:
                            target_column['max_width'] = len(opt) +2
                        target_column['rows']+=1
                        
                        target_opts_column['values'].append(target.options[opt])
                        if len(target.options[opt]) +2  > target_opts_column['max_width']:
                            target_opts_column['max_width'] = len(target.options[opt]) +2
                        target_opts_column['rows']+=1
                    
                    for match in rule.matches:
                        # match name row
                        match_column['values'].append(match.name)
                        match_column['max_width'] = len(match.name)
                        match_column['rows']+=1
                            
                        match_opts_column['values'].append("")
                        match_opts_column['max_width']
                        match_opts_column['rows']+=1
                
                        for opt in match.options.keys():
                            match_column['values'].append("- " + opt)
                            if len(opt) +2 > match_column['max_width']:
                                match_column['max_width'] = len(opt) +2
                            match_column['rows']+=1
                            
                            match_opts_column['values'].append(match.options[opt])
                            if len(match.options[opt]) > match_opts_column['max_width']:
                                match_opts_column['max_width'] = len(match.options[opt])
                            match_opts_column['rows']+=1        
                
                    # end of rule, only if not the last
                    if position != n_rules:
                        status_column['values'].append("-")
                        status_column['rows']+=1
                        position_column['values'].append("-")
                        position_column['rows']+=1
                        target_column['values'].append("-")
                        target_column['rows']+=1
                        target_opts_column['values'].append("-")
                        target_opts_column['rows']+=1
                        match_column['values'].append("-")
                        match_column['rows']+=1
                        match_opts_column['values'].append("-")
                        match_opts_column['rows']+=1
                
        
                needed_rows = max(match_column['rows'], target_column['rows'], match_opts_column['rows'], target_opts_column['rows'])
                
                field_width = status_column['max_width'] + position_column['max_width'] + target_column['max_width'] + target_opts_column['max_width'] + match_column['max_width'] + match_opts_column['max_width'] 
                
                # 1 space for each extreme fields + 1 space for each side of the 4 central fields
                spaces_width = 1 + 2 * 4 + 1
                
                total_width = field_width + spaces_width
                
                available_width = getTerminalSize()[0]
                
                diff_width = available_width - total_width
                
                if diff_width < 0:
                    #all max_width must be reduced
                    total_width = available_width
                    pass
                
                #print "needed_rows " + str(needed_rows) + " field_width " + str(field_width) + " spaces_widt" + str(spaces_width) + " total_width " + str(total_width)
        
                
                title_string = ""
                title_string = tablename + ":"+ chainname.upper() + " " +  "(" + str(n_rules) + " rules) (policy " + chain['policy']['handler_name'].upper() + ")"
                for opt in chain['policy']['options'].keys():
                    title_string += " " + opt + "="+chain['policy']['options'][opt]
                    
                text += title_string[:total_width]
                
            
                
                formatstring = '{:{fill}<' + str(total_width) + '}'
                text += formatstring.format("",fill="=")
                # header_string needed ?
                
                for row in range(needed_rows):
                    row_string = ""
                
                    if status_column=="-":
                        # end of rule
                        formatstring = '{:{fill}<' + str(total_width) + '}'
                        print formatstring.format("",fill="-")
                    else:
                        # print fields
                        # status
                        if row < status_column['rows']:
                            row_string += status_column['values'][row]
                        else: 
                            row_string += " "
                        row_string += " "
                        
                        row_string+="|"
                
                        # position
                        row_string+=" "
                        if row < position_column['rows']:
                            row_string+= position_column['values'][row]
                        else:
                            row_string+= " "
                        row_string += " "
                    
                        row_string+="|"
                    
                        # target
                        row_string+= " "
                        if row < target_column['rows']:
                            field_string = target_column['values'][row]
                            if len(field_string) > target_column['max_width']:
                                field_string = field_string[:target_column['max_width']-1] + "+"
                            formatstring = '{:<' + str(target_column['max_width']) + '}'
                            row_string+= formatstring.format(field_string)
                        else:
                            for i in range(target_column['max_width']):
                                row_string+= " "
                        row_string+=" "
                
                        row_string+="|"
                        
                        # target_opts
                        row_string+= " "
                        if row < target_opts_column['rows']:
                            field_string = target_opts_column['values'][row]
                            if len(field_string) > target_opts_column['max_width']:
                                field_string = field_string[:target_opts_column['max_width']-1] + "+"
                            formatstring = '{:<' + str(target_opts_column['max_width']) + '}'
                            row_string+= formatstring.format(field_string)
                        else:
                            for i in range(target_column['max_width']):
                                row_string+= " "
                        row_string+=" "
                        
                        row_string+="|"
                        
                        # match
                        row_string+= " "
                        if row < match_column['rows']:
                            field_string = match_column['values'][row]
                            if len(field_string) > match_column['max_width']:
                                field_string = field_string[:match_column['max_width']-1] + "+"
                            formatstring = '{:<' + str(match_column['max_width']) + '}'
                            row_string+= formatstring.format(field_string)
                        else:
                            for i in range(match_column['max_width']):
                                row_string+= " "
                        row_string+=" "
                
                        row_string+="|"
                        
                        # match_opts
                        row_string+= " "
                        if row < match_opts_column['rows']:
                            field_string = match_opts_column['values'][row]
                            if len(field_string) > match_opts_column['max_width']:
                                field_string = field_string[:match_opts_column['max_width']-1] + "+"
                            formatstring = '{:<' + str(match_opts_column['max_width']) + '}'
                            row_string+= formatstring.format(field_string)
                        else:
                            for i in range(match_opts_column['max_width']):
                                row_string+= " "
                        row_string+=" "
                    
                    text += row_string    
                        
                        
                formatstring = '{:{fill}<' + str(total_width) + '}'
        
                print formatstring.format("",fill="=")
            print text
"""





