# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import json

#from django.db import transaction

from django.http import HttpResponse



from django.shortcuts import render
from django import forms

from core.compatibility import reverse

from django.template import Template
from django.template import Context, RequestContext


from core.api.base import db_set_current_tenant_schema

from core.db_utils import transaction_commit_on_success

from webui.decorators import tenant_dependent

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class ChainForm(forms.Form):
	chain = forms.CharField(max_length=100)

@tenant_dependent
def add_new_chain(request, table_name):
	
	tenant = request.tenant
	db_set_current_tenant_schema(tenant.name)


	if request.method == 'POST':
		form = ChainForm(request.POST) # A form bound to the POST data
		if form.is_valid(): 

			chain_name = form.cleaned_data['chain']

			from entables.targets import CONTINUETarget
			from entables.datastructures import TargetMemoryObject
			from entables.tablesstore.djangostore import add_chain
			target_handler = CONTINUETarget()
			targetobj = TargetMemoryObject()
			targetobj.name = target_handler.name

			add_chain(tenant, table_name, chain_name, targetobj)

			return HttpResponse('OK') # Redirect after POST


	else:
		form = ChainForm(initial={'chain':'default', 'table':'notify'}) # An unbound form


	html_template = """
			<form id='form' action="{{action}}" method="post">{% csrf_token %}
				{{ form.as_p }}
				<input id='submit' type="submit" value="Submit" />
			</form>	
	"""

	t = Template(html_template)
	html = t.render(Context({ 
		'action': reverse('entables.views.add_new_chain', kwargs={'table_name':table_name}),
		'form': form, 
	}))

	return HttpResponse(html)

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

@tenant_dependent
def del_chain(request, table_name, chain_name):
	
	tenant = request.tenant
	db_set_current_tenant_schema(tenant.name)
	
	from entables.tablesstore.djangostore import delete_chain
	
	delete_chain(tenant, table_name, chain_name)
	
	return HttpResponse('OK')
	
	
	
#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class NewRuleForm(forms.Form):
	MODE_CHOICES=[
		('append','Append rule'),
		('insert','Insert rule')
	]
	mode     = forms.ChoiceField(required=True, choices=MODE_CHOICES)#, widget=forms.RadioSelect(), )	
	position = forms.IntegerField(required=False, min_value=1)

	def clean(self):

		if not self.errors:
			
			mode = self.cleaned_data['mode']
			if mode == 'append':
				if self.cleaned_data['position']:
					self._errors["position"] = self.error_class(["You can not specify a position when appending a rule"])
					del self.cleaned_data['position']
			else:
				
				if self.cleaned_data['position'] is None:
					self._errors["position"] = self.error_class(["This field is required"])
					del self.cleaned_data['position']
		
			return self.cleaned_data

		return super(NewRuleForm, self).clean()



@tenant_dependent
def add_rule(request, table_name, chain_name):
	
	tenant = request.tenant
	db_set_current_tenant_schema(tenant.name)



	if request.method == 'POST':
		form = NewRuleForm(request.POST) # A form bound to the POST data
		if form.is_valid(): 

			mode     = form.cleaned_data['mode']
			if mode == 'append':
				position = -1
			else:
				position = int(form.cleaned_data['position'])

			from entables.tablesstore.djangostore import insert_rule, append_rule
			from entables.datastructures import TargetMemoryObject, RuleMemoryObject
			targetobj = TargetMemoryObject()
			targetobj.name = 'continue'
			ruleobj = RuleMemoryObject(targetobj, stop_if_match=False, disabled=False)
			
			if position != -1:
				insert_rule(tenant, table_name, chain_name, ruleobj, position )
			else:
				append_rule(tenant, table_name, chain_name, ruleobj)

			return HttpResponse('OK') # Redirect after POST


	else:
		form = NewRuleForm(initial={'position':None}) # An unbound form


	html_template = """
			<form id='form' action="{{action}}" method="post">{% csrf_token %}
				{{ form.as_p }}
				<input id='submit' type="submit" value="Submit" />
			</form>	
	"""

	html_template = """
			<form id='form' action="{{action}}" method="post">{% csrf_token %}
				<!-- {{ form.as_p }} -->

				{% for hidden in form.hidden_fields %}
					{{ hidden }}
				{% endfor %}
				
				{% for e in form.non_field_errors %}
					{{e}}
				{% endfor %}
				
				<table>
					{% for field in form.visible_fields %}
					<tr>
					
						<td> 
							{{ field.label_tag }}
						</td>
						<td> 
							{{ field }}
						</td>
						<td> 
							 {{ field.help_text }}
						</td>
						<td> 
							{{ field.errors }}
						</td>
									
					<tr>
					{% endfor %}
				<table>
				
				<input id='submit' type="submit" value="{{button_label}}" />
			</form>		
	"""
	

	t = Template(html_template)
	html = t.render(Context({ 
		'action': reverse('entables.views.add_rule', kwargs={'table_name':table_name, 'chain_name': chain_name}),
		'form': form, 
		'button_label': "Confirm",
	}))

	return HttpResponse(html)

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

@tenant_dependent
def toggle_active(request, ruleid):
	
	tenant = request.tenant
	db_set_current_tenant_schema(tenant.name)
	
	rule = Rule.objects.get(tenant=tenant, uuid=ruleid)
	rule.disabled = not rule.disabled
	rule.save()
	
	return HttpResponse('OK')
	
@tenant_dependent
def toggle_stop(request, ruleid):
	
	tenant = request.tenant
	db_set_current_tenant_schema(tenant.name)
	
	rule = Rule.objects.get(tenant=tenant, uuid=ruleid)
	rule.stop_if_match = not rule.stop_if_match
	rule.save()
	
	return HttpResponse('OK')



#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

@tenant_dependent
def del_rule(request, ruleid):
	
	tenant = request.tenant
	db_set_current_tenant_schema(tenant.name)
	
	from entables.tablesstore.djangostore import delete_rule_by_pos
	
	rule = Rule.objects.get(tenant=tenant, uuid=ruleid)
	
	delete_rule_by_pos(tenant, rule.chain.table.name, rule.chain.name, rule.position)
	
	return HttpResponse('OK')

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

@tenant_dependent
def del_match(request, matchid):

	tenant = request.tenant
	db_set_current_tenant_schema(tenant.name)
	
	from entables.models import Match
	
	m = Match.objects.get(tenant=tenant, uuid=matchid)
	
	m.delete()

	return HttpResponse('OK')
	
#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

from django.forms.forms import NON_FIELD_ERRORS

class TargetForm(forms.Form):

	current     = forms.CharField(max_length=100, widget=forms.HiddenInput)

	def __init__(self, target_name, target_handler, valid_targets, *args, **kwargs):
		
		self.target_handler = target_handler

		print "FORM", args
		print "FORM", kwargs

		super(TargetForm, self).__init__(*args, **kwargs)

		# TARGET_NAME select 
		self.fields['target_name'] = forms.ChoiceField( choices=valid_targets )
		self.fields['target_name'].initial = target_name

		initials = kwargs.get('initials', {})

		# TARGET OPTIONS
		if target_handler:

			options = target_handler.get_options()

			for name, option in options.items():

				field_config = {
					'required'  : option.required,
					'help_text' : option.default,		
				}

				field_type = forms.CharField
				if option.type is int:
					field_type = forms.IntegerField
				elif option.choices is not None:
					field_type = forms.ChoiceField
					field_config = {
						'choices'   : [(c,c) for c in option.choices]
					}

				# Define field
				self.fields[name] = field_type(**field_config)

				if initials.has_key(name):
					self.fields[name].initial = initials[name]

	def clean(self):

		if not self.errors:
			
			if self.target_handler != None:
				# Get options' value
				user_options = {}
				for name, optiondef in self.target_handler.get_options().items():
					if not self.cleaned_data.has_key(name):
						continue
					user_val = self.cleaned_data[name]
					if user_val:  # ignore blank options
						user_options[name] = user_val
			
				# Validate options
				try:
				
					self.target_handler.validate_target_options(user_options)
				except Exception, e:
					msg = str(e)
					self._errors[NON_FIELD_ERRORS] = self.error_class([msg])
		
			return self.cleaned_data

		return super(TargetForm, self).clean()



def init_log():
	
	log = logging.getLogger("entablesd")

	level = 1
	formatter = logging.Formatter('%(message)s')

	log.setLevel( level )

	hdlr = logging.StreamHandler()
	hdlr.setFormatter( formatter )
	hdlr.setLevel(100)
	log.addHandler(hdlr)

	return log

import logging
from entables.models import Target, Rule, Chain, Match,  MatchOptions
from django.conf import settings
from entables.datastructures import TargetDefinitionList, MatchDefinitionList, MatchMemoryObject
from entables.modulesmanager import load_target_modules
from entables.modulesmanager import load_match_modules

def init_log():
	
	log = logging.getLogger("entablesd")

	level = 1
	formatter = logging.Formatter('%(message)s')

	log.setLevel( level )

	hdlr = logging.StreamHandler()
	hdlr.setFormatter( formatter )
	hdlr.setLevel(100)
	log.addHandler(hdlr)

	return log




@tenant_dependent
def config_target(request, targetid):
	
	tenant = request.tenant
	db_set_current_tenant_schema(tenant.name)

	target = Target.objects.get(tenant=tenant, uuid=targetid)
	selected_target_name = target.handler_name
	current = selected_target_name
	
	#
	# Troviamo tutte le chain della stessa tabella
	#
	try:
		chain  = Chain.objects.get(tenant=tenant, policy=target)
		table = chain.table
	except Chain.DoesNotExist, e:
		rule   = Rule.objects.get(tenant=tenant, target=target)
		chain  = rule.chain
		table  = chain.table
			
	chain_names = [ r[0] for r in table.chains.values_list('name').all() ]
	
	
	#
	# Carichiamo i target disponibili
	#
	
	# LOAD TARGET MODULE
	configuration = settings.ENTABLESD_SETTINGS	
	registered_targets = TargetDefinitionList()
	log = init_log()
	load_target_modules(log, configuration['targets'], registered_targets)    


	valid_targets = sorted([ (name, 'TARGET: %s' % (name)) for name in registered_targets.keys()])  + sorted([(name, 'CHAIN: %s' % (name)) for name in chain_names])


	#
	# Processiamo i dati
	#
	form = None

	button_label = 'Select'

	
	if request.method == 'POST':
		
		selected_target_name = request.POST['target_name']
		current              = request.POST['current']
		
		button_label = 'Confirm'		
		
		# E' un target vero o un JUMP o cosa?
		target_handler = registered_targets.get(selected_target_name , None)
		if target_handler is None:
			if selected_target_name not in chain_names:
				raise Exception("UNKNWON TARGET '%s': %s" % (selected_target_name , chain_names))
				
		if selected_target_name == current:
			form = TargetForm(selected_target_name, target_handler,  valid_targets, request.POST)
			if form.is_valid(): 
			
				@transaction_commit_on_success
				def update_target_options(target, target_handler, form):
			
					target.handler_name = selected_target_name
					target.save()
					
					if target_handler is not None:
						valid_options = target_handler.get_options()
					else:
						valid_options = {}

					# Cancelliamo le opzioni non piu' valide per questo target
					for opt in target.target_options.all():
						if opt.name not in valid_options.keys():
							opt.delete()
						else:
							val = form.cleaned_data[opt.name]
							if not val:
								opt.delete()

					
					if target_handler is not None:

						# Creiamo/aggiorniamo le opzioni valide
						for name, optiondef in target_handler.get_options().items():
							val = form.cleaned_data[name]
							if val:
								opt, created = target.target_options.get_or_create(tenant=tenant, name=name)
								opt.value = val 
								opt.save()
			
				update_target_options(target, target_handler, form)
			
				return HttpResponse('OK') # Redirect after POST
		else:
			button_label = 'Continue'
			current = selected_target_name
			form = None

	if form is None:

		current_values = dict([  (o.name, o.value) for o in  target.target_options.all()])
	
		form_values = {
			'target_name': selected_target_name,
			'current'    : current,
		}
		form_values.update(current_values)

		target_handler = registered_targets.get(selected_target_name , None)
		
		form = TargetForm(selected_target_name, target_handler,  valid_targets  , initial=form_values) # An unbound form
		


	html_template = """
			<form id='form' action="{{action}}" method="post">{% csrf_token %}
				<!-- {{ form.as_p }} -->

				{% for hidden in form.hidden_fields %}
					{{ hidden }}
				{% endfor %}
				
				{% for e in form.non_field_errors %}
					{{e}}
				{% endfor %}
				
				<table>
					{% for field in form.visible_fields %}
					<tr>
					
						<td> 
							{{ field.label_tag }}
						</td>
						<td> 
							{{ field }}
						</td>
						<td> 
							 {{ field.help_text }}
						</td>
						<td> 
							{{ field.errors }}
						</td>
									
					<tr>
					{% endfor %}
				<table>
				
				<input id='submit' type="submit" value="{{button_label}}" />
			</form>	
			
	"""

	t = Template(html_template)
	html = t.render(Context({ 
		'action': reverse('entables.views.config_target', kwargs={'targetid': targetid}),
		'form': form, 
		'button_label': button_label
	}))

	return HttpResponse(html)























#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#



class MatchForm(forms.Form):

	current     = forms.CharField(max_length=100, widget=forms.HiddenInput)		

	def __init__(self, match_name, match_handler, valid_matches, *args, **kwargs):
		
		super(MatchForm, self).__init__(*args, **kwargs)
		
		self.match_handler = match_handler

		# TARGET_NAME select 
		self.fields['match_name']         = forms.ChoiceField( choices=([(' ', ' ')] + valid_matches) )
		self.fields['match_name'].initial = match_name

		initials = kwargs.get('initials', {})

		# TARGET OPTIONS
		if match_handler:

			options = match_handler.get_options()

			for name, option in options.items():

				# Define field
				#self.fields[name] = forms.CharField(
				#	required   = option.required,
				#	help_text  = option.default,
				#)
				#if initials.has_key(name):
				#	self.fields[name].initial = initials[name]		




				field_config = {
					'required'  : option.required,
					'help_text' : option.default,		
				}

				field_type = forms.CharField
				if option.type is int:
					field_type = forms.IntegerField
				elif option.choices is not None:
					field_type = forms.ChoiceField
					field_config.update({
						'choices'   : [(c,c) for c in option.choices]
					})

				# Define field
				print field_config
				self.fields[name] = field_type(**field_config)

				if initials.has_key(name):
					self.fields[name].initial = initials[name]







	def clean(self):
		
		print "CLEAN"

		if not self.errors:
			print 1
			# Get options' value
			user_options = {}
			for name, optiondef in self.match_handler.get_options().items():
				if not self.cleaned_data.has_key(name):
					continue
				user_val = self.cleaned_data[name]
				if user_val:  # ignore blank options
					user_options[name] = user_val

			print 2
					
			print "USER OPTIONS", user_options
			
			# Validate options
			try:
				self.match_handler.validate_match_options(user_options)
			except Exception, e:
				msg = str(e)
				print msg
				self._errors[NON_FIELD_ERRORS] = self.error_class([msg])
		
			return self.cleaned_data

		return super(MatchForm, self).clean()




@tenant_dependent
def add_match(request, ruleid):
	
	tenant = request.tenant
	db_set_current_tenant_schema(tenant.name)

	match  = None
	rule   = Rule.objects.get(tenant=tenant, uuid=ruleid)

	match_handler = None
	selected_match_name = None
	current            = None
	

	#
	# Carichiamo i target disponibili
	#
	
	# LOAD TARGET MODULE
	configuration = settings.ENTABLESD_SETTINGS
	registered_matches = MatchDefinitionList()
	log = init_log()
	load_match_modules(log, configuration['matches'], registered_matches)    


	valid_matches = sorted([ (name, name) for name in registered_matches.keys()])


	#
	# Processiamo i dati
	#
	form = None
	
	if request.method == 'POST':
		
		selected_match_name = request.POST['match_name']
		current             = request.POST['current']
		
		# E' un target vero o un JUMP o cosa?
		match_handler = registered_matches.get(selected_match_name , None)
		
		if match_handler != None:
			try:
				match = Match.objects.get(tenant=tenant, rule=rule, handler_name=match_handler.name)
			except Match.DoesNotExist, e:
				match = None
				

		if selected_match_name == current:
			
			form = MatchForm(selected_match_name, match_handler, valid_matches, request.POST)
			if form.is_valid(): 
			

				@transaction_commit_on_success
				def safe_add_match(form, rule, match_handler):

						match, created = Match.objects.get_or_create(tenant=tenant, rule=rule, handler_name=match_handler.name)
					
						defined_options = {}
						for name, optiondef in match_handler.get_options().items():
							val = form.cleaned_data[name]
							if val:
								defined_options[name] = val
							elif optiondef.default is not None:
								defined_options[name] = optiondef.default
								
								
						if not defined_options:
							match.delete()
							return
						
						
					
						matchobj = MatchMemoryObject() 
						matchobj.name = selected_match_name
						matchobj.set_options(defined_options)			

						if match_handler is not None:
							valid_options = match_handler.get_options()
						else:
							valid_options = {}


						# Cancelliamo le opzioni non piu' valide per questo target
						for opt in match.match_options.all():
							if opt.name not in valid_options.keys():
								opt.delete()
							else:
								val = form.cleaned_data[opt.name]
								if not val:
									opt.delete()
						


						
				

						for name, value in matchobj.options.items():
							matchoption, created = MatchOptions.objects.get_or_create(tenant=tenant, match=match, name=name)
							matchoption.value = value
							matchoption.save()
									
				safe_add_match(form, rule, match_handler)
			
				return HttpResponse('OK') # Redirect after POST
			pass
		else:
			current = selected_match_name
			form = None

		pass

	if form is None:

		if match:
			current_values = dict([  (o.name, o.value) for o in  match.match_options.all()])
		else:
			current_values = {}

		form_values = {
			'match_name': selected_match_name,
			'current'   : current,
		}
		form_values.update(current_values)

		match_handler = registered_matches.get(selected_match_name , None)
		
		form = MatchForm(selected_match_name, match_handler, valid_matches, initial=form_values) # An unbound form


	html_template = """
			<form id='form' action="{{action}}" method="post">{% csrf_token %}
				<!-- {{ form.as_p }} -->

				{% for hidden in form.hidden_fields %}
					{{ hidden }}
				{% endfor %}
				
				{% for e in form.non_field_errors %}
					{{e}}
				{% endfor %}
				
				
				<table>
					{% for field in form.visible_fields %}
					<tr>
					
						<td> 
							{{ field.label_tag }}
						</td>
						<td> 
							{{ field }}
						</td>
						<td> 
							 {{ field.help_text }}
						</td>
						<td> 
							{{ field.errors }}
						</td>
									
					<tr>
					{% endfor %}
				<table>
				
				<input id='submit' type="submit" value="Update" />
			</form>	
			
	"""

	t = Template(html_template)
	html = t.render(RequestContext(request, { 
		'action': reverse('entables.views.add_match', kwargs={'ruleid': ruleid}),
		'form': form, 
	}))

	return HttpResponse(html)





