# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
########################################################################
Binary utilities
########################################################################
"""

import time
import binascii
import types

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def crc32(s):
	"""
	Compute CRC-32, the 32-bit checksum of data *s*.
	
	:param  s:  A char string
	:type s:  string
	
	:returns: An integer value
	:rtype: integer
	
	:exception: 
	
		:class:`TypeError`
			Raised if the input data is not a string of bytes.
	"""
	# FIX: write a better one?
	return abs(binascii.crc32(s))

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def bin2hex(bin):
	"""
	Convert binary byte string [0x11,0xAA,0x110xFF,0x44] to the string '11AAFF44'.
	
	:param  s:  A string of bytes.
	:type s:  string
	
	:returns: An ascii string
	:rtype: string
	
	:exception: 
	
		:class:`TypeError`
		
	"""
	return binascii.hexlify(bin)

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def macbin2blob(bin):
	"""
	TODO 
	"""
	if len(bin) == 6:
		return binascii.b2a_hex(bin)
	else:
		return bin[3:]

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def datestr():
	"""
	Esport current local time ( :method:`time.localtime` ) in the string format: YYYMMGGHHMMSS

	:returns: A string.
	:rtype: string
	"""
	year, mon, mday, hour, minu, sec, wday, yday, isdst = time.localtime(time.time())
	
	return "%04d%02d%02d%02d%02d%02d" % (year, mon , mday, hour, minu, sec)
