# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
################################################################################
Cache API
################################################################################


.. automodule:: sanet_common.cache.ramcache

.. automodule:: sanet_common.cache.rediscache

"""

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class AbstractCacheManager(object):
	
	"""
	Abstract cache handler
	"""
	
	KEY_SEP   = '.'

	def __init__(self, namespace=None, *args, **kwargs):
		if namespace == None:
			self.namespace = ''
		elif namespace == '':
			raise Exception("Invalid namespace '%s'" % (namespace))
		else:
			self.namespace = namespace

		
	def create_key(self, *key_sequence):
		key = self.KEY_SEP.join([self.namespace] + list(key_sequence) )
		return key

	def get(self, key):
		"""
		Returns:
			- None if the key is not present
			- None if the key is expired
			- a value if the key is present
		"""
		raise Exception("Not implemented")

	def set(self, key, value, expire_seconds=None):
		raise Exception("Not implemented")

	def delete(self, key):
		raise Exception("Not implemented")


	def mget(self, key, sub_key_list):
		raise Exception("Not implemented")
		
	def mset(self, key, subkey_value_dict):
		raise Exception("Not implemented")

	def clone(self):
		"""
		Returns a copy of this cache manager
		"""
		raise Exception("Not implemented")
		
	def close(self):
		raise Exception("Not implemented")

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class NullCacheManager(AbstractCacheManager):
	"""
	Dummy cache
	"""
	
	def __init__(self, namespace=None, *args, **kwargs):
		super(NullCacheManager, self).__init__(namespace=namespace, *args, **kwargs)
		
	def get(self, key):
		return None

	def set(self, key, value, expire_seconds=None):
		pass

	def delete(self, key):
		pass

	def mget(self, key, sub_key_list):
		return {}
		
	def mset(self, key, subkey_value_dict):
		pass

	def clone(self):
		return self

	def close(self):
		return
