# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
################################################################################
RAM cache
################################################################################

TODO

"""

import time

from sanet_common.cache import AbstractCacheManager

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class MemoryCacheManager(AbstractCacheManager):
	
	def __init__(self, namespace=None, *args, **kwargs):
		
		super(MemoryCacheManager, self).__init__(namespace=namespace, *args, **kwargs)
		
		self._cache = {}

	def get(self, key):

		ckey = self.create_key(key)
		
		cv = self._cache.get(ckey, None)
		if cv == None:
			return None

		expire_time, value = cv
		if expire_time == None:
			return value

		if expire_time >= time.time():
			return value

		# Expired value, remove it
		try:
			del self._cache[ckey]
		except KeyError, e:
			pass
		return None
		
	def mget(self, basekey, sub_key_list):
		
		data = {}
		
		for subkey in sub_key_list:
			
			key = self.create_key(basekey, subkey)

			data[ subkey ] = self.get(key)
			
		return data

	def set(self, key, value, expire_seconds=None):

		if expire_seconds:
			expire_time = time.time() + expire_seconds
		else:
			expire_time = None
			
		ckey = self.create_key(key)
			
		self._set(ckey, value, expire_time)

	def _set(self, ckey, value, expire_time):
		self._cache[ ckey ] = (expire_time, value)

	def mset(self, key, subkey_value_dict, expire_seconds=None):

		if expire_seconds:
			expire_time = time.time() + expire_seconds
		else:
			expire_time = None

		for subkey, value in subkey_value_dict.items():
			ckey = self.create_key(key, subkey)
			self._set(ckey, value, expire_time)

	def delete(self, key):

		ckey = self.create_key(key)
		
		# Expired value, remove it
		try:
			del self._cache[ckey]
		except KeyError, e:
			pass

