# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
################################################################################
REDIS cache
################################################################################

TODO

"""

import redis
import pickle

from sanet_common.cache import AbstractCacheManager

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class BaseRedisCacheManager(AbstractCacheManager):

	def __init__(self, namespace=None, *args, **kwargs):
		super(BaseRedisCacheManager, self).__init__(namespace=namespace, *args, **kwargs)

		c = redis.ConnectionPool(*args, **kwargs)
		self._redisclient = redis.Redis(connection_pool=c)

		self._args = (args, kwargs)

	def delete(self, key):
		
		ckey = self.create_key(key)
		
		self._redisclient.delete(ckey)

	def get(self, key):

		ckey = self.create_key(key)

		cv = self._redisclient.get(ckey)
		if cv == None:
			return None

		return cv

	def mget(self, key, sub_key_list):

		data = {}
		keys_to_load = [ self.create_key(key, subkey) for subkey in sub_key_list ]

		# The 'mget' returns a list of values.
		# Create a dictionary with <key>
		#
		values = self._redisclient.mget(keys_to_load)

		for i in xrange(len(sub_key_list)):
			if values[i] is not None:
				data[ sub_key_list[i] ] = values[i] 

		return data

	def set(self, key, value, expire_seconds=None):

		ckey = self.create_key(key)
		
		self._redisclient.set(ckey, value)
		
		if expire_seconds:
			self._redisclient.expire(ckey, expire_seconds)

	def mset(self, key, subkey_value_dict, expire_seconds=None):

		d = {}
		for subkey, value in subkey_value_dict.items():
			rkey = self.create_key(key, subkey)
			d[ rkey ] = value

		self._redisclient.mset(d)

		# FIX:
		# - set expire time for each key


	def clone(self):

		args, kwargs = self._args
		new_c = self.__class__(*args, **kwargs)

		return new_c

	def close(self):
		self._redisclient.connection_pool.disconnect()

class RedisCacheManager(BaseRedisCacheManager):
	"""
	We pickles objects stored in the cache because Redis saves strings.
	"""

	def __init__(self, *args, **kwargs):
		super(RedisCacheManager, self).__init__(*args, **kwargs)

	def get(self, key):

		pickle_value = super(RedisCacheManager, self).get(key)
		if pickle_value == None:
			return None

		try:
			return pickle.loads( pickle_value )
		except Exception, e:
			#
			# FIX: sometimes saved values contain invalid Unicode characters that will broke pickle.loads(). 
			#
			#print __file__, "WARNING: RedisCacheManager.get(%s)" % (key), type(e), str(e)
			return None
		
	def mget(self, key, sub_key_list):
		data = super(RedisCacheManager, self).mget(key, sub_key_list)
		for k,v in data.items():
			data[k] = pickle.loads(v)
		return data

	def set(self, key, value, expire_seconds=None):
		
		pickle_value = pickle.dumps(value)

		super(RedisCacheManager, self).set(key, pickle_value, expire_seconds=expire_seconds)
		
	def mset(self, key, subkey_value_dict, expire_seconds=None):
		
		pickled_values = {}
		for k,v in subkey_value_dict.items():
			pickled_values[k] = pickle.dumps(v)
			
		super(RedisCacheManager, self).mset(key, pickled_values, expire_seconds=expire_seconds)



