# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
Local DB Backend
=====================

Il modulo e': 

::
	
	sanet_common.django.auth.backends.localdb.LocalDBBackend

Questo e' il  modulo di autenticazione di default di Sanet3.

Le credenziali di accesso vengono verificate cercando l'utente nel database interno di Sanet3.

.. note:: Tutte le richieste di autenticazione vengono loggate su syslog (nei log non compaiono le password)

.. note:: Questo modulo e' praticamente equivalente al modulo di autenticazione *'django.contrib.auth.backends.ModelBackend'* (La differenza principale e' proprio il logging su syslog).

Configurazione
---------------

Esempio di configurazione: 

::
	
	AUTHENTICATION_BACKENDS = [
		'sanet_common.django.auth.backends.localdb.LocalDBBackend',
	]
"""

import syslog

from django.contrib.auth.models import User

from django.contrib.auth.backends import ModelBackend

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class LocalDBBackend(ModelBackend):

	def authenticate(self, username, password):

		try:
			username = username.encode('utf-8')
			password = password.encode('utf-8')

			self.log("Authenticating user '%s'" % (username))

			user = User.objects.get(username=username)

			if not user.is_active:
				syslog.syslog("User '%s' not active" % (username))
				return None

			if not user.check_password(password):
				syslog.syslog("User '%s': invalid password" % (username))
				return None

			# read-only
			#user.is_authenticated = True

			self.log("User '%s': authenticated" % (username))

			return user

		except User.DoesNotExist, e:
			self.log("User '%s' not found" % (username))
			return None

		except ValueError, e:
			self.log("Error while checking password for user %s: %s:%s" % (username, type(e), e))
			return None

	def log(self, msg):
		#syslog.syslog(msg)
		#print >> sys.stderr, msg
		pass
