 
# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import unittest

from sanet_common.exprparser.ExprFactory      import BaseExprFactory
from sanet_common.exprparser.ExprFactory      import SnmpValueExprTree
from sanet_common.exprparser.ExprFactory      import DataIdentifierExprTree

from sanet_common.exprparser.utils            import parseString

#------------------------------------------------------------------------#
#                                                                        #
#------------------------------------------------------------------------#

def eval_string(expr_string, context, factory):
	
	expr_tree = parseString(expr_string, factory=factory )
	val = expr_tree.evaluate(context)
	return val

def test_expressions(expressions, context, factory=None):

	for expr_string, expected_value in expressions:

		try:
			val = eval_string(expr_string, context, factory)
		except Exception, e:
			val = e

		#print "TEST: %-50s -> %-50s (%-30s) <> %-50s " % (expr_string, val, type(val), expected_value, )

		if expected_value != val:
			raise Exception("Wrong result: expected %r. Got %r" % (expected_value, val))

#------------------------------------------------------------------------#
#                                                                        #
#------------------------------------------------------------------------#

class DummyException(Exception):
	pass

class DummyException1(DummyException):
	pass

class DummyException2(DummyException1):
	pass

class DummyException3(DummyException2):
	pass

#------------------------------------------------------------------------#
#                                                                        #
#------------------------------------------------------------------------#

def exception(context, msg):
	raise DummyException3(msg)

def newSymbol(context, name, value):
	context[name] = value
	return context[name]

def getSymbol(context, name):
	return context[name]

def f(context, a):
	return int(a)

def g(context, a1,a2,a3):
	return a1 + a2 + a3

def h(context, *args):
	#for a in args:
	#	print "parametro: ", str(a)
	return 0

def my_hex(context, val):
	return hex(val)

def getHex(context):
	return lambda context, val: hex(val)

def isReachable(context, agent):
	#print agent
	#print "\n...verifica raggiungibilita per: " + agent + "... TRUE!"
	return True

def byRegexpUnique(context, oid,regexpr):
	#return "6.6.6"
	return oid + str(regexpr)

def test_str(n, string, verify):
	print "Test %s" % (n)
	print "\t", "STR  :", string
	exprTree2 = parseStringWithFactory(string, MyFactory() )
	print "\t", "TREE :", exprTree2
	val = exprTree2.evaluate(symbolTable)
	print "\t", "VALUE:", val, "(type: " + str(type(val)) + ")"
	print "\t", "VER  :", verify

	if val != verify:
		raise Exception("ERROR")

symbolTable                   = {}
symbolTable['$i']             = 1			# variabile
symbolTable['$c']             = 'a'			# variabile
symbolTable['$s']             = "mondo"			# variabile
symbolTable['$c']             = '123 \\"mondo\\" 456 \'mondo\''# variabile
symbolTable['$f']             = 3.14			# variabile

symbolTable['f']              = f			# funzione
symbolTable['g']              = g			# funzione
symbolTable['h']              = h			# funzione

symbolTable['newSymbol']      = newSymbol
symbolTable['getSymbol']      = getSymbol

symbolTable['str']            = lambda context, s: str(s)

symbolTable['hex']            = my_hex
symbolTable['$hex']           = my_hex
symbolTable['my_hex']         = my_hex
symbolTable['getHex']         = getHex

symbolTable['isReachable']    = isReachable		# funzionea
symbolTable['byRegexpUnique'] = byRegexpUnique

symbolTable['exception']      = exception

#------------------------------------------------------------------------#
#                                                                        #
#------------------------------------------------------------------------#

class MySnmpValueExprTree(SnmpValueExprTree):

	def __init__(self, exprOID, eCommunity, charType, eHost, ePort):
		super(MySnmpValueExprTree, self).__init__(exprOID, eCommunity, charType, eHost, ePort)

	def calculateSnmpValue(self, context, oid,community,ctype,host,port):
		community = community if community is not None else 'public'
		port      = port      if port      is not None else 121
		host      = host      if host      is not None else 'localhost'

		if ctype == '@':
			v = 1
		else:
			v = -1

		s = oid 
		if community != "":
			s = s + ":" + community
		s = s + ctype
		if host != "":
			s = s + host
		if port != "":
			s = s + ':' + str(port)

		#print "............. VALORE SNMP PER: " + str(s) + " = " + str(v)
		return v

class MyDataIdentifierExprTree(DataIdentifierExprTree):

	def calculateDataValue(self, context):
		#print "............", self.id
		if self.dtype == '@':
			return 1
		else:
			return -1

class MyFactory(BaseExprFactory):

	def createSnmpValue(self, exprOID, eCommunity, charType, eHost, ePort):
		return MySnmpValueExprTree(exprOID, eCommunity, charType, eHost, ePort)

	def createDataIdentifier(self, dataId):
		return MyDataIdentifierExprTree(dataId)

#------------------------------------------------------------------------#
#                                                                        #
#------------------------------------------------------------------------#

class TestStringMethods(unittest.TestCase):

	def test_strings(self):

		EXPRESSIONS = [

			#
			# Constants
			#
			('''True'''  ,   True),
			('''False''' ,   False),

			#
			# Integers
			#
			('''1'''     ,   1),
			('''1234567890'''     ,   1234567890),
			
			#
			# Floats
			#
			('''1.1'''   , 1.1),
			('''1.1'''   , 1.1),

			#
			# String
			#


			
			#
			# Single quotes (")
			#
			( """ '' """    , ""         ),
			(""" 'a' """    , "a"        ),
			(""" 'hello' """, "hello"    ),
			(r""" '\n'   """, """\n"""   ),  # \n   -> *newline*
			(r""" '\r'   """, """\r"""   ),  # \r   ->      
			(r""" '\t'   """, """\t"""   ),  # \t   -> 
			(r""" '\b'   """, """\b"""   ),  # \b   -> \b
			(r""" '\f'   """, """\f"""   ),  # \f   -> 
			(r""" '\"'   """, '''"'''    ),  # \"   -> "
			(r""" '\''   """, """'"""    ),  # \'   -> '
			(r""" '\\\\' """, '\\' + '\\'),  # \\\\ -> \\

			(r""" r'\\'  """, '\\' + '\\'),  # \\\\ -> \\

			#
			# Double quotes (")
			#
			( """ "" """    , ""         ),
			(""" "a" """    , 'a'        ),
			(""" "hello" """, "hello"    ),
			(r""" "\n"   """, """\n"""   ),  # \n   -> *newline*
			(r""" "\r"   """, """\r"""   ),  # \r   ->      
			(r""" "\t"   """, """\t"""   ),  # \t   -> 
			(r""" "\b"   """, """\b"""   ),  # \b   -> \b
			(r""" "\f"   """, """\f"""   ),  # \f   -> 
			(r""" "\""   """, '''"'''    ),  # \"   -> "
			(r""" "\'"   """, """'"""    ),  # \'   -> '
			(r""" "\\\\" """, '\\' + '\\'),  # \\\\ -> \\
			(r""" r"\\"  """, '\\' + '\\'),  # \\\\ -> \\


			("  'a b c \\\ '                     " , r'a b c \ '      ),     #    'a b c \\ '         ->    a b c \
			('  "a b c \\\ "                     ' , r'a b c \ '      ),     #    "a b c \\ "         ->    a b c \
			('  \'a \"b\" c \\\ \'               ' , r'a "b" c \ '    ),     #    'a "b" c \\ '       ->    a "b" c \
			('  "a \\"b\\" c \\\ "               ' , r'a "b" c \ '    ),     #    "a \"b\" c \\ "     ->    a "b" c \
			('  \'a \\\'b\\\' c \\\ \'           ' , r"a 'b' c \ "    ),     #    'a \'b\' c \\ '     ->    a 'b' c \
			('  "a \'b\' c \\\ "                 ' , r"a 'b' c \ "    ),     #    "a 'b' c \\ "       ->    a 'b' c \
			(r"  'a \\\'b\\\' c \\ '             " , r"a \'b\' c \ "  ),     #    'a \\\'b\\\' c \\ ' ->    a \'b\' c \
			("  \"a \\\\'b\\\\' c \\\ \"         " , r"a \'b\' c \ "  ),     #    "a \\'b\\' c \\ "   ->    a \'b\' c \
			("   'a \\\\\"b\\\\\" c \\\ '        " , r'a \"b\" c \ '  ),     #    'a \\"b\\" c \\ '   ->    a \"b\" c \
			("   \"a \\\\\\\"b\\\\\\\" c \\\ \"  " , r'a \"b\" c \ '  ),     #    "a \\\"b\\\" c \\ " ->    a \"b\" c \
			('  r"a b c \ "                      ' , r'a b c \ '      ),     #    r"a b c \ "         ->    a b c \
			("  r'a b c \ '                      " , r'a b c \ '      ),     #    r'a b c \ '         ->    a b c \
			("  r'a \"b\" c \ '                  " , r'a "b" c \ '    ),     #    r'a "b" c \ '       ->    a "b" c \
			("  r\"a 'b' c \ \"                  " , r"a 'b' c \ "    ),     #    r"a 'b' c \ "       ->    a 'b' c \
			('  r\'a \\"b\\" c \ \'              ' , r'a \"b\" c \ '  ),     #    r'a \"b\" c \ '     ->    a \"b\" c \
			("  r\"a \\'b\\' c \ \"              " , r"a \'b\' c \ "  ),     #    r"a \'b\' c \ "     ->    a \'b\' c \

		]

		test_expressions( EXPRESSIONS, {}) 


	def test_executions(self):

		EXPRESSIONS = [
			( '$i'                                  , symbolTable['$i'] ), 
			( '$c'                                  , symbolTable['$c'] ), 
			( '$s'                                  , symbolTable['$s'] ), 
			( '$c'                                  , symbolTable['$c'] ), 
			( '$f'                                  , symbolTable['$f'] ), 

			( '$c+$c'                               , symbolTable['$c']+symbolTable['$c']),
			( '$i+$i'                               , symbolTable['$i']+symbolTable['$i']),

			( 'f("10")'                             , symbolTable['f'](symbolTable, "10")),
			( 'f(666)'                              , symbolTable['f'](symbolTable, 666)),

			( 'newSymbol("$a", 10) == $a'             , True),
			( 'newSymbol("$a", 10) == getSymbol("$a")', True),
			( 'hex(10)                              ' , '0xa'),

			( '$hex(10)                             ' , '0xa'),
			( '($hex)(10)                           ' , '0xa'),

			( 'getHex()(10)                         ' , '0xa'),
			( '(getHex())(10)                       ' , '0xa'),
			
			( 'isReachable("localhost")             ' , True),
			
			( 'byRegexpUnique("1.2.3.4", ".*" )     ' , "1.2.3.4.*"),


			( '1 + 1   == 2'                            , True),
			( '(1 + 1) == 2'                            , True),

		]

		test_expressions( EXPRESSIONS, symbolTable) 


	def test_snmpepxr(self):

		EXPRESSIONS = [
			( '1.2.3.4@               ' ,  1),
			( '1.2.3.4#               ' , -1),
			( '1.2.3.4@ + 1.2.3.4#    ' ,  0),
		]

		test_expressions( EXPRESSIONS, symbolTable, factory=MyFactory() ) 

	def test_dataid(self):

		EXPRESSIONS = [
			( '{data@}               ' ,  1),
			( '{data#}               ' , -1),
		]

		test_expressions( EXPRESSIONS, symbolTable, factory=MyFactory() ) 


	def test_ifthenelse(self):

		EXPRESSIONS = [
			( 'if True  then 2 else 3     ' ,  2),
			( 'if False then 2 else 3     ' ,  3),
			( 'if 1     then 2 else 3     ' ,  2),
			( 'if 0     then 2 else 3     ' ,  3),

			( 'if (1-1) then 2 else 3     ' ,  3),

			# Equivalenti
			( 'if 1 then  if 2 then 3 else 4  else 5', 3), 
			( 'if 1 then (if 2 then 3 else 4) else 5', 3), 

			# Equivalenti
			( 'if 1 then  if 0 then 3 else  if 4 then 5 else 6  else 7', 5), 
			( 'if 1 then  if 0 then 3 else (if 4 then 5 else 6) else 7', 5), 

			# Equivalenti
			( 'if  if True then 0 else 1  then "OK" else "NO"      ' ,  "NO"),
			( 'if (if True then 0 else 1) then "OK" else "NO"      ' ,  "NO"),

			# Equivalenti
			( 'if 1 then  if False then "OK" else "NO1"  else "NO2" ' ,  "NO1"),
			( 'if 1 then (if False then "OK" else "NO1") else "NO2" ' ,  "NO1"),

			( 'if (if True then 2 else 3) + 2 == 4 then "OK" else "NO"  ' ,  "OK"),

			# Tutte equivalenti
			( 'if   if False then 2 else   3 + 2  == 4            then "OK" else "NO"  ' ,  "NO"),
			( 'if  (if False then 2 else   3 + 2  == 4)  == True  then "OK" else "NO"  ' ,  "NO"),
			( 'if ((if False then 2 else ((3 + 2) == 4)) == True) then "OK" else "NO"  ' ,  "NO"),
			
			#
			# Precedenza dell'IF rispetto agli altri operatori
			#
			( 'if 1 then "OK" else "NO" == 2 '     ,  "OK"),
			( '(if 1 then "OK" else "NO") == "2" ' ,  False),

			( 'if 1 then 2 else 3 + 2 '            ,  2),
			( '(if 1 then 2 else 3) + 2 '          ,  4),

			( 'if 1 then 2 else 3 == 3 and False '   ,  2),
			( '(if 1 then 2 else 3) == 3 and False ' ,  False),



			( 'True and if 0 then True else False'          ,  False),

			# Equivalenti
			( 'True and if 1 then True else False and False'  ,  True),
			( 'True and if 1 then True else (False and False)',  True),

			( 'False or if 1 then True else False and False'  ,  True),
			( 'False or if 1 then True else (False and False)',  True),

			( 'False or if 0 then False else  True  or False'  ,  True),
			( 'False or if 0 then False else (True  or False)' ,  True),

			( 'False or  if 1 then False else True  or False'  ,  False),
			( 'False or (if 1 then False else True) or False'  ,  False),


			#
			# Inside SNMP EXPR
			#
			( '1.3.6.1.2.1.(if 1 then 1 else 0)@'              ,   1),
			( '1.3.6.1.2.1.(if 1 then 1 else 0)#'              ,  -1),

			( '1.3.6.1.2.1.(if(1)then(1)else(0))@'             ,   1),
			( '1.3.6.1.2.1.(if(1)then(1)else(0))."ciao"@'      ,   1),
			( '1.3.6.1.2.1.(if(1)then(1)else(0))."ciao"#'      ,  -1),

		]

		test_expressions( EXPRESSIONS, symbolTable, factory=MyFactory() ) 



	def test_try(self):

		EXPRESSIONS = [
			( 'try 1   catch ()' ,  1),

			( 'try exception("TEST1") catch ()' ,  "DummyException3:TEST1"),

			( 'try exception("TEST2") catch ( str($_exception_) )' ,  "TEST2"),
			
			( 'try exception("TEST3") catch PollException ("POOL3") catch Exception       ("BASE3")'                                    , "BASE3" ),
			( 'try exception("TEST4") catch PollException ("POOL4") catch DummyException1 ("DUMMY4") catch Exception       ("BASE4")  ' , "DUMMY4"),
			( 'try exception("TEST5") catch Exception     ("BASE5") catch PollException   ("POOL5")  catch DummyException1 ("DUMMY5") ' , "BASE5" ),
		]

		test_expressions( EXPRESSIONS, symbolTable, factory=MyFactory() ) 

		try:
			eval_string( 'try (try exception("TEST1") catch Exception ( if 1 then exception("NUOVA") else -1 )) catch ZeroDivisionError ()' , symbolTable , MyFactory())
		except DummyException, e:
			if str(e) == "NUOVA":
				pass
			else:
				raise e

if __name__ == '__main__':
    unittest.main()
