 
# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
:mod:`utils`
==================

.. autofunction:: parseString

"""

import antlr3

from . import ExprScriptLexer
from . import ExprScriptParser

#------------------------------------------------------------------------------#
# Funzioni di parsing                                                          #
#------------------------------------------------------------------------------#

def parseString(string, factory=None):
	"""
	Parse the input expression *string* and returns the generated expression
	tree.
	
	Expression tree's nodes are created by using the specified *factory* or the default :class:`BaseExprFactory`.
	
	:param string: The expression string.
	:type string: string
	
	:param factory: A factory objects (optional). If not specified 
	
	:returns: The :class:`ExprTree` object rappresenting the parsed expression.
	          given value can't be retrieved.
	
	:exceptions:
		:class:`LexicalError`
		:class:`ParsingError`
	"""
	stream  = antlr3.StringStream(string)

	#
	# Lexical analysis
	#
	lexer   = ExprScriptLexer.ExprScriptLexer(stream)
	tokens  = antlr3.CommonTokenStream(lexer)

	#
	# Parsing
	#
	parser  = ExprScriptParser.ExprScriptParser(tokens)
	if factory is not None:
		parser.setExprFactory(factory)

	expr_tree = parser.start()

	return expr_tree
