# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
################################################################################
BER utilities
################################################################################

TODO

"""

import binascii

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def tlv(s):
	# This is adapted from:
	# http://code.activestate.com/recipes/577244/

	try:
		tag = s.next()
	except StopIteration:
		return {}

	if ord(tag) & 0x1f == 0x1f:
		tag += s.next()
		while ord(tag[-1]) & 0x80 == 0x80: tag += s.next()

	length = ord(s.next())
	if length & 0x80 == 0x80:
		lendata = "".join([s.next() for i in range(length & 0x7f)])
		length = int(binascii.b2a_hex(lendata), 16)

	value = "".join([s.next() for i in range(length)])
	#return {ord(tag): value}
	return [ord(tag), value]

def tlvs(s):

	seq = (b for b in s)
	rv = list()
	while True:
		d = tlv(seq)
		if not d: break
		rv.append(d)

	return rv

def decode_integer(s):

	# Contents octets give the value of the integer, base 256, in two's complement form,
	# most significant digit first, with the minimum number of octets.
	# The value 0 is encoded as a single 00 octet. 

	s = map(lambda c: ord(c), s) 
	s.reverse()
	rv = 0
	if s[-1] > 127:
		for i in range(len(s)):
			rv = rv + (s[i] ^ 0xff ) * 256 ** i
		return -rv-1
	else:
		for i in range(len(s)):
			rv = rv + s[i] * 256 ** i
		return rv

def decode_oid(s):
	# BER encoding. Primitive. Contents octets are as follows, where value1, ..., valuen denote
	#	the integer values of the components in the complete object identifier:
	#	 The first octet has value 40 * value1 + value2. (This is unambiguous,
	#	 since value1 is limited to values 0, 1, and 2; value2 is limited to
	#	 the range 0 to 39 when value1 is 0 or 1; and, according to X.208, n is
	#	 always at least 2.)
	#
	#	 The following octets, if any, encode value3, ..., valuen. Each value
	#	 is encoded base 128, most significant digit first, with as few digits
	#	 as possible, and the most significant bit of each octet except the last
	#	 in the value's encoding set to "1." 
	# 
	# Example: The first octet of the BER encoding of RSA Data Security, Inc.'s object identifier
	# is 40 * 1 + 2 = 42 = 2a16. The encoding of 840 = 6 * 128 + 4816 is 86 48 and the encoding
	# of 113549 = 6 * 1282 + 7716 * 128 + d16 is 86 f7 0d. This leads to the following BER encoding:
	# 
	# 06 06 2a 86 48 86 f7 0d

	if len(s) < 3: return ''
 
	s = map(lambda c: ord(c), s)

	rv = '%d.%d' % (s[0] / 40, s[0] % 40)

	i = 1
	while True:
		j = i
		while s[j] > 127:
			s[j] = s[j] - 128
			j = j+1
			if j > len(s)-1: return ''
		r = s[i:j+1]
		r.reverse()
		#print('i=%d, j=%d, r=%r)' % (i,j,r))
		p = 0
		for k in range(len(r)):
			p = p + r[k] * 128 ** k
		rv = rv + '.' + str(p)
		i = j+1
		if i > len(s)-1: return rv

def decode(s):

	rv = tlvs(s)

	for p in rv:
		t = p[0]
		if t & 32: # Constructed bit
			p[1] = decode(p[1])
		elif t in (2, 67): # Integer
			p[1] = decode_integer(p[1])
		elif t == 6: # OID
			p[1] = decode_oid(p[1])

	return rv
