
# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
################################################################################
Daemonize utilities
################################################################################

TODO

"""

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

import sys
import imp
import os
import fcntl
import errno
#import time

import resource
import syslog

#-----------------------------------------------------------------------------#
# PYTHON FORK BUG WORK AROUND http://bugs.python.org/issue6380
#-----------------------------------------------------------------------------#

def __fork():
	imp.acquire_lock()
	try:
		#print "forking"
		return _os_fork()
	finally:
		imp.release_lock()

python_version = sys.version_info
if python_version[0] == 2 and ((python_version[1] >= 7) or (python_version[1] == 6 and python_version[2] >= 5)):
	# This trick shouldn't be necessary anymore since python 2.6.5 - Issue #7242 - The import lock is now reinitialized after fork.
	pass
else:
	#print "redefining os.fork (%s)" % (sys.version_info, )
	try:
		_os_fork
	except NameError:
		_os_fork = os.fork
		os.fork = __fork

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def fd_is_open(fd):
	try:
		return fcntl.fcntl(0, fcntl.F_GETFD) != -1
	except IOError, e:
		if e.errno == errno.EBADF:
			return False
	return True

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def _DEBUG_DEMONIZE(msg):
	#o = open("/tmp/debug_daemonize", "a+")
	#o.write(msg+"\n")
	#o.close()
	pass

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def daemonize(UMASK=0, WORKDIR="/", MAXFD=1024, lock_file=None, close_fds=True):
	"""
	TODO
	"""

	try:
		_DEBUG_DEMONIZE("FIRST FORK")
		pid = os.fork()
	except OSError, e:
		msg = "%s [%d]" % (e.strerror, e.errno)
		_DEBUG_DEMONIZE(msg)
		raise Exception(msg)

	if (pid != 0):
		os._exit(0)

	_DEBUG_DEMONIZE("SET SID")
	os.setsid()
	
	try:
		_DEBUG_DEMONIZE("SECOND FORK")
		pid = os.fork()
	except OSError, e:
		msg = "%s [%d]" % (e.strerror, e.errno)
		_DEBUG_DEMONIZE(msg)
		raise Exception(msg)
		
	if (pid != 0):
		os._exit(0)

	os.chdir(WORKDIR)

	os.umask(UMASK)

	if close_fds:
		
		maxfd = resource.getrlimit(resource.RLIMIT_NOFILE)[1]
		if (maxfd == resource.RLIM_INFINITY):
			maxfd = MAXFD
		
		fds = range(0, maxfd)

		_DEBUG_DEMONIZE("CLOSING %s fds" % (len(fds)))
		
		closed = 0
		for fd in fds:
			try:
				if fd_is_open(fd):
					os.close(fd)
					closed += 1
			except OSError, e:
				msg = "%s - %s:%s" % (fd , type(e), str(e))
				_DEBUG_DEMONIZE(msg)
			except Exception, e:
				msg = "%s - %s:%s" % (fd , type(e), str(e))
				_DEBUG_DEMONIZE(msg)
		
		_DEBUG_DEMONIZE("CLOSED %d fds" % (closed))
		
		if (hasattr(os, "devnull")):
			REDIRECT_TO = os.devnull
		else:
			REDIRECT_TO = "/dev/null"
		
		fd = os.open(REDIRECT_TO, os.O_RDWR)
		_DEBUG_DEMONIZE("dup2 %s to 1 e 2" % (fd))
		os.dup2(fd, 1)
		os.dup2(fd, 2)
		
		#os.dup2(fd, sys.stdin.fileno())
		#os.dup2(fd, sys.stdout.fileno())
		#os.dup2(fd, sys.stderr.fileno())

	if lock_file != None:
		fcntl.lockf(lock_file, fcntl.LOCK_EX | fcntl.LOCK_NB)


def launch_daemon(path, cl=None, log_syslog=False, env=None, pre_exec_callback=None):
	"""
	TODO
	"""
	_DEBUG_DEMONIZE("launch_daemon(): spwan subprocess")

	try:
		pid = os.fork()
	except OSError, e:
		msg = "%s [%d]" % (e.strerror, e.errno)
		_DEBUG_DEMONIZE("launch_daemon() error %s" % (msg))
		raise Exception(msg)

	if pid != 0:
		return

	_DEBUG_DEMONIZE("launch_daemon(): daemonizing")
	daemonize()

	if not cl:
		cl = [path]

	_DEBUG_DEMONIZE("launch_daemon(): execv()")
	
	# HACK: be sure syslog channel is open
	if log_syslog: 
		syslog.closelog()
		syslog.openlog()
	
	try:
		if log_syslog: syslog.syslog(syslog.LOG_INFO | syslog.LOG_USER, "%s launch_daemon(): EXECV %s PARAMS %s" % (__file__, path, cl))
		if env is not None:
			os.execve(path, cl, env)
		else:
			os.execv(path, cl)
	except Exception, e:
		if log_syslog: syslog.syslog(syslog.LOG_ERR  | syslog.LOG_USER, "%s launch_daemon(): ERROR EXECUTING %s PARAMS %s: %s:%s" % (__file__, path, cl, type(e), str(e)))
	
	#
	# We reach this point only on errors on the execv().
	#   - missing file?
	#   - no permission (r,w,x) ?
	#
	# We must exit now, but if we call os.exit()/sys.exit()
	# a SystemExit exception is raised and execution will return to calling routine 
	# and will probably trigger code/exception handlers that 
	# should not be called.
	
	os._exit(1)

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

if __name__ == "__main__":
	
	import sys
	import signal
	
	signal.signal(signal.SIGCHLD, signal.SIG_IGN)
	
	launch_daemon(sys.argv[1])
