# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
########################################################################
Python utilities
########################################################################
"""

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def load_symbol_from_module( module_fullname, symbol ):
	"""
	Loads from the module *module_fullname* the symbol *symbol*.
	
	Example:
		
		::
			
			class_object = load_symbol_from_module('datetime', 'timedelta')
	
	"""

	# Load module (dlopen())
	try:
		mod = __import__(module_fullname, {}, {}, [symbol])
	except ImportError, e:
		raise Exception('Error importing handler %s: "%s"' % (module_fullname, e))
	except ValueError, e:
		raise Exception('Error importing handler! Invalid Value: %s' % (e))

	# get symbol (dlsym())
	try:
		var = getattr(mod, symbol)
	except AttributeError:
		raise Exception('Module "%s" does not define a "%s" table! ' % (module_fullname, symbol))

	return var

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def load_symbol( module_full_symbol_name ):
	"""
	Loads the python symbol *module_full_symbol_name*.
	
	Example:
	
		::
			
			class_object = module_full_symbol_name('datetime.timedelta')
	
	"""
	pos = module_full_symbol_name.rfind('.')

	module = module_full_symbol_name[0:pos]
	name   = module_full_symbol_name[pos+1:]

	return load_symbol_from_module(module, name)

