# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
################################################################################
Global constants
################################################################################

TODO

"""

import re
import types

DEFAULT_TIMEZONE = 'Europe/Rome'

#-----------------------------------------------------------------------------#
# USER & GROUPS                                                               #
#-----------------------------------------------------------------------------#

VALID_USER_NAME  = re.compile(r"^[a-z0-9._-]+$")
VALID_GROUP_NAME = re.compile(r"^[a-z0-9._-]+$")

#name_re = re.compile(r"^[a-z0-9._-]+$")
#name_re_errmsg = "User and group names must be made of lowercase ascii letters, digits, dots, underscores, hyphens"


#-----------------------------------------------------------------------------#
# Tenant                                                                      #
#-----------------------------------------------------------------------------#

VALID_TENANT_NAME = re.compile(r"^[a-z]+[a-z0-9]*$")

DEFAULT_PRIMARY_TENANT_NAME      = 'site'
DEFAULT_PRIMARY_TENANT_LONG_NAME = 'Main site'

#-----------------------------------------------------------------------------#
# Priority levels                                                             #
#-----------------------------------------------------------------------------#

PRIORITY_NONE = 0

DEFAULT_PRIORITY = PRIORITY_NONE

PRIORITY_MIN     = 1
PRIORITY_MAX     = 100

VALID_PRIORITY_LEVEL = re.compile(r"^[a-zA-Z]+([-.][a-zA-Z0-9]+)*$")

#-----------------------------------------------------------------------------#
# Clusters                                                                    #
#-----------------------------------------------------------------------------#

CLUSTER_ROLE_ACTIVE  = 0
CLUSTER_ROLE_PASSIVE = 1

CLUSTER_ROLE_MASTER    = CLUSTER_ROLE_ACTIVE
CLUSTER_ROLE_DEPENDENT = CLUSTER_ROLE_PASSIVE

CLUSTER_ROLE_STR = {
	CLUSTER_ROLE_MASTER   : 'master',
	CLUSTER_ROLE_DEPENDENT: 'dependent',
}

VALID_CLUSTER_NAME = re.compile(r"^[a-z][a-z0-9]*([-.][a-z0-9]+)*$")

#-----------------------------------------------------------------------------#
# Agent                                                                       #
#-----------------------------------------------------------------------------#

DEFAULT_AGENT_NAME = 'main-agent'

VALID_AGENT_NAME = re.compile(r"^[a-z]+([-.][a-z0-9]+)*$")

AGENT_DEFAULT_NUM_THREADS = 20

#-----------------------------------------------------------------------------#
# Monitored elements                                                          #
#-----------------------------------------------------------------------------#

ELEMENT_KIND_NODE    = 1
ELEMENT_KIND_IFACE   = 2
ELEMENT_KIND_STORAGE = 3
ELEMENT_KIND_SERVICE = 4
ELEMENT_KIND_DEVICE  = 5

ELEMENT_KIND_STR = {
	ELEMENT_KIND_NODE    : 'node',
	ELEMENT_KIND_IFACE   : 'interface',
	ELEMENT_KIND_STORAGE : 'storage',
	ELEMENT_KIND_SERVICE : 'service',
	ELEMENT_KIND_DEVICE  : 'device',
}

ELEMENT_KINDS_STR = ELEMENT_KIND_STR.values()
ELEMENT_KINDS_IDS = ELEMENT_KIND_STR.keys()

ELEMENT_KIND_VAL = {
	ELEMENT_KIND_STR[ELEMENT_KIND_NODE] : ELEMENT_KIND_NODE,
	ELEMENT_KIND_STR[ELEMENT_KIND_IFACE] : ELEMENT_KIND_IFACE,
	ELEMENT_KIND_STR[ELEMENT_KIND_STORAGE] : ELEMENT_KIND_STORAGE,
	ELEMENT_KIND_STR[ELEMENT_KIND_SERVICE] : ELEMENT_KIND_SERVICE,
	ELEMENT_KIND_STR[ELEMENT_KIND_DEVICE] : ELEMENT_KIND_DEVICE,
}


VALID_ELEMENT_NAME   = re.compile(r"^[a-zA-Z0-9]+([-.][a-zA-Z0-9]+)*$")

VALID_DNS_NODE_NAME  = re.compile(r"^([a-zA-Z0-9-])+(\.([a-zA-Z0-9-])+)*$")
VALID_IPV6_NODE_NAME = re.compile(r"^((([0-9A-Fa-f]{1,4}:){7}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){6}:[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){5}:([0-9A-Fa-f]{1,4}:)?[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){4}:([0-9A-Fa-f]{1,4}:){0,2}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){3}:([0-9A-Fa-f]{1,4}:){0,3}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){2}:([0-9A-Fa-f]{1,4}:){0,4}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){6}((\b((25[0-5])|(1\d{2})|(2[0-4]\d)|(\d{1,2}))\b)\.){3}(\b((25[0-5])|(1\d{2})|(2[0-4]\d)|(\d{1,2}))\b))|(([0-9A-Fa-f]{1,4}:){0,5}:((\b((25[0-5])|(1\d{2})|(2[0-4]\d)|(\d{1,2}))\b)\.){3}(\b((25[0-5])|(1\d{2})|(2[0-4]\d)|(\d{1,2}))\b))|(::([0-9A-Fa-f]{1,4}:){0,5}((\b((25[0-5])|(1\d{2})|(2[0-4]\d)|(\d{1,2}))\b)\.){3}(\b((25[0-5])|(1\d{2})|(2[0-4]\d)|(\d{1,2}))\b))|([0-9A-Fa-f]{1,4}::([0-9A-Fa-f]{1,4}:){0,5}[0-9A-Fa-f]{1,4})|(::([0-9A-Fa-f]{1,4}:){0,6}[0-9A-Fa-f]{1,4})|(([0-9A-Fa-f]{1,4}:){1,7}:))$")

VALID_NODE_NAME      = VALID_DNS_NODE_NAME
VALID_IFACE_NAME     = VALID_ELEMENT_NAME
VALID_STORAGE_NAME   = VALID_ELEMENT_NAME
VALID_SERVICE_NAME   = VALID_ELEMENT_NAME
VALID_DEVICE_NAME    = VALID_ELEMENT_NAME


VALID_ELEMENT_NAME_BY_KIND = {
	ELEMENT_KIND_IFACE  : VALID_IFACE_NAME  ,
	ELEMENT_KIND_STORAGE: VALID_STORAGE_NAME,
	ELEMENT_KIND_SERVICE: VALID_SERVICE_NAME,
	ELEMENT_KIND_DEVICE : VALID_DEVICE_NAME ,
}

#-----------------------------------------------------------------------------#
# Links                                                                       #
#-----------------------------------------------------------------------------#

LINK_TYPE_VIRTUAL = 0
LINK_TYPE_L2      = 2
LINK_TYPE_L3      = 3

VALID_LINK_NAME = re.compile(r"^[a-z0-9][a-z0-9]*([-.\:][a-z0-9]+)*$")

LINK_TYPE_STR = {
	LINK_TYPE_VIRTUAL:'virtual',
	LINK_TYPE_L2     :'physical',
	LINK_TYPE_L3     :'network',
}

#-----------------------------------------------------------------------------#
# Params                                                                      #
#-----------------------------------------------------------------------------#

PARAM_TYPE_STR       = 1 
PARAM_TYPE_INT       = 2 
PARAM_TYPE_FLOAT     = 3
PARAM_TYPE_BOOL      = 4 

PARAM_BOOL_TRUE  = '1'
PARAM_BOOL_FALSE = '0'

def _bool(strval):
	if strval == PARAM_BOOL_TRUE:
		return True
	elif strval == PARAM_BOOL_FALSE:
		return False
	raise Exception("Invalid bool value for parameter: '%s' (type %s)" % (strval, type(strval)))

_CONVERT_FUNCTIONS_DICT = {
	PARAM_TYPE_STR       : str,
	PARAM_TYPE_INT       : int,
	PARAM_TYPE_FLOAT     : float,
	PARAM_TYPE_BOOL      : _bool , 
}

def convert_to_param_type( vtype, value ):
	return _CONVERT_FUNCTIONS_DICT[vtype](value)
	
VALID_PARAM_NAME     = re.compile('^[a-z][a-z0-9_]*[a-z0-9]$')



#-----------------------------------------------------------------------------#
# Datasources                                                                 #
#-----------------------------------------------------------------------------#

VALID_TEMPLATE_NAME      = re.compile(r"^[a-zA-Z0-9][a-zA-Z0-9-_]*$")

VALID_DATAGROUP_NAME     = re.compile(r"^[a-zA-Z][a-zA-Z0-9-_]*$")
VALID_DATASOURCE_NAME    = re.compile(r"^[a-zA-Z][a-zA-Z0-9-_]*$")
VALID_CONDITION_NAME     = re.compile(r"^[a-zA-Z][a-zA-Z0-9-_]*$")
VALID_TIMEGRAPH_NAME     = re.compile(r"^[a-zA-Z][a-zA-Z0-9-_]*$")
VALID_SERIE_NAME         = re.compile(r"^[a-zA-Z][a-zA-Z0-9_]*$")


DATASOURCE_STATUS_OK          = 0     # value fetched successfully
DATASOURCE_STATUS_STRANGE_VAL = 10    # value (or rate) outside datasource valid range
DATASOURCE_STATUS_CASCADE     = 20    # value not calculated for cascade error
DATASOURCE_STATUS_DEPENDSON   = 30    # value not calculated for dependson
DATASOURCE_STATUS_UNCHECKABLE = 40    # value not calculated 
DATASOURCE_STATUS_ERROR       = 50    # value noc calculated because of unknown error

#DATASOURCE_STATUS_IN         = 60    # FIX: forse sarebbe il caso di definire questo stato anche per i datasource altrimenti non distinguo i datasource "sospesi"
#DATASOURCE_STATUS_UNKNOWN    = 70    # FIX: servirebbe per sapere quando un datasource COUNTER (rate) non e' calcolabile perche' non ho il dato precedente

_DSSTATE_TO_CODENAME = {
	DATASOURCE_STATUS_OK          : 'OK',
	DATASOURCE_STATUS_STRANGE_VAL : 'SV',
	DATASOURCE_STATUS_CASCADE     : 'CA',
	DATASOURCE_STATUS_DEPENDSON   : 'DO',
	DATASOURCE_STATUS_UNCHECKABLE : 'UN',
	DATASOURCE_STATUS_ERROR       : 'ER',
	
	#DATASOURCE_STATUS_IN          : 'IN', # FIX:
}

_DSSTATE_FROM_CODENAME = dict([(v,k) for k,v in _DSSTATE_TO_CODENAME.items()])

def DSSTATE_TO_CODENAME(s):
	return _DSSTATE_TO_CODENAME[s]

#-----------------------------------------------------------------------------#
# CONDITIONS                                                                  #
#-----------------------------------------------------------------------------#

CSTATE_NONE = None

CSTATE_DN = 10

CSTATE_FA = 20

CSTATE_UD = 30
CSTATE_UU = 40

CSTATE_DD = 45
CSTATE_DU = 46

CSTATE_TD = 50
CSTATE_TU = 60

CSTATE_IN = 70

CSTATE_UP = 100

CSTATE_DEFAULT = CSTATE_UU

# Macro stati DOWN
CSTATE_MACRO_DOWN = [CSTATE_DN, CSTATE_UD, CSTATE_DD]

# Macro stato UP
# NOTA: il FAILING e' considerato uno stato "ancora" UP.
CSTATE_MACRO_UP   = [CSTATE_UP, CSTATE_UU, CSTATE_DU, CSTATE_FA ]


_CSTATE_TO_CODENAME = {
        CSTATE_NONE: "NONE",
	CSTATE_DN: "DN",
	CSTATE_FA: "FA",
	CSTATE_UD: "UD", CSTATE_UU: "UU",
	CSTATE_DD: "DD", CSTATE_DU: "DU",
	CSTATE_TD: "TD", CSTATE_TU: "TU",
	CSTATE_IN: "IN",
	CSTATE_UP: "UP",
}

CODENAME_INVALID = "INVALID"

CSTATES_UNCHECKABLE = [ CSTATE_UD, CSTATE_UU, CSTATE_DD, CSTATE_DU, CSTATE_TD, CSTATE_TU ]


CSTATE_CODENAMES = _CSTATE_TO_CODENAME.values()

_CSTATE_FROM_CODENAME = dict([(v,k) for k,v in _CSTATE_TO_CODENAME.items()])


def CSTATE_TO_CODENAME(s):
	return _CSTATE_TO_CODENAME[s]

def CSTATE_FROM_CODENAME(s):
	try:
		return _CSTATE_FROM_CODENAME[s]
	except:
		return CODENAME_INVALID

def is_alarm_transition(old_state, new_state):
	#return is_down_transition(old_state, new_state)    or \
	#       is_failing_transition(old_state, new_state) or \
	return is_down_transition(old_state, new_state)    or \
	       is_up_transition(old_state, new_state)

def is_alarm_ack_needed(old_state, new_state):

	t = (old_state, new_state)
	#return  ( t in settings.TRANSITION_MACRO_STATES)
	return True

def is_up_transition(old_state, new_state):
	return ((old_state in [CSTATE_DN, CSTATE_TD, CSTATE_UD, CSTATE_DD]) and (new_state == CSTATE_UP))
	
def is_failing_transition(old_state, new_state):
	return ((old_state in [CSTATE_UP, CSTATE_FA, CSTATE_TU, CSTATE_UU, CSTATE_DU]) and (new_state == CSTATE_FA))
	
def is_down_transition(old_state, new_state):
	return ((old_state in [CSTATE_UP, CSTATE_FA, CSTATE_TU, CSTATE_UU, CSTATE_DU]) and (new_state == CSTATE_DN))
	
def is_notable_transition(old_state, new_state):
	
	# Le transizioni all'interno degli stati "down" (o "up") non ci interessano
	if old_state in CSTATE_MACRO_DOWN and new_state in CSTATE_MACRO_DOWN:
		return False

	if old_state in CSTATE_MACRO_UP and new_state in CSTATE_MACRO_UP:
		return False
	
	# Qualunque transizione da/verso macro stati "up" e "down" ci interessa.
	return True	

#-----------------------------------------------------------------------------#
# TIMEGRAPHS                                                                  #
#-----------------------------------------------------------------------------#

DEFAULT_TIMEGRAPH_SERIE_COLOR = ''

TIMEGRAPH_SERIE_STYLE_LINE      = 'line'
TIMEGRAPH_SERIE_STYLE_AREA      = 'area'
TIMEGRAPH_SERIE_STYLE_TICK      = 'tick'
TIMEGRAPH_SERIE_STYLE_INVISIBLE = 'invisible'

#-----------------------------------------------------------------------------#
# Events                                                                      #
#-----------------------------------------------------------------------------#

EVENT_CONDITION = 1
EVENT_SYSLOG    = 2
EVENT_SNMPTRAP  = 3
EVENT_CUSTOM    = 4

#-----------------------------------------------------------------------------#
# Alarms                                                                      #
#-----------------------------------------------------------------------------#

ALARM_NORMAL = 'normal'
ALARM_PUSHED = 'pushed'

#-----------------------------------------------------------------------------#
# PERMISSIONS                                                                 #
#-----------------------------------------------------------------------------#

AUTH_READONLY  = 1
AUTH_READWRITE = 2

#-----------------------------------------------------------------------------#
# Directories                                                                 #
#-----------------------------------------------------------------------------#

DIRENTRY_AUTH_READONLY  = AUTH_READONLY
DIRENTRY_AUTH_READWRITE = AUTH_READWRITE

VALID_TAGTREE_NAME      = re.compile(r"^[a-zA-Z][a-zA-Z0-9-_]*[a-zA-Z0-9-_]$")

#-----------------------------------------------------------------------------#
# Directories                                                                 #
#-----------------------------------------------------------------------------#

MAP_TYPE_LOGIC = 1
MAP_TYPE_DIR   = 2  # obsolete
MAP_TYPE_TAG   = 3

MAPENTITY_TYPE_SIMPLE     = 1
MAPENTITY_TYPE_BOX        = 2
#MAPENTITY_TYPE_CONTAINER  = MAPENTITY_TYPE_BOX

#-----------------------------------------------------------------------------#
# CONFIGURATION                                                               #
#-----------------------------------------------------------------------------#

SOURCE_LIBRARY = 1
SOURCE_LOCAL   = 2

SOURCE_DESCR = {
	SOURCE_LIBRARY : {'name': 'library', 'descr': 'Library'},
	SOURCE_LOCAL   : {'name': 'site'   , 'descr': 'Site'},
}

def SOURCE_NAME(source):
	return SOURCE_DESCR[source]['name']


PATH_PART_ELEMENT_POS   = 0
PATH_PART_TEMPLATE_POS  = 1
PATH_PART_DATAGROUP_POS = 2

DATAGROUP_PATH_SEP    = ';'
DATAGROUP_PATH_SUBSEP = ':'



#-----------------------------------------------------------------------------#
# FIELDS                                                                      #
#-----------------------------------------------------------------------------#

FIELD_TYPE_STR       = 'str'
FIELD_TYPE_INT       = 'int'
FIELD_TYPE_FLOAT     = 'float'
FIELD_TYPE_BOOL      = 'bool'
FIELD_TYPE_TIMESTAMP = 'timestamp'


def _fbool(v):
	if v == 'true':# or v == '1':
		return True
	elif v == 'false':# or v == '0':
		return False
	raise Exception("Invalid bool value '%s' for field type bool" % (v))
	
def _int(v):
	try:
		return int(float(v))
	except Exception, e:
		pass	
	try:
		return int(v)
	except Exception, e:
		raise e

_FIELD_CONVERT_FUNCTIONS_DICT = {
	FIELD_TYPE_STR       : str,
	FIELD_TYPE_INT       : _int,
	FIELD_TYPE_FLOAT     : float,
	FIELD_TYPE_BOOL      : _fbool , 
	FIELD_TYPE_TIMESTAMP : _int,
}

def convert_to_field_type( ftype, value ):
	return _FIELD_CONVERT_FUNCTIONS_DICT[ftype]( value )

VALID_PARAM_NAME     = re.compile('^[a-z][a-z0-9_]*[a-z0-9]$')


def get_store_repr_value(value):
	
	if isinstance(value, types.BooleanType):
		return str(value).lower()

	if isinstance(value, types.IntType) or isinstance(value, types.IntType):
		return str(value)

	if isinstance(value, types.FloatType):
		return str(value)

	if isinstance(value, types.StringType):
		return value


	raise Exception("Unable to get store rappresentation for value: %s:%s" % (type(value),value))


#-----------------------------------------------------------------------------#
# CLASSIFICATIONS                                                             #
#-----------------------------------------------------------------------------#

DEFAULT_IFACE_CLASSIFICATION_TRAFFIC_IN  = 'network.interface.traffic.in'
DEFAULT_IFACE_CLASSIFICATION_TRAFFIC_OUT = 'network.interface.traffic.out'
