# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.
"""
########################################################################
Text table formatting utilities
########################################################################
"""

import sys, operator, fcntl, struct, os, tty, copy
import types
import cgi

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def html_escape(s):
	"""
	TODO
	"""
	return cgi.escape(s).encode('ascii', 'xmlcharrefreplace')

def safe_unicode(e):
	"""
	TODO
	"""
	try:
		return unicode(e)
	except UnicodeDecodeError:
		return e.__repr__()

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class table(object):
	"""
	TODO
	"""

	def __init__(self, headers, column_widths=None, rows=[]):
		
		# headers is a list of labels
		super(table, self).__init__()
		
		self.headers              = headers
		self.headers_num_by_value = {}
		self.width                = len(self.headers)
		self.column_widths        = map(lambda x: len(x), self.headers)
		self.byvalue              = dict()
		self.row_count            = 0
		self.rows                 = []
		
		i = 0
		for header in headers:
			if self.byvalue.has_key(header):
				raise ValueError("Duplicate header label %r" % header)
			self.byvalue[header] = dict()
			self.headers_num_by_value[header] = i
			i = i+1
			
		for row in rows:
			self.add_row(row)

	def add_row(self, row):

		if len(row) != self.width:
			raise ValueError("Wrong count of items in row (want %d, got %d)" % (self.width, len(row)))

		for i in range(self.width):
			if row[i] is None:
				row[i] = ''
			try:
				if self.byvalue[self.headers[i]].has_key(row[i]):
					self.byvalue[self.headers[i]][row[i]].append(self.row_count)
				else:
					self.byvalue[self.headers[i]][row[i]] = [self.row_count]
			except TypeError:
				pass
		self.rows.append(copy.copy(row))
		self.row_count = self.row_count + 1

	def get_rows(self):
		return self.rows

	def __len__(self):
		return self.row_count

	def get_header_num(self, header):
		try:
			c = self.headers_num_by_value[header]
		except KeyError:
			raise ValueError("No header label %r" % header)
		return c

	def indexes_by_value(self, header, value):
		try:
			c = self.headers_num_by_value[header]
		except KeyError:
			raise ValueError("No header label %r" % header)
		try:
			return self.byvalue[header][value]
		except KeyError:
			return []

	def rows_by_value(self, header, value):
		return map(lambda x: self.rows[x], self.indexes_by_value(header, value))

	def row_by_index(self, index):
		return self.rows[index]

	def column_by_index(self, index):
		return map(lambda x: x[index], self.rows)

	def column_by_header(self, header):
		try:
			c = self.headers_num_by_value[header]
		except KeyError:
			raise ValueError("No header label %r" % header)
		return self.column_by_index(c)

	def text(self, max_width=None, separator=None):


		
		if max_width is None:
			if sys.stdout.isatty():
				buf = 'abcdefgh'
				buf = fcntl.ioctl(sys.stdout.fileno(), tty.TIOCGWINSZ, buf)
				row, col, rpx, cpx = struct.unpack('hhhh', buf)
			else:
				col = int(os.environ.get('COLUMNS', 0))
				
			max_width = col - 5
			
		if separator is None:
			separator = os.environ.get('COLSEP', ' ')
			
			
		col_widths = self.column_widths 
		
		verses = map(lambda x: '', self.headers) # '' right, '-' left
		numerics = len(self.headers)
		
		for row in self.rows:
			for i, el in enumerate(row):
				
				#
				# Se la cella contiene una tupla di 3 elementi allora
				# la tupla contiene,
				#     ( color_function, color_code, cell value)
				#
				if isinstance(el, types.TupleType) or isinstance(el, types.ListType):
					if len(el) == 3:
						color_fun, val, el = el
				
				if verses[i] == '' and not safe_unicode(el).isdigit():
					verses[i] = '-'
					numerics = numerics - 1
					
			if numerics <= 0:
				break
				
		max_col_widths = copy.copy(col_widths)
		
		for row in self.rows:
			for i,el in enumerate(row):
				
				#
				# Se la cella contiene una tupla di 3 elementi allora
				# la tupla contiene,
				#     ( color_function, color_code, cell value)
				#
				if isinstance(el, types.TupleType) or isinstance(el, types.ListType):
					if len(el) == 3:
						color_fun, val, el = el
				
				el = safe_unicode(el)
				max_col_widths[i] = max(max_col_widths[i], len(el))
				
		if max_width and sum(max_col_widths) >= max_width:
			remaining = max_width - sum(col_widths)
			if remaining > 0:
				diff_cols_widths = map(lambda x : sum([x[0],-x[1]]) , zip(max_col_widths, col_widths))
				s = sum(diff_cols_widths)
				diff_cols_percent = map(lambda x : operator.truediv(x, s), diff_cols_widths)
				for i in range(self.width):
					col_widths[i] += int(remaining*diff_cols_percent[i])
		else:
			col_widths = max_col_widths
			
		cell_template = []
		row_template = u""
		for i in range(self.width):
			
			ct = u"%" + verses[i] + str(col_widths[i]) + "s" + separator
			cell_template.append(ct)
			row_template += ct
			
		#row_template += separator
		#row_template = row_template[:-1] + "\n"
		
		row_template = row_template + "\n"
		
		#print "TEMPLATE", row_template
		
		rv = u""
		rv += row_template % tuple(self.headers)
		rv += row_template % tuple(map(lambda col_width : "-"*col_width, col_widths))
		
		for row in self.rows:
			row_cut = []
			for i, el in enumerate(row):
				
				#
				# Se la cella contiene una tupla di 3 elementi allora
				# la tupla contiene,
				#     ( color_function, color_code, cell value)
				#
				color_fun = None
				if isinstance(el, types.TupleType) or isinstance(el, types.ListType):
					if len(el) == 3:
						color_fun, val, el = el
				
				el = safe_unicode(el)
				col_width = col_widths[i]
				if len(el) > col_width:
					el = el[:col_width-1] + "+"


				el = cell_template[i] % (el)

				if color_fun is not None:
					el = color_fun(val, el)
					
				#row_cut.append(el)
				rv += el
					
			#rv += row_template % tuple(row_cut) 
			rv += '\n'
			
		return rv

	def html(self, tableid=None):
		rv=''
		if tableid:
			attr = 'id="%s"' % tableid
		else:
			attr=''
		rv = rv + '<table border=1 %s><thead><tr>' % attr
		for header in self.headers:
			rv = rv + '<th>' + html_escape(header) + '</th>'
		rv = rv + '</tr></thead><tbody>\n'
		for row in self.rows:
			rv = rv + '<tr>'
			for el in row:
				rv = rv + '<td>' + html_escape(safe_unicode(el)) + '</td>'
			rv = rv + '</tr>\n'
		rv = rv + '</tbody></table>'
		return rv

if __name__ == '__main__':
	width=10
	height=400000
	minint=1
	maxint=10
	hdrs=[]
	import random
	random.seed()
	print("Making a table with %d columns and %d rows, filled with random integers between %d and %d..." % (width, height, minint, maxint))
	for i in range(width):
		hdrs.append('Head%02d' % i)
	t = table(hdrs)
	for i in range(height):
		row = []
		for j in range(width):
			row.append(random.randint(minint,maxint))
		t.add_row(row)
	print("Done, press Enter to exit.")
	raw_input()
	
		
