# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
########################################################################
UUIDs utilities
########################################################################

========================================================================
Introduction
========================================================================

This module defines functions for UUIDs manipulation.

The canonical rappresentation of an UUID is a 32-char string with 4 separators ('-').

::
	
	550e8400-e29b-41d4-a716-446655440000

The UUIDs returnded by these functions do not contain the separators:

::
	
	550e8400e29b41d4a716446655440000
	<----------- 32 char ---------->

========================================================================
Functions
========================================================================
"""

import uuid

#-----------------------------------------------------------------------------#
# CONSTANTS                                                                   #
#-----------------------------------------------------------------------------#

#: Max length of a UUID
MAX_UUID_LEN = 32

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def create_global_uuid():
	"""
	Returns a new UUID string.
	
	:returns: a UUID string
	:rtype: string
	
	:exception: 
	        None
	"""

	return uuid.uuid4().hex

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def is_uuid(s):
	"""
	Returns *true* if *s* is a valid UUID string.
	
	:param  s:  A char string
	:type s:  string
	
	:returns: *true* or *false*
	:rtype: boolean
	
	:exception: 
	        None
	"""

	if len(s) == MAX_UUID_LEN and s.isalnum():
		return True
	else:
		return False
