# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os
import re
import sys

from setuptools import setup
from setuptools import find_packages

from setuptools import setup


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

from setuptools.command.install import install

class MyInstall(install):

	def run(self):

		print "INSTALLING MANUALLY ANTLR3"
		
		ret = os.system("easy_install ./required_packages/antlr_python_runtime-3.1.3-py2.5.egg")
		if ret is not 0:
			# Easy_install returns an error, but antlr3 is probably fine. Check if we can import it.
			ret = os.system("python -c 'import antlr3'")
			if ret is not 0:
				print >> sys.stderr, "#" * 80
				print >> sys.stderr, "#" * 80
				print >> sys.stderr, "# WARNING: someting gone wrong while installing antlr3 python package. "
				print >> sys.stderr, "#          "
				print >> sys.stderr, "#          Please verify antlr3 packages was installed properly"
				print >> sys.stderr, "#          "
				print >> sys.stderr, "#                 import antlr3"
				print >> sys.stderr, "#" * 80
				print >> sys.stderr, "#" * 80
			
		install.run(self)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def extract_symbol(version_file, symbol):

	# __version__ = "...."
	# __version__ = '....'
	VSRE = r"^__%s__\s*=\s*['\"]([^'\"]*)['\"]" % (symbol)

	verstrline = open(version_file, "rt").read()

	mo = re.search(VSRE, verstrline, re.M)
	if not mo:
		raise RuntimeError("Unable to find %s string in %s." % (symbol, version_file,))

	return mo.group(1)

def get_version_from_file():
	try:
		f = open('VERSION')
		data = f.read()
		f.close()
		return data.strip()
	except Exception, e:
		return None

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

print "Reading version from file"
VERSION = get_version_from_file()
if not VERSION:
	print "Reading version from sourcecode"
	VERSION = extract_symbol('sanet_common/__init__.py', 'version')

if not VERSION:
	raise Exception("Unable to get version")

print "Package version: ", VERSION
print "Found following packages:"
PACKAGES = find_packages()
for name in PACKAGES:
	print "\t", name


REQUIREMENTS = [


	# Queste dipendenze dovrebbero essere "suggested" perche' se vuoi installare sanet-common sincolarmente
	# forse non ti servono, ma e' quasi sicuro che Sanet le usi nel 99.999999% dei casi quindi rendiamole
	#  obbligatorie cosi' nessuno se ne dimentica (e non bestemmia dopo perche' qualcosa non funziona)

	# apt-get intall python-ipaddr
	'ipaddr',

	# apt-get install python-setproctitle
	'setproctitle',

	# Nota: Il server redis quasi certamente serve, ma non sarebbe comunque installabile con setuptools.
	#       Per scrupolo basterebbe questo:
	#
	# 		apt-get install 'redis-server'     
	#
	# apt-get install 'python-redis'
	'redis' , 

	#
	# DIPENDENZA SUGGESTED  ma quando installiamo il modulo con setuptools meglio
	#                       gestirla come OBBLIGATORIA cosi' a posteriori non 
	#                       ci saranno problemi di moduli utili ma mancanti perche'
	#                       dimenticati.
	#
	# apt-get install python-ldap
	'python-ldap',


	#
	# DIPENDENZA SUGGESTED  ma quando installiamo il modulo con setuptools meglio
	#                       gestirla come OBBLIGATORIA cosi' a posteriori non 
	#                       ci saranno problemi di moduli utili ma mancanti perche'
	#                       dimenticati.
	#
	# apt-get install python-pyrad
	'pyrad',
	
	#
	# DIPENDENZA CRITICA:      sanet_common contiene moduli pensati per django
	#                          ma sono moduli "opzionali" di autenticazione 
	#                          pensati per Sanet.
	#                          Questa dovrebbe essere SUGGESTED ma non
	#                          c'e' modo con setuptools di rendere la dipendenza
	#                          opzionale.
	#                          Chi installera' sanet installera anche sanet-common + django
	#                          quindi questa dipendenza in un modo o nell'altro
	#                          verra' comunque soddisfatta.
	#
	#apt-get install django
	#'django==1.4.5',

	'django==1.11.15',
	
	
	# ANTLR 3
	# ===============================
	#
	#      DISABLED: setup_tools can not find antl3 package.
	#                we'll install it manually. See MyInstall() class.
	#
	#'antlr==3.1.3' DISABLED
	
]

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

setup(
	cmdclass={'install': MyInstall},
	
	name             = 'sanet-common',
	version          = VERSION,
	description      = 'SANET3 Common libraries used by server and agents',

	packages         = PACKAGES,
	
	install_requires = REQUIREMENTS,

	# Metadata
	author           = 'dbilli',
	author_email     = 'dbilli@labs.it',
	url              = 'http://labs.it/',
	license          = "GNU",
)

