# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os
import re

from setuptools import setup
from setuptools import find_packages

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def extract_symbol(version_file, symbol):

	# __version__ = "...."
	# __version__ = '....'
	VSRE = r"^__%s__\s*=\s*['\"]([^'\"]*)['\"]" % (symbol)

	verstrline = open(version_file, "rt").read()

	mo = re.search(VSRE, verstrline, re.M)
	if not mo:
		raise RuntimeError("Unable to find %s string in %s." % (symbol, version_file,))

	return mo.group(1)

def get_version_from_file():
	try:
		f = open('VERSION')
		data = f.read()
		f.close()
		return data.strip()
	except Exception, e:
		return None
		
#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

print "Reading version from file"
VERSION = get_version_from_file()
if not VERSION:
	print "Reading version from sourcecode"
	VERSION = extract_symbol('sanet_libsnmp/__init__.py', "version")

if not VERSION:
	raise Exception("Unable to get version")

print "Package version: ", VERSION
print "Found following packages:"
PACKAGES = find_packages()
for name in PACKAGES:
	print "\t", name


REQUIREMENTS = [

	#
	# NETSNMP
	# =================
	# 
	#     DISABLED: setuptools can not find the right packages (python setup.py install)
	#
	#		        The module MUST BE INSTALLED WITHOUT PYTHON TOOLS.
	#
	# DEBAIN
	# -------------
	#
	# Under Debian 9 the python package is installed with:
	# 
	#        apt-get install python-netsnmp
	#
	# but it is called 'netsnmp_python' instead of 'netsnmp' 
	#        

	#'netsnmp>=5.6',    DISABLED
	
]

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

setup(
	name             = 'sanet_libsnmp',
	version          = VERSION,
                         
	description      = 'SANET3 SNMP Library',
                         
	packages         = PACKAGES,
	
	install_requires = REQUIREMENTS,

	# Metadata
	author           = 'Diego Billi',
	author_email     = 'dbilli@labs.it',
	url              = 'http://labs.it/',
	license          = "GNU",
)

