# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

AgentUpdateEvent
==================

Un oggetto AgentUpdateEvent rappresenta un singolo aggiornamento "atomico" che un agente
produce dopo aver effettuato il check di un datagroup.

        PDataGroup --->  DataGroupChecker
                                |
                                |.get_datagroup_all_updates()
                                |
                                V
                          [DA, DA, ...]  update data
                          \___________/
                                |
                        AgentUpdateEvent

Un evento e' un oggetto di tipo:

	{
		'ts'       : <ts>,
		'tenant_id': <tenant_id>,
		'agent_id' : <agent_id>,
		'data'     : <data>,
	}

Il DataGroupChecker produce una lista di blocchi dati, ogniuno che rappresenta un 
singolo datagroup con il metodo get_datagroup_all_updates(). 

Questa lista viene incapsulata all'interno dell'oggetto AgentUpdateEvent.

Questo e' un sempio di "blocchi" di aggiornamenti:

	data = [

		RESUME DI UN DATAGROUP

		{
		  'additional_updates': None,
		  'conditions_updates': {u'8e84342846c94ea8a12731489423b37f': {'fields': {'lastdone': 1526628198.695635,
											  'mtime': 1526628192,
											  'status': 100,
											  'statuslast': 70,
											  'statuslastchange': 1526628198.695635},
									       'infos': ['Resumed by poller (at 1526628198.7)'],
									       'smtime': 1526628192,
									       'time': 1526628198.695635,
									       'uuid': u'8e84342846c94ea8a12731489423b37f'}},
		  'datagroup_id': u'3ac32eaab24846ccb724759e82086568',
		  'datagroup_updates': {'fields': {'lastdone': 1526628198.695635,
						   'suspend_reason': '',
						   'suspend_resume_time': None,
						   'suspend_time': None,
						   'suspended': False},
					'smtime': 1526559200,
					'time': 1526628198.695635,
					'uuid': u'3ac32eaab24846ccb724759e82086568'},
		  'datasources_updates': {u'27258a7dbf964237a2c763bf350b4b55': {'fields': {'lastdone': 1526628198.695635,
											   'status': 0},
										'infos': ['Resumed by poller (at 1526628198.7)'],
										'name': u'rttmin',
										'smtime': 1526628192,
										'time': 1526628198.695635,
										'uuid': u'27258a7dbf964237a2c763bf350b4b55'},
					  u'a72cf6c9788240ba8c2c04eec4a06fcf': {'fields': {'lastdone': 1526628198.695635,
											   'status': 0},
										'infos': ['Resumed by poller (at 1526628198.7)'],
										'name': u'lossperc',
										'smtime': 1526628192,
										'time': 1526628198.695635,
										'uuid': u'a72cf6c9788240ba8c2c04eec4a06fcf'},
					  u'ed1a87912bdf419dbb570a331a7b18de': {'fields': {'lastdone': 1526628198.695635,
											   'status': 0},
										'infos': ['Resumed by poller (at 1526628198.7)'],
										'name': u'rttavg',
										'smtime': 1526628192,
										'time': 1526628198.695635,
										'uuid': u'ed1a87912bdf419dbb570a331a7b18de'},
					  u'eefb72f0e09b4ad8afce0a616c9f7cdf': {'fields': {'lastdone': 1526628198.695635,
											   'status': 0},
										'infos': ['Resumed by poller (at 1526628198.7)'],
										'name': u'rttmax',
										'smtime': 1526628192,
										'time': 1526628198.695635,
										'uuid': u'eefb72f0e09b4ad8afce0a616c9f7cdf'}},
		  'element_id': u'c61ed1a27be04db698624959a0ce4a63',
		  'elements': {},
		  'saved_variables': {},
		  'ts': 1526628198.695635
		 },
		 

		 CHECK VERO E PROPRIO

		 
		 {'additional_updates': [],
		  'conditions_updates': {u'8e84342846c94ea8a12731489423b37f': {'fields': {'lastchecktotaltime': 0.004088878631591797,
											  'lastdone': 1526628199,
											  'status': 20,
											  'statuslast': 100,
											  'statuslastchange': 1526628199,
											  'tries': 1,
											  'verbstatus': u' {lossperc@} < 1 expanded to (1 < 1)'},
									       'infos': [u' {lossperc@} < 1 expanded to (1 < 1)'],
									       'smtime': 1526628192,
									       'time': 1526628199,
									       'uuid': u'8e84342846c94ea8a12731489423b37f'}},
		  'datagroup_id': u'3ac32eaab24846ccb724759e82086568',
		  'datagroup_updates': {'fields': {'lastchecktotaltime': 1.1525990962982178,
						   'lastdone': 1526628199},
					'smtime': 1526559200,
					'time': 1526628199,
					'uuid': u'3ac32eaab24846ccb724759e82086568'},
		  'datasources_updates': {u'27258a7dbf964237a2c763bf350b4b55': {'fields': {'lastchecktotaltime': 0.0012857913970947266,
											   'lastdone': 1526628199,
											   'lastvalue': None,
											   'status': 40},
										'infos': ['Error while evaluating expression: Unable to calculate rttmin. Packet loss=1. '],
										'name': u'rttmin',
										'smtime': 1526628192,
										'time': 1526628199,
										'uuid': u'27258a7dbf964237a2c763bf350b4b55'},
					  u'a72cf6c9788240ba8c2c04eec4a06fcf': {'fields': {'lastchecktotaltime': 0.0033140182495117188,
											   'lastdone': 1526628199,
											   'lastvalue': 1},
										'infos': ['ICMP Timeout on all addresses',
											  u"expression getICMPStat('loss', $node, $probesize) expanded to 1"],
										'name': u'lossperc',
										'smtime': 1526628192,
										'time': 1526628199,
										'uuid': u'a72cf6c9788240ba8c2c04eec4a06fcf'},
					  u'ed1a87912bdf419dbb570a331a7b18de': {'fields': {'lastchecktotaltime': 0.014344215393066406,
											   'lastdone': 1526628199,
											   'lastvalue': None,
											   'status': 40},
										'infos': ['Error while evaluating expression: Unable to calculate rttavg. Packet loss=1. '],
										'name': u'rttavg',
										'smtime': 1526628192,
										'time': 1526628199,
										'uuid': u'ed1a87912bdf419dbb570a331a7b18de'},
					  u'eefb72f0e09b4ad8afce0a616c9f7cdf': {'fields': {'lastchecktotaltime': 0.006451129913330078,
											   'lastdone': 1526628199,
											   'lastvalue': None,
											   'status': 40},
										'infos': ['Error while evaluating expression: Unable to calculate rttmax. Packet loss=1. '],
										'name': u'rttmax',
										'smtime': 1526628192,
										'time': 1526628199,
										'uuid': u'eefb72f0e09b4ad8afce0a616c9f7cdf'}},
		  'element_id': u'c61ed1a27be04db698624959a0ce4a63',
		  'elements': {},
		  'saved_variables': {'free_variables': {}, 'snmp_variables': {}},
		  'ts': 1526628199}
	]
"""

def AgentUpdateEvent(ts, tenant_id, agent_id, data):
	
	return  {
		'ts'       : ts,
		'tenant_id': tenant_id,
		'agent_id' : agent_id,
		'data'     : data,
	}
