# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>. 

#
#	PollException
#		ExternalErrorException
#
#
#		UndefinedValueException
#			UndefinedSnmpValueException
#
#		TimeoutException
#
#		ForceUncheckableException
#
#		ClusterException
#
#		RemoteQueryException
#
#		ResourceNotFoundException
#			ElementNotFoundException
#			DataGroupNotFoundException
#			DataSourceNotFoundException
#			ConditionNotFoundException
#			SchedulingGroupNotFoundException




class PollException(Exception):
	"""
	Poller's base exception.
	"""
	pass

class ExternalErrorException(PollException):
	"""
	Wrapper exception for every exception raised by an external library or module. For example:
	
		* SNMP network error, timeout, ecc...
		* RRD problems
	"""
	pass

class UndefinedValueException(PollException):
	pass


class UndefinedSnmpValueException(UndefinedValueException):
	"""
	Raised where a expected SNMP values can't be read or are not present and
	an expression evaluation can not be completed.
	"""
	pass

class TimeoutException(PollException):
	"""
	Raised where a expected SNMP values can't be read or are not present and
	an expression evaluation can not be completed.
	"""
	pass
	
class ForceUncheckableException(PollException):
	"""
	Special exception used to wrap any exception that must produce an "uncheckable result in expression evaluation.
	"""
	# HISTORY: Added for ticket #1885.
	pass

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class ClusterException(PollException):
	"""
	Raised when there are problems with cluster manangement functions
	"""
	pass

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class ResourceNotFoundException(PollException):
	"""
	Raised when a required resource is not present inside internal data
	structures (inside internal cache).
	"""
	pass

class ElementNotFoundException(ResourceNotFoundException):
	pass

class DataGroupNotFoundException(ResourceNotFoundException):
	pass

class DataSourceNotFoundException(ResourceNotFoundException):
	pass

class ConditionNotFoundException(ResourceNotFoundException):
	pass

class SchedulingGroupNotFoundException(ResourceNotFoundException):
	pass


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class RemoteQueryException(PollException):
	"""
	Raised when an IPC error occurs when the agent queries the sanet server for data.
	"""
	pass
	
