"""

Cluster management
========================================================================

isAssociated
---------------------------------------------

    **Syntax**

      **isAssociated** **(** **)**

    **Description**

      Returns *true* if the current node (Access point) is associated to the cluster which it belongs to.

    **Input**
 
      NONE
 
    **Output**

      *True* or *False*

    **Errors**

      An exception is raised if the cluster association can not be calculated.

    **Examples**

      +------------------------------------------------------+------------------------------------------------------------------------+---------------------+
      | Expression                                           | Description                                                            | Return string value |
      +======================================================+========================================================================+=====================+
      | isAssociated()                                       |                                                                        | *true*              |
      +------------------------------------------------------+------------------------------------------------------------------------+---------------------+


clusterNode
---------------------------------------------

    **Syntax**

      **clusterNode** **(** **)**

    **Description**

      Returns the name of the *master* node the current node is associated to.

    **Input**

      None

    **Output**

      A string with the give node.

    **Errors**

      An exception is raised if the cluster association can not be calculated.

    **Examples**

      +------------------------------------------------------+------------------------------------------------------------------------+---------------------+
      | Expression                                           | Description                                                            | Return string value |
      +======================================================+========================================================================+=====================+
      | clusterNode()                                        |                                                                        | *wlc1*              |
      +------------------------------------------------------+------------------------------------------------------------------------+---------------------+


clusterCommunity
---------------------------------------------

    **Syntax**

      **clusterCommunity** **(** **)**

    **Description**

      Returns the SNMP community configured for the *master* node the current node is associated to.

    **Input**

      None

    **Output**

      A string with the give community.

    **Errors**

      An exception is raised if the cluster association can not be calculated.

    **Examples**

      +------------------------------------------------------+------------------------------------------------------------------------+---------------------+
      | Expression                                           | Description                                                            | Return string value |
      +======================================================+========================================================================+=====================+
      | clusterCommunity()                                   |                                                                        | *public123*         |
      +------------------------------------------------------+------------------------------------------------------------------------+---------------------+

clusterSnmpPort
---------------------------------------------

    **Syntax**

      **clusterSnmpPort** **(** **)**

    **Description**

      Returns the SNMP port configured for the *master* node the current node is associated to.

    **Input**

      None

    **Output**

      A number with the give SNMP port.

    **Errors**

      An exception is raised if the cluster association can not be calculated.

    **Examples**

      +------------------------------------------------------+------------------------------------------------------------------------+---------------------+
      | Expression                                           | Description                                                            | Return string value |
      +======================================================+========================================================================+=====================+
      | clusterSnmpPort()                                    |                                                                        | *161*               |
      +------------------------------------------------------+------------------------------------------------------------------------+---------------------+


clusterInstance
---------------------------------------------

    **Syntax**

      **clusterInstance** **(** **)**

    **Description**

      Returns the SNMP instance for the current node (AP) inside the MIB of the controller (*master node*).

    **Input**

      None

    **Output**

      A string with the give instance.

    **Errors**

      An exception is raised if the cluster association can not be calculated.

    **Examples**

      +------------------------------------------------------+------------------------------------------------------------------------+---------------------+
      | Expression                                           | Description                                                            | Return string value |
      +======================================================+========================================================================+=====================+
      | clusterInstance()                                    |                                                                        | *144.23.51.8*       |
      +------------------------------------------------------+------------------------------------------------------------------------+---------------------+

"""


from sanet_poller.lib.exceptions import ClusterException

#---------------------------------------------------------------------#
# Cluster functions                                                   #
#---------------------------------------------------------------------#

def fun_isAssociated(context):

        cluster_node = None
        info = ''
 
        try:
		# This will trigger cluster association procedure.
                # Check if the context contains the $cluster_node symbol.
                # If it doesn't, an association does not exists
                cluster_node     = context.get('$cluster_node', None)
                cluster_instance = context.get('$cluster_instance', None)
                info = '%s - %s' % (cluster_node, cluster_instance)
        except ClusterException, e:
                info = str(e)

        context.appendReason(info)

        if cluster_node != None:
                return True
        else:
                return False

def fun_clusterNode(context):
	context.getLog().debug("clusterNode()")
	# This will trigger cluster association procedure or an exception
        node_name = context['$cluster_node']

	r = node_name

	# If we are here '__cluster_node__' MUST be in the context.
	pnode = context.get('__cluster_node__', None)
	if pnode is not None: # Too much: this check is unecessary because pnode exists.

		# We want to contact the $cluster_node.
		# Calculate the effective name/ip name to use!
		effective_ip = pnode.calculate_effective_ip()
		if effective_ip != None:
			context.getLog().debug("\tCalculated effective ip for clusterNode() '%s' = %s", node_name, effective_ip)
			r = effective_ip
	
	return r

def fun_clusterCommunity(context):
	context.getLog().debug("clusterCommunity()")
	# This will trigger cluster association procedure 
        return context['$cluster_community']

def fun_clusterSnmpPort(context):
	context.getLog().debug("clusterSnmpPort()")
	# This will trigger cluster association procedure 
        return context['$cluster_snmpport']

def fun_clusterInstance(context):
	context.getLog().debug("clusterInstance()")
	# This will trigger cluster association procedure
        return context['$cluster_instance']

EXPR_SYMBOLS = {
        'isAssociated'     : fun_isAssociated,
        'clusterNode'      : fun_clusterNode,
        'clusterCommunity' : fun_clusterCommunity,
        'clusterInstance'  : fun_clusterInstance,
        'clusterSnmpPort'  : fun_clusterSnmpPort,
}
