# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

test
=======================

    **Syntax**
    
     **test()**

    **Description**

     It always returns 0.

    **Input**

     None

    **Outpu**

     Returns 0.

    **Errors**

     None

    **Examples**

     +----------------------------------------------------+-------------------------------------+-------------------------------------------------------+
     | Expression                                         | Return value                        |  Description                                          |
     +====================================================+=====================================+=======================================================+
     | test()                                             | 0                                   |                                                       |
     +----------------------------------------------------+-------------------------------------+-------------------------------------------------------+

pollerror
=======================

    **Syntax**
    
     **pollerror()**

    **Description**

     Always raises an execution exception and the current expression goes uncheckable.
     
     *NOTE*: used only for debug purpose.

    **Input**

     None

    **Outpu**

     Returns 0.

    **Errors**

     Raise and

    **Examples**

     +----------------------------------------------------+-------------------------------------+-------------------------------------------------------+
     | Expression                                         | Return value                        |  Description                                          |
     +====================================================+=====================================+=======================================================+
     |  pollerror()                                       | *exception*                         |                                                       |
     +----------------------------------------------------+-------------------------------------+-------------------------------------------------------+


unknownerror
=======================

    **Syntax**
    
     **unknownerror()**

    **Description**

     Always raises an unspecified exception. 

     *NOTE*: used only for debug purpose.

    **Input**

     None

    **Outpu**

     Returns 0.

    **Errors**

     Raise and

    **Examples**

     +----------------------------------------------------+-------------------------------------+-------------------------------------------------------+
     | Expression                                         | Return value                        |  Description                                          |
     +====================================================+=====================================+=======================================================+
     |  pollerror()                                       | *exception*                         |                                                       |
     +----------------------------------------------------+-------------------------------------+-------------------------------------------------------+

"""
import os
import time

import random
random.seed(os.getpid())

from sanet_poller.lib.exceptions import PollException

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def fun_test(context):

	log = context.getLog()

	log.debug("test()....")

	context.appendExecutionInfo("The quick brown fox jumps over the lazy dog.")

	return 0

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def fun_pollerror(context, info=None):
	raise PollException(info or "Dummy debug error")

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def fun_unknownerror(context):
	raise Exception("Unknown debug error")

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def fun_log_snmpval(context, snmp_val):

	log = context.getLog()

	context.appendExecutionInfo("<%s %s %s %s %s %s>" % (snmp_val.host, snmp_val.oid, snmp_val.sv, snmp_val.community, snmp_val.port, snmp_val.auth))

	log.info("host      = %s", snmp_val.host      )
	log.info("oid       = %s", snmp_val.oid       )
	log.info("port      = %s", snmp_val.port      )
	log.info("sv        = %s", snmp_val.sv        )
	log.info("community = %s", snmp_val.community )
	log.info("auth      = %s", snmp_val.auth      )

	return True

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def fun_args2dict(context, *args):

	d = {}
	for i in xrange(len(args)/2):
		d[ args[i*2] ] = args[(i*2)+1]

	return d

def fun_array(context, *args):
	return list(args)

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def fun_sleep(context, sleep_time):
	time.sleep(int(sleep_time))
	return sleep_time


def fun_randint(context, a, b):
	return random.randint(a,b)
	return True
#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def fun_ENVIRONEMNT(context, name):
	
	v = os.environ.get(name, None)
	if v is None:
		raise PollException("%s not in enviroment" % (name))
		
	return v

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

EXPR_SYMBOLS = {
	'test'                  : fun_test,

	'pollerror'             : fun_pollerror,

	'unknownerror'          : fun_unknownerror,

	'log_snmpval'           : fun_log_snmpval,

	'args2dict'             : fun_args2dict,

	'array'                 : fun_array,

	'sleep'                 : fun_sleep,

	'randint'                 : fun_randint,
	
	'ENVIRONMENT'           : fun_ENVIRONEMNT,
}





#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def _debug_status(agent):
	
	agent.logger.info("Calculating internal status...")
	
	status = True
	msg    = "DEBUG STATUS OK"

	return (status, msg)


INTERNAL_STATUS_FUNCTIONS = {
	'debug_status': _debug_status,
}
