"""
SNMP utils
========================================================================

extended_data
---------------------------------------------

    **Syntax**

      **extended_data** **(** *data* **,** *search_key* **)**

    **Description**

      Reads the multi-line text string *data* and returns the line containing the string *search_key*.
      
      The line format of *data* must be: ::
      
        <key> = <value> \n

      Example: ::
      
	adapter.packets.received = 29502579279
	adapter.bytes.transmitted = 7460746248
	adapter.0.packets.received = 29502579279 
	adapter.0.bytes.transmitted = 7460746248

    **Input**

      * *data*: multi-line text string
      * *seach_key*: string

    **Output**

      Returns a *string*

    **Errors**
    
      An exception is raised if the input data format is not valid.

    **Examples**

	 +----------------------------------------------------------------+----------------------------------+----------------------------------------------------------------+
	 | Expression                                                     |           Return value           | Description                                                    |
	 +----------------------------------------------------------------+----------------------------------+----------------------------------------------------------------+
	 | extended_data( $data , "adapter.0.packets.received" )          | 29502579279                      |                                                                |
	 +----------------------------------------------------------------+----------------------------------+----------------------------------------------------------------+

"""

from sanet_poller.lib.evaluation import SnmpValueType

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

"""
GLOBAL_KEY = '_global_'

def _parse_data(string_data):

	#adapter.packets.received = 29502579279 \n
	#adapter.bytes.transmitted = 7460746248 \n
	#adapter.0.packets.received = 29502579279 \n
	#adapter.0.bytes.transmitted = 7460746248 \n

	entries = {}
	
	for l in string_data.split('\n'):
		try:
			
			key, val = l.split('=')
			
			# Remove white spaces
			key = key.strip()
			val = float(val.strip())
			
			data = key.split('.')
			if len(data) == 4:
				# adapter.0.<name>
				index = int(data[1])
				name  = data[2] + '.' + data[3]
			else:
				# adapter.<name>
				index = GLOBAL_KEY
				name  = data[1] + '.' + data[2]

			entry = entries.get(index, {})
			entry[name] = val
			entries[index] = entry
			
				
		except Exception, e:
			#print "eccezione", e
			#print l
			#print len(l)
			pass
		
	return entries




def fun_extended_data2(context, data, name, index=None):
	
	if isinstance(data, SnmpValueType):
		data = data.getvalue()

	entries = _parse_data(data)
	
	index = index if (index != None) else GLOBAL_KEY

	e   = entries[index]
	val = e[name]
	
	return val
"""

def fun_extended_data(context, data, search_key):
	
	if isinstance(data, SnmpValueType):
		data = data.getvalue()

        for l in data.split('\n'):

		key, val = l.split('=')
			
		# Remove white spaces
		key = key.strip()
		val = float(val.strip())

		if key == search_key:
			return val

	raise Exception("Key %s not found in:\n%s" % (search_key, data))

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#
	
EXPR_SYMBOLS = {
	'extended_data': fun_extended_data, #SANET 3
}
