"""

====================
Filesystem functions
====================

mtime
---------------------------------------------

    **Syntax**

      **mtime** **(** *fname* **)**

    **Description**

      Performs a stat() system call on the given path *fname* and returns the time of most recent content modification.

      If an error occurrs the function returns 0.

    **Output**

      Returns a floating point value.

    **Examples**

      +----------------------+-----------------------+
      | Expression           | Return string value   |
      +----------------------+-----------------------+
      | mtime("/etc/passwd") | 1291971078.92         |
      +----------------------+-----------------------+
      | mtime("/etc2/foo")   | 0                     |
      +----------------------+-----------------------+
     
readFile
---------------------------------------------

    **Syntax**

      **readFile** **(** *path* **)**

    **Description**

      The function returns the content of the given file.

    **Input**

      *path*: string containing the file absolute path

    **Output**

      Returns a *string*.

    **Errors**

      Raises an exceptions if an error occurs while opening/reading the file

    **Examples**

	 +------------------------------+-------------------------+---------------+
	 | Expression                   | Return value            | Description   |
	 +------------------------------+-------------------------+---------------+
	 | readFile("/etc/hosts")       | *127.0.0.1 localhost*   |               |
	 +------------------------------+-------------------------+---------------+
	 | readFile("/tmp/nonexistent") | -                       | **EXCEPTION** |
	 +------------------------------+-------------------------+---------------+     
"""

import os
import sys

from sanet_poller.lib.exceptions import *

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def fun_mtime(context, fname):

	# If this tracebacks, then there is a config error and you will see it in the log
	fname = str(fname)
	try:
		s = os.stat(fname)
		return s.st_mtime
	except OSError:
		return 0
		
#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def fun_fileExists(context, fname):

	# If this tracebacks, then there is a config error and you will see it in the log
	fname = str(fname)
	
	return os.path.exists(fname)

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def fun_readFile(context, file_path):

	log = context.getLog()

	file_path = str(file_path)

	f = None
	try:
		f = open(file_path, "r")

		data = f.read()

		return data
	except Exception, e:
		raise PollException("Unable to open file %s: %s" % (file_path, str(e),) )
		raise e
	finally:
		try:
			if f != None:
				f.close()
		except Exception, e:
			pass

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

EXPR_SYMBOLS = {
	'mtime'                 : fun_mtime,
	'fileExists'            : fun_fileExists,
	'readFile'              : fun_readFile,
}		
