# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>. 

"""
===============
JSON Functions
===============

loadjson
---------------------------------------------

    **Syntax**

      **loadjson** **(** *string* **)**

    **Description**
    
      Parse the JSON string *string* and returns the memory objects.
    
    **Input**
    
      * *string*: JSON string.
    
    **Output**
    
      Return any valid value loaded from the string.
    
    **Errors**
    
      Raise an exception if the input string is not a valid JSON data.
    
    **Examples**

      +------------------------------------------------+------------------------------------------------------------------------+---------------------+
      | Expression                                     | Description                                                            | Return string value |
      +================================================+========================================================================+=====================+
      | loadjson('{ "hello": "world" }')['hello']      | Convert the JSON string '{ "hello": "world" }'                         | "world"             |
      +------------------------------------------------+------------------------------------------------------------------------+---------------------+
      | loadjson( readFile("/tmp/data.json")['hello']  | Read JSON data from file ( '{ "hello": "world" }' )                    | "world"             |
      +------------------------------------------------+------------------------------------------------------------------------+---------------------+

"""

import json

from sanet_common.exprparser.Types import SnmpValueType

from sanet_poller.lib.exceptions import PollException

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def fun_loadjson(context, s):
	
	# La expr era probabilmente qualcosa del tipo  loadjson( 1.3.6.1.2.3.4.5.6.7@ )
	if isinstance(s, SnmpValueType):
		s = s.getvalue()
	
	try:
		return json.loads(s)
	except Exception, e:
		raise PollException("Unable to convert json to data: %s" % (e))

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

EXPR_SYMBOLS = {
	'loadjson'             : fun_loadjson,
}
