"""

OIDs Utilities
==============

--------
oidFirst
--------

    **Syntax**

      **oidFirst** **(** *oid* **)**

    **Description**

      The function take in input a dot-separted OID string and returns the first *node* of the sequence.

    **Input**

      *oid*: a dot-separated string.

    **Output**

      Returns a *string*

    **Errors**

    **Examples**

     +-------------------------------------------+--------------------------+------------------------------------------------------+
     | Expression                                | Return string value      | Description                                          |
     +-------------------------------------------+--------------------------+------------------------------------------------------+
     | oidFirst("9.8.7.8.5.4.3.2.1")             | *9*                      |                                                      |
     +-------------------------------------------+--------------------------+------------------------------------------------------+
     | oidFirst( 1.3.6.1.2.1.1.2.0@"mason" )     | *1*                      | Extracts the first *node* from the node's SysOID     |
     +-------------------------------------------+--------------------------+------------------------------------------------------+

-------
oidLast
-------


    **Syntax**

     **oidLast** **(** *oid* **)**

    **Description**

     The function takes in input a dot-separted OID string and returns the last *node* of the sequence.

    **Input**

     *oid*: a dot-separated string.

    **Output**

     Returns a *string*

    **Errors**

    **Examples**

     +----------------------------------------+---------------------+----------------------------------------------------+
     | Expression                             | Return string value | Description                                        |
     +----------------------------------------+---------------------+----------------------------------------------------+
     | oidFirst("9.8.7.8.5.4.3.2.1")          | *1*                 |                                                    |
     +----------------------------------------+---------------------+----------------------------------------------------+
     | oidFirst( 1.3.6.1.2.1.1.2.0@"mason" )  | *1*                 | Extracts the last *node* from the node's SysOID    |
     +----------------------------------------+---------------------+----------------------------------------------------+



-----
oidIp
-----

    **Syntax**

      **oidIp** **(** *string* **)**

    **Description**

      The function takes in input a string representing an IPv6 address and returns an OID made with the 16 bytes of the address (16). It can be passed a valid IPv4 address, and it will simply return the same address. The function throws an error (becomes uncheckable) if the argument is not a valid IPv4 or IPv6 address.

    **Input**

      *string*: an ASCII string.

    **Output**

      Returns an *OID*

    **Errors**

    **Examples**

     +-------------------------------+-------------------------------------------+
     | Expression                    | Return string value                       |
     +-------------------------------+-------------------------------------------+
     | oidIp("2a01:4f8:110:3383::1") | '42.1.4.248.1.16.51.131.0.0.0.0.0.0.0.1'  |
     +-------------------------------+-------------------------------------------+


---------
oidString
---------

    **Syntax**

      **oidString** **(** *string* **)**

    **Description**

      The function takes in input a string and returns an OID made with the length of the string as the first item, and the ASCII values as subsequent items. The function throws an error (becomes uncheckable) if the string is not ASCII.

    **Input**

      *string*: an ASCII string.

    **Output**

      Returns an *OID*

    **Errors**

    **Examples**

     +----------------------------+----------------------------------------------------+
     | Expression                 | Return string value                                |
     +----------------------------+----------------------------------------------------+
     | oidString("abc")           | *3.97.98.99*                                       |
     +----------------------------+----------------------------------------------------+
     | oidString("FIXME please" ) | *12.70.73.88.77.69.32.112.108.101.97.115.101*      |
     +----------------------------+----------------------------------------------------+




"""

import ipaddr

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def fun_oidFirst(context, oid):
	
	# FIX: oid e' una stringa? o puo' essere un'espressione SNMP?
	(first, remainder) = str(oid).split('.',1)
	
	return first

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def fun_oidLast(context, oid):

	return oid.split('.')[-1]

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def fun_oidString(context, s):
	try:
		s = s.encode('ascii')
	except Exception, e:
		raise PollException("oidString: cannot encode to ascii: %s" % str(e))
	r = str(len(s))
	for c in s:
		r = r + '.' + str(ord(c))
	return r
	
#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#
	
def fun_oidIp(context, v):
       # From a text IPv4 or IPv6 address, returns an OID fragment made with the bytes
       try:
               i = ipaddr.IPAddress(v)
       except:
               raise PollException('Invalid IP address: %s' % v)

       return '.'.join(map(lambda x: str(ord(x)),i.packed))	

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

EXPR_SYMBOLS = {
	'oidFirst'              : fun_oidFirst,
	'oidLast'               : fun_oidLast,
	'oidString'             : fun_oidString,
	'oidIp'                 : fun_oidIp,
}
