"""

Redis Utils
============


     **Syntax**

       **redis** **(**  *key* [ **,** *host* ] [ **,** *port* ] [ **,** *db* ] [ **,** *default* ] **)**

     **Description**

       Tries to get the value for the key *key* from a Redis DB. The parameter *host*, *port* and *db* are facultatives.
       Their default values are: host='localhost', port=6379, db=0.
      
       An exception is raised when the key is not found and *default* is not specified.
     
     ** Output **
     
       A single value. 
      
     **Examples**

       +-------------------------------------------------------+--------------------------+------------------------------------------------------+
       | Expression                                            | Return string value      | Description                                          |
       +=======================================================+==========================+======================================================+
       | redis("a.key.in.the.db")                              | *foo*                    |                                                      |
       +-------------------------------------------------------+--------------------------+------------------------------------------------------+
       | redis("xxx")                                          | ERROR                    | Key *xxx* does not exist.                            |
       +-------------------------------------------------------+--------------------------+------------------------------------------------------+
       | redis("xxx", default=10)                              | 10                       | Default returned.                                    |
       +-------------------------------------------------------+--------------------------+------------------------------------------------------+
      
"""

import redis

from sanet_poller.lib.exceptions import PollException

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def fun_redis(context, name, host='localhost', port=6379, db=0, **options):
	
	log = context.getLog()
	
	try:
		port = int(port)
		db = int(db)
	except:
		raise PollException('port and db must be integers')
		
	try:
		r = redis.Redis(host=host, port=port, db=db)
	except Exception, e:
		raise PollException('Cannot contact redis: %s' % (e))
		
	rv = r.get(name)
	
	if rv is not None:
		return rv

	if options.has_key('default'):
		rv = options['default']
		return rv

	raise PollException('key %s not present' % name)

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

EXPR_SYMBOLS = {
	'redis'                   : fun_redis,
}
