"""
*********************
SANET Service utils
*********************

Ping & RTT
==========


getICMPStat
-----------

    **Syntax**

      *isServiceRunning* **(** **)**

    **Description**

      Returns true if the value of *swRunIndex* for the current service can be calculated.

    **Input**

      None
      
    **Output**

      Returns a boolean.

    **Errors**

      An exception is raised when if *swRunIndex* can not be calculated*.

    **Examples**

      +------------------------------------+------------------------------------------------------------------------+---------------------+
      | Expression                         | Description                                                            | Return string value |
      +------------------------------------+------------------------------------------------------------------------+---------------------+
      | isServiceRunning()                 |                                                                        | *true*              |
      +------------------------------------+------------------------------------------------------------------------+---------------------+

"""
from sanet_poller.lib.exceptions import UndefinedValueException

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

	
def fun_isServiceRunning(context):
	"""
	Check if system can calculate the swrunindex of the service element
	related to the condition calling this function.
	"""
	
	ret  = False
	info = ''

	try:
		#
		# Try to calculate swrunindex and catch the exception
		#
		swrunindex = context.get('$swrunindex', None)

		info = "Service running with swrunindex = %s" % (swrunindex)
		ret = True

	except UndefinedValueException, e:
		info = str(e) + " " + str(type(e))

	context.appendReason( info )
	return ret

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

EXPR_SYMBOLS = {
	'isServiceRunning'      : fun_isServiceRunning,
}
