# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it> 
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
================================
STP/Bridge utils
================================

vtpCommunity(context, host, comm, ifindex):
	

ifIndex2stp
------------

    **Syntax**::
    
      *ifIndex2stp* **(** host **,** ifindex [ **,** community] [ **,** sv ] **)**

    **Description**

      Returns the port number for the given *host*/*ifindex*.

    **Input**

      * *host*: 
      * *ifindex*: 
      * *communty*: 
      * *sv*: 

    **Outpu**

      Returns a number

    **Errors**

      Raises and exception if an error occurs.

    **Examples**

      +------------------------------+----------------------------------------------+----------------------------------------------+
      | Expression                   | Return value                                 | Description                                  |
      +==============================+==============================================+==============================================+
      | ifIndex2stp($node, $ifindex) | 23                                           |                                              |
      +------------------------------+----------------------------------------------+----------------------------------------------+


vtpCommunity
------------

    **Syntax**::
    
      vtpCommunity **(** host **,** comm **,** ifindex **)**

    **Description**

      TODO

    **Input**

      * *host*: 
      * *communty*:
      * *ifindex*: 

    **Outpu**

      Returns a string

    **Errors**

      Raises and exception if an error occurs.

    **Examples**

      +-------------------------------------------+----------------------------------------------+----------------------------------------------+
      | Expression                                | Return value                                 | Description                                  |
      +===========================================+==============================================+==============================================+
      | vtpCommunity($node, $community, $ifindex) | "public"                                     |                                              |
      +-------------------------------------------+----------------------------------------------+----------------------------------------------+

"""

from sanet_poller.lib.exceptions import PollException

from sanet_poller.lib.snmp.stp  import ifIndex2stp
from sanet_poller.lib.snmp.stp  import ciscoExtendCommunity

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def fun_ifIndex2stp(context, host, ifindex, community=None, sv=None):
	
	log = context.getLog()

	community = community or context.get('$community'  , None)
	sv        = sv        or context.get('$snmpversion', None)

	
	if community == None: raise PollException('Parameter $community missing or invalid')
	if sv == None:        raise PollException('Parameter $sv missing or invalid')
	
	# Be sure the ifindex is a valid interger value.
	ifindex = int(ifindex)

	# 
	# Call to this function is expensive. Calculate function's return value once 
	# and cache it inside the execution context.
	#
	retval_context_key = "ifIndex2stp/%s/%s/%s/%s" % (community, host, ifindex, sv)
	if context.has_key(retval_context_key):
		stpInst = context[retval_context_key]
		log.debug("    -> cached return value (%s) = %s" % (retval_context_key, stpInst))	
		return stpInst

	#
	# CALCULATE
	#	
	c = ciscoExtendCommunity(log, host, community, ifindex, sv)
	
	stpInst = ifIndex2stp(log, host, ifindex, c, sv)
	
	log.debug("ifIndex2stp returns: %s", stpInst)

	#
	# Cache function's return value for this execution
	#
	context[retval_context_key] = stpInst

	return stpInst

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def fun_vtpCommunity(context, host, comm, ifindex):
	
	log = context.getLog()	
	
	host    = host    or context.get('$node'       , None)
	comm    = comm    or context.get('$community'  , None)
	ifindex = ifindex or context.get('$ifindex'    , None)
	sv      =            context.get('$snmpversion', None)
	
	if host    == None: raise PollException('Parameter $node missing or invalid')
	if comm    == None: raise PollException('Parameter $community missing or invalid')
	if ifindex == None: raise PollException('Parameter $ifindex missing or invalid')
	if sv      == None: raise PollException('Parameter $snmpversion missing or invalid')

	# Be sure the ifindex is a valid interger value.
	ifindex = int(ifindex)
	
	# 
	# Call to this function is expensive. Calculate function's return value once 
	# and cache it inside the execution context.
	#
	retval_context_key = "vtpCommunity/%s/%s/%s/%s" % (comm, host, ifindex, sv)
	if context.has_key(retval_context_key):
		vtpcomm = context[retval_context_key]
		log.debug("    -> cached return value (%s) = %s" % (retval_context_key, vtpcomm))	
		return vtpcomm
	
	#
	# CALCULATE
	#
	vtpcomm = ciscoExtendCommunity(log, host, comm, ifindex, sv)
	
	#
	# Cache function's return value for this execution
	#
	context[retval_context_key] = vtpcomm
	
	return vtpcomm

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

EXPR_SYMBOLS = {
	'ifIndex2stp' : fun_ifIndex2stp,
	'vtpCommunity': fun_vtpCommunity,
}
