# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
================
String functions
================

strlen
------------------------

    **Syntax**

      strlen   **(** data **)**

    **Description**

     Return the lenght of the given string data.

    **Input**

     * *data*: a string

    **Output**

     Returns an *integer* value.

    **Errors**

    **Examples**

     +-------------------------------------------+----------------------------------------------+----------------------------------------------+
     | Expression                                | Return value                                 | Description                                  |
     +===========================================+==============================================+==============================================+
     | strlen("foo")                             | 3                                            |                                              |
     +-------------------------------------------+----------------------------------------------+----------------------------------------------+


strrep
------------------------

    **Syntax**

      strrep   **(** str_orig, str_pattern **,** str_rep **)**

    **Description**

     Replaces the string *str_pattern* with *str_rep* inside *str_orig*, and returns the result string.

    **Input**

     * *str_orig*   : string
     * *str_pattern*: string
     * *str_rep*    : string

    **Output**

     Returns an *string*.

    **Errors**

    **Examples**

     +-------------------------------------------+----------------------------------------------+----------------------------------------------+
     | Expression                                | Return value                                 | Description                                  |
     +===========================================+==============================================+==============================================+
     | strrep("123", "2", "B")                   | 1B3                                          |                                              |
     +-------------------------------------------+----------------------------------------------+----------------------------------------------+


strpos
--------------------------------
    
    **Syntax**

      strpos   **(** str1 **,** str2 ** ] )**

    **Description**

     Returns the position of *str2* inside *str1*. The first valid position is 0.

    **Input**

     * *str1*: string
     * *str2*: string

    **Output**

     Returns an *integer* value. Returns -1 when *str2* is not found.

    **Errors**

    **Examples**

     +-------------------------------------------+----------------------------------------------+----------------------------------------------+
     | Expression                                | Return value                                 | Description                                  |
     +===========================================+==============================================+==============================================+
     | strpos("hello world", "orl")              | 7                                            |                                              |
     +-------------------------------------------+----------------------------------------------+----------------------------------------------+
     | strpos("foo foo foo", "bar")              | -1                                           |                                              |
     +-------------------------------------------+----------------------------------------------+----------------------------------------------+


strsub
--------------------------------

    **Syntax**

      strsub  **(** str1 [ **,** startpos ] [ **,** length  **)**

    **Description**

      TODO

    **Input**

      * *str*: string
      * *startpos*: interger
      * *length*: interger

    **Output**

     Returns an *integer* value.

    **Errors**

    **Examples**
    
    
strsplit
----------------------------------

    **Syntax**

      strsplit **(** str1 [**,** splitchar ] [**,**  count ])**
      
    **Description**

     Splits *str1* in tokens. The split char used is *spitchar* (default is whitespace " "). The max number of splits is given by *count*.

    **Input**

     * *str1*: string
     * *splitchar*: a character
     * *count*: integer

    **Output**

     Returns a *vector* of strings.

    **Errors**

    **Examples**
    
     +-------------------------------------------+----------------------------------------------+----------------------------------------------+
     | Expression                                | Return value                                 | Description                                  |
     +===========================================+==============================================+==============================================+
     | strsplit("a b c")                         | ["a", "b", "c"]                              |                                              |
     +-------------------------------------------+----------------------------------------------+----------------------------------------------+
     | strsplit("a b c", count=1)                | ["a", "bc"]                                  |                                              |
     +-------------------------------------------+----------------------------------------------+----------------------------------------------+
    
    
strstrip
----------------------------------

    **Syntax**

      strstrip **(** data **)**

    **Description**

     Remove whitespaces (newline, whitespace, tabs, ecc...) from the head and the tail of the given string.

    **Input**

     * *data*: string

    **Output**

     Returns an *string*

    **Errors**

    **Examples**
    
     +-------------------------------------------+----------------------------------------------+----------------------------------------------+
     | Expression                                | Return value                                 | Description                                  |
     +===========================================+==============================================+==============================================+
     | strstrip("   hello     ")                 | *hello*                                      |                                              |
     +-------------------------------------------+----------------------------------------------+----------------------------------------------+

strjoin
----------------------------------

    **Syntax**

        strjoin **(** data  [**,** *join_char* ] **)**

    **Description**

        Returns a string in which the element of *data* have been joined by separator *join_char*. If *join_char* is not specified, the
        white space characeter is used as separator.

    **Input**

        * *data*: vectory of values (string values)

    **Output**

        Returns an *string*

    **Errors**
    
        An error is raised if *data* is not suitable to join.

    **Examples**
    
       +-------------------------------------------+----------------------------------------------+----------------------------------------------+
       | Expression                                | Return value                                 | Description                                  |
       +===========================================+==============================================+==============================================+
       | strjoin( array(1,2,3) )                   | "1 2 3"                                      |                                              |
       +-------------------------------------------+----------------------------------------------+----------------------------------------------+
       | strjoin( array(1,2,3) , "#")              | "1#2#3"                                      |                                              |
       +-------------------------------------------+----------------------------------------------+----------------------------------------------+

head
-----------------------------------------    
    

    **Syntax**

      head     **(** data **,** num **)**

    **Description**

     Return a vector containing the first *num* strings of the given multi-line data string.

    **Input**

     * *data*: multi-line string
     * *num*: integer

    **Output**

     Returns an *vector* of strings.

    **Errors**

    **Examples**

     +-------------------------------------------+----------------------------------------------+----------------------------------------------+
     | Expression                                | Return value                                 | Description                                  |
     +===========================================+==============================================+==============================================+
     | head("line1\\nline2\\nline3\\nline4, 2)      | ["line1", "line2"]                           |                                              |
     +-------------------------------------------+----------------------------------------------+----------------------------------------------+
     | head("line1\\nline2\\nline3\\nline4, 2)[1]   | "line1"                                      |                                              |
     +-------------------------------------------+----------------------------------------------+----------------------------------------------+
    

tail
-------------------------------------

    **Syntax**

      tail     **(** data **,** num **)**

    **Description**

      Return a vector containing the last *num* strings of the given multi-line data string.

    **Input**

      * *data*: multi-line string
      * *num*: integer

    **Output**

      Returns an *vector* of strings.

    **Errors**

    **Examples**

      +-------------------------------------------+----------------------------------------------+----------------------------------------------+
      | Expression                                | Return value                                 | Description                                  |
      +===========================================+==============================================+==============================================+
      | tail("line1\\nline2\\nline3\\nline4, 2)      | ["line3", "line4"]                           |                                              |
      +-------------------------------------------+----------------------------------------------+----------------------------------------------+
      | tail("line1\\nline2\\nline3\\nline4, 1)      | ["line4"]                                    |                                              |
      +-------------------------------------------+----------------------------------------------+----------------------------------------------+
      | tail("line1\\nline2\\nline3\\nline4, 1)[0]   | "line4"                                      |                                              |
      +-------------------------------------------+----------------------------------------------+----------------------------------------------+
    
"""

from sanet_common.exprparser.Types import SnmpValueType

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def fun_strlen(context, data):

	if isinstance(data, SnmpValueType):
		data = data.getvalue()

	return len(data)

def fun_strrep(context, str_orig, str_pattern, str_rep):

	if isinstance(str_orig, SnmpValueType):
		str_orig = str_orig.getvalue()

	return str(str_orig).replace(str_pattern, str_rep)

def fun_strpos(context, str1, str2):
	
	if isinstance(str1, SnmpValueType):
		str1 = str1.getvalue()
	if isinstance(str2, SnmpValueType):
		str2 = str2.getvalue()

	return str(str1).find(str(str2))

def fun_strsub(context, str1, startpos=None, length=None):

	if isinstance(str1, SnmpValueType):
		str1 = str1.getvalue()

	return str(str1)[startpos:length]


def fun_strsplit(context, str1, splitchar=' ', count=None):

	if isinstance(str1, SnmpValueType):
		str1 = str1.getvalue()
		
	

	if count != None:
		r = str(str1).split(str(splitchar), count)
	else:
		r = str(str1).split(str(splitchar))

	return r


def fun_strstrip(context, data):

	if isinstance(data, SnmpValueType):
		data = data.getvalue()

	return data.strip()

def fun_strjoin(context, data, char=' '):
	return char.join([str(a) for a in data])

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def fun_head(context, data, num):
	return '\n'.join(data.split('\n')[0:num])

def fun_tail(context, data, num):
	return '\n'.join(data.split('\n')[-(num+1):])

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

EXPR_SYMBOLS = {

	'strlen': fun_strlen,
	'strpos': fun_strpos,
	'strrep': fun_strrep,
	'strsub': fun_strsub,
	'strsplit': fun_strsplit,	
	'strstrip': fun_strstrip,

	'strjoin' : fun_strjoin,

	'head' : fun_head,
	'tail' : fun_tail,
	
	
	
}
