# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it> 
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import urllib2
import httplib, socket
from urllib import addinfourl 
from urllib2 import Request
import urlparse

from ntlm import ntlm

#
# Code taken from python-ntlm. Trying to support redirect after
# succesfull connection with a proxy with NTLM authorization.
#

class LabsAbstractNtlmAuthHandler:

	def __init__(self, password_mgr=None, debuglevel=0):

		if password_mgr is None:
			password_mgr = HTTPPasswordMgr()

		self.passwd = password_mgr
		self.add_password = self.passwd.add_password
		self._debuglevel = debuglevel

	def set_http_debuglevel(self, level):
		self._debuglevel = level


	def http_error_authentication_required(self, auth_header_field, req, fp, headers):
		auth_header_value = headers.get(auth_header_field, None)
		if auth_header_field:
			if auth_header_value is not None and 'ntlm' in auth_header_value.lower():
				fp.close()
				return self.retry_using_http_NTLM_auth(req, auth_header_field, None, headers)

	def retry_using_http_NTLM_auth(self, req, auth_header_field, realm, headers):

		user, pw = self.passwd.find_user_password(realm, req.get_full_url())
		if pw is None:
			return None

		if True:
		
			user_parts = user.split('\\', 1)
			if len(user_parts) == 1:
				UserName = user_parts[0]
				DomainName = ''
				type1_flags = ntlm.NTLM_TYPE1_FLAGS & ~ntlm.NTLM_NegotiateOemDomainSupplied
			else:
				DomainName = user_parts[0].upper()
				UserName = user_parts[1]
				type1_flags = ntlm.NTLM_TYPE1_FLAGS

			# ntlm secures a socket, so we must use the same socket for the complete handshake
			headers = dict(req.headers)
			headers.update(req.unredirected_hdrs)


			#
			# TO FIX: la funzione create_NTLM_NEGOTIATE_MESSAGE()  quanti parametri in input prende? E' cambiato il numero di parametri tra una release della liberia e l'altra?
			#
			try:
				auth = 'NTLM %s' % ntlm.create_NTLM_NEGOTIATE_MESSAGE(user, type1_flags)
			except Exception, e:
				auth = 'NTLM %s' % ntlm.create_NTLM_NEGOTIATE_MESSAGE(user)
	

			if req.headers.get(self.auth_header, None) == auth:
				return None
			headers[self.auth_header] = auth

			host = req.get_host()
			if not host:
				raise urllib2.URLError('no host given')
			h = None
			if req.get_full_url().startswith('https://'):
				h = httplib.HTTPSConnection(host) # will parse host:port
			else:
				h = httplib.HTTPConnection(host) # will parse host:port
			h.set_debuglevel(self._debuglevel)
			# we must keep the connection because NTLM authenticates the connection, not single requests
			headers["Connection"] = "Keep-Alive"
			headers = dict((name.title(), val) for name, val in headers.items())
			h.request(req.get_method(), req.get_selector(), req.data, headers)



			r = h.getresponse()
			r.begin()
			r._safe_read(int(r.getheader('content-length')))
			if r.getheader('set-cookie'):
				# this is important for some web applications that store authentication-related info in cookies (it took a long time to figure out)
				headers['Cookie'] = r.getheader('set-cookie')
				
			r.fp = None # remove the reference to the socket, so that it can not be closed by the response object (we want to keep the socket open)
			auth_header_value = r.getheader(auth_header_field, None)
			(ServerChallenge, NegotiateFlags) = ntlm.parse_NTLM_CHALLENGE_MESSAGE(auth_header_value[5:])
			auth = 'NTLM %s' % ntlm.create_NTLM_AUTHENTICATE_MESSAGE(ServerChallenge, UserName, DomainName, pw, NegotiateFlags)
			headers[self.auth_header] = auth
			headers["Connection"] = "Close"
			headers = dict((name.title(), val) for name, val in headers.items())

			try:
				h.request(req.get_method(), req.get_selector(), req.data, headers)
				response = h.getresponse()

				"""
				NOT WORKING

				if response.status in [301,302]:

				    code = response.status
				    headers2 = dict(response.getheaders())
				    print headers2
					
				    if 'location' in headers2:
					newurl = headers2['location']
				    elif 'uri' in headers:
					newurl = headers2['uri']
				    else:
					return

				    newurl = urlparse.urljoin(req.get_full_url(), newurl)

				    print "----------------- REDIRECTED REQUEST ----------------------"
				    print newurl
				    print headers
				    h.request(req.get_method(), newurl, req.data, headers)
				    response = h.getresponse()

				    print response.status
				    print response.msg
				    

				    #m = req.get_method()
				    #if not (code in (301, 302, 303, 307) and m in ("GET", "HEAD") or code in (301, 302, 303) and m == "POST"):
				    #    raise HTTPError(req.get_full_url(), code, msg, headers, fp)
				    #newurl = newurl.replace(' ', '%20')
				    #new_request = Request(newurl, headers=headers)#, origin_req_host=req.get_origin_req_host(), unverifiable=True)
				    #headers[self.auth_header] = auth
				    headers["Connection"] = "Close"
				    #print "---------------------------------"
				    #print "NEW REQUEST"
				    #print newurl
				    #print headers
				    #return self.parent.open(new_request)
				"""

				def notimplemented():
					raise NotImplementedError
					
				response.readline = notimplemented

				infourl = addinfourl(response, response.msg, req.get_full_url())
				infourl.code = response.status
				infourl.msg = response.reason
				return infourl

			except socket.error, err:
				raise urllib2.URLError(err)


class LabsHTTPNtlmAuthHandler(LabsAbstractNtlmAuthHandler, urllib2.BaseHandler):

	auth_header = 'Authorization'

	def http_error_401(self, req, fp, code, msg, headers):
		#return self.http_error_authentication_required('www-authenticate', req, fp, headers)
		return self.http_error_authentication_required('WWW-Authenticate', req, fp, headers)


class LabsProxyNtlmAuthHandler(LabsAbstractNtlmAuthHandler, urllib2.BaseHandler):
	""" 
	CAUTION: this class has NOT been tested at all!!! 
	use at your own risk
	"""
	auth_header = 'Proxy-authorization'

	def http_error_407(self, req, fp, code, msg, headers):
		return self.http_error_authentication_required('proxy-authenticate', req, fp, headers)
