# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import struct
import re

from sanet_poller.lib.exceptions import UndefinedSnmpValueException

from sanet_poller.lib.snmp.base import getvar, gettable

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def areCdpAdj(log, host1, host2, port1, port2, comm1, comm2, sv1, sv2):

	log.debug( "areCdpAdj(%s,%s,%s,%s,%s,%s,%s,%s)", host1,host2, port1,port2, comm1,comm2, sv1,sv2)
	
	SNMPPORT1=161
	SNMPPORT2=161
	
	port1_ifdescr = getvar(host1 , comm1, 3, '1.3.6.1.2.1.2.2.1.2.%s' % (port1) , SNMPPORT1, sv1)
	if port1_ifdescr == None:
		raise UndefinedSnmpValueException("can't get ifdescr for port %s of %s" % (port1, host1))
		
	port2_ifdescr = getvar(host2 , comm2, 3, '1.3.6.1.2.1.2.2.1.2.%s' % (port2) , SNMPPORT1, sv2)	
	if port1_ifdescr == None:
		raise UndefinedSnmpValueException("can't get ifdescr for port %s of %s" % (port2, host2))
	
	info = "(%s,%s,%s,%s) " % (host1, port1_ifdescr, host2, port2_ifdescr)
	
	#
	#
	#
	log.debug("looking inside CDP table of host1 %s" % (host1))
	host1_cdp_table = _get_cdp_table(host1, comm1, sv1, port1)	
	(result, rinfo) = _analize_cdp_data(log, host1, host2, port1, port2, comm1, comm2, sv1, sv2, port1_ifdescr, port2_ifdescr, host1_cdp_table)
	if result == True:
		return (result, info + rinfo)
	
	log.debug("No match found inside CDP table of host1 %s" % (host1) )

	#
	#
	#
	log.debug("looking inside CDP table of host2 %s" % (host2))
	host2_cdp_table = _get_cdp_table(host2, comm2, sv2, port2)
	(result, rinfo) = _analize_cdp_data(log, host2, host1, port2, port1, comm2, comm1, sv2, sv1, port2_ifdescr, port1_ifdescr, host2_cdp_table)
	if result == True:
		return (result, info + rinfo)
		
	info += " no match."
	
	return (result, info)
	
	
def _get_cdp_table(host, comm, sv, port):
	
	SNMPPORT=161
	
	#
	#   1.3.6.1.4.1.9.9.23.1.2.1.1. 6 . [ <ifindex> . <device index> ] =  <deviceid>   (type=DisplayString)
	#   1.3.6.1.4.1.9.9.23.1.2.1.1. 7 . [ <ifindex> . <device index> ] =  <deviceport> (type=DisplayString)
	#                                   <----------instance---------->
	#
	#  Return a dictionary:
	#
	#  	<device index> => <deviceid, deviceport>
	#

	# HACK: #2970 Cisco does not support walks on 1.3.6.1.4.1.9.9.23.1.2.1.1.6.<ifindex>  
	#             but only on 1.3.6.1.4.1.9.9.23.1.2.1.1.6.
	#
	#             We have filter entries manually
	#
	#CDP_DEVICEID   = '1.3.6.1.4.1.9.9.23.1.2.1.1.6.%s' % (port)
	#CDP_DEVICEPORT = '1.3.6.1.4.1.9.9.23.1.2.1.1.7.%s' % (port)
	CDP_DEVICEID   = '1.3.6.1.4.1.9.9.23.1.2.1.1.6'
	CDP_DEVICEPORT = '1.3.6.1.4.1.9.9.23.1.2.1.1.7'

	
	host_cdp_table = {}
	
	host_cdp_entries = gettable(host, comm, 3, CDP_DEVICEID, SNMPPORT, sv)	
	if host_cdp_entries == None:
		raise UndefinedSnmpValueException("DeviceID mib of %s can't be read (%s)!" % (host, CDP_DEVICEID))
	for (row_oid, row_val) in host_cdp_entries:
		instance = row_oid[1+len(CDP_DEVICEID) + 1: ]

		# HACK: #2970
		if_index, dev_index = instance.split('.')
		if if_index != port:
			continue
		instance = dev_index 

		entry = host_cdp_table.get( instance , {} )
		entry['deviceid'] = row_val
		host_cdp_table[instance] = entry
		
	host_cdp_entries = gettable(host, comm, 3, CDP_DEVICEPORT, SNMPPORT, sv)	
	if host_cdp_entries == None:
		raise UndefinedSnmpValueException("DevicePort mib of %s can't be read (%s)!" % (host, CDP_DEVICEPORT))
	for (row_oid, row_val) in host_cdp_entries:
		instance = row_oid[1+len(CDP_DEVICEPORT) + 1: ]

		# HACK: #2970
		if_index, dev_index = instance.split('.')
		if if_index != port:
			continue
		instance = dev_index 

		entry = host_cdp_table.get( instance , {} )
		entry['deviceport'] = row_val
		host_cdp_table[instance] = entry
		
	return host_cdp_table

def _analize_cdp_data(log, host1, host2, port1, port2, comm1, comm2, sv1, sv2, port1_ifdescr, port2_ifdescr, host1_cdp_table):
	#SNMPVERSION1=1
	#SNMPVERSION2=1
	SNMPPORT1=161
	SNMPPORT2=161

	# FIX: 
	# Every device should advertise it's  cdpGlobalDeviceID and we should compare these ids.
	# The format of this string depends on value of cdpGlobalDeviceIdFormatCpb
	# and cdpGlobalDeviceIdFormat BUT some devices don't export these OIDs.
	#
	# 	serialNumber(0) = serial number as the format for its DeviceId.
	# 	macAddress(1)   = layer 2 MAC address as the format for its DeviceId.
	# 	other(2)        = platform specific format as the format for its DeviceId. For example: ASCII string contains serialNumber appended/prepened with system name."	
	#
	# In some cases the GlobalDeviceID (es. sw2524 HP?) is saved in the 
	# neighbors' tables (DeviceID) with a different format. 
	#
	# Es: on 2524 HP
	#   host's cdpGlobalDeviceID               = "labs-sw2524-1(000a57-34bdc0)"
	#   DeviceID value in the neighbor's table = 000a57-34bdc0
	#
	
	#CDP_GLOBAL_DEVICEID        = '1.3.6.1.4.1.9.9.23.1.3.4.0'
	#CDP_GLOBAL_DEVICEID_CAP    = '1.3.6.1.4.1.9.9.23.1.3.7.0'
	#CDP_GLOBAL_DEVICEID_FORMAT = '1.3.6.1.4.1.9.9.23.1.3.6.0'
	#
	#host1_id = getvar(host1 , comm1, 3, CDP_GLOBAL_DEVICEID, SNMPPORT1, sv1)
	#host2_id = getvar(host2 , comm2, 3, CDP_GLOBAL_DEVICEID, SNMPPORT2, sv2)
	#
	#log.debug("host1 %s has %s id" % (host1, host1_id))			
	#log.debug("host1 %s has %s id" % (host1, host1_id))			
	

	host2_globaldeviceid = getvar(host2 , comm2, 3, '1.3.6.1.4.1.9.9.23.1.3.4.0', SNMPPORT1, sv2)	
	if host2_globaldeviceid == None:
		raise UndefinedSnmpValueException("Can't get cdpGlobalDeviceID for %s" % (host2))

	log.debug("\t"+"host2 %s has cdpGlobalDeviceID %s" % (host2, host2_globaldeviceid))


	result = False
	info   = ""
	
	
	for i in host1_cdp_table:
		deviceinfo = host1_cdp_table[i]
		
		id2 = deviceinfo['deviceid']
		p2  = deviceinfo['deviceport']

		#				
                # DeviceID's type is a "DisplayString". This means that it 
                # could be a Hex-String (probably a MAC Address).
                #
                val = None
                is_mac = False
                try:
			(val,) = struct.unpack('6s', id2)
			id2    = macbin2blob(val)                        
			is_mac = True
                	log.debug("\t" + "device id is an binary string %s (mac address?)" % (id2))			
                except Exception,e :
                        pass
		
		
		log.debug("\t" + "comparing host %r (globaldeviceid=%s) port %r (ifindex:%r) with CDP entry <%r,%r>" % (host2, host2_globaldeviceid, port2_ifdescr, port2, id2, p2))			
		
		#
		# DevicePort could be the ifDescr or the ifIndex (es: with HP?)
		#
		if port2_ifdescr == p2:	
			log.debug("\t" + "device port '%s' matched ifdescr '%s'" % (p2, port2) )
		elif str(port2) == p2:
			# On some devices the DeviceID is not the ifDescr but the
			# ifindex of the device.
			log.debug("\t" + "device port '%s' matched ifindex '%s'" % (p2, port2) )
		else:
			log.debug("\t" + "ports not mached!")
			continue
			
		# id2 is the CDP's DeviceID.
		if host2_globaldeviceid.lower() == id2.lower():
			result = True
			info   = "matched <%s,%s> with CDP entry <%s,%s>" % (host2, port2_ifdescr, id2, p2)
			break
		
		# On some HP GlobalDeviceID can be something like  "foo(001321-e5e400)"
		# where <id> is the DeviceID stored in Hex-String in neighbours' CDP tables.
		#
		m = re.match(r".*\((.*)\)", host2_globaldeviceid)
		if m != None:
			id = m.group(1)
			if id2.replace('-','') == id.replace('-',''):
				msg = "%s matches cdpGlobalDeviceID (%s) of %s" % (id2, host2_globaldeviceid, host2)
				log.debug("\t" + msg)
				
				result = True
				info   = "linked! found CDP entry <%s,%s> for (%s,%s). %s" % (id2, p2, host2, port2_ifdescr, msg)
				break
		
		
		# HexString
		#  000a57346140  (seen with some HP, it could be be the BaseBridgeAddress)
			
		if is_mac==True:
			
			# Check if the DeviceID looks like the MacAddress of the device
			id2_mac = id2.replace('-','') 
			
			host2_bridgemac = None
			host2_bridgemac = getvar(host2 , comm2, 3, '1.3.6.1.2.1.17.1.1.0', SNMPPORT1, sv2)	
			if host2_bridgemac != None:
				if bin2hex(host2_bridgemac) == id2_mac:
					msg = "%s is BaseBridgeAddress of %s" % (id2, host2)
					log.debug("\t" + msg)
					
					result = True
					info   = "linked! found CDP entry <%s,%s> for (%s,%s). %s" % (id2, p2, host2, port2_ifdescr, msg)
					break

	return (result, info)
