# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import struct
import re

from sanet_poller.lib.exceptions import UndefinedSnmpValueException

from sanet_poller.lib.snmp.base import getvar, gettable

from sanet_poller.lib.snmp.stp import ifIndex2stp

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#


SNMPPORT = 161

def areEdpAdj(log, host1, host2, ifindex1, ifindex2, comm1, comm2, sv1, sv2):

	result = False
	reason = 'match on EDP table not found'

	h2 = getvar(host2 , comm2, 3, '.1.3.6.1.2.1.1.5.0' , SNMPPORT, sv2)
	p2 = ifIndex2stp(log, host2, ifindex2, comm2, sv2)	

	log.debug('Linked node sysname: %s', h2)
	log.debug("Linked node port   : %s", p2)

	edp_table = _get_edp_table(log, host1, ifindex1, comm1, sv1)

	log.debug("EDP entries for ifindex %s: %s", ifindex1, len(edp_table))

	for remote_name, remote_port in edp_table:

		log.debug("Comparing EDP tuple <%s:%s> with linked %s:%s", remote_name, remote_port, h2, p2)

		if remote_name == str(h2)  and str(remote_port) == str(p2):
			reason = 'Neighbor %s (sysname %s) with remote iface %s(ifindex) (bridge port %s) found in EDP entry (%s:%s) for local interface %s (ifindex)' % (host2, h2, ifindex2, p2, remote_name, remote_port, ifindex1)
			result = True
			break



	return (result, reason )



def _get_edp_table(log, host, ifindex, comm, sv):

	ifindex = str(ifindex)

	print host, ifindex, comm, sv


	extremeEdpEntry        = '1.3.6.1.4.1.1916.1.13.2.1'
	extremeEdpNeighborName = '1.3.6.1.4.1.1916.1.13.2.1.3'
	extremeEdpNeighborPort = '1.3.6.1.4.1.1916.1.13.2.1.6'

	oid = extremeEdpNeighborName 
	entries = gettable(host, comm, 3, oid, SNMPPORT, sv)	
	if entries == None:
		raise UndefinedSnmpValueException("extremeEdpNeighborName (%s) not found on %s! " % (oid, host) )

	r = []
	for o, v in entries:

		instance = o[1+len(oid) + 1:]

		instance_ifindex, neighbor_id  = instance.split('.', 1)

		#print instance, instance_ifindex, neighbor_id, str(ifindex)

		if ifindex != instance_ifindex:
			continue

		oid2 = extremeEdpNeighborPort + '.' + instance
		p = getvar(host , comm, 3, oid2, SNMPPORT, sv)
		if p == None:
			raise UndefinedSnmpValueException("Can't get %s for %s" % (oid2, host))

		r.append( (v,p) )

	return r

