# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import binascii

from sanet_poller.lib.exceptions import UndefinedSnmpValueException

from sanet_poller.lib.snmp.base import getvar, gettable

#------------------------------------------------------------------------------#
# LLDP Adjacency functions                                                     #
#------------------------------------------------------------------------------#

def areLldpAdj(log, host1, host2, port1, port2, comm1, comm2, sv1, sv2):

	# FIX: we assume hosts' LLDP IDs (see ChassisIdSubtype) are MAC Addresses 
	#      and ports' IDs are ifDescrs (see PortIdSubtype).

        SNMPPORT1 = 161
        SNMPPORT2 = 161

        #
        # Get interfaces ID (ifDescr)
        #
        port1_ifdescr = getvar(host1 , comm1, 3, '1.3.6.1.2.1.2.2.1.2.%s' % (port1) , SNMPPORT1, sv1)
        if port1_ifdescr == None:
                raise UndefinedSnmpValueException("can't get ifdescr for port %s of %s" % (port1, host1))

        log.debug("port1 ifdescr: %s", port1_ifdescr)

        port2_ifdescr = getvar(host2 , comm2, 3, '1.3.6.1.2.1.2.2.1.2.%s' % (port2) , SNMPPORT1, sv2)
        if port1_ifdescr == None:
                raise UndefinedSnmpValueException("can't get ifdescr for port %s of %s" % (port2, host2))

        log.debug("port2 ifdescr: %s ", port2_ifdescr)


        #
        # Get Host LLDP ID (macaddress)
        #

        lldpLocChassisId = "1.0.8802.1.1.2.1.3.2.0"


        host1_id = getvar(host1 , comm1, 3, lldpLocChassisId , SNMPPORT1, sv1)
        if host1_id == None:
                raise UndefinedSnmpValueException("can't get lldpLocChassisId for %s:%s" % (port1, host1))

        host1_id = binascii.hexlify(host1_id)

        log.debug("host1 ID: %s", host1_id)

        host2_id = getvar(host2 , comm2, 3, lldpLocChassisId , SNMPPORT2, sv2)
        if host2_id == None:
                raise UndefinedSnmpValueException("can't get lldpLocChassisId for %s:%s" % (port1, host2))

        host2_id = binascii.hexlify(host2_id)

        log.debug("host2 ID: %s", host2_id)

        (result1, info1) = search_lldp_match(log, host1, host2, comm1, comm2, sv1,sv2, port1_ifdescr, port2_ifdescr, host1_id, host2_id)
        if result1:
                return (result1, info1)

        (result2, info2) = search_lldp_match(log, host2, host1, comm2, comm1, sv2, sv1, port2_ifdescr, port1_ifdescr, host2_id, host1_id)

        return (result2, "%s; %s" % (info1, info2))

def search_lldp_match(log, host1, host2, comm1, com2, sv1,sv2, port1_ifdescr, port2_ifdescr, host1_id, host2_id):

        SNMPPORT1 = 161
        SNMPPORT2 = 161

        #
        # Find local LLDP entry for port1
        #
        lldpLocPortId = "1.0.8802.1.1.2.1.3.7.1.3"
        table = gettable(host1, comm1, 3, lldpLocPortId, SNMPPORT1, sv1)
        if table == None:
                raise UndefinedSnmpValueException("lldpLocPortId table  not found on host1: %s" % (host1))
        host1_port_num_by_port_id = {}
        for (row_oid, row_val) in table:
                port_num = row_oid[1+len(lldpLocPortId) + 1: ]
                port_id  = row_val
                host1_port_num_by_port_id[ port_id ] = port_num
        port1_num = host1_port_num_by_port_id.get( port1_ifdescr , None )
        
        if port1_num == None:
		lldpLocPortDescr = "1.0.8802.1.1.2.1.3.7.1.4"
		table = gettable(host1, comm1, 3, lldpLocPortDescr, SNMPPORT1, sv1)
		if table == None:
			raise UndefinedSnmpValueException("lldpLocPortDescr table  not found on host1: %s" % (host1))
		host1_port_num_by_port_id = {}
		for (row_oid, row_val) in table:
			port_num = row_oid[1+len(lldpLocPortDescr) + 1: ]
			port_id  = row_val
			host1_port_num_by_port_id[ port_id ] = port_num
		port1_num = host1_port_num_by_port_id.get( port1_ifdescr )



        log.debug("port1  port number= %s", port1_num)
        #
        # Search host2:port2 inside host1's LLDP table
        #
        lldp_mib = '1.0.8802.1.1.2.1.4.1.1'
        #TimeMark        =  '1'  # TABULAR      not-accessible  TimeFilter                      1.0.8802.1.1.2.1.4.1.1.1
        #LocalPortNum    =  '2'  # TABULAR      not-accessible  LldpPortNumber                  1.0.8802.1.1.2.1.4.1.1.2
        #Index           =  '3'  # TABULAR      not-accessible  Integer32 ( 1..2147483647 )     1.0.8802.1.1.2.1.4.1.1.3
        ChassisIdSubtype =  '4'  #      TABULAR read-only       LldpChassisIdSubtype            1.0.8802.1.1.2.1.4.1.1.4
        ChassisId        =  '5'  # TABULAR      read-only       LldpChassisId                   1.0.8802.1.1.2.1.4.1.1.5
        PortIdSubtype    =  '6'  # TABULAR      read-only       LldpPortIdSubtype               1.0.8802.1.1.2.1.4.1.1.6
        PortId           =  '7'  # TABULAR      read-only       LldpPortId                      1.0.8802.1.1.2.1.4.1.1.7
        PortDescr        =  '8'  # TABULAR      read-only       LldpPortDescr                   1.0.8802.1.1.2.1.4.1.1.8

        base_mib = lldp_mib + '.' + ChassisId
        table = gettable(host1, comm1, 3, base_mib, SNMPPORT1, sv1)
        if table == None:
                raise UndefinedSnmpValueException("lldpRemTable not found on host1: %s" % (host1))

        log.debug("Searching (%s,%s) in (%s,%s)'s LLDP remote table", host2, host2_id, host1, host1_id)
        for (row_oid, row_val) in table:
                instance = row_oid[1+len(base_mib) + 1: ]

                (time, port_num, index) = instance.split('.')
                if port_num != port1_num:
                        continue

                remote_host_id = binascii.hexlify(row_val)
                if remote_host_id.lower() != host2_id.lower():
                        continue

                log.debug("Found matching entry on port (%s,%s)", port1_ifdescr, port_num)

                log.debug("Getting remote port id/descr...")
                remote_port_id_oid    =  lldp_mib +'.'+ PortId +'.'+ instance
                remote_port_id        = getvar(host1, comm1, 3, remote_port_id_oid, SNMPPORT1, sv1)

	        remote_port_descr_oid =  lldp_mib +'.'+ PortDescr +'.'+ instance
        	remote_port_descr     = getvar(host1, comm1, 3, remote_port_descr_oid, SNMPPORT1, sv1)



                log.debug("Remote port id %s, descr %s", remote_port_id, remote_port_descr)

                if remote_port_id == port2_ifdescr or remote_port_descr == port2_ifdescr:
                        log.debug("LLDP link found")
                        return (True, "found entry (host %s, id=%s, port id=%s) in %s's table for iface (port id=%s, port num=%s)" % (host2, remote_host_id, port2_ifdescr, host1, port1_ifdescr, port_num))

        return (False, "Host2 %s:%s not found in  %s's LLDP tables." % (host2, port2_ifdescr, host1))

