# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import getopt
import sys
import logging

from sanet_poller.lib.evaluation import *
from sanet_poller.lib.functions import load_system_expr_functions
from sanet_poller.lib.functions import load_extra_expr_functions

#from sanet_poller.lib.icmp import SanetIcmpPinger, SanetIcmpPingerDaemon

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def handle_num(v):
	return float(v)	

def handle_str(v):
	return v

def handle_bool(v):
	return not(not v or v == '0')

type_handlers = {
	'num'  : handle_num,
	'bool' : handle_bool,
	'str'  : handle_str,
	'text' : handle_str,
}


def  convert_str_param_to_type(name, value_str, type_descr ):
	#try:
		return type_handlers[type_descr] ( value_str )
	#except ValueError, e:
	#	raise PollException("Parameter '%s' value '%s' invalid for type %s" % (name, value_str, type_descr))

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def show_help():
	print "python %s [-X dir] <host> <expr>" % (sys.argv[0])

def main():
	
	opt_extra_pkgs = None
	opt_verbose   = False

	try:
		opts, args = getopt.getopt(sys.argv[1:], "hX:")
		for o, a in opts:
			if   o in ("-h"):
				show_help()
				sys.exit(0)
			elif o in ("-v"):
				opt_verbose   = True
			elif o in ("-X"):
				opt_extra_pkgs = a

	except getopt.error, msg:
		print "Error while parsing options: %s" % msg
		show_help()
		sys.exit()
	except ValueError, ex:
		print "Invalid value for input parameter: %s" % ex
		show_help()
		sys.exit()
	

	if len(args) != 2:
		show_help()
		sys.exit()
		
	host = args[0]
	expr = args[1]
	
	
	log = logging.getLogger('poller')
	log.setLevel( 1 )
	hdlr = logging.StreamHandler()
	hdlr.setFormatter( logging.Formatter('%(asctime)s %(levelname)s> %(message)s') )
	hdlr.setLevel(1)
	log.addHandler(hdlr)
	
	
	
	
	params = {}
	
	print "* enter variables and expression *"
	input_e = ''
	expression_started = False
	lines = sys.stdin.readlines()
	for l in lines:
		l = l.strip()
		if len(l) == 0:
			expression_started = True
			continue
			
		if not expression_started:
			if l.find("=") < 0:
				continue
			param_def = l.split('=')
			if len(param_def) == 2:
				#  $var = value
				name = param_def[0].strip()
				val  = param_def[1].strip()
				params[ name ] = val
			else:
				# $var = type = value
				name  = param_def[0].strip()
				atype = param_def[1].strip()
				val   = param_def[2].strip()
				params[ name ] = convert_str_param_to_type(name, val, atype)
		else:
			input_e += l
			
	input_e = input_e.strip()
	if input_e:
		if expr == None:
			expr = input_e
		else:
			log.warning("Expression already specified in the command line")




	#
	# Check expression
	#
	
	#
	# LOAD SYMBOLS
	#
	symbols = {}
	
	print "Loading system symbols"
	symbols.update(  load_system_expr_functions() )
	
	
	if opt_extra_pkgs:
		print "Loading symbols from extra packages", opt_extra_pkgs 
		extra_s = load_extra_expr_functions( opt_extra_pkgs.split(':') )
		symbols.update( extra_s )
		
	for symbol in sorted(params):
		v = params[symbol]
		log.debug('\t%s = %r %s', symbol, v, type(v))


	for s,v in sorted(symbols.items()):
		log.debug('\t%s = %r %s', s, v, type(v))

	#
	# COMPILE EXPRESSION
	#
	# FIX: cache compiled expr tree!
	expr_tree = compile_str_expr(log, expr)
	
	#
	# EVALUATE
	#
	context = ExecutionContext()
	context.setLog( log )
	context.update(params)
	context.update(symbols)
	
	context['$node'] = host
	
	print "Evaluating: ", str(expr_tree)
	# Evaluate expression
	value = expreval(log, context, expr_tree)
	expr_tree.evaluated()
	
	info = context.getReason()
	
	print "VALUE: ", value, "(", type(value), ")"
	print "INFO : ", info


if __name__ == "__main__":
	main()
