# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os
import re

from setuptools import setup
from setuptools import find_packages

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def extract_symbol(version_file, symbol):

	# __version__ = "...."
	# __version__ = '....'
	VSRE = r"^__%s__\s*=\s*['\"]([^'\"]*)['\"]" % (symbol)

	verstrline = open(version_file, "rt").read()

	mo = re.search(VSRE, verstrline, re.M)
	if not mo:
		raise RuntimeError("Unable to find %s string in %s." % (symbol, version_file,))

	return mo.group(1)

def get_version_from_file():
	try:
		f = open('VERSION')
		data = f.read()
		f.close()
		return data.strip()
	except Exception, e:
		return None
		
#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

print "Reading version from file"
VERSION = get_version_from_file()
if not VERSION:
	print "Reading version from sourcecode"
	VERSION = extract_symbol('sanet_poller/__init__.py', 'version')

if not VERSION:
	raise Exception("Unable to get version")

print "Package version: ", VERSION
print "Found following packages:"
PACKAGES = find_packages()
for name in PACKAGES:
	print "\t", name


REQUIREMENTS = [

	# SANET Base library 
	'sanet_libsnmp',
	'sanet_common',
	
	# Pacchetti richiesti ma teoricamente gia' installati con sanet_common
	'ipaddr',
	
	# Nota: Il server redis sarebbe opzionale ma il poller "de facto" usa redis per gestire la cache, quindi e' praticamente obbligatorio.
	#       Non comunque e' possibile specificare tramite 'setuptools' di installare il server redis.
	#
	# 	apt-get install 'redis-server'     
	#

	# Alcune funzioni del poller permettono di parlare direttamente con redis
	# per implementare controlli. Forse non verrano mai usate, ma meglio obbligare la presenza
	# della libreria per comodita.
	# apt-get install 'python-redis'
	'redis' , 
	
	# Alcune funzion del poller richiedono pycurl
	'pycurl',
]

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

setup(
	name             = 'sanet-poller',
	version          = VERSION,
	description      = 'SANET3 (Netsentry) Agent',
                         
	packages         = PACKAGES,

	install_requires = REQUIREMENTS,

	#scripts=[
	#	'scripts/sanet3-poller',
	#],

	# Metadata
	author           = 'dbilli',
	author_email     = 'dbilli@labs.it',
	url              = 'http://labs.it/',
	license          = "GNU",
)

