# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

################################################################################
Datagroup Checker
################################################################################

TODO

"""


import os
import time
import logging
import pprint
import traceback

from optparse import OptionParser
from optparse import make_option

#----------------------------------------------------------------------#

from sanet_common.pythonutils import load_symbol
from sanet_common.uuids import is_uuid

from sanet_common.ipc import IPCConnection

#----------------------------------------------------------------------#

from sanet_common.sanetdefs.constants import *

from sanet_poller.lib.classes import DataGroupChecker
from sanet_poller.lib.functions import load_system_expr_functions

from remotepoller.lib.remotepolling import RemoteConfiguration

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#



class RemoteDataGroupChecker(object):

	option_list = (
		#make_option('--tenant', action='store'     , dest='tenant'   , default=None , help='Select the tenant.'),
		make_option('-l'      , action='store'     , dest='loglevel' , default=20   , help='Set log level (<20 DEBUG, <30 INFO, <40 WARNING, <50 ERROR.'),
		make_option('--broker', action='store'     , dest='broker'   , default="sanet/sanet@localhost" , help='Send check result to the main server.'),
		#make_option('--update', action='store_true', dest='do_update', default=False, help='Updata datagroup data.'),
	)

	def __init__(self):
		self.logger             = None
		self.log_handler        = None
		self.log_handler_stream = None

		self.cache_handler_name   = None
		self.cache_handler_params = None

	def dispose(self):
		if self.log_handler        is not None: self.logger.removeHandler(self.log_handler)
		if self.log_handler_stream is not None: self.logger.removeHandler(self.log_handler_stream)

	def init_log(self, log_level, stream=None):

		format_str = '%(asctime)s %(levelname)s %(threadName)s > %(message)s'

		logger = logging.getLogger('test')
		logger.setLevel( log_level )

		self.logger = logger

		hdlr = logging.StreamHandler(stream=stream)
		hdlr.setFormatter( logging.Formatter(format_str) )
		hdlr.setLevel( log_level )
		logger.addHandler(hdlr)

		self.log_handler = hdlr

		hdlr = logging.StreamHandler()
		hdlr.setFormatter( logging.Formatter(format_str) )
		hdlr.setLevel( log_level )
		logger.addHandler(hdlr)	

		self.log_handler_stream = hdlr

		return logger

	def set_cache_params(self, handler_module_name, init_params):
		self.cache_handler_name   = handler_module_name
		self.cache_handler_params = init_params

	def _init_cache_manager(self, handler_module_name, init_params):
		handler_class = load_symbol( handler_module_name )
		handler       = handler_class(**init_params)
		
		return handler

	def create_connection(self, logger, broker_address):

		connect_options = {}

		logger.debug("Broker address: %s", broker_address )

		broker_auth, broker_address = broker_address.split('@')

		broker_auth    = broker_auth.split('/')
		broker_address = broker_address.split(':')

		host = broker_address[0]
		if len(broker_address) > 1: 
			try:
				if broker_address[1]:
					connect_options['port']     = int(broker_address[1])
			except Exception, e:
				pass

		if broker_auth:
			connect_options['username'] = broker_auth[0]
			if len(broker_auth) > 1:
				connect_options['password'] = broker_auth[1]

		logger.debug("Opening connection with server %s", host )
		logger.debug("Options:")
		for k,v in connect_options.items():
			logger.debug("\t%s = %s", k, v)

		c = IPCConnection( host, **connect_options)
		c.open()
		logger.debug("Connection created!")

		return c

	def handle(self):

		parser = OptionParser(option_list=self.option_list)
		(options, args) = parser.parse_args()
		options = options.__dict__

		do_update      = False #options['do_update']
		broker_address = options['broker']

		datagroup_id = args[0]

		if do_update:
			if not (broker_address):
				raise Exception("Cannot update without broker or in direct mode.")

		log_level = int(options['loglevel'])

		#
		# Init ad execute
		#
		from StringIO import StringIO
		stream = StringIO()

		logger = self.init_log(log_level, stream=stream)
		connection = self.create_connection(logger, broker_address)

		updates_data = self.execute(logger, connection,  datagroup_id)

		return

	def execute(self, logger, connection, tenant_id, datagroup_id, ipc_factory):

		#
		# Creo l'oggetto per accedere alla cache (Redis)
		#
		logger.debug( "Loading cache manager: %s %s...", self.cache_handler_name, self.cache_handler_params)
		#cache_handler = "sanet_common.cache.rediscache.RedisCacheManager"
		#cache_params  = {}
		#cache_manager    = self.init_cache_manager(cache_handler, cache_params)
		cache_manager    = self._init_cache_manager(self.cache_handler_name, self.cache_handler_params)


		# Creiamo l'oggetto che ha il compito fornire i vari oggetti del monitoraggio
		# (nodi,interfacce,ecc...) che potranno servire durante la valutazione
		# del datagroup.
		configuration = RemoteConfiguration(connection, logger, tenant_id, cache_manager)

		#
		# Creo l'oggetto per notificare
		#
		#if do_notify:
		#	mail_handler_class = load_mail_handler_class(conf.SANETD_MAIL_HANDLER)
		#	mail_handler       = mail_handler_class(logger, conf.SANETD_MAIL_HANDLER_PARAMS)
		#	mail_sender        = SanetdSimpleMailSender(self.logger, mail_handler)
		#	mail_sender.start()

		#
		# Creo l'oggetto per gestire il database dei datasource (RRDTool)
		#
		#logger.debug( "Loading datasource store...")
		#datasource_store = init_datasource_store(conf.DATASOURCE_STORAGE_HANDLER, conf.DATASOURCE_STORAGE_PARAMS)

		try:
			logger.debug( "Loading data...")

			#tenant_name = options['tenant']
			#if tenant_name == None:
			#	tenant  = tenant_get_primary()
			#	if tenant == None:
			#		raise Exception("No primary tenant found. You must specify a tenant name")
			#	logger.debug( "Using default tenant: %s", tenant)
			#else:
			#	tenant = tenant_get(tenant_name)
			#db_set_current_tenant_schema(tenant.name)
				
			#
			# Verfifichiamo se l'ID inserito dall'utente e' un UUID o un
			# path (mason;;reachability). Per uniformare il codice
			# quello che useremo alla fine e' comunque UUID.
			#
			#if not is_uuid(datagroup_id):
			#	logger.debug("UUID: %s", datagroup_id)
			#	datagroup = DataGroup.objects.get(path=datagroup_id)
			#	datagroup_id = datagroup.uuid

			
			#
			# Carichiamo in sequenza
			#  - le funzioni standard del poller
			#  - eventuali funzioni contenute in moduli python esterni
			#    specificati tramite nel file di configurazione di SANET  (LOCAL_POLLER_EXTRA_MODULES)
			#
			symbols = {}

			logger.debug( "Loading system symbols")
			system_symbols = load_system_expr_functions()

			#logger.debug("Loading extra symbols from %s", conf.LOCAL_POLLER_EXTRA_MODULES)
			#extra_symbols = load_extra_expr_functions(conf.LOCAL_POLLER_EXTRA_MODULES)
			extra_symbols = {}

			symbols.update(system_symbols)
			symbols.update(extra_symbols)

			configuration.set_global_symbols(symbols) 

			# Otteniamo l'oggetto PDataGroup associato al datagroup selezioanto
			pdg = configuration.get_datagroup(datagroup_id)

			# Istanziamo la classe che contiene tutta la logica per la valutazione di un 
			# datagroup.
			dgc = DataGroupChecker(configuration, logger, cache_manager, ipc_factory, pdg )#, system_symbols)

			logger.debug( "########################################################" )
			logger.debug( "###################### CHECKING ########################" )
			logger.debug( "########################################################" )

			logger.info( "Start checking...")

			# CHECK
			t1 = time.time()
			dgc.check( time.time() )
			t2 = time.time()
			
			logger.info( "Check finished! Elapsed %s", t2 - t1)

			# Ci facciamo dare tutti i dati aggiornati/modificati durante il check
			# che dovrebbero essere mandati al processo SANETD per la memorizzazione
			# - valori dei datasource
			# - stato delle condition
			# - aggiornamenti su dati degli elementi (ifindex, ecc...)

			updates_data = dgc.get_datagroup_all_updates()

			logger.debug("\n%s", pprint.pformat(updates_data))

			for data in updates_data:

				logger.info("Update block:")

				for uuid, updates in data['elements'].items():
					#e = MonitoredElement.objects.get(uuid=uuid)
					#e = e.get_specific_element()
					e = configuration.get_element_by_id(uuid)
					logger.info("\tElement: %s", e.path)
					for f, v in updates.items():
						logger.info("\t\t%s =  %s", f, v)

				#dg = DataGroup.objects.get(uuid=data['datagroup_id'])
				dg = configuration.get_datagroup(data['datagroup_id'])

				#dg = DataGroup.objects.get(uuid=data['datagroup_updates']['uuid'])
				
				logger.info( "\tDataGroup: %s", dg.path)
				if data['datagroup_updates']:
					for field, value in data['datagroup_updates']['fields'].items():
						logger.info( "\t\t%s = %s", field, value)

				for uuid, ds_data in data['datasources_updates'].items():

					ds = configuration.get_datasource(uuid)

					logger.info( "\t\tDatasource: %s", ds.path)
					logger.info( "\t\t\tInfo: %s", ds_data['infos'])
					for field, value in ds_data['fields'].items():
						if field == "status":
							value = DSSTATE_TO_CODENAME(value)
						logger.info( "\t\t\t%s = %s", field, value)

				for uuid, c_data in data['conditions_updates'].items():

					c = configuration.get_condition(uuid)

					logger.info( "\t\tCondition: %s", c.path)
					logger.info( "\t\t\tInfo: %s", c_data['infos'])
					for field, value in c_data['fields'].items():
						if field in ["status","statuslast", "statuslastnotable"]:
							value = CSTATE_TO_CODENAME(value)
						logger.info( "\t\t\t%s = %s", field, value)

			return updates_data

		except Exception, e:
			logger.error("Error: %s %s -> %s", type(e), e, traceback.format_exc())

		finally:
			# Quando questo comando viene eseguito dentro la CLI 
			# facciamo cleanup altrimenti ad ogni execute creiamo logger
			# aggiuntivi.
			#self.remove_logger()
			pass

