# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
################################################################################
Remote polling
################################################################################

TODO

"""


import os

from sanet_common.ipc import RESPONSE_SUCCESS
from sanet_common.ipc import encode_request, decode_response

from sanet_common.sanetdefs.constants import *

from sanet_common.sanetdefs.ipc import SANETD_CLT_QUEUE, TEMP_REPLY_QUEUE

from sanet_poller.lib.exceptions import *
from sanet_poller.lib.classes import AbstractLocalPollerConfiguration

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class RemoteConfiguration(AbstractLocalPollerConfiguration):
	"""
	Questa versione di PollerConfiguration carica i dati per il poller
	interrogando il server sanetd remoto.
	"""

	def __init__(self, connection, logger, tenant_id, cache_manager):

		super(RemoteConfiguration, self).__init__(logger, tenant_id, cache_manager)

		self.cache_manager = cache_manager

		self._loaded_conditions = {}

		self._connection = connection

		self._connection.settimeout(3)

		#self._connection.bind("temp-remote_exec-pid-%s" % (os.getpid()))
		bind_address = TEMP_REPLY_QUEUE("remote_exec-pid-%s" % (os.getpid()))
		self._connection.bind(bind_address)

		self.tenant_id = tenant_id

	def send_and_receive(self, api_command, arguments):

		command      = "remote_agent_api"
		command_args = [api_command,] + arguments  

		c = self._connection

		# SEND
		request = encode_request(command, *command_args)
		#c.send_msg( 'sanetd-ctl' , request, blocking=True)
		c.send_msg( SANETD_CLT_QUEUE , request, blocking=True)

		# RECEIVE
		addr, data = c.receive_msg(ack_msg=True)
		if data is None:
			raise Exception("Timeout")
		# EXIT
		c.close()

		response = decode_response(data)
		if response['status'] == RESPONSE_SUCCESS:
			return response['content']
		else:
			raise Exception("Error from remote host: %s" % (response['content']))	

	def _get_cluster(self, cid):
		try:
			return self.send_and_receive("get_cluster", [self.tenant_id, cid])
		except Exception, e:
			raise ClusterException("cluster %s not found. (not loaded? %s)" % (cid, e))

	def _get_cluster_elements_ids(self, cid):
		try:
			return self.send_and_receive("get_cluster_elements_ids", [self.tenant_id, cid])
		except Exception, e:
			raise ClusterException("cluster %s not found. (not loaded? %s)" % (cid, e))

	def _get_datagroup(self, dgid):
		try:
			return self.send_and_receive("get_datagroup", [self.tenant_id, dgid])
		except Exception, e:
			raise DataGroupNotFoundException("DataGroup %s not found (%s:%s)" % (dgid, type(e), e))

	def _get_datagroup_params(self, dgid):
		try:
			return self.send_and_receive("get_datagroup_params", [self.tenant_id, dgid])
		except Exception, e:
			raise DataGroupNotFoundException("DataGroup %s not found (%s:%s)" % (dgid, type(e), e))

	def _get_datasource(self, dsid):
		try:
			return self.send_and_receive("get_datasource", [self.tenant_id, dsid])
		except Exception, e:
			raise DataSourceNotFoundException("DataSource %s not found. %s" % (dsid, e))

	def _get_condition(self, cid):
		try:
			return self.send_and_receive("get_condition", [self.tenant_id, cid])
		except Exception, e:
			raise ConditionNotFoundException("Condition %s not found. %s" % (cid, e))

	def _get_condition_by_path(self, cid):
		try:
			return self.send_and_receive("get_condition_by_path", [self.tenant_id, cid])
		except Exception, e:
			raise ConditionNotFoundException("Condition %s not found. %s" % (cid, e))

	def _get_condition_by_classification(self, path, node_id, subelement_id):
		try:
			return self.send_and_receive("get_condition_by_classification", [self.tenant_id, node_id, subelement_id])
		except Exception, e:
			raise ConditionNotFoundException("Condition %s not found. %s" % (cid, e))

	def _get_element_by_id(self, e_id):
		try:
			return self.send_and_receive("get_element_by_id", [self.tenant_id, e_id])
		except Exception, ex:
			raise ResourceNotFoundException("Element %s not found. %s" % (e_id, ex))

	def _get_element_by_parent_id(self, e_id, kind):
		try:
			return self.send_and_receive("get_element_by_parent_id", [self.tenant_id, e_id, kind])
		except Exception, ex:
			raise ResourceNotFoundException("Error while getting elements by parent id %s: %s" % (e_id, ex))


	def _get_linked_iface_by_id(self, iface_id, link_type = None):
		try:
			data = self.send_and_receive("get_linked_iface_by_id", [self.tenant_id, iface_id, link_type])
			return data
		except Exception, e:
			raise ResourceNotFoundException("DataSource %s not found. %s" % (iface_id, e))
