# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

################################################################################
Logic
################################################################################

TODO

"""

import os
import sys
import time
import json

from sanet_poller.main import launch_agent
from sanet_poller.main import __hack_keyboard_interrupt

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

#
# FIX: remove this switch
#
from sanet_poller.agent               import BaseAgent
from sanet_poller.agent_new_scheduler import BaseAgentAdvancedScheduling

SANET_NO_SCHEDULINGGROUPS = os.environ.get('SANET_NO_SCHEDULINGGROUPS', False)
if SANET_NO_SCHEDULINGGROUPS:
	Sanet3Agent = BaseAgent
else:
	Sanet3Agent = BaseAgentAdvancedScheduling

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class Sanet3RemoteAgent(Sanet3Agent):

	def wait_for_stop(self):
		
		super(Sanet3RemoteAgent, self).wait_for_stop()
		
		self.logger.info("*"*30);
		self.logger.info("*"*30);
		self.logger.info("*"*30);
		self.logger.info("*"*30);
		self.logger.info("*"*30);
		self.logger.info("*"*30);
		
		supervisor_process_pid = os.getppid()
		
		# Try killing the daemon process       
		try:
			# Send signals until the process dies
			while 1:
				print "Sending SIGHUP signal to supervisor (%s)..." % (supervisor_process_pid),
				os.kill(supervisor_process_pid, signal.SIGHUP)

				print "Waiting"
				time.sleep(1)
		except OSError, e:
			
			# did we kill the process or what else?
			if e.errno == errno.ESRCH:
				print "Process terminated...",
				#if os.path.exists(PID_FILE):
				#	os.remove(PID_FILE)
			else:
				sys.stderr.write("Error while sending signal to process %s: %s" % (supervisor_process_pid, e))
				sys.exit(1)		

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def read_config_file(file_path):

	GLOBAL_SECTION = 'configuration'

	import ConfigParser

	try:
		config = ConfigParser.ConfigParser()
		config.readfp(open(file_path, 'r'))

		settings = config.items(GLOBAL_SECTION)
		d = dict( settings )

		d = set_settings_defaults(d)

		return d

	except ConfigParser.Error, e:
		raise Exception("Error while reading configuration file '%s': %s" % (file_path, str(e),))


def set_settings_defaults(settings):

	assert( settings['server_host'] )

	assert( settings['tenant'] )
	assert( settings['agent']  )

	#
	# Set defaults
	#
	if not settings.get('sanet_user'         , None): settings['sanet_user'         ] = 'sanet'
	if not settings.get('sanet_password'     , None): settings['sanet_password'     ] = 'sanet'

	# Log
	if not settings.get('log_file'           , None): settings['log_file'           ] = "/var/log/sanet_remote_%s.log" % (settings['agent'])
	if not settings.get('log_level'          , None): settings['log_level'          ] = 20

	settings['log_level'] = int(settings['log_level'])

	# Caching
	if not settings.get('cache_module'       , None): settings['cache_module'       ] = "sanet_common.cache.rediscache.RedisCacheManager"
	if not settings.get('cache_config'       , None): settings['cache_config'       ] = '{}'

	json.loads(settings['cache_config'])

	# Modules
	if not settings.get('extra_pkgs'         , None): settings['extra_pkgs'         ]   = ''

	# Discovery
	if not settings.get('autodiscovery'      , None): settings['autodiscovery'      ] = 0
	if not settings.get('discovery_log_file' , None): settings['discovery_log_file' ] = "/var/log/sanet_remote_%s_discovery.log" % (settings['agent'])
	if not settings.get('discovery_log_level', None): settings['discovery_log_level'] = 20
	settings['discovery_log_level'] = int(settings['discovery_log_level'])

	# SnmpTrap
	if not settings.get('snmptrap'               , None): settings['snmptrap'               ] = 0
	if not settings.get('snmptrap_log_file'      , None): settings['snmptrap_log_file'      ] = "/var/log/sanet_remote_%s_trapper.log" % (settings['agent'])
	if not settings.get('snmptrap_log_level'     , None): settings['snmptrap_log_level'     ] = 20
	settings['snmptrap_log_level']   = int(settings['snmptrap_log_level'])

	if not settings.get('snmptrap_listen_address', None): settings['snmptrap_listen_address'] = '0.0.0.0'

	if not settings.get('snmptrap_listen_port'   , None): settings['snmptrap_listen_port'   ] = 162
	settings['snmptrap_listen_port'] = int(settings['snmptrap_listen_port'])

	return settings

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def launch_remote_poller_from_settings(settings, logtostdout=False):

	settings = set_settings_defaults(settings)

	try:
		cache_params = json.loads(settings['cache_config'])
	except Exception, e:
		raise Exception("Error while reading 'cache_config' option: %s" % (str(e)))

	agent = Sanet3RemoteAgent()

	launch_agent(
		agent               = agent, 

		address             = ( settings['server_host']          , settings.get('server_port'    , None) ),
		auth                = ( settings.get('server_user', None), settings.get('server_password', None) ),

		tenantid            = settings['tenant'],

		agentid             = settings['agent'],

		threads_number      = settings.get('threads',None),

		logfile             = settings.get('log_file', None),
		loglevel            = int(settings.get('log_level', 20)),
		logtostdout         = logtostdout,

		cache_module        = settings.get('cache_module', None),
		cache_module_params = cache_params,

		extra_pkgs          = settings.get('extra_pkgs', None),
	)

	# never returns
	return

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

if __name__ == "__main__":

	print "*** DEBUG MODE ***"
	print "*** DEBUG MODE ***"
	print "*** DEBUG MODE ***"
	print "*** DEBUG MODE ***"
	print "*** DEBUG MODE ***"

	config_file_name = sys.argv[1]
	print "config file:", config_file_name

	__hack_keyboard_interrupt()

	settings = read_config_file(config_file_name)

	launch_remote_poller_from_settings(settings, logtostdout=True)

	# never returns

