# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

################################################################################
Main
################################################################################

TODO

"""


import os
import sys
import getopt
import errno 
import signal
import time
import traceback
from operator import attrgetter

BASE_DIR = os.path.abspath(os.path.join(os.path.dirname(os.path.abspath(__file__)), ".." ))
sys.path.append( BASE_DIR )

#-----------------------------------------------------------------------------#
# IMPORTS                                                                     #
#-----------------------------------------------------------------------------#

from sanet_common.osutils.daemonize import daemonize
from sanet_common.osutils.proctitle import get_proc_title, set_proc_title
from sanet_common.osutils.osutils   import convert_exit_status_to_verb_reason


from remotepoller.logic import read_config_file

#-----------------------------------------------------------------------------#
# CONSTANTS                                                                   #
#-----------------------------------------------------------------------------#

VAR_DIR = '/var/run'

LOG_DIR = '/var/log'

#-----------------------------------------------------------------------------#
# MAIN PROGRAM                                                                #
#-----------------------------------------------------------------------------#

def sanet_remote_poller_unix_main(argv=None):

	agent_name   = ''
	config_file  = ''
	pid_file     = ''
	log_file     = ''
	do_daemonize = True

	if argv is None:
		argv = sys.argv

	try:
		opts, args = getopt.getopt(sys.argv[1:], "hc:p:n:k")
		for o, a in opts:

			if   o in ("-h"):
				print_help()
				sys.exit()

			elif o in ("-n"):
				agent_name  = a

			elif o in ("-c"):
				config_file = a

			elif o in ("-p"):
				pid_file = a

			elif o in ("-k"):
				do_daemonize = False

	except getopt.error, msg:
		sys.stderr.write("Error while parsing options: %s\n" % (msg,))
		print_help()
		sys.exit()
	except ValueError, e:
		sys.stderr.write("Invalid value for input parameter: %s\n" % (e))
		print_help()
		sys.exit()

	if len(args) != 1:
		sys.stderr.write("Wrong number of arguments: %s\n" % (args))
		print_help()
		sys.exit(-1)

	cmd = args[0]

	if   cmd == 'start':

		if not pid_file:
			sys.stderr.write("Pid file '%s' not found\n" % (pid_file))
			sys.exit(-1)

		if not config_file:
			sys.stderr.write("Config file '%s' not found\n" % (config_file))
			sys.exit(-1)

		start(agent_name, pid_file, config_file, do_daemonize=do_daemonize)

	elif cmd == 'stop':

		if not pid_file:
			sys.stderr.write("Pid file '%s' not found\n" % (pid_file))
			sys.exit(-1)

		stop(agent_name, pid_file)
	else:
		sys.stderr.write("Invalid command: %s\n" % (cmds[0]))

		print_help()

		sys.exit(-1)

	sys.exit(0)

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

log_file_name = None

def init_log(file_name):

	global log_file_name

	log_file_name = file_name

def log(text):
	pid = os.getpid()
	text = "%s [supervisor] [pid %s] %s\n" % (time.ctime(), pid, text)

	print text
	log_to_file(log_file_name, text)

def log_to_file(file_name, text):
	try:
		f = open(file_name,'a')
		f.write(text )
		f.close()
	except:
		pass

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def start(agent_name, pid_file, config_file, do_daemonize=True):

	settings = read_config_file(config_file)

	settings['agent_name'] = agent_name

	init_log(settings['log_file'])

	try:
		try:
			pid = read_pid_file(pid_file)
		except IOError, e:
			print e
			pid = None

		if pid != None:
			if is_supervisor_process(pid):
				sys.stderr.write("Already running. (PID %s)\n" % pid)
				sys.exit(-1)
			else:
				if is_process_running(pid):
					sys.stderr.write("Unknown process with pid %s!\n" % pid)
					sys.exit(-1)
				else:
					# if the process is dead, go on!
					sys.stderr.write("PID file not empty, but process %s not running!\n" % pid)

		print "Launching remote poller daemon"

		if do_daemonize:
			daemonize()

		# Do after daemonize!
		write_pid_on_file(pid_file)

		init_children_jobs(settings)

		respawn_children_until_stop()

		remove_pid_file(pid_file)

	except Exception, e:

		text = traceback.format_exc()

		sys.stdout.write(text)

		log( "Error: %s" % (text) )

	return 0

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def stop(agent_name, pid_file):

	try:
		pid = read_pid_file(pid_file)
	except IOError, e:
		sys.stderr.write("Unable to read pid file %s: %s\n" % (pid_file, e) )
		pid = None

	print "Stopping agent: %s (pid: %s - %s)" % (agent_name, pid, pid_file)

	if not pid:
		sys.stderr.write("pidfile %s does not exist. Process not running?\n" % (pid_file) )
		sys.exit(1)

	# Try killing the daemon process       
	try:
		# Send signals until the process dies
		while 1:
			print "Sending term signal to supervisor (%s)..." % (pid),
			os.kill(pid, signal.SIGTERM)

			print "Waiting"
			time.sleep(1)
	except OSError, e:
		# did we kill the process or what else?
		if e.errno == errno.ESRCH:
			print "Process terminated...",
			#if os.path.exists(PID_FILE):
			#	os.remove(PID_FILE)
		else:
			sys.stderr.write("Error while sending signal to process %s: %s" % (pid, e))
			sys.exit(1)

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def write_pid_on_file(pid_file):
	pid = str(os.getpid())
	f = open(pid_file, 'w+')
	f.write("%s\n" % (pid))
	f.close()

def read_pid_file(name):
	f = file(name,'r')
	pid = int(f.read().strip())
	f.close()
	return pid

def remove_pid_file(pid_file):
	try:
		if os.path.exists(pid_file):
			os.remove(pid_file)
	except Excetion, e:
		print e
		pass

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def is_supervisor_process(pid):
	
	return False
	
	"""
	try:
		f = file('/proc/%s/cmdline' % pid,'r')
		cmd_argv = f.read().split('\x00')
		f.close()
	except IOError, er:
		sys.stderr.write("Process %s does not exist.\n" % pid)
		return False

	# Without setproctitle
	print "Checking running process: %s" % ( cmd_argv )
	
	if cmd_argv[0] == 'python' and cmd_argv[1].find('sanetd/main.py') != -1:
		return True

	# With setproctitle
	if cmd_argv[0].find('python') != -1 and cmd_argv[0].find('sanetd/main.py') != -1:
		return True
	
	return False
	"""

def is_process_running(pid):

	process_filename = '/proc/%s/cmdline' % pid

	if os.path.exists(process_filename):
		return True
	else:
		return False

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def start_real_remote_poller_child_process(params=None):

	settings = params

	from remotepoller.logic import launch_remote_poller_from_settings

	try:
		log("%s" % (settings))
		launch_remote_poller_from_settings(settings)
	except Exception, e:
		text = traceback.format_exc()
		log( "Error while launching agent: %s" % (text) )

	# Never returns
	return

#-----------------------------------------------------------------------------#
                                
def start_remote_poller_discovery_process(params=None):

	settings = params

	from remotepoller.discovery.agent import launch_remote_discovery_from_settings

	try:
		log("%s" % (settings))
		launch_remote_discovery_from_settings(settings)
	except Exception, e:
		text = traceback.format_exc()
		log( "Error while launching agent: %s" % (text) )

	# Never returns
	return

#-----------------------------------------------------------------------------#

def start_remote_poller_checker_process(params=None):

	settings = params

	from remotepoller.checker.agent import launch_remote_checker_from_settings

	try:
		log("%s" % (settings))
		launch_remote_checker_from_settings(settings)
	except Exception, e:
		text = traceback.format_exc()
		log( "Error while launching agent: %s" % (text) )

	# Never returns
	return

#-----------------------------------------------------------------------------#

def start_remote_poller_snmptrap_process(params=None):

	settings = params

	from remotepoller.snmptrapper.agent import launch_remote_trapper_from_settings

	try:
		log("%s" % (settings))
		launch_remote_trapper_from_settings(settings, logtostdout=False)
	except Exception, e:
		text = traceback.format_exc()
		log( "Error while launching agent: %s" % (text) )

	# Never returns
	return

def start_remote_poller_netinfo_process(params=None):
	settings = params

	from remotepoller.netinfo.agent import launch_remote_netinfo_from_settings

	try:
		log("%s" % (settings))
		launch_remote_netinfo_from_settings(settings, logtostdout=False)
	except Exception, e:
		text = traceback.format_exc()
		log( "Error while launching agent: %s" % (text) )

	# Never returns
	return

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

# FIX: move to default_settings.py?
WAITTIME_AFTER_KILL = 1   #seconds


class SubProcessInfo(object):

	def __init__(self, name, action, pidfile_name, params={}, priority=0):

		super(SubProcessInfo, self).__init__()

		self.name        = name
		self.action      = action
		self.params      = params
		self.priority    = priority

		self.pid         = 0
		self.pidfile     = pidfile_name
		self.term_sent   = 0   # time the term signal was sent
		self.graceperiod = 10  # seconds. Interval between the (soft) term signal and the kill signal

	def __str__(self):
		return '<%s:%s>' % (self.name, self.pid if self.pid != None else '-')

	def run_action(self):
		self.action(self.params)



STOP_SUPERVISOR = False
#RESTART_ALL     = False

subprocesses = []


def init_children_jobs(settings):

	global subprocesses

	log("Preparing subprocesses...")

	#
	# MAIN PROCESS (POLLER)
	#
	child_name    = '%s-%s' % (settings['agent_name'], settings['agent'])
	child_pidfile = '%s.pid' % (child_name)

	job = SubProcessInfo(name=child_name, 
	                     pidfile_name=child_pidfile, 
	                     action=start_real_remote_poller_child_process,
	                     params=settings,
	                    )  
	subprocesses.append(job)

	#
	# CHECKER PROCESS
	#
	child_name    = '%s-%s-checker' % (settings['agent_name'], settings['agent'])
	child_pidfile = '%s.pid' % (child_name)

	job = SubProcessInfo(name=child_name, 
	                     pidfile_name=child_pidfile, 
	                     action=start_remote_poller_checker_process,
	                     params=settings,
	                    )  
	subprocesses.append(job)


	#
	# NETINFO HELPER PROCESS
	#
	child_name    = '%s-%s-netinfo' % (settings['agent_name'], settings['agent'])
	child_pidfile = '%s.pid' % (child_name)

	job = SubProcessInfo(name=child_name, 
			     pidfile_name=child_pidfile, 
			     action=start_remote_poller_netinfo_process,
			     params=settings,
			    )  
	subprocesses.append(job)


	#
	# AUTODISCOVERY PROCESS
	#
	if str(settings['autodiscovery']) == "1":

		child_name    = '%s-%s-discovery' % (settings['agent_name'], settings['agent'])
		child_pidfile = '%s.pid' % (child_name)

		job = SubProcessInfo(name=child_name, 
		                     pidfile_name=child_pidfile, 
		                     action=start_remote_poller_discovery_process,
		                     params=settings,
		                    )  
		subprocesses.append(job)

	#
	# SNMP TRAP PROCESS
	#
	if str(settings['snmptrap']) == "1":
		child_name    = '%s-%s-snmptrap' % (settings['agent_name'], settings['agent'])
		child_pidfile = '%s.pid' % (child_name)

		job = SubProcessInfo(name=child_name, 
		                     pidfile_name=child_pidfile, 
		                     action=start_remote_poller_snmptrap_process,
		                     params=settings,
		                    )  
		subprocesses.append(job)



	for p in subprocesses:
		log("Loaded job: %s" % (p.name))

	return


def supervisor_on_sigterm(signum, frame):
	global STOP_SUPERVISOR
	STOP_SUPERVISOR = True


#def supervisor_on_sighup(signum, frame):
#	global RESTART_ALL
#	RESTART_ALL = True

def respawn_children_until_stop():

	global STOP_SUPERVISOR, subprocesses

	not_running_by_name = dict( [ (p.name, p) for p in subprocesses ] )
	running_by_pid      = {}

	try:
		STOP_SUPERVISOR = False

		supervisor_pid = os.getpid()

		signal.signal(signal.SIGTERM, supervisor_on_sigterm)
		#signal.signal(signal.SIGHUP , supervisor_on_sighup)

		original_proc_title = get_proc_title()

		set_proc_title( original_proc_title + ' (supervisor)' )

		log("Starting main loop")

		while True:
			if not STOP_SUPERVISOR: # or RESTART_ALL):

				for child in not_running_by_name.values():

					pid = os.fork()
					if pid:
						# father
						child.pid = pid

						del not_running_by_name[ child.name ] 
						running_by_pid[ pid ] = child

						log("spawned %s" % (child))
					else:
						# son
						signal.signal(signal.SIGTERM, signal.SIG_DFL )

						my_pid = os.getpid()
						log("started child %s" % (child))
						write_pid_on_file(child.pidfile)

						#set_proc_title( original_proc_title + ' (%s)' % (child.name,))
						set_proc_title( child.name )

						try:
							child.run_action() # This should not return
						except Exception, e:
							log("EXCEPTION child %s: %s" % (child, e))
						exit() # This should not happen
			else:
				children = running_by_pid.values()

				children = sorted(children, key=attrgetter('priority'))
				children.reverse()

				for child in children:

					if not child.term_sent:
						log("sending TERM to %s " % (child))
						os.kill(child.pid, signal.SIGTERM)
						child.term_sent = time.time()

					elif time.time() - child.term_sent < child.graceperiod:
						log("re-sending TERM to %s" % (child))
						os.kill(child.pid, signal.SIGTERM)

					else:
						log("sending KILL to %s" % (child))
						os.kill(child.pid, signal.SIGKILL)

					break # Go through a wait before killing next one

				if len(running_by_pid) == 0:

					log("no more children alive.")
					if STOP_SUPERVISOR:
						log("exiting")
						return

					#elif RESTART_ALL:
					#	log("now restarting")
					#	continue
				else:
					time.sleep(WAITTIME_AFTER_KILL)

			try:
				(pid, status) = os.wait()
			except OSError, e:
				log("Wait interrupted with error %d: %s" % (e.errno, os.strerror(e.errno)))
				log("Stop signal recieved: %s" % ( STOP_SUPERVISOR ), )
			else:
				child = running_by_pid[ pid ]

				try:
					reason = convert_exit_status_to_verb_reason(status)
				except Exception, e:
					reason = "Unreadable reason: %s" % (str(e),)

				log("Child %s exited with status %d: %s" % (child, status, reason))

				if os.path.exists(child.pidfile):
					os.remove(child.pidfile)

				child.pid = 0

				del running_by_pid[pid]
				not_running_by_name[child.name] = child

	except Exception, e:
		log("Unknown error while executing supervisor: %s: %s -> %s" % (type(e), str(e), traceback.format_exc()))

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def print_help():
	print "program -c <config file> -p <pid file>  start"
	print "program                  -p <pid file>  stop"

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

if __name__ == "__main__":

	sanet_remote_poller_unix_main()

