# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
sanetcli
--------

Questo comando lancia la CLI di sanet. I seguenti comandi sono equivalenti: ::

	$ sanet-cli
	
	$ sanet-manage sanetcli
"""
import os
import sys

from django.core.management.base import BaseCommand

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

from cli.prompts import cli_register_menu
from cli.prompts import cli_get_registered_menus

from cli.main import register_apps_cli_hooks

from cli.prompts.show   import show_commands
from cli.prompts.common import common_commands
from cli.prompts.common import exec_commands

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class Command(BaseCommand):

	help = "*** INTERNAL COMMAND. DO NOT USE ***"

	sanet_internal = True

	help = ''
	args = ''

	def add_arguments(self, parser):
		super(Command, self).add_arguments(parser)
		#
		# Positional arguments
		#
		parser.add_argument('filename', nargs='?', default=None)


	def handle(self, *args, **options):

		filename = options['filename']

		f = open( filename , "w+")

		TITLE = "Cli Command Reference"

		print >>f, '#' * len(TITLE)
		print >>f, TITLE
		print >>f, '#' * len(TITLE)
		print >>f, ""
		print >>f, ""
		print >>f, """
		
.. toctree::
   :hidden:

.. contents:: Comandi
   :depth: 2
"""

		register_apps_cli_hooks()

		cli_register_menu('__show_commands__'  , 'base_show_commands'   , show_commands)	
		cli_register_menu('__common_commands__', 'base_common_commands' , common_commands)
		cli_register_menu('__common_commands__', 'other_common_commands', exec_commands)


		generate_help( f, cli_get_registered_menus('__show_commands__')   , "View commands")

		generate_help( f, cli_get_registered_menus('__common_commands__') , "Common commands")

		generate_help( f, cli_get_registered_menus('__apps_commands__')   , "Applications commands")

		f.close()


from cli.clilib.ioslike import T_Pipe

from cli.clilib.ioslike import SORTED_KEYWORDS

levels = '=-_~+^";'

LEVELS = dict([ (i,levels[i]) for i in xrange(len(levels)) ])


def generate_help(f, tree, title):
	
	print >>f, title
	print >>f, '*' * len(title)
	print >>f, ''

	rows = []
	
	for e in SORTED_KEYWORDS(tree):
		rows += __r_help_entry(e, '')
		
	for level, cmd, doc in rows:
		print >>f, cmd
		print >>f,  LEVELS.get(level,'"') * len(cmd)
		print >>f, ""
		print >>f, doc
		print >>f, ""


def __r_help_entry(e, cmdstr, level=0):

	rows = []

	# Remove the all the pipe commands (every entry after the Pipe )
	selected_entries = []
	for sub_e in SORTED_KEYWORDS(e.get_subtree()):
		if isinstance(sub_e, T_Pipe): continue
		selected_entries.append(sub_e)

	cmdstr += e.token_repr

	if hasattr(e, 'help'):
		doc = getattr(e, 'help')
	#if (e.callback):
	#	doc = e.help
	else:
		doc = None

	rows += [ (level, cmdstr, doc)  ]

	for sub_e in selected_entries:
		rows += __r_help_entry(sub_e, cmdstr + ' ', level=level+1)

	return rows

