# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

sanetcli
--------

Si rimanda alla sezione: :ref:`CLI-MAIN`.

"""
import os
import sys
import traceback

import getpass

from sanet_common.termutils import term_red

from optparse import make_option

from django.conf import settings
from django.core.management.base import BaseCommand

from django.utils.translation import activate as lang_activate
from django.utils.translation import deactivate_all as lang_deactivate_all

from django.utils.translation import ugettext as _

from django.contrib.auth.models import User
from django.contrib.auth import authenticate


from core.exceptions import ApplicationException

from core.api.base import tenant_get_primary

from core.api.datasource import check_datasource_store_conf

from cli.main import FatalCliException

from cli.main import launch_sanet_cli

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class Command(BaseCommand):

	help = _("Sanet CLI")

	# Metadata about this command.
	def add_arguments(self, parser):
		parser.add_argument('-d', '--debug'         , action='store_true', dest='debug'          , default=False, help=_('Debug mode')),
                                                                                                 
		parser.add_argument('-l', '--library'       , action='store_true', dest='library'        , default=False, help=_('Configure the library')),
		parser.add_argument('-c', '--config'        , action='store_true', dest='config'         , default=False, help=_('Go straight into configuration mode')),
		parser.add_argument('-t', '--tenant'        , action='store'     , dest='tenant'         , default=None , help=_('Select tenant for configuration mode')),
		parser.add_argument('-T', '--deftenant'     , action='store_true', dest='deftenant'      , default=False, help=_('Use default tenant in configuration mode')),
                                                                                                 
		parser.add_argument('-e', '--echo'          , action='store_true', dest='echomode'       , default=False, help=_('Echo mode on/off')),
                                                                                                 
		parser.add_argument('--colors'              , action='store'     , dest='colormode'      , default='yes', help=_('Enable colors')),
                                                                                                 
		parser.add_argument('-U', '--user'          , action='store'     , dest='username'       , default=None , help=_('User')),
		parser.add_argument('-P', '--password'      , action='store'     , dest='password'       , default=None , help=_('Password')),
		
		parser.add_argument('-X', '--exit-on-error' , action='store_true', dest='exit_on_error'  , default=False , help=_('Close CLI on any error.')),
	

	help = ''
	args = ''

	def handle(self, cmd='', *args, **options):

		# Handle special characters always with UTF8
		reload(sys)
		sys.setdefaultencoding('UTF8')	


		check_datasource_store_conf()

		#
		# Localization
		#
		lang_deactivate_all()
		lang = os.environ.get('LANG', None) or settings.LANGUAGE_CODE
		lang_activate(lang)

		#
		# Cerchiamo di calcolare quale utente utilizzare per l'accesso
		#
		try:
			user = self.calculate_access_user(options)

		except Exception, e:
			msg = str(e)+'\n'
			sys.stderr.write( term_red(msg) )
			sys.exit(-1)

		#
		# Current Tenant
		#
		tenant_name = None
		if options['deftenant'] == True:
			tenant_name = tenant_get_primary().name
		elif options['tenant']:
			tenant_name = options['tenant']

		if tenant_name:
			if options['library']:
				msg = _("You can not configure the library and set the current tenant at the same time.")
				sys.stderr.write( msg + '\n')
				sys.exit(-1)

		#
		# Launch
		#
		try:
			launch_sanet_cli(
				debug         = options['debug'],
				library       = options['library'],
				config        = options['config'],
				tenant_name   = tenant_name,
				echo_mode     = options['echomode'],
				colors_mode   = (options['colormode'] == 'yes'),
				user          = user,
				exit_on_error = options['exit_on_error'],
			)

		except FatalCliException, ex:
			msg = "FATAL ERROR:" + str(ex) + '\n'
			sys.stderr.write( term_red( msg ) )
			sys.exit(-1)

		except ApplicationException, ex:
			msg = "APPLICATION ERROR: " + str(ex) + '\n'
			sys.stderr.write( term_red( msg ) )
			sys.exit(-1)

		except Exception, ex:
			msg = "UNKNOWN EXCEPTION: " + str(ex) + '\n' + traceback.format_exc()
			sys.stderr.write( term_red( msg ) )
			sys.exit(-1)

	def calculate_access_user(self, options): 

		if not int(os.environ.get('CLI_AUTHENTICATED',1)) or not settings.CLI_AUTHENTICATED : 
			return None

		try:
			shell_user = getpass.getuser()

			if shell_user not in settings.CLI_SHELL_ROOT_USERS:
				
				if options['username'] and options['username'] != shell_user:
					raise Exception("You can not specify a different username.")
					
				login_user = options['username'] or shell_user
			else:
				if options['username']:
					login_user = options['username']
				else:
					if settings.CLI_DEFAULT_ADMIN_USER:
						login_user = settings.CLI_DEFAULT_ADMIN_USER
					else:	
						login_user = shell_user


			password = options['password']
			if password is None and shell_user not in settings.CLI_SHELL_ROOT_USERS:
				password = getpass.getpass()

			if not password:
				current_user = User.objects.get(username=login_user)
				return current_user

			current_user = authenticate(username=login_user, password=password)
			if current_user is not None:
				return current_user


			raise Exception("User %s not defined or invalid password!\n" % (login_user))

		except User.DoesNotExist, e:
			raise Exception( _("User '%s' not defined inside Sanet\n") % (login_user))



