# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from sanet_common.sanetdefs.constants import *

from conf import get_configuration

from core.exceptions import ApplicationException
from core.api.base import tenant_rename
from core.api.templates import datagroup_template_rename_in_library

from cli.clilib.ioslike import FatalCliException

from cli.clilib.ioslike import T_Keyword, T_Anyword, T_ValidWord
from cli.clilib.ioslike import SORTED_KEYWORDS, BLANK_ENTRY

from cli.tokens import T_Tenantcompleter

from cli.tokens import *

from cli.prompts import cli_get_registered_menus

from cli.prompts.show.library import show_commands_library

from cli.prompts.config.users import cmd_user_config_prompt
from cli.prompts.config.users import cmd_delete_user
from cli.prompts.config.users import cmd_user_config_rename
from cli.prompts.config.users import cmd_group_config_rename

from cli.prompts.config.users import cmd_group_config_prompt
from cli.prompts.config.users import cmd_delete_group

from cli.prompts.config.tenants import cmd_tenant_config_prompt, cmd_tenant_remove, cmd_tenant_config_default_prompt, cmd_tenant_config_prompt_direct
from cli.prompts.config.tenants import cmd_tenant_create_and_config_prompt

from cli.prompts.config.templates.datagroups import cmd_datagroup_template_config_prompt

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

configuration = get_configuration()

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_rename_datagroup_template_in_library(cli):
	
	# 0      1                  2      3       4
	# rename datagroup-template <name> <name>  fix-logs
	
	env         = cli.env
	
	source      = env['SOURCE']
	
	user        = env['CURRENT_USER']
	
	old_name = cli.canonical_words[2]
	new_name = cli.canonical_words[3]
	
	fix_logs = cli.canonical_words[4] == 'fix-logs'
	
	if old_name == new_name:
		cli.error("Name not changed. Skip")
		return
		
	if fix_logs:
		cli.warning("Update existing logs could take a very long time.")
		cli.warning("Are you sure to continue [y/N]")
		response = cli.read_next_input_line(show_prompt=False)
		if response.strip().lower() != 'y':
			cli.info("Cancelled")
			return
	else:
		cli.warning("Renaming a template without fixing existing logs could cause problems for future data elaborations.")
		cli.warning("Are you sure to continue [y/N]")
		response = cli.read_next_input_line(show_prompt=False)
		if response.strip().lower() != 'y':
			cli.info("Cancelled")
			return
	
	def _feedback(msg):
		cli.info(msg)
	
	datagroup_template_rename_in_library(old_name, new_name, fix_logs=fix_logs, feedback_callback=_feedback)
	
	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#


config_library_prompt = [
	T_Keyword('datagroup-template', help='Configure a datagroup template', subtree=[
		T_DataGroupTemplate(help='Configure a datagroup template', callback=cmd_datagroup_template_config_prompt, pipe=None,),
		T_ValidWord(VALID_DATAGROUP_NAME , help='Configure a datagroup template', callback=cmd_datagroup_template_config_prompt, pipe=None),
	]),
	
	T_KeywordWithHook('rename', help='' , subtree=[
		T_KeywordWithHook('datagroup-template', help='' , subtree=[
			T_DataGroupTemplate(help='Rename template', pipe=None, subtree=[
				T_ValidWord(VALID_DATAGROUP_NAME, help='', pipe=None, subtree=[
					T_KeywordWithHook('fix-logs'   , help='' , callback=cmd_rename_datagroup_template_in_library ),
					T_KeywordWithHook('no-fix-logs', help='' , callback=cmd_rename_datagroup_template_in_library ),
				]),
			]),
		]),
	]),
	
	T_Keyword('no', help='Remove an object from library', subtree=[
		T_Keyword('datagroup-template', help='Remove a datagroup template', subtree=[
			T_DataGroupTemplate(help='Remove a datagroup template', pipe=None, subtree=[
				T_DataGroupTemplate(help='Remove a datagroup template', callback=cmd_datagroup_template_config_prompt, subtree=[
					T_Keyword('force', help='Remove a datagroup template even if used', callback=cmd_datagroup_template_config_prompt)
				]),
			]),
		]),	
	]),
]


def cmd_config_library_prompt(cli):

	env = cli.env.clone()

	env['CURRENT_TENANT'] = None
	env['CONFIG_TENANT'] = None

	env['SOURCE'] = SOURCE_LIBRARY

	env['PROMPT'] = '' #config-library# '
	env['MODE'  ] = 'configuration'

	cmds = SORTED_KEYWORDS(config_library_prompt) + [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + show_commands_library )

	c = cli.get_class()( cmds, env, debug_interpreter=env['DEBUG'] )
	c.cmdloop()


#
# FIX: opent ticket #398
#
#def cmd_tenant_rename(cli):
#	
#	# rename tenant <name> <name>
#	env = cli.env.clone()
#	
#	name  = cli.canonical_words[2]
#	name2 = cli.canonical_words[3]
#	
#	if name == name2:
#		return
#
#	tenant_rename(name, name2)
#
#	env['CURRENT_TENANT'] = name2

config_prompt = [

	T_Keyword('user', help='Configure a user', subtree=[
		T_Usercompleter(help='Username', callback=cmd_user_config_prompt, pipe=None),
		T_ValidWord(VALID_USER_NAME, help='New username', callback=cmd_user_config_prompt, pipe=None),
	]),

	T_Keyword('group', help='Define a group of users', subtree=[
		T_Groupcompleter(help='Group name', callback=cmd_group_config_prompt, pipe=None),
		#T_Anyword(help='Create a new group', callback=cmd_create_group, pipe=None ),
		T_ValidWord(VALID_GROUP_NAME, help='Create a new group', callback=cmd_group_config_prompt, pipe=None ),
	]),

	T_Keyword('tenant', help='Configure tenant', subtree=[
		T_Tenantcompleter(help="Configure existing tenant name", callback=cmd_tenant_config_prompt, pipe=None),
		T_ValidWord(VALID_TENANT_NAME, help='Create and configure new tenant (tenant name must be in "lowercase")', callback=cmd_tenant_create_and_config_prompt, pipe=None),
	]),
	
	T_Keyword('rename', help='Rename an object', subtree=[
		T_Keyword('user', help='Rename an user', subtree=[
			T_Usercompleter(help='Username', subtree=[
				T_ValidWord(VALID_USER_NAME, help='Rename a user', callback=cmd_user_config_rename),
			])
		]),

		T_Keyword('group', help='Rename a group', subtree=[
			T_Groupcompleter(help='Rename a group', subtree=[
				T_ValidWord(VALID_GROUP_NAME, help='Rename a group', callback=cmd_group_config_rename),
			]),
		]),

		#T_Keyword('tenant', help='Rename a tenant', subtree=[
		#	T_Tenantcompleter(help="", subtree=[
		#		T_ValidWord(VALID_GROUP_NAME, help='Rename a group', callback=cmd_tenant_rename)
		#	]),
		#]),
	]),
	

	T_Keyword('library', help='Configure library' , callback=cmd_config_library_prompt, pipe=None, ),


	T_KeywordWithHook('no', help='Remove existing elements from the configuration', 
	
		hook_name='configuration:no',
	
		subtree=[
		T_Keyword('user', help='Remove existing user', subtree=[
			T_Usercompleter(help='Remove existing user', pipe=None, subtree=[
				T_Usercompleter(help='Remove existing user', callback=cmd_delete_user, pipe=None
				),
			]),
		]),
		T_Keyword('group', help='Remove existing Gruop', subtree=[
			T_Groupcompleter(help='Remove existing Gruop', pipe=None, subtree=[
				T_Groupcompleter(help='Remove existing Gruop', callback=cmd_delete_group, pipe=None
				),
			]),
		]),
		
		T_Keyword('tenant', help='Remove a tenant', subtree=[
			T_Tenantcompleter(help='Remove a tenant', pipe=None, subtree=[
				T_Tenantcompleter(help='Remove a tenant', callback=cmd_tenant_remove, pipe=None,)
			]),
		]),
		
		
		T_Keyword('datagroup-template', help='Remove a datagroup template', subtree=[
			T_DataGroupTemplate(help='Remove a datagroup template', pipe=None, subtree=[
			T_DataGroupTemplate(help='Remove a datagroup template', callback=cmd_datagroup_template_config_prompt, pipe=None, subtree=[
				T_Keyword('force', help='Remove a datagroup template even if used', callback=cmd_datagroup_template_config_prompt)
			]),
			]),
		]),	

	]),
]

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def LOCKED_HANDLER( handler ):

	def _locked_handler(cli):

		from sanet_common.osutils.lockfile import FileLocker
		
		try:
			os.makedirs(configuration.PID_DIR)
		except OSError, e:
			if e.errno == 17:
				pass
			else:
				raise e
			
		
		filename = os.path.join(configuration.PID_DIR, 'cli.pid')
		
		flock = FileLocker(filename)
		
		if not flock.lock():

			msg = 'configuration is locked by another process (lock on file %s)' % (filename)
			
			if cli.env['EXIT_ON_ERROR']:
				raise FatalCliException(msg)
			else:
				raise ApplicationException(msg)

		try:
			handler(cli)
		finally:
			flock.unlock()
		
	return _locked_handler

def cmd_config_prompt(cli):
	
	env = cli.env.clone()
	
	env['CONFIG_TENANT'] = None

	#env['PROMPT'] = 'config#'
	env['PROMPT'] = ''
	env['MODE'  ] = 'configuration'
	
	cmds = SORTED_KEYWORDS(config_prompt) \
		+ [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') \
		+ cli_get_registered_menus('__show_commands__')) \
		+  SORTED_KEYWORDS(cli_get_registered_menus('configuration')) 

	c = cli.get_class()( cmds, env, debug_interpreter=env['DEBUG'] )
	c.cmdloop()

config_prompt_command = [

	T_Keyword('configuration', help='Enter configuration mode' , callback=LOCKED_HANDLER(cmd_config_prompt), pipe=None , 
		subtree=[
			T_Keyword('tenant', help='Configure default tenant' , callback=LOCKED_HANDLER(cmd_tenant_config_default_prompt), pipe=None, subtree=[
				T_Tenantcompleter(help="Configure existing tenant", callback=cmd_tenant_config_prompt_direct, pipe=None),	
			]),
			T_Keyword('library', help='Configure system library' , callback=LOCKED_HANDLER(cmd_config_library_prompt), pipe=None, ),
		]
	)

]
