# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from core.db_utils import transaction_commit_on_success

from core.api.base import *
from core.api.elements import *

from cli.clilib.ioslike import *
from cli.clilib.ioslike import SORTED_KEYWORDS

from cli.tokens import T_AdditionalDataGroup
from cli.tokens import T_CDataGroupParameter

from cli.prompts.defs import *

from cli.prompts import cli_get_registered_menus

from cli.prompts.config.templates.datagroups import CREATE_DATASOURCE_FIELD_COMMANDS
from cli.prompts.config.templates.datagroups import CREATE_CONDITION_FIELD_COMMANDS
from cli.prompts.config.templates.datagroups import CREATE_DATAGROUP_COMMANSDS
from cli.prompts.config.templates.datagroups import CREATE_TIMEGRAPH_FIELD_COMMANDS
from cli.prompts.config.templates.datagroups import CREATE_TIMEGRAPH_SERIE_FIELD_COMMANDS

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_config_additional_datasource_field(cli):

	env = cli.env

	if cli.env.get('PARSE_ONLY', False):
		return

	user                    = env['CURRENT_USER']

	tenant_name             = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	element                 = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']

	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']
	datasource_name         = env['CONFIG_DATASOURCE']

	# <field> <value>
	# no <field>
	if cli.canonical_words[0] != 'no':
		field_name      = cli.canonical_words[0]
		field_name      = CLI_FIELD_TO_DATASOURCE_FIELD(field_name)

		field_str_value = cli.canonical_words[1]

		element_additional_datasource_field_set(element, datagroup_instance_name, datasource_name, field_name, field_str_value, tenant, user=user)

		return
	else:
		field_name      = cli.canonical_words[1]
		field_name      = CLI_FIELD_TO_DATASOURCE_FIELD(field_name)

		element_additional_datasource_field_unset(element, datagroup_instance_name, datasource_name, field_name, tenant, user=user)

		return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_config_additional_condition_field(cli):

	env = cli.env

	if cli.env.get('PARSE_ONLY', False):
		return

	user                    = env['CURRENT_USER']

	tenant_name             = env['CONFIG_TENANT']

	tenant = tenant_get(tenant_name)

	element                 = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']

	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']
	condition_name          = env['CONFIG_CONDITION']

	# <field> <value>
	# no <field>
	if cli.canonical_words[0] != 'no':

		field_name      = cli.canonical_words[0]
		field_name      = CLI_FIELD_TO_CONDITION_FIELD(field_name)

		field_str_value = cli.canonical_words[1]
		field_value     = CLI_FIELD_VALUE_TO_CONDITION_FIELD_VALUE(field_name, field_str_value)

		element_additional_condition_field_set(element, datagroup_instance_name, condition_name, field_name, field_value, tenant, user=user)
	else:
		field_name      = cli.canonical_words[1]
		field_name      = CLI_FIELD_TO_CONDITION_FIELD(field_name)

		element_additional_condition_field_unset(element, datagroup_instance_name, condition_name, field_name, tenant, user=user)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_config_additional_timegraph_field(cli):

	env = cli.env

	if cli.env.get('PARSE_ONLY', False):
		return

	user                    = env['CURRENT_USER']

	tenant_name             = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)

	element                 = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']

	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']
	timegraph_name          = env['CONFIG_TIMEGRAPH']

	if cli.canonical_words[0] != 'no':

		field_name      = cli.canonical_words[0]
		field_name      = CLI_FIELD_TO_TIMEGRAPH_FIELD(field_name)

		field_str_value = cli.canonical_words[1]

		element_additional_timegraph_field_set(element, datagroup_instance_name, timegraph_name, field_name, field_str_value, tenant, user=user)
	else:
		field_name      = cli.canonical_words[1]
		field_name      = CLI_FIELD_TO_TIMEGRAPH_FIELD(field_name)

		element_additional_timegraph_field_unset(element, datagroup_instance_name, timegraph_name, field_name, tenant, user=user)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_config_additional_timegraph_serie_field(cli):

	env = cli.env

	if cli.env.get('PARSE_ONLY', False):
		return
		
	user                    = env['CURRENT_USER']

	tenant_name             = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)

	element                 = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']

	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']
	timegraph_name          = env['CONFIG_TIMEGRAPH']
	serie_name              = env['CONFIG_TIMEGRAPH_SERIE']

	if cli.canonical_words[0] != 'no':

		field_name      = cli.canonical_words[0]
		field_name      = CLI_FIELD_TO_TIMEGRAPHSERIE_FIELD(field_name)

		field_str_value = cli.canonical_words[1]

		element_additional_timegraph_serie_field_set(element, datagroup_instance_name, timegraph_name, serie_name, field_name, field_str_value, tenant, user=user)
	else:
		field_name      = cli.canonical_words[1]
		field_name      = CLI_FIELD_TO_TIMEGRAPHSERIE_FIELD(field_name)

		element_additional_timegraph_serie_field_unset(element, datagroup_instance_name, timegraph_name, serie_name, field_name, tenant, user=user)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

mt_config_additional_datasource            = CREATE_DATASOURCE_FIELD_COMMANDS(cmd_config_additional_datasource_field, include_deleted=True)

mt_config_additional_condition             = CREATE_CONDITION_FIELD_COMMANDS(cmd_config_additional_condition_field, include_deleted=True)

mt_config_additional_timegraph_serie_field = CREATE_TIMEGRAPH_SERIE_FIELD_COMMANDS(cmd_config_additional_timegraph_serie_field, include_deleted=True)



def mt_config_additional_datasource_prompt(cli):

	env = cli.env

	if not cli.env.get('PARSE_ONLY', False):

		# datasource <name>

		tenant_name             = env['CONFIG_TENANT']
		tenant = tenant_get(tenant_name)
		db_set_current_tenant_schema(tenant.name)

		element = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']	
		element_name = element.name
		datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
		datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']

		datasource_name = cli.canonical_words[1]


		# Verify elements
		from core.models import TDataGroup

		tdg = TDataGroup.objects.get(name=datagroup_template_name, source=SOURCE_LOCAL, tenant=tenant)

		cdg = element_additional_datagroup_get(element, datagroup_instance_name, tenant)	
		tds = cdg.find_tdatasource(datasource_name)

		env = cli.env.clone()
		#env['PROMPT'] = 'config-%s#' % (PROMPT_SEP.join([tenant_name, element_name, datagroup_instance_name, datasource_name]),)
		env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, element_name, datagroup_instance_name, datasource_name]),)
		env['CONFIG_DATASOURCE'] = datasource_name

	cmds = mt_config_additional_datasource + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))

	#cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	#cli.cmdloop()
	cli.enter_subcli(cmds, env)

	return

def mt_config_additional_condition_prompt(cli):
	# datasource <name>
	
	env = cli.env

	if not cli.env.get('PARSE_ONLY', False):

		env = cli.env
		tenant_name             = env['CONFIG_TENANT']
		element                 = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']
		element_name = element.name

		datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
		datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']

		condition_name = cli.canonical_words[1]

		tenant = tenant_get(tenant_name)
		db_set_current_tenant_schema(tenant.name)

		# Verify elements
		from core.models import TDataGroup

		cdg = element_additional_datagroup_get(element, datagroup_instance_name, tenant)

		tc = cdg.find_tcondition(condition_name)

		env = cli.env.clone()
		#env['PROMPT'] = 'config-%s#' % (PROMPT_SEP.join([tenant_name, element_name, datagroup_instance_name, condition_name]))
		
		env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, element_name, datagroup_instance_name, condition_name]))
		
		env['CONFIG_CONDITION'] = condition_name

	cmds = mt_config_additional_condition + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))

	#cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	#cli.cmdloop()
	cli.enter_subcli(cmds, env)

	return

def cmd_config_additional_timegraph_serie_prompt(cli):

	env = cli.env

	if not cli.env.get('PARSE_ONLY', False):

		tenant_name             = env['CONFIG_TENANT']
		element                 = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']
		element_name            = element.name
		datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
		datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']
		timegraph_name          = env['CONFIG_TIMEGRAPH']

		serie_name      = cli.canonical_words[1]

		tenant = tenant_get(tenant_name)

		db_set_current_tenant_schema(tenant.name)

		# Verify elements
		from core.models import TDataGroup
		from core.models import TTimeGraph
		from core.models import TTimeGraphSerie

		tdg  = TDataGroup.objects.get(name=datagroup_template_name, tenant=tenant)

		#ttg  = TTimeGraph.objects.get(tdatagroup=tdg, name=timegraph_name, tenant=tenant)
		ttg = tdg.get_last_defined_timegraph(timegraph_name)
		
		#ttgs = TTimeGraphSerie.objects.get(tenant=tenant, ttimegraph=ttg, name=serie_name)
		ttgs = tdg.get_last_defined_timegraphserie(timegraph_name, serie_name)

		cdg = element_additional_datagroup_get(element, datagroup_instance_name, tenant)

		env = cli.env.clone()

		env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, element_name, datagroup_instance_name, timegraph_name, serie_name]))
		env['CONFIG_TIMEGRAPH_SERIE'] = serie_name


	cmds = mt_config_additional_timegraph_serie_field + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))

	#cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	#cli.cmdloop()

	cli.enter_subcli(cmds, env)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#



def mt_config_additional_timegraph_prompt(cli):
	# datasource <name>

	env = cli.env

	if not cli.env.get('PARSE_ONLY', False):

		tenant_name             = env['CONFIG_TENANT']
		element                 = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']
		element_name            = element.name

		datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
		datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']

		timegraph_name = cli.canonical_words[1]

		tenant = tenant_get(tenant_name)

		db_set_current_tenant_schema(tenant.name)

		# Verify elements
		from core.models import TDataGroup
		from core.models import TTimeGraph

		tdg = TDataGroup.objects.get(name=datagroup_template_name, tenant=tenant)
		
		#ttg = TTimeGraph.objects.get(tdatagroup=tdg, name=timegraph_name, tenant=tenant)
		ttg = tdg.get_last_defined_timegraph(timegraph_name)

		cdg = element_additional_datagroup_get(element, datagroup_instance_name, tenant)

		env = cli.env.clone()
		#env['PROMPT'] = 'config-%s#' % (PROMPT_SEP.join([tenant_name, element_name, datagroup_instance_name, timegraph_name]))
		env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, element_name, datagroup_instance_name, timegraph_name]))
		env['CONFIG_TIMEGRAPH'] = timegraph_name

	cmds = mt_config_additional_timegraph + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))

	#cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	#cli.cmdloop()
	cli.enter_subcli(cmds, env)

	return

mt_config_additional_timegraph = CREATE_TIMEGRAPH_FIELD_COMMANDS(
	cmd_config_additional_timegraph_field,
	cmd_config_additional_timegraph_serie_prompt,
	include_deleted=True,
)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_config_additional_datagroup_field(cli):

	if cli.env.get('PARSE_ONLY', False):
		return

	env = cli.env

	tenant_name             = env['CONFIG_TENANT']
	element = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']

	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']

	# <field> <value>
	# no <field>
	if cli.canonical_words[0] != 'no':

		field_name      = cli.canonical_words[0]
		field_str_value = cli.canonical_words[1]
		
		field_name      = CLI_FIELD_TO_DATAGROUP_FIELD(field_name)

		tenant = tenant_get(tenant_name)

		element_additional_datagroup_field_set(element, datagroup_template_name, datagroup_instance_name, field_name, field_str_value, tenant=tenant)
		
	else:
		field_name      = cli.canonical_words[1]
		field_name      = CLI_FIELD_TO_DATAGROUP_FIELD(field_name)

		tenant = tenant_get(tenant_name)

		element_additional_datagroup_field_unset(element, datagroup_template_name, datagroup_instance_name, field_name, tenant=tenant)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def cmd_config_additional_datagroup_parameter(cli):

	if cli.env.get('PARSE_ONLY', False):
		return

	env = cli.env

	user                    = env['CURRENT_USER']
	
	tenant_name             = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	element = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']

	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']


	# parameter <name> ....
	# no parameter <name> <name>
	if cli.canonical_words[0] != 'no':

		# parameter <name> <type> <value>
		param_name      = cli.canonical_words[1]
		param_type_name = cli.canonical_words[2]
		param_value     = cli.canonical_words[3]

		# Validate param value. This will raise an exception
		ptype, pvalue = convert_param_value(param_type_name, param_value)

		element_additional_datagroup_parameter_set(element, datagroup_template_name, datagroup_instance_name, param_name, ptype, param_value, tenant=tenant, user=user)
	else:

		param_name      = cli.canonical_words[2]

		element_additional_datagroup_parameter_unset(element, datagroup_template_name, datagroup_instance_name, param_name, tenant=tenant, user=user)

	return



def cmd_config_additional_datagroup_conditions_priority(cli):

	env = cli.env

	user                    = env['CURRENT_USER']

	tenant_name             = env['CONFIG_TENANT']
	element = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']

	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']
	
	priority = int(cli.canonical_words[1])	

	element_additional_datagroup_set_conditions_priority(element, datagroup_template_name, datagroup_instance_name, priority, user=user)


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

mt_config_additional_datagroup = CREATE_DATAGROUP_COMMANSDS(
	cmd_config_additional_datagroup_field,
	cmd_config_additional_datagroup_parameter,
	mt_config_additional_datasource_prompt,
	mt_config_additional_condition_prompt,
	mt_config_additional_timegraph_prompt, 
	include_no_commands=False,
	include_deleted=False,
	
	include_scheduling_commands=True,
	
	tok_no_param_completer = T_CDataGroupParameter,
) + [

		T_Keyword('priority-conditions'     , help='Set priority of all conditions', subtree=[
			T_Number(help='Set priority of all conditions', callback=cmd_config_additional_datagroup_conditions_priority)
		]),
]


def cmd_additional_datagroup_config_prompt(cli):

	env = cli.env

	if not cli.env.get('PARSE_ONLY', False):

		user         = env['CURRENT_USER']

		tenant_name  = env['CONFIG_TENANT']
		tenant = tenant_get(tenant_name)

		element      = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']
		element_name = element.name

		# datagroup <name> [ template <name> ]
		if len(cli.canonical_words) == 4:
			datagroup_template_name = cli.canonical_words[3]
			datagroup_instance_name = cli.canonical_words[1]
		else:
			datagroup_template_name = cli.canonical_words[1]
			datagroup_instance_name = datagroup_template_name


		tenant.db_set_current_schema()

		cdg = element_additional_datagroup_get(element, datagroup_instance_name, tenant)

		if cdg == None:
			
			# Get or create from the library
			from cli.prompts.config.templates.datagroups import get_or_duplicate_tdg_from_library
			tdg = get_or_duplicate_tdg_from_library(cli, tenant, datagroup_template_name, create_if_missing=False)
			
			if tdg == None:
				raise Exception("Datagroup template %s does not exist" % (datagroup_template_name))
		else:	
			tdg = cdg.tdatagroup

		if cdg == None:
			cdg = element_datagroup_create_additional(element, tdg, datagroup_instance_name, tenant=tenant, user=user)

		env = cli.env.clone()
		
		#env['PROMPT'] = 'config-%s#' % (PROMPT_SEP.join([tenant_name, element_name, datagroup_instance_name]),)
		env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, element_name, datagroup_instance_name]),)
		
		env['CONFIG_DATAGROUP_TEMPLATE'] = cdg.get_tdatagroup().name
		env['CONFIG_DATAGROUP_INSTANCE'] = cdg.name

	cmds = SORTED_KEYWORDS(mt_config_additional_datagroup) + [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))
	
	#cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	#cli.cmdloop()
	cli.enter_subcli(cmds, env)

	#
	# FIX: cos'altro fare qui? qualche refresh?
	#



#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#


def cmd_additional_datagroup_remove(cli):

	# no datagroup 1 1
	env = cli.env
	tenant_name          = env['CONFIG_TENANT']

	#(kind, element_name) = env['CONFIG_ELEMENT']
	element = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']

	dg_name  = cli.canonical_words[2]
	dg_name2 = cli.canonical_words[3]
	if dg_name != dg_name2:
		raise Exception("Different names")

	element_datagroup_remove_additional(element, dg_name)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#


mt_config_additional_datagroup_no = T_Keyword('datagroup', help='', subtree=[
		T_AdditionalDataGroup(help='', subtree=[
			T_AdditionalDataGroup(help='', callback=cmd_additional_datagroup_remove, )
		])
])

