# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from core.api.base import agent_get
from core.api.base import agent_create
from core.api.base import agent_set_enabled
from core.api.base import agent_set_remote
from core.api.base import agent_set_primary
from core.api.base import agent_remove
from core.api.base import agent_update
from core.api.base import agent_rename
from core.api.base import tenant_get

from cli.clilib.ioslike import T_Keyword, SORTED_KEYWORDS, T_Line, T_Number

from cli.prompts import cli_get_registered_menus
from cli.prompts.defs import PROMPT_SEP


#------------------------------------------------------------------------------#
# AGENT CONFIGURATION                                                          #
#------------------------------------------------------------------------------#

def cmd_agent_set_attribute(cli):
	
	# enabled
	# primary
	# remote
	# no enabled
	# no primary
	# no remote
	
	warning_msg = None

	tenant_name  = cli.env['CONFIG_TENANT']
	agent_name   = cli.env['CONFIG_AGENT']

	agent = agent_get(agent_name, tenant_name=tenant_name)
	
	
	RESTART_AGENT_REQUIRED = """This option requires agent restart:\n\n\trestart agent %s\n""" % (agent_name)
	
	
	if cli.canonical_words[0] != 'no':

		param = cli.canonical_words[0]

		if param == 'description':
			val = cli.canonical_words[1]
			
			# FIX: update only description + mtime
			agent.description = val
			
			agent_update(agent, 'description')
		elif param == 'enabled':
			agent_set_enabled(agent, True)
		elif param == 'threads':
			val = cli.canonical_words[1]
			val = int(val)
			agent.threads = val
			agent_update(agent, 'threads')

			warning_msg = RESTART_AGENT_REQUIRED

		elif param == 'disabled':
			agent_set_enabled(agent, False)
		elif param == 'local':
			agent_set_remote(agent, False)
		elif param == 'remote':
			agent_set_remote(agent, True)
		elif param == 'primary':
			agent_set_primary(agent)
		elif param == 'update-minperiod':
			val = int(cli.canonical_words[1])
			agent.config_update_minperiod = val
			agent_update(agent, 'config_update_minperiod')

			warning_msg = RESTART_AGENT_REQUIRED

		elif param in ['scheduling-min-gap', 'scheduling-max-parallel','scheduling-max-deadline'] :
			field_name = param.lower().replace('-','_')
			val = int(cli.canonical_words[1])
			#agent.config_update_minperiod = val
			#agent_update(agent, 'config_update_minperiod')
			
			old_val = getattr(agent, field_name)
			if old_val != val:
				
				setattr(agent, field_name, val)
				agent_update(agent, field_name)

				warning_msg = RESTART_AGENT_REQUIRED

		else:
			raise Exception("Invalid parameter '%s'" % (param))

	else:
		param = cli.canonical_words[1]
		
		if param in ['scheduling-min-gap', 'scheduling-max-parallel','scheduling-max-deadline'] :
			field_name = param.lower().replace('-','_')
			val = None
			old_val = getattr(agent, field_name)
			if old_val != val:
				
				setattr(agent, field_name, val)
				agent_update(agent, field_name)

				warning_msg = RESTART_AGENT_REQUIRED

		else:
			raise Exception("Invalid parameter '%s'" % (param))

	if warning_msg:
		cli.warning(warning_msg)

mt_config_agent = [

	T_Keyword('description', help='', subtree=[
		T_Line(help='', callback=cmd_agent_set_attribute,),
	]),
	T_Keyword('enabled' , help='', callback=cmd_agent_set_attribute,),
	T_Keyword('disabled', help='', callback=cmd_agent_set_attribute,),
	
	T_Keyword('primary' , help='', callback=cmd_agent_set_attribute,),
	
	T_Keyword('remote'  , help='',  callback=cmd_agent_set_attribute,),
	T_Keyword('local'   , help='',  callback=cmd_agent_set_attribute,),
	
	T_Keyword('threads' , help='',  subtree=[
		T_Number(help='', callback=cmd_agent_set_attribute),
	]),
	
	T_Keyword('update-minperiod' , help='',  subtree=[
		T_Number(help='', callback=cmd_agent_set_attribute),
	]),

	T_Keyword('scheduling-min-gap'      , help='',  subtree=[ T_Number(help='', callback=cmd_agent_set_attribute), ]),
	T_Keyword('scheduling-max-parallel' , help='',  subtree=[ T_Number(help='', callback=cmd_agent_set_attribute), ]),
	T_Keyword('scheduling-max-deadline' , help='',  subtree=[ T_Number(help='', callback=cmd_agent_set_attribute), ]),

	T_Keyword('no' , help='',  subtree=[

		T_Keyword('scheduling-min-gap'      , help='', callback=cmd_agent_set_attribute),
		T_Keyword('scheduling-max-parallel' , help='', callback=cmd_agent_set_attribute),
		T_Keyword('scheduling-max-deadline' , help='', callback=cmd_agent_set_attribute),
	]),
]

def cmd_agent_config_prompt(cli):

	user         = cli.env['CURRENT_USER']
	
	tenant_name  = cli.env['CONFIG_TENANT']

	name = cli.canonical_words[1]

	a = agent_get(name, tenant_name=tenant_name)
	if a == None:
		a = agent_create(name, primary_if_first=True, tenant_name=tenant_name)
		cli.info("Created agent %s" % (name))

	a.assert_permission_readwrite(user)

	env = cli.env.clone()
	#env['PROMPT'] = 'config-%s#' % ( PROMPT_SEP.join([ tenant_name, name ]) )
	env['PROMPT'] = '%s' % ( PROMPT_SEP.join([ tenant_name, name ]) )
	env['CONFIG_AGENT'] = name

	cmds = mt_config_agent  + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))

	cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	cli.cmdloop()

def cmd_agent_remove(cli):
	# no agent <name>
	tenant_name  = cli.env['CONFIG_TENANT']

	name = cli.canonical_words[2]

	agent_remove(name, tenant_name=tenant_name)

def cmd_agent_rename(cli):
	# rename agent <name> <new name>
	tenant_name  = cli.env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)

	name     = cli.canonical_words[2]
	new_name = cli.canonical_words[3]
	
	agent_rename(tenant, name, new_name)
