# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.


from core.models import Tenant
from core.models import Cluster
from core.models import SchedulingGroup

from core.models import User, Group

from core.api.base import cluster_get, cluster_remove, cluster_save, cluster_rename, cluster_create

from cli.clilib.ioslike import T_Keyword, T_Alternatives, SORTED_KEYWORDS, T_Line, T_Number

from cli.tokens import AUTH_READONLY, AUTH_READWRITE
from cli.tokens import T_Usercompleter
from cli.tokens import T_Groupcompleter

from cli.prompts import cli_get_registered_menus
from cli.prompts.defs import PROMPT_SEP
from cli.prompts.defs import CLI_PERM_OPTIONS


#------------------------------------------------------------------------------#
# CLUSTER                                                                      #
#------------------------------------------------------------------------------#

def cmd_cluster_set_attribute(cli):

	user         = cli.env['CURRENT_USER']

	if cli.env.get('PARSE_ONLY', False):
		return

	tenant_name  = cli.env['CONFIG_TENANT']
	cluster_name = cli.env['CONFIG_CLUSTER']
	
	tenant = Tenant.objects.get(name=tenant_name)
	
	cluster = cluster_get(cluster_name, tenant_name=tenant_name)

	if cli.canonical_words[0] != 'no':

		param = cli.canonical_words[0]
		value = cli.canonical_words[1]


		if param == 'description':

			# FIX: <<<<<<<<<<<<< MOVE TO LIBRARY
			
			if user:
				cluster.assert_permission_readwrite(user)

			cluster.description = value
			cluster_save(cluster, 'description')
			
			# >>>>>>>>>>>>>>

		elif param in [ 'scheduling-min-gap' , 'scheduling-max-deadline', 'scheduling-max-parallel' ]:

			if user:
				cluster.assert_permission_readwrite(user)
			
			#name = 'cluster:%s' % (cluster.name)
			name   = cluster.implicit_scheduling_group_name
			
			sg, created = SchedulingGroup.objects.get_or_create(tenant=tenant, name=name)

			sg.is_real=False
			sg.save()
			
			if created:
				sg.min_gap      = 3
				sg.max_parallel = 1
				sg.save()

			cluster.scheduling_group = sg
			cluster.save()


			field_name = param.replace('-', '_')[11:]

			new_val = int(value)
			old_val = getattr(sg, field_name)
			
			if old_val != new_val:
				
				updated_fields = []
				updated_fields.append(field_name)
				setattr(sg, field_name, int(value))
				
				#if field_name == 'min_gap':
				#	if sg.scheduling_max_parallel is None:
				#		sg.scheduling_max_parallel = 1
				#		updated_fields.append("max_parallel")
				#		cli.warning("Setting default value for scheduling-max-parallel = %s" % (cluster.scheduling_max_parallel))
				#	print cluster.scheduling_max_parallel
				#	
				#cluster_save(cluster, *updated_fields)
				sg.save()
				
			if created:
				cli.info("Scheduling settings set for the first time (min-gap %s, max-paralleld %s" % (sg.min_gap, sg.max_parallel))
				
		else:
			raise Exception("Invalid param %s" % (param))

	else:

		param = cli.canonical_words[1]

		if param in [ 'scheduling-min-gap' , 'scheduling-max-deadline', 'scheduling-max-parallel' ]:

			if user:
				cluster.assert_permission_readwrite(user)

			sg = cluster.scheduling_group
			
			if sg is None:
				return
				
			field_name = param.replace('-', '_')[11:]
			setattr(sg, field_name, None)
			sg.save()
			
			if sg.min_gap is None and sg.max_parallel is None:
				cli.info("Removing scheduling settings...")
				sg.delete()
		else:
			raise Exception("Invalid param %s" % (param))


def cmd_cluster_rename(cli):

	# rename cluster <name> <name>
	
	user         = cli.env['CURRENT_USER']
	tenant_name  = cli.env['CONFIG_TENANT']
	
	tenant = Tenant.objects.get(name=tenant_name)

	name     = cli.canonical_words[2]
	new_name = cli.canonical_words[3]

	cluster_rename(tenant, name, new_name, action_user=user)

def cmd_cluster_set_owner(cli):

	#print "TO IMPLEMENT", cli.canonical_words

	user        = cli.env['CURRENT_USER']
	#tenant_name = cli.env['CONFIG_TENANT']
	#name        = cli.env['CONFIG_SCHEDULINGGROUP']
	
	#tenant  = tenant_get(tenant_name)

	#sg = SchedulingGroup.objects.get(tenant=tenant, name=name)
	
	tenant_name  = cli.env['CONFIG_TENANT']
	cluster_name = cli.env['CONFIG_CLUSTER']

	cluster = cluster_get(cluster_name, tenant_name=tenant_name)
	

	if cli.canonical_words[0] != 'no':

		username = cli.canonical_words[1]

		owner = User.objects.get(username=username)

		cluster.set_owner(owner, action_user=user)
	else:
		cluster.unset_owner(action_user=user)


CLI_PERM_TO_AUTH_PERM = {
	'read-write': AUTH_READWRITE,
	'read-only' : AUTH_READONLY, 
}

def cmd_config_cluster_permission(cli):

	user = cli.env['CURRENT_USER']

	tenant_name = cli.env['CURRENT_TENANT']

	name        = cli.env['CONFIG_CLUSTER']

	#tenant = tenant_get(tenant_name)
	tenant = Tenant.objects.get(name=tenant_name)
	
	c = Cluster.objects.get(tenant=tenant, name=name)

	if cli.canonical_words[0] != 'no':

		group_name = cli.canonical_words[1]

		if len(cli.canonical_words) > 2:
			permission = CLI_PERM_TO_AUTH_PERM[ cli.canonical_words[2] ]
		else:
			permission = AUTH_READONLY

		g = Group.objects.get(name=group_name)

		c.add_group_permission(g, permission, user)
	else:

		group_name = cli.canonical_words[2]

		g = Group.objects.get(name=group_name)

		c.remove_group_permission(g, user)



mt_config_cluster = [

	T_Keyword('description', help='', subtree=[
		T_Line(help='', callback=cmd_cluster_set_attribute,),
	]),

	T_Keyword('owner', help='', subtree=[
		T_Usercompleter(help='', callback=cmd_cluster_set_owner),
	]),

	T_Keyword('group', help='', subtree=[
		T_Groupcompleter(help='', callback=cmd_config_cluster_permission, subtree=[ 
			T_Alternatives( CLI_PERM_OPTIONS, callback=cmd_config_cluster_permission )
		]),
	]),


	T_Keyword('scheduling-min-gap'     , help='Set min-gap between datagroups.'       , subtree=[ T_Number(help='Set min-gap between datagroups.'       , callback=cmd_cluster_set_attribute,), ]),
	T_Keyword('scheduling-max-deadline', help='Set datagroup execution deadline.'     , subtree=[ T_Number(help='Set datagroup execution deadline.'     , callback=cmd_cluster_set_attribute,), ]),
	T_Keyword('scheduling-max-parallel', help='Set max number of parallel datagroups.', subtree=[ T_Number(help='Set max number of parallel datagroups.', callback=cmd_cluster_set_attribute,), ]),

	T_Keyword('no', help='Unset cluster parameters.', subtree=[ 

		T_Keyword('owner', help='', callback=cmd_cluster_set_owner),

		T_Keyword('description'            , help=''   , callback=cmd_cluster_set_attribute),

		T_Keyword('scheduling-min-gap'     , help='Unset min-gap.'                          , callback=cmd_cluster_set_attribute),
		T_Keyword('scheduling-max-deadline', help='Unset deadline.'                         , callback=cmd_cluster_set_attribute),
		T_Keyword('scheduling-max-parallel', help='Unset max number of parallel datagroups.', callback=cmd_cluster_set_attribute),

	]),
]

def cmd_cluster_config_prompt(cli):

	user = cli.env['CURRENT_USER']

	if not cli.env.get('PARSE_ONLY', False):

		tenant_name  = cli.env['CONFIG_TENANT']

		name = cli.canonical_words[1]

		c = cluster_get(name, tenant_name=tenant_name)
		if c == None:
			c = cluster_create(name, tenant_name=tenant_name, owner=user)
			cli.info("Created cluster %s" % (name))

		if user:
			c.assert_permission_readwrite(user)

		env = cli.env.clone()

		env['PROMPT'] = '%s' % ( PROMPT_SEP.join([ tenant_name, name ]) )
		env['CONFIG_CLUSTER'] = c.name
	else:
		env = cli.env.clone()
	
	cmds = mt_config_cluster  + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))

	#cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	#cli.cmdloop()
	cli.enter_subcli(cmds, env)


def cmd_cluster_remove(cli):

	# no agent <name>
	tenant_name  = cli.env['CONFIG_TENANT']

	name = cli.canonical_words[2]

	cluster_remove(name, tenant_name=tenant_name)
