# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from sanet_common.sanetdefs.constants import ELEMENT_KIND_DEVICE

from sanet_poller.lib.evaluation import compile_str_expr

from core.db_utils import transaction_commit_on_success

from core.api.base import *

from cli.clilib.ioslike import *

from cli.clilib.ioslike import SORTED_KEYWORDS

from cli.tokens import T_Iconcompleter

from cli.prompts.defs import *

from cli.prompts import cli_get_registered_menus

from cli.prompts.config.element_config import mt_config_element_template_and_datagroups
from cli.prompts.config.element_config import mt_config_element_template_and_datagroups_no

from cli.prompts.config.element_config import MT_CONFIG_ELEMENT_SCHEDULING_COMMANDS
from cli.prompts.config.element_config import MT_CONFIG_ELEMENT_SCHEDULING_COMMANDS_NO

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def cmd_device_set_attribute(cli):

	if cli.env.get('PARSE_ONLY', False):
		return



	env          = cli.env

	user         = env['CURRENT_USER']

	tenant_name  = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)

	device = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']

	mtime = time.time()

	field_name = cli.canonical_words[0]
	value      = cli.canonical_words[1]

	field_name = field_name.replace('-','_')  # BAD IDEA BUT IT WORKS

	if field_name == 'xform':
		value = xform_alias_to_xform('devices', value)
		try:
			compile_str_expr(None, value)
		except Exception, e:
			cli.error("Error while parsing expression: %s" % (e))
			return

	try:
		value = device.convert_field_str_value(field_name, value)
	except Exception, e:
		raise e

	#
	# FIX: move to library
	#

	if user:
		user.assert_capability(tenant, 'core.element.conf')

	setattr(device, field_name, value)

	# FIX: update only the selected field
	device_update(device)

	if field_name == 'priority':
		device.refresh_elements_priority(mtime=mtime)		

@transaction_commit_on_success
def cmd_device_unset_attribute(cli):

	if cli.env.get('PARSE_ONLY', False):
		return


	env          = cli.env

	user         = env['CURRENT_USER']

	tenant_name  = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)


	device = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']

	mtime = time.time()

	field_name = cli.canonical_words[1]

	field_name = field_name.replace('-','_')  # BAD IDEA BUT IT WORKS

	value = device.get_fields_default_value_dict().get(field_name, None)	

	#
	# FIX: move to library
	#

	if user:
		user.assert_capability(tenant, 'core.element.conf')

	setattr(device, field_name, value)

	# FIX: update only the selected field
	device_update(device, field_name, mtime=mtime)

	if field_name == 'priority':
		device.refresh_elements_priority(mtime=mtime)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

mt_config_device = SORTED_KEYWORDS([
	T_Keyword('description'   , help='', subtree=[ T_Line(help='', callback=cmd_device_set_attribute,), ]),
	T_Keyword('xform'         , help='', subtree=[ T_Line(help='', callback=cmd_device_set_attribute,), ]),
	T_Keyword('distinguisher' , help='', subtree=[ T_Line(help='', callback=cmd_device_set_attribute,  ), ]),
	T_Keyword('devindex'      , help='', subtree=[ T_Line(help='', callback=cmd_device_set_attribute,  ), ]),
	T_Keyword('icon'          , help='', subtree=[ T_Iconcompleter(help='', callback=cmd_device_set_attribute,),]),
	T_Keyword('priority'      , help='', subtree=[ T_PositiveInteger(help='', callback=cmd_device_set_attribute,) ] ),	
]) + MT_CONFIG_ELEMENT_SCHEDULING_COMMANDS

mt_config_device_no = [

	T_Keyword('no', help='', subtree=[
			T_Keyword('distinguisher', help='', callback=cmd_device_unset_attribute,),
			T_Keyword('xform'        , help='', callback=cmd_device_unset_attribute,),
			T_Keyword('priority'     , help='', callback=cmd_device_unset_attribute,),
			T_Keyword('icon'         , help='', callback=cmd_device_unset_attribute,),
		]
		+
		mt_config_element_template_and_datagroups_no(ELEMENT_KIND_DEVICE)
		+ 
		MT_CONFIG_ELEMENT_SCHEDULING_COMMANDS_NO
		
	),
]
#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_device_config_prompt(cli):
	
	#   0        1      2     3   4    5             
	# interface eth0 instance 1 xform "byRegexpUnique('prova','123')"


	env = cli.env

	if not cli.env.get('PARSE_ONLY', False):

		tenant_name = env['CURRENT_TENANT']
		node_name   = env['CURRENT_NODE']
		
		node = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']
		node_name = node.name
		
		
		name = cli.canonical_words[1]
		instance = None
		xform = None
		if len(cli.canonical_words) >= 4:
			instance = cli.canonical_words[3]
		if len(cli.canonical_words) >= 6:
			xform = cli.canonical_words[5]

		device = device_get(node_name, name, tenant_name=tenant_name)

		if device == None:
			cli.info("Creating device %s" % (name))
			device = device_create(node, name)

		updated = False
		
		if instance != None:
			if device.distinguisher != instance:
				device.distinguisher = instance
				updated=True

		if xform != None:
			xform = xform_alias_to_xform('devices', xform)
			
			try:
				compile_str_expr(None, xform)
			except Exception, e:
				cli.error("Invalid xform: %s" % (e))
				return		
						
			if device.xform != xform:
				device.xform = xform
				updated=True
				

		if updated:
			# FIX: update only instance or xform or both, not the entire object
			device_update(device)



		env = cli.env.clone()

		#env['PROMPT'] = 'config-%s#' % (PROMPT_SEP.join([tenant_name, node_name, name]),)
		env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, node_name, name]),)
		env['CONFIG_DEVICE']  = name
		
		env['CURRENT_ELEMENT'] = (ELEMENT_KIND_DEVICE, name)
		env['CONFIG_ELEMENT' ] = (ELEMENT_KIND_DEVICE, name)

		env['CONFIG_ELEMENT_DATA'] = device
		env['CURRENT_ELEMENT_DATA'] = device

	else:
		env = cli.env.clone()


	cmds = SORTED_KEYWORDS(mt_config_device) + \
	       [BLANK_ENTRY] + \
	       mt_config_element_template_and_datagroups(ELEMENT_KIND_DEVICE) + \
	       [BLANK_ENTRY] + \
	       mt_config_device_no + \
	       [BLANK_ENTRY] + \
	       SORTED_KEYWORDS(cli_get_registered_menus('configuration:device')) + \
	       [BLANK_ENTRY] + \
	       SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))
	
	#cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	#cli.cmdloop()
	cli.enter_subcli(cmds, env)


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#
	
	
from cli.prompts.config.ifaces import _cmd_element_from_template_config_prompt

def cmd_device_from_template_config_promt(cli):

	return _cmd_element_from_template_config_prompt(cli, ELEMENT_KIND_DEVICE, mt_config_device, mt_config_device_no, 'configuration:device')	

