# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import time
import os

from conf import get_configuration
DEBUG = get_configuration().DEBUG_CONFIG or os.environ.get('DEBUG_CONFIG',False)

from core.db_utils import transaction_commit_on_success

from core.models import CConditionFieldValue
from core.models import CDataGroup
from core.models import CDataSourceFieldValue
from core.models import CTimeGraphFieldValue
from core.models import CTimeGraphSerieFieldValue
from core.models import TCondition
from core.models import TEDataGroup
from core.models import TTimeGraph
from core.models import TTimeGraphSerie

from core.models import ElementTemplate
from core.api.base import *

from core.api.elements import *
from core.api.elements import element_set_template, element_unset_template
from core.api.templates import ElementTemplate

from cli.clilib.ioslike import *
from cli.tokens import T_Clustercompleter
from cli.tokens import T_DataGroupTemplate
from cli.tokens import T_LibraryDataGroupTemplate
from cli.tokens import T_SchedulingGroupcompleter

from cli.clilib.ioslike import SORTED_KEYWORDS

from cli.tokens import T_ElementTemplateDataGroup
from cli.tokens import T_ParametricElementTemplate

from cli.prompts.defs import *

from cli.prompts import cli_get_registered_menus

from cli.prompts.config.templates.datagroups import CREATE_DATAGROUP_COMMANSDS
from cli.prompts.config.templates.datagroups import CREATE_DATASOURCE_FIELD_COMMANDS
from cli.prompts.config.templates.datagroups import CREATE_CONDITION_FIELD_COMMANDS
from cli.prompts.config.templates.datagroups import CREATE_TIMEGRAPH_FIELD_COMMANDS
from cli.prompts.config.templates.datagroups import CREATE_TIMEGRAPH_SERIE_FIELD_COMMANDS

from cli.prompts.config.additional import cmd_additional_datagroup_config_prompt
from cli.prompts.config.additional import mt_config_additional_datagroup_no

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def datagroup_from_template_in_element_set_field(element, element_template_name, datagroup_template_name, datagroup_instance_name, field_name, field_str_value, tenant, user=None):

	mtime = time.time()

	db_set_current_tenant_schema(tenant.name)

	if user:
		user.assert_capability(tenant, 'core.element.conf_monitoring')

	# Troviamo il template associato all'elemento
	e_t     = ElementTemplate.objects.get(melement=element, telement__name=element_template_name)
	telement = e_t.telement

	# Troviamo il datagroup nel template che corrisponde a quello che vogliamo modificare (deve esistere)
	tedg = TEDataGroup.objects.get(telement=telement, name=datagroup_instance_name)
	tdg  = tedg.tdatagroup
	datagroup_template_name = tdg.name

	cdg = CDataGroup.objects.get(tenant=tenant, element=element, tdatagroup=None, tedatagroup=tedg, name=datagroup_instance_name)

	dg = DataGroup.objects.get(tenant=tenant, element=element, cdatagroup=cdg)

	tf   = TField.objects.get(fclass=TField.FIELD_DATAGROUP, name=field_name)
	
	real_value = tf.validate_value(field_str_value)
	
	try:
		cdgf = CDataGroupFieldValue.objects.get(tenant=tenant, cdatagroup=cdg, tfield=tf)
	except CDataGroupFieldValue.DoesNotExist, e:
		cdgf = CDataGroupFieldValue(tenant=tenant, cdatagroup=cdg, tfield=tf)
		cdgf.save()

	# FIX: validate
	if cdgf.value == field_str_value:
		return

	cdgf.value = field_str_value
	#cdgf.tenant = cdg.tenant
	#cdgf.mtime  = time.time()
	#cdgf.save()
	cdgf.concurrent_save(['value'], new_sentinel_value=mtime)

	# FIX: usare
	#element_datagroup_field_set(element, datagroup_template_name, datagroup_instance_name, field_name, field_str_value, tenant=None):	

	# Update 
	dg.update_single_field_and_update(cdgf, mtime=mtime)


	#
	# Suspend the datagroup if any of the suspended field is set.
	#
	if tf.name in  ['suspended']: #, 'suspend_time', 'suspend_reason','suspend_resume_time']:

		# Suspending?
		if (real_value):

			# Keep the current value for the following fields

			# Keep current settings, if any
			suspend_time   = dg.suspend_time
			resume_time    = dg.suspend_resume_time
			suspend_reason = dg.suspend_reason

			dg.suspend(
				resume_time  = resume_time, 
				suspend_time = suspend_time, 
				reason       = suspend_reason, 
				force=True
			)
		else:
			# suspended false
			resume_time = int(time.time())
			reason      = "No reason. (Resumed at %s)" % (resume_time)

			dg.resume(resume_time=int(time.time()), reason=reason, force=True)




	return

@transaction_commit_on_success
def datagroup_from_template_in_element_unset_field(element, element_template_name, datagroup_template_name, datagroup_instance_name, field_name, tenant, user=None):

	mtime = time.time()

	db_set_current_tenant_schema(tenant.name)

	if user:
		user.assert_capability(tenant, 'core.element.conf_monitoring')

	# Troviamo il template associato all'elemento
	e_t     = ElementTemplate.objects.get(melement=element, telement__name=element_template_name)
	telement = e_t.telement

	# Troviamo il datagroup nel template che corrisponde a quello che vogliamo modificare (deve esistere)
	tedg = TEDataGroup.objects.get(telement=telement, name=datagroup_instance_name)
	tdg  = tedg.tdatagroup
	datagroup_template_name = tdg.name

	cdg = CDataGroup.objects.get(tenant=tenant, element=element, tdatagroup=None, tedatagroup=tedg, name=datagroup_instance_name)

	dg = DataGroup.objects.get(tenant=tenant, element=element, cdatagroup=cdg)

	tf   = TField.objects.get(fclass=TField.FIELD_DATAGROUP, name=field_name)

	try:
		cdgf = CDataGroupFieldValue.objects.get(cdatagroup=cdg, tfield=tf)
	except CDataGroupFieldValue.DoesNotExist, e:
		if DEBUG: print "Field %s.%s.%s.%s.%s not configured" % (element.name, element_template_name, datagroup_template_name, datagroup_instance_name, field_name)
		return

	cdgf.delete()

	# Calcoliamo il valore che dovrebbe avere questo campo
	fv = cdg.get_configured_fields([tf.name])[0] #FIX: non molto elegante

	dg.update_single_field_and_update(fv, mtime=mtime)

@transaction_commit_on_success
def datagroup_from_template_in_element_set_parameter(element, element_template_name, datagroup_template_name, datagroup_instance_name, param_name, param_vtype, param_value, tenant, user=None):

	db_set_current_tenant_schema(tenant.name)

	if user:
		user.assert_capability(tenant, 'core.element.conf_monitoring')

	# Troviamo il template associato all'elemento
	e_t     = ElementTemplate.objects.get(melement=element, telement__name=element_template_name)
	telement = e_t.telement
	
	# Troviamo il datagroup nel template che corrisponde a quello che vogliamo modificare (deve esistere)
	tedg = TEDataGroup.objects.get(telement=telement, name=datagroup_instance_name)
	
	cdg = CDataGroup.objects.get(tenant=tenant, element=element, tdatagroup=None, tedatagroup=tedg, name=datagroup_instance_name)
	
	cdgpv, created = CDataGroupParameterValue.objects.get_or_create(tenant=tenant, cdatagroup=cdg, name=param_name)
		
	convert_to_param_type(param_vtype, param_value)
		
	if cdgpv.value == param_value:
		return
	
	mtime = time.time()
	
	# FIX: update related datagroups
	#cdgpv.tenant = cdg.tenant
	cdgpv.vtype  = param_vtype
	cdgpv.value  = param_value 
	#cdgpv.mtime  = mt
	#cdgpv.save()
	cdgpv.concurrent_save(['vtype','value'], new_sentinel_value=mtime)
	
	# Update 
	dg = DataGroup.objects.get(tenant=tenant, element=element, cdatagroup=cdg)
	
	dgp, created = DataGroupParameter.objects.get_or_create(tenant=tenant, datagroup=dg, name=param_name)
	dgp.vtype = param_vtype
	dgp.value = param_value
	#dgp.mtime = mt
	# FIX: usa update
	#dgp.save()
	dgp.concurrent_save(['vtype','value'], new_sentinel_value=mtime)
	
	return

@transaction_commit_on_success
def datagroup_from_template_in_element_unset_parameter(element, element_template_name, datagroup_template_name, datagroup_instance_name, param_name, tenant, user=None):

	mtime = time.time()

	db_set_current_tenant_schema(tenant.name)

	if user:
		user.assert_capability(tenant, 'core.element.conf_monitoring')

	# Troviamo il template associato all'elemento
	e_t     = ElementTemplate.objects.get(melement=element, telement__name=element_template_name)
	telement = e_t.telement

	# Troviamo il datagroup nel template che corrisponde a quello che vogliamo modificare (deve esistere)
	tedg = TEDataGroup.objects.get(telement=telement, name=datagroup_instance_name)

	cdg = CDataGroup.objects.get(tenant=tenant, element=element, tdatagroup=None, tedatagroup=tedg, name=datagroup_instance_name)

	dg = DataGroup.objects.get(tenant=tenant, element=element, cdatagroup=cdg)

	# Rimuoviamo il parametro definito
	try:
		cdgpv = CDataGroupParameterValue.objects.get(tenant=tenant, cdatagroup=cdg, name=param_name)
	except CDataGroupParameterValue.DoesNotExist, e:
		cdgpv = None

	if cdgpv is None:
		return

	cdgpv.delete()

	# Ricalcoliamo il valore del parametro (se e' definito almeno in un punto
	# altrimenti lo cancelliamo definitivamente
	pv = cdg.get_configured_param(param_name)
	if pv != None:

		# Update 

		dg.update_single_parameter_and_save(pv, mtime=mtime)
	else:
		# Delete
		dgp = DataGroupParameter.objects.get(tenant=tenant, datagroup=dg, name=param_name)
		dgp.delete()

		dg.concurrent_save(['mtime'], new_sentinel_value=mtime)



@transaction_commit_on_success
def datagroup_from_template_in_element_set_conditions_priority(element, datagroup_template_name, datagroup_instance_name, priority, user=None):

	mtime = time.time()

	tenant = element.tenant

	db_set_current_tenant_schema(tenant.name)

	if user:
		user.assert_capability(tenant, 'core.element.conf_monitoring')

	tf   = TField.objects.get(fclass=TField.FIELD_CONDITION, name='priority')

	# Troviamo il template associato all'elemento
	e_t     = ElementTemplate.objects.get(melement=element)
	telement = e_t.telement

	# Troviamo il datagroup nel template che corrisponde a quello che vogliamo modificare (deve esistere)
	tedg = TEDataGroup.objects.get(telement=telement, name=datagroup_instance_name)
	tdg  = tedg.tdatagroup
	
	cdg = CDataGroup.objects.get(tenant=tenant, element=element, tdatagroup=None, tedatagroup=tedg, name=datagroup_instance_name)

	# Update the working tables
	dg = DataGroup.objects.get(tenant=tenant, element=element, cdatagroup=cdg, name=datagroup_instance_name)
	
	
	for c in dg.conditions.all():
		
		if DEBUG: print "Setting priority for condition", c.name
	
		tc   = tdg.get_last_defined_condition(c.name)

		try:
			ccfv = CConditionFieldValue.objects.get(tenant=tenant, cdatagroup=cdg, tcondition=tc, tfield=tf)
			if DEBUG: print "\t", "updated"
		except CConditionFieldValue.DoesNotExist, e:
			ccfv = CConditionFieldValue(tenant=tenant, cdatagroup=cdg, tcondition=tc, tfield=tf)
			ccfv.save()
			if DEBUG: print "\t", "set"
		
		ccfv.value = str(priority)
		ccfv.concurrent_save(['value'], new_sentinel_value=mtime)

		c.update_single_field_and_save(ccfv, mtime=mtime)



@transaction_commit_on_success
def datasource_from_template_in_element_set_field(element, element_template_name, datagroup_instance_name, datasource_name, field_name, field_str_value, tenant, user=None):

	mtime = time.time()

	db_set_current_tenant_schema(tenant.name)

	if user:
		user.assert_capability(tenant, 'core.element.conf_monitoring')

	# Troviamo il datagroup nel element template che corrisponde a quello che vogliamo modificare (deve esistere)
		
	# Troviamo il template associato all'elemento
	e_t     = ElementTemplate.objects.get(melement=element, telement__name=element_template_name)
	telement = e_t.telement

	tf  = TField.objects.get(fclass=TField.FIELD_DATASOURCE, name=field_name)

	tf.validate_value(field_str_value)

	tedg = TEDataGroup.objects.get(telement=telement, name=datagroup_instance_name)
	tdg  = tedg.tdatagroup

	# Troviamo il datasource
	#tds = TDataSource.objects.get(tenant=tenant, tdatagroup=tdg, name=datasource_name)
	tds = tdg.get_last_defined_datasource(datasource_name)

	# Troviamo il configured datasource effettivo per l'elemento
	cdg = CDataGroup.objects.get(tenant=tenant, element=element, tdatagroup=None, tedatagroup=tedg, name=datagroup_instance_name)

	# Update the working tables
	dg = DataGroup.objects.get(tenant=tenant, element=element, cdatagroup=cdg, name=datagroup_instance_name)
	ds = DataSource.objects.get(tenant=tenant, datagroup=dg, name=datasource_name)

	old_configured = CDataSourceFieldValue.objects.filter(tenant=tenant, cdatagroup=cdg, tdatasource__name=tds.name, tfield=tf).exclude(tdatasource=tds)
	if len(old_configured) > 0:
		if DEBUG: print "Deleting old configured datasource field (%s entries)..." % (len(old_configured))
		old_configured.delete()

	try:
		cdsfv = CDataSourceFieldValue.objects.get(tenant=tenant, cdatagroup=cdg, tdatasource=tds, tfield=tf)
	except CDataSourceFieldValue.DoesNotExist, e:
		cdsfv = CDataSourceFieldValue(tenant=tenant, cdatagroup=cdg, tdatasource=tds, tfield=tf)
		cdsfv.save()

	# FIX: validate

	if cdsfv.value == field_str_value:
		if DEBUG: print "not updated (%s)" % (field_str_value)
		return

	cdsfv.value = field_str_value
	#cdsfv.tenant = tenant
	#cdsfv.save()
	cdsfv.concurrent_save(['value'], new_sentinel_value=mtime)

	ds.update_single_field_and_save(cdsfv, mtime=mtime)

	return

@transaction_commit_on_success
def datasource_from_template_in_element_unset_field(element, element_template_name, datagroup_instance_name, datasource_name, field_name, tenant, user=None):

	mtime = time.time()

	db_set_current_tenant_schema(tenant.name)

	if user:
		user.assert_capability(tenant, 'core.element.conf_monitoring')

	# Troviamo il datagroup nel element template che corrisponde a quello che vogliamo modificare (deve esistere)

	# Troviamo il template associato all'elemento
	e_t     = ElementTemplate.objects.get(melement=element, telement__name=element_template_name)
	telement = e_t.telement

	tf  = TField.objects.get(fclass=TField.FIELD_DATASOURCE, name=field_name)

	tedg = TEDataGroup.objects.get(telement=telement, name=datagroup_instance_name)
	tdg  = tedg.tdatagroup

	# Troviamo il configured datagroup effettivo per l'elemento
	cdg = CDataGroup.objects.get(tenant=tenant, element=element, tdatagroup=None, tedatagroup=tedg, name=datagroup_instance_name)

	# Troviamo il datasource
	#tds = TDataSource.objects.get(tenant=tenant, tdatagroup=tdg, name=datasource_name)
	tds = tdg.get_last_defined_datasource(datasource_name)

	# Update the working tables
	#dg = DataGroup.objects.get(tenant=tenant, element=element, name=datagroup_instance_name)
	dg = DataGroup.objects.get(tenant=tenant, element=element, cdatagroup=cdg)
	ds = DataSource.objects.get(tenant=tenant, datagroup=dg, name=datasource_name)

	#cdsfv = CDataSourceFieldValue.objects.get(tenant=tenant, cdatagroup=cdg, tdatasource=tds, tfield=tf)
	try:
		cdsfv = CDataSourceFieldValue.objects.get(tenant=tenant, cdatagroup=cdg, tdatasource__name=tds.name, tfield=tf)
	except CDataSourceFieldValue.DoesNotExist, e:
		if DEBUG: print "Field %s.%s.%s.%s.%s not configured" % (element.name, element_template_name, datagroup_instance_name, datasource_name, field_name)
		return

	cdsfv.delete()

	fv = cdg.get_configured_datasource_field_or_default(datasource_name, tf.name)

	ds.update_single_field_and_save(fv, mtime=mtime)

	return

@transaction_commit_on_success
def condition_from_template_in_element_set_field(element, element_template_name, datagroup_instance_name, condition_name, field_name, field_str_value, tenant, user=None):

	mtime = time.time()

	db_set_current_tenant_schema(tenant.name)

	if user:
		user.assert_capability(tenant, 'core.element.conf_monitoring')

	# Troviamo il template associato all'elemento
	e_t     = ElementTemplate.objects.get(melement=element, telement__name=element_template_name)
	telement = e_t.telement

	# Troviamo il datagroup nel element template che corrisponde a quello che vogliamo modificare (deve esistere)
	tf   = TField.objects.get(fclass=TField.FIELD_CONDITION, name=field_name)

	tf.validate_value(field_str_value)

	tedg = TEDataGroup.objects.get(telement=telement, name=datagroup_instance_name)
	tdg  = tedg.tdatagroup

	# Troviamo il condition
	#tc   = TCondition.objects.get(tenant=tenant, tdatagroup=tdg, name=condition_name)
	tc   = tdg.get_last_defined_condition(condition_name)

	# Troviamo il configured datasource effettivo per l'elemento
	cdg  = CDataGroup.objects.get(tenant=tenant, element=element, tdatagroup=None, tedatagroup=tedg, name=datagroup_instance_name)

	# Update the working tables
	dg = DataGroup.objects.get(tenant=tenant, element=element, cdatagroup=cdg, name=datagroup_instance_name)
	c  = Condition.objects.get(tenant=tenant, datagroup=dg, name=condition_name)

	old_configured = CConditionFieldValue.objects.filter(tenant=tenant, cdatagroup=cdg, tcondition__name=tc.name, tfield=tf).exclude(tcondition=tc)
	if len(old_configured) > 0:
		if DEBUG: print "Deleting old configured condition (%s entries)..." % (len(old_configured))
		old_configured.delete()

	try:
		cdsfv = CConditionFieldValue.objects.get(tenant=tenant, cdatagroup=cdg, tcondition=tc, tfield=tf)
	except CConditionFieldValue.DoesNotExist, e:
		cdsfv = CConditionFieldValue(tenant=tenant, cdatagroup=cdg, tcondition=tc, tfield=tf)
		cdsfv.save()

	# FIX: validate

	if cdsfv.value == field_str_value:
		if DEBUG: print "not updated (%s)" % (field_str_value)
		return

	cdsfv.value = field_str_value
	#cdsfv.save()
	cdsfv.concurrent_save(['value'], new_sentinel_value=mtime)

	changed = c.update_single_field_and_save(cdsfv, mtime=mtime)
	if changed:
		if field_name == 'primary':
			for dg in DataGroup.objects.filter(tenant=tenant, dependson=c):
				dg.update_from_config(mtime=mtime)

	return

@transaction_commit_on_success
def condition_from_template_in_element_unset_field(element, element_template_name, datagroup_instance_name, condition_name, field_name, tenant, user=None):

	mtime = time.time()

	db_set_current_tenant_schema(tenant.name)

	if user:
		user.assert_capability(tenant, 'core.element.conf_monitoring')

	# Troviamo il template associato all'elemento
	e_t      = ElementTemplate.objects.get(melement=element, telement__name=element_template_name)
	telement = e_t.telement

	# Troviamo il datagroup nel element template che corrisponde a quello che vogliamo modificare (deve esistere)

	tf   = TField.objects.get(fclass=TField.FIELD_CONDITION, name=field_name)

	#tf.validate_value(field_value)

	tedg = TEDataGroup.objects.get(telement=telement, name=datagroup_instance_name)
	tdg  = tedg.tdatagroup

	# Troviamo il condition
	#tc   = TCondition.objects.get(tenant=tenant, tdatagroup=tdg, name=condition_name)
	tc   = tdg.get_last_defined_condition(condition_name)

	cdg  = CDataGroup.objects.get(tenant=tenant, element=element, tdatagroup=None, tedatagroup=tedg, name=datagroup_instance_name)

	dg = DataGroup.objects.get(tenant=tenant, element=element, cdatagroup=cdg, name=datagroup_instance_name)
	c  = Condition.objects.get(tenant=tenant, datagroup=dg, name=condition_name)

	# Troviamo il configured datasource effettivo per l'elemento
	cdg  = CDataGroup.objects.get(tenant=tenant, element=element, tdatagroup=None, tedatagroup=tedg, name=datagroup_instance_name)

	cdsfv = CConditionFieldValue.objects.filter(tenant=tenant, cdatagroup=cdg, tcondition__name=tc.name, tfield=tf)

	#
	# FIX
	#
	cdsfv.delete()

	# Update the working tables
	fv = cdg.get_configured_condition_field_or_default(condition_name, tf.name)

	changed = c.update_single_field_and_save(fv, mtime=mtime)
	if changed:
		if field_name == 'primary':
			for dg in DataGroup.objects.filter(tenant=tenant, dependson=c):
				dg.update_from_config(mtime=mtime)

	return


@transaction_commit_on_success
def timegraph_from_template_in_element_set_field(element, element_template_name, datagroup_instance_name, timegraph_name, field_name, field_str_value, tenant, user=None):

	mtime = time.time()

	db_set_current_tenant_schema(tenant.name)

	if user:
		user.assert_capability(tenant, 'core.element.conf_monitoring')

	# Troviamo il template associato all'elemento
	e_t     = ElementTemplate.objects.get(melement=element, telement__name=element_template_name)
	telement = e_t.telement

	# Troviamo il datagroup nel element template che corrisponde a quello che vogliamo modificare (deve esistere)

	tf   = TField.objects.get(fclass=TField.FIELD_TGRAPH, name=field_name)

	tf.validate_value(field_str_value)

	tedg = TEDataGroup.objects.get(telement=telement, name=datagroup_instance_name)
	tdg  = tedg.tdatagroup
	
	#ttg   = TTimeGraph.objects.get(tenant=tenant, tdatagroup=tdg, name=timegraph_name)
	ttg   = tdg.get_last_defined_timegraph(timegraph_name)


	# Troviamo il configured datasource effettivo per l'elemento
	cdg  = CDataGroup.objects.get(tenant=tenant, element=element, tdatagroup=None, tedatagroup=tedg, name=datagroup_instance_name)

	# Update the working tables
	dg = DataGroup.objects.get(tenant=tenant, element=element, cdatagroup=cdg, name=datagroup_instance_name)
	tg = TimeGraph.objects.get(tenant=tenant, datagroup=dg, name=timegraph_name)

	try:
		ctgfv = CTimeGraphFieldValue.objects.get(tenant=tenant, cdatagroup=cdg, ttimegraph=ttg, tfield=tf)
	except CTimeGraphFieldValue.DoesNotExist, e:
		ctgfv = CTimeGraphFieldValue(tenant=tenant, cdatagroup=cdg, ttimegraph=ttg, tfield=tf)
		ctgfv.save()

	# FIX: validate

	ctgfv.value = field_str_value
	#ctgfv.save()
	ctgfv.concurrent_save(['value'], new_sentinel_value=mtime)

	tg.update_single_field_and_save(ctgfv, mtime=mtime)

	return

@transaction_commit_on_success
def timegraph_from_template_in_element_unset_field(element, element_template_name, datagroup_instance_name, timegraph_name, field_name, tenant, user=None):

	mtime = time.time()

	db_set_current_tenant_schema(tenant.name)

	if user:
		user.assert_capability(tenant, 'core.element.conf_monitoring')

	# Troviamo il template associato all'elemento
	e_t     = ElementTemplate.objects.get(melement=element, telement__name=element_template_name)
	telement = e_t.telement

	# Troviamo il datagroup nel element template che corrisponde a quello che vogliamo modificare (deve esistere)

	tf   = TField.objects.get(fclass=TField.FIELD_TGRAPH, name=field_name)

	tedg = TEDataGroup.objects.get(telement=telement, name=datagroup_instance_name)
	tdg  = tedg.tdatagroup

	#ttg   = TTimeGraph.objects.get(tenant=tenant, tdatagroup=tdg, name=timegraph_name)
	ttg   = tdg.get_last_defined_timegraph(timegraph_name)

	# Troviamo il configured datasource effettivo per l'elemento
	cdg  = CDataGroup.objects.get(tenant=tenant, element=element, tdatagroup=None, tedatagroup=tedg, name=datagroup_instance_name)

	try:
		ctgfv = CTimeGraphFieldValue.objects.get(tenant=tenant, cdatagroup=cdg, ttimegraph=ttg, tfield=tf)
	except CTimeGraphFieldValue.DoesNotExist, e:
		if DEBUG: print "Field %s.%s.%s.%s.%s not configured." % (element.name, element_template_name, datagroup_instance_name, timegraph_name, field_name)
		return

	ctgfv.delete()

	fv = cdg.get_configured_timegraph_field(timegraph_name, tf.name)

	# Update the working tables
	dg = cdg.get_datagroup()
	tg = TimeGraph.objects.get(tenant=tenant, datagroup=dg, name=timegraph_name)

	tg.update_single_field_and_save(fv, mtime=mtime)

	return

@transaction_commit_on_success
def timegraphserie_from_template_in_element_set_field(element, element_template_name, datagroup_instance_name, timegraph_name, serie_name, field_name, field_str_value, tenant, user=None):

	mtime = time.time()

	db_set_current_tenant_schema(tenant.name)

	if user:
		user.assert_capability(tenant, 'core.element.conf_monitoring')

	# Troviamo il template associato all'elemento
	e_t     = ElementTemplate.objects.get(melement=element, telement__name=element_template_name)
	telement = e_t.telement

	# Troviamo il datagroup nel element template che corrisponde a quello che vogliamo modificare (deve esistere)

	tf   = TField.objects.get(fclass=TField.FIELD_TSERIE, name=field_name)

	tf.validate_value(field_str_value)

	tedg = TEDataGroup.objects.get(telement=telement, name=datagroup_instance_name)
	tdg  = tedg.tdatagroup

	#ttg   = TTimeGraph.objects.get(tenant=tenant, tdatagroup=tdg, name=timegraph_name)
	#ttgs  = TTimeGraphSerie.objects.get(tenant=tenant, ttimegraph=ttg, name=serie_name)
	ttgs = tdg.get_last_defined_timegraphserie(timegraph_name, serie_name)

	# Troviamo il configured datasource effettivo per l'elemento
	cdg  = CDataGroup.objects.get(tenant=tenant, element=element, tdatagroup=None, tedatagroup=tedg, name=datagroup_instance_name)

	# Update the working tables
	dg = DataGroup.objects.get(tenant=tenant, element=element, cdatagroup=cdg, name=datagroup_instance_name)
	tg = TimeGraph.objects.get(tenant=tenant, datagroup=dg, name=timegraph_name)
	ts = TimeGraphSerie.objects.get(tenant=tenant, timegraph=tg, name=serie_name)

	try:
		ctgsfv = CTimeGraphSerieFieldValue.objects.get(tenant=tenant, cdatagroup=cdg, ttimegraphserie=ttgs, tfield=tf)
	except      CTimeGraphSerieFieldValue.DoesNotExist, e:
		ctgsfv = CTimeGraphSerieFieldValue(tenant=tenant, cdatagroup=cdg, ttimegraphserie=ttgs, tfield=tf)
		ctgsfv.save()

	# FIX: validate

	ctgsfv.value = field_str_value
	#ctgsfv.save()
	ctgsfv.concurrent_save(['value'], new_sentinel_value=mtime)

	ts.update_single_field_and_save(ctgsfv, mtime=mtime)

	return

@transaction_commit_on_success
def timegraphserie_from_template_in_element_unset_field(element, element_template_name, datagroup_instance_name, timegraph_name, serie_name, field_name, tenant, user=None):

	mtime = time.time()

	db_set_current_tenant_schema(tenant.name)

	if user:
		user.assert_capability(tenant, 'core.element.conf_monitoring')

	# Troviamo il template associato all'elemento
	e_t     = ElementTemplate.objects.get(melement=element, telement__name=element_template_name)
	telement = e_t.telement

	# Troviamo il datagroup nel element template che corrisponde a quello che vogliamo modificare (deve esistere)

	tf   = TField.objects.get(fclass=TField.FIELD_TSERIE, name=field_name)

	tedg = TEDataGroup.objects.get(telement=telement, name=datagroup_instance_name)
	tdg  = tedg.tdatagroup

	#ttg   = TTimeGraph.objects.get(tenant=tenant, tdatagroup=tdg, name=timegraph_name)
	#ttgs  = TTimeGraphSerie.objects.get(tenant=tenant, ttimegraph=ttg, name=serie_name)
	ttgs = tdg.get_last_defined_timegraphserie(timegraph_name, serie_name)

	# Troviamo il configured datasource effettivo per l'elemento
	cdg  = CDataGroup.objects.get(tenant=tenant, element=element, tdatagroup=None, tedatagroup=tedg, name=datagroup_instance_name)

	try:
		ctgsfv = CTimeGraphSerieFieldValue.objects.get(tenant=tenant, cdatagroup=cdg, ttimegraphserie=ttgs, tfield=tf)
	except CTimeGraphSerieFieldValue.DoesNotExist, e:
		if DEBUG: print "Field %s.%s.%s.%s.%s.%s not configured" % (element.name, element_template_name, datagroup_instance_name, timegraph_name, serie_name, field_name)
		return

	ctgsfv.delete()

	fv = cdg.get_configured_timegraphserie_field(timegraph_name, serie_name, field_name)

	# Update the working tables
	dg = cdg.get_datagroup()
	#dg = DataGroup.objects.get(tenant=tenant, element=element, name=datagroup_instance_name)
	tg = TimeGraph.objects.get(tenant=tenant, datagroup=dg, name=timegraph_name)
	ts = TimeGraphSerie.objects.get(tenant=tenant, timegraph=tg, name=serie_name)

	ts.update_single_field_and_save(fv, mtime=mtime)

	return

#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#

from core.models import SchedulingGroup

def cmd_set_element_scheduling_attribute(cli):

	if cli.env.get('PARSE_ONLY', False):
		return

	# template <name>

	env               = cli.env

	user              = env['CURRENT_USER']

	tenant_name       = env['CONFIG_TENANT']
	tenant            = tenant_get(tenant_name)

	element           = cli.get_config_element()
	kind              = element.kind
	
	
	if cli.canonical_words[0] != 'no':
	
		field_name  = cli.canonical_words[0]
		field_value = cli.canonical_words[1]
		
		if field_name == 'scheduling-group':
			
			
			if element.scheduling_group is not None and element.scheduling_group.is_real is False:
				cli.info("Removing old scheduling settings...")
				element.scheduling_group.delete()

			#
			# Scheduling group esplicito
			#
			
			name = field_value
			
			sg = SchedulingGroup.objects.get(tenant=element.tenant, name=name)
			
			if element.scheduling_group != sg:
				cli.info("Updated scheduling settings.")
				element.scheduling_group = sg
				element.save()
			
		else:
			#
			# Scheduling group implicito
			#
			created = False
			
			if element.scheduling_group is not None and element.scheduling_group.is_real:
				element.scheduling_group = None

			if element.scheduling_group is None:
				
				#sg_name = '%s:%s' % (element.kind_str, element.path)
				sg_name = element.implicit_scheduling_group_name
				
				sg, created = SchedulingGroup.objects.get_or_create(tenant=tenant, name=sg_name)
				sg.is_real = False
				sg.save()
				
				if created:
					sg.min_gap      = 3
					sg.max_parallel = 1
					sg.save()
				
				element.scheduling_group = sg
				element.save()
			else:
				sg = element.scheduling_group
			
			field_name  = field_name.replace('-','_')[11:]
			field_value = int(field_value)
			
			old_value = getattr(sg, field_name)
			if old_value != field_value:
				setattr(sg, field_name, field_value)
				sg.save()

			if created:
				cli.info("Scheduling settings set for the first time (min-gap %s, max-paralleld %s" % (sg.min_gap, sg.max_parallel))

	else:
		field_name  = cli.canonical_words[1]

		if field_name == 'scheduling-group':

			if element.scheduling_group != None:
				element.scheduling_group = None
				element.save()

		else:
			sg = element.scheduling_group
			if sg is None:
				return
			
			field_name  = field_name.replace('-','_')[11:]

			old_value = getattr(sg, field_name)
			if old_value != None:
				setattr(sg, field_name, None)
				sg.save()

				if sg.min_gap is None and sg.max_parallel is None:
					cli.info("Removing scheduling settings...")
					sg.delete()

		return


MT_CONFIG_ELEMENT_SCHEDULING_COMMANDS = [
	T_Keyword('scheduling-min-gap'     , help='Set min-gap between datagroups.'       , subtree=[ T_Number                  (help='Set min-gap between datagroups.'       , callback=cmd_set_element_scheduling_attribute) ] ),
	T_Keyword('scheduling-max-deadline', help='Set datagroup execution deadline.'     , subtree=[ T_Number                  (help='Set datagroup execution deadline.'     , callback=cmd_set_element_scheduling_attribute) ] ),
	T_Keyword('scheduling-max-parallel', help='Set max number of parallel datagroups.', subtree=[ T_Number                  (help='Set max number of parallel datagroups.', callback=cmd_set_element_scheduling_attribute) ] ),
	T_Keyword('scheduling-group'       , help='Assign elemento to scheduling group.'  , subtree=[ T_SchedulingGroupcompleter(help='Assign elemento to scheduling group.'  , callback=cmd_set_element_scheduling_attribute) ] ),
]

MT_CONFIG_ELEMENT_SCHEDULING_COMMANDS_NO = [
	T_Keyword('scheduling-min-gap'     , help='Unset min-gap.'                          , callback=cmd_set_element_scheduling_attribute),
	T_Keyword('scheduling-max-deadline', help='Unset deadline.'                         , callback=cmd_set_element_scheduling_attribute),
	T_Keyword('scheduling-max-parallel', help='Unset max number of parallel datagroups.', callback=cmd_set_element_scheduling_attribute),
	T_Keyword('scheduling-group'       , help='Remove element from scheduling group.'   , callback=cmd_set_element_scheduling_attribute),
]

#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#

def MT_CONFIG_ELEMENT_COMMON_COMMANDS(set_cluster, set_attribute):

	return  [
		T_Keyword('description'          , help='Add a description of this element'            , subtree=[T_Line(help=''   , callback=set_attribute,),]),

		T_Keyword('is-abstract'          , help='Specify if this element is a abstract element', subtree=[T_Boolean(help='', callback=set_attribute,), ]),

		T_Keyword('cluster'              , help='Cluster name'                                 , subtree=[T_Clustercompleter(help='', callback=set_cluster,),]),

		T_Keyword('cluster-role'         , help='Role of this cluster'                         , subtree=[
			T_Alternatives(CLI_CLUSTER_ROLE_TO_NAME_DESCR.values(), help='cluster role', callback=set_attribute,)
		]),

		T_Keyword('cluster-distinguisher', help='Unique distinguisher within the cluster'      , subtree=[T_Line(help='', callback=set_attribute,),]),
		T_Keyword('cluster-xform'        , help='Cluster xform'                                , subtree=[T_Line(help='', callback=set_attribute,),]),
		T_Keyword('cluster-instance'     , help='Cluster instance'                             , subtree=[T_Line(help='', callback=set_attribute,),]),
	]

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_set_element_template(cli):

	if cli.env.get('PARSE_ONLY', False):
		return	

	# template <name>

	env               = cli.env
	

	user              = env['CURRENT_USER']

	tenant_name       = env['CONFIG_TENANT']
	tenant            = tenant_get(tenant_name)

	element           = cli.get_config_element()
	kind              = element.kind

	element_category_name = cli.canonical_words[1]

	tenant = element.tenant

	telement = TElement.objects.get(tenant=tenant, kind=kind, name=element_category_name)

	#
	# TEMP: Questo warning e' messo solo come "avvertenza" dopo l'introduzione dei multi-template. Vverra' rimosso tra non molto.
	#
	if ElementTemplate.objects.filter(tenant=tenant, melement=element, telement=telement).count() == 0:
		cli.info("*** IMPORTANT: adding '%s' to element '%s'. THIS TEMPLATE WILL NOT REPLACE CURRENT ASSIGNED TEMPLATES. ***" % (element_category_name, element.path))

	element_set_template(element, telement, user=user)

	return

def cmd_unset_element_template(cli):

	# 0  1        2                 3
	# no template <elementtemplate> <elementtemplate>

	env               = cli.env

	user              = env['CURRENT_USER']

	tenant_name       = env['CONFIG_TENANT']
	tenant            = tenant_get(tenant_name)

	element           = cli.get_config_element() 

	element_template_name1 = cli.canonical_words[2]
	element_template_name2 = cli.canonical_words[3]

	if element_template_name1 != element_template_name2:
		cli.error("template name does not match")
		return

	element_unset_template(element, element_template_name1, user=user)

	return

def cmd_replace_all_element_template(cli):
	
	# 0                          1
	# replace-all-template-with  <elementtemplate>
	
	env               = cli.env

	user              = env['CURRENT_USER']

	tenant_name       = env['CONFIG_TENANT']
	tenant            = tenant_get(tenant_name)

	element           = cli.get_config_element()
	kind              = element.kind
	
	element_template_name1 = cli.canonical_words[1]
	
	'''e_t = ElementTemplate.objects.filter(tenant=tenant, melement=element)
	
	for t in e_t:
		cli.warning('*** Sto rimuovendo il template %s ***' % (str(t.telement.name)))
		element_unset_template(element, t.telement.name, user=user)'''
		
	element_unset_template(element, user=user)
	
	telement = TElement.objects.get(tenant=tenant, kind=kind, name=element_template_name1)
		
	element_set_template(element, telement, user=user)
	
#------------------------------------------------------------------------------#
# DATAGROUP FIELD                                                              #
#------------------------------------------------------------------------------#

def cmd_config_datagroup_field_from_template_in_element(cli):

	if cli.env.get('PARSE_ONLY', False):
		return

	env = cli.env

	user           = env['CURRENT_USER']

	tenant_name    = env['CONFIG_TENANT']
	tenant         = tenant_get(tenant_name)

	element        = cli.get_config_element()

	element_template_name   = env['CONFIG_ELEMENT_TEMPLATE'  ]
	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']

	if cli.canonical_words[0] != 'no':
		field_name      = cli.canonical_words[0]
		field_str_value = cli.canonical_words[1]

		field_name      = cli.canonical_words[0]
		field_name      = CLI_FIELD_TO_DATAGROUP_FIELD(field_name)

		datagroup_from_template_in_element_set_field  (element, element_template_name, datagroup_template_name, datagroup_instance_name, field_name, field_str_value, tenant, user=user)
	else:
		field_name      = cli.canonical_words[1]
		field_name      = CLI_FIELD_TO_DATAGROUP_FIELD(field_name)

		datagroup_from_template_in_element_unset_field(element, element_template_name, datagroup_template_name, datagroup_instance_name, field_name, tenant, user=user)

	return

#------------------------------------------------------------------------------#
# PARAMETER                                                                    #
#------------------------------------------------------------------------------#

def cmd_config_datagroup_parameter_from_template_in_element(cli):

	if cli.env.get('PARSE_ONLY', False):
		return

	env = cli.env

	user           = env['CURRENT_USER']

	tenant_name    = env['CONFIG_TENANT']
	tenant         = tenant_get(tenant_name)

	element = cli.get_config_element() 

	element_template_name   = env['CONFIG_ELEMENT_TEMPLATE'  ]
	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']

	# parameter <name> <type> <value>
	# no parameter <name>
	if cli.canonical_words[0] != 'no':
		param_name      = cli.canonical_words[1]
		param_type_name = cli.canonical_words[2]
		param_value     = cli.canonical_words[3]

		param_value     = cli_input_to_real_param_value(param_type_name, param_value)

		# Validate param value. This will raise an exception
		ptype, pvalue = convert_param_value(param_type_name, param_value)

		datagroup_from_template_in_element_set_parameter  (element, element_template_name, datagroup_template_name, datagroup_instance_name, param_name, ptype, param_value, tenant=tenant, user=user)
	else:
		param_name      = cli.canonical_words[2]
		
		datagroup_from_template_in_element_unset_parameter(element, element_template_name, datagroup_template_name, datagroup_instance_name, param_name, tenant=tenant, user=user)
	return

#------------------------------------------------------------------------------#
# DATASOURCE FIELDS                                                            #
#------------------------------------------------------------------------------#

def cmd_config_datasource_in_element_field(cli):

	if cli.env.get('PARSE_ONLY', False):
		return

	env = cli.env

	user           = env['CURRENT_USER']

	tenant_name    = env['CONFIG_TENANT']
	tenant         = tenant_get(tenant_name)

	element = cli.get_config_element()

	element_template_name   = env['CONFIG_ELEMENT_TEMPLATE'  ]
	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']
	datasource_name         = env['CONFIG_DATASOURCE']

	if cli.canonical_words[0] != 'no':

		field_name      = cli.canonical_words[0]
		field_name      = CLI_FIELD_TO_DATASOURCE_FIELD(field_name)

		field_str_value = cli.canonical_words[1]

		datasource_from_template_in_element_set_field(element, element_template_name, datagroup_instance_name, datasource_name, field_name, field_str_value, tenant, user=user)
	else:
		field_name      = cli.canonical_words[1]
		field_name      = CLI_FIELD_TO_DATASOURCE_FIELD(field_name)

		datasource_from_template_in_element_unset_field(element, element_template_name, datagroup_instance_name, datasource_name, field_name, tenant, user=user)

	return

#------------------------------------------------------------------------------#

mt_config_datasource_in_element = CREATE_DATASOURCE_FIELD_COMMANDS(cmd_config_datasource_in_element_field)

def cmd_datasource_from_template_in_element(cli):

	# datasource <name>

	env = cli.env

	if not cli.env.get('PARSE_ONLY', False):

		tenant_name    = env['CONFIG_TENANT']
		tenant         = tenant_get(tenant_name)
		db_set_current_tenant_schema(tenant.name)

		element = cli.get_config_element() 

		element_template_name   = env['CONFIG_ELEMENT_TEMPLATE'  ]
		datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
		datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']

		datasource_name = cli.canonical_words[1]

		# Troviamo il template associato all'elemento
		e_t     = ElementTemplate.objects.get(melement=element, telement__name=element_template_name)
		telement = e_t.telement

		# Troviamo il datagroup nel template che corrisponde a quello che vogliamo modificare (deve esistere)

		tedg = TEDataGroup.objects.get(telement=telement, name=datagroup_instance_name)
		tdg  = tedg.tdatagroup

		tds = tdg.get_last_defined_datasource(datasource_name)

		datagroup_template_name = tdg.name

		cdg = CDataGroup.objects.get(tenant=tenant, element=element, tdatagroup=None, tedatagroup=tedg, name=datagroup_instance_name)

		env = cli.env.clone()
		if element.parent != None:
			parent_name = element.parent.name
			env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, parent_name, element.name, datagroup_instance_name, datasource_name]),)
		else:
			env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, element.name, datagroup_instance_name, datasource_name]),)

		env['CONFIG_DATASOURCE'] = datasource_name

	cmds = mt_config_datasource_in_element + [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))

	#cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	#cli.cmdloop()
	cli.enter_subcli(cmds, env)



#------------------------------------------------------------------------------#
# CONDITION FIELDS                                                             #
#------------------------------------------------------------------------------#


def cmd_config_condition_in_element_field(cli):

	if cli.env.get('PARSE_ONLY', False):
		return

	env = cli.env

	user        = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant      = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant.name)

	element = cli.get_config_element()

	element_template_name   = env['CONFIG_ELEMENT_TEMPLATE'  ]
	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']
	condition_name          = env['CONFIG_CONDITION']

	# <field> <value>
	# no <field> 
	if cli.canonical_words[0] != 'no':

		field_name      = cli.canonical_words[0]
		field_name      = CLI_FIELD_TO_CONDITION_FIELD(field_name)

		field_str_value = cli.canonical_words[1]
		field_str_value = CLI_FIELD_VALUE_TO_CONDITION_FIELD_VALUE(field_name, field_str_value)

		condition_from_template_in_element_set_field  (element, element_template_name, datagroup_instance_name, condition_name, field_name, field_str_value, tenant, user=user)
	else:
		field_name      = cli.canonical_words[1]
		field_name      = CLI_FIELD_TO_CONDITION_FIELD(field_name)
		
		condition_from_template_in_element_unset_field(element, element_template_name, datagroup_instance_name, condition_name, field_name, tenant, user=user)

#------------------------------------------------------------------------------#

mt_config_condition_in_element  = CREATE_CONDITION_FIELD_COMMANDS(cmd_config_condition_in_element_field)


def cmd_condition_from_template_in_element(cli):

	# datasource <name>
	
	env = cli.env

	if not cli.env.get('PARSE_ONLY', False):

		tenant_name = env['CONFIG_TENANT']
		tenant      = tenant_get(tenant_name)
		db_set_current_tenant_schema(tenant.name)

		element = cli.get_config_element()

		element_template_name   = env['CONFIG_ELEMENT_TEMPLATE'  ]
		datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
		datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']

		condition_name = cli.canonical_words[1]

		# Troviamo il template associato all'elemento
		e_t     = ElementTemplate.objects.get(melement=element, telement__name=element_template_name)
		telement = e_t.telement


		# Troviamo il datagroup nel template che corrisponde a quello che vogliamo modificare (deve esistere)
		tedg = TEDataGroup.objects.get(telement=telement, name=datagroup_instance_name)
		tdg  = tedg.tdatagroup

		tc = tdg.get_last_defined_condition(condition_name)

		datagroup_template_name = tdg.name

		cdg = CDataGroup.objects.get(tenant=tenant, element=element, tdatagroup=None, tedatagroup=tedg, name=datagroup_instance_name)

		env = cli.env.clone()
		if element.parent != None:
			parent_name = element.parent.name
			env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, parent_name, element.name, datagroup_instance_name, condition_name]),)
		else:
			env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, element.name, datagroup_instance_name, condition_name]),)

		env['CONFIG_CONDITION'] = condition_name

	cmds = mt_config_condition_in_element + [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))
	
	#cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	#cli.cmdloop()
	cli.enter_subcli(cmds, env)
	

#------------------------------------------------------------------------------#
# TIMEGRAPH                                                                    #
#------------------------------------------------------------------------------#

def cmd_config_timegraph_in_element_field(cli):


	if cli.env.get('PARSE_ONLY', False):
		return

	# <field> <value>

	env = cli.env

	user        = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant      = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant.name)

	element                 = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']

	element_template_name   = env['CONFIG_ELEMENT_TEMPLATE'  ]
	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']
	timegraph_name          = env['CONFIG_TIMEGRAPH']

	if cli.canonical_words[0] != 'no':

		field_name      = cli.canonical_words[0]
		field_name      = CLI_FIELD_TO_TIMEGRAPH_FIELD(field_name)
		
		field_str_value = cli.canonical_words[1]

		timegraph_from_template_in_element_set_field(element, element_template_name, datagroup_instance_name, timegraph_name, field_name, field_str_value, tenant, user=user)
	else:
		field_name      = cli.canonical_words[1]
		field_name      = CLI_FIELD_TO_TIMEGRAPH_FIELD(field_name)
		
		timegraph_from_template_in_element_unset_field(element, element_template_name, datagroup_instance_name, timegraph_name, field_name, tenant, user=user)


def cmd_config_timegraph_serie_field_in_element(cli):

	if cli.env.get('PARSE_ONLY', False):
		return

	env                     = cli.env

	user        = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant      = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant.name)

	element                 = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']
	
	element_template_name   = env['CONFIG_ELEMENT_TEMPLATE'  ]
	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']
	timegraph_name          = env['CONFIG_TIMEGRAPH']
	serie_name              = env['CONFIG_TIMEGRAPH_SERIE']

	if cli.canonical_words[0] != 'no':

		field_name      = cli.canonical_words[0]
		field_name      = CLI_FIELD_TO_TIMEGRAPHSERIE_FIELD(field_name)
		field_str_value = cli.canonical_words[1]

		timegraphserie_from_template_in_element_set_field(element, element_template_name, datagroup_instance_name, timegraph_name, serie_name, field_name, field_str_value, tenant, user=user)
	else:
		field_name      = cli.canonical_words[1]
		field_name      = CLI_FIELD_TO_TIMEGRAPHSERIE_FIELD(field_name)

		timegraphserie_from_template_in_element_unset_field(element, element_template_name, datagroup_instance_name, timegraph_name, serie_name, field_name, tenant, user=user)

	return

mt_config_timegraph_serie_field_in_element = CREATE_TIMEGRAPH_SERIE_FIELD_COMMANDS(cmd_config_timegraph_serie_field_in_element)


def cmd_config_timegraph_serie_in_element(cli):

	env                     = cli.env

	if not cli.env.get('PARSE_ONLY', False):

		tenant_name = env['CONFIG_TENANT']
		tenant      = tenant_get(tenant_name)
		db_set_current_tenant_schema(tenant.name)
		
		element                 = cli.get_config_element()
		
		element_template_name   = env['CONFIG_ELEMENT_TEMPLATE'  ]
		datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
		datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']
		timegraph_name          = env['CONFIG_TIMEGRAPH']
		
		serie_name              = cli.canonical_words[1]
			
		# Troviamo il template associato all'elemento
		e_t     = ElementTemplate.objects.get(melement=element, telement__name=element_template_name)
		telement = e_t.telement

		# Troviamo il datagroup nel element template che corrisponde a quello che vogliamo modificare (deve esistere)
		tedg = TEDataGroup.objects.get(telement=telement, name=datagroup_instance_name)
		tdg  = tedg.tdatagroup
		
		ttgs   = tdg.get_last_defined_timegraphserie(timegraph_name, serie_name)
		
		# Troviamo il configured datasource effettivo per l'elemento
		cdg  = CDataGroup.objects.get(tenant=tenant, element=element, tdatagroup=None, tedatagroup=tedg, name=datagroup_instance_name)
		
		env = cli.env.clone()
		if element.parent != None:
			parent_name = element.parent.name
			env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, parent_name, element.name, datagroup_instance_name, timegraph_name, serie_name]),)
		else:
			env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, element.name, datagroup_instance_name, timegraph_name, serie_name]),)	
		
		env['CONFIG_TIMEGRAPH_SERIE'] = serie_name

	cmds = mt_config_timegraph_serie_field_in_element + [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))

	#cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	#cli.cmdloop()
	cli.enter_subcli(cmds, env)
	

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

mt_config_timegraph_in_element = CREATE_TIMEGRAPH_FIELD_COMMANDS(
	cmd_config_timegraph_in_element_field,
	cmd_config_timegraph_serie_in_element,
)

def cmd_timegraph_from_template_in_element(cli):
	
	# timegraph <name>
	env = cli.env

	if not cli.env.get('PARSE_ONLY', False):

		tenant_name = env['CONFIG_TENANT']
		tenant      = tenant_get(tenant_name)
		db_set_current_tenant_schema(tenant.name)
		
		element                 = cli.get_config_element()
		
		element_template_name   = env['CONFIG_ELEMENT_TEMPLATE'  ]
		datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
		datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']
		
		timegraph_name = cli.canonical_words[1]
		
		# Elemento da modificare		

		# Troviamo il template associato all'elemento
		e_t      = ElementTemplate.objects.get(melement=element, telement__name=element_template_name)
		telement = e_t.telement


		# Troviamo il datagroup nel template che corrisponde a quello che vogliamo modificare (deve esistere)
		
		tedg = TEDataGroup.objects.get(telement=telement, name=datagroup_instance_name)
		tdg  = tedg.tdatagroup

		#ttg = TTimeGraph.objects.get(tenant=tenant, tdatagroup=tdg, name=timegraph_name)
		ttg = tdg.get_last_defined_timegraph(timegraph_name)

		datagroup_template_name = tdg.name

		cdg = CDataGroup.objects.get(tenant=tenant, element=element, tdatagroup=None, tedatagroup=tedg, name=datagroup_instance_name)

		env = cli.env.clone()
		
		if element.parent != None:
			parent_name = element.parent.name
			env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, parent_name, element.name, datagroup_instance_name, timegraph_name]),)
		else:
			env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, element.name, datagroup_instance_name, timegraph_name]),)	
		
		env['CONFIG_TIMEGRAPH'] = timegraph_name
	
	cmds = mt_config_timegraph_in_element + [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))
	
	#cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	#cli.cmdloop()
	cli.enter_subcli(cmds, env)


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_config_all_datagroup_conditions_priority(cli):
	env = cli.env
	
	tenant_name    = env['CONFIG_TENANT']
	tenant         = tenant_get(tenant_name)
	
	element        = cli.get_config_element()

	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']

	priority = int(cli.canonical_words[1])

	datagroup_from_template_in_element_set_conditions_priority(element, datagroup_template_name, datagroup_instance_name, priority)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

from cli.tokens import T_CDataGroupParameter

mt_datagroup_from_template_in_element = CREATE_DATAGROUP_COMMANSDS( 
                                               cmd_config_datagroup_field_from_template_in_element    ,
                                               cmd_config_datagroup_parameter_from_template_in_element,
                                               
                                               cmd_datasource_from_template_in_element,
                                               cmd_condition_from_template_in_element, 
                                               cmd_timegraph_from_template_in_element,
                                               
                                               include_scheduling_commands=True,
                                               
                                               tok_no_param_completer      = T_CDataGroupParameter,
                                               
                                               include_no_commands=False,
                                        ) + [
                                               T_Keyword('priority-conditions'     , help='Set priority of all conditions', subtree=[
                                                   T_Number(help='Set priority of all conditions', callback=cmd_config_all_datagroup_conditions_priority)
                                               ]),
]

def cmd_datagroup_template_in_element_config_prompt(cli):
	
	# 0                  1        2    3
	# template-datagroup <name> [ from <element-template> ]

	env = cli.env
	
	if not cli.env.get('PARSE_ONLY', False):
				
		tenant_name = env['CONFIG_TENANT']
		tenant      = tenant_get(tenant_name)
		db_set_current_tenant_schema(tenant.name)
		
		element                 = cli.get_config_element()

		# template-datagroup <instance name>
		datagroup_instance_name = cli.canonical_words[1]
		
		element_template_name   = cli.canonical_words[3] if (len(cli.canonical_words) > 3) else None


		# Troviamo il datagroup-template che stiamo cercando di personalizzare.
		#
		if element_template_name:

			# E' stato esplicitato l'element-template dove cercare guardiamo in quello
			e_t      = ElementTemplate.objects.get(melement=element, telement__name=element_template_name)
			telement = e_t.telement

			# Troviamo il datagroup nel template che corrisponde a quello che vogliamo modificare (deve esistere)
			tedg = TEDataGroup.objects.get(telement=telement, name=datagroup_instance_name)
			
			tdg  = tedg.tdatagroup
			datagroup_template_name = tdg.name

		else:
			# Troviamo tutti gli element-template associati all'elemento
			used_telements = [ e.telement for e in ElementTemplate.objects.filter(melement=element).all() ]

			# Troviamo il datagroup dentro un element-template 
			tedg = TEDataGroup.objects.get(telement__in=used_telements, name=datagroup_instance_name)
			
			tdg  = tedg.tdatagroup
			datagroup_template_name = tdg.name
			
			telement = tedg.telement

		cdg = CDataGroup.objects.get(tenant=tenant, element=element, tdatagroup=None, tedatagroup=tedg, name=datagroup_instance_name)


		env = cli.env.clone()

		if element.parent != None:
			parent_name = element.parent.name
			env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, parent_name, element.name, datagroup_instance_name]),)
		else:
			env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, element.name, datagroup_instance_name]),)

		env['CONFIG_DATAGROUP_TEMPLATE'  ] = datagroup_template_name
		env['CONFIG_DATAGROUP_INSTANCE'  ] = datagroup_instance_name
		env['CONFIG_ELEMENT_TEMPLATE'    ] = telement.name

		env['ELEMENT_TEMPLATE_DATAGROUP' ] = tedg.name

	cmds = SORTED_KEYWORDS(mt_datagroup_from_template_in_element) + [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))
	
	#cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	#cli.cmdloop()
	cli.enter_subcli(cmds, env)


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def set_priority_primary(cli):

	env               = cli.env

	user              = env['CURRENT_USER']

	tenant_name       = env['CONFIG_TENANT']
	tenant            = tenant_get(tenant_name)

	element = cli.get_config_element() 

	priority_value = int(cli.canonical_words[1])

	_set_priority_primary(element, priority_value, user=user)

def _set_priority_primary(element, priority_value, user=None):

	c = element.get_primary_condition()
	if c is None:
		return  

	dg = c.datagroup

	if DEBUG: print "Setting priority for primary condition", c.path

	cdg = dg.cdatagroup

	if cdg.tedatagroup is not None:
		if DEBUG: print "SETTING PRIMARY FROM TEMPLATE-DATAGROUP"

		datagroup_instance_name = cdg.name
		condition_name          = c.name
		field_name              = 'priority'
		field_str_value         = priority_value

		condition_from_template_in_element_set_field(element, cdg.tedatagroup.telement.name, datagroup_instance_name, condition_name, field_name, field_str_value, element.tenant, user=user)
	else:
		if DEBUG: print "SETTING ADDITIONAL"
		datagroup_instance_name = cdg.name
		condition_name          = c.name
		field_name              = 'priority'
		field_str_value         = priority_value
		
		element_additional_condition_field_set(element, datagroup_instance_name, condition_name, field_name, field_str_value, element.tenant, user=user)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

VALID_DATAGROUP_NAME_HACK = re.compile(r"^[a-zA-Z][a-zA-Z0-9-+_]*$")


def mt_config_element_template_and_datagroups(kind, can_set_template=True, can_configure_template_datagroups=True, can_configure_additional_datagroups=True):
	
	entries = []
		
	entries += [
		T_Keyword('primary-priority'     , help='Set priority of the primary datagroup', subtree=[T_Line(help=''   , callback=set_priority_primary,),]),
	]

	if can_set_template:
		entries += [
			# Category and datagroup settings
			T_Keyword('template', help='Specify element template', subtree=[
				T_ParametricElementTemplate(kind, help='', callback=cmd_set_element_template, )
			]),
			
			# replace-all-template-with
			T_Keyword('replace-all-template-with', help='Specify element template to use for replace others', subtree=[
				T_ParametricElementTemplate(kind, help='', callback=cmd_replace_all_element_template, )
			]),
		]
	
	if can_configure_template_datagroups:
		entries += [
			T_Keyword('template-datagroup', help='Modify datagroup from the element category', subtree=[
				T_ElementTemplateDataGroup(help='', callback=cmd_datagroup_template_in_element_config_prompt, subtree=[
					T_Keyword('from', help='', subtree=[
						T_ParametricElementTemplate(kind, help='', callback=cmd_datagroup_template_in_element_config_prompt, )
					])
				]),
			]),
		]
	
	if can_configure_additional_datagroups:
		entries += [
			# Additional datagroups
			T_Keyword('datagroup', help='Specify datagroup for this element', subtree=[
				T_DataGroupTemplate(help='Datagroup template name', callback=cmd_additional_datagroup_config_prompt, subtree=[]),
				T_LibraryDataGroupTemplate(help='Datagroup template name', callback=cmd_additional_datagroup_config_prompt, pipe=None,),
				#T_Anyword(help='Custom name', callback=cmd_additional_datagroup_config_prompt, subtree=[
				T_ValidWord(VALID_DATAGROUP_NAME_HACK, help='Custom name', callback=cmd_additional_datagroup_config_prompt, subtree=[
					T_Keyword('template', help='Datagroup template', subtree=[
						T_DataGroupTemplate(help='Datagroup template name', callback=cmd_additional_datagroup_config_prompt),
						T_LibraryDataGroupTemplate(help='Datagroup template name', callback=cmd_additional_datagroup_config_prompt, pipe=None,),
					]),
				]),
			]),	
		]
	
	return entries


def mt_config_element_template_and_datagroups_no(kind):

	entries = [
		T_Keyword('template', help='', subtree=[
			T_ParametricElementTemplate(kind, help='',  subtree=[
				T_ParametricElementTemplate(kind, help='',  callback=cmd_unset_element_template)
			]),
		]),
		mt_config_additional_datagroup_no
	]
	return entries
