# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from sanet_common.sanetdefs.constants import ELEMENT_KIND_IFACE

from sanet_poller.lib.evaluation import compile_str_expr

from core.db_utils import transaction_commit_on_success

from core.api.base import *

from cli.clilib.ioslike import *

from cli.clilib.ioslike import SORTED_KEYWORDS

from cli.tokens import T_KeywordWithHook
from cli.tokens import T_Iconcompleter

from cli.prompts.defs import *

from cli.prompts import cli_get_registered_menus

from cli.prompts.config.element_config import mt_config_element_template_and_datagroups
from cli.prompts.config.element_config import mt_config_element_template_and_datagroups_no

from cli.prompts.config.element_config import MT_CONFIG_ELEMENT_SCHEDULING_COMMANDS
from cli.prompts.config.element_config import MT_CONFIG_ELEMENT_SCHEDULING_COMMANDS_NO

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def cmd_iface_set_attribute(cli):

	if cli.env.get('PARSE_ONLY', False):
		return

	env         = cli.env

	user        = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)

	node_name   = env['CONFIG_NODE']

	i   = cli.get_config_element() 

	mtime = time.time()

	field_name = cli.canonical_words[0]
	value      = cli.canonical_words[1]

	field_name = field_name.replace('-','_')  # BAD IDEA BUT IT WORKS

	if field_name == 'xform':
		value = xform_alias_to_xform('interfaces', value)
		try:
			compile_str_expr(None, value)
		except Exception, e:
			cli.error("Error while parsing expression: %s" % (e))
			return

	try:
		value = i.convert_field_str_value(field_name, value)
	except Exception, e:
		cli.error("Error in value: %s" % (str(e)))

	#
	# FIX: MOve to API
	#

	if user:
		user.assert_capability(tenant, 'core.element.conf')

	setattr(i, field_name, value)

	iface_update(i, field_name, mtime=mtime)

	if field_name == 'priority':
		i.refresh_elements_priority(mtime=mtime)

	return

@transaction_commit_on_success
def cmd_iface_unset_attribute(cli):

	if cli.env.get('PARSE_ONLY', False):
		return

	# no <field> 
	
	env         = cli.env

	user        = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)

	node_name   = env['CONFIG_NODE']
	i           = cli.get_config_element()

	#
	# FIX: move to library
	#

	if user:
		user.assert_capability(tenant, 'core.element.conf')

	mtime = time.time()

	field_name = cli.canonical_words[1]

	field_name = field_name.replace('-','_')  # BAD IDEA BUT IT WORKS

	value = i.get_fields_default_value_dict().get(field_name, None)

	setattr(i, field_name, value)

	iface_update(i, field_name, mtime=mtime)

	if field_name == 'priority':
		i.refresh_elements_priority(mtime=mtime)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

mt_config_iface = SORTED_KEYWORDS([
	T_Keyword('description'   , help='Unset description'     , subtree=[ T_Line           (help='', callback=cmd_iface_set_attribute) ]),
	T_Keyword('distinguisher' , help='Unset distinguisher'   , subtree=[ T_Line           (help='', callback=cmd_iface_set_attribute) ]),
	T_Keyword('xform'         , help='Unset xform'           , subtree=[ T_Line           (help='', callback=cmd_iface_set_attribute) ]),
	T_Keyword('ifindex'       , help='Unset ifindex'         , subtree=[ T_Line           (help='', callback=cmd_iface_set_attribute) ]),
	T_Keyword('icon'          , help='Unset default priority', subtree=[ T_Iconcompleter  (help='', callback=cmd_iface_set_attribute) ]),
	T_Keyword('priority'      , help='Unset icon'            , subtree=[ T_PositiveInteger(help='', callback=cmd_iface_set_attribute) ]),
	T_Keyword('backbone'      , help='Unset backbone flag'   , subtree=[ T_Boolean        (help='', callback=cmd_iface_set_attribute) ]),
	T_Keyword('speed'         , help='Unset nominal speed'   , subtree=[ T_PositiveInteger(help='', callback=cmd_iface_set_attribute) ]),
]) + MT_CONFIG_ELEMENT_SCHEDULING_COMMANDS

mt_config_iface_no = [
	T_KeywordWithHook('no', help='Unset/Remove properties', 
		hook_name='configuration:interface:no',
		subtree=[
			T_Keyword('description'   , help='Unset description'     , callback=cmd_iface_unset_attribute) ,
			T_Keyword('distinguisher' , help='Unset distinguisher'   , callback=cmd_iface_unset_attribute) ,
			T_Keyword('xform'         , help='Unset xform'           , callback=cmd_iface_unset_attribute) ,
			T_Keyword('ifindex'       , help='Unset ifindex'         , callback=cmd_iface_unset_attribute) ,
			T_Keyword('priority'      , help='Unset default priority', callback=cmd_iface_unset_attribute) ,
			T_Keyword('icon'          , help='Unset icon'            , callback=cmd_iface_unset_attribute) ,
			T_Keyword('backbone'      , help='Unset backbone flag'   , callback=cmd_iface_unset_attribute) ,
			T_Keyword('speed'         , help='Unset nominal speed'   , callback=cmd_iface_unset_attribute) ,
		]
		+
		mt_config_element_template_and_datagroups_no(ELEMENT_KIND_IFACE)
		+ 
		MT_CONFIG_ELEMENT_SCHEDULING_COMMANDS_NO
	),
]


def cmd_iface_config_prompt(cli):

	#   0        1      2     3   4    5             
	# interface eth0 instance 1 xform "byRegexpUnique('prova','123')"

	env = cli.env

	if not cli.env.get('PARSE_ONLY', False):

		tenant_name = env['CURRENT_TENANT']
		node_name   = env['CURRENT_NODE']

		name = cli.canonical_words[1]
		instance = None
		xform = None

		if len(cli.canonical_words) >= 4:
			instance = cli.canonical_words[3]

		if len(cli.canonical_words) >= 6:
			xform = cli.canonical_words[5]

		iface = iface_get(node_name=node_name, iface_name=name, tenant_name=tenant_name)
		if iface == None:
			cli.info("Creating iface %s" % (name))

			iface = iface_create(node_name=node_name, iface_name=name, tenant_name=tenant_name)
		else:
			if iface.telement_generator is not None:
				cli.warning("Interface %s already exists from template %s" % (name, iface.telement_generator.name))
				return
			else:
				cli.info("Interface '%s' already defined" % (name))

		updated=False

		if instance != None:
			if iface.distinguisher != instance:
				iface.distinguisher = instance
				updated=True

		if xform != None:
			xform = xform_alias_to_xform('interfaces', xform)

			try:
				compile_str_expr(None, xform)
			except Exception, e:
				cli.error("Invalid xform: %s" % (e))
				return

			if iface.xform != xform:
				#xform = ifdescr_to_xform(xform)
				iface.xform = xform
				updated=True

		if updated:
			iface_update(iface)

		env = cli.env.clone()

		env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, node_name, name]),)
		env['CONFIG_IFACE'] = name
		
		env['CURRENT_ELEMENT'] = (ELEMENT_KIND_IFACE, name)

		env['CONFIG_ELEMENT_DATA'] = iface
		env['CURRENT_ELEMENT_DATA'] = iface

	#else:
	#	env = cli.env.clone()

	cmds = SORTED_KEYWORDS(mt_config_iface) + \
	       [BLANK_ENTRY] + \
	       mt_config_element_template_and_datagroups(ELEMENT_KIND_IFACE) + \
	       [BLANK_ENTRY] + \
	       mt_config_iface_no + \
	       [BLANK_ENTRY] + \
	       SORTED_KEYWORDS(cli_get_registered_menus('configuration:interface')) + \
	       [BLANK_ENTRY] + \
	       SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__')) 

	#cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	#cli.cmdloop()
	cli.enter_subcli(cmds, env)


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_iface_from_template_config_promt(cli):

	return _cmd_element_from_template_config_prompt(cli, ELEMENT_KIND_IFACE, mt_config_iface, mt_config_iface_no, 'configuration:interface')


def _cmd_element_from_template_config_prompt(cli, kind, mt_config_menu, mt_config_no_menu, menu_hook_id):
	
	env = cli.env

	if not cli.env.get('PARSE_ONLY', False):

		# 0                  1
		# template-interface <name>

		env = cli.env
		tenant_name = env['CURRENT_TENANT']
		node_name   = env['CURRENT_NODE']

		name = cli.canonical_words[1]

		#
		# Find monitored element
		#
		element = MonitoredElement.objects.get(tenant__name=tenant_name, parent__name=node_name, name=name, telement_generator__isnull=False)

		element = element.get_specific_element()

		#
		# NOTHING TO DO?
		#
		env = cli.env.clone()

		env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, node_name, name]),)
		env['CONFIG_IFACE'] = name

		env['CURRENT_ELEMENT'] = (kind, name)

		env['CONFIG_ELEMENT_DATA'] = element
		env['CURRENT_ELEMENT_DATA'] = element

		#cmds = SORTED_KEYWORDS(mt_config_iface) + \
		#       [BLANK_ENTRY] + \
		#       mt_config_element_template_and_datagroups(kind, can_set_template=False) + \
		#       [BLANK_ENTRY] + \
		#       mt_config_iface_no + \
		#       [BLANK_ENTRY] + \
		#       SORTED_KEYWORDS(cli_get_registered_menus('configuration:interface')) + \
		#       [BLANK_ENTRY] + \
		#       SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__')) 

		       #mt_config_element_template_and_datagroups(kind, can_set_template=False) + \

	cmds = SORTED_KEYWORDS(mt_config_menu) + \
	       [BLANK_ENTRY] + \
	       mt_config_element_template_and_datagroups(kind) + \
	       [BLANK_ENTRY] + \
	       mt_config_no_menu + \
	       [BLANK_ENTRY] + \
	       SORTED_KEYWORDS(cli_get_registered_menus(menu_hook_id)) + \
	       [BLANK_ENTRY] + \
	       SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__')) 


	#cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	#cli.cmdloop()

	cli.enter_subcli(cmds, env)


