# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from sanet_common.sanetdefs.constants import ELEMENT_KIND_NODE

from core.db_utils import transaction_commit_on_success

from core.api.base import *
from core.api.elements import *
from core.api.elements import element_rename

from cli.clilib.ioslike import *

from cli.tokens import T_Usercompleter
from cli.tokens import T_KeywordWithHook
from cli.tokens import T_Agentcompleter
from cli.tokens import T_Iconcompleter

from cli.clilib.ioslike import SORTED_KEYWORDS

from cli.tokens import T_SubInterfaceCompleter
from cli.tokens import T_SubStorageCompleter
from cli.tokens import T_SubServiceCompleter
from cli.tokens import T_SubDeviceCompleter

from cli.tokens import T_SubInterfaceFromTemplateCompleter
from cli.tokens import T_SubStorageFromTemplateCompleter  
from cli.tokens import T_SubServiceFromTemplateCompleter  
from cli.tokens import T_SubDeviceFromTemplateCompleter   

from cli.prompts.defs import *

from cli.prompts import cli_get_registered_menus

from cli.prompts.config.ifaces     import cmd_iface_config_prompt
from cli.prompts.config.storages   import cmd_storage_config_prompt
from cli.prompts.config.services   import cmd_service_config_prompt
from cli.prompts.config.devices    import cmd_device_config_prompt

from cli.prompts.config.ifaces     import cmd_iface_from_template_config_promt
from cli.prompts.config.storages   import cmd_storage_from_template_config_promt
from cli.prompts.config.services   import cmd_service_from_template_config_promt
from cli.prompts.config.devices    import cmd_device_from_template_config_promt

from cli.prompts.config.element_config import MT_CONFIG_ELEMENT_COMMON_COMMANDS
from cli.prompts.config.element_config import mt_config_element_template_and_datagroups
from cli.prompts.config.element_config import mt_config_element_template_and_datagroups_no

from cli.prompts.config.element_config import MT_CONFIG_ELEMENT_SCHEDULING_COMMANDS
from cli.prompts.config.element_config import MT_CONFIG_ELEMENT_SCHEDULING_COMMANDS_NO

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

"""
	interface/storage/device/service x instance y xform z
	....
	exit
"""
import types

def NODE_SUB_RESOURCE_TEMPLATE(name, valid_re, callback, t_completer=T_Anyword, help_str='', xform_tok=None):

	#
	#   <name>   <valid_re>  distinguisher|instance  <distinguisher>  xform  <tokens>|<string>
	#

	DISTINGUISHERS_TOKS = (
		('distinguisher', 'String'),
		('instance'     , 'String (synonym of distinguisher)')
	)

	subtree=[]

	#
	# <tokens>|<string>
	#
	xformtree = []
	if xform_tok:

		if not isinstance(xform_tok, types.ListType):
			xform_tok = [ xform_tok, ]

		for tok in xform_tok:
			xformtree += [
				T_Keyword(tok, help='', callback=callback, pipe=None),
			]

	xformtree += [
		T_Anyword(help='', callback=callback, pipe=None, ), 
	]

	if t_completer != None:
	
		subtree = [
			t_completer(help='Existing subelement', callback=callback, pipe=None, subtree=[
				T_Alternatives(DISTINGUISHERS_TOKS, help='Specify a distinguisher', pipe=None, subtree=[
					T_Anyword(help='Distinguisher string value', pipe=None, callback=callback, subtree=[
						T_Keyword('xform', help='Specify xform expression', pipe=None, subtree=xformtree),
					],),
				],),
			],)
		]

	subtree += [
			T_ValidWord(valid_re, help='New %s' % (name), callback=callback, pipe=None, subtree=[
				T_Alternatives(DISTINGUISHERS_TOKS, help='Specify a distinguisher', pipe=None, subtree=[
					T_Anyword(help='Distinguisher string value', pipe=None, callback=callback, subtree=[
						T_Keyword('xform', help='Specify xform expression', pipe=None, subtree=xformtree),
					],),
				],),
			],)
	]

	t = T_Keyword(name, help=help_str, subtree=subtree, pipe=None)		

	return t

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_node_set_cluster(cli):

	if cli.env.get('PARSE_ONLY', False):
		return

	env               = cli.env

	user              = env['CURRENT_USER']

	tenant_name       = env['CONFIG_TENANT']

	element = cli.get_config_element() 

	node_name = element.name

	cluster_name = cli.canonical_words[1]

	c    = cluster_get(cluster_name, tenant_name=tenant_name)
	node = node_get(node_name, tenant_name=tenant_name)

	node_set_cluster(node, c, user=user)

def cmd_node_remove_cluster(cli):

	env               = cli.env

	user              = env['CURRENT_USER']

	tenant_name       = env['CONFIG_TENANT']

	element = cli.get_config_element()
	node_name = element.name

	node = node_get(node_name, tenant_name=tenant_name)

	node_set_cluster(node, None, user=user)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def cmd_node_set_attribute(cli):

	if cli.env.get('PARSE_ONLY', False):
		return

	env               = cli.env

	user              = env['CURRENT_USER']

	tenant_name       = env['CONFIG_TENANT']
	tenant            = Tenant.objects.get(name=tenant_name)

	element = cli.get_config_element() 

	node_name = element.name

	mtime = time.time()


	field_name  = cli.canonical_words[0]
	field_value = cli.canonical_words[1]


	#
	# FIX: MOVE TO core.lib
	#

	if user:
		user.assert_capability(tenant, 'core.element.conf')

	field_name = field_name.replace('-','_')  # BAD IDEA BUT IT WORKS

	n = node_get(node_name, tenant_name=tenant_name)


	#
	# FIX: this is only for RETRO-COMPATIBILITY. REMOVE AS SOON AS POSSIBLE.
	#
	
	#if field_name == 'snmp-community':
	#	if n.snmp_version == 3:
	#		from sanet_poller.lib.snmp.base import parse_snmpv3_auth_options
	#		if parse_snmpv3_auth_options(field_value):
	#			cli.warning("The use of this field is deprecated: use snmp-v3-authdata instead!")

	if field_name in n.CLUSTER_FIELDS:
		
		if n.cluster == None:
			raise Exception("Unable to set field without a cluster")
		
		if field_name == 'cluster_role':
			field_value = CLI_NAME_TO_ROLE[field_value]

	try:
		value = n.convert_field_str_value(field_name, field_value)
	except Exception, e:
		raise e

	setattr(n, field_name, value)

	node_update(n, field_name, mtime=mtime)

	if field_name == 'priority':
		n.refresh_elements_priority(mtime=mtime)


@transaction_commit_on_success	
def cmd_node_set_attribute_no(cli):

	if cli.env.get('PARSE_ONLY', False):
		return


	# no <field> 
	env               = cli.env

	user              = env['CURRENT_USER']

	tenant_name       = env['CONFIG_TENANT']
	tenant            = Tenant.objects.get(name=tenant_name)
	
	#(kind, node_name) = env['CONFIG_ELEMENT' ]
	element = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']
	
	#kind      = element.kind
	node_name = element.name
	
	mtime = time.time()

	field_name  = cli.canonical_words[1]

	field_name = field_name.replace('-','_')  # BAD IDEA BUT IT WORKS


	#
	# FIX: MOVE TO core.lib
	#

	if user:
		user.assert_capability(tenant, 'core.element.conf')
	
	n = node_get(node_name, tenant_name=tenant_name)

	#val = None
	val = n.get_fields_default_value_dict().get(field_name, None)

	setattr(n, field_name, val)

	# FIX: update only selected fields
	node_update(n, field_name, mtime=mtime)

	if field_name == 'priority':
		n.refresh_elements_priority(mtime=mtime)

	
def cmd_node_set_agent(cli):

	if cli.env.get('PARSE_ONLY', False):
		return

	env         = cli.env

	user        = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant      = Tenant.objects.get(name=tenant_name)

	element     = cli.get_config_element() 
	node_name   = element.name
	
	agent_name = cli.canonical_words[1]
	
	a    = agent_get(agent_name, tenant_name=tenant_name)
	node = node_get(node_name, tenant_name=tenant_name)
	
	node_set_agent(node, a, user=user)
	
	if node.cluster is not None:
		for n in MonitoredNode.objects.filter(cluster=node.cluster).all():
			if n.uuid != n.uuid:
				if node.agent != n.agent:
					cli.warning("Node '%s' belong to cluster '%s' which is monitored by agent '%s', but cluster node '%s' is assigned to an other agent '%s'. You must change node's agent!" % (node.name, node.cluster.name, node.agent.name, n.name, n.agent.name))

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_node_remove_interface(cli):
	cmd_node_remove_sub_element(cli, ELEMENT_KIND_IFACE)

def cmd_node_remove_storage(cli):
	cmd_node_remove_sub_element(cli, ELEMENT_KIND_STORAGE)

def cmd_node_remove_service(cli):
	cmd_node_remove_sub_element(cli, ELEMENT_KIND_SERVICE)

def cmd_node_remove_device(cli):
	cmd_node_remove_sub_element(cli, ELEMENT_KIND_DEVICE)

@transaction_commit_on_success
def cmd_node_remove_sub_element(cli, subkind):

	if cli.env.get('PARSE_ONLY', False):
		return

	env = cli.env

	user              = env['CURRENT_USER']

	tenant_name       = env['CONFIG_TENANT']
	tenant  = tenant_get(tenant_name)

	element = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']
	#kind      = element.kind
	node_name = element.name
	

	name  = cli.canonical_words[2]
	name2 = cli.canonical_words[3]

	if name != name2:
		raise Exception("Name different")
		
	node = node_get(node_name, tenant_name=tenant_name)

	# Up to now
	if subkind == ELEMENT_KIND_IFACE:
		iface_delete(tenant, node_name, name, user=user)
	else:
		# Generic remove for the other kind of elements
		node_remove_sub_element(node, subkind, name, user=user)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_node_rename_interface(cli):
	cmd_node_rename_sub_element(cli, ELEMENT_KIND_IFACE)

def cmd_node_rename_storage(cli):
	cmd_node_rename_sub_element(cli, ELEMENT_KIND_STORAGE)

def cmd_node_rename_service(cli):
	cmd_node_rename_sub_element(cli, ELEMENT_KIND_SERVICE)

def cmd_node_rename_device(cli):
	cmd_node_rename_sub_element(cli, ELEMENT_KIND_DEVICE)
	
def cmd_node_rename_sub_element(cli, kind):

	if cli.env.get('PARSE_ONLY', False):
		return

	# rename <type> <name> <new name>

	env = cli.env

	user        = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant      = tenant_get(tenant_name)

	element     = cli.get_config_element() 

	name  = cli.canonical_words[2]
	name2 = cli.canonical_words[3]

	sub_element = MonitoredElement.objects.get(tenant=tenant, kind=kind, parent=element, name=name)

	element_rename(sub_element, name2, user=user)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_node_set_owner(cli):

	if cli.env.get('PARSE_ONLY', False):
		return

	# owner <user>
	# no owner

	user        = cli.env['CURRENT_USER']

	tenant_name = cli.env['CONFIG_TENANT']
	
	tenant  = tenant_get(tenant_name)
	
	element = cli.get_config_element() 

	if cli.canonical_words[0] != 'no':

		username = cli.canonical_words[1]

		owner = User.objects.get(username=username)

		element.set_owner(owner, action_user=user)
	else:
		element.unset_owner(action_user=user)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

SNMP_VERSIONS = [
	('0', 0),  # 
	
	('1', 1), 
	('2', 2), 
	('3', 3)
	
]

mt_config_common_node = MT_CONFIG_ELEMENT_COMMON_COMMANDS(cmd_node_set_cluster, cmd_node_set_attribute)

mt_config_specific_node = [
	T_Keyword('dns-name'             , help='A name'             , pipe=None, subtree=[T_Anyword        (help='A name'             , callback=cmd_node_set_attribute, pipe=None                                             )]), #FIX: valid word
	T_Keyword('dns6-name'            , help='AAAA name'          , pipe=None, subtree=[T_Anyword        (help='AAAA name'          , callback=cmd_node_set_attribute, pipe=None                                             )]), #FIX: valid word
	T_Keyword('ip4-mgt'              , help='IPv4 address'       , pipe=None, subtree=[T_Anyword        (help='IPv4 address'       , callback=cmd_node_set_attribute, pipe=None                                             )]), #FIX: valid word
	T_Keyword('ip6-mgt'              , help='IPv6 address'       , pipe=None, subtree=[T_Anyword        (help='IPv6 address'       , callback=cmd_node_set_attribute, pipe=None                                             )]), #FIX: valid word
	T_Keyword('use-ipv4'             , help='IP protocol version', pipe=None, subtree=[T_Boolean        (help='IP protocol version', callback=cmd_node_set_attribute, pipe=None                                             )]), #FIX: valid word
	T_Keyword('timezone'             , help='Timezone'           , pipe=None, subtree=[T_Anyword        (help='Timezone'           , callback=cmd_node_set_attribute, pipe=None                                             )]), #FIX: valid word
	T_Keyword('snmp-port'            , help='Snmp UDP port'      , pipe=None, subtree=[T_PositiveInteger(help='Snmp UDP port'      , callback=cmd_node_set_attribute, pipe=None                                             )]), #FIX: valid word
	T_Keyword('snmp-version'         , help='Snmp Version'       , pipe=None, subtree=[T_Alternatives   (help='Snmp Version'       , callback=cmd_node_set_attribute, pipe=None, alternatives=SNMP_VERSIONS                 )]), #FIX: valid word
	T_Keyword('snmp-community'       , help='Snmp Community'     , pipe=None, subtree=[T_Line           (help='Snmp Community'     , callback=cmd_node_set_attribute, pipe=None                                             )]), #FIX: valid word
	T_Keyword('snmp-v3-authdata'     , help='Snmp V3 Auth Data'  , pipe=None, subtree=[T_Line           (help='Snmp V3 Auth Data'  , callback=cmd_node_set_attribute, pipe=None                                             )]), #FIX: valid word
	T_Keyword('dft-timeout'          , help='Default timeout'    , pipe=None, subtree=[T_PositiveInteger(help='Default timeout'    , callback=cmd_node_set_attribute, pipe=None                                             )]), #FIX: valid word
	T_Keyword('icon'                 , help='Icon'               , pipe=None, subtree=[T_Iconcompleter  (help='Icon'               , callback=cmd_node_set_attribute, pipe=None                                             )]),
	T_Keyword('priority'             , help='Priority'           , pipe=None, subtree=[T_PositiveInteger(help='Priority'           , callback=cmd_node_set_attribute, pipe=None                                             )]), #FIX: valid priority

	T_Keyword('notify-address'       , help='Notify Address'     , pipe=None, subtree=[T_Line           (help='Notify Address'     , callback=cmd_node_set_attribute, pipe=None                                             )]), #FIX: valid priority

	T_Keyword('agent'                , help='Agent'              , pipe=None, subtree=[T_Agentcompleter (help='Agent'              , callback=cmd_node_set_agent    , pipe=None                                             )]),
] 

mt_config_node_subresources = [
	NODE_SUB_RESOURCE_TEMPLATE('interface', VALID_IFACE_NAME  , cmd_iface_config_prompt  , t_completer=T_SubInterfaceCompleter, xform_tok=xform_aliases('interfaces'), help_str='Configure interface.'),
	NODE_SUB_RESOURCE_TEMPLATE('storage'  , VALID_STORAGE_NAME, cmd_storage_config_prompt, t_completer=T_SubStorageCompleter  , xform_tok=xform_aliases('storages')  , help_str='Configure storage.'),
	NODE_SUB_RESOURCE_TEMPLATE('service'  , VALID_SERVICE_NAME, cmd_service_config_prompt, t_completer=T_SubServiceCompleter  , xform_tok=xform_aliases('services')  , help_str='Configure service.'),
	NODE_SUB_RESOURCE_TEMPLATE('device'   , VALID_DEVICE_NAME , cmd_device_config_prompt , t_completer=T_SubDeviceCompleter   , xform_tok=xform_aliases('devices')   , help_str='Configure device.'),
] + [
	T_Keyword('rename', help='Rename sub-elements.' , subtree=[
		T_Keyword('interface', help='Rename interface.', subtree=[
			T_SubInterfaceCompleter(help='Rename interface',  subtree=[
				T_ValidWord (VALID_IFACE_NAME, help='Rename interface', callback=cmd_node_rename_interface, pipe=None,)
			]),
		]),
		T_Keyword('storage', help='Rename storage.', subtree=[
			T_SubStorageCompleter(help='Rename storage',  subtree=[
				T_ValidWord (VALID_STORAGE_NAME, help='Rename storage', callback=cmd_node_rename_storage, pipe=None,)
			]),
		]),
		T_Keyword('service', help='Rename service.', subtree=[
			T_SubServiceCompleter(help='Rename service',  subtree=[
				T_ValidWord(VALID_SERVICE_NAME, help='Rename service', callback=cmd_node_rename_service, pipe=None,)
			]),
		]),
		T_Keyword('device', help='Rename device.', subtree=[
			T_SubDeviceCompleter(help='Rename device.',  subtree=[
				T_ValidWord(VALID_DEVICE_NAME, help='Rename device.', callback=cmd_node_rename_device, pipe=None,)
			]),
		]),
	]),
] + [
	T_Keyword('template-interface', help='Configure interface from template.', subtree=[
		T_SubInterfaceFromTemplateCompleter(help='', callback=cmd_iface_from_template_config_promt),
	]),
	T_Keyword('template-storage', help='Configure storage from template.', subtree=[
		T_SubStorageFromTemplateCompleter  (help='Configure storage from template.', callback=cmd_storage_from_template_config_promt),
	]),
	T_Keyword('template-service', help='Configure service from template.', subtree=[
		T_SubServiceFromTemplateCompleter  (help='Configure service from template.', callback=cmd_service_from_template_config_promt),
	]),
	T_Keyword('template-device', help='Configure device from template.', subtree=[
		T_SubDeviceFromTemplateCompleter   (help='Configure devices from template.', callback=cmd_device_from_template_config_promt),
	]),
] + [
	T_Keyword('owner', help='', subtree=[
		T_Usercompleter(help='', callback=cmd_node_set_owner),
	])
] + MT_CONFIG_ELEMENT_SCHEDULING_COMMANDS


	


mt_config_node_no = [

	T_KeywordWithHook('no', help='Remove configuration sections.', 
		hook_name='configuration:node:no'
		,
		subtree=[

		T_Keyword('icon'                 , help='', callback=cmd_node_set_attribute_no ),
		T_Keyword('snmp-port'            , help='', callback=cmd_node_set_attribute_no ),
		T_Keyword('snmp-version'         , help='', callback=cmd_node_set_attribute_no ),
		T_Keyword('snmp-community'       , help='', callback=cmd_node_set_attribute_no ),
		T_Keyword('snmp-v3-authdata'     , help='', callback=cmd_node_set_attribute_no ),
		T_Keyword('dft-timeout'          , help='', callback=cmd_node_set_attribute_no ),

		T_Keyword('dns-name'             , help='', callback=cmd_node_set_attribute_no ),
		T_Keyword('dns6-name'            , help='', callback=cmd_node_set_attribute_no ),
		T_Keyword('ip4-mgt'              , help='', callback=cmd_node_set_attribute_no ),
		T_Keyword('ip6-mgt'              , help='', callback=cmd_node_set_attribute_no ),

		T_Keyword('timezone'             , help='', callback=cmd_node_set_attribute_no ),

		T_Keyword('priority'             , help='', callback=cmd_node_set_attribute_no ),

		T_Keyword('notify-address'       , help='', callback=cmd_node_set_attribute_no ),

		T_Keyword('cluster'              , help='Remove reference to cluster.', callback=cmd_node_remove_cluster 
		),

		T_Keyword('interface', help='Remove interface.', subtree=[
			T_SubInterfaceCompleter(help='Remove interface.', subtree=[
				T_SubInterfaceCompleter(help='Remove interface.', callback=cmd_node_remove_interface) #FIX: completer
			]),
		]),
		T_Keyword('storage'  , help='Remove storage.', subtree=[
			T_SubStorageCompleter(help='Remove storage.', subtree=[
				T_SubStorageCompleter(help='Remove storage.', callback=cmd_node_remove_storage,) #FIX: completer
			]),
		]),
		T_Keyword('service'  , help='Remove service.', subtree=[
			T_SubServiceCompleter(help='Remove service.', subtree=[
				T_SubServiceCompleter(help='Remove service.', callback=cmd_node_remove_service,) #FIX: completer
			]),
		]),
		T_Keyword('device'   , help='Remove device.', subtree=[
			T_SubDeviceCompleter(help='Remove device.', subtree=[
				T_SubDeviceCompleter(help='Remove device.', callback=cmd_node_remove_device,) #FIX: completer
			]),
		]),

		]
		+
		mt_config_element_template_and_datagroups_no(ELEMENT_KIND_NODE)
		+ 
		[
		T_Keyword('owner'       , help='', callback=cmd_node_set_owner ),
		]
		+ 
		MT_CONFIG_ELEMENT_SCHEDULING_COMMANDS_NO
	
	),
]

def cmd_node_config_prompt(cli):

	env = cli.env

	if not cli.env.get('PARSE_ONLY', False):

		user        = env['CURRENT_USER']
		tenant_name = env['CURRENT_TENANT']

		tenant = Tenant.objects.get(name=tenant_name)

		name = cli.canonical_words[1]

		node = node_get(name, tenant_name=tenant_name)
		if node == None:
			node = node_create(name, tenant_name=tenant_name, owner=user)

			cli.info("Creating node %s" % (name))

		node.assert_permission_readwrite(user)

		env = cli.env.clone()

		env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, name]),)

		env['CONFIG_NODE']  = name
		env['CURRENT_NODE'] = name

		env['CURRENT_ELEMENT'] = (ELEMENT_KIND_NODE, name)
		env['CONFIG_ELEMENT' ] = (ELEMENT_KIND_NODE, name)

		env['CURRENT_ELEMENT_DATA'] = node
		env['CONFIG_ELEMENT_DATA'] = node

	else:
		env = cli.env.clone()

	cmds = SORTED_KEYWORDS(mt_config_common_node) + \
	       [BLANK_ENTRY] + \
	       SORTED_KEYWORDS(mt_config_specific_node) + \
	       [BLANK_ENTRY] + \
	       mt_config_element_template_and_datagroups(ELEMENT_KIND_NODE) + \
	       [BLANK_ENTRY] + \
	       SORTED_KEYWORDS(mt_config_node_subresources) + \
	       [BLANK_ENTRY] + \
	       mt_config_node_no + \
	       [BLANK_ENTRY] + \
	       SORTED_KEYWORDS(cli_get_registered_menus('configuration:node')) + \
	       [BLANK_ENTRY] + \
	       SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__')
	       )

	#cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	#cli.cmdloop()
	cli.enter_subcli(cmds, env)

