# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.


from core.models import Tenant
from core.models import SchedulingGroup

from cli.clilib.ioslike import T_Keyword, T_Alternatives, T_PositiveInteger, SORTED_KEYWORDS

from cli.tokens import AUTH_READONLY, AUTH_READWRITE
from cli.tokens import T_Usercompleter
from cli.tokens import T_Groupcompleter

from cli.prompts import cli_get_registered_menus
from cli.prompts.defs import PROMPT_SEP
from cli.prompts.defs import CLI_PERM_OPTIONS

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_config_scheduling_group_field(cli):

	tenant_name  = cli.env['CONFIG_TENANT']

	name = cli.env['CONFIG_SCHEDULINGGROUP']

	#tenant = tenant_get(tenant_name)
	tenant = Tenant.objects.get(name=tenant_name)

	sg = SchedulingGroup.objects.get(tenant=tenant, name=name)
	
	if cli.canonical_words[0] != 'no':

		field_name   = cli.canonical_words[0]
		field_value  = int(cli.canonical_words[1])

		field_name = field_name.replace('-', '_')

		old_value = getattr(sg, field_name)

		if old_value != field_value:

			setattr(sg, field_name, field_value)
			sg.validate()
			sg.save()

			if field_name == 'min_gap':
				if sg.max_parallel is None:
					sg.max_parallel = 1
					sg.save()
					cli.info("Set max-parallel to default = %s" % (sg.max_parallel))

	else:
		field_name   = cli.canonical_words[1]

		field_name = field_name.replace('-', '_')
		
		dft_value = sg.get_fields_default_value_dict().get(field_name, None)

		setattr(sg, field_name, dft_value)
		sg.validate()
		sg.save()


def cmd_config_scheduling_group_owner(cli):

	if cli.env.get('PARSE_ONLY', False):
		return

	# owner <user>
	# no owner

	user        = cli.env['CURRENT_USER']
	tenant_name = cli.env['CONFIG_TENANT']
	name        = cli.env['CONFIG_SCHEDULINGGROUP']
	
	tenant  = tenant_get(tenant_name)

	sg = SchedulingGroup.objects.get(tenant=tenant, name=name)

	if cli.canonical_words[0] != 'no':

		username = cli.canonical_words[1]

		owner = User.objects.get(username=username)

		sg.set_owner(owner, action_user=user)
	else:
		sg.unset_owner(action_user=user)


CLI_PERM_TO_AUTH_PERM = {
	'read-write': AUTH_READWRITE,
	'read-only' : AUTH_READONLY, 
}

def cmd_config_scheduling_group_permission(cli):

	user = cli.env['CURRENT_USER']

	tenant_name = cli.env['CURRENT_TENANT']

	name        = cli.env['CONFIG_SCHEDULINGGROUP']

	#name = cli.env['CONFIG_ELEMENT_TEMPLATE'    ]
	#kind = cli.env['CONFIG_ELEMENT_TEMPLATE_KIND']

	tenant = tenant_get(tenant_name)
	
	sg = SchedulingGroup.objects.get(tenant=tenant, name=name)

	if cli.canonical_words[0] != 'no':

		group_name = cli.canonical_words[1]

		if len(cli.canonical_words) > 2:
			permission = CLI_PERM_TO_AUTH_PERM[ cli.canonical_words[2] ]
		else:
			permission = AUTH_READONLY

		g = Group.objects.get(name=group_name)

		sg.add_group_permission(g, permission, user)
	else:

		group_name = cli.canonical_words[2]

		g = Group.objects.get(name=group_name)

		sg.remove_group_permission(g, user)




mt_config_schedulinggroup = [

	T_Keyword('owner', help='', subtree=[
		T_Usercompleter(help='', callback=cmd_config_scheduling_group_owner),
	]),

	T_Keyword('group', help='', subtree=[
		T_Groupcompleter(help='', callback=cmd_config_scheduling_group_permission, subtree=[ 
			T_Alternatives( CLI_PERM_OPTIONS, callback=cmd_config_scheduling_group_permission )
		]),
	]),

	T_Keyword('min-gap', help='Set min-gap between datagroups.', pipe=None, subtree=[
		T_PositiveInteger(help='Set min-gap between datagroups.', pipe=None, callback=cmd_config_scheduling_group_field, ),
	]),
	T_Keyword('max-deadline', help='Set datagroup execution deadline.', pipe=None, subtree=[
		T_PositiveInteger(help='Set datagroup execution deadline.', pipe=None, callback=cmd_config_scheduling_group_field, ),
	]),

	T_Keyword('max-parallel', help='Set max number of parallel datagroups.', pipe=None, subtree=[
		T_PositiveInteger(help='Set max number of parallel datagroups.', pipe=None, callback=cmd_config_scheduling_group_field, ),
	]),

	T_Keyword('no', help='Unset parameters', subtree=[

		T_Keyword('owner', help='', callback=cmd_config_scheduling_group_owner),

		T_Keyword('min-gap'     , help='Unset min-gap.'                          , pipe=None, callback=cmd_config_scheduling_group_field ),
		T_Keyword('max-deadline', help='Unset deadline.'                         , pipe=None, callback=cmd_config_scheduling_group_field ),
		T_Keyword('max-parallel', help='Unset max number of parallel datagroups.', pipe=None, callback=cmd_config_scheduling_group_field ),
	]),
]

def cmd_schedulinggroup_config_prompt(cli):

	# scheduling-group n
	
	# no scheduling-group n n force

	user         = cli.env['CURRENT_USER']
	tenant_name  = cli.env['CONFIG_TENANT']

	#tenant = tenant_get(tenant_name)
	tenant = Tenant.objects.get(name=tenant_name)

	if cli.canonical_words[0] != 'no':

		name = cli.canonical_words[1]

		# >>>>>>>>>
		# FIX: Spostare nel core
		# >>>>>>>>>

		tenant.db_set_current_schema()
		sg, created = SchedulingGroup.objects.get_or_create(tenant=tenant, name=name)
		
		if created:
			sg.owner = user

			# Impostiamo dei default sensati
			sg.min_gap      = 3
			sg.max_parallel = 1
			sg.save()

			cli.info("Scheduling settings set for the first time (min-gap %s, max-paralleld %s" % (sg.min_gap, sg.max_parallel))

		# <<<<<<<<<<
		# <<<<<<<<<<
		# <<<<<<<<<<

		if user:
			sg.assert_permission_readwrite(user)


		env = cli.env.clone()

		env['PROMPT'] = '%s' % ( PROMPT_SEP.join([ tenant_name, name ]) )
		env['CONFIG_SCHEDULINGGROUP'] = sg.name
		
		cmds = mt_config_schedulinggroup  + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))

		cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
		cli.cmdloop()
	else:
		name  = cli.canonical_words[2]
		name2 = cli.canonical_words[3]
		force = True if len(cli.canonical_words) > 4 and (cli.canonical_words[4] == 'force') else False
		
		if name != name2:
			raise Exception("Name mismatch")

		# >>>>>>>>>
		# FIX: Spostare nel core
		# >>>>>>>>>
		tenant.db_set_current_schema()
		
		sg = SchedulingGroup.objects.get(tenant=tenant, name=name)
		
		sg.check_and_delete(force=force)
		# <<<<<<<<<<
		# <<<<<<<<<<
		# <<<<<<<<<<
