# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from cli.clilib.ioslike import *

from cli.tokens import T_Groupcompleter, T_Usercompleter

from cli.tokens import T_DataGroupTemplateParameter

from cli.tokens import T_DataGroupTemplateDefinedDataSource
from cli.tokens import T_DataGroupTemplateDefinedCondition
from cli.tokens import T_DataGroupTemplateDefinedTimegraph
from cli.tokens import T_DataGroupTemplateDefinedTimegraphSerie

from cli.tokens import T_DataGroupTemplateLocalDefinedDataSource
from cli.tokens import T_DataGroupTemplateLocalDefinedCondition
from cli.tokens import T_DataGroupTemplateLocalDefinedTimegraph
from cli.tokens import T_DataGroupTemplateLocalDefinedTimegraphSerie

from cli.prompts.defs import *
from cli.prompts.defs import CLI_PERM_OPTIONS

from cli.utils import dot_terminated_text

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def multi_line_callback( token, callback ):
	"""
	Questa callback legge da standard input una stringa multilinea
	e al termine invoca la "callback" passando in input un oggetto CLI
	come se l'utente avesse dato il comando:
	
		<token>  <stringa valore>
	"""
	def _read_input_and_real_callback(cli):
		
		# Lettura stinga-multilinea
		text = dot_terminated_text()
		
		# Fingiamo che l'utente abbia dato il comando:
		#
		#       <token>  <stringa valore>
		#
		cli.canonical_words[0] = token
		if len(cli.canonical_words) > 1:
			cli.canonical_words[1] = text
		else:
			cli.canonical_words.append( text )
		
		# Callback vera e propria
		callback(cli)

	return _read_input_and_real_callback

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def CREATE_DATASOURCE_FIELD_COMMANDS(callback, include_deleted=True):

	commands = [
		T_Keyword('title'         , help='Descriptive title'                              , pipe=None, subtree=[ T_Line            (help='Descriptive title'                              , callback=callback, ), ]),
		T_Keyword('expr'          , help='Expression'                                     , pipe=None, subtree=[ T_Line            (help='Expression'                                     , callback=callback, ), ]),
		T_Keyword('order'         , help='Computation order when checking datagroup'      , pipe=None, subtree=[ T_PositiveInteger (help='Computation order when checking datagroup'      , callback=callback, ), ]),
		T_Keyword('save-value'    , help='Store serie value inside DB'                    , pipe=None, subtree=[ T_Boolean         (help='Store serie value inside DB'                    , callback=callback, ), ]),
		T_Keyword('min-val'       , help='Minimum valid value'                            , pipe=None, subtree=[ T_Float           (help='Minimum valid value'                            , callback=callback, ), ]),
		T_Keyword('max-val'       , help='Maximum valid value'                            , pipe=None, subtree=[ T_Float           (help='Maximum valid value'                            , callback=callback, ), ]),
		T_Keyword('gauge'         , help='Treat expression value as a "gauge" value.'     , pipe=None, subtree=[ T_Boolean         (help='Treat expression value as a "gauge" value.'     , callback=callback, ), ]),
		T_Keyword('cascade'       , help='On error do not check the following datasources', pipe=None, subtree=[ T_Boolean         (help='On error do not check the following datasources', callback=callback, ), ]),
		T_Keyword('classification', help='Logical classification'                         , pipe=None, subtree=[ T_Line            (help='Logical classification'                         , callback=callback, ), ]),
		T_Keyword('storage-spec'  , help='Internal parameters for DB storage system'      , pipe=None, subtree=[ T_Line            (help='Internal parameters for DB storage system'      , callback=callback, ), ]),	
	]
	
	if include_deleted:
		commands += [ T_Keyword('deleted'       , help='Set to "true" to ignore this datasource', pipe=None, subtree=[ T_Boolean         (help='', callback=callback, ), ]), ]
	
	no_commands = [
		T_Keyword('title'         , help='', pipe=None, callback=callback, ),
		T_Keyword('expr'          , help='', pipe=None, callback=callback, ),
		T_Keyword('order'         , help='', pipe=None, callback=callback, ),
		T_Keyword('save-value'    , help='', pipe=None, callback=callback, ),
		T_Keyword('min-val'       , help='', pipe=None, callback=callback, ),
		T_Keyword('max-val'       , help='', pipe=None, callback=callback, ),
		T_Keyword('gauge'         , help='', pipe=None, callback=callback, ),
		T_Keyword('cascade'       , help='', pipe=None, callback=callback, ),
		T_Keyword('classification', help='', pipe=None, callback=callback, ),
		T_Keyword('storage-spec'  , help='', pipe=None, callback=callback, ),
	]
	
	if include_deleted:
		no_commands += [ T_Keyword('deleted'       , help='', pipe=None, callback=callback, ), ]
	
	return commands + [
		T_Keyword('no'  , help='Remove parameters', subtree=no_commands),
	]

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def CREATE_CONDITION_FIELD_COMMANDS(callback, include_deleted=True):

	commands = [
		T_Keyword('dependson'     , help='Path of the condition this condition depends on', pipe=None, subtree=[ T_Line            (help='', pipe=None, callback=callback, ), ]),

		T_Keyword('order'         , help='Computation order when checking datagroup'      , pipe=None, subtree=[ T_PositiveInteger (help='Computation order when checking datagroup'      , callback=callback, ), ]),
		T_Keyword('cascade'       , help='On error do not check the following datasources', pipe=None, subtree=[ T_Boolean         (help='On error do not check the following datasources', callback=callback, ), ]),
		
		T_Keyword('title'         , help='Descriptive title', pipe=None, subtree=[ T_Line            (help='', pipe=None, callback=callback, ), ]),
		T_Keyword('primary'       , help='Sets the element\' primary condition', pipe=None, subtree=[ T_Boolean         (help='', pipe=None, callback=callback, ), ]),
		T_Keyword('expr'          , help='Expression', pipe=None, subtree=[ T_Line            (help='', pipe=None, callback=callback, ), ]),
		T_Keyword('max-tries'     , help='Max number of checks before DOWN state.', pipe=None, subtree=[ T_PositiveInteger (help='', pipe=None, callback=callback, ), ]),
		
		T_Keyword('uncheckable-fallback', help='Forced state when the expression is uncheckable', pipe=None, subtree=[ 
			T_Alternatives([
				('UP', 'Up status'), 
				('DN', "Down status")
			], help='', pipe=None, callback=callback, )
		]),
		T_Keyword('statuschange-action' , help='External program/script executed when the status changes', pipe=None, subtree=[ T_Line            (help='', pipe=None, callback=callback, ), ]),
		T_Keyword('priority'            , help='Set priority'                                            , pipe=None, subtree=[ T_PositiveInteger (help='', pipe=None, callback=callback, ), ]),
		T_Keyword('classification'      , help='Logical classification'                                  , pipe=None, subtree=[ T_Line            (help='', pipe=None, callback=callback, ), ]),
		T_Keyword('email'               , help='Default recipient for DOWN (and UP) notifications'       , pipe=None, subtree=[ T_Line            (help='', pipe=None, callback=callback, ), ]),
		T_Keyword('upemail'             , help='Force recipient for UP notifications'                    , pipe=None, subtree=[ T_Line            (help='', pipe=None, callback=callback, ), ]),
		T_Keyword('msg-downsubj'        , help='Subject for DOWN notifications'                          , pipe=None, subtree=[ T_Line            (help='', pipe=None, callback=callback, ), ]),
		T_Keyword('msg-downbody'        , help='Message for DOWN notifications'                          , pipe=None, subtree=[ T_Line            (help='', pipe=None, callback=callback, ), ]),
		T_Keyword('msg-upsubj'          , help='Subject for UP notifications'                            , pipe=None, subtree=[ T_Line            (help='', pipe=None, callback=callback, ), ]),
		T_Keyword('msg-upbody'          , help='Message for UP notifications'                            , pipe=None, subtree=[ T_Line            (help='', pipe=None, callback=callback, ), ]),
		
		# Comandi di comodo che leggono il valore multilinea 
		T_Keyword('msg-upbody-multiline'  , help='Message for UP   notifications (multi-line input)'     , pipe=None, callback=multi_line_callback('msg-upbody'  , callback) ),
		T_Keyword('msg-downbody-multiline', help='Message for DOWN notifications (multi-line input)'     , pipe=None, callback=multi_line_callback('msg-downbody', callback) ),
	]
	
	if include_deleted:
		commands += [ T_Keyword('deleted'             , help='Set to "true" to ignore this condition'    , pipe=None, subtree=[ T_Boolean         (help='', pipe=None, callback=callback, ), ]), ]
	
	no_commands = [
			T_Keyword('dependson'           , help='', pipe=None, callback=callback, ),
			T_Keyword('title'               , help='', pipe=None, callback=callback, ),
			T_Keyword('primary'             , help='', pipe=None, callback=callback, ),
			T_Keyword('expr'                , help='', pipe=None, callback=callback, ),
			T_Keyword('max-tries'           , help='', pipe=None, callback=callback, ),
			T_Keyword('uncheckable-fallback', help='', pipe=None, callback=callback, ),
			T_Keyword('statuschange-action' , help='', pipe=None, callback=callback, ),
			T_Keyword('priority'            , help='', pipe=None, callback=callback, ),
			T_Keyword('classification'      , help='', pipe=None, callback=callback, ),
			T_Keyword('email'               , help='', pipe=None, callback=callback, ),
			T_Keyword('upemail'             , help='', pipe=None, callback=callback, ),
			T_Keyword('msg-downsubj'        , help='', pipe=None, callback=callback, ),
			T_Keyword('msg-downbody'        , help='', pipe=None, callback=callback, ),
			T_Keyword('msg-upsubj'          , help='', pipe=None, callback=callback, ),
			T_Keyword('msg-upbody'          , help='', pipe=None, callback=callback, ),
			
	]

	if include_deleted:
		no_commands += [ T_Keyword('deleted'             , help='', pipe=None, callback=callback, ), ]
		
	return commands + [
		T_Keyword('no', help='Remove parameters', subtree=no_commands),
	]

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def CREATE_TIMEGRAPH_FIELD_COMMANDS(field_callback, 
                                    serie_callback, 
                                    no_serie_callback       = None, 
                                    
                                    include_no_commands     = True, 
                                    #include_rename_commands = True,
                                    tok_serie_completer     = T_DataGroupTemplateDefinedTimegraphSerie,
                                    tok_no_serie_completer  = T_DataGroupTemplateLocalDefinedTimegraphSerie,
                                    include_deleted = True
):

	cmds = [
		T_Keyword('title'         , help='Descriptive title', pipe=None, subtree=[ T_Line   (help='', pipe=None, callback=field_callback, ), ]),
		T_Keyword('primary'       , help='Set element\'s primary timegraph', pipe=None, subtree=[ T_Boolean(help='', pipe=None, callback=field_callback, ), ]),
		T_Keyword('stacked'       , help='Timegraph\'s series can be stacked.', pipe=None, subtree=[ T_Boolean(help='', pipe=None, callback=field_callback, ), ]),
		T_Keyword('ylegend'       , help='Y-axis legend', pipe=None, subtree=[ T_Line   (help='', pipe=None, callback=field_callback, ), ]),
		T_Keyword('classification', help='Logic classification', pipe=None, subtree=[ T_Line   (help='', pipe=None, callback=field_callback, ), ]),
		
		T_Keyword('serie'  , help='Define a serie', pipe=None, subtree=[
			tok_serie_completer(help='name', pipe=None, callback=serie_callback),
			T_ValidWord(VALID_SERIE_NAME, help='name', pipe=None, callback=serie_callback)
		]),
	]
	if include_deleted:
		cmds += [  T_Keyword('deleted'       , help='Set to "true" to ignore this timegraph', pipe=None, subtree=[ T_Boolean(help='', pipe=None, callback=field_callback, ), ]),  ]
	
	no_cmds = [
		T_Keyword('title'  , help='', pipe=None, callback=field_callback, ),
		T_Keyword('primary', help='', pipe=None, callback=field_callback, ),
		T_Keyword('stacked', help='', pipe=None, callback=field_callback, ),
		T_Keyword('ylegend', help='', pipe=None, callback=field_callback, ),
	]
	if include_deleted:
		no_cmds += [ T_Keyword('deleted', help='', pipe=None, callback=field_callback, ), ]
	
	if include_no_commands:
		no_cmds += [
			T_Keyword('serie', help='Remove a serie.', pipe=None, subtree=[
				tok_no_serie_completer(help='name' , pipe=None, subtree=[
					tok_no_serie_completer(help='name', pipe=None, callback=no_serie_callback)
				]),
			])
		]
		
	cmds += [ 
		T_Keyword('no', help='Remove parameters/series', pipe=None, subtree=no_cmds )
	]
	
	return cmds

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def CREATE_TIMEGRAPH_SERIE_FIELD_COMMANDS(field_callback, include_deleted=True):

	commands = [
		T_Keyword('legend'          , help='Legend of the serie', pipe=None, subtree=[ T_Line   (help='', pipe=None, callback=field_callback, ), ]),
		T_Keyword('color'           , help='RGB Color (#rrggbb)', pipe=None, subtree=[ T_Line   (help='', pipe=None, callback=field_callback, ), ]),
		T_Keyword('render-order'    , help='Rendering order inside the graph', pipe=None, subtree=[ T_Number (help='', pipe=None, callback=field_callback, ), ]),
		T_Keyword('var-value'       , help='Name of the datasource related to this serie', pipe=None, subtree=[ T_Line   (help='', pipe=None, callback=field_callback, ), ]),
		T_Keyword('computed-value'  , help='Value\'s expression', pipe=None, subtree=[ T_Line   (help='', pipe=None, callback=field_callback, ), ]),
		T_Keyword('style'           , help='Rendering style (line, area, ecc.)', pipe=None, subtree=[ 
			T_Alternatives(
				CLI_TG_STYLE_TO_NAME_DESCR.values()
				,
				help='style',
				pipe=None,
				callback=field_callback,
			),
		]),
	]
	
	if include_deleted == True:
		commands += [ T_Keyword('deleted'         , help='Disable this serie', pipe=None, subtree=[ T_Boolean(help='', pipe=None, callback=field_callback, ), ]), ]
	
	no_commands = [
			T_Keyword('legend'          , help='', pipe=None, callback=field_callback, ),
			T_Keyword('color'           , help='', pipe=None, callback=field_callback, ),
			T_Keyword('render-order'    , help='', pipe=None, callback=field_callback, ),
			T_Keyword('var-value'       , help='', pipe=None, callback=field_callback, ),
			T_Keyword('computed-value'  , help='', pipe=None, callback=field_callback, ),
			T_Keyword('style'           , help='', pipe=None, callback=field_callback, ),
	]
	if include_deleted == True:
		no_commands += [  T_Keyword('deleted'         , help='', pipe=None, callback=field_callback, ), ]
	
	return commands + [
		T_Keyword('no'              , help='Remove parameters', pipe=None, subtree=no_commands )
	]

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def CREATE_DATAGROUP_COMMANSDS(xcmd_config_datagroup_template_field,
                               xcmd_config_datagroup_template_parameter,

                               xcmd_datasource_template_config_prompt,

                               xcmd_condition_template_config_prompt,

                               xcmd_timegraph_template_config_prompt,

                               tok_datasource_completer = T_DataGroupTemplateDefinedDataSource,
                               tok_condition_completer  = T_DataGroupTemplateDefinedCondition,
                               tok_timegraph_completer  = T_DataGroupTemplateDefinedTimegraph,
                               tok_param_completer      = T_DataGroupTemplateParameter,

                               tok_no_datasource_completer = T_DataGroupTemplateLocalDefinedDataSource,
                               tok_no_condition_completer  = T_DataGroupTemplateLocalDefinedCondition,
                               tok_no_timegraph_completer  = T_DataGroupTemplateLocalDefinedTimegraph,
                               tok_no_param_completer      = T_DataGroupTemplateParameter,

                               include_no_commands     = True,
                               include_no_fields       = True,
                               include_rename_commands = True, 
                               include_deleted         = True,
                               
                               include_group_commands  = False,
                               xcmd_config_datagroup_template_group_access=None,
                               
                               include_owner_commands  = False,
                               xcmd_config_datagroup_template_owner=None,
                               
                               include_scheduling_commands = False,

                               template_definition = False,

):

	r = [
		T_Keyword('title', help='Descriptive title', pipe=None, subtree=[
			T_Line(help='', pipe=None, callback=xcmd_config_datagroup_template_field, ),
		]),
		
		T_Keyword('dependson', help='The condition this datagroup depends on',  pipe=None,subtree=[
			T_Line(help='', pipe=None, callback=xcmd_config_datagroup_template_field, ),
		]),
		
		T_Keyword('minperiod', help='Rescheduling interval in seconds', pipe=None, subtree=[
			T_PositiveInteger(help='', pipe=None, callback=xcmd_config_datagroup_template_field, ),
		]),
		T_Keyword('shorttries', help='Defaul number of retries when performing any network operation', pipe=None, subtree=[
			T_PositiveInteger(help='', pipe=None, callback=xcmd_config_datagroup_template_field, ),
		]),
		T_Keyword('timeout', help='Default timeout for network operations.', pipe=None, subtree=[
			T_PositiveInteger(help='', pipe=None, callback=xcmd_config_datagroup_template_field, ),
		]),
		T_Keyword('classification', help='Logical classification', pipe=None, subtree=[
			T_Line(help='', pipe=None, callback=xcmd_config_datagroup_template_field, ),
		]),
		
		T_Keyword('passive-match', help='Matching string for passive checks', pipe=None, subtree=[
			T_Line(help='', pipe=None, callback=xcmd_config_datagroup_template_field, ),
		]),
		
		T_Keyword('parameter', help='Define a new expression parameter', pipe=None, subtree=[
		
			tok_param_completer(help='name', pipe=None, subtree=[
				T_Alternatives(
					CLI_PTYPE_TO_NAME_DESCR.values()
					,
					help='type', pipe=None,
					subtree=[
						T_Line(help='value', pipe=None, callback=xcmd_config_datagroup_template_parameter),
					]
				),
			]),
			T_ValidWord(VALID_PARAM_NAME, help='name', pipe=None, subtree=[
				T_Alternatives(
					CLI_PTYPE_TO_NAME_DESCR.values()
					,
					help='type', pipe=None,
					subtree=[
						T_Line(help='value', pipe=None, callback=xcmd_config_datagroup_template_parameter),
					]
				),
			]),
		
		]),
		
		BLANK_ENTRY,
	]
	
	r += [
		T_Keyword('suspended', help='Set datagroup as suspended', pipe=None, subtree=[
			T_Boolean(help='', pipe=None, callback=xcmd_config_datagroup_template_field, ),
		]),

		T_Keyword('suspend-time', help='Set suspend start time', pipe=None, subtree=[
			T_PositiveInteger(help='', pipe=None, callback=xcmd_config_datagroup_template_field, ),
		]),

		T_Keyword('suspend-resume-time', help='Suspend time', pipe=None, subtree=[
			T_PositiveInteger(help='', pipe=None, callback=xcmd_config_datagroup_template_field, ),
		]),		
		T_Keyword('suspend-reason', help='Suspend reason', pipe=None, subtree=[
			T_Line(help='', pipe=None, callback=xcmd_config_datagroup_template_field, ),
		]),

	]

	if template_definition:
		r += [
			T_Keyword('datasource', help='Define a datasource', pipe=None, subtree=[
				tok_datasource_completer(callback=xcmd_datasource_template_config_prompt),
				T_ValidWord(VALID_DATASOURCE_NAME, help='name', pipe=None, callback=xcmd_datasource_template_config_prompt),
			]),

			T_Keyword('condition' , help='Define a condition', pipe=None, subtree=[
				tok_condition_completer(callback=xcmd_condition_template_config_prompt),
				T_ValidWord(VALID_CONDITION_NAME, help='name', pipe=None, callback=xcmd_condition_template_config_prompt),
			]),

			T_Keyword('timegraph' , help='Define a timegraph', pipe=None, subtree=[
				tok_timegraph_completer(callback=xcmd_timegraph_template_config_prompt),
				T_ValidWord(VALID_TIMEGRAPH_NAME, help='name', pipe=None, callback=xcmd_timegraph_template_config_prompt),
			]),
		]

	else:
		r += [
			T_Keyword('datasource', help='Define a datasource', pipe=None, subtree=[
				tok_datasource_completer(callback=xcmd_datasource_template_config_prompt),
			]),

			T_Keyword('condition' , help='Define a condition', pipe=None, subtree=[
				tok_condition_completer(callback=xcmd_condition_template_config_prompt),
			]),

			T_Keyword('timegraph' , help='Define a timegraph', pipe=None, subtree=[
				tok_timegraph_completer(callback=xcmd_timegraph_template_config_prompt),
			]),
		]

	if include_scheduling_commands:
		r += [
			T_Keyword('scheduling-group', help='Assign datagroup a ascheduling group', pipe=None, subtree=[
				T_Line(help='', pipe=None, callback=xcmd_config_datagroup_template_field, ),
			]),
		]
	
	if include_owner_commands and xcmd_config_datagroup_template_owner:
		
		r += [ 
			T_Keyword('owner', help='', subtree=[
				T_Usercompleter(help='', callback=xcmd_config_datagroup_template_owner),
			])
		]
	
	if include_group_commands and xcmd_config_datagroup_template_group_access:
		
		r += [ 
			T_Keyword('group', help='', subtree=[
				T_Groupcompleter(help='', callback=xcmd_config_datagroup_template_group_access, subtree=[ 
					T_Alternatives( CLI_PERM_OPTIONS, callback=xcmd_config_datagroup_template_group_access )
				]),
			])
		]
	
	if include_deleted:
		r += [ 
			T_Keyword('deleted', help='Disable this datagroup', pipe=None, subtree=[
				T_Boolean(help='', pipe=None, callback=xcmd_config_datagroup_template_field, ),
			]),
		]
	
	#if include_rename_commands:
	#	r += [
	#		T_Keyword('rename', help='', pipe=None, subtree=[
	#			T_Keyword('parameter', help='', pipe=None, subtree=[
	#				tok_param_completer(help='name', pipe=None, subtree=[
	#					T_ValidWord(VALID_PARAM_NAME, help='name', pipe=None, callback=xcmd_datasource_template_config_prompt)
	#				])
	#			]),
	#			T_Keyword('datasource', help='', pipe=None, subtree=[
	#				tok_datasource_completer(help='', subtree=[
	#					T_ValidWord(VALID_DATASOURCE_NAME, help='name', pipe=None, callback=xcmd_datasource_template_config_prompt),
	#				])
	#			]), 
	#			T_Keyword('condition' , help='', pipe=None, subtree=[
	#				tok_condition_completer(help='', subtree=[
	#					T_ValidWord(VALID_CONDITION_NAME, help='name', pipe=None, callback=xcmd_condition_template_config_prompt),
	#				])
	#			]), 
	#			T_Keyword('timegraph' , help='', pipe=None, subtree=[
	#				tok_timegraph_completer(help='', subtree=[
	#					T_ValidWord(VALID_TIMEGRAPH_NAME, help='name', pipe=None, callback=xcmd_timegraph_template_config_prompt),
	#				])
	#			]),
	#		])
	#	]
	
	no_commands = [
		T_Keyword('title'         , help='', pipe=None, callback=xcmd_config_datagroup_template_field, ),
		T_Keyword('dependson'     , help='', pipe=None, callback=xcmd_config_datagroup_template_field, ),
		T_Keyword('minperiod'     , help='', pipe=None, callback=xcmd_config_datagroup_template_field, ),
		T_Keyword('shorttries'    , help='', pipe=None, callback=xcmd_config_datagroup_template_field, ),
		T_Keyword('timeout'       , help='', pipe=None, callback=xcmd_config_datagroup_template_field, ),
		T_Keyword('classification', help='', pipe=None, callback=xcmd_config_datagroup_template_field, ),
		T_Keyword('passive-match' , help='', pipe=None, callback=xcmd_config_datagroup_template_field, ),
	
		T_Keyword('parameter'     , help='', pipe=None, subtree=[
			tok_no_param_completer(help='', pipe=None, subtree=[
				tok_no_param_completer(callback=xcmd_config_datagroup_template_parameter),
			]),
		]), 
	]
	
	if include_scheduling_commands:	
		no_commands += [
			T_Keyword('scheduling-group'       , help='Unlink datagroup from a scheduling group', pipe=None, callback=xcmd_config_datagroup_template_field, ),
		]
	
	if include_owner_commands and xcmd_config_datagroup_template_owner:
		no_commands += [
			T_Keyword('owner', help='', callback=xcmd_config_datagroup_template_owner),
		]
	
	if include_group_commands and xcmd_config_datagroup_template_group_access:
		no_commands += [
			T_Keyword('group', help='', pipe=None, subtree=[
				T_Groupcompleter(help='', callback=xcmd_config_datagroup_template_group_access),
			])
		]
	
	if include_deleted:
		no_commands += [ T_Keyword('deleted'       , help='', pipe=None, callback=xcmd_config_datagroup_template_field, ), ]
	
	if include_no_commands:
		no_commands += [
			
			T_Keyword('datasource', help='Remove datagroup configuration', pipe=None, subtree=[
				tok_no_datasource_completer(help='', pipe=None, subtree=[
					tok_no_datasource_completer(callback=xcmd_datasource_template_config_prompt),
				])
			]), 
			
			T_Keyword('condition' , help='Remove datagroup configuration', pipe=None, subtree=[
				tok_no_condition_completer(help='', pipe=None, subtree=[
					tok_no_condition_completer(callback=xcmd_condition_template_config_prompt),
				])
			]), 
			
			T_Keyword('timegraph' , help='Remove datagroup configuration', pipe=None, subtree=[
				tok_no_timegraph_completer(help='', pipe=None, subtree=[
					tok_no_timegraph_completer(callback=xcmd_timegraph_template_config_prompt),
				])
			]), 
		]
	
	if no_commands:
		r += [
			T_Keyword('no', help='Remove configuration (fields/parameters/conditions/datagroups/timegraphs)', pipe=None, subtree=no_commands)
		]
	
	return r

