# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from core.models import SOURCE_LIBRARY, SOURCE_LOCAL

from core.models import *
from core.api.base import *

from core.api.templates import datagroup_template_move_to_library

from core.api.templates import datagroup_template_get
from core.api.templates import datagroup_template_create
from core.api.templates import datagroup_template_delete

from core.api.templates import datagroup_template_group_permission

from core.api.templates import datagroup_template_description_update
from core.api.templates import datagroup_library_template_duplicate_on_tenant
from core.api.templates import datagroup_template_set_parent
from core.api.templates import datagroup_template_set_field
from core.api.templates import datagroup_template_unset_field

from core.api.templates import datagroup_template_update_parameter
from core.api.templates import datagroup_template_remove_parameter
from core.api.templates import datagroup_template_rename_parameter

from core.api.templates import datasource_template_get
from core.api.templates import datasource_template_create
from core.api.templates import datasource_template_remove
from core.api.templates import datasource_template_rename
from core.api.templates import datasource_template_unset_field
from core.api.templates import datasource_template_update_field

from core.api.templates import condition_template_get
from core.api.templates import condition_template_create
from core.api.templates import condition_template_remove
from core.api.templates import condition_template_rename
from core.api.templates import condition_template_update_field
from core.api.templates import condition_template_unset_field

from core.api.templates import timegraph_template_create
from core.api.templates import timegraph_template_remove
from core.api.templates import timegraph_template_rename
from core.api.templates import timegraph_template_field_update
from core.api.templates import timegraph_template_field_unset

from core.api.templates import timegraph_serie_create
from core.api.templates import timegraph_serie_remove
from core.api.templates import timegraph_serie_template_rename
from core.api.templates import timegraph_serie_template_update_field
from core.api.templates import timegraph_serie_template_unset_field


from cli.clilib.ioslike import *
from cli.prompts.defs import *

from cli.clilib.ioslike import SORTED_KEYWORDS

from cli.tokens import T_DataGroupTemplate, T_AllDataGroupTemplate

from cli.prompts.defs import *

from cli.prompts import cli_get_registered_menus

from cli.prompts.config.templates.base import *

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def PROMPT_STRING(*args):
	args = [ a for a in args if a ]
	return PROMPT_SEP.join(args)

#------------------------------------------------------------------------------#
# DATASOURCE                                                                   #
#------------------------------------------------------------------------------#

def cmd_config_datasource_template_field(cli):

	env             = cli.env

	user            = env['CURRENT_USER']
	source          = env['SOURCE']

	# FIX: ASSERT
	tenant = None
	if source == SOURCE_LOCAL:
		tenant_name = env['CONFIG_TENANT']
		tenant = tenant_get(tenant_name)

	datagroup_name  = env['CONFIG_DATAGROUP_TEMPLATE']
	datasource_name = env['CONFIG_DATASOURCE_TEMPLATE']

	# <field> <value>
	# no <field>
	if cli.canonical_words[0] != 'no':

		field_name  = cli.canonical_words[0]
		field_name  = CLI_FIELD_TO_DATASOURCE_FIELD(field_name)

		field_value = cli.canonical_words[1]

		datasource_template_update_field(
			datagroup_name, 
			datasource_name,
			field_name,
			field_value, 
			source=source,
			tenant=tenant,
			user=user,
		)
	else:
		field_name  = cli.canonical_words[1]
		field_name  = CLI_FIELD_TO_DATASOURCE_FIELD(field_name)

		datasource_template_unset_field(
			datagroup_name, 
			datasource_name,
			field_name,
			source=source,
			tenant=tenant,
			user=user,
		)


mt_config_datasource = CREATE_DATASOURCE_FIELD_COMMANDS(cmd_config_datasource_template_field)


def cmd_datasource_template_config_prompt(cli):

	env = cli.env

	user        = env['CURRENT_USER']
	source      = env['SOURCE']

	# FIX: ASSERT
	tenant = None
	if source == SOURCE_LOCAL:
		tenant_name = env['CONFIG_TENANT']
		tenant = tenant_get(tenant_name)

	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']

	# datasource <n>
	# no datasource <n>
	if cli.canonical_words[0] != 'no':

		name = cli.canonical_words[1]

		ds = datasource_template_get(datagroup_template_name, name, source=source, tenant=tenant)
		if ds == None:
			ds = datasource_template_create(datagroup_template_name, name, source=source, tenant=tenant, user=user)
			cli.info("Creating datasource %s in template %s" % (name, datagroup_template_name))

		env = cli.env.clone()

		env['PROMPT'] = '%s' % (PROMPT_STRING(datagroup_template_name, name),)

		env['CONFIG_DATASOURCE_TEMPLATE'] = name

		cmds = SORTED_KEYWORDS(mt_config_datasource) + [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))
		cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
		cli.cmdloop()

	else:
		name = cli.canonical_words[2]

		datasource_template_remove(datagroup_template_name, name, source=source, tenant=tenant, user=user)

#------------------------------------------------------------------------------#
# CONDITION                                                                    #
#------------------------------------------------------------------------------#

def cmd_config_condition_template_field(cli):

	env  = cli.env

	user            = env['CURRENT_USER']
	source          = env['SOURCE']

	# FIX: ASSERT
	tenant = None
	if source == SOURCE_LOCAL:
		tenant_name = env['CONFIG_TENANT']
		tenant = tenant_get(tenant_name)

	datagroup_name = env['CONFIG_DATAGROUP_TEMPLATE']
	condition_name = env['CONFIG_CONDITION_TEMPLATE']

	# <field> <value>
	# no field <field>
	if cli.canonical_words[0] != 'no':
		field_name  = cli.canonical_words[0]
		field_name  = CLI_FIELD_TO_CONDITION_FIELD(field_name)

		field_value = cli.canonical_words[1]
		field_value = CLI_FIELD_VALUE_TO_CONDITION_FIELD_VALUE(field_name, field_value)

		condition_template_update_field(
			datagroup_name, 
			condition_name,
			field_name, 
			field_value, 
			source=source,
			tenant=tenant,
			user=user,
		)
	else:
		field_name  = cli.canonical_words[1]
		field_name  = CLI_FIELD_TO_CONDITION_FIELD(field_name)

		condition_template_unset_field(
			datagroup_name, 
			condition_name,
			field_name, 
			source=source,
			tenant=tenant,
			user=user,
		)

	return


mt_config_condition_template = CREATE_CONDITION_FIELD_COMMANDS( cmd_config_condition_template_field ) 



def cmd_condition_template_config_prompt(cli):

	env  = cli.env

	user   = env['CURRENT_USER']
	source = env['SOURCE']

	# FIX: ASSERT
	tenant = None
	if source == SOURCE_LOCAL:
		tenant_name = env['CONFIG_TENANT']
		tenant = tenant_get(tenant_name)

	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']

	# condition <c>
	# no condition <c>
	if cli.canonical_words[0] != 'no':

		name = cli.canonical_words[1]

		c = condition_template_get(datagroup_template_name, name, source=source, tenant=tenant)
		if c == None:
			cli.info("Creating condition template %s" % (name))
			c = condition_template_create(datagroup_template_name, name, source=source, tenant=tenant, user=user)

		env = cli.env.clone()

		env['PROMPT'] = '%s' % (PROMPT_STRING(datagroup_template_name, name),)

		env['CONFIG_CONDITION_TEMPLATE'] = name

		cmds = SORTED_KEYWORDS(mt_config_condition_template) + [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))
		cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
		cli.cmdloop()
	else:
		name = cli.canonical_words[2]

		condition_template_remove(datagroup_template_name, name, source=source, tenant=tenant, user=user)

#------------------------------------------------------------------------------#
# TIMEGRAPH SERIE                                                              #
#------------------------------------------------------------------------------#

def cmd_config_timegraph_serie_field_template(cli):

	env            = cli.env

	user           = env['CURRENT_USER']
	source         = env['SOURCE']

	tenant = None
	if source == SOURCE_LOCAL:
		tenant_name = env['CONFIG_TENANT']
		tenant = tenant_get(tenant_name)

	datagroup_name = env['CONFIG_DATAGROUP_TEMPLATE']
	timegraph_name = env['CONFIG_TIMEGRAPH']
	serie_name     = env['CONFIG_TIMEGRAPH_SERIE']

	if cli.canonical_words[0] != 'no':
		# <field> <value>
		field_name  = cli.canonical_words[0]
		field_name  = CLI_FIELD_TO_TIMEGRAPHSERIE_FIELD(field_name)

		field_value = cli.canonical_words[1]

		timegraph_serie_template_update_field(
			datagroup_name, 
			timegraph_name,
			serie_name,
			field_name, 
			field_value, 
			source=source,
			tenant=tenant,
			user=user
		)
	else:
		field_name  = cli.canonical_words[1]
		field_name  = CLI_FIELD_TO_TIMEGRAPHSERIE_FIELD(field_name)

		timegraph_serie_template_unset_field(
			datagroup_name, 
			timegraph_name,
			serie_name,
			field_name, 
			source=source,
			tenant=tenant,
			user=user
		)

	return
	

mt_config_timegraph_serie_template = CREATE_TIMEGRAPH_SERIE_FIELD_COMMANDS(cmd_config_timegraph_serie_field_template)

def cmd_config_timegraph_serie_template(cli):

	env            = cli.env

	user           = env['CURRENT_USER']
	
	tenant_name    = env['CURRENT_TENANT']
	datagroup_name = env['CONFIG_DATAGROUP_TEMPLATE']
	timegraph_name = env['CONFIG_TIMEGRAPH']
	source         = env['SOURCE']

	name = cli.canonical_words[1]

	tenant = None
	if source == SOURCE_LOCAL:
		tenant = tenant_get(tenant_name)
		db_set_current_tenant_schema(tenant.name)

	#
	# CREATE
	#
	ttgs = timegraph_serie_create(
		datagroup_name,
		timegraph_name,
		name,
		source=source,
		tenant=tenant,
		user=user,
	)

	#
	# New prompt
	#
	env = cli.env.clone()

	env['PROMPT'] = '%s' % (PROMPT_STRING(tenant_name, datagroup_name, timegraph_name, name),)
	
	env['CONFIG_TIMEGRAPH_SERIE'] = name

	cmds = SORTED_KEYWORDS(mt_config_timegraph_serie_template) + [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))
	cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	cli.cmdloop()

#------------------------------------------------------------------------------#

def cmd_config_timegraph_serie_template_remove(cli):

	# no serie <name> <name>
	env            = cli.env

	user           = env['CURRENT_USER']

	source         = env['SOURCE']
	#tenant_name    = env['CURRENT_TENANT']
	tenant = None
	if source == SOURCE_LOCAL:
		tenant_name = env['CONFIG_TENANT']
		tenant = tenant_get(tenant_name)
	
	datagroup_name = env['CONFIG_DATAGROUP_TEMPLATE']
	timegraph_name = env['CONFIG_TIMEGRAPH']

	name = cli.canonical_words[2]
	name2 = cli.canonical_words[3]
	if name != name2:
		raise Exception("different names") # FIX: same
	
	
	timegraph_serie_remove(
		datagroup_name,
		timegraph_name,
		name,
		source,
		tenant,
		user=user
	)

#------------------------------------------------------------------------------#
# TIMEGRAPH                                                                    #
#------------------------------------------------------------------------------#

def cmd_config_timegraph_template_field(cli):

	env            = cli.env

	user           = env['CURRENT_USER']
	source         = env['SOURCE']

	tenant = None
	if source == SOURCE_LOCAL:
		tenant_name = env['CONFIG_TENANT']
		tenant = tenant_get(tenant_name)

	datagroup_name = env['CONFIG_DATAGROUP_TEMPLATE']
	timegraph_name = env['CONFIG_TIMEGRAPH']

	if cli.canonical_words[0] != 'no':
		# <field> <value>
		field_name  = cli.canonical_words[0]
		field_name  = CLI_FIELD_TO_TIMEGRAPH_FIELD(field_name)	

		field_value = cli.canonical_words[1]

		timegraph_template_field_update(
			datagroup_name,
			timegraph_name,
			field_name,
			field_value,
			tenant,
			source,
			user=user,
		)
	else:
		# no <field> <value>
		field_name  = cli.canonical_words[1]
		field_name  = CLI_FIELD_TO_TIMEGRAPH_FIELD(field_name)	

		timegraph_template_field_unset(
			datagroup_name,
			timegraph_name,
			field_name,
			tenant,
			source,
			user=user,
		)

	return

def cmd_timegraphserie_template_rename(cli):
	# rename serie <name> <name>
	
	env            = cli.env

	user           = env['CURRENT_USER']
	source         = env['SOURCE']

	tenant = None
	tenant_name = None
	if source == SOURCE_LOCAL:
		tenant_name = env['CONFIG_TENANT']
		tenant = tenant_get(tenant_name)
		db_set_current_tenant_schema(tenant_name)

	datagroup_name = env['CONFIG_DATAGROUP_TEMPLATE']
	timegraph_name = env['CONFIG_TIMEGRAPH']

	name     = cli.canonical_words[2]
	new_name = cli.canonical_words[3]

	tdg = TDataGroup.objects.get(tenant=tenant, name=datagroup_name, source=source)
	
	timegraph_serie_template_rename(tenant, tdg, timegraph_name, name, new_name, user=user)

	return


mt_config_timegraph_template = CREATE_TIMEGRAPH_FIELD_COMMANDS(
	cmd_config_timegraph_template_field,
	cmd_config_timegraph_serie_template,
	cmd_config_timegraph_serie_template_remove,
)

mt_config_timegraph_template += [
	T_Keyword('rename'  , help='', pipe=None, subtree=[
		T_Keyword('serie'  , help='', pipe=None, subtree=[
			T_DataGroupTemplateDefinedTimegraphSerie(help='name', pipe=None, subtree=[
				T_ValidWord(VALID_SERIE_NAME, help='name', pipe=None, callback=cmd_timegraphserie_template_rename)
			])
		]),
	]),
]


def cmd_timegraph_template_config_prompt(cli):

	env            = cli.env

	user           = env['CURRENT_USER']
	source         = env['SOURCE']

	tenant = None
	tenant_name = None
	if source == SOURCE_LOCAL:
		tenant_name = env['CONFIG_TENANT']
		tenant = tenant_get(tenant_name)
		db_set_current_tenant_schema(tenant_name)

	datagroup_name = env['CONFIG_DATAGROUP_TEMPLATE']


	# datasource <n>
	# no datasource <n>
	if cli.canonical_words[0] != 'no':


		name = cli.canonical_words[1]

		tdg = TDataGroup.objects.get(tenant=tenant, name=datagroup_name, source=source)

		ttg = timegraph_template_create(tenant, tdg, name, user=user)

		#
		# FIX:
		#

		env = cli.env.clone()

		env['PROMPT'] = '%s' % (PROMPT_STRING(tenant_name, datagroup_name, name),)
		
		env['CONFIG_TIMEGRAPH'] = name

		cmds = SORTED_KEYWORDS(mt_config_timegraph_template) + [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))
		cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
		cli.cmdloop()
	else:
		name = cli.canonical_words[2]
		
		timegraph_template_remove(datagroup_name, name, source=source, tenant=tenant)
	

#------------------------------------------------------------------------------#
# DATAGROUP                                                                    #
#------------------------------------------------------------------------------#

SOURCE_DESCR_TUPLE = [ (a['name'], a['descr']) for a in SOURCE_DESCR.values() ]	

SOURCE_NAME_TO_CODE = dict( [ (d['name'],c) for c,d in SOURCE_DESCR.items() ] )


def cmd_config_datagroup_template_field(cli):

	env  = cli.env

	user        = env['CURRENT_USER']
	source      = env['SOURCE']

	# FIX: ASSERT
	tenant_name = env['CONFIG_TENANT']
	tenant = None
	if source == SOURCE_LOCAL:
		if tenant_name != None:
			tenant = tenant_get(tenant_name)

	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']

	# <field> <value>
	# no field 
	if cli.canonical_words[0] != 'no':

		field_name  = cli.canonical_words[0]
		field_name  = CLI_FIELD_TO_DATAGROUP_FIELD(field_name)

		field_value = cli.canonical_words[1]

		datagroup_template_set_field(
			datagroup_template_name, 
			field_name, 
			field_value, 
			source=source,
			tenant=tenant,
			user=user,
		)
	else:

		field_name  = cli.canonical_words[1]
		field_name  = CLI_FIELD_TO_DATAGROUP_FIELD(field_name)

		datagroup_template_unset_field(
			datagroup_template_name, 
			field_name, 
			source=source,
			tenant=tenant,
			user=user,
		)

	return

def cmd_config_datagroup_template_parameter(cli):

	env  = cli.env

	user   = env['CURRENT_USER']
	source = env['SOURCE']

	tenant = None
	if source == SOURCE_LOCAL:
		tenant_name = env['CONFIG_TENANT']
		tenant = tenant_get(tenant_name)

	# parameter <name> <type> <val>
	# no parameter <name>
	if cli.canonical_words[0] != 'no':

		datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']

		# parameter <name> <type> <value>
		param_name      = cli.canonical_words[1]
		param_type_name = cli.canonical_words[2]
		param_value     = cli.canonical_words[3]

		param_value     = cli_input_to_real_param_value(param_type_name, param_value)

		# Validate param value. This will raise an exception
		ptype, pvalue = convert_param_value(param_type_name, param_value)

		datagroup_template_update_parameter(
			datagroup_template_name,
			param_name,
			ptype,
			param_value,
			source=source,
			tenant=tenant,
			user=user,
		)
	else:
		datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']

		# parameter <name> <type> <value>
		param_name      = cli.canonical_words[2]

		datagroup_template_remove_parameter(
			datagroup_template_name,
			param_name,
			source=source,
			tenant=tenant,
			user=user,
		)

def cmd_config_datagroup_template_parent(cli):

	env  = cli.env

	user   = env['CURRENT_USER']

	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']

	source = env['SOURCE']

	tenant = None
	if source == SOURCE_LOCAL:
		tenant_name = env['CONFIG_TENANT']
		tenant = tenant_get(tenant_name)

	# parent <datagroup>
	# parent <datagroup> library

	parent_name  = cli.canonical_words[1]

	forced_source = source
	if len(cli.canonical_words) > 2:
		if cli.canonical_words[2] == 'library':  #FIX: use constant
			forced_source = SOURCE_LIBRARY

	cli.info("Setting '%s'(%s) as parent of %s (%s)" % (parent_name, SOURCE_NAME(forced_source), datagroup_template_name, SOURCE_NAME(source) ) )

	tdg = datagroup_template_get(datagroup_template_name, source=source, tenant=tenant)

	if parent_name == 'none':
		datagroup_template_set_parent(tdg, None, user=user)
	else:
		if tdg.parent != None:

			# Non posso cambiare la gerarchia di un datagroup-template "clone" di uno in library
			if tdg.name == tdg.parent.name and tdg.source == SOURCE_LOCAL and tdg.parent.source == SOURCE_LIBRARY:

				if tdg.parent.name != parent_name:
					cli.error("Can not change parent of datagroup template '%s' inside tenant because it was 'cloned' from library template '%s'." % (tdg.name, tdg.parent.name))
					return
				else:
					cli.info("Cloned datagroup template '%s' from '%s'" % (tdg.name, tdg.parent.name))

			# Non sto effettuando nessun cambiamento. Usciamo.
			if tdg.parent.name == parent_name:
				cli.info("Parent not changed\n")
				return

		if source == SOURCE_LIBRARY:
			tdg_parent = datagroup_template_get(parent_name, source=source)
			if tdg_parent == None:
				cli.error("Parent datagroup template '%s' not found in library." % (parent_name))
				return
		else:
			# Sono nel sito, vediamo se c'e' un datagroup definito nel sito
			tdg_parent = datagroup_template_get(parent_name, source=source, tenant=tenant)
			if tdg_parent == None:
				# Forse intendevo usare uno dalla library?
				tdg_parent = datagroup_template_get(parent_name, source=SOURCE_LIBRARY, tenant=None)
			else:
				# WORKAROUND: Questo caso non dovrebbe esistere mai. 
				# a causa di un bug forse in giro ci sono gerarchie strane per cui un template "clonato"
				# non e' figlio di un template della library con lo stesso nome.
				if tdg.name == tdg_parent.name:
					# Forse intendevo usare uno dalla library?
					tdg_parent = datagroup_template_get(parent_name, source=SOURCE_LIBRARY, tenant=None)

	if tdg_parent == tdg:
		cli.error("Can not set parent of itself")
		return

	datagroup_template_set_parent(tdg, tdg_parent, user=user)
	
	return

def cmd_config_datagroup_template_description(cli):

	env  = cli.env

	user   = env['CURRENT_USER']

	source = env['SOURCE']
	

	tenant = None
	if source == SOURCE_LOCAL:
		tenant_name = env['CONFIG_TENANT']
		tenant = tenant_get(tenant_name)

	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']

	description = cli.canonical_words[1]

	datagroup_template_description_update(datagroup_template_name, description, source, tenant, user=user)

def cmd_config_datagroup_template_source(cli):

	env  = cli.env

	source          = env['SOURCE']

	tenant = None
	tenant_name = None
	if source == SOURCE_LOCAL:
		tenant_name = env['CONFIG_TENANT']
		tenant = tenant_get(tenant_name)

	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']

	new_source = SOURCE_NAME_TO_CODE[ cli.canonical_words[1] ]

	#
	# VOLENDO POTREMMO PERMETTERE DI SPOSTARE UN TEMPLATE DAL SITO
	# ALLA LIBRARY, MA DOVREMMO GESTIRE LA "CLONAZIONE" DEL TEMPLATE
	# TRA LIBRARY E SITO.
	#
	# PER IL MOMENTO CONTROLLIAMO SOLO LA VALIDA' DEL COMANDO
	#

	if new_source != source:
		cli.error("You are wordking inside '%s'. You can not set source to '%s'" % (SOURCE_NAME(source), SOURCE_NAME(new_source)))
		return

	return


def cmd_config_datagroup_template_group_access(cli):

	# group  <name>         <type>
	# no     group  <name>  

	env  = cli.env

	user   = env['CURRENT_USER']
	source = env['SOURCE']

	tenant = None
	if source == SOURCE_LOCAL:
		tenant_name = env['CONFIG_TENANT']
		tenant = tenant_get(tenant_name)

	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']

	if cli.canonical_words[0] != 'no':

		group_name = cli.canonical_words[1]

		if len(cli.canonical_words) > 2:
			permission = CLI_PERM_TO_AUTH_PERM[ cli.canonical_words[2] ]
		else:
			permission = AUTH_READONLY

		datagroup_template_group_permission(tenant, datagroup_template_name, group_name, permission, user=user)
	else:
		group_name = cli.canonical_words[2]

		datagroup_template_group_permission(tenant, datagroup_template_name, group_name, None, user=user)





# rename condition  <name> <new>
# rename datasource <name> <new>
# rename timegraph  <name> <new>
# rename parameter  <name> <new>
# rename serie <name> <new>

def cmd_parameter_template_rename(cli):

	env  = cli.env

	user        = env['CURRENT_USER']
	source      = env['SOURCE']

	tenant = None
	if source == SOURCE_LOCAL:
		tenant_name = env['CONFIG_TENANT']
		tenant = tenant_get(tenant_name)

	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']

	name     = cli.canonical_words[2]
	new_name = cli.canonical_words[3]

	datagroup_template_rename_parameter(
		datagroup_template_name,
		name,
		new_name,
		source=source,
		tenant=tenant,
		user=user,
	)

	return

def cmd_datasource_template_rename(cli):

	env  = cli.env

	user   = env['CURRENT_USER']
	source = env['SOURCE']

	tenant = None
	if source == SOURCE_LOCAL:
		tenant_name = env['CONFIG_TENANT']
		tenant = tenant_get(tenant_name)

	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']

	name     = cli.canonical_words[2]
	new_name = cli.canonical_words[3]

	datasource_template_rename(datagroup_template_name, name, new_name, source=source, tenant=tenant, user=user)

def cmd_condition_template_rename(cli):

	env  = cli.env

	user   = env['CURRENT_USER']
	source = env['SOURCE']

	tenant = None
	if source == SOURCE_LOCAL:
		tenant_name = env['CONFIG_TENANT']
		tenant = tenant_get(tenant_name)

	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']

	name     = cli.canonical_words[2]
	new_name = cli.canonical_words[3]


	cli.warning("This renaming will not update sanet logs. This could cause problems with future data elaborations.")
	cli.warning("Are you sure to continue [y/N]")
	response = cli.read_next_input_line(show_prompt=False)
	if response.strip().lower() != 'y':
		cli.info("Cancelled")
		return

	condition_template_rename(datagroup_template_name, name, new_name, source=source, tenant=tenant, user=user)


def cmd_timegraph_template_rename(cli):

	env            = cli.env

	user           = env['CURRENT_USER']
	source         = env['SOURCE']

	tenant = None
	tenant_name = None
	if source == SOURCE_LOCAL:
		tenant_name = env['CONFIG_TENANT']
		tenant = tenant_get(tenant_name)
		db_set_current_tenant_schema(tenant_name)

	datagroup_name = env['CONFIG_DATAGROUP_TEMPLATE']

	name     = cli.canonical_words[2]
	new_name = cli.canonical_words[3]

	tdg = TDataGroup.objects.get(tenant=tenant, name=datagroup_name, source=source)
	ttg = timegraph_template_create(tenant, tdg, name)

	timegraph_template_rename(tenant, tdg, name, new_name, user=user)



def cmd_config_datagroup_template_owner(cli):

	env            = cli.env
	source         = env['SOURCE']

	tenant = None
	tenant_name = None
	if source == SOURCE_LOCAL:
		tenant_name = env['CONFIG_TENANT']
		tenant = tenant_get(tenant_name)
		db_set_current_tenant_schema(tenant_name)

	datagroup_name = env['CONFIG_DATAGROUP_TEMPLATE']


	tdg = TDataGroup.objects.get(tenant=tenant, name=datagroup_name, source=source)

	# owner <user> 
	# no owner
	if cli.canonical_words[0] != 'no':

		username = cli.canonical_words[1]

		user = User.objects.get(username=username)

		tdg.set_owner(user)
	else:
		tdg.unset_owner()

	return


CLI_PERM_TO_AUTH_PERM = {
	'read-write': AUTH_READWRITE,
	'read-only' : AUTH_READONLY, 
}

#from cli.tokens import T_Groupcompleter

mt_config_datagroup_1 = [

		T_Keyword('source', help='', subtree=[
			T_Alternatives(SOURCE_DESCR_TUPLE,  callback=cmd_config_datagroup_template_source, ),  # FIX: COMPLETER
		]),

		T_Keyword('parent', help='', subtree=[
			T_Keyword('none', callback=cmd_config_datagroup_template_parent, ),

			#T_DataGroupTemplate(help='', callback=cmd_config_datagroup_template_parent), #subtree=[
			#	T_Keyword('library', help='', callback=cmd_config_datagroup_template_parent),
			#]),  # FIX: COMPLETER

			T_AllDataGroupTemplate(help='', callback=cmd_config_datagroup_template_parent, subtree=[
				T_Keyword('library', help='', callback=cmd_config_datagroup_template_parent),
			]),  # FIX: COMPLETER

		]),

		T_Keyword('description', help='', subtree=[
			T_Line(help='', callback=cmd_config_datagroup_template_description, ),
		]),
	] 

mt_config_datagroup_2 =	CREATE_DATAGROUP_COMMANSDS(
		cmd_config_datagroup_template_field, 
		cmd_config_datagroup_template_parameter,

		cmd_datasource_template_config_prompt,

		cmd_condition_template_config_prompt,

		cmd_timegraph_template_config_prompt,

		include_group_commands=True,
		xcmd_config_datagroup_template_group_access=cmd_config_datagroup_template_group_access,

		include_owner_commands=True,
		xcmd_config_datagroup_template_owner=cmd_config_datagroup_template_owner,
		template_definition=True,
	) 

mt_config_datagroup_library = CREATE_DATAGROUP_COMMANSDS(
		cmd_config_datagroup_template_field, 
		cmd_config_datagroup_template_parameter,

		cmd_datasource_template_config_prompt,

		cmd_condition_template_config_prompt,

		cmd_timegraph_template_config_prompt,

		#include_group_commands=True,
		#xcmd_config_datagroup_template_group_access=cmd_config_datagroup_template_group_access,
		#
		#include_owner_commands=True,
		#xcmd_config_datagroup_template_owner=cmd_config_datagroup_template_owner
		template_definition=True,
	) 

mt_config_datagroup_rename = [
		T_Keyword('rename', help='Rename parameters/conditions/datasources', subtree=[
			T_Keyword('parameter' , help='', subtree=[
				T_DataGroupTemplateParameter        (help='', subtree=[
					T_ValidWord(VALID_PARAM_NAME, help='name',callback=cmd_parameter_template_rename),
				])
			]),
			T_Keyword('datasource', help='', subtree=[
				T_DataGroupTemplateDefinedDataSource(help='', subtree=[
					T_ValidWord(VALID_DATASOURCE_NAME, help='name',callback=cmd_datasource_template_rename),
				])
			]),
			T_Keyword('condition' , help='', subtree=[
				T_DataGroupTemplateDefinedCondition (help='', subtree=[
					T_ValidWord(VALID_CONDITION_NAME, help='name',callback=cmd_condition_template_rename),
				])
			]),
			T_Keyword('timegraph' , help='', subtree=[
				T_DataGroupTemplateDefinedTimegraph (help='', subtree=[
					T_ValidWord(VALID_TIMEGRAPH_NAME, help='name',callback=cmd_timegraph_template_rename),
				])
			]),
		]),
	]

#mt_config_datagroup_group = [
#		T_Keyword('group', help='', subtree=[
#			T_Groupcompleter(help='', callback=cmd_config_datagroup_template_group_access, subtree=[ 
#			
#				T_Alternatives(
#					[('read-write',''), ('read-only','')],
#					callback=cmd_config_datagroup_template_group_access
#				)
#			
#			]),
#		])
#]



def get_or_duplicate_tdg_from_library(cli, tenant, name, create_if_missing=True, owner_user=None):

	#if owner_user: owner_user.assert_capability(tenant, 'core.template.dg.edit')

	# E' definito localmente? usiamo quello?
	tdg = datagroup_template_get(name, source=SOURCE_LOCAL, tenant=tenant)
	if tdg != None:
		return tdg

	# Se nella library esiste, duplichiamolo e usiamo quello.
	library_tdg = datagroup_template_get(name, source=SOURCE_LIBRARY)
	if library_tdg != None:
		cli.info("Duplicating template datagroup  %s" % (name))
		tdg = datagroup_library_template_duplicate_on_tenant(library_tdg, tenant, owner_user=owner_user)
		return tdg

	# Creiamo localmente se manca?
	if create_if_missing:
		cli.info("Creating datagroup template %s" % (name))
		tdg = datagroup_template_create(name, source=SOURCE_LOCAL, tenant=tenant, owner_user=owner_user)

	return tdg

def cmd_datagroup_template_config_prompt(cli):

	# datagroup-template <name>
	# no datagroup-template <name> <name> [force]

	env         = cli.env

	source      = env['SOURCE']

	user        = env['CURRENT_USER']

	# FIX: ASSERT
	tenant = None
	tenant_name = None
	if source == SOURCE_LOCAL:
		tenant_name = env['CONFIG_TENANT']
		tenant = tenant_get(tenant_name)

	if cli.canonical_words[0] != 'no':

		name = cli.canonical_words[1]

		# Se siamo nella library, aggiungiamo semplicemente il template
		# Se siamo in un tenant. Se siamo in un tenant
		# - o il template e' definito nella library, quindi lo "cloniamo"
		# - o lo aggiungiamo semplicemente.
		if source == SOURCE_LIBRARY:

			tdg = datagroup_template_get(name, source=SOURCE_LIBRARY)
			if tdg == None:
				cli.info("Creating datagroup template in library: %s" % (name))
				tdg = datagroup_template_create(name, source=SOURCE_LIBRARY)
		else:
			tdg = get_or_duplicate_tdg_from_library(cli, tenant, name, owner_user=user)


		env = cli.env.clone()

		env['PROMPT'] = '%s' % ( PROMPT_STRING(tenant_name, name), )
		env['CONFIG_DATAGROUP_TEMPLATE']  = name

		#cmds = mt_config_datagroup 
		
		cmds  = mt_config_datagroup_1
		cmds += mt_config_datagroup_2  if  source == SOURCE_LOCAL else mt_config_datagroup_library
		cmds += mt_config_datagroup_rename

		cmds = cmds + [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))

		cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
		cli.cmdloop()

	else:
		# no datagroup <name> <name> [force]

		tdg1 = cli.canonical_words[2]
		tdg2 = cli.canonical_words[3]

		force = False
		name = cli.canonical_words[1]
		if len(cli.canonical_words) >=5:
			if cli.canonical_words[4] == 'force':
				force = True

		if tdg1 != tdg2:
			cli.error("%s != %s" % (tdg1, tdg2))
			return

		datagroup_template_delete(tdg1, force=force, source=source, tenant=tenant, user=user)



def cmd_datagroup_template_move_to_library(cli):

	# move to-library <name>

	env         = cli.env

	# FIX: ASSERT
	tenant = None
	tenant_name = None
	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)

	name = cli.canonical_words[2]

	datagroup_template_move_to_library(name, source=SOURCE_LOCAL, tenant=tenant)


