# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os

from sanet_common.sanetdefs.constants import *

from core.db_utils import transaction_commit_on_success

from core.exceptions import InvalidConfigurationException

from core.api.base import tenant_get
from core.api.templates import element_template_get, element_template_create

from core.api.templates import element_template_set_switch
from core.api.templates import element_template_set_router
from core.api.templates import element_template_update
from core.api.templates import element_template_delete
from core.api.templates import element_template_datagroup_create
from core.api.templates import element_template_datagroup_remove

from core.models import *
from core.api.base import db_set_current_tenant_schema
from core.api.templates import *

from cli.clilib.ioslike import *
from cli.clilib.ioslike import BLANK_ENTRY

from cli.tokens import T_DataGroupTemplate
from cli.tokens import T_LibraryDataGroupTemplate
from cli.tokens import T_DataGroupInElementTemplateCompleter

from cli.prompts.defs import *

from cli.prompts import cli_get_registered_menus

from cli.prompts.config.templates.datagroups import CREATE_DATAGROUP_COMMANSDS

from cli.prompts.config.templates.datagroups import CREATE_DATASOURCE_FIELD_COMMANDS
from cli.prompts.config.templates.datagroups import CREATE_CONDITION_FIELD_COMMANDS

from cli.prompts.config.templates.datagroups import CREATE_TIMEGRAPH_FIELD_COMMANDS
from cli.prompts.config.templates.datagroups import CREATE_TIMEGRAPH_SERIE_FIELD_COMMANDS




@transaction_commit_on_success
def element_template_datagroup_set_field(tenant, kind, element_template, datagroup_template_name, datagroup_instance_name, field_name, field_value):

	mtime = time.time()

	db_set_current_tenant_schema(tenant.name)
	
	te     = TElement.objects.get(tenant=tenant, kind=kind, name=element_template)
	tdg    = datagroup_template_get(datagroup_template_name, tenant=tenant)
	if tdg == None:
		raise Exception("non trovata TDG %s" % (datagroup_template_name))
		
	tedg   = TEDataGroup.objects.get(tenant=tenant, telement=te, tdatagroup=tdg, name=datagroup_instance_name)
	tf     = TField.objects.get(fclass=TField.FIELD_DATAGROUP, name=field_name)

	tf.validate_value(field_value)
	
	try:
		tedgfv = TEDataGroupFieldValue.objects.get(tenant=tenant, tedatagroup=tedg, tfield=tf)
	except TEDataGroupFieldValue.DoesNotExist, e:
		tedgfv = TEDataGroupFieldValue(tenant=tenant, tedatagroup=tedg, tfield=tf)
		tedgfv.save()
		
	if tedgfv.value == field_value:
		return
		
	tedgfv.value = field_value
	#tedgfv.mtime = mtime
	#tedgfv.save()
	tedgfv.concurrent_save(['value'], new_sentinel_value=mtime)

	#
	# FIX : REFACTOR THIS
	#
	
	from core.api.templates import _update_single_datagroup_in_element_template_field_value
	_update_single_datagroup_in_element_template_field_value(tedg, tedgfv, mtime)
	
	return
			
@transaction_commit_on_success
def element_template_datagroup_unset_field(tenant, kind, element_template, datagroup_template_name, datagroup_instance_name, field_name):
	
	mtime = time.time()

	db_set_current_tenant_schema(tenant.name)
	
	te     = TElement.objects.get(tenant=tenant, kind=kind, name=element_template)
	tdg    = datagroup_template_get(datagroup_template_name, tenant=tenant)
	if tdg == None:
		raise Exception("non trovata TDG %s" % (datagroup_template_name))
		
	tedg   = TEDataGroup.objects.get(tenant=tenant, telement=te, tdatagroup=tdg, name=datagroup_instance_name)
	tf     = TField.objects.get(fclass=TField.FIELD_DATAGROUP, name=field_name)

	try:
		tedgfv = TEDataGroupFieldValue.objects.get(tenant=tenant, tedatagroup=tedg, tfield=tf)
	except TEDataGroupFieldValue.DoesNotExist, e:
		#print "Field %s.%s.%s.%s not configured" % (element_template, datagroup_template_name, datagroup_instance_name, field_name)
		return
	
	
	tedgfv.delete()

	# FIX : REFACTOR THIS
	
	# Calcolo il valore che dovrebbe avere il campo a questo livello
	# e uso questo valore per aggiornare tutti gli oggetti sottostanti
	
	fv = tedg.get_configured_fields([field_name])[0] 
	
	from core.api.templates import _update_single_datagroup_in_element_template_field_value
	_update_single_datagroup_in_element_template_field_value(tedg, fv, mtime)
	
	return

@transaction_commit_on_success
def element_template_datagroup_set_parameter(tenant, kind, element_template, datagroup_template_name, datagroup_instance_name, param_name, param_type, param_value):

	mtime = time.time()

	ptype, pvalue = convert_param_value(param_type, param_value)

	te     = TElement.objects.get(tenant=tenant, kind=kind, name=element_template)

	#tdg    = datagroup_template_get(datagroup_template_name, tenant_name=tenant_name)
	tdg    = datagroup_template_get(datagroup_template_name, tenant=tenant)
	tedg   = TEDataGroup.objects.get(tenant=tenant, telement=te, tdatagroup=tdg, name=datagroup_instance_name)


	# Check if the parameter already exists (in the anchestors too) with the same type
	configured_params = tedg.get_configured_params_dict(filter_names=[param_name,])

	param = configured_params.get( param_name, None )
	if param:
		if param.vtype != ptype:
			raise ValueError("Can not change the param type (from type %s to %s)" % ( param.vtype, ptype))

	try:
		tedgpv = TEDataGroupParameterValue.objects.get(tenant=tenant, tedatagroup=tedg, name=param_name, vtype=ptype)
	except TEDataGroupParameterValue.DoesNotExist,e:
		tedgpv = TEDataGroupParameterValue(
			tenant=tenant,
			tedatagroup=tedg, 
			name=param_name, 
			vtype=ptype
		)
		tedgpv.save()

	if tedgpv.value == param_value:
		return

	tedgpv.value = param_value
	#tedgpv.mtime = mtime
	#tedgpv.save()
	tedgpv.concurrent_save(['value'], new_sentinel_value=mtime)

	#
	# FIX : REFACTOR THIS
	#
	from core.api.templates import _update_single_datagroup_in_element_template_parameter_value
	_update_single_datagroup_in_element_template_parameter_value(tedg, tedgpv, mtime)
	
	return

@transaction_commit_on_success
def element_template_datagroup_unset_parameter(tenant, kind, element_template, datagroup_template_name, datagroup_instance_name, param_name):
	
	mtime = time.time()

	te     = TElement.objects.get(tenant=tenant, kind=kind, name=element_template)

	tdg    = datagroup_template_get(datagroup_template_name, tenant=tenant)
	tedg   = TEDataGroup.objects.get(tenant=tenant, telement=te, tdatagroup=tdg, name=datagroup_instance_name)

	try:
		tedgpv = TEDataGroupParameterValue.objects.get(tenant=tenant, tedatagroup=tedg, name=param_name)
	except TEDataGroupParameterValue.DoesNotExist, e:
		tedgpv = None

	if tedgpv is None:
		return

	tedgpv.delete()

	#
	# FIX : REFACTOR THIS
	#
	from core.api.templates import _update_single_datagroup_in_element_template_parameter_value

	try:
		pv = tedg.get_configured_param(param_name)

		_update_single_datagroup_in_element_template_parameter_value(tedg, pv, mtime)	

	except KeyError, e:
		# Il parametro non e' definito nei datagroup template.
		# Scendiamo nell'albero con un parametro fittizio dal volore nullo per "cancellare" quello che c'e' sotto (se non sovrascritto)

		pv = TEDataGroupParameterValue(name=param_name, value=None)

		_update_single_datagroup_in_element_template_parameter_value(tedg, pv, mtime)	


@transaction_commit_on_success
def element_template_datasource_set_field(tenant, kind, element_template, datagroup_template_name,  datagroup_instance_name, datasource_template_name, field_name, field_value):

	mtime = time.time()

	tdg    = datagroup_template_get(datagroup_template_name, tenant=tenant)
	tds    = tdg.get_last_defined_datasource(datasource_template_name)
	
	te     = TElement.objects.get(tenant=tenant, kind=kind, name=element_template)
	tedg   = TEDataGroup.objects.get(tenant=tenant, telement=te, tdatagroup=tdg, name=datagroup_instance_name)
	
	
	tf     = TField.objects.get(fclass=TField.FIELD_DATASOURCE, name=field_name)
	
	tf.validate_value(field_value)
	
	old_configured =  TEDataSourceFieldValue.objects.filter(tenant=tenant, tedatagroup=tedg, tdatasource__name=datasource_template_name, tfield=tf).exclude(tdatasource=tds)
	if len(old_configured) > 0:
		#print "Deleting old configured datasource field (%s entries)..." % (len(old_configured))
		old_configured.delete()

	try:
		tedsfv = TEDataSourceFieldValue.objects.get(tenant=tenant, tedatagroup=tedg, tdatasource=tds, tfield=tf)
	except TEDataSourceFieldValue.DoesNotExist, e:
		tedsfv = TEDataSourceFieldValue(tenant=tenant, tedatagroup=tedg, tdatasource=tds, tfield=tf)
		tedsfv.save()
	
	if tedsfv.value == field_value:
		return
		
	tedsfv.value = field_value
	#tedsfv.mtime = mtime
	#tedsfv.save()
	tedsfv.concurrent_save(['value'], new_sentinel_value=mtime)
	
	#
	# FIX : REFACTOR THIS
	#
	from core.api.templates import _update_single_datasource_in_element_template_field_value
	_update_single_datasource_in_element_template_field_value(tedg, tds.name, tedsfv, mtime)
	
	return

@transaction_commit_on_success
def element_template_datasource_unset_field(tenant, kind, element_template, datagroup_template_name, datagroup_instance_name, datasource_template_name, field_name):

	mtime = time.time()

	tdg    = datagroup_template_get(datagroup_template_name, tenant=tenant)
	tds    = tdg.get_last_defined_datasource(datasource_template_name)
	
	te     = TElement.objects.get(tenant=tenant, kind=kind, name=element_template)
	tedg   = TEDataGroup.objects.get(tenant=tenant, telement=te, tdatagroup=tdg, name=datagroup_instance_name)
	
	tf     = TField.objects.get(fclass=TField.FIELD_DATASOURCE, name=field_name)
	
	try:
		tedsfv = TEDataSourceFieldValue.objects.get(tenant=tenant, tedatagroup=tedg, tdatasource__name=datasource_template_name, tfield=tf)
	except TEDataSourceFieldValue.DoesNotExist, e:
		#print "Field %s.%s.%s.%s.%s not configured" % (element_template, datagroup_template_name, datagroup_instance_name, datasource_template_name, field_name)
		return
	
	tedsfv.delete()
	
	#
	# FIX : REFACTOR THIS
	#
	fv = tedg.get_configured_datasource_field(tds.name, field_name)
	
	from core.api.templates import _update_single_datasource_in_element_template_field_value
	_update_single_datasource_in_element_template_field_value(tedg, tds.name, fv, mtime)
	
	return




@transaction_commit_on_success
def element_template_condition_set_field(tenant, kind, element_template, datagroup_template_name, datagroup_instance_name, condition_template_name, field_name, field_value):


	mtime = time.time()

	tdg    = datagroup_template_get(datagroup_template_name, tenant=tenant)
	tc     = tdg.get_last_defined_condition(condition_template_name)
	
	te     = TElement.objects.get(tenant=tenant, kind=kind, name=element_template)
	tedg   = TEDataGroup.objects.get(tenant=tenant, telement=te, tdatagroup=tdg, name=datagroup_instance_name)
	
	tf     = TField.objects.get(fclass=TField.FIELD_CONDITION, name=field_name)
	
	tf.validate_value(field_value)
	
	old_configured = TEConditionFieldValue.objects.filter(tenant=tenant, tedatagroup=tedg, tcondition__name=condition_template_name, tfield=tf).exclude(tcondition=tc)
	if len(old_configured) > 0:
		#print "Deleing old configured condition field (%s entries)..." % (len(old_configured))
		old_configured.delete()

	try:
		tecfv = TEConditionFieldValue.objects.get(tenant=tenant, tedatagroup=tedg, tcondition=tc, tfield=tf)
	except TEConditionFieldValue.DoesNotExist, e:
		tecfv = TEConditionFieldValue(tenant=tenant, tedatagroup=tedg, tcondition=tc, tfield=tf)
		tecfv.save()
	
	if tecfv.value == field_value:
		return
		
	tecfv.value = field_value
	#tecfv.mtime = mtime
	#tecfv.save()
	tecfv.concurrent_save(['value'], new_sentinel_value=mtime)

	#
	# FIX : REFACTOR THIS
	#
	from core.api.templates import _update_single_condition_in_element_template_field_value
	_update_single_condition_in_element_template_field_value(tedg, tc.name, tecfv, mtime)
	
	return

@transaction_commit_on_success
def element_template_condition_unset_field(tenant, kind, element_template, datagroup_template_name,  datagroup_instance_name, condition_template_name, field_name):

	mtime = time.time()

	tdg    = datagroup_template_get(datagroup_template_name, tenant=tenant)
	#tc     = TCondition.objects.get(tdatagroup=tdg, name=condition_template_name)
	tc     = tdg.get_last_defined_condition(condition_template_name)

	te     = TElement.objects.get(tenant=tenant, kind=kind, name=element_template)
	tedg   = TEDataGroup.objects.get(tenant=tenant, telement=te, tdatagroup=tdg, name=datagroup_instance_name)

	tf     = TField.objects.get(fclass=TField.FIELD_CONDITION, name=field_name)

	try:
		tecfv = TEConditionFieldValue.objects.get(tenant=tenant, tedatagroup=tedg, tcondition__name=tc.name, tfield=tf)
	except TEConditionFieldValue.DoesNotExist, e:
		#print "Field %s.%s.%s.%s.%s not configured" % (element_template, datagroup_template_name,  datagroup_instance_name, condition_template_name, field_name)
		return

	tecfv.delete()
	
	#
	# FIX : REFACTOR THIS
	#
	fv = tedg.get_configured_condition_field(tc.name, field_name)
	
	from core.api.templates import _update_single_condition_in_element_template_field_value
	_update_single_condition_in_element_template_field_value(tedg, tc.name, fv, mtime)
	
	return



@transaction_commit_on_success
def element_template_timegraph_set_field(tenant, kind, element_template, datagroup_template_name, datagroup_instance_name, timegraph_name, field_name, field_value):

	mtime = time.time()
	
	tdg    = datagroup_template_get(datagroup_template_name, tenant=tenant)
	
	ttg    = tdg.get_last_defined_timegraph(timegraph_name)
	
	te     = TElement.objects.get(tenant=tenant, kind=kind, name=element_template)
	tedg   = TEDataGroup.objects.get(tenant=tenant, telement=te, tdatagroup=tdg, name=datagroup_instance_name)
	
	tf     = TField.objects.get(fclass=TField.FIELD_TGRAPH, name=field_name)

	tf.validate_value(field_value)
	
	try:
		tetgfv = TETimeGraphFieldValue.objects.get(tenant=tenant, tedatagroup=tedg, ttimegraph=ttg, tfield=tf)
	except TETimeGraphFieldValue.DoesNotExist, e:
		tetgfv = TETimeGraphFieldValue(tenant=tenant, tedatagroup=tedg, ttimegraph=ttg, tfield=tf)
		tetgfv.save()
	
	if tetgfv.value == field_value:
		return
		
	tetgfv.value = field_value
	#tetgfv.mtime = time.time()
	#tetgfv.save()
	tetgfv.concurrent_save(['value'], new_sentinel_value=mtime)

	# FIX: udpate related entities
	from core.api.templates import _update_single_timegraph_in_element_template_field_value
	_update_single_timegraph_in_element_template_field_value(tedg, ttg.name, tetgfv, mtime)
		
	return

@transaction_commit_on_success
def element_template_timegraph_unset_field(tenant, kind, element_template, datagroup_template_name, datagroup_instance_name, timegraph_name, field_name):

	mtime = time.time()

	tdg    = datagroup_template_get(datagroup_template_name, tenant=tenant)
	
	ttg    = tdg.get_last_defined_timegraph(timegraph_name)
	
	te     = TElement.objects.get(tenant=tenant, kind=kind, name=element_template)
	tedg   = TEDataGroup.objects.get(tenant=tenant, telement=te, tdatagroup=tdg, name=datagroup_instance_name)
	
	tf     = TField.objects.get(fclass=TField.FIELD_TGRAPH, name=field_name)

	try:
		tetgfv = TETimeGraphFieldValue.objects.get(tenant=tenant, tedatagroup=tedg, ttimegraph=ttg, tfield=tf)
	except TETimeGraphFieldValue.DoesNotExist, e:
		#print "Field %s.%s.%s.%s.%s not configured" % (element_template, datagroup_template_name, datagroup_instance_name, timegraph_name, field_name)
		return
		
	tetgfv.delete()
	
	# FIX: refresh	
	
	fv = tedg.get_configured_timegraph_field(ttg.name, field_name)
	
	from core.api.templates import _update_single_timegraph_in_element_template_field_value
	_update_single_timegraph_in_element_template_field_value(tedg, ttg.name, fv, mtime)
	
	return
	

@transaction_commit_on_success
def element_template_timegraphserie_set_field(tenant, kind, element_template, datagroup_template_name, datagroup_instance_name, timegraph_name, serie_name, field_name, field_value):

	mtime = time.time()

	tf     = TField.objects.get(fclass=TField.FIELD_TSERIE, name=field_name)

	tf.validate_value(field_value)

	tdg    = datagroup_template_get(datagroup_template_name, tenant=tenant)
	
	ttgs    = tdg.get_last_defined_timegraphserie(timegraph_name, serie_name)
	
	te     = TElement.objects.get(tenant=tenant, kind=kind, name=element_template)
	tedg   = TEDataGroup.objects.get(tenant=tenant, telement=te, tdatagroup=tdg, name=datagroup_instance_name)

	try:
		tetgsfv = TETimeGraphSerieFieldValue.objects.get(tenant=tenant, tedatagroup=tedg, ttimegraphserie=ttgs, tfield=tf)
	except       TETimeGraphSerieFieldValue.DoesNotExist, e:
		tetgsfv = TETimeGraphSerieFieldValue(tenant=tenant, tedatagroup=tedg, ttimegraphserie=ttgs, tfield=tf)
		tetgsfv.save()

	if  tetgsfv.value == field_value:
		return

	tetgsfv.value = field_value
	#tetgsfv.mtime = mtime
	#tetgsfv.save()
	tetgsfv.concurrent_save(['value'], new_sentinel_value=mtime)

	# FIX: udpate related entities
	
	# FIX: udpate related entities
	from core.api.templates import _update_single_timegraphserie_in_element_template_field_value
	_update_single_timegraphserie_in_element_template_field_value(tedg, timegraph_name, serie_name, tetgsfv, mtime)
	
	return
	

@transaction_commit_on_success
def element_template_timegraphserie_unset_field(tenant, kind, element_template, datagroup_template_name, datagroup_instance_name, timegraph_name, serie_name, field_name):

	mtime = time.time()

	tf     = TField.objects.get(fclass=TField.FIELD_TSERIE, name=field_name)

	tdg    = datagroup_template_get(datagroup_template_name, tenant=tenant)

	ttgs    = tdg.get_last_defined_timegraphserie(timegraph_name, serie_name)

	te     = TElement.objects.get(tenant=tenant, kind=kind, name=element_template)
	tedg   = TEDataGroup.objects.get(tenant=tenant, telement=te, tdatagroup=tdg, name=datagroup_instance_name)

	try:
		tetgsfv = TETimeGraphSerieFieldValue.objects.get(tenant=tenant, tedatagroup=tedg, ttimegraphserie=ttgs, tfield=tf)
	except TETimeGraphSerieFieldValue.DoesNotExist, e:
		#print "Field %s.%s.%s.%s.%s.%s not configured" % (element_template, datagroup_template_name, datagroup_instance_name, timegraph_name, serie_name, field_name)
		return

	tetgsfv.delete()

	# FIX: udpate related entities	
	fv = tedg.get_configured_timegraphserie_field(timegraph_name, serie_name, field_name)

	from core.api.templates import _update_single_timegraphserie_in_element_template_field_value
	_update_single_timegraphserie_in_element_template_field_value(tedg, timegraph_name, serie_name, fv, mtime)

	return

#------------------------------------------------------------------------------#
# DATAGROUP  PARAMETER                                                         #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def cmd_element_template_datagroup_parameter(cli):

	env = cli.env
	tenant_name             = env['CURRENT_TENANT']
	tenant = tenant_get(tenant_name)

	element_template        = env['CONFIG_ELEMENT_TEMPLATE'    ]
	kind                    = env['CONFIG_ELEMENT_TEMPLATE_KIND']
	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']

	
	if cli.canonical_words[0] != 'no':
		# parameter <name> <type> <value>

		param_name  = cli.canonical_words[1]
		
		param_type  = cli.canonical_words[2]
		param_value = cli.canonical_words[3]
		
		param_value     = cli_input_to_real_param_value(param_type, param_value)
		
		element_template_datagroup_set_parameter(tenant, kind, element_template, datagroup_template_name, datagroup_instance_name, param_name, param_type, param_value)
	else:
		# no parameter <name> <name>
		param_name  = cli.canonical_words[2]
		param_name2 = cli.canonical_words[3]
		
		if param_name != param_name2:
			raise Exception("You must enter the same parameter twice")
		
		element_template_datagroup_unset_parameter(tenant, kind, element_template, datagroup_template_name, datagroup_instance_name, param_name)

	return

#------------------------------------------------------------------------------#
# DATASOURCE                                                                   #
#------------------------------------------------------------------------------#

def cmd_element_template_datasource_config_field(cli):
	env  = cli.env
	tenant_name              = env['CURRENT_TENANT']
	tenant = tenant_get(tenant_name)

	element_template         = env['CONFIG_ELEMENT_TEMPLATE']
	kind                     = env['CONFIG_ELEMENT_TEMPLATE_KIND']
	datagroup_template_name  = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name  = env['CONFIG_DATAGROUP_INSTANCE']
	datasource_template_name = env['CONFIG_DATASOURCE_TEMPLATE']

	tenant = tenant_get(tenant_name)	

	if cli.canonical_words[0] != 'no':
		# field <value>
		field_name  = cli.canonical_words[0]
		field_name  = CLI_FIELD_TO_DATASOURCE_FIELD(field_name)

		field_value = cli.canonical_words[1]

		element_template_datasource_set_field(tenant, kind, element_template, datagroup_template_name, datagroup_instance_name, datasource_template_name, field_name, field_value)
	else:
		# no <field>
		field_name  = cli.canonical_words[1]
		field_name  = CLI_FIELD_TO_DATASOURCE_FIELD(field_name)

		element_template_datasource_unset_field(tenant, kind, element_template, datagroup_template_name, datagroup_instance_name, datasource_template_name, field_name)

	return

mt_config_element_template_datasource = CREATE_DATASOURCE_FIELD_COMMANDS(cmd_element_template_datasource_config_field, )


def cmd_element_template_datasource_config_prompt(cli):

	env  = cli.env
	tenant_name             = env['CURRENT_TENANT']
	tenant = tenant_get(tenant_name)

	element_template        = env['CONFIG_ELEMENT_TEMPLATE'    ]
	kind                     = env['CONFIG_ELEMENT_TEMPLATE_KIND']
	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']

	name = cli.canonical_words[1]

        tdg         = datagroup_template_get(datagroup_template_name, tenant=tenant)
        tdatasource = tdg.get_last_defined_datasource(name)

        if tdatasource == None:
                raise Exception("'%s' not a valid datasource for the given datagroup" % (name,))


	env = cli.env.clone()
	env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, element_template, datagroup_instance_name, name]),)
	env['CONFIG_DATASOURCE_TEMPLATE'] = name

	cmds = SORTED_KEYWORDS(mt_config_element_template_datasource) + [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))
	cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	cli.cmdloop()	



#------------------------------------------------------------------------------#
# CONDITION                                                                   #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def cmd_element_template_condition_config_field(cli):
	env  = cli.env
	tenant_name              = env['CURRENT_TENANT']
	tenant = tenant_get(tenant_name)

	element_template         = env['CONFIG_ELEMENT_TEMPLATE'    ]
	kind                     = env['CONFIG_ELEMENT_TEMPLATE_KIND']
	datagroup_template_name  = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name  = env['CONFIG_DATAGROUP_INSTANCE']
	condition_template_name  = env['CONFIG_CONDITION_TEMPLATE']


	if cli.canonical_words[0] != 'no':
		field_name  = cli.canonical_words[0]
		field_name  = CLI_FIELD_TO_CONDITION_FIELD(field_name)

		field_value = cli.canonical_words[1]
		field_value = CLI_FIELD_VALUE_TO_CONDITION_FIELD_VALUE(field_name, field_value)

		element_template_condition_set_field(tenant, kind, element_template, datagroup_template_name, datagroup_instance_name, condition_template_name, field_name, field_value)		

	else:
		field_name  = cli.canonical_words[1]
		field_name  = CLI_FIELD_TO_CONDITION_FIELD(field_name)

		element_template_condition_unset_field(tenant, kind, element_template, datagroup_template_name, datagroup_instance_name, condition_template_name, field_name)

	return


mt_config_element_template_condition = CREATE_CONDITION_FIELD_COMMANDS( cmd_element_template_condition_config_field )


def cmd_element_template_condition_config_prompt(cli):
	
	env  = cli.env
	
	tenant_name             = env['CURRENT_TENANT']
	element_template        = env['CONFIG_ELEMENT_TEMPLATE'    ]
	kind                    = env['CONFIG_ELEMENT_TEMPLATE_KIND']
	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']

	name = cli.canonical_words[1]

	tenant = tenant_get(tenant_name)

        tdg        = datagroup_template_get(datagroup_template_name, tenant=tenant)
        tcondition = tdg.get_last_defined_condition(name)

	if tcondition == None:
		raise Exception("'%s' not a valid condition for the given datagroup" % (name,))


	env = cli.env.clone()

	env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, element_template, datagroup_instance_name, name]),)
	env['CONFIG_CONDITION_TEMPLATE'] = name

	cmds = SORTED_KEYWORDS(mt_config_element_template_condition) + [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))
	cli  = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	cli.cmdloop()

#------------------------------------------------------------------------------#
# TIMEGRAPH SERIE                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def cmd_config_element_template_timegraph_serie_field(cli):

	env  = cli.env
	tenant_name             = env['CURRENT_TENANT']
	tenant = tenant_get(tenant_name)
	element_template        = env['CONFIG_ELEMENT_TEMPLATE'    ]
	kind                    = env['CONFIG_ELEMENT_TEMPLATE_KIND']
	
	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']
	timegraph_name          = env['CONFIG_TIMEGRAPH']
	serie_name              = env['CONFIG_TIMEGRAPH_SERIE']

	if cli.canonical_words[0] != 'no':
		field_name  = cli.canonical_words[0]
		field_name  = CLI_FIELD_TO_TIMEGRAPHSERIE_FIELD(field_name)

		field_value = cli.canonical_words[1]
		
		element_template_timegraphserie_set_field(tenant, kind, element_template, datagroup_template_name, datagroup_instance_name, timegraph_name, serie_name, field_name, field_value)
		
	else:
		field_name  = cli.canonical_words[1]
		field_name  = CLI_FIELD_TO_TIMEGRAPHSERIE_FIELD(field_name)
		
		element_template_timegraphserie_unset_field(tenant, kind, element_template, datagroup_template_name, datagroup_instance_name, timegraph_name, serie_name, field_name)

	return


mt_config_element_template_timegraph_serie_field =  CREATE_TIMEGRAPH_SERIE_FIELD_COMMANDS(cmd_config_element_template_timegraph_serie_field)

def cmd_config_element_template_timegraph_serie(cli):

	env  = cli.env
	tenant_name             = env['CURRENT_TENANT']
	tenant = tenant_get(tenant_name)
	element_template        = env['CONFIG_ELEMENT_TEMPLATE'    ]
	kind                    = env['CONFIG_ELEMENT_TEMPLATE_KIND']
	
	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']
	timegraph_name          = env['CONFIG_TIMEGRAPH']
	
	serie_name  = cli.canonical_words[1]


	db_set_current_tenant_schema(tenant_name)
	
	#tdg    = datagroup_template_get(datagroup_template_name, tenant_name=tenant_name)
	tdg    = datagroup_template_get(datagroup_template_name, tenant=tenant)

	#print "XXXXXXXXXXXX", "tdatagroup=", tdg, "name=", timegraph_name
	#for ttg in TTimeGraph.objects.filter(tdatagroup=tdg).all():
	#	print ttg
	#print "XXXXXXXXXXXX"
	
	#ttg    = TTimeGraph.objects.get(tdatagroup=tdg, name=timegraph_name)
	ttg     = tdg.get_last_defined_timegraph(timegraph_name)

	#ttgs   = TTimeGraphSerie.objects.get(ttimegraph=ttg, name=serie_name)     
	ttgs    = tdg.get_last_defined_timegraphserie(timegraph_name, serie_name) # FIX: sarebbe piu' corretto che questo metodo fosse un metodo del TTimeGraph e on del TDatagroup

	te     = TElement.objects.get(tenant=tenant, kind=kind, name=element_template)
	tedg   = TEDataGroup.objects.get(tenant=tenant, telement=te, tdatagroup=tdg, name=datagroup_instance_name)

	env = cli.env.clone()

	#env['PROMPT'] = 'config-%s#' % (PROMPT_SEP.join([tenant_name, element_template, datagroup_instance_name, timegraph_name, serie_name]),)
	env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, element_template, datagroup_instance_name, timegraph_name, serie_name]),)

	env['CONFIG_TIMEGRAPH_SERIE'] = serie_name

	cmds = SORTED_KEYWORDS(mt_config_element_template_timegraph_serie_field) + \
	       [BLANK_ENTRY] + \
	       SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))
	       
	cli  = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	cli.cmdloop()
	
#------------------------------------------------------------------------------#
# TIMEGRAPH                                                                    #
#------------------------------------------------------------------------------#

def cmd_config_element_template_timegraph_field(cli):
	env  = cli.env
	tenant_name             = env['CURRENT_TENANT']
	tenant = tenant_get(tenant_name)
	element_template        = env['CONFIG_ELEMENT_TEMPLATE'    ]
	kind                    = env['CONFIG_ELEMENT_TEMPLATE_KIND']
	
	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']
	timegraph_name          = env['CONFIG_TIMEGRAPH']


	if cli.canonical_words[0] != 'no':
		field_name  = cli.canonical_words[0]
		field_name  = CLI_FIELD_TO_TIMEGRAPH_FIELD(field_name)
		
		field_value = cli.canonical_words[1]

		element_template_timegraph_set_field(tenant, kind, element_template, datagroup_template_name, datagroup_instance_name, timegraph_name, field_name, field_value)
	else:
		field_name  = cli.canonical_words[1]
		field_name  = CLI_FIELD_TO_TIMEGRAPH_FIELD(field_name)
		
		element_template_timegraph_unset_field(tenant, kind, element_template, datagroup_template_name, datagroup_instance_name, timegraph_name, field_name)
		
	return

	
mt_config_element_template_timegraph = CREATE_TIMEGRAPH_FIELD_COMMANDS(
	cmd_config_element_template_timegraph_field, 
	cmd_config_element_template_timegraph_serie,
	#cmd_config_element_template_timegraph_serie_remove,
	include_no_commands=False
)

def cmd_element_template_timegraph_config_prompt(cli):

	env  = cli.env
	tenant_name             = env['CURRENT_TENANT']
	element_template        = env['CONFIG_ELEMENT_TEMPLATE'    ]
	kind                    = env['CONFIG_ELEMENT_TEMPLATE_KIND']
	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']

	timegraph_name = cli.canonical_words[1]

	tenant = tenant_get(tenant_name)

	#ttimegraph = condition_template_get(datagroup_template_name, name, tenant_name=tenant_name)
	#if tcondition == None:
	#raise Exception("'%s' not a valid condition for the given datagroup" % (name,))
	

	env = cli.env.clone()

	#env['PROMPT'] = 'config-%s#' % (PROMPT_SEP.join([tenant_name, element_template, datagroup_instance_name, timegraph_name]),)
	env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, element_template, datagroup_instance_name, timegraph_name]),)

	env['CONFIG_TIMEGRAPH'] = timegraph_name

	cmds = SORTED_KEYWORDS(mt_config_element_template_timegraph) + [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))
	cli  = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	cli.cmdloop()
	
#------------------------------------------------------------------------------#
# DATAGROUP                                                                    #
#------------------------------------------------------------------------------#

def cmd_element_template_datagroup_field(cli):
	
	env = cli.env
	tenant_name             = env['CURRENT_TENANT']
	tenant = tenant_get(tenant_name)
	element_template        = env['CONFIG_ELEMENT_TEMPLATE'    ]
	kind                    = env['CONFIG_ELEMENT_TEMPLATE_KIND']
	datagroup_template_name = env['CONFIG_DATAGROUP_TEMPLATE']
	datagroup_instance_name = env['CONFIG_DATAGROUP_INSTANCE']


	# <field> <value>
	# no <field>
	if cli.canonical_words[0] != 'no':

		field_name  = cli.canonical_words[0]
		field_name  = CLI_FIELD_TO_TIMEGRAPH_FIELD(field_name)
		
		field_value = cli.canonical_words[1]
		
		element_template_datagroup_set_field(tenant, kind, element_template, datagroup_template_name, datagroup_instance_name, field_name, field_value)
	else:
		field_name  = cli.canonical_words[1]
		field_name  = CLI_FIELD_TO_TIMEGRAPH_FIELD(field_name)
		
		element_template_datagroup_unset_field(tenant, kind, element_template, datagroup_template_name, datagroup_instance_name, field_name)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

from cli.tokens import T_TEDataGroupTemplateParameter

mt_element_template_datagroup = CREATE_DATAGROUP_COMMANSDS( 
	cmd_element_template_datagroup_field    ,
	cmd_element_template_datagroup_parameter,

	cmd_element_template_datasource_config_prompt,
	cmd_element_template_condition_config_prompt, 
	cmd_element_template_timegraph_config_prompt,
	include_no_commands=False,
	
	
	tok_param_completer=T_TEDataGroupTemplateParameter,
	tok_no_param_completer=T_TEDataGroupTemplateParameter,
)


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_element_template_datagroup_config_delete(cli):
	
	# no datagroup <instance>
	env = cli.env

	tenant_name = env['CURRENT_TENANT']
	name = env['CONFIG_ELEMENT_TEMPLATE'    ]
	kind = env['CONFIG_ELEMENT_TEMPLATE_KIND']

	datagroup_template_name = cli.canonical_words[2]
	datagroup_instance_name = datagroup_template_name

	tenant = tenant_get(tenant_name)
		
	element_template_datagroup_remove(tenant, kind, name, datagroup_instance_name)
	
	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_element_template_datagroup_config_prompt(cli):
	
	env = cli.env
	
	user        = env['CURRENT_USER']
	
	tenant_name = env['CURRENT_TENANT']
	tenant = tenant_get(tenant_name)
	name = env['CONFIG_ELEMENT_TEMPLATE'    ]
	kind = env['CONFIG_ELEMENT_TEMPLATE_KIND']
	
	# datagroup <name> [ template <name> ]
	if len(cli.canonical_words) == 4:
		datagroup_template_name = cli.canonical_words[3]
		datagroup_instance_name = cli.canonical_words[1]
	else:
		datagroup_template_name = cli.canonical_words[1]
		datagroup_instance_name = datagroup_template_name
		
	# Get or create from the library
	from cli.prompts.config.templates.datagroups import get_or_duplicate_tdg_from_library
	tdg = get_or_duplicate_tdg_from_library(cli, tenant, datagroup_template_name)
	
	tedg = element_template_datagroup_create(tenant, kind, name, datagroup_template_name, datagroup_instance_name, user=user)
	
	env = env.clone()
	
	#env['PROMPT'] = 'config-%s#' % (PROMPT_SEP.join([tenant_name, name, datagroup_instance_name]),)
	env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, name, datagroup_instance_name]),)
	
	env['CONFIG_DATAGROUP_TEMPLATE'] = datagroup_template_name
	env['CONFIG_DATAGROUP_INSTANCE'] = datagroup_instance_name
	
	cmds = SORTED_KEYWORDS(mt_element_template_datagroup) + [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))
	
	cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	cli.cmdloop()



#------------------------------------------------------------------------------#
# NODE-TEMPLATE                                                                #
#------------------------------------------------------------------------------#

def cmd_element_template_set_attr(cli):
	
	env =  cli.env
	
	tenant_name = env['CURRENT_TENANT']
	
	name = env['CONFIG_ELEMENT_TEMPLATE'    ]
	kind = env['CONFIG_ELEMENT_TEMPLATE_KIND']
	
	tenant = tenant_get(tenant_name)
	
	te = element_template_get(kind, name, tenant)
	
	attr = cli.canonical_words[0]
	
	if attr == 'description':
		v = cli.canonical_words[1]
		if te.description != v:
			te.description = v
			element_template_update(te, 'description')


def cmd_node_template_set_attr(cli):

	env =  cli.env

	user        = env['CURRENT_USER']

	tenant_name = env['CURRENT_TENANT']

	name = env['CONFIG_ELEMENT_TEMPLATE'    ]
	kind = env['CONFIG_ELEMENT_TEMPLATE_KIND']

	tenant = tenant_get(tenant_name)

	te = element_template_get(kind, name, tenant)

	if cli.canonical_words[0] != 'no':

		attr = cli.canonical_words[0]
		if attr == 'switch':
			element_template_set_switch(te, True, user=user)
		if attr == 'router':
			element_template_set_router(te, True, user=user)
	else:

		attr = cli.canonical_words[1]

		if attr == 'switch':
			element_template_set_switch(te, False, user=user)
		if attr == 'router':
			element_template_set_router(te, False, user=user)


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

from cli.tokens import T_Groupcompleter

CLI_PERM_TO_AUTH_PERM = {
	'read-write': AUTH_READWRITE,
	'read-only' : AUTH_READONLY, 
}

def cmd_element_template_set_group(cli):

	# group <name> [ <perm> ]
	# no group <name>

	user = cli.env['CURRENT_USER']

	tenant_name = cli.env['CURRENT_TENANT']

	name = cli.env['CONFIG_ELEMENT_TEMPLATE'    ]
	kind = cli.env['CONFIG_ELEMENT_TEMPLATE_KIND']

	tenant = tenant_get(tenant_name)
	
	te = element_template_get(kind, name, tenant)

	if cli.canonical_words[0] != 'no':

		group_name = cli.canonical_words[1]

		if len(cli.canonical_words) > 2:
			permission = CLI_PERM_TO_AUTH_PERM[ cli.canonical_words[2] ]
		else:
			permission = AUTH_READONLY

		g = Group.objects.get(name=group_name)

		te.add_group_permission(g, permission, user)
	else:

		group_name = cli.canonical_words[2]

		g = Group.objects.get(name=group_name)

		te.remove_group_permission(g, user)


from cli.prompts.defs import CLI_PERM_OPTIONS


mt_base_element_template = [

	T_Keyword('description', help='Specify description for the template.', subtree=[
		T_Line(help='Specify description for the template.', callback=cmd_element_template_set_attr,)
	]),

	T_Keyword('group', help='', subtree=[
		T_Groupcompleter(help='', callback=cmd_element_template_set_group, subtree=[ 
			T_Alternatives( CLI_PERM_OPTIONS, callback=cmd_element_template_set_group )
		]),
	]),

	T_Keyword('datagroup', help='Add refrence to datagroup template.', subtree=[
		T_ValidWord(VALID_DATAGROUP_NAME, help='Add refrence to datagroup template with the given instance name', subtree=[
			T_Keyword('template', help='Add refrence to datagroup template.', subtree=[
				T_DataGroupTemplate(help='Add refrence to datagroup template.', callback=cmd_element_template_datagroup_config_prompt),
				T_LibraryDataGroupTemplate(help='Add refrence to datagroup template.', callback=cmd_element_template_datagroup_config_prompt),
			]),
		]),
		T_DataGroupTemplate(help='Add refrence to datagroup template (SITE)', callback=cmd_element_template_datagroup_config_prompt, subtree=[
			T_Keyword('template', help='Add refrence to datagroup template (SITE)', subtree=[
				T_DataGroupTemplate(help='Add refrence to datagroup template (SITE)', callback=cmd_element_template_datagroup_config_prompt),
				T_LibraryDataGroupTemplate(help='Add refrence to datagroup template (SITE)', callback=cmd_element_template_datagroup_config_prompt),
			]),
		]),
		T_LibraryDataGroupTemplate(help='Add refrence to datagroup template (LIBRARY)', callback=cmd_element_template_datagroup_config_prompt, subtree=[
			T_Keyword('template', help='Add refrence to datagroup template (LIBRARY)', subtree=[
				T_DataGroupTemplate(help='Add refrence to datagroup template (LIBRARY)', callback=cmd_element_template_datagroup_config_prompt),
				T_LibraryDataGroupTemplate(help='Add refrence to datagroup template (LIBRARY)', callback=cmd_element_template_datagroup_config_prompt),
			]),
		]),
	]),
]

mt_base_element_template_no = [
	T_Keyword('datagroup', help='Remove datagroup from element template.', subtree=[
		T_DataGroupInElementTemplateCompleter(help='Remove datagroup from element template.', subtree=[
			T_DataGroupInElementTemplateCompleter(help='Remove datagroup from element template.', callback=cmd_element_template_datagroup_config_delete, pipe=False),
		])
	]),

	T_Keyword('group', help='', subtree=[
		T_Groupcompleter(help='', callback=cmd_element_template_set_group)
	]),
]


mt_other_element_template = mt_base_element_template + [
	T_Keyword('no', help='Remove configuration', subtree=mt_base_element_template_no )

]

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_element_template_subelement_assign_template(cli):
	
	env = cli.env
	
	# 0        1        2      3
	# template <name>
	# no       template <name> <name>
	
	tenant_name = env['CURRENT_TENANT']
	name = env['CONFIG_ELEMENT_TEMPLATE'    ]
	kind = env['CONFIG_ELEMENT_TEMPLATE_KIND']
	
	tenant = tenant_get(tenant_name)
	te = element_template_get(kind, name, tenant)
	
	if te is None:
		raise Exception("IPOSSIBILE 1")
	
	if cli.canonical_words[0] == 'template':
		
		apply_name = cli.canonical_words[1]
		
		#
		# Creiamo l'associazione tra il TElement del sotto elemento ed il TElement usato
		#
		telementapplied = element_template_get(kind, apply_name, tenant)
		if telementapplied is None:
			raise Exception("IPOSSIBILE 2")
		
		_template_sub_element_assign_template(tenant, te, telementapplied)
	else:
		apply_name  = cli.canonical_words[2]
		apply_name2 = cli.canonical_words[3]
		
		if apply_name != apply_name2:
			cli.error("Names '%s' and '%s' do not match" % (apply_name, apply_name2)) 
			return
		
		_template_sub_element_remove_template(tenant, te, apply_name)
		
		return



#
#          (F)             (E)
#     DataGroup---<>---CDataGroup-----<>-----TEDataGroup
#         |           /       .                     |
#         |          /        .                     |
#        < >       < >        .                    < >
#         |        /          .                     |
#         |       /           .   (C)               |
# MonitoredElement------<elementtemplate>------- TElement
#         |                     |                 /     \
#         |                     |           normal       sub-+
#         |                     |                |       | | |
#         |                     |                +--<A>--+ | |
#         |                    <B>-------------------------+ |
#         |                                                  |
#        <D>-------------------------------------------------+
#

from core.api.templates import _refresh_element
from core.api.elements import _element_unset_template_clean_pending_datagroups

#
# FIX: Spostare in CORE
# 
@transaction_commit_on_success
def _template_sub_element_assign_template(tenant, te, telementapplied):

	try:
		pos = TElementAppliedTemplate.objects.filter(tenant=tenant, telement=te).order_by('applyorder')[-1].applyorder
	except Exception, e:
		pos = 0

	teat, created = TElementAppliedTemplate.objects.get_or_create(tenant=tenant, telement=te, telementapplied=telementapplied)
	if not created:
		teat.applyorder = pos

	teat.save()

	#
	# Il sotto-elemento TElement potrebbe essere associato a (aver generato automaticamente) dei MonitoredElement. (D)
	#
	# Ora che abbiamo creato una associazione tra il sub-element template e un elemente template (A) 
	# dobbiamo creare anche una relazione tra "telementapplied" e l'elemento attraverso una "ElementTemplate" (C)
	#
	for me in MonitoredElement.objects.filter(telement_generator=te).all():
		
		et, created2 = ElementTemplate.objects.get_or_create(tenant=tenant, melement=me, telement=telementapplied)

		# Caso veramente stronzo: ad un elemento di monitoraggio e' stato gia' associato il template "telementapplied", ma direttamente! NO, non va bene! Annulliamo tutto.
		if created and not created2:
			raise InvalidConfigurationException("Can not add reference to '%s' inside '%s' because the element '%s' (generend by '%s') was configured with the same reference" % (telementapplied.name, te.name, me.path, te.name ))

		if created2:

			et.telementappliedtemplate = teat
			et.save()

			refresh_element_template(tenant, telementapplied, me)

#
# FIX: Spostare in CORE
# 
@transaction_commit_on_success
def _template_sub_element_remove_template(tenant, te, template_name):

	telementapplied = TElement.objects.get(tenant=tenant, name=template_name)

	#
	# Cancelliamo l'associazione tra il sub-element-template e i template che usa (A)
	#
	teats = TElementAppliedTemplate.objects.filter(tenant=tenant, telement=te, telementapplied=telementapplied)
	for appliedtemplate in teats:
		# Questa operazione cancella in cascata anche gli elementi della relazione ElementTemplate (C) che 
		# associavano l'elmento di monitoraggio e il template che gli era stato assegnato dalla relazione (A)
		appliedtemplate.delete()

	# Le cancellazioni (espliciti e in cascata) al passo prima non hanno rimosso i CDataGroup
	# creati dal sistema quando era stato applicato il template "telementapplied" (E, F)
	for element in MonitoredElement.objects.filter(telement_generator=te).all():

		# Cancelliamo i CDataGroup (e quindi i Datagroup) (E, F)
		_element_unset_template_clean_pending_datagroups(tenant, element, telementapplied)

		# Refresh configuration dell'elemento
		_refresh_element(element)

	return


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

from cli.tokens import T_ParametricElementTemplate, T_TElementAppliedTemplate

def mt_sub_element_template(kind):
	
	return [
		
		T_Keyword('template', help='Apply element template for this sub-element template.', subtree=[
			T_ParametricElementTemplate(kind, help='Specify element template for this sub-element template.', callback=cmd_element_template_subelement_assign_template)
		]),
		
		T_Keyword('no', help='Remove configuration for this sub-element template.', subtree=[
			T_Keyword('template', help='Remove reference to element-template.', subtree=[
				T_TElementAppliedTemplate(help='Remove reference to element-template.', subtree=[
					T_TElementAppliedTemplate(help='Remove reference to element-template.', callback=cmd_element_template_subelement_assign_template)
				])
			])
		])
	]

#
# FIX: SPOSTARE IN CORE
#
@transaction_commit_on_success
def _create_sub_element_in_template(tenant, te, sub_kind, subelement_name, distinguisher=None, xform=None):

	template_subelement_name = '%s:%s' % (te.name, subelement_name)
	
	#
	# Prima di aggiungere un elemento al template verifichiamo se qualche nodo
	# gia' non dichiari un sotto-elemento con lo stesso nome che non c'entra
	# niente con questo template
	#
	conflicts = []
	for et in ElementTemplate.objects.filter(telement=te).select_related('melement').all():
		
		search_filter = et.melement.get_sub_elements(sub_kind)
		search_filter = search_filter.filter(name=subelement_name)
		search_filter = search_filter.exclude(telement_generator__name=template_subelement_name)
		
		count = search_filter.count()
		if count > 0:
			conflicts.append( "Element %s already defines a sub-element with name '%s'" % (et.melement.path, subelement_name) )

	if conflicts:
		raise InvalidConfigurationException("Configuration conflict!", conflicts)

	#
	# CREATE or UPDATE
	#
	
	updated_fields = {}
	old_values = {}
	
	sub_te, created = TElement.objects.get_or_create(tenant=tenant, parent=te, kind=sub_kind, name=template_subelement_name)
	
	if created:
		sub_te.owner = te.owner
		sub_te.save()
	
	old_values['distinguisher'] = sub_te.distinguisher
	old_values['xform']         = sub_te.xform

	if sub_te.distinguisher != distinguisher:
		sub_te.distinguisher = distinguisher
		updated_fields['distinguisher'] = distinguisher
	
	if sub_te.xform != xform:
		sub_te.xform = xform
		updated_fields['xform'] = xform
	
	if updated_fields:
		sub_te.save()
	
	#
	# Creiamo gli elementi di monitoraggio
	#
	for et in ElementTemplate.objects.filter(telement=te).select_related('melement').all():
		element = et.melement
		
		sub = element.create_sub_element(subelement_name, sub_kind)

		changed = False
		
		if sub.telement_generator != sub_te:
			sub.telement_generator = sub_te
			changed = True
		
		for field_name, field_new_value in updated_fields.items():
			
			# Se i campi dell'oggetto di monitoraggio hanno un valore diverso da quello
			# che c'era prima, vuol dire che erano stati "customizzati"
			# a livello di elemento quindi non dobbiamo sovrascrivere quel valore.
			if getattr(sub, field_name) != old_values[field_name]:
				continue
				
			if getattr(sub, field_name) != field_new_value:
				setattr(sub, field_name, field_new_value)
				changed = True
			
		if changed:
			sub.save()
		
	return sub_te


def cmd_element_template_subelement_config(cli):
	
	# 0          1         2             3       4     5
	# interface  <name>  [ distinguisher <val> [ xform <val> ] ]
	# storage    <name>  [ distinguisher <val> [ xform <val> ] ]
	# service    <name>  [ distinguisher <val> [ xform <val> ] ]
	# device     <name>  [ distinguisher <val> [ xform <val> ] ]

	env =  cli.env
	
	tenant_name = env['CURRENT_TENANT']
	name = env['CONFIG_ELEMENT_TEMPLATE'    ]
	kind = env['CONFIG_ELEMENT_TEMPLATE_KIND']
	
	if kind != ELEMENT_KIND_NODE:
		raise Exception("IMPOSSIBILE 666")
	
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant.name)
	
	te = element_template_get(kind, name, tenant)

	str_to_kind = { 
		'interface' : ELEMENT_KIND_IFACE   ,
		'storage'   : ELEMENT_KIND_STORAGE ,
		'service'   : ELEMENT_KIND_SERVICE ,
		'device'    : ELEMENT_KIND_DEVICE  ,
	}
	sub_kind = str_to_kind[ cli.canonical_words[0] ]

	subelement_name = cli.canonical_words[1]

	distinguisher   = cli.canonical_words[3] if len(cli.canonical_words) > 3 else None
	xform           = cli.canonical_words[5] if len(cli.canonical_words) > 5 else None


	#
	# Validate xform alias
	#
	from sanet_poller.lib.evaluation import compile_str_expr
	if xform:
		xform = xform_alias_to_xfrom_by_kind( sub_kind, xform)
		try:
			compile_str_expr(None, xform)
		except Exception, e:
			cli.error("Error while parsing expression: %s" % (e))
			return	

	# 
	# Proviamo a creare la sotto entita' nel template (e in cascata le entita' in tutti gli elementi che usano quel template)
	#

	try:
		tsub = _create_sub_element_in_template(tenant, te, sub_kind, subelement_name, distinguisher=distinguisher, xform=xform)
	except InvalidConfigurationException, e:
		cli.error(str(e))
		return
	
	
	#
	# Enter submenu
	#

	template_subelement_name = tsub.name

	env = cli.env.clone()
	env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, name, subelement_name]),)
	env['CONFIG_ELEMENT_TEMPLATE'    ]  = template_subelement_name
	env['CONFIG_ELEMENT_TEMPLATE_KIND']  = sub_kind
	
	menu = mt_sub_element_template(sub_kind)
	
	cmds = SORTED_KEYWORDS(menu) + [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))
	cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	cli.cmdloop()	


	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#


def cmd_element_template_subelement_config_no(cli):
	# 0  1          2       3
	# no interface  <name>  <name> 
	# no storage    <name>  <name>
	# no service    <name>  <name>
	# no device     <name>  <name>

	env =  cli.env
	
	tenant_name = env['CURRENT_TENANT']

	name = env['CONFIG_ELEMENT_TEMPLATE'    ]
	kind = env['CONFIG_ELEMENT_TEMPLATE_KIND']
	
	if kind != ELEMENT_KIND_NODE:
		raise Exception("IMPOSSIBILE 666")
	
	tenant = tenant_get(tenant_name)
	te = element_template_get(kind, name, tenant)
	
	subelement_name  = cli.canonical_words[2]
	subelement_name2 = cli.canonical_words[3]
	
	if subelement_name != subelement_name2:
		cli.error("Names '%s' and '%s' do not match" % (subelement_name, subelement_name2))
		return
	
	template_subelement_name = '%s:%s' % (te.name, subelement_name)

	# Teoricamente dovrebbe esserci un solo sub-element-template.
	sub_all = TElement.objects.filter(tenant=tenant, parent=te, name=template_subelement_name)
	for sub_te in sub_all:
		sub_te.delete()

		#
		# FIX: dobbiamo refreshare qualcosa?
		# 

	return 



#
# FIX: move to CORE
#
from core.api.elements import element_rename_raw

@transaction_commit_on_success
def element_template_sub_element_rename(telement_sub, subelement_name_new):

	template_subelement_name_new = '%s:%s' % (telement_sub.parent.name, subelement_name_new)


	# Rename
	telement_sub.name = template_subelement_name_new
	telement_sub.save()
	
	#
	# Controlliamo se non esistono conflitti con il nuovo nome
	#
	
	conflicts = []
	for me in MonitoredElement.objects.filter(tenant=telement_sub.tenant, parent__isnull=False, kind=telement_sub.kind, name=subelement_name_new):
		conflicts.append( "Element '%s' already has a '%s' sub-element" % (me.parent, me.name))
	
	if conflicts:
		raise InvalidConfigurationException("Element '%s' already exists" % (subelement_name_new), conflicts)
	
	# Effettuiamo il rename di tutti gli elementi legati al template
	for me in MonitoredElement.objects.filter(tenant=telement_sub.tenant, telement_generator=telement_sub):
		element_rename_raw(me, subelement_name_new)
	


def cmd_element_template_subelement_rename(cli):
	
	# 0      1         2     3
	# rename interface <old> <new>
	
	env =  cli.env
	
	tenant_name = env['CURRENT_TENANT']

	name = env['CONFIG_ELEMENT_TEMPLATE'    ]
	kind = env['CONFIG_ELEMENT_TEMPLATE_KIND']
	
	if kind != ELEMENT_KIND_NODE:
		raise Exception("IMPOSSIBILE 666")
	
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant.name)
	
	te = element_template_get(kind, name, tenant)
	
	#
	# Parametri
	#
	subelement_name     = cli.canonical_words[2]
	subelement_name_new = cli.canonical_words[3]
	
	
	template_subelement_name     = '%s:%s' % (te.name, subelement_name)
	
	sub_te = TElement.objects.get(tenant=tenant, parent=te, name=template_subelement_name)

	try:
		element_template_sub_element_rename(sub_te, subelement_name_new)
	except InvalidConfigurationException, e:
		
		cli.error(str(e))



#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

from cli.tokens import T_TemplateSubElementOrAny, T_TemplateSubElement


mt_node_element_template_no = mt_base_element_template_no + [
	#
	# Parametri globali
	#
	T_Keyword('switch'  , help='Disable "switch" flag.', callback=cmd_node_template_set_attr, pipe=None,),
	T_Keyword('router'  , help='Disable "router" flag.', callback=cmd_node_template_set_attr, pipe=None,),
	#
	# Comandi per sotto-elementi in un node-template
	#
	T_Keyword('interface', help='Remove sub-interface definition.', pipe=None, subtree=[
		T_TemplateSubElement(ELEMENT_KIND_IFACE, help='Remove sub-interface definition.', pipe=None, subtree=[
			T_TemplateSubElement(ELEMENT_KIND_IFACE, help='Remove sub-interface definition.', pipe=None, callback=cmd_element_template_subelement_config_no)
		])
	]),
	T_Keyword('storage', help='Remove sub-storage definition.', pipe=None, subtree=[
		T_TemplateSubElement(ELEMENT_KIND_STORAGE, help='Remove sub-storage definition.', pipe=None, subtree=[
			T_TemplateSubElement(ELEMENT_KIND_STORAGE, help='Remove sub-storage definition.', pipe=None, callback=cmd_element_template_subelement_config_no)
		])
	]),
	T_Keyword('service', help='Remove sub-service definition.', pipe=None, subtree=[
		T_TemplateSubElement(ELEMENT_KIND_SERVICE, help='Remove sub-service definition.', pipe=None, subtree=[
			T_TemplateSubElement(ELEMENT_KIND_SERVICE, help='Remove sub-service definition.', pipe=None, callback=cmd_element_template_subelement_config_no)
		])
	]),
	T_Keyword('device', help='Remove sub-device definition.', pipe=None, subtree=[
		T_TemplateSubElement(ELEMENT_KIND_DEVICE, help='Remove sub-device definition.', pipe=None, subtree=[
			T_TemplateSubElement(ELEMENT_KIND_DEVICE, help='Remove sub-device definition.', pipe=None, callback=cmd_element_template_subelement_config_no)
		])
	]),
]


mt_node_element_template = mt_base_element_template + [
	#
	# Parametri globali
	#
	T_Keyword('switch'  , help='Enable "switch" flag.', callback=cmd_node_template_set_attr, pipe=None,),
	T_Keyword('router'  , help='Enable "router" flag.', callback=cmd_node_template_set_attr, pipe=None,),
	#
	# Comandi per sotto-elementi in un node-template
	#
	T_Keyword('interface' , help='Add sub-interface.', pipe=None, subtree=[
		T_TemplateSubElementOrAny(ELEMENT_KIND_IFACE, help='Add sub-interface definition.', pipe=None, callback=cmd_element_template_subelement_config, subtree=[
			T_Keyword('distinguisher' , help='Add sub-interface with distinguisher.', pipe=None, subtree=[
				T_Anyword(help='Add sub-interface with distinguisher.', pipe=None, callback=cmd_element_template_subelement_config, subtree=[
					T_Keyword('xform' , help='Add sub-interface with distinguisher and xform.', pipe=None, subtree=[
						T_Line(help='Add sub-interface with distinguisher and xform.', pipe=None, callback=cmd_element_template_subelement_config)
					])
				])
			])
		]),
	]),
	T_Keyword('storage' , help='Add sub-storage definition.', pipe=None, subtree=[
		T_TemplateSubElementOrAny(ELEMENT_KIND_STORAGE, help='Add sub-storage definition.', pipe=None, callback=cmd_element_template_subelement_config, subtree=[
			T_Keyword('distinguisher' , help='Add sub-interface with distinguisher.', pipe=None, subtree=[
				T_Anyword(help='Add sub-interface with distinguisher.', pipe=None, callback=cmd_element_template_subelement_config, subtree=[
					T_Keyword('xform' , help='Add sub-storage with distinguisher and xform.', pipe=None, subtree=[
						T_Line(help='Add sub-storage with distinguisher and xform.', pipe=None, callback=cmd_element_template_subelement_config)
					])
				])
			])
		]),
	]),
	T_Keyword('service' , help='Add sub-service definition.', pipe=None, subtree=[
		T_TemplateSubElementOrAny(ELEMENT_KIND_SERVICE, help='Add sub-service definition.', pipe=None, callback=cmd_element_template_subelement_config, subtree=[
			T_Keyword('distinguisher' , help='Add sub-service with distinguisher.', pipe=None, subtree=[
				T_Anyword(help='Add sub-service with distinguisher.', pipe=None, callback=cmd_element_template_subelement_config, subtree=[
					T_Keyword('xform' , help='Add sub-service with distinguisher and xform.', pipe=None, subtree=[
						T_Line(help='Add sub-service with distinguisher and xform.', pipe=None, callback=cmd_element_template_subelement_config)
					])
				])
			])
		]),
	]),
	T_Keyword('device' , help='Add sub-device definition.', pipe=None, subtree=[
		T_TemplateSubElementOrAny(ELEMENT_KIND_DEVICE, help='Add sub-device definition.', pipe=None, callback=cmd_element_template_subelement_config, subtree=[
			T_Keyword('distinguisher' , help='Add sub-device with distinguisher.', pipe=None, subtree=[
				T_Anyword(help='Add sub-device with distinguisher.', pipe=None, callback=cmd_element_template_subelement_config, subtree=[
					T_Keyword('xform' , help='Add sub-device with distinguisher and xform.', pipe=None, subtree=[
						T_Line(help='Add sub-device with distinguisher and xform.', pipe=None, callback=cmd_element_template_subelement_config)
					])
				])
			])
		
		]),
	]),
	
	T_Keyword('rename', help='Rename sub-elements.', pipe=None, subtree=[
		T_Keyword('interface', help='Rename sub-interface', pipe=None, subtree=[
			T_TemplateSubElement(ELEMENT_KIND_IFACE, help='Rename sub-interface', pipe=None, subtree=[
				T_Anyword(help='Rename sub-interface', pipe=None, callback=cmd_element_template_subelement_rename)
			])
		]),
		T_Keyword('storage', help='Rename sub-storage.', pipe=None, subtree=[
			T_TemplateSubElement(ELEMENT_KIND_STORAGE, help='Rename sub-interface', pipe=None, subtree=[
				T_Anyword(help='Rename sub-interface', pipe=None, callback=cmd_element_template_subelement_rename)
			])
		]),
		T_Keyword('service', help='Rename sub-service.', pipe=None, subtree=[
			T_TemplateSubElement(ELEMENT_KIND_SERVICE, help='Rename sub-interface', pipe=None, subtree=[
				T_Anyword(help='Rename sub-interface', pipe=None, callback=cmd_element_template_subelement_rename)
			])
		]),
		T_Keyword('device', help='Rename sub-device.', pipe=None, subtree=[
			T_TemplateSubElement(ELEMENT_KIND_DEVICE, help='Rename sub-interface', pipe=None, subtree=[
				T_Anyword(help='Rename sub-interface', pipe=None, callback=cmd_element_template_subelement_rename)
			])
		]),
	]),

	T_Keyword('no', help='Remove configuration sections for this template.', subtree=mt_base_element_template_no + mt_node_element_template_no
	)
]



#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_node_template_config_prompt(cli): 
	return __cmd_element_template_config_prompt(ELEMENT_KIND_NODE    , cli)
	
def cmd_interface_template_config_prompt(cli): 
	return __cmd_element_template_config_prompt(ELEMENT_KIND_IFACE   , cli)
	
def cmd_storage_template_config_prompt(cli): 
	return __cmd_element_template_config_prompt(ELEMENT_KIND_STORAGE , cli)
	
def cmd_service_template_config_prompt(cli): 
	return __cmd_element_template_config_prompt(ELEMENT_KIND_SERVICE , cli)
	
def cmd_device_template_config_prompt(cli): 
	return __cmd_element_template_config_prompt(ELEMENT_KIND_DEVICE  , cli)


def __cmd_element_template_config_prompt(kind, cli):

	env  = cli.env
	
	user = env['CURRENT_USER']
	
	tenant = None
	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	
	te_name = cli.canonical_words[1]

	te = element_template_get(kind, te_name, tenant=tenant)
	if te == None:
		te = element_template_create(kind, te_name, tenant=tenant, owner_user=user)


	env = cli.env.clone()
	
	env['PROMPT'] = '%s' % (PROMPT_SEP.join([tenant_name, te_name]),)
	
	env['CONFIG_ELEMENT_TEMPLATE'    ]  = te_name
	env['CONFIG_ELEMENT_TEMPLATE_KIND']  = kind
	
	
	if kind == ELEMENT_KIND_NODE:
		menu = mt_node_element_template
	else:
		menu = mt_other_element_template

	
	cmds = SORTED_KEYWORDS(menu) + [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))
	
	cli = cli.get_class()(cmds, env, debug_interpreter=env['DEBUG'])
	cli.cmdloop()	

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#



def cmd_node_template_remove(cli): 
	return __cmd_element_template_remove(ELEMENT_KIND_NODE    , cli)

def cmd_interface_template_remove(cli): 
	return __cmd_element_template_remove(ELEMENT_KIND_IFACE   , cli)

def cmd_storage_template_remove(cli): 
	return __cmd_element_template_remove(ELEMENT_KIND_STORAGE , cli)

def cmd_service_template_remove(cli): 
	return __cmd_element_template_remove(ELEMENT_KIND_SERVICE , cli)

def cmd_device_template_remove(cli): 
	return __cmd_element_template_remove(ELEMENT_KIND_DEVICE  , cli)


def __cmd_element_template_remove(kind, cli):

	user = cli.env['CURRENT_USER']

	# no *-template <t> <t>
	#tenant_name = cli.env['CONFIG_TENANT']

	tenant = None
	tenant_name = cli.env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)


	te1 = cli.canonical_words[2]
	te2 = cli.canonical_words[3]

	if te1 != te2:
		cli.error("Names do not match: '%s' '%s'" % (te1, te2))
		return
		
	te = element_template_get(kind, te1, tenant)
	if te is None:
		cli.error("Names do not match: '%s' '%s'" % (te1, te2))
	
	element_template_delete( te , user)


