# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from cli.prompts.defs import *

import re

from django.contrib.auth.models import User, Group

from core.db_utils import transaction_commit_on_success

from core.api.base import *
from core.api.base import tenant_addgroup
from core.models import TenantUserSettings

#from webui.capabilities import CAPABILITIES
#from webui.models import Capability

from core.capabilities import CAPABILITIES
from core.models import Capability


from cli.clilib.ioslike import *
from cli.tokens import *
from cli.textformat import *

from cli.clilib.ioslike import SORTED_KEYWORDS

from cli.prompts import cli_get_registered_menus

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def put_user_in_group(c):
	group_name = c.env['CONFIG_GROUP']
	
	user_name = c.canonical_words[1]
	
	group_adduser(group_name, user_name)


def remove_user_from_group(c):
	# no user <n>
	group_name = c.env['CONFIG_GROUP']
	user_name  =  c.canonical_words[2]
	
	group_removeuser(group_name, user_name)

def set_group_capability(cli):
	
	# min-priority <val>

	group_name  = cli.env['CONFIG_GROUP']
	tenant_name = cli.env['CURRENT_TENANT']

	tenant = tenant_get(tenant_name)
	group  = Group.objects.get(name=group_name)

	capability_code  = cli.canonical_words[1]
	
	if capability_code not in CAPABILITIES:
		cli.warning( "Invalid capability: %s" % (capability_code) )

	c, created = Capability.objects.get_or_create(tenant=tenant, group=group, code=capability_code)

def unset_group_capability(cli):

	group_name  = cli.env['CONFIG_GROUP']
	tenant_name = cli.env['CURRENT_TENANT']

	tenant = tenant_get(tenant_name)
	group  = Group.objects.get(name=group_name)

	capability_code  = cli.canonical_words[2]
	
	try:
		c = Capability.objects.get(tenant=tenant, group=group, code=capability_code)
		c.delete()
	except Capability.DoesNotExist, e:
		pass



def set_group_min_priority(cli):
	# min-priority <val>

	group_name   = cli.env['CONFIG_GROUP']
	tenant_name = cli.env['CURRENT_TENANT']

	level       = cli.canonical_words[1]
	
	if level == 'none':
		level = PRIORITY_NONE
	else:
		level = int(level)
		
	group  = Group.objects.get(name=group_name)
	tenant = tenant_get(tenant_name)
		
	_set_group_min_priority(group, tenant, level)
	
	return
		
@transaction_commit_on_success
def _set_group_min_priority(group, tenant, min_priority):
	#
	# FIX: 
	#	- move to api. 
	#	- Controllare che non ci siano race-conditions
	#	
	settings = TenantGroup.objects.get(tenant=tenant, group=group)	
	settings.default_min_priority = min_priority
	
	settings.concurrent_save(['default_min_priority'])


from webui.capabilities import CAPABILITIES_LIST

mt_config_group_tenant = [

	T_Keyword('capability'  , help='Activate capability', subtree=[
		T_Alternatives(  CAPABILITIES_LIST  , help='Activate capability', callback=set_group_capability, ),
		T_Line(help='Activate capability', callback=set_group_capability, ),

	]),

	T_Keyword('min-priority', help='Set default priority for this tenant', subtree=[
		T_Keyword('none', help='Hash in algo$salt$hash format', callback=set_group_min_priority, ),
		T_Number(         help='Hash in algo$salt$hash format', callback=set_group_min_priority, ),
	]),

	T_Keyword('no'  , help='', subtree=[
		T_Keyword('capability', help='Remove capability', subtree=[
			T_Alternatives(CAPABILITIES_LIST, help='Remove capability', callback=set_group_capability, ),
			T_Line(help='Remove capability', callback=unset_group_capability, ),
		]),
	]),
]


def cmd_group_tenant_config_prompt(cli):

	# tenant <name>
	#
	groupname = cli.env['CONFIG_GROUP']
	
	tenant_name = cli.canonical_words[1]
	
	group       = Group.objects.get(name=groupname)
	tenant      = tenant_get(tenant_name)
	if tenant == None:
		raise Exception("Invalid tenant")

	#
	# Action
	#
	tenant_addgroup(tenant_name, groupname)

	#
	# Subsection
	#
	env = cli.env.clone()

	env['PROMPT'] = '%s' % (PROMPT_SEP.join([groupname, tenant_name]),)
	
	env['CURRENT_TENANT'] = tenant_name

	cmds = SORTED_KEYWORDS(mt_config_group_tenant) + [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))
	
	cli = cli.get_class()( cmds, env, debug_interpreter=env['DEBUG'])
	cli.cmdloop()



def cmd_group_set_attr(cli):
	env = cli.env
	groupname = env['CONFIG_GROUP']
	
	val = cli.canonical_words[1]
	
	data = group_get_data(groupname)
	
	data.description = val
	data.save()

@transaction_commit_on_success
def cmd_group_tenant_remove(cli):
	
	# no tenant <name> 
	groupname = cli.env['CONFIG_GROUP']
	
	tenant_name = cli.canonical_words[2]
	
	tenant_removegroup(tenant_name, groupname)
	
	return
	
#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#


mt_config_group = [

	T_Keyword('description', help='Set first name, set empty to delete', subtree=[
		T_Line(help='First name', callback=cmd_group_set_attr),
	]),

	T_Keyword('user', help='Put user in group', subtree=[
		T_Usercompleter(help='Name of the group', callback=put_user_in_group,)
	]),

	
	T_Keyword('tenant', help='Tenant specific options', subtree=[
		T_Tenantcompleter(help='Tenant name', callback=cmd_group_tenant_config_prompt ),
	]),	
	
	
	T_Keyword('no', help='Tenant specific options', subtree=[
	
		T_Keyword('user', help='Put user in group', subtree=[
			T_Usercompleter(help='Name of the group', callback=remove_user_from_group,)
		]),
		
		T_Keyword('tenant', help='Tenant specific options', subtree=[
			T_VisibleTenantByGroupCompleter(help='Tenant name', callback=cmd_group_tenant_remove ),
		]),
	]),
	
]


def cmd_group_config_prompt(cli):
	
	groupname = cli.canonical_words[1]
	
	if not VALID_GROUP_NAME.match(groupname):
		raise Exception("Invalid name '%s'" % (groupname))
	
	
	g = group_get(groupname)
	if g == None:
		cli.outln("Creating group %s" % (groupname))
		g = group_create(groupname)
	
	env = cli.env.clone()
		
	env['PROMPT'] = '%s' % (PROMPT_SEP.join([groupname]),)
	
	env['CONFIG_GROUP'] = groupname
	env['CURRENT_GROUP'] = groupname
	
	cmds = SORTED_KEYWORDS(mt_config_group) + [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))
	
	cli = cli.get_class()( cmds, env, debug_interpreter=env['DEBUG'])
	cli.cmdloop()	


def cmd_delete_group(cli):
	
	name = cli.canonical_words[2]
	name2 = cli.canonical_words[3]
	
	if name != name2:
		cli.outln("The two names differ")
		return

	group_delete(name)


def cmd_group_config_rename(cli):

	#rename user <name> <name>

	name = cli.canonical_words[2]
	name2 = cli.canonical_words[3]
	
	g = Group.objects.get(name=name)

	g.name = name2
	g.save()

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def set_firstname(c):
	
	user_name = c.env['CONFIG_USER']

	fn = c.canonical_words[1]
	
	user = user_find(user_name)
	
	if  fn == user.first_name:
		return
		
	user.first_name = fn
	
	user_update(user)

#------------------------------------------------------------------------------#

def set_lastname(c):
	user_name = c.env['CONFIG_USER']

	v = c.canonical_words[1]

	user = user_find(user_name)
	
	if v == user.last_name:
		return	
		
	user.last_name = v
	
	user_update(user)

#------------------------------------------------------------------------------#

def disable(c):
	user_name = c.env['CONFIG_USER']

	user = user_find(user_name)

	user_set_enabled(user, False)

#------------------------------------------------------------------------------#

def enable(c):
	user_name = c.env['CONFIG_USER']

	user = user_find(user_name)

	user_set_enabled(user, True)

#------------------------------------------------------------------------------#

def set_email(c):

	user_name = c.env['CONFIG_USER']
	
	email = c.canonical_words[1]

	user = user_find(user_name)
	
	if user.email == email: # do not update
		return

	user.email = c.canonical_words[1]
	
	user_update(user)

#------------------------------------------------------------------------------#

def set_password(c):
	user_name = c.env['CONFIG_USER']

	password = c.canonical_words[1]
	
	#
	# FIX: use a function in API to validate cleartext password
	#
	if not password:
		raise Exception("Invalid password")
	
	user = user_find(user_name)

	user_set_password(user, password)

#------------------------------------------------------------------------------#

def set_secret(c):
	user_name = c.env['CONFIG_USER']

	password = c.canonical_words[1]

	user = user_find(user_name)
	
	# FIX: use a function in API to validate secret
	if user.password == password:
		return

	user_set_password(user, password, encripted=False)

#------------------------------------------------------------------------------#

def set_super(c):
	user_name = c.env['CONFIG_USER']
	user = user_find(user_name)
	if user.is_superuser:
		return
		
	user.is_superuser = True

	# FIX: change authorizations?
	user_update(user)
	
#------------------------------------------------------------------------------#

def clear_super(c):
	user_name = c.env['CONFIG_USER']
	user = user_find(user_name)
	if not user.is_superuser:
		return
		
	user.is_superuser = False
	
	# FIX: change authorizations?
	user_update(user)

#------------------------------------------------------------------------------#

def set_user_min_priority(cli):
	# min-priority <val>

	user_name   = cli.env['CONFIG_USER']
	tenant_name = cli.env['CURRENT_TENANT']

	level       = cli.canonical_words[1]
	
	if level == 'none':
		level = PRIORITY_NONE
	else:
		level = int(level)

	user   = user_find(user_name)
	tenant = tenant_get(tenant_name)

	_set_user_min_priority(user, tenant, level)
	
	return

@transaction_commit_on_success
def _set_user_min_priority(user, tenant, min_priority):
	#
	# FIX: 
	#	- move to api. 
	#	- controllare se non ci sono delle racecondition 
	from core.models import TenantUserSettings
	try:
		settings = TenantUserSettings.objects.get(tenant=tenant, user=user)
	except Exception, e:
		settings = TenantUserSettings(tenant=tenant, user=user)
		settings.save()
		
	settings.default_min_priority = min_priority
	
	settings.concurrent_save(['default_min_priority'])
	
#------------------------------------------------------------------------------#

def set_user_default_tenant(cli):

	user_name   = cli.env['CONFIG_USER']
	tenant_name = cli.env['CURRENT_TENANT']

	user   = user_find(user_name)
	tenant      = tenant_get(tenant_name)
	if tenant == None:
		raise Exception("Invalid tenant")

	is_default = False
	if cli.canonical_words[0] == 'default':
		is_default = True

	_set_user_default_tenant(user, tenant, is_default)

@transaction_commit_on_success
def _set_user_default_tenant(user, tenant, is_default):
	# FIX: 
	#	- move to api. 
	#	- controllare se non ci sono delle racecondition 
	from core.models import TenantUserSettings

	for setting in TenantUserSettings.objects.exclude(tenant=tenant):
		setting.is_default = False
		setting.concurrent_save(['is_default'])

	settings, created = TenantUserSettings.objects.get_or_create(tenant=tenant, user=user)
	
	# "Settings" dovrebbe gia' essere sul DB (con o senza created=True) quindi possiamo aggiornare col metodo concurrent_save
	settings.is_default = is_default

	settings.concurrent_save(['is_default'])

#------------------------------------------------------------------------------#

def cmd_user_tenant_remove(cli):

	# no tenant <name>
	user_name = cli.env['CONFIG_USER']

	tenant_name = cli.canonical_words[2]

	user   = User.objects.get(username=user_name)
	tenant = tenant_get(tenant_name)

	TenantUserSettings.objects.get(tenant=tenant, user=user).delete()	

#------------------------------------------------------------------------------#


mt_config_user_tenant = [

	T_Keyword('default', help='Set as default tenant for this user', callback=set_user_default_tenant),

	T_Keyword('min-priority', help='Set default priority for this tenant', subtree=[
		T_Keyword('none', help='Hash in algo$salt$hash format', callback=set_user_min_priority, ),
		T_Number(         help='Hash in algo$salt$hash format', callback=set_user_min_priority, ),
	]),

	T_Keyword('no', help='Unset setting', subtree=[
		T_Keyword('default', help='Unset defaul tenant', callback=set_user_default_tenant, ),
	]),

]



#------------------------------------------------------------------------------#

def cmd_user_tenant_config_prompt(cli):

	# tenant <name>
	#
	user = cli.env['CONFIG_USER']
	
	tenant_name = cli.canonical_words[1]
	tenant      = tenant_get(tenant_name)
	if tenant == None:
		raise Exception("Invalid tenant")

	#
	# FIX: move to api
	#
	settings, created = TenantUserSettings.objects.get_or_create(tenant=tenant, user=user)

	#
	# Subsection
	#
	env = cli.env.clone()
		
	env['PROMPT'] = '%s' % (PROMPT_SEP.join([user.username, tenant_name]),)
	
	env['CURRENT_TENANT'] = tenant_name
	
	cmds = SORTED_KEYWORDS(mt_config_user_tenant) + [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))
	
	cli = cli.get_class()( cmds, env, debug_interpreter=env['DEBUG'])
	cli.cmdloop()

#------------------------------------------------------------------------------#

def cmd_delete_user(c):

	if c.canonical_words[2] != c.canonical_words[3]:
		c.outln("The two names differ")
		return

	user_delete(c.canonical_words[2])

#------------------------------------------------------------------------------#


def cmd_user_config_rename(cli):

	#rename user <name> <name>

	name = cli.canonical_words[2]
	name2 = cli.canonical_words[3]
	
	if name == name2:
		return
		
	user_rename(name, name2)
	
#------------------------------------------------------------------------------#


mt_config_user = [

	T_Keyword('disable', help='Disable the user', callback=disable ),
	
	T_Keyword('email', help='Set user email', subtree=[
		T_Anyword(help='Email address, like user@domain.com', callback=set_email,),
	]),
	
	T_Keyword('enable', help='Enable the user', callback=enable,),
	
	T_Keyword('firstname', help='Set first name, set empty to delete', subtree=[
		T_Anyword(help='First name', callback=set_firstname),
	]),
	
	#T_Keyword('group', help='Put user in group', subtree=[
	#	T_Groupcompleter(help='Name of the group', callback=put_user_in_group,),
	#]),
	
	T_Keyword('lastname', help='Set last name, set empty to delete', subtree=[
		T_Anyword(help='Last name', callback=set_lastname ),
	]),
	
	T_Keyword('password', help='Set cleartext password, will show as hash in config', subtree=[
		T_Anyword(help='Cleartext password', callback=set_password,
		),
	]),
	
	T_Keyword('secret', help='Set password as hash', subtree=[
		T_Anyword(help='Hash in algo$salt$hash format', callback=set_secret, ),
	]),
	
	T_Keyword('superuser', help='Make this user a superuser (permission to see and do anything)',
		callback=set_super,
	),

	T_Keyword('tenant', help='Tenant specific options', subtree=[
		T_VisibleTenantByUserCompleter(help='Tenant name', callback=cmd_user_tenant_config_prompt ),
	]),


	T_Keyword('no', help='Negate a command', subtree=[

	#	T_Keyword('group', help='Remove user from group', subtree=[
	#		T_Groupcompleter(help='Name of the group', callback=remove_user_from_group ),
	#	]),

		T_Keyword('tenant', help='Tenant specific options', subtree=[
			T_VisibleTenantByUserCompleter(help='Tenant name', callback=cmd_user_tenant_remove ),
		]),

		T_Keyword('superuser', help='Revoke this user the permission to see and do anything', callback=clear_super ),
	]),
]


def cmd_user_config_prompt(cli):
	
	user_name = cli.canonical_words[1]
	
	try:
		user = User.objects.get(username=user_name)
	except User.DoesNotExist, e:
		
		if not VALID_USER_NAME.match(user_name):
			cli.outln("INVALID USER NAME") # FIX: better message
			return
			
		cli.outln("Creating username %s" % (user_name))
		
		user = User(username=user_name, is_active=True, is_staff=False, is_superuser=False)
		user.set_unusable_password()
		user.save()
		
		#Siteattr.update_auth_config_timestamp()
	else:
		cli.debug("Modifying configuration for user %s" % user.username)
	
	env = cli.env.clone()
	
	env['PROMPT'] = '%s' % (PROMPT_SEP.join([user_name]),)
	
	env['CONFIG_USER'] = user
	
	#env['CURRENT_USER'] = user
	
	cmds = SORTED_KEYWORDS(mt_config_user) + [BLANK_ENTRY] + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))
	
	cli = cli.get_class()( cmds, env, debug_interpreter=env['DEBUG'])
	cli.cmdloop()

