# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from django.utils.translation import ugettext as _

from django.conf import settings

from sanet_common.sanetdefs.constants import *

from sanet_common.sanetdefs.constants import PARAM_TYPE_STR
from sanet_common.sanetdefs.constants import PARAM_TYPE_FLOAT
from sanet_common.sanetdefs.constants import PARAM_TYPE_BOOL
from sanet_common.sanetdefs.constants import convert_to_param_type

from sanet_common.sanetdefs.constants import *

from sanet_common.termutils import term_color, TERM_COLOR_RED, TERM_COLOR_GREEN, TERM_COLOR_YELLOW

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

PROMPT_SEP = '/'

"""
RESERVED_WORDS = [
	'tenant',
	'agent',
	'user', 'group',
	'node', 'interface', 'storage', 'service', 'device',
	'datagroup','datasource','timegraph','serie',
	'parameter',
]

def cli_assert_reserved_word(w):
	if s in RESERVED_WORDS:
		raise Exception("'%s' is a reserved word" % (w))
"""

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def PIQJ_IFDEFINED(cli, l, name, v):
	if v != None:
		cli.piqj(l, name , v)

CLI_TRUE  = "true"
CLI_FALSE = "false"

def CLI_BOOL(v):
	if v != None:
		if bool(v):
			return CLI_TRUE 
		else:
			return CLI_FALSE

	return v


def CLI_INT(v):
	if v != None:
		return str(v)
	return v

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

LINK_TYPE_TO_CLI = {
	LINK_TYPE_VIRTUAL: 'virtual',
	LINK_TYPE_L2     : 'physical',
	LINK_TYPE_L3     : 'network',
}

LINK_ALTERNATIVES = (
	('virtual' , _("Virtual link")  ),
	('physical', _("Physical link") ),
	('network' , _("Network link")  ),
)

LINK_CLI_TO_TYPE = dict( (v,k) for k,v in LINK_TYPE_TO_CLI.items() )

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def CLI_FIELD_TO_DATAGROUP_FIELD(f_name):
	return f_name.replace('-', '_')

def DATAGROUP_FIELD_TO_CLI_FIELD(f_name):
	return f_name.replace('_','-')


def CLI_FIELD_TO_DATASOURCE_FIELD(f_name):
	return f_name.replace('-', '_')

def DATASOURCE_FIELD_TO_CLI_FIELD(f_name):
	return f_name.replace('_','-')
	

def CLI_FIELD_TO_CONDITION_FIELD(f_name):
	return f_name.replace('-', '_')
	
def CONDITION_FIELD_TO_CLI_FIELD(f_name):
	return f_name.replace('_', '-')


FALLBACK_UP = 'UP'
FALLBACK_DN = 'DN'
_FALLBACK_TO_STATE = {
	FALLBACK_UP: str(CSTATE_UP),
	FALLBACK_DN: str(CSTATE_DN),
}
_STATE_TO_FALLBACK = {
	str(CSTATE_UP): FALLBACK_UP,
	str(CSTATE_DN): FALLBACK_DN,
}

def CLI_FIELD_VALUE_TO_CONDITION_FIELD_VALUE(f_name, f_str_value):
	if f_name == 'uncheckable_fallback':
		return _FALLBACK_TO_STATE[f_str_value]
	return f_str_value
	
def CONDITION_FIELD_VALUE_TO_CLI_FIELD_VALUE(f_name, f_value):
	if f_name == 'uncheckable_fallback':
		return _STATE_TO_FALLBACK[f_value]
	return f_value



	
def CLI_FIELD_TO_TIMEGRAPH_FIELD(f_name):
	return f_name.replace('-', '_')
	
def TIMEGRAPH_FIELD_TO_CLI_FIELD(f_name):
	return f_name.replace('_','-')		


def CLI_FIELD_TO_TIMEGRAPHSERIE_FIELD(f_name):
	if   f_name == 'var-value'     : f_name = 'variable_value'
	elif f_name == 'computed-value': f_name = 'computed_value_expr'
	return f_name.replace('-', '_')

def TIMEGRAPHSERIE_FIELD_TO_CLI_FIELD(f_name):
	if   f_name == 'variable_value': f_name = 'var-value'
	elif f_name == 'computed_value_expr': f_name = 'computed-value'
	return f_name.replace('_', '-')
	
#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#


CLI_BOOL_TO_PARAM_BOOL = {
	CLI_TRUE : PARAM_BOOL_TRUE,
	CLI_FALSE: PARAM_BOOL_FALSE,
}	

PARAM_BOOL_TO_CLI_BOOL = {
	PARAM_BOOL_TRUE  : CLI_TRUE,
	PARAM_BOOL_FALSE : CLI_FALSE,
}


CLI_PTYPE_TO_NAME_DESCR = {
	PARAM_TYPE_STR       : ('str' , _("String value") ),
	PARAM_TYPE_FLOAT     : ('num' , _("Numeric value")),
	PARAM_TYPE_BOOL      : ('bool', _("Boolean value")),
}

def CLI_PTYPE_TO_NAME(ptype):
	return CLI_PTYPE_TO_NAME_DESCR[ptype][0]

CLI_NAME_TO_TYPE = dict([ (name_descr[0], t) for t,name_descr in CLI_PTYPE_TO_NAME_DESCR.items()] )


def CLI_PTYPE_VAL_TO_CLI_VAL(ptype, pval):

	if ptype == PARAM_TYPE_BOOL:
		return PARAM_BOOL_TO_CLI_BOOL[pval]
	else:
		return pval



def cli_input_to_real_param_value(param_type_name, str_input_value):
	
	t = CLI_NAME_TO_TYPE[ param_type_name ]
	
	if t == PARAM_TYPE_BOOL:
		try:
			return CLI_BOOL_TO_PARAM_BOOL[str_input_value]
		except KeyError, e:
			raise Exception(_("Invalid value '%(value)s' for type '%(type)s'") % { 'value': str_input_value, 'type': param_type_name })
	else:
		return str_input_value
		

def convert_param_value( param_type_name, str_value ):

	t = CLI_NAME_TO_TYPE[ param_type_name ]

	v = convert_to_param_type( t , str_value )

	return (t, v)



#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

CLI_CLUSTER_ROLE_TO_NAME_DESCR = {
	CLUSTER_ROLE_MASTER    : ('master'    , _('Main element' )     ),
	CLUSTER_ROLE_DEPENDENT : ('dependent' , _('Dependent element') )
}

def CLI_CLUSTER_ROLE_TO_NAME(role):
	return CLI_CLUSTER_ROLE_TO_NAME_DESCR[role][0]

CLI_NAME_TO_ROLE = dict([ (name_descr[0], t) for t,name_descr in CLI_CLUSTER_ROLE_TO_NAME_DESCR.items()] )


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

CLI_TG_STYLE_TO_NAME_DESCR = {
	TIMEGRAPH_SERIE_STYLE_LINE      : ( 'line'      , _('Line style') ),
	TIMEGRAPH_SERIE_STYLE_AREA      : ( 'area'      , _('Area style') ),
	TIMEGRAPH_SERIE_STYLE_TICK      : ( 'tick'      , _('Tick style') ),
	TIMEGRAPH_SERIE_STYLE_INVISIBLE : ( 'invisible' , _('Invisible' ) ),
}





#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

_kind_to_xalias_section = {
	ELEMENT_KIND_IFACE  : 'interfaces',
	ELEMENT_KIND_STORAGE: 'storages',
	ELEMENT_KIND_SERVICE: 'services',
	ELEMENT_KIND_DEVICE : 'devices',
}

def xform_aliases(section):
	XFORMS_ALIASES = settings.XFORMS_ALIASES
	try:
		return XFORMS_ALIASES[section].keys()
	except KeyError, e:
		return []

def xform_alias_to_xform(section, s):
	XFORMS_ALIASES = settings.XFORMS_ALIASES
	try:
		return XFORMS_ALIASES[section][s][0]
	except KeyError, e:
		return s

def xform_to_xform_alias(section, s):
	try:
	
		XFORMS_ALIASES = settings.XFORMS_ALIASES
		
		for xform_name, xforms in XFORMS_ALIASES[section].items():
			for xform in xforms:
				if xform == s:
					return xform_name
		return s

	except KeyError, e:
		return s

def xform_alias_to_xfrom_by_kind(kind, s):
	return xform_alias_to_xform( _kind_to_xalias_section[kind], s)

def xform_to_xform_alias_by_kind(kind, s):
	return xform_to_xform_alias( _kind_to_xalias_section[kind], s)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

SYSTEM_FIELDS = [
		'uuid',
]

COMMON_FIELDS = SYSTEM_FIELDS + [
		'name',
		'description',
		'ctime',
		'mtime',
		'icon',
		'priority',
		'parent',
		'agent',
		'notify_address',
]

IFACE_FIELDS = [
	'distinguisher',
	'xform',
	'ifindex',
	'description_snmp',
]

STORAGE_FIELDS = [
	'distinguisher',
	'xform',
	'stindex',
]

SERVICE_FIELDS = [
	'distinguisher',
	'xform',
	'swrunindex',
]

DEVICE_FIELDS = [
	'distinguisher',
	'xform',
	'devindex',
]



NODE_FIELDS = [
		 'use_ipv4'
		,'dns_name'
		,'dns6_name'
		,'ip4_mgt'
		,'ip6_mgt'
		,'effective_ip'
]

NODE_FIELDS += [
		'timezone',
		'dft_timeout',
		'is_switch',
		'is_router',
]

SNMP_FIELDS = [		
		'snmp_version',
		'snmp_port',
		'snmp_community',
		'snmp_v3_authdata',
]

CLUSTER_FIELDS = [

		 'cluster'
		,'cluster_role'
		,'cluster_distinguisher'
		,'cluster_xform'
		,'cluster_instance'
		
		,'cluster_instance_time'
		,'cluster_element'

]

HIDDEN_FIELDS = [
	'tenant',
	'mtime',
	'kind',
	'monitoredelement_ptr',
	'agent_runningon',
]


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#



def COLORED_STATUS_TEXT(status, text):
	if   status == CSTATE_DN: return term_color( TERM_COLOR_RED   , text)
	elif status == CSTATE_FA: return term_color( TERM_COLOR_YELLOW, text)
	elif status == CSTATE_UP: return term_color( TERM_COLOR_GREEN , text)
	else:
		return text 

def COLORED_DSSTATUS_TEXT(status, text):
	if   status == DATASOURCE_STATUS_ERROR      : return term_color( TERM_COLOR_RED   , text)
	elif status == DATASOURCE_STATUS_STRANGE_VAL: return term_color( TERM_COLOR_YELLOW, text)
	elif status == DATASOURCE_STATUS_OK         : return term_color( TERM_COLOR_GREEN , text)
	else:
		return text 

def ELEMENT_STATUS_TEXT(status, text):
	if   status == "DN": return term_color( TERM_COLOR_RED   , text)
	elif status == "UP": return term_color( TERM_COLOR_GREEN , text)
	elif status == "FA": return term_color( TERM_COLOR_YELLOW, text) 
	else:
		return text 


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

VERBOSE_OPTIONS = {
	'normal': (10, 'Debugging mode' ),
	'full'  : ( 1, 'Very verbose debugging mode'),
}

VERBOSE_OPTIONS_LABEL = [ (k, v[1]) for k,v in VERBOSE_OPTIONS.items() ]

def _cli_verbose_mode_to_loglevel(canonical_words):

	loglevel = 20

	# verbose [<mode>]
	verbose  = True if (len(canonical_words) > 0 and canonical_words[0] == 'verbose') else False
	if verbose:
		loglevel = 10
		if len(canonical_words) > 1:
			mode     = canonical_words[1]
			loglevel = VERBOSE_OPTIONS.get(mode,(20,''))[0]

	return loglevel

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

CLI_PERM_TO_AUTH_PERM = {
	# keyword      value           Description
	'read-write': (AUTH_READWRITE, 'Read/Write'),
	'read-only' : (AUTH_READONLY , 'Read only' ), 
}

CLI_PERM_OPTIONS = [ (k, v[1]) for k,v in CLI_PERM_TO_AUTH_PERM.items() ]

AUTH_PERM_TO_CLI_PERM = dict( [ (value[0],label) for (label, value) in CLI_PERM_TO_AUTH_PERM.items() ])

