# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from django.utils.translation import ugettext as _

#from sanet_common.sanetdefs.constants import ELEMENT_KIND_STR

from sanet_common.sanetdefs.constants import ELEMENT_KIND_NODE, ELEMENT_KIND_IFACE, ELEMENT_KIND_STORAGE, ELEMENT_KIND_SERVICE, ELEMENT_KIND_DEVICE

from cli.clilib.ioslike import *
from cli.textformat import *

from core.exceptions import ApplicationException

from core.api.base import agent_select_all
from core.api.base import agent_get

from cli.prompts.common import output_table

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_agents(cli):

	# show agent [<name>]

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	current_user   = cli.env.get('CURRENT_USER'  , None)
	cols = cli.env['COLS']

	if len(cli.canonical_words) < 3:

		agents = agent_select_all(current_tenant)

		r = [[
			[_('TIME'), 12],

			[_('UUID'), 32],
			[_('NAME'), -4],
			[_('PRIMARY'), 7],
			[_('ENABLED'), 3],
			[_('REMOTE'), 3],
			[_('THREADS'), 30],
			[_('UPDATE'), 5],
			[_('DESCRIPTION'), 30],
		]]
		for a in agents:
			r.append([
				a.mtime, 
				a.uuid, 
				
				a.name,
				'*' if a.is_primary else '',
				'*' if a.is_enabled else '',
				'*' if a.is_remote  else '',
				a.threads,
				a.config_update_minperiod,
				a.description,
			])

		output_table(cli, r)

def cmd_show_agent(cli):

	# show agent <name> <cmd>
	tenant_name = cli.env['CURRENT_TENANT']

	agent_name = cli.canonical_words[2]

	agent = agent_get(agent_name, tenant_name=tenant_name)

	totals = agent.monitored_elements_counters()

	totals_by_kind = dict([ (e['kind'],e['total']) for e in totals ])

	cli.prnt("Nodes     : %s" % (totals_by_kind.get(ELEMENT_KIND_NODE   , 0)))
	cli.prnt("Interfaces: %s" % (totals_by_kind.get(ELEMENT_KIND_IFACE  , 0)))
	cli.prnt("Storages  : %s" % (totals_by_kind.get(ELEMENT_KIND_STORAGE, 0)))
	cli.prnt("Services  : %s" % (totals_by_kind.get(ELEMENT_KIND_SERVICE, 0)))
	cli.prnt("Devices   : %s" % (totals_by_kind.get(ELEMENT_KIND_DEVICE , 0)))

        # FIX: disabled because buffering support is not working
	#if agent.is_remote:
	#if 1:
	#	_print_agent_remote_heuristics(cli, agent, totals_by_kind)
	

def _print_agent_remote_heuristics(cli, agent, totals_by_kind):

	UPDATE_EVENT_SIZE = {
		ELEMENT_KIND_NODE    : 2000,
		ELEMENT_KIND_IFACE   : 1000,
		ELEMENT_KIND_STORAGE : 1000,
		ELEMENT_KIND_SERVICE : 1000,
		ELEMENT_KIND_DEVICE  : 1000,
	}

	min_buffer_size = 0
	min_buffer_size += totals_by_kind.get(ELEMENT_KIND_NODE   , 0) * UPDATE_EVENT_SIZE[ ELEMENT_KIND_NODE    ]
	min_buffer_size += totals_by_kind.get(ELEMENT_KIND_IFACE  , 0) * UPDATE_EVENT_SIZE[ ELEMENT_KIND_IFACE   ]
	min_buffer_size += totals_by_kind.get(ELEMENT_KIND_STORAGE, 0) * UPDATE_EVENT_SIZE[ ELEMENT_KIND_STORAGE ]
	min_buffer_size += totals_by_kind.get(ELEMENT_KIND_SERVICE, 0) * UPDATE_EVENT_SIZE[ ELEMENT_KIND_SERVICE ]
	min_buffer_size += totals_by_kind.get(ELEMENT_KIND_DEVICE , 0) * UPDATE_EVENT_SIZE[ ELEMENT_KIND_DEVICE  ]

	CHECK_PER_MINUTE = 1
	MIN_NUM_EVENTS = CHECK_PER_MINUTE * 60

	cli.prnt("Minimum send buffer size (1h) = %s" % (min_buffer_size * MIN_NUM_EVENTS) )


def cmd_show_agent_info(cli):

	# show agent <name> <cmd>
	tenant_name = cli.env['CURRENT_TENANT']

	agent_name = cli.canonical_words[2]

	agent = agent_get(agent_name, tenant_name=tenant_name)

	cmd = cli.canonical_words[3]
	args = cli.canonical_words[4:]

	if cmd == 'schedule':
		cmd = 'show_schedule'

	try:

		from django.core.management import call_command

		params = [cmd] + args

		options = {
			'tenant'        : tenant_name,
			'agent'         : agent_name,
			'interactive'   : False, 
			'verbosity'     : True, 
			'show_traceback': True,
		}

		call_command('agentinfo', *params, **options)

	except Exception, e:

		raise ApplicationException(e)
