# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from django.utils.translation import ugettext as _

from cli.clilib.ioslike import *
from cli.textformat import *

from cli.prompts.defs import *

from core.api.base import tenant_get
from core.models import Cluster

from cli.prompts.common import output_table

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_clusters(cli):

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	current_user   = cli.env.get('CURRENT_USER'  , None)
	cols = cli.env['COLS']

	tenant = tenant_get(current_tenant)
	tenant.db_set_current_schema()
	
	
	# show clusters [<name>]
	if len(cli.canonical_words) < 3:

		clusters = Cluster.get_visible_by_user(tenant, current_user)

		r = [[
			[_('TIME'), 12],

			[_('NAME'), -4],
			[_('COUNT'), 5],
			[_('DESCRIPTION'), 32],
		]]
		for c in clusters:
			r.append([
				c.mtime, 
				
				c.name,
				c.monitoredelement_set.count(),
				c.description,
			])

		output_table(cli, r)
		
	else:
		name = cli.canonical_words[2]
		
		clusters = Cluster.get_visible_by_user(tenant, current_user)
		cluster = clusters.get(name=name)
		
		cli.prnt(
			generic_details(
				cluster,
				#suppress=[
				#],
			)
		)
		
		cli.prnt("")

		r = [[
			[_('NODE'), 12],
			[_('ROLE'), -4],
			[_('DISTINGUISHER'), 5],
			[_('INSTANCE'), 10],
			[_('XFORM'), 32],
		]]
		
		for n in cluster.monitoredelement_set.order_by('name', 'cluster_role').all():
			r.append([
				n.name, 
				CLI_CLUSTER_ROLE_TO_NAME(n.cluster_role), 
				n.cluster_distinguisher, 
				n.cluster_instance,
				n.cluster_xform
			])
		cli.prnt(text_table(r, cols))
