# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os
import operator 

from django.contrib.auth.models import User, Group

from sanet_common.sanetdefs.constants import *

# FIX: 
from core.models import *

from core.api.base import *
from core.api.templates import *

from cli.clilib.ioslike import T_Keyword, T_Anyword
from cli.tokens import *

from cli.prompts.defs import *

from cli.prompts import cli_get_registered_hooks


DEBUG = settings.DEBUG_CONFIG or os.environ.get('DEBUG_CONFIG', False)

#------------------------------------------------------------------------------#
#  DATAGROUP CONFIGURATION                                                     #
#------------------------------------------------------------------------------#

def base_cmd_show_conf_datagroup_templates(cli, source, tenant, name=None):

	user = cli.env['CURRENT_USER']

	#
	# FIX: Estrarre i datagroup in ordine di dipendenza e di nome
	#
	if name != None:
		tdgs = TDataGroup.objects.filter(name=name, source=source, tenant=tenant)
	else:
		#tdgs = TDataGroup.objects.filter(source=source, tenant=tenant).order_by('name')
		tdgs = TDataGroup.get_visible_by_user(user, tenant).filter(source=source) #, tenant=tenant)

	tdgs = tdgs.order_by('name')

	if len(tdgs) == 0:
		if source == SOURCE_LIBRARY:
			cli.error("No template found in library")
		else:
			cli.error("No template found in tenant %s" % (tenant.name) )
		return

	for tdg in tdgs:
		do_show_conf_one_datagroup_template(cli, tdg)


def do_show_conf_one_datagroup_template(cli, dgt, level=0):

	cli.piqj(level, cli.CLI_KEYWORD("datagroup-template"), dgt.name)

	level += 1

	cli.piqj(level, cli.CLI_KEYWORD("source"), SOURCE_DESCR[dgt.source]['name'])

	if dgt.parent != None:
		p = dgt.parent
		if dgt.source != p.source or dgt.source == SOURCE_LIBRARY:
			cli.piqj(level, cli.CLI_KEYWORD("parent"), p.name, cli.CLI_KEYWORD(SOURCE_DESCR[p.source]['name']))
		else:
			cli.piqj(level, cli.CLI_KEYWORD("parent"), p.name)


	#cli.piqj(level, cli.CLI_COMMENT("# >>>>>>>>>>>> Permission >>>>>>>>>>>>"))
	if dgt.owner:
		cli.piqj(level, cli.CLI_KEYWORD("owner"), dgt.owner.username)

	for gp in dgt.get_group_permissions():
			cli.piqj(level, cli.CLI_KEYWORD("group"), gp.group.name, cli.CLI_KEYWORD( AUTH_PERM_TO_CLI_PERM[gp.auth_type] ))

	#cli.piqj(level, cli.CLI_COMMENT("# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<"))
	if dgt.description:
		cli.piqj(level, cli.CLI_KEYWORD("description"), dgt.description)

	fields = dgt.get_local_configured_field_dict()
	for f_name, f_val in fields.items():
		cli.piqj(level, cli.CLI_KEYWORD(DATAGROUP_FIELD_TO_CLI_FIELD(f_name)), f_val)
	
	params  = dgt.get_local_configured_parameters()
	for p in params:
		cli.piqj(level, cli.CLI_KEYWORD("parameter"), p.name, CLI_PTYPE_TO_NAME(p.vtype), CLI_PTYPE_VAL_TO_CLI_VAL(p.vtype, p.value))	


	datasources = TDataSource.objects.filter(tdatagroup=dgt).order_by('name')
	for dst in datasources:
		show_conf_datasource_template(cli, level, dst)

	conditions = TCondition.objects.filter(tdatagroup=dgt).order_by('name')
	for ct in conditions:
		show_conf_condition_template(cli, level, ct)

	timegraphs = TTimeGraph.objects.filter(tdatagroup=dgt)
	for tg in timegraphs:
		show_conf_timegraph_template(cli, level, tg)

	level -= 1
	cli.piqj(level, cli.CLI_KEYWORD("exit"))

def show_conf_datasource_template(cli, l, dst):

	cli.piqj(l, cli.CLI_KEYWORD("datasource"), dst.name)

	fields = TDataSourceFieldValue.objects.filter(tdatasource=dst).order_by('tfield__name')
	for f in fields:
		cli.piqj(l+1, cli.CLI_KEYWORD(DATASOURCE_FIELD_TO_CLI_FIELD(f.name)), f.value)

	cli.piqj(l, cli.CLI_KEYWORD("exit"))

def show_conf_condition_template(cli, l, ct):
	
	cli.piqj(l, cli.CLI_KEYWORD("condition"), ct.name)

	fields = TConditionFieldValue.objects.filter(tcondition=ct).order_by('tfield__name')
	
	for f in fields:
		cli.piqj(l+1, cli.CLI_KEYWORD(CONDITION_FIELD_TO_CLI_FIELD(f.name)), CONDITION_FIELD_VALUE_TO_CLI_FIELD_VALUE(f.name, f.value) )

	cli.piqj(l, cli.CLI_KEYWORD("exit"))
	

def show_conf_timegraph_template(cli, l, ttg):

	cli.piqj(l, cli.CLI_KEYWORD("timegraph"), ttg.name)

	fields = TTimeGraphFieldValue.objects.filter(ttimegraph=ttg).order_by('tfield__name')
	
	for f in fields:
		cli.piqj(l+1, cli.CLI_KEYWORD(TIMEGRAPH_FIELD_TO_CLI_FIELD(f.tfield.name)), f.value)
		
	series = TTimeGraphSerie.objects.filter(ttimegraph=ttg).order_by('name')
	for s in series: 
		show_conf_timegraph_serie_template(cli, l+1, s)

	cli.piqj(l, cli.CLI_KEYWORD("exit"))

def show_conf_timegraph_serie_template(cli, l, serie):
	
	cli.piqj(l, cli.CLI_KEYWORD("serie") , serie.name)
	
	fields = TTimeGraphSerieFieldValue.objects.filter(ttimegraphserie=serie).order_by('tfield__name')
	for f in fields:
		cli.piqj(l+1, cli.CLI_KEYWORD(TIMEGRAPHSERIE_FIELD_TO_CLI_FIELD(f.tfield.name)), f.value)

	cli.piqj(l, cli.CLI_KEYWORD("exit") )

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_conf_users(cli):

	_cmd_show_conf_users(cli)

def _cmd_show_conf_users(cli, normal_data=True, auth_data=True ):

	# show configuration users <name>
	if len(cli.canonical_words) == 4:

		name = cli.canonical_words[3]
		users = User.objects.filter(username=name).all()
	else:
		users = User.objects.all()

	for u in users:
		_show_config_oneuser(cli, u, normal_data=normal_data, auth_data=auth_data )


def _show_config_oneuser(cli, u, level=0, normal_data=True, auth_data=True ):

	l = level
	cli.piqj(l, cli.CLI_KEYWORD('user'), u.username)
	l += 1

	if normal_data:

		if not u.is_active:
			cli.piqj(l, cli.CLI_KEYWORD('disable'))

		if u.is_superuser:
			cli.piqj(l, cli.CLI_KEYWORD('superuser'))

		if u.email:
			cli.piqj(l, cli.CLI_KEYWORD('email'), u.email)

		if u.first_name:
			cli.piqj(l, cli.CLI_KEYWORD('firstname'), u.first_name)
			
		if u.last_name:
			cli.piqj(l, cli.CLI_KEYWORD('lastname'), u.last_name)

		if u.password:
			cli.piqj(l, cli.CLI_KEYWORD('secret'), u.password)

	if auth_data:
		_show_config_oneuser_auth(cli, u, level=l)

	l -= 1
	cli.piqj(l, cli.CLI_KEYWORD('exit'))

def _show_config_oneuser_auth(cli, u, level=0):
	
	l = level

	for ts in TenantUserSettings.objects.filter(user=u):
		
		if not ts.is_configured:  # is there any configuration ?
			continue

		cli.piqj(l, cli.CLI_KEYWORD('tenant'), ts.tenant.name)
		l += 1
		
		if ts.is_default:
			cli.piqj(l, cli.CLI_KEYWORD('default') )

		if ts.default_min_priority != PRIORITY_NONE:
			cli.piqj(l, cli.CLI_KEYWORD('min-priority') , str(ts.default_min_priority))

		l -= 1
		cli.piqj(l, cli.CLI_KEYWORD('exit'))

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_conf_groups(cli, show_user_section=True, show_tenant_section=True):
	
	# show configuration groups <name>
	if len(cli.canonical_words) == 4:

		name = cli.canonical_words[3]
		groups = Group.objects.filter(name=name).all()
	else:
		groups = Group.objects.all()

	if show_user_section:
		cli.pij(0, cli.CLI_COMMENT('# Global configuration') )
		for g in groups:
			_show_config_one_group(cli, g, show_users_section=True, show_tenant_section=False)
			print ""

	if show_tenant_section:
		
		cli.pij(0, cli.CLI_COMMENT('# Tenant dependent') )
		for g in groups:
			_show_config_one_group(cli, g, show_users_section=False, show_tenant_section=True)
			print ""

	return

def _show_config_one_group(cli, group, level=0, show_users_section=True, show_tenant_section=True):
	l = level
	
	cli.piqj(l, cli.CLI_KEYWORD('group'), group.name)
	l += 1
	
	groupdata = group_get_data(group.name)
	
	if show_users_section:
		
		if groupdata.description:
			cli.piqj(l, cli.CLI_KEYWORD('description'), groupdata.description)
	
		## group gname
		for u in group.user_set.order_by('username'):
			cli.piqj(l, cli.CLI_KEYWORD('user'), u.username)

	if show_tenant_section:
		
		for tg in TenantGroup.objects.filter(group=group):
			
			cli.piqj(l, cli.CLI_KEYWORD('tenant'), tg.tenant.name)
			l+=1

			for code in sorted(group.get_capabilities(tg.tenant)):
				cli.piqj(l, cli.CLI_KEYWORD('capability'), code);


			if tg.default_min_priority != PRIORITY_NONE:
				cli.piqj(l, cli.CLI_KEYWORD('min-priority'), str(tg.default_min_priority))

			l-=1
			cli.piqj(l, cli.CLI_KEYWORD('exit') )

	l -= 1
	cli.piqj(l, cli.CLI_KEYWORD('exit') )

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_conf_tenants(cli):
	
	user = cli.env['CURRENT_USER']
	
	# show configuration tenant <name>
	if len(cli.canonical_words) == 4:
		tenant_name = cli.canonical_words[3]
		tenants = [ tenant_get(tenant_name), ]
	else:
		#tenants = tenant_select_all()
		tenants = Tenant.get_visible_by_user(user)
		tenants = tenants.order_by('name')
		tenants = tenants.all()
	
	for t in tenants:
		show_conf_one_tenant(cli, t)

def show_conf_one_tenant(cli, t):

	db_set_current_tenant_schema(t.name)

	
	user = cli.env['CURRENT_USER']
	
	l=0
	cli.piqj(l, cli.CLI_KEYWORD('tenant'), t.name)
	l+=1

	cli.piqj(l, cli.CLI_KEYWORD('long-name') , t.long_name )
	cli.piqj(l, cli.CLI_KEYWORD('timezone' ) , t.timezone)

	if t.is_primary:
		cli.piqj(l, cli.CLI_KEYWORD('primary') )

	#
	# Priority labels
	#
	levels = PriorityLabel.objects.all().order_by('min_level')
	if levels:
		cli.pij(l, cli.CLI_COMMENT('#') )
		cli.pij(l, cli.CLI_COMMENT('# Priority levels') )
		cli.pij(l, cli.CLI_COMMENT('#') )
		for pl in levels:
			cli.piqj(l, cli.CLI_KEYWORD('priority-level'), pl.label, str(pl.min_level), str(pl.max_level), pl.description)

	#
	# Agents
	#
	if user is None or user.is_superuser:
		agents = agent_select_all(t.name)
		if agents:
			cli.pij(l, cli.CLI_COMMENT('#') )
			cli.pij(l, cli.CLI_COMMENT('# AGENTS') )
			cli.pij(l, cli.CLI_COMMENT('#') )
			for a in agents:
				show_config_one_agent(cli, a, level=l)
	else:
		cli.pij(l, cli.CLI_WARNING("# Only admin can see agents configuration") )

	#
	# Scheduling groups
	#
	#schedulinggroups = SchedulingGroup.objects.filter(tenant=t).order_by('name').all()
	schedulinggroups = SchedulingGroup.get_visible_by_user(t, user) #objects.filter(tenant=t).order_by('name').all()
	if schedulinggroups:
		cli.pij(l, cli.CLI_COMMENT('#') )
		cli.pij(l, cli.CLI_COMMENT('# Scheduling groups') )
		cli.pij(l, cli.CLI_COMMENT('#') )
		for sg in schedulinggroups:
			show_config_one_schedulinggroup(cli, sg, level=l)

	#
	# Cluster
	#
	clusters = Cluster.objects.filter(tenant=t).order_by('name')
	if clusters:
		cli.pij(l, cli.CLI_COMMENT('#') )
		cli.pij(l, cli.CLI_COMMENT('# CLUSTERS') )
		cli.pij(l, cli.CLI_COMMENT('#') )
		for c in clusters:
			_show_config_one_cluster(cli, c, level=l)

	#
	# Mostriamo i datagorup-template (visisbili dall'utente.)
	#

	#dg_templates = TDataGroup.objects.filter(tenant=t, source=SOURCE_LOCAL).order_by('name')

	dg_templates = TDataGroup.get_visible_by_user(user, tenant=t).exclude(source=SOURCE_LIBRARY)
	dg_templates = dg_templates.order_by('name').all()
	if dg_templates:
		cli.pij(l, cli.CLI_COMMENT('#') )
		cli.pij(l, cli.CLI_COMMENT('# DATAGROUPS TEMPLATES') )
		cli.pij(l, cli.CLI_COMMENT('#') )
		for tdg in dg_templates:
			do_show_conf_one_datagroup_template(cli, tdg, level=l)

	#
	# Mostriamo tutti gli element-template (visibili dall'utente)
	#
	
	#
	# NOTA: poiche' i node-template possono contenere la definizione di sotto elementi che usano
	#       gli altri tipi di template e' FONDAMENTALE che la configurazione
	#       venga mostrata elencando i node-template per ultimi. Per questo motivo
	#       sortiamo per '-kind' che ci assicura che i node-template vengano mostrati per ultimi.
	#
	#element_templates = TElement.objects.filter(tenant=t)
	element_templates = TElement.get_visible_by_user(t, user)

	element_templates = element_templates.filter(parent__isnull=True)
	element_templates = element_templates.order_by('-kind','name').all()
	
	if element_templates:
		cli.pij(l, cli.CLI_COMMENT('#') )
		cli.pij(l, cli.CLI_COMMENT('# ELEMENT TEMPLATES') )
		cli.pij(l, cli.CLI_COMMENT('#') )
		for te in element_templates:
			_show_conf_one_element_template(cli, te, level=l)

	
	hooks = cli_get_registered_hooks('show:tenant:configuration-pre-nodes')
	if hooks:
		for h in hooks:
			h(cli, t, level=l)

	#nodes = node_select_all(t)
	nodes = MonitoredNode.get_visible_by_user(t, user)
	if nodes:
		cli.pij(l, cli.CLI_COMMENT('#') )
		cli.pij(l, cli.CLI_COMMENT('# NODES') )
		cli.pij(l, cli.CLI_COMMENT('#') )
		for n in nodes:
			show_config_one_node(cli, n, level=l)

	if user is None or user.is_superuser:
		links = link_select_all(tenant_name=t.name)
		if links:
			cli.pij(l, cli.CLI_COMMENT('#') )
			cli.pij(l, cli.CLI_COMMENT('# LINKS') )
			cli.pij(l, cli.CLI_COMMENT('#') )
			for link in links:
				show_conf_one_link(cli, link, level=l)
	else:
		cli.pij(l, cli.CLI_WARNING("# Only admin can see link configuration"))

	hooks = cli_get_registered_hooks('show:tenant:configuration')
	if hooks:
		for h in hooks:
			h(cli, t, level=l)

	l+=1
	cli.piqj(0, cli.CLI_KEYWORD('exit'))

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_conf_clusters(cli):

	tenant_name = cli.env['CURRENT_TENANT']

	# show configuration cluster [ <name> ]
	if len(cli.canonical_words) == 4:
		name = cli.canonical_words[3]
		clusters = [ cluster_get(name, tenant_name=tenant_name) , ]
	else:
		#clusters = cluster_select_all(tenant_name)
		clusters = Cluster.objects.filter(tenant__name=tenant_name).order_by('name')

	for c in clusters:
		_show_config_one_cluster(cli, c)
		
def _show_config_one_cluster(cli, cluster, level=0):

	l = level
	cli.piqj(l, cli.CLI_KEYWORD('cluster')     , cluster.name)
	l += 1

	cli.piqj(l, cli.CLI_KEYWORD('description') , cluster.description)


	if cluster.owner:
		cli.piqj(l, cli.CLI_KEYWORD('owner') , CLI_INT(cluster.owner))

	for cgp in cluster.clustergrouppermission_set.all():
		cli.piqj(l, cli.CLI_KEYWORD('group') , cgp.group.name,  AUTH_PERM_TO_CLI_PERM[cgp.auth_type] )


	sg = cluster.scheduling_group
	if sg is not None:
		if sg.min_gap      is not None: cli.piqj(l, cli.CLI_KEYWORD('scheduling-min-gap'     ) , CLI_INT( sg.min_gap      ) )
		if sg.max_deadline is not None: cli.piqj(l, cli.CLI_KEYWORD('scheduling-max-deadline') , CLI_INT( sg.max_deadline ) )
		if sg.max_parallel is not None: cli.piqj(l, cli.CLI_KEYWORD('scheduling-max-parallel') , CLI_INT( sg.max_parallel ) )

	l -= 1
	cli.piqj(l, cli.CLI_KEYWORD('exit') )

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_conf_agents(cli):
	
	tenant_name = cli.env['CURRENT_TENANT']

	# show configuration users <name>
	if len(cli.canonical_words) == 4:
		name = cli.canonical_words[3]
		agents = [ agent_get(name, tenant_name=tenant_name) , ]	
	else:
		agents = agent_select_all(tenant_name=tenant_name)
		
	for a in agents:
		show_config_one_agent(cli, a)
		
def show_config_one_agent(cli, agent, level=0):

	l = level

	cli.piqj(l, cli.CLI_KEYWORD('agent') , agent.name)
	l += 1

	cli.piqj(l, cli.CLI_KEYWORD('description') , agent.description)

	if not agent.is_enabled:
		cli.piqj(l, cli.CLI_KEYWORD('disabled') )

	if agent.is_remote:
		cli.piqj(l, cli.CLI_KEYWORD('remote') )

	if agent.is_primary:
		cli.piqj(l, cli.CLI_KEYWORD('primary') )

	cli.piqj(l, cli.CLI_KEYWORD('update-minperiod') , str(agent.config_update_minperiod))

	cli.piqj(l, cli.CLI_KEYWORD('threads') , CLI_INT(agent.threads))

	PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('scheduling-min-gap'              ), CLI_INT(agent.scheduling_min_gap     ) )
	PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('scheduling-max-parallel'         ), CLI_INT(agent.scheduling_max_parallel) )
	PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('scheduling-max-deadline'         ), CLI_INT(agent.scheduling_max_deadline) )

	l -= 1
	cli.piqj(l, cli.CLI_KEYWORD('exit') )

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_conf_nodes(cli):

	user        = cli.env['CURRENT_USER']

	tenant_name = cli.env['CURRENT_TENANT']

	tenant = tenant_get(tenant_name)


	# show configuration node <name> 
	if len(cli.canonical_words) == 4:
		name     = cli.canonical_words[3]
		nodes = [ node_get(name, tenant_name=tenant_name) , ]

		for n in nodes:
			show_config_one_node(cli, n)

	# show configuration node <name> no-subelements 
	elif len(cli.canonical_words) == 5 and cli.canonical_words[4] == 'no-subelements':
		name = cli.canonical_words[3]
		n    = node_get(name, tenant_name=tenant_name) 

		show_config_one_node(cli, n, sub_elements=False)


	# Ticket: #3391
	# NON ANCORA SUPPORTATO PERCHE' NON CI SONO I COMPLETER E QUESTO RENDE IL COMANDO
	# SCOMODISSIMO.
	# show configuration node <name> datagroup <name> [template <name>]
	#elif len(cli.canonical_words) > 5 and cli.canonical_words[4] == 'datagroup':
	#
	#	name          = cli.canonical_words[3]
	#	dg_name       = cli.canonical_words[5]
	#
	#	kind = 1
	#
	#	if len(cli.canonical_words) > 6:
	#		node_template = cli.canonical_words[7]
	#
	#		cdg = CDataGroup.objects.get(tenant=tenant, element__name=name, element__kind=kind, name=dg_name, tedatagroup__telement__name=node_template)
	#	else:
	#
	#		cdg = CDataGroup.objects.get(tenant=tenant, element__name=name, element__kind=kind, name=dg_name, tdatagroup__isnull=False)
	#
	#	if cdg.tdatagroup is not None:
	#		show_configuration_element_configured_datagroup(cli, cdg, 0, force_show=True)
	#	else:
	#		show_configuration_element_configured_datagroup(cli, cdg, 0, force_show=True, prefix='template')

	# show configuration node <name> interface|storage|service|device <name> 
	elif len(cli.canonical_words) > 5:
		name     = cli.canonical_words[3]
		sub_name = cli.canonical_words[5]

		subelement = MonitoredElement.objects.get(name=sub_name, parent__name=name)
		subelement = subelement.get_specific_element()

		show_configuration_sub_element(cli, subelement)	

	# show configuration node 
	else:
		nodes = MonitoredNode.get_visible_by_user(tenant, user)

		for n in nodes:
			show_config_one_node(cli, n)


def show_config_element_common_fields(cli, e, level=0, is_node=False):

	l = level

	# Campo visibile solo a livello di nodo
	if e.kind == ELEMENT_KIND_NODE and e.owner:
		PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('owner'          ), e.owner.username)

	configured_fields = e.not_default_fields()
	
	if 'description' in configured_fields:
		PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('description'          ), e.description)

	
	if 'is_abstract' in configured_fields:
		PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('is_abstract'          ), CLI_BOOL(e.is_abstract))

	if is_node and e.agent:
		PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('agent'                ), e.agent.name )

	if 'priority' in configured_fields: 
		PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('priority'             ), CLI_INT(e.priority))

	if e.cluster:
		PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('cluster'              ), e.cluster.name )
		PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('cluster-role'         ), CLI_CLUSTER_ROLE_TO_NAME(e.cluster_role))
		PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('cluster-distinguisher'), e.cluster_distinguisher )
		PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('cluster-xform'        ), e.cluster_xform )
		PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('cluster-instance'     ), e.cluster_instance )

	if e.scheduling_group:
		if e.scheduling_group.is_real:
			PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('scheduling-group'     ), CLI_INT(e.scheduling_group.name))
		else:
			sg = e.scheduling_group
			PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('scheduling-min-gap'     ), CLI_INT(sg.min_gap))
			PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('scheduling-max-deadline'), CLI_INT(sg.max_deadline))
			PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('scheduling-max-parallel'), CLI_INT(sg.max_parallel))

	if e.icon:
		PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('icon'                 ), e.icon )

def cmd_show_conf_scheduling_groups(cli, l=0):

	user        = cli.env['CURRENT_USER'  ]
	tenant_name = cli.env['CURRENT_TENANT']

	tenant = tenant_get(tenant_name)

	tenant.db_set_current_schema()

	# 0    1    2                  3
	# show conf scheduling-group [ <name> ]

	#sgs = SchedulingGroup.objects.filter(tenant=tenant).filter(is_real=True)

	sgs = SchedulingGroup.get_visible_by_user(tenant, user)

	sgs = sgs.filter(is_real=True)
	
	sgs = sgs.order_by('name')
	
	if len(cli.canonical_words) > 3:

		name = cli.canonical_words[3]

		sgs = sgs.filter(name=name)
	else:
		sgs = sgs

	for sg in sgs.all():
		show_config_one_schedulinggroup(cli, sg, level=l)

def show_config_one_schedulinggroup(cli, sg, level=0):

	sg.tenant.db_set_current_schema()

	l = level

	cli.piqj(l, cli.CLI_KEYWORD('scheduling-group'), sg.name)
	l += 1

	if sg.owner:
		cli.piqj(l, cli.CLI_KEYWORD('owner') , CLI_INT(sg.owner))
		
	for sgp in sg.get_group_permissions():
		cli.piqj(l, cli.CLI_KEYWORD('group') , sgp.group.name, cli.CLI_KEYWORD( AUTH_PERM_TO_CLI_PERM[sgp.auth_type] ))

	if sg.min_gap:
		cli.piqj(l, cli.CLI_KEYWORD('min-gap'), str(sg.min_gap))

	if sg.max_deadline:
		cli.piqj(l, cli.CLI_KEYWORD('max-deadline'), str(sg.max_deadline))

	if sg.max_parallel:
		cli.piqj(l, cli.CLI_KEYWORD('max-parallel'), str(sg.max_parallel))

	l -= 1
	cli.piqj(l, cli.CLI_KEYWORD('exit'))


def show_config_one_node(cli, n, level=0, sub_elements=True):

	l = level

	cli.piqj(l, cli.CLI_KEYWORD('node'), n.name)

	l += 1

	show_config_element_common_fields(cli, n, l, is_node=True)

	configured_fields = n.not_default_fields()
	
	if 'dft_timeout'      in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('dft-timeout'     ) , CLI_INT(n.dft_timeout))
	if 'dns_name'         in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('dns-name'        ) , n.dns_name)
	if 'dns6_name'        in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('dns6-name'       ) , n.dns6_name)
	if 'ip4_mgt'          in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('ip4-mgt'         ) , n.ip4_mgt)
	if 'ip6_mgt'          in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('ip6-mgt'         ) , n.ip6_mgt)
	if 'use_ipv4'         in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('use-ipv4'        ) , CLI_BOOL(n.use_ipv4))
	if 'timezone'         in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('timezone'        ) , n.timezone)
	if 'snmp_port'        in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('snmp-port'       ) , CLI_INT(n.snmp_port))
	if 'snmp_community'   in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('snmp-community'  ) , n.snmp_community)
	if 'snmp_version'     in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('snmp-version'    ) , CLI_INT(n.snmp_version))
	if 'snmp_v3_authdata' in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('snmp-v3-authdata') , n.snmp_v3_authdata)

	if 'agent_runningon'  in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('agent-runningon' ) , CLI_BOOL(n.agent_runningon))
	
	if 'notify_address'   in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('notify-address'  ) , n.notify_address)
	
	hooks = cli_get_registered_hooks('show:node:configuration')
	for h in hooks:
		h(cli, n, level=l)
				
	show_configuration_element_configured_category_and_datagroups(cli, n, l)

	# Show sub elements (interface, storage, ecc...)
	
	if sub_elements:

		for e in MonitoredElement.objects.filter(parent=n).order_by('kind', 'name'):

			e = e.get_specific_element()
			show_configuration_sub_element(cli, e, l)

	l -= 1
	
	cli.piqj(l, cli.CLI_KEYWORD('exit') )

def show_configuration_sub_element(cli, e, l=0):
	"""
	Questa funzione visualizzza la configurazione di sotto-elementi:
	
	  * creati direttamente dentro un nodo
	  * creati autoamticamente da un sub-element-template.
	  
	
	
	"""
	
	configured_fields = e.not_default_fields(exclude_fields=['mtime', 'uuid', 'kind', 'name'])
	
	if e.telement_generator is None:
		#
		# Elemento normale
		#
		show_xform = True
		if e.distinguisher and e.xform:
			xform = e.xform
			if   e.kind == ELEMENT_KIND_STORAGE: xform = xform_to_xform_alias('storages'  , xform) #xform_to_storagedescr(e.xform)
			elif e.kind == ELEMENT_KIND_SERVICE: xform = xform_to_xform_alias('services'  , xform) #xform_to_procdescr(e.xform)
			elif e.kind == ELEMENT_KIND_IFACE  : xform = xform_to_xform_alias('interfaces', xform) #xform_to_ifdescr(e.xform)
			elif e.kind == ELEMENT_KIND_DEVICE : xform = xform_to_xform_alias('devices'   , xform) #xform_to_ifdescr(e.xform)
			
			cli.piqj(l, cli.CLI_KEYWORD(ELEMENT_KIND_STR[e.kind]), e.name, cli.CLI_KEYWORD('distinguisher'), e.distinguisher, cli.CLI_KEYWORD('xform'), xform)
			show_xform = False
		elif e.distinguisher:
			cli.piqj(l, cli.CLI_KEYWORD(ELEMENT_KIND_STR[e.kind]), e.name, cli.CLI_KEYWORD('distinguisher'), e.distinguisher)
			show_xform = False
		else:
			cli.piqj(l, cli.CLI_KEYWORD(ELEMENT_KIND_STR[e.kind]), e.name)
	else:
		
		# 
		# Alcuni campi deli elementi generati da un sub-element template alcuni campi (distinguisher, xform, ecc.)
		# potrebbero contenere il valore "proveniente dal template". In questo caso non va visualizzata
		# la configurazione.
		#
		# I campi *index non ha senso farli vedere in configurazione, o forse si? Per ora non li facciamo vedere.
		#

		fields_to_hide = ['ifindex', 'stindex', 'swrunindex', 'devindex']

		# Prendiamo tutti i campi valorizzati nel template
		telement_configured_fields = e.telement_generator.get_configured_element_fields_dict()

		# Vediamo tutti i campi (non da nascondere) non configurabili nel sub-element template oppure che sono configurabili e che sono anche stati "personalizzati"
		configured_fields = [ field_name for field_name in configured_fields if ( (field_name not in fields_to_hide) and (field_name not in telement_configured_fields) or ((field_name in telement_configured_fields) and (telement_configured_fields[field_name] != getattr(e, field_name))) ) ]
		
		# Ci sono configurazioni nei singoli datagroup?
		configured_datagroups = CDataGroup.objects.filter(tenant=e.tenant, element=e).count() > 0
		
		# Se non c'e' niente da visualizzare, tutta la sezione di queesto elemento si puo' saltare.
		
		if not (configured_fields or configured_datagroups):
			return

		#
		# Elemento from Sub-element template
		#
		show_xform = True
		
		prefix = 'template-%s' % (ELEMENT_KIND_STR[e.kind])
		cli.piqj(l, cli.CLI_KEYWORD(prefix), e.name)
		
	l += 1
	
	show_config_element_common_fields(cli, e, l)
	
	#
	# Mostriamo i campi
	#
	if   e.kind == ELEMENT_KIND_IFACE:
		if show_xform:
			if 'distinguisher' in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('distinguisher') , e.distinguisher)
			if 'xform'         in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('xform'        ) , e.xform)
			if 'ifindex'       in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('ifindex'      ) , str(e.ifindex))

		if e.backbone:
			PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('backbone')       , 'true')

		if e.speed is not None:
			PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('speed'   )       , str(e.speed))
		
	elif e.kind == ELEMENT_KIND_STORAGE:
		if show_xform:
			if 'distinguisher' in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('distinguisher') , e.distinguisher)
			if 'xform'         in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('xform'        ) , e.xform)
			if 'stindex'       in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('stindex'      ) , str(e.stindex))
	elif e.kind == ELEMENT_KIND_DEVICE:
		if show_xform:
			if 'distinguisher' in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('distinguisher') , e.distinguisher)
			if 'xform'         in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('xform'        ) , e.xform)
			if 'devindex'      in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('devindex'     ) , str(e.devindex))
	elif e.kind == ELEMENT_KIND_SERVICE:
		if show_xform:
			if 'distinguisher' in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('distinguisher') , e.distinguisher)
			if 'xform'         in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('xform'        ) , e.xform)
			if 'swrunindex'    in configured_fields: PIQJ_IFDEFINED(cli, l, cli.CLI_KEYWORD('swrunindex'   ) , str(e.swrunindex))
	#
	# Mostriamo i datagroup configurati a livello di elemento
	#
	show_configuration_element_configured_category_and_datagroups(cli, e, l)

	#
	# Hook di configurazione
	#
	if   e.kind == ELEMENT_KIND_IFACE  : hooks = cli_get_registered_hooks('show:interface:configuration')
	elif e.kind == ELEMENT_KIND_STORAGE: hooks = cli_get_registered_hooks('show:storage:configuration')
	elif e.kind == ELEMENT_KIND_DEVICE : hooks = cli_get_registered_hooks('show:device:configuration')
	elif e.kind == ELEMENT_KIND_SERVICE: hooks = cli_get_registered_hooks('show:service:configuration')
	for h in hooks:
		h(cli, e, level=l)

	l -= 1
	cli.piqj(l, cli.CLI_KEYWORD('exit') )


def show_configuration_element_configured_category_and_datagroups(cli, e, level=0):

	n = e
	l = level

	#
	# Per "aiutare" mostro tutti i template che sono stati associati a partire da un sub-element template
	#
	templates = n.elementtemplate_set.filter(telementappliedtemplate__isnull=False).all()
	for template in templates:
		cli.pij(l, cli.CLI_COMMENT('# template %s from %s' % (template.telement.name, template.telementappliedtemplate.telement.name)) )

	#
	# Mostro tutti i template associati direttamente all'elemento.
	#
	templates = n.elementtemplate_set.filter(telementappliedtemplate__isnull=True).all()
	for template in templates:
		cli.pij(l, cli.CLI_KEYWORD('template'), template.telement.name)

	# Configured datagroup from element template
	datagroups = CDataGroup.objects.filter(element=n, tedatagroup__isnull=False).order_by('tedatagroup__telement__name', 'name')
	if datagroups:
		for cdg in datagroups:
			show_configuration_element_configured_datagroup(cli, cdg, l, prefix='template')

	# Configured datagrup (additional)
	datagroups = CDataGroup.objects.filter(element=n, tdatagroup__isnull=False).order_by('name')
	if datagroups:
		for cdg in datagroups:
			show_configuration_element_configured_datagroup(cli, cdg, l, force_show=True)

	return

def show_configuration_element_configured_datagroup(cli, cdg, level=0, prefix=None, force_show=False):
	
	l=level
	
	# Find customizations
	fields = CDataGroupFieldValue.objects.filter(cdatagroup=cdg)
	params = CDataGroupParameterValue.objects.filter(cdatagroup=cdg)
	
	tdatagroup  = cdg.get_tdatagroup()
	
	datasources_by_name = cdg.get_local_defined_datasources_fields_dict()
	
	conditions_by_name = cdg.get_local_defined_conditions_fields_dict()
	
	timegraphs_by_name = cdg.get_local_defined_timegraphs_fields_dict()
	
	if not (fields or params or datasources_by_name or conditions_by_name or timegraphs_by_name or force_show):
		if DEBUG: cli.pij(l, '#DEBUG datagroup %s' % ( cdg ))
		#cli.out('\n')
		return
	
	#cli.out('\n')
	if DEBUG: cli.pij(l, '#DEBUG cdatagroup %s' % (cdg))
		
	if prefix:
		if cdg.tedatagroup_id is not None:
			cli.piqj(l, cli.CLI_KEYWORD('%s-datagroup' % (prefix)), cdg.name, cli.CLI_KEYWORD('from'), cdg.tedatagroup.telement.name ) 
		else:
			cli.piqj(l, cli.CLI_KEYWORD('%s-datagroup' % (prefix)), cdg.name)
	else:
		if cdg.name != cdg.tdatagroup.name:
			cli.piqj(l, cli.CLI_KEYWORD('datagroup'), cdg.name, cli.CLI_KEYWORD('template'), cdg.tdatagroup.name )
		else:
			cli.piqj(l, cli.CLI_KEYWORD('datagroup'), cdg.name)

	l += 1
	
	#
	# Fields
	#

	#for f in sorted(fields, key=operator.attrgetter('name')):
	#	cli.piqj(l, cli.CLI_KEYWORD(DATAGROUP_FIELD_TO_CLI_FIELD(f.name)), f.value)
	
	SUSPEND_FIELDS_ORDER = ['suspended', 'suspend_resume_time', 'suspend_reason', 'suspend_time' ]
	
	suspend_fields_by_name = dict([ (f.name, f) for f in fields  if f.name in SUSPEND_FIELDS_ORDER ])
	other_fields_by_name   = dict([ (f.name, f) for f in fields  if f.name not in suspend_fields_by_name ])

	# Show "suspend" fields only if suspended
	f = suspend_fields_by_name.get('suspended', None)
	if f and f.value == 'true':
		for name in SUSPEND_FIELDS_ORDER:
			f = suspend_fields_by_name.get(name, None)
			if f:
				cli.piqj(l, cli.CLI_KEYWORD(DATAGROUP_FIELD_TO_CLI_FIELD(f.name)), f.value)
	

	for f in sorted(other_fields_by_name.values(), key=operator.attrgetter('name')):
		cli.piqj(l, cli.CLI_KEYWORD(DATAGROUP_FIELD_TO_CLI_FIELD(f.name)), f.value)
	
	#
	# Parameters
	#
	for p in sorted(params, key=operator.attrgetter('name')):
		cli.piqj(l, cli.CLI_KEYWORD('parameter'), p.name, CLI_PTYPE_TO_NAME(p.vtype), CLI_PTYPE_VAL_TO_CLI_VAL(p.vtype, p.value))
	
	_show_configured_datasources_conditions_timegraphs(cli, datasources_by_name, conditions_by_name, timegraphs_by_name, l=l)

	l -= 1
	cli.piqj(l, cli.CLI_KEYWORD('exit'))
	
def _show_configured_datasources_conditions_timegraphs(cli, datasources_by_name, conditions_by_name, timegraphs_by_name, l=0):
		
	for ds_name, configured_fields in sorted(datasources_by_name.items()):
		cli.piqj(l, cli.CLI_KEYWORD('datasource'), ds_name)
		l += 1
		for name, cf in sorted(configured_fields.items()):
			cli.piqj(l, cli.CLI_KEYWORD(DATASOURCE_FIELD_TO_CLI_FIELD(cf.name)), cf.value)
		l -= 1
		cli.piqj(l, cli.CLI_KEYWORD('exit') )

	for c_name, configured_fields in sorted(conditions_by_name.items()):
		cli.piqj(l, cli.CLI_KEYWORD('condition'), c_name)
		l += 1
		for name, cf in sorted(configured_fields.items()):
			cli.piqj(l, cli.CLI_KEYWORD(CONDITION_FIELD_TO_CLI_FIELD(cf.name)), CONDITION_FIELD_VALUE_TO_CLI_FIELD_VALUE(cf.name, cf.value))
		l -= 1
		cli.piqj(l, cli.CLI_KEYWORD('exit') )
		
	for t_name, configured in sorted(timegraphs_by_name.items()):
		
		cli.piqj(l, cli.CLI_KEYWORD('timegraph'), t_name)
		l += 1
		
		for name, cf in sorted(configured['fields'].items()):
			cli.piqj(l, cli.CLI_KEYWORD(TIMEGRAPH_FIELD_TO_CLI_FIELD(cf.name)), cf.value)
		
		if configured['series']:
			for s_name, configured in sorted(configured['series'].items()):
				cli.piqj(l, cli.CLI_KEYWORD('serie'), s_name)
				l += 1
				for name, cf in sorted(configured.items()):
					cli.piqj(l, cli.CLI_KEYWORD(TIMEGRAPHSERIE_FIELD_TO_CLI_FIELD(cf.name)), cf.value)
				l -= 1
				cli.piqj(l, cli.CLI_KEYWORD('exit'))
		
		l -= 1
		cli.piqj(l, cli.CLI_KEYWORD('exit'))


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_conf_links(cli):
	
	# show conf links node ....
	tenant_name  = cli.env.get('CURRENT_TENANT', None)

	
	links = []
	
	if len(cli.canonical_words) <= 3:
		links = link_select_all(tenant_name=tenant_name)
	elif cli.canonical_words[3] == 'node':
		node_name = cli.canonical_words[4]
		n = MonitoredNode.objects.get(name=node_name)
		
		links = link_select_all(tenant_name=tenant_name, node=n)

		
	for l in links:
		show_conf_one_link(cli, l)

def show_conf_one_link(cli, link, level=0):
	
	line = [ cli.CLI_KEYWORD('link') ]
	
	line += [ 
		link.node1.name, 
		link.iface1.name, 
		link.node2.name,  
		link.iface2.name 
	]
	
	line += [ 
		link.name,  
		cli.CLI_KEYWORD(LINK_TYPE_TO_CLI[link.ltype]), 
		link.description 
	]
	
	cli.piqj(level, *line)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_conf_tenant_datagroup_templates(cli):

	source = SOURCE_LOCAL
	tenant_name = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(tenant_name)

	if len(cli.canonical_words) == 4:
		name = cli.canonical_words[3]
	else:
		name = None

	base_cmd_show_conf_datagroup_templates(cli, source, tenant, name)
	
	return

def cmd_show_conf_datagroup_templates_library(cli):

	source = SOURCE_LIBRARY
	tenant = None

	name = cli.canonical_words[3]
	
	base_cmd_show_conf_datagroup_templates(cli, source, tenant, name)

	return



#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

# show configuration *-template <name>

def cmd_show_conf_node_templates(cli):
	return cmd_show_conf_element_templates(cli, ELEMENT_KIND_NODE )

def cmd_show_conf_iface_templates(cli):
	return cmd_show_conf_element_templates(cli, ELEMENT_KIND_IFACE )

def cmd_show_conf_storage_templates(cli):
	return cmd_show_conf_element_templates(cli, ELEMENT_KIND_STORAGE )

def cmd_show_conf_service_templates(cli):
	return cmd_show_conf_element_templates(cli, ELEMENT_KIND_SERVICE )

def cmd_show_conf_device_templates(cli):
	return cmd_show_conf_element_templates(cli, ELEMENT_KIND_DEVICE )
	
def cmd_show_conf_element_templates(cli, kind):
	
	user         = cli.env['CURRENT_USER']

	tenant_name  = cli.env.get('CURRENT_TENANT', None)
	
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)

	if len(cli.canonical_words) == 4:
		element_template_name = cli.canonical_words[3]
		
		te = TElement.objects.get(tenant=tenant, name=element_template_name, kind=kind)
		
		tes = [ te ]
		
	else:
		#tes = TElement.objects.filter(tenant=tenant, kind=kind).all().order_by('kind','name')
		tes = TElement.get_visible_by_user(tenant, user)
		
		tes = tes.filter(kind=kind)
		tes = tes.order_by('kind','name')
	
	for te in tes:
		_show_conf_one_element_template(cli, te)

def cmd_show_conf_one_element_template(cli, tenant, kind, name):

	te = TElement.objects.get(tenant=tenant, kind=kind, name=name)

	if te.parent is not None:
		te = te.parent

	_show_conf_one_element_template(cli, te)


def _show_conf_one_element_template(cli, te, level=0):

	l = level
	
	name = te.get_name()

	cli.piqj(l, cli.CLI_KEYWORD("%s-template" % (ELEMENT_KIND_STR[te.kind])), te.name)
	l += 1


	if te.owner:
		cli.piqj(l, cli.CLI_KEYWORD("owner"), te.owner.username )
		
	for tgp in te.get_group_permissions():
		cli.piqj(l, cli.CLI_KEYWORD("group"), tgp.group.name,  cli.CLI_KEYWORD( AUTH_PERM_TO_CLI_PERM[tgp.auth_type] ) )



	if te.description:
		cli.piqj(l, cli.CLI_KEYWORD("description"), te.description)

	if te.is_switch:
		cli.piqj(l, cli.CLI_KEYWORD("switch") )
		
	if te.is_router:
		cli.piqj(l, cli.CLI_KEYWORD("router") )
	
	for tedg in TEDataGroup.objects.filter(telement=te):
		
		if tedg.name != tedg.tdatagroup.name:
			cli.piqj(l, cli.CLI_KEYWORD("datagroup"), tedg.name, cli.CLI_KEYWORD("template"), tedg.tdatagroup.name)
		else:
			cli.piqj(l, cli.CLI_KEYWORD("datagroup"), tedg.name)
			
		l += 1
		# fields
		for tedgfv in TEDataGroupFieldValue.objects.filter(tedatagroup=tedg):
			cli.piqj(l, cli.CLI_KEYWORD(tedgfv.tfield.name), tedgfv.value)
			
		# params
		for tedgpv in TEDataGroupParameterValue.objects.filter(tedatagroup=tedg):
			cli.piqj(l, cli.CLI_KEYWORD('parameter'), tedgpv.name, CLI_PTYPE_TO_NAME(tedgpv.vtype), CLI_PTYPE_VAL_TO_CLI_VAL(tedgpv.vtype, tedgpv.value))
		
		datasources_by_name = tedg.get_local_defined_datasources_fields_dict()
		
		conditions_by_name  = tedg.get_local_defined_conditions_fields_dict()
		
		timegraphs_by_name  = tedg.get_local_defined_timegraphs_fields_dict()
		
		_show_configured_datasources_conditions_timegraphs(cli, datasources_by_name, conditions_by_name, timegraphs_by_name, l=l)

		l -= 1
		cli.piqj(l, cli.CLI_KEYWORD("exit") )


	for sub in TElement.objects.filter(tenant=te.tenant, parent=te).order_by('kind', 'name'):
		
		name = sub.get_element_name_to_apply()

		section_command = [ cli.CLI_KEYWORD( "%s" % (ELEMENT_KIND_STR[sub.kind]) ), name ]
		
		if sub.distinguisher:
			section_command += [ cli.CLI_KEYWORD("distinguisher") , sub.distinguisher ]

		if sub.xform:
			
			xform_to_show = xform_to_xform_alias_by_kind(sub.kind, sub.xform)
			
			section_command += [ cli.CLI_KEYWORD("xform") ,  xform_to_show ]
		
		cli.piqj(l, *section_command)
		
		l += 1
		applied_templates = TElementAppliedTemplate.objects.filter(telement=sub).order_by('applyorder')
		for at in applied_templates:
			cli.piqj(l, cli.CLI_KEYWORD("template"), at.telementapplied.name)
		l -=1
		
		cli.piqj(l, cli.CLI_KEYWORD("exit") )

	l -= 1
	cli.piqj(l, cli.CLI_KEYWORD("exit") )
 
#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_conf_this(cli):

	if cli.env.get('SOURCE') == SOURCE_LIBRARY:

		datagroup_name = cli.env.get('CONFIG_DATAGROUP_TEMPLATE', None)
		if datagroup_name:
			d = datagroup_template_get(datagroup_name, source=SOURCE_LIBRARY)
			do_show_conf_one_datagroup_template(cli, d)
			return

		return

	user_name = cli.env.get('CONFIG_USER', None)
	if  user_name:
		u = User.objects.get(username=user_name)
		_show_config_oneuser(cli, u)
		return

	group_name = cli.env.get('CONFIG_GROUP', None)
	if  group_name:
		g = Group.objects.get(name=group_name)

		cli.pij(0, cli.CLI_COMMENT("#") )
		cli.pij(0, cli.CLI_COMMENT("# Global") )
		cli.pij(0, cli.CLI_COMMENT("#") )
		_show_config_one_group(cli, g, show_users_section=True, show_tenant_section=False)

		cli.pij(0, cli.CLI_COMMENT("#") )
		cli.pij(0, cli.CLI_COMMENT("# Tenant dependent") )
		cli.pij(0, cli.CLI_COMMENT("#") )
		_show_config_one_group(cli, g, show_users_section=False, show_tenant_section=True)
		return

	tenant_name  = cli.env.get('CONFIG_TENANT', None)

	cluster_name = cli.env.get('CONFIG_CLUSTER', None)
	if cluster_name:
		c = cluster_get(cluster_name, tenant_name=tenant_name)
		_show_config_one_cluster(cli, c)
		return

	agent_name = cli.env.get('CONFIG_AGENT', None)
	if agent_name:
		a = agent_get(agent_name, tenant_name=tenant_name)
		show_config_one_agent(cli, a)
		return


	subelement = cli.env.get('CONFIG_ELEMENT_DATA', None)
	sub = cli.env.get('CONFIG_IFACE', None) or cli.env.get('CONFIG_STORAGE', None) or cli.env.get('CONFIG_DEVICE', None) or cli.env.get('CONFIG_SERVICE', None)
	if sub:
		subelement = MonitoredElement.objects.get(uuid=subelement.uuid)
		subelement = subelement.get_specific_element()
		
		show_configuration_sub_element(cli, subelement)
		return

	node_name = cli.env.get('CONFIG_NODE', None)
	if node_name:
		n = node_get(node_name, tenant_name=tenant_name)
		show_config_one_node(cli, n)
		return

	schedulinggroup_name = cli.env.get('CONFIG_SCHEDULINGGROUP', None)
	if schedulinggroup_name:
		sg = SchedulingGroup.objects.get(tenant__name=tenant_name, name=schedulinggroup_name)
		show_config_one_schedulinggroup(cli, sg)
		return


	tenant_name = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(tenant_name)

	#
	# HOOK
	#
	hooks = cli_get_registered_hooks('show:configuration:this')
	for h in hooks:
		if h(cli):
			return

	element_template = cli.env.get('CONFIG_ELEMENT_TEMPLATE'     , None)
	kind             = cli.env.get('CONFIG_ELEMENT_TEMPLATE_KIND', None)
	if element_template:
		cmd_show_conf_one_element_template(cli, tenant, kind, element_template)
		return

	datagroup_name = cli.env.get('CONFIG_DATAGROUP_TEMPLATE', None)
	if datagroup_name:
		d = datagroup_template_get(datagroup_name, tenant=tenant)#tenant_name=tenant_name)
		do_show_conf_one_datagroup_template(cli, d)
		return

	if tenant_name:
		tenant = tenant_get(tenant_name)
		show_conf_one_tenant(cli, tenant)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_all_configuration(cli):
	
	user = cli.env['CURRENT_USER']

	if cli.env['SOURCE'] == SOURCE_LIBRARY:
		cmd_show_conf_datagroup_templates(cli)
		return

	if cli.env['SOURCE'] == SOURCE_LOCAL:

		if user is None or user.is_superuser:

			cli.pij(0, cli.CLI_COMMENT( "" ) )
			cli.pij(0, cli.CLI_COMMENT( "#" ) )
			cli.pij(0, cli.CLI_COMMENT( "# USERS " ) )
			cli.pij(0, cli.CLI_COMMENT( "# ==============================================================================" ) )
			
			_cmd_show_conf_users(cli, normal_data=True, auth_data=False )

			cli.pij(0, cli.CLI_COMMENT( "" ) )
			cli.pij(0, cli.CLI_COMMENT( "#" ) )
			cli.pij(0, cli.CLI_COMMENT( "# GROUPS" ) )
			cli.pij(0, cli.CLI_COMMENT( "# ==============================================================================" ) )

			cmd_show_conf_groups(cli, show_user_section=True, show_tenant_section=False)


		cli.pij(0, cli.CLI_COMMENT( "" ) )
		cli.pij(0, cli.CLI_COMMENT( "#" ) )
		cli.pij(0, cli.CLI_COMMENT( "# TENANTS " ) )
		cli.pij(0, cli.CLI_COMMENT( "# ==============================================================================" ) )

		cmd_show_conf_tenants(cli)

		if user is None or user.is_superuser:

			cli.pij(0, cli.CLI_COMMENT( "" ) )
			cli.pij(0, cli.CLI_COMMENT( "#" ) )
			cli.pij(0, cli.CLI_COMMENT( "# USERS / TENANT CONFIGURATION" ) )
			cli.pij(0, cli.CLI_COMMENT( "# ==============================================================================" ) )

			_cmd_show_conf_users(cli, normal_data=False, auth_data=True )

			cli.pij(0, cli.CLI_COMMENT( "" ) )
			cli.pij(0, cli.CLI_COMMENT( "#" ) )
			cli.pij(0, cli.CLI_COMMENT( "# GROUPS / TENANT CONFIGURATION" ) )
			cli.pij(0, cli.CLI_COMMENT( "# ==============================================================================" ) )

			cmd_show_conf_groups(cli, show_user_section=False, show_tenant_section=True)

		return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

show_conf_tenant_commands = [

	T_Keyword('scheduling-group' , help='xxxxxx', callback=cmd_show_conf_scheduling_groups, subtree=[
		T_SchedulingGroupcompleter(help='xxxxxxx', callback=cmd_show_conf_scheduling_groups, ),
	]),

	T_Keyword('cluster' , help='Show cluster configuration', callback=cmd_show_conf_clusters, subtree=[
		T_Clustercompleter(help='Show cluster configuration', callback=cmd_show_conf_clusters, ),
	]),

	T_Keyword('agent' , help='Show agent settings', callback=cmd_show_conf_agents, subtree=[
		T_Agentcompleter(help='Show agent settings', callback=cmd_show_conf_agents, ),
	]),

	T_Keyword('node' , help='Show node configuration', callback=cmd_show_conf_nodes, subtree=[
		T_MonitoredNodecompleter(help='Show node configuration', callback=cmd_show_conf_nodes, subtree=[

			T_Keyword('no-subelements' , help="Show node's subelements configurations only", callback=cmd_show_conf_nodes),

			# Ticket: #3391
			# DA IMPLEMENTARE TUTTI I COMPLETER
			#T_Keyword('datagroup' , help='', subtree=[
			#	T_Anyword(help='', callback=cmd_show_conf_nodes, subtree=[
			#		T_Keyword('template' , help='', subtree=[
			#			T_Anyword(help='', callback=cmd_show_conf_nodes)
			#		])
			#	])
			#]),

			T_Keyword('interface' , help="Show interface's configuration of the given node", subtree=[
				T_InlineIfaceCompleter(help="Show interface's configuration of the given node", node_name_token_pos=-2, callback=cmd_show_conf_nodes, subtree=[
					# Ticket: #3391
					# DA IMPLEMENTARE I COMPLETER
					#T_Keyword('datagroup' , help='', subtree=[
					#	T_Anyword(help='', callback=cmd_show_conf_nodes, subtree=[
					#		T_Keyword('template' , help='', subtree=[
					#			T_Anyword(help='', callback=cmd_show_conf_nodes)
					#		])
					#	])
					#]),
				])
			]),

			T_Keyword('storage' , help="Show storage's configuration of the given node", subtree=[
				T_InlineStorageCompleter(help="Show storage's configuration of the given node", node_name_token_pos=-2, callback=cmd_show_conf_nodes, subtree=[
					# Ticket: #3391
					# DA IMPLEMENTARE I COMPLETER
					#T_Keyword('datagroup' , help='', subtree=[
					#	T_Anyword(help='', callback=cmd_show_conf_nodes, subtree=[
					#		T_Keyword('template' , help='', subtree=[
					#			T_Anyword(help='', callback=cmd_show_conf_nodes)
					#		])
					#	])
					#]),
				])
			]),

			T_Keyword('service' , help="Show service's configuration of the given node", subtree=[
				T_InlineServiceCompleter(help="Show service's configuration of the given node", node_name_token_pos=-2, callback=cmd_show_conf_nodes, subtree=[
					# Ticket: #3391
					# DA IMPLEMENTARE I COMPLETER
					#T_Keyword('datagroup' , help='', subtree=[
					#	T_Anyword(help='', callback=cmd_show_conf_nodes, subtree=[
					#		T_Keyword('template' , help='', subtree=[
					#			T_Anyword(help='', callback=cmd_show_conf_nodes)
					#		])
					#	])
					#]),
				])
			]),

			T_Keyword('device' , help="Show device's configuration of the given node", subtree=[
				T_InlineDeviceCompleter(help="Show device's configuration of the given node", node_name_token_pos=-2, callback=cmd_show_conf_nodes, subtree=[
					# Ticket: #3391
					# DA IMPLEMENTARE I COMPLETER
					#T_Keyword('datagroup' , help='', subtree=[
					#	T_Anyword(help='', callback=cmd_show_conf_nodes, subtree=[
					#		T_Keyword('template' , help='', subtree=[
					#			T_Anyword(help='', callback=cmd_show_conf_nodes)
					#		])
					#	])
					#]),
				])
			]),

		]),
	]),

	T_Keyword('links' , help='Show configured links', callback=cmd_show_conf_links, subtree=[
		T_Keyword('node' , help='Show configured links by node', subtree=[
			T_MonitoredNodecompleter(help='Show configured links by node', callback=cmd_show_conf_links, ),
		]),
	]),

	T_Keyword('datagroup-template' , help='Show datagroup template configuration', callback=cmd_show_conf_tenant_datagroup_templates, subtree=[
		T_AllDataGroupTemplate(help='Show datagroup template configuration', callback=cmd_show_conf_tenant_datagroup_templates, subtree=[
			T_Keyword('library' , help='Show datagroup template configuration from library',callback=cmd_show_conf_datagroup_templates_library)
		]),
	]),

	T_Keyword('node-template' , help='Show node template configuration', callback=cmd_show_conf_node_templates, subtree=[
		T_NodeTemplate(help='Show node template configuration', callback=cmd_show_conf_node_templates, ),
	]),
	T_Keyword('interface-template' , help='Show interface template configuration', callback=cmd_show_conf_iface_templates, subtree=[
		T_IfaceTemplate(help='Show interface template configuration', callback=cmd_show_conf_iface_templates, ),
	]),
	T_Keyword('storage-template' , help='Show storage template configuration', callback=cmd_show_conf_storage_templates, subtree=[
		T_StorageTemplate(help='Show storage template configuration', callback=cmd_show_conf_storage_templates, ),
	]),
	T_Keyword('service-template' , help='Show service template configuration', callback=cmd_show_conf_service_templates, subtree=[
		T_ServiceTemplate(help='Show service template configuration', callback=cmd_show_conf_service_templates, ),
	]),
	T_Keyword('device-template' , help='Show device template configuration', callback=cmd_show_conf_device_templates, subtree=[
		T_DeviceTemplate(help='Show device template configuration', callback=cmd_show_conf_device_templates, ),
	]),
]

show_conf_command = [

	T_KeywordWithHook('configuration' , help='Show configuration', callback=cmd_show_all_configuration, 

		hook_name='show:configuration',

		subtree=[
		T_Keyword('this' , help='Show configuration for the current element', callback=cmd_show_conf_this, subtree=[
		]),

		T_Keyword('user' , help='Show user configuration', callback=cmd_show_conf_users, subtree=[
			T_Usercompleter(help='Show user configuration', callback=cmd_show_conf_users, ),
		]),

		T_Keyword('group' , help='Show group configuration', callback=cmd_show_conf_groups, subtree=[
			#T_Anyword(help='Show group configuration', callback=cmd_show_conf_groups, ),
			T_Groupcompleter(help='Show group configuration', callback=cmd_show_conf_groups, ),
		]),

		T_Keyword('tenant' , help='Show tenant configuration', callback=cmd_show_conf_tenants, subtree=[
			T_Tenantcompleter(help='Show tenant configuration', callback=cmd_show_conf_tenants),
		]),

		] 
		+ 
		show_conf_tenant_commands
	),
] 

