# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.


from sanet_common.timeutils import timestamp_to_isostr

from sanet_common.sanetdefs.constants import SOURCE_DESCR

from django.utils.translation import ugettext as _

from cli.clilib.ioslike import *
from cli.textformat import *

from core.models import *
from core.api.base import tenant_get
from core.api.base import db_set_current_tenant_schema

from cli.prompts.common import output_table

from cli.prompts.show.utils import get_groups_by_datagroup_template_id

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_datagroup_templates(cli):

	user             = cli.env['CURRENT_USER']

	current_tenant   = cli.env.get('CURRENT_TENANT'  , None)

	tenant = None
	if current_tenant:
		tenant = tenant_get(current_tenant)
		db_set_current_tenant_schema(tenant.name)

	cols = cli.env['COLS']

	# show templates      datagroup-template [<name>]
	# show templates used datagroup-template [<name>]

	words = cli.canonical_words

	show_used_only = False
	if words[2] == 'used':
		show_used_only = True
		words = words[3:]
	else:
		words = words[2:]

	if len(words) < 2:
		r = [[
			[_('TIME'       ), 12],
			[_('NAME'       ), -4],
			[_('SOURCE'     ), 20],
			[_('TENANT'     ), 20],
			[_('OWNER'      ), 10],
			[_('COUNT'      ),  5],
			[_('ICOUNT'     ),  5],
			[_('PARENT'     ),  5],
			[_('DESCRIPTION'), 30],
			[_('GROUPS'     ), 30],
		]]

		# FIX: use api
		datagroups = TDataGroup.get_visible_by_user(user, tenant)

		datagroups = datagroups.order_by('name', 'source')

		groups_by_tdg_id = get_groups_by_datagroup_template_id( [e[0]for e in datagroups.values_list('uuid')] )


		for d in datagroups.all():

			n = d.used_count()
			n2 = d.indirect_used_count()

			if show_used_only:
				if ((n+n2) == 0):
					continue

			r.append([
				timestamp_to_isostr(d.mtime),
				d.name,

				'library'        if d.source == SOURCE_LIBRARY else str(d.source) ,

				d.tenant.name    if d.tenant is not None       else '',
				d.owner.username if d.owner_id                 else '',

				n,
				n2,

				('%s (%s)' % (d.parent.name, SOURCE_DESCR[d.parent.source]['name'])) if d.parent else '' ,
				d.description[0:20],

				','.join(groups_by_tdg_id.get(d.uuid,[])),
			])

		output_table(cli, r)

	else:
		source = cli.env['SOURCE']

		name = words[1]

		datagroups = TDataGroup.objects

		datagroup = datagroups.get(tenant=tenant, source=source, name=name)

		cli.prnt(
			generic_details(
				datagroup,
			)
		)
