
import pprint

from django.utils.translation import ugettext as _

from sanet_common.sanetdefs.constants import CSTATE_TO_CODENAME

from sanet_common.timeutils import timestamp_to_isostr

from sanet_poller.agentctl import agent_ctl

from cli.clilib.ioslike import *
from cli.textformat import *

from cli.utils import *
from cli.textformat import *
from cli.prompts.defs import *

from cli.prompts import *

from core.models import *
from core.api.base import tenant_get
from core.api.base import db_set_current_tenant_schema

from core.api.cache import init_cache_manager_from_conf

from cli.prompts.common import output_table

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_datagroup(cli):

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	current_user   = cli.env.get('CURRENT_USER'  , None)

	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	#datagroups = DataGroup.active_objects.filter(tenant=tenant)

	datagroups = tenant.get_all_datagroups_by_user(current_user)
	
	datagroups = datagroups.extra(select={'path2':"replace(path,';','1')"}, order_by=['path2'])  #.order_by('path')

	if len(cli.canonical_words) < 4:
		_show_datagroups_list(cli, datagroups)



def cmd_show_datagroup_by_node(cli):

	# show datagroup node <name> [path]

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	node_name  = cli.canonical_words[3]
	
	element = MonitoredNode.objects.get(name=node_name)
	
	datagroups = element.get_all_datagroups()
	
	_show_datagroups_list(cli, datagroups)

	return

def _cmd_show_datagroup_by_element(cli, kind):

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	node_name    = cli.canonical_words[3]
	element_name = cli.canonical_words[4]

	parent  = MonitoredNode.objects.get(name=node_name)
	element = MonitoredElement.objects.get(kind=kind, name=element_name, parent=parent)
	
	datagroups = element.get_all_datagroups()
	

	_show_datagroups_list(cli, datagroups)


def cmd_show_datagroup_by_iface(cli):
	# show datagroup interface <node> <iface> [path]
	
	return _cmd_show_datagroup_by_element(cli, ELEMENT_KIND_IFACE)

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	node_name    = cli.canonical_words[3]
	element_name = cli.canonical_words[4]

	parent  = MonitoredNode.objects.get(name=node_name)
	element = MonitoredElement.objects.get(kind=ELEMENT_KIND_IFACE, name=element_name, parent=parent)
	
	datagroups = element.get_all_datagroups()
	

	_show_datagroups_list(cli, datagroups)

	return


def cmd_show_datagroup_by_storage(cli):
	# show datagroup storage <node> <s> [path]
	
	return _cmd_show_datagroup_by_element(cli, ELEMENT_KIND_STORAGE)
	

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	

	node_name  = cli.canonical_words[3]
	element_name = cli.canonical_words[4]

	parent  = MonitoredNode.objects.get(name=node_name)
	element = MonitoredElement.objects.get(kind=ELEMENT_KIND_STORAGE, name=element_name, parent=parent)
	
	datagroups = element.get_all_datagroups()

	_show_datagroups_list(cli, datagroups)

	return


def cmd_show_datagroup_by_service(cli):

	# show datagroup service <node> <s> [path]
	
	return _cmd_show_datagroup_by_element(cli, ELEMENT_KIND_SERVICE)
	

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	

	node_name  = cli.canonical_words[3]
	element_name = cli.canonical_words[4]

	parent  = MonitoredNode.objects.get(name=node_name)
	element = MonitoredElement.objects.get(kind=ELEMENT_KIND_SERVICE, name=element_name, parent=parent)
	
	datagroups = element.get_all_datagroups()

	_show_datagroups_list(cli, datagroups)

	return

def cmd_show_datagroup_by_device(cli):

	# show datagroup device <node> <d> [path]

	return _cmd_show_datagroup_by_element(cli, ELEMENT_KIND_DEVICE)


	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	node_name  = cli.canonical_words[3]
	element_name = cli.canonical_words[4]

	parent  = MonitoredNode.objects.get(name=node_name)
	element = MonitoredElement.objects.get(kind=ELEMENT_KIND_DEVICE, name=element_name, parent=parent)
	
	datagroups = element.get_all_datagroups()

	_show_datagroups_list(cli, datagroups)

	return


def cmd_show_datagroup_by_template(cli):

	# show datagroup template <template name> and-subtemplates

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	template_name  = cli.canonical_words[3]
	
	show_subtemplates = False
	if len(cli.canonical_words) >= 5:
		if cli.canonical_words[4] == 'and-subtemplates':
			show_subtemplates = True
	
	#
	tdg = TDataGroup.objects.get(name=template_name, tenant=tenant)
	
	templates = [ tdg, ]
	if show_subtemplates:
		templates += list(tdg.get_recursive_children())
	
	f = Q()
	f = f | Q(cdatagroup__tdatagroup__in=templates)
	f = f | Q(cdatagroup__tedatagroup__tdatagroup__in=templates)
	
	datagroups = DataGroup.active_objects.filter(tenant=tenant).filter(f).all()
	
	_show_datagroups_list(cli, datagroups)


def cmd_show_datagroup_by_scheduling_group(cli):

	# 0    1         2                3
	# show datagroup scheduling-group <name>

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	sg_name  = cli.canonical_words[3]
	
	sg = SchedulingGroup.objects.get(tenant=tenant, name=sg_name)
	
	f = Q(scheduling_group=sg)
	datagroups = DataGroup.active_objects.filter(tenant=tenant).filter(f).all()
	
	_show_datagroups_list(cli, datagroups)

def _show_datagroups_list(cli, datagroups):
	
	datagroups = datagroups.order_by('path')

	cache_manager = init_cache_manager_from_conf()

	cols = cli.env['COLS']

	r = [[
		[_('TIME')     , 12],
		[_('PATH')     , 30],
		[_('TITLE')    , 25],
		[_('DEPENDSON'), 15],
		[_('PERIOD')   ,  3],
		[_('LASTDONE') , 19],
		[_('LASTTIME') ,  5],
		[_('TRIES')    ,  3],
		[_('TIMEOUT')  ,  3],
		[_('SUSPEND')  , 20],
		[_('OWNER')    , 10],
		
	]]
	
	# FIX: use api
	for e in datagroups:
		
		e.load_fields_from_cache(cache_manager)
		
		r.append([
			timestamp_to_isostr(e.mtime),
			e.path,
			e.expanded_title(cache_manager=cache_manager), 
			e.dependson.path if e.dependson else '',
			e.minperiod,
			timestamp_to_isostr(e.lastdone),
			"%.3f" % (e.lastchecktotaltime) if e.lastchecktotaltime else '-',
			e.shorttries,
			e.timeout,
			('Yes, reason: ' + e.suspend_reason) if e.suspended else '',
			e.owner.username if e.owner else '',
		])

	output_table(cli, r)
	

def show_datagroup_details(cli):

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	# show datagroup path <path>
	
	datagroup_path = cli.canonical_words[3]
	
	dg = DataGroup.active_objects.get(path=datagroup_path)
	
	cache_manager = init_cache_manager_from_conf()
	
	_show_datagroup_details(cli, dg, cache_manager=cache_manager)
	

	
	
def _show_datagroup_details(cli, dg, cache_manager=None):
	
	display_fields = [
		[_('element')            , dg.element ],
		[_('path')               , dg.path ],
		[_('name')               , dg.name ],
		[_('title')              , dg.title ],
		[_('expanded title')     , dg.expanded_title(cache_manager=cache_manager)],
		[_('dependson')          , dg.dependson ],
		[_('minperiod')          , dg.minperiod ],
		[_('shorttries')         , dg.shorttries ],
		[_('timeout')            , dg.timeout ],
		[_('deleted')            , dg.deleted ],
		[_('suspended')          , dg.suspended ],
		[_('suspend_reason')     , dg.suspend_reason ],
		[_('suspend_time')       , timestamp_to_isostr(dg.suspend_time) ],
		[_('suspend_resume_time'), timestamp_to_isostr(dg.suspend_resume_time) ],
		[_('classification')     , dg.classification ],
		[_('agent')              , dg.agent ],
	]

	cli.prnt(text_details(display_fields))
	
	params = dg.datagroupparameter_set.all()
	if params:
		
		t = [[
			[_('PARAM NAME'), 10],
			[_('TYPE')      , 20],
			[_('VALUE')     , 20],
		]]
		for p in params:
			t = t + [[p.name, CLI_PTYPE_TO_NAME(p.vtype), p.value]]
		cli.prnt(text_table(t))


def cmd_show_datagroup_details_runstate(cli):

	# show datagroup path <path> runstate

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	path = cli.canonical_words[3]

	dg = DataGroup.active_objects.get(path=path)

	# Send to agent
	broker  = get_broker_connection_string()
	agent   = dg.element.agent
	cmd     = "datagroups"
	args    = [ path ]
	timeout = 3

	response = agent_ctl(broker, agent.uuid, cmd, list(args), timeout=timeout)

	# Print data
	data = json.loads(response)

	pprint.pprint(data)
	
	
def cmd_show_datasource_details_runstate(cli):

	# show datagroup path <path> runstate

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	path = cli.canonical_words[3]

	try:
		ds = DataSource.active_objects.get(path=path)
	except DataSource.DoesNotExist, e:
		ds = DataSource.active_objects.get(uuid=path)

	# Send to agent
	broker  = get_broker_connection_string()
	agent   = ds.element.agent
	cmd     = "datasources"
	args    = [ path ]
	timeout = 3

	response = agent_ctl(broker, agent.uuid, cmd, list(args), timeout=timeout)

	# Print data
	data = json.loads(response)

	pprint.pprint(data)
	
def cmd_show_datasource_details_storage(cli):

	# show datagroup path <path> storeinfo

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	path = cli.canonical_words[3]

	from django.core.management import call_command
	
	params = [ path ]
	options = {}
	call_command('get_ds_storeinfo', *params, **options)	

def cmd_show_condition_details_runstate(cli):

	# show datagroup path <path> runstate

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	path = cli.canonical_words[3]

	c = Condition.active_objects.get(path=path)

	# Send to agent
	broker  = get_broker_connection_string()
	agent   = c.element.agent
	cmd     = "conditions"
	args    = [ path ]
	timeout = 3
	
	response = agent_ctl(broker, agent.uuid, cmd, list(args), timeout=timeout)

	# Print data
	data = json.loads(response)

	pprint.pprint(data)
	
	return


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_datasource(cli):

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	current_user   = cli.env.get('CURRENT_USER'  , None)

	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)


	#datagroups = tenant.get_all_datagroups_by_user(current_user)
	#datagroups = datagroups.extra(select={'path2':"replace(path,';','1')"}, order_by=['path2'])  #.order_by('path')

	datasources = tenant.get_all_datasources_by_user(current_user)
	datasources = datasources.order_by('path')
	datasources = datasources.all()

	_show_datasources_list(cli, datasources)


def cmd_show_datasource_by_node(cli):
	# show datagroup node <name> [path]
	
	current_tenant = cli.env.get('CURRENT_TENANT', None)
	current_user   = cli.env.get('CURRENT_USER'  , None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	name = cli.canonical_words[3]

	element = MonitoredElement.objects.get(tenant=tenant, kind=ELEMENT_KIND_NODE, name=name)
	
	datasources = element.get_all_datasources()
	
	_show_datasources_list(cli, datasources)
	
	return

def cmd_show_datasource_by_iface(cli):
	# show datagroup interface <node> <iface> 
	_show_element_datasource(cli, ELEMENT_KIND_IFACE)

def cmd_show_datasource_by_storage(cli):
	_show_element_datasource(cli, ELEMENT_KIND_STORAGE)

def cmd_show_datasource_by_service(cli):
	_show_element_datasource(cli, ELEMENT_KIND_SERVICE)

def cmd_show_datasource_by_device(cli):
	_show_element_datasource(cli, ELEMENT_KIND_DEVICE)

def _show_element_datasource(cli, ekind):
	
	current_tenant = cli.env.get('CURRENT_TENANT', None)
	current_user   = cli.env.get('CURRENT_USER'  , None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	node_name  = cli.canonical_words[3]
	element_name = cli.canonical_words[4]

	node  = MonitoredElement.objects.get(tenant=tenant, kind=ELEMENT_KIND_NODE, name=node_name)
	
	element = MonitoredElement.objects.get(tenant=tenant, kind=ekind, name=element_name, parent=node)
	
	datasources = element.get_all_datasources()
	
	_show_datasources_list(cli, datasources)
	
	return

def cmd_show_datasource_by_template(cli):

	user = cli.env['CURRENT_USER']

	# show datagroup template <template name> and-subtemplates

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	template_name  = cli.canonical_words[3]
	
	show_subtemplates = False
	if len(cli.canonical_words) >= 5:
		if cli.canonical_words[4] == 'and-subtemplates':
			show_subtemplates = True

	tdg = TDataGroup.objects.get(name=template_name, tenant=tenant)
	
	templates = [ tdg, ]
	if show_subtemplates:
		templates += list(tdg.get_recursive_children())

	f = Q()
	f = f | Q(datagroup__cdatagroup__tdatagroup__in=templates)
	f = f | Q(datagroup__cdatagroup__tedatagroup__tdatagroup__in=templates)

	#datasources = DataSource.active_objects

	datasources = tenant.get_all_datasources_by_user(user)
	datasources = datasources.filter(tenant=tenant).filter(f).all()

	_show_datasources_list(cli, datasources)


def cmd_show_datasource_by_state(cli):

	user = cli.env['CURRENT_USER']

	STATE_OPTIONS = {
		'ok'         : DATASOURCE_STATUS_OK,
		'uncheckable': DATASOURCE_STATUS_UNCHECKABLE,
		'strangeval' : DATASOURCE_STATUS_STRANGE_VAL,
		'cascade'    : DATASOURCE_STATUS_CASCADE,
		'error'      : DATASOURCE_STATUS_ERROR,
	}

	# show datasource state <code>

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	state = cli.canonical_words[3]

	if STATE_OPTIONS.has_key(state):
		state = STATE_OPTIONS[state]
	else:
		raise Exception("invalid state: %s" % (state))

	#datasources = DataSource.active_objects
	datasources = tenant.get_all_datasources_by_user(user)
	
	datasources = datasources.filter(status=state).order_by('path').all()

	_show_datasources_list(cli, datasources)


def _show_datasources_list(cli, datasources):

	cache_manager = init_cache_manager_from_conf()

	cols = cli.env['COLS']

	r = [[
		#['TIME', 19],
		#['ELEMENT', -1],
		#['AGENT', 12],
		[_('UUID')    , 32],
		[_('PATH')    , 12],
		[_('TITLE')   , 25],
		[_('STATUS')  , 12],
		[_('SAVE')    ,  1],
		[_('O')       ,  3],
		[_('C')       ,  1],
		[_('G')       ,  1],
		[_('MIN')     ,  5],
		[_('MAX')     ,  5],
		[_('PERIOD')  ,  6],
		[_('LASTDONE'), 19],
		[_('LASTTIME'),  5],
		[_('INFO')    , 10],
		[_('OWNER')   , 10],
		#['EXPR', 30],
		#['INFO', 30],
	]]
	
	# FIX: use api
	datasources = datasources.select_related('datagroup').all()
	for e in datasources:
		
		e.load_fields_from_cache(cache_manager)
		
		v = e.get_scaled_last_value()
		
		if v is not None:
			if e.gauge:
				#if e.lastvalue is not None:
				#v = str(e.lastvalue)
				info = _('Last value') + ':' + v
			else:
				#if e.lastvaluec is not None:
				#v = str(e.lastvaluec)
				info = _('Last rate' ) + ':' + v

		else:
			info = e.verbstatus if e.verbstatus is not None else ''
		
		r.append([
			# e.element.name       
			#,e.agent.name      
			e.uuid    
			,e.path           
			#, e.title
			,e.expanded_title(cache_manager=cache_manager)
			
			, (COLORED_DSSTATUS_TEXT, e.status,  DSSTATE_TO_CODENAME(e.status)) 
			
			
			, 'S' if e.save_value else ''
			,e.order          
			, 'S' if e.cascade else ''
			, 'S' if e.gauge else ''
			,e.min_val if e.min_val is not None else ''        
			,e.max_val if e.max_val is not None else ''
			
			, e.datagroup.minperiod
			
			, timestamp_to_isostr(e.lastdone) # from cache
			, "%.3f" % (e.lastchecktotaltime) if e.lastchecktotaltime else '-'
			
			, info[0:40] #from cache
			
			, e.owner.username if e.owner else '',

			#,e.expr      
			  
			#,e.verbstatus   
		])

	output_table(cli, r)


def cmd_show_datasource_details(cli):

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	# show datasource path <path>
	
	path = cli.canonical_words[3]

	try:
		ds = DataSource.active_objects.get(path=path)
	except DataSource.DoesNotExist, e:
		ds = DataSource.active_objects.get(uuid=path)
	
	_show_datasource_details(cli, ds)


def _ds_status(cli, s):
	# L'output colorato va usato solo se e' abilitato e l'output e' in formato normale.
	# Il csv non deve contenere carattereri speciali
	if cli.env.get('COLORS', False) == True and cli.env.get('OUTPUT_FORMAT','normal') == 'normal':
		return COLORED_DSSTATUS_TEXT(s, DSSTATE_TO_CODENAME(s))
	else:
		return DSSTATE_TO_CODENAME(s)

def _show_datasource_details(cli, ds):
	
	cache_manager = init_cache_manager_from_conf()
	
	ds.load_fields_from_cache(cache_manager)

	display_fields = [
		[_('uuid')          , ds.uuid ],
		[_('element')       , ds.element ],
		[_('datagroup')     , ds.datagroup ],
		[_('path')          , ds.path ],
		[_('agent')         , ds.agent ],
		[_('name')          , ds.name ],
		[_('title')         , ds.title ],
		[_('expanded title'), ds.expanded_title(cache_manager=cache_manager)],
		[_('expr')          , ds.expr ],
		[_('order'         ), ds.order ],
		[_('cascade'       ), ds.cascade ],
		[_('save_value'    ), ds.save_value ],
		[_('gauge'         ), ds.gauge ],
		[_('min_val'       ), ds.min_val ],
		[_('max_val'       ), ds.max_val ],
		[_('storage_spec'  ), ds.storage_spec ],
		[_('classification'), ds.classification ],
		[_('element'       ), ds.element ],
		[_('agent'         ), ds.agent ],
		[_('last done'     ), timestamp_to_isostr(ds.lastdone) ],
		[_('last check time'), ds.lastchecktotaltime ],
		[_('status'        ), _ds_status(cli, ds.status) ],
		[_('verbstatus'    ), ds.verbstatus],
		[_('last value'    ), ds.lastvalue ],
	]

	cli.prnt(text_details(display_fields))


from core.api.datasource import init_datasource_store_from_conf


def cmd_show_datasource_details_data(cli):

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	# show datasource path <path> data [<n>]
	
	path = cli.canonical_words[3]

	if len(cli.canonical_words) >= 6:
		seconds = int(cli.canonical_words[5])
	else:
		seconds = 60
	
	try:
		ds = DataSource.active_objects.get(path=path)
	except DataSource.DoesNotExist, e:
		ds = DataSource.active_objects.get(uuid=path)
	
	store = init_datasource_store_from_conf()
	
	#data = store.fetch_last(ds, 5)
	ts_end   = int(time.time())
	ts_start = ts_end - seconds
	
	data = store.fetch_values(ds.tenant_id, ds.uuid, ts_start=ts_start, ts_end=ts_end)
	
	
	#print data['info']
	ts_start = data['scalex'][0]
	ts_step  = data['scalex'][2]
	
	for d in data['dataseries']:
		print ts_start, timestamp_to_isostr(ts_start), d
		ts_start += ts_step



#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_condition(cli):
	current_tenant = cli.env.get('CURRENT_TENANT', None)

	current_user   = cli.env.get('CURRENT_USER'  , None)

	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)


	#conditions = Condition.active_objects.all()
	conditions = tenant.get_all_conditions_by_user(current_user)

	conditions = conditions.select_related('datagroup', 'element','element__parent').extra(select={'path2':"replace(condition.path,';','1')"}, order_by=['path2'])  #.order_by('path')
	
	
	_show_conditions_list(cli, conditions)

def cmd_show_condition_by_value(cli):

	# show condition priority level  <s>
	current_tenant = cli.env.get('CURRENT_TENANT', None)
	current_user   = cli.env.get('CURRENT_USER'  , None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	if len(cli.canonical_words) == 4 and cli.canonical_words[3] == "undefined":
		value = 0
	elif len(cli.canonical_words) == 5:
		value = int(cli.canonical_words[4])
	else:
		raise Exception("Malformed command")

	#conditions = Condition.active_objects
	conditions = tenant.get_all_conditions_by_user(current_user)
	
	conditions = conditions.filter(priority=value).select_related('element','element__parent').all()
	
	_show_conditions_list(cli, conditions)


def cmd_show_condition_by_min_priority(cli):
	# show condition priority level  <s>
	current_tenant = cli.env.get('CURRENT_TENANT', None)
	current_user   = cli.env.get('CURRENT_USER'  , None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	level = int(cli.canonical_words[4])

	#conditions = Condition.active_objects.filter(priority__gte=level).select_related('element','element__parent').all()
	
	conditions = tenant.get_all_conditions_by_user(current_user)
	conditions = conditions.filter(priority__gte=level).select_related('element','element__parent').all()
	
	_show_conditions_list(cli, conditions)


def cmd_show_condition_by_priority_level(cli):
	# show condition priority level  <s>
	current_tenant = cli.env.get('CURRENT_TENANT', None)
	current_user   = cli.env.get('CURRENT_USER'  , None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	level = cli.canonical_words[4]
	
	pl = PriorityLabel.objects.get(label=level)

	conditions = pl.get_conditions()
	
	_show_conditions_list(cli, conditions)


def cmd_show_condition_by_status(cli):
	# show condition status  <s>
	current_tenant = cli.env.get('CURRENT_TENANT', None)
	current_user   = cli.env.get('CURRENT_USER'  , None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	status_name = cli.canonical_words[3]

	if status_name == 'down':
		states = [ CSTATE_DN, ]
	elif status_name == 'up':
		states = [ CSTATE_UP, ]
	elif status_name == 'suspended':
		states = [ CSTATE_IN, ]
	elif status_name == 'uncheckable':
		states = [ CSTATE_UU, CSTATE_UD, ]
	else:
		raise Exception("Invalid filter '%s'" % (status_name) )
	
	#conditions = Condition.active_objects
	conditions = tenant.get_all_conditions_by_user(current_user)
	if states:
		conditions = conditions.filter(status__in=states)

	conditions = conditions.order_by('path').all()
	
	_show_conditions_list(cli, conditions)
	
	return

def cmd_show_condition_by_node(cli):
	# show datagroup node <name> [path]

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	current_user   = cli.env.get('CURRENT_USER'  , None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	name = cli.canonical_words[3]

	element = MonitoredElement.objects.get(tenant=tenant, kind=ELEMENT_KIND_NODE, name=name)
	condition = element.get_all_conditions().select_related('element','element__parent')
	condition = condition.extra(select={'path2':"replace(condition.path,';','1')"}, order_by=['-primary', 'order', 'path2'])  #.order_by('path')

	_show_conditions_list(cli, condition)

	pass


def cmd_show_condition_by_iface(cli):
	# show datagroup interface <node> <iface> [path]
	_show_element_conditions(cli, ELEMENT_KIND_IFACE)
	pass

def cmd_show_condition_by_storage(cli):
	# show datagroup storage <node> <s> [path]
	_show_element_conditions(cli, ELEMENT_KIND_STORAGE)
	pass

def cmd_show_condition_by_service(cli):
	# show datagroup service <node> <s> [path]
	_show_element_conditions(cli, ELEMENT_KIND_SERVICE)
	pass

def cmd_show_condition_by_device(cli):
	# show datagroup device <node> <d> [path]
	_show_element_conditions(cli, ELEMENT_KIND_DEVICE)
	pass		
	
	
	

def _show_element_conditions(cli, ekind):
	
	current_tenant = cli.env.get('CURRENT_TENANT', None)
	current_user   = cli.env.get('CURRENT_USER'  , None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	node_name  = cli.canonical_words[3]
	element_name = cli.canonical_words[4]

	node  = MonitoredElement.objects.get(tenant=tenant, kind=ELEMENT_KIND_NODE, name=node_name)
	
	element = MonitoredElement.objects.get(tenant=tenant, kind=ekind, name=element_name, parent=node)
	
	condition = element.get_all_conditions().select_related('element','element__parent').extra(select={'path2':"replace(condition.path,';','1')"}, order_by=['path2'])  #.order_by('path')
	
	_show_conditions_list(cli, condition)
	
	return




def cmd_show_condition_details(cli):

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	# show datasource path <path>
	path = cli.canonical_words[3]

	c = Condition.active_objects.get(path=path)

	_show_condition_details(cli, c)


def cmd_show_conditions_by_template(cli):

	user = cli.env['CURRENT_USER']

	# show datagroup template <template name> and-subtemplates

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	template_name  = cli.canonical_words[3]
	
	show_subtemplates = False
	if len(cli.canonical_words) >= 5:
		if cli.canonical_words[4] == 'and-subtemplates':
			show_subtemplates = True

	tdg = TDataGroup.objects.get(name=template_name, tenant=tenant)
	
	templates = [ tdg, ]
	if show_subtemplates:
		templates += list(tdg.get_recursive_children())

	f = Q()
	f = f | Q(datagroup__cdatagroup__tdatagroup__in=templates)
	f = f | Q(datagroup__cdatagroup__tedatagroup__tdatagroup__in=templates)

	conditions = Condition.active_objects.filter(tenant=tenant).filter(f).all()
	
	conditions = tenant.get_all_conditions_by_user(user)
	conditions = conditions.filter(tenant=tenant).filter(f).all()

	_show_conditions_list(cli, conditions)
	
	return
	
def _show_conditions_list(cli, conditions):

	cache_manager = init_cache_manager_from_conf()
	
	cols = cli.env['COLS']

	r = [[
		#[_('TIME'      ), 19],
		#['UUID', 32],
		 [_('TYPE'      ), 10],
		#[_('ELEMENT'   ), -1],
		#['AGENT', 12],
		[_('PATH'      ), 12],

		[_('P'         ),  1],
		[_('O'         ),  2],

		#['DEPENDSON', 12],
		[_('TITLE'     ), 25],
		

		[_('PRI'       ),  5],

		[_('PERIOD'    ),  6],
		[_('LASTDONE'  ), 19],
		
		[_('CUR.'      ), 4],
		[_('OLD'       ), 3],

		#[_('NOTABLE'    ), 12],
		[_('LAST CHANGE'  ), 19],
		[_('LAST NOTABLE' ), 19],
		[_('LASTTIME'  ),  5],
		
		[_('MAX'       ),  3],
		[_('TRIES'     ),  3],
		
		[_('OWNER'     ),  10],
		
		#['EXPR', 30],
		#['INFO', 30],
	]]
	
	def _status(cli, s):
		# L'output colorato va usato solo se e' abilitato e l'output e' in formato normale.
		# Il csv non deve contenere carattereri speciali
		if cli.env.get('COLORS', False) == True and cli.env.get('OUTPUT_FORMAT','normal') == 'normal':
			return (COLORED_STATUS_TEXT, s     , CSTATE_TO_CODENAME(s)     )
		else:
			return CSTATE_TO_CODENAME(s)
	
	conditions = conditions.select_related('element')
	
	# FIX: use api
	#conditions = list(conditions)
	
	#cli.info("Loading %s conditions..." % (conditions.count()))
	
	for e in conditions.all():
		
		e.load_fields_from_cache(cache_manager)
		
		r.append([
			 #e.uuid,
			#timestamp_to_isostr(e.mtime)
			 
			  ELEMENT_KIND_STR[e.element.kind]
			#, e.element.path
			#,e.agent.name          
			, e.path

			, '*' if e.primary else ''
			, e.order

			#,e.title
			#,e.dependson.path if e.dependson else ''
			, e.expanded_title(cache_manager=cache_manager)
			
			, e.priority if e.priority is not PRIORITY_NONE else 'n/a'
			
			,e.datagroup.minperiod
			,timestamp_to_isostr(e.lastdone)
			
			, _status(cli, e.status    )
			, _status(cli, e.statuslast)

			,timestamp_to_isostr(e.statuslastchange)        if e.statuslastchange        != 0 else ''
			,timestamp_to_isostr(e.statuslastnotablechange) if e.statuslastnotablechange != 0 else ''
			, "%.3f" % (e.lastchecktotaltime) if e.lastchecktotaltime else '-'

			,e.max_tries
			,e.tries

			,e.owner.username if e.owner else ''
			#,e.expr           
			#,e.verbstatus
		])

	#cli.info("Preparing output...")
	
	output_table(cli, r)


def _show_condition_details(cli, c):
	
	cache_manager = init_cache_manager_from_conf()

	c.load_fields_from_cache(cache_manager)

	def _status(cli, s):
		# L'output colorato va usato solo se e' abilitato e l'output e' in formato normale.
		# Il csv non deve contenere carattereri speciali
		if cli.env.get('COLORS', False) == True and cli.env.get('OUTPUT_FORMAT','normal') == 'normal':
			return COLORED_STATUS_TEXT(s, CSTATE_TO_CODENAME(s))
		else:
			return CSTATE_TO_CODENAME(s)
	
	display_fields = [
		[_('uuid'             ), c.uuid    ],
		[_('element'          ), c.element ],
		[_('path'             ), c.path ],
		[_('agent'            ), c.agent  ],       
		[_('title'            ), c.title ],
		[_('expanded title'   ), c.expanded_title(cache_manager=cache_manager)],
	        [_('classification'   ), c.classification ],	
		[_('Tries'            ), "%s / %s" % (c.tries, c.max_tries) ],
		[_('email'       ), c.email ],
		[_('upemail'     ), c.upemail ],
		[_('msg_downsubj'), c.msg_downsubj ],
		[_('msg_downbody'), c.msg_downbody ],
		[_('msg_upsubj'  ), c.msg_upsubj ],
		[_('msg_upbody'  ), c.msg_upbody ],

		[_('fallback'         ), CSTATE_TO_CODENAME(c.uncheckable_fallback) if c.uncheckable_fallback else ''],
		[_('expr'             ), c.expr ],
		[_('lastdone'         ), timestamp_to_isostr(c.lastdone) if c.lastdone != None else ''],
		[_('last check time'), c.lastchecktotaltime ],
		[_('priority'         ), c.priority ],
		[_('status'           ), _status(cli, c.status) ],
		[_('statuslast'       ), _status(cli, c.statuslast) ],
		[_('statuslastchange' ), timestamp_to_isostr(c.statuslastchange) ],
		[_('verbstatus'       ), c.verbstatus if c.verbstatus else '' ],

		#[_('msg-downbody'     , c.msg_downbody ],
		#[_('msg-downsubject'     , c.msg_downbody ],
		#[_('msg-upbody'     , c.msg_downbody ],
		#[_('msg-upsubjectbody'     , c.msg_downbody ],
	]

	cli.prnt(text_details(display_fields))

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#


def cmd_show_timegraph(cli):
	# show datagroups [<name>]

	current_tenant = cli.env.get('CURRENT_TENANT', None)

	current_user   = cli.env.get('CURRENT_USER'  , None)

	tenant = tenant_get(current_tenant)
	tenant.db_set_current_schema()

	timegraphs = tenant.get_all_timegraphs_by_user(current_user)

	timegraphs = timegraphs.order_by('datagroup__path', 'name')  # FIX: add timegraph.path and order on this field

	_show_timegraphs_list(cli, timegraphs)


def cmd_show_timegraph_by_node(cli):
	# show datagroup node <name> [path]

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	current_user   = cli.env.get('CURRENT_USER'  , None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	name = cli.canonical_words[3]

	element = MonitoredElement.objects.get(tenant=tenant, kind=ELEMENT_KIND_NODE, name=name)

	timegraphs = element.get_all_timegraphs()

	timegraphs = timegraphs.order_by('datagroup__path', 'name')  # FIX: add timegraph.path and order on this field

	_show_timegraphs_list(cli, timegraphs)

	pass



def cmd_show_timegraph_by_iface(cli):
	# show datagroup interface <node> <iface> [path]
	_show_element_timegraphs(cli, ELEMENT_KIND_IFACE)
	pass

def cmd_show_timegraph_by_storage(cli):
	# show datagroup storage <node> <s> [path]
	_show_element_timegraphs(cli, ELEMENT_KIND_STORAGE)
	pass

def cmd_show_timegraph_by_service(cli):
	# show datagroup service <node> <s> [path]
	_show_element_timegraphs(cli, ELEMENT_KIND_SERVICE)
	pass

def cmd_show_timegraph_by_device(cli):
	# show datagroup device <node> <d> [path]
	_show_element_timegraphs(cli, ELEMENT_KIND_DEVICE)
	pass		


def _show_element_timegraphs(cli, ekind):
	
	current_tenant = cli.env.get('CURRENT_TENANT', None)
	current_user   = cli.env.get('CURRENT_USER'  , None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	node_name  = cli.canonical_words[3]
	element_name = cli.canonical_words[4]

	node  = MonitoredElement.objects.get(tenant=tenant, kind=ELEMENT_KIND_NODE, name=node_name)

	element = MonitoredElement.objects.get(tenant=tenant, kind=ekind, name=element_name, parent=node)

	l = element.get_all_timegraphs()

	_show_timegraphs_list(cli, l)
	
	return



def _show_timegraphs_list(cli, timegraphs):

	cols = cli.env['COLS']

	cache_manager = init_cache_manager_from_conf()

	r = [[
		[_('TIME'          ), 19],
		[_('UUID'          ), 34],
		[_('DATAGROUP'     ), 12],
		[_('OWNER'         ), 12],
		[_('NAME'          ), 12],
		[_('TITLE'         ), 12],
		[_('PRIMARY'       ), 12],
		[_('STACKABLE'     ), 12],
		[_('YLEGEND'       ), 12],
		[_('CLASSIFICATION'), 12],
	]]
	
	# FIX: use api
	for e in timegraphs:
		
		r.append([
			timestamp_to_isostr(e.mtime),
			e.uuid,
			e.datagroup.path if e.datagroup is not None else '',
			e.name,         
			e.expanded_title(cache_manager=cache_manager),
			'y' if e.primary else ' ',   
			'y' if e.stacked else ' ', 
			e.ylegend,      
			e.classification,
			e.owner.username if e.owner else ''
		])

	#cli.prnt(text_table(r, cols))

	output_table(cli, r)

	return


def cmd_show_timegraph_details(cli):

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	# show timegraph path <path>

	path = cli.canonical_words[3]

	dg_path, name = path.rsplit(";", 1)

	tg = TimeGraph.active_objects.get(datagroup__path=dg_path, name=name)

	_show_timegraph_details(cli, tg)

def _show_timegraph_details(cli, t):

	#cache_manager = init_cache_manager_from_conf()
	#t.load_fields_from_cache(cache_manager)

	display_fields = [
		[ _('uuid'          ), t.uuid           ],
		[ _('element'       ), t.element        ],
		[ _('path'          ), t.path           ],
		[ _('title'         ), t.title          ],
		[ _('name'          ), t.name           ], 
		[ _('primary'       ), t.primary        ], 
		[ _('stacked'       ), t.stacked        ], 
		[ _('ylegend'       ), t.ylegend        ], 
		[ _('classification'), t.classification ], 
	]

	cli.prnt(text_details(display_fields))	

	#
	# Series
	#
	r = [[
		[_('NAME'           ), 19],
		[_('LEGEND'         ), 19],
		[_('COLOR'          ), 19],
		[_('STYLE'          ), 19],
		
		[_('DATASOURCE'    ), 34],
		[_('STATUS'        ), 12],
		[_('COMPUTED'      ), 30],
	]]

	datasources = t.datagroup.datasources.all()
	datasource_by_name = dict([ (ds.name, ds) for ds in datasources ])

	series = t.series.order_by('name').all()

	for s in series:

		if s.variable_value:
			ds = datasource_by_name.get(s.variable_value, None)
			if ds:
				path   = ds.path
				status = _ds_status(cli, ds.status)
			else:
				path   = '*ERROR*'
				status = '-'
		else:
			path   = '-'
			status = '-'

		r.append([
			s.name,
			s.legend,
			s.color,
			s.style,

			path,
			status,
			s.computed_value_expr
		])

	output_table(cli, r)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_timegraphserie(cli):
	
	current_tenant = cli.env.get('CURRENT_TENANT', None)

	current_user   = cli.env.get('CURRENT_USER'  , None)

	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	# show datagroups [<name>]

	#timegraphseries = TimeGraphSerie.active_objects.order_by('timegraph__element', 'timegraph__datagroup').all()
	
	timegraphseries = tenant.get_all_timegraphseries_by_user(current_user)
	timegraphseries = timegraphseries.order_by('timegraph__element', 'timegraph__datagroup').all()
	
	_show_timegraphserie(cli, timegraphseries)
	
	
def cmd_show_timegraphserie_by_datagroup(cli):

	# show timegraphserie datagroup <path>
	
	current_tenant = cli.env.get('CURRENT_TENANT', None)
	current_user   = cli.env.get('CURRENT_USER'  , None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	path = cli.canonical_words[3]
	
	datagroup = DataGroup.active_objects.get(path=path)
	
	timegraphseries = TimeGraphSerie.active_objects.filter(timegraph__datagroup=datagroup).all()
	
	_show_timegraphserie(cli, timegraphseries)

def cmd_show_timegraphserie_by_node(cli):

	# show timegraphserie node <name>
	
	current_tenant = cli.env.get('CURRENT_TENANT', None)
	current_user   = cli.env.get('CURRENT_USER'  , None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	name = cli.canonical_words[3]
	
	element   = MonitoredNode.objects.get(name=name)
	
	timegraphseries = TimeGraphSerie.active_objects.filter(timegraph__element=element).all()
	_show_timegraphserie(cli, timegraphseries)


def cmd_show_timegraphserie_by_iface(cli):
	return _cmd_show_timegraphserie_by_element(cli, ELEMENT_KIND_IFACE)
	
def cmd_show_timegraphserie_by_storage(cli):
	return _cmd_show_timegraphserie_by_element(cli,  ELEMENT_KIND_STORAGE)

def cmd_show_timegraphserie_by_service(cli):
	return _cmd_show_timegraphserie_by_element(cli,  ELEMENT_KIND_SERVICE)

def cmd_show_timegraphserie_by_device(cli):
	return _cmd_show_timegraphserie_by_element(cli, ELEMENT_KIND_DEVICE)

def _cmd_show_timegraphserie_by_element(cli, kind):

	# show timegraphserie iface <node> <iface>
	
	current_tenant = cli.env.get('CURRENT_TENANT', None)
	current_user   = cli.env.get('CURRENT_USER'  , None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	node_name    = cli.canonical_words[3]
	element_name = cli.canonical_words[4]
	
	element   = MonitoredElement.objects.get(kind=kind, parent__name=node_name, name=element_name)
	
	timegraphseries = TimeGraphSerie.active_objects.filter(timegraph__element=element).select_related('element', 'timegraph', 'timegraph__element', 'timegraph__datagroup').all()
	_show_timegraphserie(cli, timegraphseries)


	

def _show_timegraphserie(cli, timegraphseries):

	cols = cli.env['COLS']

	r = [[
		[_('TIME'     ), 19],
		[_('NODE'     ), 12],
		[_('DATAGROUP'), 30],
		[_('TIMEGRAPH'), -1],
		[_('NAME'     ), 12],
		[_('LEGEND'   ), 30],
		[_('COLOR'    ), 12],
		[_('STYLE'    ),  5],
		[_('ORDER'    ), 12],
		[_('VARIABLE' ), 12],
		[_('COMPUTED' ), 12],
		[_('OWNER'    ), 10],
	]]

	# FIX: use api
	for e in timegraphseries:
		r.append([
			timestamp_to_isostr(e.mtime),
			e.timegraph.element.name,
			e.timegraph.datagroup.path,
			e.timegraph.name,
			e.name,
			e.legend,
			e.color,
			e.style,
			e.render_order,
			e.variable_value,
			e.computed_value_expr,
			e.owner.username if e.owner else '',
		])

	output_table(cli, r)
	

