# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from django.utils.translation import ugettext as _

from core.api.base import db_set_current_tenant_schema

from core.api.base import tenant_get

from cli.textformat import generic_details
from cli.utils import convert_special_fields_val_to_cli_val

from cli.prompts import cli_get_registered_hooks
from cli.prompts.defs import COMMON_FIELDS
from cli.prompts.common import output_table

from cli.prompts.show.utils import get_templates_by_element_id


from core.models import MonitoredDevice
from cli.prompts.defs import DEVICE_FIELDS

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_devices(cli):

	user   = cli.env['CURRENT_USER']

	# show interfaces node <node> <iface>
	
	current_tenant = cli.env.get('CURRENT_TENANT', None)
	current_user   = cli.env.get('CURRENT_USER'  , None)
	cols = cli.env['COLS']

	tenant = tenant_get(current_tenant)

	db_set_current_tenant_schema(tenant.name)

	if len(cli.canonical_words) == 2:

		#devices = MonitoredDevice.objects.filter(tenant=tenant)

		devices = MonitoredDevice.get_visible_by_user(tenant, user)

		devices = devices.order_by('parent__name', 'name')

		_show_devices(cli, tenant, devices)

	elif len(cli.canonical_words) == 4  and cli.canonical_words[2] == "node":

		node_name = cli.canonical_words[3]

		devices = MonitoredDevice.objects.filter(tenant=tenant)
		devices = devices.filter(parent__name=node_name)
		devices = devices.order_by('parent__name', 'name')

		_show_devices(cli, tenant, devices)

	elif len(cli.canonical_words) == 4  and cli.canonical_words[2] == "template":

		template_name = cli.canonical_words[3]

		devices = MonitoredDevice.filter_by_template(template_name)
		devices = devices.filter(tenant=tenant)
		devices = devices.order_by('parent__name', 'name')

		_show_devices(cli, tenant, devices)
	else:
		node_name = cli.canonical_words[3]

		device_name = cli.canonical_words[4]

		e = MonitoredDevice.objects.get(tenant=tenant, parent__name=node_name, name=device_name)
		
		#
		visible_fields  = COMMON_FIELDS
		cli.prnt(generic_details(e, visible=visible_fields, convert_val_fun=convert_special_fields_val_to_cli_val))

		#
		visible_fields  = DEVICE_FIELDS
		cli.prnt(generic_details(e,visible=visible_fields, convert_val_fun=convert_special_fields_val_to_cli_val))
		
		#
		cli.prnt("Applied templates")
		cli.prnt("-----------------")
		
		for t in e.get_used_templates():
			cli.prnt("\t- %s %s" % (t.name, t.description))
		cli.prnt("")
		
		#
		hooks = cli_get_registered_hooks('show:device')
		for h in hooks:
			h(cli, e)

def _show_devices(cli, tenant, devices):

	templates_by_elementid = get_templates_by_element_id(tenant, [e.uuid for e in devices] )
	
	r = []
	r.append([
		[_('TIME'         ),  9],
		[_('NODE'         ), 10],
		[_('NAME'         ), -4],
		[_('OWNER'        ), 10],
		[_('AGENT'        ), 10],
		[_('TEMPLATE'     ), 10],
		[_('FROM-TEMPLATE'), 12],
		[_('XFORM'        ), 30],
		[_('DISTINGUISHER'), 10],
		[_('DEVINDEX'     ),  5],
		[_('DESCRIPTION'  ), 30],
	])
	for s in devices:
		
		name = s.name
		
		templates = templates_by_elementid.get(s.uuid,'')

		r.append([
			s.mtime, 

			s.parent.name,
			name,

			s.owner.username if s.owner else '',

			s.agent.name,

			templates,
			s.telement_generator.name if s.telement_generator_id is not None else '',

			s.xform,
			s.distinguisher,
			s.devindex,


			s.description,
		])

	output_table(cli, r)	
	
