# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from django.utils.translation import ugettext as _

from core.api.base import db_set_current_tenant_schema

from core.api.base import tenant_get

from cli.textformat import generic_details
from cli.utils import convert_special_fields_val_to_cli_val

from cli.prompts import cli_get_registered_hooks
from cli.prompts.defs import COMMON_FIELDS
from cli.prompts.common import output_table

from cli.prompts.show.utils import get_templates_by_element_id


from core.models import MonitoredIface
from cli.prompts.defs import IFACE_FIELDS


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_ifaces(cli):

	user   = cli.env['CURRENT_USER']

	# 0    1          2        3      4
	# show interfaces
	# show interfaces node     <node>
	# show interfaces node     <node> <iface>
	# show interfaces template <name>
	
	current_tenant = cli.env.get('CURRENT_TENANT', None)
	current_user   = cli.env.get('CURRENT_USER'  , None)
	cols = cli.env['COLS']

	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	if len(cli.canonical_words) == 2:

		#ifaces = MonitoredIface.objects.filter(tenant=tenant)
		ifaces = MonitoredIface.get_visible_by_user(tenant, user)
		
		ifaces = ifaces.order_by('parent__name', 'name')

		_show_ifaces(cli, tenant, ifaces)

	elif len(cli.canonical_words) == 4 and cli.canonical_words[2] == "node":

		node_name = cli.canonical_words[3]

		#ifaces = MonitoredIface.objects.filter(tenant=tenant)
		ifaces = MonitoredIface.get_visible_by_user(tenant, user)

		ifaces = ifaces.filter(parent__name=node_name)
		ifaces = ifaces.order_by('parent__name', 'name')

		_show_ifaces(cli, tenant, ifaces)


	elif len(cli.canonical_words) == 5 and cli.canonical_words[2] == "node":

		from core.api.cache import init_cache_manager_from_conf
		cm = init_cache_manager_from_conf()

		node_name = cli.canonical_words[3]
		iface_name = cli.canonical_words[4]
		
		e = MonitoredIface.objects.get(tenant=tenant, parent__name=node_name, name=iface_name)

		e.set_cache_manager(cm)

		visible_fields  = COMMON_FIELDS
		cli.prnt(generic_details(e, visible=visible_fields, convert_val_fun=convert_special_fields_val_to_cli_val))
		
		visible_fields  = IFACE_FIELDS
		cli.prnt(generic_details(e,visible=visible_fields, convert_val_fun=convert_special_fields_val_to_cli_val))
		
		cli.prnt("Applied templates")
		cli.prnt("-----------------")
		
		for t in e.get_used_templates():
			cli.prnt("\t- %s %s" % (t.name, t.description))
		cli.prnt("")
		
		
		hooks = cli_get_registered_hooks('show:interface')
		for h in hooks:
			h(cli, e)

	elif len(cli.canonical_words) == 4 and cli.canonical_words[2] == "template":

		template_name = cli.canonical_words[3]

		objects = MonitoredIface.filter_by_template(template_name)
		objects = objects.filter(tenant=tenant)
		
		_show_ifaces(cli, tenant, objects)


def _show_ifaces(cli, tenant, ifaces):

	templates_by_elementid = get_templates_by_element_id(tenant, [e.uuid for e in ifaces] )

	r = []
	r.append([
		[_('TIME'         ),  9],
		[_('NODE'         ), 10],
		[_('NAME'         ), -4],
		[_('OWNER'        ), 10],
		[_('AGENT'        ), 10],
		[_('TEMPLATE'     ), 10],
		[_('FROM-TEMPLATE'), 12],
		[_('XFORM'        ), 30],
		[_('DISTINGUISHER'), 10],
		[_('IFINDEX'      ),  5],
		[_('BACKBONE'     ),  8],
		[_('DESCRIPTION'  ), 30],
	])
	for i in ifaces:
		
		name = i.name
		
		templates = templates_by_elementid.get(i.uuid, '')

		r.append([
			i.mtime, 
			
			i.parent.name,
			
			name,
			
			i.owner.username if i.owner else '',
			
			i.agent.name,
			
			templates,
			i.telement_generator.name if i.telement_generator_id is not None else '',
			
			i.xform,
			i.distinguisher,
			i.ifindex,

			'*' if i.backbone else '',
			
			
			i.description,
		])

	output_table(cli, r)	

