# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from django.utils.translation import ugettext as _

from cli.clilib.ioslike import T_Keyword
from cli.clilib.ioslike import SORTED_KEYWORDS

from cli.prompts.show.configuration import do_show_conf_one_datagroup_template

from cli.prompts.show.configuration import base_cmd_show_conf_datagroup_templates

from cli.tokens import T_DataGroupTemplate
from cli.prompts.show.datagroup_templates import cmd_show_datagroup_templates

from sanet_common.sanetdefs.constants import SOURCE_LIBRARY
from core.models import TDataGroup

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_conf_library(cli):

	tdgs = TDataGroup.objects.filter(tenant=None, source=SOURCE_LIBRARY, parent=None).order_by('name')
	for tdg in tdgs:
		_r_show_conf(cli, tdg)
		
def _r_show_conf(cli, tdg):
	
	do_show_conf_one_datagroup_template(cli, tdg)
	
	for tdg2 in TDataGroup.objects.filter(tenant=None, source=SOURCE_LIBRARY, parent=tdg).order_by('name'):
	
		_r_show_conf(cli, tdg2)
	

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_conf_library_datagroup_templates(cli):

	source = SOURCE_LIBRARY

	tenant = None

	if len(cli.canonical_words) == 4:
		name = cli.canonical_words[3]
	else:
		name = None

	base_cmd_show_conf_datagroup_templates(cli, source, tenant, name)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_this_library(cli):
	
	from core.api.templates import datagroup_template_get
	from cli.prompts.show.configuration import do_show_conf_one_datagroup_template
	
	if cli.env.get('SOURCE') == SOURCE_LIBRARY:
		
		datagroup_name = cli.env.get('CONFIG_DATAGROUP_TEMPLATE', None)
		if datagroup_name:
			d = datagroup_template_get(datagroup_name, source=SOURCE_LIBRARY)
			do_show_conf_one_datagroup_template(cli, d)
			return
			
		return

	
#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

show_commands_library = [
	
	T_Keyword('show'  , help=_('Show running system information'), subtree=SORTED_KEYWORDS([
	
		T_Keyword('configuration',  help='', callback=cmd_show_conf_library, subtree=[
		
			T_Keyword('datagroup-template' , help='', subtree=[
				T_DataGroupTemplate(help='', callback=cmd_show_conf_library_datagroup_templates, ),
			]),
			
			T_Keyword('this' , help='', callback=cmd_show_this_library),
		]),

		T_Keyword('template' , help='', subtree=[
		
			T_Keyword('datagroup-template' , help='', callback=cmd_show_datagroup_templates, subtree=[
				T_DataGroupTemplate(help='', callback=cmd_show_datagroup_templates, ),
			]),	
			
		]),

		
		
	])),
]
