# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from sanet_common.timeutils import timestamp_to_isostr

from django.utils.translation import ugettext as _

from core.models import *
from core.api.base import *

from cli.clilib.ioslike import *
from cli.textformat import *

from cli.prompts.defs import COLORED_STATUS_TEXT

from cli.prompts.common import output_table

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_log_by_id(cli):

	user   = cli.env['CURRENT_USER']

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	
	# show log id <id>
	uuid = cli.canonical_words[3]
	
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	
	log = tenant.get_logs().get(uuid=uuid)


	# Normal user can view "sensibile data" only with the capability. #545
	if user is not None:
		if 'gui.view_sensible_data' not in user.get_capabilities(tenant):
			log.hide_sensible_data()


	visible_fields = [
		'uuid',             
		'tenant',           
		'timestamp',        
		'element_path',     
	]

	if log.kind == EVENT_CONDITION:
		visible_fields += [
			'condition_path',
			'priority',  
			'status',
			'statuslast',
			'statuslastchange',
			'msg',
		]
		
		try:
			a = Alarm.objects.get(event=log)
			log.alarmid = a.uuid
			visible_fields += ['alarmid']
		except Alarm.DoesNotExist, e:
			pass

	cli.prnt(
		generic_details(
			log,
			visible=visible_fields
		)
	)


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_logs(cli):
	
	# show logs [ last <n> ]

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	words = cli.canonical_words[2:]
	
	return _show_last_resource_logs(cli, tenant, words)
	
#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_node_logs(cli):
	# show logs node <n> [last <n>]
	return _cmd_show_sub_logs(cli, ELEMENT_KIND_NODE, cli.canonical_words[3], words=cli.canonical_words[4:])

def cmd_show_interface_logs(cli):
	# show logs interface <n> <i> [last <n>]
	return _cmd_show_sub_logs(cli, ELEMENT_KIND_IFACE  , cli.canonical_words[4], node_name=cli.canonical_words[3], words=cli.canonical_words[5:])

def cmd_show_storage_logs(cli):
	# show logs storage <n> <i> [last <n>]
	return _cmd_show_sub_logs(cli, ELEMENT_KIND_STORAGE, cli.canonical_words[4], node_name=cli.canonical_words[3], words=cli.canonical_words[5:])

def cmd_show_service_logs(cli):
	# show logs service <n> <i> [last <n>]
	return _cmd_show_sub_logs(cli, ELEMENT_KIND_SERVICE, cli.canonical_words[4], node_name=cli.canonical_words[3], words=cli.canonical_words[5:])

def cmd_show_device_logs(cli):
	# show logs device <n> <i> [last <n>]
	return _cmd_show_sub_logs(cli, ELEMENT_KIND_DEVICE , cli.canonical_words[4], node_name=cli.canonical_words[3], words=cli.canonical_words[5:])

def _cmd_show_sub_logs(cli, kind, element_name, node_name=None, words=[]):

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	parent = None
	if node_name:
		parent = MonitoredNode.objects.get(tenant=tenant, name=node_name)
	
	element = MonitoredElement.objects.get(kind=kind, parent=parent, name=element_name)
	
	_show_last_resource_logs(cli, element, words=words)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_datagroup_logs(cli):
	# show logs datagroup <path> [ last <n> ]
	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	words = cli.canonical_words[4:]

	dg = DataGroup.objects.get(tenant=tenant, path=cli.canonical_words[3])
	
	return _show_last_resource_logs(cli, dg, words)


def cmd_show_condition_logs(cli):
	# show logs datagroup <path> [ last <n> ]
	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	words = cli.canonical_words[4:]

	c = Condition.objects.get(tenant=tenant, path=cli.canonical_words[3])

	return _show_last_resource_logs(cli, c, words)

def cmd_show_status_logs(cli):
	# show logs status <state> [ last <n> ]
	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)
	
	#words = cli.canonical_words[4:]
	#c = Condition.objects.get(tenant=tenant, path=cli.canonical_words[3])
	#return _show_last_resource_logs(cli, c, words)
	
	
	status_name = cli.canonical_words[3]
	words = cli.canonical_words[4:]
	

	if status_name == 'down':
		states = [ CSTATE_DN, ]
	elif status_name == 'up':
		states = [ CSTATE_UP, ]
	elif status_name == 'suspended':
		states = [ CSTATE_IN, ]
	elif status_name == 'uncheckable':
		states = [ CSTATE_UU, CSTATE_UD, ]
	else:
		raise Exception("Invalid filter '%s'" % (status_name) )

	if words:
		count = int(words[1])
	else:
		count = 50

	logs = Event.objects
	logs = logs.filter(status__in=states)
	
	_show_last_logs(cli, logs, last=count)
	
	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def _show_last_resource_logs(cli, resource, words):
	# ...... last <n>
	
	if words:
		count = int(words[1])
	else:
		count = 50

	current_user = cli.env.get('CURRENT_USER', None)
	#return _show_resource_logs(cli, resource, last=count)
	
	logs = resource.get_logs_by_user(current_user)
	
	_show_last_logs(cli, logs, last=count)

#def _show_resource_logs(cli, element, last=50):
#	logs = element.get_logs().order_by('-timestamp')
#	
#	_show_last_logs(cli, logs, last)
	
	
def _show_last_logs(cli, logs, last):

	user   = cli.env['CURRENT_USER']


	cols = cli.env['COLS']

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	tenant.db_set_current_schema()

	
	logs = logs.order_by('-timestamp')
	logs = logs[0:last]
	
	logs = [ l for l in logs ]
	logs.reverse()

	r = [[
		[_('ID'       ), 32],
		[_('TIME'     ), 19],
		[_('ELEMENT'  ), 15],
		[_('CONDITION'), 20],
		[_('OLD'      ),  3],
		[_('NEW'      ),  3],
		[_('MSG'      ), 60],
	]]	
	for l in logs:

		# Normal user can view "sensibile data" only with the capability. #545
		if user is not None:
			if 'gui.view_sensible_data' not in user.get_capabilities(tenant):
				l.hide_sensible_data()
		
		if cols is not None and int(cols) == 0:
			msg = l.msg
		else:
			MAX_LEN=60
			if len(l.msg) > MAX_LEN:
				msg = l.msg[0:(MAX_LEN)-3].replace('\n', '\\n')+'...'
			else:
				msg = l.msg
		
		r.append([
			l.uuid,
			timestamp_to_isostr(l.timestamp),
			l.element_path  ,
			l.condition_path,
			#CSTATE_TO_CODENAME(l.statuslast)    ,
			#CSTATE_TO_CODENAME(l.status)        ,

			(COLORED_STATUS_TEXT, l.statuslast, CSTATE_TO_CODENAME(l.statuslast))    ,
			(COLORED_STATUS_TEXT, l.status    , CSTATE_TO_CODENAME(l.status))        ,
			
			msg,
		])
	
	output_table(cli, r)


