# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import json
import pprint

from django.utils.translation import ugettext as _

from sanet_poller.agentctl import agent_ctl

from conf import get_broker_connection_string

from core.models import MonitoredNode
from core.api.base import db_set_current_tenant_schema

from cli.prompts import cli_get_registered_hooks
from cli.prompts.defs import *
from cli.prompts.common import output_table
from cli.prompts.show.utils import get_templates_by_element_id

from cli.clilib.ioslike import *

from cli.textformat import generic_details

from cli.utils import *

from core.api.base import tenant_get, node_get, node_select_all
from core.api.cache import init_cache_manager_from_conf	

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_nodes(cli):
	
	user   = cli.env['CURRENT_USER']

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	current_user   = cli.env.get('CURRENT_USER'  , None)
	cols = cli.env['COLS']

	tenant = tenant_get(current_tenant)
	tenant.db_set_current_schema()
	
	cache_manager = init_cache_manager_from_conf()

	# show nodes .... [<name>]
	if len(cli.canonical_words) == 2:

		#nodes = node_select_all(tenant)
		
		nodes = MonitoredNode.get_visible_by_user(tenant, user)
		
		nodes = nodes.select_related('agent', 'cluster')  # django 1.11  select_related('agent__name', 'cluster__name') do not work

		_show_nodes(cli, tenant, nodes)
	
	elif cli.canonical_words[2] == 'agent':

		agent_name = cli.canonical_words[3]

		#nodes = node_select_all(tenant)
		nodes = MonitoredNode.get_visible_by_user(tenant, user)
		
		nodes = nodes.select_related('agent', 'cluster')  # django 1.11  select_related('agent__name', 'cluster__name') do not work
		
		nodes = nodes.filter(agent__name=agent_name)
		
		_show_nodes(cli, tenant, nodes)

	elif cli.canonical_words[2] == 'template':
		template_name = cli.canonical_words[3]

		#nodes = node_select_all(tenant)
		nodes = MonitoredNode.filter_by_template(template_name)
		nodes = nodes.filter(tenant=tenant)
		nodes = nodes.select_related('agent', 'cluster')  # django 1.11  select_related('agent__name', 'cluster__name') do not work
		
		#nodes = nodes.filter_by_template(template_name)
		
		_show_nodes(cli, tenant, nodes)

	elif cli.canonical_words[2] == 'cluster':
		
		cluster_name = template_name = cli.canonical_words[3]
		
		nodes = MonitoredNode.objects.filter(cluster__name=cluster_name)
		nodes = nodes.filter(tenant=tenant)
		nodes = nodes.select_related('agent', 'cluster')  # django 1.11  select_related('agent__name', 'cluster__name') do not work

		_show_nodes(cli, tenant, nodes)

	else:

		node_name = cli.canonical_words[3]

		e = node_get(node_name, current_tenant)
		
		e.load_fields_from_cache(cache_manager=cache_manager)
		
		
		# Normal user can view "sensibile data" only with the capability. #545
		if user is not None:
			if 'gui.view_sensible_data' not in user.get_capabilities(tenant):
				e.hide_sensible_data()
		
		visible_fields  = COMMON_FIELDS
		cli.prnt(generic_details(e, visible=visible_fields, convert_val_fun=convert_special_fields_val_to_cli_val))

		visible_fields  = NODE_FIELDS
		cli.prnt(generic_details(e, visible=visible_fields, convert_val_fun=convert_special_fields_val_to_cli_val))

		visible_fields  = SNMP_FIELDS
		cli.prnt(generic_details(e, visible=visible_fields,  convert_val_fun=convert_special_fields_val_to_cli_val))

		if e.cluster is not None:
			visible_fields  = CLUSTER_FIELDS
			cli.prnt(generic_details(e, visible=visible_fields, convert_val_fun=convert_special_fields_val_to_cli_val))
			
			
		cli.prnt("Applied templates")
		cli.prnt("-----------------")
		
		for t in e.get_used_templates():
			cli.prnt("\t- %s %s" % (t.name, t.description))

		cli.prnt("")

		hooks = cli_get_registered_hooks('show:node')
		for h in hooks:
			h(cli, e)



def _show_nodes(cli, tenant, nodes):
	
	nodes.select_related('agent', 'cluster')
	
	templates_by_elementid = get_templates_by_element_id(tenant,  [n.uuid for n in nodes] )
	
	#
	# PRINT
	#
	r = [[
		[_('NAME'       ), -4],
		[_('AGENT'      ), 10],
		[_('COMMUNITY'  ), 10],
		[_('TIMEOUT'    ), 10],
		[_('ICON'       ), 10],
		[_('S'          ),  1],
		[_('R'          ),  1],
		[_('CLUSTER'    ),  5],
		[_('P.'         ),  3],
		[_('OWNER'      ), 10],
		[_('TEMPLATE'   ), 10],
		[_('EFFECTIVE IP'), 15],
		[_('DESCRIPTION'), 20],
	]]
	for n in nodes:
		
		name = n.name
		
		templates = templates_by_elementid.get(n.uuid, '')

		r.append([
			name,
			
			n.agent.name,

			n.snmp_community,
			n.dft_timeout,
			
			n.icon if n.icon else '',
			
			'*' if n.is_switch else '',
			'*' if n.is_router else '',

			n.cluster.name if n.cluster != None else None,
			n.priority,

			n.owner.username if n.owner is not None else '',
			
			templates , 

			n.effective_ip,
			n.description,
			
		])

	output_table(cli, r)


def cmd_show_nodes_runstate(cli):

	# show nodes name <name> runstate
	
	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	
	name = cli.canonical_words[3]
	e = MonitoredNode.objects.get(tenant=tenant, name=name)

	# Send to agent
	broker  = get_broker_connection_string()
	agent   = e.agent
	cmd     = "nodes"
	args    = [ e.name ]
	timeout = 3

	response = agent_ctl(broker, agent.uuid, cmd, list(args), timeout=timeout)

	# Print data
	data = json.loads(response)

	pprint.pprint(data)	
	
