# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from django.utils.translation import ugettext as _

from sanet_common.sanetdefs.constants import *

from core.models import *
from core.api.base import db_set_current_tenant_schema

from cli.clilib.ioslike import *
from cli.textformat import *

from cli.prompts.common import output_table

from cli.prompts.show.utils import get_groups_by_element_template_id

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_all_templates(cli):
	return cmd_show_element_templates(cli, show_all=True)
	
def cmd_show_node_templates(cli):
	return cmd_show_element_templates(cli, kind=ELEMENT_KIND_NODE)

def cmd_show_interface_templates(cli):
	return cmd_show_element_templates(cli, kind=ELEMENT_KIND_IFACE)

def cmd_show_service_templates(cli):
	return cmd_show_element_templates(cli, kind=ELEMENT_KIND_SERVICE)

def cmd_show_storage_templates(cli):
	return cmd_show_element_templates(cli, kind=ELEMENT_KIND_STORAGE)

def cmd_show_device_templates(cli):
	return cmd_show_element_templates(cli, kind=ELEMENT_KIND_DEVICE)

def cmd_show_element_templates(cli, kind=None, show_all=False):

	user = cli.env['CURRENT_USER']

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	cols = cli.env['COLS']

	tenant = Tenant.objects.get(name=current_tenant)
	db_set_current_tenant_schema(current_tenant)

	# show template <type> [<name>]

	if len(cli.canonical_words) < 4:

		# FIX: use API
		templates = TElement.get_visible_by_user(tenant, user)

		if kind != None:
			templates = templates.filter(kind=kind)

		if not show_all:
			templates = templates.filter(parent__isnull=True)

		templates = templates.order_by('kind', 'name')

		templates = templates.select_related('tenant','owner', 'parent')

		r = [[
			[_('TEMPLATE KIND'), 20],
			[_('NAME'         ), -4],
			[_('OWNER'        ), 10],
			[_('USED'         ),  5],
			[_('PARENT'       ), 10],
			[_('DESCRIPTION'  ), 30],
			[_('GROUPS'       ), 10],
		]]

		groups_by_te_id = get_groups_by_element_template_id( [e[0]for e in templates.values_list('id')] )

		for d in templates.all():
			r.append([
				ELEMENT_KIND_STR[d.kind],
				d.name,
				d.owner.username if d.owner_id is not None else '',
				d.get_reference_count(),  #FIX: COSTA TROPPO QUESTA CHIAMATA
				d.parent.name if d.parent is not None else '',
				d.description,
				
				','.join(groups_by_te_id.get(d.id,[]))
			])

		output_table(cli, r)

	else:
		name = cli.canonical_words[3]
		
		t = TElement.objects.get(kind=kind, name=name, tenant=tenant)
		
		display_fields = [
			[_('name'          ), t.name        ],
			[_('description'   ), t.description ],
		]

		cli.prnt(text_details(display_fields))		
		
		
		cli.prnt(_("Elements"))
		cli.prnt("--------")
		
		for e in t.elementtemplate_set.select_related('parent').order_by('melement__parent__name', 'melement__name').all():
			cli.prnt(e.melement.path)
			

