# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from django.utils.translation import ugettext as _

from cli.clilib.ioslike import *
from cli.textformat import *

from core.models import Tenant
from core.models import MonitoredNode, MonitoredIface,  MonitoredStorage, MonitoredService, MonitoredDevice, Link, Cluster, Condition, DataSource

from core.api.base import tenant_get
from core.api.base import db_set_current_tenant_schema 
#, tenant_select_all


from cli.prompts.common import output_table

from cli.prompts import cli_get_registered_hooks

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def CHECKED(b):
	if b: return '*'
	return ''


def cmd_show_tenants(cli):
	
	user = cli.env['CURRENT_USER']
	
	
	
	#tenants = tenant_select_all()
	#tenants = user.get_profile().get_visible_tenants().all()
	tenants = Tenant.get_visible_by_user(user)
	
	tenants = tenants.all()

	r = [[
		[_('NAME'     ), -4],
		[_('LONG NAME'), 12],
		[_('TZ'       ), 32],
		[_('PRIMARY'  ),  5],
		[_('UUID'     ), 36],
	]]
	for t in tenants:
		r = r + [[
			t.name,
			t.long_name,
			t.timezone,
			CHECKED(t.is_primary),
			t.uuid,
		]]

	output_table(cli, r)
	
def cmd_show_tenant_details(cli):
	
	env = cli.env
	
	if len(cli.canonical_words) > 2:
		tenant_name = cli.canonical_words[2]
	else:
		tenant_name = env['CURRENT_TENANT']

	current_user = env.get('CURRENT_USER', None)
	
	t = tenant_get(tenant_name)
	db_set_current_tenant_schema(t.name)

	cli.prnt('ID       : %s' % (t.uuid))
	cli.prnt('Name     : %s' % (t.name))
	cli.prnt('Long name: %s' % (t.long_name))
	
	cli.prnt('Configuration defines:')
	cli.prnt('')
	cli.prnt('\t%5s nodes'       % (MonitoredNode.objects.count()))
	cli.prnt('\t%5s interfaces'  % (MonitoredIface.objects.count()))
	cli.prnt('\t%5s storages'    % (MonitoredStorage.objects.count()))
	cli.prnt('\t%5s services'    % (MonitoredService.objects.count()))
	cli.prnt('\t%5s devices'     % (MonitoredDevice.objects.count()))
	cli.prnt('')
	cli.prnt('\t%5s links'       % (Link.objects.count()))
	cli.prnt('')
	cli.prnt('\t%5s clusters'    % (Cluster.objects.count()))
	cli.prnt('')
	cli.prnt('\t%5s conditions'  % (Condition.objects.count()))
	cli.prnt('\t%5s datasources' % (DataSource.objects.count()))
	

	hooks = cli_get_registered_hooks('show:tenant')
	for h in hooks:
		h(cli, t)

	return 

