# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from django.utils.translation import ugettext as _

from django.contrib.auth.models import User, Group

from core.models import Tenant
from core.api.base import group_get_data

from webui.capabilities import CAPABILITIES

from cli.textformat import *

from cli.prompts.common import output_table

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_users(c):
	
	env = c.env
	
	cols = env['COLS']
	
	# show users [<uname>]
	if len(c.canonical_words) < 3:

		users =  User.objects.all().order_by('username')

		t = [[
			[_('ID'        ), -4],
			[_('USERNAME'  ), 12],
			[_('FIRST_LAST'), 32],
			[_('EMAIL'     ), 32],
			[_('ACTIVE'    ),  6],
			[_('SUPER'     ),  6],
			[_('LASTLOGIN' ), 20],
		]]
		for u in users:
			t = t + [[
				u.id,
				u.username,
				u.first_name + ' ' + u.last_name,
				u.email,
				u.is_active,
				u.is_superuser,
				u.last_login,
			]]

		output_table(c, t)
	
	else:
		username = c.canonical_words[2]
		
		try:
			u = User.objects.get(username=username)
		except User.DoesNotExist:
			c.error("No such user: %s" % username)
			return
			
		c.prnt(
			generic_details(
				u,
				suppress=['is_staff', 'is_superuser', 'date_joined'],
			)
		)
		
		if u.groups.count() == 0:
			c.prnt('Not member of any group.')
		else:
			c.prnt('Member of groups: %s' % ', '.join(map(lambda x: x.name, u.groups.all())))

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_users_permissions(cli):

	# show user <name> permissions

	current_user = cli.env['CURRENT_USER']

	env = cli.env

	username = cli.canonical_words[2]

	#if current_user:
	#	if not (current_user.is_superuser or current_user.username == username):
	#		cli.error("NOT ALLOWD")
	#		return

	from tags.models import TagTreeGroupPermission

	try:
		user = User.objects.get(username=username)

		tenants = Tenant.objects.order_by('name').all()

		for tenant in tenants:

			tenant.db_set_current_tenant_schema(tenant.name)
			
			print "%-80s" % (tenant.name)
			print "-" * 80

			#
			# CAPABILITIES
			#

			capabilities = sorted(user.get_capabilities(tenant))

			for c in capabilities:
				print tenant.name, c

			print ""

			#
			# Permission sui tagtree
			#
			t = [[
				[_('TENANT'        ), -4],
				[_('TREE'  ), 12],
				[_('AUTH'), 32],
				[_('GROUP'     ), 32],
			]]

			permissions = TagTreeGroupPermission.objects.filter(group=user.groups.all()).order_by('dirtree__name').all()

			for ttgp in permissions:
				t.append( [ tenant.name, ttgp.dirtree.name, ttgp.str_auth_type(), ttgp.group.name ] )

			output_table(cli, t)

			print ""

	except User.DoesNotExist:
		c.outlnd("No such user: %s" % username)
		return



def cmd_show_groups_permissions(cli):
	print "NO"
	
#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_groups(c):
	
	env = c.env
	
	cols = env['COLS']
	
	# show users [<uname>]
	if len(c.canonical_words) < 3:
		t = [[
			[_('ID'         ), -4],
			[_('NAME'       ), 12],
			[_('DESCRIPTION'), 30],
		]]
		for g in Group.objects.all():
			
			d = group_get_data(g.name)
			
			t = t + [[
				g.id,
				g.name,
				d.description,
			]]

		output_table(c, t)

	else:

		name = c.canonical_words[2]
		
		try:
			g = Group.objects.get(name=name)
		except Group.DoesNotExist:
			c.outln("No such group: %s" % (name,))
			return
		
		d = group_get_data(g.name)
		
		setattr(g, 'description', d.description)

		c.prnt(
			generic_details(
				g,
				suppress=['is_staff', 'is_superuser', 'date_joined'],
			)
		)


		t = [[
			[_('TENANT'             ), 30],
			[_('CAPABILITY'         ), 30],
			[_('DESCR'              ), 30],
		]]
		
		last = None
		for tenant in Tenant.objects.all():
			
			for cap in sorted(g.get_capabilities(tenant)):

				t = t + [[
					tenant.name if tenant.name != last else "",
					cap,
					CAPABILITIES.get(cap, '???')
				]]
				
				last = tenant.name

		c.prnt(text_table(t, cols))


