# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.


from core.models import Group, TDataGroupPermission
from core.models import TElement
from core.models import ElementTemplate
from core.models import TElementPermission

from core.api.base import db_set_current_tenant_schema

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def get_templates_by_element_id(tenant, elements_uuids):

	db_set_current_tenant_schema(tenant.name)

	elementtemplate_by_node_id =  {}

	telement_by_id = dict([ (te.id, te) for te in TElement.objects.all() ])

	for et in ElementTemplate.objects.filter(tenant=tenant, melement__in=elements_uuids).all():

		s = elementtemplate_by_node_id.get(et.melement_id  , elementtemplate_by_node_id.setdefault(et.melement_id, '' ) )
		if s:
			s += ',' + telement_by_id[et.telement_id].name
		else:
			s += telement_by_id[et.telement_id].name

		elementtemplate_by_node_id[et.melement_id] = s

	return elementtemplate_by_node_id



def get_groups_by_datagroup_template_id(datagroup_uuids):

	group_name_by_uuid = dict( [ (g.id, g.name) for g in Group.objects.all() ] )

	r =  {}

	for tdatagroup_id, group_id in TDataGroupPermission.objects.values_list('tdatagroup_id', 'group_id'):

		l = r.get( tdatagroup_id, r.setdefault( tdatagroup_id, [] ) )

		l.append(group_name_by_uuid[group_id])

	return r

def get_groups_by_element_template_id(telement_ids):
	
	group_name_by_uuid = dict( [ (g.id, g.name) for g in Group.objects.all() ] )

	r =  {}

	for te_id, group_id in TElementPermission.objects.values_list('telement_id', 'group_id'):

		l = r.get( te_id, r.setdefault( te_id, [] ) )

		l.append(group_name_by_uuid[group_id])

	return r
