# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import sys
import traceback

from django.db import connection
from django.contrib.auth.models import User, Group

from sanet_common.sanetdefs.constants import *

from core.models import Cluster
from core.models import PriorityLabel
from core.models import MonitoredNode 
from core.models import MonitoredIface 
from core.models import MonitoredDevice 
from core.models import MonitoredService
from core.models import MonitoredStorage
from core.models import CDataGroup
from core.models import DataGroup
from core.models import DataSource
from core.models import Condition

from core.models import TElement, TEDataGroup

from core.models import SOURCE_LIBRARY

from core.api.base import db_set_current_tenant_schema
from core.api.base import agent_select_all
from core.api.base import cluster_select_all
from core.api.base import node_get
from core.api.base import icon_select_all

from core.api.templates import datagroup_template_select_all

#from cli.clilib.ioslike import *
from cli.clilib.ioslike import T_Keyword, T_Anyword, Token, T_Pipe, HelpingString
#from cli.clilib.ioslike import SORTED_KEYWORDS
#from cli.clilib.ioslike import BLANK_ENTRY

from tags.models import *

from cli.prompts import cli_get_registered_menus

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_KeywordWithHook(T_Keyword):

	def __init__(self, *args, **kwargs):

		self.hook_name = None

		super(T_KeywordWithHook, self).__init__(*args, **kwargs)

	def get_subtree(self):

		r = [ t for t in self.subtree ]

		if self.hook_name:
			t2 = cli_get_registered_menus(self.hook_name)
			if t2 != None:
				r += [ t for t in t2 ]

		if isinstance(self.pipe, T_Pipe) and not self.pipe in r:
			r.append(self.pipe)

		return r

class T_AnywordWithHook(T_Anyword):

	def __init__(self, *args, **kwargs):

		self.hook_name = None

		super(T_AnywordWithHook, self).__init__(*args, **kwargs)

	def get_subtree(self):

		r = [ t for t in self.subtree ]

		if self.hook_name:
			t2 = cli_get_registered_menus(self.hook_name)
			if t2 != None:
				r += [ t for t in t2 ]

		if isinstance(self.pipe, T_Pipe) and not self.pipe in r:
			r.append(self.pipe)

		return r

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_Resourcecompleter(Token):

	"""
	Completes something that has a name and a descr
	"""
	expand_help  = True
	klass        = None
	key          = 'name'

	def __init__(self, *args, **kwargs):
		if hasattr(self.klass, 'all_objects'):
			self.manager = self.klass.all_objects
		else:
			self.manager = self.klass.objects

		super(T_Resourcecompleter, self).__init__(*args, **kwargs)

		self.token_repr = '<' + self.klass.__name__.replace("monitored", "").upper() + '>'
		
	def set_tenant(self, cli):
		current_tenant_name = cli.env['CURRENT_TENANT']
		#tenant = tenant_get(current_tenant_name)
		db_set_current_tenant_schema(current_tenant_name)

	def get_base_queryset(self, cli, text):
		self.set_tenant(cli)
		return self.manager.filter(**{self.key + '__startswith': text})

	def matches(self, cli, word):
		return self.get_base_queryset(cli, word).count() > 0

	def completion(self, cli, text):

		rv = []
		#c = connection.cursor()
		#q = 'select ' + self.key + ',descr from ' + self.klass._meta.db_table + ' where  ' + self.key + ' like %s ORDER BY ' + self.key
		#c.execute(q, [text + '%'])
		#data = c.fetchall()
		
		data = self.get_base_queryset(cli, text).order_by(self.key).values_list(self.key)

		for n in data:
			rv.append(HelpingString(n[0], n[0]))

		return rv

	def canonical(self, cli, word):
		results = self.get_base_queryset(cli, word).order_by(self.key).values_list(self.key)
		
		if len(results) > 0:
			return results[0][0]
		return None
		
		#print results
		#print self.manager
		
		if len(results) == 1:
			return results[0]
		elif len(results) > 1:
				for r in results:
					if r[0] == word:
						return r
		
		return word
		
		#return super(T_Resourcecompleter, self).canonical(cli, word)


		#try:
		#	print "canonical", 1
		#	r = self.manager.get(**{self.key + '__startswith': word})
		#	print r
		#	return getattr(r, self.key)
                #
		#except self.klass.DoesNotExist, e:
		#	print "canonical", 2
		#	return (None)
                #
		#except self.klass.MultipleObjectsReturned, e:
		#	print "canonical", 3
		#	try:
		#		
		#		results = self.manager.filter(**{self.key: word}).order_by(self.key).values_list(self.key)
		#		print results
		#		for r in results:
		#			if r[0] == word:
		#				return r
		#			
		#		print r
		#		return getattr(r,self.key)
		#	except:
		#		print "canonical", 4
		#		return(super(T_Resourcecompleter, self).canonical(cli, word))

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_Usercompleter(T_Resourcecompleter):

	klass  = User
	key    = 'username'

	def completion(self, cli, text):

		# FIX: do not access django from here

		rv = []
		c = connection.cursor()
		q = 'select ' + self.key + ',first_name,last_name from ' + self.klass._meta.db_table + ' where  ' + self.key + ' like %s ORDER BY ' + self.key
		c.execute(q, [text + '%'])
		data = c.fetchall()

		for n in data:
			rv.append(HelpingString(n[0], ' '.join(n[1:])))
		return rv

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_Groupcompleter(T_Resourcecompleter):

	klass = Group
	key   = 'name'

	def completion(self, cli, text):
		# FIX: do not access django from here

		rv = []
		c = connection.cursor()
		q = 'select ' + self.key + ' from ' + self.klass._meta.db_table + ' where  ' + self.key + ' like %s ORDER BY ' + self.key
		c.execute(q, [text + '%'])
		data = c.fetchall()

		for n in data:
			rv.append(HelpingString(n[0], ''))

		return rv

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_Tenantcompleter(T_Anyword):

	def __init__(self, *args, **kwargs):
		super(T_Tenantcompleter, self).__init__(*args, **kwargs)
		self.token_repr = '<TENANT>'

	def get_base_queryset(self, cli):
		user = cli.env['CURRENT_USER']
		return Tenant.get_visible_by_user(user)

	def matches(self, cli, word):

		tenants = self.get_base_queryset(cli)

		return tenants.filter(name__startswith=word).count()

	def completion(self, cli, text):
		try:
			rv = []

			tenants = self.get_base_queryset(cli)

			tenants = tenants.filter(name__startswith=text)
			tenants = tenants.order_by('name').all()

			for t in tenants:
				rv.append(HelpingString(t.name, t.long_name))

			return rv

		except Exception, e:
			raise e

	def canonical(self, cli, word):

		tenants = self.get_base_queryset(cli).filter(name__startswith=word).order_by('name').all()
		
		if len(tenants) >= 1:
			return tenants[0].name

		return None

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_VisibleTenantByGroupCompleter(T_Anyword):

	def __init__(self, *args, **kwargs):

		super(T_VisibleTenantByGroupCompleter, self).__init__(*args, **kwargs)

		self.token_repr = '<TENANT>'

	def matches(self, cli, word):
		group_name = cli.env['CURRENT_GROUP']
		
		return TenantGroup.objects.filter(group__name=group_name, tenant__name=word).count() > 0

	def completion(self, cli, text):

		group_name = cli.env['CURRENT_GROUP']

		try:
			rv = []

			for tg in TenantGroup.objects.filter(group__name=group_name, tenant__name__startswith=text):
				rv.append(HelpingString(tg.tenant.name, tg.tenant.long_name))

		except Exception, e:
			raise e

		return rv

	def canonical(self, cli, word):

		group_name = cli.env['CURRENT_GROUP']

		tenants = TenantGroup.objects.filter(group__name=group_name, tenant__name__startswith=word)
		if len(tenants) >= 1:
			return tenants[0].tenant.name

		return None


class T_VisibleTenantByUserCompleter(T_Anyword):

	def __init__(self, *args, **kwargs):
		super(T_VisibleTenantByUserCompleter, self).__init__(*args, **kwargs)
		self.token_repr = '<TENANT>'

	def matches(self, cli, word):
		user_name = cli.env['CURRENT_USER']

		groups = User.objects.get(username=user_name).groups.all()

		return TenantGroup.objects.filter(group__in=groups, tenant__name=word).count() > 0

	def completion(self, cli, text):

		user_name = cli.env['CURRENT_USER']

		groups = User.objects.get(username=user_name).groups.all()

		try:
			rv = []
			
			tenants = dict([(tg.tenant.name, tg.tenant) for tg in TenantGroup.objects.filter(group__in=groups, tenant__name__startswith=text).all()])
			for k,t in sorted(tenants.items()):
				rv.append(HelpingString(t.name, t.long_name))

		except Exception, e:
			raise e
		return rv

	def canonical(self, cli, word):

		user_name = cli.env['CURRENT_USER']

		groups = User.objects.get(username=user_name).groups.all()

		tenants = dict([(tg.tenant.name, tg.tenant) for tg in TenantGroup.objects.filter(group__in=groups, tenant__name__startswith=word).all()])

		if len(tenants) >= 1:
			first_name = sorted(tenants.keys())[0]
			return tenants[first_name].name

		return None

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_PriorityLevelCompleter(T_Anyword):

	def __init__(self, *args, **kwargs):
		super(T_PriorityLevelCompleter, self).__init__(*args, **kwargs)		
		self.token_repr = '<level>'

	def matches(self, cli, word, ):
		current_tenant_name = cli.env['CURRENT_TENANT']

		db_set_current_tenant_schema(current_tenant_name)

		return PriorityLabel.objects.filter(**{'label__startswith': word }).count() > 0

	def completion(self, cli, text):

		try:
			current_tenant_name = cli.env['CURRENT_TENANT']

			db_set_current_tenant_schema(current_tenant_name)

			res = PriorityLabel.objects.filter( **{'label__startswith' :text } ).all()
			rv = []
			for n in res:
				rv.append(HelpingString( n.label, n.description or '* no description *'))

			return rv

		except Exception, e:
			raise e

	def canonical(self, cli, word):

		current_tenant_name = cli.env['CURRENT_TENANT']

		try:
			db_set_current_tenant_schema(current_tenant_name)

			res = PriorityLabel.objects.get( **{'label': word } )
			return res.label

		except PriorityLabel.DoesNotExist, e:
			return None

		except PriorityLabel.MultipleObjectsReturned, e:
			try:
				db_set_current_tenant_schema(current_tenant_name)

				return PriorityLabel.objects.get(**{'name': word}).label
			except:
				return "**ERROR**"
		except Exception, e:
			raise e

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_Clustercompleter(T_Anyword):

	def __init__(self, *args, **kwargs):
		super(T_Clustercompleter, self).__init__(*args, **kwargs)
		self.token_repr = '<CLUSTER>'

	def get_base_queryset(self, cli):

		user        = cli.env['CURRENT_USER']
		tenant_name = cli.env['CURRENT_TENANT']

		tenant = Tenant.objects.get(name=tenant_name)
		tenant.db_set_current_schema()

		clusters = Cluster.get_visible_by_user(tenant, user)

		return clusters


	def matches(self, cli, word):
		
		#current_tenant_name = cli.env['CURRENT_TENANT']
		
		#return len(cluster_select_all(current_tenant_name, name_filter_str=word)) > 0
		return self.get_base_queryset(cli).filter(name__startswith=word).order_by('name').all()

	def completion(self, cli, text):
		
		current_tenant_name = cli.env['CURRENT_TENANT']

		try:
			rv = []
			#if not text:
			#	text = None

			#clusters = cluster_select_all(current_tenant_name, name_filter_str=text)
			clusters = self.get_base_queryset(cli).filter(name__startswith=text).order_by('name').all()
			for t in clusters:
				rv.append(HelpingString(t.name, t.name))

		except Exception, e:
			raise e

		return rv

	def canonical(self, cli, word):

		current_tenant_name = cli.env['CURRENT_TENANT']

		#tenants = cluster_select_all(current_tenant_name, name_filter_str=word)
		clusters = self.get_base_queryset(cli).filter(name__startswith=word).order_by('name').all()

		if len(clusters) >= 1:
			return clusters[0].name

		return None

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_Agentcompleter(T_Anyword):

	def __init__(self, *args, **kwargs):
		super(T_Agentcompleter, self).__init__(*args, **kwargs)
		self.token_repr = '<AGENT>'

	def matches(self, cli, word):

		current_tenant_name = cli.env['CURRENT_TENANT']

		return len(agent_select_all(current_tenant_name, name_filter_str=word)) > 0

	def completion(self, cli, text):

		current_tenant_name = cli.env['CURRENT_TENANT']

		try:
			rv = []
			if not text:
				text = None

			tenants = agent_select_all(current_tenant_name, name_filter_str=text).order_by('name')
			for t in tenants:
				rv.append(HelpingString(t.name, t.name))
		except Exception, e:
			raise e
		return rv

	def canonical(self, cli, word):

		current_tenant_name = cli.env['CURRENT_TENANT']

		tenants = agent_select_all(current_tenant_name, name_filter_str=word).order_by('name')

		if len(tenants) >= 1:
			return tenants[0].name

		return None

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_MonitoredElementcompleter(T_AnywordWithHook):

	expand_help  = True
	klass        = None   
	key          = 'name'
	
	token_regexp = VALID_ELEMENT_NAME

	def __init__(self, *args, **kwargs):

		self.manager = self.klass

		super(T_MonitoredElementcompleter, self).__init__(*args, **kwargs)

		self.token_repr = '<' + self.klass.__name__.lower().replace("monitored", "").upper() + '>'

	def get_base_queryset(self, cli):

		user                = cli.env['CURRENT_USER']
		current_tenant_name = cli.env['CURRENT_TENANT']

		tenant = Tenant.objects.get(name=current_tenant_name)

		db_set_current_tenant_schema(current_tenant_name)

		#return self.manager.objects
		return self.manager.get_visible_by_user(tenant, user)

	def matches(self, cli, word, ):

		objects = self.get_base_queryset(cli)

		return objects.filter(**{'name__startswith': word}).count() > 0

	def completion(self, cli, text):

		try:
			current_tenant_name = cli.env['CURRENT_TENANT']
			db_set_current_tenant_schema(current_tenant_name)

			objects = self.get_base_queryset(cli)

			res = objects.filter( **{'name__startswith' :text} ).order_by('name').values_list('name').all()
			rv = []
			for n in res:
				rv.append(HelpingString( n[0], n[0]))
			return rv
		except Exception, e:
			raise e

	def canonical(self, cli, word):

		objects = self.get_base_queryset(cli)

		res = objects.filter( **{'name__startswith': word} ).values_list('name').order_by('name')
		if len(res) >= 1:
			return res[0][0]
			# FIX: review completition semantic
		return None


class T_MonitoredNodecompleter(T_MonitoredElementcompleter):

	token_regexp = VALID_NODE_NAME

	klass  = MonitoredNode

class T_MonitoredIfacecompleter(T_MonitoredElementcompleter):

	token_regexp = VALID_IFACE_NAME

	klass  = MonitoredIface

class T_MonitoredStoragecompleter(T_MonitoredElementcompleter):

	token_regexp = VALID_STORAGE_NAME

	klass  = MonitoredStorage

class T_MonitoredServicecompleter(T_MonitoredElementcompleter):

	token_regexp = VALID_SERVICE_NAME

	klass  = MonitoredService

class T_MonitoredDevicecompleter(T_MonitoredElementcompleter):

	token_regexp = VALID_DEVICE_NAME

	klass  = MonitoredDevice

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

#
# FIX: questo filtro non tiene conto se l'utente puo' effettivamente vedere il nodo ed i suoi sotto-elementi.
#      Questo compleater funziona perche' comunque viene sempre usato insieme ad un completer che effettua il controllo di visibilita'.
#
class T_InlineMonitoredElementCompleter(T_Anyword):

	klass = None

	def __init__(self, *args, **kwargs):

		self.node_name_token_pos = -1

		super(T_InlineMonitoredElementCompleter, self).__init__(*args, **kwargs)

		self.token_repr = '<' + self.klass.__name__.lower().replace("monitored","").upper() + '>'
		
		self.node_name_token_pos = kwargs.get('node_name_token_pos', -1)

	def get_params(self, cli):

		filter_params = {}

		current_tenant_name = cli.env['CURRENT_TENANT']

		#node_name = cli.canonical_words[-1]
		node_name = cli.canonical_words[self.node_name_token_pos]

		node = node_get(node_name, tenant_name=current_tenant_name)

		if node != None:
			filter_params['parent'] = node

		return filter_params

	def matches(self, cli, word, ):
		filter_params = self.get_params(cli)
		filter_params.update({'name__startswith': word })
		return self.klass.objects.filter(**filter_params).count() > 0

	def completion(self, cli, text):
		try:
			filter_params = self.get_params(cli)
			filter_params.update({'name__startswith' :text})

			res = self.klass.objects.filter( **filter_params ).all()
			rv = []
			for n in res:
				rv.append(HelpingString( n.name, n.name))

			return rv

		except Exception, e:
			raise e

	def canonical(self, cli, word):
		try:
			filter_params = self.get_params(cli)
			filter_params.update({ 'name': word })

			res = self.klass.objects.get( **filter_params  )
			return res.name

		except self.klass.DoesNotExist, e:
			return None
		except self.klass.MultipleObjectsReturned, e:
			try:
				filter_params = { 'name': word }

				node_name = cli.canonical_words[-1]
				node = node_get(node_name, tenant_name=current_tenant_name)				
				if node != None:
					filter_params['parent'] = node

				return self.klass.objects.get(**filter_params).name
			except:
				return "**ERROR**"
		except Exception, e:
			raise e	
	
class T_InlineIfaceCompleter(T_InlineMonitoredElementCompleter):
	klass = MonitoredIface

class T_InlineStorageCompleter(T_InlineMonitoredElementCompleter):
	klass = MonitoredStorage

class T_InlineServiceCompleter(T_InlineMonitoredElementCompleter):
	klass = MonitoredService

class T_InlineDeviceCompleter(T_InlineMonitoredElementCompleter):
	klass = MonitoredDevice

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_DataGroupTemplate(T_Anyword):

	def __init__(self, *args, **kwargs):
		super(T_DataGroupTemplate, self).__init__(*args, **kwargs)
		self.token_repr = '<datagroup template>'

	def select_templates(self,cli, word):
		source = cli.env['SOURCE']
		tenant = None
		if source == SOURCE_LOCAL:
			current_tenant_name = cli.env['CURRENT_TENANT']
			tenant = tenant_get(current_tenant_name)

		return datagroup_template_select_all(name_filter_str=word, source=source, tenant=tenant)

	def matches(self, cli, word):
		r = self.select_templates(cli, word)
		return len(r) > 0

	def completion(self, cli, text):
		try:
			rv = []
			if not text:
				text = None

			templates = self.select_templates(cli, text)

			d = {}
			for t in templates:
				d[t.name] = t

			for name, t in sorted(d.items()):
				rv.append(HelpingString(name, "(%s) %s" % (str(SOURCE_DESCR[t.source]['name']), t.description)))
		except Exception, e:
			raise e
		return rv

	def canonical(self, cli, word):

		templates = self.select_templates(cli, word)

		# FIX: Remove duplicates in a better way
		d = {}
		for t in templates:
			d[t.name] = t
		# Remove duplicates
		if len(d) >= 1:
			return sorted(d.keys())[0]
		return None

class T_AllDataGroupTemplate(T_DataGroupTemplate):

	def __init__(self, *args, **kwargs):
		super(T_AllDataGroupTemplate, self).__init__(*args, **kwargs)

	def select_templates(self,cli, word):
		return datagroup_template_select_all(name_filter_str=word)

	def completion(self, cli, text):
		try:
			rv = []
			if not text:
				text = None

			templates = self.select_templates(cli, text)

			templates = sorted([(t.source,t.name,t) for t in templates])

			for source, name, t in templates:
				rv.append(HelpingString(name, "(%s) %s" % (str(SOURCE_DESCR[source]['name']), t.description)))
		except Exception, e:
			raise e
		return rv

	def canonical(self, cli, word):

		templates = self.select_templates(cli, word)

		# FIX: Remove duplicates in a better way
		d = {}
		for t in templates:
			d[t.name] = t
		# Remove duplicates
		if len(d) >= 1:
			return sorted(d.keys())[0]
		return None

class T_LibraryDataGroupTemplate(T_Anyword):

	def __init__(self, *args, **kwargs):
		super(T_LibraryDataGroupTemplate, self).__init__(*args, **kwargs)

		self.token_repr = '<datagroup template>'

	def matches(self, cli, word):
		source = SOURCE_LIBRARY
		return len(datagroup_template_select_all(name_filter_str=word, source=source)) > 0

	def completion(self, cli, text):
		source = SOURCE_LIBRARY
		try:
			rv = []
			if not text:
				text = None
			templates = datagroup_template_select_all(name_filter_str=text, source=source)
			# FIX: Remove duplicates in a better way
			d = {}
			for t in templates:
				d[t.name] = t
			for name, t in sorted(d.items()):
				rv.append(HelpingString(name, name+ ' '+str(SOURCE_DESCR[t.source]['name'])))
		except Exception, e:
			raise e
		return rv

	def canonical(self, cli, word):

		source = SOURCE_LIBRARY
		templates = datagroup_template_select_all(name_filter_str=word, source=source)
		# FIX: Remove duplicates in a better way
		d = {}
		for t in templates:
			d[t.name] = t
		# Remove duplicates
		if len(d) >= 1:
			return sorted(d.keys())[0]
		return None

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_AbastractElementTemplateCompleter(T_Anyword):

	kind = None

	def __init__(self, *args, **kwargs):
		super(T_AbastractElementTemplateCompleter, self).__init__(*args, **kwargs)
		self.token_repr = '<element template>'

	def get_base_queryset(self, cli):

		user        = cli.env['CURRENT_USER']
		tenant_name = cli.env['CURRENT_TENANT']

		tenant = Tenant.objects.get(name=tenant_name)

		db_set_current_tenant_schema(tenant.name)

		templates = TElement.get_visible_by_user(tenant, user)
		templates = templates.filter(kind=self.kind)
		templates = templates.filter(parent__isnull=True)
		templates = templates.order_by('name')

		return templates

	def matches(self, cli, word):
		return self.get_base_queryset(cli).filter(name__startswith=word).count() > 0

	def completion(self, cli, text):
		try:
			rv = []
			templates = self.get_base_queryset(cli).filter(name__startswith=text).all()
			# FIX: Remove duplicates in a better way
			d = {}
			for t in templates:
				d[t.name] = t
			for name, t in sorted(d.items()):
				rv.append(HelpingString(name, t.description))
		except Exception, e:
			raise e
		return rv

	def canonical(self, cli, word):
		templates = self.get_base_queryset(cli).filter(name__startswith=word).all()
		# FIX: Remove duplicates in a better way
		d = {}
		for t in templates:
			d[t.name] = t
		# Remove duplicates
		if len(d) >= 1:
			sorted_tuples = sorted(d.items())
			name, template =  sorted_tuples[0]
			return name
		return None

class T_NodeTemplate(T_AbastractElementTemplateCompleter):
	kind = ELEMENT_KIND_NODE

class T_IfaceTemplate(T_AbastractElementTemplateCompleter):
	kind = ELEMENT_KIND_IFACE

class T_StorageTemplate(T_AbastractElementTemplateCompleter):
	kind = ELEMENT_KIND_STORAGE

class T_ServiceTemplate(T_AbastractElementTemplateCompleter):
	kind = ELEMENT_KIND_SERVICE

class T_DeviceTemplate(T_AbastractElementTemplateCompleter):
	kind = ELEMENT_KIND_DEVICE

class T_ParametricElementTemplate(T_AbastractElementTemplateCompleter):

	def __init__(self, kind, *args, **kwargs):
		super(T_ParametricElementTemplate, self).__init__(*args, **kwargs)
		self.kind = kind

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_TemplateSubElement(T_Anyword):

	def __init__(self, *args, **kwargs):
		super(T_TemplateSubElement, self).__init__(*args, **kwargs)
		self.token_repr = '<applied template>'

	def get_base_queryset(self, cli, text):

		CONFIG_TENANT                = cli.env['CONFIG_TENANT'               ]
		CONFIG_ELEMENT_TEMPLATE      = cli.env['CONFIG_ELEMENT_TEMPLATE'     ]
		CONFIG_ELEMENT_TEMPLATE_KIND = cli.env['CONFIG_ELEMENT_TEMPLATE_KIND']

		s = '%s:%s' % (CONFIG_ELEMENT_TEMPLATE, text)
		
		p = TElement.objects.filter(tenant__name=CONFIG_TENANT, name=CONFIG_ELEMENT_TEMPLATE, kind=CONFIG_ELEMENT_TEMPLATE_KIND)

		q = TElement.objects.filter(parent=p)
		q = q.filter(name__startswith=s)
		q = q.order_by('name')

		r = [ (t.name.split(':')[1], t) for t in q.all()  ]
		return r

	def matches(self, cli, word):
		return len(self.get_base_queryset(cli, word)) > 0

	def completion(self, cli, text):
		try:
			rv = []
			templates = self.get_base_queryset(cli, text)
			# FIX: Remove duplicates in a better way
			d = {}
			for name, t in templates:
				d[name] = t
			for name, t in sorted(d.items()):
				rv.append(HelpingString(name, t.description))
		except Exception, e:
			print >> sys.stderr, type(e), str(e), traceback.format_exc()
			raise e
		return rv

	def canonical(self, cli, word):
		templates = self.get_base_queryset(cli, word)
		# FIX: Remove duplicates in a better way
		d = {}
		for name, t in templates:
			d[name] = t
		# Remove duplicates
		if len(d) >= 1:
			sorted_tuples = sorted(d.items())
			name, template =  sorted_tuples[0]
			return name

		return None

class T_TemplateSubElementOrAny(T_TemplateSubElement):

	def __init__(self, kind, *args, **kwargs):

		super(T_TemplateSubElementOrAny, self).__init__(*args, **kwargs)

		self.token_repr = '<sub-element name>'

		self.kind = kind

	def matches(self, cli, word):
		return VALID_ELEMENT_NAME_BY_KIND[self.kind].match(word)

	def canonical(self, cli, word):
		return super(T_TemplateSubElementOrAny, self).canonical(cli, word) or word

class T_TElementAppliedTemplate(T_Anyword):

	def __init__(self, *args, **kwargs):
		super(T_TElementAppliedTemplate, self).__init__(*args, **kwargs)
		self.token_repr = '<applied template>'

	def get_base_queryset(self, cli, text):
		tenant_name = cli.env['CURRENT_TENANT']

		element_template_name = cli.env['CONFIG_ELEMENT_TEMPLATE'    ]
		element_template_kind = cli.env['CONFIG_ELEMENT_TEMPLATE_KIND']

		q = TElementAppliedTemplate.objects.filter(tenant__name=tenant_name, telement__name=element_template_name, telement__kind=element_template_kind)
		q = q.filter(telementapplied__name__startswith=text)

		return q.order_by('telementapplied__name')

	def matches(self, cli, word):
		return self.get_base_queryset(cli, word).count() > 0

	def completion(self, cli, text):
		try:
			rv = []
			templates = self.get_base_queryset(cli, text).all()
			# FIX: Remove duplicates in a better way
			d = {}
			for t in templates:
				d[t.telementapplied.name] = t.telementapplied
			for name, t in sorted(d.items()):
				rv.append(HelpingString(name, t.description))
		except Exception, e:
			raise e
		return rv

	def canonical(self, cli, word):
		templates = self.get_base_queryset(cli, word).all()
		# FIX: Remove duplicates in a better way
		d = {}
		for t in templates:
			d[t.telementapplied.name] = t.telementapplied
		# Remove duplicates
		if len(d) >= 1:
			sorted_tuples = sorted(d.items())
			name, template =  sorted_tuples[0]
			return name
		return None

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_SubElementCompleter(T_Anyword):
	"""
	Completer base per estrarre un monitoredelement associato ad un nodo.
	
	Questo compleater esclude i sub-monitoredelement generati da un element-template (ovvero che hanno il campo telement_generator != null )
	"""

	expand_help  = True
	klass        = None
	key          = 'name'

	def __init__(self, *args, **kwargs):
		super(T_SubElementCompleter, self).__init__(*args, **kwargs)		

		self.manager = self.klass
		self.token_repr = '<' + self.klass.__name__.upper() + '>'

	def get_query_set(self, cli):
		current_tenant_name = cli.env['CURRENT_TENANT']
		db_set_current_tenant_schema(current_tenant_name)

		element             = cli.env['CURRENT_ELEMENT_DATA']

		# Tutti i subelement 
		queryset = self.manager.objects.filter(parent=element)

		# Escludiamo quelli generati in automatico da un element-template
		queryset = queryset.exclude(telement_generator__isnull=False)

		return queryset

	def matches(self, cli, word, ):
		return self.get_query_set(cli).filter(name__startswith=word).count() > 0

	def completion(self, cli, text):
		try:
			res = self.get_query_set(cli).filter(name__startswith=text).all()
			rv = []
			for n in res:
				rv.append(HelpingString( n.name, n.description or '* no description *'))
			return rv
		except Exception, e:
			raise e
			
	def canonical(self, cli, word):
		try:
			res = self.get_query_set(cli).get(name=word)
			return res.name
			
		except self.manager.DoesNotExist, e:
			return None
		except self.manager.MultipleObjectsReturned, e:
			try:
				return self.get_query_set(cli).get(name=word).name
			except:
				return "**ERROR**"
		except Exception, e:
			raise e


class T_SubInterfaceCompleter(T_SubElementCompleter):
	klass = MonitoredIface

class T_SubStorageCompleter(T_SubElementCompleter):
	klass = MonitoredStorage

class T_SubServiceCompleter(T_SubElementCompleter):
	klass = MonitoredService

class T_SubDeviceCompleter(T_SubElementCompleter):
	klass = MonitoredDevice


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_SubElementFromTemplateCompleter(T_SubElementCompleter):
	"""
	Complater per estrarre i sotto-elementi generati da un element-template.
	"""
	
	def get_query_set(self, cli):
		current_tenant_name = cli.env['CURRENT_TENANT']
		db_set_current_tenant_schema(current_tenant_name)
		
		element             = cli.env['CURRENT_ELEMENT_DATA']
		
		queryset = self.manager.objects.filter(parent=element)
		
		# Solo gli sub-element generati da un element-template
		queryset = queryset.filter(telement_generator__isnull=False)
		
		return queryset

class T_SubInterfaceFromTemplateCompleter(T_SubElementFromTemplateCompleter):
	klass = MonitoredIface

class T_SubStorageFromTemplateCompleter(T_SubElementFromTemplateCompleter):
	klass = MonitoredStorage

class T_SubServiceFromTemplateCompleter(T_SubElementFromTemplateCompleter):
	klass = MonitoredService

class T_SubDeviceFromTemplateCompleter(T_SubElementFromTemplateCompleter):
	klass = MonitoredDevice

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_ElementTemplateDataGroup(T_Anyword):

	def __init__(self, *args, **kwargs):
		super(T_ElementTemplateDataGroup, self).__init__(*args, **kwargs)

		self.token_repr = '<datagroup template>'
		
	def get_context(self, cli):
		current_tenant_name = cli.env['CURRENT_TENANT']
		db_set_current_tenant_schema(current_tenant_name)
		element = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']
		return element

	def matches(self, cli, word):
		element = self.get_context(cli)

		return CDataGroup.objects.filter(element=element, tdatagroup=None, name__startswith=word).count() > 0

	def completion(self, cli, text):
		try:
			element = self.get_context(cli)
			
			rv = []
			
			params  = {}
			if text:
				params['name__startswith'] = text
								
			for cdg in CDataGroup.objects.filter(element=element, tdatagroup=None).filter(**params):
				rv.append(
					HelpingString(cdg.name, cdg.name +  ' ' + '**nodescription**')
				)

		except Exception, e:
			raise e
		return rv

	def canonical(self, cli, word):
		try:
			element = self.get_context(cli)
			
			# FIX: DOBBIAMO SELEZIONARE IL CDATAGROUP IN BASE ALL'ELEMENT-TEMPLATE !!
			res = CDataGroup.objects.get( element=element, tdatagroup=None, name=word )
			return res.name
			
		except CDataGroup.DoesNotExist, e:
			return None
		except CDataGroup.MultipleObjectsReturned, e:
			# FIX: DOBBIAMO SELEZIONARE IL CDATAGROUP IN BASE ALL'ELEMENT-TEMPLATE !!
			try:
				#return self.manager.get(**{'name': word}).name
				return CDataGroup.objects.filter( element=element, tdatagroup=None).filter(name=word )[0].name
			except:
				return "**ERROR**"
		except Exception, e:
			raise e

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_PathCompleter(T_Anyword):
	
	klass = None

	def __init__(self, *args, **kwargs):

		self.all_path = None

		super(T_PathCompleter, self).__init__(*args, **kwargs)

		self.token_repr = '<PATH>'
		self.all_path = kwargs.get('all_path', True)
		
	def get_element(self, cli):
		current_tenant_name = cli.env['CURRENT_TENANT']
		db_set_current_tenant_schema(current_tenant_name)

		if self.all_path:
			return None
			
		return cli.env.get('CURRENT_ELEMENT_DATA', None)
		
	def get_filters(self, cli, word):
		filters = {}
		element = self.get_element(cli)
		if element:
			filters['element'] = element
		filters['path__startswith'] = word
		return filters

	def get_base_queryset(cli):
		raise ExceptioN("NOT IMPLEMENTED")

	def get_objects(self, cli, word):
		filters = self.get_filters(cli, word)

		elements = self.get_base_queryset(cli)

		return elements.filter(**filters).values_list('path')

	def matches(self, cli, word):
		return self.get_objects(cli, word).count() > 0

	def completion(self, cli, text):
		try:
			rv = []
			for dg in self.get_objects(cli, text).order_by('path'):
				rv.append(
					HelpingString(dg[0], dg[0])
				)
		except Exception, e:
			raise e
		return rv

	def canonical(self, cli, word):
		try:
			res = self.get_objects(cli, word).order_by('path')
			if len(res) > 0:
				#return res[0].path
				return res[0][0]
			return None
		except self.klass.DoesNotExist, e:
			return None
		except Exception, e:
			raise e

class T_DataGroupCompleter(T_PathCompleter):
	klass = DataGroup

	def get_base_queryset(self, cli):
		user = cli.env['CURRENT_USER']

		tenant_name = cli.env['CURRENT_TENANT']
		tenant = Tenant.objects.get(name=tenant_name)

		return tenant.get_all_datagroups_by_user(user)


class T_DataSourceCompleter(T_PathCompleter):
	klass = DataSource

	def get_base_queryset(self, cli):
		user = cli.env['CURRENT_USER']

		tenant_name = cli.env['CURRENT_TENANT']
		tenant      = Tenant.objects.get(name=tenant_name)

		return tenant.get_all_datasources_by_user(user)

class T_ConditionCompleter(T_PathCompleter):
	klass = Condition

	def get_base_queryset(self, cli):
		user = cli.env['CURRENT_USER']

		tenant_name = cli.env['CURRENT_TENANT']
		tenant      = Tenant.objects.get(name=tenant_name)

		return tenant.get_all_conditions_by_user(user)

class T_TimeGraphCompleter(T_PathCompleter):
	klass = TimeGraph

	def get_base_queryset(self, cli):
		user = cli.env['CURRENT_USER']

		tenant_name = cli.env['CURRENT_TENANT']
		tenant      = Tenant.objects.get(name=tenant_name)

		tgs = tenant.get_all_timegraphs_by_user(user)

		return tgs

	def get_filters(self, cli, word):

		filters = {}
		element = self.get_element(cli)
		if element:
			filters['element'] = element

		# <--------------- datagroup path -------------------------->
		# <element>:<element>;                  ;<datagroup-template> ;<name>
		# <element>:<element>;<element-template>;<datagroup-template> ;<name>
		
		if word.count(';') == 3:
			parts = word.rsplit(';',1)
			datagroup_path, tg_name = parts
			filters['datagroup__path__startswith'] = datagroup_path
			filters['name'           ] = tg_name
		else:
			datagroup_path = word
			filters['datagroup__path__startswith'] = datagroup_path

		return filters

	def get_objects(self, cli, word):

		elements = self.get_base_queryset(cli)

		filters = self.get_filters(cli, word)

		return elements.filter(**filters).values_list('datagroup__path', 'name')

	def matches(self, cli, word):

		elements = self.get_objects(cli, word)

		return elements.count() > 0

	def completion(self, cli, text):

		try:
			elements = self.get_objects(cli, text).order_by('datagroup__path', 'name')
			
			rv = []
			for dg_path, tg_name in elements.all():
				tg_path = dg_path + ';' + tg_name
				rv.append(
					HelpingString(tg_path, tg_path)
				)
		except Exception, e:
			raise e
		return rv

	def canonical(self, cli, word):
		try:
			res = self.get_objects(cli, word).order_by('datagroup__path', 'name')
			if len(res) > 0:
				dg_path, tg_name = res[0]
				tg_path = dg_path + ';' + tg_name
				return tg_path
			else:
				return None
		except self.klass.DoesNotExist, e:
			return None
		except Exception, e:
			raise e

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

from core.api.base import tenant_get
from core.api.templates import datagroup_template_get
from core.models import TDataSource

class T_DataGroupTemplateParameter(T_Anyword):

	def __init__(self, *args, **kwargs):
		super(T_DataGroupTemplateParameter, self).__init__(*args, **kwargs)
		self.token_repr = '<datagroup path>'

	def get_context(self, cli):
		
		tenant = None
		tenant_name = cli.env.get('CONFIG_TENANT')
		if tenant_name:
			tenant = tenant_get(tenant_name)
			db_set_current_tenant_schema(tenant_name)
		source = cli.env['SOURCE']
		datagroup_template_name = cli.env['CONFIG_DATAGROUP_TEMPLATE']

		tdg = datagroup_template_get(datagroup_template_name, source=source, tenant=tenant)
		params = tdg.get_configured_params()
		
		return params
		

	def matches(self, cli, word):
		params = self.get_context(cli)

		matching = [ p for p in params if p.name.startswith(word) ]

		return len(matching) > 0

	def completion(self, cli, text):
		params = self.get_context(cli)
		
		rv = []
		for cp in params:
			if cp.name.startswith(text):
				rv.append(
					HelpingString(cp.name, cp.name)
				)
		return rv	

	def canonical(self, cli, word):
		params = self.get_context(cli)

		for p in params:
			if p.name == word:
				return p.name

		return None
		


class T_TEDataGroupTemplateParameter(T_DataGroupTemplateParameter):

	def get_context(self, cli):
		
		tenant = None
		tenant_name = cli.env.get('CONFIG_TENANT')

		tenant = tenant_get(tenant_name)
		db_set_current_tenant_schema(tenant_name)

		source = cli.env['SOURCE']

		dg_name           = cli.env['CONFIG_DATAGROUP_INSTANCE']
		dg_template       = cli.env['CONFIG_DATAGROUP_TEMPLATE']
		elemente_template = cli.env['CONFIG_ELEMENT_TEMPLATE']
		
		tedg = TEDataGroup.objects.get(tenant=tenant, name=dg_name, telement__name=elemente_template, tdatagroup__name=dg_template)

		params = tedg.get_configured_params()

		return params

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#


class T_DataGroupTemplateSubElement(T_Anyword):

	queryset_name = None
	SELECT_METHOD = None

	def __init__(self, *args, **kwargs):
		super(T_DataGroupTemplateSubElement, self).__init__(*args, **kwargs)
		self.token_repr = '<token>'
		
	def get_context(self, cli):
		tenant = None
		tenant_name = cli.env.get('CONFIG_TENANT')
		if tenant_name:
			tenant = tenant_get(tenant_name)
			db_set_current_tenant_schema(tenant_name)
			
		source = cli.env['SOURCE']
		datagroup_template_name = cli.env['CONFIG_DATAGROUP_TEMPLATE']

		tdg = datagroup_template_get(datagroup_template_name, source=source, tenant=tenant)

		return tdg

	def matches(self, cli, word):
		tdg = self.get_context(cli)
		return getattr(tdg, self.SELECT_METHOD)().filter(name__startswith=word).count() > 0

	def completion(self, cli, text):
		tdg = self.get_context(cli)
		ret = getattr(tdg, self.SELECT_METHOD)().filter(name__startswith=text).all()

		# remove duplicated names
		ret = sorted( list(set([tds.name for tds in ret])))  

		rv = []
		for tds_name in ret:
			rv.append(
				HelpingString(tds_name, 'no description')
			)

		return rv

	def canonical(self, cli, word):
		tdg = self.get_context(cli)
		try:
			tds = getattr(tdg, self.SELECT_METHOD)().filter(name__startswith=word).order_by('name')
			tds = tds[0]
			return tds.name
		except TDataSource.DoesNotExist, e:
			return None
		except Exception, e:
			return None

class T_DataGroupTemplateDefinedDataSource(T_DataGroupTemplateSubElement):
	SELECT_METHOD = 'get_defined_datasources'
	def __init__(self, *args, **kwargs):
		super(T_DataGroupTemplateDefinedDataSource, self).__init__(*args, **kwargs)
		self.token_repr = '<datasource name>'

class T_DataGroupTemplateDefinedCondition(T_DataGroupTemplateSubElement):
	SELECT_METHOD = 'get_defined_conditions'
	def __init__(self, *args, **kwargs):
		super(T_DataGroupTemplateDefinedCondition, self).__init__(*args, **kwargs)
		self.token_repr = '<condition name>'

class T_DataGroupTemplateDefinedTimegraph(T_DataGroupTemplateSubElement):
	SELECT_METHOD = 'get_defined_timegraphs'
	def __init__(self, *args, **kwargs):
		super(T_DataGroupTemplateDefinedTimegraph, self).__init__(*args, **kwargs)
		self.token_repr = '<timegraph name>'

class T_DataGroupTemplateDefinedTimegraphSerie(T_Anyword):

	def __init__(self, *args, **kwargs):
		super(T_DataGroupTemplateDefinedTimegraphSerie, self).__init__(*args, **kwargs)
		self.token_repr = '<timegraph serie name>'

	def get_context(self, cli):
		tenant = None
		tenant_name = cli.env.get('CONFIG_TENANT')
		if tenant_name:
			tenant = tenant_get(tenant_name)
			db_set_current_tenant_schema(tenant_name)

		source = cli.env['SOURCE']
		datagroup_template_name = cli.env['CONFIG_DATAGROUP_TEMPLATE']
		timegraph_template_name = cli.env['CONFIG_TIMEGRAPH']

		tdg = datagroup_template_get(datagroup_template_name, source=source, tenant=tenant)
		ttg = tdg.get_defined_timegraphs().filter(name=timegraph_template_name)
		
		series = TTimeGraphSerie.objects.filter(tenant=tenant, ttimegraph__in=ttg)
		return series, timegraph_template_name
		

	def matches(self, cli, word):
		series, timegraph_template_name = self.get_context(cli)

		return  series.filter(name__startswith=word).count() > 0
	

	def completion(self, cli, text):
		series, timegraph_template_name = self.get_context(cli)

		ret = series.filter(name__startswith=text).all()

		# remove duplicated names
		d = dict([ (s.name, s) for s in ret ])
		ret = d.values()

		rv = []
		for r in ret:
			rv.append(
				HelpingString(r.name, 'no description')
			)
		return rv

	def canonical(self, cli, word):
		series, timegraph_template_name = self.get_context(cli)
		
		try:
			series = series.filter(name__startswith=word).all()
			series = series[0]
			return series.name
		except TTimeGraphSerie.DoesNotExist, e:
			return None
		except Exception, e:
			return None



class T_DataGroupTemplateLocalDefinedDataSource(T_DataGroupTemplateSubElement):
	SELECT_METHOD = 'get_local_defined_datasources'
	def __init__(self, *args, **kwargs):
		super(T_DataGroupTemplateLocalDefinedDataSource, self).__init__(*args, **kwargs)
		self.token_repr = '<datasource name>'

class T_DataGroupTemplateLocalDefinedCondition(T_DataGroupTemplateSubElement):
	SELECT_METHOD = 'get_local_defined_conditions'
	def __init__(self, *args, **kwargs):
		super(T_DataGroupTemplateLocalDefinedCondition, self).__init__(*args, **kwargs)
		self.token_repr = '<condition ame>'

class T_DataGroupTemplateLocalDefinedTimegraph(T_DataGroupTemplateSubElement):
	SELECT_METHOD = 'get_local_defined_timegraphs'
	def __init__(self, *args, **kwargs):
		super(T_DataGroupTemplateLocalDefinedTimegraph, self).__init__(*args, **kwargs)
		self.token_repr = '<timegraph name>'


class T_DataGroupTemplateLocalDefinedTimegraphSerie(T_Anyword):

	def __init__(self, *args, **kwargs):
		super(T_DataGroupTemplateLocalDefinedTimegraphSerie, self).__init__(*args, **kwargs)
		self.token_repr = '<timegraph serie name>'

	def get_context(self, cli):
		tenant = None
		tenant_name = cli.env.get('CONFIG_TENANT')
		if tenant_name:
			tenant = tenant_get(tenant_name)
			db_set_current_tenant_schema(tenant_name)

		source = cli.env['SOURCE']
		datagroup_template_name = cli.env['CONFIG_DATAGROUP_TEMPLATE']
		timegraph_template_name = cli.env['CONFIG_TIMEGRAPH']

		tdg = datagroup_template_get(datagroup_template_name, source=source, tenant=tenant)

		series = tdg.get_defined_timegraph_series(timegraph_template_name)

		return series

	def matches(self, cli, word):
		series = self.get_context(cli)
		return  series.filter(name__startswith=word).count() > 0

	def completion(self, cli, text):
		ret = self.get_context(cli)
		ret = set([ s.name for s in ret.filter(name__startswith=text).all().order_by('name') ])
		rv = []
		for name in ret:
			rv.append(
				HelpingString(name, 'no description')
			)
		return rv

	def canonical(self, cli, word):
		series = self.get_context(cli)
		try:
			series = series.filter(name__startswith=word).all().order_by('name')
			series = series[0]
			return series.name
		except TTimeGraphSerie.DoesNotExist, e:
			return None
		except Exception, e:
			return None

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_DataGroupInElementTemplateCompleter(T_Anyword):
	"""
	Completa tutte le istanze di datagroup nel template di un elemento
	"""
	
	def __init__(self, *args, **kwargs):
		super(T_DataGroupInElementTemplateCompleter, self).__init__(*args, **kwargs)		

		self.token_repr = '<datagroup>'
		
	def get_context(self, cli):
		tenant_name = cli.env.get('CONFIG_TENANT')
		tenant = tenant_get(tenant_name)
		db_set_current_tenant_schema(tenant_name)

		current_tenant_name = cli.env['CURRENT_TENANT']

		element_template_name = cli.env['CONFIG_ELEMENT_TEMPLATE']
		element_template_kind = cli.env['CONFIG_ELEMENT_TEMPLATE_KIND']
		
		te = TElement.objects.get(tenant=tenant, kind=element_template_kind, name=element_template_name)

		return tenant, te
		
	def matches(self, cli, word, ):
	
		tenant, te = self.get_context(cli)
		
		count = TEDataGroup.objects.filter(tenant=tenant, telement=te, name__startswith=word).count()
		
		return count > 0

	def completion(self, cli, text):

		tenant, te = self.get_context(cli)
		
		try:
			
			tedgs = TEDataGroup.objects.filter(tenant=tenant, telement=te, name__startswith=text).all()
			rv = []
			for tedg in tedgs:
				rv.append(HelpingString( 
					tedg.name, 
					"Instance of:  %s - %s" % (tedg.tdatagroup.name, tedg.tdatagroup.description))
				)
			return rv
		except Exception, e:
			raise e
			
	def canonical(self, cli, word):

		tenant, te = self.get_context(cli)

		try:
			res = TEDataGroup.objects.get(tenant=tenant, telement=te, name=word)
			return res.name
			
		except TEDataGroup.DoesNotExist, e:
			return None
		except MultipleObjectsReturned, e:
			return '*** ERRORE ***'
		except Exception, e:
			raise e

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_Iconcompleter(T_Anyword):

	def __init__(self, *args, **kwargs):

		super(T_Iconcompleter, self).__init__(*args, **kwargs)

		self.token_repr = '<icon>'

	def matches(self, cli, word):

		current_tenant_name = cli.env['CURRENT_TENANT']

		return len(icon_select_all(current_tenant_name, name_filter_str=word)) > 0

	def completion(self, cli, text):
		current_tenant_name = cli.env['CURRENT_TENANT']

		try:
			rv = []
			if not text:
				text = None

			icons = icon_select_all(current_tenant_name, name_filter_str=text)
			for i in icons:
				rv.append(HelpingString(i, i))

		except Exception, e:
			raise e
		return rv

	def canonical(self, cli, word):

		current_tenant_name = cli.env['CURRENT_TENANT']

		#ictenants = agent_select_all(current_tenant_name, name_filter_str=word)

		icons = icon_select_all(current_tenant_name, name_filter_str=word)

		if len(icons) >= 1:
			return icons[0]

		return None

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_AdditionalDataGroup(T_Anyword):
	
	def __init__(self, *args, **kwargs):
		super(T_AdditionalDataGroup, self).__init__(*args, **kwargs)
		self.token_repr = '<datagroup template>'
		
	def get_element(self, cli):
		element = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']
		return element
		
	def matches(self, cli, word):
		element = self.get_element(cli)
		
		return element.cdatagroup_set.filter(name__startswith=word).count() > 0
		
	def completion(self, cli, text):
		element = self.get_element(cli)

		rv = []
		try:
			cdgs = element.cdatagroup_set.filter(name__startswith=text).all()

			for cdg in cdgs:
				rv.append(HelpingString(cdg.name, cdg.name) )
		except Exception, e:
			raise e
		return rv

	def canonical(self, cli, word):
		element = self.get_element(cli)
			
		cdgs = element.cdatagroup_set.filter(name__startswith=word).all()
			
		# FIX: Remove duplicates in a better way
		d = {}
		for cdg in cdgs:
			d[cdg.name] = cdg
			
		# Remove duplicates
		if len(d) >= 1:
			return sorted(d.keys())[0]

		return None

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_CDataGroupParameter(T_Anyword):

	def __init__(self, *args, **kwargs):
		super(T_CDataGroupParameter, self).__init__(*args, **kwargs)

		self.token_repr = '<parameter>'

	def get_cdg_parameters(self, cli):
		element = cli.get_config_element() #cli.env['CONFIG_ELEMENT_DATA']
		dg_name = cli.env['CONFIG_DATAGROUP_INSTANCE']

		inside_element_template = cli.env.get('ELEMENT_TEMPLATE_DATAGROUP', None)

		if inside_element_template:
			cdg = element.cdatagroup_set.get(name=dg_name, tedatagroup__isnull=False, tdatagroup__isnull=True)
		else:
			cdg = element.cdatagroup_set.get(name=dg_name, tedatagroup__isnull=True , tdatagroup__isnull=False)

		return cdg.cdatagroupparametervalue_set

	def matches(self, cli, word):
		elements = self.get_cdg_parameters(cli)
		return elements.filter(name__startswith=word).count() > 0

	def completion(self, cli, text):
		elements = self.get_cdg_parameters(cli)

		rv = []
		try:
			elements = elements.filter(name__startswith=text).all()

			for e in elements:
				rv.append(HelpingString(e.name, e.name) )
		except Exception, e:
			raise e
		return rv

	def canonical(self, cli, word):

		elements = self.get_cdg_parameters(cli)			
		elements = elements.filter(name__startswith=word).all()
			
		# FIX: Remove duplicates in a better way
		d = {}
		for e in elements:
			d[e.name] = e
			
		# Remove duplicates
		if len(d) >= 1:
			return sorted(d.keys())[0]

		return None

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_LinkCompleter(T_Anyword):

	def __init__(self, *args, **kwargs):
		super(T_LinkCompleter, self).__init__(*args, **kwargs)
		self.token_repr = '<LINK NAME>'

	def get_links(self, cli):
		current_tenant_name = cli.env['CURRENT_TENANT']
		tenant = tenant_get(current_tenant_name)
		db_set_current_tenant_schema(tenant.name)

		#
		# FIX: visibilita' per utente non gestita
		#
		links = Link.objects.filter(tenant=tenant)

		return links

	def matches(self, cli, word):
		return self.get_links(cli).filter(name__startswith=word).count() > 0

	def completion(self, cli, text):
		try:
			rv = []

			links = self.get_links(cli)
			if text:
				links = links.filter(name__startswith=text)

			links = links.order_by('name')

			for e in links:
				rv.append(HelpingString(e.name, e.description))

		except Exception, e:
			raise e
		return rv

	def canonical(self, cli, word):

		links = self.get_links(cli).filter(name__startswith=word).order_by('name')
		if len(links) >= 1:
			return links[0].name

		return None

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_ValidPriorityValue(Token):

	def __init__(self, *args, **kwargs):
		super(T_ValidPriorityValue, self).__init__(*args, **kwargs)
		self.token_repr = '<1-100>'

	def matches(self, cli, word):
		try:
			if PRIORITY_MIN <= int(word) and int(word) <= PRIORITY_MAX:
				return True
			else:
				return False
		except ValueError, e:
			return False
	ordering = 2

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class T_SchedulingGroupcompleter(T_Resourcecompleter):

	token_regexp = VALID_ELEMENT_NAME

	klass  = SchedulingGroup
	key          = 'name'

	def get_base_queryset(self, cli, text):
		qs = super(T_SchedulingGroupcompleter, self).get_base_queryset(cli, text)

		# Mostriamo quelli "espliciti" (quelli impliciti non devono essere visibili all'utente)
		qs = qs.filter(is_real=True)

		return qs

