# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it> 
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

# Utility functions that do not require importing of many heavy modules

import sys

from django.utils.translation import ugettext as _

from core.models import MonitoredElement

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def _dot_terminated_text():
	
	#
	# Inverse of dot_stuff
	#
	buf=''

	if sys.stdin.isatty():
		sys.stdout.write(_("Enter lines of input. Terminate with a single dot on a line by itself.\n"))
		sys.stdout.write(_("To insert a line with a leading dot, use two dots.\n"))

	while True:
		try:
			line = raw_input(_("(dot-terminated text)#"))
			line = line + '\n'
		except EOFError, e:
			return buf

		#
		# FIX: use this code?
		#
		#if sys.stdin.isatty():
		#	line = raw_input(_("(dot-terminated text)#"))
		#else:
		#	line = raw_input()
		#
		#line += '\n'
		
		if line == ".\n":
			return buf
			
		if line[0:2] == '..':
			line = line[1:]
		
		#
		# FIX: qsingle?
		#	
		#add = qsingle(line)
		add = line
		
		if len(add)>1 and add[-2:] == '\n\n':
			add = add[:-1]
			
		buf = buf + add

def dot_terminated_text():
	return _dot_terminated_text()
	
	
def dot_stuff(text):
	#
	# Inverse of dot_terminated_text
	#
	out=[]
	for line in text.split('\n'):
		if line != '' and line[0] == '.':
			out.append('.'+line)
		else:
			out.append(line)
	return('\n'.join(out) + '\n.\n')

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def parse_suspend_upto(buf):
	# Formats: "YYYY-MM-DD HH:MM:SS", "forever", number of seconds
	if buf.lower() == "forever":
		return 0
	try:
		d=datetime.datetime.strptime(buf, DATETIME_FORMAT)
		return int(time.mktime(d.timetuple()))
	except:
		pass
	try:
		return int(time.time()) + int(buf)
	except:
		raise ValueError('Expected number of seconds, or date and time in "YYYY-MM-DD HH:MM:SS" format (with quotes), or "forever", got:' + buf)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def generic_details_aoa(o, timestamps=[], suppress=None, visible=None, convert_val_fun=None):

	d = []
	
	if visible is not None:
		fields = visible
	else:
		fields = [ f.name for f in o._meta.fields ]
	

	#for f in o._meta.fields:
	for fname in fields:

		if suppress is not None:
			#if f.name in suppress:
			if fname in suppress:
				continue
		#if visible is not None:
		#	if f.name not in visible:
		#		continue

		#name = f.name
		#val  = getattr(o, f.name)

		name = fname
		val  = getattr(o, fname)

		if convert_val_fun is not None:
			val = convert_val_fun(val)
			
		if val is None:
			val = ''
		elif fname in timestamps:
			val = '%s %s' % (val, timestamp_to_isostr(val))
		
		val = '%s' % (val) if val is not None else ''
			
		d.append( [ name, val ] )

	return d

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

#_TERM_COLOR = "\033[%sm"
#
#TERM_COLOR_RED    = 91 #'\033[91m'
#TERM_COLOR_GREEN  = 92 #'\033[92m'
#TERM_COLOR_YELLOW = 93 #'\033[93m'
#TERM_COLOR_CYAN   = 32 #'\033[93m'
#
#TERM_COLOR_ENDC   = _TERM_COLOR % (0)
#
#TERM_BOLD         = "1;36"
#
#if sys.stdout.isatty():
#	def term_color(color, msg):
#		return _TERM_COLOR % (color) + msg + TERM_COLOR_ENDC
#
#	def term_bold(msg):
#		return _TERM_COLOR % (TERM_BOLD) + msg + TERM_COLOR_ENDC
#else:
#	def term_color(color, msg):
#		return msg
#
#	def term_bold(msg):
#		return msg
#
#def term_red(msg):
#	return term_color(TERM_COLOR_RED, msg)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def convert_special_fields_val_to_cli_val(v):

	if isinstance(v, MonitoredElement):
		v = v.name

	return v


