# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

#
# DO NOT EDIT THIS FILE !!!!!
# DO NOT EDIT THIS FILE !!!!!
# DO NOT EDIT THIS FILE !!!!!
# DO NOT EDIT THIS FILE !!!!!
# DO NOT EDIT THIS FILE !!!!!
# DO NOT EDIT THIS FILE !!!!!
#

"""

.. _sanet-default_dirs:

default_dirs 
========================================================================

Questo modulo definisce le costanti per tutte le directory principali di Sanet.


La costante SANET_INSTALL_DIR  *non deve mai essere usata nel codice*. 

Il codice di sanet deve usare solo
queste costanti.

	+----------------+-----------------------------------------------------------------------------------+
	| Constant       | Description                                                                       |
	+================+===================================================================================+
	| BASE_DIR       | Base directory of Sanet main package                                              |
	+----------------+-----------------------------------------------------------------------------------+
	| DATA_DIR       | Base directory where to save sanet files                                          |
	+----------------+-----------------------------------------------------------------------------------+
	| LOG_DIR        | Log directory                                                                     |
	+----------------+-----------------------------------------------------------------------------------+
	| PID_DIR        | Run file directory                                                                |
	+----------------+-----------------------------------------------------------------------------------+
	| MEDIA_DIR      | Base directory where media files are stored                                       |
	+----------------+-----------------------------------------------------------------------------------+
	| MEDIA_WEB_ROOT | Base directory where web media files are stored (should be the same of MEDIA_DIR) |
	+----------------+-----------------------------------------------------------------------------------+

Queste costanti **DEVONO** essere usate nel codice per comporre i path dove salvare i file creati da Sanet.

In fase di sviluppo tutte queste costanti possono essere *forzate* a piacimento nel file *settings.py*, ma in fase di deploy
non **dovrebero essere** essere modificate dagli *amministratori di sistema*, salvo che una installazione
molto particolare non lo richieda.

.. warning:: Il file *default_settings.py* utilizza il valore di queste costanti per impostarne altre.
             
             Se si modifica il valore di queste costanti, bisogna accertarsi che il file *settings.py*
             gestisca correttamente anche tutte le altre costanti che *dipendono* da queste.


Tenant Media directory
========================

All'interno della directroy {{MEDIA_DIR}} viene creata una directory per ogni tenant:

	::
		
		{{ MEDIA_DIR }}/<tenant uuid>/

E un link simbolico alla directory stsessa:

	::
		
		{{ MEDIA_DIR }}/<tenant name>/   -->   {{ MEDIA_DIR }}/<tenant uuid>/


All'interno di questa directory, viene creata una directory utilizzata
per salvare file *multimediali* accessibili dal web.

	::
		
		{{ MEDIA_DIR }} / <tenant uuid> / {{ TENANT_MEDIA_DIRNAME }}

.. autodata:: TENANT_MEDIA_DIRNAME


Gestione di default delle directory con SANET_INSTALL_DIR e SANET_HOME
--------------------------------------------------------------------------

Quando il modulo viene importato, verifica la presenza della variabile d'ambiente SANET_HOME
e decide in che modo impostare le directory di default.

.. note:: Il valore di queste costanti puo' essere comunque *sovrascritto* specificando le costanti nel file settings.py.

Questo modulo gestisce i valori di default delle costanti in questo modo:

	+------------------------+---------------------------------------------------+---------------------------------------------------------+
	| Directory              | Posizione directory di Default                    | Valore con SANET_HOME impostata                         |
	+========================+===================================================+=========================================================+
	| BASE_DIR               | SANET_INSTALL_DIR                                 | $SANET_HOME/                                            |
	+------------------------+---------------------------------------------------+---------------------------------------------------------+
	| DATA_DIR               | /var/lib/sanet3                                   | $SANET_HOME/var/lib                                     |
	+------------------------+---------------------------------------------------+---------------------------------------------------------+
	| PID_DIR                | /var/run/sanet3                                   | $SANET_HOME/var/run                                     |
	+------------------------+---------------------------------------------------+---------------------------------------------------------+
	| LOG_DIR                | /var/log/sanet3                                   | $SANET_HOME/var/log                                     |
	+------------------------+---------------------------------------------------+---------------------------------------------------------+

"""

import os, sys

from os.path import join, abspath

#
# Sanet directories. You can set each one differently, but be careful.
#
BASE_DIR       = None # Base directory of Sanet main package
DATA_DIR       = None # Base directory where to save sanet files
LOG_DIR        = None # Log directory
PID_DIR        = None # Run file directory
MEDIA_DIR      = None # Base directory where media files are stored
MEDIA_WEB_ROOT = None # Base directory where web media files are stored (should be the same of MEDIA_DIR)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

#
# Try to autodetect the installation directory
#

try:
	from .install_dir import SANET_INSTALL_DIR
except ImportError, e:
	SANET_INSTALL_DIR = None

SANET_INSTALL_DIR = os.environ.get("SANET_HOME", SANET_INSTALL_DIR)
if not SANET_INSTALL_DIR:
	SANET_INSTALL_DIR = abspath(os.path.join(os.path.dirname(abspath(__file__)), ".." ))
	
#
# Django requires that the project main directory is in the PYTHONPATH.
#
sys.path.append( SANET_INSTALL_DIR )

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

#:
#: Name of media directory for every tenant
#:
#:   {{MEDIA_DIR}} / <tenant uuid> / {{TENANT_MEDIA_DIRNAME}}
#:
TENANT_MEDIA_DIRNAME = 'media'

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

#
# Try to set default dirs (conf, log, ecc....)
#
if os.environ.has_key('SANET_HOME'):

	BASE_DIR       = os.environ['SANET_HOME']
	LOG_DIR        = os.path.join( BASE_DIR, "var", "log" )
	PID_DIR        = os.path.join( BASE_DIR, "var", "run" )	
	DATA_DIR       = os.path.join( BASE_DIR, "var", "lib" )
	MEDIA_DIR      = DATA_DIR
	MEDIA_WEB_ROOT = MEDIA_DIR

else:

	# Put all files under the /var/..../sanet3/
	_OS_VAR_DIR    = '/var'
	_OS_LOG_DIR    = os.path.join( _OS_VAR_DIR, "log" )
	_OS_RUN_DIR    = os.path.join( _OS_VAR_DIR, "run" )
	_OS_DATA_DIR   = os.path.join( _OS_VAR_DIR, "lib" )
	_SANET_DIR_NAME = "sanet3"

	BASE_DIR       = SANET_INSTALL_DIR
	LOG_DIR        = os.path.join( _OS_LOG_DIR , _SANET_DIR_NAME )
	PID_DIR        = os.path.join( _OS_RUN_DIR , _SANET_DIR_NAME )
	DATA_DIR       = os.path.join( _OS_DATA_DIR, _SANET_DIR_NAME )
	MEDIA_DIR      = DATA_DIR
	MEDIA_WEB_ROOT = MEDIA_DIR
