# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
WSGI config for SANET3 project.

This module contains the WSGI application used by Django's development server
and any production WSGI deployments. It should expose a module-level variable
named ``application``. Django's ``runserver`` and ``runfcgi`` commands discover
this application via the ``WSGI_APPLICATION`` setting.

Usually you will have the standard Django WSGI application here, but it also
might make sense to replace the whole Django WSGI application with a custom one
that later delegates to the Django one. For example, you could introduce WSGI
middleware here, or combine a Django application with an application of another
framework.

"""

import os, sys

#==============================================================================#

SANET_INSTALL_DIR=None

SANET_INSTALL_DIR = os.environ.get("SANET_HOME", SANET_INSTALL_DIR)

from os.path import join, dirname, abspath
if not SANET_INSTALL_DIR:
	SANET_INSTALL_DIR = abspath(os.path.join(os.path.dirname(abspath(__file__)),".."))

sys.path.append( SANET_INSTALL_DIR )

#==============================================================================#

#
# HACK: If we import compressed (zipped) Eggs, we could get this error from Apache2:
#      
#     The following error occurred while trying to extract file(s) to the Python egg cache:
#
#         [Errno 13] Permission denied: '/var/www/.cache'
#
# Avoid any permission problem by setting Eggs cache inside a writable directory.

os.environ['PYTHON_EGG_CACHE'] = '/tmp'

#==============================================================================#

#
# By default SANET will look for the following
#
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "conf.wsgi_settings")

# Django 1.11
import django
django.setup()


# This application object is used by any WSGI server configured to use this
# file. This includes Django's development server, if the WSGI_APPLICATION
# setting points here.
from django.core.wsgi import get_wsgi_application


# HACK: wsgi does not receive webserver's (apache) environment as expected. 
#       Be sure to use SANET_HOME (od other environement variables) if set.
def application(environ, start_response):

	ENV_VARIABLES_TO_PASS = [
		'SANET_HOME', 
		'RRDCACHED_ADDRESS',
	]

	# pass the WSGI environment variables on through to os.environ
	for var in ENV_VARIABLES_TO_PASS:
		v = environ.get(var, None)
		if v is not None:
			os.environ[var] = v

	#
	# Is useful to know when code is running inside the a webserver or not.
	#
	os.environ['SANET_IN_WSGI'] = 'yes'

	#
	# FIX: web server (apache) call application() for every request.
	#      move this code somewhere else.
	#
	#from core.api.sysstats import check_system_settings
	#
	#check_system_settings()

	#
	# GO
	#
	real_application = get_wsgi_application()

	return real_application(environ, start_response)


# Apply WSGI middleware here.
# from helloworld.wsgi import HelloWorldApplication
# application = HelloWorldApplication(application)




