# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from sanet_common.pythonutils import load_symbol

from conf import get_configuration

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def init_cache_manager(handler_module_name, init_params):
	
	handler_class = load_symbol( handler_module_name )
	handler       = handler_class(**init_params)
	
	return handler

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def init_cache_manager_from_conf():
	
	config = get_configuration()
	
	return  init_cache_manager(config.CACHE_HANDLER, config.CACHE_HANDLER_PARAMS)


#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

# Per pulire la cache dobbiamo istanizare un cache_manager, ma non possiamo
# ricrearlo tutte le volte che si scatena un evento di update altrimenti
# apriamo e chiudiamo troppe connessioni TCP che rimangono in TIME_WAIT
# e puo' succedere che il kernel dopo un po' abbia troppe connessioni 
# attive.
#
# Istanziamo con un "singleton" un solo cache manager. 
#
# Non e' bello avere un cache_manager definito globalmente, perche' potrebbe
# essere condiviso ma i cache_manager dovrebbero essere threadsafe (usano un pool di connessioni)


__GLOBAL_CACHE_MANAGER = None

def _GET_GLOBAL_CACHE_MANGER():

	global __GLOBAL_CACHE_MANAGER

	if __GLOBAL_CACHE_MANAGER is None:
		__GLOBAL_CACHE_MANAGER = init_cache_manager_from_conf()

	return __GLOBAL_CACHE_MANAGER
