# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.


"""

Null DataSource Store module
=======================================================================

Questo modulo non salva e non restituisce dati. 

Il suo utilizzo e' prevalentemente a scopo di debug.

.. note: questo modulo puo' essere usato per disabilitare temporaneamente il salvataggio dei dati senza alterare il normale funzionamento di Sanet.

Configurazione modulo
-------------------------------------------

Per utilizzarlo bisogna impostare nel file di configurazione di Sanet (settings.py):

	::
		
		DATASOURCE_STORAGE_HANDLER = 'core.api.datasource.nullstore.NullDataSourceStore'

.. note:: Il parametro DATASOURCE_STORAGE_PARAMS viene completamente ignorato


Il modulo *non e'* parametrizzabile.

.. note:: Il parametro di configurazione DATASOURCE_STORAGE_PARAMS viene completamente ignorato. 


"""

from core.api.datasource import AbstractDataSourceStore, DataSourceStoreException

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class NullDataSourceStore(AbstractDataSourceStore):
	"""
	This special store does nothing. 
	
	For test/debug only.
	"""

	def __init__(seflf, *params, **kwargs):
		pass

	def init_store(self, datasource):
		return True

	def check_sanet_conf(self):
		return True

	def init_tenant(self, tenant):
		return True

	def exists(self, tenant_id, datasource_id):
		return True

	def save_value(self, tenant_id, datasource_id, timestamp, value):
		return True

	def fetch_values(self, tenant_id, datasource_id, ts_start=None, ts_end=None, resolution=None, min_value=None, max_value=None):
		msg = "No data with this Datasource Store"
		print __name__, msg
		raise DataSourceStoreException( msg )

	@staticmethod
	def validate_storage_spec_options(options_string):
		return True

	def check_and_update_storage_spec(self, tenant_id, datasource_id, minperiod, gauge_flag, storage_spec_string):
		return True

	def stats(self, timeout=None):
		return { "info": "NullDataSourceStore is working"}
