
import socket


from django.db.models import Q

from core.models import MonitoredNode


def guess_node_from_ip(logger, source_ip):

			logger.info("Sanet node not found with source ip %s", source_ip)

			localhost_name = socket.gethostname()
			logger.info("Localhost has host name = %s", localhost_name)
			
			localhost_fqdn = socket.getfqdn(localhost_name)
			logger.info("Localhost has fqdn = %s", localhost_fqdn)
			
			localhost_domain = localhost_fqdn.replace(localhost_name, "")
			logger.info("Localhost has domain = %s", localhost_domain)
			
			# Risolviamo il nome dell'host
			node_fqdn = socket.getfqdn(source_ip)
			logger.info("Remote host has fqdn = %s", node_fqdn)
			
			# cerchiamo di ottenere l'hostname
			if node_fqdn.find(".") > -1:
				hostname, domain = node_fqdn.split(".", 1)
			else:
				hostname = node_fqdn

			# Vediamo se il nodo e' monitorato con il suo hostname
			sanet_node = None
			try:
				logger.info("Searching sanet node with name = %s...", hostname)
				sanet_node = MonitoredNode.objects.get(name=hostname)
			except MonitoredNode.DoesNotExist, e:
				pass
				
			# O con tutto il dominio
			if sanet_node is None:
				try:
					logger.info("Searching sanet node with hostname/fqdn = %s...", node_fqdn)
					sanet_node = MonitoredNode.objects.get(Q(name=node_fqdn)|Q(dns_name=node_fqdn))
				except MonitoredNode.DoesNotExist, e:
					logger.info("No sanet node found with name: %s", node_fqdn)
					pass
				except MonitoredNode.MultipleObjectsReturned, e:
					logger.info("Too many nodes found with name: %s", node_fqdn)
					pass
					
			if sanet_node is None:
				sanet_node_name = source_ip
			else:
				logger.info("Found node: %s", sanet_node)
				sanet_node_name = sanet_node.name

			return sanet_node, sanet_node_name
