# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os
import time

#from django.db import transaction

from sanet_common.sanetdefs.constants import *
from sanet_common.exceptions import *

from conf import get_configuration

from core.db_utils import transaction_commit_on_success

from core.exceptions import InvalidConfigurationException

from core.models import convert_to_param_type
from core.models import SOURCE_LIBRARY, SOURCE_LOCAL

from core.models import DataGroup, DataSource, Condition

from core.models import Tenant
from core.models import Event
from core.models import TDataGroup
from core.models import TField
from core.models import TDataSource, TDataGroupFieldValue, TDataGroupParameterValue
from core.models import TDataSourceFieldValue
from core.models import TCondition, TConditionFieldValue 
from core.models import TTimeGraph, TTimeGraphFieldValue
from core.models import TTimeGraphSerie, TTimeGraphSerieFieldValue

from core.models import TElement
from core.models import TEDataGroup, TEConditionFieldValue, TEDataSourceFieldValue, TETimeGraphFieldValue, TETimeGraphSerieFieldValue
from core.models import ElementTemplate

from core.models import CDataGroup, CConditionFieldValue, CDataSourceFieldValue, CTimeGraphFieldValue, CTimeGraphSerieFieldValue

from core.api.base import db_set_current_tenant_schema
from core.api.base import tenant_get



DEBUG = get_configuration().DEBUG_CONFIG or os.environ.get('DEBUG_CONFIG',False)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#


@transaction_commit_on_success
def datagroup_template_move_to_library(name, source, tenant):

	mtime = time.time()

	#
	# FIX: Controllare che nessun altro tenant definisca lo stesso datagroup template
	#

	tdg = datagroup_template_get(name, source=source, tenant=tenant)
	if tdg.parent != None and tdg.parent.source != SOURCE_LIBRARY: # per il momento permettiamo questo spostamento
		raise Exception("Datagroup template %s has a parent datagroup (%s) but it's not in the library." % (tdg.name, tdg.parent.name))

	if tdg.tenant:
		db_set_current_tenant_schema(tdg.tenant.name)
		

	#
	# Move datagroup to library
	#
	if DEBUG: print "Moving to library"

	tdg.source = SOURCE_LIBRARY
	tdg.tenant = None
	#tdg.mtime  = mtime
	#tdg.save()
	tdg.concurrent_save(['source','tenant'], new_sentinel_value=mtime)
	
	
	for tdgfv in tdg.tdatagroupfieldvalue_set.all():
		if DEBUG: print tdgfv
		tdgfv.tenant = None
		#tdgfv.mtime  = mtime
		#tdgfv.save()
		tdgfv.concurrent_save(['tenant'], new_sentinel_value=mtime)

	for tdgpv in tdg.tdatagroupparametervalue_set.all():
		if DEBUG: print tdgpv
		tdgpv.tenant = None
		#tdgpv.mtime  = mtime
		#tdgpv.save()
		tdgpv.concurrent_save(['tenant'], new_sentinel_value=mtime)
	
	for tds in tdg.tdatasource_set.all():
		if DEBUG: print tds
		tds.tenant = None
		#tds.mtime = mtime
		#tds.save()
		tds.concurrent_save(['tenant'], new_sentinel_value=mtime)
		
		for tdsfv in tds.tdatasourcefieldvalue_set.all():
			if DEBUG: print tdsfv
			tdsfv.tenant=None
			#tdsfv.mtime = mtime
			#tdsfv.save()
			tdsfv.concurrent_save(['tenant'], new_sentinel_value=mtime)
		
	for tc in tdg.tcondition_set.all():
		if DEBUG: print tc
		tc.tenant = None
		#tc.mtime = mtime
		#tc.save()
		tc.concurrent_save(['tenant'], new_sentinel_value=mtime)

		for tcfv in tc.tconditionfieldvalue_set.all():
			if DEBUG: print tcfv
			tcfv.tenant=None
			#tcfv.mtime = mtime
			#tcfv.save()
			tcfv.concurrent_save(['tenant'], new_sentinel_value=mtime)
	
	for ttg in tdg.ttimegraph_set.all():
		if DEBUG: print ttg
		ttg.tenant = None
		#ttg.mtime = mtime
		#ttg.save()
		ttg.concurrent_save(['tenant'], new_sentinel_value=mtime)
		
		for ttgfv in ttg.ttimegraphfieldvalue_set.all():
			if DEBUG: print ttgfv
			ttgfv.tenant = None
			#ttgfv.mtime = mtime
			#ttgfv.save()
			ttgfv.concurrent_save(['tenant'], new_sentinel_value=mtime)

		for ttgs  in ttg.ttimegraphserie_set.all():
			if DEBUG: print ttgs
			ttgs.tenant = None
			#ttgs.mtime = mtime
			#ttgs.save()
			ttgs.concurrent_save(['tenant'], new_sentinel_value=mtime)
			
			for ttgsfv in ttgs.ttimegraphseriefieldvalue_set.all():
				if DEBUG: print ttgsfv
				ttgsfv.tenant  = None
				#ttgsfv.mtime = mtime
				#ttgsfv.save()
				ttgsfv.concurrent_save(['tenant'], new_sentinel_value=mtime)


	#
	# Create local datagroup template
	#
	if DEBUG: print "Creating local copy"
	tdg_local = TDataGroup(
		tenant = tenant,
		source = SOURCE_LOCAL,
		name   = tdg.name,
		parent = tdg,
		description = "Automatically created by CLI when moved datagroup to library section (%s)" % (time.ctime(mtime)),
	)
	tdg_local.mtime = mtime
	tdg_local.save()

	#
	# Colleghiamo tutti i tedatagroup negli element template al nuovo datagroup-template
	#
	if DEBUG: print "Changing reference in element templates"
	for tedg in TEDataGroup.objects.filter(tenant=tenant, tdatagroup=tdg):
		if DEBUG: print "\t", "\t", tedg
		tedg.tdatagroup = tdg_local
		tedg.mtime = mtime
		tedg.save()
		#tedg.concurrent_save(['tdatagroup'], new_sentinel_value=mtime)

	#
	# Colleghiamo tutti i configured datagroup
	#
	if DEBUG: print "Changing reference in element templates"
	for cdg in CDataGroup.objects.filter(tenant=tenant, tdatagroup=tdg):
		if DEBUG: print "\t", "\t", cdg
		cdg.tdatagroup = tdg_local
		#cdg.mtime = mtime
		#cdg.save()
		cdg.concurrent_save(['tdatagroup'], new_sentinel_value=mtime)
		
	if DEBUG: print "Finish"


	tdg.concurrent_save(['source','tenant'], new_sentinel_value=mtime)

	return
		
#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def datagroup_template_get(name, source=None, tenant=None):

	if source == None:
		source = SOURCE_LOCAL

	try:
		tc = TDataGroup.objects.get(name=name, source=source, tenant=tenant)
		return tc
	except TDataGroup.DoesNotExist, e:
		return None
		

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def datagroup_template_select_all(name_filter_str=None, source=None, tenant=None):

	tdatagroups = TDataGroup.objects
	
	if source != None:
		tdatagroups = tdatagroups.filter(source=source)
		
	if tenant != None:
		tdatagroups = tdatagroups.filter(tenant=tenant)
	
	if name_filter_str:
		pattern = '%s' % (name_filter_str)
		tdatagroups = tdatagroups.filter(name__startswith=pattern).order_by('name')
		
	return tdatagroups.all()
	
#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def datagroup_template_create(name, source=None, tenant=None, owner_user=None):

	mtime = time.time()

	if source==None: 
		source=SOURCE_LOCAL

	assert ( (source == SOURCE_LIBRARY and tenant==None) or (source == SOURCE_LOCAL and tenant != None) )

	if tenant and owner_user:
		owner_user.assert_capability(tenant, 'core.template.dg.create')


	# EVITIAMO CHE GENTE CREI IN TEMPI DIVERSI DATAGROUP CON LO STESSO NOME
	# PRIMA IN UN SITO POI NELLA LIBRARY.
	if source == SOURCE_LIBRARY:
		# Search in every tenant
		defined_tdgs = TDataGroup.objects.filter(name=name, source=SOURCE_LOCAL) 
		if defined_tdgs:
				raise Exception("Datagroup template '%s' alread defined outside the library (found %s datagroups: %s)" % (name, len(defined_tdgs), ','.join([str(t) for t in defined_tdgs]) ))

	if tenant:
		db_set_current_tenant_schema(tenant.name)

	tdg = TDataGroup(name=name, source=source, tenant=tenant, owner=owner_user)
	tdg.mtime = mtime
	tdg.save()
	
	return tdg

from django.contrib.auth.models import Group

def datagroup_template_group_permission(tenant, name, group_name, permission, user=None):

	tdg = TDataGroup.objects.get(name=name, source=SOURCE_LOCAL) 

	group = Group.objects.get(name=group_name)

	if tenant and user:

		user.assert_capability(tdg.tenant, 'core.template.dg.acl.edit')

		tdg.assert_permission_readwrite(user)

		user.assert_in_group(group_name)

	tdg.add_group_permission(group, permission)


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def datagroup_template_delete(name, source=None, tenant=None, force=False, user=None):

	if source == None: 
		source = SOURCE_LOCAL

	assert ( (source == SOURCE_LIBRARY and tenant==None) or (source == SOURCE_LOCAL and tenant != None) )

	if tenant:
		db_set_current_tenant_schema(tenant.name)

	tdg = TDataGroup.objects.get(tenant=tenant, name=name, source=source)

	if user: 
		user.assert_capability(tenant, 'core.template.dg.delete')
		tdg.assert_permission_readwrite(user)

	#
	# Checks
	#
	if not force:
		# Check if it's parent of other templates
		l = TDataGroup.objects.filter(parent=tdg)
		if l.count() > 0:
			raise Exception("Datagroup template '%s' used in configuration.(%s)" % (tdg, ','.join([str(t) for t in l]) ))

		# Check if it's used inside element templates
		l = TEDataGroup.objects.filter(tdatagroup=tdg)
		if l.count() > 0:
			raise Exception("Datagroup template '%s' used in element template. (%s)" % (tdg, ','.join([str(t) for t in l]) ))

		# Check if it's used inside element templates
		l = CDataGroup.objects.filter(tdatagroup=tdg)
		if l.count() > 0:
			raise Exception("Datagroup template '%s' used in elements. (%s)" % (tdg, ','.join([str(t) for t in l]) ))

	#
	# FIX: WE MUST REFRESH NODES CONFIGURATION
	#
	
	TEDataGroup.objects.filter(tdatagroup=tdg).delete()
	
	CDataGroup.objects.filter(tdatagroup=tdg).delete()
	
	tdg.delete()

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def datagroup_library_template_duplicate_on_tenant(tdg, tenant, owner_user=None):

	return _datagroup_library_template_duplicate_on_tenant(tdg, tenant, owner_user=owner_user)

def _datagroup_library_template_duplicate_on_tenant(tdg, tenant, owner_user=None):

	source = SOURCE_LOCAL

	if owner_user: owner_user.assert_capability(tenant, 'core.template.dg.create')

	new_tdg = TDataGroup(
		source = source, 
		tenant = tenant, 
		parent = tdg,
		name   = tdg.name, 
		mtime  = time.time(),
		owner  = owner_user,
	)

	new_tdg.save()

	return new_tdg
	
#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def datagroup_template_description_update(datagroup_template_name, description, source=None, tenant=None, user=None):
	
	mtime = time.time()
	
	assert ( (source == SOURCE_LIBRARY and tenant==None) or (source == SOURCE_LOCAL and tenant != None) )
		
	if tenant:
		db_set_current_tenant_schema(tenant.name)
		
	if user: user.assert_capability(tenant, 'core.template.dg.edit')

	# FIX: usa UPDATE and not SAVE
	tdg        = datagroup_template_get(datagroup_template_name, source=source, tenant=tenant)
	
	tdg.description = description
	tdg.mtime = mtime
	tdg.save() #update_fields=['description','mtime']) django 1.5
	
	return
	
#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def datagroup_template_set_parent(t, tp, user=None):

	mtime = time.time()

	if user: user.assert_capability(t.tenant, 'core.template.dg.edit')

	if t.parent == tp:
		return

	if tp != None:

		# Se cerco di impostare come padre un template della library lo devo prima
		# duplicare sul tenant
		if t.source == SOURCE_LOCAL and tp.source == SOURCE_LIBRARY:

			# Se i nomi sono uguali, sto "rimettendo" a posto un gerarchia sbagliata quindi
			# non devo creare niente sul sito
			if t.name != tp.name:
				tp = _datagroup_library_template_duplicate_on_tenant(tp, tenant=t.tenant)

	#
	# Update parameters
	#
	t.parent = tp

	t.mtime  = mtime
	t.save()  # FIX: update_fields django 1.5

	#
	# FIX: QUesto refresh e' molto dispendioso. E' possibile refreshare tutti
	#      i template collegati in maniera rapida?
	#

	#_refresh_datagroup_template(t)

	_refresh_datagroup_template_tree(t, mtime)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def refresh_datagroup_template(t):
	mtime = time.time()
	return _refresh_datagroup_template(t, mtime)

@transaction_commit_on_success
def refresh_datagroup_template_tree(t):
	mtime = time.time()
	return _refresh_datagroup_template_tree(t, mtime)

def _refresh_datagroup_template(t, mtime):

	tenant = t.tenant

	if tenant:
		db_set_current_tenant_schema(tenant.name)

	#
	# Additional Datagroups
	#
	cdgs = CDataGroup.objects.filter(tenant=tenant, tdatagroup=t, tedatagroup__isnull=True).all()
	for cdg in cdgs:
		if DEBUG: print cdg
		for dg in cdg.datagroup_set.all():
			if DEBUG: print "Refreshing dg", dg
			dg.update_from_config(mtime=mtime)

	#
	# Element Datagroups
	#
	cdgs = CDataGroup.objects.filter(tenant=tenant, tdatagroup__isnull=True, tedatagroup__tdatagroup=t).all()
	for cdg in cdgs:
		if DEBUG: print cdg
		for dg in cdg.datagroup_set.all():
			if DEBUG: print "Refreshing dg", dg
			dg.update_from_config(mtime=mtime)



def _refresh_datagroup_template_tree(t, mtime, select_datasources=None, selected_conditions=None, selected_timegraphs=None):

	assert( (t.source == SOURCE_LIBRARY and t.tenant == None) or  (t.source == SOURCE_LOCAL and t.tenant != None) )

	if t.tenant != None:
		# Se questo template e' locale, allora ci potrebbero essere dei datagroup
		# configurati per usarlo. Refreshiamoli
		_refresh_datagroup_template(t, mtime)

	subtemplates = t.tdatagroup_set.all()
	for st in subtemplates:
		_refresh_datagroup_template_tree(st, mtime)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def refresh_datagroup(dg):

	mtime = time.time()

	dg.update_from_config(mtime=mtime)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def refresh_element(e):

	_refresh_element(e)


def _refresh_element(e, mtime=None, only_fields_names=None):
	"""
	Effettua il refresh di tutti i datagroup di un elemento (compresi quelli dei sotto-elementi)
	"""
	if DEBUG: print "_refresh_element()", e, "(only fields=",only_fields_names,")"

	if mtime is None:
		mtime = time.time()

	tenant = e.tenant
	db_set_current_tenant_schema(tenant.name)

	# Aggiorniamo l'elemento
	_refresh_element_config(e, mtime, only_fields_names=only_fields_names)


	# Aggiorniamo tutti i sotto-elementi

	# XXXXXXXXXXXXXXXXXXXXXXXXXXX CODICE DUPLICATO?
	#for sube in e.get_sub_elements().all():            
	#	_refresh_element(sube, mtime=mtime, only_fields_names=only_fields_names)

	for sube in e.get_all_sub_elements():
		_refresh_element(sube, mtime=mtime, only_fields_names=only_fields_names)


def _refresh_element_config(e, mtime, only_fields_names=None):
	#
	# FIX: questo refresh va cambiato.
	#      bisogna aggiornare in base all'element template e non semplicemente
	#      tutti i datagroup associati all'elemento.
	#

	cdgs = CDataGroup.objects.filter(tenant=e.tenant, element=e).all()
	for cdg in cdgs:
		for dg in cdg.datagroup_set.all():
			if DEBUG: print "Refreshing dg", dg, "(only fields=",only_fields_names,")"
			#dg.update_from_config_and_save(mtime)
			dg.update_from_config(mtime, only_fields_names=only_fields_names)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def datagroup_template_set_field(dg_name, f_name, f_str_value, source=SOURCE_LOCAL, tenant=None, user=None):

	mtime = time.time()

	if tenant:
		if user: user.assert_capability(tenant, 'core.template.dg.edit')


	tdg = TDataGroup.objects.get(name=dg_name, source=source, tenant=tenant)

	tf = TField.objects.get(fclass=TField.FIELD_DATAGROUP, name=f_name)

	tf.validate_value(f_str_value)

	created = False
	try:
		tdgfv = TDataGroupFieldValue.objects.get(tdatagroup=tdg, tfield=tf, tenant=tenant)
	except TDataGroupFieldValue.DoesNotExist, e:
		tdgfv = TDataGroupFieldValue(tdatagroup=tdg, tfield=tf, tenant=tenant)
		created = True

	if tdgfv.value == f_str_value:
		return

	tdgfv.tenant = tenant
	tdgfv.value = f_str_value
	tdgfv.mtime = mtime
	tdgfv.save() # FIX: use update

	if DEBUG: print "Updated field", tdgfv

	#
	# Propagate
	#

	_update_configured_datagroup_field_value(tdg, tdgfv, mtime=mtime)
	_propagate_configured_datagroup_field_value(tdg, tdgfv, mtime=mtime)
	_update_datagroup_in_element_template_field_value(tdg, tdgfv, mtime=mtime)
	_propagate_datagroup_in_element_template_field_value(tdg, tdgfv, mtime=mtime)

	return

@transaction_commit_on_success
def datagroup_template_unset_field(dg_name, f_name, source=SOURCE_LOCAL, tenant=None, user=None):

	mtime = time.time()

	if tenant:
		if user: user.assert_capability(tenant, 'core.template.dg.edit')

	tdg = TDataGroup.objects.get(name=dg_name, source=source, tenant=tenant)

	tf = TField.objects.get(fclass=TField.FIELD_DATAGROUP, name=f_name)

	try:
		tdgfv = TDataGroupFieldValue.objects.get(tdatagroup=tdg, tfield=tf, tenant=tenant)
	except TDataGroupFieldValue.DoesNotExist, e:
		if DEBUG: print "value not set"
		return

	tdgfv.delete()

	#
	# Propagate
	#
	fv = tdg.get_configured_field_or_default(tf.name)

	_update_configured_datagroup_field_value            (tdg, fv, mtime)
	_propagate_configured_datagroup_field_value         (tdg, fv, mtime)
	_update_datagroup_in_element_template_field_value   (tdg, fv, mtime)
	_propagate_datagroup_in_element_template_field_value(tdg, fv, mtime)

	return 

#------------------------------------------------------------------------------#

def _update_configured_datagroup_field_value(tdg, fv, mtime):

	tenant = tdg.tenant
	if tenant != None:
		db_set_current_tenant_schema(tenant.name)

	if DEBUG: print "Refreshing related to ", tdg

	cdgs = CDataGroup.objects.filter(tenant=tenant, tdatagroup=tdg, tedatagroup=None).all()

	for cdg in cdgs:
		if DEBUG: print "\t", "cdatagroup", cdg

		cdfv = cdg.get_local_configured_field_value(fv.tfield)
		if cdfv:
			continue

		if DEBUG: print "\t", "\t", "Updating real datagroup"
		for dg in cdg.datagroup_set.all():
			dg.update_single_field_and_update(fv, mtime=mtime)

	return

def _propagate_configured_datagroup_field_value(tdg, tdgfv, mtime):

	childrens = tdg.get_childrens()
	for c in childrens:

		if DEBUG: print "Checking", c
		# Is configured inside this template? 
		local_config = c.get_local_configured_field_value(tdgfv.tfield)
		if local_config:
			if DEBUG: print "\t", "local override. skip"
			continue

		_update_configured_datagroup_field_value(c, tdgfv, mtime)
		_propagate_configured_datagroup_field_value(c, tdgfv, mtime)

	return
	
#------------------------------------------------------------------------------#

def _update_datagroup_in_element_template_field_value(tdg, fv, mtime):

	if DEBUG: print "Updating datagroups in elemente templats"
		
	for tedg in TEDataGroup.objects.filter(tdatagroup=tdg).all():

		if DEBUG: print "\t", "Checking", tedg
		
		local_value = tedg.get_local_configured_field_value(fv.tfield)
		if DEBUG: print "\t", "\t", "Local value", 
		if local_value:
			if DEBUG: print "\t", "\t", "locally changed. Skip", tedg.telement
			continue

		_update_single_datagroup_in_element_template_field_value(tedg, fv, mtime)

	return


def _update_single_datagroup_in_element_template_field_value(tedg, fv, mtime):
			
	tenant = tedg.tenant
	db_set_current_tenant_schema(tenant.name)

	cdgs = CDataGroup.objects.filter(tenant=tenant, tedatagroup=tedg, tdatagroup=None).all()
	for cdg in cdgs:
		if DEBUG: print "\t", "cdatagroup", cdg
	
		cdfv = cdg.get_local_configured_field_value(fv.tfield)
		if cdfv:
			continue
						
		for dg in cdg.datagroup_set.all():
			if DEBUG: print "\t", "\t", "Updating real datagroup", dg
			dg.update_single_field_and_update(fv, mtime=mtime)
			
	return

def _propagate_datagroup_in_element_template_field_value(tdg, fv, mtime):

	childrens = tdg.get_childrens()
	for c in childrens:

		if DEBUG: print "Checking", c
		# Is configured inside this template? 
		local_config = c.get_local_configured_field_value(fv.tfield)
		if local_config:
			if DEBUG: print "\t", "local override. skip"
			continue

		_update_datagroup_in_element_template_field_value(c, fv, mtime)
		_propagate_datagroup_in_element_template_field_value(c, fv, mtime)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def datagroup_template_update_parameter(dg_name, param_name, param_type, param_str_val, source=SOURCE_LOCAL, tenant=None, user=None):

	mtime = time.time()

	tdg = TDataGroup.objects.get(name=dg_name, source=source, tenant=tenant)


	if tenant:
		if user:
			user.assert_capability(tenant, 'core.template.dg.edit')
			tdg.assert_permission_readwrite(user)

	#
	# Get all configured params (from anchestors)
	#
	configured_params = tdg.get_configured_params_dict(filter_names=[param_name,])
	param = configured_params.get( param_name, None )
	if param:
		if param.vtype != param_type:
			raise ValueError("Can not change the param type (from type %s to %s)" % ( param.vtype, param_type))

	#
	# Create local param configuration
	#
	creating = False
	try:
		tdgp = TDataGroupParameterValue.objects.get(tdatagroup=tdg, name=param_name, tenant=tenant)
	except TDataGroupParameterValue.DoesNotExist, e:
		tdgp = None
		
	if tdgp == None:
		tdgp = TDataGroupParameterValue(name=param_name, tdatagroup = tdg, vtype=param_type, tenant=tenant)
		tdgp.save()
		creating = True
		
	if not creating:
		if tdgp.vtype != param_type:
			raise Exception("Can not update parameter type")

	param_val2 = convert_to_param_type(param_type, param_str_val)
	
	if tdgp.value == param_str_val:
		return
	
	tdgp.value = param_str_val
	tdgp.concurrent_save(['value'], new_sentinel_value=mtime)

	#
	# Propagate
	#
	_update_configured_datagroup_parameter_value(tdg, tdgp, mtime=mtime)
	_propagate_configured_datagroup_parameter_value(tdg, tdgp, mtime=mtime)	
	_update_datagroup_in_element_template_parameter_value(tdg, tdgp, mtime=mtime)
	_propagate_datagroup_in_element_template_parameter_value(tdg, tdgp, mtime=mtime)
	
	return 
	
#------------------------------------------------------------------------------#

	
def _update_configured_datagroup_parameter_value(tdg, tdgpv, mtime):

	tenant = tdg.tenant
	if tenant != None:
		db_set_current_tenant_schema(tenant.name)

	if DEBUG: print "Refreshing related to ", tdg

	cdgs = CDataGroup.objects.filter(tenant=tenant, tdatagroup=tdg, tedatagroup=None).all()
	
	for cdg in cdgs:
		if DEBUG: print "\t", "cdatagroup", cdg
		
		if tenant:
			db_set_current_tenant_schema(tenant.name)
		
		v = cdg.get_local_configured_parameter_value(tdgpv.name)
		if v:
			continue
		
		for dg in cdg.datagroup_set.all():
			if DEBUG: print "\t", "\t", "Updating real datagroup", dg
			dg.update_single_parameter_and_save(tdgpv, mtime=mtime)



def _propagate_configured_datagroup_parameter_value(tdg, tdgpv, mtime):

	childrens = tdg.get_childrens()
	for c in childrens:

		if DEBUG: print "Checking", c

		# Is configured inside this template? 
		local_config = c.get_local_configured_parameter_value(tdgpv.name)
		if local_config:
			if DEBUG: print "\t", "local override. skip"
			continue

		_update_configured_datagroup_parameter_value(c, tdgpv, mtime)		
		_propagate_configured_datagroup_parameter_value(c, tdgpv, mtime)

	return

#------------------------------------------------------------------------------#

def _update_datagroup_in_element_template_parameter_value(tdg, pv, mtime):
	
	if DEBUG: print "Updating datagroups in elemente templats"
		
	for tedg in TEDataGroup.objects.filter(tdatagroup=tdg).all():

		if DEBUG: print "\t", "Checking", tedg
		
		local_value = tedg.get_local_configured_parameter_value(pv.name)
		if local_value:
			if DEBUG: print "\t", "\t", "locally changed. Skip", tedg.telement
			continue

		_update_single_datagroup_in_element_template_parameter_value(tedg, pv, mtime)

	return


def _update_single_datagroup_in_element_template_parameter_value(tedg, pv, mtime):

	tenant = tedg.tenant
	db_set_current_tenant_schema(tenant.name)

	cdgs = CDataGroup.objects.filter(tenant=tenant, tedatagroup=tedg, tdatagroup=None).all()

	for cdg in cdgs:
		if DEBUG: print "\t", "cdatagroup", cdg

		v = cdg.get_local_configured_parameter_value(pv.name)
		if v:
			continue

		for dg in cdg.datagroup_set.all():
			if DEBUG: print "\t", "\t", "Updating real datagroup", dg
			dg.update_single_parameter_and_save(pv, mtime=mtime)

	return

def _propagate_datagroup_in_element_template_parameter_value(tdg, tdgpv, mtime):

	childrens = tdg.get_childrens()
	for c in childrens:

		if DEBUG: print "Checking", c
		# Is configured inside this template? 
		local_config = c.get_local_configured_parameter_value(tdgpv.name)
		if local_config:
			if DEBUG: print "\t", "local override. skip"
			continue
		
		_update_datagroup_in_element_template_parameter_value(c, tdgpv, mtime=mtime)	
		_propagate_datagroup_in_element_template_parameter_value(c, tdgpv, mtime=mtime)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def datagroup_template_remove_parameter(dg_name, param_name, source=SOURCE_LOCAL, tenant=None, user=None):
	
	mtime = time.time()

	tdg = TDataGroup.objects.get(name=dg_name, source=source, tenant=tenant)

	if tenant:
		if user:
			user.assert_capability(tenant, 'core.template.dg.edit')
			tdg.assert_permission_readwrite(user)

	try:
		tdgp = TDataGroupParameterValue.objects.get(tdatagroup=tdg, name=param_name, tenant=tenant)
	except TDataGroupParameterValue.DoesNotExist, e:
		tdgp = None
	
	if tdgp is None:
		return

	# FIX: unbound related parameters in other templates
	tdgp.delete()
	
	#
	# FIX: too expencive, 
	#
	_refresh_datagroup_template_tree(tdg, mtime)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def datasource_template_get(datagroup_name, datasource_name, source=SOURCE_LOCAL, tenant=None, user=None):

	try:
		tds = TDataSource.objects.get(name=datasource_name, tdatagroup__name=datagroup_name, tdatagroup__source=source, tenant=tenant)
		return tds
	except TDataSource.DoesNotExist, e:
		return None

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def datasource_template_create(datagroup_name, datasource_name, source=SOURCE_LOCAL, tenant=None, user=None):

	mtime = time.time()

	tdg = TDataGroup.objects.get(name=datagroup_name, source=source, tenant=tenant)

	if tenant:
		if user: 
			user.assert_capability(tenant, 'core.template.dg.edit')
			tdg.assert_permission_readwrite(user)

	tds = TDataSource(name=datasource_name, tdatagroup = tdg, tenant=tenant)

	tds.mtime = time.time()
	tds.save()

	_refresh_datagroup_template_tree(tdg, mtime=mtime)

	return tds

#------------------------------------------------------------------------------#

@transaction_commit_on_success
def datasource_template_remove(datagroup_name, datasource_name, source=SOURCE_LOCAL, tenant=None, user=None):

	mtime = time.time()

	tds = TDataSource.objects.get(
		tdatagroup__name=datagroup_name, 
		tdatagroup__source=source, 
		name=datasource_name, 
		tenant=tenant,
	)

	if tenant:
		if user:
			user.assert_capability(tenant, 'core.template.dg.edit')
			tds.tdatagroup.assert_permission_readwrite(user)

	#
	# Se ci sono delle TCondition configurate in datagroup padre dobbiamo
	# aggiornare le foreigh-key delle entita' collegate a "tc" sostituend
	# "tc" con la TCondition "piu' vicina'" in configurazione,
	# altrimenti il tc.delete() cancellera tutte le configurazioni collegate a "tc".
	#
	
	anchestor = tds.get_ancestors_tree(include_self=False)
	if len(anchestor) > 0:

		closest_anchestor = anchestor[-1]
		
		for tedsfv in TEDataSourceFieldValue.objects.filter(tdatasource=tds).all():
			tedsfv.tdatasource = closest_anchestor
			tedsfv.save()
		
		for cdsfv in CDataSourceFieldValue.objects.filter(tdatasource=tds).all():
			cdsfv.tdatasource = closest_anchestor
			cdsfv.save()

	tds.delete()
	
	# FIX: expensive
	tdg = tds.tdatagroup
	
	_refresh_datagroup_template_tree(tdg, mtime)
	
	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def datasource_template_update_field(dg_name, ds_name, f_name, f_value, source=SOURCE_LOCAL, tenant=None, user=None):

	mtime = time.time()

	tdg = TDataGroup.objects.get(tenant=tenant, source=source, name=dg_name)

	if tenant:
		if user:
			user.assert_capability(tenant, 'core.template.dg.edit')
			tdg.assert_permission_readwrite(user)

	# FIX: usare tdatagroup__name
	tds = TDataSource.objects.get(name=ds_name, tdatagroup__name=dg_name, tdatagroup__source=source, tenant=tenant)

	tf  = TField.objects.get(fclass=TField.FIELD_DATASOURCE, name=f_name)

	#
	# Validate
	#
	tf.validate_value(f_value)

	created = False
	try:
		tdsfv = TDataSourceFieldValue.objects.get(tdatasource=tds, tfield=tf, tenant=tenant)
	except TDataSourceFieldValue.DoesNotExist, e:
		tdsfv = TDataSourceFieldValue(tdatasource=tds, tfield=tf, tenant=tenant)
		tdsfv.save()

	if tdsfv.value == f_value:
		return

	#tdsfv.tenant = tenant
	tdsfv.value  = f_value
	#tdsfv.mtime  = time.time()
	#tdsfv.save()
	tdsfv.concurrent_save(['value'], new_sentinel_value=mtime)

	#
	# Propagate
	#
	tdg = tds.tdatagroup

	_update_configured_datasource_field_value            (tdg, tds.name, tdsfv, mtime)
	_propagate_configured_datasource_field_value         (tdg, tds.name, tdsfv, mtime)
	_update_datasource_in_element_template_field_value   (tdg, tds.name, tdsfv, mtime)
	_propagate_datasource_in_element_template_field_value(tdg, tds.name, tdsfv, mtime)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def datasource_template_unset_field(dg_name, ds_name, f_name, source=SOURCE_LOCAL, tenant=None, user=None):

	mtime = time.time()

	tdg = TDataGroup.objects.get(tenant=tenant, name=dg_name, source=source)

	if tenant:
		if user:
			user.assert_capability(tenant, 'core.template.dg.edit')
			tdg.assert_permission_readwrite(user)

	# FIX: usare tdatagroup__name
	tds = TDataSource.objects.get(name=ds_name, tdatagroup__name=dg_name, tdatagroup__source=source, tenant=tenant)

	tf  = TField.objects.get(fclass=TField.FIELD_DATASOURCE, name=f_name)

	try:
		tdsfv = TDataSourceFieldValue.objects.get(tdatasource=tds, tfield=tf, tenant=tenant)
	except TDataSourceFieldValue.DoesNotExist, e:
		return

	tdsfv.delete()

	#
	# Propagate
	#
	tdg = tds.tdatagroup

	fv = tdg.get_configured_datasource_field_value_or_default(ds_name, f_name)

	_update_configured_datasource_field_value            (tdg, ds_name, fv, mtime)
	_propagate_configured_datasource_field_value         (tdg, ds_name, fv, mtime)
	_update_datasource_in_element_template_field_value   (tdg, ds_name, fv, mtime)
	_propagate_datasource_in_element_template_field_value(tdg, ds_name, fv, mtime)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def _update_configured_datasource_field_value(tdg, ds_name, fv, mtime):


	tenant = tdg.tenant
	if tenant != None:
		db_set_current_tenant_schema(tenant.name)

	if DEBUG: print "Refreshing related to ", tdg
	cdgs = CDataGroup.objects.filter(tenant=tenant, tdatagroup=tdg, tedatagroup=None).all()

	for cdg in cdgs:

		if DEBUG: print "\t", cdg, ds_name

		v = cdg.get_local_configured_datasource_field_value(ds_name, fv.tfield)
		if v:
			if DEBUG: print "\t", "local override. skip"
			continue

		for dg in cdg.datagroup_set.all():
			ds = dg.datasources.get(name=ds_name)
			ds.update_single_field_and_save(fv, mtime)
			
	return

def _propagate_configured_datasource_field_value(tdg, ds_name, tdsfv, mtime):

	childrens = tdg.get_childrens()
	for c in childrens:
		
		# Is the field configured inside this template? 
		v = c.get_local_configured_datasource_field_value(ds_name, tdsfv.tfield)
		if v:
			if DEBUG: print "\t", "local override. skip"
			continue

		_update_configured_datasource_field_value(c, ds_name, tdsfv, mtime)
		_propagate_configured_datasource_field_value(c, ds_name, tdsfv, mtime)
		
	return
	
#------------------------------------------------------------------------------#

def _update_datasource_in_element_template_field_value(tdg, ds_name, fv, mtime):

	if DEBUG: print "Updating datagroups in elemente templats"
		
	for tedg in TEDataGroup.objects.filter(tdatagroup=tdg).all():

		if DEBUG: print "\t", "Checking", tedg
		
		v = tedg.get_local_configured_datasource_field_value(ds_name, fv.tfield)
		if v:
			if DEBUG: print "\t", "\t", "locally changed. Skip", tedg.telement
			continue

		_update_single_datasource_in_element_template_field_value(tedg, ds_name, fv, mtime)

	return

def _update_single_datasource_in_element_template_field_value(tedg, ds_name, fv, mtime):

	tenant = tedg.tenant
	db_set_current_tenant_schema(tenant.name)

	cdgs = CDataGroup.objects.filter(tenant=tenant, tedatagroup=tedg, tdatagroup=None).all()

	for cdg in cdgs:
		if DEBUG: print "\t", "cdatagroup", cdg

		v = cdg.get_local_configured_datasource_field_value(ds_name, fv.tfield)
		if v:
			if DEBUG: print "\t", "local override. skip"
			continue

		for dg in cdg.datagroup_set.all():
			ds = dg.datasources.get(name=ds_name)
			ds.update_single_field_and_save(fv, mtime=mtime)
	
	return

def _propagate_datasource_in_element_template_field_value(tdg, ds_name, fv, mtime):

	childrens = tdg.get_childrens()
	for c in childrens:

		if DEBUG: print "Checking", c

		# Is configured inside this template? 
		v = c.get_local_configured_datasource_field_value(ds_name, fv.tfield)
		if v:
			if DEBUG: print "\t", "local override. skip"
			continue

		_update_datasource_in_element_template_field_value(c, ds_name, fv, mtime)
		_propagate_datasource_in_element_template_field_value(c, ds_name, fv, mtime)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def condition_template_get(datagroup_name, condition_name, source=SOURCE_LOCAL, tenant=None):

	try:
		tc = TCondition.objects.get(name=condition_name, tdatagroup__name=datagroup_name, tdatagroup__source=source, tenant=tenant)
		return tc
	except TCondition.DoesNotExist, e:
		return None

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def condition_template_create(datagroup_name, condition_name, source=SOURCE_LOCAL, tenant=None, user=None):

	mtime = time.time()


	tdg = TDataGroup.objects.get(name=datagroup_name, source=source, tenant=tenant)

	if tenant:
		if user:
			user.assert_capability(tenant, 'core.template.dg.edit')
			tdg.assert_permission_readwrite(user)

	tc  = TCondition(tdatagroup=tdg, name=condition_name, tenant=tenant )
	tc.mtime = time.time()
	tc.save()

	_refresh_datagroup_template_tree(tdg, mtime)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def condition_template_remove(datagroup_name, condition_name, source=SOURCE_LOCAL, tenant=None, user=None):

	mtime = time.time()

	tdg = TDataGroup.objects.get(tenant=tenant, source=source, name=datagroup_name)



	tc  = TCondition.objects.get(
		tdatagroup__name = datagroup_name, 
		name             = condition_name, 
		tenant           = tenant,
	)

	if tenant:
		if user:
			user.assert_capability(tenant, 'core.template.dg.edit')
			tc.tdatagroup.assert_permission_readwrite(user)

	#
	# Se ci sono delle TCondition configurate in datagroup padre dobbiamo
	# aggiornare le foreigh-key delle entita' collegate a "tc" sostituend
	# "tc" con la TCondition "piu' vicina'" in configurazione,
	# altrimenti il tc.delete() cancellera tutte le configurazioni collegate a "tc".
	#

	anchestor = tc.get_ancestors_tree(include_self=False)
	if len(anchestor) > 0:

		closest_anchestor = anchestor[-1]

		for tecfv in TEConditionFieldValue.objects.filter(tcondition=tc).all():
			tecfv.tcondition = closest_anchestor
			tecfv.save()

		for ccfv in CConditionFieldValue.objects.filter(tcondition=tc).all():
			ccfv.tcondition = closest_anchestor
			ccfv.save()

	tc.delete()

	tdg = tc.tdatagroup
	_refresh_datagroup_template_tree(tdg, mtime)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def condition_template_update_field(dg_name, c_name, f_name, f_value, source=SOURCE_LOCAL, tenant=None, user=None):

	mtime = time.time()


	tdg = TDataGroup.objects.get(tenant=tenant, source=source, name=dg_name)

	if tenant:
		if user:
			user.assert_capability(tenant, 'core.template.dg.edit')
			tdg.assert_permission_readwrite(user)

	# FIX: usare tdatagroup__name
	tc = TCondition.objects.get(name=c_name, 
		#tdatagroup__name   = dg_name,
		#tdatagroup__source = source,
		tdatagroup=tdg,
		tenant=tenant,
	)

	tf  = TField.objects.get(fclass=TField.FIELD_CONDITION, name=f_name)

	tf.validate_value(f_value)

	created = False
	try:
		tcfv = TConditionFieldValue.objects.get(tcondition=tc, tfield=tf, tenant=tenant)
	except TConditionFieldValue.DoesNotExist, e:
		tcfv = TConditionFieldValue(tcondition=tc, tfield=tf, tenant=tenant)
		tcfv.save()
		created = True

	if tcfv.value == f_value:
		return

	#tcfv.tenant = tenant
	tcfv.value  = f_value
	#tcfv.mtime  = time.time()
	#tcfv.save()
	tcfv.concurrent_save(['value'], new_sentinel_value=mtime)
	
	#
	# Propagate
	#
	tdg = tc.tdatagroup

	_update_configured_condition_field_value            (tdg, tc.name, tcfv, mtime)
	_propagate_configured_condition_field_value         (tdg, tc.name, tcfv, mtime)
	_update_condition_in_element_template_field_value   (tdg, tc.name, tcfv, mtime)
	_propagate_condition_in_element_template_field_value(tdg, tc.name, tcfv, mtime)

	return
	
#------------------------------------------------------------------------------#


@transaction_commit_on_success
def condition_template_unset_field(dg_name, c_name, f_name, source=SOURCE_LOCAL, tenant=None, user=None):

	mtime = time.time()

	tdg = TDataGroup.objects.get(tenant=tenant, source=source, name=dg_name)
	if tenant:
		if user:
			user.assert_capability(tenant, 'core.template.dg.edit')
			tdg.assert_permission_readwrite(user)


	# FIX: usare tdatagroup__name
	tc = TCondition.objects.get(name=c_name, 
		tdatagroup__name   = dg_name,
		tdatagroup__source = source,
		tenant=tenant,
	)


	tf  = TField.objects.get(fclass=TField.FIELD_CONDITION, name=f_name)

	try:
		tcfv = TConditionFieldValue.objects.get(tcondition=tc, tfield=tf, tenant=tenant)
	except TConditionFieldValue.DoesNotExist, e:
		if DEBUG: print "%s.%s.%s not set" % (dg_name, c_name, f_name)
		return

	tcfv.delete()

	#
	# Propagate
	#
	tdg = tc.tdatagroup

	fv = tdg.get_configured_condition_field_value_or_default(c_name, f_name)

	_update_configured_condition_field_value            (tdg, tc.name, fv, mtime)
	_propagate_configured_condition_field_value         (tdg, tc.name, fv, mtime)
	_update_condition_in_element_template_field_value   (tdg, tc.name, fv, mtime)
	_propagate_condition_in_element_template_field_value(tdg, tc.name, fv, mtime)

	return

#-----------------------------------------------------------------------------#

def _update_configured_condition_field_value(tdg, c_name, fv, mtime):

	tenant = tdg.tenant
	if tenant != None:
		db_set_current_tenant_schema(tenant.name)

	if DEBUG: print "Refreshing related to ", tdg
	cdgs = CDataGroup.objects.filter(tenant=tenant, tdatagroup=tdg, tedatagroup=None).all()

	for cdg in cdgs:

		if DEBUG: print "\t", cdg, c_name
		
		# Alcuni  campi sono "sovrascritti" da configurazioni nell'elemento e il valore
		# non va propagato:
		if fv.name == 'priority':
			
			# L'elemento o il nodo padre definisce la priority quindi questa configurazione non va propagata
			if cdg.element.get_effective_priority() != PRIORITY_NONE: 
				continue
		

		v = cdg.get_local_configured_condition_field_value(c_name, fv.tfield)
		if v:
			if DEBUG: print "\t", "local override. skip"
			continue

		for dg in cdg.datagroup_set.all():
			c = dg.conditions.get(name=c_name)
			changed = c.update_single_field_and_save(fv, mtime)
			if changed:
				if fv.name == 'primary':
					for dg in DataGroup.objects.filter(tenant=tenant, dependson=c):
						dg.update_from_config(mtime=mtime)

def _propagate_configured_condition_field_value(tdg, c_name, tcfv, mtime):

	childrens = tdg.get_childrens()
	for c in childrens:

		# Is the field configured inside this template? 
		v = c.get_local_configured_condition_field_value(c_name, tcfv.tfield)
		if v:
			if DEBUG: print "\t", "local override. skip"
			continue

		_update_configured_condition_field_value(c, c_name, tcfv, mtime)
		_propagate_configured_condition_field_value(c, c_name, tcfv, mtime)

	return

#------------------------------------------------------------------------------#

def _update_condition_in_element_template_field_value(tdg, c_name, fv, mtime):

	if DEBUG: print "Updating datagroups in elemente templats"
		
	for tedg in TEDataGroup.objects.filter(tdatagroup=tdg).all():

		if DEBUG: print "\t", "Checking", tedg
		
		v = tedg.get_local_configured_condition_field_value(c_name, fv.tfield)
		if v:
			if DEBUG: print "\t", "\t", "locally changed. Skip", tedg.telement
			continue

		_update_single_condition_in_element_template_field_value(tedg, c_name, fv, mtime)

	return

def _update_single_condition_in_element_template_field_value(tedg, c_name, fv, mtime):

	tenant = tedg.tenant
	db_set_current_tenant_schema(tenant.name)

	cdgs = CDataGroup.objects.filter(tenant=tenant, tedatagroup=tedg, tdatagroup=None).all()
	for cdg in cdgs:
		if DEBUG: print "\t", "cdatagroup", cdg


		# Alcuni  campi sono "sovrascritti" da configurazioni nell'elemento e il valore
		# non va propagato:
		if fv.name == 'priority':
			
			# L'elemento o il nodo padre definisce la priority quindi questa configurazione non va propagata
			if cdg.element.get_effective_priority() != PRIORITY_NONE: 
				continue


		v = cdg.get_local_configured_condition_field_value(c_name, fv.tfield)
		if v:
			if DEBUG: print "\t", "local override. skip"
			continue

		for dg in cdg.datagroup_set.all():
			c = dg.conditions.get(name=c_name)
			changed = c.update_single_field_and_save(fv, mtime=mtime)
			if changed:
				if fv.name == 'primary':
					for dg in DataGroup.objects.filter(tenant=tenant, dependson=c):
						dg.update_from_config(mtime=mtime)
					
	
	return

def _propagate_condition_in_element_template_field_value(tdg, c_name, fv, mtime):

	childrens = tdg.get_childrens()
	for c in childrens:

		if DEBUG: print "Checking", c
		
		# Is configured inside this template? 
		v = c.get_local_configured_condition_field_value(c_name, fv.tfield)
		if v:
			if DEBUG: print "\t", "local override. skip"
			continue
		
		_update_condition_in_element_template_field_value(c, c_name, fv, mtime)
		_propagate_condition_in_element_template_field_value(c, c_name, fv, mtime)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def timegraph_template_create(tenant, tdg, name, user=None):

	mtime = time.time()

	if tenant:
		if user:
			user.assert_capability(tenant, 'core.template.dg.edit')
			tdg.assert_permission_readwrite(user)


	try:
		ttg = TTimeGraph.objects.get(tenant=tenant, tdatagroup=tdg, name=name)
	except TTimeGraph.DoesNotExist, e:
		ttg = None

	if ttg == None:
		ttg  = TTimeGraph(tenant=tenant, tdatagroup=tdg, name=name)
		ttg.mtime = time.time()
		ttg.save()

		_refresh_datagroup_template_tree(tdg, mtime=mtime)

	return ttg


@transaction_commit_on_success
def timegraph_template_remove(datagroup_name, timegraph_name, source=SOURCE_LOCAL, tenant=None, user=None):

	mtime = time.time()

	ttg  = TTimeGraph.objects.get(
		tdatagroup__name=datagroup_name, 
		tdatagroup__source=source,
		name=timegraph_name, 
		tenant=tenant,
		#source=source,
	)

	if tenant:
		if user:
			user.assert_capability(tenant, 'core.template.dg.edit')
			ttg.tdatagroup.assert_permission_readwrite(user)
	
	#
	# Se ci sono delle TCondition configurate in datagroup padre dobbiamo
	# aggiornare le foreigh-key delle entita' collegate a "tc" sostituend
	# "tc" con la TCondition "piu' vicina'" in configurazione,
	# altrimenti il tc.delete() cancellera tutte le configurazioni collegate a "tc".
	#
	
	anchestor = ttg.get_ancestors_tree(include_self=False)
	if len(anchestor) > 0:

		closest_anchestor = anchestor[-1]
		
		for tetgfv in TETimeGraphFieldValue.objects.filter(ttimegraph=ttg).all():
			tetgfv.ttimegraph = closest_anchestor
			tetgfv.save()
		
		for ctgfv in CTimeGraphFieldValue.objects.filter(ttimegraph=ttg).all():
			ctgfv.ttimegraph = closest_anchestor
			ctgfv.save()	
	
	
	ttg.delete()
	
	tdg = ttg.tdatagroup
	_refresh_datagroup_template_tree(tdg, mtime)


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def timegraph_template_field_update(
		datagroup_name,
		timegraph_name,
		field_name,
		field_value,
		tenant=None,
		source=None,
		user=None
):
	mtime = time.time()

	if tenant:
		db_set_current_tenant_schema(tenant.name)

	tdg = TDataGroup.objects.get(tenant=tenant, name=datagroup_name, source=source)

	if tenant:
		if user:
			user.assert_capability(tenant, 'core.template.dg.edit')
			tdg.assert_permission_readwrite(user)


	ttg = TTimeGraph.objects.get(tenant=tenant, tdatagroup=tdg, name=timegraph_name)

	tf  = TField.objects.get(fclass=TField.FIELD_TGRAPH, name=field_name)

	tf.validate_value(field_value)

	created = False
	try:
		ttgfv = TTimeGraphFieldValue.objects.get(tenant=tenant, ttimegraph=ttg, tfield=tf)
	except TTimeGraphFieldValue.DoesNotExist, e:
		ttgfv = TTimeGraphFieldValue(tenant=tenant, ttimegraph=ttg, tfield=tf)
		created = True

	if ttgfv.value == field_value:
		return

	ttgfv.value  = field_value
	ttgfv.mtime = time.time()
	ttgfv.save()

	#
	# Propagate
	#
	_update_configured_timegraph_field_value            (tdg, ttg.name, ttgfv, mtime)
	_propagate_configured_timegraph_field_value         (tdg, ttg.name, ttgfv, mtime)
	_update_timegraph_in_element_template_field_value   (tdg, ttg.name, ttgfv, mtime)
	_propagate_timegraph_in_element_template_field_value(tdg, ttg.name, ttgfv, mtime)

	return

#------------------------------------------------------------------------------#

@transaction_commit_on_success
def timegraph_template_field_unset(
		datagroup_name,
		timegraph_name,
		field_name,
		tenant=None,
		source=None,
		user=None,
):
	mtime = time.time()

	if tenant:
		db_set_current_tenant_schema(tenant.name)

	tdg = TDataGroup.objects.get(tenant=tenant, name=datagroup_name, source=source)

	if tenant:
		if user:
			user.assert_capability(tenant, 'core.template.dg.edit')
			tdg.assert_permission_readwrite(user)

	tdg = TDataGroup.objects.get(tenant=tenant, name=datagroup_name, source=source)
	ttg = TTimeGraph.objects.get(tenant=tenant, tdatagroup=tdg, name=timegraph_name)

	tf  = TField.objects.get(fclass=TField.FIELD_TGRAPH, name=field_name)

	try:
		ttgfv = TTimeGraphFieldValue.objects.get(tenant=tenant, ttimegraph=ttg, tfield=tf)
	except TTimeGraphFieldValue.DoesNotExist, e:
		#not configured, return
		return

	ttgfv.delete()

	#
	# Propagate
	#
	fv = tdg.get_configured_timegraph_field_value_or_default(timegraph_name, field_name)

	_update_configured_timegraph_field_value             (tdg, ttg.name, fv, mtime)
	_propagate_configured_timegraph_field_value          (tdg, ttg.name, fv, mtime)
	_update_timegraph_in_element_template_field_value   (tdg, ttg.name, fv, mtime)
	_propagate_timegraph_in_element_template_field_value(tdg, ttg.name, fv, mtime)

	return

#-----------------------------------------------------------------------------#

def _update_configured_timegraph_field_value(tdg, tg_name, fv, mtime):

	tenant = tdg.tenant
	if tenant != None:
		db_set_current_tenant_schema(tenant.name)

	if DEBUG: print "Refreshing configured timegraph related to ", tdg
	cdgs = CDataGroup.objects.filter(tenant=tenant, tdatagroup=tdg, tedatagroup=None).all()

	for cdg in cdgs:

		if DEBUG: print "\t", cdg, tg_name

		v = cdg.get_local_configured_timegraph_field_value(tg_name, fv.tfield)
		
		if v:
			if DEBUG: print "\t", "local override. skip"
			continue

		for dg in cdg.datagroup_set.all():
			
			tg = dg.timegraphs.get(name=tg_name)
			
			tg.update_single_field_and_save(fv, mtime)
	return

def _propagate_configured_timegraph_field_value(tdg, tg_name, ttgfv, mtime):

	if DEBUG: print "Refreshing children datagroup templates of ", tdg

	childrens = tdg.get_childrens()
	
	for c in childrens:
		
		# Is the field configured inside this template? 
		v = c.get_local_configured_timegraph_field_value(tg_name, ttgfv.tfield)
		if v:
			if DEBUG: print "\t", "local override. skip"
			continue
		
		_update_configured_timegraph_field_value(c, tg_name, ttgfv, mtime)
		_propagate_configured_timegraph_field_value(c, tg_name, ttgfv, mtime)

	return
	
#------------------------------------------------------------------------------#

def _update_timegraph_in_element_template_field_value(tdg, c_name, fv, mtime):

	if DEBUG: print "Updating datagroups in elemente templats"
		
	for tedg in TEDataGroup.objects.filter(tdatagroup=tdg).all():

		if DEBUG: print "\t", "Checking", tedg
		
		v = tedg.get_local_configured_timegraph_field_value(c_name, fv.tfield)
		if v:
			if DEBUG: print "\t", "\t", "locally changed. Skip", tedg.telement
			continue

		_update_single_timegraph_in_element_template_field_value(tedg, c_name, fv, mtime)

	return
	
	
def _update_single_timegraph_in_element_template_field_value(tedg, tg_name, fv, mtime):

	tenant = tedg.tenant
	db_set_current_tenant_schema(tenant.name)

	cdgs = CDataGroup.objects.filter(tenant=tenant, tedatagroup=tedg, tdatagroup=None).all()
	for cdg in cdgs:
		if DEBUG: print "\t", "cdatagroup", cdg

		v = cdg.get_local_configured_timegraph_field_value(tg_name, fv.tfield)
		if v:
			if DEBUG: print "\t", "local override. skip"
			continue

		for dg in cdg.datagroup_set.all():
			tg = dg.timegraphs.get(name=tg_name)
			tg.update_single_field_and_save(fv, mtime=mtime)
	
	return



def _propagate_timegraph_in_element_template_field_value(tdg, c_name, fv, mtime):

	childrens = tdg.get_childrens()
	for c in childrens:

		if DEBUG: print "Checking", c
		
		# Is configured inside this template? 
		v = c.get_local_configured_timegraph_field_value(c_name, fv.tfield)
		if v:
			if DEBUG: print "\t", "local override. skip"
			continue
		
		_update_timegraph_in_element_template_field_value(c, c_name, fv, mtime)
		_propagate_timegraph_in_element_template_field_value(c, c_name, fv, mtime)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def timegraph_serie_create(
		datagroup_name,
		timegraph_name,
		name,
		source=None,
		tenant=None,
		user=None,
):

	mtime = time.time()

	tdg = TDataGroup.objects.get(tenant=tenant, name=datagroup_name, source=source)

	if tenant:
		if user:
			user.assert_capability(tenant, 'core.template.dg.edit')
			tdg.assert_permission_readwrite(user)

	ttg = TTimeGraph.objects.get(tenant=tenant, tdatagroup=tdg, name=timegraph_name)

	try:
		ttgs = TTimeGraphSerie.objects.get(tenant=tenant, ttimegraph=ttg, name=name)
	except TTimeGraphSerie.DoesNotExist, e:
		ttgs = None

	if ttgs == None:
		ttgs  = TTimeGraphSerie(tenant=tenant, ttimegraph=ttg, name=name)
		ttgs.save()

		_refresh_datagroup_template_tree(tdg, mtime=mtime)

	return ttgs

#------------------------------------------------------------------------------#

@transaction_commit_on_success
def timegraph_serie_remove(
		datagroup_name,
		timegraph_name,
		name,
		source=None,
		tenant=None,
		user=None,
):
	mtime = time.time()

	if tenant:
		db_set_current_tenant_schema(tenant.name)

	tdg = TDataGroup.objects.get(tenant=tenant, name=datagroup_name, source=source)

	if tenant:
		if user:
			user.assert_capability(tenant, 'core.template.dg.edit')
			tdg.assert_permission_readwrite(user)

	try:
		ttg = TTimeGraph.objects.get(tenant=tenant, tdatagroup=tdg, name=timegraph_name)
	except TTimeGraph.DoesNotExist, e:
		ttg = None

	try:
		ttgs = TTimeGraphSerie.objects.get(tenant=tenant, ttimegraph=ttg, name=name)
	except TTimeGraphSerie.DoesNotExist, e:
		ttgs = None

	if ttg is None or ttgs is None:
		return

	#
	# Se ci sono delle TCondition configurate in datagroup padre dobbiamo
	# aggiornare le foreigh-key delle entita' collegate a "tc" sostituend
	# "tc" con la TCondition "piu' vicina'" in configurazione,
	# altrimenti il tc.delete() cancellera tutte le configurazioni collegate a "tc".
	#

	anchestor = ttgs.get_ancestors_tree(include_self=False)
	if len(anchestor) > 0:

		closest_anchestor = anchestor[-1]

		for tetgsfv in TETimeGraphSerieFieldValue.objects.filter(ttimegraphserie=ttgs).all():
			tetgsfv.ttimegraphserie = closest_anchestor
			tetgsfv.save()

		for ctgsfv in CTimeGraphSerieFieldValue.objects.filter(ttimegraphserie=ttgs).all():
			ctgsfv.ttimegraphserie = closest_anchestor
			ctgsfv.save()

	ttgs.delete()

	# FIX: expensive
	_refresh_datagroup_template_tree(tdg, mtime=mtime)

#------------------------------------------------------------------------------#

@transaction_commit_on_success
def timegraph_serie_template_update_field(
		datagroup_name, 
		timegraph_name,
		serie_name,
		field_name, 
		field_value, 
		source=None,
		tenant=None,
		user=None
):
	mtime = time.time()

	if tenant:
		db_set_current_tenant_schema(tenant.name)

	tdg  = TDataGroup.objects.get(tenant=tenant, name=datagroup_name, source=source)

	if tenant:
		if user:
			user.assert_capability(tenant, 'core.template.dg.edit')
			tdg.assert_permission_readwrite(user)

	ttg  = TTimeGraph.objects.get(tenant=tenant, tdatagroup=tdg, name=timegraph_name)
	ttgs = TTimeGraphSerie.objects.get(tenant=tenant, ttimegraph=ttg, name=serie_name)

	tf  = TField.objects.get(fclass=TField.FIELD_TSERIE, name=field_name)

	tf.validate_value(field_value)

	created = False
	try:
		ttgsfv = TTimeGraphSerieFieldValue.objects.get(tenant=tenant, ttimegraphserie=ttgs, tfield=tf)
	except TTimeGraphSerieFieldValue.DoesNotExist, e:
		ttgsfv = TTimeGraphSerieFieldValue(tenant=tenant, ttimegraphserie=ttgs, tfield=tf)
		ttgsfv.save()
		created = True

	if ttgsfv.value == field_value:
		return

	ttgsfv.mtime  = time.time()
	ttgsfv.value  = field_value
	ttgsfv.save()

	#
	# VERY EXPENCIVE
	#
	#_refresh_datagroup_template_tree(tdg, mtime=mtime)

	_update_configured_timegraphserie_field_value            (tdg, ttg.name, serie_name, ttgsfv, mtime)
	_propagate_configured_timegraphserie_field_value         (tdg, ttg.name, serie_name, ttgsfv, mtime)
	_update_timegraphserie_in_element_template_field_value   (tdg, ttg.name, serie_name, ttgsfv, mtime)
	_propagate_timegraphserie_in_element_template_field_value(tdg, ttg.name, serie_name, ttgsfv, mtime)

	return

#------------------------------------------------------------------------------#

@transaction_commit_on_success
def timegraph_serie_template_unset_field(
		datagroup_name, 
		timegraph_name,
		serie_name,
		field_name, 
		source=None,
		tenant=None,
		user=None
):
	if tenant:
		db_set_current_tenant_schema(tenant.name)

	tdg  = TDataGroup.objects.get(tenant=tenant, name=datagroup_name, source=source)

	if tenant:
		if user:
			user.assert_capability(tenant, 'core.template.dg.edit')
			tdg.assert_permission_readwrite(user)


	mtime = time.time()

	ttg  = TTimeGraph.objects.get(tenant=tenant, tdatagroup=tdg, name=timegraph_name)
	ttgs = TTimeGraphSerie.objects.get(tenant=tenant, ttimegraph=ttg, name=serie_name)

	tf  = TField.objects.get(fclass=TField.FIELD_TSERIE, name=field_name)

	created = False
	try:
		ttgsfv = TTimeGraphSerieFieldValue.objects.get(tenant=tenant, ttimegraphserie=ttgs, tfield=tf)
	except TTimeGraphSerieFieldValue.DoesNotExist, e:
		if DEBUG: print "%s.%s.%s.%s not set" % (datagroup_name, timegraph_name, serie_name, field_name)
		return

	ttgsfv.delete()

	# FIX: too expencive
	#_refresh_datagroup_template_tree(tdg, mtime)

	fv = tdg.get_configured_timegraphserie_field_value_or_default(timegraph_name, serie_name, field_name)

	_update_configured_timegraphserie_field_value            (tdg, ttg.name, serie_name, fv, mtime)
	_propagate_configured_timegraphserie_field_value         (tdg, ttg.name, serie_name, fv, mtime)
	_update_timegraphserie_in_element_template_field_value   (tdg, ttg.name, serie_name, fv, mtime)
	_propagate_timegraphserie_in_element_template_field_value(tdg, ttg.name, serie_name, fv, mtime)

	return

def _update_configured_timegraphserie_field_value            (tdg, ttg_name, serie_name, ttgsfv, mtime):

	tenant = tdg.tenant
	if tenant != None:
		db_set_current_tenant_schema(tenant.name)

	if DEBUG: print "Refreshing configured timegraphserie related to ", tdg, "with", ttgsfv.get_value(), type(ttgsfv.get_value())
	cdgs = CDataGroup.objects.filter(tenant=tenant, tdatagroup=tdg, tedatagroup=None).all()

	for cdg in cdgs:

		if DEBUG: print "\t", cdg, ttg_name

		v = cdg.get_local_configured_timegraphserie_field_value(ttg_name, serie_name, ttgsfv.tfield)
		if v:
			if DEBUG: print "\t", "local override. skip"
			continue

		for dg in cdg.datagroup_set.all():
			tg = dg.timegraphs.get(name=ttg_name) # FIX: use a single get with join
			tgs = tg.series.get(name=serie_name)
			tgs.update_single_field_and_save(ttgsfv, mtime)


def _propagate_configured_timegraphserie_field_value         (tdg, ttg_name, serie_name, ttgsfv, mtime):

	if DEBUG: print "Refreshing children datagroup templates of ", tdg

	childrens = tdg.get_childrens()

	for c in childrens:

		# Is the field configured inside this template? 
		v = c.get_local_configured_timegraphserie_field_value(ttg_name, serie_name, ttgsfv.tfield)
		if v:
			if DEBUG: print "\t", "local override. skip"
			continue

		_update_configured_timegraphserie_field_value(c, ttg_name, serie_name, ttgsfv, mtime)
		_propagate_configured_timegraphserie_field_value(c, ttg_name, serie_name, ttgsfv, mtime)

	return

def _update_timegraphserie_in_element_template_field_value   (tdg, ttg_name, serie_name, ttgsfv, mtime):

	if DEBUG: print "Updating datagroups in elemente templats"

	for tedg in TEDataGroup.objects.filter(tdatagroup=tdg).all():

		if DEBUG: print "\t", "Checking", tedg

		v = tedg.get_local_configured_timegraphserie_field_value(ttg_name, serie_name, ttgsfv.tfield)
		if v:
			if DEBUG: print "\t", "\t", "locally changed. Skip", tedg.telement
			continue

		_update_single_timegraphserie_in_element_template_field_value(tedg, ttg_name, serie_name, ttgsfv, mtime)

	return

def _update_single_timegraphserie_in_element_template_field_value(tedg, tg_name, serie_name, fv, mtime):

	tenant = tedg.tenant
	db_set_current_tenant_schema(tenant.name)

	#print tenant.name

	cdgs = CDataGroup.objects.filter(tenant=tenant, tedatagroup=tedg, tdatagroup=None).all()
	for cdg in cdgs:
		if DEBUG: print "\t", "cdatagroup", cdg

		v = cdg.get_local_configured_timegraphserie_field_value(tg_name, serie_name, fv.tfield)
		if v:
			if DEBUG: print "\t", "local override. skip"
			continue

		for dg in cdg.datagroup_set.all():
			tg = dg.timegraphs.get(name=tg_name)
			if DEBUG: print tg, serie_name
			#for s in tg.series.all(): print s, s.name
			ts = tg.series.get(name=serie_name)
			ts.update_single_field_and_save(fv, mtime=mtime)

	return

def _propagate_timegraphserie_in_element_template_field_value(tdg, ttg_name, serie_name, ttgsfv, mtime):

	childrens = tdg.get_childrens()
	for c in childrens:

		if DEBUG: print "Checking", c

		# Is configured inside this template? 
		v = c.get_local_configured_timegraphserie_field_value(ttg_name, serie_name, ttgsfv.tfield)
		if v:
			if DEBUG: print "\t", "local override. skip"
			continue

		_update_timegraphserie_in_element_template_field_value(c, ttg_name, serie_name, ttgsfv, mtime)
		_propagate_timegraphserie_in_element_template_field_value(c, ttg_name, serie_name, ttgsfv, mtime)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def element_template_get(kind, name, tenant=None):

	if tenant != None:
		db_set_current_tenant_schema(tenant.name)

	try:
		return TElement.objects.get(kind=kind, name=name, tenant=tenant)
	except TElement.DoesNotExist,e:
		return None

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def element_template_create(kind, name, tenant=None, owner_user=None):

	db_set_current_tenant_schema(tenant.name)

	if owner_user: 
		owner_user.assert_capability(tenant, 'core.template.element.create')

	te = TElement(
		tenant = tenant,
		kind   = kind, 
		name   = name,
		owner  = owner_user,
	)
	te.save()

	# FIX: mtime

	return te

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def element_template_delete(telement, user):

	# CHECK CAPABILITY/Permission
	if user: 
		user.assert_capability(telement.tenant, 'core.template.element.delete')

		telement.assert_permission_readwrite(user)

	# FIX:
	telement.delete()

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def element_template_select_all(kind=None, name_filter_str=None, tenant_name=None):

	if tenant_name != None:
		db_set_current_tenant_schema(tenant_name)
		t = tenant_get(tenant_name)
	else:
		t = tenant_get_primary()
		db_set_current_tenant_schema(t.name)

	element_templates = TElement.objects.filter(tenant=t, kind=kind)

	if kind != None:
		element_templates = element_templates.filter(kind=kind)

	if name_filter_str != None:
		pattern = '%s' % (name_filter_str)
		element_templates = element_templates.filter(name__startswith=pattern)

	return element_templates

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def element_template_update(te, *fields_name):

	mtime = time.time()

	# update mtime?
	#te.mtime = time.time()
	#te.save()
	te.concurrent_save(fields_name, new_sentinel_value=mtime)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def element_template_set_switch(te, is_switch, user):

	mtime = time.time()

	if user:
		user.assert_capability(te.tenant, 'core.template.element.edit')
		te.assert_permission_readwrite(user)

	if te.is_switch == is_switch:
		return

	# Update template
	te.is_switch = is_switch
	#te.mtime = time.time()
	#te.save()
	#te.concurrent_save(['is_switch'], new_sentinel_value=mtime)

	# FIX: ottimizzare questo update con meno accessi al DB
	objs = ElementTemplate.objects.filter(telement=te)
	for o in objs:
		node = o.melement.get_specific_element()
		node.is_switch = te.is_switch
		# FIX: use UPDATE
		#node.mtime = time.time()
		#node.save()
		node.concurrent_save(['is_switch'], new_sentinel_value=mtime)

	# confirm transaction with this last update
	te.concurrent_save(['is_switch'], new_sentinel_value=mtime)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def element_template_set_router(te, is_router, user):

	mtime = time.time()

	if user:
		user.assert_capability(te.tenant, 'core.template.element.update')
		te.assert_permission_readwrite(user)


	if te.is_router == is_router:
		return

	te.is_router = is_router
	#te.mtime = time.time()
	#te.save()

	# FIX: ottimizzare questo update con meno accessi al DB
	objs = ElementTemplate.objects.filter(telement=te)
	for o in objs:
		node = o.melement.get_specific_element()
		node.is_router = te.is_router
		# FIX: use UPDATE
		#node.mtime = time.time()
		#node.save()
		node.concurrent_save(['is_router'], new_sentinel_value=mtime)
		
	te.concurrent_save(['is_router'], new_sentinel_value=mtime)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def refresh_template_element_all(tenant, telement, selected_tdgs=None):

	db_set_current_tenant_schema(tenant.name)

	# Troviamo tutti gli elementi (es. nodi) che usano il template e refreshiamo
	# tutti i datagroup corrispondenti
	for association in ElementTemplate.objects.filter(tenant=tenant, telement=telement):

		element = association.melement

		refresh_element_template(tenant, telement, element, selected_tdgs=selected_tdgs)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def refresh_element_template(tenant, telement, element, selected_tdgs=None):
	
	mtime = time.time()
	
	from core.api.elements import element_datagroup_create_from_element_template


	# Alcuni template modificano anche i campi dell'elemento a cui sono associati.
	if element.kind == ELEMENT_KIND_NODE:

		# Calcoliamo i flag da applicare (additivamente)
		is_router = telement.is_router
		is_switch = telement.is_switch
		for te in ElementTemplate.objects.filter(tenant=tenant, melement=element).order_by('applyorder'):
			is_router = te.telement.is_router or is_router
			is_switch = te.telement.is_switch or is_switch
		
		# Aggiorniamo i dati sui flag nella tabella del nodo
		node_element = element.get_specific_element()
		
		changed_fields = []
		if node_element.is_router != is_router:
			element.is_router = is_router
			changed_fields.append('is_router')

		if node_element.is_switch != is_switch:
			element.is_switch = is_switch
			changed_fields.append('is_switch')
			
		if changed_fields:
			node_element.concurrent_save(fields_name=changed_fields, new_sentinel_value=mtime)
			
			element.mtime = node_element.mtime    # FIX: node_element = element   ma il campo mtime va tenuto allineato. Si puo' fare diversamente?

	#
	# Gestione dei datagroup definiti dentro un template
	#

	# Troviamo tutti i datagroup-template associati all'elemente template
	tedgs = TEDataGroup.objects.filter(tenant=tenant, telement=telement)
	
	# Vediamo se dobbiamo refreshare solo alcuni tipi di datagroup-template
	if selected_tdgs is not None:
		tedgs = tedgs.filter(tdatagroup__in=selected_tdgs)
	
	# Creiamo un datagroup sul database (e la sua configurazione) per ogni datagroup-template
	# associato all'element-template.
	for tedg in tedgs.all():
		tdg                     = tedg.tdatagroup
		datagroup_instance_name = tedg.name 
		
		cdg = element_datagroup_create_from_element_template(element, tedg, datagroup_instance_name, tenant=tenant, mtime=mtime)

	#
	# Gestione dei sub-element definiti dentro il template
	#

	# Troviamo tutte le definizioni dei sub-element nel template
	sub_element_templates = telement.get_sub_elements()
	for sub_element_template in sub_element_templates.all():

		#print "Verify subelement %s" % (sub_element_template.name)

		name = sub_element_template.get_element_name_to_apply()

		# Vediamo se esiste gia' un qualche elemento non generato da questo template che 
		if element.get_sub_elements().filter(kind=sub_element_template.kind, name=name).exclude(telement_generator=sub_element_template).count() > 0:
			raise InvalidConfigurationException("Duplicated element '%s'" % (sub_element_template.name), [])

		# Troviamo o creiamo l'elemento che dovrebbe essere associato al sub-elemente-template
		automatic_sub_element = element.create_sub_element(name, sub_element_template.kind)

		automatic_sub_element.telement_generator = sub_element_template
		automatic_sub_element.distinguisher      = sub_element_template.distinguisher
		automatic_sub_element.xform              = sub_element_template.xform
		automatic_sub_element.save()

		# Refresh di tutti i template associati all'elemento
		for telementappliedtemplate in sub_element_template.get_applied_templates():

			telement_to_apply = telementappliedtemplate.telementapplied

			if DEBUG: print "Appling", telement_to_apply.name

			et, created = ElementTemplate.objects.get_or_create(tenant=tenant, melement=automatic_sub_element, telement=telement_to_apply, telementappliedtemplate=telementappliedtemplate)
			if created:
				if DEBUG: print "created", telement_to_apply.name
				refresh_element_template(tenant, telement_to_apply, automatic_sub_element)

	if DEBUG: print ""
	if DEBUG: print "refresh_element_template() now refresh", element
	if DEBUG: print ""

	# IMPORTANTE:
	# Ora chi abbiamo creato tutti i datagroup in base all'element-template,
	# assicuriamoci che le dipendenze siano soddisfatte.
	_refresh_element(element, mtime=mtime, only_fields_names=['dependson'])

	# last update for transactional commit	
	element.concurrent_save(new_sentinel_value=mtime)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def element_template_datagroup_create(tenant, kind, name, datagroup_template_name, datagroup_instance_name, user):

	db_set_current_tenant_schema(tenant.name)

	te   = TElement.objects.get(tenant=tenant, kind=kind, name=name)

	if user:
		user.assert_capability(tenant, 'core.template.element.create')
		te.assert_permission_readwrite(user)


	tdg  = datagroup_template_get(datagroup_template_name, tenant=tenant)
	if tdg == None:
		raise Exception("Datagroup template %s does not exist" % (datagroup_template_name))

	tedg, created = TEDataGroup.objects.get_or_create(tenant=tenant, telement=te, tdatagroup=tdg, name=datagroup_instance_name)

	if created:
		tedg.mtime = time.time()
		tedg.save()

	if created:
		#from core.api.templates import refresh_template_element_all

		refresh_template_element_all(tenant, te, selected_tdgs=[tdg,])

	return tedg

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def element_template_datagroup_remove(tenant, kind, name, datagroup_instance_name):
	
	db_set_current_tenant_schema(tenant.name)
	
	te   = TElement.objects.get(tenant=tenant, kind=kind, name=name)

	tedg = TEDataGroup.objects.get(tenant=tenant, telement=te, name=datagroup_instance_name)	
	tedg.delete()

	#from core.api.templates import refresh_template_element_all
	
	refresh_template_element_all(tenant, te)


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def datagroup_template_rename_parameter(dg_name, param_name, new_name, source=SOURCE_LOCAL, tenant=None, user=None):
	
	if tenant and user:
		user.assert_capability(tenant, 'core.template.dg.edit')
		tdg.assert_permission_readwrite(user)

	tdg = TDataGroup.objects.get(name=dg_name, source=source, tenant=tenant)
	
	# Non posso rinominare un parametro in un datagroup template figlio di un altro datagroup template
	anchestors = tdg.get_ancestors_tree(include_self=False)
	count = TDataGroupParameterValue.objects.filter(tdatagroup__in=anchestors, name=param_name).count()
	if count > 0:
		raise Exception("Can not rename a parameter defined in a parent template")
	

	# Update parameter in related objects
	_rename_datagroup_parameter(tdg, param_name, new_name)

	# Recursive update
	for tdg_child in tdg.get_recursive_children():
		_rename_datagroup_parameter(tdg_child, param_name, new_name)
		
def _rename_datagroup_parameter(tdg, param_name, new_name):

	if DEBUG: print "Renaming parameter in tdg", tdg, param_name, new_name
	tdg.rename_parameter(param_name, new_name)
	
	for tedg in tdg.tedatagroup_set.all():

		if DEBUG: print "Renaming in tedg", tedg, param_name, new_name	
		tedg.rename_parameter(param_name, new_name)

		for cdg in tedg.telement_datagroups.all():
			cdg.rename_parameter(param_name, new_name)
			
			dg = cdg.get_datagroup()
			dg.rename_parameter(param_name, new_name)
	
	for cdg in tdg.additional_datagroups.all():

		if DEBUG: print "Renaming in additiona dg", cdg, param_name, new_name
		cdg.rename_parameter(param_name, new_name)
		
		dg = cdg.get_datagroup()
		dg.rename_parameter(param_name, new_name)

	return

#------------------------------------------------------------------------------#

@transaction_commit_on_success
def datasource_template_rename(datagroup_name, datasource_name, new_name, source=SOURCE_LOCAL, tenant=None, user=None):

	tdg = TDataGroup.objects.get(name=datagroup_name, source=source, tenant=tenant)

	if tenant and user:
		user.assert_capability(tenant, 'core.template.dg.edit')
		tdg.assert_permission_readwrite(user)

	tds = TDataSource(name=datasource_name, tdatagroup = tdg, tenant=tenant)

	# Non posso rinominare un oggetto in un datagroup template figlio di un altro datagroup template
	anchestors = tdg.get_ancestors_tree(include_self=False)
	count = TDataSource.objects.filter(tdatagroup__in=anchestors, name=datasource_name).count()
	if count > 0:
		raise Exception("Can not rename a datasource defined in a parent template")

	# Update parameter in related objects
	_rename_datagroup_datasource(tdg, datasource_name, new_name)

	# Recursive update
	for tdg_child in tdg.get_recursive_children():
		_rename_datagroup_datasource(tdg_child, datasource_name, new_name)


def _rename_datagroup_datasource(tdg, old_name, new_name):

	if tdg.tenant:
		tdg.tenant.db_set_current_schema()

	if DEBUG: print "Renaming datasourcein tdg", tdg, old_name, new_name
	tdg.rename_datasource(old_name, new_name)

	for tedg in tdg.tedatagroup_set.all():
		if DEBUG: print "Renaming in tedg", tedg, old_name, new_name
		for cdg in tedg.telement_datagroups.all():
			dg = cdg.get_datagroup()
			dg.rename_datasource(old_name, new_name)

	for cdg in tdg.additional_datagroups.all():
		if DEBUG: print "Renaming in additiona dg", cdg, old_name, new_name
		dg = cdg.get_datagroup()
		dg.rename_datasource(old_name, new_name)


#------------------------------------------------------------------------------#

@transaction_commit_on_success
def condition_template_rename(datagroup_name, condition_name, new_name, source=SOURCE_LOCAL, tenant=None, user=None):

	tdg = TDataGroup.objects.get(name=datagroup_name, source=source, tenant=tenant)
	
	if tenant and user:
		user.assert_capability(tenant, 'core.template.dg.edit')
		tdg.assert_permission_readwrite(user)
	
	tc  = TCondition(tdatagroup=tdg, name=condition_name, tenant=tenant )

	# Non posso rinominare un oggetto in un datagroup template figlio di un altro datagroup template
	anchestors = tdg.get_ancestors_tree(include_self=False)
	count = TCondition.objects.filter(tdatagroup__in=anchestors, name=condition_name).count()
	if count > 0:
		raise Exception("Can not rename a condition defined in a parent template")

	# Update parameter in related objects
	_rename_datagroup_condition(tdg, condition_name, new_name)

	# Recursive update
	for tdg_child in tdg.get_recursive_children():
		_rename_datagroup_condition(tdg_child, condition_name, new_name)
	
def _rename_datagroup_condition(tdg, old_name, new_name):

	if tdg.tenant:
		tdg.tenant.db_set_current_schema()

	if DEBUG: print "Renaming condition in tdg", tdg, old_name, new_name
	tdg.rename_condition(old_name, new_name)

	for tedg in tdg.tedatagroup_set.all():
		if DEBUG: print "Renaming in tedg", tedg, old_name, new_name

		for cdg in tedg.telement_datagroups.all():
			dg = cdg.get_datagroup()
			dg.rename_condition(old_name, new_name)

	for cdg in tdg.additional_datagroups.all():
		if DEBUG: print "Renaming in additiona dg", cdg, old_name, new_name
		dg = cdg.get_datagroup()
		dg.rename_condition(old_name, new_name)

	return
	
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def timegraph_template_rename(tenant, tdg, name, new_name, user=None):

	ttg = TTimeGraph.objects.get(tenant=tenant, tdatagroup=tdg, name=name)
	
	if user is not None:
		user.assert_capability(tenant, 'core.template.dg.edit')
		tdg.assert_permission_readwrite(user)
	
	# Non posso rinominare un oggetto in un datagroup template figlio di un altro datagroup template
	anchestors = tdg.get_ancestors_tree(include_self=False)
	count = TTimeGraph.objects.filter(tdatagroup__in=anchestors, name=name).count()
	if count > 0:
		raise Exception("Can not rename a timegraph defined in a parent template")

	# Update parameter in related objects
	_rename_datagroup_timegraph(tdg, name, new_name)

	# Recursive update
	for tdg_child in tdg.get_recursive_children():
		_rename_datagroup_timegraph(tdg_child, name, new_name)
	
def _rename_datagroup_timegraph(tdg, old_name, new_name):

	if tdg.tenant:
		tdg.tenant.db_set_current_schema()

	if DEBUG: print "Renaming condition in tdg", tdg, old_name, new_name
	tdg.rename_timegraph(old_name, new_name)

	for tedg in tdg.tedatagroup_set.all():
		if DEBUG: print "Renaming in tedg", tedg, old_name, new_name

		for cdg in tedg.telement_datagroups.all():
			dg = cdg.get_datagroup()
			dg.rename_timegraph(old_name, new_name)

	for cdg in tdg.additional_datagroups.all():
		if DEBUG: print "Renaming in additiona dg", cdg, old_name, new_name
		dg = cdg.get_datagroup()
		dg.rename_timegraph(old_name, new_name)

	return

#------------------------------------------------------------------------------#

@transaction_commit_on_success
def timegraph_serie_template_rename(tenant, tdg, tg_name, name, new_name, user=None):


	if user is not None:
		user.assert_capability(tenant, 'core.template.dg.edit')
		tdg.assert_permission_readwrite(user)

	ttg = TTimeGraph.objects.get(tenant=tenant, tdatagroup=tdg, name=tg_name)
	
	ttgs = ttg.ttimegraphserie_set.get(name=name)

	# Non posso rinominare un oggetto in un datagroup template figlio di un altro datagroup template
	anchestors = tdg.get_ancestors_tree(include_self=False)
	count = TTimeGraphSerie.objects.filter(ttimegraph__tdatagroup__in=anchestors, ttimegraph__name=tg_name, name=name).count()
	if count > 0:
		raise Exception("Can not rename a timegraph defined in a parent template")

	# Update parameter in related objects
	_rename_datagroup_timegraphserie(tdg, tg_name, name, new_name)

	# Recursive update
	for tdg_child in tdg.get_recursive_children():
		_rename_datagroup_timegraphserie(tdg_child, tg_name, name, new_name)
	
def _rename_datagroup_timegraphserie(tdg, tg_name, old_name, new_name):

	if tdg.tenant:
		tdg.tenant.db_set_current_schema()

	if DEBUG: print "Renaming condition in tdg", tdg, old_name, new_name
	tdg.rename_timegraphserie(tg_name, old_name, new_name)

	for tedg in tdg.tedatagroup_set.all():
		if DEBUG: print "Renaming in tedg", tedg, old_name, new_name

		for cdg in tedg.telement_datagroups.all():
			dg = cdg.get_datagroup()
			dg.rename_timegraphserie(tg_name, old_name, new_name)

	for cdg in tdg.additional_datagroups.all():
		if DEBUG: print "Renaming in additiona dg", cdg, old_name, new_name
		dg = cdg.get_datagroup()
		dg.rename_timegraphserie(tg_name, old_name, new_name)

	return

#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def datagroup_template_rename_in_library(old_name, new_name, fix_logs=True, feedback_callback=None):

	def _null(msg):
		pass

	if feedback_callback is None: feedback_callback = _null


	feedback_callback("Renaming in library...")
	
	tdg_l = TDataGroup.objects.get(tenant=None, source=SOURCE_LIBRARY, name=old_name)
	tdg_l.name = new_name
	tdg_l.save()
	
	for tenant in Tenant.objects.order_by('name').all():
		
		feedback_callback("Fixing in tenant '%s'" % (tenant.name))
		
		tenant.db_set_current_schema()
		
		tdgs = TDataGroup.objects.filter(tenant=tenant, parent=tdg_l, name=old_name).all()
		
		for tdg in tdgs:
			_rename_tdg(tenant, tdg, new_name, fix_logs=fix_logs, feedback_callback=feedback_callback)


@transaction_commit_on_success
def datagroup_template_rename_in_tenant(tenant, old_name, new_name, fix_logs=True, feedback_callback=None):
	
	tenant.db_set_current_schema()
	
	tdg = TDataGroup.objects.get(tenant=tenant, name=old_name)
	
	_rename_tdg(tenant, tdg, new_name, fix_logs=fix_logs, feedback_callback=feedback_callback)
	
	return

def _rename_tdg(tenant, tdg, new_name, fix_logs=True, feedback_callback=None):
	
	t1 = time.time()
	
	def _null(msg):
		pass
	
	if feedback_callback is None: feedback_callback = _null
	
	old_name = tdg.name
	
	tdg.name = new_name
	tdg.save()
	
	condition_paths_to_update = []
	
	feedback_callback( "\tRenaming in element templates..." )
	
	for tedg in tdg.tedatagroup_set.all():
		
		if tedg.name == old_name:
			tedg.name = new_name
			tedg.save()
		
		for cdg in tedg.telement_datagroups.all():
			
			if cdg.name == old_name:
				cdg.name = new_name
				cdg.save()
			
			dg = cdg.get_datagroup()
			
			if dg.name != new_name:        # FIx: 2880
				dg.name = new_name
				dg.save()
			
			feedback_callback( "\t\t%s" % (dg.path) )
			
			old_path = dg.path
			if dg.update_path():
				
				dg.save()
				
				for ds in DataSource.objects.filter(datagroup=dg).all():
					if feedback_callback is not None: feedback_callback( "\t\t\t%s" % (ds.path) )
					ds.path = dg.path + DATAGROUP_PATH_SEP + ds.name
					ds.save()
				
				for c in Condition  .objects.filter(datagroup=dg).all():

					old_c_path = c.path

					if feedback_callback is not None: feedback_callback( "\t\t\t%s" % (c.path) )
					c.path  = dg.path + DATAGROUP_PATH_SEP + c.name
					c.save()

					condition_paths_to_update.append( (old_c_path, c.path) )
	
	
	feedback_callback( "\tRenaming additional datagorup in elements" )
	
	for cdg in tdg.additional_datagroups.all():
		
		if cdg.name == old_name:
			cdg.name = new_name
			cdg.save()
		
		dg = cdg.get_datagroup()
		
		feedback_callback( "\t\t: %s" % (dg.path) )
		
		old_path = dg.path
		if dg.update_path():
			
			dg.save()
			
			for ds in DataSource.objects.filter(datagroup=dg).all():
				if feedback_callback is not None: feedback_callback( "\t\t\t%s" % (ds.path) )
				ds.path = dg.path + DATAGROUP_PATH_SEP + ds.name
				ds.save()
			
			for c in Condition  .objects.filter(datagroup=dg).all():
				if feedback_callback is not None: feedback_callback( "\t\t\t%s" % (c.path) )
				
				old_c_path = c.path
				c.path  = dg.path + DATAGROUP_PATH_SEP + c.name
				c.save()
				
				condition_paths_to_update.append( (old_c_path, c.path) )
	
	if fix_logs:
		feedback_callback("\tFixing logs..." )
		for old_c_path, new_path in condition_paths_to_update:
			feedback_callback("\t\t%s" % (old_c_path) )
			changed = Event.objects.filter(condition_path=old_c_path).update(condition_path=new_path)
			feedback_callback("\t\t\tChanged %s logs." % (changed) )

	t2 = time.time()
	
	feedback_callback("Update took: %s seconds" % (t2-t1) )

	return


