# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from sanet_common.timeutils import get_working_timestamps_in_range

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

def get_ds_avg_in_timespec(datasource, ts_start, ts_end, timespec=None):

	"""
	Questa funzione restituisce la media dei valori di un datagroup all'interno dell'arco
	temporale (ts_start, ts_end) negli intervalli specificati da timespec.

	:param datasource: Oggetto di classe DataSource
	:param ts_start: timestamp di inizio intervallo
	:param ts_end: timestamp di fine intervallo
	:param timespec: intervalli temporali specificati nel formato di entables
	:return: il valore numerico della media (int o float)
	"""

	if timespec:
		timestamps = get_working_timestamps_in_range(ts_start, ts_end, timespec)
	else:
		timestamps = [[ts_start, ts_end]]

	tmp = []
	for start, end in timestamps:
		tmp += datasource.get_rrd_values(start, end)['dataseries']

	tmp = [x for x in tmp if x is not None]

	if len(tmp)>0:
		avg = reduce(lambda x, y: x+y, tmp)/len(tmp)
	else:
		avg = 0

	return avg


def get_ds_max_in_timespec(datasource, ts_start, ts_end, timespec=None):

	"""
	Questa funzione restituisce il massimo tra i valori di un datagroup all'interno dell'arco
	temporale (ts_start, ts_end) negli intervalli specificati da timespec.

	:param datasource: Oggetto di classe DataSource
	:param ts_start: timestamp di inizio intervallo
	:param ts_end: timestamp di fine intervallo
	:param timespec: intervalli temporali specificati nel formato di entables
	:return: il valore maaggiore tra quelli selezionati (int o float)
	"""

	if timespec:
		timestamps = get_working_timestamps_in_range(ts_start, ts_end, timespec)
	else:
		timestamps = [[ts_start, ts_end]]

	tmp = []
	for start, end in timestamps:
		tmp += datasource.get_rrd_values(start, end)['dataseries']

	return max(tmp)


def get_ds_min_in_timespec(datasource, ts_start, ts_end, timespec=None):

	"""
	Questa funzione restituisce il minimo tra i valori di un datagroup all'interno dell'arco
	temporale (ts_start, ts_end) negli intervalli specificati da timespec.

	:param datasource: Oggetto di classe DataSource
	:param ts_start: timestamp di inizio intervallo
	:param ts_end: timestamp di fine intervallo
	:param timespec: intervalli temporali specificati nel formato di entables
	:return: il valore minimo tra quelli selezionati (int o float)
	"""

	if timespec:
		timestamps = get_working_timestamps_in_range(ts_start, ts_end, timespec)
	else:
		timestamps = [[ts_start, ts_end]]

	tmp = []
	for start, end in timestamps:
		tmp += datasource.get_rrd_values(start, end)['dataseries']

	tmp = [x for x in tmp if x is not None]

	return min(tmp)
