# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class AbstractCachedFieldModel(object):
	"""
	Classe astratta per caricare gestire dinamicamente campi del modello
	solo tramite cache
	"""

	cached_fields = []

	#
	# Cache check data
	#
	def cache_key(self):
		"""
		Must return the unique key for the given model instance
		"""
		return NotImplementedError("cache_key() non implementato da: %s" % (self.__class__.__name__))

	def save_fields_in_cache(self, cache_manager):

		key = self.cache_key()

		data_to_cache = {}

		for n in self.cached_fields:
			data_to_cache[n] = getattr(self, n, None)

		AbstractCachedFieldModel.raw_save_fields_in_cache(cache_manager, key, data_to_cache)

	@staticmethod
	def raw_save_fields_in_cache(cache_manager, key, data_to_cache):

		if not data_to_cache:
			return False

		cache_manager.mset(key, data_to_cache)

		return True

	@staticmethod
	def raw_load_fields_from_cache(cache_manager, key, cached_fields):
		d = cache_manager.mget(key, cached_fields)
		return d

	def load_fields_from_cache(self, cache_manager):

		if not self.cached_fields:
			return False

		key = self.cache_key()

		data = cache_manager.mget(key, self.cached_fields)
		for n in self.cached_fields:
			v = data.get(n,None)
			setattr(self, n, v)


