# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import time

from django import db
from django.db import connection

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def preserve_db_schema_on_exception(fun):
	"""
	Questo decoratore assicura che il search_path sulla connessione al DB
	di DJANGO resti lo stesso prima e dopo la chiamata ad un metodo/funzione.
	
	Serve per risolvere un problema nella gestione delle eccezioni di DJANGO e il decorate
	
		@transaction_commit_on_success
	
	Quando una eccezione viene individuata in un blocco transazionale, il
	search_path viene "resettato".
	"""

	def _fun(*args, **kwargs):

		c = connection.cursor()
		c.execute('show search_path;')
		schema_names = c.fetchall()[0]
		c.close()
		#print __file__, "preserve_schema", "current search_path", schema_names

		try:
			#print __file__, "preserve_schema", "calling", fun.__name__
			return fun(*args, **kwargs)
		finally:
			#print __file__, "preserve_schema", "RESTORING search_path", schema_names
			c = connection.cursor()
			search_path = ','.join(schema_names)
			c.execute('set search_path to %s' % (search_path))
			c.close()

	return _fun

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def debug_print_queries(fun):
	"""
	Decoratore di comodo per debuggare il numero di query effettuate da un metodo od una funzione.
	"""

	def _fun(*args, **kwargs):

		db.reset_queries()

		t1 = time.time()
		try:
			return fun(*args, **kwargs)
		finally:
			t2 = time.time()

			print "*" * 80
			print "DEBUG", "debug_print_queries", "Queries:%s, elapsed:%s" % (len(connection.queries), t2 - t1)

			for q in connection.queries:
				print "DEBUG", q['time'], q['sql']

	return _fun
