# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.
"""

Gerarchia 
==========

Tutte le eccezioni sollevate da Sanet hanno come tipo base *BaseSanetException*.

::
    
        BaseSanetException
        |                     SanetCommon
    ____|___________________________________
        |                     Sanet Server
        | 
        `--- SanetServerException
             | 
             |--- DatabaseException
             | 
             |--- ConcurrentUpdateException
             |
             `--- ApplicationException
                  |
                  `--- InvalidConfigurationException
"""
from sanet_common.exceptions import  BaseSanetException

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class SanetServerException(BaseSanetException):
	"""
	Eccezione base per problemi sul server centrale di sanet
	"""
	pass
	

class DatabaseException(SanetServerException):
	"""
	Problemi a basso livello di schema del DB.
	* namespace non definiti
	* schema non aggioranto
	* indici non definiti
	* eccc..
	"""
	pass

#------------------------------------------------------------------------------#

class ConcurrentUpdateException(SanetServerException):
	"""
	Raised when a concurrent update on the DB is detected.
	"""
	pass

#------------------------------------------------------------------------------#

class ApplicationException(SanetServerException):
	"""
	Errore generico nella logica applicativa:
	
		* Vincoli "software" non rispettati
		* Configurazione 
		* ...
	"""
	pass


class InvalidConfigurationException(ApplicationException):
	"""
	Raised when a an invalid configuration is detected.
	
	L'eccezione ha due attributi:
		* msg   : messaggio testuale di errore singolo
		* errors: tutte le anomalie da segnalare.
	"""
	def __init__(self, msg, errors=None):
		self.msg    = msg
		self.errors = errors
	
	def __str__(self):
		s = self.msg
		if self.errors:
			s +=  '\n' + '\n'.join([str(e) for e in self.errors])
		return s
