# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os
import errno
from pwd import getpwnam
from grp import getgrnam

import logging
from logging.handlers import WatchedFileHandler


def mkdir_p(path):
	
	"""http://stackoverflow.com/a/600612/190597 (tzot)"""
	
	try:
		os.makedirs(path, exist_ok=True)  # Python>3.2
	except TypeError:
		try:
			os.makedirs(path)
		except OSError as exc: # Python >2.5
			if exc.errno == errno.EEXIST and os.path.isdir(path):
				pass
			else:
				raise

class MakeWatchedFileHandler(WatchedFileHandler):
	"""
	Questo logger logga su file accertandosi che tutte le directory del path
	esistano.

	Imposta i permessi di accesso e l'owner del file in maniera che
	tutte le componenti di sanet riescano ad aprire il file.
	"""

	def __init__(self, **kwargs):

		mkdir_p(os.path.dirname(kwargs['filename']))

		owner_id = -1
		group_id = -1
		mod      = 0

		if 'owner' in kwargs:
			owner_id = getpwnam(kwargs['owner']).pw_uid
			del kwargs['owner']

		if 'group' in kwargs:
			group_id = getgrnam(kwargs['group']).gr_gid
			del kwargs['group']
		
		if 'mod' in kwargs:
			mod = int(str(kwargs['mod']), 8)
			del kwargs['mod']

		WatchedFileHandler.__init__(self, **kwargs)

		#
		# If specified in settings, change owner, group and mod of log file
		#
		# IMPORTANT: only root can do that. For this reason, the first
		#            time this code MUST be executed as root.
		# 
		#            This is true if "sanetdata" as root is called after every install/update.
		#
		if os.getuid() == 0:
			os.chown(kwargs['filename'], owner_id, group_id)

			if mod:
				os.chmod(kwargs['filename'], mod)

