# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import logging

from optparse import make_option

from django.core.management.base import BaseCommand

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class SanetBaseCommand(BaseCommand):

	# Questo flag serve per indicare se il comando e' un comando "interno" (debug o altro)
	# o se puo' essere usato dagli utenti normali.
	#
	# I comandi "interni" non vengono visualizzati dal comando:
	#
	#     sanet-manage help 
	#
	sanet_internal = False

	def init_stdout_log(self, log_level):

		format_str = '%(asctime)s %(levelname)s > %(message)s'
		logger = logging.getLogger('test')
		logger.setLevel( log_level )

		self.logger = logger

		hdlr = logging.StreamHandler()
		hdlr.setFormatter( logging.Formatter(format_str) )
		hdlr.setLevel( log_level )
		logger.addHandler(hdlr)

		self.log_hdlr = hdlr

		return logger

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class SanetInternalCommand(BaseCommand):
	"""
	Internal commands are meant for special mantainance (or debugging) and are not visible with
	the default help command:
	
		sanet-manage help
	
	"""
	sanet_internal = True

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class TenantBasedCommand(SanetBaseCommand):

	def add_arguments(self, parser):
		super(TenantBasedCommand, self).add_arguments(parser)
		parser.add_argument('--tenant'  , action='store'     , dest='tenant'  , default=None , help='')

	def get_tenant(self, options):

		from core.models import Tenant

		tenant_name = options['tenant']

		#if tenant_name == None:
		#	tenant  = tenant_get_primary()
		#else:
		#	tenant  = tenant_get(tenant_name)
		#	if tenant is None:
		#		raise Exception("Invalid tenant '%s'" % (tenant_name))
		#
		#if tenant == None:
		#	raise Exception("No primary tenant found. You must specify a tenant name")

		try:
			if not tenant_name:
				tenant = Tenant.objects.get(is_primary=True)
			else:
				tenant = Tenant.objects.get(name=tenant_name)
		except Tenant.DoesNotExist, e:
			if tenant_name:
				raise Exception("Invalid tenant '%s'" % (tenant_name))
			else:
				raise Exception("No primary tenant found. You must specify a tenant name")

		tenant.db_set_current_schema()

		return tenant
