# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

agentinfo
======================================================================

Si rimanda :ref:`agentinfo`.

"""


import os
import sys
import time
from optparse import make_option
import json

from django.utils.translation import ugettext as _

from sanet_poller.agentctl import DEFAULT_AGENT_CTL_TIMEOUT
from sanet_poller.agentctl import agent_ctl, agent_ctl_local

from conf import get_configuration

from core.models import Agent
from core.api.base import agent_get_primary, agent_get, agent_get_by_id, agent_select_all

from core.management.commands import TenantBasedCommand

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(TenantBasedCommand):

	help = _("Query running agent.")

	#option_list = TenantBasedCommand.option_list + (
	#	make_option('--broker'          , action='store'     , dest='broker'         , default=None                     , help=''),
	#	make_option('--tcp'             , action='store_true', dest='tcp'            , default=False                    , help=''),
	#	make_option('--agent'           , action='store'     , dest='agent'          , default=None                     , help=''),
	#	make_option('--timeout'         , action='store'     , dest='timeout'        , default=DEFAULT_AGENT_CTL_TIMEOUT, help=''),
        #
	#	make_option('--checkall'        , action='store_true', dest='checkall'       , default=False                    , help=''),
	#	make_option('--checkall-details', action='store_true', dest='checkalldetails', default=False                    , help=''),
	#)

	def add_arguments(self, parser):
		super(Command, self).add_arguments(parser)
		
		parser.add_argument('--broker'          , action='store'     , dest='broker'         , default=None                     , help=''),
		parser.add_argument('--tcp'             , action='store_true', dest='tcp'            , default=False                    , help=''),
		parser.add_argument('--agent'           , action='store'     , dest='agent'          , default=None                     , help=''),
		parser.add_argument('--timeout'         , action='store'     , dest='timeout'        , default=DEFAULT_AGENT_CTL_TIMEOUT, help=''),

		parser.add_argument('--checkall'        , action='store_true', dest='checkall'       , default=False                    , help=''),
		parser.add_argument('--checkall-details', action='store_true', dest='checkalldetails', default=False                    , help=''),

		#
		# Positional arguments
		#
		parser.add_argument('cmd', nargs='?', default=None)

	#def handle(self, cmd='', *args, **options):
	def handle(self, *args, **options):
		
		cmd = options['cmd']

		t0 = time.time()

		config = get_configuration()

		timeout = int(options['timeout'])

		#if options['tcp']:
		#	broker = None
		#else:
		#	broker = options['broker']
		#	if broker == None:
		#		broker = '%s/%s@%s' % (config.BROKER_AUTH[0], config.BROKER_AUTH[1], config.BROKER_ADDRESS[0])

		broker = options['broker']
		if broker == None:
			broker = '%s/%s@%s' % (config.BROKER_AUTH[0], config.BROKER_AUTH[1], config.BROKER_ADDRESS[0])

		tenant = self.get_tenant(options)

		if options['checkall'] is False and options['checkalldetails'] is False:

			agent_id = options['agent']
			if agent_id == None:
				agent = agent_get_primary(tenant)
			else:
				agent = agent_get(agent_id, tenant.name)
				if agent == None:
					agent = agent_get_by_id(agent_id, tenant)

			if agent == None:
				raise Exception("Agent not found")

			if not cmd:
				cmd = 'info'

			#if broker:
			#	response = agent_ctl(broker, agent.uuid, cmd, list(args), timeout=timeout)
			#else:
			#	response = agent_ctl_local(agent.name, cmd,  list(args), timeout=timeout)

			response = agent_ctl(broker, agent.uuid, cmd, list(args), timeout=timeout)

			print response
		else:
			cmd  =  'infojson'
			args = []

			agents = agent_select_all(tenant.name)

			for agent in agents:

				if not agent.is_enabled:
					continue

				try:
					#if broker:
					#	response = agent_ctl(broker, agent.uuid, cmd, list(args), timeout=timeout)
					#else:
					#	response = agent_ctl_local(agent.name, cmd,  list(args), timeout=timeout)

					response = agent_ctl(broker, agent.uuid, cmd, list(args), timeout=timeout)

					response = json.loads(response)
					if response is None:
						raise Exception("Unable to get status from agent. Agent returned None")

					status = 'RUNNING'
					if len(response['critical_problems']) > 0:
						status = 'ERROR'

					if options['checkalldetails']:

						d = []
						for k, v in sorted(response.items()):
							if k == 'critical_problems':
								v = ','.join(v)
							d.append( '%s=%s' % (k, v) )

						print "%s:%s:%s %s" % (agent.tenant.name, agent.name, status , ';'.join(d) )

					else:
						print "%s:%s:%s" % (agent.tenant.name, agent.name, status)

				except Exception, e:
					print "%s:%s:%s %s" % (agent.tenant.name, agent.name, 'ERROR', str(e))
