# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
broker_info
========================

Mostra informazioni sullo stato del sistema di comunicazione tra i processi del sistema: 

	::
		
		broker_info [--flat] [ --broker <connection string> ] [queues]

.. warning:: This is an internal Sanet command. It is not recommended to use this command.

"""

import os
import sys
from optparse import make_option

from django.utils.translation import ugettext as _

from conf import get_configuration

from core.management.commands import SanetInternalCommand

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(SanetInternalCommand):

	help = _("Show IPC broker informations.")

	#option_list = SanetBaseCommand.option_list + (
	#	make_option('--broker', action='store'     , dest='broker', default=None , help=''),
	#	make_option('--flat'  , action='store_true', dest='flat'  , default=False, help=''),
	#)

	def add_arguments(self, parser):
		super(Command, self).add_arguments(parser)
		parser.add_argument('--broker', action='store'     , dest='broker', default=None , help=''),
		parser.add_argument('--flat'  , action='store_true', dest='flat'  , default=False, help=''),
		
		#
		# Positional arguments
		#
		parser.add_argument('cmd', nargs='?', default=None)

	#def handle(self, cmd='', *args, **options):
	def handle(self, *args, **options):
		
		cmd = options['cmd']
		
		if not cmd:
			cmd = 'queues'

		config = get_configuration()

		#broker = options['broker']
		#if broker == None:
		#	broker = config.BROKER_ADDRESS[0]
		#	if len(config.BROKER_ADDRESS)>1 and config.BROKER_ADDRESS[1] is not None:
		#		broker = '%s:%s' % (broker, config.BROKER_ADDRESS[1])
		#	if config.BROKER_AUTH:
		#		broker = '%s/%s@%s' % (config.BROKER_AUTH[0], config.BROKER_AUTH[1], broker)

		if cmd == 'queues':
			#self.show_queues(broker, options['flat'])
			self.show_queues(options['flat'])
		else:
			self.print_help()


	#def show_queues(self, broker, flat):
	def show_queues(self, flat):

		import redis
		from sanet_common.ipc.redisipc import _USE_REDIS_IPC_DB
		from cli.textformat import text_table

		pattern = "QUEUE_"

		r = redis.Redis()

		r.execute_command("SELECT", _USE_REDIS_IPC_DB)

		keys = r.keys(pattern+"**")

		if flat:
			for key in keys:
				print key[len(pattern):], r.llen(key)
		else:
			rows = [[ ["QUEUE", 30], ["SIZE", 5] ]]
			for key in sorted(keys):
				rows.append([ 
					key[len(pattern):], 
					r.llen(key)
				])
			print text_table(rows)

	def print_help(self, *args, **kwargs):
		print "Syntax: [options] <action>"
		print ""
		print "Options:"
		print "    --broker <address>  Specify broker address"
		print "    --flat              Plain text output" 
		print ""
		print "Actions:"
		print "    queues - Show broker queue info"
