# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
cache_info
========================

Syntax:
	::
		
		sanet-manage cache_info info
		
		sanet-manage cache_info keys [ <patter> [ sorted ] ]

Extract informations about Sanet caching subsystem.

.. warning:: This is an internal Sanet command. It is not recommended to use this command.

"""

import os
import sys
import redis

from django.utils.translation import ugettext as _

from core.management.commands import SanetInternalCommand

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(SanetInternalCommand):

	help = _("Show cache informations.")

	#option_list = SanetInternalCommand.option_list

	def add_arguments(self, parser):
		
		super(Command, self).add_arguments(parser)
		
		parser.add_argument('parameters', action='store', nargs='*', help='')

		#
		# Positional arguments
		#
		parser.add_argument('cmd', nargs='?', default=None)

	#def handle(self, cmd='', *args, **options):
	def handle(self, *args, **options):

		cmd = options['cmd']

		parameters = options['parameters']

		if not cmd:
			cmd = 'info'

		r = redis.Redis()

		if cmd == 'info':

			for k,v in sorted(r.info().items()):
				print "%30s: %s" % (k.upper(), v)

		elif cmd == 'keys':
		
			#pattern = args[0] if len(args) > 0 else '*'
			pattern = parameters[0] if len(parameters) > 0 else '*'
			
			#do_sort = True if (len(args) > 1 and args[1] == 'sorted') else False
			do_sort = True if (len(parameters) > 1 and parameters[1] == 'sorted') else False
			
			keys = r.keys(pattern)
			if do_sort:
				keys = sorted(keys)

			for key in keys:
				print key

	def print_help(self, *args, **kwargs):

		print "Syntax: <action>"
		print ""
		print "    info                           Show caching system's internal informations."
		print "    keys [ <patter> [ sorted ] ]   Show keys matching <pattern>."
