# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.
"""

check_active_rrds
========================================================================

Syntax:

	::
		
		sanet-manage check_active_rrds

Prints informations about RRD files.

.. warning:: This is an internal Sanet command. It is not recommended to use this command.

"""

import os
import sys
import getpass
import pytz

from django.utils.translation import ugettext as _

from sanet_common.sanetdefs.constants import DSSTATE_TO_CODENAME

from conf import get_configuration

from core.models import Tenant, DataSource
from core.api.datasource import init_datasource_store

from core.management.commands import SanetBaseCommand

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(SanetBaseCommand):

	help = "Print old/unused datasources files (RRD files)"

	#option_list = SanetBaseCommand.option_list 

	def handle(self, *args, **options):

		conf = get_configuration()
		
		datasource_store = init_datasource_store(conf.DATASOURCE_STORAGE_HANDLER, conf.DATASOURCE_STORAGE_PARAMS)

		FORMAT = "%-20s %-36s %-10s %-5s %-10s %-10s  %s"
		print FORMAT % ('TENANT', 'UUID', 'DS_STATUS', 'VALID', 'CTIME', 'MTIME', 'PATH')

		for tenant in Tenant.objects.all():

			#
			# Load datasoruces
			#
			tenant.db_set_current_schema()

			valid_ids = dict([ (r['uuid'], r) for r in DataSource.objects.all().values('uuid', 'status')])

			#
			#
			#
			base_path = datasource_store._get_tenant_dir(tenant.uuid)

			for fname in os.listdir(base_path):

				name = fname.replace('.rrd','')

				fpath = os.path.join(base_path, fname)

				file_info = os.stat(fpath)

				data = valid_ids.get(name, None)

				if data: print FORMAT %  (tenant.name, name, DSSTATE_TO_CODENAME( data['status'] ), "valid", int(file_info.st_ctime), int(file_info.st_mtime), fpath)
				else   : print FORMAT %  (tenant.name, name, '-'                                  , "old  ", int(file_info.st_ctime), int(file_info.st_mtime), fpath)
