# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
check_logs_consistency
========================================================================

Syntax:

	::
		
		sanet-manage check_logs_consistency [--start YYYY-MM-DD] [--end YYYY-MM-DD]

Verifies inconsistent Condition logs selected between *start* and *end* dates.

.. warning:: This is an internal Sanet command. It is not recommended to use this command.


"""

import os
import sys
import time
from optparse import make_option

from django.utils.translation import ugettext as _

from sanet_common.sanetdefs.constants import EVENT_CONDITION

from core.models import Event
from core.management.commands import SanetInternalCommand, TenantBasedCommand

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

def isodatetime_to_ts(s):
	return time.mktime(time.strptime(s, "%Y-%m-%d"))

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(SanetInternalCommand, TenantBasedCommand):

	#sanet_internal = True

	help = _("Check if there are incosistencies in conditions logs")

	#option_list = TenantBasedCommand.option_list + (
	#	#make_option('--tenant', action='store'     , dest='tenant' , default=None , help='Select the tenant.'),
	#	make_option('--start' , action='store'     , dest='start'  , default=None , help='Select the tenant.'),
	#	make_option('--end'   , action='store'     , dest='end'    , default=None , help='Select the tenant.'),
	#)

	def add_arguments(self, parser):
		super(Command, self).add_arguments(parser)

		parser.add_argument('--start' , action='store'     , dest='start'  , default=None , help='Select the tenant.'),
		parser.add_argument('--end'   , action='store'     , dest='end'    , default=None , help='Select the tenant.'),

	def handle(self, *args, **options):

		#
		# Set the right tenant
		#

		tenant = self.get_tenant(options)

		ts_start = None
		ts_end   = None

		if options['start'] == None and options['end'] is None:
			ts_end   = time.time()
			ts_start = ts_end - 60 * 60 * 24
		elif options['start'] is not None and options['end'] == None:
			ts_start = isodatetime_to_ts(options['start'])
			ts_end   = time.time()
		else:
			ts_start = isodatetime_to_ts(options['start'])
			ts_end   = isodatetime_to_ts(options['end'])

		logs = Event.objects
		logs = logs.filter(kind=EVENT_CONDITION)
		logs = logs.filter(timestamp__range=[ts_start, ts_end])
		logs = logs.order_by('timestamp')

		prev_state_by_condition = {}

		for log in logs:

			print log.uuid, log.status, log.statuslast, log.timestamp, log.condition_path

			path = log.condition_path

			prev = prev_state_by_condition.get(path, None)
			if prev is None:
				prev = log.statuslast

			if prev != log.statuslast:
				print "\t", "wrong", log.uuid

			prev = log.status

			prev_state_by_condition[path] = log.status
