# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.
"""

check_multiple_primary
======================================================================

Syntax:

	::
		
		sanet-manage check_multiple_primary [options]

Options:

	::
		
		--tenant <name>    Limit the check to the given tenant.

Verifies if there are multiple primary conditions configured for a monitored element.

.. warning:: This is an internal Sanet command. It is not recommended to use this command.

"""


import os
import sys
import time

from django.db import connection
from django.utils.translation import ugettext as _

from core.models import Tenant

from core.management.commands import TenantBasedCommand

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(TenantBasedCommand):

	help = _("Checks for multiple primary condition in monitored elements.")

	#option_list = TenantBasedCommand.option_list 

	def add_arguments(self, parser):
		super(Command, self).add_arguments(parser)
		
		#parser.add_argument('--broker'          , action='store'     , dest='broker'         , default=None                     , help=''),
		#parser.add_argument('--tcp'             , action='store_true', dest='tcp'            , default=False                    , help=''),
		#parser.add_argument('--agent'           , action='store'     , dest='agent'          , default=None                     , help=''),
		#parser.add_argument('--timeout'         , action='store'     , dest='timeout'        , default=DEFAULT_AGENT_CTL_TIMEOUT, help=''),
		#
		#parser.add_argument('--checkall'        , action='store_true', dest='checkall'       , default=False                    , help=''),
		#parser.add_argument('--checkall-details', action='store_true', dest='checkalldetails', default=False                    , help=''),
		#
		##
		## Positional arguments
		##
		#parser.add_argument('cmd', nargs='?', default=None)

	#def handle(self, cmd='', *args, **options):
	def handle(self, **options):

		if options['tenant']:
			tenants = Tenant.objects.filter(name=options['tenant']).all()
		else:
			tenants = Tenant.objects.all()

		for tenant in tenants:
			
			tenant.db_set_current_schema()
			
			c = connection.cursor()
			
			c.execute("""
select   case when me.parent_id is not null then p.name  else me.name end as NODO
       , case when me.parent_id is not null then me.name else ''      end as SUB
  from monitoredelement me left join monitoredelement p on (me.parent_id = p.uuid) 
 where me.uuid in ( 
                   select condition.element_id 
                     from condition 
                    where "condition"."primary" = true 
                    group by condition.element_id 
                   having count(*) > 1 
                  )
"""
			)

			data = c.fetchall()

			if data:
				print "Tenant:", tenant.name
				for r in data:
					print r[0], r[1]
				print ""
