# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.
"""

check_scheduling_config
======================================================================

TODO

"""


import os
import sys
import time

from django.utils.translation import ugettext as _

from core.models import SchedulingGroup, DataGroup, Agent, MonitoredNode, MonitoredIface, MonitoredStorage, MonitoredService, MonitoredDevice

from core.management.commands import TenantBasedCommand

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

MAX = 2**32

class Command(TenantBasedCommand):

	help = _("Show problematic configuration about configured scheduling groups (implicit or explicit)")

	#option_list = TenantBasedCommand.option_list 

	def handle(self, cmd='', *args, **options):

		tenant = self.get_tenant(options)
		
		agents = Agent.objects.filter(tenant=tenant).all()

		for agent in agents:

			elements_by_uuid = {}
			elements_by_uuid.update(dict([ (e['uuid'].encode('ascii'),e) for e in MonitoredNode   .load_for_agent(tenant, agent) ]))
			elements_by_uuid.update(dict([ (e['uuid'].encode('ascii'),e) for e in MonitoredIface  .load_for_agent(tenant, agent) ]))
			elements_by_uuid.update(dict([ (e['uuid'].encode('ascii'),e) for e in MonitoredStorage.load_for_agent(tenant, agent) ]))
			elements_by_uuid.update(dict([ (e['uuid'].encode('ascii'),e) for e in MonitoredService.load_for_agent(tenant, agent) ]))
			elements_by_uuid.update(dict([ (e['uuid'].encode('ascii'),e) for e in MonitoredDevice .load_for_agent(tenant, agent) ]))

			datagroups = DataGroup.load_for_agent(tenant, agent)

			configured_groups_by_uuid = dict([ (sg['uuid'], sg) for sg in SchedulingGroup.load_for_agent(tenant) ])

			#
			# Raggruppamento per gruppi configurati o automatici (cluster o nodo)
			#
			dg_by_group = {}
			for dg in datagroups:

				if dg['scheduling_group_id'] is not None:
					sg_uuid = dg['scheduling_group_id']
				else:
					# GRUPPI AUTOMATICI
					e = elements_by_uuid[dg['element']]
					
					# Fa parte di un cluster? -> gruppo cluster
					if e['cluster_id']          is not None: 
						sg_uuid = e['cluster_id']
						
					# E' un nodo? -> gruppo nodo
					elif e['parent_id']         is     None: 
						sg_uuid = e['uuid']
					else:
						# E' un sotto-elemento -> gruppo nodo (parent)
						p = elements_by_uuid[e['parent_id']]
						sg_uuid = p['uuid']

				#print "\t", "\t", dg['path'], sg_uuid
				
				l = dg_by_group.get(sg_uuid, dg_by_group.setdefault(sg_uuid, []))
				l.append(dg)
			
			#
			# Verifica per gruppi
			#
			for g_uuid, dgs in dg_by_group.items():

				error = False

				min_minperiod = min([ dg['minperiod'] for dg in dgs ])
				
				# Gruppo configurato?
				sg = configured_groups_by_uuid.get(g_uuid, None)
				if sg:
					g_type  = 'GROUP'
					g_name  = sg['name']

					min_gap = sg['min_gap']
				else:
					# Gruppo automatico. I parametri sono quelli dell'agente
					min_gap  = agent.scheduling_min_gap

					# Il gruppo e' associato ad un elemento? -> NODO
					e = elements_by_uuid.get( g_uuid , None)
					if e is not None:
						g_type = 'AUTO'
						g_name = e['name']
					else:
						# Gruppo associato al cluster evidenmente.
						g_type = 'AUTO'
						g_name = 'CLUSTER'
				
				# Verifica invariaente di stabilita'
				if min_gap:
					size = min_gap * len(dgs)
					if size > min_minperiod:
						error = True

					print agent.name, g_type, sg_uuid, g_name, "num datagroups=%s" % (len(dgs)), ",", "min_gap=%s" % (min_gap), ",", size, "(required time window)", ">", 'min minperiod=%s' % (min_minperiod), "ERRORE" if error else ''
