# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

create_datasource_store 
====================================================================================

Syntax:

	::
		
		sanet-manage create_datasource_store [optioni] <path | id>

Initialize the storage (RRD file) for the given *datasource*.

Options:

	::
		
		--tenant <name>   Specify the tenant, otherwise the primary tenant is selected. 

Example:

	::
		
		sanet-manage create_datasource_store 'localhost;;datagroup-test;datasource1'
		
		sanet-manage create_datasoruce_store 117634a4962841f2952695a92c3b6c1d

.. warning:: This is an internal Sanet command. It is not recommended to use this command.

"""

import os
import sys

from django.db.models import Q

from conf import get_configuration

from core.models import DataSource

from core.api.datasource import init_datasource_store

from core.management.commands import TenantBasedCommand, SanetInternalCommand

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(SanetInternalCommand, TenantBasedCommand):

	help = "Create datasource store file (RRD file)"

	#option_list = TenantBasedCommand.option_list 

	def add_arguments(self, parser):
		
		super(Command, self).add_arguments(parser)

		#
		# Positional arguments
		#
		parser.add_argument('ds_id', nargs='?', default=None)

	#def handle(self, ds_id, *args, **options):
	def handle(self, *args, **options):

		ds_id = options['ds_id']

		configuration = get_configuration()

		tenant = self.get_tenant(options)

		ds = DataSource.objects.get(Q(path=ds_id)|Q(uuid=ds_id))

		configuration = get_configuration()

		datasource_store = init_datasource_store(configuration.DATASOURCE_STORAGE_HANDLER, configuration.DATASOURCE_STORAGE_PARAMS)
		
		if datasource_store.exists(tenant.uuid, ds.uuid):
			print "Already exists."
			return
			
		datasource_store.init_store(ds)
