# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
datasource_produce_random_values
===============================================================================================

Syntax:

	::
		
		sanet-manage datasource_produce_random_values [ --tenant <name> ] <path datasource> <DD-MM-YYYY inizio> <DD-MM-YYYY fine> <minvalue>-<max>

Generate and stores random values for the given datasource, between the selected time range.

.. warning:: This is an internal Sanet command. It is not recommended to use this command.

Example:

	::
		
		sanet-manage datasource_produce_random_values 'localhost;;cpu-hr;cpuperc-hr' '01-01-2016' '01-03-2016' 0-1000

"""

import os
import sys
import time
import shutil
import datetime

from random import uniform
from random import randint

from optparse import make_option

from conf import get_configuration

from core.models import DataSource

from core.api.datasource import init_datasource_store

from core.management.commands import TenantBasedCommand, SanetInternalCommand

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class Command(TenantBasedCommand, SanetInternalCommand):

	help = "Produce random values for the specified datasource."

	#option_list = TenantBasedCommand.option_list + (
	#	make_option('--counter'  , action='store_true' , dest='counter' , default=False, help=''),
	#)

	def add_arguments(self, parser):
		super(Command, self).add_arguments(parser)

		parser.add_argument('--force'         , action='store_true' , dest='force'        , default=False, help=''),
		parser.add_argument('--counter'       , action='store_true' , dest='counter'      , default=False, help=''),
		
		parser.add_argument('parameters', action='store'     , nargs='*', help=''),

	def handle(self, *args, **options):

		parameters = options['parameters']
		
		if len(parameters) != 4:
			print "usage: command -- <path> <start_date ex: '12-02-2015'> <end_date ex: '12-02-2015'> <values-range ex: '0-100'> [COUNTER]"
			sys.exit(-1)

		configuration = get_configuration()

		tenant = self.get_tenant(options)

		datasource_store = init_datasource_store(configuration.DATASOURCE_STORAGE_HANDLER, configuration.DATASOURCE_STORAGE_PARAMS)

		counter      = options['counter']
		force_create = options['force_create']

		path_or_uuid = parameters[0]
		start_date   = parameters[1]
		end_date     = parameters[2]
		values_range = parameters[3]


		start_date   = time.strptime(start_date, "%Y-%m-%d")
		start_date   = int( time.mktime(start_date) )

		if end_date == 'now':
			end_date = int( time.time() )
		else:
			end_date = time.strptime(end_date, "%Y-%m-%d")
			end_date = int( time.mktime(end_date) )

		values_range = values_range.split('-')

		try:
			values_range[0] = int(values_range[0])
			values_range[1] = int(values_range[1])
		except ValueError:
			values_range[0] = float(values_range[0])
			values_range[1] = float(values_range[1])


		try:
			datasource = DataSource.objects.get(uuid=path_or_uuid)
		except DataSource.DoesNotExist, e:
			datasource = DataSource.objects.get(path=path_or_uuid)

		try:
			file_exits = datasource_store.init_store(datasource) == False
		except Exception, e:
			print e
			sys.exit(-1)


		print "Values range   :  %f - %f" % tuple(values_range)

		step = datasource.datagroup.minperiod
		if step == 0:
			step = 60

		print "Start timestamp: ", start_date
		print "End timestamp  : ", end_date
		print "Values range   :  %f - %f" % tuple(values_range)
		print "Step           : ", step, datasource.datagroup.minperiod

		start_ts = start_date-10

		if file_exits:

			if not force_create:
				c = raw_input("File already exists:\n\t(m)make a new one, (e)exit: ")
				if c == 'm':
					force_create = True

			if force_create:
				ds_file_path = datasource_store._get_datasource_rrd_filename(tenant.uuid, datasource.uuid)[1]

				shutil.move(ds_file_path, ds_file_path + '.bkp')

				#i = ''
				#while os.path.exists(ds_file_path + '.bkp.' + datetime.date.today().isoformat() + '(' + i + ')'):
				#	if i == '':
				#		i = 0
				#	else:
				#		i += 1
				#
				#if i == '':
				#	shutil.move(ds_file_path, ds_file_path + '.bkp')
				#else:
				#	shutile.move(ds_file_path, ds_file_path + '.bkp.' + datetime.date.today().isoformat() + '(' + i + ')')
				

				datasource_store.create_rrd_file(ds_file_path, datasource, start_ts=start_ts)

			elif c == 'e':
				sys.exit(0)

		#
		# FIXME: this "flush" is necessary but specific to RRD
		#
		rrdcached_address = configuration.DATASOURCE_STORAGE_PARAMS.get('RRDCACHED_ADDRESS', None) or os.environ.get('RRDCACHED_ADDRESS', None)
		if rrdcached_address:
			ds_file_path = datasource_store._get_datasource_rrd_filename(tenant.uuid, datasource.uuid)[1]
			code, msg = _send_forget_to_rrdcached(rrdcached_address, ds_file_path)

		if counter:
			i = 0
			for ts in range(start_date, end_date, step):
				datasource_store.save_value(tenant.uuid, datasource.uuid, ts, i)
				i += randint(values_range[0], values_range[1])

		elif type(values_range[1]) == int:
			for ts in range(start_date, end_date, step):
				datasource_store.save_value(tenant.uuid, datasource.uuid, int(ts), randint(values_range[0], values_range[1]))

		else:
			for ts in range(start_date, end_date, step):
				datasource_store.save_value(tenant.uuid, datasource.uuid, ts, uniform(values_range[0], values_range[1]))


import socket

def _send_forget_to_rrdcached(rrdcached_address, filename):

	rrdcached_address = rrdcached_address.replace("unix:","")

	# Create a UDS socket
	sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)

	# Connect the socket to the port where the server is listening
	sock.connect(rrdcached_address)

	sock.sendall("FORGET %s\n" % (filename))

	data = ''
	while True:
		buff = sock.recv(1)
		if not buff or buff == '\n':
			break
		data += buff

	sock.close()

	code, msg = data.split(' ', 1)
	code = int(code)

	return code, msg
