# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
dbusage
========================

Syntax:

	::
		
		sanet-manage dbusage

Show informations about DataBase's running queries.

.. warning:: This is an internal Sanet command. It is not recommended to use this command.


"""

import os
import sys

from core.management.commands import SanetInternalCommand

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(SanetInternalCommand):

	help = "Show DataBase usage (pending connections/queries)"

	#option_list = SanetInternalCommand.option_list 

	def handle(self, cmd='', *args, **options):
		
		import psycopg2
		from django.conf import settings as settings_mod
		from cli.textformat import text_table

		kw={}
		kw['database'] = settings_mod.DATABASES['default']['NAME']
		kw['user']     = settings_mod.DATABASES['default']['USER']
		kw['password'] = settings_mod.DATABASES['default']['PASSWORD']
		kw['host']     = settings_mod.DATABASES['default']['HOST']
		kw['port']     = settings_mod.DATABASES['default']['PORT']


		conn = psycopg2.connect(**kw)
		c = conn.cursor()

		c.execute('select version()');
		data = c.fetchall()[0][0]
		
		# PostgreSQL 9.6.3 on x86_64-pc-linux-gnu, compiled by gcc (Debian 6.3.0-18) 6.3.0 20170516, 64-bit
		# ---------- ----- --------------------------------------------------------------------------------
		version = data.split(' ', 3)[1]   # 9.6.3
		version = version[0:3]            # 9.6
		
		#print version
		
		if version.startswith('9.6'):
			t=[[
				['PID'                  ,  8],
				['WAIT_EVENT'           ,  3],
				['STATE'                , 10],
				['XACT_START           ',  9],
				['QUERY_START          ',  9],
				['BACKEND_START        ',  9],
				['QUERY'                , 15],
			]]
			
			proc_field   = 'pid'
			state_filter = "   and state not like '%%idle%%'"
		
		elif version.startswith('9.4'):
		
			t=[[
				['PID'                  ,  8],
				['STATE'                , 10],
				['WAITING'              ,  3],
				['XACT_START           ',  9],
				['QUERY_START          ',  9],
				['BACKEND_START        ',  9],
				['QUERY'                , 15],
			]]
			
			proc_field   = 'pid'
			state_filter = "   and state not like '%%idle%%'"
		
		else:
			t=[[
				['PROCPID'              ,  8],
				['WAITING'              ,  3],
				['XACT_START           ',  9],
				['QUERY_START          ',  9],
				['BACKEND_START        ',  9],
				['CURRENT_QUERY'        , 15],
			]]
			proc_field   = 'procpid'
			state_filter = ''
		
		query_fields = map(lambda x: x[0].lower(), t[0])
		
		c.execute('select pg_backend_pid()')
		pid = int(c.fetchall()[0][0])
		
		query  = ""
		query += "SELECT " + ', '.join(query_fields)
		query += "  FROM pg_stat_activity "
		query += " WHERE %(proc_field)s <> %(pid)s" % { 'proc_field': proc_field, 'pid':pid }
		query += state_filter
		c.execute(query)

		rv = c.fetchall()
		for l in rv:
			t.append(map(lambda x:str(x).replace('\n', ' '), l))
		conn.close()

		print text_table(t, term_width=0)



