# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

exec_datagroup
=============================================

Launch: 

	::
		
		sanet-manage exec_datagroup [options] ( <UUID> | <path> )


Available options are: 

	::
		
		-l <n>            Imposta il livello di logging al valore <n>, 1 debug massimo, 10 debug normale, 20 info.
		--tenant <nome>   Esegue il datagroup del tenant specificato. Default: Tenant primario
        
Examples
--------    

Check the datagroup *xha;;icmp-reachability* 

	::
		
		# python manage.py exec_datagroup 'xha;;icmp-reachability' -l 10
		2013-07-25 19:03:19,656 DEBUG MainThread > Loading cache manager...
		2013-07-25 19:03:19,662 DEBUG MainThread > Loading datasource store...
		2013-07-25 19:03:19,672 DEBUG MainThread > Loading data...
		2013-07-25 19:03:19,687 DEBUG MainThread > Using default tenant: <Tenant:10a279a6ce5c4438a98f9c33c0b77cde:site>
		2013-07-25 19:03:19,692 DEBUG MainThread > Loading system symbols
		2013-07-25 19:03:19,728 INFO MainThread > Loading extra symbols from ['sanet_extra.agent.functions.rtt_pingerd']
		2013-07-25 19:03:19,740 DEBUG MainThread > ########################################################
		2013-07-25 19:03:19,741 DEBUG MainThread > ###################### CHECKING ########################
		2013-07-25 19:03:19,741 DEBUG MainThread > ########################################################
		2013-07-25 19:03:19,741 DEBUG MainThread > Checking datagroup 1081cb2edf8f4a5ca49e68e6bdc83004 at 1374771799.74
		2013-07-25 19:03:19,741 DEBUG MainThread > Loading previeous execution state...
		2013-07-25 19:03:19,743 DEBUG MainThread > Loading free parameters
		2013-07-25 19:03:19,748 DEBUG MainThread >   params 'probesize' = (3) 1472 
		2013-07-25 19:03:19,763 DEBUG MainThread > -------------------------- Calculating datasource rttmin ---------------- 
		2013-07-25 19:03:19,764 DEBUG MainThread > Evaluating datasource xha;;icmp-reachability;rttmin
		2013-07-25 19:03:19,764 DEBUG MainThread > $delta = 11.4565660954
		2013-07-25 19:03:19,765 DEBUG MainThread > getRttInfo_pingerd(ipv=None, host=2a01:4f8:110:3383::1, tries=None, probe=1472.0, timeout=None, frag=False, bind=None)
		2013-07-25 19:03:19,765 WARNING MainThread > PINGERD_ADDRESS not defined in environment! Using normal ping
		2013-07-25 19:03:19,765 DEBUG MainThread > Resolving ipv=6, host=2a01:4f8:110:3383::1
		2013-07-25 19:03:19,767 DEBUG MainThread > 2a01:4f8:110:3383::1: Sending IPv6 pings.
		2013-07-25 19:03:19,767 DEBUG MainThread > exec_ping: ping6 -c 1 -s 1472 -A -W 1.0 -n '2a01:4f8:110:3383::1'
		2013-07-25 19:03:19,875 DEBUG MainThread > exec_ping: ping6 -c 1 -s 1472 -A -W 1.0 -n '2a01:4f8:110:3383::1'
		2013-07-25 19:03:19,980 DEBUG MainThread > exec_ping: ping6 -c 1 -s 1472 -A -W 1.0 -n '2a01:4f8:110:3383::1'
		2013-07-25 19:03:20,087 DEBUG MainThread > exec_ping: ping6 -c 1 -s 1472 -A -W 1.0 -n '2a01:4f8:110:3383::1'
		2013-07-25 19:03:20,193 DEBUG MainThread > exec_ping: ping6 -c 1 -s 1472 -A -W 1.0 -n '2a01:4f8:110:3383::1'
		2013-07-25 19:03:20,300 DEBUG MainThread > 2a01:4f8:110:3383::1: got ping answer #0 in 0.107907 s
		2013-07-25 19:03:20,301 DEBUG MainThread > 2a01:4f8:110:3383::1: got ping answer #1 in 0.104681 s
		2013-07-25 19:03:20,301 DEBUG MainThread > 2a01:4f8:110:3383::1: got ping answer #2 in 0.106491 s
		2013-07-25 19:03:20,301 DEBUG MainThread > 2a01:4f8:110:3383::1: got ping answer #3 in 0.106536 s
		2013-07-25 19:03:20,301 DEBUG MainThread > 2a01:4f8:110:3383::1: got ping answer #4 in 0.106056 s
		2013-07-25 19:03:20,302 DEBUG MainThread > avg=0.1063342, min=0.104681, max=0.107907, loss=0.0
		2013-07-25 19:03:20,303 DEBUG MainThread > Check result 1374771799.76, SV, 0.104681, [u"expression getICMPStat('rttmin', $node, $probesize) expanded to 0.104681"]
		2013-07-25 19:03:20,303 DEBUG MainThread > Saving rttmin = 0.104681
		2013-07-25 19:03:20,303 DEBUG MainThread > Saving rttmin/lastgoodcheck = 1374771800.303601
		2013-07-25 19:03:20,304 DEBUG MainThread > -------------------------- Calculating datasource rttmax ---------------- 
		2013-07-25 19:03:20,304 DEBUG MainThread > Evaluating datasource xha;;icmp-reachability;rttmax
		2013-07-25 19:03:20,304 DEBUG MainThread > $delta = 11.9944171906
		2013-07-25 19:03:20,306 DEBUG MainThread > getRttInfo_pingerd(ipv=None, host=2a01:4f8:110:3383::1, tries=None, probe=1472.0, timeout=None, frag=False, bind=None)
		2013-07-25 19:03:20,307 DEBUG MainThread > SIMPLEVALUE CACHE HIT: 2a01:4f8:110:3383::1:1472:5:1.0:False:None:6 = (0.1063342, 0.104681, 0.107907, 0.0, []) <type 'tuple'>
		2013-07-25 19:03:20,307 DEBUG MainThread > Check result 1374771800.3, SV, 0.107907, [u"expression getICMPStat('rttmax', $node, $probesize) expanded to 0.107907"]
		2013-07-25 19:03:20,307 DEBUG MainThread > Saving rttmax = 0.107907
		2013-07-25 19:03:20,307 DEBUG MainThread > Saving rttmax/lastgoodcheck = 1374771800.307629
		2013-07-25 19:03:20,308 DEBUG MainThread > -------------------------- Calculating datasource rttavg ---------------- 
		2013-07-25 19:03:20,308 DEBUG MainThread > Evaluating datasource xha;;icmp-reachability;rttavg
		2013-07-25 19:03:20,308 DEBUG MainThread > $delta = 11.9975709915
		2013-07-25 19:03:20,309 DEBUG MainThread > getRttInfo_pingerd(ipv=None, host=2a01:4f8:110:3383::1, tries=None, probe=1472.0, timeout=None, frag=False, bind=None)
		2013-07-25 19:03:20,309 DEBUG MainThread > SIMPLEVALUE CACHE HIT: 2a01:4f8:110:3383::1:1472:5:1.0:False:None:6 = (0.1063342, 0.104681, 0.107907, 0.0, []) <type 'tuple'>
		2013-07-25 19:03:20,309 DEBUG MainThread > Check result 1374771800.31, SV, 0.1063342, [u"expression getICMPStat('rttavg', $node, $probesize) expanded to 0.1063342"]
		2013-07-25 19:03:20,310 DEBUG MainThread > Saving rttavg = 0.1063342
		2013-07-25 19:03:20,310 DEBUG MainThread > Saving rttavg/lastgoodcheck = 1374771800.31033
		2013-07-25 19:03:20,310 DEBUG MainThread > -------------------------- Calculating datasource lossperc ---------------- 
		2013-07-25 19:03:20,310 DEBUG MainThread > Evaluating datasource xha;;icmp-reachability;lossperc
		2013-07-25 19:03:20,311 DEBUG MainThread > $delta = 11.9995350838
		2013-07-25 19:03:20,311 DEBUG MainThread > getRttInfo_pingerd(ipv=None, host=2a01:4f8:110:3383::1, tries=None, probe=1472.0, timeout=None, frag=False, bind=None)
		2013-07-25 19:03:20,312 DEBUG MainThread > SIMPLEVALUE CACHE HIT: 2a01:4f8:110:3383::1:1472:5:1.0:False:None:6 = (0.1063342, 0.104681, 0.107907, 0.0, []) <type 'tuple'>
		2013-07-25 19:03:20,312 DEBUG MainThread > Check result 1374771800.31, SV, 0.0, [u"expression getICMPStat('loss', $node, $probesize) expanded to 0"]
		2013-07-25 19:03:20,312 DEBUG MainThread > Saving lossperc = 0.0
		2013-07-25 19:03:20,312 DEBUG MainThread > Saving lossperc/lastgoodcheck = 1374771800.312801
		2013-07-25 19:03:20,319 DEBUG MainThread > -------------------------- Calculating condition reach ---------------- 
		2013-07-25 19:03:20,319 DEBUG MainThread > Checking condition reach
		2013-07-25 19:03:20,319 DEBUG MainThread > $delta = 12.0010859966
		2013-07-25 19:03:20,320 DEBUG MainThread > Saving reach/lastgoodcheck = 1374771800.320371
		2013-07-25 19:03:20,320 DEBUG MainThread > Check result 1374771800.32, result=True, infos=[u' {lossperc@} < 1 expanded to (0 < 1)']
		2013-07-25 19:03:20,320 DEBUG MainThread > Old state  : UP
		2013-07-25 19:03:20,320 DEBUG MainThread > New state  : UP
		2013-07-25 19:03:20,321 DEBUG MainThread > Updating condition state
		2013-07-25 19:03:20,321 DEBUG MainThread >      UP -> UP
		2013-07-25 19:03:20,321 DEBUG MainThread > Updating penalty--->TODO
		2013-07-25 19:03:20,321 DEBUG MainThread > Checking all changed fields...
		2013-07-25 19:03:20,321 DEBUG MainThread > ----------------------- Saving execution state ----------------------
		2013-07-25 19:03:20,322 DEBUG MainThread >      lossperc = 0.0
		2013-07-25 19:03:20,322 DEBUG MainThread >      lossperc/lastgoodcheck = 1374771800.312801
		2013-07-25 19:03:20,322 DEBUG MainThread >      reach/lastgoodcheck = 1374771800.320371
		2013-07-25 19:03:20,322 DEBUG MainThread >      rttavg = 0.1063342
		2013-07-25 19:03:20,323 DEBUG MainThread >      rttavg/lastgoodcheck = 1374771800.31033
		2013-07-25 19:03:20,323 DEBUG MainThread >      rttmax = 0.107907
		2013-07-25 19:03:20,323 DEBUG MainThread >      rttmax/lastgoodcheck = 1374771800.307629
		2013-07-25 19:03:20,323 DEBUG MainThread >      rttmin = 0.104681
		2013-07-25 19:03:20,324 DEBUG MainThread >      rttmin/lastgoodcheck = 1374771800.303601
		2013-07-25 19:03:20,324 DEBUG MainThread > ----------------------- Element fields to update ----------------------
		2013-07-25 19:03:20,324 DEBUG MainThread > ----------------------- Pushing updates ----------------------
		2013-07-25 19:03:20,325 DEBUG MainThread > Check finished. Update data = 
		2013-07-25 19:03:20,331 DEBUG MainThread > 
		[{'conditions_updates': {u'74d5c331cbeb44b093d76310cb65bcce': {'fields': {'lastdone': 1374771800.320735},
									       'infos': [u' {lossperc@} < 1 expanded to (0 < 1)'],
									       'time': 1374771800.319544,
									       'uuid': u'74d5c331cbeb44b093d76310cb65bcce'}},
		  'datagroup_id': u'1081cb2edf8f4a5ca49e68e6bdc83004',
		  'datagroup_updates': {},
		  'datasources_updates': {u'09893adb00874763ade0ffaece3a58da': {'fields': {'lastdone': 1374771799.764363,
											   'lastvalue': 0.104681},
										'infos': [u"expression getICMPStat('rttmin', $node, $probesize) expanded to 0.104681"],
										'time': 1374771799.764363,
										'uuid': u'09893adb00874763ade0ffaece3a58da'},
					  u'5dca34f65bc949d0bf22d504b67ba976': {'fields': {'lastdone': 1374771800.308346,
											   'lastvalue': 0.1063342},
										'infos': [u"expression getICMPStat('rttavg', $node, $probesize) expanded to 0.1063342"],
										'time': 1374771800.308346,
										'uuid': u'5dca34f65bc949d0bf22d504b67ba976'},
					  u'5f2df9582cad41c5bc4b279a099cdef1': {'fields': {'lastdone': 1374771800.304476,
											   'lastvalue': 0.107907},
										'infos': [u"expression getICMPStat('rttmax', $node, $probesize) expanded to 0.107907"],
										'time': 1374771800.304476,
										'uuid': u'5f2df9582cad41c5bc4b279a099cdef1'},
					  u'8fb653003b914bba92c61e13ffc19d6f': {'fields': {'lastdone': 1374771800.31103,
											   'lastvalue': 0.0},
										'infos': [u"expression getICMPStat('loss', $node, $probesize) expanded to 0"],
										'time': 1374771800.31103,
										'uuid': u'8fb653003b914bba92c61e13ffc19d6f'}},
		  'element_id': u'6bd8081044cf4f58a932f3c0ee265f13',
		  'elements': {},
		  'mtime': 1373626873.01769}]
"""

import os
import sys
import time
import pprint
import logging
import traceback
from optparse import make_option

# Global Sanet libraries
from sanet_common.uuids import is_uuid, create_global_uuid
from sanet_common.mailing import load_mail_handler_class
from sanet_common.ipc import *

from sanet_common.ipc import IPCConnection

from sanet_common.termutils import term_color, TERM_COLOR_GREEN, TERM_COLOR_YELLOW
#TERM_COLOR_RED, 

from sanet_poller.lib.debug import *
from sanet_poller.lib.functions import load_system_expr_functions
from sanet_poller.lib.functions import load_extra_expr_functions
from sanet_poller.lib.classes   import DataGroupChecker
from sanet_poller.lib.communication import AgentUpdateEvent
from sanet_poller.agent import BaseAgent

# Local libraries (Sanet+Django related)
from django.utils.translation import ugettext as _


from conf import get_configuration

from core.management.commands import TenantBasedCommand

from core.models import *

from core.api.cache import init_cache_manager_from_conf
from core.api.datasource import init_datasource_store_from_conf

from core.localpolling import LocalConfiguration

#from server.sanetd.sanetdlogic import create_ipc_connection_from_config

from server.sanetd.sanetdlogic import ServerIPCFactory

#from server.sanetd.sanetdlogic import process_update_event
#from server.sanetd.sanetdlogic import send_alarm_to_entables_or_fallback
from server.sanetd.mailsender  import SanetdSimpleMailSender

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(TenantBasedCommand):

	help = _("Execute an existing datagroup")

	#option_list = TenantBasedCommand.option_list + (
	#	make_option('-l'      , action='store'     , dest='loglevel' , default=20   , help='Set log level (<20 DEBUG, <30 INFO, <40 WARNING, <50 ERROR.'),
	#	make_option('--broker', action='store'     , dest='broker'   , default=None , help='Send check result to the main server.'),
	#	make_option('--direct', action='store_true', dest='direct'   , default=False, help='Directly update on DB. (DO NOT USE. FOR DEBUG ONLY)'),
	#	make_option('--update', action='store_true', dest='do_update', default=False, help='Updata datagroup data. (DO NOT USE. FOR DEBUG ONLY)'),
	#	make_option('--notify', action='store_true', dest='do_notify', default=False, help='Send alarms to entables. (DO NOT USE. FOR DEBUG ONLY)'),
	#)

	def add_arguments(self, parser):
		super(Command, self).add_arguments(parser)
		
		parser.add_argument('-l'      , action='store'     , dest='loglevel' , default=20   , help='Set log level (<20 DEBUG, <30 INFO, <40 WARNING, <50 ERROR.')
		parser.add_argument('--broker', action='store'     , dest='broker'   , default=None , help='Send check result to the main server.'),
		parser.add_argument('--direct', action='store_true', dest='direct'   , default=False, help='Directly update on DB. (DO NOT USE. FOR DEBUG ONLY)')
		parser.add_argument('--update', action='store_true', dest='do_update', default=False, help='Updata datagroup data. (DO NOT USE. FOR DEBUG ONLY)')
		parser.add_argument('--notify', action='store_true', dest='do_notify', default=False, help='Send alarms to entables. (DO NOT USE. FOR DEBUG ONLY)')

		parser.add_argument('--save_update_to_json', action='store'     , dest='save_update_to_json', default=None , help='Save updates to json file. (DO NOT USE. FOR DEBUG ONLY)'),

		#
		# Positional arguments
		#
		parser.add_argument('datagroup_id', action='store', default=None, nargs='?', help='')

	def handle(self, *args, **options):
		
		datagroup_id = options['datagroup_id']
		
		if datagroup_id is None:
			raise Exception("Missing datagroup id/path")

		#
		# Set the right tenant
		#
		tenant = self.get_tenant(options)

		#
		# Verfifichiamo se l'ID inserito dall'utente e' un UUID o un
		# path (mason;;reachability). Per uniformare il codice
		# quello che useremo alla fine e' comunque UUID.
		#
		if not is_uuid(datagroup_id):
			datagroup = DataGroup.objects.get(path=datagroup_id)
			datagroup_id = datagroup.uuid
		else:
			datagroup = DataGroup.objects.get(uuid=datagroup_id)

		#
		# Init Log
		#
		loglevel = int(options['loglevel'])
		self.logger = None
		logger = self.init_log(loglevel)


		#
		# Se il datagroup e' associato ad un elemento monitorato da un agente remoto
		# il check lo facciamo fare all'agente remoto
		#
		try:
			logger.debug("Checking datagroup %s. Agent: %s %s", datagroup.path, datagroup.agent, datagroup.agent.is_remote)
			
			if datagroup.agent.is_remote == True:
				self.execute_remotely(datagroup, loglevel)
			else:
				self.execute_local(logger, tenant, datagroup_id, options)

		except Exception, e:
			print "XXXXXXXXXXXXXXXXXX", type(e), str(e), traceback.format_exc()
		finally:
			# Quando questo comando viene eseguito dentro la CLI 
			# facciamo cleanup altrimenti ad ogni execute creiamo logger
			# aggiuntivi.
			self.remove_logger()

	def init_log(self, log_level):

		format_str = '%(asctime)s %(levelname)s %(threadName)s > %(message)s'
		logger = logging.getLogger('test')
		logger.setLevel( log_level )

		self.logger = logger

		hdlr = logging.StreamHandler()
		hdlr.setFormatter( logging.Formatter(format_str) )
		hdlr.setLevel( log_level )
		logger.addHandler(hdlr)

		self.log_hdlr = hdlr

		return logger

	def remove_logger(self):

		if self.logger:
			self.logger.removeHandler(self.log_hdlr)


	def execute_local(self, logger, tenant, datagroup_id, options):

		do_update           = options['do_update']
		do_notify           = options['do_notify']
		broker_address      = options['broker']
		save_update_to_json = options['save_update_to_json']



		update_data = self.do_check_local(logger, tenant, datagroup_id, options)

		if save_update_to_json:
			with open(save_update_to_json, 'w') as outfile:
				json.dump(update_data, outfile)

		if do_update:
			if broker_address and do_notify:
				raise Exception("Cannot send alarms when broker is specified.")

		if do_notify and not do_update:
				raise Exception("Cannot notify without update.")

		self.do_update_local(logger, tenant, datagroup_id, options, update_data)


	def do_check_local(self, logger, tenant, datagroup_id, options):

		conf = get_configuration()

		#
		# Creo l'oggetto per accedere alla cache (Redis)
		#
		logger.debug( "Loading cache manager...")

		cache_manager    =init_cache_manager_from_conf()

		ipc_factory = ServerIPCFactory(logger)

		#
		# Creo l'oggetto per notificare
		#
		#if do_notify:
		#	mail_handler_class = load_mail_handler_class(conf.SANETD_MAIL_HANDLER)
		#	mail_handler       = mail_handler_class(logger, conf.SANETD_MAIL_HANDLER_PARAMS)
		#	mail_sender        = SanetdSimpleMailSender(self.logger, mail_handler)
		#	mail_sender.start()

		#
		# Creo l'oggetto per gestire il database dei datasource (RRDTool)
		#
		logger.debug( "Loading datasource store...")
		datasource_store = init_datasource_store_from_conf()

		#
		# Carichiamo in sequenza
		#  - le funzioni standard del poller
		#  - eventuali funzioni contenute in moduli python esterni
		#    specificati tramite nel file di configurazione di SANET  (LOCAL_POLLER_EXTRA_MODULES)
		#
		symbols = {}

		logger.debug( "Loading system symbols")
		system_symbols = load_system_expr_functions()

		logger.debug("Loading extra symbols from %s", conf.LOCAL_POLLER_EXTRA_MODULES)
		extra_symbols = load_extra_expr_functions(conf.LOCAL_POLLER_EXTRA_MODULES)

		symbols.update(system_symbols)
		symbols.update(extra_symbols)

		# Creiamo l'oggetto che ha il compito fornire i vari oggetti del monitoraggio
		# (nodi,interfacce,ecc...) che potranno servire durante la valutazione
		# del datagroup.
		c = LocalConfiguration(logger, tenant.uuid, cache_manager)

		c.set_global_symbols(symbols) 

		# Otteniamo l'oggetto PDataGroup associato al datagroup selezioanto
		pdg = c.get_datagroup(datagroup_id)

		# Istanziamo la classe che contiene tutta la logica per la valutazione di un 
		# datagroup.
		dgc = DataGroupChecker(c, logger, cache_manager, ipc_factory, pdg )#, system_symbols)

		logger.debug( "########################################################" )
		logger.debug( "###################### CHECKING ########################" )
		logger.debug( "########################################################" )

		logger.info( "Start checking...")

		# CHECK
		t1 = time.time()
		dgc.check( time.time() )
		t2 = time.time()

		logger.info( "Check finished! Elapsed %s", t2 - t1)

		# Ci facciamo dare tutti i dati aggiornati/modificati durante il check
		# che dovrebbero essere mandati al processo SANETD per la memorizzazione
		# - valori dei datasource
		# - stato delle condition
		# - aggiornamenti su dati degli elementi (ifindex, ecc...)

		updates_data = dgc.get_datagroup_all_updates()

		logger.debug("\n%s", pprint.pformat(updates_data))
		
		fake_event = AgentUpdateEvent(time.time(), tenant.uuid, None, updates_data)
		fake_data  = BaseAgent.encode_update_events(fake_event)
		
		logger.debug("Data to send would be: %s", len(fake_data))

		for data in updates_data:

			logger.info("Update block:")

			for uuid, updates in data['elements'].items():
				e = MonitoredElement.objects.get(uuid=uuid)
				e = e.get_specific_element()
				logger.info("\tElement: %s", e.path)
				for f, v in updates.items():
					logger.info("\t\t%s =  %s", f, v)

			dg = DataGroup.objects.get(uuid=data['datagroup_id'])

			logger.info( "\tDataGroup: %s", dg.path)
			if data['datagroup_updates']:
				for field, value in data['datagroup_updates']['fields'].items():
					logger.info( "\t\t%s = %s", field, value)

			for uuid, ds_data in data['datasources_updates'].items():

				ds = DataSource.objects.get(uuid=uuid)

				logger.info( "\t\tDatasource: %s", ds.path)
				logger.info( "\t\t\tInfo: %s", ds_data['infos'])
				logger.info( "\t\t\tSave: %s", 0 if ds_data.get('nosave', 0) else 1) 
				for field, value in ds_data['fields'].items():
					if field == "status":
						value = DSSTATE_TO_CODENAME(value)
					logger.info( "\t\t\t%s = %s", field, value)

			for uuid, c_data in data['conditions_updates'].items():

				c = Condition.objects.get(uuid=uuid)

				logger.info( "\t\tCondition: %s", c.path)
				logger.info( "\t\t\tInfo: %s", c_data['infos'])
				for field, value in c_data['fields'].items():
					if field in ["status","statuslast", "statuslastnotable"]:
						value = CSTATE_TO_CODENAME(value)
					logger.info( "\t\t\t%s = %s", field, value)

		return updates_data

	def do_update_local(self, logger, tenant, datagroup_id, options, updates_data):

		do_update      = options['do_update']
		broker_address = options['broker']

		agent = None
		if not do_update:
			return

		logger.debug( "########################################################" )
		logger.debug( "###################### UPDATING ########################" )
		logger.debug( "########################################################" )

		logger.debug( "Creating update event" )
		fake_event = AgentUpdateEvent(time.time(), tenant.uuid, agent, updates_data)

		logger.debug("Sending update data: %s", '\n'.join([ 'DATA> %s' % (l) for l in  pprint.pformat(fake_event, width=75).split('\n')]) )

		if broker_address is not None:

			logger.info("Sending update data to sanetd broker" )

			events = [fake_event, ]

			connection = self.create_connection(broker_address)
			connection.open()

			events = BaseAgent.encode_update_events(events)

			connection.send_msg( 'sanetd-events', events, blocking=False)

			connection.close()
			logger.info("Sent. Exit." )

		return


	def execute_remotely(self, datagroup, loglevel):

		from sanet_common.sanetdefs.ipc import AGENT_CHECKER_QUEUE

		agent = datagroup.agent

		BROKER_ADDRESS = 'sanet/sanet@localhost'

		# Create
		connection = self.create_connection(BROKER_ADDRESS)
		connection.open()

		# Bind
		reply_address = 'temp-check-%s-%s' % (os.getpid(), create_global_uuid())
		connection.bind(reply_address)

		# Send request
		requestid = str(time.time())
		request = encode_request_with_id("exec_datagroup", requestid, {'datagroup_id': datagroup.uuid, 'loglevel': loglevel })
		connection.send_msg( AGENT_CHECKER_QUEUE(agent.uuid), request, blocking=False)		

		# Receive
		(addr, msg) = connection.receive_msg()
		response = decode_response(msg)

		# Close
		connection.close()

		if response['id'] != requestid:
			raise Exception("Received response %s instead of %s" % (response['id'], requestid))

		if response['status'] == RESPONSE_SUCCESS:

			data = response['content']

			text     = data['log']            # execution log 
			updates  = data['updates_data']   # update data generated by execution

			print term_color(TERM_COLOR_GREEN , "Response from remote agent:")
			for line in text.split('\n'):
				print term_color(TERM_COLOR_GREEN ,'AGENT RESPONSE>'), line
		else:

			error_text = response['content']

			print term_color(TERM_COLOR_YELLOW, "Error on remote agent:")
			for line in error_text.split('\n'):
				print term_color(TERM_COLOR_GREEN ,'AGENT RESPONSE>'), line

		return


	def create_connection(self, broker_address):

		connect_options = {}
		self.logger.debug("Broker address: %s", broker_address )

		broker_auth, broker_address = broker_address.split('@')

		broker_auth    = broker_auth.split('/')
		broker_address = broker_address.split(':')

		host = broker_address[0]
		if len(broker_address) > 1: 
			try:
				if broker_address[1]:
					connect_options['port']     = int(broker_address[1])
			except Exception, e:
				pass

		if broker_auth:
			connect_options['username'] = broker_auth[0]
			if len(broker_auth) > 1:
				connect_options['password'] = broker_auth[1]

		self.logger.debug("Opening connection with server %s", host )
		self.logger.debug("Options:")
		for k,v in connect_options.items():
			self.logger.debug("\t%s = %s", k, v)

		c = IPCConnection( host, **connect_options)

		self.logger.debug("Connection created!")

		return c

