# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.
"""

extract_conditions
====================================================================================

Syntax:

	::
		
		sanet-manage extract_conditions [options]

List all the defined conditions.

Options:

	::
		
		--tenant <tenant>          Specify the tenant to use.
		
		--priority_value <num>     Filter by priority value *num*.
		--priority_label <label>   Filter by priority level *name*.

Output format:

	::
		
		<element type> , <primary flag> , <path>

Example:

	::
		
		node,*,localhost;;icmp-reachability;reach

"""

from optparse import make_option

from sanet_common.sanetdefs.constants import ELEMENT_KIND_STR

from core.management.commands import TenantBasedCommand

from core.models import Condition, PriorityLabel

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(TenantBasedCommand):

	help = "Estract conditions"

	#option_list = TenantBasedCommand.option_list + (
	#	make_option('--priority_value'  , action='store'     , dest='priority_value' , default=None , help=''),
	#	make_option('--priority_label'  , action='store'     , dest='priority_label' , default=None , help=''),
	#)

	def add_arguments(self, parser):
		super(Command, self).add_arguments(parser)

		parser.add_argument('--priority_value'  , action='store'     , dest='priority_value' , default=None , help=''),
		parser.add_argument('--priority_label'  , action='store'     , dest='priority_label' , default=None , help=''),
	
	def handle(self, *args, **options):

		tenant = self.get_tenant(options)

		conditions = Condition.active_objects

		if options['priority_value']:

			v = int(options['priority_value'])

			filters = {}
			filters['priority'] = v
			conditions = conditions.filter(**filters)

		if options['priority_label']:

			label = options['priority_label']

			p = PriorityLabel.objects.get(label=label)
			conditions = p.get_conditions()

		conditions = conditions.select_related('element__kind').order_by('path').values_list('element__kind', 'primary', 'path')

		for r in conditions:
			print "%s,%s,%s" % (ELEMENT_KIND_STR[r[0]], '*' if r[1] else ' ', r[2])

