# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
fetch_ds_rrddata
========================================================================

Sintassi:

	::
		
		sanet-manage fetch_ds_data [-h] [--tenant <name> ] -- <path | uuid>  [-s <start>] [-e <end>]

.. important:: E' fondamentale che la sequenza di escape '--' venga indicata prima del <path> o <uuid>.

Opzioni:

	::
		
		-h               Mostra l'help
		--tenant <name>  Specifica su quale tenant cercare il datasource <path>
		-s <start>       Istante temporale di inizio estrazione.
		-e <end>         Istante finale di inizio estrazione.

I parametri *start* e *end* permettono di specificare un valore nei seguenti formati:

	* timestamp (es. 121342342)
	* range tempolae supportato da RRDTOOL (es: -400s, -8h, ecc.)

Esempio:

	::
		
		sanet-manage fetch_ds_rrddata -- 'localhost;;cpu-hr;cpuperc-hr' -s -408s

"""

import os
import sys
import subprocess

from conf import get_configuration

from core.models import DataSource

from core.api.datasource import init_datasource_store

from core.management.commands import TenantBasedCommand

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(TenantBasedCommand):

	help = "Fetch RRD data for the specified datasource."

	#option_list = TenantBasedCommand.option_list 

	def add_arguments(self, parser):

		super(Command, self).add_arguments(parser)

		#
		# Positional
		#
		parser.add_argument('path_or_uuid', action='store'     , nargs='?'       , default=None, help=''),

		parser.add_argument('rrdargs'     , action='store'     , nargs='*'           , help='')

	def handle(self, *args, **options):

		path_or_uuid = options['path_or_uuid']
		args         = options['rrdargs']

		if not args:
			print "usage: command -- <path> <rrdtool options>"
			sys.exit(-1)

		configuration = get_configuration()

		tenant = self.get_tenant(options)

		datasource_store = init_datasource_store(configuration.DATASOURCE_STORAGE_HANDLER, configuration.DATASOURCE_STORAGE_PARAMS)

		try:
			ds = DataSource.objects.get(uuid=path_or_uuid)
		except DataSource.DoesNotExist,e:
			ds = DataSource.objects.get(path=path_or_uuid)

		dirname, filename = datasource_store._get_datasource_rrd_filename(tenant.name, ds.uuid)

		#args = sys.argv[3:]

		command_args = [ 'rrdtool', 'fetch', filename, 'AVERAGE' ] + args

		subprocess.call(command_args)

