# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.
"""

filter_poller_log
========================================================================

This command is for debug only. Do not use.

"""

import os
import json
import csv
import time
import datetime

from optparse import make_option

#from django.core.management.base import BaseCommand

from core.management.commands import SanetInternalCommand, TenantBasedCommand

from core.models import SchedulingGroup, MonitoredElement, DataGroup
from core.api.base import db_set_current_tenant_schema

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class ALIGN:
	LEFT, RIGHT = '-', ''

class Column(list):
	def __init__(self, name, data, align=ALIGN.RIGHT):
		list.__init__(self, data)
		self.name = name
		width = max(len(x) for x in data + [name])
		self.format = ' %%%s%ds ' % (align, width)

class Table:
	def __init__(self, *columns):
		self.columns = columns
		self.length = max(len(x) for x in columns)
	def get_row(self, i=None):
		for x in self.columns:
			if i is None:
				yield x.format % x.name
			else:
				yield x.format % x[i]
	def get_rows(self):
		yield '|'.join(self.get_row(None))
		for i in range(0, self.length):
			yield '|'.join(self.get_row(i))

	def __str__(self):
		return '\n'.join(self.get_rows())

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

import datetime

DATETIME_FORMAT = '%H:%M:%S'

def TS_FORMAT(ts):
	if ts:
		return datetime.datetime.fromtimestamp(ts).strftime(DATETIME_FORMAT)
	else:
		return '-'

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

def tail(f, n, offset=0):
	"""Reads a n lines from f with an offset of offset lines."""

	avg_line_length = 74
	to_read = n + offset

	while 1:
		try:
			f.seek(-(avg_line_length * to_read), 2)

		except IOError:
			# woops.  apparently file is smaller than what we want
			# to step back, go to the beginning instead
			f.seek(0)

		pos = f.tell()

		lines = f.read().splitlines()
		if len(lines) >= to_read or pos == 0:
			return lines[-to_read:offset and -offset or None]

		avg_line_length *= 1.3

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(SanetInternalCommand, TenantBasedCommand):

	help = "Analize poller log looking for data of given scheduling group."

	#option_list = BaseCommand.option_list + (
	#	make_option('--scheduling_group', action='store', dest='sched_g', type='string', help='scheduling group name'),
	#	make_option('--lines', action='store', dest='linen', type='int', help='max number of lines to analize'),
	#	make_option('--tenant', action='store', dest='tenant', default=None, help='tenant name'),
	#)

	def add_arguments(self, parser):
		super(Command, self).add_arguments(parser)

		parser.add_argument('--scheduling_group', action='store', dest='sched_g', type='string', help='scheduling group name'),
		parser.add_argument('--lines', action='store', dest='linen', type='int', help='max number of lines to analize'),
		parser.add_argument('--tenant', action='store', dest='tenant', default=None, help='tenant name'),

	def handle(self, *args, **kwargs):

		if kwargs['tenant']:
			db_set_current_tenant_schema(kwargs['tenant'])

		if kwargs['sched_g']:
			min_gap, max_parallel, max_deadline, uuid = SchedulingGroup.objects.filter(name=kwargs['sched_g']).values_list('min_gap', 'max_parallel', 'max_deadline', 'uuid')[0]
			sched_g = kwargs['sched_g']

		else:
			print 'Usage Error'
			return

		element_list = []

		try:
			tmp = MonitoredElement.objects.filter(scheduling_group_id=uuid).values_list('name')
			element_list += [e[0] for e in tmp]
		except IndexError:
			pass

		try:
			tmp = DataGroup.objects.filter(scheduling_group_id=uuid).values_list('path')[0]
			element_list += [e[0] for e in tmp]
		except IndexError:
			pass

		SANET_HOME = os.environ['SANET_HOME']
		path = os.path.join(SANET_HOME, 'var/log/poller.log')
		linen = kwargs['linen']
	
		lines = ''
		problems = [[], [], [], []]
	
		with open(path, 'r') as fd:
			lines = tail(fd, linen)

		date_list = []
		path_list = []

		if not lines:
			print 'Nothing to show'
			return

		new_lines = []
	
		for line in lines:
			line1 = line.split()

			if line1[6] == '[scheduler]' and any(el in line1[11] for el in element_list):
				new_lines.append(line)

		for idx, line in enumerate(new_lines):
			line = line.split()

			try:
				line2 = new_lines[idx-1].split()
			except IndexError:
				pass

			if len(line) < 12:
				continue

			if idx > 0:
				t2 = time.mktime(datetime.datetime.strptime(line[0] + ' ' + line[1], '%Y-%m-%d %H:%M:%S,%f').timetuple())
				t1 = time.mktime(datetime.datetime.strptime(line2[0] + ' ' + line2[1], '%Y-%m-%d %H:%M:%S,%f').timetuple())

				if t2 - t1 < min_gap:
					problems[0].append(line[0] + ' ' + line[1])
					problems[1].append(line[11])
					problems[2].append(line2[0] + ' ' + line2[1])
					problems[3].append(line2[11])

			date_list.append(line[0] + ' ' + line[1])
			path_list.append(line[11])

		print "Scheduling group", sched_g
		print "\tmin_gap:", min_gap
		print "\tmax_parallel:", max_parallel
		print "\tmax_deadline:", max_deadline
		print "------------------------------------------------------------------------------"

		print Table (
			Column('TIME', date_list),
			Column('DATAGROUP', path_list)
		)

		print "\nProblems found:"

		print Table (
			Column('FIRST DG_EXEC TIME', problems[0]),
			Column('FIRST DG_EXEC PATH', problems[1]),
			Column('SECOND DG_EXEC TIME', problems[2]),
			Column('SECOND DG_EXEC PATH', problems[3])
		)
