# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
fix_dead_users
========================================================================

Poiche' il database e' suddiviso in schemi, quando un utente viene cancellato
il sistema di "delete cascade" di django potrebbe non "azzerare" correttamente
i field "owner" in diverse tabelle.

Questo comando verifica e azzera il field "owner" se associato ad un User.id
non piu' esistente.

"""
import logging

from django.utils.translation import ugettext as _

from django.db import connection
#from django.db import transaction

from django.contrib.auth.models import User

from core.db_utils import transaction_commit_on_success, transaction_set_dirty, transaction_commit

from core.models import Tenant
from core.models import OwnerUserModel

from core.api.base import db_version

from core.management.commands import TenantBasedCommand

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(TenantBasedCommand):

	help = _("Check if there are incosistencies with owner user")

	sanet_internal = True

	#option_list = TenantBasedCommand.option_list 

	@transaction_commit_on_success
	def handle(self, *args, **options):
		
		logger = logging.getLogger('sanet.global')

		if db_version() < 33:
			return

		ids = ','.join([ str(u.id) for u in User.objects.all() ])
		
		if not ids:
			return

		tenant_name = options['tenant']
		if tenant_name == None:
			tenants  = Tenant.objects.all()
		else:
			tenants  = [ Tenant.objects.get(name=tenant_name) ]

		cursor = connection.cursor()

		for tenant in tenants:

			logger.info( "Checking for deleted owners in '%s'", tenant.name)

			tenant.db_set_current_schema(db_cursor=cursor)

			for table in OwnerUserModel.OWNER_TABLES:

				query = 'UPDATE %s set owner_id = null WHERE owner_id not in (%s)' % (table, ids)
				cursor.execute(query)

				if cursor.rowcount:
					logger.info("\t" + "UPDATED %s = %s", table, cursor.rowcount)

			transaction_set_dirty()

		transaction_commit()
