# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os
import sys
import time
import pprint
import logging
from optparse import make_option


from sanet_common.uuids import is_uuid
from sanet_common.sanetdefs.constants import CSTATE_FROM_CODENAME, CSTATE_UP, CSTATE_DN

from django.conf import settings
from core.models import DataGroup
from core.api.mail import init_mail_sender_from_conf

from server.sanetd.sanetdlogic import force_datagroup_status
from server.sanetd.sanetdlogic import load_alarm_handlers

from core.management.commands import TenantBasedCommand

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(TenantBasedCommand):

	help = "Force status change for conditions of a given datagroup."

	#option_list = TenantBasedCommand.option_list + (
	#	make_option('-l'          , action='store'     , dest='loglevel'  , default=20   , help='Set log level (<20 DEBUG, <30 INFO, <40 WARNING, <50 ERROR.'),
	#	make_option('--status'    , action='store'     , dest='status'    , default="UP" , help='Specify reason.'),
	#	make_option('--reason'    , action='store'     , dest='reason'    , default=None , help='Specify reason.'),
	#	make_option('--conditions', action='store'     , dest='conditions', default=None , help='Specify conditions to update.'),
	#)

	def add_arguments(self, parser):
		super(Command, self).add_arguments(parser)

		parser.add_argument('-l'          , action='store'     , dest='loglevel'  , default=20   , help='Set log level (<20 DEBUG, <30 INFO, <40 WARNING, <50 ERROR.'),
		parser.add_argument('--status'    , action='store'     , dest='status'    , default="UP" , help='Specify reason.'),
		parser.add_argument('--reason'    , action='store'     , dest='reason'    , default=None , help='Specify reason.'),
		parser.add_argument('--conditions', action='store'     , dest='conditions', default=None , help='Specify conditions to update.'),

		#
		# Positional arguments
		#
		parser.add_argument('datagroup_id', nargs='?', default='')

	def handle(self, *args, **options):
		
		datagroup_id = options['datagroup_id']

		status_code = options['status']
		status = CSTATE_FROM_CODENAME(status_code)
		if (status not in [CSTATE_UP, CSTATE_DN]):
			raise Exception("Invalid state %s" % (status_code))

		reason = options['reason']
		if reason is None:
			reason = 'Status changed by administrator'

		conditions = options['conditions']
		if conditions is not None:
			conditions = conditions.split(',')

		#
		# Set the right tenant
		#
		tenant = self.get_tenant(options)
		
		#
		# Get Datagroup
		#
		if not is_uuid(datagroup_id):
			datagroup = DataGroup.objects.get(path=datagroup_id)
			datagroup_id = datagroup.uuid
		else:
			datagroup = DataGroup.objects.get(uuid=datagroup_id)

		if not datagroup.is_passive():
			raise Exception("Datagroup is not passive (minperiod=%s > 0)" % (datagroup.minperiod))

		#
		# Init Log
		#
		l = int(options['loglevel'])
		self.logger = None
		logger = self.init_log(l)

		alarms_to_send = force_datagroup_status(datagroup, logger, status, time.time(), reason=reason, selected_condition_names=conditions)

		logger.info("FORCING")
		
		if not alarms_to_send:
			return
	
		#
		# Process Alarm (send to entables or something else)
		#
		logger.info("Loading alarm handlers = %s", settings.SANET_ALARM_PROCESSORS)
		alarm_handlers = load_alarm_handlers(logger)
		
		mail_sender = init_mail_sender_from_conf(logger)


		for alarm_data in alarms_to_send:

			for handler in alarm_handlers:
				logger.info("[alarm] Processing alarm with handler %s", handler)
				
				processed = handler.process_alarm(mail_sender, alarm_data)
				if processed:
					self.logger.info("[alarm] Processing alarm with handler %s", handler)
					break


	def init_log(self, log_level):

		format_str = '%(asctime)s %(levelname)s %(threadName)s > %(message)s'
		logger = logging.getLogger('test')
		logger.setLevel( log_level )

		self.logger = logger

		hdlr = logging.StreamHandler()
		hdlr.setFormatter( logging.Formatter(format_str) )
		hdlr.setLevel( log_level )
		logger.addHandler(hdlr)

		self.log_hdlr = hdlr

		return logger		
