# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

get_config
========================================================================

Lancio: 

	::
		
		sanet-manage get_config [ option_name ]

Produce un dump su standard output della configurazione corrente del sistema caricata dal file di configurazione "settings.py".

Questo comando e' utile per estrarre informazioni parziali di configurazione: 

	::
		
		$ sanet-manage get_config  SANET_INSTALL_DIR
		/usr/share/sanet

Per avere tutta la configurazione: 

	::
		
		$ sanet-manage get_config
		
		...
		BASE_DIR /opt/sanet3
		BASE_URL_PREFIX 
		BASE_WEB_URL_PREFIX web
		BROKER_ADDRESS ('localhost',)
		BROKER_AUTH ('sanet', 'sanet')
		COMMENTS_ALLOW_PROFANITIES False
		ADMINS ()
		ADMIN_FOR ()
		ALLOWED_INCLUDE_ROOTS ()
		APPEND_SLASH True
		AUTHENTICATION_BACKENDS ('sanet_common.django.auth.backends.dummy.DummyBackend',)
		BASE_DIR /opt/sanet3
		BASE_URL_PREFIX 
		BASE_WEB_URL_PREFIX web
		BROKER_ADDRESS ('localhost',)
		BROKER_AUTH ('sanet', 'sanet')
		CACHES {'default': {'LOCATION': '', 'BACKEND': 'django.core.cache.backends.locmem.LocMemCache'}}
		...
		...
		...
		...
"""

from django.conf import settings

from core.management.commands import SanetBaseCommand

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

def _getattr_safe(d, k):
	try:
		return d[k]
	except Exception, e:
		return getattr(d, k)

class Command(SanetBaseCommand):

	help = "Print Sanet configuration parameters on standard output."

	#option_list = SanetBaseCommand.option_list

	def add_arguments(self, parser):
		parser.add_argument('parameters'     , action='store'     , nargs='*'           , help='')

	def handle(self, *args, **options):

		if len(options['parameters']) > 0:
			if options['parameters'][0] == 'DATABASE_NAME':
				return settings.DATABASES['default']['NAME']
			else:
				data = settings

				k = options['parameters'][0]
				while k.find('.') > -1:
					k, k2 = k.split('.', 1)
					data = _getattr_safe(data, k)
					k = k2

				print _getattr_safe(data, k)
		else:
			print "DATABASE_NAME", settings.DATABASES['default']['NAME']

			for s in sorted(dir(settings)):
				if s.startswith('__'): continue
				print s, getattr(settings, s)

