# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

get_ds_filename
=================================================

Read datasources' path (or UUID) from stdin and returns the path of the related store file (RRD FILE)

Uso:

	::
		sanet-manage get_ds_filename [options]

Opzioni:

	::
		
		--tenant name   	Seleziona il tenant di riferimento.
		--usauuid       	Legge da standainput UUID di datasource al posto di path.

Esempio 1:

	::
		
		sanet-manage get_ds_filename
		localhost;;cpu-hr;cpuperc-hr
		Cltr+D
		
		localhost;;cpu-hr;cpuperc-hr /home/dbilli/labs/sanet_data2/var/lib/site/rrds/481c58593e2e43aba411b39ce2c71936.rrd


Esempio 2:

	::
		
		sanet-manage get_ds_filename --useuuid
		481c58593e2e43aba411b39ce2c71936
		Cltr+D
		
		481c58593e2e43aba411b39ce2c71936 /home/dbilli/labs/sanet_data2/var/lib/site/rrds/481c58593e2e43aba411b39ce2c71936.rrd


"""

import os
import sys

from optparse import make_option

from conf import get_configuration

from core.models import DataSource
from core.api.datasource import init_datasource_store

from core.management.commands import TenantBasedCommand

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(TenantBasedCommand):

	help = "Read datasources' UUID from stdin and returns the path of the related store file (RRD FILE)"

	#option_list = TenantBasedCommand.option_list + (
	#	make_option('--useuuid'  , action='store_true', dest='use_uuid', default=False, help=''),
	#)

	def add_arguments(self, parser):
		super(Command, self).add_arguments(parser)

		parser.add_argument('--useuuid'  , action='store_true', dest='use_uuid', default=False, help=''),

	#def handle(self, cmd='', *args, **options):
	def handle(self, *args, **options):
		
		use_uuid    = options['use_uuid']
		
		configuration = get_configuration()

		tenant = self.get_tenant(options)

		datasource_store = init_datasource_store(configuration.DATASOURCE_STORAGE_HANDLER, configuration.DATASOURCE_STORAGE_PARAMS)

		idval = sys.stdin.readline()
		while idval:

			idval = idval.strip()
			try:
				if use_uuid:
					ds = DataSource.objects.get(uuid=idval)
				else:
					ds = DataSource.objects.get(path=idval)
			except DataSource.DoesNotExist,e:
				print idval, e
				break

			dirname, filename = datasource_store._get_datasource_rrd_filename(tenant.name, ds.uuid)

			print idval, filename

			idval = sys.stdin.readline()
