# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

get_ds_storeinfo
====================================================================================================

Syntax:

	::
		
		sanet-manage get_ds_storeinfo <path|id>

Returns RRDTOOL informations about the RRD file linked to the given *datasource*.

Example:

	::
		
		# sanet-manage get_ds_storeinfo c620e819a12249df8771d5919556f280
		
		filename = "............/rrds/c620e819a12249df8771d5919556f280.rrd"
		rrd_version = "0003"
		step = 60
		last_update = 1522005473
		header_size = 584
		ds[data].index = 0
		ds[data].type = "GAUGE"
		ds[data].minimal_heartbeat = 3600
		ds[data].min = NaN
		ds[data].max = NaN
		ds[data].last_ds = "U"
		ds[data].value = NaN
		ds[data].unknown_sec = 53
		rra[0].cf = "AVERAGE"
		rra[0].rows = 525600
		rra[0].cur_row = 216338
		rra[0].pdp_per_row = 1
		rra[0].xff = 5.0000000000e-01
		rra[0].cdp_prep[0].value = NaN
		rra[0].cdp_prep[0].unknown_datapoints = 0

"""

import os
import sys

from django.db.models import Q

from conf import get_configuration

from core.models import DataSource
from core.api.datasource import init_datasource_store

from core.management.commands import TenantBasedCommand

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(TenantBasedCommand):

	help = "Returns the datasource store file (RRD file) informations."

	#option_list = TenantBasedCommand.option_list 

	def add_arguments(self, parser):
		super(Command, self).add_arguments(parser)

		#
		# Positional arguments
		#
		parser.add_argument('uuid_or_path', nargs='?', default=None)

	#def handle(self, uuid_or_path, *args, **options):
	def handle(self, *args, **options):
		
		uuid_or_path = options['uuid_or_path']

		configuration = get_configuration()

		tenant = self.get_tenant(options)

		try:
			ds = DataSource.objects.get(Q(uuid=uuid_or_path)|Q(path=uuid_or_path))

			datasource_store = init_datasource_store(configuration.DATASOURCE_STORAGE_HANDLER, configuration.DATASOURCE_STORAGE_PARAMS)
			dirname, filename = datasource_store._get_datasource_rrd_filename(tenant.name, ds.uuid)

			os.system("rrdtool info %s" % (filename))

		except DataSource.DoesNotExist,e:
			print idval, e

