# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

helpme 
==============================================================================

Syntax:

	::
		
		sanet-manage helpme


Show Sanet's shell commands.

Example:

	::
		
		# sanet-manage helpme
		
		[core]
		    agentinfo                        Query running agent.
		    broker_info                      Show broker informations.
		    cache_info                       Show cache informations.
		    clean_rrds                       Print old/unused datasources files (RRD files)
		    create_datasource_store          Create datasource store file (RRD file)
		    datasource_produce_random_values Produce random values for the specified datasource.
		    dbshell2                         
		    dbusage                          Show DataBase usage (pending connections/queries)
		    exec_datagroup                   Execute an existing datagroup
		    exec_expr                        Execute a generic expression.
		    exec_timegraph                   Processe and produce timegraph's series
		    extract_conditions               Estract conditions
		    ...
		    ...
		    ...


.. note: Questo comando e' una versione modificata del comando "help" di django.

"""

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

#
# Tutti i django commands definiti nelle subapp possono dichiare un attributo di classe
# 
# 	sanet_internal = True
# 	
# Quanto questo attributo e' definito, il django command e' considerato "interno" e non viene elencato.
# 

from django.conf import settings
#from django.core.management.base import NoArgsCommand
from django.core.management.base import BaseCommand

from django.core.management import get_commands, load_command_class

from django.utils.translation import ugettext as _

from sanet_common.termutils import term_color, TERM_COLOR_CYAN, term_red

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(BaseCommand):

	#option_list = NoArgsCommand.option_list + (
	#)

	help = _("Show Sanet's shell commands.")


	def add_arguments(self, parser):
		super(Command, self).add_arguments(parser)
		parser.add_argument('--show-internals'  , action='store_true', dest='show_internals' , default=None , help=''),

	#def handle_noargs(self, **options):
	def handle(self, *args, **options):

		max_len = 0

		commands_by_app = {}
		for cmd_name, app_name in get_commands().items(): 

			if app_name.startswith('django'): 
				continue
			try:
				klass = load_command_class(app_name, cmd_name)
			except Exception, e:
				print term_red("UNABLE TO LOAD command %s.%s: %s %s" % (app_name, cmd_name, type(e), str(e)))
				continue

			is_internal = True if (hasattr(klass, 'sanet_internal') and klass.sanet_internal == True) else False
				
			# Do not show internal commands.
			if is_internal:
				if not options['show_internals']:
					continue
			else:
				if options['show_internals']:
					continue

			l = commands_by_app.get(app_name, commands_by_app.setdefault(app_name, []))
			l.append((cmd_name, klass))
			
			max_len = max(max_len, len(cmd_name))

		ROW_FORMAT = "%%-%ss %%s" % (max_len)

		for app_name in settings.INSTALLED_APPS:

			commands = commands_by_app.get(app_name, [])

			if not commands:
				continue

			section_title = "[%s]" % (app_name)

			print term_color(TERM_COLOR_CYAN, section_title)

			for cmd_name, klass in sorted(commands):
				s = "    " + ROW_FORMAT % (cmd_name, klass.help)
				print s

			print ""
