# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.
"""

nodes_csv_export
========================================================================

Syntax:

	::
		
		sanet-manage nodes_csv_export <filter pattern> <outputfile>

Produce a csv file, in the current working directory, with the requested info about searched nodes in tenant.

Example: Export all nodes:

	::
		
		sanet-manage nodes_csv_export '' /tmp/list.csv

Example: Export all nodes with name containing 'sw-':

	::
		
		sanet-manage nodes_csv_export 'sw-' /tmp/list.csv

"""

import csv
from optparse import make_option

from core.models import Tenant
from core.management.commands import TenantBasedCommand

from customfield.models import CustomGroup, CustomField

from webui.views.objects.nodes import COL_MODEL_DICT
from webui.views.objects.nodes import perform_query, _csv_struct_encode

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

def get_custom_fields(tenant):
	
	custom_groups = CustomGroup.objects.filter(tenant=tenant)
	custom_list   = []
	custom_fields = []
	
	for group in custom_groups:
		
		fields = CustomField.objects.filter(custom_group=group)
		
		for field in fields:
			custom_list.append('"c_value_' + group.name + '_' + field.name + '".value AS "c_value_' + group.name + '_' + field.name + '"')
			custom_fields.append({ 'group': group.name, 'field_name': field.name })
	
	return custom_list, custom_fields

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(TenantBasedCommand):

	help = "Produce a csv file, in the current working directory, with the requested info about searched nodes in tenant"

	#option_list = TenantBasedCommand.option_list + (
	#	make_option('--length'      , action='store'     , dest='length'       ,type='int',    help=''),
	#	make_option('--column_list' , action='store'     , dest='column_list'  ,type='string', help=''),
	#)

	def add_arguments(self, parser):
		super(Command, self).add_arguments(parser)

		parser.add_argument('--length'      , action='store'     , dest='length'       ,type=int,    help=''),
		parser.add_argument('--column_list' , action='store'     , dest='column_list'  ,type=str, help=''),

		#
		# Positional
		#
		parser.add_argument('search'     , action='store'     , nargs='?'           , help='')
		parser.add_argument('output_file', action='store'     , nargs='?'           , help='')


	#def handle(self, search, output_file, *args, **options):
	def handle(self, *args, **options):
		
		search      = options['search']
		output_file = options['output_file']
		
		tenant = self.get_tenant(options)
		
		length = options['length']
		if length is not None:
			offset = 0
		else:
			offset = None

		custom_list, custom_fields = get_custom_fields(tenant)
		
		if options['column_list'] is not None:
			col_file    = kwargs['column_list']

			fd          = open(col_file)
			column_list = fd.read().splitlines()
			fd.close()
		else:
			column_list = []
			for k, column in COL_MODEL_DICT.items():
				column_list.append(column)

			column_list += custom_list

		order = (column_list[0], 'asc')

		#
		# PERFORM
		#
		res = perform_query(tenant, search, order, column_list, [], custom_fields, None, "csv", limit=length, offset=offset)

		#
		# WRITE
		#
		res = _csv_struct_encode(res)

		fd = open(output_file, 'w+')
		
		cw = csv.writer(fd)
		cw.writerows(res)
		
		fd.close()
