# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
push_alarm
============================================================================

Push a raw alarm in Entables pipeline.

TODO

"""

import os
import sys
import time
import pprint
import logging
from optparse import make_option

# Global Sanet libraries
from sanet_common.uuids import create_global_uuid
from sanet_common.ipc import *

from sanet_common.sanetdefs.constants import ALARM_PUSHED

# Local libraries (Sanet+Django related)
from core.management.commands import TenantBasedCommand

from server.sanetd.sanetdlogic import create_ipc_connection_from_config

from entables.lib import encode_alarm

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(TenantBasedCommand):

	help = "Push a raw alarm in Entables pipeline."

	#option_list = TenantBasedCommand.option_list + (
	#	make_option('-L'             , action='store'     , dest='loglevel'     , default=20   , help='Set log level (<20 DEBUG, <30 INFO, <40 WARNING, <50 ERROR.'),
	#	
	#	make_option('--stdin'        , action='store_true', dest='use_stdin'    , default=False, help='Load event data from standard input in JSON format'),
	#	make_option('--use-env'      , action='store_true', dest='use_env'      , default=False, help='Load event data from environment symbols PUSH_EVENT_<name>'),
	#)

	def add_arguments(self, parser):
		super(Command, self).add_arguments(parser)

		parser.add_argument('-L'             , action='store'     , dest='loglevel'     , default=20   , help='Set log level (<20 DEBUG, <30 INFO, <40 WARNING, <50 ERROR.')

		parser.add_argument('--stdin'        , action='store_true', dest='use_stdin'    , default=False, help='Load event data from standard input in JSON format')
		parser.add_argument('--use-env'      , action='store_true', dest='use_env'      , default=False, help='Load event data from environment symbols PUSH_EVENT_<name>')

		#
		# Positional
		#
		parser.add_argument('parameters'     , action='store'     , nargs='*'           , help='')

	def handle(self, *args, **options):

		tenant = self.get_tenant(options)
		
		parameters = options['parameters']

		#
		# Init Log
		#
		l = int(options['loglevel'])
		self.logger = None
		logger = self.init_log(l)


		#
		# Read raw alarm
		#

		if len(parameters) > 0:
			logger.debug("Reading alarm from command line argument")
			raw_alarm = parameters[0]
		elif options['use_stdin']:
			logger.debug("Reading alarm from standard input")
			raw_alarm = sys.stdin.read()
		elif options['use_env']:
			logger.debug("Reading alarm from environment")
			
			d = {}
			for k, v in os.environ.items():
				if k.startswith('PUSH_EVENT_'):
					name = k[ len('PUSH_EVENT_') : ]
					d[ name ] = v
					
			raw_alarm = json.dumps(d)
		else:
			raise Exception("You must specify alarm\'s data")

		alarm_data = json.loads(raw_alarm)

		#
		# Add minimum fields
		#
		if not alarm_data.has_key('tenantid'): 
			logger.warning("Adding tenant ID to message: %s %s", tenant.name, tenant.uuid)			
			alarm_data['tenantid'] = tenant.uuid

		if not alarm_data.has_key(u'uuid'): 
			alarm_id = create_global_uuid()
			logger.warning("Adding unique ID to alarm: %s", alarm_id)
			alarm_data['uuid'    ] = alarm_id
			
		alarm_data['alarmtype'] = ALARM_PUSHED
		
		#Test raw binary values
		#alarm_data['xxx'] = '\x07\xe1\n\x06\x0f\x1d(\x00+\x02\x00'

		#
		# ENCODE
		#
		msg = encode_alarm(alarm_data)
		
		#
		# Sending data to entables
		#
		logger.debug("Init qpid connection")
                
		alarm_connection = create_ipc_connection_from_config(logger)
		alarm_connection.open()
                
		logger.debug("Sending")
                
                alarm_connection.send_msg( 'entables-alarms' , msg, blocking=False)                
		alarm_connection.close()

		logger.debug("Sent")

		self.remove_logger()
		
		return


	def init_log(self, log_level):

		format_str = '%(asctime)s %(levelname)s %(threadName)s > %(message)s'
		logger = logging.getLogger('test')
		logger.setLevel( log_level )

		self.logger = logger

		hdlr = logging.StreamHandler()
		hdlr.setFormatter( logging.Formatter(format_str) )
		hdlr.setLevel( log_level )
		logger.addHandler(hdlr)

		self.log_hdlr = hdlr

		return logger

	def remove_logger(self):

		if self.logger:
			self.logger.removeHandler(self.log_hdlr)

