# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.
"""
remove_old_logs
========================

Rimuove (per uno o tutti i tenant) alcuni dati piu' vecchi di una data specificata.

La sintassi e':

::
	
	remove_old_logs [options] <YYYY-MM-GG>



I dati che vengono rimossi sono:
- log delle variazioni di stato delle condition
- allarmi di entables


TODO

"""
import time
from optparse import make_option

from django.db import connection
from django.db import transaction
from django.core.management.base import BaseCommand
from django.core.management.base import CommandError

from core.models import Tenant
from core.api.base import tenant_get, tenant_get_primary

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(BaseCommand):
	
	DATE_FORMAT = "%Y-%m-%d"
	
	help = "Delete logs/alarms from db older thant the specified date."
	
	#option_list = BaseCommand.option_list + (
	#	make_option('--tenant', action='store'     , dest='tenant'    , default=None , help=''),
	#	make_option('--all   ', action='store_true', dest='tenant_all', default=False, help=''),
	#)

	def add_arguments(self, parser):

		super(Command, self).add_arguments(parser)

		parser.add_argument('--tenant', action='store'     , dest='tenant'    , default=None , help=''),
		parser.add_argument('--all   ', action='store_true', dest='tenant_all', default=False, help=''),

		#
		# Positional
		#
		parser.add_argument('date', action='store'     , nargs='?'       , default=None, help=''),

	#def handle(self, date='', *args, **options):
	def handle(self, *args, **options):

		date = options['date']

		#
		# Set the right tenant
		#
		if options['tenant_all']:
			tenants = Tenant.objects.all()
		else:
			tenant_name = options['tenant']
			if tenant_name == None:
				tenant  = tenant_get_primary()
				if tenant == None:
					raise Exception("No primary tenant found. You must specify a tenant name")
			else:
				tenant = tenant_get(tenant_name)
			
			tenants = [tenant]
		
		try:
			ts = int(time.mktime(time.strptime(date, self.DATE_FORMAT)))
		except Exception, e:
			raise CommandError("You must specify a date with format YYYY-MM-DD")
			
		r = raw_input("Are you sure [y/N] ?")
		if r != "y":
			return

		with transaction.atomic():

			for tenant in tenants:
				tenant.db_set_current_schema()

				print "Tenant:", tenant.name

				cursor = connection.cursor()

				print "Removing entables log..."
				cursor.execute("DELETE FROM entables_alarmarchive WHERE ctime::timestamp  <  to_timestamp( %s );" % (ts) )
				print "\tDeleted %s records." % (cursor.rowcount)                           

				print "Removing alarms..."                                                   
				cursor.execute("DELETE FROM alarm                 WHERE mtime             <                %s  ;" % (ts) )
				print "\tDeleted %s records." % (cursor.rowcount)                           

				print "Removing events..."                                                   
				cursor.execute("DELETE FROM event                 WHERE timestamp         <                %s  or mtime < %s;" % (ts, ts) )
				print "\tDeleted %s records." % (cursor.rowcount)

				#transaction.commit_unless_managed()

			#aconnection.close()
