# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

.. _sanetdinfo:

sanetdinfo
==========================================================================================

Send commands to Sanet main daemon (sanetd).

.. warning:: This is an internal Sanet command. It is not recommended to use this command.

"""

import os
import sys
import time
from optparse import make_option

from django.conf import settings

from core.management.commands import SanetBaseCommand

from server.sanetd.sanetdctl import sanetd_ctl

from conf import get_configuration

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(SanetBaseCommand):

	help = "Send commands to Sanet main daemon (sanetd)"

	#option_list = SanetBaseCommand.option_list + (
	#	make_option('--instance', action='store', dest='instance'     , default=None, help=''),
	#)

	def add_arguments(self, parser):
		super(Command, self).add_arguments(parser)
		parser.add_argument('--instance'     , action='store', dest='instance'     , default=False, help=''),
		#
		# Positional
		#
		parser.add_argument('cmd', action='store'     , nargs='?'       , default=None, help=''),

	#def handle(self, cmd='', *args, **options):
	def handle(self, *args, **options):

		cmd = options['cmd']

		t0 = time.time()

		config = get_configuration()

		if not cmd:
			cmd = 'info'

		broker = '%s/%s@%s' % (config.BROKER_AUTH[0], config.BROKER_AUTH[1], config.BROKER_ADDRESS[0])

		instance_id = options['instance']
		if instance_id:
			instances = [ instance_id ]
		else:
			instances = settings.SANETD_INSTANCES

		for instance_id in instances:
			try:
				response = sanetd_ctl(broker, cmd, list(args), instance_id=instance_id)
			except Exception, e:
				response = "Error with instance %s: %s:%s" % (instance_id, type(e), str(e))

			print response
			print ""
