# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.
"""

Make statistics about scheduling groups
========================================================================


"""

import os
import sys

import json
import csv

from optparse import make_option

#from django.core.management.base import BaseCommand
from core.management.commands import TenantBasedCommand

#from sanet_poller.lib.debug import DebugSchedulingGroupDataGroupChecker

from core.models import SchedulingGroup
#from core.api.base import db_set_current_tenant_schema

# ----------------------------------------------------------------------#
#                                                                       #
# ----------------------------------------------------------------------#

class ALIGN:
	LEFT, RIGHT = '-', ''

class Column(list):
	def __init__(self, name, data, align=ALIGN.RIGHT):
		list.__init__(self, data)
		self.name = name
		width = max(len(x) for x in data + [name])
		self.format = ' %%%s%ds ' % (align, width)

class Table:
	def __init__(self, *columns):
		self.columns = columns
		self.length = max(len(x) for x in columns)
	def get_row(self, i=None):
		for x in self.columns:
			if i is None:
				yield x.format % x.name
			else:
				yield x.format % x[i]
	def get_rows(self):
		yield '|'.join(self.get_row(None))
		for i in range(0, self.length):
			yield '|'.join(self.get_row(i))

	def __str__(self):
		return '\n'.join(self.get_rows())

# ----------------------------------------------------------------------#
#                                                                       #
# ----------------------------------------------------------------------#

import datetime

DATETIME_FORMAT = '%H:%M:%S'

def TS_FORMAT(ts):
	if ts:
		return datetime.datetime.fromtimestamp(ts).strftime(DATETIME_FORMAT)
	else:
		return ''

# ----------------------------------------------------------------------#
#                                                                       #
# ----------------------------------------------------------------------#

class Command(TenantBasedCommand):
	
	help = "Produce statistics based on scheduling groups activities."
	
	#option_list = TenantBasedCommand.option_list + (
	#	make_option('--scheduling_group', action='store', dest='sched_g', type='string', default=None, help=''),
	#)

	def add_arguments(self, parser):
		super(Command, self).add_arguments(parser)

		parser.add_argument('--scheduling_group', action='store', dest='sched_g', type='string', help=''),
	
	def handle(self, *args, **kwargs):
		return
		if 'DEBUG_SCHEDULINGGROUP_CHECKER' not in os.environ:
			print >>sys.stderr, ""
			print >>sys.stderr, ""
			print >>sys.stderr, ""
			print >>sys.stderr, "DEBUG_SCHEDULINGGROUP_CHECKER not defined in your environment!"
			print >>sys.stderr, ""
			print >>sys.stderr, "Set: "
			print >>sys.stderr, "    export SANET_DEBUG_SCHEDULINGGROUP_CHECKER=1"
			print >>sys.stderr, ""
			print >>sys.stderr, "and relaunch your agents."
			print >>sys.stderr, ""
			sys.exit(-1)
		
		tenant = self.get_tenant(kwargs)

		sched_g = kwargs['sched_g']
			
		path = DebugSchedulingGroupDataGroupChecker.get_log_file(sched_g)


		#
		# Elaborazione
		#
		count_by_group = {}

		data = []

		with open(path, 'r') as fd:
			for line in fd:
				datalog = json.loads(line)
	
				ltype = datalog.get('type',None)
				gname = datalog.get('group',None) 
				if gname:
					v = count_by_group.get(gname, 0)
					
					if ltype == 'start':
						v += 1
					else:
						v = max(0, v-1)
						
					count_by_group[gname] = v
					
					datalog['P'] = v
					datalog['type'] = ltype
					#print gname, ltype, v


				#if datalog.get('type',None) == 'start':
				#	continue
				data.append(datalog)



		stats = {}

		type_list      = []
		datagroup_list = []
		minperiod_list = []
		lastdone_list  = []
		scheduled_list = []
		start_list     = []
		end_list       = []
		delay_list     = []
		s_delay_list  = []
		execution_time = []

		parallel_list  = []

		for elem in data:
			type_list.append(  elem['type'] )
			datagroup_list.append( str(elem['datagroup'         ]) )
			minperiod_list.append( str(elem['minperiod'         ]) )
			#lastdone_list .append( str(elem['lastdone human'    ]) )
			#scheduled_list.append( str(elem['scheduled ts human']) )
			#start_list    .append( str(elem['ts start human'    ]) )
			#end_list      .append( str(elem['ts end human'      ]) )


			lastdone_list .append( TS_FORMAT(elem['lastdone'    ]) )
			scheduled_list.append( TS_FORMAT(elem['scheduled ts']) )
			start_list    .append( TS_FORMAT(elem['ts start'    ]) )
			end_list      .append( TS_FORMAT(elem['ts end'      ]) )
			delay_list    .append( str(elem['ts start'          ] - elem['scheduled ts']) )
			#waiting_time  .append( str(elem['waiting time'      ]) )

			parallel_list  .append( str(elem.get('P',0)) )

			s_delay_list.append( str(elem['scheduling delay'] if 'scheduling delay' in elem.keys() else '-') )
			v = elem['time elapsed']

			execution_time.append( ("%.6f" % float(v)) if v is not None else '-' )

		print Table(

			Column('TYPE'           , type_list     ),
			Column('DATAGROUP'      , datagroup_list),
			Column('MINPERIOD'      , minperiod_list),
			Column('LASTDONE'       , lastdone_list ),
			Column('SCHEDULED'      , scheduled_list),
			Column('SCHED DELAY'    , s_delay_list  ),
			Column('EXECUTION START', start_list    ),
			Column('EXECUTION END'  , end_list      ),
			Column('EXECUTION DELAY', delay_list    ),
			#Column('WAITING TIME'   , waiting_time  ),
			Column('EXECUTION TIME' , execution_time),
			Column('PARALLEL'       , parallel_list ),
		)

		#print "\n"
		#print "--------------------------------------"
		#print "Scheduling group: ", kwargs['sched_g']
		#print "\tmin_gap: ", min_gap
		#print "\tmax_parallel: ", max_parellel
		#print "\tmax_deadline: ", max_deadline

		#for elem in data:
		#
		#	tmp = {
		#		'lastdone': elem['lastdone'],
		#		'scheduled_ts': elem['scheduled ts human'],
		#		'start': elem['ts start human'],
		#		'end': elem['ts end human'],
		#		'waiting_time': elem['waiting time'],
		#		'execution_time': elem['time elapsed']
		#	}
		#
		#	try:
		#		stats[ elem['datagroup'] ]['executions_data'].append(tmp)
		#
		#	except KeyError:
		#
		#		try:
		#			stats[ elem['datagroup'] ]['executions_data'] = []
		#
		#		except KeyError:
		#			stats[ elem['datagroup'] ] = {}
		#			stats[ elem['datagroup'] ]['executions'] = 0
		#			stats[ elem['datagroup'] ]['executions_data'] = []
		#
		#		stats[ elem['datagroup'] ]['executions_data'].append(tmp)
		#
		#	finally:
		#		stats[ elem['datagroup'] ]['executions'] += 1
		#
		#for elem in stats:
		#
