
from __future__ import unicode_literals

import logging

from django.db import migrations, models
from django.db import connection

from core.models import Tenant

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def _add_cascade_column_to_tenant_schemas(apps, schema_editor):

	logger = logging.getLogger('sanet.global')

	db_cursor = connection.cursor()
	db_cursor.execute("SET search_path TO public")

	schemas = [ t.name for t in Tenant.objects.all() ]
	for name in schemas:
		logger.debug("Tenant: %s", name)
		db_cursor.execute("SET search_path TO %s" % (name))

		query = 'ALTER TABLE "condition" ADD COLUMN "cascade" boolean  DEFAULT false NOT NULL'
		logger.debug("Tenant: %s", name)
		db_cursor.execute(query)

		query = 'CREATE INDEX "condition_cascade_d72f5a64" ON "condition" ("cascade");'
		logger.debug("Tenant: %s", name)
		db_cursor.execute(query)

	db_cursor.execute("SET search_path TO public")

def _add_order_column_to_tenant_schemas(apps, schema_editor):

	logger = logging.getLogger('sanet.global')

	db_cursor = connection.cursor()

	db_cursor.execute("SET search_path TO public")

	schemas = [ t.name for t in Tenant.objects.all() ]
	for name in schemas:
		logger.debug("Tenant: %s", name)
		db_cursor.execute("SET search_path TO %s" % (name))

		query = 'ALTER TABLE "condition" ADD COLUMN "order"   smallint DEFAULT 0     NOT NULL CHECK ("order" >= 0)'
		logger.debug("Tenant: %s", name)
		db_cursor.execute(query)

		query = 'CREATE INDEX "condition_order_04058e64"   ON "condition" ("order");'
		logger.debug("Tenant: %s", name)
		db_cursor.execute(query)


	db_cursor.execute("SET search_path TO public")


class Migration(migrations.Migration):

	dependencies = [
	('core', '0001_initial'),
	]

	operations = [

		#
		#
		#
		migrations.AddField(
		    model_name='condition',
		    name='cascade',
		    field=models.BooleanField(db_index=True, default=False),
		),

		migrations.RunPython(_add_cascade_column_to_tenant_schemas),

		#
		#
		#
		migrations.AddField(
		    model_name='condition',
		    name='order',
		    field=models.PositiveSmallIntegerField(db_index=True, default=0),
		),

		migrations.RunPython(_add_order_column_to_tenant_schemas),
	]



