# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
Gestione del calendario delle festivita'.
========================================================================

Per alcune operazioni Sanet (ed entables) ha bisogno di distinguere tra 
giorni normali e giorni "festivi".

Per gestire il calendario Sanet utilizza l'oggetto di tipo:

::
	
	sanet_common.sanetdefs.timing.AbstractCalendarHandler

Per istanziare questo oggetto deve essere utilizzata la funzione globale:

::
	
	init_holidays_converter()


Anche Entables importa e utilzza questa funzione per "farsi dare" da Sanet 
il CalendarHandler giusto da usare.

"""

import types

from django.conf import settings

from sanet_common.sanetdefs.timing import SimpleCalendarHandler  

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class Sanet3SettingsSimpleCalendarHandler(SimpleCalendarHandler):

	def __init__(self, dict_calendar, forced_wday ):
		super(Sanet3SettingsSimpleCalendarHandler, self).__init__(dict_calendar, forced_wday )

		# Nel file settings il contesto di default e' identificato dalla chiave 'DEFAULT'
		# per rendere piu' facile e comprensibile la scrittura della configurazione
		# nel file settings.py.
		#
		# Internamente il contesto di default e' identificato da None e non dalla
		# stringa "DEFAULT".

		self._default_time_context = None

		# Fix del riferimento al calendario di default
		self.calendar[self._default_time_context] = self.calendar['DEFAULT']

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def init_holidays_converter():
	"""
	Carica la configurazione del calendario delle festivita' e restituisce un oggettto di tipo CalendarHandler.
	"""

	#
	# FIX: la gestione del calendario dovrebbe essere salvata / configurata nel database di Sanet.
	#
	#      Bisognerebbe definire un oggetto tipo:
	#
	#             class SanetDBCalendarHandler(SimpleCalendarHandler):
	#                 ...
	#

	try:
		calendar     = settings.HOLIDAYS
		holiday_wday = settings.HOLIDAYS_AS_WDAY

		#
		# Retrocompatibilita' con i calendari esistenti 
		#

		# Vecchio modo
		if isinstance(calendar, set):
			calendar = { }
			calendar[Sanet3SettingsSimpleCalendarHandler.DEFAULT_CONTEXT] = calendar

		if not isinstance(calendar, types.DictType):
			raise Exception("Invalid Holidays Calendar definition")

		holydays_converter = Sanet3SettingsSimpleCalendarHandler(calendar, holiday_wday)

		return holydays_converter

	except Exception, e:
		raise Exception("Error while loading special holidays settings: %s" % (str(e)))
