# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import datetime

from django.db import transaction
from django.contrib.contenttypes.models import ContentType

from core.db_utils import transaction_commit_on_success

from core.api.base import db_set_current_tenant_schema

from notes.models import *

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def usernotes_add(resource, user, text, date=None, action_user=None):

	db_set_current_tenant_schema(resource.tenant.name)


	print __file__, usernotes_add, user, action_user

	if action_user:
		UserNote.assert_edit_element_notes(resource.tenant, action_user, resource)

	date = date if date != None else datetime.datetime.now()

	if isinstance(user,User):
		user_name = user.username
	else:
		user_name = user
		user = None

	text = text.strip()


	u, created = UserNote.objects.get_or_create(
		tenant = resource.tenant,

		content_type = ContentType.objects.get_for_model(resource),
		object_pk    = resource._get_pk_val(),

		user      = user,
		user_name = user_name,

		submit_date = date,
		text = text, 
	)
	u.save()

	return u

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def usernotes_delete(note, user=None):

	#db_set_current_tenant_schema(note.tenant.name)

	#
	# FIX: soft-delete it? 
	#
	
	entity = note.content_object

	# TEMP FIX >>>>>>>>>>>>>>>
	#     Ticket:  https://develtrac/redmine/issues/2902
	#           Alcune note non sono associate a nessuna risorsa.
	#           La cancellazione di qualche entita' ha lasciato sporco.
	#           Bisogna capire perche', nel frattempo, queste
	#           note facciamo finta di assegnarle al tenant.
	if entity is None:
		entity = Tenant.objects.get(is_primary=True)
	# <<<<<<<<<<<<<<<<<<<<<<<<<

	if user:
		UserNote.assert_edit_element_notes(entity.tenant, user, entity)

	note.delete()

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def usernotes_unlink_user(user):

	#db_set_current_tenant_schema(t.name)

	#
	# FIX: what to do?
	#
	UserNote.objects.update(user=None)


