# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import datetime

from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext as _

from core.api.base import *

from cli.clilib.ioslike import *
from cli.tokens import *

from cli.utils import dot_terminated_text, dot_stuff

from notes.models import *
from notes.api.base import usernotes_add

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_add_note(cli):
	
	tenant_name = cli.env['CONFIG_TENANT']
	t = tenant_get(tenant_name)	
	
	if cli.env.get('CONFIG_ELEMENT_DATA', None):
		element = cli.get_config_element()
		if element:
			_add_not_to_resource(cli, t, element)
			return

	#
	# FIX: non funziona
	#
	#c = cli.env.get('CONFIG_CONDITION', None)
	#if c:
	#	_add_not_to_resource(cli, t, c)
	#	return
	#
	
	_add_not_to_resource(cli, t, t)

def _add_not_to_resource(cli, tenant, resource):
	
	DATETIME_FORMAT = "%Y-%m-%d %H:%M:%S"
	
	from pytz import timezone
	utc = timezone("UTC")

	# note [ ... ]
	if len(cli.canonical_words) > 1:

		# Date 
		date_str = cli.canonical_words[1]
		if date_str == 'now':
			d = datetime.datetime.utcnow().replace(tzinfo=utc)
		else:
			d = datetime.datetime.strptime(date_str, DATETIME_FORMAT).replace(tzinfo=utc)
	else:
		d = datetime.datetime.utcnow().replace(tzinfo=utc)

	# Author
	if len(cli.canonical_words) > 2:
		author = cli.canonical_words[2]
	else:
		author = settings.DEFAULT_ADMIN_USERNAME
	
	try:
		user   = user_find(author)
		author = user
	except User.DoesNotExist, e:
		user = None
		cli.warning( _("User '%s' does not exist.") % (author, author) )
	
	text = dot_terminated_text()
	
	usernotes_add(resource, author, text, date=d)
	
	return
	

menu_config_note = [
	T_Keyword('note', help=_('Assign a note to this tenant (input dot-terminated text)'), pipe=None, callback=cmd_add_note,
		subtree=[
			T_Keyword("now", help=_('Create a note with the current date"'), pipe=None, callback=cmd_add_note,
				subtree=[
					T_Anyword(token_repr="<USER>", help=_('Create a note with the current date and the given author'), pipe=None, callback=cmd_add_note)
				],
			),
			T_Anyword(token_repr="<DATE>", help=_('Create a note with the given date and time (format: "YYYY-MM-DD HH:MM:SS" or "now", defaults to now)'), pipe=None, callback=cmd_add_note,
				subtree=[
					T_Anyword(token_repr="<USER>", help=_('Create a note with the given date/time and author'), pipe=None, callback=cmd_add_note)
				],
			),
		],
	),
]

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#


def cmd_remove_note(cli):
	# no note id <id>
	
	tenant_name = cli.env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)

	note_id = cli.canonical_words[3]

	note = UserNote.objects.get(tenant=tenant, uuid=note_id)
	note.delete()
	

menu_config_note_remove = [
	T_Keyword('note', help=_('Delete a note'), pipe=None, subtree=[
		T_Keyword('id', help=_('Delete a note by id'), pipe=None, subtree=[
			T_Anyword(help=_('Delete a note by id'), pipe=None, callback=cmd_remove_note,),
		]),
	]),
]

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def notes_show_conf_tenant_notes(cli, element, level): 
	_show_conf_resource_notes(cli, element, element, level)

def notes_show_conf_node_notes(cli, element, level): 
	_show_conf_resource_notes(cli, element.tenant, element, level)

def notes_show_conf_interface_notes(cli, element, level): 
	_show_conf_resource_notes(cli, element.tenant, element, level)

def notes_show_conf_storage_notes(cli, element, level): 
	_show_conf_resource_notes(cli, element.tenant, element, level)

def notes_show_conf_service_notes(cli, element, level): 
	_show_conf_resource_notes(cli, element.tenant, element, level)

def notes_show_conf_device_notes(cli, element, level): 
	_show_conf_resource_notes(cli, element.tenant, element, level)

#def notes_show_conf_condition_notes(cli, element, level): 
#	_show_conf_resource_notes(cli, element.tenant, element, level)


def _show_conf_resource_notes(cli, tenant, element, level):
	
	db_set_current_tenant_schema(tenant.name)

	#resource_type  = ContentType.objects.get_for_model(element) 
	#resource_id    = element.uuid

	#notes = UserNote.objects.filter(content_type=resource_type , object_pk=resource_id)
	
	notes = element.notes()

	if notes:
		
		cli.pij(level, cli.CLI_COMMENT("#")      )
		cli.pij(level, cli.CLI_COMMENT("# Notes"))
		cli.pij(level, cli.CLI_COMMENT("#")      )

		for n in notes:
			args = [cli.CLI_KEYWORD('note'), n.submit_date.strftime("%Y-%m-%d %H:%M:%S"), n.user_name]
			cli.pij(level, cli.CLI_COMMENT("# --start--"))
			cli.piqj(level, *args)
			cli.prnt(dot_stuff(n.text))
			cli.pij(level, cli.CLI_COMMENT("# --end--"))

	return
	
