# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from django.utils.translation import ugettext as _

from core.api.base import db_set_current_tenant_schema, tenant_get, tenant_select_all

from notes.models import *

from cli.clilib.ioslike import *
from cli.textformat import *

from cli.prompts.common import output_table

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#
def _print_notes(cli, notes):

	if len(notes) == 0:
		cli.prnt("* NO NOTES *")
		cli.prnt("")
		return

	t = [[
		['ID', -4],
		['DATE' ,17],
		['AUTHOR', 15],
		['RESOURCE TYPE',10],
		['RESOURCE  ',20],
		['CONTENT' , 40],
	]]

	for n in notes:
		t += [[ 
			n.uuid, 
			n.submit_date.strftime("%Y-%m-%d %H:%M:%S"), 
			n.user_name, 
			n.content_type, 				# FIX: visualizzare 'node', 'interface', 'storage', ecc. e non il nome della classe
			n.content_object.path  if n.content_object else '', 
			n.text.replace('\n', '\\n')
		]]

	output_table(cli, t)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_notes(cli):
	
	user = cli.env['CURRENT_USER']

	tenant_name = cli.env.get('CURRENT_TENANT',None)

	if tenant_name != None:
		tenants = [ tenant_get(tenant_name) , ]
	else:
		tenants = tenant_select_all()
	
	notes = []
	for t in tenants:
		db_set_current_tenant_schema(t.name)

		notes += t.notes_by_user(user)

	_print_notes(cli, notes)
	

menu_show_note = [
	T_Keyword('notes' , help=_('Show notes'), callback=cmd_show_notes, ),
]


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def notes_show_resource_notes(cli, element):
	
	tenant  = element.tenant
	
	db_set_current_tenant_schema(tenant.name)

	notes = element.notes()
	
	_print_notes(cli, notes)
