# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings

from core.compatibility import reverse

# Signals
from webui.models import WebuiPageExtension

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class NotesPageExtension(WebuiPageExtension):

	def get_info_widgets(self):

		page = self.page

		# Notes are not visible in the "info" section of the tenant page
		if page.pagetype == "tenant":
			return []
		else:
			return self._get_widget(page)

	def get_advanced_info_widgets(self):

		page = self.page

		# Notes
		if page.pagetype == "tenant":
			return self._get_widget(page)
		else:
			return [ ]

	def _get_widget(self, page):

		from webui.api.widgets import NotesWidget

		n = page.resource

		widget_notes = NotesWidget(
			id        = "%s_%s_%s" % (n.uuid, n.resource_type, 'notelist'),

			tab       = page.TAB_GENERAL,
			container = page.LAYOUT_INFO,

			remote    = "%s_%s.%s" % (n.resource_type, n.resource_id, 'notelist'),
			rtimer    = settings.WEBUI_REFRESH_TIMERS['notes_list'],
		)

		return [ widget_notes.to_dict() ]

	def get_extension_additional_jsfile_urls(self):

		urls = []

		urls += [ reverse('static.file.serve' , kwargs={'path': 'widgets2/jswlib_notelist.js'              }) ]

		return urls
