# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import time

from django.conf import settings

from django.http import HttpResponse, HttpResponseForbidden, HttpResponseNotFound

from django.contrib.auth.models import User

from core.models import Tenant, get_user_default_tenant
from core.api.base import tenant_get_primary

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def rest_api_enabled(view_fun):

	def _rest_api_enabled_wrapper(request, *args, **kwargs):
		
		ts_start = time.time()

		try:
			username = request.GET.get('username', None)
			tenantid = request.GET.get('tenantid', None)

			cookie_value = request.COOKIES.get('sanet3auth', None)

			if cookie_value is None:
				cookie_value = request.GET.get('sanet3auth', None)

				if cookie_value is None:
					return HttpResponseForbidden("Authentication error. Check rest_api authentication cookie.")

			if cookie_value != settings.REST_API_AUTH_COOKIE:
				return HttpResponseForbidden("Invalid authentication cookie.")

			if username or tenantid:

				tenant = None

				if tenantid is not None:
					if username is not None:
						user = User.objects.get(username=username)
					else:
						user = User.objects.filter(is_superuser = True)[0]

					try:
						tenant = Tenant.objects.get(uuid=tenantid)
					except Tenant.DoesNotExist, e:
						try:
							tenant = Tenant.objects.get(name=tenantid)
						except Tenant.DoesNotExist, e:
							return HttpResponseNotFound("Unable to find the right tenant for the request")

				else:
					user = User.objects.get(username=username)
					tenant = get_user_default_tenant(user)

			else:
				tenant = tenant_get_primary()
				user = User.objects.filter(is_superuser = True)[0]

			request.user = user
			request.tenant = tenant
			
			return view_fun(request, *args, **kwargs)
		finally:
			
			ts_end = time.time()
			
			print "SANET REST-API. Request:", request.path, "Took:", ts_end - ts_start

	return _rest_api_enabled_wrapper


