#!/bin/bash

SANET_INSTALL_DIR=

if test -n "$SANET_HOME"; then
	SANET_INSTALL_DIR=$SANET_HOME
fi
if test -z $SANET_INSTALL_DIR; then
	echo "WARNING: sanet install dir is unknown!"
	exit -1
fi

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

COLOR_GREEN='\e[0;32m'
COLOR_RED_BOLD='\e[1;31m'
COLOR_OFF='\e[0m'

function msg()
{
	echo -en $COLOR_GREEN
	echo -en $@
	echo -e  $COLOR_OFF
}

function error()
{
	echo -en $COLOR_RED_BOLD
	echo -en $@
	echo -e  $COLOR_OFF
}

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

cd $SANET_INSTALL_DIR

DEBUG_STATE=$(python manage.py get_config DEBUG)
if [[ ! "$DEBUG_STATE" == "False" ]]; then
	error '######################'
	error '### DEBUG MODE ON! ###'
	error '######################'
fi



LOG_DIR=$(python manage.py get_config LOG_DIR)
LOG_FILE="$LOG_DIR/entables_log"

PID_DIR=$(python manage.py get_config PID_DIR)
PID_FILE="$PID_DIR/entablesd_pid"
LOCK_FILE="$PID_DIR/entablesd_pid.lock"


case "$1" in

	"start")
		msg "Starting entablesd..."

		#echo "Using SANET_INSTALL_DIR:" $SANET_INSTALL_DIR
		
		if [ -f $PID_FILE ]; then

			ps -p $(cat $PID_FILE 2> /dev/null) > /dev/null
			RET=$?
			if [ "$RET" -eq "0" ]; then
				error "entablesd already started. See " $PID_FILE
				exit -1
			else
				echo "entablesd process not running but pid file exists. " $PID_FILE
				echo "Removing pid file"
				rm $PID_FILE
				echo "Removing lockfile..."
				rm $LOCK_FILE
			fi
		else
			#echo "Removing lockfile..."
			rm $LOCK_FILE 2> /dev/null
		fi	

		#echo "Launching entablesd..."
		#echo "Log file: " $LOG_FILE
		#echo "Pid file: " $PID_FILE

		python manage.py entables_server -l $LOG_FILE -p $PID_FILE $*
		
		RET=$?
		if [[ "$RET" -eq "0" ]]; then
			msg "\t" "Started"
		else
			error "\t" "Unable to start entablesd."
		fi

		;;

	"stop")
		msg "Stopping entablesd..." 

		ENTABLES_PID=$(cat $PID_FILE 2>  /dev/null)

		#echo "Pid file:" $PID_FILE ", PID=" $ENTABLES_PID

		kill -s HUP "$ENTABLES_PID"
		
		RET=$?
		if [[ "$RET" -eq "0" ]]; then
			msg "\t" "Stopped"
			rm $PID_FILE
		else
			error "\t" "Unable to stop entablesd."
		fi
		;;  

	"restart")
		$0 stop
		$0 start
		;;

	"status")
		PID=$(pgrep -f $PID_FILE)
		RET=$?
		if [ "$RET" -eq "0" ]; then
			echo "entables is running" $PID
		else
			echo "entables not running" $PID
		fi
		;;

	*)
		echo "Invalid option"
		echo $0 "[start|stop]"
		;;
esac
