#!/bin/bash

### BEGIN INIT INFO
# Provides:          sanet3
# Required-Start:    $network  $local_fs $remote_fs $syslog $postgresql $rrdcached $redis-server
# Required-Stop:     $local_fs $remote_fs $syslog
# Should-Start:      $time postgresql rrdcached redis-server postfix
# Should-Stop:       $time postgresql rrdcached redis-server postfix
# Default-Start:     2 3 4 5
# Default-Stop:      0 1 6
# Short-Description: Sanet 3
### END INIT INFO

#WORKING_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"
WORKING_DIR=$(dirname $(readlink -f $0))  # not POSIX


COLOR_GREEN='\e[0;32m'
COLOR_RED_BOLD='\e[1;31m'
COLOR_OFF='\e[0m'

function msg()
{
	echo -en $COLOR_GREEN
	echo -en $@
	echo -e  $COLOR_OFF
}

function error()
{
	echo -en $COLOR_RED
	echo -en $@
	echo -e  $COLOR_OFF
}

function check_config()
{
	$WORKING_DIR/../config-check.sh
	ret=$?
	if [ "$ret" != "0" ]; then
		exit -1
	fi
}

case "$1" in

	'start')
		check_config
		
		msg "Starting sanet3..."

		$WORKING_DIR/entablesd.sh start
		$WORKING_DIR/sanetd.sh    start
		$WORKING_DIR/poller.sh    start
		;;

	'stop')
		check_config
	
		msg "Stopping sanet3..."

		$WORKING_DIR/entablesd.sh stop
		$WORKING_DIR/sanetd.sh    stop
		$WORKING_DIR/poller.sh    stop
		;;

	'restart')
		$0 stop
		$0 start
		;;

	'nightly')
		check_config

		# Stop processes and avoid memory leaks
		# - stop poller
		# - stop sanetd ( discoveryd )
		$WORKING_DIR/poller.sh    stop
		$WORKING_DIR/sanetd.sh    stop

		# FIX: dump current-cli-config

		# clean redis garbage
		$WORKING_DIR/../utils/clean_queues.sh
		
		echo "cleaning temp web data..."
		$WORKING_DIR/../../bin/sanet-manage clean_temp_web_data

		# Relaunch all
		$WORKING_DIR/sanetd.sh    start
		$WORKING_DIR/poller.sh    start
		;;

	'cli-save')
		check_config

		echo "Saving current configuration..."
		$WORKING_DIR/../config-save.sh
		;;

	*)
		echo ""
		echo "Usage: $0 <args>"
		echo ""
		echo "Command can be:"
		echo ""
		echo "start            Start sanet3"
		echo "stop             Stop sanet3"
		echo "restart          Stop & start sanet3"
		echo "nightly          Perform nighly procedures and restarts sanet3"
		echo "cli-save         Dump Sanet's configuration (CLI + Entables) "
		echo ""
		;;
esac	
