#!/bin/bash

SANET_INSTALL_DIR=

if test -n "$SANET_HOME"; then
	SANET_INSTALL_DIR=$SANET_HOME
fi

if test -z $SANET_INSTALL_DIR; then
	echo "WARNING: sanet install dir is unknown!"
	exit -1
fi

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

COLOR_GREEN='\e[0;32m'
COLOR_YELLOW='\e[0;33m'
COLOR_RED_BOLD='\e[1;31m'
COLOR_OFF='\e[0m'

function msg()
{
	echo -en $COLOR_GREEN
	echo -en $@
	echo -e  $COLOR_OFF
}

function warning()
{
	echo -en $COLOR_YELLOW
	echo -en $@
	echo -e  $COLOR_OFF
}

function error()
{
	echo -en $COLOR_RED_BOLD
	echo -en $@
	echo -e  $COLOR_OFF
}

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

export HPRI="nice -n -5"

#------------------------------------------------------------------------------#
# MAIN                                                                         #
#------------------------------------------------------------------------------#

cd $SANET_INSTALL_DIR


DEBUG_STATE=$(python manage.py get_config DEBUG)
if [[ ! "$DEBUG_STATE" == "False" ]]; then
	error '######################'
	error '### DEBUG MODE ON! ###'
	error '######################'
fi

case "$1" in

	'start')
		msg "Starting sanetd..."

		ulimit -n 16000 2>&1 > /dev/null
		if [[ "$RET" -eq "0" ]]; then
			warning "Unable to set ulimit"
		fi
		
		$HPRI python server/sanetd/main.py start
		RET=$?
		if [[ "$RET" -eq "0" ]]; then
			msg "\t" "Started"
		else
			error "\t" "Unable to start sanetd."
			exit -1
		fi
		;;

	'stop')
		msg "Stopping sanetd..."

		python server/sanetd/main.py stop
		RET=$?
		if [[ "$RET" -eq "0" ]]; then
			msg "\t" "Stopped"
		else
			error "\t" "Unable to stop sanetd."
			exit -1
		fi
		;;

	'restart')
		$0 stop
		$0 start
		;;

	*)
		echo ""
		echo "Usage: $0 <command> <args>"
		echo ""
		echo "Command can be:"
		echo ""
		echo "start            Start checks and measures"
		echo "stop             Stop checks and measures"
		echo ""
		;;
esac
