
# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET3
# SANET3 is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.


import os, sys

#==============================================================================#

SANET_INSTALL_DIR=None

SANET_INSTALL_DIR = os.environ.get("SANET_HOME", SANET_INSTALL_DIR)

from os.path import join, dirname, abspath, exists
if not SANET_INSTALL_DIR:
	SANET_INSTALL_DIR = abspath(os.path.join(os.path.dirname(abspath(__file__)),"..",".." ))

sys.path.append( SANET_INSTALL_DIR )

#==============================================================================#



import sys
import os
import getopt
import time
import errno 
import signal
from operator import attrgetter

"""
# PYTHON FORK BUG WORK AROUND http://bugs.python.org/issue6380
def __fork():
	imp.acquire_lock()
	try:
		#print "forking"
		return _os_fork()
	finally:
		imp.release_lock()
		
python_version = sys.version_info
if python_version[0] == 2 and ((python_version[1] >= 7) or (python_version[1] == 6 and python_version[2] >= 5)):
	# This trick shouldn't be necessary anymore since python 2.6.5 - Issue #7242 - The import lock is now reinitialized after fork.
	pass
else:
	#print "redefining os.fork (%s)" % (sys.version_info, )
	try:
		_os_fork
	except NameError:
		_os_fork = os.fork
		os.fork = __fork
"""
#-----------------------------------------------------------------------------#
# IMPORT CONFIGURATION FROM DANGO                                             #
#-----------------------------------------------------------------------------#

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "conf.settings")

# Changed for django 1.11
import django
django.setup()

#-----------------------------------------------------------------------------#

import logging
logging._srcfile     = None # dont' collect information about where calls were made from.
logging.logThreads   = 0    # dont' collect threading information.
logging.logProcesses = 0    # dont' collect process information.
import logging.handlers

#-----------------------------------------------------------------------------#
# SHARED LIBRARIES                                                            #
#-----------------------------------------------------------------------------#

from sanet_common.osutils.daemonize import daemonize
from sanet_common.osutils.proctitle import get_proc_title, set_proc_title
from sanet_common.osutils.osutils   import convert_exit_status_to_verb_reason

#-----------------------------------------------------------------------------#
# IMPORT CONFIGURATION FROM DANGO                                             #
#-----------------------------------------------------------------------------#

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "conf.settings")

from conf import get_configuration

#from core.api.datasource import check_datasource_store_conf

from core.api.sysstats import check_system_settings


#-----------------------------------------------------------------------------#
# Program Options                                                             #
#-----------------------------------------------------------------------------#

PROGRAM_NAME = 'sanetd_supervisor'

#-----------------------------------------------------------------------------#
# Global files                                                                #
#-----------------------------------------------------------------------------#

supervisor_log_filename = PROGRAM_NAME
supervisor_log_file = None
supervisor_pid_file = None

# The main logger object.
log = None

#-----------------------------------------------------------------------------#
# Program Options                                                             #
#-----------------------------------------------------------------------------#

# -h
opt_help = False

opt_daemonize = True


# -L log level
opt_loglevel = logging.DEBUG

log_levels = {
	'DEBUG': logging.DEBUG,
	'INFO' : logging.INFO,
}


#-----------------------------------------------------------------------------#
# MAIN PROGRAM                                                                #
#-----------------------------------------------------------------------------#

def sanetd_supervisor_main(argv=None):

	global  opt_help, \
	        opt_loglevel, \
	        opt_daemonize

	if argv is None:
		argv = sys.argv

	try:
		opts, args = getopt.getopt(sys.argv[1:], "hL:n")
		for o, a in opts:
			if   o in ("-h"):
				opt_help = True
			if   o in ("-n"):
				opt_daemonize = False
			elif o in ("-L"):
				if log_levels.has_key(a):
					opt_loglevel = log_levels[a]
				else:
					opt_loglevel = int(a)

	except getopt.error, msg:
		sys.stderr.write("Error while parsing options: %s\n" % (msg,))
		print_help()
		sys.exit()
	except ValueError, e:
		sys.stderr.write("Invalid value for input parameter: %s\n" % (e))
		print_help()
		sys.exit()

	if opt_help:
		print_help()
		sys.exit()

	if len(args) != 1:
		sys.stderr.write("Wrong number of arguments: %s\n" % (args))
		print_help()
		sys.exit(-1)

	#
	# 
	#
	#check_datasource_store_conf()

	check_system_settings()

	load_conf()

	cmd = args[0]

	if   cmd == 'start':
		start_supervisor(opt_daemonize)
		
	elif cmd == 'stop':
		stop_supervisor()
		
	elif cmd == 'test':
		show_processes()

	else:
		sys.stderr.write("Invalid command: %s\n" % (cmds[0]))
		
		print_help()
		
		sys.exit(-1)


#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def print_help():
	print "program [options]"
	print "program [options] start"
	print "program [options] stop"

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def create_wordking_dirs():

	try:
		os.makedirs(get_configuration().LOG_DIR)
	except OSError, e:
		pass

	try:	
		os.makedirs(get_configuration().PID_DIR)
	except OSError, e:
		pass


def load_conf():

	global supervisor_pid_file, \
	       supervisor_log_file 

	supervisor_log_file = os.sep.join([get_configuration().LOG_DIR, supervisor_log_filename + '.log'])

	supervisor_pid_file = os.sep.join([get_configuration().PID_DIR, PROGRAM_NAME + '.pid'])

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def start_supervisor(daemon_mode=True):
	
	try:
		create_wordking_dirs()
		
		# Check for a pidfile to see if the daemon is already running
		try:
			pid = read_pid_file(supervisor_pid_file)
		except IOError, e:
			pid = None
			
		if pid != None:
			if is_supervisor_process(pid):
				sys.stderr.write("Already running. (PID %s)\n" % pid)
				sys.exit(-1)
			else:
				if is_process_running(pid):
					sys.stderr.write("Unknown process with pid %s!\n" % pid)
					sys.exit(-1)
				else:
					# if the process is dead, go on!
					sys.stderr.write("PID file not empty, but process %s not running!\n" % pid)


		if daemon_mode:
			daemonize()

		init_log()

		# Do after daemonize!
		write_pid_on_file(supervisor_pid_file)

		init_children_jobs()
		
		respawn_children_until_stop()
		
		remove_pid_file(supervisor_pid_file)

	except Exception, e:
		sys.stdout.write(str(e))
		__log( os.getpid(), "Error: %s" % (e) )
		
	return 0

def show_processes():
	
	from pipes import quote

	init_log()

	init_children_jobs()

	for child in subprocesses:
		print ' '.join([ quote(a) for a in child.params['args']  ])
		
	return 0

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def start_subprocess(params=None):
	
	try:
		args = params['args']
		
		__log(0, "%s execlp....goodbye" % (str(args)))
		
		os.execlp( *args )
		
	except Exception, e:
		__log(0, "Error while launching execd: %s" % (e))

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def stop_supervisor():
	
	try:
		pid = read_pid_file(supervisor_pid_file)
	except IOError, e:
		sys.stderr.write("Unable to read pid file %s: %s\n" % (supervisor_pid_file, e) )
		pid = None

	if not pid:
		sys.stderr.write("pidfile %s does not exist. Process not running?\n" % (supervisor_pid_file) )
		sys.exit(1)
	else:
		if not is_supervisor_process(pid):
			sys.stderr.write("Unknown process with pid %s!\n" % pid)
			sys.exit(1)

	#
	# Send command to commit and stop
	#

	# Try killing the daemon process       
	try:
		sys.stdout.write("Sending term signal to Sanetd (pid=%s)" % (pid))
		sys.stdout.flush()

		# Send signals until the process dies
		while 1:
			os.kill(pid, signal.SIGTERM)

			sys.stdout.write(".")
			sys.stdout.flush()
			
			time.sleep(1)
	except OSError, e:
		# did we kill the process or what else?
		if e.errno == errno.ESRCH:
			print ""
			print "Process terminated.",
		else:
			sys.stderr.write("Error while sending signal to process %s: %s" % (pid, e))
			sys.exit(1)

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def init_log():

	global log, opt_loglevel

	log = logging.getLogger(PROGRAM_NAME)
	log.setLevel( opt_loglevel )

	hdlr = logging.FileHandler(supervisor_log_file, "a")
	hdlr.setFormatter( logging.Formatter('%(asctime)s %(levelname)s> %(message)s') )
	hdlr.setLevel( opt_loglevel )
	log.addHandler(hdlr)

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

# FIX: move to default_settings.py?
WAITTIME_AFTER_KILL = 1   #seconds


class SubProcessInfo(object):

	def __init__(self, name, action, pidfile_name, params={}, priority=0):

		super(SubProcessInfo, self).__init__()

		self.name        = name
		self.action      = action
		self.params      = params
		self.priority    = priority

		self.pid         = 0
		self.pidfile     = os.sep.join( [get_configuration().PID_DIR, pidfile_name] )
		self.term_sent   = 0   # time the term signal was sent
		self.graceperiod = 10  # seconds. Interval between the (soft) term signal and the kill signal
		
	def __str__(self):
		return '<%s:%s>' % (self.name, self.pid if self.pid != None else '-')
		
	def run_action(self):
		self.action(self.params)



STOP_SUPERVISOR = False

subprocesses = []


def init_children_jobs():

	global subprocesses

	log.info("Preparing subprocesses...")

	#
	# SANETD (and instances)
	#
	for instance_id in get_configuration().SANETD_INSTANCES:

		args = [
			"python",
			"python",
			os.path.join( get_configuration().BASE_DIR, 'server', 'sanetd', 'sanetdserver.py'),
			get_configuration().BASE_DIR,

			# params
			'-I', instance_id,
		]

		job = SubProcessInfo(
			name        = 'sanetd-%s' % (instance_id), 
			action      = start_subprocess, 
			pidfile_name= 'sanetd-%s.pid' % (instance_id), 
			priority    = 0, 
			params      = {'args':args}
		)  
		subprocesses.append(job)

	# STATED
	args = [
		"python",
		"python",
		os.path.join( get_configuration().BASE_DIR, 'server', 'statusd', 'main.py'),
		get_configuration().BASE_DIR,
		str(12345),
	]
	job = SubProcessInfo(name='stated'    , action=start_subprocess    , pidfile_name='stated.pid', priority=0, params={'args':args})  
	subprocesses.append(job)

	# WDOG
	if get_configuration().SANETD_WATCHDOG_CONF.get('enabled', False):
		log.info("Watchdog enabled. Setting wdog process")		
		args = [
			"python",
			"python",
			os.path.join( get_configuration().BASE_DIR, 'server', 'sanetd', 'sanetdwdog.py'),
			get_configuration().BASE_DIR,
		]                		
		job = SubProcessInfo(name='sanetdwdog', action=start_subprocess, pidfile_name='sanetdwdog.pid', priority=1, params={'args':args})  
		subprocesses.append(job)

	## AUTOCONF
	#if 'autoconf' in get_configuration().INSTALLED_APPS:
	#	if get_configuration().DISCOVERYD_ENABLED:
	#		log.info("Autoconf enabled. Setting autoconf process")
	#		args = [
	#			"python",
	#			"python",
	#			os.path.join( get_configuration().BASE_DIR, 'autoconf', 'server', 'autoconfd.py'),
	#			get_configuration().BASE_DIR,
	#		]
	#		job = SubProcessInfo(name='autoconfd'    , action=start_subprocess, pidfile_name='autoconfd.pid', priority=0, params={'args':args})  
	#		subprocesses.append(job)
	#
	# TRAPD
	#if get_configuration().SNMPTRAP_LISTEN is not None:
	#	args = [
	#		"python",
	#		"python",
	#		os.path.join( get_configuration().BASE_DIR, 'server', 'trapd', 'main.py'),
	#		get_configuration().BASE_DIR,
	#	]
	#	job = SubProcessInfo(name='trapd'    , action=start_subprocess    , pidfile_name='trapd.pid', priority=0, params={'args':args})  
	#	subprocesses.append(job)


	# TAGSD
	#if "tags" in get_configuration().INSTALLED_APPS:
	#	args = [
	#		"python",
	#		"python",
	#		os.path.join( get_configuration().BASE_DIR, 'tags', 'server', 'tagsd', 'main.py'),
	#		get_configuration().BASE_DIR,
	#	]
	#	job = SubProcessInfo(name='tagsd'    , action=start_subprocess    , pidfile_name='tagsd.pid', priority=0, params={'args':args})  
	#	subprocesses.append(job)

	log.info("Loading extra sub-processes")

	for name, pidfile, priority, args in get_configuration().SANETD_EXTRA_PROCESSES:
		job = SubProcessInfo(name=name, action=start_subprocess    , pidfile_name=pidfile, priority=priority, params={'args':args})  
		subprocesses.append(job)


	for p in subprocesses:
		log.info("Loaded job: %s" % (p.name))

	return



def supervisor_on_sigterm(signum, frame):
	global STOP_SUPERVISOR
	STOP_SUPERVISOR = True


def respawn_children_until_stop():

	global STOP_SUPERVISOR, subprocesses

	not_running_by_name = dict( [ (p.name, p) for p in subprocesses ] )
	running_by_pid      = {}

	try:

		STOP_SUPERVISOR = False

		supervisor_pid = os.getpid()

		signal.signal(signal.SIGTERM, supervisor_on_sigterm)

		original_proc_title = get_proc_title()

		set_proc_title( original_proc_title + ' (supervisor)' )

		__log(supervisor_pid, "Starting main loop")

		while True:
			if not STOP_SUPERVISOR:

				for child in not_running_by_name.values():

					pid = os.fork()
					if pid:
						# father
						child.pid = pid

						del not_running_by_name[ child.name ] 
						running_by_pid[ pid ] = child

						#__log(supervisor_pid, "spawned %s PID: %s" % (child.name, child.pid))
						__log(supervisor_pid, "spawned %s" % (child))
					else:
						# son
						signal.signal(signal.SIGTERM, signal.SIG_DFL )

						my_pid = os.getpid()
						__log(my_pid, "started child %s" % (child))
						write_pid_on_file(child.pidfile)

						set_proc_title( original_proc_title + ' (%s)' % (child.name,))

						try:
							child.run_action() # This should not return
						except Exception, e:
							__log(my_pid, "EXCEPTION child %s: %s" % (child, e))
						exit() # This should not happen
			else:
				children = running_by_pid.values()

				children = sorted(children, key=attrgetter('priority'))
				children.reverse()

				for child in children:

					if not child.term_sent:
						__log(supervisor_pid, "sending TERM to %s " % (child))
						os.kill(child.pid, signal.SIGTERM)
						child.term_sent = time.time()

					elif time.time() - child.term_sent < child.graceperiod:
						__log(supervisor_pid, "re-sending TERM to %s" % (child))
						os.kill(child.pid, signal.SIGTERM)

					else:
						__log(supervisor_pid, "sending KILL to %s" % (child))
						os.kill(child.pid, signal.SIGKILL)

					break # Go through a wait before killing next one

				if len(running_by_pid) == 0:
					__log(supervisor_pid, "no more children, exiting")
					return
				else:
					time.sleep(WAITTIME_AFTER_KILL)
			try:
				(pid, status) = os.wait()
			except OSError, e:
				__log(supervisor_pid, "Wait interrupted with error %d: %s" % (e.errno, os.strerror(e.errno)))
				__log(supervisor_pid, "Stop signal recieved: %s" % ( STOP_SUPERVISOR ), )
			else:
				child = running_by_pid[ pid ]

				#__log(supervisor_pid, "Child %s exited" % (child))

				try:
					reason = convert_exit_status_to_verb_reason(status)
				except Exception, e:
					reason = "Unreadable reason: %s" % (str(e),)

				__log(supervisor_pid, "Child %s exited with status %d: %s" % (child, status, reason))

				if os.path.exists(child.pidfile):
					os.remove(child.pidfile)

				child.pid = 0

				del running_by_pid[pid]
				not_running_by_name[child.name] = child

	except Exception, e:
		log.fatal("%s", e)

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def __log(pid, status):
	
	global log
	msg = "%s" % (status)
	print msg
	log.info(msg)
	return
	
	"""
	#global supervisor_log_file
	
	supervisor_log_file = "./supervisor.log"
	
	try:
		filename = supervisor_log_file
		f = open(filename,'a')
		f.write("%s [main] [pid %s] %s\n" % (timestamp_to_str(get_time()), pid, status) )
		f.close()
	except:
		pass
	"""

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def write_pid_on_file(pid_file):
	pid = str(os.getpid())
	f = open(pid_file, 'w+')
	f.write("%s\n" % (pid))
	f.close()

def remove_pid_file(pid_file):
	try:
		if os.path.exists(pid_file):
			os.remove(pid_file)
	except Exception, e:
		print "Error while removing pid file: ", e
		pass

def read_pid_file(name):
	f = file(name,'r')
	pid = int(f.read().strip())
	f.close()
	return pid

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def is_supervisor_process(pid):

	try:
		f = file('/proc/%s/cmdline' % pid,'r')
		cmd_argv = f.read().split('\x00')
		f.close()
	except IOError, er:
		sys.stderr.write("Process %s does not exist.\n" % pid)
		return False

	# Without setproctitle
	if cmd_argv[0] == 'python' and cmd_argv[1].find('sanetd/main.py') != -1:
		return True

	# With setproctitle
	if cmd_argv[0].find('python') != -1 and cmd_argv[0].find('sanetd/main.py') != -1:
		return True

	return False

def is_process_running(pid):

	process_filename = '/proc/%s/cmdline' % pid

	if os.path.exists(process_filename):
		return True
	else:
		return False

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

if __name__ == "__main__":
	sanetd_supervisor_main()

#BDCA


