# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os
import socket
import sys
import getopt
import json

from sanet_common.ipc import IPCConnection


from sanet_common.ipc import RESPONSE_SUCCESS
from sanet_common.ipc import decode_response, encode_request, encode_request

from sanet_common.sanetdefs.ipc import SANETD_INFO_QUEUE

from sanet_common.sanetdefs.ipc import SANETD_CLT_QUEUE

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def sanetd_ctl(broker, command, arguments, timeout=3, instance_id=None):

	if not instance_id:
		instance_id = 'main'

	queue_name = SANETD_INFO_QUEUE % (instance_id)

	#return _sanetd_ctl(broker, SANETD_CLT_QUEUE, command, arguments, timeout=timeout)
	return _sanetd_ctl(broker, queue_name, command, arguments, timeout=timeout)



def _sanetd_ctl(broker, queue_name, command, arguments, timeout=3):

	request = encode_request(command, *arguments)

	if broker is None:
		broker = "sanet/sanet@localhost"
	
	host = None
	port = None
	user = None
	password = None
	
	auth, addr = broker.split('@')
	user,password = auth.split('/')
	addr = addr.split(':')
	host = addr[0]
	if len(addr) > 1:
		port = int(addr[1])
	
	connect_options = {}
	if port     != None: connect_options['port']     = port
	if user     != None: connect_options['username'] = user
	if password != None: connect_options['password'] = password
	
	c = IPCConnection( host, **connect_options)
	
	c.settimeout(3)
	
	c.open()
	
	c.bind("temp-sanetdctl-pid-%s" % (os.getpid()), persistent=False)

	#
	# SEND
	#
	
	#c.send_msg( 'sanetd-ctl' , request, blocking=True)
	c.send_msg( queue_name , request, blocking=True)
	
	#
	# RECEIVE
	#

	addr, data = c.receive_msg(ack_msg=True, timeout=timeout)
	if data is None:
		raise Exception("Timeout")
	c.close()

	#
	# Return response
	#
	response = decode_response(data)

	if response['status'] == RESPONSE_SUCCESS:
		return response['content']
	else:
		raise Exception("Error from remote host: %s" % (response['content']))

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def sanetd_ctl_socket(address, command, arguments):

	if address:
		address = address.split(':')
		address = (address[0], int(address[1]))
	else:
		address = ('127.0.0.1', 3456)

	c = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	c.connect(address)


	request = encode_request(command, *arguments)
	
	c.sendall(request)
	
	data = ''
	tmp = c.recv(1024)
	while tmp:
		data += tmp
		tmp = c.recv(1024)

	c.close()
	response = decode_response(data)
	
	if response['status'] == RESPONSE_SUCCESS:
		return response['content']
	else:
		raise Exception("Error from remote host: %s" % (response['content']))

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

#def sanetd_ctl_main():
#
#	opt_broker         = None
#	opt_socket         = False
#	opt_instanceid     = 'main'
#	
#	argv = sys.argv
#
#	try:
#		opts, args = getopt.getopt(sys.argv[1:], "hB:sI:")
#		print opts, args
#		for o, a in opts:
#			if   o in ("-h"):
#				print_help()
#				sys.exit(0)
#				
#			elif o in ("-B"):
#				opt_broker = a
#
#			elif o in ("-s"):
#				opt_socket = True
#
#			elif o in ("-I"):
#				print __file__
#				opt_instanceid = a
#
#			else:
#				sys.stderr.write("Unknown option: %s" % (o))
#				print_help()
#				sys.exit(-1)
#
#	except getopt.error, msg:
#		sys.stderr.write("Error while parsing options: %s" % (msg))
#		print_help()
#		sys.exit(-1)
#
#	except ValueError, e:
#		sys.stderr.write("Invalid value for input parameter: %s" % (e))
#		print_help()
#		sys.exit(-1)
#
#	if len(args) < 1:
#		sys.stderr.write("Missing command")
#		print_help()
#		sys.exit(-1)
#
#	command   = args[0]
#	arguments = args[1:]
#	
#	if not opt_socket:
#		print sanetd_ctl(opt_broker, command, arguments, instance_id=opt_instanceid)
#	else:
#		print sanetd_ctl_socket(opt_broker, command, arguments)
#
#def print_help():
#	sys.stderr.write('program -B "<user>/<password>@<host>"  <command> <options>\n')
#
#if __name__ == "__main__":
#
#	sanetd_ctl_main()
