# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET3
# SANET3 is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.
"""
Sanetd
================

Sanetd e' il server principale di sanet. Comunica con gli agenti ed effettua gli aggiornamenti sul DB.

Il processo legge da due code:

	* SANETD_CLT_QUEUE
	* SANETD_EVENTS_QUEUE

.. image:: ../_static/sanetd.png
	:width: 800

"""


import os
import sys
import pprint

SANET_INSTALL_DIR=None

from os.path import join, dirname, abspath, exists
if not SANET_INSTALL_DIR:
	print "SANET_INSTALL_DIR not set. Auto detect..."
	SANET_INSTALL_DIR = abspath(os.path.join(os.path.dirname(abspath(__file__)),"..",".." ))
	print "SANET_INSTALL_DIR set to ", SANET_INSTALL_DIR
sys.path.append( SANET_INSTALL_DIR )

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "conf.settings")

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

import time
import sys
import os
import socket
import traceback
import logging
import logging.handlers
import threading
import json
import traceback

import Queue

from sanet_poller.agent import BaseAgent

from sanet_common.sanetdefs.ipc import SANETD_INFO_QUEUE

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class SanetdThread(threading.Thread):
	"""
	Demone che si occupa di stare in ascolto di messaggi dall'esterno.
	
	- Messaggi da agenti che devono caricare configurazioni
	- Messaggi di agenti con valori di datasource da aggiornare
	- Messaggi di agenti con eventi su cambi di stato di condition
	"""

	def __init__(self, name, run_callback):

		# Do not block main program on exit
		super(SanetdThread, self).__init__()

		# Don't block main program on exit
		# FIX: hack for old API version.
		if (hasattr(self, "daemon")):
			self.daemon = True

		if (hasattr(self, "daemon")):
			self.setDaemon(True)

		self.name = name
		self.run_callback = run_callback

	def run(self):
		self.run_callback()

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class UpdateEvents(object):
	"""
	Classe Wrapper
	"""

	def __init__(self, events):
		self.events = events

	def __str__(self):
		return "events(%s)" % (len(self.events))

	def __len__(self):
		return len(self.events)

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

from sanet_common.threadutils import AbstractDaemonThread

class SanetServerThreadPool(object):
	"""
	Semplice thread pool per gestire le richieste concorrenti
	"""

	def __init__(self, server, num_workers=5):

		self.server = server

		self.num_workers        = num_workers
		self.workers            = []
		self.work_queue         = Queue.Queue(-1)
		self.work_queue_timeout = -1

	def init_workers(self):
		for i in xrange(self.num_workers):
			t = SanetServerListenWorkerThread(i, self.server, self)
			self.workers.append(t)
			t.start()

	def assign_client_to_worker(self, handler_function, params):
		self.work_queue.put_nowait( (handler_function, params) )

	def wait_for_workers(self):
		for worker in self.workers:
			if worker.isAlive():
				worker.join(1)

	def get_info_text(self, all_threads_info=True):

		infos = self.get_info(all_threads_info=all_threads_info)

		t_info = []
		format_str = '  '.join(    ('%10s'    , '%7s'   , '%5s'  , '%4s'  , '%5s' , '%s'  ) )
		header     = format_str  % ('thread'  , 'status', 'done' , 'errs' , 'time', 'job' )
		t_info.append(header)
		
		for line in infos['threads']:
			t_line = format_str % line
			t_info.append(t_line)

		t_info.append('Request queue size: %s (max:%s)' % (infos['wait_queue'], infos['wait_queue_max']   ) )

		return '\n'.join(t_info)

	def get_info(self, all_threads_info=True):

		now = time.time()

		infos = {
		}

		#
		# Threads
		#
		t_info = []
		for t in self.workers:

			line = (
				t.tid,
				t.status,
				t.count,
				t.errors,
				"%5.2f" % (now - t.current_start, ) if t.current_start else '-',
				t.current_work[1] if t.current_work else '',
			)

			t_info.append( line )

		infos['threads'] = t_info

		#
		# Other 
		#
		infos['wait_queue']     = self.work_queue.qsize()
		infos['wait_queue_max'] = self.work_queue.maxsize

		return infos

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class SanetServerListenWorkerThread(AbstractDaemonThread):

	def __init__(self, tid, server, thread_pool):

		super(SanetServerListenWorkerThread, self).__init__()

		self.tid           = tid
		self.thread_pool = thread_pool

		self.server = server

		self.status        = 'w'
		self.count         = 0
		self.errors        = 0
		self.current_work  = None
		self.current_start = 0

	def run(self):
		while 1:

			self.status        = 'w'
			self.current_work  = None
			self.current_start = 0

			try:
				data = self.thread_pool.work_queue.get(True, 1)

				self.current_work = data

				self.server.logger.log(1, "Listen-Worker(%s): working on %s", self.tid, data )

				handler_function, params = data

				self.status       = 'r'
				self.current_start = time.time()

				handler_function(*params)

				self.count += 1

			except Queue.Empty, er:
				data = None

				self.errors += 1


class SanetServer(object):

	SANETD_CLT_QUEUE    = 'sanetd-ctl'

	SANETD_EVENTS_QUEUE = 'sanetd-events'

	def __init__(self, instanceid, logger, mail_handler, ipc_server, ipc_auth, datasource_store, cache_manager, cache_manager_conf,  alarm_handlers,  working_threads, working_thread_queue_size):#, watchdog_mail):

		self.instanceid = instanceid

		self._stop = False

		self.logger = logger
		self.mail_handler = mail_handler

		self.queue_broker_address = ipc_server
		self.queue_broker_auth    = ipc_auth

		self.datasource_store = datasource_store

		# cache for confgiuration and ..
		self.cache_manager   = cache_manager
		self.cache_manager_conf = cache_manager_conf

		self.tenants_by_id = {}
		self.agents_by_id  = {}

		
		self.threads = {}

		#
		# Thread pool per gestione dei comandi dall'esterno
		#
		self.thread_pool = None

		#
		# Coda IPC per comandi di diagnostica
		#
		self.queue_info = None

		#
		# Coda IPC per ricevere gli eventi
		#
		self.queue_events      = None
		self.queue_read_count  = 0
		self.queue_read_last   = 0
		
		self.events_chunks     = 0
		self.events_processed  = 0

		# 
		self.pending_events_to_process = []


		#
		# Thread pool per il processing degli eventi
		#
		self.events_threadpool = None

		# max number of working thread
		self.num_events_working_threads = working_threads

		# set queue size max for 60 threads
		self.queue_size_working_threads = working_thread_queue_size


		#
		# Coda interna per i messaggi di allarme
		#
		self.alarms_queue       = None
		self.alarms_queue_count = None

		self.alarms_sent = 0
		self.alarms_error = 0

		self.logger.debug("Initialized")
		
		self.alarm_handlers = alarm_handlers

		return

	def load_active_tenant_agents(self):

		self.logger.info("Loading valid tenants and agents")

		self.tenants_by_id = {}
		self.agents_by_id  = {}

		tenants = tenant_select_all() 
		for t in tenants:
			self.logger.info("Tenant: %s", t)

			self.tenants_by_id[t.uuid] = t

			for a in agent_select_all(t.name):
				self.logger.info("\tAgent: %s", a)
				self.agents_by_id[a.uuid] = a

	def is_valid_tenant(self, uuid):
		return uuid in self.tenants_by_id

	def get_valid_tenant(self, uuid):
		return self.tenants_by_id.get(uuid, None)

	def is_valid_agent(self, uuid):
		return uuid in self.agents_by_id

	def get_valid_agent(self, uuid):
		return self.agents_by_id.get(uuid, None)

	def start(self):
		self.logger.debug("Started")

		self._stop = False

		self.logger.debug("Loading tenants & agents info")

		# FIX: more initialization here?
		self.load_active_tenant_agents()

		self.run()


	def stop(self):
		self._stop = True

	def run(self):
		self.logger.info("Running...")

		self.logger.info("Init internal listen threadpool")
		self.thread_pool = SanetServerThreadPool(self)
		self.thread_pool.init_workers()

		self.logger.info("Init mail sender...")
		self.mail_sender = SanetdSimpleMailSender(self.logger, self.mail_handler)
		self.mail_sender.start()

		self.logger.info("Launching info thread...")
		it = SanetdThread('INFO', self.handleInfoCommands)
		it.start()
		self.threads[it.name] = it

		self.logger.info("Launching ctl thread...")
		ct = SanetdThread('CTL', self.handleCtlCommands)
		ct.start()
		self.threads[ct.name] = ct

		#self.logger.info("Launching ctl socket thread...")
		#ct = SanetdThread('CTLSOCK', self.handleCtlCommandsFromSocket)
		#ct.start()
		#self.threads[ct.name] = ct

		self.init_event_thread_pool()

		self.logger.info("Launching event thread...")
		et = SanetdThread('EVENT', self.handleEventsFromAMQP)
		et.start()
		self.threads[et.name] = et

		self.logger.info("Launching alarm thread...")
		self.alarms_queue = Queue.Queue(-1)
		self.alarms_queue_count = 0
		self.alarms_queue_timeout = 1 # second
		et = SanetdThread('NOTIFY', self.handleAlarms)
		et.start()
		self.threads[et.name] = et

		#
		# Wait for termination: wait all thread
		#

		self.thread_pool.wait_for_workers()

		for t in self.threads.values():
			self.logger.info("Waiting for thread %s..." % (t.name))
			while t.isAlive():
				t.join(1)
			self.logger.info("Thread %s stopped" % (t.name))

		self.logger.info("Stopping mail sender")
		self.sender.stop()

		self.logger.info("Exiting...")

	def init_event_thread_pool(self):

		self.logger.info("Creating threads for events (workers: %s)" % (self.num_events_working_threads))

		self.events_threadpool = PollThreadPool(self.logger, queue_size=self.queue_size_working_threads)

		self.events_threadpool.createWorkers(self.num_events_working_threads, pre_run_callback=self._init_events_working_thread_callback)

		return

	def _init_events_working_thread_callback(self, thread_obj):
		log = thread_obj.getLogger()
		log.debug("Thread[%s] started (%s)", thread_obj.id, get_current_thread())

		#
		# NOTA: per cercare di migliorare le performance vediamo se una copia locale per thread
		#       velocizza i working thread del server
		# 
		log.info("Thread[%s] creating cache manger (%s)", thread_obj.id, get_current_thread())
		local_cache = self.cache_manager.clone()

		thread_obj.thread_cache_manager = local_cache

		return

	def is_stopping(self):
		return self._stop

	#--------------------------------------------------------------#
	#                                                              #
	#--------------------------------------------------------------#

	def create_ipc_connection(self):
		return create_ipc_connection(self.logger, self.queue_broker_address, self.queue_broker_auth)

	#--------------------------------------------------------------#
	#                                                              #
	#--------------------------------------------------------------#

	#def handleCtlCommandsFromSocket(self):
	#
	#	self.logger.info("[ctl socket] Start...")
	#
	#	sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	#	sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
	#	server_address = ('127.0.0.1', 3456)
	#	sock.bind(server_address)
	#	sock.listen(5)
	#
	#	while True:
	#		self.logger.debug("[clt socket] listen...")
	#
	#		try:
	#			connection, addr = sock.accept()
	#
	#			self.logger.info("[ctl socket] accepted from %s", addr)
	#
	#			msg = connection.recv(1024)
	#
	#			try:
	#				request = decode_request(msg)
	#			except Exception, e:
	#				self.logger.error("[ctl socket] Invalid request format: %s", e)
	#				return
	#
	#			request_id = request['id']
	#			self.logger.log(9,"[ctl] Received ctl from %s", addr)
	#			self.logger.log(9,"[ctl]   id     = %s" % (request_id,))
	#			self.logger.log(9,"[ctl]   cmd    = %s" % (request['type'],))
	#			self.logger.log(9,"[ctl]   params = %s" % (request['params'],))
	#
	#			h_name = 'cmd_' + request['type']
	#			if not hasattr(self, h_name):
	#				self.logger.warning("[ctl] Received unknown command: %s, %s", request['type'], request['params'])
	#				return
	#
	#			handler = getattr(self, h_name) 
	#			try:
	#				response = handler(addr, request_id, request['params'])
	#			except Exception, e:
	#				msg = str(e) + traceback.format_exc()
	#				self.logger.error("[ctl] error while processing request %s: %s", request['type'], msg)
	#				response = encode_response_with_id( RESPONSE_ERROR, request_id, msg )
	#
	#			connection.sendall(response)
	#
	#			connection.close()
	#
	#		except Exception, e:
	#			self.logger.error(str(e))
	#			connection.close()
	#
	#	self.logger.info("[ctl socket] Exit")
	#
	#	return

	#--------------------------------------------------------------#
	#                                                              #
	#--------------------------------------------------------------#

	def handleInfoCommands(self):

			self.logger.info("[info] Start...")

			while True:

				try:
					queue_id = SANETD_INFO_QUEUE % (self.instanceid)

					self.queue_info = self.create_ipc_connection()

					self.queue_info.open()
					self.queue_info.bind(queue_id, persistent=False)

					self.logger.log(9, "Reading commands from %s", queue_id)

					while not self.is_stopping():
						self.receive_commands(self.queue_info)

				except IPCConnectionError, e:
					self.logger.error("Problem on connection. Trying reconnecting")
				except Exception, e:
					self.logger.error("Problem on connection. %s:%s", type(e), str(e))
				finally:
					try:
						if self.queue_info:
							self.queue_info.close()
					except Exception, e:
						pass

				time.sleep(3)

			self.logger.info("[info] Exit")

	def handleCtlCommands(self):

		self.logger.info("[ctl] Start...")

		while True:

			try:
				self.queue_ctl = self.create_ipc_connection()

				self.queue_ctl.open()
				self.queue_ctl.bind(self.SANETD_CLT_QUEUE, persistent=False)

				self.logger.log(9, "Reading commands from %s", self.SANETD_CLT_QUEUE)

				while not self.is_stopping():
					#self.receive_commands()
					self.receive_commands(self.queue_ctl)

			except IPCConnectionError, e:
				self.logger.log(9,"Problem on connection. Trying reconnecting")
			except Exception, e:
				self.logger.log(9,"Problem on connection. %s:%s", type(e), str(e))
			finally:
				try:
					if self.queue_ctl:
						self.queue_ctl.close()
				except Exception, e:
					pass

			time.sleep(3)

		self.logger.info("[ctl] Exit")

	def receive_commands(self, queue):

		try:
			self.logger.log(9, "[ctl] waiting...")

			(addr, msg) = queue.receive_msg(timeout=1, ack_msg=True)

			if msg == None:
				self.logger.log(9, "[ctl] waiting...")
				return

			try:
				request = decode_request(msg)
			except Exception, e:
				self.logger.error("[ctl] Invalid request format: %s", e)
				return

			self.handle_received_command(queue, addr, request)

		except IPCConnectionError, e:
			self.logger.error("[ctl] Error on connection: %s" % (str(e)))
			raise e

		except IPCException, e:
			self.logger.error("[ctl] Error on connection: %s" % (str(e)))
			raise e

		except Exception, e:
			self.logger.error("[ctl] Error while processing message: %s (%s - %s)", e, type(e), dir(e))
			self.logger.error("TYPE %s", type(e))
			self.logger.error("DATA %s", dir(e))

	def handle_received_command(self, queue, addr, request):
		self.thread_pool.assign_client_to_worker(self._handle_received_command, [queue, addr, request])

	def _handle_received_command(self, queue, addr, request):

		try:
			request_id = request['id']

			self.logger.log(9, "[ctl] Received ctl from %s", addr)
			self.logger.log(9, "[ctl]   id     = %s" % (request_id,))
			self.logger.log(9, "[ctl]   cmd    = %s" % (request['type'],))
			self.logger.log(9, "[ctl]   params = %s" % (request['params'],))

			h_name = 'cmd_' + request['type']
			if not hasattr(self, h_name):
				msg = "Received unknown command: %s, %s", request['type'], request['params']
				self.logger.warning("[ctl] %s", msg )
				response = encode_response_with_id( RESPONSE_ERROR, request_id, msg )
			else:
				handler = getattr(self, h_name) 
				try:
					response = handler(addr, request_id, request['params'])
				except IPCException, e:
					raise e
				except Exception, e:
					msg = str(e) + traceback.format_exc()
					self.logger.error("[ctl] error while processing request %s: %s", request['type'], msg)
					response = encode_response_with_id( RESPONSE_ERROR, request_id, msg )

			if response and addr != None:
				self.logger.info("[ctl] Sending response to %s", addr)

				queue.send_msg( addr, response, blocking=False)

		except IPCConnectionError, e:
			self.logger.error("[ctl] Error while sending response to %s: %s:%s -> %s", addr, type(e), e, traceback.format_exc())
			raise e

		except IPCException, e:
			self.logger.error("[ctl] Error while sending response to %s: %s:%s -> %s", addr, type(e), e, traceback.format_exc())
			raise e

		except Exception, e:
			self.logger.error("[ctl] Error while processing request from %s: %s:%s -> %s", addr, type(e), e, traceback.format_exc())


	#--------------------------------------------------------------#

	def cmd_help(self, addr, request_id, params):
		
		response = []
		for k in dir(self):
			if k.startswith('cmd_'):
				doc = getattr(self, k).__doc__
				if doc is None:
					doc = ''
				else:
					doc = doc.strip()
				response.append(
					"%s - %s" % (k[4:], doc)
				)

		response = '\n'.join(response)

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, response )

		return response

	#--------------------------------------------------------------#

	def cmd_stop(self, addr, request_id, params):
		"""
		Stop server
		"""
		self.logger.info("[ctl] Stop received")
		self.stop()

	#--------------------------------------------------------------#

	def cmd_info(self, addr, request_id, params):

		self.logger.info("[ctl] Info received")

		response = self._load_info_data()['text_version']

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, response )

		return response

	def cmd_info_json(self, addr, request_id, params):

		self.logger.info("[ctl] Info json received")

		response = json.dumps( self._load_info_data() )

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, response )

		return response

	def _load_info_data(self):

		data = {
			'instance': self.instanceid,

			'threads': [],
			'threadpool': 'not started',

			# Coda in INPUT
			'queue': {
				'size'     : 0,
				'read'     : 0,
				'last'     : 0,
				'processed': 0,
				'pending'  : 0,
			},

			# Coda del thread pool
			'processqueue': {
				'size': 0,
				'rate': 0,
			},

			# Thread
			'process': {
				'min' : 0,
				'max' : 0,
				'avg' : 0,
			},

			'critical_problems': [],
		}

		rows = []
		for t in self.threads.values():
			alive = t.isAlive()
			data['threads'].append( { 'name': t.name, 'alive': alive } )

			if not alive:
				data['critical_problems'].append("Thread %s not running" % (t.name))

		if self.events_threadpool:
			data['threadpool'] = 'started'
		
		# INPUT QUEUE
		data['queue']['size'     ] = self.get_event_queue_size()
		data['queue']['read'     ] = self.queue_read_count
		data['queue']['last'     ] = time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(self.queue_read_last))
		data['queue']['processed'] = self.events_processed
		data['queue']['chunks'   ] = self.events_chunks
		
		# PROCESSING QUEUE
		pqueue_infos = self.events_threadpool.get_info(all_threads_info=False)
		data['processqueue']['size'     ] = pqueue_infos['wait_queue']
		data['processqueue']['rate'     ] = pqueue_infos['rate'      ]
		data['processqueue']['processed'] = pqueue_infos['processed' ]
		
		print type(pqueue_infos), pqueue_infos
		
		data['process']['min'] = pqueue_infos['process']['min']
		data['process']['max'] = pqueue_infos['process']['max']
		data['process']['avg'] = pqueue_infos['process']['avg']

		# OUTPUT ALARMS
		data['alarms'] = self.get_alarms_info_data()

		data['text_version'] = self._format_info_data(data)

		return data


	def _format_info_data(self, data):

		response = ''

		rows = []

		rows += [
			"Instance: %s" % (data['instance'])
		]

		for t in data['threads']:
			rows.append( "Main Thread %s: %s" % (t['name'], t['alive']) )


		# EVENT QUEUE
		rows += ["Events Queue:"]
		rows += [
			"    size: %s" % ( data['queue']['size'     ] ),
			"    read: %s" % ( data['queue']['read'     ] ),
			"    last: %s" % ( data['queue']['last'     ] ),
			"  events: %s" % ( data['queue']['processed'] ),
			"  chunks: %s" % ( data['queue']['chunks'] ),
		]
		response += '\n'.join(rows)
		response += '\n'

		# EVENTS THREAD POOL
		response += "Thread pool: %s\n" % (data['threadpool'])
		if self.events_threadpool is not None:
			response += self.events_threadpool.get_info_text()
		response += '\n'

		# ALARMS
		rows = self.get_alarms_info_text(data['alarms'])
		response += '\n'.join(rows)
		response += '\n'

		# REQUESTS THREAD POOL
		response += '\n'
		response += 'Request thread pool:\n'
		if self.thread_pool is not None:
			response += self.thread_pool.get_info_text()
		response += '\n'


		if data['critical_problems']:
			rows = [ 'CRITICAL PROBLEMS' ] + data['critical_problems']
			response += 'CRITICAL PROBLEMS\n' + '\n'.join( [ '* %s' % (p) for p in data['critical_problems'] ] )
			response += '\n'

		return response

	#--------------------------------------------------------------#

	def cmd_set_log_level(self, addr, request_id, params):

		level = int(params[0])

		self.logger.info("[ctl] Setting log level to %s", level)

		self.logger.setLevel(level)
		for handler in self.logger.handlers:
			handler.setLevel( level )

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, "ok" )

		return response

	#--------------------------------------------------------------#

	def cmd_get_agent_config(self, addr, request_id, params):

		self.logger.debug("[ctl] get_agent_config")

		tenant_name = params[0]
		agent_name  = params[1]

		self.logger.info("[ctl] from %s:%s id=%s", tenant_name, agent_name, request_id)

		ret = load_agent_config(tenant_name, agent_name)

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, ret )

		return response

	#--------------------------------------------------------------#

	def cmd_get_clusters(self, addr, request_id, params):

		tenantid = params[0]
		agentid  = params[1]

		self.logger.debug("[ctl] from %s:%s", tenantid, agentid)

		ret = COMPRESS_DATA( load_cluster_config(tenantid, self.cache_manager_conf) )

		self.logger.debug("[ctl] sending %s clusters", len(ret))

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, ret )
		return response

	def cmd_get_nodes(self, addr, request_id, params):

		tenantid = params[0]
		agentid  = params[1]

		self.logger.debug("[ctl] from %s:%s", tenantid, agentid)

		ret = COMPRESS_DATA( load_nodes_config(tenantid, agentid, self.cache_manager_conf) )

		self.logger.debug("[ctl] sending %s nodes", len(ret))

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, ret )
		return response

	def cmd_get_ifaces(self, addr, request_id, params):

		tenantid = params[0]
		agentid  = params[1]

		self.logger.debug("[ctl] from %s:%s", tenantid, agentid)

		ret = COMPRESS_DATA( load_ifaces_config(tenantid, agentid, self.cache_manager_conf) )

		self.logger.debug("[ctl] sending %s ifaces", len(ret))

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, ret )
		return response

	def cmd_get_storages(self, addr, request_id, params):

		tenantid = params[0]
		agentid  = params[1]

		self.logger.debug("[ctl] from %s:%s", tenantid, agentid)

		ret = COMPRESS_DATA( load_storages_config(tenantid, agentid, self.cache_manager_conf) )

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, ret )
		return response

	def cmd_get_services(self, addr, request_id, params):

		tenantid = params[0]
		agentid  = params[1]

		self.logger.debug("[ctl] from %s:%s", tenantid, agentid)

		ret = COMPRESS_DATA( load_services_config(tenantid, agentid, self.cache_manager_conf) )

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, ret )
		return response

	def cmd_get_devices(self, addr, request_id, params):

		tenantid = params[0]
		agentid  = params[1]

		self.logger.debug("[ctl] from %s:%s", tenantid, agentid)

		ret = COMPRESS_DATA( load_devices_config(tenantid, agentid, self.cache_manager_conf) )

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, ret )
		return response

	def cmd_get_links(self, addr, request_id, params):

		tenantid = params[0]
		agentid  = params[1]

		self.logger.debug("[ctl] from %s:%s", tenantid, agentid)

		ret = COMPRESS_DATA( load_links_config(tenantid, self.cache_manager_conf) )

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, ret )
		return response

	def cmd_get_datagroups_params(self, addr, request_id, params):

		tenantid = params[0]
		agentid  = params[1]

		self.logger.debug("[ctl] from %s:%s", tenantid, agentid)

		ret = COMPRESS_DATA( load_datagroup_params_config(tenantid, agentid, self.cache_manager_conf) )

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, ret )
		return response

	def cmd_get_datagroups(self, addr, request_id, params):

		tenantid = params[0]
		agentid  = params[1]

		self.logger.debug("[ctl] from %s:%s", tenantid, agentid)

		ret = COMPRESS_DATA( load_datagroups_config(tenantid, agentid, self.cache_manager_conf) )

		self.logger.debug("[ctl] sending %s datagroups", len(ret))

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, ret )
		return response

	def cmd_get_datasources(self, addr, request_id, params):

		tenantid = params[0]
		agentid  = params[1]

		self.logger.debug("[ctl] from %s:%s", tenantid, agentid)
		
		#ret = COMPRESS_DATA( load_datasources_config(tenantid, agentid, self.cache_manager_conf) )
		
		cache_manager = init_cache_manager_from_conf()
		ret = COMPRESS_DATA( load_datasources_config(tenantid, agentid, cache_manager) )
		cache_manager.close()

		self.logger.debug("[ctl] sending %s datasources", len(ret))

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, ret )
		return response

	def cmd_get_conditions(self, addr, request_id, params):

		tenantid = params[0]
		agentid  = params[1]

		self.logger.debug("[ctl] from %s:%s", tenantid, agentid)

		ret = COMPRESS_DATA( load_conditions_config(tenantid, agentid, self.cache_manager_conf) )

		self.logger.debug("[ctl] sending %s conditions", len(ret))

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, ret )
		return response

	#
	# Metodi utilizzati dagli agenti remoti per caricare on-demand dati per effettuare operazioni di check
	# fingendo che i dati siano caricati localmente.
	#
	def cmd_remote_agent_api(self, addr, request_id, params):

		# FIX: add parameters for tenant
		#tenant = tenant_get_primary()
		#db_set_current_tenant_schema(tenant.name)

		command = params[0]

		handler_name = 'remote_cmd_%s' % (command)
		handler = getattr(self, handler_name) 

		response = handler(addr, request_id, params[1:])
		return response

	def remote_cmd_get_cluster(self, addr, request_id, params):

		tid = params[0]
		cid = params[1]

		#tenant = Tenant.objects.get(uuid=tid)
		#tenant.db_set_current_schema()
		#self.logger.log(9, "Loading cluster %s", cid)
		#c = Cluster.objects.get(uuid=cid)
		#data = c.get_dict()

		local_configuration = LocalToRemoteConfiguration(self.logger, tid, self.cache_manager)
		data = local_configuration.get_cluster(cid)

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, data )
		return response	

	def remote_cmd_get_cluster_elements_ids(self, addr, request_id, params):

		tid = params[0]
		cid = params[1]

		#tenant = Tenant.objects.get(uuid=tid)
		#tenant.db_set_current_schema()
		#self.logger.log(9, "Loading cluster %s", cid)
		#c = Cluster.objects.get(uuid=cid)
		#elements = MonitoredElement.objects.filter(cluster=c).values_list("uuid")
		#data = [row[0] for row in elements]

		local_configuration = LocalToRemoteConfiguration(self.logger, tid, self.cache_manager)
		data = local_configuration.get_cluster_elements_ids(cid)

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, data )
		return response

	def remote_cmd_get_datagroup(self, addr, request_id, params):

		tid  = params[0]
		dgid = params[1]

		#tenant = Tenant.objects.get(uuid=tid)
		#tenant.db_set_current_schema()
		#self.logger.log(9, "Loading datagroup %s from db", dgid)
		#try:
		#	dg = DataGroup.objects.get(uuid=dgid)
		#except DataGroup.DoesNotExist, e:
		#	dg = DataGroup.objects.get(path=dgid)
		#dg.load_fields_from_cache(self.cache_manager)
		#dg_data = dg.get_dict_with_ds_and_c()

		local_configuration = LocalToRemoteConfiguration(self.logger, tid, self.cache_manager)
		data = local_configuration.get_datagroup(dgid)

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, data )
		return response

	def remote_cmd_get_datagroup_params(self, addr, request_id, params):

		tid  = params[0]
		dgid = params[1]

		#tenant = Tenant.objects.get(uuid=tid)
		#tenant.db_set_current_schema()
		#self.logger.log(9, "Loading datagroup %s params from db", dgid)
		#try:
		#	dg = DataGroup.objects.get(uuid=dgid)
		#except DataGroup.DoesNotExist, e:
		#	dg = DataGroup.objects.get(path=dgid)
		#data = [ p.get_dict() for p in dg.datagroupparameter_set.all() ]

		local_configuration = LocalToRemoteConfiguration(self.logger, tid, self.cache_manager)
		data = local_configuration.get_datagroup_params(dgid)

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, data )
		return response

	def remote_cmd_get_element_by_id(self, addr, request_id, params):

		tid = params[0]
		eid = params[1]

		#tenant = Tenant.objects.get(uuid=tid)
		#tenant.db_set_current_schema()
		#me = MonitoredElement.objects.get(uuid=eid)
		#e = me.get_specific_element()
		#e.load_fields_from_cache(self.cache_manager)
		#data = e.get_dict()

		local_configuration = LocalToRemoteConfiguration(self.logger, tid, self.cache_manager)
		data = local_configuration.get_element_by_id(eid)

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, data )
		return response

	def remote_cmd_get_element_by_parent_id(self, addr, request_id, params):

		tid = params[0]

		p_id = params[1]
		kind = int(params[2])

		#tenant = Tenant.objects.get(uuid=tid)
		#tenant.db_set_current_schema()
		#data = []
		#for me in MonitoredElement.objects.filter(parent__uuid=p_id, kind=kind):
		#	e = me.get_specific_element()
		#	e.load_fields_from_cache(self.cache_manager)
		#	data.append(e.get_dict())

		local_configuration = LocalToRemoteConfiguration(self.logger, tid, self.cache_manager)
		data = local_configuration.get_element_by_parent_id(p_id, kind)

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, data )
		return response

	def remote_cmd_get_datasource(self, addr, request_id, params):

		tid  = params[0]
		dsid = params[1]

		#tenant = Tenant.objects.get(uuid=tid)
		#tenant.db_set_current_schema()
		#self.logger.log(9, "Loading datasource from local db")
		#ds = DataSource.objects.get(uuid=dsid)
		#ds.load_fields_from_cache(self.cache_manager)
		#data = ds.get_dict()

		local_configuration = LocalToRemoteConfiguration(self.logger, tid, self.cache_manager)
		data = local_configuration.get_datasource(dsid)

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, data )
		return response

	def remote_cmd_get_condition(self, addr, request_id, params):

		tid = params[0]
		cid = params[1]

		#tenant = Tenant.objects.get(uuid=tid)
		#tenant.db_set_current_schema()
		#self.logger.log(9, "Loading condition from local db")
		#c = Condition.objects.get(uuid=cid)
		#c.load_fields_from_cache(self.cache_manager)
		#data = c.get_dict()

		local_configuration = LocalToRemoteConfiguration(self.logger, tid, self.cache_manager)
		data = local_configuration.get_condition(cid)

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, data )
		return response	

	def remote_cmd_get_condition_by_path(self, addr, request_id, params):

		tid  = params[0]
		path = params[1]

		#tenant = Tenant.objects.get(uuid=tid)
		#tenant.db_set_current_schema()
		#self.logger.log(9, "Loading condition from local db")
		#c = Condition.objects.get(path=cid)
		#c.load_fields_from_cache(self.cache_manager)
		#data = c.get_dict()

		local_configuration = LocalToRemoteConfiguration(self.logger, tid, self.cache_manager)
		data = local_configuration.get_condition_by_path(path)

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, data )
		return response

	def remote_cmd_get_condition_by_classification(self, addr, request_id, params):

		tid = params[0]
		cid             = params[1]
		node_id         = params[2]
		element_id      = params[3]

		#tenant = Tenant.objects.get(uuid=tid)
		#tenant.db_set_current_schema()
		#self.logger.log(9, "Loading condition from local db")
		#c = Condition.objects.get(path=cid)
		#c.load_fields_from_cache(self.cache_manager)
		#data = c.get_dict()

		local_configuration = LocalToRemoteConfiguration(self.logger, tid, self.cache_manager)
		data = local_configuration.get_condition_by_classification(cid, node_id, element_id)

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, data )
		return response	

	def remote_cmd_get_linked_iface_by_id(self, addr, request_id, params):

		tid = params[0]

		iface_id  = params[1]
		link_type = params[2]

		#tenant = Tenant.objects.get(uuid=tid)
		#tenant.db_set_current_schema()
		#try:
		#	i = MonitoredIface.objects.get(uuid=iface_id)
		#except Exception, e:
		#	raise ResourceNotFoundException("Interface %s not found" % (iface_id))
		#
		#link_elements = LinkElement.objects.filter(iface=i)
		#if link_type != None:
		#	link_elements = link_elements.filter(link__ltype=link_type)
		#link_elements = link_elements.all()
		#
		#data = None
		#if link_elements: 
		#	le1 = link_elements[0]
		#	link = le1.link
		#	try:
		#		le2 = LinkElement.objects.filter(link=link).exclude(iface=i)
		#		le2 = le2[0]
		#		data = le2.iface.get_dict()
		#	except Exception, e:
		#		raise ResourceNotFoundException("Linked iface %s for %s not found" % (iid, iface_id))

		local_configuration = LocalToRemoteConfiguration(self.logger, tid, self.cache_manager)
		data = local_configuration.get_linked_iface_by_id(iface_id, link_type)

		response = encode_response_with_id( RESPONSE_SUCCESS, request_id, data )
		return response	

	#--------------------------------------------------------------#
	#                                                              #
	#--------------------------------------------------------------#

	#
	# FIX: move 
	#

	def handleEventsFromAMQP(self):

		self.logger.info("[event] Start...")


		while True:

			try:
				self.logger.log(9,"[event] Opening connection: %s", self.SANETD_EVENTS_QUEUE)
				self.queue_events = self.create_ipc_connection()

				self.queue_events.open()
				self.queue_events.bind(self.SANETD_EVENTS_QUEUE, persistent=True)

				self.logger.log(9,"[event] Reading event from %s", self.SANETD_EVENTS_QUEUE)

				self.receive_events()

				self.logger.log(9,"[event] stop reading event from %s", self.SANETD_EVENTS_QUEUE)

			except IPCConnectionError, e:
				self.logger.error("[event] Problem on connection. Trying reconnecting")
			except Exception, e:
				self.logger.error("[event] Unkonwn: %s:%s -> %s" % (type(e), e, traceback.format_exc()))
			finally:
				pass

			try:
				self.logger.warning("[event] Closing event queue")
				if self.queue_events:
					self.queue_events.close()
			except Exception, e:
				self.logger.warning("[event] error while closing queue_events: %s:%s -> %s", e, traceback.format_exc())
				self.logger.warning("[event] Restart: %s", e)
				pass

			self.logger.info("[event] Problems. Waiting before retry loop.")
			time.sleep(2)

		self.logger.debug("[event] Exit")

	def get_event_queue_size(self):

		if self.queue_events == None:
			return None

		return self.queue_events.available()

	def receive_events(self):

		self.un_acked = 0

		self.pending_events_to_process = []
		
		ts_last_push = 0

		while not self.is_stopping():

			try:
				working_queue_len = self.events_threadpool.get_queue_size()
				self.logger.log(19, "Events to process: %s " % (working_queue_len))

				#
				# READ FROM QUEUE
				#
				(addr, msg) = self.queue_events.receive_msg(timeout=1, ack_msg=False)
				

				#
				# PROCESS
				#
				if msg != None:
	
					self.logger.log(19, "received = %s", len(msg))

					self.queue_read_count += 1
					self.queue_read_last   = time.time()

					self.un_acked += 1

					#
					# DECODE INPUT DATA
					#

					# IMPORTANT: from string (JSON or BASE64/BINARY) to memory
					events = BaseAgent.decode_update_events(msg)
					
					#import pprint
					#pprint.pprint(events)

					self.pending_events_to_process.extend( events )

				if self.pending_events_to_process:
					#
					# Need push?
					#
					MAX_PUSH_SECONDS = 1

					to_push      = len(self.pending_events_to_process)
					push_elapsed = time.time() - ts_last_push;

					self.logger.log(9, "To push=%s, elapsed=%.2f/%s",to_push, push_elapsed, MAX_PUSH_SECONDS)

					if to_push < 30 and (push_elapsed < MAX_PUSH_SECONDS):
						continue

					#
					# PUSH
					#
					while True:

						self.logger.log(9, "PUSHING: %s", len(self.pending_events_to_process) )

						try:
							self.process_events_with_worker(self.pending_events_to_process)

							# Update statistics
							self.events_chunks     += 1
							self.events_processed  += len(self.pending_events_to_process)

							# Reset push data
							ts_last_push = time.time()
							self.pending_events_to_process = []
			

							break

						except Queue.Full, e:
							self.logger.error("[event] Queue FULL: %s (type %s).", str(e), type(e) )
							#time.sleep(0.5)
						break

				#
				# ACK
				#
				if self.un_acked > 10 or (msg == None and self.un_acked > 0):
					self.queue_events.ack_pending()
					self.logger.log(9,"Acked messages (timeout=%s)", msg==None)
					self.un_acked = 0

			except IPCConnectionError, e:
				self.logger.error("[event] Error: %s (type %s)", str(e), type(e) )
				raise e

			except Exception, e:
				self.logger.error("[event] Unknown error: %s (type %s)", str(e), type(e) )
				self.logger.error("[event] RAISE")
				raise e

	def process_events_with_worker(self, events):
		
		self.alarms_queue_count += 1
		
		# Assegna gli eventi ricevuti al primo thread libero
		#params = [ UpdateEvents(events), ]
		params = [events, ]
		self.events_threadpool.execJob(self.event_worker_process_callback, params )
		
	def event_worker_process_callback(self, workingthread, queue_delay, update_events):

		cache_manager = workingthread.thread_cache_manager

		errors = False
		
		db.reset_queries()
		
		for event in update_events:
			error = self.process_event(cache_manager, event) or errors

		# close DB connection every time
		if errors:
			connection.close()

		self.logger.debug("process total db queries: %s", len(connection.queries))

		return

	def process_event(self, cache_manager, event):

		errors = False

		try:
			# FIX: Skip invalid events. More checks?
			tenant = self.get_valid_tenant(event['tenant_id'])
			if not tenant:
				self.logger.warning("Event from invalid tenant %s", event['tenant_id'])
				return

			#if not self.is_valid_agent(event['agent_id']):
			#agent = self.get_valid_agent(event['agent_id'])
			#if not agent:
			#	self.logger.warning("Event from invalid agent %s", event['agent_id'])
			#	return 

			new_alarms_to_send = process_update_event(self.logger, cache_manager, self.datasource_store, tenant, event['agent_id'], event)
			
			# FIX: disabled, buffering not working yet.
			#if event.has_key('buffered'):
			#	for a in new_alarms_to_send:
			#		a['buffered'] = 1

			#
			# Enqueue alarms
			#
			for new_alarm in new_alarms_to_send:
				self.push_alarm(new_alarm)

		except ConcurrentUpdateException, e:
			self.logger.warning("Concurrent update: %s", e)
			errors = True
			
		except Exception, e:
			self.logger.error("Error while processing event: %s %s -> %s", type(e), e, traceback.format_exc() )
			errors = True
			
		return errors

	def push_alarm(self, alarm_data):
		self.alarms_queue.put_nowait(alarm_data)

	#--------------------------------------------------------------#
	#                                                              #
	#--------------------------------------------------------------#

	def handleAlarms(self):

		self.alarms_sent  = 0
		self.alarms_error = 0

		self.logger.info("[alarm] Start...")

		while True:

			try:
				if self.is_stopping():
					self.logger.info("[alarm] waiting for alarms to notify")
					break

				alarm_data = None
				try:
					self.logger.debug("[alarm] waiting for alarms to notify")
					alarm_data = self.alarms_queue.get(True, self.alarms_queue_timeout) # block waiting
				except Queue.Empty, er:
					pass

				if not alarm_data:
					continue

				self._handler_alarm_with_processors(alarm_data)
				
			except Exception, e:
				self.logger.error("[alarm] Unknown error while processing alarm: %s:%s -> %s", type(e), str(e), traceback.format_exc())

		return

	def _handler_alarm_with_processors(self, alarm_data):

		tenant = Tenant.objects.get(uuid=alarm_data['tenantid'])
		tenant.db_set_current_schema()
		
		processed = False
		for handler in self.alarm_handlers:
			
			self.logger.info("[alarm] Processing alarm with handler %s", handler)
			
			processed = handler.process_alarm(self.mail_sender, alarm_data)
			if processed:
				self.logger.info("[alarm] Processing alarm with handler %s", handler)
				break 

		self.logger.info("[alarm] Finish processing")
	
		if processed:
			self.alarms_sent += 1
		else:
			self.alarms_error += 1

		return


	def get_alarms_info_text(self, data=None):
		
		if data is None:
			data = self.get_alarms_info_data()
		
		r = []
		
		r.append( "Alarms:" )
		r.append( "    pending: %s" % (data['pending']) )
		r.append( "    sent   : %s" % (data['sent'   ]) )
		r.append( "    errors : %s" % (data['errors' ]) )
		
		return r


	def get_alarms_info_data(self):
		
		return {
			'pending': self.alarms_queue.qsize() if self.alarms_queue is not None else -1,
			'sent'   : self.alarms_sent,
			'errors' : self.alarms_error,
		}

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class SanetServerWithSchedulingGroups(SanetServer):
	
	"""
	Nuova versione del server centrale che aggiunge supporto per gli scheduling-groups
	"""

	def cmd_get_scheduling_groups(self, addr, request_id, params):
        
		tenantid = params[0]
		agentid  = params[1]
        
		self.logger.debug("[ctl] from %s:%s", tenantid, agentid)
        
		ret = COMPRESS_DATA( load_scheduling_group_config(tenantid, self.cache_manager_conf) )
        
		self.logger.debug("[ctl] sending %s scheduling groups", len(ret))
        
		response = encode_response_with_id(RESPONSE_SUCCESS, request_id, ret)
		return response

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def load_configuration(self):

	conf = get_configuration()

	# Set log 
	self.log_level = conf.SANETD_LOG_LEVEL
	self.log_file  = os.sep.join([conf.LOG_DIR, 'sanetd.log'])

	self.queue_broker_address = conf.BROKER_ADDRESS
	self.queue_broker_auth    = conf.BROKER_AUTH

def init_sanetd_logger(name, level, file_name, stdout=False):

	logger = logging.getLogger(name)
	logger.setLevel( level )

	log_format = '%(asctime)s %(levelname)s ' + name + ' > %(message)s'

	hdlr = logging.handlers.WatchedFileHandler(file_name, "a")
	hdlr.setFormatter( logging.Formatter(log_format) )
	hdlr.setLevel( level )
	logger.addHandler(hdlr)

	if stdout:
		hdlr = logging.StreamHandler()
		hdlr.setFormatter( logging.Formatter(log_format) )
		hdlr.setLevel( level )
		logger.addHandler(hdlr)

	return logger

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def print_help():
	print "Options: <sanet dir> [options]"
	print ""
	print "\t", "-h         ", "Show help"
	print "\t", "-l <file>  ", "Specify log file"
	print "\t", "-L <level> ", "Log level"
	print "\t", "-v         ", "Log on standard output"

def sanetd_main(argv):

	import sys
	import getopt

	opt_help       = False
	opt_log_level  = None
	opt_log_file   = None
	opt_stdout     = False

	opt_instanceid = 'main'

	log_levels = {
		'DEBUG': logging.DEBUG,
		'INFO' : logging.INFO,
	}

	try:
		opts, args = getopt.getopt(argv, "hl:L:vI:")
		for o, a in opts:
			if   o in ("-h"):
				opt_help = True
				print_help()
				sys.exit(0)
			elif o in ("-l"):
				opt_log_file = a
			elif o in ("-L"):
				if log_levels.has_key(a):
					opt_log_level = log_levels[a]
				else:
					opt_log_level = int(a)

				print "Selected log level: %s", opt_log_level
			elif o in ("-v"):
				opt_stdout = True

			elif o in ("-I"):
				opt_instanceid = a


	except getopt.error, msg:
		print "Error while parsing options: %s" % msg
		print_help()
		sys.exit()
	except ValueError, ex:
		print "Invalid value for input parameter: %s" % ex
		print_help()
		sys.exit()

	conf = get_configuration()


	if opt_log_file == None:
		#opt_log_file = os.sep.join([conf.LOG_DIR, 'sanetd.log'])
		opt_log_file = os.sep.join([conf.LOG_DIR, 'sanetd-%s.log' % (opt_instanceid), ])

	if opt_log_level == None:
		opt_log_level = conf.SANETD_LOG_LEVEL
		
	logger = init_sanetd_logger('sanetd-%s' % (opt_instanceid), opt_log_level, opt_log_file, opt_stdout)
	
	ipc_server = conf.BROKER_ADDRESS
	ipc_auth   = conf.BROKER_AUTH
	logger.info("Broker address %s", ipc_server)

	ipc_server = conf.BROKER_ADDRESS
	ipc_auth   = conf.BROKER_AUTH
	
	logger.info("Loading datasource store handler = %s %s", conf.DATASOURCE_STORAGE_HANDLER, conf.DATASOURCE_STORAGE_PARAMS)
	datasource_store = init_datasource_store(conf.DATASOURCE_STORAGE_HANDLER, conf.DATASOURCE_STORAGE_PARAMS)
	
	logger.info("Loading cache handler")
	cache_manager      = init_cache_manager(conf.CACHE_HANDLER, conf.CACHE_HANDLER_PARAMS)
	cache_manager_conf = init_cache_manager(conf.CACHE_HANDLER, conf.CACHE_HANDLER_PARAMS)
	
	logger.info("Loading mail handler = %s %s", conf.SANETD_MAIL_HANDLER, conf.SANETD_MAIL_HANDLER_PARAMS)
	mail_handler_class = load_mail_handler_class(conf.SANETD_MAIL_HANDLER)
	mail_handler       = mail_handler_class(logger, conf.SANETD_MAIL_HANDLER_PARAMS)

	logger.info("Loading alarm handlers = %s", conf.SANET_ALARM_PROCESSORS)
	alarm_handlers = load_alarm_handlers(logger)

	logger.info("Loaded alarm handlers = %s", alarm_handlers)

	logger.info("Creating server")

	# FIX: Remove this switch 
	SANET_NO_SCHEDULINGGROUPS = os.environ.get('SANET_NO_SCHEDULINGGROUPS', False)
	if SANET_NO_SCHEDULINGGROUPS:
		server_class = SanetServer
	else:
		server_class = SanetServerWithSchedulingGroups


	s = server_class(

		opt_instanceid,

		logger, 

		mail_handler, 
		ipc_server, 
		ipc_auth, 
		datasource_store, 
		cache_manager,
		cache_manager_conf,
		
		alarm_handlers,
		
		conf.SANETD_WORKING_THREADS_NUM, 
		conf.SANETD_WORKING_THREADS_QUEUE_SIZE
	)
	s.start()

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

if __name__ == "__main__":

	reload(sys)
	sys.setdefaultencoding('UTF8')

	# Be sure django will find modules with the right path
	if len(sys.argv) < 1:
		print "usage: <sanet dir> [options]"
		print sys.exit(-1)

	if not os.path.exists(sys.argv[1]):
		print "Invalid directory: %s" % (sys.argv[1])
		sys.exit(-1)

	sanet_install_dir = sys.argv[1]
	if not sanet_install_dir in sys.path:
		print __file__, "USING:", sanet_install_dir
		sys.path.insert(0, sanet_install_dir)


	print "Using SANET_INSTALL_DIR=", sanet_install_dir
	
	#
	# DJANGO SETUP
	#

	os.environ.setdefault("DJANGO_SETTINGS_MODULE", "conf.settings")

	# Added for django 1.11
	import django
	django.setup()

	#
	# Imports
	#

	from django.db import connection


	from sanet_common.ipc import *
	from sanet_common.mailing import load_mail_handler_class


	from conf import get_configuration

	from core.models import *
	from core.models import Condition
	from core.models import Agent, Tenant, DataGroup, DataSource, Condition, Event, Alarm
	from core.models import MonitoredIface
	from core.exceptions import ConcurrentUpdateException
	from core.localpolling import LocalToRemoteConfiguration

	from core.api.base import db_set_current_tenant_schema
	from core.api.base import * # FIX BAD
	from core.api.datasource import init_datasource_store

	from core.api.cache import init_cache_manager
	from core.api.cache import init_cache_manager_from_conf
	
	#-----------------------------------------------------------------------------#
	#                                                                             #
	#-----------------------------------------------------------------------------#

	from sanet_poller.lib.threadutils import PollThreadPool, get_current_thread
	from sanet_poller.agent import COMPRESS_DATA

	from server.sanetd.sanetdlogic import load_agent_config
	from server.sanetd.sanetdlogic import load_scheduling_group_config
	from server.sanetd.sanetdlogic import load_cluster_config
	from server.sanetd.sanetdlogic import load_nodes_config
	from server.sanetd.sanetdlogic import load_ifaces_config
	from server.sanetd.sanetdlogic import load_storages_config
	from server.sanetd.sanetdlogic import load_devices_config
	from server.sanetd.sanetdlogic import load_services_config
	from server.sanetd.sanetdlogic import load_links_config
	from server.sanetd.sanetdlogic import load_datagroup_params_config
	from server.sanetd.sanetdlogic import load_datagroups_config
	from server.sanetd.sanetdlogic import load_datasources_config
	from server.sanetd.sanetdlogic import load_conditions_config

	from server.sanetd.sanetdlogic import create_ipc_connection

	from server.sanetd.sanetdlogic import process_update_event
	
	from server.sanetd.sanetdlogic import load_alarm_handlers

	from server.sanetd.mailsender import SanetdSimpleMailSender

	#
	# Il server centrale puo' eseguire comandi esterni *demonizzati*.
	#
	# Dobbiamo essere sicuri che il kernel liberi le risorse associate a questi
	# processi quando terminano.
	#
	import signal
	signal.signal(signal.SIGCHLD, signal.SIG_IGN)

	
	argv = sys.argv[2:]
	sanetd_main(argv)

