# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os
import time


from sanet_common.sanetdefs.ipc import SANETD_STATE_QUEUE

from sanet_common.ipc import IPCConnection
from sanet_common.ipc import RESPONSE_SUCCESS
from sanet_common.ipc import decode_response, encode_request, encode_request, decode_response
from sanet_common.ipc import encode_request_with_id

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def statusd_ctl(command, arguments):

	mid = str(int(time.time()))

	request = encode_request_with_id(command, mid, arguments)


	broker = "sanet/sanet@localhost"
	
	host = None
	port = None
	user = None
	password = None
	
	auth, addr = broker.split('@')
	user,password = auth.split('/')
	addr = addr.split(':')
	host = addr[0]
	if len(addr) > 1:
		port = int(addr[1])
	
	connect_options = {}
	if port     != None: connect_options['port']     = port
	if user     != None: connect_options['username'] = user
	if password != None: connect_options['password'] = password

	c = IPCConnection( host, **connect_options)
	
	c.settimeout(3)
	
	c.open()
	
	c.bind("temp-statusddctl-pid-%s" % (os.getpid()), persistent=False)

	c.send_msg( SANETD_STATE_QUEUE , request, blocking=True)
	
	addr, data = c.receive_msg(ack_msg=True)
	if data is None:
		raise Exception("Timeout")
	c.close()
	response = decode_response(data)
	
	if response['status'] == RESPONSE_SUCCESS:
		data = response['content']['data']
		return data
	else:
		raise Exception("Error from remote host: %s" % (response['content']))

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def statusd_get_server_status():
	
	status_data = statusd_ctl('get_server_status', [])
	
	return status_data
