/**
 * Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
 * Created: 5/27/13, 3:37 PM
 * Author: Antonio Vaccarino
 */


/*format = {
    "url" : string
    "data" : {
        "headers": [],
        "values": [], // rows as [], each with headers.length inside
        "ref": [] // one per row, if object can contain properties foldicon with id of fontawesome symbol and foldtitle with title to be displayed on tooltip
    },
    "filters" : {
        "$fieldname": {
            "type": string ("text", "date", "select") // defaults to text
            "select": [] (list of options)
        }
    },
    "ctx": {
        "results": int, // total results
        "offset": int, // position of the first element in values respective to results
        "paged": int, // number of results for each page, can be * to mean it's all ONE PAGE
    },
    "req" : {
        "filter": {
            "enabled": boolean,
            "field": string??
            "criteria": string
        },
        "sort": {
            "field": string,
            "dir": boolean
        }
    }
};

also supports WPARAMS['hidefilter'] = true
*/

ajaxtable = {};


// VERBATIM FROM DYNAMIC TABLE
ajaxtable.assemble = function ()
{

//    console.log("Starting assemble on dynamic table");

    // does NOT call winit on the children since we do not know yet what kind they are

    var displaydiv = lwp.toDisplay.call(this);
    $(this).addClass("jswlib_dyntable_master");

    var id_ctrl = lwp.run.call(this, 'getid_ctrl');
    var id_table = lwp.run.call(this, 'getid_table');

    var div_ctrl = $("<div/>");
    div_ctrl.attr('id', id_ctrl);
    div_ctrl.addClass("jswlib_dyntable_ctrl");

    var div_table = $("<div/>");
    div_table.attr('id', id_table);
    div_table.addClass("jswlib_dyntable_table");

    displaydiv.append(div_ctrl);
    displaydiv.append(div_table);

//    console.log("Finished assemble on dynamic table");

};

// VERBATIM FROM DYNAMIC TABLE
ajaxtable.getid_table = function ()
{
    return $(this).attr('id')+"_sub_table";
};

// VERBATIM FROM DYNAMIC TABLE
ajaxtable.getdiv_table = function ()
{
    return $(this).find("#"+lwp.run.call(this, 'getid_table'));
};

// VERBATIM FROM DYNAMIC TABLE
ajaxtable.getid_ctrl = function ()
{
    return $(this).attr('id')+"_sub_ctrl";
};

// VERBATIM FROM DYNAMIC TABLE
ajaxtable.getdiv_ctrl= function ()
{
    return $(this).find("#"+lwp.run.call(this, 'getid_ctrl'));
};

ajaxtable.validate = function (data)
{
    console.log("Table dataset ",data);
//    console.log("STARTING VALIDATION");
    if (!$.isPlainObject(data))
    {
        console.error("Ajax table dataset is not a dict-like object ",typeof data);
        return false;
    }

    if (data.hasOwnProperty("filters") && !$.isPlainObject(data['filters']))
    {
        console.warn("Filters list for ajax table is in invalid format, defaulting");
        delete(data['filters']);
    }

    if (!data.hasOwnProperty('data') ||
        !data.hasOwnProperty('ctx') ||
        !data.hasOwnProperty('req'))
    {
        console.log("Missing one of three main keys: data, ctx or req");
        return false;
    }

    if (!data['data'].hasOwnProperty('headers') ||
        !data['data'].hasOwnProperty('values'))
    {
        console.log("Missing headers or values element in table data");
        return false;
    }

//    console.log("checking ref length match");
    if (data['data'].hasOwnProperty('ref') &&
        data['data']['ref'].length > 0 &&
        data['data']['ref'].length != data['data']['values'].length)
    {
        console.log("Length mismatch between values and ref in foldable");
        return false;
    }
    else if (!data['data']['ref'])
    {
        data['data']['ref'] = [];
    }
//    console.log("ref matches values");

    if (data['data'].hasOwnProperty('classes') &&
        data['data']['classes'].length > 0 &&
        data['data']['classes'].length != data['data']['values'].length)
    {
        console.log("Length mismatch between values and row classes");
        return false;
    }
    else if (!data['data']['classes'])
    {
        data['data']['classes'] = [];
    }

    if (!data['req'].hasOwnProperty('filter') ||
        !data['req'].hasOwnProperty('sort'))
    {
        console.log("Missing filter or sort elemeent in table req");
        return false;
    }


    // arrayfication of ref objects
    for (var i = 0; i < data['data']['ref'].length; i++)
    {
        var cref = data['data']['ref'][i];
        if (!$.isArray(cref))
        {
            console.log ("A row must always contain an array of reference objects");
            return false;
        }
    }

    console.log("Completed validation");

    return true;

};

ajaxtable.ftypes = ["text", "date", "select"];

ajaxtable.emerge = function ()
{
    console.log("performing emerge");
    var data = $(this).data('wdata');
//    console.log("SOURCE DATA: ",data);

    if (!data.hasOwnProperty('filters'))
    {
        data['filters'] = {};
        var headers = data['data']['headers'];
        for (var i = 0; i < headers.length; i++)
        {
            var ckey = headers[i];
            data['filters'][ckey] = {
                type: "text"
            };
        }
    }
    else
    {
        var filterlist = Object.keys(data['filters']);
        for (var i = 0; i < filterlist.length; i++)
        {
            var ckey = filterlist[i];

//            if (data['data']['headers'].indexOf(ckey)==-1)
//            {
//                delete(data['filters'][ckey]);
//                continue;
//            }

            if (!data['filters'][ckey].hasOwnProperty("type") ||
                ajaxtable.ftypes.indexOf(data['filters'][ckey]['type'])==-1 ||
                (
                    data['filters'][ckey]['type'] == "select" &&
                    (
                        !data['filters'][ckey].hasOwnProperty('select') ||
                        !$.isArray(data['filters'][ckey]['select'])
                    )
                )
            )
            {
                data['filters'][ckey] = "text";
            }
        }
    }



    if (!data['ctx'].hasOwnProperty('results'))
    {
        data['ctx']['results'] = data['data']['values'].length;
    }

    if (!data['ctx'].hasOwnProperty('offset'))
    {
        data['ctx']['offset'] = 0;
    }

    if (!data['ctx']['paged'])
    {
        data['ctx']['paged'] = "*";
    }

    if (!data['req']['filter']['enabled'] ||
        !data['req']['filter']['field'] ||
        !data['req']['filter']['criteria'])
    {
        data['req']['filter']['enabled'] = false;
    }

    if (!data['req']['sort']['enabled'] ||
        !data['req']['sort']['field'])
    {
        data['req']['sort']['enabled'] = false;
    }

    if (!data['req']['sort']['dir'] !== true)
    {
        data['req']['sort']['dir'] = false;
    }

	data['flags'] = {};
	data['flags']['hidefilters'] = ($(this).data('wparams'))['hidefilters'] === true;

    console.log("Emerged data as ",data);
    lwp.toWlive.call(this, data, 'tabledata');
};

ajaxtable.ctrl = {};

ajaxtable.ctrl.rendercontroller = function ()
{


    var data = lwp.fromWlive.call(this, 'tabledata');
    var req = data['req'];

    // main controls order
    var controller = $("<div/>");
    controller.addClass("jswlib_ajaxtable_ctrl_toolbar");

    // control to decide how to filter the results
    var sd_filter = $("<div/>");
    sd_filter.addClass("jswlib_ajaxtable_ctrl_block");
    sd_filter.addClass("jswlib_ajaxtable_ctrl_filter");

    var sdf_fieldsel = $("<select/>");
    sdf_fieldsel.addClass("jswlib_ajaxtable_ctrl_filter_fieldsel");
    var filterfields = Object.keys(data['filters']);
    filterfields.unshift("");
	var num_filters = filterfields.length;
    for (var i = 0; i < num_filters; i++)
    {
        var cfield = filterfields[i];
        var option = $("<option/>");
        option.val(cfield);
        option.append(cfield);
        sdf_fieldsel.append(option);
    }

    var sdf_criteria = $("<input type=\"text\"/>");
    sdf_criteria.addClass("jswlib_ajaxtable_ctrl_filter_criteria");

    if (req['filter']['enabled'])
    {
        sdf_criteria.val(req['filter']['criteria']);
        sdf_fieldsel.val(req['filter']['field']);
    }

    var sdf_on = $("<span/>");
    sdf_on.addClass("jswlib_ajaxtable_ctrl_filter_on");
    sdf_on.append('<i class="icon-filter"></i>');
    sdf_on.addClass("jswlib_btncontrol");
    sdf_on.addClass("jswlib_flag_notextsel");
    sdf_on.bind('click', {widget: this}, ajaxtable.ctrl.requestFilter);

    var sdf_off = $("<span/>");
    sdf_off.addClass("jswlib_ajaxtable_ctrl_filter_off");
    sdf_off.append('<i class="icon-remove"></i>');
    sdf_off.addClass("jswlib_btncontrol");
    sdf_off.addClass("jswlib_flag_notextsel");
    sdf_off.bind('click', {widget: this}, ajaxtable.ctrl.requestClean);

    sd_filter.append(sdf_fieldsel);
    sdf_fieldsel.bind('change', {widget: this}, ajaxtable.ctrl.redrawFilterCriteria);
    sd_filter.append(sdf_criteria);
    sd_filter.append(sdf_on);
    sd_filter.append(sdf_off);

    // control to decide which page of results we check
    var sd_pager = $("<div/>");
    sd_pager.addClass("jswlib_ajaxtable_ctrl_pager");
    sd_pager.addClass("jswlib_ajaxtable_ctrl_block");

    var resnum = data['ctx']['results'];
    var offset = data['ctx']['offset'];
    var pagesize = data['ctx']['paged'];
    var paged = pagesize != "*" && pagesize < resnum;

    if (paged)
    {
        var pagelist = [];
        var prange = 2;
        var pages = Math.ceil(resnum/pagesize);
        var cpage = Math.ceil(offset/pagesize);
//        console.log("Data renderd at page "+cpage+" of "+pages+" with pagesize "+pagesize+" offset by "+offset);

        var lowerend = cpage - prange;
        var upperend = cpage + prange;

        var addlower = upperend >= pages ? upperend-(pages-1) : 0;
        var addupper = lowerend < 0 ? -lowerend : 0;

        lowerend = Math.max (0, lowerend-addlower);
        upperend = Math.min (pages-1, upperend+addupper);

        // single entry bookends do not use the << < ... > >> links
        if (lowerend == 1)
        {
            lowerend = 0;
        }

        if (upperend == pages-2)
        {
            upperend = pages-1;
        }

        if (lowerend > 1)
        {
            // bigger bookend
//            console.log("PAGING |<< < ...");

            var entry = $("<span/>");
            entry.addClass("jswlib_ajaxtable_ctrl_pagerlink");
            entry.addClass("jswlib_flag_notextsel");
            entry.addClass("jswlib_ajaxtable_ctrl_tostart");
            entry.append('<i class="icon-fast-backward"></i>');
            entry.bind('click', {pagenum: 0, widget: this}, ajaxtable.ctrl.requestPage);

            pagelist.push(entry);

//            pagelist.push(" ... ");
        }


        if (lowerend < cpage)
        {
            var entry = $("<span/>");
            entry.addClass("jswlib_ajaxtable_ctrl_pagerlink");
            entry.addClass("jswlib_flag_notextsel");
            entry.append('<i class="icon-backward"></i>');
            entry.bind('click', {pagenum: cpage-1, widget: this}, ajaxtable.ctrl.requestPage);
            pagelist.push(entry);
        }

        for (var p = lowerend; p <= upperend; p++)
        {
//            console.log("PAGING ",p);

            var entry = $("<span/>");
            entry.addClass("jswlib_flag_notextsel");
            if (p == cpage)
            {
                entry.addClass("jswlib_ajaxtable_ctrl_pagerlink_current");
            }
            else
            {
                entry.addClass("jswlib_ajaxtable_ctrl_pagerlink");
                entry.bind('click', {pagenum: p, widget: this}, ajaxtable.ctrl.requestPage);
            }
            entry.append(p+1);

            pagelist.push(entry);
        }

        if (upperend > cpage)
        {
            var entry = $("<span/>");
            entry.addClass("jswlib_ajaxtable_ctrl_pagerlink");
            entry.addClass("jswlib_flag_notextsel");
            entry.append('<i class="icon-forward"></i>');
            entry.bind('click', {pagenum: cpage+1, widget: this}, ajaxtable.ctrl.requestPage);
            pagelist.push(entry);
        }

        if (upperend < pages-2)
        {
            // bigger bookend
//            console.log("PAGING ... > >>|");

            var entry = $("<span/>");
            entry.addClass("jswlib_ajaxtable_ctrl_pagerlink");
            entry.addClass("jswlib_flag_notextsel");
            entry.addClass("jswlib_ajaxtable_ctrl_toend");
            entry.append('<i class="icon-fast-forward"></i>');
            entry.bind('click', {pagenum: pages-1, widget: this}, ajaxtable.ctrl.requestPage);
//            pagelist.push(" ... ");

            pagelist.push(entry);

        }

        for (var e = 0; e < pagelist.length; e++)
        {
            sd_pager.append(pagelist[e]);
        }


    }

    // RENDERING ON WIDGET
	if (!data['flags']['hidefilters'])
	{
		controller.append(sd_filter);
	}
    controller.append(sd_pager);

    if (paged)
    {
        // simpler sizer, passive
        var sd_sizer = $("<div/>");
        sd_sizer.addClass("jswlib_ajaxtable_ctrl_sizer");
        sd_sizer.addClass("jswlib_ajaxtable_ctrl_block");
        var sizer_from = cpage * pagesize + 1;
        var sizer_to = (cpage+1) * pagesize;
        sd_sizer.append(sizer_from+" - "+sizer_to);

        controller.append(sd_sizer);
    }

    var div_ctrl = ajaxtable.getdiv_ctrl.call(this);
    div_ctrl.append(controller);

};

ajaxtable.ctrl.redrawFilterCriteria = function (ev)
{
    var widget = ev['data']['widget'];
    var target = $(widget).find(".jswlib_ajaxtable_ctrl_filter_criteria");

    target.unbind();
    console.log("Redraw filter %o has event data %o", this, ev);

};

ajaxtable.ctrl.requestSort = function (ev)
{

    var context = ev['data']['widget'];
    var sortinfo = ev['data']['sort'];

    var req = lwp.fromWlive.call(context, 'tabledata')['req'];

    var filtered = req['filter']['enabled'];

    var req_dict = {
        sort: {
            field: req['sort']['field'],
            dir: req['sort']['dir']
        }
    };

    req_dict['filter'] = filtered ? req['filtered'] : false;

    console.log("Requesting sort ",req_dict);
    ajaxtable.ctrl.requestTable.call(context, req_dict);
};

ajaxtable.ctrl.requestFilter = function (ev)
{
    var context = ev['data']['widget'];
    var req = lwp.fromWlive.call(context, 'tabledata')['req'];

    var criteria = ajaxtable.getdiv_ctrl.call(context).find(".jswlib_ajaxtable_ctrl_filter_criteria").val();
    var field = ajaxtable.getdiv_ctrl.call(context).find(".jswlib_ajaxtable_ctrl_filter_fieldsel").val();
    var req_dict = {
        filter: {
            criteria: criteria,
            field: field
        }
    };

    var sorted = req['sort']['enabled'];
    req_dict['sort'] = sorted ? req['sorted'] : false;

    console.log("Requesting filter ",req_dict);
    ajaxtable.requestTable.call(context, req_dict);
};

ajaxtable.ctrl.requestPage = function (ev)
{
//    console.log("Requesting new page with ",ev, " from ",this);

    // changes page on existing filter/sort setup

    var pagenum = ev['data']['pagenum'];
    var context = ev['data']['widget'];

    console.log("Requesting page ",pagenum);

    var req = lwp.fromWlive.call(context, 'tabledata')['req'];
    var ctx = lwp.fromWlive.call(context, 'tabledata')['ctx'];

    var filtered = req['filter']['enabled'];
    var sorted = req['sort']['sorted'];

    var req_dict = {
        page: pagenum,
        pagesize: ctx['paged']
    };

    req_dict['filter'] = filtered ? req['filter'] : false;
    req_dict['sort'] = sorted ? req['sorted'] : false;

    ajaxtable.requestTable.call(context, req_dict);

};

ajaxtable.ctrl.requestClean = function (ev)
{
    // calls the table as unfiltered
    var context = ev['data']['widget'];

    var req = lwp.fromWlive.call(context, 'tabledata')['req'];
    var ctx = lwp.fromWlive.call(context, 'tabledata')['ctx'];

    var sorted = req['sort']['sorted'];

    var req_dict = {};

    req_dict['filter'] = false;
    req_dict['sort'] = sorted ? req['sorted'] : false;

    ajaxtable.requestTable.call(context, req_dict);
};

ajaxtable.requestTable = function (req)
{
    var url = lwp.fromWlive.call(this, 'tabledata')['url'];

    req.requester = "table";

    $.ajax({
        url: url,
        data: req,
        context: this
    }).done (
        function (data) {
            // reload widget with new data
            var tablejson = typeof data == "string" ? JSON.parse(data) : data;
            console.log("rebuilding widget with data ",tablejson);
            lwp.dropWlive.call(this);
            var widget = $(this);
            widget.lwp('winit', 'ajaxtable');
            widget.lwp('populate', tablejson);
            widget.lwp('render');
            ajaxtable.reflate.call(widget);
            console.log("finished reloading table with updated data");
        }
    ).fail(
        function () {
            // show error message in place of the table, unique function for all changes of this kind
            ajaxtable.requestFail.call(this);
        }
    );
};

ajaxtable.requestFail = function ()
{
    var div_vis = ajaxtable.getdiv_table.call(this);
    div_vis.empty();
    div_vis.lwp('winit', 'customdiv');
    div_vis.css('height', "2em");
    lwp.toDisplay.call(div_vis).css('height', '100%');
    lwp.warn.call(div_vis);
    ajaxtable.reflate.call(this);
};



ajaxtable.reflate = function ()
{


    var ctrl_height = ajaxtable.getdiv_ctrl.call(this).height();
    var table_height = ajaxtable.getdiv_table.call(this).height();

    console.log("reinflation for ",this, " with "+ctrl_height+"+"+table_height);

    var maxheight = ctrl_height + table_height;
//    lwp.inflate_v.call(this, maxheight);

    //lwp.toDisplay.call(this).height(maxheight);
    //$(this).height(maxheight);
    
    //console.log("Widget reinflated to ",maxheight);
    //console.log("Actual final height for widget ",this,' is ',$(this).height());
};

ajaxtable.wrender = function ()
{
//    var data = lwp.fromWlive.call(this, 'tabledata');

//	var hidefilters = $(this).data('wparams')['hidefilters'] == true;
    ajaxtable.getdiv_ctrl.call(this).empty();
    ajaxtable.getdiv_table.call(this).empty();
    ajaxtable.ctrl.rendercontroller.call(this);
    ajaxtable.vis.rendertable.call(this);

    console.log("Reinflating widget with ",this);

//    ajaxtable.reflate.call(this);

    var self = this;
    var reflater = function () {
        ajaxtable.reflate.call(self);
    };

    if (NS.delay)
    {
        setTimeout(reflater, NS.delay);
    }


};

ajaxtable.vis = {};

ajaxtable.vis.isdynamicresource = function (val)
{
    // verifies if val is an object that requires special actions such as ajax
    // or if we can simply render it by the way of textcell or other functions

    if (typeof val != 'object')
    {
        return false;
    }
    else
    {
        try
        {
            // note: right now we only check for a widgetloader capability, but more may be added in the future
            return val['type'] === 'wloader';
        }
        catch (err)
        {
            return false;
        }
    }
};

ajaxtable.vis.switchfold = function (ev)
{
    console.log("Switching fold ",$(this));


    var master = lwp.getWidgetSelf.call(this);
    var subwidget = $(this).closest(".jswlib_widget");
    var targetrow = $(this).closest("tbody").find(".jswlib_subrefrow");
    var targetcell = targetrow.find(".jswlib_subreftd");
    var targetdiv = $(targetcell.find("div").get(0));

    // ev['data']['active'] == this;
    var folds =  ev['data']['context'].find(".jswlib_foldbutton");
    var refid = ev['data']['refid'];

    var subdiv = $("<div/>");
    subdiv.addClass("jswlib_loadedwidget");
    subdiv.prop('id', targetdiv.prop('id')+"_unfolded");
    subdiv.css('height', '100%');

    var class_fold_open = "jswlib_switchfold_open";
    var class_fold_closed = "jswlib_switchfold_closed";

//    console.log("Class data on switcher ", $(this).className);
    if (!$(this).hasClass(class_fold_open))
    {
        console.log("Switching open");
        folds.removeClass(class_fold_open);
        folds.addClass(class_fold_closed);
        $(this).addClass(class_fold_open);
        $(this).removeClass(class_fold_closed);

        var refcontent = targetdiv.data('wdata')[refid];
        targetdiv.empty();
        console.log("Refcontent data in widget ");
        console.log(refcontent);
        if (!ajaxtable.vis.isdynamicresource(refcontent))
        {
             console.log("rendering non-dynamic data");
            // not dynamic, render
            subdiv.append(lwp.rendertextcell(refcontent));
            targetdiv.append(subdiv);
            ajaxtable.reflate.call(master);
        }
        else
        {
            subdiv.data('wtype', refcontent['wtype']);
            var divheight = '3em';
            if (refcontent.hasOwnProperty('wheight'))
            {
                divheight = refcontent['wheight'];
            }
            //            console.log("setting height: "+divheight);
            subdiv.css('height', divheight);
            console.log("Launching ajax on refcontent "+refcontent.url);
            console.log(targetdiv.get(0));
            targetdiv.append(subdiv);
            var refparams = refcontent.hasOwnProperty('wparams') ? refcontent['wparams'] : null;
            // dynamic, load and apply callback
            $.ajax({
                url: refcontent.url,
                context: {
                    tdiv: $(targetdiv.get(0)),
                    params: refparams
                },
                contentType: 'application/json',
                cache: false
            }).done(
                function (data, textStatus, jqXHR) {
                                        console.log("DONE AJAX: ");
                                        console.log(this);
                                        console.log(typeof data);
                    var dataset = JSON.parse(data);
                                        console.log("Subwidget data from ajax: ", dataset);
                                        console.log("Context for subw: ",this);
                    var wdiv = this.tdiv;
                    var wparams = this.params;

                    var widget = $(wdiv.find('.jswlib_loadedwidget').get(0));

                    console.log("Working on widgetn entity ",widget);
                    console.log("Widget data info ",widget.data());

                    lwp.run.call(widget, 'winit');
                    widget.width(wdiv.innerWidth());
                    lwp.toDisplay.call(widget).css('height', widget.css('height'));
                    lwp.toDisplay.call(widget).width(widget.innerWidth());

//                      console.log("Widget Init done");
//                      console.log($(widget).data());
                    var populated = lwp.populate.call(widget, dataset);
                    if (populated)
                    {
                        if (wparams != null)
                        {
//                            console.log("passing to subw params ",wparams);
                            widget.data('wparams', wparams);
                        }
                        //  console.log("Data for subw: ",widget.data());
                        lwp.render.call(widget);
//                        console.log("rendered subwidget ",widget," sized with height ",widget.height()," actual height ", widget.get(0).scrollHeight);
                        widget.height(widget.get(0).scrollHeight);
                    }
                    else
                    {
                        widget.data('wtype', 'customdiv');
                        widget.lwp('winit', 'customdiv');
                        widget.css('height', "2em");
                        var cdiv_vis = lwp.toDisplay.call(widget);
                        cdiv_vis.css('height', '100%');
                        lwp.warn.call(widget);
                        var errorspan = $("<span/>");
                        errorspan.append("No data!");
                        errorspan.addClass("jswlib_loaderrormess");
                        cdiv_vis.find(".jswlib_warning").append(errorspan);

                    }
//                    console.log("Widget inited");
                }
            ).fail(
                function (jqXHR, textStatus, errorThrown) {
//                    console.log("FAILED AJAX: ");
                    var wdiv = this.tdiv;
                    var widget = $(wdiv.find('.jswlib_loadedwidget').get(0));
                    widget.lwp('winit', 'customdiv');
                    widget.css('height', "2em");
                    var cdiv_vis = lwp.toDisplay.call(widget);
                    cdiv_vis.css('height', '100%');
                    lwp.warn.call(widget);
                    var errorspan = $("<span/>");
                    errorspan.append("No data!");
                    errorspan.addClass("jswlib_loaderrormess");
                    cdiv_vis.find(".jswlib_warning").append(errorspan);
                }
            ).always(
                    function () {
                        var wdiv = this.tdiv;
                        var widget = lwp.getWidgetSelf.call(wdiv);
                        console.log("autoresizing table widget ",widget);
                        ajaxtable.reflate.call(widget);

                    }
                );

        }
        targetrow.show();
    }
    else
    {
        console.log("Switching closed");
        folds.removeClass(class_fold_open);
        folds.addClass(class_fold_closed);
        try
        {
            targetdiv.empty();
        }
        catch (err)
        {
            // do nothing intentionally
        }

//        console.log("Target div emptied");
//        console.log(targetdiv);
        targetrow.hide();
        ajaxtable.reflate.call(master);
    }
};

ajaxtable.vis.rendertable = function ()
{
    var data = lwp.fromWlive.call(this, 'tabledata')['data'];
    var ctx = lwp.fromWlive.call(this, 'tabledata')['ctx'];
    var req = lwp.fromWlive.call(this, 'tabledata')['req'];

    var vistable = $("<table/>");
    vistable.addClass('jswlib_ajaxtable_vis');

    var foldable = data['ref'].length > 0;
    var classed = data['classes'].length > 0;

    // draw headers
    var headers = $("<thead/>");
    var trheaders = $("<tr/>");
    if (foldable)
    {
        // foldable addition;
        trheaders.append($("<th/>"));
    }
    for (var h = 0; h < data['headers'].length; h++)
    {
        var cheader = data['headers'][h];
        trheaders.append('<th>'+cheader+'</th>');
    }
    headers.append(trheaders);
    vistable.append(headers);

    // draw rows
    var numrows = typeof ctx['paged'] == 'number' ? Math.min(data['values'].length, ctx['paged']) : data['values'].length;
    for (var r = 0; r < numrows; r++)
    {
        var rendersection = $("<tbody/>");
        rendersection.addClass('jswlib_foldbody');
        var valuesrow = $('<tr class=jswlib_valuesrow/>');
        if (classed)
        {
            var rowclass = data['classes'][r];
            if (rowclass != null && rowclass != "")
            {
                valuesrow.addClass(rowclass);
            }
        }

        if (foldable)
        {
            var subrow = $('<tr class=jswlib_subrefrow><td class=jswlib_subreftd/></tr>');
            var rowrefs = data['ref'][r];
            subrow.find('td').append('<div style="width: 100%"/>');
            subrow.find('td').find('div').data('wdata', rowrefs);
            subrow.find('td').find('div').prop('id', lwp.getDisplayId.call($(this))+"_sub_"+r);
            subrow.find('td').find('div').addClass('jswlib_foldableslot');

            var switchercell = $('<td/>');
            switchercell.addClass("jswlib_foldcell");
            for (var i = 0; i < rowrefs.length; i++)
            {
                var refobj = rowrefs[i];
                if (typeof refobj != "undefined" && refobj != null && refobj != "")
                {
                    var switcher = $("<span/>");
                    switcher.width(19);
                    switcher.addClass('jswlib_foldbutton');
                    var iconid = refobj.hasOwnProperty("foldicon") ? refobj['foldicon'] : "icon-chevron-sign-down";
                    var ftitle = refobj.hasOwnProperty("foldtitle") ? refobj['foldtitle'] : "";

                    switcher.append('<i class="'+iconid+'" title="'+ftitle+'"></i>');
                    switcher.bind('click', {context: switchercell, active: switcher, refid: i}, ajaxtable.vis.switchfold);
                    switcher.addClass("jswlib_switchfold_closed");
                }
                switchercell.append(switcher);
            }
            switchercell.width(20*rowrefs.length);
            valuesrow.append(switchercell);
        }
        rendersection.append(valuesrow);

        var colnum = data['values'][r].length;
        for (var c = 0; c < colnum; c++)
        {
            var valuetd = $("<td/>");
            valuetd.append(lwp.rendertextcell(data['values'][r][c]));
            valuesrow.append(valuetd);
        }

        rendersection.append(valuesrow);
        if (foldable)
        {
            subrow.find("td").prop("colspan", colnum+1);
            rendersection.append(subrow);
            // all hidden by default
            subrow.hide();
        }
        vistable.append(rendersection);

    }

    var div_table = ajaxtable.getdiv_table.call(this);
    div_table.empty();
    div_table.append(vistable);


};










jswlib['ajaxtable'] = ajaxtable;
//jswlib['tablevis'] = tablevis;
buildWidgetsTree();

console.log("All widgets now ",jswlib);
