/**
 * Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
 * Created: 8/27/13, 11:26 AM
 * Author: Antonio Vaccarino
 */


// development version of the ajax table widget


/* FORMAT GEN 3
{
	"url": string, // url called for new requests
	"data":
	{
		"headers": [{
			id: str, internal title
			title: str, human title,
			sortable: bool, if can be sorted
		}],
		"values" [], // rows as [], each of headers.length length (mandatory),
		"ref": [] // one per row, if object can contain properties foldicon with id of fontawesome symbol and foldtitle with title to be displayed on tooltip,
		"classes": [str], must be either empty or values.length length
	},
	"ctx":
	{
		"filters": [[str, {}]], // list of available filters for this table, each tuple has [0] as the ftype and [1] as the creation parameters
		"results": int, // total results for the search/filter combination, cannot be smaller than values.length
		"offset": int, // position of the first element in values respective to results
		"paged": int, // amount of results per page, cannot be smaller than values.length, can be longer ONLY if it's the last page, throws exception; 0 to remove the check, automatically set as 1st page (verified with offset==0 and results == values.length)
	},
	// description of the original request and parameters
	"req":
	{
		"filters":
		[
			{
				ftype: str // name of the filter type to be used for this filter instance
				fparams: object // {} dict with filter-specific parameters
				fand: boolean // if filter is concatenated inclusively (OR) or exclusively (AND), cannot be null
			}
		],
		"sort": // can be empty if no filter
		{
			"field": str // identifier of the field to be sorted, can be null
			"dir" boolean // small to big or big to smalle
		}
	}
}
*/

ajaxtable = {};

ajaxtable.filters = {};

ajaxtable.CONST = {};
ajaxtable.CONST.class_div_ctrl = "jswlib_atable_ctrl";
ajaxtable.CONST.class_div_table = "jswlib_atable_table";
ajaxtable.CONST.class_div_filter = "jswlib_atable_filter";
ajaxtable.CONST.class_div_master = "jswlib_atable_master";


ajaxtable.assemble = function ()
{
	console.log("Starting assemble on dynamic table");

	// does NOT call winit on the children since we do not know yet what kind they are

	var displaydiv = lwp.toDisplay.call(this);
	$(this).addClass(ajaxtable.CONST.class_div_master);

	// console.log("Finished assemble on dynamic table");

	var tableW = new ajaxtable.Widget(displaydiv, $(this).attr('id'));
	lwp.toWlive.call(this, tableW, 'tableW');
};

ajaxtable.emerge = function ()
{
	var i;
	var raw = $(this).data('wdata');

	console.log("Data summary for emerge ", raw);

	// checking structure
	var wellformed = true;
	var issues = [];
	if (!raw['url'])
	{
		wellformed = false;
		issues.push("Missing url for further requests");
	}
	if (!raw['data'])
	{
		wellformed = false;
		issues.push("Missing table data");
	}
	else
	{
		var rdata = raw['data'];
		if (!$.isArray(rdata['headers']) || !$.isArray(rdata['values']) || !$.isArray(rdata['ref']))
		{
			wellformed = false;
			issues.push("Incomplete table data structure");
		}
		else
		{
			var ncols = rdata['headers'].length;
			var rows = rdata['values'];
			var refs = rdata['ref'];
			for (i = 0; i < rows.length; i++)
			{
				if (!$.isArray(rows[i]) || rows[i].length != ncols)
				{
					wellformed = false;
					issues.push("Values/headers field number mismatch");
					break;
				}
			}

			if (!$.isArray(refs) || refs.length != rows.length)
			{
				wellformed = false;
				issues.push("Values/references number mismatch");
			}
			if (rdata.hasOwnProperty('classes'))
			{
				var classes = rdata['classes'];
				if (!$.isArray(classes) || ((classes.length != 0) && (classes.length != rows.length)))
				{
					wellformed = false;
					issues.push("Values/classes number mismatch");
				}
				else
				{
					for (i = 0; i < classes.length; i++)
					{
						if (typeof(classes[i])!='string')
						{
							wellformed = false;
							issues.push("Row classes contain non valid values (should be string)");
							break;
						}
					}
				}
			}
		}
	}
	if (!raw['ctx'])
	{
		wellformed = false;
		issues.push("Missing context for table data");
	}
	else
	{
		// only check if other verifications were ok, if table structure is broken in itself we stop earlier despite losing a bit of debug values
		if (wellformed)
		{
			var rctx = raw['ctx'];
			if (!(typeof(rctx["results"]) == 'number' && rctx["results"] >= 0 &&
				typeof(rctx["offset"]) == 'number' && rctx["offset"] >= 0 &&
				typeof(rctx["paged"]) == 'number' && rctx["paged"] > 0 &&
				rctx["results"] >= rctx["offset"]+rows.length && rctx["paged"] >= rows.length))
			{
				wellformed = false;
				issues.push("Invalid pagination data");
				console.log("BAD pagination data: ",rctx," with rows ",rows.length);
			}
		}

		// checking for available filters
		if (!rctx.hasOwnProperty('filters'))
		{
			rctx['filters'] = [];
		}
		else
		{
			var avfilters = rctx['filters'];
			if (!$.isArray(avfilters))
			{
				wellformed = false;
				issues.push("Available filters list is not an array.");
			}
			else
			{
				for (i = 0; i < avfilters.length; i++)
				{
					var cfilter = avfilters[i];
					if (!$.isArray(cfilter) || typeof(cfilter[0])!='string' || typeof(cfilter[1])!='object')
					{
						wellformed = false;
						issues.push("Invalid filter configuration in available filters.");
						break;
					}
					if (!ajaxtable.Filters.hasOwnProperty(cfilter[0]))
					{
						wellformed = false;
						issues.push("Requested nonexistent filter "+cfilter[0]);
					}
				}
			}
		}
	}

	var reqsort = raw['req']['sort'];
	if (!reqsort)
	{
		raw['req']['sort'] = { "field": null, "dir": null };
	}
	else
	{
		if (typeof reqsort['field'] != "string" || (reqsort['dir'] != null && typeof reqsort['dir']) != "boolean")
		{
			wellformed = false;
			issues.push("Invalid sorting syntax");
		}
	}

	var reqfilters = raw['req']['filters'];
	if (!reqfilters)
	{
		raw['req']['filters'] = [];
	}
	else
	{
		var rf = reqfilters;
		if (!$.isArray(rf))
		{
			wellformed = false;
			issues.push ("Badly formed requested filters syntax");
		}
		else
		{
			for (i = 0; i < reqfilters.length; i++)
			{
				var cf = reqfilters[i];
				if (typeof(cf['type']!='string' || typeof cf['fparams'] != "object" || typeof cf['fseq'] != 'boolean'))
				{
					wellformed = false;
					issues.push ("Badly formed requested filters syntax");
				}
				else if (!ajaxtable.Filters.hasOwnProperty(cf['ftype']))
				{
					wellformed = false;
					issues.push ("Non existent filter in request array");
				}
			}
		}
	}

	var tableW = lwp.fromWlive.call(this, 'tableW');
	tableW.inject(raw);

	// checking REQ section, with filters and sorts
	if (!wellformed)
	{
		throw "Invalid json data structure: "+issues.join(", ");
	}
};

ajaxtable.wrender = function ()
{

};

ajaxtable.Widget = function Widget (jqdiv, wid)
{
	this.id = wid;
	this.div = jqdiv;
	console.log("inited widget object "+this.id+" with div ",this.div);
};

ajaxtable.Widget.prototype.inject = function (raw)
{
	// to be launched from widget emerge call
	console.log("injecting validated data into widget: ",raw);
	this.ajaxurl = raw['url'];
};

//
// SECTION: GRAPHICAL ELEMENTS
//

ajaxtable.AjaxTable = function AjaxTable () {
	// the actual table subwidget
};
ajaxtable.PagerWidget = function PagerWidget () {
	// pager subwidget
};
ajaxtable.FilterSummary = function FilterSummary () {
	// the widget selector and summarizer
};


// generates an instance of a filter
ajaxtable.FilterFactory = function FilterFactory (filtertype, filterdefinition) {

	// function to create a filter from a typestring and related data, does not include sequencing with AND/OR
	return new ajaxtable.Filters[filtertype] (filterdefinition);
};

ajaxtable.Filters = {};

// general filter, for subclassing
ajaxtable.Filters.filter = function (definition) {
	// saving raw definition
	this.rawdef = definition;
	this.define (definition);
};

ajaxtable.Filters.filter.prototype.render = function () {};
ajaxtable.Filters.filter.prototype.define = function () {
	console.log ("Raw filter definition hook, implemented by descendants");
};

// INDIVIDUAL FILTER WIDGETS

ajaxtable.Filters.fulltext = function (definition) {
	ajaxtable.Filters.filter.call (this, definition);
};
ajaxtable.Filters.fulltext.prototype = Object.create(ajaxtable.Filters.filter.prototype);
ajaxtable.Filters.fulltext.prototype.define = function (definition)
{
//	var definition = this.rawdef;
	console.log("Running definition for  filter fulltext");
	// user input is checked in case we want to allow only specific expressions
	var typingcheck;
	if (definition['charset'])
	{
		var charset = definition['charset'];
		typingcheck = function (lastchar)
		{
			return charset.indexOf(lastchar)!=-1;
		};
	}
	else
	{
		typingcheck = function ()
		{
			return true;
		};
	}
	this.typingCheck = typingcheck;

	var suggestions = [];
	if ($.isArray(definition['suggestions'])) {
		suggestions = definition['suggestions'];
	}
	this.suggestions = suggestions;
};


















jswlib['ajaxtable'] = ajaxtable;
//jswlib['tablevis'] = tablevis;
buildWidgetsTree();

console.log("All widgets now ",jswlib);