/**
 * Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
 * Created: 5/22/13, 10:42 AM
 * Author: Antonio Vaccarino
 */


//container for a table widget that spawns a microtable internally

dynamictable = {};

// this widget assembles its subdiv structure in the assemble method
// actual widgets are spawned with the emerge command because the controller needs to know which data it is to work with and the table widget needs to know its own render

dynamictable.assemble = function ()
{

//    console.log("Starting assemble on dynamic table");

    // does NOT call winit on the children since we do not know yet what kind they are

    var displaydiv = lwp.toDisplay.call(this);
    $(this).addClass("jswlib_dyntable_master");

    var id_ctrl = lwp.run.call(this, 'getid_ctrl');
    var id_table = lwp.run.call(this, 'getid_table');

    var div_ctrl = $("<div/>");
    div_ctrl.attr('id', id_ctrl);
    div_ctrl.addClass("jswlib_dyntable_ctrl");

    var div_table = $("<div/>");
    div_table.attr('id', id_table);
    div_table.addClass("jswlib_dyntable_table");

    displaydiv.append(div_ctrl);
    displaydiv.append(div_table);

//    console.log("Finished assemble on dynamic table");

};

dynamictable.getid_table = function ()
{
    return $(this).attr('id')+"_sub_table";
};

dynamictable.getdiv_table = function ()
{
    return $(this).find("#"+lwp.run.call(this, 'getid_table'));
};

dynamictable.getid_ctrl = function ()
{
    return $(this).attr('id')+"_sub_ctrl";
};

dynamictable.getdiv_ctrl= function ()
{
    return $(this).find("#"+lwp.run.call(this, 'getid_ctrl'));
};

dynamictable.validate = function (message)
{
    // validates the container message and the data for the subwidget

    // the subvalidation is executed BY the underlying subwidget
    // after validating the type (but not with spawning the data, which implies a double check though)

    /*

    expected message structure
    {
        tabletype: string,

        tabledata: {message for table widget, contents may vary}
    }
    */

    var tabletype;
    try
    {
        // tabletype may be missing or pointing at a non existing widget
        tabletype = message['tabletype'];
//        console.log("Validating tabledata according to "+tabletype);
//        console.log(jswlib[tabletype]);
//        console.log(message['tabledata']);
        if (jswlib[tabletype].hasOwnProperty('validate'))
        {
            var subvalidation = jswlib[tabletype].validate(message['tabledata']);
            if (!subvalidation)
            {
//                console.log("Dynamic table validate failed");
                return false;
            }
        }
    }
    catch (err)
    {
//        console.log("Dynamic table validate failed with exception");
        return false;
    }

//    console.log("Dynamic table validate succeeded");
    return true;
};


dynamictable.emerge = function ()
{

//    console.log("Started dynamic table emerge process");
//    console.log($(this).data());

    var div_ctrl = lwp.run.call(this, 'getdiv_ctrl');
    var div_table = lwp.run.call(this, 'getdiv_table');

    var dataset = $(this).data('wdata');

    var tabledata = dataset['tabledata'];

    if (dataset.hasOwnProperty('presort'))
    {
        var presort = dataset['presort'];
        lwp.toWlive.call(this, presort, 'sortedby');
    }


    var isdynamic = dataset.hasOwnProperty('dynamic') ? dataset.dynamic === true : false;

    lwp.toWlive.call(this, isdynamic, "dynamic");
    lwp.toWlive.call(this, dataset.context['offset'], 'offset');

//    console.log("Preparing div_table for "+dataset['tabletype']);

    div_table.lwp('winit', dataset['tabletype']);

//    console.log("Done winit "+dataset['tabletype']);
//    console.log(div_table);
    // populate is validate + copy + emerge, but we already handled the validate
    //lwp.populate.call(div_table, $(this).data('wdata')['tabledata']);
    //div_table.data('wdata', tabledata);
//    console.log("passed data to div_table");
    lwp.run.call(div_table, 'populate', [tabledata]);

//    console.log("Preparing div_ctrl");

    var controllerdata = {
        dynamic: isdynamic,
        columns: tabledata.headers,
        // lines actually used in this dataset
        rows: tabledata.values.length,
        // total results, may be different from rows if table is on another system and seen through a query
        results: dataset.context['results'],
        offset: dataset.context['offset']
    };

    div_ctrl.lwp('winit', 'tablecontroller');
    lwp.populate.call(div_ctrl, controllerdata);

};

dynamictable.askredraw = function ()
{
    /*
        launches an ajax wave to receive updated data and render it on return
        takes parameters from wlive
     */

    // TODO: placeholder, implement

    // from
    var div_ctrl = lwp.run.call(this, 'getdiv_ctrl');


};

dynamictable.filter = function ()
{

//    console.log("dynamic table filtering");

    var div_ctrl = lwp.run.call(this, 'getdiv_ctrl');
    var div_table = lwp.run.call(this, 'getdiv_table');


    // we do the on the fly filtering only on non-dynamic tables.
    // In dynamic ones the filter is used by the original query and not repeated
    // filtered contains the values, filterctx the stats for the controller widget
    if (!lwp.fromWlive.call(this, 'dynamic'))
    {
        var filtercol = lwp.fromWlive.call(this, 'filtercolumn');
        var filterval = lwp.fromWlive.call(this, 'filtervalue');

        // erasing current filter is needed
        lwp.dropWlive.call(this, 'filtered');
        lwp.dropWlive.call(this, 'filterctx');

        var filtered = {};

        if (filtercol && filterval)
        {
            var source = $(div_table).data('wdatafinal');
//            console.log("Filter source");
//            console.log(source);
            var ctxsource = $(div_ctrl).data('wdata');
            var filterctx= jQuery.extend(true, {}, ctxsource);
            filtered.headers = source.headers;
            filtered.foldable = source.foldable;
            filtered.data = [];
            var critcol = filtered.headers.indexOf(filtercol);
//            console.log("Filtering on col n."+critcol+" named "+filtercol);
            var stringref = $.trim(filterval.toLowerCase());

            var strictsearch = stringref[0] == '"';
            if (strictsearch)
            {
                var closer = stringref.lastIndexOf('"');
                if (closer != -1)
                {
                    stringref = stringref.substring(1,closer);
                }
                else
                {
                    stringref = stringref.substring(1);
                }
            }
            else
            {
                stringref = stringref.split(" ");
            }

            for (var i = 0; i < source.data.length; i++)
            {
                // note: for now we ALWAYS treat both as strings
                try
                {
//                    console.log(source.data[i].val[critcol]+" vs "+filterval);
//                    var stringcmp = lwp.rendertextcell(source.data[i].val[critcol]).toLowerCase();
                    var rawdata = source.data[i].val[critcol];
                    var rendereddata = lwp.rendertextcell(rawdata);
                    var stringcmp = typeof rawdata == "string" ? (lwp.rendertextcell(rawdata)).toLowerCase() : JSON.stringify(rawdata).toLowerCase();
//                    console.log("Filter checking ",stringcmp," vs ",stringref," as ",stringcmp.indexOf(stringref)!=-1);
                    if (strictsearch)
                    {
                        if (stringcmp == stringref)
                        {
                            filtered.data.push(source.data[i]);
                        }
                    }
                    else
                    {
                        for (var r = 0; r < stringref.length; r++)
                        {
                            if (stringcmp.indexOf(stringref[r])!=-1)
                            {
                                filtered.data.push(source.data[i]);
                                break;
                            }
                        }
                    }

                }
                catch (err)
                {
                    console.log("FILTERING ISSUE ", err.message);
                    // if data formats are incompatible between source and filter, we skip
                }
            }
            //console.log("Filtered rows");
            //console.log(filtered);
            filterctx['results'] = filtered.data.length;
            filterctx['rows'] = filtered.data.length;
//            console.log("RESULTS: ",filterctx['rows']);
//            console.log(filterctx);
            lwp.toWlive.call(this, filtered, 'filtered');
            lwp.toWlive.call(this, filterctx, 'filterctx');
        }
        // since we just modified the filter criteria we are automatically at page 0
        // this happens also if we just emptied the filter
        lwp.toWlive.call(this, 0, 'offset');

    }


};

dynamictable.wrender = function ()
{
    // TEMPORARY FIX TO re-enable correct paging behaviour, more research needed
    // TODO: check the reason for this block, 12/05/2013
//    console.log("pre-render filtering");
//    lwp.run.call(this, 'filter');

    var div_ctrl = lwp.run.call(this, 'getdiv_ctrl');
    var div_table = lwp.run.call(this, 'getdiv_table');

    lwp.render.call(div_ctrl);
    //var height_ctrl = div_ctrl.find('table').get(0).scrollHeight;
    var height_ctrl = $(div_ctrl.find('table')).height();

//    console.log("Stretching div_ctrl to pixels "+height_ctrl+" or "+div_ctrl.height());
    lwp.inflate_v.call(div_ctrl, height_ctrl);

    //div_table.css('top', height_ctrl);
    var height_table = $(this).height()-height_ctrl;
    lwp.render.call(div_table);
//    console.log("rendered actual table part");
    lwp.inflate_v.call(div_table, height_table);

//    console.log("Heights: ",div_table," has ",height_table," from main height ",$(this).height()," and control height ",height_ctrl," (based on el ",$(div_ctrl.find('table')),")");

    if ($(this).data('wparams')['autosize'] === true)
    {
        dynamictable.autosize.call(this);
    }

};

dynamictable.autosize = function ()
{
    var div_ctrl = lwp.run.call(this, 'getdiv_ctrl');
    var div_table = lwp.run.call(this, 'getdiv_table');
    $(div_table).css('height','');
    lwp.toDisplay.call(div_table).css('height', '');
    lwp.toDisplay.call(this).css('height', '');
    $(this).css('height', '');
};



dynamictable.isfiltered = function()
{
    if (lwp.fromWlive.call(this, 'dynamic')===true)
    {
        return false;
    }
    else
    {
//        console.log(lwp.fromWlive.call(this));
        // the wlive objects are removed when the filter is reset, so if they exist they can be expected
        // to be usable without further checkings
        var response = ($.isPlainObject(lwp.fromWlive.call(this, 'filtered')) && $.isPlainObject(lwp.fromWlive.call(this, 'filterctx')));
//        console.log(response);
        return response;
    }
};


// tablevis replaces both datatable and foldtable inside a dynamictable system.
// contains  both working modes according to having or not a REF item in the data


tablevis = {};

tablevis.validate = function (dataset)
{
    //todo: placeholder, implement
    var validstructure = dataset.hasOwnProperty('values') && $.isArray(dataset.values);
    // the dataset can be devoid of optional field ref (reference for fold) but if it has, it must be in the same quantity as the actual values (-1 compared to values as 1st row of values is header names)

    var optional = (!dataset.hasOwnProperty('ref')) || ($.isArray(dataset.ref) && dataset.ref.length == dataset.values.length);

//    console.log("Performed tablevis validation: structure "+validstructure+" / optional "+optional);

    return validstructure && optional;

};

tablevis.emerge = function ()
{

    //todo: placeholder, implement
    // using this to make the structure cleaner and non-heuristic

    var dataset = $(this).data('wdata');
    var foldable = dataset.hasOwnProperty('ref');
    var classed = dataset.hasOwnProperty('classes');


    var headers = [];
    if (dataset.hasOwnProperty('headers'))
    {
        headers = dataset.headers;
    }

    var series = [];
    for (var i = 0; i < dataset.values.length; i++)
    {
        var row = {};
        row.val = dataset.values[i];
        if (foldable)
        {
            row.ref = dataset.ref[i];
        }
        if (classed)
        {
            try
            {
                row.class = dataset.classes[i];
            }
            catch (err)
            {
                row.class = "";
            }
        }
        else
        {
            row.class = "";
        }
        series.push(row);
    }

    var emerged = {
        headers:    headers,
        data:       series,
        foldable:   foldable
    };

    $(this).data('wdatafinal', emerged);

};

tablevis.getRowsNum = function ()
{
    // retrieves the number of rows to be rendered from the tablecontroller widget
    // OR renders the full length of the data

    var rowsnum = 0;
    // 0 means show ALL rows
    var parentwidget = $(this).parents('.jswlib_widget');
    if (parentwidget.data('wtype') == 'dynamictable')
    {
        var liverows = lwp.fromWlive.call(parentwidget, 'rowsnum');
//        console.log("live rows: "+liverows);
        if (typeof liverows == 'undefined')
        {
            try
            {
                var target = dynamictable.getdiv_ctrl.call(parentwidget);
                if (target)
                {
//                    console.log("Rowsnum Selector ");
//                    console.log(target);
                    rowsnum = tablecontroller.getMaxRows.call(target);
                }
            }
            catch (err)
            {
                // do nothing, we keep to the default value 0
            }
        }
        else
        {
            rowsnum = liverows;
        }
    }

//    console.log("Rows num for this control is "+rowsnum);
    return rowsnum;
};

tablevis.isdynamicresource = function (val)
{

    // verifies if val is an object that requires special actions such as ajax
    // or if we can simply render it by the way of textcell or other functions

    if (typeof val != 'object')
    {
        return false;
    }
    else
    {
        try
        {
            // note: right now we only check for a widgetloader capability, but more may be added in the future
            return val['type'] === 'wloader';
        }
        catch (err)
        {
            return false;
        }
    }

};

tablevis.switchfold = function ()
{
//    console.log("Fold switching on");
//    console.log(this);
    var subwidget = $(this).closest(".jswlib_widget");
    var targetrow = $(this).closest("tbody").find(".jswlib_subrefrow");
    var targetcell = targetrow.find(".jswlib_subreftd");
    var targetdiv = $(targetcell.find("div").get(0));
//    console.log("FOLDING ON DIV WITH ID "+targetdiv.prop('id'));

    var subdiv = $("<div/>");
    subdiv.addClass("jswlib_loadedwidget");
    subdiv.prop('id', targetdiv.prop('id')+"_unfolded");
    subdiv.css('height', '100%');
    //subdiv.css('width', '100%');
    //subdiv.width(subwidget.outerWidth());

//    console.log(subwidget);
    if (!targetcell.is(':visible'))
    {
        var refcontent = targetdiv.data('wdata');
//        console.log("Refcontent data in widget ");
//        console.log(refcontent);
        if (!lwp.run.call(subwidget, 'isdynamicresource', [refcontent]))
        {
//            console.log("rendering non-dynamic data");
            // not dynamic, render
            subdiv.append(lwp.rendertextcell(refcontent));
            targetdiv.append(subdiv);

        }
        else
        {
//            console.log("Loading and rendering dynamic data as ",refcontent['wtype']);
//            console.log("from pre-dataset ",refcontent);
            subdiv.data('wtype', refcontent['wtype']);
            var divheight = '3em';
            if (refcontent.hasOwnProperty('wheight'))
            {
                divheight = refcontent['wheight'];
            }
//            console.log("setting height: "+divheight);
            subdiv.css('height', divheight);
//            console.log("Launching ajax on refcontent "+refcontent.url);
//            console.log(targetdiv.get(0));

            targetdiv.append(subdiv);

            var refparams = refcontent.hasOwnProperty('wparams') ? refcontent['wparams'] : null;

            // dynamic, load and apply callback
            $.ajax({
                url: refcontent.url,
                context: {
                    tdiv: $(targetdiv.get(0)),
                    params: refparams
                },
                contentType: 'application/json',
                cache: false
            }).done(
                function (data, textStatus, jqXHR) {
//                    console.log("DONE AJAX: ");
//                    console.log(this);
//                    console.log(typeof data);
                    var dataset = JSON.parse(data);
//                    console.log("Subwidget data from ajax: ", dataset);

//                    console.log("Context for subw: ",this);
                    var wdiv = this.tdiv;
                    var wparams = this.params;

                    var widget = $(wdiv.find('.jswlib_loadedwidget').get(0));

                    lwp.run.call(widget, 'winit');
                    widget.width(wdiv.innerWidth());
                    lwp.toDisplay.call(widget).css('height', widget.css('height'));
                    lwp.toDisplay.call(widget).width(widget.innerWidth());

//                    console.log("Widget Init done");
//                    console.log($(widget).data());
                    var populated = lwp.populate.call(widget, dataset);
                    if (populated)
                    {
                        if (wparams != null)
                        {
                            widget.data('wparams', wparams);
                        }
//                        console.log("Data for subw: ",widget.data());
                        lwp.render.call(widget);
                    }
                    else
                    {
                        widget.data('wtype', 'customdiv');
                        widget.lwp('winit', 'customdiv');
                        widget.css('height', "2em");
                        lwp.toDisplay.call(widget).css('height', '100%');
                        lwp.warn.call(widget);
                    }
//                    console.log("Widget inited");
                }
            ).fail(
                function (jqXHR, textStatus, errorThrown) {
//                    console.log("FAILED AJAX: ");
                    var wdiv = this.tdiv;
                    var widget = $(wdiv.find('.jswlib_loadedwidget').get(0));
                    widget.lwp('winit', 'customdiv');
                    widget.css('height', "2em");
                    lwp.toDisplay.call(widget).css('height', '100%');
                    lwp.warn.call(widget);
                }
            );



        }
        $(this).html('&#9650;');
        targetrow.show();
    }
    else
    {
        $(this).html('&#9660;');
        try
        {
            targetdiv.empty();
        }
        catch (err)
        {
            // do nothing intentionally
        }

//        console.log("Target div emptied");
//        console.log(targetdiv);
        targetrow.hide();
    }

};

tablevis.setSorting = function (event)
{

//    console.log("Setting sorting on table column");
//    console.log("Event ",event, " data ", event.data);

    var widget = lwp.toMaster.call(this);
//    console.log("Master widget ",widget);
//    console.log("master data: ",widget.data());
//    console.log("master wlive: ", lwp.fromWlive.call(widget));

    var sortcol = event.data.colidx;
    var oldsortedby = lwp.fromWlive.call(widget, 'sortedby');

    var sortedby = {
        col: sortcol,
        dir: true
    };

    if (oldsortedby != null && oldsortedby['col'] == sortcol && typeof oldsortedby['dir'] == 'boolean')
    {
        sortedby.dir = !oldsortedby.dir;
    }

//    console.log("updated sorting criteria to ",sortedby);

    lwp.toWlive.call(widget, sortedby, 'sortedby');
    lwp.render.call(widget);


};

tablevis.sortby = function (dataset, sortedby)
{


//    console.log("Sorting ",dataset," by ",sortedby);

    var sortedset = clone(dataset);

    sortedset['data'].sort(function (a,b)
    {
        var sortcol = sortedby['col'];
        var sortdir = sortedby['dir'];
//        console.log("SORTED BY ",sortedby);
//        console.log("SORTING SET WITH CONTEXT ",this);
        var aSorter = $.trim($("<div/>").append(lwp.rendertextcell(a['val'][sortcol])).text());
        var bSorter = $.trim($("<div/>").append(lwp.rendertextcell(b['val'][sortcol])).text());

        if (sortdir)
        {
            return (aSorter < bSorter) ? -1 : (aSorter > bSorter) ? 1 : 0;
        }
        else
        {
            return (bSorter < aSorter) ? -1 : (bSorter > aSorter) ? 1 : 0;
        }


    });



    return sortedset;

};

tablevis.wrender  = function ()
{
    // based on foldtable renderer
    // TODO: clean up and replace, make REF conditional to rendering



    lwp.toDisplay.call(this).empty();
//    console.log("Cleaned up render div");
//    console.log(lwp.toDisplay.call(this));

    var master = lwp.toMaster.call(this);

    var dataset;
    if (lwp.run.call(master, 'isfiltered'))
    {
        dataset = lwp.fromWlive.call(master, 'filtered');
    }
    else
    {
        dataset = $(this).data('wdatafinal');
    }

    var sortedby = lwp.fromWlive.call(master, 'sortedby');
    if (sortedby != null)
    {
        dataset = tablevis.sortby.call(this, dataset, sortedby);
    }

//    console.log("rendering tablevis on dataset");
//    console.log(dataset);

    var rendertable = $("<table/>");
    rendertable.addClass('jswlib_tablevis');

    var header = $('<thead/>');
    var trheader = $('<tr/>');

    if (dataset.foldable)
    {
        trheader.append($('<th/>'));
    }

    for (var h = 0; h < dataset.headers.length; h++)
    {
        var sortbutton = $("<span/>");
        sortbutton.addClass("jswlib_btncontrol");
        sortbutton.addClass("jswlib_flag_notextsel");
        sortbutton.append('<i class="icon-sort"></i>');
        sortbutton.attr('title', 'Sorting');
        sortbutton.bind('click', {colidx: h}, tablevis.setSorting);

        var ccell = $('<th/>');
        ccell.append($("<span/>").append(dataset.headers[h]));
        ccell.append(sortbutton);
        trheader.append(ccell);
    }

    header.append(trheader);
    rendertable.append(header);

    var livedata = lwp.fromWlive.call(master);
    var rowoffset = 0;
//    console.log("with livedata");
//    console.log(livedata);
    try
    {
        if (typeof livedata['offset'] === 'number')
        {
            rowoffset = livedata['offset'];
        }
    }
    catch (err)
    {
    }

    var dynamic = lwp.fromWlive.call(master, 'dynamic');
//    console.log("DYNAMIC: "+dynamic);

    var dataoffset = 0;
    if (!dynamic)
    {
        dataoffset = rowoffset;
    }
//    console.log("Actual offset relative to present dataset: "+dataoffset);
    var rowsnum = lwp.run.call(this, 'getRowsNum');
    var datalen = dataset.data.length;
//    console.log("Rowsnum setting "+rowsnum+" on len "+datalen+" maxpoint "+(rowsnum+dataoffset));
    if (rowsnum == 0 || rowsnum+dataoffset > datalen)
    {
        rowsnum = datalen-dataoffset;
    }

    for (var i = 0; i < rowsnum; i++)
    {
//        console.log("Rendering row "+(i+dataoffset));
        var row = dataset.data[(i+dataoffset)];

        var rendersection = $("<tbody/>");
        rendersection.addClass('jswlib_foldbody');
        var colnum = row.val.length;
        var valuesrow = $('<tr class=jswlib_valuesrow/>');

        if (row.class != null && row.class != "")
        {
            valuesrow.addClass(row.class);
        }

        if (dataset.foldable)
        {
            var subrow = $('<tr class=jswlib_subrefrow><td class=jswlib_subreftd/></tr>');
            subrow.find('td').append('<div style="width: 100%"/>');
            subrow.find('td').find('div').data('wdata', row.ref);
            subrow.find('td').find('div').prop('id', lwp.getDisplayId.call($(this))+"_sub_"+i);
            subrow.find('td').find('div').addClass('jswlib_foldableslot');

            var switcher = $('<td/>');
            if (typeof row.ref != "undefined" && row.ref != null && row.ref != "")
            {
                switcher.addClass('jswlib_foldbutton');
                switcher.click(tablevis.switchfold);

                switcher.html('&#9660;');
            }
            valuesrow.append(switcher);
        }

        for (var n = 0; n < colnum; n++)
        {
            var valuetd = $('<td/>');
            valuetd.append(lwp.rendertextcell(row.val[n]));
            valuesrow.append(valuetd);
        }

        rendersection.append(valuesrow);
        if (dataset.foldable)
        {
            subrow.find("td").prop("colspan", colnum+1);
            rendersection.append(subrow);
            // all hidden by default
            subrow.hide();
        }


        rendertable.append(rendersection);
    }

    rendertable.css('width', '100%');
    var tablecontainer = $('<div/>');
    tablecontainer.addClass("jswlib_dyntablecontainer");
    tablecontainer.append(rendertable);
    lwp.toDisplay.call(this).append(tablecontainer);


};



tablecontroller = {};

tablecontroller.emerge = function ()
{
        //todo: placeholder, implement
//    console.log("Emerging table controller data");
//    console.log($(this).data());
};

tablecontroller.getMaxRows = function ()
{
    //jswlib_tablectrl_maxrows_select
    var target = $(this).find(".jswlib_tablectrl_maxrows_select").val();
    return target.val();
};

tablecontroller.wrender = function ()
{

//    console.log("Tablecontroller rendering start");

    lwp.toDisplay.call(this).empty();

    var master = lwp.toMaster.call(this);
    var dataset;
    if (lwp.run.call(master, 'isfiltered'))
    {
//        console.log("Rendering from filtered");
        dataset = lwp.fromWlive.call(master, 'filterctx');
    }
    else
    {
//        console.log("Rendering from original");
        dataset = $(this).data('wdata');
    }
//    console.log(dataset);

    var offset = 0;
    try
    {
        var liveoffset = lwp.fromWlive.call(lwp.toMaster.call(this), 'offset');
        if (typeof liveoffset == 'number')
        {
            offset = liveoffset;
        }
        else
        {
            var dataoffset = 0;
            try
            {
                dataoffset = dataset['offset'];
            }
            catch (err)
            {
                //nothing, offset not provided
            }

            if (typeof dataoffset == 'number')
            {
                offset = dataoffset;
            }
        }
    }
    catch (err)
    {

    }

//    console.log("Called with offset "+offset);


    var div_wrapper = $("<table/>");
    div_wrapper.css('height', '2em');

//    console.log("Am I dynamic? "+dataset.dynamic);


    // filter section, allowing a filter on a single column, has launcher button since we type stuff in
    var hasfilter = lwp.run.call(master, 'isfiltered');
    var div_filter = $("<td/>");
    div_filter.addClass("jswlib_tablectrl_filter");
    div_filter.css('text-align', 'left');
    var filter_selector = $("<select/>");
    filter_selector.addClass('jswlib_tablectrl_filter_sel');
//    console.log("Creating filter");
    for (var i = 0; i < dataset.columns.length; i++)
    {
        var option = $("<option/>");
        option.attr('value', dataset.columns[i]);
        option.append(""+dataset.columns[i]);
        filter_selector.append(option);
    }
    var filter_launcher = $('<span/>');
    filter_launcher.append('<i class="icon-filter"></i>');
    filter_launcher.addClass("jswlib_tablectrl_filterupdate");
    filter_launcher.addClass("jswlib_btncontrol");
    filter_launcher.addClass("jswlib_flag_notextsel");
    var filter_query = $('<input type="text"/>');
    filter_query.addClass('jswlib_tablectrl_filter_query');
    if (hasfilter)
    {
        filter_query.val(lwp.fromWlive.call(master, 'filtervalue'));
        filter_selector.val(lwp.fromWlive.call(master, 'filtercolumn'));
    }

    var filter_cancel = $("<span/>");
    filter_cancel.addClass("jswlib_tablectrl_filtercancel");
    filter_cancel.addClass("jswlib_btncontrol");
    filter_cancel.addClass("jswlib_flag_notextsel");
    filter_cancel.append('<i class="icon-remove-sign"></i>');
    filter_cancel.click(tablecontroller.unfilter);

    div_filter.append(filter_selector);
    div_filter.append(filter_query);
    div_filter.append(filter_launcher);
    div_filter.append(filter_cancel);

    var div_maxrows = $("<td/>");
    div_maxrows.addClass("jswlib_tablectrl_maxrows");
    div_maxrows.css('text-align', 'right');
    var rowspans = lwp.getRowSpans();
    var maxrows_selector = $("<select/>");
    maxrows_selector.addClass("jswlib_tablectrl_maxrows_select");

//    console.log("Creating max rows selector");

    var option;
    if (dataset.dynamic === false)
    {
        // if the data can all be loaded at once, we add a null option
        option = $("<option/>");
        option.attr('value', 0);
        maxrows_selector.append(option);
    }
    for (var i = 0; i < rowspans.length; i++)
    {
        option = $("<option/>");
        option.attr('value', rowspans[i]);
        option.append(""+rowspans[i]);
        maxrows_selector.append(option);
    }
    div_maxrows.append(maxrows_selector);


    // gtetting the preset value from wlive
    var vis = lwp.run.call(master, 'getdiv_table');

    var rowsnum = lwp.run.call(vis, 'getRowsNum');
    //var datalen = dataset.data.length;
    //    console.log("Setting table controller to rows "+rowsnum);
    maxrows_selector.val(rowsnum);

//    console.log("Created maxrows selector");

    var div_pager = $("<td/>");
    div_pager.css('text-align', 'center');

    var totalrows = dataset.dynamic ? dataset.results : dataset.rows;
    if (rowsnum == 0) { rowsnum = totalrows }

    var pagesnum = Math.floor(totalrows/rowsnum);
    if (dataset.results%rowsnum != 0)
    {
        pagesnum+=1;
    }

    // number of pages we can reach directly via their number.
    var pagesdefault = 5;
    var pages = Math.min(pagesdefault, pagesnum);

    var cpage = Math.floor(offset/rowsnum);

//    console.log("Current page is "+cpage);
//    console.log("(FF check: from rowsnum "+rowsnum+" and offset "+offset+")");

    var selspan;

    if (pages==pagesnum)
    {
        //todo: implement
        //all result pages stay in the selector

        if (pages > 1)
        {
            for (var i = 0; i < pages; i++)
            {
                selspan = $("<span/>");
                selspan.data('woffset', i*rowsnum);
                selspan.append(i+1);
                if (i == cpage)
                {
                    // different since the currently activated one should not trigger any change in offset
                    selspan.addClass("jswlib_tablectrl_pagerlink_current");
                }
                else
                {
                    selspan.addClass("jswlib_tablectrl_pagerlink");
                }
                div_pager.append(selspan);

            }
        }

    }
    else
    {
        var extrapages = Math.floor(pages/2);
        // some result pages don't fit
        var lowerend = cpage - extrapages;
        var upperend = cpage + extrapages;

//        console.log("From "+lowerend+" to "+upperend);

        if (lowerend < 0)
        {
            upperend += (-lowerend);
            lowerend = 0;
        }
        else if (upperend > pagesnum)
        {
            lowerend += (pagesnum - upperend);
            upperend = pagesnum;
        }


        if (cpage > 0)
        {
            // doing FIRST PAGE span
            selspan = $("<span/>");
            selspan.data('woffset', 0);
            selspan.addClass("jswlib_tablectrl_pagerlink");
            selspan.append("&lt;&lt;");
            div_pager.append(selspan);

            // doing PREVIOUS PAGE span
            selspan = $("<span/>");
            selspan.data('woffset', (cpage-1)*rowsnum);
            selspan.addClass("jswlib_tablectrl_pagerlink");
            selspan.append("&lt;");
            div_pager.append(selspan);
        }

        if (lowerend > 0)
        {
            div_pager.append("...");
        }

        for (var i = lowerend; i <= upperend; i++)
        {
            selspan = $("<span/>");
            selspan.data('woffset', i*rowsnum);
            selspan.append(i+1);
            if (i == cpage)
            {
                // different since the currently activated one should not trigger any change in offset
                selspan.addClass("jswlib_tablectrl_pagerlink_current");
            }
            else
            {
                selspan.addClass("jswlib_tablectrl_pagerlink");
            }
            div_pager.append(selspan);

        }

        if (upperend != pagesnum)
        {
            div_pager.append("...");
        }

        if (cpage < pagesnum)
        {
            // doing NEXT PAGE span
            selspan = $("<span/>");
            selspan.data('woffset', (cpage+1)*rowsnum);
            selspan.addClass("jswlib_tablectrl_pagerlink");
            selspan.append("&gt;");
            div_pager.append(selspan);

            // doing LAST PAGE span
            selspan = $("<span/>");
            selspan.data('woffset', (pagesnum-1)*rowsnum);
            selspan.addClass("jswlib_tablectrl_pagerlink");
            selspan.append("&gt;&gt;");
            div_pager.append(selspan);
        }

    }
    div_pager.addClass("jswlib_flag_notextsel");


    //jswlib_tablectrl_pagerlink


    div_wrapper.append($("<tr/>"));

    div_wrapper.find('tr').append(div_filter);
    div_wrapper.find('tr').append(div_pager);
    div_wrapper.find('tr').append(div_maxrows);
    var div_container = $("<div/>");
    div_container.css('width', '100%');
    div_container.append(div_wrapper);
    div_wrapper.css("width", "100%");
    div_wrapper.find('td').css('vertical-align', 'text-top');

    lwp.toDisplay.call(this).append(div_container);

    div_filter.find('.jswlib_tablectrl_filterupdate').off('click');
    div_filter.find('.jswlib_tablectrl_filterupdate').on('click', tablecontroller.refilter);

    div_pager.find('.jswlib_tablectrl_pagerlink').off('click');
    div_pager.find('.jswlib_tablectrl_pagerlink').on('click', tablecontroller.switchpage);

    maxrows_selector.off('change');
    maxrows_selector.on('change', tablecontroller.setrownum);



//    console.log("Finished rendering controls");
};

tablecontroller.unfilter = function (evt)
{
    var widget = lwp.toMaster.call(this);
/*    var formfilterval = $.trim(widget.find(".jswlib_tablectrl_filter_query").val());
    if (formfilterval!="")
    {
    }*/

    widget.find(".jswlib_tablectrl_filter_query").val("");
    widget.find(".jswlib_tablectrl_filterupdate").click();

};

tablecontroller.refilter = function (evt)
{

//    console.log("setting new filter and redrawing");
//    console.log($(this));
//    console.log($(this).data());
//    console.log(evt);

    var widget = lwp.toMaster.call(this);
    var isdynamic = widget.data('wdata').dynamic === true;

    var formfiltercol = widget.find(".jswlib_tablectrl_filter_sel").val();
    var formfilterval = $.trim(widget.find(".jswlib_tablectrl_filter_query").val());

    // getting the current filter settings (excluding pagination)
    var cfiltercol = lwp.fromWlive.call(widget, 'filtercolumn');
    var cfilterval = lwp.fromWlive.call(widget, 'filtervalue');
    if (typeof cfilterval == 'undefined')
    {
        cfilterval = "";
    }

//    console.log ("Filtering col "+formfiltercol+" with "+formfilterval);
//    console.log ("Previously col "+formfiltercol+" with "+formfilterval);

    var updated = (formfilterval != "" && formfiltercol != cfiltercol) || (formfilterval != cfilterval);

//    console.log ("Filter updated? "+updated);

    if (updated)
    {
        if (formfilterval != "")
        {
            lwp.toWlive.call(widget, formfiltercol, 'filtercolumn');
            lwp.toWlive.call(widget, formfilterval, 'filtervalue');
        }
        else
        {
            lwp.dropWlive.call(widget, 'filtercolumn');
            lwp.dropWlive.call(widget, 'filtervalue');
        }

        // updating the filter data in wlive
        lwp.run.call(widget, 'filter');

        // redrawing now or differed
        if (isdynamic)
        {
//            console.log('asking for differed redraw');
            lwp.run.call(widget, 'askredraw');
        }
        else
        {
//            console.log("redrawing now");
            lwp.render.call(widget);
        }
    }



};



tablecontroller.switchpage = function (evt)
{
//    console.log("setting new page (offset) and redrawing");
//    console.log($(this));
//    console.log($(this).data());
//    console.log(evt);

    var newoffset = $(this).data('woffset');
    var widget = $(this).closest(".jswlib_widget").parents(".jswlib_widget");
    var isdynamic = $(widget).data('wdata').dynamic === true;
    lwp.toWlive.call(widget, newoffset, 'offset');
//    console.log("wlive statu BEFORE paging: ",widget," -> ",lwp.fromWlive.call(widget));
    if (isdynamic)
    {
//        console.log('asking for differed redraw');
        lwp.run.call(widget, 'askredraw');

    }
    else
    {
//        console.log("redrawing now");
        lwp.render.call(widget);
    }
//    console.log("wlive statu AFTER paging: ",widget," -> ",lwp.fromWlive.call(widget));

};


tablecontroller.setrownum = function (evt)
{
/*    console.log("setting number of drawable rows and redrawing");
    console.log(this);
    console.log(evt);*/

    var value = Number($(this).val());


    var widget = $(this).closest(".jswlib_widget").parents(".jswlib_widget");

    var isdynamic = $(widget).data('wdata').dynamic === true;

//    console.log("Value to "+value+" (dynamic: "+isdynamic+")");

    if (isdynamic)
    {
        if (value == 0)
        {
            // we never show all the rows in an ajax table
            // if has complete structure for better doc/readability only
        }
        else
        {
            lwp.toWlive.call(widget, value, 'rowsnum');
            lwp.run.call(widget, 'askredraw');
        }
    }
    else
    {
        lwp.toWlive.call(widget, value, 'rowsnum');
        lwp.render.call(widget);
    }



};

jswlib['dynamictable'] = dynamictable;
jswlib['tablecontroller'] = tablecontroller;
jswlib['tablevis'] = tablevis;
buildWidgetsTree();