/**
 * Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
 * Created: 5/22/13, 10:24 AM
 * Author: Antonio Vaccarino
 */

// traced graph + timeline
// can use bands or series according to subclasses

compositegraph = {};

compositegraph.assemble = function ()
{
    var displaydiv = lwp.toDisplay.call(this);
    $(this).addClass("jswlib_timegraph_master");

    var id_ctrl = lwp.run.call(this, 'getid_ctrl');
    var id_vis = lwp.run.call(this, 'getid_vis');
    var id_legend = lwp.run.call(this, 'getid_legend');

    var div_ctrl = $("<div/>");
    div_ctrl.attr('id', id_ctrl);
    div_ctrl.addClass("jswlib_timegraph_ctrl");
    // for testing, previously 33% but now we are adding the legend
    // moved to wrender
    //div_ctrl.css("height", "20%");

    var div_vis = $("<div/>");
    div_vis.attr('id', id_vis);
    div_vis.addClass("jswlib_timegraph_vis");
    // moved to wrender
    //div_vis.css("height", "67%");

    var div_legend = $("<div/>");
    div_legend.attr('id', id_legend);
    div_legend.addClass("jswlib_timegraph_legend");

    displaydiv.append(div_vis);
    displaydiv.append(div_ctrl);
    displaydiv.append(div_legend);

    div_vis.lwp('winit', 'timegraph_vis');
    div_ctrl.lwp('winit', 'timegraph_ctrl');
    div_legend.lwp('winit', 'graphedlegend');
};

compositegraph.wupdate = function ()
{
//    console.log("Wupdating composite graph, creating filtered data");

//    console.log('Retrieving wajax data');
    var cajax = $(this).data('wajax');
//    console.log(cajax);

    if (($.isArray(cajax) && cajax.length == 0) || Object.keys(cajax).length == 0)
    {

        // not an actual update but the initial set up
        if ($(this).data('wincoming'))
        {
            /*
            console.log("Pushing data from wincoming to wdata in "+$(this).attr('id'));
            */
            lwp.run.call(this, 'populate', [$(this).data('wincoming')]);
        }
    }



};

compositegraph.getid_ctrl = function ()
{
    return $(this).attr('id')+"_sub_ctrl";
};

compositegraph.getdiv_ctrl = function()
{
    return $(this).find("#"+lwp.run.call(this, 'getid_ctrl'));
};

compositegraph.getid_vis = function ()
{
    return $(this).attr('id')+"_sub_vis";
};

compositegraph.getdiv_vis = function()
{
    return $(this).find("#"+lwp.run.call(this, 'getid_vis'));
};

compositegraph.getid_legend = function ()
{
    return $(this).attr('id')+"_sub_legend";
};

compositegraph.getdiv_legend = function()
{
    return $(this).find("#"+lwp.run.call(this, 'getid_legend'));
};

compositegraph.isdynamic = function()
{
    try
    {
        return ($(this).data('wconf')['dynamic']===true);
    }
    catch (err)
    {
        return false;
    }
};

compositegraph.emerge = function ()
{
/*
    console.log("Core emerge of dynamic timegraph");
*/

    // this widget is SUPPOSED to be subclassed, so
    // if there is an emerge specific to the derived widget, that applies
    // if there is no emerge, then the data is copied plainly to the inner widgets
    // (that cannot access params and conf, though)
    var div_ctrl = lwp.run.call(this, 'getdiv_ctrl');
    lwp.populate.call(div_ctrl, $(this).data('wdata'));
    var div_vis = lwp.run.call(this, 'getdiv_vis');
    lwp.populate.call(div_vis, $(this).data('wdata'));
    var div_legend = lwp.run.call(this, 'getdiv_legend');
    lwp.populate.call(div_legend, $(this).data('wdata'));



};


compositegraph.wrender = function ()
{

    var height_full = $(this).height();

//    console.log("Rendering TIMEGRAPH COMPONENTS");
//    console.log("of height: "+height_full);

    var div_ctrl = lwp.run.call(this, 'getdiv_ctrl');
    var div_vis = lwp.run.call(this, 'getdiv_vis');
    var div_legend = lwp.run.call(this, 'getdiv_legend');

    var opts = {};
    if ($(this).data('wdata')['renderopts'].hasOwnProperty('wconf'))
    {
        opts = $(this).data('wdata')['renderopts']['wconf'];
    }
    //console.log("Rendering opts for "+$(this).attr('id'), opts);
    var conf = $(this).data('wconf');

    var hidelegend = false;
    if (conf.hasOwnProperty('legend'))
    {
        hidelegend = conf['legend'] === false;
    }
    if (opts.hasOwnProperty('legend'))
    {
        hidelegend = opts['legend'] === false;
    }

    var hidecontrols = false;
    if (conf.hasOwnProperty('controls'))
    {
        hidecontrols = conf['controls'] === false;
    }
    if (opts.hasOwnProperty('controls'))
    {
        hidecontrols = opts['controls'] === false;
    }

    var hidetimeline = false;
    if (hidecontrols)
    {
        hidetimeline = true;
    }
    else
    {
        if (conf.hasOwnProperty('timeline'))
        {
            hidetimeline = conf['timeline'] === false;
        }
        if (opts.hasOwnProperty('timeline'))
        {
            hidetimeline = opts['timeline'] === false;
        }
    }

    // re-expressing predefined zoom in msec
    // dataset OVERRIDES
    var prezoom = false;
    if (conf.hasOwnProperty('prezoom') && typeof conf['prezoom'] == 'number' && conf['prezoom'] > 0 )
    {
        prezoom = conf['prezoom']*60000;
    }
    if (opts.hasOwnProperty('prezoom')  && typeof opts['prezoom'] == 'number' && opts['prezoom'] > 0)
    {
        prezoom = opts['prezoom'] * 60000
    }



/*    // debug
    div_ctrl.css( "border", "1px solid #ff0000");
    div_vis.css( "border", "1px solid #00ff00");
    div_legend.css( "border", "1px solid #0000ff");*/

    var height_legend = 0;
    if (!hidelegend)
    {
        lwp.run.call(div_legend, 'wrender');
        height_legend = lwp.toDisplay.call(div_legend).find(".jswlib_hlegend").outerHeight();
    }
    else
    {
        div_legend.hide();
    }

//    console.log("Heights: FULL "+height_full+" LEGEND "+height_legend);

    var height_graphs = height_full - height_legend;

    var height_timeline = 0;

    if (height_graphs <= 128)
    {
        hidetimeline = true;
    }

    if (!hidetimeline)
    {
        height_timeline = 64;
    }
    else
    {
        height_timeline = 1;

    }

    div_ctrl.height(height_timeline);
    lwp.toDisplay.call(div_ctrl).height(height_timeline);
    lwp.run.call(div_ctrl, 'wrender');
    if (hidetimeline)
    {
        div_ctrl.css('visibility', 'hidden');
    }
    var height_vis = height_graphs - height_timeline;
    div_vis.height(height_vis);
    lwp.toDisplay.call(div_vis).height(height_vis);
//    console.log("Graph height for main render: "+height_vis);
    lwp.run.call(div_vis, 'wrender');


    // default zoom
    if (prezoom)
    {
//        console.log("applying predefined zoom of "+prezoom+" msec");
        compositegraph.livezoom.call(this, 'preset', {zoomed: true, range: prezoom});
    }


};

compositegraph.livezoom = function (eventype, eventdata)
{

/*
    console.log("called livezoom routine w eventype "+eventype+" and data");
    console.log(eventdata);
*/

    // 1. get the time range according to type (if applicable)
    var deltaperc = 0.2;
    var min = $(this).data('wparams')['timescale'][0];
    var max = $(this).data('wparams')['timescale'][1];
    var dynamic = lwp.run.call(this, 'isdynamic');

    var minrange = $(this).data('wparams')['timescale'][2]*5;

    var zoomed;
    var zoomfrom;
    var zoomto;
    var czoomfrom;
    var czoomto;
    var czoomwidth;
    var czoomdelta;

    // we erase any existing zoom data so the "pre-paint" of ajaxed widgets can know it is working on optical for the time being
    lwp.dropWlive.call(this, 'zoomdata');

    if (eventype == 'graphselect')
    {
        // selection on any widget: zoom on this time selection
        zoomed = true;
        zoomfrom = eventdata.x1;
        zoomto = eventdata.x2;

        var req_range = zoomto-zoomfrom;
        if (req_range < minrange)
        {
            var exc_range = minrange - req_range;
            zoomfrom = Math.max(min, zoomfrom - exc_range);
            zoomto = Math.min(max, zoomfrom+minrange);
        }
    }
/*  REPLACED WITH PRESET
    else if (eventype == 'unzoom')
    {
        // single click on controller widget: unzoom
        zoomed = false;
        zoomfrom = min;
        zoomto = max;
    }*/
    else if (eventype == 'zoomout')
    {
        zoomed = true;
        czoomfrom = lwp.fromWlive.call(this, 'zoomfrom');
        czoomto = lwp.fromWlive.call(this, 'zoomto');
//        console.log("Zooming out based on "+czoomfrom+" to "+czoomto);
        if (isNaN(czoomfrom)) { czoomfrom = min; }
        if (isNaN(czoomto)) { czoomto = max; }
        if (czoomfrom == min && czoomto == max)
        {
            return;
        }
        czoomwidth = czoomto - czoomfrom;
        czoomdelta = (czoomwidth/2)*(deltaperc/2);
        zoomfrom = Math.max(min, czoomfrom - czoomdelta);
        zoomto = Math.min(max, czoomto + czoomdelta);
        if (zoomfrom == min && zoomto == max)
        {
            zoomed = false;
            zoomfrom = min;
            zoomto = max;
        }
    }
    else if (eventype == 'zoomin')
    {
        // button - on vis widget: zoom out
        zoomed = true;
        czoomfrom = lwp.fromWlive.call(this, 'zoomfrom');
        czoomto = lwp.fromWlive.call(this, 'zoomto');
        if (isNaN(czoomfrom)) { czoomfrom = min; }
        if (isNaN(czoomto)) { czoomto = max; }
//        console.log("Zooming in based on "+czoomfrom+" to "+czoomto);
        czoomwidth = czoomto - czoomfrom;
        czoomdelta = (czoomwidth/2)*(deltaperc/2);
        zoomfrom = czoomfrom + czoomdelta;
        zoomto = czoomto - czoomdelta;

        var req_range = zoomto-zoomfrom;
        if (req_range < minrange)
        {
            var exc_range = minrange - req_range;
            zoomfrom = Math.max(min, zoomfrom - exc_range);
            zoomto = Math.min(max, zoomfrom+minrange);
        }

    }
    else if (eventype == 'scrollfwd')
    {
        zoomed = true;
        czoomfrom = lwp.fromWlive.call(this, 'zoomfrom');
        czoomto = lwp.fromWlive.call(this, 'zoomto');
//        console.log("Zooming out based on "+czoomfrom+" to "+czoomto);
        if (isNaN(czoomfrom)) { czoomfrom = min; }
        if (isNaN(czoomto)) { czoomto = max; }
        if (czoomfrom == min && czoomto == max)
        {
            return;
        }
        czoomwidth = czoomto - czoomfrom;
        czoomdelta = (czoomwidth/2)*(deltaperc/2);
        zoomto = Math.min(max, czoomto + czoomdelta);
        zoomfrom = Math.max (min, zoomto - czoomwidth);
        if (zoomfrom == min && zoomto == max)
        {
            zoomed = false;
            zoomfrom = min;
            zoomto = max;
        }
    }
    else if (eventype == 'scrollback')
    {
        zoomed = true;
        czoomfrom = lwp.fromWlive.call(this, 'zoomfrom');
        czoomto = lwp.fromWlive.call(this, 'zoomto');
//        console.log("Zooming out based on "+czoomfrom+" to "+czoomto);
        if (isNaN(czoomfrom)) { czoomfrom = min; }
        if (isNaN(czoomto)) { czoomto = max; }
        if (czoomfrom == min && czoomto == max)
        {
            return;
        }
        czoomwidth = czoomto - czoomfrom;
        czoomdelta = (czoomwidth/2)*(deltaperc/2);
        zoomfrom = Math.max(min, czoomfrom - czoomdelta);
        zoomto = Math.min(max, zoomfrom + czoomwidth);
        if (zoomfrom == min && zoomto == max)
        {
            zoomed = false;
            zoomfrom = min;
            zoomto = max;
        }
    }
    else if (eventype == 'preset')
    {
        // button preset, includes one unzoom option
        zoomed = eventdata['zoomed'];
        if (zoomed)
        {
            zoomfrom = Math.max(min, max-eventdata['range']);
            zoomto = max;
        }
        else
        {
            zoomfrom = min;
            zoomto = max;
        }
    }

    // 2. set wlive with it
    lwp.toWlive.call(this, zoomed, 'zoomed');
    lwp.toWlive.call(this, zoomfrom, 'zoomfrom');
    lwp.toWlive.call(this, zoomto, 'zoomto');

    // 3. launch wrender OR askrender on children
    var div_vis = lwp.run.call(this, 'getdiv_vis');
    var div_ctrl = lwp.run.call(this, 'getdiv_ctrl');

//    console.log("Rendering section livezoom");

    // simply draws the selection for the new zoom
    // it's actually a double call if zoom came from controller select
    lwp.run.call(div_ctrl, 'renderzoom');
    // we always render the "optical" zoom anyway, in case the data comes slow
//    console.log(div_vis);
    lwp.render.call(div_vis);

    if (dynamic && zoomed == true)
    {
        lwp.run.call(div_vis, 'renderwait');
        var payload = {
            timespan: [ Math.round(zoomfrom/1000), Math.round(zoomto/1000) ]
        };
//        console.log("Sending ajax wave timegraphzoom and data");
//        console.log(payload);
        lwp.wave.call(this, 'timegraphzoom', payload);
    }

    if (zoomed === false)
    {
        // if unzommed, we remove all calls from the queue
        lwp.flushwave.call(this);
    }

//    console.log("Completed livezoom operation");

};

compositegraph_band = clone (compositegraph);

compositegraph_band.assemble = function ()
{

    compositegraph.assemble.call(this);
    // adding binding for hover action
    var div_graph= timegraph_vis.getdiv_graph.call(compositegraph.getdiv_vis.call(this));
    var div_vis = div_graph.get(0);
//    console.log("Setting bindings on display ",div_vis);
    $(window.document).on('mouseleave', ".jswlib_timegraph_vis,.jsqlib_timegraph_ctrl", function(evt) {
        var widget = $(evt.currentTarget).closest(".jswlib_timegraph_master");
        widget.find('.jswlib_callout').remove();
    });

    Flotr.EventAdapter.observe (div_vis, 'flotr:mousemove', function(evt) {

/*        console.log("Received FLOTR event on BANDGRAPH");
        console.log(evt);*/

        var div_vis = $(evt.currentTarget).closest(".jswlib_widget");
        var div_master = div_vis.closest(".jswlib_timegraph_master");
        var graph = div_vis.data('wrendered');

        var cval = graph.lastMousePos.x;
        console.log("mousemove event on bandgraph ",cval);

        var bandseq = div_master.data('wdata')['values'][1];
        for (var i = 0; i < bandseq.length; i++)
        {

            if (cval >= bandseq[i][0] && cval <= bandseq[i][1])
            {
//                console.log("Found band for callout", bandseq[i]);

                var calloutstring = "";

                var pubDate = new Date();
                pubDate.setTime(cval);
                calloutstring += "<b>"+lwp.renderTimeString(pubDate)+"</b><br>";


//                console.log("Moved into band n."+i);
                try
                {
                    var info = lwp.rendertextcell(bandseq[i][3]);
                    if ($.trim(info) == "" && typeof bandseq[i][3] != 'undefined')
                    {
                        info = "Nessuna informazione per l'intervallo";
                    }
                    else if (typeof bandseq[i][3] == 'undefined')
                    {
                        info = "";
                    }
                    calloutstring += info;
                }
                catch (err)
                {
                    //console.log("NO DATA ENTRY HERE");
                }

                lwp.spawncallout(evt, calloutstring);
            }
        }

    });


};

compositegraph_band.emerge = function ()
{

    //console.log("BAND GRAPH: emerging data");
    //console.log($(this).data('wdata'));

    var graphed = [];
    var labels = $(this).data('wdata')['values'][0];
    var dataset = $(this).data('wdata')['values'][1];
    var dataconf = $(this).data('wdata')['renderopts'];
    var conf = $(this).data('wconf');
    var timescale = [];

    var precolors = lwp.getColors.call(this);

    // building ordered list
    var zsorted = [];
    try
    {
        zsorted = $(this).data('wdata')['renderopts']['zlist'];
        if (typeof zsorted == 'undefined')
        {
            zsorted = [];
        }
    }
    catch (err)
    {

    }

    //console.log("BAND GRAPH: initial zsorted value");
    //console.log(zsorted);

    // adding non-specified series
    for (var i = 0; i < labels.length; i++)
    {
        // setting colors
        if (dataconf['series'].hasOwnProperty(i) && dataconf['series'][i].hasOwnProperty('color'))
        {
            precolors[i] = dataconf['series'][i]['color'];
        }

        if (zsorted.indexOf(i) == -1)
        {
            zsorted.push(i);
        }
    }

    //console.log("BAND GRAPH: created zorder");
    //console.log(zsorted);


    //console.log("BAND GRAPH: adapting dataset");
    //console.log(dataset);
    //console.log(dataset);
    // changing from one series to multiseries
    var zseries = [];
    var zaggregate = [];
    for (var i = 0; i < dataset.length; i++)
    {
        dataset[i][0] *= 1000;
        dataset[i][1] *= 1000;
        var ctimefrom = dataset[i][0];
        var ctimeto = dataset[i][1];
        var cseriesord = dataset[i][2];
        var cseriesord_z = zsorted.indexOf(cseriesord);


        // TODO: temporary, fix for correct graphic styling
        var valueseqfrom = [-1, 1];
        var valueseqto = [1, -1];

        if (!zseries.hasOwnProperty(cseriesord_z))
        {
            zseries[cseriesord_z] = [];
            zaggregate[cseriesord_z] = {
                times: 0,
                avgspan: 0,
                maxspan: 0,
                minspan: Infinity,
                totspan: 0
            };
        }

        var cspan = (ctimeto - ctimefrom)/60000;
        zaggregate[cseriesord_z].totspan += cspan;
        zaggregate[cseriesord_z].avgspan = (zaggregate[cseriesord_z].avgspan * zaggregate[cseriesord_z].times + cspan) / (zaggregate[cseriesord_z].times + 1);
        zaggregate[cseriesord_z].times++;
        if (cspan > zaggregate[cseriesord_z].maxspan)
        {
            zaggregate[cseriesord_z].maxspan = cspan;
        }
        if (cspan < zaggregate[cseriesord_z].minspan)
        {
            zaggregate[cseriesord_z].minspan = cspan;
        }

        zseries[cseriesord_z].push([ctimefrom, valueseqfrom[0]]);
        zseries[cseriesord_z].push([ctimefrom, valueseqfrom[1]]);
        //zseries[cseriesord_z].push([ctimefrom, valueseqfrom[2]]);
        zseries[cseriesord_z].push([ctimeto, valueseqto[0]]);
        zseries[cseriesord_z].push([ctimeto, valueseqto[1]]);
        //zseries[cseriesord_z].push([ctimeto, valueseqto[2]]);

        //updating timescale
        if (!(ctimefrom > timescale[0]))
        {
            timescale[0] = ctimefrom;
        }
        //updating timescale
        if (!(ctimeto < timescale[1]))
        {
            timescale[1] = ctimeto;
        }
    }
    //console.log("Adapted dataset to zseries");
    //console.log(zseries);

/*    console.log("Aggregated data");
    console.log(zaggregate);*/

    // creates aggregate data for use with the graphedlegend widget
    var zcolors = [];
    // setting data for drawing
    for (var z = zsorted.length; z > 0; z--)
    {
        // zsorting colors array
        var i = zsorted[z-1];
        zcolors.push(precolors[i%precolors.length]);

        // TODO: cleanup
        var graphtype = "band";
/*        try
        {
            graphtype = dataconf['series'][i]['dataType'];
            if (typeof graphtype == 'undefined')
            {
                graphtype = "band";
            }
        }
        catch (err)
        {
        }*/

        if (graphtype === "band")
        {
/*
            console.log("Rendering "+i+" in order "+z+" as band");
*/
            graphed.push({
                data:   zseries[z-1],
                label:  labels[i],
                lines: {
                    fill: true,
                    fillOpacity: 0.8
                }
            });
        }
    }

/*    console.log("BAND GRAPH: colorlist Zed");
    console.log(zcolors);*/

/*
      %h: hours
  %H: hours (left-padded with a zero)
  %M: minutes (left-padded with a zero)
  %S: seconds (left-padded with a zero)
  %d: day of month (1-31), use %0d for zero-padding
  %m: month (1-12), use %0m for zero-padding
  %y: year (four digits)
  %b: month name (customizable)
  %p: am/pm, additionally switches %h/%H to 12 hour instead of 24
  %P: AM/PM (uppercase version of %p)
  */

    var opts = {
        yaxis: {
            max: 1,
            min: 0,
            ticks: null,
            showLabels: false
        },
        xaxis: {
            noTicks: Math.ceil($(this).width()/160),
            mode : 'time',
            timeMode: 'local',
            // timeformat:  "%d/%m %H:%M",
            tickFormatter: lwp.dateTickFormatter
        },
        colors: zcolors,
        shadowSize: 0,
        legend: { show: false },
        HtmlText: false,
        fontSize: 6

    };

    // selection is graphically enabled only if the graph is actives
    var passive = (conf.hasOwnProperty('controls') && conf['controls']===false);
    if (!passive)
    {
        opts['selection'] = { mode: 'x', color: "#000000" };
    }

    // separated from graphing routine for cleanness (and easier removal if needed)
    var zlegend = [];
    for (var i = 0; i  < zsorted.length; i++)
    {
        z = zsorted[i];
        zlegend.push(
            {
                key: labels[z],
                color:  zcolors[zsorted.length-(i+1)],
                aggregate: zaggregate[z]
            }
        )
    }

/*
    console.log("Prepared data for graphedlegend");
    console.log(zlegend);
*/

    var div_legend = lwp.run.call(this, 'getdiv_legend');
    lwp.run.call(div_legend, 'populate', [zlegend]);

    $(this).data('wgraphed', graphed);
/*    console.log("BAND GRAPH: emerged following ");
    console.log(graphed);*/
    lwp.updateparams.call(this, 'renderopts', opts);
    lwp.updateparams.call(this, 'timescale', timescale);

    var div_vis = lwp.run.call(this, 'getdiv_vis');
    var div_ctrl = lwp.run.call(this, 'getdiv_ctrl');

    div_vis.data('wparams', $(this).data('wparams'));
    div_ctrl.data('wparams', $(this).data('wparams'));
    div_vis.data('wconf', $(this).data('wconf'));
    div_ctrl.data('wconf', $(this).data('wconf'));
    lwp.run.call(div_vis, 'populate', [graphed]);
    lwp.run.call(div_ctrl, 'populate', [graphed]);



/*    console.log("EMERGE END TIME:");
    console.log(new Date());*/

};

// compositetimegraph for multiseries (several lines already split)

compositegraph_multi = clone (compositegraph);

/*compositegraph_multi.wupdate = function ()
{
    // added to get out of the inherited wupdate
    lwp.run.call(this, 'populate', [$(this).data('wincoming')]);
};*/

compositegraph_multi.assemble = function ()
{

    compositegraph.assemble.call(this);
    // adding binding for hover action
    var div_graph= timegraph_vis.getdiv_graph.call(compositegraph.getdiv_vis.call(this));
    var div_vis = div_graph.get(0);
    $(window.document).on('mouseleave', ".jswlib_timegraph_vis,.jsqlib_timegraph_ctrl", function(evt) {
        var widget = $(evt.currentTarget).closest(".jswlib_timegraph_master");
        widget.find('.jswlib_callout').remove();
    });


    Flotr.EventAdapter.observe (div_vis, 'flotr:mousemove', function(evt) {

        //console.log("Received FLOTR event on MULTIGRAPH");
        //console.log(evt);

        var div_vis = $(evt.currentTarget).closest(".jswlib_widget");
        var div_master = div_vis.closest(".jswlib_timegraph_master");

        var graph = div_vis.data('wrendered');
        //console.log("X "+evt.offsetX+" offsetby "+graph.axes.xO);

        //console.log(graph);
        //console.log(evt.pos);


        //console.log("EVT: Xpixel "+graph.lastMousePos.relX+" Xpos "+graph.lastMousePos.x+" from "+graph.axes.x.min);
        var timescale = div_master.data('wparams')['timescale'];
        var graphed = div_master.data('wgraphed');
        var values = div_master.data('wdata').values;

        var cx = graph.lastMousePos.x;

        if (cx >= graph.axes.x.min && cx <= graph.axes.x.max)
        {
/*
            console.log("calculating callout data");
*/
            var cpoint = Math.round((cx - timescale[0])/timescale[2]);

            cpoint = Math.min (cpoint, graphed[0].data.length-1);

            var calloutstring = "";
            var pubDate = new Date();
            pubDate.setTime(cx);
            calloutstring += "<b>"+lwp.renderTimeString(pubDate)+"</b><br>";

//            console.log("Point specific data for "+pubDate+" - "+cpoint);
//            console.log(graphed);
            var pointdata = {};
            var zindex = lwp.fromWlive.call(div_master, 'zindex');
//            console.log("Callout with zindex distrib ",zindex);
            for (var i = 0; i < graphed.length; i++)
            {
                var datalabel = graphed[i].label;
                var datavalue;
                var valuez = zindex.indexOf(i);
                var z = (graphed.length-1)-valuez;
                /*console.log("rendering series "+i);
                console.log(div_master.data('wdata'));
//                var seriestype = div_master.data('wdata')['renderopts']['series'][i]['dataType']; */

                var seriestype = graphed[i].type;

                //var pointval = graphed[i].data[cpoint][1];
                var pointval = values[2][cpoint][z];


                if (seriestype == 'tick' || seriestype == 'bartick')
                {
                    //console.log(graphed[i].data[cpoint][1]);
                    var summarised = graphed[i].data[cpoint][1] == 1;
                    var pure = pointval;

                    if ($.isNumeric(pure))
                    {
                        pure = lwp.getRoundedTo(pure, 3);
                    }

                    datavalue = ""+summarised;
                    if (pure != null && pure != 0)
                    {
                        datavalue += " ("+pure+")";
                    }
                }
                else
                {
                    datavalue = pointval;
                    if ($.isNumeric(datavalue))
                    {
                        datavalue = lwp.getRoundedTo(datavalue, 3);
                    }

                }
                pointdata[datalabel] = datavalue;
                var outarray = {};
                outarray [datalabel] = datavalue;
                calloutstring += lwp.rendertextcell(outarray)+"<br>";

            }

//            calloutstring += lwp.rendertextcell(pointdata);

            lwp.spawncallout(evt, calloutstring);

        }


    });



};

compositegraph_multi.emerge = function ()
{

    // determines the core scale, used for the controller widget and as a temporary render on the passive widget while waiting for the ajaxwave return

//    console.log("EMERGING ON");
//    console.log(this);

    // note: for testing purposes we will add a fourth number, avg of 1 and 2
//    console.log("EMERGE START TIME FOR MULTIGRAPH:");
//    console.log(new Date());
    var graphed = [];
    var timescale = lwp.toJavascriptTimeStamp($(this).data('wdata')['values'][0]);
    lwp.updateparams.call(this, 'timescale', timescale);
    var labels = $(this).data('wdata')['values'][1];
    var rawdataset = $(this).data('wdata')['values'][2];
    var dataconf = $(this).data('wdata')['renderopts'];
    var conf = $(this).data('wconf');

//    console.log("Multigraph: prepared for rescale");


    var dataset = lwp.run.call(this, 'rescale', [rawdataset, dataconf]);
/*    console.log("REBUILT DATASET AS");
    console.log(dataset);
    console.log(rawdataset);*/

/*    // TEMPORARY TEST STUFF, to be pulled out
    // defining series by type: line, area, tick, each with a style set and filter?


    console.log("Rescaled dataset");
    //console.log(dataset);

    console.log("Getting colors");*/
    var precolors = lwp.getColors.call(this);
/*
    console.log("Received colors");
    console.log(precolors);
*/

    // building ordered list
    var zsorted = [];
    try
    {
        zsorted = dataconf['zlist'];
    }
    catch (err)
    {
        zsorted = [];
    }

/*    console.log("Adding left labels to zsorted");
    console.log(labels);*/
    if (typeof zsorted == 'undefined')
    {
        zsorted = [];
    }

    for (var i = 0; i < labels.length; i++)
    {

        if (dataconf['series'].hasOwnProperty(i) && dataconf['series'][i].hasOwnProperty('color'))
        {
            precolors[i] = dataconf['series'][i]['color'];
        }

        if (zsorted.indexOf(i) == -1)
        {
            zsorted.push(i);
        }
    }

    // Z ORDER IS DEFINED ONLY NOW

/*    console.log("Added labels");
    console.log("Created zsorting list");
    console.log(zsorted);*/



    // creates aggregate data for use with the graphedlegend widget
    var zaggregate = [];
    var zcolors = [];
    var zunits = [];
    for (var z = zsorted.length; z > 0; z--)
    {
        var i = zsorted[z-1];
        zcolors.push(precolors[i%precolors.length]);
//        console.log("Rendering "+i+" in order "+z);
        var aggr = [];
        var cseries = [];
        for (var t = 0; t < dataset.length; t++)
        {
            var ctime = timescale[0] + t * timescale[2];
            var cvalue = dataset[t][i];
            cseries.push([ctime, cvalue]);
            if (cvalue != null)
            {
                var aggrval = Math.round(rawdataset[t][i]*10000)/10000;
                aggr.push(aggrval);
            }
        }

        var graphtype = "line";
        try
        {
            graphtype = dataconf['series'][i]['dataType'];
        }
        catch (err)
        {
        }

        var cunit = null;
        if (dataconf['series'][i].hasOwnProperty('unit'))
        {
            cunit = dataconf['series'][i]['unit'];
        }
//        console.log("Adding ",cunit," to zunits");
        zunits.push(cunit);

        if (graphtype === "tick")
        {
/*
            console.log("Rendering "+i+" in order "+z+" as tick");
*/
            graphed.push({
                data:   cseries,
                label:  labels[i],
                lines: {show: false},
                type: graphtype,
                bars: {
                    show: true,
                    lineWidth: 1,
                    barWidth: 0,
                    fill: false
                }
            });


        }
        else if (graphtype == 'bartick')
        {
            graphed.push({
                data:   cseries,
                label:  labels[i],
                lines: {show: false},
                type: graphtype,
                bars: {
                    show: true,
                    lineWidth: 1,
                    barWidth: timescale[2],
                    fill: true,
                    fillopacity: 0.4
                }
            });
        }
        else if (graphtype == 'bars')
        {
            graphed.push({
                data:   cseries,
                label:  labels[i],
                type: graphtype,
                lines: { show: false},
                bars: {
                    show: true,
                    lineWidth: 1,
                    barWidth: timescale[2],
                    fill: true,
                    fillOpacity: 0.6

                }
            });

        }
        else if (graphtype === 'area')
        {
            // line used as catchall
            graphed.push({
                data:   cseries,
                label:  labels[i],
                type: graphtype,
                lines:  {
                    fill: true,
                    fillopacity: 0.6,
                    lineWidth: 1
                }
            });

        }
        else
        {
            // line used as catchall
            graphed.push({
                data:   cseries,
                label:  labels[i],
                type: graphtype,
                lines:  {
                    show: true,
                    fill: false,
                    lineWidth: 1
                }
            });




        }

        // aggregation data
        if (graphtype != 'tick' && graphtype != 'bartick')
        {
            var linqer = Enumerable.From(aggr);

//            console.log("Aggregating via linqer ",aggr);


            try
            {
                zaggregate.push(
                    {   max: linqer.Max(),
                        min: linqer.Min(),
                        avg: Math.round(linqer.Average()*10000)/10000
                    }
                );
            }
            catch (err)
            {
                zaggregate.push(
                    {
                        max: "N/A",
                        min: "N/A",
                        avg: "N/A"
                    }
                );
            }
        }
        else
        {
            // TODO: add some aggregator function?
            zaggregate.push({});
        }
    }

    // separated from graphing routine for cleanness (and easier removal if needed)
    var zlegend = [];
    for (var i = 0; i  < zsorted.length; i++)
    {
        z = zsorted[i];
        zlegend.push(
            {
                key: labels[z],
                color:  zcolors[zsorted.length-(i+1)],
                aggregate: zaggregate[zsorted.length-(i+1)],
                unit: zunits[zsorted.length-(i+1)]
            }
        )
    }

//    console.log("Prepared data for graphedlegend");
//    console.log(zlegend);

    var div_legend = lwp.run.call(this, 'getdiv_legend');
    lwp.run.call(div_legend, 'populate', [zlegend]);




    // NOTE: THIS SET PERMANENTLY the OPTS for the CONTROLLER SUBWIDGET
    var opts = {
        yaxis: {
            max: 1,
            min: 0
        },
        xaxis: {
            noTicks: Math.ceil($(this).width()/160),
            mode : 'time',
            'timeMode': 'local',
            //timeformat:  "%d/%m %H:%M"
            tickFormatter: lwp.dateTickFormatter
        },
        colors: zcolors,
        shadowSize: 0,
        legend: { show: false }

    };


    // selection is graphically enabled only if the graph is actives
    var passive = (conf.hasOwnProperty('controls') && conf['controls']===false);
    if (!passive)
    {
        opts['selection'] = { mode: 'x', color: "#000000" };
    }


    $(this).data('wgraphed', graphed);
/*    console.log("MULTILINE GRAPH: emerged following ");
    console.log(graphed);*/

    lwp.updateparams.call(this, 'renderopts', opts);
    lwp.updateparams.call(this, 'timescale', timescale);

/*    console.log("Updated params to ");
    console.log($(this).data('wparams'));*/

    var div_vis = lwp.run.call(this, 'getdiv_vis');
    var div_ctrl = lwp.run.call(this, 'getdiv_ctrl');

    div_vis.data('wparams', $(this).data('wparams'));
    div_ctrl.data('wparams', $(this).data('wparams'));
    div_vis.data('wconf', $(this).data('wconf'));
    div_ctrl.data('wconf', $(this).data('wconf'));
    lwp.run.call(div_vis, 'populate', [graphed]);
    lwp.run.call(div_ctrl, 'populate', [graphed]);

    lwp.toWlive.call(this, zsorted, 'zindex');

/*    console.log("EMERGE END TIME:");
    console.log(new Date());*/


};

compositegraph_multi.rescale = function (dataset, dataconf)
{
    // handles the normalisation of data according to how the various lines are combined

//     console.log("Normalizing dataset ",dataset," with dataconf ",dataconf);


    // setting the normalisation scales where applicable

    var datalimits = {};
    var unscaledlimits = {};
    var graphtype = [];

    var forcedscales = $.isArray(dataconf['fscales']) ? dataconf['fscales'] : [];
    var forced = [];
    for (var i = 0; i < forcedscales.length; i++)
    {
        try
        {
            if (typeof forcedscales[i][0] == 'number' && typeof forcedscales[i][1] == 'number')
            {
                datalimits[i] = {min: forcedscales[i][0], max: forcedscales[i][1]};
                if (forcedscales[i].length == 3 && forcedscales[i][2] != null)
                {
                    datalimits[i]['unit'] = forcedscales[i][2];
                }
                forced.push(i);
            }
        }
        catch (err)
        {
            // invalid scale, reverting to autoscale;
        }
    }

    // we are moving PER SERIES
    for (var i = 0; i < dataset[0].length; i++)
    {

        var scaled = true;
        graphtype[i] = 'line';

        try
        {
            var scaleid = dataconf['series'][i]['visNorm'];
            if (scaleid == -1)
            {
                scaled = false;
            }
            // possible lack of dataconf['series'][i] altogether has been already caught by trying to set scaleid
            if (dataconf['series'][i].hasOwnProperty('dataType'))
            {
                graphtype[i] = dataconf['series'][i]['dataType'];
            }
        }
        catch (err)
        {
            // if this series has no global scale to normalise on, we fall back to self-normalisation
            scaleid = -1;
            scaled = false;
        }
/*
        console.log("Rendering series "+i+" as "+graphtype);
*/
        var nomin = true;
        var nomax = true
        if (forced.indexOf(scaleid)!=-1)
        {
            nomin = isNaN(datalimits[scaleid]['min']);
            nomax = isNaN(datalimits[scaleid]['max']);
        }

        if (forced.indexOf(scaleid)==-1 || nomin || nomax)
        {
            var cmin = NaN;
            var cmax = NaN;
            var cunit = null;

            if (scaled)
            {
                if (datalimits.hasOwnProperty(scaleid))
                {
//                    console.log("Starting from existing scale "+scaleid, datalimits[scaleid]);
                    if (nomin)
                    {
                        cmin = datalimits[scaleid]['min'];
                    }

                    if (nomax)
                    {
                        cmax = datalimits[scaleid]['max'];
                    }

                    if (datalimits[scaleid].hasOwnProperty('unit'))
                    {
                        cunit = datalimits[scaleid]['unit'];
                    }

                }
                else
                {
//                    console.log("Creating new scale with scaleid "+scaleid);
                }
            }

            for (var t = 0; t < dataset.length; t++)
            {
                if (dataset[t][i] === null)
                {
                    continue;
                }

                if (cmin > dataset[t][i] || isNaN(cmin))
                {
                    cmin = dataset[t][i];
                }

                if (cmax < dataset[t][i] || isNaN(cmax))
                {
                    cmax = dataset[t][i];
                }
            }


            if (dataconf['series'][i].hasOwnProperty('unit') && dataconf['series'][i]['unit'] != null)
            {
                cunit = dataconf['series'][i]['unit'];
            }

            var unpadded = false;
            if (isNaN(cmax) && isNaN(cmin))
            {
                unpadded = true;
                cmax = 1;
                cmin = 0;
            }

            if (cmax == cmin)
            {
                unpadded = true;
                var coreval = cmax;
                if (coreval > 0)
                {
                    cmin = 0;
                    cmax = coreval*2;
                }
                else if (coreval < 0)
                {
                    cmax = 0;
                    cmin = coreval*2;
                }
                else if (coreval == 0)
                {
                    cmax = 1;
                    cmin = -1;
                }
            }

            // slight padding to make graph more readable
            var delta = cmax-cmin;
            var padding = !unpadded ? Math.abs(delta)*0.015 : 0;

            cmax = cmax > 0 ? cmax + padding : Math.min(cmax+padding, 0);
            cmin = cmin < 0 ? cmin - padding : Math.max(cmin-padding, 0);


            if (scaled)
            {
                datalimits[scaleid] = { min: cmin, max: cmax };
    /*            console.log("Scale "+scaleid);
                console.log(datalimits[scaleid]);*/
                if (cunit != null)
                {
                    datalimits[scaleid]['unit'] = cunit;
//                    console.log("Set scaled "+scaleid+" unit to "+cunit);
                }
            }
            else
            {
                unscaledlimits[i] = { min: cmin, max: cmax };
    /*            console.log("Unscaled series "+i);
                console.log(unscaledlimits[i]);*/
                if (cunit != null)
                {
                    unscaledlimits[i]['unit'] = cunit;
//                    console.log("Set unscaled "+scaleid+" unit to "+cunit);
                }
            }
        }
    }

    // applying normalisation scales (for now we consider them to be equally spaced) inside

//    console.log("Datalimits pre:", datalimits);

    var scaledset = [];
    var cscale = null;

    for (var i = 0; i < dataset[0].length; i++)
    {
        var cmin = NaN;
        var cmax = NaN;
        try
        {
            scaleid = dataconf['series'][i]['visNorm'];
            if ( scaleid == -1)
            {
                cscale = unscaledlimits[i];
/*                console.log("Series "+i+" normalised as unscaled");*/
            }
            else
            {
/*                console.log("Series "+i+" normalised to scaled "+scaleid);*/
                cscale = datalimits[scaleid];
            }
        }
        catch (err)
        {
            cscale = unscaledlimits[i];
        }

        if (cscale['min'] != NaN && cscale['max'])
        {
            cmin = cscale['min'];
            cmax = cscale['max'];
        }
        else
        {
            cmin = 0;
            cmax = 1;

        }
        var cdiff = cmax - cmin;

        for (var t = 0; t < dataset.length; t++)
        {
            if (!scaledset[t])
            {
                scaledset[t] = [];
            }

            if (graphtype[i] !== 'tick' && graphtype[i] !== 'bartick' )
            {
                scaledset[t][i] = dataset[t][i] !== null ? (dataset[t][i] - cmin) / cdiff : null;
            }
            else
            {
                // ticks are rendered as full height bars

                if (dataset[t][i] !== 0 && dataset[t][i] != null)
                {
                    scaledset[t][i] = 1;
                }
                else
                {
                    // replace with null so we show ONLY valid tick values
                    scaledset[t][i] = null;
                }
            }
        }
    }
/*

    console.log("RESCALE RESULTS (unscaledlimits + datalimits and wlivesaved)");
    console.log(unscaledlimits);
    console.log(datalimits);
    console.log(lwp.fromWlive.call(this, 'scales'));
*/


//    console.log("DATALIMITS FOR GRAPH: ",datalimits);

    // setting wlive scale only if not existing
    if (typeof lwp.fromWlive.call(this, 'scales') == 'undefined')
    {
        /*        console.log("Setting scales to wlive");*/
        lwp.toWlive.call(this, datalimits, 'scales');
    }


    return scaledset;

};



// visualization version of the timegraph widget
// can do local zoom and launch ajaxwave if required
timegraph_vis = {};

timegraph_vis.assemble = function ()
{

    var conf = $(this).data('wconf');

    console.log("DATA FOR TGVIS context ", $(this).data());

    var passive = (conf.hasOwnProperty('controls') && conf['controls'] === false);




    var div_graph = $("<div/>");
    div_graph.addClass(".jswlib_timegraph_canvas");
    div_graph.attr('id', timegraph_vis.getid_graph.call(this));
    lwp.toDisplay.call(this).append(div_graph);

    // creates the bindings for this widget
    //var container = lwp.toDisplay.call(this).get(0);
    var container = div_graph.get(0);

    // controls are enabled and respond only if the widget is not passive
    // replicates selection changes from vis to ctrl
    Flotr.EventAdapter.observe (container, 'flotr:select', function(evt) {

        var widget = $(container).parents('.jswlib_widget');
        var master = $(widget).parents('.jswlib_widget');
        var controller = lwp.run.call(master, 'getdiv_ctrl');
        var controllerdisplay = lwp.toDisplay.call(controller);

        controller.data('wrendered').selection.setSelection(evt, true);
        $(widget).find(".jswlib_timegraph_timeselector").prop('selectedIndex', 0);

        Flotr.EventAdapter.fire(controllerdisplay.get(0), 'flotr:select', evt);
    });

};

timegraph_vis.getid_graph = function ()
{
    return $(this).attr('id')+"_graph";
};

timegraph_vis.getdiv_graph = function ()
{
    return lwp.toDisplay.call(this).find("#"+timegraph_vis.getid_graph.call(this));
};

timegraph_vis.wrender = function ()
{
//    console.log("Rendering tgv graph pure");

//    console.log("Rendering vis for tg subwidget ",this);

    var widget = lwp.toMaster.call(this);
    var timescale = $(widget).data('wparams')['timescale'];

    var conf = $(this).data('wconf');
    var passive = (conf.hasOwnProperty('controls') && conf['controls'] === false);
    var graphcontainer = timegraph_vis.getdiv_graph.call(this);

    if (passive)
    {
//        console.log("Unbinding flotr:select event");
        // unbinding flotr events
        //var container = lwp.toDisplay.call(this).get(0);
        Flotr.EventAdapter.stopObserving(graphcontainer, 'flotr:select');
    }

//    console.log("setting zoom for tg");

    var zoomfrom = lwp.fromWlive.call(widget, 'zoomfrom');
    var zoomto = lwp.fromWlive.call(widget, 'zoomto');
    if (typeof zoomfrom == 'undefined' || typeof zoomto == 'undefined' || zoomfrom == null || zoomto == null)
    {
        zoomfrom = timescale[0];
        zoomto = timescale[1];


    }

    console.log("Showing "+$(widget).attr('id')+" from ",new Date(zoomfrom)," to ",new Date(zoomto));

    var renderparams = $(this).data('wparams')['renderopts'];


    // VISUALISATION ONLY, NOT FOR CONTROLLER
    renderparams['xaxis'] = {
        noTicks: Math.ceil($(this).width()/160),
        min: zoomfrom,
        max: zoomto,
        mode: 'time',
        timeMode: 'local',
        //timeformat:  "%d/%m %H:%M"
        tickFormatter: lwp.dateTickFormatter
    };

    var masterWidgetType= $(widget).data('wtype');
    if (masterWidgetType != 'compositegraph_band')
    {
        renderparams.yaxis.showLabels = true;
    }

    if (!passive)
    {
        renderparams['crosshair'] = {
            mode:   'xy',
            color:  '#666666'
        };
    }

//        console.log("Graph container ",timegraph_vis.getdiv_graph.call(this), "element ", document.getElementById(timegraph_vis.getid_graph.call(this)));

    if (widget.data('wtype') == 'compositegraph_multi')
    {
        var livescale = lwp.fromWlive.call(widget, 'scales');
//        console.log("LIVE SCALE FOR TG",livescale);
        var scalekeys = Object.keys(livescale).sort();
//        console.log("scale keys",scalekeys);
        var mainscale = livescale[scalekeys[0]];
        var scalemin = mainscale['min'];
        var scalemax = mainscale['max'];
        var unit = mainscale.hasOwnProperty('unit') ? mainscale['unit'] : null;
        var scaledelta = scalemax - scalemin;

        var vertspace = lwp.toDisplay.call(this).height();
        var nbands = Math.round(vertspace/32);

        var array_scale = [];
        //var magnitude = lwp.getRoundingRange(scalemin, scalemax, 2);

        //array_scale.push([0, Number(scalemin).toFixed(magnitude)]);
        //array_scale.push([1, Number(scalemax).toFixed(magnitude)]);
        array_scale.push([0, lwp.renderNumeric(Number(scalemin), unit)]);
        array_scale.push([1, lwp.renderNumeric(Number(scalemax), unit)]);

        for (var b = 1; b < nbands; b++)
        {
            var tickpos = (1 / nbands) * b;
            var tickval = lwp.renderNumeric(Number(scalemin+scaledelta*tickpos), unit);
//            console.log("TICK POS "+tickpos+" on max "+scalemax+": "+tickval);
            //var tickval = Number(scalemax*tickpos).toFixed(magnitude);
            array_scale.push([tickpos, tickval]);
        }
        renderparams.yaxis.ticks = array_scale;
    }

//    console.log("controls rendering");
    // draws the graph and assigns it to wrendered
    if (!passive)
    {
        if (timegraph_vis.getdiv_toolbar.call(this).length==0)
        {
            lwp.run.call(this, 'rendercontrols');
        }
        var toolbar_div = timegraph_vis.getdiv_toolbar.call(this);
        graphcontainer.height(lwp.getWidgetSelf.call(this).height()-toolbar_div.height());

    }
    else
    {
        graphcontainer.height(lwp.getWidgetSelf.call(this).height());
    }

//    console.log("height of graph container: ",graphcontainer.height());


    console.log("RENDER PARAMS: ",renderparams);

//    $(this).data('wrendered', Flotr.draw(document.getElementById(lwp.getDisplayId.call(this)), $(this).data('wgraphed'), renderparams));
    $(this).data('wrendered', Flotr.draw(document.getElementById(timegraph_vis.getid_graph.call(this)), $(this).data('wgraphed'), renderparams));

    console.log("Rendered object: ",$(this).data('wrendered'));


    //    console.log("Rendered tgv graph pure");
};

timegraph_vis.getdiv_toolbar = function ()
{
    return lwp.getWidgetSelf.call(this).find(".jswlib_timegraph_toolbar");
};

timegraph_vis.rendercontrols = function ()
{

    var widget = lwp.toMaster.call(this);
    var master = lwp.getWidgetSelf.call(this);
    var masterid = master.attr('id');

    var timescale = $(this).data('wparams')['timescale'];
    var bottom = timescale[0];
    var top = timescale[1];

    // draws additional UI controls

    var ajaxloader = null;
    var rawstamps;
    console.log("Widget data reference", $(widget), $(widget).data());
    if ($(widget).data('wdata')['renderopts'].hasOwnProperty('ajaxrange'))
    {

        rawstamps = [$(widget).data('wdata')['renderopts']['ajaxrange'][0], $(widget).data('wdata')['renderopts']['ajaxrange'][1]];
        rawstamps[0]=rawstamps[0]*1000;
        rawstamps[1]=rawstamps[1]*1000;

        var dateFrom = new Date (rawstamps[0]);
        var dateTo = new Date (rawstamps[1]);

        var pickfrom = $("<input type='text'/>");

        pickfrom.attr('id', masterid+'_subrange_from');
        pickfrom.addClass("jswlib_daterange_from");
        pickfrom.datepicker({
            minDate: dateFrom,
            maxDate: dateTo,
            onClose: function( selectedDate ) {
                var dest = lwp.getWidgetSelf.call(this).find(".jswlib_daterange_to");
//                console.log("Selected date on picker FROM ",$(this));
                $(dest).datepicker( "option", "minDate", selectedDate );
            }
        });

        var pickto = $("<input type='text'/>");
        pickto.addClass("jswlib_daterange_to");
        pickto.attr('id', masterid+'_subrange_to');
        pickto.datepicker({
            minDate: dateFrom,
            maxDate: dateTo,
            onClose: function( selectedDate ) {
                var dest = lwp.getWidgetSelf.call(this).find(".jswlib_daterange_from");
//                console.log("Selected date on picker TO ",$(this));
                $(dest).datepicker( "option", "maxDate", selectedDate );
            }
        });

        console.log("Built datepickers");

        var btn_ajaxload = $("<span/>");
        btn_ajaxload.addClass("jswlib_btncontrol");
        btn_ajaxload.addClass("jswlib_flag_notextsel");
        btn_ajaxload.append('Update');
        // todo: placeholder link to real function
        btn_ajaxload.click(function () { console.log("WIP disabled!")});

        ajaxloader = $("<span/>");
        ajaxloader.addClass("jswlib_daterange_selector");
        ajaxloader.css('margin-left', '2em');
        ajaxloader.append("From ");
        ajaxloader.append(pickfrom);
        ajaxloader.append(" to ");
        ajaxloader.append(pickto);
        ajaxloader.append(btn_ajaxload);

    }
    console.log("Setting datepicker over ",rawstamps);




    //console.log("Rendering timegraphpassive controls with presets ");
    //console.log(presets);

    var toolbar = $("<div/>");
    toolbar.addClass("jswlib_timegraph_toolbar");


    var destination = master.find(".jswlib_timegraph_vis");
/*    console.log("Max "+master.data('wrendered').axes.x.max+" Top "+top+" Min "+ master.data('wrendered').axes.x.min);*/

    //var cinterval = master.data('wrendered').axes.x.max - master.data('wrendered').axes.x.min;

/*
    console.log("Current interval = "+cinterval+", on "+(top-bottom));
*/

    var zoomer = $(' ' +
   //     '<input class="jswlib_timegraph_control jswlib_timegraph_preset" id="'+masterid+'_preset_full" type="button" value="*">' +
        '<input class="jswlib_timegraph_control jswlib_timegraph_zoom jswlib_timegraph_zoom_out" id="'+masterid+'_subzoom_in" type="button" value="-">' +
        '<input class="jswlib_timegraph_control jswlib_timegraph_zoom jswlib_timegraph_zoom_in" id="'+masterid+'_subzoom_out" type="button" value="+">'+
        '<input class="jswlib_timegraph_control jswlib_timegraph_zoom jswlib_timegraph_scroll_back" id="'+masterid+'_subscroll_back" type="button" value="&lt;&lt;">'+
    '<input class="jswlib_timegraph_control jswlib_timegraph_zoom jswlib_timegraph_scroll_fwd" id="'+masterid+'_subscroll_fwd" type="button" value="&gt;&gt;">'
        );



    var maxrange = top-bottom;
//    console.log("MAXRANGE: "+maxrange+" ("+top+" to "+bottom+")");
    var timespans = lwp.getTimespans();
//    console.log(timespans);
    timespans.unshift({label: '*', span: 0});
//    console.log(timespans);
    for (var i=timespans.length-1; i >= 0; i--)
    {
        var spanlabel = timespans[i].label;
        var spanlen = timespans[i].span;
        if (spanlen <= maxrange)
        {
            var cbutton = $('<input type="button"/>');
            cbutton.addClass('jswlib_timegraph_control');
            cbutton.addClass('jswlib_timegraph_preset');
            cbutton.prop('id', masterid+'_preset_'+spanlabel);
            cbutton.data('span', spanlen);
            cbutton.val(spanlabel);
//            console.log(cbutton);
            toolbar.append(cbutton);
        }
    }

//    console.log(toolbar);

    toolbar.find(".jswlib_timegraph_preset").bind('click', function(evt)
    {
        // zoomed range
        var widget = lwp.toMaster.call($(this));
        var zoomrange = $(this).data('span');
        var iszoomed = zoomrange != 0;
//        console.log("Prelaunching preset zoom");
        lwp.run.call(widget, 'livezoom', ['preset', {range: zoomrange, zoomed: iszoomed}]);
    });

    toolbar.append(zoomer);

    toolbar.append(ajaxloader);

    toolbar.find(".jswlib_timegraph_zoom_out").bind('click', function(evt)
    {
        // zoomed range
        var widget = lwp.toMaster.call($(this));
//        console.log("Prelaunching button zoom out");
        lwp.run.call(widget, 'livezoom', ['zoomout', {}]);
    });

    toolbar.find(".jswlib_timegraph_zoom_in").bind('click', function(evt)
    {
        // zoomed range
        var widget = lwp.toMaster.call($(this));
//        console.log("Prelaunching button zoom in");
        lwp.run.call(widget, 'livezoom', ['zoomin', {}]);
    });

    toolbar.find(".jswlib_timegraph_scroll_back").bind('click', function(evt)
    {
        // zoomed range
        var widget = lwp.toMaster.call($(this));
//        console.log("Prelaunching button scroll back");
        lwp.run.call(widget, 'livezoom', ['scrollback', {}]);
    });

    toolbar.find(".jswlib_timegraph_scroll_fwd").bind('click', function(evt)
    {
        // zoomed range
        var widget = lwp.toMaster.call($(this));
//        console.log("Prelaunching button scroll forward");
        lwp.run.call(widget, 'livezoom', ['scrollfwd', {}]);
    });



    lwp.toDisplay.call(this).prepend(toolbar);
};



timegraph_vis.renderwait = function ()
{

//    console.log("Rendering tgv wait overlay");
    // renders the waiting overlay

    var displaydiv = lwp.toDisplay.call(this);

    var subdiv = $("<div class=jswlib_widgetoverlay/>");

    displaydiv.append(subdiv);

    var divheight = subdiv.height();
    var divwidth = subdiv.width();
    var bannersize = divheight < divwidth ? divheight : divwidth;

    var overlaybanner = $('<img src="'+lwp.asset.icon_wait+'">');
    overlaybanner.css('width', bannersize);
    overlaybanner.css('height', bannersize);

    subdiv.append(overlaybanner);

//    console.log("Rendered tgv wait overlay");

};

timegraph_vis.emerge = function ()
{
    // no need to alter the entering data, for now (takes from compositegraph
/*    console.log("Emerging wdata to wgraphed on timegraph_vis");
    console.log($(this).data('wdata'));*/
    $(this).data('wgraphed', $(this).data('wdata'));
};


timegraph_ctrl = {};


timegraph_ctrl.assemble = function ()
{

    var container = lwp.toDisplay.call(this).get(0);
    // mapping the click action to event unzoom
    Flotr.EventAdapter.observe (container , 'flotr:click', function(evt)
    {
        var widget = lwp.toMaster.call($(container));
        lwp.run.call(widget, 'livezoom', ['unzoom']);
    });

    Flotr.EventAdapter.observe (container, 'flotr:select', function(evt)
    {
        var widget = lwp.toMaster.call($(container));
        lwp.run.call(widget, 'livezoom', ['graphselect', evt]);
    });

};




timegraph_ctrl.renderzoom = function ()
{

//    console.log("Rendering tgc zoom selection");

    var widget = lwp.toMaster.call(this);
    var zoomed = false;
    try
    {
        zoomed = lwp.fromWlive.call(widget, 'zoomed')===true;
    }
    catch (err)
    {
        // nothing, we keep the default value
    }

    if (zoomed)
    {
        // drawing selection
        var evt = {
            x1: lwp.fromWlive.call(widget, 'zoomfrom'),
            x2: lwp.fromWlive.call(widget, 'zoomto')
        };
        // true avoids propagating the selection event so we don't enter a loop
        $(this).data('wrendered').selection.setSelection(evt, true);
    }
    else
    {
        // erasing selection
        $(this).data('wrendered').selection.clearSelection();
    }


//    console.log("Rendered tgc zoom selection");

};


timegraph_ctrl.resetzoom = function ()
{

/*
    console.log("Requested zoom reset from control");
*/

    var master = $(this).parents('.jswlib_widget');
    var slave = lwp.run.call(master, 'getdiv_vis');
    var opts = slave.data('wparams')['renderopts'];
    opts['xaxis'] = {
        noTicks: Math.ceil($(this).width()/160),
        mode: 'time',
        'timeMode': 'local',
        //timeformat:  "%d/%m %H:%M"
        tickFormatter: lwp.dateTickFormatter

    };
    lwp.render.call(slave);
    opts = $(this).data('wparams')['renderopts'];
    opts['xaxis'] = {
        noTicks: Math.ceil($(this).width()/160),
        mode: 'time',
        'timeMode': 'local',
        //timeformat:  "%d/%m %H:%M"
        tickFormatter: lwp.dateTickFormatter

    };
    lwp.render.call(this);
    lwp.flushwave.call(master);
};


timegraph_ctrl.smartzoom = function (timespan)
{

    var timescale = $(this).data('wparams')['timescale'];
    var evt = {};
    if (timespan!="" && !isNaN(parseInt(timespan)))
    {
        evt = {x1: timescale[1]-timespan, x2: timescale[1]};
/*        console.log("Smartzoomed to ");
        console.log(evt);*/
        $(this).data('wrendered').selection.setSelection(evt, true);

        Flotr.EventAdapter.fire(lwp.toDisplay.call(this).get(0), 'flotr:select', evt);
    }
    else
    {
        lwp.run.call(this, 'resetzoom');
    }
};

timegraph_ctrl.emerge = function ()
{
    // no need to alter the entering data, for now (takes from compositegraph
    $(this).data('wgraphed', $(this).data('wdata'));
};

timegraph_ctrl.wrender = function ()
{
/*    console.log("Wrender function for timegraph_ctrl");
    console.log($(this).data());*/

    var renderparams = $(this).data('wparams')['renderopts'];
    renderparams['legend'] = { 'show': false };

    var widget = lwp.toMaster.call(this);



    if (widget.data('wtype') == 'compositegraph_multi')
    {
        var livescale = lwp.fromWlive.call(widget, 'scales');
        var scalekeys = Object.keys(livescale).sort();
        var mainscale = livescale[scalekeys[0]];
        var scalemin = mainscale['min'];
        var scalemax = mainscale['max'];
        var unit = mainscale.hasOwnProperty('unit') ? mainscale['unit'] : null;

        var scaledelta = scalemax-scalemin;

        var vertspace = lwp.toDisplay.call(this).height();
        var nbands = Math.round(vertspace/32);

        var array_scale = [];
        var magnitude = lwp.getRoundingRange(scalemin, scalemax, 2);

//        array_scale.push([0, Number(scalemin).toFixed(magnitude)]);
//        array_scale.push([1, Number(scalemax).toFixed(magnitude)]);
        array_scale.push([0, lwp.renderNumeric(Number(scalemin), unit)]);
        array_scale.push([1, lwp.renderNumeric(Number(scalemax), unit)]);


        for (var b = 1; b < nbands; b++)
        {
            var tickpos = (1 / nbands) * b;
            //var tickval = Number(scalemax*tickpos).toFixed(magnitude);
            var tickval = lwp.renderNumeric(Number(scalemin+scaledelta*tickpos), unit);
            array_scale.push([tickpos, tickval]);
        }
        renderparams.yaxis.ticks = array_scale;
    }


    $(this).data('wrendered', Flotr.draw(document.getElementById(lwp.getDisplayId.call(this)), $(this).data('wgraphed'), renderparams));
};



// sparkline based linegraph

sparklinegraph = {};

sparklinegraph.rescale = compositegraph_multi.rescale;

sparklinegraph.emerge = function ()
{

    /*
     evolved from multilinegraph; main differences are:
     - rendering options
     - colors shifted in rendering options section
     - graphtype is ignored, if present
     - no legend or aggregated data
     */


//    console.log("EMERGE START TIME FOR SPARKLINEGRAPH:");
//    console.log(new Date());
    var graphed = [];
    var timescale = lwp.toJavascriptTimeStamp($(this).data('wdata')['values'][0]);
    lwp.updateparams.call(this, 'timescale', timescale);
    var labels = $(this).data('wdata')['values'][1];
    var rawdataset = $(this).data('wdata')['values'][2];
    var dataconf = $(this).data('wdata')['renderopts'];

//    console.log("Sparklinegraph: prepared for rescale");

    var dataset = lwp.run.call(this, 'rescale', [rawdataset, dataconf]);
    // TEMPORARY TEST STUFF, to be pulled out
    // defining series by type: line, area, tick, each with a style set and filter?

//    console.log("Rescaled dataset");
    //console.log(dataset);

//    console.log("Getting colors");
    var precolors = lwp.getColors.call(this);
//    console.log("Received colors");
//    console.log(precolors);

    // building ordered list
    var zsorted = [];
    try
    {
        zsorted = $(this).data('wdata')['renderopts']['zlist'];
    }
    catch (err)
    {
        zsorted = [];
    }

//    console.log("Adding left labels to zsorted");
//    console.log(labels);
    if (typeof zsorted == 'undefined')
    {
        zsorted = [];
    }
    // adding non-specified series
    for (var i = 0; i < labels.length; i++)
    {
        if (zsorted.indexOf(i) == -1)
        {
            zsorted.push(i);
        }
    }
//    console.log("Added labels");

//    console.log("Created zsorting list");
//    console.log(zsorted);

    // creates aggregate data for use with the graphedlegend widget
    var zcolors = [];
    for (var z = zsorted.length; z > 0; z--)
    {

        var i = zsorted[z-1];

        zcolors.push(precolors[i%precolors.length]);
//        console.log("Rendering "+i+" in order "+z);

        var cseries = [];
        for (var t = 0; t < dataset.length; t++)
        {
            var ctime = timescale[0] + t * timescale[2];
            var cvalue = dataset[t][i];
            cseries.push([ctime, cvalue]);
        }

        // line used as catchall
        graphed.push({
            data:   cseries,
            label:  labels[i],
            options: {
                type: 'line',
                lineColor: zcolors[z]
            }
        });

    }

    $(this).data('wgraphed', graphed);
//    console.log("SPARKLINE GRAPH: emerged following ");
//    console.log(graphed);

    //lwp.updateparams.call(this, 'renderopts', opts);
    lwp.updateparams.call(this, 'timescale', timescale);

//    console.log("Updated params to ");
//    console.log($(this).data('wparams'));

};

sparklinegraph.wrender = function ()
{
    var dataset = $(this).data('wgraphed');
//    console.log("Spark rendering dataset");
    //console.log(dataset);

    var sparkcontainer = $("<span/>");
    lwp.toDisplay.call(this).append(sparkcontainer);

    for (var i = 0; i < dataset.length; i++)
    {
        var cseries = dataset[i].data;
        var clabel = dataset[i].label;
        var copts = dataset[i].options;
        copts.drawNormalOnTop = true;
        copts.width = lwp.toDisplay.call(this).innerWidth()+"px";

//        console.log("Options for series "+clabel);
//        console.log(copts);
        $(sparkcontainer).sparkline(cseries, copts);
    }

};




/*
    WIDGET registration: rebuilds the WidgetTree
 */
jswlib['compositegraph_band'] = compositegraph_band;
jswlib['compositegraph_multi'] = compositegraph_multi;
jswlib['sparklinegraph'] = sparklinegraph;
jswlib['timegraph_vis'] = timegraph_vis;
jswlib['timegraph_ctrl'] = timegraph_ctrl;
buildWidgetsTree();