/**
 * Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
 * Created: 10/10/13, 4:01 PM
 * Author: Antonio Vaccarino
 */


// QuickGrid

/*
FORMAT =
{

	categories: [		// columns
		{
			symbol: string, fontawesome icon name, // optional
			text: string, // optional
			image: string, URL // optional, overrides symbol
			styler:
			{
				relevant: boolean, // applies predefined style elements to whole category/column
				color: string (color hexstring), // font color for all cells in category/column
				background: string (color hexstring), // background for all cells in category/column
			}
			lstyle: {} // same as styler but only applies to category header cell
		}
	],
	groups: [		// rows
		{
			symbol: string, fontawesome icon name, // optional
			text: string, // optional
			image: string, URL // optional, overrides symbol
			styler:
			{
				relevant: boolean, // applies predefined style elements to whole group/row
				color: string (color hexstring), // font color for all cells in group/row
				background: string (color hexstring), // background for all cells in group/row
			}
			lstyle: {} // same as styler but only applies to group header cell
		}
	],
	values: [
		[
			string/object for rendertextcell // if object, can use lstyle to define cell-specific styling
		]
	]

}

NOTES:
- Categories headers are displayed as either (first that applies): image+text, icon+title, title
- Groups headers are displayed as either (first that applies): image, icon+symbol, icon
- Stylers are applied as follows: lstyle (cell-specific) first, if no lstyle applies then the category styler, if missing the group styler.

 */


/*
EXAMPLE
     var quickgriddata = {

         categories: [
             {
                 symbol: "icon-warning-sign",
                 text: 'critical',
                 styler: {
                     relevant: true,
                     color: "#ffffff",
                     background: "#220000"
                 }

             },
             {
                 text: 'high'
             },
             {
                 image: "../imgs/lightbulbjabber.png",
                 text: 'average'
             },
             {
                 symbol: "icon-time",
 //                text: 'low'
             }
         ],
         groups: [
             {
                 image: "../imgs/My-Network-Places-icon.png",
                 text:   "networked stuff"
             },
             {
                 symbol: "icon-laptop"
             },
             {
                 symbol: "icon-windows",
                 text: "bill's machines"
             }
         ],
         values: [
             [5,4,31,8],
             [{value: 3, details: [
         "macchina1", "macchina2", "macchina3"
     ], ref: "#", action: ""},6,1,2],
             [5,22,11,87]
         ]

     };
*/

qgrid = {};

qgrid.assemble = function ()
{
	var div_display = lwp.toDisplay.call(this);
	div_display.css({
		"position": "relative"
	});
	var div_vis = $("<div/>");
	div_vis.addClass("jswlib_qgrid_table");
	div_vis.addClass("jswlib_flag_notextsel");
	div_vis.attr('id', qgrid.getid_vis.call(this));
	div_vis.css({
//		"position": "absolute",
		"width":  div_display.width()
//		"height": div_display.height()
	});
	div_display.append(div_vis);

	var gridW = new qgrid.Widget (div_vis);
	lwp.toWlive.call(this, gridW, 'gridW');
};

qgrid.getid_vis = function ()
{
	return lwp.toDisplay.call(this).attr('id')+"_grid";
};

qgrid.getdiv_vis = function ()
{
	return $("#"+qgrid.getid_vis.call(this));
};

qgrid.emerge = function ()
{
	console.log("Emerging");
	var rawdata = $(this).data('wdata');
	try
	{
		var data = typeof(rawdata) == "string" ? JSON.parse(rawdata) : rawdata;
	}
	catch (err)
	{
		console.trace();
		console.log ("Wrong data format, must be JSON dict: ",err);
		throw ("Wrong data format for matrix");
	}
	console.log("Rawdata for qgrid: ", data);

	var num_categories = data['categories'].length;
	var num_groups = data['groups'].length;
	var matrix_rows = data['values'].length;
	if (matrix_rows != num_groups)
	{
		throw ("matrix rows/groups mismatch")
	}
	for (var row = 0; row < num_groups; row++)
	{
		var matrix_cols = data['values'][row].length;
		if (matrix_cols != num_categories)
		{
			throw ("matrix cols/categories mismatch");
		}
	}
	console.log("lengths match ok");



	var gridW = lwp.fromWlive.call(this, 'gridW');

	gridW.loadMatrix(data);
};

qgrid.wrender = function ()
{
	var gridW = lwp.fromWlive.call(this, 'gridW');
	gridW.render();
};

qgrid.Widget = function Widget (div)
{
	// graphic grid structure
	this.div = div;
	div.empty();

	this.categories = [];
	this.groups = [];

	this.styles = {
		categories: [],
		groups: []
	};

};

qgrid.Widget.prototype.loadMatrix = function (data)
{
	this.categories = data['categories'];
	this.groups = data['groups'];
	this.matrix = new qgrid.Matrix(data['values']);

	var i, cstyle, cset;
	for (i = 0; i < this.categories.length; i++)
	{
		cset = this.categories[i];
		cstyle = {
			relevance: false,
			colors: null
		};
		if (cset.hasOwnProperty('styler'))
		{
			if ((cset['styler']['color'] || cset['styler']['background']))
			{
				cstyle.colors = {};
				if (cset['styler']['color'])
				{
					cstyle.colors.font = cset['styler']['color'];
				}
				if (cset['styler']['background'])
				{
					cstyle.colors.background = cset['styler']['background'];
				}
			}
			cstyle.relevance = cset['styler']['relevant'] === true;
		}
		this.styles.categories[i] = cstyle;
	}
	for (i = 0; i < this.groups.length; i++)
	{
		cset = this.groups[i];
		cstyle = {
			relevance: false,
			colors: null
		};
		if (cset.hasOwnProperty('styler'))
		{
			if (cset['styler']['color'] || cset['styler']['background'])
			{
				cstyle.colors = {};
				if (cset['styler']['color'])
				{
					cstyle.colors.font = cset['styler']['color'];
				}
				if (cset['styler']['background'])
				{
					cstyle.colors.background = cset['styler']['background'];
				}
			}
			cstyle.relevance = cset['styler']['relevant'] === true;
		}
		this.styles.groups[i] = cstyle;
	}
	console.log("STYLES: ",this.styles);
};

qgrid.Widget.prototype.renderCell = function(celldata)
{
	var div = $("<div/>");
	var content = celldata.value;
	var span;
	if (celldata['ref'])
	{
		span = $("<a/>");
		span.attr("href", celldata['ref']);
		span.append(content);
	}
	else
	{
		span = $("<span/>");
		span.append(content);
	}

	var action = celldata['action'];
//    console.log(celldata, action);
	if (action)
	{
		span.attr('onclick', action);
		span.addClass("jswlib_qgrid_clickable");
	}

	div.append(span);
	return div;
};

qgrid.Widget.prototype.render = function ()
{
	var r, c, symboltag, imagetag, lstyle;
	this.div.empty();

	var matrix = this.matrix;
	var cols = matrix.cols;
	var rows = matrix.rows;

	var corner = $("<div/>");
	corner.addClass("jswlib_qgrid_corner");

	var headers = $("<div/>");
	headers.addClass("jswlib_qgrid_headers");

	headers.append(corner);
	for (c = 0; c < cols; c++)
	{
		var header = $("<div/>");
		header.addClass("jswlib_qgrid_category");

		var ctext = this.categories[c]['text'];
		var cimage = this.categories[c]['image'];
		var csymbol = this.categories[c]['symbol'];

		lstyle = this.categories[c]['lstyle'];
		if (lstyle)
		{
			lstyle['background'] && header.css("background-color", lstyle['background']);
			lstyle['color'] && header.css("color", lstyle['color']);
			lstyle['relevant']===true && header.addClass("jswlib_qgrid_relevant");
		}

		if (cimage)
		{
			imagetag = $("<img/>");
			imagetag.addClass("jswlib_qgrid_catimage");
			imagetag.attr('src', cimage);
			header.append(imagetag);
		}
		else if (csymbol)
		{
			symboltag = $("<i/>");
			symboltag.addClass(csymbol);
			if (!gtext)
			{
				symboltag.addClass("jswlib_qgrid_catsymbol");
			}
			header.append(symboltag);
		}

		if (ctext)
		{
			var textdiv = $("<div/>");
			textdiv.addClass("jswlib_qgrid_header");
			textdiv.append(ctext);
			header.append(textdiv);
		}
		else
		{
			header.css({
				"vertical-align": "top"
			});
		}

		headers.append(header);

	}

	this.div.append(headers);
	for (r = 0; r < rows; r++)
	{
		var grouprow = $("<div/>");
		grouprow.addClass("jswlib_qgrid_group");

		var group = $("<div/>");
		group.addClass("jswlib_qgrid_grouptag");

		var gtext = this.groups[r]['text'];
		var gimage = this.groups[r]['image'];
		var gsymbol = this.groups[r]['symbol'];

		if (gimage)
		{
			imagetag = $("<img/>");
			imagetag.addClass("jswlib_qgrid_groupimage");
			imagetag.attr('src', gimage);
			group.append(imagetag);
		}
		else
		{
			var divtext = $("<div/>");
			divtext.addClass("jswlib_qgrid_grouptext");

			if (gtext)
			{
				divtext.append(gtext);
			}

			if (gsymbol)
			{
				symboltag = $("<i/>");
				symboltag.addClass(gsymbol);
				if (!gtext)
				{
					symboltag.addClass("jswlib_qgrid_groupsymbol");
				}
				divtext.prepend(symboltag);
			}
			group.append(divtext);
		}

		lstyle = this.groups[r]['lstyle'];
		if (lstyle)
		{
			lstyle['background'] && group.css("background-color", lstyle['background']);
			lstyle['color'] && group.css("color", lstyle['color']);
			lstyle['relevant']===true && group.addClass("jswlib_qgrid_relevant");
		}


		grouprow.append(group);

		// CELL RENDERING CODE
		for (c = 0; c < cols; c++)
		{
			// actual cell values rendering
			var celldata = matrix.get(r,c);
//            console.log("Rendering cell ",celldata);
			// check for local styling
			lstyle = celldata['lstyle'];


			var cell = this.renderCell(celldata);

			cell.addClass("jswlib_qgrid_cell");
			if (r == 0)
			{
				cell.addClass("jswlib_qgrid_nobordertop");
			}
			if (c == 0)
			{
				cell.addClass("jswlib_qgrid_noborderleft");
			}

			if (lstyle==null)
			{
				if (this.styles.categories[c]['relevance']===true || this.styles.groups[r]['relevance']===true)
				{
					cell.addClass("jswlib_qgrid_relevant");
				}
				var catstyle = this.styles.categories[c]['colors'];
				var grstyle = this.styles.groups[r]['colors'];
				if (catstyle)
				{
					catstyle['background'] && cell.css("background-color", catstyle['background']);
					catstyle['font'] && cell.css("color", catstyle['font']);
				}
				else
				{
					if (grstyle)
					{
						grstyle['background'] && cell.css("background-color", grstyle['background']);
						grstyle['font'] && cell.css("color", grstyle['font']);
					}
				}
			}
			else
			{
				lstyle['background'] && cell.css("background-color", lstyle['background']);
				lstyle['color'] && cell.css("color", lstyle['color']);
				lstyle['relevant']===true && cell.addClass("jswlib_qgrid_relevant");
			}



			grouprow.append(cell);
		}

		this.div.append(grouprow);
	}

	var cimages = this.div.find(".jswlib_qgrid_groupimage");
	var gimages = this.div.find(".jswlib_qgrid_catimage");

	var cells = this.div.find(".jswlib_qgrid_cell");
	console.log("checking "+cells.length+" cells ");
	var cwidth = 0;
	var cheight = 0;
	var ccell;
	for (c = 0; c < cells.length; c++)
	{
		ccell = $(cells[c]);
		cwidth = Math.max(ccell.width(), cwidth);
		cheight = Math.max(ccell.height(), cheight);
	}
	for (c = 0; c < cells.length; c++)
	{
		ccell = $(cells[c]);
		ccell.width(cwidth);
		ccell.height(cheight);
	}

	console.log("rows with height ",cheight);

	var symbols;
	// first we do a run through the groups
	gimages.css({
			"max-height": cheight
		});
	symbols = this.div.find(".jswlib_qgrid_groupsymbol");
	symbols.css({
			"font-size": cheight+"px"
		});

	// run through categories, keeping separate from group for future differentiation
	cimages.css({
			"max-height": cheight
		});
	cimages.attr('height', cheight);
	symbols = this.div.find(".jswlib_qgrid_catsymbol");
	symbols.css({
			"font-size": cheight+"px"
	});

	var gtags;
	gtags = this.div.find(".jswlib_qgrid_group");
	gtags.css({
		"height": cheight,
		"max-height": cheight
	});
	gtags = this.div.find(".jswlib_qgrid_grouptext");
	for (var i = 0; i < gtags.length; i++)
	{
		var tag = $(gtags[i]);
		var baseheight = tag.height();
		var vmargin = Math.max(0, (cheight-baseheight)/2);
		tag.css({
			"max-height": cheight,
			"padding-top": vmargin,
			"padding-bottom": vmargin
		});
	}



};

qgrid.Matrix = function Matrix (source)
{
	// matrix representing the actual data
	// already verified in the emerge function of the widget

	console.log("Creating matrix from source ",source);

	this.matrix = [];
	this.rows = source.length;
	try
	{
		this.cols = source[this.rows-1].length;
	}
	catch (err)
	{
		console.error("ZERO length for categories");
		// zero length
		this.cols = 0;
	}


	console.log("calculated rows "+this.rows+" cols "+this.cols+", filling ");
	for (var r = 0; r < this.rows; r++)
	{
		var row = [];

		for (var c = 0; c < this.cols; c++)
		{
			var cell = {
				value: null,
				details: null,
				ref: null,
				action: null,
				lstyle: null
			};

			var candidate = source[r][c];
			if (typeof(candidate)=='object')
			{
				(candidate.hasOwnProperty('value')) && (cell.value = candidate.value);
				(candidate.hasOwnProperty('details')) && (cell.details = candidate.details);
				(candidate.hasOwnProperty('ref')) && (cell.ref = candidate.ref);
				(candidate.hasOwnProperty('action')) && (cell.action = candidate.action);
				(candidate.hasOwnProperty('lstyle')) && (cell.lstyle = candidate.lstyle);
			}
			else
			{
				cell.value = candidate.toString();
			}
			row.push(cell);
		}
		this.matrix.push(row);
	}
	console.log("Filled matrix ",this.matrix);
};

qgrid.Matrix.prototype.get  = function (row, col)
{
	return this.matrix[row][col];
};

/*************************************
SPARKGRAPH WIDGET
**************************************/


/*
FORMAT =
{
	notip: boolean,
	series: [
		{
			label: str, optional
			color: str (#hex),
			shade: boolean (area graph or not)
			values: []
		}
	],
	scale: {
		min: number,
		max: number,
	},
	thresh: {
		normal: boolean (if amount should be used compared to scale, implies the minmax being between 0 and 1, otherwise does not validate)
		min: number,
		max: number,
		color:  str (#hex)
	}
}
*/

sparkgraph = {};

sparkgraph.assemble = function ()
{
	var div_display = lwp.toDisplay.call(this);
	div_display.css({
		"position": "relative"
	});

	var div_vis = $("<div/>");
	div_vis.addClass("jswlib_sparkgraph");
	div_vis.addClass("jswlib_flag_notextsel");
	div_vis.attr('id', sparkgraph.getid_vis.call(this));
	div_vis.css({
		"position": "absolute",
		"width":  div_display.width(),
		"height": div_display.height()
	});
	div_display.append(div_vis);

	var graphW = new sparkgraph.Widget (div_vis);
	lwp.toWlive.call(this, graphW, 'graphW');
};

sparkgraph.getid_vis = function ()
{
	return lwp.toDisplay.call(this).attr('id')+"_graph";
};

sparkgraph.getdiv_vis = function ()
{
	return $("#"+sparkgraph.getid_vis.call(this));
};



sparkgraph.emerge = function ()
{
	// to do: handle custom scaling
	var i;

	var raw = $(this).data('wdata');

	var wellformed = true;
	var issues = [];

	var hasmin, hasmax;
	if (raw.hasOwnProperty('scale'))
	{
		hasmin = raw['scale'].hasOwnProperty('min');
		hasmax = raw['scale'].hasOwnProperty('max');

		if ((hasmin && typeof raw['scale']['min'] != 'number') ||
			(hasmax && typeof raw['scale']['max'] != 'number'))
		{
			wellformed = false;
			issues.push ("Malformed data in scale definition");
		}

		if (hasmin && hasmax && raw['scale']['min'] > raw['scale']['max'])
		{
			wellformed = false;
			issues.push ("Inconsistent scale definition")
		}
	}

	if (raw.hasOwnProperty('thresh'))
	{
		var thresh = raw['thresh'];

		hasmin = thresh.hasOwnProperty('min');
		hasmax = thresh.hasOwnProperty('max');

		if ((hasmin && typeof thresh['min'] != 'number') ||
			(hasmax && typeof thresh['max'] != 'number') ||
			(thresh.hasOwnProperty('color') && !lwp.isColorString(thresh['color'])) ||
			(thresh.hasOwnProperty('normal') && typeof(thresh['normal'])!='boolean') ||
			(thresh['normal'] && !hasmin && !hasmax))
		{
			wellformed = false;
			issues.push ("Malformed data in thresh definition")
		}

		var consistent = true;
		if (thresh['normal'])
		{
			if (hasmin && (thresh['min'] < 0 || thresh['min'] > 1))
			{
				consistent = false;
			}
			if (hasmax && (thresh['max'] < 0 || thresh['max'] > 1))
			{
				consistent = false;
			}
		}
		if (hasmin && hasmax && (thresh['min'] > thresh['max']))
		{
			consistent = false;
		}
		if (!consistent)
		{
			wellformed = false;
			issues.push ("Inconsistent threshold definition");
		}


	}

	if (!$.isArray(raw['series']) || raw['series'].length == 0)
	{
		wellformed = false;
		issues.push ("Missing series data");
	}
	else
	{
		var fallbackcoloridx = 0;
		var fallbackcolors = lwp.getColors();

		for (i = 0; i < raw['series'].length; i++)
		{
			var cs = raw['series'][i];

			cs['shade'] = cs['shade'] === true;

			if (!cs['values'])
			{
				wellformed = false;
				issues.push ("Missing values for series");
				break;
			}

			if (!lwp.isColorString(cs['color']))
			{
				cs['color'] = fallbackcolors[fallbackcoloridx];
				fallbackcoloridx = (fallbackcoloridx+1)%fallbackcolors.length;
			}

		}
	}


	// checking REQ section, with filters and sorts
	if (!wellformed)
	{
		throw "Invalid json data structure: "+issues.join(", ");
	}

	var widget = lwp.fromWlive.call(this, 'graphW');
	widget.inject(raw);
};

sparkgraph.wrender = function()
{
	var widget = lwp.fromWlive.call(this, 'graphW');
	widget.render();
};

sparkgraph.Widget = function Widget(jqdiv)
{
	this.div = jqdiv;

};

sparkgraph.Widget.prototype.inject = function (raw)
{
//	console.log("DATA", raw);
	var i, e, min, max, cs, thresh, hasmin, hasmax, hasscale;

	hasscale = raw.hasOwnProperty('scale');
	hasmin = hasscale && raw['scale'].hasOwnProperty('min');
	hasmax = hasscale && raw['scale'].hasOwnProperty('max');

	var unit = raw.hasOwnProperty('unit') && typeof raw['unit'] == 'string' ? raw['unit'] : "";

	if (!hasmin || !hasmax)
	{
		min = hasmin ? raw['scale']['min'] : Infinity;
		max = hasmax ? raw['scale']['max'] : -Infinity;
		for (i = 0; i < raw['series'].length; i++)
		{
			cs = raw.series[i];
//			console.log("Inspecting series ",cs," for scale");
			for (e = 0; e < cs['values'].length; e++)
			{
				var val = cs['values'][e];
				if (!hasmin && val != null && val < min)
				{
					min = val;
				}
				if (!hasmax && val != null && val > max)
				{
					max = val;
				}
			}
		}
	}
	else
	{
//		console.log("Scale preset to ",raw['scale']);
		min = raw['scale']['min'];
		max = raw['scale']['max'];
	}

	// if thresh is missing we do NOT use it at all
	thresh = raw['thresh'];
	if (thresh && (thresh.hasOwnProperty('min') || thresh.hasOwnProperty('max')))
	{
		if (!thresh.hasOwnProperty('min'))
		{
			thresh['min'] = min;
		}
		else if (thresh['normal'])
		{
			thresh['min'] = (thresh['min'] * (max-min)) + min;
		}
		if (!thresh.hasOwnProperty('max'))
		{
			thresh['max'] = max;
		}
		else if (thresh['normal'])
		{
			thresh['max'] = (thresh['max'] * (max-min)) + min;
		}
	}
	else
	{
		thresh = false;
	}

//	console.log("Set scale ",min," to ",max);
//	if (thresh)
//	{
//		console.log("Set threshold to ",thresh);
//	}

	var width = this.div.width();
	var height = this.div.height();

	var sparkopts = [];
	var sparkvals = [];
	for (i = 0; i < raw['series'].length; i++)
	{
		cs = raw['series'][i];
		var copts = {
			fillColor: cs['shade'] ? cs['color'] : false,
			chartRangeMin: min,
			chartRangeMax: max,
			composite: i != 0,
			lineColor: cs['color'],
			width: width,
			height: height,
			chartRangeClip: true,
			spotColor: false,
			minSpotColor: false,
			maxSpotColor: false,
			highlightSpotColor: false,
			tooltipSuffix: unit,
			numberFormatter: lwp.renderNumeric
		};
		if (i == 0 && thresh)
		{
			copts['normalRangeMax'] = thresh['max'];
			copts['normalRangeMin'] = thresh['min'];
			if (lwp.isColorString(thresh['color']))
			{
				copts['normalRangeColor'] = thresh['color'];
			}
		}
		if (cs['label'])
		{
			copts['tooltipPrefix'] = cs['label']+": ";
		}
		if (raw['notip']===true)
		{
			copts['disableTooltips'] = true;
		}
		sparkopts.push(copts);
		sparkvals.push(cs['values']);
	}

	this.sparkopts = sparkopts;
	this.sparkvals = sparkvals;
};

sparkgraph.Widget.tooltipFormatter = function (data)
{
//	$.spformat('{{value}}', 'tooltip-class')
	console.log("Formatting ",data);
};

sparkgraph.Widget.prototype.render = function ()
{

//	console.log("Rendering data for sparkW ",this);
	this.div.empty();
//	this.div.unbind('sparklineRegionChange');

	for (var i = 0; i < this.sparkvals.length; i++)
	{
		$(this.div).sparkline(this.sparkvals[i], this.sparkopts[i]);
	}

};


/*************************************
 ORDLIST WIDGET
 **************************************/

// ORDLIST (similar to dictlist but with an order, i.e. list based

/*

FORMAT = {
	items: [
		{
			key: string/object (rendertextcell),
			content: string/object (rendertextcell),
			css: string (css class)
		}
	],
	gen: {
		title:	string,
		scrollable: boolean, // if the widget should scroll vertically instead of overflowing
	}
}

 */


ordlist = {};

ordlist.assemble = function ()
{
	var div_display = lwp.toDisplay.call(this);
	div_display.css({
		"position": "relative"
	});

	var div_vis = $("<div/>");
	div_vis.addClass("jswlib_ordlist");
	div_vis.addClass("jswlib_flag_notextsel");
	div_vis.attr('id', ordlist.getid_vis.call(this));
	div_vis.css({
		"position": "absolute",
		"width":  div_display.width(),
		"height": div_display.height()
	});
	div_display.append(div_vis);

	var listW = new ordlist.Widget (div_vis);
	lwp.toWlive.call(this, listW, 'listW');
};

ordlist.getid_vis = function ()
{
	return lwp.toDisplay.call(this).attr('id')+"_table";
};

ordlist.getdiv_vis = function ()
{
	return $("#"+ordlist.getid_vis.call(this));
};

ordlist.emerge = function ()
{
	var raw = $(this).data('wdata');
	var wellformed = true;
	var issues = [];

	console.log("checking ",raw['items']);
	if (!$.isArray(raw['items']) || raw['items'].length == 0)
	{
		wellformed = false;
		issues.push("No items in list");
	}

	var qt = raw['items'].length;
	for (var i = 0; i < qt; i++)
	{
		var citem = raw['items'][i];
		if (!citem.hasOwnProperty('key') || !citem.hasOwnProperty('content'))
		{
			wellformed = false;
			issues.push("Invalid item structure for item "+i);
		}

		if (citem.hasOwnProperty('css') && typeof citem['css'] != 'string')
		{
			wellformed = false;
			issues.push("Invalid item class value for item "+i);
		}
	}


	if (!wellformed)
	{
		throw "Invalid json data structure: "+issues.join(", ");
	}

	var widget = lwp.fromWlive.call(this, 'listW');
	widget.inject(raw);

};

ordlist.wrender = function ()
{
	var widget = lwp.fromWlive.call(this, 'listW');
	console.log("Rendering widget ",widget);
	widget.render();
//	var height = widget.div[0].scrollHeight;

//	lwp.toDisplay.call(this).height(height);
//	lwp.inflate_v.call(this, height);

};

ordlist.Widget = function Widget(jqdiv)
{
	this.div = jqdiv;

};

ordlist.Widget.prototype.inject = function (raw)
{

	this.list = raw.items;
	this.title = raw['gen'] && raw['gen']['title'] ? raw['gen']['title'] : null;
	this.scrollable = raw['gen'] ? raw['gen']['scrollable'] === true : false;
};

ordlist.Widget.const = {};
ordlist.Widget.const.CLASS_WIDGET = 'jswlib_ordlist_frame';
ordlist.Widget.const.CLASS_ROW = 'jswlib_ordlist_row';
ordlist.Widget.const.CLASS_CELL = 'jswlib_ordlist_cell';
ordlist.Widget.const.CLASS_CELL_KEY = 'jswlib_ordlist_itemkey';
ordlist.Widget.const.CLASS_CELL_CONT = 'jswlib_ordlist_itemcontent';
ordlist.Widget.const.CLASS_TITLE = 'jswlib_ordlist_title';

ordlist.Widget.prototype.render = function ()
{
	this.master = this.div.closest(".jswlib_widget_ordlist");

	this.div.empty();


	var rendered = $("<table/>");
	rendered.addClass(ordlist.Widget.const.CLASS_WIDGET);

	var titlediv;
	if (this['title'])
	{
		titlediv = $("<div/>");
		titlediv.addClass(ordlist.Widget.const.CLASS_TITLE);
		titlediv.append(lwp.rendertextcell(this['title']));
	}
	else
	{
		titlediv = null;
	}

	this.div.append(titlediv);
	var titleheight = titlediv.height();

	var qt = this.list.length;
	for (var i = 0; i < qt; i++)
	{
		var item = this.list[i];
		var row = $("<tr/>");
		row.addClass(ordlist.Widget.const.CLASS_ROW);

		console.log("Adding item ",item);
		var cssclass = item['css'];

		var td, textrender;
		td = $("<td/>");
		td.addClass(ordlist.Widget.const.CLASS_CELL);
		td.addClass(ordlist.Widget.const.CLASS_CELL_KEY);
		textrender = lwp.rendertextcell(item['key']);
		td.append(textrender);
		if (cssclass)
		{
			td.addClass(cssclass);
		}
		row.append(td);

		console.log("added key td");

		td = $("<td/>");
		td.addClass(ordlist.Widget.const.CLASS_CELL);
		td.addClass(ordlist.Widget.const.CLASS_CELL_CONT);
		textrender = lwp.rendertextcell(item['content']);
		td.append(textrender);

		if (cssclass)
		{
			td.addClass(cssclass);
		}
		row.append(td);

//		console.log("added cont td");

		rendered.append(row);

//		console.log("added row");
	}

//	console.log("prepared table");

	this.div.append(rendered);
	var neededheight = this.div.height();
	var tableheight = rendered.outerHeight();
	rendered.css("min-height", neededheight-titleheight);

	if (this['scrollable'])
	{
		this.div.css('overflow-y', 'auto');
	}
	else
	{
//		console.log("DIV HEIGHT ",tableheight," for master ",this.master," on div ",this.div);
		this.master.css("min-height", tableheight+titleheight);
	}

};














/*
	WIDGET registration: rebuilds the WidgetTree
 */
jswlib['qgrid'] = qgrid;
jswlib['sparkgraph'] = sparkgraph;
jswlib['ordlist'] = ordlist;
buildWidgetsTree();


