/**
 * Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
 * Created: 2/28/14, 9:57 AM
 * Author: Antonio Vaccarino
 */


/**
 * JavaScript Widgets Lib - CORE
 * base widget class and standard functionalities
 */


/**
 * Features to be supported:
 * - jquery plugin, extended by specific widgets
 * - data and parameters passed by wdata
 * - controls/ui connecgted only loosely
 * - walkability (if available)
 */

/**
 * DEPENDENCY:
 * jquery (testing on 1.11.0)
 *
 * SUPPORTS
 * Google Chrome
 * Mozilla Firefox
 * Internet Explorer 6+
 *
 * NOTES
 * In case of migration to JQ 2.0 we lose support for IE <= 8
 *
 */

/**
 * DEPENDENCY:
 * ICanHaz (testing on 0.10.2)
 * for the renderTextObject functionality
 */

/**
 * DEPENDENCY:
 * jquery-ui (testing on 1.10.4)
 *
 */

// namespace for wlive etc.
/**
 * The WDS namespace replaces wdata and others from the old version of the widget; wdata and such can still be used for the init stage of the widget, but they are NEVER used after that and NEVER written by the widget itself
 */

WDS = {};

WDS.wlib = {};
WDS.wstore = {};





// core widget functionalitites
// all widgets are build adding to this one
var wcore = {};

wcore.CONST = {};

wcore.CONST.ID_DISPLAY_SUFFIX = "_wvis";

wcore.CONST.WPREFIX = "jsw2";

wcore.CONST.WIDGETCLASS_CORE   = wcore.CONST.WPREFIX+"_widget";
wcore.CONST.WIDGETCLASS_PREFIX = wcore.CONST.WPREFIX+"_class_";
wcore.CONST.WCLASS_CONTAINER   = wcore.CONST.WPREFIX+"_container";
wcore.CONST.WCLASS_DISPLAY     = wcore.CONST.WPREFIX+"_display";

wcore.CONST.WCLASS_REFRESH_BTN = wcore.CONST.WPREFIX+"_refresh_button";

wcore.CONST.COLOR_EXPANDER = "#999999";
wcore.CONST.COLOR_EXPGLYPH = "#ffffff";


wcore.CONST.STR_AUTO_REFRESH_ON  = "Auto-refresh ON";
wcore.CONST.STR_AUTO_REFRESH_OFF = "Auto-refresh OFF";


wcore.winit = function (wtype)
{
	//console.log("wcore.winit(" + wtype + ")");
	
	var WIDGET = $(this);
	
	// ID	
	var wid = WIDGET.attr('id');
	//console.log("\t id = " + wid);

	// WTYPE
	if (!wtype) 
		wtype = WIDGET.data('wtype');
	
	if (!wtype)
		throw ("Widget type not declared in constructor or tag wtype");
		
	WIDGET.data("wtype", wtype);

	/*
	 * Registriamo i dati del widget
	 */
	w = { 
		'wtype': wtype 
	};

	WDS[wid] = w;


	/*
	 * Inizializziamo il DIV
	 */
	WIDGET.addClass(wcore.CONST.WIDGETCLASS_PREFIX + wtype);
	WIDGET.addClass(wcore.CONST.WIDGETCLASS_CORE);
	WIDGET.css('position', 'relative');
	WIDGET.empty();

	/*
	 * REFRESH BUTTON
	 */ 
	var has_autorefresh_button   = this.data('autorefresh_button') != undefined;

	//console.log("\tInit refresh button: " + has_autorefresh_button);

	if (has_autorefresh_button) {

		WIDGET.addClass('refresh-controller');

		//console.log("\tInit refresh button");

		VISIBLE = 1;
		NORMAL  = 0.1;

		var refreshbutton = $("<button class='" + wcore.CONST.WCLASS_REFRESH_BTN + " gui-button gui-gradient' title=''><i class='icon-refresh icon16x16'/> &nbsp; <span>"+wcore.CONST.STR_AUTO_REFRESH_ON+"</span></button>");
		WIDGET.append(refreshbutton);
		
		refreshbutton.addClass('refresh-button-normal');

		refreshbutton.click({element: refreshbutton, widget: WIDGET }, function(e) {
			var widget = e.data.widget;
			//console.log("wcore.refreshbutton.click()");
			//console.log("\t" + widget.attr('id') );
			// Fix autorefresh in subwidges
			wcore.toggleSubWidgetsAutorefresh.call(widget);
		});
	}

	/*
	 * WIDGET CONTENT DIV (DISPLAY)
	 */ 
	//console.log("\tCreating display");

	var displaydiv = $("<div/>");
	displaydiv.addClass(wcore.CONST.WCLASS_DISPLAY);
	displaydiv.attr('id', wcore.getDisplayId.call(WIDGET));

	displaydiv.css('position','relative');
	displaydiv.css('z-indez' , 1);
	WIDGET.append(displaydiv);


	//console.log("\tSetting refresh/remote params");
	wcore.trySetRemote.call(WIDGET);


	// making it chainable
	return WIDGET;
};



wcore.destroy = function (){
        console.log("wcore destroy");

        var widget = $(this);

        if (wcore.has.call(this, 'destroy'))
        {
                        wcore.run.call(this, 'destroy');
        }
};


wcore.toggleSubWidgetsAutorefresh = function (forcedisabled)
{
	var widget = this;
	

	// Troviamo il pulsante
	var refreshbutton = $('.' +  wcore.CONST.WCLASS_REFRESH_BTN, widget);

	//console.log('wcore.toggleSubWidgetsAutorefresh('+forcedisabled+')' + refreshbutton);

	//
	// Update button
	//
	var button_icon = $('i', refreshbutton[0]);
	
	var is_refresh_enabled = ! widget.hasClass('prevent-refresh');
	

	var enable_refresh = ! is_refresh_enabled;
	if (forcedisabled != undefined)
		enable_refresh = forcedisabled;

	//console.log('\tenable refresh? ' + enable_refresh);

	if (enable_refresh) {
		refreshbutton.removeClass   ('refresh-button-visible');
		refreshbutton.addClass      ('refresh-button-normal');

		$('span', refreshbutton).empty().append(wcore.CONST.STR_AUTO_REFRESH_ON);

		button_icon.addClass('icon-refresh');
		button_icon.removeClass('icon-stop');
		button_icon.removeClass('refresh-stop');

		// Enable refresh
		widget.removeClass('prevent-refresh');
	}
	else {
		
		refreshbutton.addClass('refresh-button-visible');
		refreshbutton.removeClass   ('refresh-button-normal');
		
		$('span', refreshbutton).empty().append(wcore.CONST.STR_AUTO_REFRESH_OFF);

		button_icon.removeClass('icon-refresh');
		button_icon.addClass('icon-stop');
		button_icon.addClass('refresh-stop');


		// Disable refresh
		widget.addClass('prevent-refresh');

	}

	if (enable_refresh) 
		widget.removeClass("autorefresh-disabled");
	else
		widget.addClass("autorefresh-disabled");

	
	$('.jsw2_widget', widget).each( function(i, el) {
		var el_obj = $(el);

		//console.log("\tsubwidget " + i + " " + el_obj.attr('id') );

		if (enable_refresh) 
			el_obj.removeClass("autorefresh-disabled");
		else
			el_obj.addClass("autorefresh-disabled");
	});	
}

wcore.blockAutorefresh = function ()
{
	var widget = $(this);
	
	//console.log("wcore.blockAutorefresh");
	//console.log(widget);
	
	widget.closest('.refresh-controller').each(function(i, el) {
		var w = $(el);
		wcore.toggleSubWidgetsAutorefresh.call(w, false);
	});
}



wcore.trySetRemote = function ()
{
	var premote  = this.data('premote');
	var rtimer   = this.data('rtimer');
	var data_pre = this.data('wdata');

	var hasRemote = typeof(premote) == 'string' && premote.length > 0;
	var hasTimer  = typeof(rtimer)  == 'number' && rtimer         > 0;
	
	// I dati presi dal wdata non sono un oggetto vuoto?
	var hasData   = $.isPlainObject(data_pre) && Object.keys(data_pre).length > 0;



	if (hasRemote)
	{
		//console.log("TrySetRemote for "+this.attr('id'));
		if (!hasData)
		{
			// calling it NOW even without a timer
			wcore.cycleRemote.call(this, true);
		}
		else if (hasTimer)
		{
			// has data AND timer, we call it with a delay
			// otherwise (data ready but no timer in parameters) we don't call it at all
			wcore.cycleRemote.call(this);
		}
	}
	else
	{
//		console.warn("Widget ",this.attr('id'),'has no registered update capability');
	}

};

wcore.cycleRemote = function (startnow)
{
	var WIDGET = this;

	var premote  = WIDGET.data('premote');
	var rtimer   = WIDGET.data('rtimer');

	var hasTimer = typeof(rtimer) == 'number' && rtimer > 0;

	// starts immediately if timer is not valid or if forced
	startnow = !hasTimer || startnow === true;

	if (startnow)
	{
		wcore.update.call(WIDGET);
	}

	if (hasTimer)
	{
		var countdown = wcore.aux.sec2msec(rtimer);
		
		window.setInterval(function ()
		{
			console.log("*** REFRESH TIMER: " + WIDGET.attr('id'));

			var id = WIDGET.attr('id');
			
			var w = $('#'+id);
			
			//
			// FIX: se un widget non e' visibile non effettiuamo l'aggiornamento
			//      Dovremmo utilizzare un flag e non guardare una proprieta' CSS ma per il momento facciamo cosi'.
			//
			
			if (w.is(':visible') == false) {
				return;
			}
			
			console.log("\tautorefresh-disabled = " + w.hasClass("autorefresh-disabled"));
			
			if (w.hasClass("autorefresh-disabled") == true) {
				console.log("\t\tNot updating");
				return;
			}

			wcore.update.call(WIDGET);
		}, countdown);
	}

};

/*
wcore.getRemoteData = function ()
{
	wcore.update.call(this);

	wcore.trySetRemote.call(this);
};
*/

wcore.getWType = function ()
{
	// should we use WDS?
//	return $(this).data('wtype');
	//return WDS[$(this).attr('id')].wtype;
	
	return $(this).data('wtype');
};

wcore.getW = function ()
{
	return $(this).closest("."+wcore.CONST.WIDGETCLASS_CORE);
};

wcore.getDisplayId = function ()
{
	return this.attr('id')+wcore.CONST.ID_DISPLAY_SUFFIX;
};

/*
wcore.setDisplay = function ()
{
	this.empty();
	var displaydiv = $("<div/>");
	displaydiv.addClass(wcore.CONST.WCLASS_DISPLAY);
	displaydiv.attr('id', wcore.getDisplayId.call(this));
	this.append(displaydiv);
};
*/

wcore.getDisplay = function ()
{
//	var wdisplay = wcore.getW.call(this).find("."+wcore.CONST.WCLASS_DISPLAY);
//	console.log("Display from root ",wcore.getW.call(this));
	var wdisplay = wcore.getW.call(this).find("#"+wcore.getDisplayId.call(this));
//	console.log("Retrieved display ",wdisplay);
	return wdisplay.length > 0 ? wdisplay : null;
};

/*
wcore.setDivClass = function setDivClass ()
{
	// removing display class(es) and setting the new
	try
	{
		var classes = this.attr('class').split(" ");
		for (var i = 0; i < classes.length; i++)
		{
			var cclass = classes[i];
			if (cclass.substring(0, wcore.CONST.WIDGETCLASS_PREFIX.length).equals(wcore.CONST.WIDGETCLASS_PREFIX))
			{
				this.removeClass(cclass);
			}
		}
	}
	catch (err)
	{
		// do nothing, no class set or something
	}
	this.addClass(wcore.CONST.WIDGETCLASS_PREFIX+wcore.getWType.call(this));
	this.addClass(wcore.CONST.WIDGETCLASS_CORE);
};
*/

wcore.layout = function ()
{
	// NOTE: TRYING to go via css
	var wdisplay = wcore.getDisplay.call(this);
	if (wdisplay != null)
	{
		wdisplay.clear();
	}

	wdisplay = $("<div/>");
	wdisplay.addClass(wcore.CONST.WCLASS_DISPLAY);

	return this;
};

wcore.update = function (stateparams, forcenow)
{
	//console.log("wcore.update()");

	// forcing parameters can be used to get a new view of the widget
	if (!stateparams)
	{
		stateparams = wcore.getWstate.call(this);
	}
//	console.log("wcore.update with request payload ",stateparams);

	forcenow = forcenow === true;

	var withlock = $(this).data(wcore.CONST.FIELD_SHOWSPINNER) === true;
	var premote = $(this).data('premote');

	//console.log("\tlock   = " + withlock);
	//console.log("\tremote = " + premote);
	
//	console.log("PRemote for w ",$(this),": ",premote);
	if (NS && NS.hasOwnProperty('delay_dl')) {
//		console.log("Refresh via NS.delay_dl");
		// if the page offers a download+init mechanism, use it
		if (withlock)
		{
			//console.log("SPINNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNE");
			wcore.async.lock.call(this);
		}
//		console.log("updating widget via forced remote ",NS['delay_dl']);
		NS['delay_dl'](premote, stateparams, forcenow);
	}
	else if (!!premote) {
		var widget = this;
//		console.log("Refresh via straight ajax");
		// otherwise parse as generic url
		var ajaxparams = {
			custom: true,
			silent: true,
			lock: withlock,
			callback: function (data)
				{
					widget.data('wdata', data);
//					console.log("loading ",data," in wdata of ",subw);
					widget['wcore']('populate');
					widget['wcore']('render');
				}
		};

//		console.log("updating widget via ajax");
		wcore.async.ajax.call(this, premote, stateparams, ajaxparams);

	}
	else {
//		console.warn("No update capability detected for widget ",this.attr('id'));
	}


};


wcore.setWstate = function (data)
{
	wcore.write.call(this, data, "wstate");
};

wcore.getWstate = function ()
{
	return wcore.read.call(this, "wstate");
};

wcore.write = function (data, key, subkey)
{
//    console.log("TOWLIVE",key,subkey,data);

	var wid = $(this).attr('id');

	if (!WDS.hasOwnProperty(wid))
	{
		WDS[wid] = {};
	}

//    console.log("Putting data on wlive for widget "+wid+" in key "+key);

	if (!key)
	{
		if ($.isPlainObject(data))
		{
			for (var i = 0; i < Object.keys(data).length; i++)
			{
				var ckey = Object.keys(data)[i];
				WDS[wid][ckey] = data[ckey];
			}
		}
		else
		{
			throw "Wlive needs a dict type object";
		}
	}
	else
	{
		if (!subkey)
		{
			WDS[wid][key] = data;
		}
		else
		{
			if (!WDS[wid].hasOwnProperty(key))
			{
				WDS[wid][key] = {};
			}
			WDS[wid][key][subkey] = data;

		}
//        console.log("NOWLIVE",NS.wlive[wid][key]);
	}


};

wcore.read = function (keyname)
{
	var wid = $(this).attr('id');
	if (!keyname)
	{
		try
		{
			return WDS[wid];
		}
		catch (err)
		{
			return undefined;
		}
	}
	else
	{
		try
		{
			return WDS[wid][keyname];
		}
		catch (err)
		{
			return undefined;
		}
	}
};

wcore.delete = function (key)
{
	var wid = $(this).attr('id');

	if (!key)
	{
		WDS[wid] = {};
	}
	else
	{
		try
		{
			delete WDS[wid][key];
		}
		catch (err)
		{
			// do nothing, key already removed/missing
		}
	}
};

wcore.run = function (funcname, args)
{
		/*
	 Runs a function inside the plugin of the current object with arguments from the args array
	 */
//	console.log("wcore.run func+args", funcname , " ", args);

	var wtype = wcore.getWType.call(this);

	if (!wtype)
	{
		console.error("wcore.run: Could not retrieve a widget for (jq) context ",this);
		throw "No widget instance for launching function "+funcname;
	}

	var wclass = WDS.wlib[wtype];
	if (!wclass)
	{
		console.error("wcore.run: Widget type "+wtype+" is not compiled into the widget library (check template file for the appropriate js import)");
		throw "Widget "+wtype+" not registered into library namespace";
	}

	var handler = WDS.wlib[wtype][funcname];
	if (!handler)
	{
		console.error("wcore.run: "+funcname+" is not registered as a function specific for widget "+wtype);
		throw "Function is not registered into widget namespace";
	}

	return handler.apply(this, args);
};

wcore.has = function (candidate)
{

	// returns whether the widget type for this widget has a certain method
	var wtype = wcore.getWType.call(this);
	
	return WDS.wlib[wtype].hasOwnProperty(candidate);
//	return jswidgets[$(this).data('wtype')].hasOwnProperty(methodname);

};

wcore.reflow = function ()
{
	// resizes the current widget and forces recalculation up the hierarchy

	var canvas = wcore.getDisplay.call($(this));
	var elems = canvas.children();
	var height = 0;
	for (var i = 0; i < elems.length; i++)
	{
		height += elems.get(i).scrollHeight;
	}
//	console.log("Reflowing ",wcore.getW()," to height "+height);
	canvas.height(height);
};

wcore.reflowUpwards = function ()
{
	// resizes the current widget and forces recalculation up the hierarchy
	wcore.reflow.call(this);

	var nextUpwards = $(this).parent().closest("."+wcore.CONST.WIDGETCLASS_CORE);
	if (nextUpwards.length > 0)
	{
		wcore.reflowUpwards.call(nextUpwards);
	}
};


/*
 * functions to handle network and async based workflow
 */

wcore.CONST.CLASS_DIV_LOCKER = "jsw2_wlock";
wcore.CONST.CLASS_DIV_SPINNER = "jsw2_wlock_spinner";
wcore.CONST.FIELD_SHOWSPINNER = "showspinner";

wcore.async = {};

wcore.CONST.SPINNER_SIZE = 48;

wcore.async.lock = function ()
{
	// intervenes ONLY if a lock is not already present

	if ($(this).find("."+wcore.CONST.CLASS_DIV_LOCKER).length == 0)
	{
		var locker = $("<div />");
		locker.addClass(wcore.CONST.CLASS_DIV_LOCKER);
		
		var spinner = wcore.aux.renderFA("spinner", null, ["spin"]);
		
		var spinholder = $("<div />");
		spinholder.addClass(wcore.CONST.CLASS_DIV_SPINNER);

		var wwidth = $(this).width();
		var wheight = $(this).height();

		var shortside = Math.min(wwidth, wheight);
		var spinnersize = Math.min(wcore.CONST.SPINNER_SIZE, shortside-8);
		
		var spinnersize = wcore.CONST.SPINNER_SIZE;

		var offsetX = ($(this).width()-spinnersize)/2;
		var offsetY = ($(this).height()-spinnersize)/2;

		spinholder.css({
			"top": offsetY+"px",
			"left": offsetX+"px",
			"font-size": spinnersize+"px"
		});
		spinholder.append(spinner);
		locker.append(spinholder);

		locker.bind('mousewheel wheel scroll click', function(ev)
		{
			ev.preventDefault();
			ev.stopPropagation();
		});
		this.append(locker);
	}
};

wcore.async.unlock = function ()
{
	this.find("."+wcore.CONST.CLASS_DIV_LOCKER).remove();
};

wcore.async.verifyMessage = function (payload)
{
	if (typeof payload != 'object')
	{
		return false;
	}
	else
	{
		if (!payload.hasOwnProperty('status') ||
			!payload.hasOwnProperty('info') ||
			!payload.hasOwnProperty('data'))
		{
			return false;
		}
	}
	return true;
};

wcore.async.ajaxFormatException = function (badmessage)
{
	console.warn("Invalid SANET ajax message: ",badmessage);
	return {
		name: "AjaxFormatException",
		message: "Message has non valid data"
	}
};

wcore.CONST.MESSAGE_AJAX_ERROR = "Errore di trasmissione.";
wcore.CONST.MESSAGE_AJAX_MALFORMED = "Risposta non prevista dal server.";
wcore.CONST.MESSAGE_AJAX_ALLOK = "Operazione eseguita correttamente.";
wcore.CONST.MESSAGE_AJAX_OPFAILED = "Operazione fallita.";

wcore.async.ajax = function (url, payload, params)
{

	// makes an ajax call and handles the return before passing it to the callback

	// jq note: use .done(data) .fail(jqXHR, textStatus) .always(data|jqXHR, textStatus, jqXHR|errorThrown)

	/*
		All calls MUST return:
		{
			status: string, // success|error (maybe others in the future)
			info: string, // message describing the output, optional
			data: mixed, // data returned, usually only for success
		}
	*/

	// parameters
//	console.log("Call made with params ",params);

	// if the message from the server has a specific structure that does not conform with the standard message structure expectation as of VerifyMessageStructure
	var customResponse = params['custom'] === true;

	var contentType = !!params['content'] ? params['content'] : "application/json";
	
	var usesJSON = contentType === "application/json";

	// if the call locks the widget while it's done (e.g. user requesting an explicit update) or not (automatic refresh)
	var isLocking = params['lock'] === true;

	// if the call displays dialogs to report status or simply passes the result of a successful call to the callback
	var isSilent = params['silent'] === true;

	// the callback function
	var callback = params['callback'];
	// to find which way of calling the callback is correct
	var isCallable = typeof(callback) === 'function';
	var hasCallback = isCallable || typeof (callback) == 'string';

	var widget = this;


	if (isLocking) {
		wcore.async.lock.call(widget);
	}

	$.ajax({
		url: url,
		contentType: contentType,
		cache: false,
		type: "POST",
		data: payload,
		context: {
			"widget": widget
		}
	}).done( function (data, textStatus, jqXHR) {

			if (isLocking) {
				wcore.async.unlock.call(widget);
			}

			// parsing json if needed
			try {
				if (usesJSON) {
					
					data = typeof(data) === 'string' ? JSON.parse(data) : data;
					
					if (!customResponse && !wcore.async.verifyMessage(data)) {
						throw wcore.async.ajaxFormatException();
					}
					else if (!customResponse) {
						if (data['status'] === 'success') {
							
							if (!isSilent) {
								wcore.aux.feedbackDialog.call(widget, wcore.CONST.MESSAGE_AJAX_ALLOK);
							}
						}
						else if (data['status'] === 'error') {
							if (!isSilent) {
								wcore.aux.feedbackDialog.call(widget, wcore.CONST.MESSAGE_AJAX_OPFAILED, data['info']);
							}
						}
						else {
							// separate from basic verification exception because we might add more states in the future
							throw wcore.async.ajaxFormatException();
						}
					}
				}

				// performing callback
				// NOTE that callback is performed in the background even as the feedback dialog is still on display
				if (hasCallback)
				{
//					console.log('Default handlers done, moving to specified callback');
					// passing the original req payload so it can be used for reference
					try {
						if (isCallable) {
							callback.call(widget, data, payload);
						}
						else {
							// calling from function name as string, assuming it's a member function of the caller widget type
							wcore.run.call(widget, callback, [data, payload]);
						}
					}
					catch (err) {
						console.error("Error in callback ",callback," after ajax on "+url+": ",err);
					}

				}

			}
			catch (err) {
				// message must have a certain structure
				if (!isSilent) {
					wcore.aux.feedbackDialog.call(widget, wcore.CONST.MESSAGE_AJAX_MALFORMED);
				}
			}

	}).fail( function (jqXHR, textStatus) {

		if (isLocking) {
			wcore.async.unlock.call(widget);
		}

		// communication error
		console.warn("Ajax call to "+url+" has failed with status ",textStatus);
		if (!isSilent) {
			wcore.aux.feedbackDialog.call(widget, wcore.CONST.MESSAGE_AJAX_ERROR+"<br>HTTP status: "+textStatus);
		}
	 });
};







wcore.async.ajax_json = function (url, payload, params)
{
	var widget = this;


	// makes an ajax call and handles the return before passing it to the callback

	// jq note: use .done(data) .fail(jqXHR, textStatus) .always(data|jqXHR, textStatus, jqXHR|errorThrown)

	/*
		All calls MUST return:
		{
			status: string, // success|error (maybe others in the future)
			info: string, // message describing the output, optional
			data: mixed, // data returned, usually only for success
		}
	*/

	// if the call locks the widget while it's done (e.g. user requesting an explicit update) or not (automatic refresh)
	var isLocking = params['lock'] === true;

	// the callback function
	var callback = params['callback'];
	
	// to find which way of calling the callback is correct
	var isCallable = typeof(callback) === 'function';
	var hasCallback = isCallable || typeof (callback) == 'string';


	var success_callback = params['success_callback'];
	var error_callback = params['error_callback'];
	

	if (isLocking) wcore.async.lock.call(widget);

	$.ajax({
		url: url,
		//contentType: "application/json",
		cache: false,
		type: "POST",
		data: payload,
		context: {
			"widget": widget
		}
	}).done( function (data, textStatus, jqXHR) {

			if (isLocking) wcore.async.unlock.call(widget);

			// parsing json if needed
			try {
				data = typeof(data) === 'string' ? JSON.parse(data) : data;
				
				if (!wcore.async.verifyMessage(data)) 
					throw wcore.async.ajaxFormatException();

				if (data['status'] === 'success') {
					if (success_callback != undefined) 
						success_callback.call(widget, data, payload)
				}
				else if (data['status'] === 'error') {
					if (error_callback != undefined) 
						error_callback.call(widget, data, payload)
				}
				else {
					// separate from basic verification exception because we might add more states in the future
					throw wcore.async.ajaxFormatException();
				}
			}
			catch (err) {
				wcore.aux.feedbackDialog.call(widget, wcore.CONST.MESSAGE_AJAX_MALFORMED);
			}

	}).fail( function (jqXHR, textStatus) {

		if (isLocking) wcore.async.unlock.call(widget);

		// communication error
		console.warn("Ajax call to "+url+" has failed with status ",textStatus);
		
		wcore.aux.feedbackDialog.call(widget, wcore.CONST.MESSAGE_AJAX_ERROR+"<br>HTTP status: "+textStatus);
	 });
};






























/*
 * UTILITY FUNCTIONS
 */

wcore.aux = {};

wcore.CONST.CLASS_MODAL = "jsw2_dialog_modal";

wcore.aux.feedbackDialog = function (message, details)
{
	var dialogdiv = $("<div/>");
	dialogdiv.append(message);
	
	if (details != undefined)
		dialogdiv.append(details);
	
	var widget = wcore.getW.call(this);
	
	dialogdiv.dialog({
		autoOpen: true,
		modal: true,
		closeOnEscape: false,
		dialogClass: wcore.CONST.CLASS_MODAL,
		buttons: {
			"ok": function ()
			{
				$(this).dialog("destroy");
				wcore.async.unlock.call(widget);
			}
		}
	});
};


wcore.aux.ShowDialog = function (title, message, details)
{

	var widget = wcore.getW.call(this);

	var dialogdiv = $("<div/>");
	dialogdiv.append("<p>" + message + "</p>");
	
	if (details != undefined) {
		dialogdiv.append("<br/>");
		dialogdiv.append("<h3>Details</h3>");
		dialogdiv.append("<br/>");
		dialogdiv.append("<p>"+details+"</p>");
	}
	
	
	dialogdiv.dialog({
		autoOpen: true,
		modal: true,
		closeOnEscape: false,
		dialogClass: wcore.CONST.CLASS_MODAL,
		title: "ERROR",
		buttons: {
			"ok": function ()
			{
				$(this).dialog("destroy");
				wcore.async.unlock.call(widget);
			}
		}
	});

}

wcore.aux.ShowErrorDialog = function (message, details)
{
	var widget = this;
	wcore.aux.ShowDialog.call(widget, "Error", message, details);
};



// css class for button objects
wcore.CONST.CLASS_BUTTON = "jsw2_item_button";
// css class to avoid text selection on items
wcore.CONST.CLASS_UNSEL = "jsw2_flag_notextsel";

wcore.aux.setButton = function (jq, context, callback)
{
	jq.bind('click', context, callback);
	jq.addClass(wcore.CONST.CLASS_BUTTON);
	jq.addClass(wcore.CONST.CLASS_UNSEL);
};

wcore.aux.sec2msec = function (timestamp)
{
	// converts a seconds timestamp (e.g. Python) to a msec timestamp (e.g. Javascript)
	var result;
	if ($.isArray(timestamp))
	{
		result = [];
		for (var i = 0; i < timestamp.length; i++)
		{
			result.push(Math.round(timestamp*1000));
		}
	}
	else
	{
		result = Math.round(timestamp*1000);
	}

	return result;
};

wcore.aux.timeToStrings = function (timestamp, convertSecStamp)
{
		if (convertSecStamp===true)
	{
		timestamp = wcore.aux.sec2msec(timestamp);
	}

	var date = typeof(timestamp) == 'number' ? new Date(timestamp) : timestamp;
	var tz = date.getTimezoneOffset();
	var timezonemod = tz > 0 ? "-" : "+";
	var timezonehours = Math.abs(Math.round(tz / 60));
	timezonehours = timezonehours < 10 ? "0"+timezonehours : timezonehours;
	var timezonemins = tz % 60 == 0 ? "00" : "30";
	var timezonestring = ""+timezonemod+""+timezonehours+""+timezonemins;




};

wcore.aux.renderTimeStringShort = function (timestamp, convertSecStamp)
{

	if (convertSecStamp===true)
	{
		timestamp = wcore.aux.sec2msec(timestamp);
	}

	var date = typeof(timestamp) == 'number' ? new Date(timestamp) : timestamp;

	var tz = date.getTimezoneOffset();
	var timezonemod = tz > 0 ? "-" : "+";
	var timezonehours = Math.abs(Math.round(tz / 60));
	timezonehours = timezonehours < 10 ? "0"+timezonehours : timezonehours;
	var timezonemins = tz % 60 == 0 ? "00" : "30";
	var timezonestring = ""+timezonemod+""+timezonehours+""+timezonemins;

	var time_date = date.getDate();
	//console.log(typeof time_date);
	if (time_date<10) { time_date = "0"+time_date; }
	var time_month = date.getMonth()+1;
	if (time_month<10) { time_month = "0"+time_month; }
	var time_hours = date.getHours();
	if (time_hours<10) { time_hours = "0"+time_hours; }
	var time_mins = date.getMinutes();
	if (time_mins<10) { time_mins = "0"+time_mins; }

	var timestring = time_date+"/"+time_month+"<br>"	+ time_hours +":"+ time_mins;

	return timestring;
};

wcore.aux.renderTimeString = function (timestamp, convertSecStamp)
{

	if (convertSecStamp===true)
	{
		timestamp = wcore.aux.sec2msec(timestamp);
	}

	var date = typeof(timestamp) == 'number' ? new Date(timestamp) : timestamp;

	var tz = date.getTimezoneOffset();
	var timezonemod = tz > 0 ? "-" : "+";
	var timezonehours = Math.abs(Math.round(tz / 60));
	timezonehours = timezonehours < 10 ? "0"+timezonehours : timezonehours;
	var timezonemins = tz % 60 == 0 ? "00" : "30";
	var timezonestring = ""+timezonemod+""+timezonehours+""+timezonemins;

	var time_date = date.getDate();
	//console.log(typeof time_date);
	if (time_date<10) { time_date = "0"+time_date; }
	var time_month = date.getMonth()+1;
	if (time_month<10) { time_month = "0"+time_month; }
	var time_hours = date.getHours();
	if (time_hours<10) { time_hours = "0"+time_hours; }
	var time_mins = date.getMinutes();
	if (time_mins<10) { time_mins = "0"+time_mins; }
	var time_secs = date.getSeconds();
	if (time_secs<10) { time_secs = "0"+time_secs; }

	var timestring = time_date+"-"+time_month+"-"+date.getFullYear()+" "
						+ time_hours +":"+ time_mins +":"+ time_secs +" "+timezonestring;

	return timestring;
};

wcore.aux.renderLink = function (url, content)
{
	var link = $("<a/>");
	link.attr('href', url);
	if (!!content)
	{
		link.append(content);
	}
	else
	{
		link.append(url);
	}
	return link;
};

wcore.aux.renderFA = function (iconid, label, modifiers)
{
	var FAPRE = "fa-";

	var fadiv = $("<i/>");
	if (iconid.substr(0,3)!="fa-")
	{
		iconid = FAPRE+iconid;
	}
	fadiv.addClass("fa");
	fadiv.addClass(iconid);
	if (label)
	{
		fadiv.after(" "+label);
	}

	if ($.isArray(modifiers))
	{
		for (var i = 0; i < modifiers.length; i++)
		{
			fadiv.addClass(FAPRE+modifiers[i]);
		}
	}

//	console.log(fadiv);
	return fadiv;
};

wcore.CONST.EXPANDER_SIDE_CENTER = 0;
wcore.CONST.EXPANDER_SIDE_LEFT = 1;
wcore.CONST.EXPANDER_SIDE_RIGHT = 2;
wcore.CONST.CLASS_EXPANDER_BUTTON = "jsw2_expander_btn";
wcore.CONST.CLASS_EXPANDER_DIV = "jsw2_expander";

wcore.aux.killEvent = function (ev)
{
	// shorthand to use on any event we do not want to spread
	ev.stopPropagation();
	ev.preventDefault();
	return false;
};

wcore.aux.renderExpander = function (side, cb_open, cb_close, start_open, label, class_btn)
{
	if(label == undefined){
		label = "";
	}
	if(class_btn == undefined){
		class_btn = wcore.CONST.CLASS_EXPANDER_BUTTON;
	}
	
	// renders, binds (if available) and returns an expander element
	// always closed by default

	// start_open if false by default, i.e. the Expander by default shows a button that opens something
	start_open = start_open === true;

	var variantclass = [
		"jsw2_expander_side_center",
		"jsw2_expander_side_left",
		"jsw2_expander_side_right"
	];

	if (typeof(side)!="number" || side >= variantclass.length)
	{
		side = 0;
	}

	var icon_open = wcore.aux.renderFA("chevron-down", label);
	var icon_close = wcore.aux.renderFA("chevron-up");

	var button_icon = start_open ? icon_close : icon_open;

	var exdiv = $("<div/>");
	exdiv.addClass(wcore.CONST.CLASS_EXPANDER_DIV);
	exdiv.addClass(wcore.CONST.CLASS_UNSEL);

	var button = $("<div/>");
	button.addClass(wcore.CONST.CLASS_EXPANDER_BUTTON);
	button.addClass(class_btn);
	button.addClass(wcore.CONST.CLASS_UNSEL);
	button.addClass(variantclass[side]);
	button.append(button_icon);

	var onOpen = function (ev) {
		var widget = ev.data['widget'];
		var button = ev.data['button'];
		button.empty();
		button.append(icon_close);
		if (cb_open)
		{
			cb_open.call(widget, button);
		}
//		wcore.reflowUpwards.call(widget);
		button.unbind();
		button.bind('click', {widget: widget, button: button}, onClose);
	};

	var onClose = function (ev) {
		var widget = ev.data['widget'];
		var button = ev.data['button'];
		button.empty();
		button.append(icon_open);
//		console.log("Running closer ",cb_close);
		if (cb_close)
		{
			cb_close.call(widget, button);
		}
//		wcore.reflowUpwards.call(widget);
		button.unbind();
		button.bind('click', {widget: widget, button: button}, onOpen);

	};

//	console.log("Binding expander to ",this);
	if (start_open)
	{
		button.bind('click', {widget: this, button: button}, onClose);
	}
	else
	{
		button.bind('click', {widget: this, button: button}, onOpen);
	}


	exdiv.append(button);
	return exdiv;

};


/* TEXT RENDERING FUNCTIONS for in-widget use */

wcore.CONST.CLASS_LISTKEY = "jsw2_rt_listkey";
wcore.CONST.CLASS_BOOLOBJ = "jsw2_rt_boolobj";
wcore.CONST.MARKUP_LBREAK = "<br>";
wcore.CONST.MARKUP_WBREAK = "<wbr/>";

wcore.CONST.COLOR_BOOL_TRUE = "#318600";
wcore.CONST.COLOR_BOOL_FALSE = "#d30500";
wcore.CONST.COLOR_BOOL_UNDEF = "#ffa400";
wcore.CONST.ICON_BOOL_TRUE = "ok-sign";
wcore.CONST.ICON_BOOL_FALSE = "remove-sign";
wcore.CONST.ICON_BOOL_UNDEF = "question-sign";

wcore.aux.getTallest = function (sel)
{
	var maxtall = 0;
	var tallid = null;
	for (var i = 0; i < sel.length; i++)
	{
		var cel = $(sel.get(i));
		var ch = cel.height();
		if (ch >= maxtall)
		{
			tallid = i;
			maxtall = ch;
		}
	}

	return $(sel.get(tallid));
};

wcore.aux.hyphenString = function (source)
{

	// returns a string that the browsers can break in lines more easily

	var match = "\\.,/@|?&=";
	var newstr = "";
	try
	{
		var len = source.length;
		if (typeof(len) != 'number')
		{
			return source;
		}
	}
	catch (err)
	{
		return source;
	}


	var intag = false;
	for (var i = 0; i < len; i++)
	{
		var csymbol = source[i];
		newstr += csymbol;
		if (csymbol === "<")
		{
			intag = true;
		}
		if (csymbol === ">")
		{
			intag = false;
		}
		if (!intag && match.indexOf(csymbol) != -1)
		{
			newstr += wcore.CONST.MARKUP_WBREAK;
		}
	}
	return newstr;
};

wcore.aux.markTextLinks = function (src)
{
	// finds hyperlinks in a string and wraps them in an anchor tag

	var marked;

	// todo: placeholder, implement
	marked = src;



	return marked;
};


wcore.aux.jqToMarkup = function (jqobj)
{
	// returns the complete markup of a jquery entity as string
	// (workaround for the limitations of the $.html() function

	var temp = $("<div/>");
	temp.append(jqobj);
	return temp.html();
};

wcore.aux.renderTextObject = function renderTextObject (src)
{
	// renders a single complex object into html
	// containers and self-rendering entities are handled by renderText

	var rendered = "";
	var sep = "";

	var hasValidType = src.hasOwnProperty('type');
	if (hasValidType)
	{
		// specialized renderers
		if (src.type === 'url')
		{
			var linkel = $("<a/>");
			var label = src.hasOwnProperty('label') && src.label!="" ? src.label : src.value;
			linkel.append(wcore.aux.hyphenString(label));
			linkel.prop('href', src.value);
			rendered = linkel;
		}
		else if (src.type === 'icon')
		{
			rendered = wcore.aux.renderFA(src['icontype'],src['icontext']);
		}
		else if (src.type === 'datetime')
		{
			var timestring = "";
			if (src.value != 0)
			{
				// assuming all timestamps we get come from Python
				var timestamp = new Date (wcore.aux.sec2msec(src.value));
				timestring = wcore.renderTimeString(timestamp);
			}
			rendered = wcore.aux.hyphenString(timestring);
		}
		else if (src.type === 'template')
		{
			// ICanHaz template renderer, requires properties: type(==template), template(name of the template), value(actual content)
			try
			{
				var templateString = wcore.aux.hyphenString(src.value);
				rendered = ich[src.template](templateString);
			}
			catch (err)
			{
				// missing iCanHaz or different kind of template passed inadvertedly
				hasValidType = false;
			}
		}
		else if (src.type === 'boolean')
		{
			rendered = $("<span/>");

			var iconstring = wcore.CONST.ICON_BOOL_UNDEF;
			var iconcolor = wcore.CONST.COLOR_BOOL_UNDEF;

			if (typeof src['value'] != 'undefined')
			{
				if (src['value'] === true)
				{
					iconstring =  wcore.CONST.ICON_BOOL_TRUE;
					iconcolor = wcore.CONST.COLOR_BOOL_TRUE;
				}
				else if (src['value'] === false)
				{
					iconstring = wcore.CONST.ICON_BOOL_FALSE;
					iconcolor = wcore.CONST.COLOR_BOOL_FALSE;
				}
			}
			var iconlabel = src.hasOwnProperty('note') ? wcore.aux.hyphenString(src.note) : null;
			var iconspan = wcore.aux.renderFA(iconstring, iconlabel);
			iconspan.addClass(wcore.CONST.CLASS_BOOLOBJ);
			iconspan.css("color", iconcolor);
			rendered.append(iconspan);
		}
		else if (src.type === 'note')
		{
			var spanel = $("<span/>");

			var pubDate = new Date();
			pubDate.setTime (wcore.aux.sec2msec(src.time));

			var objlink = "";
			if (src.hasOwnProperty('objlink'))
			{
				objlink = $("<a/>");
				objlink.attr('href', src['objlink']);
				objlink.text(src['objtext']);
			}
			else
			{
				objlink = src['objtext'];
			}

			var noteimg = "";
			if (src.hasOwnProperty('noteimage'))
			{
				noteimg = $('<img/>');
				noteimg.attr('src', src['noteimage']);
			}

			var colorspan = "";
			if (src.hasOwnProperty('color'))
			{
				colorspan = $('<span/>');
				colorspan.css('background-color', src['color']);
				colorspan.append("&nbsp;&nbsp;");
				colorspan.css('width', '2em');
			}

			spanel.append(colorspan);
			spanel.append(" ");
			spanel.append(objlink);
			spanel.append(" - ");
			spanel.append(src['user']);
			spanel.append(": ");
			spanel.append(pubDate);
			spanel.append("("+wcore.aux.hyphenString(src['notetext'])+")");

			rendered = spanel;
		}
		else if (src.type == 'wloader')
		{
			// verifying wloader structure
//			console.warn("WLOADER structure: ",src)
			if (!!src['wid'] && !!src['wtype'])
			{
				try
				{
					rendered = wcore.aux.preheat (src);
					// NOTE: since this is done outside the DOM, there may be rendering issues on certain widgets
					// hence we stop at the init+assemble though it may STILL have issues on size based widgets, that should be re-inited later
					var wlib = !!src['wlib'] ? src['wlib'] : "wcore";
					
					rendered[wlib]('winit', src['wtype']);
					
					//console.warn("pre-heated widget "+src['wid']+" _ "+src['wtype']);
				}
				catch (err)
				{
					console.warn("Error when trying to spawn wloader ",src," falling back to text render");
					hasValidType = false;
				}
			}
			else
			{
				console.warn("Textobj ",src," described as wloader but has the wrong structure, falling back to text render");
				hasValidType = false;
			}

		}
		else
		{
			// fallback to generic dict style rendering
			hasValidType = false;
		}
	}

	if (!hasValidType)
	{
		// generic dict style rendering
		var keys = Object.keys(src);
		rendered = $("<div/>");
		for (var i = 0; i < keys.length; i++)
		{
			var k = keys[i];
			var citem = wcore.aux.renderText(src[k]);
			var keyhead = $("<span/>");
			keyhead.addClass(wcore.CONST.CLASS_LISTKEY);
			keyhead.append(k);
			rendered.append(wcore.aux.jqToMarkup(keyhead)+": "+citem);
			sep = wcore.CONST.MARKUP_LBREAK;
		}
	}

	return rendered;

};

// FUNCTIONS TO KNOW IF A GIVEN DIV HOSTS A WIDGET (incl support for old LWP)

wcore.aux.isValidWidget = function (div)
{
	var hasWClass = div.hasClass(wcore.CONST.WIDGETCLASS_CORE);
	var isRegistered = !!div.data('wtype') && WDS.wlib.hasOwnProperty(div.data('wtype'));

	return hasWClass && isRegistered;
};

wcore.aux.isLegacyWidget = function (div)
{
	if (typeof(jswidgets) == 'undefined')
	{
		// if the old library is no longer registered itself no legacy widget can possibly exists
		return false;
	}

	var hasWClass = div.hasClass('jswlib_widget');
	var isRegistered = !!div.data('wtype') && jswidgets.hasOwnProperty(div.data('wtype'));
	return hasWClass && isRegistered;

};

wcore.aux.cook = function (div)
{
	console.log("wcore.aux.cook(" + div.data('id') + " " + div.data('wtype') + ")");
	
	var wtype = div.data('wtype');

	var wlib;
	if (wcore.aux.isValidWidget(div))
		wlib = 'wcore';
	else if (wcore.aux.isLegacyWidget(div))
		wlib = 'lwp';
	else
		throw "NoApplicableWidgetLibrary";

	div[wlib]('winit', wtype);
	div[wlib]('populate');
	div[wlib]('render');

};

wcore.aux.preheat = function (wloader)
{

	// returns a NON-inited widget div DOM structure

	if (typeof(wloader)!='object')
	{
		throw ("InvalidWloaderEntityException");
	}

	var widget = $("<div/>");
	if (!!wloader['remote'])
	{
//		var premote = wid+"."+wloader['remote'];
//		widget.data('premote', wloader['remote']);
		widget.attr('data-premote', wloader['remote']);
		widget.addClass("datadset_"+wloader['remote'].replace(".","_"));
	}

	if (!!wloader['rtimer'])
	{
		widget.data('rtimer', wloader['rtimer']);
	}

	if (wloader.hasOwnProperty('wdata'))
	{
		widget.data('wdata', wloader['wdata']);
	}

	var i;
	if ($.isArray(wloader['classes']))
	{
		for (i = 0; i < wloader['classes'].length; i++)
		{
			widget.addClass(wloader['classes'][i]);
		}
	}
	else if (typeof('wloader')['classes']=="string")
	{
		widget.addClass(wloader['classes']);
	}

	if ($.isPlainObject(wloader['attrs']))
	{
		widget.attr(wloader['attrs']);
	}

	// returns a rendered widget according to the parameters in the wloader object
	var wid = wloader['wid'];
	var wtype = wloader['wtype'];
	widget.attr('id', wid);
	widget.data('wtype', wtype);
	// added last to avoid overwriting by wloader['attrs']

	// default library is wcore
	var wlib = !!wloader['wlib'] ? wloader['wlib'] : 'wcore';

	// additional check that we are calling a valid widget type, only for wcore though
	if (wlib === 'wcore')
	{
		if (!WDS.wlib.hasOwnProperty(wtype))
		{
			throw "UnregisteredWidgetTypeError";
		}
	}

	return widget;

};

wcore.aux.renderText = function renderText (src)
{
	// "smart" HTML rendering for a string or object

	if (src === null)
	{
		src = "";
	}

	var datatype = typeof src;
	var rendered;
	var sep = "";
	if (datatype == "object")
	{
		if (src.hasOwnProperty('toString'))
		{
			// complex object but with its own rendering
			rendered = src.toString();
			rendered = wcore.aux.hyphenString(rendered);
		}
		else if ($.isArray(src))
		{
			rendered = "";
			// object is a container for other objects
			for (var i = 0; i < src.length; i++)
			{
				var citem = renderText(src[i]);
				rendered += sep + citem;
				sep = wcore.CONST.MARKUP_LBREAK;
			}
		}
		else
		{
			// no dedicated renderer built in
			rendered = wcore.aux.renderTextObject (src);
		}
	}
	else
	{
		// simple string or number
//		console.log("Rendering as clean text");
		rendered = $("<span/>").append(wcore.aux.hyphenString(src));
	}


	return rendered;

};


/*
 *  WIDGET LIFECYCLE SECTION (except for init)
 */

wcore.CONST.F_VALIDATE = 'validate';
wcore.CONST.F_EMERGE = 'emerge';
wcore.CONST.F_RENDER = 'wrender';

wcore.aux.extractDataSet = function (jsondict)
{
	// quick hack to get around data coming with or without a wrapping message
	return jsondict.hasOwnProperty('status') && jsondict.hasOwnProperty('info') && jsondict.hasOwnProperty('data') ? jsondict.data : jsondict;
};

wcore.populate = function (rawdata)
{
	/*
	 Populates the dataset with the data from which we will create the display
	 erases any pre-existing data in the WDS.data section
	 */

	var fval = wcore.CONST.F_VALIDATE;
	var fem = wcore.CONST.F_EMERGE;

//	console.warn("Rawdata for "+this.attr('id')+" is actually ",rawdata);

	if (typeof(rawdata)=='undefined')
	{
		// in case data is not supplied, trying to take it implicitly from wdata
		var divdata = $(this).data('wdata');
		if (typeof(divdata) == 'undefined')
		{
			console.warn("No data available for widget: " + $(this).attr('id'));
			return;
		}
		else
		{
			rawdata = $(this).data('wdata');
		}
	}

	var data = wcore.aux.extractDataSet(typeof(rawdata)!='object' ? JSON.parse(rawdata) : rawdata);

	// optional validation stage if supported by widget
	if (wcore.has.call(this, fval))
	{
		try
		{
			wcore.run.call(this, fval, [data]);
		}
		catch (err)
		{
			console.trace();
			console.log("POPULATE validation error: ",err+"\n",err.message);
		}
	}

//	console.log("Writing to data space: ",data);
	wcore.write.call(this, data, 'data');

	// optional emerge stage, otherwise data is funneled straight to WDS[wid][data]
	if (wcore.has.call(this, fem))
	{
		try
		{
			wcore.run.call(this, fem);
		}
		catch (err)
		{
			console.trace();
			console.log("POPULATE emerge error: ",err+"\n",err.message);
		}
	}

};

wcore.render = function ()
{
	var widget = $(this);
	
	//console.log(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>");

	//console.log("rendering " + widget.attr('id'));

	// to handle automatic updates
	wcore.async.unlock.call(widget);
	
	if (wcore.has.call(widget, 'wrender'))
	{

		try
		{
			wcore.run.call(widget, 'wrender');
		}
		catch (err)
		{
			var div_context = wcore.getW.call(widget);
			var wtype = wcore.getWType.call(widget);

			console.log("************************************************************************************************");
			var msg = "RENDERING error: " + err + " " + err.message + "\n" + "Error context: widget type '"+wtype+"' on div '" + div_context + "'";
			console.error(msg);
			console.log("----------TRACEBACK ------------");
			console.trace();
			console.log("--------------------------------");
			console.log("************************************************************************************************");
		}
	}

	//console.log("<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<");
};


/* BACKWARDS COMPATIBILITY SECTION */
wcore.hydrate = function ()
{
	if (wcore.has.call(this, 'wupdate'))
	{
		try
		{
			wcore.run.call(this, 'wupdate');
		}
		catch (err)
		{
			console.error(err);
			console.error("WUPDATE ERROR: "+err);
		}
	}
	else
	{
		// if a wupdate is missing we try using populate and render
		// but only if safe
		if ($(this).data('wincoming'))
		{
//			console.log("Running hydrate" );
			wcore.populate.call(this, $(this).data('wincoming'));
			//wcore.run.call(this, 'populate', [$(this).data('wincoming')]);
		}
	}
};

wcore.rejenerate = function ()
{
	/* redundant since the check is done by populate */

//	var raw = $(this).data('wdata');
//	var parsed = typeof(raw) == 'object' ? raw : JSON.parse(raw);
//	$(this).data('wdata', parsed);
//	console.log("Running rejenerate" );
	$(this)['wcore']('populate');
	
};



























// static functions for library handling etc.
// will contain initContainedWidget, buildWidgets and buildWidgetsTree

WSTAT = {};

WSTAT.clone = function clone (clonable)
{
	if (typeof clonable != 'object' || clonable == null)
	{
		return clonable;
	}

	var cloned, i;
	if (!$.isArray(clonable))
	{
		cloned = {};
		var keys = Object.keys(clonable);
		for (i = 0; i < keys.length; i++)
		{
			cloned[keys[i]] = clone (clonable[keys[i]]);
		}
	}
	else
	{
		cloned = [];
		for (i = 0; i < clonable.length; i++)
		{
			cloned.push(clone (clonable[i]));
		}
	}

	return cloned;
};

WSTAT.initContainedWidget = function initContainedWidget ($, widgetid, methods)
{
	$.fn[widgetid] = function (method)
	{
		// Method calling logic
		if ( methods[method] ) {
			return methods[ method ].apply( this, Array.prototype.slice.call( arguments, 1 ));
		} 
		else if ( typeof method === 'object' || !method ) {
			return methods.init.apply( this, arguments );
		} 
		else {
			$.error( 'Method ' +  method + ' does not exist on this plugin' );
		}
		//console.log("Created plugin named "+widgetid);
		return this.each(function() {});
	};
};

WSTAT.buildWidgets = function buildWidgets (widgettree, defaultmethods)
{
	var jswidgets = {};

	for (var i in Object.keys(widgettree))
	{
		var id = Object.keys(widgettree)[i];
		var methods = $.extend({}, defaultmethods, widgettree[id]);
		
		jswidgets[id] = methods;
		WSTAT.initContainedWidget(jQuery, id, methods);
	}
};

WSTAT.buildWidgetsTree = function buildWidgetsTree ()
{
	var jswlib = WDS.wlib;
	var widgettree = {};

	for (var i in Object.keys(jswlib))
	{
		var wid = Object.keys(jswlib)[i];
		widgettree[wid] = jswlib[wid];
	}

	WSTAT.buildWidgets(widgettree, wcore);
};



WDS.wlib['wcore'] = wcore;
