/**
 * Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
 * Created: 3/7/14, 11:41 AM
 * Author: Antonio Vaccarino
 */

/**
 * DEPENDENCY:
 * Mustache
 * for the templating functionality
 */

// simple HTML renderer, renders a string to straight html or via Mustache;

/* FORMAT
{
	"structure": Mustache template object if needed, use null or remove key to avoid,
	"values": data for rendering (string or object)
}
 */


var customdiv = {};

customdiv.validate = function validate (dataset)
{
	try
	{
		return typeof (dataset.values) =='string' || typeof (dataset.values) =='object';
	}
	catch (err)
	{
		return false;
	}
};

customdiv.wrender = function ()
{
	var widget = $(this);
	
	var canvas = wcore.getDisplay.call(widget);
	canvas.empty();
	
	var packed = wcore.read.call(widget, 'data');
	
	if (packed === undefined)
		return;
	
	// Proof of concet: Il dataset restituito puo' contenere un campo "error" per indicare
	//                  che ci sono stati problemi nel calcolare il dataset.
	var error  = packed['error'];
	if (error !== undefined) {
		canvas.append(wcore.aux.renderText(error));
		return;
	}
	
	
	var structure = packed['structure'];
	var data = packed['values'];

	var isValidTemplate = packed.hasOwnProperty('structure') && packed['structure']['template'];

	if (isValidTemplate)
	{
		try
		{
			var html = Mustache.to_html(structure['template'],data);
			canvas.append(html);
			
			if (structure.hasOwnProperty('active'))
			{
				var parent_id = wcore.getDisplay.call(widget).closest('.widget').attr('id').substring(4);
				
				var activable_element = canvas.find(structure.active);
				
				activable_element.data('parent_id', parent_id);
				
				activable_element.on('mouseover', function(ev){ $(ev.target).css('cursor', 'pointer'); });
				activable_element.on('mouseout' , function(ev){ $(ev.target).css('cursor', null);      });
				
				activable_element.on('click', function(e) {
					$(window.document).trigger('toggle_deps', [ $(this).data('parent_id'), $(e.target) ]);
				});
			}
		}
		catch (err)
		{
			// missing mustache or invalid template, falling back to rendertextcell
			console.error("Missing Mustache library or invalid template, falling back to wcore renderer");
			isValidTemplate = false;
		}
	}

	if (!isValidTemplate)
	{
		canvas.append(wcore.aux.renderText(data));
	}

};




// saving all widgets to library for finalization and use

WDS.wlib['customdiv'] = customdiv;
