/**
 * Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
 * Created: 3/13/14, 10:06 AM
 * Author: Antonio Vaccarino
 */


/*
	WIDGET per la creazione dei cartellinji degli elementi monjitorati in una lista
	Ogni istanza del widget renderizza un solo cartellino


FORMATO:
{
    "id": string // monitoring element id
    "type": string // type of element: tag | node | iface | storage | service
	"name": string,
	"clickable": boolean, // if the tag can be opened clicking, defaults to false
    "state":
    {
        "alarms"      : number, // number of issues on this element
        "img"         : string, // url icona (gia' colorata)
         "overlay"  : string // url dell'icona di overlay da aggiungere SOPRA all'immagine
    },
    "details":
    {
        "url_geo": string,
        "url_page": string
    },
	"measures": {
		 "type":  stringa  "BAR" (tipo x storage) o "PAIR"
		 "data":  valore multiplo
					val                     quando type = "BAR"
				   ( val1, val2 )           quando type = "PAIR"
	}
}


Struttura
========================================

    +--------------------------------------------------------------------------+
    |badge                                                                     |
    |  +-----------+  +-+  +------------------------------------------------+  |
    |  |quickview  |  | |  | details                                        |  |
    |  |  +-----+  |  | |  |  +------------------------------------------+  |  |
    |  |  |icon |  |  | |  |  | nametag                                  |  |  |
    |  |  |     |  |  | |  |  |  +------------------------------------+  |  |  |
    |  |  |     |  |  | |  |  |  |                                    |  |  |  |
    |  |  |     |  |  | |  |  |  +------------------------------------+  |  |  |
    |  |  |     |  |  | |  |  |                                          |  |  |
    |  |  +-----+  |  | |  |  |  +--+                                    |  |  |
    |  |           |  | |  |  |  |  |                                    |  |  |
    |  |           |  | |  |  |  +--+                                    |  |  |
    |  |           |  | |  |  +------------------------------------------+  |  |
    |  |           |  | |  |                                                |  |
    |  |           |  | |  |  +------------------------------------------+  |  |
    |  |           |  | |  |  | description                              |  |  |
    |  |           |  | |  |  |                                          |  |  |
    |  |           |  | |  |  |                                          |  |  |
    |  |           |  | |  |  |  +----------------------------------------+ |  |
    |  |           |  | |  |  |  | overmark                               | |  |
    |  |           |  | |  |  +--+                   V                    | |  |
    |  |           |  | |  |     +----------------------------------------+ |  |
    |  |           |  | |  |                                                |  |
    |  +-----------+  +-+  +------------------------------------------------+  |
    |                                                                          |
    +--------------------------------------------------------------------------+


*/


elementbadge = {};

elementbadge.CLASS_BADGE         = "jswlib2_badge_badge";
elementbadge.CLASS_SUB           = "jswlib2_badge_sub";
elementbadge.CLASS_LSIDE         = "jswlib2_badge_quickview";
elementbadge.CLASS_RSIDE         = "jswlib2_badge_details";
elementbadge.CLASS_ICON          = "jswlib2_badge_icon";
elementbadge.CLASS_NAMETAG       = "jswlib2_badge_nametag";
elementbadge.CLASS_NAME_TEXT     = "jswlib2_badge_name_text";
elementbadge.CLASS_NAMEEXPAND    = "jswlib2_badge_nameexpand";
elementbadge.CLASS_SPACER        = "jswlib2_badge_spacer";
elementbadge.CLASS_LINK          = "jswlib2_badge_link";
elementbadge.CLASS_OVERMARK      = "jswlib2_badge_overmark";

elementbadge.CLASS_BADGE_WARN    = "jswlib2_badge_warn";
elementbadge.CLASS_BADGE_ISSUES  = "jswlib2_badge_issues";
elementbadge.CLASS_BADGE_UNCHECK = "jswlib2_badge_uncheck";

elementbadge.GLYPH_GEO = "globe";
elementbadge.GLYPH_ELPAGE = "external-link";

elementbadge.CLASS_MINIWIDGET = "jswlib2_badge_miniw";
elementbadge.CLASS_MINIWIDGET_ICON = "jswlib2_badge_miniw_icon";
elementbadge.CLASS_MINIWIDGET_MINIGAUGE = "jswlib2_badge_miniw_gauge";
elementbadge.CLASS_MINIWIDGET_MINIGAUGE_INNER = "jswlib2_badge_miniw_gauge_inner";

elementbadge.MINIGAUGE_WIDTH = 46;

/* ***************************************************************************************************************** */
/* ***************************************************************************************************************** */
/* ***************************************************************************************************************** */
/* ***************************************************************************************************************** */
/* ***************************************************************************************************************** */


/*
elementbadge.ELTYPE_TAG = "tag";
elementbadge.ELTYPE_NODE = "node";
elementbadge.ELTYPE_IFACE = "iface";
elementbadge.ELTYPE_STORAGE = "storage";
elementbadge.ELTYPE_SERVICE = "service";

elementbadge.TYPES_ALL = [
	elementbadge.ELTYPE_TAG,
	elementbadge.ELTYPE_NODE,
	elementbadge.ELTYPE_IFACE,
	elementbadge.ELTYPE_STORAGE,
	elementbadge.ELTYPE_SERVICE
];

elementbadge.MEASURES_ALL = [ "BAR", "PAIR" ];

elementbadge.validate = function (data)
{

	console.warn("Validating ",data," for widget elementbadge");

	if (!data['id'] || !data['details'] || !data['state'])
	{
		console.error("Missing structure (id|details|status) in data: ",data);
		return false;
	}

	// not in use right now

//	if (elementbadge.TYPES_ALL.indexOf(data['type'])==-1)
//	{
//		console.error("Invalid or missing monitoring element type in data: ",data);
//		return false;
//	}

	if (!data.hasOwnProperty('name'))
	{
		console.error("Missing name in element data ",data);
		return false;
	}

	if (data.hasOwnProperty('measures'))
	{
		var mtype = data['measures']['type'];
		var mdata = data['measures']['data'];
		if (!mtype || !mdata)
		{
			console.error("Invalid measures data structure: ",data);
			return false;
		}
		else if (elementbadge.MEASURES_ALL.indexOf(mtype)==-1)
		{
			console.error("Unsupported measure type "+mtype);
			return false;
		}
		else if (mtype == "PAIR" && (!$.isArray(mdata) || mdata.length!=2))
		{
			console.error("Invalid measures data for type "+mtype+": ",mdata);
			return false;
		}
		else if (mtype == "BAR" && typeof(mdata)!='number')
		{
			console.error("Invalid measures data for type "+mtype+": ",mdata);
			return false;
		}
	}
};


elementbadge.wrender = function ()
{
	var display = wcore.getDisplay.call(this);
	display.empty();
	var data = wcore.read.call(this, 'data');

	var badge = $("<div/>");
	badge.addClass(elementbadge.CLASS_BADGE);

	var quickview = $("<div/>");
	quickview.addClass(elementbadge.CLASS_SUB);
	quickview.addClass(elementbadge.CLASS_LSIDE);

	var details = $("<div/>");
	details.addClass(elementbadge.CLASS_SUB);
	details.addClass(elementbadge.CLASS_RSIDE);

	var img = $("<img/>");
	img.addClass(elementbadge.CLASS_ICON);
	img.attr('src', data['state']['img']);
	quickview.append(img);

	var nametag = $("<div/>");
	nametag.addClass(elementbadge.CLASS_NAMETAG);
	nametag.append(data['name']);
	var spacer = "&nbsp;";
	if (data['details']['url_page'])
	{
		nametag.append(spacer);
		var icon_goto = wcore.aux.renderFA(elementbadge.GLYPH_ELPAGE);
		var span_goto = wcore.aux.renderLink(data['details']['url_page'], icon_goto);
		span_goto.addClass(elementbadge.CLASS_LINK);
		nametag.append(span_goto);
	}
	if (data['details']['url_geo'])
	{
		nametag.append(spacer);
		var icon_geo = wcore.aux.renderFA(elementbadge.GLYPH_GEO);
		var span_geo = wcore.aux.renderLink(data['details']['url_geo'], icon_geo);
		span_geo.addClass(elementbadge.CLASS_LINK);
		nametag.append(span_geo);
	}
	details.append(nametag);

	// layout element to avoid hiding spacers in margins 
	var spacerdiv = $("<div/>");
	spacerdiv.addClass(elementbadge.CLASS_SPACER);
	spacerdiv.addClass(elementbadge.CLASS_SUB);


	if (data['measures'])
	{
		// data has been already validated in .validate, so we assume the structure to be correct
		var measuredata = data['measures']['data'];
		var measuretype = data['measures']['type'];
		if (elementbadge.MINIWIDGETS.hasOwnProperty(measuretype))
		{
//			console.log("Spawning: ",elementbadge.MINIWIDGETS[measuretype]," for "+measuretype);
//			console.log(elementbadge.MINIWIDGETS);
			var subw = elementbadge.MINIWIDGETS[measuretype](measuredata);
			quickview.append(subw);
		}
	}


	badge.append(quickview);
	badge.append(spacerdiv);
	badge.append(details);

	var alarms = data['state']['alarms'];
	var hasalarms = typeof(alarms)=='number' && alarms > 0;
	var overlay = data['state']['overlay'];
	var hasoverlay = typeof(overlay) == 'string' && overlay.length > 0;

//	console.log("Has alarms? "+hasalarms+" (",alarms,")");
//	console.log("Has overlay? "+hasoverlay+" (",overlay,")");

	if (hasalarms)
	{
		var counter = $("<div/>");
		counter.addClass(elementbadge.CLASS_BADGE_ISSUES);
		counter.append(alarms);
		quickview.append(counter);
	}

	if (hasoverlay)
	{
		var warn = $("<img/>");
		warn.addClass(elementbadge.CLASS_BADGE_WARN);
		warn.attr("src", data['state']['overlay']);
		badge.append(warn);
	}


	display.append(badge);


	elementbadge.reflowDetails.call(this);
};

// miniwidgets static functions


elementbadge.renderMiniTraffic = function (data)
{
	var div = $("<div/>");
	div.addClass(elementbadge.CLASS_MINIWIDGET);

	var glyph_dl = wcore.aux.renderFA("long-arrow-down");
    var glyph_ul = wcore.aux.renderFA("long-arrow-up");
	glyph_dl.addClass(elementbadge.CLASS_MINIWIDGET_ICON);
	glyph_ul.addClass(elementbadge.CLASS_MINIWIDGET_ICON);

	var dl = data[0];
	var ul = data[1];
	var lbr = "<br>";
	var spacer = "&nbsp;";

    div.append(glyph_dl).append(spacer).append(dl).append(lbr).append(glyph_ul).append(spacer).append(ul);
    return div;

};

elementbadge.renderMiniGauge = function (data)
{
	var div = $("<div/>");
	div.addClass(elementbadge.CLASS_MINIWIDGET);
	div.addClass(elementbadge.CLASS_MINIWIDGET_MINIGAUGE);

	var inner = $("<div/>");
	inner.addClass(elementbadge.CLASS_MINIWIDGET_MINIGAUGE_INNER);

	if (typeof(data)!='number')
	{
		data = parseInt(data);
	}

	// forcing 0-100 scale
	var val = Math.max(Math.min(Math.round(data), 100), 0);

	// using a hardcoded constant because the CSS will only be applied when entered into the actual DOM
	// so we cannot know the padding needed to not overflow/touch borders with the container
	inner.width((elementbadge.MINIGAUGE_WIDTH * val)/100);
	div.append(inner);
	div.after(val+"%");

	return div;
};

// assembling the miniwidgets
elementbadge.MINIWIDGETS = {};
elementbadge.MINIWIDGETS["PAIR"] = elementbadge.renderMiniTraffic;
elementbadge.MINIWIDGETS["BAR"] = elementbadge.renderMiniGauge;
*/





/* ***************************************************************************************************************** */
/* ***************************************************************************************************************** */
/* ***************************************************************************************************************** */
/* ***************************************************************************************************************** */
/* ***************************************************************************************************************** */


elementbadge.wrender = function ()
{
	var display = wcore.getDisplay.call(this);
	display.empty();

	var data = wcore.read.call(this, 'data');

//	console.log("Data for badge: ",data);

	var clickable = data['clickable'] === true;

	if (data['measures']) {
		//
		// Poiche' MustacheJs non supporta costrutti IF/ELSE nei template dobbiamo
		// creare delle variabili fittizione per semplificare il flusso nel template.
		//
		if (data['measures']['type'] == 'BAR') {

			var val = data['measures']['data'];
			val = Math.max(Math.min(Math.round(val), 100), 0);

			var bar_width = (elementbadge.MINIGAUGE_WIDTH * val) / 100;

			data['measures']['gauge'] = {
				'bar_width': bar_width
			};
		}
		else {
			data['measures']['pair'] = {
				'0': data['measures']['data'][0],
				'1': data['measures']['data'][1]
			};
		}

	}

//	console.log("Data for templating ",data);
	var textspanner = function (text)
	{
		if (!text)
		{
			return text;
		}

		var spanned = $("<span/>");
		// spansize value
		var ss = 6;

		var breakers = [
			"-",
			"_",
			".",
			"/",
			"\\",
			"\n",
			"\t",
		];

		var splitted = [];
		var splitters = [];

		var csub = "";
		var wasUpper = true;
		for (var c = 0; c < text.length; c++)
		{
			var cchar = text[c];
			var isUpper = cchar == cchar.toUpperCase();
			if (breakers.indexOf(cchar)==-1)
			{
				csub += cchar;
			}
			else if (isUpper && wasUpper != isUpper)
			{
				// detecting camelcase wordchange
				csub += cchar;
				splitted.push(csub);
				splitters.push("");
				csub = "";
			}
			else
			{
				splitted.push(csub);
				splitters.push(cchar);
				csub = "";
			}
		}
		splitted.push(csub);
		splitters.push("");

//		var splitted = text.split(/[\s]/);
		for (var s = 0; s < splitted.length; s++)
		{
//			var prespan = splitted[s];
//			var len = prespan.length;
//			var spans = Math.ceil(len/ss);
//
//			for (var i = 0; i < spans; i++)
//			{
//				var cstr = prespan.substr(i * ss, ss);
//				if (cstr.length > 0)
//				{
//					spanned.append("<wbr>");
//					var subspan = $("<span/>").append(cstr);
//					spanned.append(subspan);
//
//				}
//			}
			spanned.append(splitted[s]);
			spanned.append(splitters[s]);
			spanned.append("<wbr>");
		}


		return spanned.html();

	};

	var details_name = textspanner(data['name']);

	var CONTENT_TEMPLATE = '\
			\
			<div class="jswlib2_badge_badge  jswlib2_badge_clickable"> \
					<div class="jswlib2_badge_sub jswlib2_badge_quickview"> \
						{{#state}} \
								<img class="jswlib2_badge_icon" src="{{img}}"> \
						{{/state}} \
						{{#measures}} \
							{{#gauge}} \
								<div class="jswlib2_badge_miniw jswlib2_badge_miniw_gauge"> \
									<div class="jswlib2_badge_miniw_gauge_inner" style="width: 34.5px;"> </div> \
								</div> \
								{{data}} % \
							{{/gauge}} \
							{{#pair}} \
								<div class="jswlib2_badge_miniw"> \
									<i class="fa fa-long-arrow-down"> </i> {{0}} \
									<br /> \
									<i class="fa fa-long-arrow-up"> </i>   {{1}} \
								</div> \
							{{/pair}} \
						{{/measures}} \
						{{#state}} \
							{{#alarms}}\
							<div class="jswlib2_badge_issues" title="There are {{alarms}} alarm(s) inside this element"> \
								{{alarms}} \
							</div> \
							{{/alarms}} \
						{{/state}} \
					</div> \
					\
					<div class="jswlib2_badge_spacer jswlib2_badge_sub"> \
					</div> \
					\
					<div class="jswlib2_badge_sub jswlib2_badge_details"> \
						<div class="jswlib2_badge_nametag"> \
							\
							{{#state}} \
								<div class="jswlib2_badge_name_title jswlib2_badge_name_text"> \
									'+details_name+'\
								</div> \
							{{/state}} \
							\
							{{#details}}\
								\
								{{#url_page}}\
									<a target="_blank" href="{{url_page}}" class="gui-button jswlib2_badge_link"><i class="fa fa-external-link "></i></a> \
								{{/url_page}}\
								{{^url_page}} \
								{{/url_page}}\
								\
								\
								{{#url_map}}\
									<a target="_blank" href="{{url_map}}" class="gui-button jswlib2_badge_link "><i class="fa fa-code-fork"></i></a>\
								{{/url_map}} \
								{{^url_map}} \
								{{/url_map}} \
								\
								\
								{{#url_adj}}\
									<a target="_blank" href="{{url_adj}}" class="gui-button jswlib2_badge_link "><i class="fa fa-link"></i></a>\
								{{/url_adj}} \
								{{^url_adj}} \
								{{/url_adj}} \
								\
								{{#url_geo}}\
									<a target="_blank" href="{{url_geo}}" class="gui-button jswlib2_badge_link "><i class="fa fa-globe"></i></a> \
								{{/url_geo}} \
								{{^url_geo}} \
								{{/url_geo}} \
								\
								</div>\
								\
								{{#url_delete}}\
									<div onclick="if (confirm(\'Confirm to delete this item from Tag?\')) { {{url_delete}};return false;} else return false;" class="jswlib2_delete_link"> \
										<i class="fa fa-times  "></i>\
									</div>\
								{{/url_delete}}\
							\
							{{/details}}\
							\
							{{#state}} \
								<div class="jswlib2_badge_name_description jswlib2_badge_name_text" > \
									{{description}}\
								</div> \
							{{/state}} \
					</div> \
					\
					{{#state}} \
						{{#overlay}}\
							<img title="Alarms for this element are above the selected critical level or created \'explosive\' states." class="jswlib2_badge_warn {{overlay_add_class}}" src="{{overlay}}"> \
						{{/overlay}}\
						{{^overlay}}\
						{{/overlay}}\
					{{/state}} \
			</div> \
	';
	
	//console.log(">>>>>>>>>>>>>>>>>>>>>>>>>>")
	//console.log(data)
	//console.log("<<<<<<<<<<<<<<<<<<<<<<<<<<")

	var html_content = Mustache.to_html(CONTENT_TEMPLATE, data);
	var badgel = $(html_content);
	var shortid = $(this).attr('id').substr(4);

	display.append( badgel );

	var checkable = display.find("." + elementbadge.CLASS_NAMETAG);
	var checkH = checkable.height();

	elementbadge.reflowDetails.call(this);

	if (clickable)
	{
		display.unbind();
		display.bind('click', function(e)
			{
				$(window.document).trigger('toggle_deps',[shortid, $(e.target)]);
			}
		);

		var anchors = display.find("a");
		anchors.unbind();
		anchors.bind('click', function (e)
		{
			// when clicking on links we do not open the tag
			e.stopPropagation();
		});
	
		var delete_link = display.find(".jswlib2_delete_link");
		delete_link.unbind();
		delete_link.bind('click', function (e)
		{
			// when clicking on links we do not open the tag
			e.stopPropagation();
		});
	}



};


/* ***************************************************************************************************************** */
/* ***************************************************************************************************************** */
/* ***************************************************************************************************************** */
/* ***************************************************************************************************************** */
/* ***************************************************************************************************************** */

elementbadge.reflowDetails = function ()
{
	var display = wcore.getDisplay.call(this);

	var container = $(display.find("." + elementbadge.CLASS_RSIDE)[0]);

	// removing a previous marker in case we are updating
	var oldovermark = display.find("." + elementbadge.CLASS_OVERMARK);
	oldovermark.remove();

	var maxH = container.height();

	//
	// Calcoliamo l'altezza effettiva di nome + icona + descrizione
	//
	var actualH = 0;
	container.children().each(function() {
		var h = $(this).prop('scrollHeight');
		actualH += h;
	});

	var overflowing = maxH < actualH;
	//console.log("Badge overflowing? "+overflowing+" "+actualH+" over "+maxH);

	if (overflowing) {
		var overmark = $("<div/>");

		overmark.addClass(elementbadge.CLASS_OVERMARK);
		overmark.append(wcore.aux.renderFA("chevron-down"));
		container.append(overmark);

		overmark.bind('click', {widget: this}, elementbadge.openOverflow);
	}
};

elementbadge.openOverflow = function (ev)
{
	var WIDGET = ev.data.widget;

	var data = wcore.read.call(WIDGET, 'data');

	var clickable = data['clickable'] === true;
	var shortid = WIDGET.attr('id').substr(4);

	ev.preventDefault();
	ev.stopPropagation();

	var widget = ev.data.widget;

	var display   = wcore.getDisplay.call(widget);
	var container = display.find("." + elementbadge.CLASS_BADGE );

	var content   = display.find("." + elementbadge.CLASS_RSIDE);

	var description = display.find(".jswlib2_badge_name_description");
	var overmark = content.find("." + elementbadge.CLASS_OVERMARK)



	container.removeClass('jswlib2_badge_clickable');
	container.css({'overflow': 'visible'});
	container.css({'z-index': 99999999 });

	description.addClass("jswlib2_badge_background");
	description.addClass("jswlib2_badge_clickable");

	overmark.hide();

	/* Resettiamo qualunque evento di click e lo reimpostiamo */
	display.off('click');
	display.bind('click', function(e) {
			ev.preventDefault();
			ev.stopPropagation();

			container.addClass('jswlib2_badge_clickable');
			container.css({'overflow': 'hidden'});
			container.css({'z-index': 'initial' });

			description.removeClass("jswlib2_badge_background");
			description.removeClass("jswlib2_badge_clickable");

			overmark.bind('click', {widget: this}, elementbadge.openOverflow);
			overmark.show();

			if (clickable)
			{
				display.unbind();
				display.bind('click', function(e)
					{
						$(window.document).trigger('toggle_deps',[shortid, $(e.target)]);
					}
				);
			}
	});

	return false;
};

// saving all widgets to library for finalization and use

WDS.wlib['elementbadge'] = elementbadge;
