/**
 * Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
 * Created: 2/28/14, 3:39 PM
 * Author: Antonio Vaccarino
 */


// handles sanet specific structures and elements


var notelist = {};

/* RENDERS THE LIST OF NOTES FOR THIS NODE
{
    "meta": {
        "element_id": string, // uuid of the element we are reading/adding notes tp
        "visible": number, // number of notes that are always visible (compact mode), total is taken from content length, 0 to show all notes by default
        "urls": {
            "refresh": string (url), // url to retrieve updated data (retrieves the whole creator message)
            "ops_remove": string (url), // url to remove a note,
            "ops_add": string (url), // url to add a note
        }
    },
    "content": [ NOTEOBJ ], // notes, sorted by time
}

NOTEOBJ = {
    "id": string (uuid), // id of the note as parameter for removal
    "date": number (timestamp), // timestamp for when the note was written, in seconds
    "user": string, // user that wrote the note
    "elem": string, // name of element (rootfs, eth0, etc)
    "url_elem": string (url), // url to sub element page
    "text": string, // text of the note
}

*/


/*



 */


notelist.validate = function (data)
{
	if (!data.hasOwnProperty('meta') || !data.hasOwnProperty('content'))
	{
		return false;
	}
	return true;
};

notelist.wrender = function ()
{
	var display = wcore.getDisplay.call(this);
	display.empty();
	
	var data = wcore.read.call(this, 'data');
	
	var listdiv = notelist.rendernotes.call(this, data, 10);
	display.append(listdiv);

//	wcore.reflowUpwards.call(this);
};

notelist.css = {};
notelist.css.CLASS_NOTE_ITEM   = 'jsw2_note_item';
notelist.css.CLASS_NOTE_HEADER = 'jsw2_note_header';
notelist.css.CLASS_NOTE_INFO   = 'jsw2_note_info';
notelist.css.CLASS_NOTE_BTN    = 'jsw2_note_btn';
notelist.css.ICON_ADD_NOTE     = "pencil-square-o";
notelist.css.ICON_DEL_NOTE     = "trash-o";
notelist.css.CLASS_NOTELIST    = "jsw2_notelist_holder";


ONE_SECOND = 1000

function replaceURLWithHTMLLinks(text) {
	var exp = /(\b(https?|ftp|file):\/\/[-A-Z0-9+&@#\/%?=~_|!:,.;]*[-A-Z0-9+&@#\/%=~_|])/i;
	return text.replace(exp,"<a href='$1' target='parent'>$1</a>");
} 


notelist.rendersinglenote = function (notedata)
{
	
	
	var ctx = {
		widget: this
	};
	var notediv = $("<div/>");
	notediv.addClass(notelist.css.CLASS_NOTE_ITEM);

//	var notedata = notes[i];
	ctx['noteid'] = notedata['id'];

	var noteheader = $("<div/>");
	noteheader.addClass(notelist.css.CLASS_NOTE_HEADER);

	var note_del = $("<div/>");
	note_del.addClass(notelist.css.CLASS_NOTE_BTN);
	note_del.append(wcore.aux.renderFA(notelist.css.ICON_DEL_NOTE));
	wcore.aux.setButton(note_del, ctx, notelist.requestRemoveNote);
	noteheader.append(note_del);

	var headerstr = $("<span/>");
	
	var datestr = wcore.aux.renderTimeString(notedata.date * ONE_SECOND);
	headerstr.append(datestr);
	
	if (notedata['elem'])
	{
		var elemlink = notedata['url_elem'];
//			console.log("link: "+elemlink);
		if (elemlink)
		{
			var elemspan = $("<a/>");
			elemspan.attr('href', elemlink);
			elemspan.append(notedata.elem);
		}
		else
		{
			elemspan = notedata.elem;
		}
		headerstr.append(" - ").append(elemspan);
	}
	headerstr.append(" ("+notedata.user+")");
	
	noteheader.append(headerstr);



	var noteinfo = $("<div/>");
	noteinfo.addClass(notelist.css.CLASS_NOTE_INFO);
	
	var visible_text = replaceURLWithHTMLLinks(notedata.text);
	
	noteinfo.append(visible_text);

	notediv.append(noteheader);
	notediv.append(noteinfo);

	return notediv;
};



notelist.rendernotes2 = function (data, max)
{
	var notes = data['content'];
	
	var items = notes.length;

	if (typeof(max)!='number')
	{
		max = items;
	}
	var needexpand = max < items;

	var ctx = {
		widget: this
	};

	var listdiv = $("<div/>");
	listdiv.addClass(notelist.css.CLASS_NOTELIST);
	
	var listdiv_overflow = $("<div/>");
	listdiv_overflow.addClass("notelist_overflow");

	var listhead = $("<div/>");

	var note_add = $("<div/>");
	note_add.addClass(notelist.css.CLASS_NOTE_BTN);
	note_add.append(wcore.aux.renderFA(notelist.css.ICON_ADD_NOTE));
	wcore.aux.setButton(note_add, ctx, notelist.requestAddNote);

	listhead.append(note_add);

	var list_title = $("<div/>");
	list_title.append("NOTES");
	list_title.addClass("jsw2_widget_title");
	listhead.append(list_title);

	listdiv.append(listhead);

	for (var i = 0; i < items; i++)
	{
		var notediv = notelist.rendersinglenote.call(this, notes[i]);

		if (i < max)
		{
			listdiv.append(notediv);
		}
		else
		{
			listdiv_overflow.append(notediv);
		}

	}
	listdiv.append(listdiv_overflow);


	var showsome = function (button)
	{
		this.find(".notelist_overflow").hide();
	};

	var showall = function (button)
	{
		this.find(".notelist_overflow").show();
	};

	if (needexpand)
	{
		var expander = wcore.aux.renderExpander.call(this, wcore.CONST.EXPANDER_SIDE_CENTER, showall, showsome);
		listdiv.append(expander);
		listdiv.css("margin-bottom", expander.height());
		showsome.call(listdiv)
	}

	return listdiv;

};


function ISODateString(d){
	
    function _pad(n){return n<10 ? '0'+n : n}
    
    return     d.getFullYear()    + '-'
        + _pad(d.getMonth()   +1) + '-'
        + _pad(d.getDate()      ) + ' '
        + _pad(d.getHours()     ) + ':'
        + _pad(d.getMinutes()   ) + ':'
        + _pad(d.getSeconds()   );
}


notelist.rendernotes = function (notes, max)
{
	var data = notes;
	
	data['can_edit'] = notes['meta']['can_edit'];
	
	var CONTENT_TEMPLATE = ' \
	<div class="jsw2_notelist_holder">  \
		<div>  \
			<!-- \
			<button class="jsw2_item_button jsw2_flag_notextsel add_note gui-button gui-gradient" \
			        style="{{^can_edit}}color:red;{{/can_edit}} icon16x16" \
				{{#can_edit}}title="Aggiungi nota"{{/can_edit}} \
				{{^can_edit}}title="Non hai i permessi per modificare le note"{{/can_edit}} \
			> \
				<span class="fa fa-pencil-square-o"></span>  \
			</button> \
			-->  \
			<b> Notes </b>  \
		</div> \
		<table width="100%" border=0 cellpading=0 cellspacing=0> \
			<tr> \
				<td align="center"> \
					<button \
						{{#can_edit}}\
							class="jsw2_item_button jsw2_flag_notextsel add_note gui-button gui-gradient" \
							title="Aggiungi nota"\
							style="icon16x16" \
						{{/can_edit}}\
						{{^can_edit}}\
							class="jsw2_item_button jsw2_flag_notextsel add_note gui-button gui-gradient gui-disabled" \
							title="Non hai i permessi per modificare le note" \
							style="icon16x16" \
						{{/can_edit}}\
					> \
						<span class="fa fa-pencil-square-o"></span>  New note \
					</button> \
				</td> \
			</tr> \
		{{#content}} \
			<tr> \
				<td> \
					<div class="jsw2_note_item">  \
						<div class="jsw2_note_header">  \
							\
							<div class="jsw2_note_btn jsw2_item_button jsw2_flag_notextsel" >  \
								\
								<button id="{{id}}" \
									\
									{{#can_edit}}\
										class="remove_note gui-button gui-gradient" \
										title="Cancella nota" \
										style="icon16x16" \
									{{/can_edit}}\
									{{^can_edit}}\
										class="remove_note gui-button gui-gradient gui-disabled" \
										title="Non hai i permessi per modificare le note" \
										style="icon16x16; cursor:pointer;" \
									{{/can_edit}}\
									\
									style="{{^can_edit}}color:red;"{{/can_edit}}; " \
								> \
									<i class="fa fa-trash-o"></i>  \
								</button> \
							</div>  \
							\
							<span>  \
								{{#timestamp2iso}}{{date}}{{/timestamp2iso}}  \
								-  \
								<a href="/web/page/node/{{id}}"> <a href="{{url_elem}}">{{elem}}</a>  \
								{{user}} \
							</span>  \
						</div>  \
						<div class="jsw2_note_info" style="word-break: break-all"> \
							{{#htmllinks}} \
							{{text}} \
							{{/htmllinks}} \
						</div>  \
					</div>  \
				</td> \
			</tr> \
		{{/content}} \
		{{^content}} \
			<tr> \
				<td align="center"> \
					* no notes * \
				</td> \
			</tr> \
		{{/content}} \
		</table> \
		<div class="notelist_overflow"></div>  \
	</div>  \
	';
	
	data['timestamp2iso'] = function () {
		return function (text, render) {
			return (ISODateString(new Date(parseInt(render(text)*1000))));
		}
	};	
	
	function replaceURLWithHTMLLinks(text) {
		
		// http(s) , ftp, file
		var url_exp    = /(\b(https?|ftp|file):\/\/[-a-zA-Z0-9+&@#\/%?=~_|!:,.;]*[-a-zA-Z0-9+&@#\/%=~_|])/gi;
		
		// mailto
		var mailto_exp = /(?:\b(mailto:([-a-zA-Z0-9+&@#\/%?=~_|!:,.;]*[-a-zA-Z0-9+&@#\/%=~_|])))/gi;

		var r = text;
		r = r.replace(url_exp   , "<a href='$1' target='parent'>$1</a>");
		r = r.replace(mailto_exp, "<a href='$1' target='parent'>$2</a>");

		//console.log(text, r);
		return r;
	} 
	
	data['htmllinks'] = function () {
		return function (text, render) {
			var s = render(text);
			s = s.replace(/&#x2F;/g, '/');
			return (replaceURLWithHTMLLinks(s));
		}
	};
	
	var html_content = Mustache.to_html(CONTENT_TEMPLATE, data)
	
	var element = $(html_content);
	

	var eventdata = {
		widget: this
	};	
	
	if (data['can_edit']) {
	
		element.find('.add_note'   ).bind('click', eventdata, notelist.requestAddNote);
	
		//ctx['noteid'] = notedata['id'];	
		element.find('.remove_note').bind('click', eventdata, notelist.requestRemoveNote);
	}
	
	return element
}


notelist.requestRemoveNote = function (ev)
{
	var widget = ev.data.widget;
	
	if ($(ev.delegateTarget).hasClass('gui-disabled'))
		return;
	
	var noteid = $(ev.delegateTarget).attr('id');

	var urls = wcore.read.call(widget, 'data')['meta']['urls'];

	var data = wcore.read.call(widget)['data'];
	var url = data['meta']['urls']['ops_remove'];
	
	var params = {
		lock: true,
		silent: false,
		success_callback: notelist.eraseNote,
		error_callback  : notelist.eraseNote_error
	};

	var payload = {
		"noteid": noteid
	};

	wcore.async.ajax_json.call(widget, url, payload, params);
};

notelist.eraseNote_error = function (data, request)
{
	var widget = this;
	wcore.aux.ShowErrorDialog.call(widget, wcore.CONST.MESSAGE_AJAX_OPFAILED, data['info']);
}

notelist.eraseNote = function (data, request)
{
	// removes note from rendering after note has been removed server side
	// todo: placeholder, implement;

	//console.log("Received erasure callback data as ",data," from req ",request);

	var noteid = data['data']['uuid'];
	var notes = wcore.read.call(this)['data']['content'];

	//console.log("Removing from ",notes);
	for (var i = 0; i < notes.length; i++)
	{
		var cid = notes[i]['id'];
		if (cid === noteid)
		{
			notes.splice(i, 1);
			break;
		}
	}
	notelist.wrender.call(this);

};

notelist.FORM_ROWS = 4;
notelist.FORM_COLS = 50;
notelist.FORM_HINT = "Descrizione della nota";
notelist.FORM_TITLE = "Nuova nota";
notelist.CLASS_FORM_ADD = "jsw2_notelist_form_add";

notelist.requestAddNote = function (ev)
{

	var widget = wcore.getW.call(this);

	//console.warn("requestAddNote context ev ",ev," this ",this," widget ",widget);

	var formdiv = $("<div/>");

	var textfield = $("<textarea/>");
	textfield.attr({
		rows: notelist.FORM_ROWS,
		cols: notelist.FORM_COLS,
		wrap: "hard",
		placeholder: notelist.FORM_HINT
	});
	textfield.css({
		width: "100%"
	  });

	formdiv.append(textfield);

	formdiv.dialog({
		autoOpen: true,
		modal: true,
		title: notelist.FORM_TITLE,
		closeOnEscape: true,
		dialogClass: notelist.CLASS_FORM_ADD,
		buttons: {
			"create": {
				text: "crea nota",
				click: function ()
				{
					var notetext = textfield.val();
					notelist.sendRequestCreateNote.call(widget, notetext);
					$(this).dialog('destroy');
					$(this).remove();
				}
			},
			"cancel": {
				text: "annulla",
				click: function ()
				{
					$(this).dialog('destroy');
					$(this).remove();
				}
			}
		}
	});
};

notelist.sendRequestCreateNote = function (text)
{

	var widget = $(this);
	//console.log('Sending request for note '+text+' creation from ',widget);
	//console.log("Data space: ",wcore.read.call(widget));

	var data = wcore.read.call(widget)['data'];
	var url = data['meta']['urls']['ops_add'];
	var params = {
		lock: true,
		silent: false,
		success_callback: notelist.addNewNote,
		error_callback  : notelist.addNewNote_error
	};

	var payload = {
		element_id: data['meta']['element_id'],
		text      : text
	};

	wcore.async.ajax_json.call(widget, url, payload, params);
};

notelist.addNewNote_error = function (data, request)
{
	var widget = this;
	wcore.aux.ShowErrorDialog.call(widget, wcore.CONST.MESSAGE_AJAX_OPFAILED, data['info']);
}

notelist.addNewNote = function (data)
{
	console.log("Received data from ajax return ",data);

	var wdata = wcore.read.call(this)['data'];

	//console.log("adding to ",wdata);

	var notedata = data['data'];

	wdata.content.unshift(notedata);

	notelist.wrender.call(this);
};





// saving all widgets to library for finalization and use

WDS.wlib['notelist'] = notelist;
