/**
 * Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
 * Created: 3/26/14, 10:28 AM
 * Author: Antonio Vaccarino
 */

/*
FORMAT

{
	cols: [ COLDEF  ],
	rows: [ ROWDEF ],
	styles: {
		stylename: object, // css style dict description, stylename is the classname associated to it
	},
	tablestyle: string|object, // css class descriptor if string, css dict if object

}

COLDEF
{
	label: string|object, // header for the column, treated as a renderText entity
	width: string, // css definition, will be passed as it is to css, optional
	weight: int, // relative size definition, the table takes all cols with a WEIGHT and divides a % size definition amonmg them according to their relative weight, optional and ignored if width is defined too
	style: string|object, // css class descriptor if string, css dict if object
}
NOTE: weight and width should never be mixed together if at all possible (since weight is translated to percentage)

ROWDEF
{
	label: string|object, // header for the row, treated as a renderText entity
	style: string|object, // css class descriptor if string, css dict if object
	height: int, // height in pixels, optional (needed for some widgets)
	content: [ CELLDEF ]
}

CELLDEF
{
	content: string|object, // data rendered by RenderText function
	style: string|object, // css class descriptor if string, css dict if object
}

NOTE: if the style string in COLDEF/ROWDEF/CELLDEF describes a class that exists in the STYLES dict of the FORMAT, the widget does not assign that class but simply fill the CSS for the element with the values defined for the in-widget STYLE
 */

tablegrid = {};

tablegrid.assemble = function ()
{
	//console.log("Assembling tablegrid (placeholder)");

};

tablegrid.validate = function ()
{
	//console.log("Validating tablegrid (placeholder)");
	//todo: placeholder, implement
	return true;
};

/*tablegrid.emerge = function ()
{
	//todo: placeholder, implement
	// so we add the keys that are optional in case they are missing
	// and simplify the wrender (no data structure checks)
		console.log("Emerging tablegrid (placeholder)");

};*/



tablegrid.CLASS_TABLE_MAIN = "jswlib2_tg_main";
tablegrid.CLASS_TABLE_ROW = "jswlib2_tg_row";


tablegrid.wrender = function ()
{

	//console.log("wrendering tablegrid");
	var display = wcore.getDisplay.call(this);
	display.empty();

	var data = wcore.read.call(this, 'data');

	var styles = data['styles'];

	var i, ccol, html;

	var table = $("<table style='border: 0px' />");
	var thead = $("<thead/>");

	var tablestyle = data['tablestyle'];
	if (tablestyle != undefined && tablestyle != null)
	{
		tablegrid.applyStyleDef(table, tablestyle, styles);
	}


	//
	// Le righe hanno la label in testa orizzontale?
	//
	var rows_have_label = false;
	for (var r = 0; r < data['rows'].length; r++)
		if (data['rows'][r]['label'] != undefined && data['rows'][r]['label'].length > 0)
			rows_have_label = true;

	var cols_have_title = false;
	for (var c = 0; c < data['cols'].length; c++)
		if (data['cols'][c]['label'] != undefined && data['cols'][c]['label'].length > 0)
			cols_have_title = true;

	//
	// Calcoliamo i pesi di ogni colonna e il peso complessivo in modo da poter definire la larghezza delle colonne
	//
	var weights = [];
	var weightsum = 0;
	for (i = 0; i < data['cols'].length; i++)
	{
		var ccol = data['cols'][i];
		
		if (ccol['width'] == null && typeof (ccol['weight']) == 'number')
		{
			weights[i] = ccol['weight'];
			weightsum += ccol['weight'];
		}
		else
		{
			weights[i] = null;
		}
	}

	//
	// La prima colonna e' quella delle etichetthe delle righe (se ci sono)
	//
	if (rows_have_label) {
		table.append($("<colgroup><col/></colgroup>"));
		thead.append($("<th/>"));
	}

	for (i = 0; i < data['cols'].length; i++)
	{
		var ccol = data['cols'][i];

		var colgroup = $("<colgroup/>");

		// Titolo della colonna
		if (cols_have_title) {
			var headcell = $("<th/>");
			headcell.append(ccol['label'])
			thead.append(headcell);
		}

		var html = $("<col/>");

		if (weights[i] != null)
		{
			// set as weight
			var wval = (ccol['weight'] / weightsum) * 100;
			
			//console.log("Setting size of col "+i+" to weight "+wval);
			colgroup.css('width', wval+"%");
		}
		else if (ccol['width'] != null)
		{
			// set as width
			//console.log("Setting size of col "+i+" to width "+ccol['width']);
			colgroup.css('width', ccol['width']);
		}

		if (ccol['style'] != undefined && ccol['style'] != null)
			tablegrid.applyStyleDef (html, ccol['style'], styles);

		colgroup.append(html);
		table.append(colgroup)
	}

	if (cols_have_title) {
		table.append(thead);
	}

	var spawned = [];

	for (i = 0; i < data['rows'].length; i++)
	{
		var crow = data['rows'][i];
		
		html = $("<tr/>");
		
		if (crow['style'] != undefined && crow['style'] != null)
			tablegrid.applyStyleDef (html, crow['style'], styles);

		// La prima colonna e' la label della riga
		if (rows_have_label) {
			var rowhead = $("<td/>");
			rowhead.append(crow['label']);
			html.append(rowhead);
		}

		// Le altre colonne
		for (var c = 0; c < crow['content'].length; c++)
		{
			var ccell = crow['content'][c];

			var el = $("<td/>");

			if (ccell['style'] != undefined && ccell['style'] != null)
				tablegrid.applyStyleDef (el, ccell['style'], styles);

			var rendered = wcore.aux.renderText(ccell['content']);

			el.append(rendered);

			if (wcore.aux.isValidWidget(rendered) || wcore.aux.isLegacyWidget(rendered))
			{
				spawned.push(rendered);
			}

			html.append(el);
		}
		table.append(html);
	}
	display.append(table);



	// activating widgets
	for (i = 0; i < spawned.length; i++)
	{
		wcore.aux.cook($(spawned)[i]);
	}

};


tablegrid.applyStyleDef = function (jq, styledef, styles)
{
	// wraps the detection of available styles and strings so we don't repeat it in several points of wrender
	// made basically static to avoid reopening the data() space of the widget (wrender already opened it)
	if (!styles)
	{
		styles = {};
	}
	else{
		jq.css(styles);
	}
	
	if (typeof(styledef)=='object')
	{
		jq.css(styledef);
	}
	else if (typeof(styledef)=='string')
	{
		if (styles.hasOwnProperty(styledef))
		{
			jq.css(styles[styledef])
		}
		else
		{
			jq.addClass(styledef);
		}
	}

	return jq;
};

WDS.wlib['tablegrid'] = tablegrid;
