/**
 * Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
 * Created: 3/10/14, 10:01 AM
 * Author: Antonio Vaccarino
 */
/**
 * Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
 * Created: 3/10/14, 10:01 AM
 * Author: Antonio Vaccarino
 */

/*

Defines a vertical box that holds a number of objects that can be also contained in sub-boxes
NOTE: the widgets defined inside the layout are all inited. if there is data for  them, they are also populated and rendered after the main rendering,
whether they are visible or not (the flap acts as a hide/show control only)

FORMAT
{
	"struct": [ CONTAINED ]
}

CONTAINED = {
	"type": string, // "wframesub"|"widget"|"legacy" (legacy is for lwp widgets only and should NOT be used if at all possible)
	"hideable": boolean, // whether the object is under a flap button or not, false by default
	"visible": boolean, // default state, ignored if !hideable
	"buttonpos": int, // 0|1|2 (CENTER|LEFT|RIGHT) position for the flap button, ignored if !hideable
	"contents": Object, // [CONTAINED]|WLOADER, [CONTAINED] if "wframesub", WLOADER if "widget"
}

WLOADER = {
	"id": string, // widget id
	"wtype": string, // widget type as per library dict
	"rdata": string/Object, // data for populating and rendering the widget, optional; note: if string, it will be JSON.parsed
	"remote": string, // url to be used to retrieve wdata in case wdata is missing; used only if wdata is missing or empty (0 len inside)
	"rtimer": number, // seconds between refreshes of the widget (ignored if 0)

}

*/

widgetframe = {};

widgetframe.validate = function (data)
{

	// note: we can assume data IS an object (so keys etc) because wcore handles that transformation before passing the data to validate

	var validated = true;
	if (!$.isArray(data['struct']))
	{
		validated = false;
	}
	else
	{
		validated = widgetframe.validate_struct (data.struct);
	}
	return validated;
};

// field names in the WLOADER structure
// NOTE: wid/wtype/wdata are the same fields used by the PAGE
widgetframe.FIELDS_SUBW_WID = "id";
widgetframe.FIELDS_SUBW_WTYPE = "wtype";
widgetframe.FIELDS_SUBW_WDATA = "rdata";
widgetframe.FIELDS_SUBW_WURL = "remote";
widgetframe.FIELDS_SUBW_TIMER = "rtimer";
widgetframe.FIELDS_SUBW_SPINNER = "showspinner";


widgetframe.validate_struct = function validate_struct (struct)
{
	var validated = true;
	
	for (var i = 0; i < struct.length; i++)
	{
		var el = struct[i];
		if (!el.hasOwnProperty('type'))
		{
			validated = false;
			break;
		}
		else if (el.type == 'wframesub')
		{
			// if sub-container, we inspect the containe
			if (!$.isArray(el['contents']))
			{
				validated = false;
				break;
			}
			else
			{
				validated = validate_struct(el['contents']);
				if (!validated)
				{
					break;
				}
			}
		}
		else if (el.type == 'widget')
		{
			if (typeof (el['contents']) != 'object' || !el.contents.hasOwnProperty(widgetframe.FIELDS_SUBW_WID) || !el.contents.hasOwnProperty(widgetframe.FIELDS_SUBW_WTYPE))
			{
				//console.error("Malformed widget declaration in widgetframe: ",el);
				validated = false;
			}
			else if (!WDS.wlib.hasOwnProperty(el.contents[widgetframe.FIELDS_SUBW_WTYPE]))
			{
				//console.error("Unknown widget requested in widgetframe: ",el);
				validated = false;
			}
		}
		else if (el.type == 'legacy')
		{
			if (typeof (el['contents']) != 'object' || !el.contents.hasOwnProperty(widgetframe.FIELDS_SUBW_WID) || !el.contents.hasOwnProperty(widgetframe.FIELDS_SUBW_WTYPE))
			{
				//console.error("Malformed widget declaration in widgetframe: ",el);
				validated = false;
			}
		}
		else
		{
			// unrecognized type
			console.error ("Unrecognized widgetframe sub element type "+el.type+" from element ",el);
			validated = false;
		}
	}
	return validated;
};


widgetframe.wrender = function wrender ()
{
	var display = wcore.getDisplay.call(this);
	var data = wcore.read.call(this, 'data');

	// data has been already validated, no need to do an explicit check now
//	console.log("Rendering widgetframe with data ", data);

	var render = widgetframe.renderContents.call(this, data.struct);
	display.append(render);
};



// div framing sub-elements
widgetframe.CLASS_FRAME = "jswlib2_wframe_frame";
// objects under a fold button (css: class_frame > class_foldable)
widgetframe.CLASS_FOLDABLE = "jswlib2_wframe_foldable";
// objects HIDDEN under a fold button
widgetframe.CLASS_FOLDED = "jswlib2_wframe_folded";

widgetframe.showContents = function (button)
{
	var parent  = button.closest("."+wcore.CONST.CLASS_EXPANDER_DIV);
	
	var children = parent.children("."+widgetframe.CLASS_FOLDABLE);

//	var firstChild = $(children.get(0));
//	firstChild.css("margin-top", button.height());
//	console.log("Showing children ",children," of parent ",parent," from button ",button);

	children.show();

/*	for (var i = 0; i < children.length; i++)
	{
		var cchild = $(children.get(i));
		var widgetsel = cchild.find("."+wcore.CONST.WIDGETCLASS_CORE);
		for (var w = 0; w < widgetsel.length; w++)
		{
			var cw = $(widgetsel.get(w));
			wcore.reflow.call(cw);
		}
	}*/

//	wcore.reflowUpwards.call(this);
};

widgetframe.showContentsUpdateAll = function(button)
{
	var parent   = button.closest("."+wcore.CONST.CLASS_EXPANDER_DIV);
	var children = parent.children("."+widgetframe.CLASS_FOLDABLE);
	
	children.show();
	
	var widgets = button.parents('.widget').find('.jsw2_widget');
	
	for( var subwidget = 0; subwidget < widgets.length; subwidget++ )
		$(widgets[subwidget])['wcore']('update');
}

widgetframe.hideContents = function (button)
{
	var parent = button.closest("."+wcore.CONST.CLASS_EXPANDER_DIV);
	var children = parent.children("."+widgetframe.CLASS_FOLDABLE);
//	console.log("Hiding children ",children," of parent ",parent," from button ",button);
	children.hide();


//	wcore.reflowUpwards.call(this);
};

widgetframe.renderContents = function renderContents (contents)
{

	var render    = $("<div/>");
	var updateAll = false;
	render.addClass(widgetframe.CLASS_FRAME);

	for (var i = 0; i < contents.length; i++)
	{
		// first we render the object, then  we bind it to the flap if neededa
		var el = contents[i];
		var struct = el.contents;

//		console.warn("CONTENTS FOR SUB: ",struct);

		var isSub = el.type == "wframesub";


		var div;
		if (!isSub)
		{
			// rendering as widget

			var wlib;
			if (el.type == 'widget')
			{
				wlib = "wcore";
			}
			else if (el.type == "legacy")
			{
				wlib = "lwp";
			}

			div = $("<div/>");
			var div_id = struct[widgetframe.FIELDS_SUBW_WID];
			var div_type = struct[widgetframe.FIELDS_SUBW_WTYPE];
			div.attr('id', div_id);

			var hasurl = !!struct[widgetframe.FIELDS_SUBW_WURL];
			var hastimer = !!struct[widgetframe.FIELDS_SUBW_TIMER];

			if (hasurl)
			{
				var url = struct[widgetframe.FIELDS_SUBW_WURL];
				div.addClass("datadset_"+url.replace(".","_"));
//				console.log("Setting premote for ",div," as ",url);
				div.data('premote', url);
			}

			if (hastimer)
			{
				var timer = struct[widgetframe.FIELDS_SUBW_TIMER];
//				console.log("Setting rtimer for ",div," as ",timer);
				div.data('rtimer', timer);
			}

			if (struct[widgetframe.FIELDS_SUBW_SPINNER]===true)
			{
				div.data(widgetframe.FIELDS_SUBW_SPINNER, true);
			}


			div[wlib]('winit', div_type);

			var owndata = struct[widgetframe.FIELDS_SUBW_WDATA];
			var wdata_type = typeof(owndata);
			
			var isEmpty = (owndata == null || wdata_type == 'undefined');
			
			var possiblyEmpty = false;
			
			if (isEmpty) {
				possiblyEmpty = true;
			}
			else {
				//var possiblyEmpty = isEmpty;
				
				if ($.isArray(owndata) || wdata_type == 'string')
				{
					possiblyEmpty = owndata.length == 0;
				}
				else if (wdata_type == 'object')
				{
					possiblyEmpty = Object.keys(owndata).length == 0;
				}
			}



//			console.log("Subwidget "+div_id+" ("+div_type+") generation: isEmpty "+isEmpty+" or possiblyEmpty "+possiblyEmpty+" with url "+hasurl);

			if (!possiblyEmpty || (possiblyEmpty && !hasurl))
			{
//				console.log("Direct render");
				div[wlib]('populate', owndata);
				div[wlib]('render');
			}
			else if (hasurl)
			{
				var subw = div;

//				console.log("Retrieving sub data from "+url);

				// todo: find more integrated solution
				if (NS && NS.hasOwnProperty('delay_dl'))
				{
//					console.log("Network render via NS.delay_dl with lib "+wlib);
					// if the page offers a download+init mechanism, use it
					NS['delay_dl'](url, null, true);
				}
				else
				{
//					console.log("Network render via straight ajax");
					// otherwise parse as generic url
					var ajaxparams = {
						custom: true,
						silent: true,
						lock: false,
						callback: function (data)
							{
								subw.data('wdata', data);
								console.log("loading ",data," in wdata of ",subw," with lib "+wlib);
								subw[wlib]('populate');
								subw[wlib]('render');
							}
					};

					wcore.async.ajax.call(this, url, null, ajaxparams);
				}



			}
		}
		else
		{
			// rendering as sub section
			div = renderContents(struct);
			updateAll = 'updateAll' in el ? el['updateAll'] : false;
		}

		var foldable = el['hideable'] === true;
		if (!foldable)
		{
			render.append(div);

		}
		else
		{
			div.addClass(widgetframe.CLASS_FOLDABLE);
			div.addClass(wcore.CONST.CLASS_UNSEL);
			
			if (updateAll)
				var expander = wcore.aux.renderExpander.call(this, wcore.CONST.EXPANDER_SIDE_LEFT, widgetframe.showContentsUpdateAll, widgetframe.hideContents);
			
			else
				var expander = wcore.aux.renderExpander.call(this, wcore.CONST.EXPANDER_SIDE_LEFT, widgetframe.showContents, widgetframe.hideContents);
			
			
			expander.append(div);


			render.append(expander);
			if (el['visible']===true)
			{
				// expanders are closed at creation time
				var button = expander.find(wcore.CONST.CLASS_EXPANDER_BUTTON);
				
//				console.log("pre-opening expander via ",button);
				button.trigger('click');
			}
			else
			{
				div.hide();
			}
		}
		
		
	}
	return render;
};


// saving all widgets to library for finalization and use

WDS.wlib['widgetframe'] = widgetframe;
