# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from cli.prompts import cli_register_menu, cli_register_hook

from tags.cli.prompts.show import _menu_show_tag
from tags.cli.prompts.show import _menu_show_log_tag
from tags.cli.prompts.show import _menu_show_conf_tagtree

from tags.cli.prompts.show import _menu_show_node_tag
from tags.cli.prompts.show import _menu_show_datagroup_tag
from tags.cli.prompts.show import _menu_show_datasource_tag
from tags.cli.prompts.show import _menu_show_condition_tag
from tags.cli.prompts.show import _menu_show_timegraph_tag

from tags.cli.prompts.show import show_conf_tags
from tags.cli.prompts.show import show_configuration_this_tagtree
from tags.cli.prompts.show import tag_show_node_tag_meta


from tags.cli.prompts.config import menu_configure_tagtree
from tags.cli.prompts.config import no_menu_configure_tagtree
from tags.cli.prompts.config import menu_rename_tagtree

from tags.cli.prompts.config import menu_configure_tag, menu_tag_diff

from cli.clilib.ioslike import T_Keyword
from cli.tokens import T_MonitoredNodecompleter

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_launch_tag_dialog(cli):
	
	# tag node <name>

	name         = cli.canonical_words[2]
	
	from django.core.management import call_command
	
	call_command('tag_element', name, interactive=False, verbosity=True, show_traceback=True)

	return

mt_tag_commands = [

	T_Keyword('tag', help='Show tag informatinos', subtree=[
	
		T_Keyword('node', help='Show tag informatinos', subtree=[
			T_MonitoredNodecompleter(help="Show tag's informations", callback=cmd_launch_tag_dialog )
		]),
	]),
]

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

from tags.server.tagsd.api import ClientTagStateSocketUDP

def cmd_status(cli):

	client = ClientTagStateSocketUDP()
	
	#client.connect()
	
	if   cli.canonical_words[-1] == 'conditions': client.reload_conditions()
	
	elif cli.canonical_words[-1] == 'refresh'   : client.refresh_all()
	
	elif cli.canonical_words[-1] == 'force'     : client.refresh_all_force()
	
	client.close()

menu_execute_tag_commands = [
	T_Keyword('status', help='', pipe=None, subtree=[
		T_Keyword('reload', help='', pipe=None, subtree=[
			T_Keyword('conditions', help='', pipe=None, callback=cmd_status),
		]),
		T_Keyword('refresh', help='', pipe=None, callback=cmd_status, subtree=[
			T_Keyword('force', help='', pipe=None, callback=cmd_status)
		])
	])
]

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def register_cli_hooks():

	#
	# Show section
	#
	cli_register_menu('show'                     , 'tag', _menu_show_tag)                 # show       tag ....
	cli_register_menu('show:log'                 , 'tag', _menu_show_log_tag)             # show log   tag ...
	cli_register_menu('show:configuration'       , 'tag', _menu_show_conf_tagtree)        # show conf  tagtree ....

	cli_register_menu('show:node'                , 'tag', _menu_show_node_tag)            # show node  tagtree ....
	cli_register_menu('show:datagroup'           , 'tag', _menu_show_datagroup_tag)       # show datagroup  tagtree ....
	cli_register_menu('show:datasource'          , 'tag', _menu_show_datasource_tag)      # show datasource tagtree ....
	cli_register_menu('show:condition'           , 'tag', _menu_show_condition_tag)       # show condition  tagtree ....
	cli_register_menu('show:timegraph'           , 'tag', _menu_show_timegraph_tag)       # show timegraph  tagtree ....

	cli_register_hook('show:node'                , 'tag', tag_show_node_tag_meta)            # show node  tagtree ....

	cli_register_hook('show:tenant:configuration', 'tag', show_conf_tags)    
	cli_register_hook('show:configuration:this'  , 'tag', show_configuration_this_tagtree)

	
	#
	# Config section
	#
	cli_register_menu('configuration:tenant'       , 'tagtree', menu_configure_tagtree)
	cli_register_menu('configuration:tenant:no'    , 'tagtree', no_menu_configure_tagtree)
	cli_register_menu('configuration:tenant:rename', 'tagtree', menu_rename_tagtree)

	#
	# Copy tag between nodes and show node tag difference
	#
	
	cli_register_menu('configuration:tenant', 'copy', menu_configure_tag)
	cli_register_menu('show', 'diff', menu_tag_diff)
	
	#cli_register_menu('__apps_commands__'          , 'tag'    , mt_tag_commands)

	cli_register_menu('tenant:execute'          , 'tag'   , menu_execute_tag_commands)
