# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
     tag Pontecchio
     
            condition foo;;datagroup;condition1
            
            datagroup foo;;datagroup
		no condition2
            exit

            node xha                                   -- tag di tutto quello che c'e' in XHA
            
                datagroup dg1
                    no condition check1
                exit

                interface eth1 

                    datagroup status
                        no timegraph t2
                    exit

                    no datagroup cdp
                exit

                no interface eth2
            exit
     exit
"""     

import sys

from django.utils.translation import ugettext as _

from core.db_utils import transaction_commit_on_success

from core.api.base import *

from cli.clilib.ioslike import *


#from cli.tokens import *
from cli.tokens import T_SubInterfaceCompleter
from cli.tokens import T_SubStorageCompleter
from cli.tokens import T_SubServiceCompleter
from cli.tokens import T_SubDeviceCompleter
from cli.tokens import T_MonitoredNodecompleter
from cli.tokens import T_InlineIfaceCompleter
from cli.tokens import T_InlineStorageCompleter
from cli.tokens import T_InlineServiceCompleter
from cli.tokens import T_InlineDeviceCompleter
from cli.tokens import T_ConditionCompleter
from cli.tokens import T_NodeTemplate
from cli.tokens import T_Clustercompleter

from cli.tokens import T_KeywordWithHook
from cli.tokens import T_Usercompleter


from cli.textformat import *

#from cli.tokens import *
from cli.tokens import T_Groupcompleter

from cli.prompts.defs import CLI_TRUE
from cli.prompts.defs import PROMPT_SEP
from cli.prompts.defs import CLI_PERM_OPTIONS

from cli.prompts import cli_get_registered_menus
from cli.prompts.common import output_table

#from tags.cli.tokens import *

from tags.models import Tag
from tags.models import TagEntryInsertRule
from tags.models import TagTreeGroupPermission
from tags.models import ElementTag
from tags.models import ELEMENT_KIND_TO_TAG_ELEMENTTYPE
from tags.models import MODEL_TO_TAG_ELEMENTTYPE

from tags.cli.tokens import T_TagTreeCompleter
from tags.cli.tokens import T_TagCompleter
from tags.cli.tokens import T_TaggedDataGroupCompleter
from tags.cli.tokens import T_TaggedConditionCompleter
from tags.cli.tokens import T_TagMetaDataCompleter


from tags.api import tagtree_get
from tags.api import tagtree_delete
from tags.api import tagtree_set_nostate

from tags.api import tagtree_recalculate_tagged_elements

from tags.api import tag_create
from tags.api import tag_remove
from tags.api import tag_rename

from tags.api import tag_element, untag_element
from tags.api import tag_element_indirect

from tags.api import tag_datagroup_indirect

from tags.api import tag_condition

from tags.api import untag_element
from tags.api import untag_datagroup
from tags.api import untag_condition

from tags.api import elementtag_set_explosive

from tags.server.tagsd.api import set_signal_tagsd_server, signal_state_refresh


from tags.models import ElementTag
from core.models import MonitoredElement

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_config_tags_untag_condition(cli):

	# no condition <name> <name2>
	env = cli.env

	user = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)

	db_set_current_tenant_schema(tenant_name)

	tree    = env['CONFIG_TREE']
	tag     = env['CONFIG_TAG']
	dg      = env['CONFIG_DATAGROUP']

	condition_name = cli.canonical_words[2]

	c = Condition.objects.get(datagroup=dg, name=condition_name)

	untag_condition(tag, c, user=user)

	return


mt_config_tags_inside_datagroup = [

	T_Keyword('no'       , help='Untag datagroup entities' , pipe=None, subtree=[
		T_Keyword('condition', help='Untag condition', pipe=None, subtree=[
			T_Anyword (help='Untag condition', callback=cmd_config_tags_untag_condition, pipe=None )
		]),
	])

]

def cmd_config_tags_inside_datagroup(cli):

	# datagroup <name>
	env = cli.env

	user = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)

	tree    = env['CONFIG_TREE']
	tag     = env['CONFIG_TAG']
	element = env['CURRENT_ELEMENT_DATA']


	name = cli.canonical_words[1]

	dg = DataGroup.objects.get(element=element, name=name)

	tag_datagroup_indirect(tag, dg, user=user)

	env = cli.env.clone()

	env['CONFIG_DATAGROUP'] = dg

	env['PROMPT'] = '%s' % ( PROMPT_SEP.join([ tenant_name, name ]) )
	
	path = [tenant_name] + env['CONFIG_TAG_PATH'] + [name]

	env['PROMPT'] = '%s' % ( PROMPT_SEP.join([ p.encode('utf-8') for p in path ] ) )
	
	cmds = mt_config_tags_inside_datagroup + \
	       [BLANK_ENTRY] + \
	       SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + \
	       cli_get_registered_menus('__show_commands__'))
	       
	#cli = Cli(cmds, env, debug_interpreter=env['DEBUG'])
	#cli.cmdloop()
	cli.enter_subcli(cmds, env)

	return
	
def cmd_config_tags_untag_interface_inside_node(cli):
	_cmd_config_tags_untag_element_inside_node(cli, MonitoredIface)
		
def cmd_config_tags_untag_storage_inside_node(cli):
	_cmd_config_tags_untag_element_inside_node(cli, MonitoredStorage)

def cmd_config_tags_untag_service_inside_node(cli):
	_cmd_config_tags_untag_element_inside_node(cli, MonitoredService)

def cmd_config_tags_untag_device_inside_node(cli):
	_cmd_config_tags_untag_element_inside_node(cli, MonitoredDevice)

def _cmd_config_tags_untag_element_inside_node(cli, model):

	# datagroup <name>
	env = cli.env

	user = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)

	tree    = env['CONFIG_TREE']
	tag     = env['CONFIG_TAG']
	element = env['CURRENT_ELEMENT_DATA']

	node = element

	name = cli.canonical_words[2]

	element = model.objects.get(parent=node, name=name)

	untag_element(tag, element, user=user)

	return

mt_config_tags_inside_element = [

	T_Keyword('datagroup'       , subtree=[
		T_Anyword   (help='', callback=cmd_config_tags_inside_datagroup, pipe=None , )
	]),

	T_Keyword('no'       , subtree=[
		T_Keyword('interface'       , subtree=[
			T_SubInterfaceCompleter(help='', callback=cmd_config_tags_untag_interface_inside_node, pipe=None , ) 
		]),
		T_Keyword('storage'       , subtree=[
			T_SubStorageCompleter  (help='', callback=cmd_config_tags_untag_storage_inside_node  , pipe=None , )
		]),
		T_Keyword('service'       , subtree=[
			T_SubServiceCompleter  (help='', callback=cmd_config_tags_untag_service_inside_node  , pipe=None , )
		]),
		T_Keyword('device'       , subtree=[
			T_SubDeviceCompleter   (help='', callback=cmd_config_tags_untag_device_inside_node   , pipe=None , )		
		])
	])
]

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_config_tags_inside_interface(cli):
	return _cmd_config_tags_inside_element(cli, ELEMENT_KIND_IFACE)

def cmd_config_tags_inside_storage  (cli):
	return _cmd_config_tags_inside_element(cli, ELEMENT_KIND_STORAGE)

def cmd_config_tags_inside_service  (cli):
	return _cmd_config_tags_inside_element(cli, ELEMENT_KIND_SERVICE)

def cmd_config_tags_inside_device   (cli):
	return _cmd_config_tags_inside_element(cli, ELEMENT_KIND_DEVICE)
	
def _cmd_config_tags_inside_element(cli, kind):

	# no condition <path>
	env = cli.env
	
	user = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)

	tree    = env['CONFIG_TREE']
	tag     = env['CONFIG_TAG']
	element = env['CURRENT_ELEMENT_DATA']
	
	name    = cli.canonical_words[1]
	
	subelement = element.get_sub_elements(kind=kind).get(name=name)
	
	tag_element_indirect(tag, subelement, user=user)
	
	env = cli.env.clone()

	env['CURRENT_ELEMENT_DATA'] = subelement

	#env['PROMPT'] = 'config-%s#' % ( PROMPT_SEP.join([ tenant_name, element.name, name ]) )
	
	path = [tenant_name] + env['CONFIG_TAG_PATH'] + [ element.name ] + [name]

	env['PROMPT'] = '%s' % ( PROMPT_SEP.join([ p.encode('utf-8') for p in path ] ) )
	
	cmds = mt_config_tags_inside_element + \
	       [BLANK_ENTRY] + \
	       SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + \
	       cli_get_registered_menus('__show_commands__'))
	       
	#cli = Cli(cmds, env, debug_interpreter=env['DEBUG'])
	#cli.cmdloop()
	cli.enter_subcli(cmds, env)

	return		


mt_config_tags_inside_node = mt_config_tags_inside_element + [

	T_Keyword('interface'       , subtree=[
		T_SubInterfaceCompleter(help='', callback=cmd_config_tags_inside_interface, pipe=None , ) 
	]),
	T_Keyword('storage'       , subtree=[
		T_SubStorageCompleter  (help='', callback=cmd_config_tags_inside_storage  , pipe=None , )
	]),
	T_Keyword('service'       , subtree=[
		T_SubServiceCompleter  (help='', callback=cmd_config_tags_inside_service  , pipe=None , )
	]),
	T_Keyword('device'       , subtree=[
		T_SubDeviceCompleter   (help='', callback=cmd_config_tags_inside_device   , pipe=None , )
	]),
	#T_Keyword('no '       , subtree=[
	#	T_Keyword('condition'       , subtree=[
	#		T_ConditionCompleter(help='', callback=cmd_tag_exclude_condition, pipe=None,)
	#	])
	#]),
]



def cmd_tag_add_node(cli):

	# node <name> [ force-retagging ]
	env = cli.env

	user = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)

	tree = env['CONFIG_TREE']
	tag  = env['CONFIG_TAG']

	name = cli.canonical_words[1]
	
	force_retagging = False
	if len(cli.canonical_words) > 2 and cli.canonical_words[2] == 'force-retagging':
		force_retagging = True

	node = MonitoredNode.objects.get(name=name)

	#
	# TAG
	#
	tag_element(tag, node, user=user, force_retagging=force_retagging)


	#
	# SUB SECTION
	#
	e = node

	env = cli.env.clone()


	env['CURRENT_ELEMENT_DATA'] = e

	env['PROMPT'] = '%s' % ( PROMPT_SEP.join([ tenant_name, e.name ]) )

	path = [tenant_name] + env['CONFIG_TAG_PATH'] + [e.name]

	env['PROMPT'] = '%s' % ( PROMPT_SEP.join([ p.encode('utf-8') for p in path ] ) )

	cmds = mt_config_tags_inside_node + \
	       [BLANK_ENTRY] + \
	       SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + \
	       cli_get_registered_menus('__show_commands__'))

	#cli = Cli(cmds, env, debug_interpreter=env['DEBUG'])
	#cli.cmdloop()

	cli.enter_subcli(cmds, env)

	return


def cmd_tag_add_iface(cli):
	_cmd_tag_add_element(cli, ELEMENT_KIND_IFACE, 'interface')
	
def cmd_tag_add_storage(cli):
	_cmd_tag_add_element(cli, ELEMENT_KIND_STORAGE, 'storage')

def cmd_tag_add_service(cli):
	_cmd_tag_add_element(cli, ELEMENT_KIND_SERVICE, 'service')

def cmd_tag_add_device(cli):
	_cmd_tag_add_element(cli, ELEMENT_KIND_DEVICE, 'device')

def _cmd_tag_add_element(cli, kind, tagelementtype):

	env = cli.env

	user = env['CURRENT_USER']

	# interface <node> <iface> [explosive [ group <n> ] ]
	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)

	tree = env['CONFIG_TREE']
	tag  = env['CONFIG_TAG']

	node_name    = cli.canonical_words[1]
	element_name = cli.canonical_words[2]

	node = MonitoredNode.objects.get(tenant=tenant, name=node_name)

	e    = MonitoredElement.objects.get(tenant=tenant, parent=node, kind=kind, name=element_name)

	tag_element(tag, e, user=user)

	env = cli.env.clone()

	env['CURRENT_ELEMENT_DATA'] = e

	env['PROMPT'] = '%s' % ( PROMPT_SEP.join([ tenant_name, node.name, e.name ]) )

	path = [tenant_name] + env['CONFIG_TAG_PATH'] + [e.name]

	env['PROMPT'] = '%s' % ( PROMPT_SEP.join([ p.encode('utf-8') for p in path ] ) )

	cmds = mt_config_tags_inside_element + \
	       [BLANK_ENTRY] + \
	       SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + \
	       cli_get_registered_menus('__show_commands__'))

	#cli = Cli(cmds, env, debug_interpreter=env['DEBUG'])
	#cli.cmdloop()

	cli.enter_subcli(cmds, env)

	return

#------------------------------------------------------------------------------#

"""
def cmd_tag_add_datagroup(cli):
	env = cli.env
	
	# condition <path>  [explosive [ group <n> ] ]
	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)
	
	tree = env['CONFIG_TREE']
	tag  = env['CONFIG_TAG']

	path    = cli.canonical_words[1]
	
	dg = DataGroup.objects.get(tenant=tenant, path=path)
	
	name = dg.name
	
	tag_datagroup(tag, dg)
	
	env['CONFIG_DATAGROUP'] = dg

	env['PROMPT'] = 'config-%s#' % ( PROMPT_SEP.join([ tenant_name, name ]) )
	
	path = [tenant_name] + env['CONFIG_TAG_PATH'] + [name]

	env['PROMPT'] = 'config-%s#' % ( PROMPT_SEP.join([ p.encode('utf-8') for p in path ] ) )
	
	cmds = mt_config_tags_inside_datagroup + \
	       [BLANK_ENTRY] + \
	       SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + \
	       cli_get_registered_menus('__show_commands__'))
	       
	cli = Cli(cmds, env, debug_interpreter=env['DEBUG'])
	cli.cmdloop()

	return
"""
	
	
	
#------------------------------------------------------------------------------#

def cmd_tag_add_condition(cli):

	env = cli.env

	user = env['CURRENT_USER']

	# condition <path>  [explosive [ group <n> ] ]
	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)
	
	tree = env['CONFIG_TREE']
	tag  = env['CONFIG_TAG']

	path    = cli.canonical_words[1]

	e = Condition.objects.get(tenant=tenant, path=path)

	tag_condition(tag, e, user=user)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_tag_remove_node(cli):

	# no node <name>

	env = cli.env
	
	user = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)

	tree = env['CONFIG_TREE']
	tag  = env['CONFIG_TAG']

	name = cli.canonical_words[2]

	node = MonitoredNode.objects.get(name=name)

	untag_element(tag, node, user=user)

	return
	
def cmd_tag_remove_iface(cli):
	_cmd_tag_remove_element(cli, ELEMENT_KIND_IFACE, 'interface')
	
def cmd_tag_remove_storage(cli):
	_cmd_tag_remove_element(cli, ELEMENT_KIND_STORAGE, 'storage')

def cmd_tag_remove_service(cli):
	_cmd_tag_remove_element(cli, ELEMENT_KIND_SERVICE, 'service')

def cmd_tag_remove_device(cli):
	_cmd_tag_remove_element(cli, ELEMENT_KIND_DEVICE, 'device')
	
def _cmd_tag_remove_element(cli, kind, tagelementtype):

	# no <kind> <node> <sub>
	env = cli.env

	user = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)

	tree = env['CONFIG_TREE']
	tag  = env['CONFIG_TAG']

	node_name    = cli.canonical_words[2]
	element_name = cli.canonical_words[3]

	node    = MonitoredNode.objects.get(tenant=tenant, name=node_name)
	element = MonitoredElement.objects.get(tenant=tenant, kind=kind, parent=node, name=element_name)

	untag_element(tag, element, user=user)

	return

#------------------------------------------------------------------------------#

def cmd_tag_remove_datagroup(cli):

	# no datagroup <path>
	env = cli.env

	user = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)

	tree = env['CONFIG_TREE']
	tag  = env['CONFIG_TAG']

	path    = cli.canonical_words[2]
	path2   = cli.canonical_words[3]

	if path != path2:
		print "Not same datagroup"
		return

	dg = DataGroup.objects.get(tenant=tenant, path=path)

	untag_datagroup(tag, dg, user=user)

#------------------------------------------------------------------------------#
	
def cmd_tag_remove_condition(cli):

	# no <kind> <node> <sub>
	env = cli.env
	
	user = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)

	tree = env['CONFIG_TREE']
	tag  = env['CONFIG_TAG']

	path    = cli.canonical_words[2]
	path2    = cli.canonical_words[3]

	if path != path2:
		raise Exception("Condition mismatch")

	condition = Condition.objects.get(tenant=tenant, path=path)

	untag_condition(tag, condition, user=user)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_config_tag_prompt(cli):

	env = cli.env

	user = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)

	tree       = env['CONFIG_TREE']
	parent_tag = env['CONFIG_TAG']

	name = cli.canonical_words[1]

	#
	# CREATE TAG
	#
	db_set_current_tenant_schema(tenant.name)

	tag = tag_create(tree, name, parent=parent_tag, user=user)

	#
	# SUB PROMPT
	#
	env = cli.env.clone()

	env['CONFIG_TAG'] = tag
	env['CONFIG_TAG_PATH'] = [ e for e in env['CONFIG_TAG_PATH']] + [ name ]

	env['CURRENT_TREE'] = env['CONFIG_TREE']
	env['CURRENT_TAG' ] = env['CONFIG_TAG' ]

	path = [tenant_name] + env['CONFIG_TAG_PATH']

	env['PROMPT'] = '%s' % ( PROMPT_SEP.join([ p.encode('utf-8') for p in path ] ) )

	cmds = mt_config_tag  + \
	       [BLANK_ENTRY] + \
	       SORTED_KEYWORDS(cli_get_registered_menus('configuration:tag')) + \
	       [BLANK_ENTRY] + \
	       SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + \
	       cli_get_registered_menus('__show_commands__'))

	#cli = Cli(cmds, env, debug_interpreter=env['DEBUG'])
	#cli.cmdloop()
	
	cli.enter_subcli(cmds, env)
	

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_tag_remove_subtag(cli):

	# no tag <name>
	env = cli.env

	user = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)

	tree = env['CONFIG_TREE']
	tag  = env['CONFIG_TAG']

	name = cli.canonical_words[2]

	#
	# FIX: move to model
	#
	tag = Tag.objects.get(tenant=tenant, parent=tag, tree=tree, name=name)

	if tag.parent is None:
		cli.warning("You are deleting the root tag. Everything in this tagging tree will be deleted.")

	tag_remove(tag, user=user)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#
	
def cmd_config_tag_rename(cli):

	# rename tag <tag> <new_name>

	env = cli.env

	user = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)

	tree       = env['CONFIG_TREE']
	parent_tag = env['CONFIG_TAG']

	tag_name = cli.canonical_words[2]
	new_name = cli.canonical_words[3]

	tag = Tag.objects.get(tenant=tenant, tree=tree, parent=parent_tag, name=tag_name)

	tag_rename(tag, new_name, user=user)

	return

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_config_tag_insert_rule(cli):

	# no insert-rule ........

	env = cli.env

	user = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)

	tree = env['CONFIG_TREE']
	tag  = env['CONFIG_TAG']

	rule = cli.canonical_words[1]
	
	#
	# FIX: move to model
	#
	if user:
		user.assert_capability(tenant, 'tags.tag.edit')
		tree.assert_permission_readwrite(user)

	r, created = TagEntryInsertRule.objects.get_or_create(tenant=tenant, tagentry=tag, rule=rule)
	r.save()

def cmd_config_tag_remove_rule(cli):

	# insert-rule ........

	env = cli.env

	user = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)

	tree = env['CONFIG_TREE']
	tag  = env['CONFIG_TAG']

	rule = cli.canonical_words[2]


	#
	# FIX: move to model
	#
	if user:
		user.assert_capability(tenant, 'tags.tag.edit')
		tree.assert_permission_readwrite(user)

	r = TagEntryInsertRule.objects.get(tenant=tenant, tagentry=tag, rule=rule)
	r.delete()

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_config_tag_attribute(cli):

	# description
	env = cli.env

	user = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)

	db_set_current_tenant_schema(tenant_name)

	tree = env['CONFIG_TREE']
	tag  = env['CONFIG_TAG']

	attr_name = cli.canonical_words[0]
	attr_val  = cli.canonical_words[1]


	#
	# FIX: move to model
	#
	if user:
		user.assert_capability(tenant, 'tags.tag.edit')
		tree.assert_permission_readwrite(user)

	if attr_name == 'description':
		tag.description = attr_val

	tag.save()

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

from tags.models import TagMetaData

def cmd_config_tag_meta(cli):
	# meta <name> <val>

	env = cli.env

	user = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)

	tree = env['CONFIG_TREE']
	tag  = env['CONFIG_TAG']

	meta_name = cli.canonical_words[1]
	meta_val  = cli.canonical_words[2]


	#
	# FIX: move to model
	#
	if user:
		user.assert_capability(tenant, 'tags.tag.edit')
		tree.assert_permission_readwrite(user)

	data, created = TagMetaData.objects.get_or_create(tenant=tenant, tag=tag, name=meta_name)

	if data.value != meta_val:
		data.value = meta_val
		data.save()


def cmd_tag_remove_metadata(cli):
	# no meta <name>

	env = cli.env

	user = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)

	tree = env['CONFIG_TREE']
	tag  = env['CONFIG_TAG']

	meta_name = cli.canonical_words[2]

	#
	# FIX: move to model
	#
	if user:
		user.assert_capability(tenant, 'tags.tag.edit')
		tree.assert_permission_readwrite(user)

	try:
		data = TagMetaData.objects.get(tenant=tenant, tag=tag, name=meta_name)
		data.delete()
	except TagMetaData.DoesNotExist, e:
		print e
		pass

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

# 0         1         2                
# explosive node      <name>         [ group <id> ]
# explosive interface <name> <name>  [ group <id> ]
# explosive storage   <name> <name>  [ group <id> ]
# explosive service   <name> <name>  [ group <id> ]
# explosive device    <name> <name>  [ group <id> ]
# explosive condition <path>         [ group <id> ]

def cmd_set_explosive_node(cli):

	name        = cli.canonical_words[2]
	group       = _get_group(cli.canonical_words[4:])

	_set_explosive_element(cli, ELEMENT_KIND_NODE, None, name, True, group)

	return 

def cmd_unset_explosive_node(cli):

	name        = cli.canonical_words[3]
	_set_explosive_element(cli, ELEMENT_KIND_NODE, None, name, False, None)

	return 

#------------------------------------------------------------------------------#

def cmd_set_explosive_iface(cli):
	_set_explosive_subelement(ELEMENT_KIND_IFACE, cli)

def cmd_unset_explosive_iface(cli):
	_unset_explosive_subelement(ELEMENT_KIND_IFACE, cli)

#------------------------------------------------------------------------------#

def cmd_set_explosive_storage(cli):
	_set_explosive_subelement(ELEMENT_KIND_STORAGE, cli)

def cmd_unset_explosive_storage(cli):
	_unset_explosive_subelement(ELEMENT_KIND_STORAGE, cli)

#------------------------------------------------------------------------------#

def cmd_set_explosive_service(cli):
	_set_explosive_subelement(ELEMENT_KIND_SERVICE, cli)

def cmd_unset_explosive_service(cli):
	_unset_explosive_subelement(ELEMENT_KIND_SERVICE, cli)

#------------------------------------------------------------------------------#

def cmd_set_explosive_device(cli):
	_set_explosive_subelement(ELEMENT_KIND_DEVICE, cli)

def cmd_unset_explosive_device(cli):
	_unset_explosive_subelement(ELEMENT_KIND_DEVICE, cli)

#------------------------------------------------------------------------------#

def _set_explosive_subelement(kind, cli):

	parent_name = cli.canonical_words[2]
	name        = cli.canonical_words[3]
	group       = _get_group(cli.canonical_words[5:])

	_set_explosive_element(cli, kind, parent_name, name, True, group)

def _unset_explosive_subelement(kind, cli):

	parent_name = cli.canonical_words[3]
	name        = cli.canonical_words[4]

	_set_explosive_element(cli, kind, parent_name, name, False, None)

#------------------------------------------------------------------------------#

def _get_group(explosive_args):

	if len(explosive_args) == 0:
		return None

	return int(explosive_args[0])

#------------------------------------------------------------------------------#

def _set_explosive_element(cli, kind, parent_name, name, explosive, explosive_group):

	env = cli.env

	user = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)

	tree = env['CONFIG_TREE']
	tag  = env['CONFIG_TAG']

	element = MonitoredElement.objects.get(tenant=tenant, kind=kind, name=name, parent__name=parent_name)

	elementtype = ELEMENT_KIND_TO_TAG_ELEMENTTYPE[element.kind]
	elementid   = element.uuid

	#
	# FIX: move to model
	#
	if user:
		user.assert_capability(tenant, 'tags.tag.edit')
		tree.assert_permission_readwrite(user)

	et = ElementTag.objects.get(tenant=tag.tenant, tag=tag, elementtype=elementtype, elementid=elementid)

	return elementtag_set_explosive(et, explosive, explosive_group)



def cmd_set_explosive_condition(cli):
	# 0         1         2        3     4
	# explosive condition <path> [ group <name>.. ]
	path = cli.canonical_words[2]

	explosive_group = _get_group(cli.canonical_words[4:])

	_set_explosive_condition(cli, path, True, explosive_group)

def cmd_unset_explosive_condition(cli):

	path = cli.canonical_words[3]

	_set_explosive_condition(cli, path, False, None)

def _set_explosive_condition(cli, path, explosive, explosive_group):

	env = cli.env

	user = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)

	tree = env['CONFIG_TREE']
	tag  = env['CONFIG_TAG']


	#
	# FIX: move to model
	#
	if user:
		user.assert_capability(tenant, 'tags.tag.edit')
		tree.assert_permission_readwrite(user)

	element = Condition.objects.get(path=path)

	elementtype = MODEL_TO_TAG_ELEMENTTYPE[element.__class__]
	elementid   = element.uuid

	et = ElementTag.objects.get(tenant=tag.tenant, tag=tag, elementtype=elementtype, elementid=elementid)

	return elementtag_set_explosive(et, explosive, explosive_group)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@transaction_commit_on_success
def cmd_config_tag_quick_tag_nodes_by_template(cli):
	
	env = cli.env
	
	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)
	
	tree = env['CONFIG_TREE']
	tag  = env['CONFIG_TAG']
	
	
	
	template_name = cli.canonical_words[3]
	nodes = MonitoredNode.filter_by_template(template_name)
	nodes = nodes.filter(tenant=tenant).order_by('name')
	
	for n in nodes:
		print "Tagging", n.name, "..."
		tag_element(tag, n)
	
	
@transaction_commit_on_success
def cmd_tag_cluster_shortcut(cli):
	
	cluster_name = cli.canonical_words[1]
	
	env = cli.env
	
	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)
	
	tree = env['CONFIG_TREE']
	tag  = env['CONFIG_TAG']

	c = Cluster.objects.get(name = cluster_name)
	for n in c.get_all_nodes().all():
		print "tagging node", n.name
		tag_element(tag, n)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

mt_config_tag = [

	T_Keyword('description', help='Set tag description', subtree=[
		T_Line       (help='Create tag', callback=cmd_config_tag_attribute , pipe=None,),
	]),

	T_Keyword('meta-data', help='Set tag metadata', subtree=[
		T_TagMetaDataCompleter(help='name', pipe=None, subtree=[
			T_Line       (help='Set tag metadata', callback=cmd_config_tag_meta , pipe=None,),
		]),
		T_Anyword       (help='name', pipe=None, subtree=[
			T_Line       (help='Set tag metadata', callback=cmd_config_tag_meta , pipe=None,),
		]),
	]),

	T_Keyword('tag', help='Configure/create a new tag', subtree=[
		T_TagCompleter  (help='Configure tag', callback=cmd_config_tag_prompt , pipe=None,),
		T_Anyword       (help='Create tag', callback=cmd_config_tag_prompt , pipe=None,),
	]),
	
	T_Keyword('node'       , subtree=[
		T_MonitoredNodecompleter(help='Name of an existing node', callback=cmd_tag_add_node, pipe=None, subtree=[
			T_Keyword('force-retagging', help='Force complete retagging.', pipe=None, callback=cmd_tag_add_node)
		]),
	]),
	T_Keyword('interface'       , subtree=[
		T_MonitoredNodecompleter(help='Node 1', subtree=[
			T_InlineIfaceCompleter(help='', callback=cmd_tag_add_iface, pipe=None , 
			),
		]),
	]),
	T_Keyword('storage'       , subtree=[
		T_MonitoredNodecompleter(help='Node 1', subtree=[
			T_InlineStorageCompleter(help='', callback=cmd_tag_add_storage, pipe=None , 
			),
		]),
	]),
	T_Keyword('service'       , subtree=[
		T_MonitoredNodecompleter(help='Node 1', subtree=[
			T_InlineServiceCompleter(help='', callback=cmd_tag_add_service, pipe=None , 
			),
		]),
	]),
	T_Keyword('device'       , subtree=[
		T_MonitoredNodecompleter(help='Node 1', subtree=[
			T_InlineDeviceCompleter(help='', callback=cmd_tag_add_device, pipe=None , 
			),
		]),
	]),

	#
	#T_Keyword('datagroup'       , subtree=[
	#	T_DataGroupCompleter(help='', callback=cmd_tag_add_datagroup, pipe=None , 
	#	),
	#]),
	
	T_Keyword('condition'       , subtree=[
		T_ConditionCompleter(help='', callback=cmd_tag_add_condition, pipe=None , 
		),
	]),
	
	T_Keyword('explosive'       , help='Node 1', subtree=[

			T_Keyword('node'       , subtree=[
				T_MonitoredNodecompleter(help='Name of an existing node', callback=cmd_set_explosive_node, pipe=None , subtree=[
								T_Keyword('group', help='', callback=cmd_set_explosive_node, subtree=[
									T_Number('<id>', help='', callback=cmd_set_explosive_node ) 
								]),
				]),
			]),
			T_Keyword('interface'       , subtree=[
				T_MonitoredNodecompleter(help='Node 1', subtree=[
					T_InlineIfaceCompleter(help='', callback=cmd_set_explosive_iface, pipe=None , subtree=[
								T_Keyword('group', help='', callback=cmd_set_explosive_iface, subtree=[
									T_Number('<id>', help='', callback=cmd_set_explosive_iface )
								]),
					]),
				]),
			]),
			T_Keyword('storage'       , subtree=[
				T_MonitoredNodecompleter(help='Node 1', subtree=[
					T_InlineStorageCompleter(help='', callback=cmd_set_explosive_storage, pipe=None , subtree=[
								T_Keyword('group', help='', callback=cmd_set_explosive_storage, subtree=[
									T_Number('<id>', help='', callback=cmd_set_explosive_storage )
								]),
					]),
				]),
			]),
			T_Keyword('service'       , subtree=[
				T_MonitoredNodecompleter(help='Node 1', subtree=[
					T_InlineServiceCompleter(help='', callback=cmd_set_explosive_service, pipe=None , subtree=[
								T_Keyword('group', help='', callback=cmd_set_explosive_service, subtree=[
									T_Number('<id>', help='', callback=cmd_set_explosive_service )
								]),
					]),
				]),
			]),
			T_Keyword('device'       , subtree=[
				T_MonitoredNodecompleter(help='Node 1', subtree=[
					T_InlineDeviceCompleter(help='', callback=cmd_set_explosive_device, pipe=None , subtree=[
								T_Keyword('group', help='', callback=cmd_set_explosive_device, subtree=[
									T_Number('<id>', help='', callback=cmd_set_explosive_device )
								]),
					]),
				]),
			]),
			T_Keyword('condition'       , subtree=[
				T_ConditionCompleter(help='', callback=cmd_set_explosive_condition, pipe=None , subtree=[
								T_Keyword('group', help='', callback=cmd_set_explosive_condition, subtree=[
									T_Number('<id>', help='', callback=cmd_set_explosive_condition )
								]),
				]),
			]),

	]),
	
	T_Keyword('rename', help='Rename existing tag', subtree=[
		
		T_Keyword('tag', help='Rename existing tag', subtree=[
			T_TagCompleter  (help='Rename existing tag', subtree=[
				T_Anyword       (help='Rename existing tag', callback=cmd_config_tag_rename , pipe=None,),
			]),
		]),
	]),


	T_Keyword('quick-tag', help='Utility command for quick tagging', subtree=[
		T_Keyword('nodes', help='Quick tag nodes', subtree=[
			T_Keyword('template', help='Quick tag nodes by node-template', subtree=[
				T_NodeTemplate(help='Quick tag nodes by the specified node template', callback=cmd_config_tag_quick_tag_nodes_by_template , pipe=None,),
			]),
		]),
	]),


	T_Keyword('insert-rule', help='Rename existing tag', subtree=[
		T_Line(help='rule', callback=cmd_config_tag_insert_rule, pipe=None,),
	]),

	T_Keyword('cluster', help='Tag all nodes of the selected cluster', subtree=[
		T_Clustercompleter(help='Tag all nodes of the selected cluster', callback=cmd_tag_cluster_shortcut, pipe=None,),
	]),


	T_KeywordWithHook('no', help='', 
	
		hook_name='configuration:tag:no',
	
		subtree=[

		T_Keyword('meta-data', help='Remove metadata', subtree=[
			T_TagMetaDataCompleter(help='name', pipe=None, callback=cmd_tag_remove_metadata) 
		]),

	
		T_Keyword('tag', help='Remove tag', subtree=[
			T_TagCompleter  (help='Remove tag', callback=cmd_tag_remove_subtag , pipe=None,),
		]),
		
		T_Keyword('node'       , subtree=[
			T_MonitoredNodecompleter(help='Name of an existing node', callback=cmd_tag_remove_node, pipe=None ),
			
		]),
		
		T_Keyword('interface'       , subtree=[
			T_MonitoredNodecompleter(help='Node 1', subtree=[
				T_InlineIfaceCompleter(help='', callback=cmd_tag_remove_iface, pipe=None ),
			]),
		]),
		T_Keyword('storage'       , subtree=[
			T_MonitoredNodecompleter(help='Node 1', subtree=[
				T_InlineStorageCompleter(help='', callback=cmd_tag_remove_storage, pipe=None ),
			]),
		]),
		T_Keyword('service'       , subtree=[
			T_MonitoredNodecompleter(help='Node 1', subtree=[
				T_InlineServiceCompleter(help='', callback=cmd_tag_remove_service, pipe=None ),
			]),
		]),
		T_Keyword('device'       , subtree=[
			T_MonitoredNodecompleter(help='Node 1', subtree=[
				T_InlineDeviceCompleter(help='', callback=cmd_tag_remove_device, pipe=None ),
			]),
		]),	

		T_Keyword('datagroup'       , subtree=[
			T_TaggedDataGroupCompleter(help='', subtree=[
				T_TaggedDataGroupCompleter(help='', callback=cmd_tag_remove_datagroup, pipe=None ),
			]),
		]),
		
		T_Keyword('condition'       , subtree=[
			T_TaggedConditionCompleter(help='', subtree=[
				T_TaggedConditionCompleter(help='', callback=cmd_tag_remove_condition, pipe=None ),
			]),
		]),
		
		T_Keyword('insert-rule', help='Rename existing tag', subtree=[
			T_Line(help='rule', callback=cmd_config_tag_remove_rule, pipe=None,),
		]),

		T_Keyword('explosive', subtree=[

			T_Keyword('node', subtree=[
				T_MonitoredNodecompleter(help='Name of an existing node', callback=cmd_unset_explosive_node, pipe=None),
			]),
			T_Keyword('interface'    , subtree=[
				T_MonitoredNodecompleter(help='Node 1', subtree=[
					T_InlineIfaceCompleter(help='', callback=cmd_unset_explosive_iface, pipe=None),
				]),
			]),
			T_Keyword('storage'      , subtree=[
				T_MonitoredNodecompleter(help='Node 1', subtree=[
					T_InlineStorageCompleter(help='', callback=cmd_unset_explosive_device, pipe=None),
				]),
			]),
			T_Keyword('service'      , subtree=[
				T_MonitoredNodecompleter(help='Node 1', subtree=[
					T_InlineServiceCompleter(help='', callback=cmd_unset_explosive_service, pipe=None),
				]),
			]),
			T_Keyword('device'       , subtree=[
				T_MonitoredNodecompleter(help='Node 1', subtree=[
					T_InlineDeviceCompleter(help='', callback=cmd_unset_explosive_device, pipe=None),
				]),
			]),
			T_Keyword('condition'    , subtree=[
				T_ConditionCompleter(help='', callback=cmd_unset_explosive_condition, pipe=None),
			]),
		]),
	])

]

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_config_tagtree_field(cli):

	env = cli.env
	
	user = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)

	tree       = env['CONFIG_TREE']

	field_name  = cli.canonical_words[0]
	field_value = cli.canonical_words[1]
	
	


	if   field_name == 'uniques':

		if field_value == CLI_TRUE:
			uniques = True
		else:
			uniques = False

		#
		# FIX: move to model
		#
		if user:
			user.assert_capability(tenant, 'tags.tagtree.edit')
			tree.assert_permission_readwrite(user)

		from django.db import IntegrityError
		try:
			tree.set_uniques(uniques)
		except IntegrityError, e:
			cli.error("Can not change this tree setting. Some elements are already tagged multiple times.")

	elif field_name == 'allnodes':

		if field_value == CLI_TRUE:
			allnodes = True
		else:
			allnodes = False

		#
		# FIX: move to model
		#
		if user:
			user.assert_capability(tenant, 'tags.tagtree.edit')
			tree.assert_permission_readwrite(user)

		tree.allnodes = allnodes
		tree.save()

	elif field_name == 'nostate':

		if field_value == CLI_TRUE:
			v = True
		else:
			v = False

		#
		# FIX: move to model
		#
		if user:
			user.assert_capability(tenant, 'tags.tagtree.edit')
			tree.assert_permission_readwrite(user)

		tagtree_set_nostate(tree, v)

	elif field_name == 'nomap':

		if field_value == CLI_TRUE:
			nomap = True
		else:
			nomap = False

		#
		# FIX: move to model
		#
		if user:
			user.assert_capability(tenant, 'tags.tagtree.edit')
			tree.assert_permission_readwrite(user)

		tree.nomap = nomap
		tree.save()


#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_tagtree_refresh(cli):

	env = cli.env
	
	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)
	
	tree       = env['CONFIG_TREE']

	tagtree_recalculate_tagged_elements(tree)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def set_tagtree_group_access(cli):
	
	# group <name> [ <permission> ]

	env = cli.env

	user        = env['CURRENT_USER']
	
	tenant_name = env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)
	
	tree       = env['CONFIG_TREE']


	# group  <name> <perm>
	group_name = cli.canonical_words[1]
	
	if len(cli.canonical_words) > 2:
		perm       = cli.canonical_words[2]
		if perm == 'read-only':
			perm = DIRENTRY_AUTH_READONLY
		elif perm == 'read-write':
			perm = DIRENTRY_AUTH_READWRITE
	else:
		perm = DIRENTRY_AUTH_READWRITE

	group = Group.objects.get(name=group_name)


	#
	# ACTION: FIX move to models
	#
	if user:
		user.assert_capability(tenant, 'tags.tagtree.acl.edit')
		tree.assert_permission_readwrite(user)
		user.assert_in_group(group)

	ttgp, created = TagTreeGroupPermission.objects.get_or_create(tenant=tenant, dirtree=tree, group=group)
	
	ttgp.auth_type = perm
	ttgp.save()


def unset_tagtree_group_access(cli):

	# no group <name> <name>

	env = cli.env

	user        = env['CURRENT_USER']

	tenant_name = env['CONFIG_TENANT']

	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)
	
	tree       = env['CONFIG_TREE']

	# group  <name> <name>
	group_name  = cli.canonical_words[2]
	group_name2 = cli.canonical_words[3]

	print group_name, group_name2

	if group_name != group_name2:
		cli.error("Name mismatch")
		return

	group = Group.objects.get(name=group_name)



	#
	# ACTION: FIX move to models
	#
	if user:
		user.assert_capability(tenant, 'tags.tagtree.acl.edit')
		tree.assert_permission_readwrite(user)
		user.assert_in_group(group)

	try:
		ttgp = TagTreeGroupPermission.objects.get(tenant=tenant, dirtree=tree, group=group)
		ttgp.delete()
	except TagTreeGroupPermission.DoesNotExist, e:
		pass


def set_tagtree_owner(cli):

	# owner <name>
	# no owner 

	env = cli.env

	user        = env['CURRENT_USER']
	
	print >> sys.stderr, __file__, "set_tagtree_owner", user

	tenant_name = env['CONFIG_TENANT']

	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant_name)

	tree       = env['CONFIG_TREE']

	if cli.canonical_words[0] != 'no':

		username = cli.canonical_words[1]

		owner = User.objects.get(username=username)

		tree.set_owner(owner, action_user=user)

	else:
		tree.unset_owner(action_user=user)



mt_config_tagtree = [

	T_Keyword('owner', help='', subtree=[
		T_Usercompleter(help='', callback=set_tagtree_owner)
	]),

	T_Keyword('group', help='', subtree=[
		T_Groupcompleter(help='Group name', pipe=None, callback=set_tagtree_group_access, subtree=[
			T_Alternatives(	CLI_PERM_OPTIONS, callback=set_tagtree_group_access
			)
		])
	]),


	T_Keyword('tag', help='', subtree=[
		T_TagCompleter  (help='', callback=cmd_config_tag_prompt , pipe=None,),
		T_Anyword       (help='', callback=cmd_config_tag_prompt , pipe=None,), #FIX: valid
	]),


	T_Keyword('uniques', help='', subtree=[
		T_Boolean(help='', callback=cmd_config_tagtree_field , pipe=None,),
	]),
	T_Keyword('allnodes', help='', subtree=[
		T_Boolean(help='', callback=cmd_config_tagtree_field , pipe=None,),
	]),

	T_Keyword('nostate' , help='', subtree=[
		T_Boolean(help='', callback=cmd_config_tagtree_field , pipe=None,),
	]),

	T_Keyword('nomap' , help='Enable/Disable map for the selected tag', subtree=[
		T_Boolean(help='', callback=cmd_config_tagtree_field , pipe=None,),
	]),

	T_Keyword('rename', help='Rename subtags', subtree=[
	
		T_Keyword('tag', help='Rename existing tag', subtree=[
			T_TagCompleter  (help='Rename existing tag', subtree=[
				T_Anyword       (help='Rename existing tag', callback=cmd_config_tag_rename , pipe=None,),
			]),
		]),
	]),

	T_Keyword('no', help='Remove subtags/Disable options', subtree=[
		T_Keyword('tag', help='Remove tag', subtree=[
			T_TagCompleter  (help='Remove tag', callback=cmd_tag_remove_subtag , pipe=None,),
		]),

		T_Keyword('owner', help='', callback=set_tagtree_owner),

		T_Keyword('group', help='', subtree=[
			T_Groupcompleter(help='Group name', pipe=None, subtree=[
				T_Groupcompleter(help='Group name', pipe=None, callback=unset_tagtree_group_access)
			])
		]),
	]), 

	T_Keyword('refresh', help='Refresh tree', callback=cmd_tagtree_refresh, ),

]


def cmd_treetag_config_prompt(cli):

	user = cli.env['CURRENT_USER']

	tenant_name  = cli.env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)

	name = cli.canonical_words[1]
	
	if not sys.stdin.isatty():
		#print "SIGNAL DISABLED"
		#print ""
		#set_signal_tagsd_server(False)
		pass

	# 
	# Create tagtree if missing
	# 
	tree = tagtree_get(tenant, name, create=True, owner=user)

	#
	# Enter in subsection
	#
	env = cli.env.clone()

	env['PROMPT'] = '%s' % ( PROMPT_SEP.join([ tenant_name, name ]) )

	env['CONFIG_TREE'] = tree
	env['CONFIG_TAG' ] = None

	env['CURRENT_TREE'] = env['CONFIG_TREE']
	env['CURRENT_TAG' ] = env['CONFIG_TAG' ]

	env['CONFIG_TAG_PATH'] = [tree.name]

	cmds = mt_config_tagtree  + SORTED_KEYWORDS(cli_get_registered_menus('__common_commands__') + cli_get_registered_menus('__show_commands__'))
	
	#cli = Cli(cmds, env, debug_interpreter=env['DEBUG'])
	#cli.cmdloop()
	cli.enter_subcli(cmds, env)

	#if not sys.stdin.isatty():
	#	set_signal_tagsd_server(True)
	#	signal_state_refresh(tenant.uuid, [tree.uuid])

	return


menu_configure_tagtree = [
	T_Keyword('tagtree', help='Configure tagtree', subtree=[
		T_TagTreeCompleter(help='Modify existing tagtree', callback=cmd_treetag_config_prompt   , pipe=None),
		T_ValidWord(VALID_TAGTREE_NAME, help='Create new tagtree', callback=cmd_treetag_config_prompt   , pipe=None),
	]),
]

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_treetag_remove(cli):

	user = cli.env['CURRENT_USER']

	# no tagtree <name> <name>
	tenant_name  = cli.env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)

	name = cli.canonical_words[2]
	name2 = cli.canonical_words[3]

	if name != name2:
		raise Exception("Names must be equal")

	#
	# ACTION
	#
	tagtree_delete(tenant, name, user=user)

	return

no_menu_configure_tagtree = [
	T_Keyword('tagtree', help='', subtree=[
		T_TagTreeCompleter(help='', subtree=[
			T_TagTreeCompleter(help='', callback=cmd_treetag_remove)
		]),
	]),
]

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_treetag_rename(cli):
	
	user = cli.env['CURRENT_USER']

	# rename tagtree <name> <name>
	tenant_name  = cli.env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)

	name = cli.canonical_words[2]
	name2 = cli.canonical_words[3]

	if name == name2:
		return

	tagtree = tagtree_get(tenant, name)

	# Il controllo sulle capability viene fatto qui
	tagtree.rename(name2, user=user)

	return


menu_rename_tagtree = [
	T_Keyword('tagtree', help='', subtree=[
		T_TagTreeCompleter(help='', subtree=[
			T_ValidWord(VALID_TAGTREE_NAME, help='', callback=cmd_treetag_rename)
		]),
	]),
]


def cmd_conf_tag_node_as(cli):
	
	tenant_name  = cli.env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant.name)
	
	name1 = cli.canonical_words[5]
	name2 = cli.canonical_words[3]
	
	node1 = MonitoredElement.objects.get(name=name1,tenant=tenant)
	node2 = MonitoredElement.objects.get(name=name2,tenant=tenant)
	
	tag_in_node1 = [ et.tag for et in ElementTag.objects.filter(elementid=node1.uuid).select_related('tag') ]
	tag_in_node2 = [ et.tag for et in ElementTag.objects.filter(elementid=node2.uuid).select_related('tag') ]
	
	for t in tag_in_node1:
		if t not in tag_in_node2:
			tag_element(t, node2)
	
	for t in tag_in_node2:
		if t not in tag_in_node1:
			untag_element(t, node2)
			
	return

def cmd_conf_tag_node_as_with_confirm(cli):
	
	tenant_name  = cli.env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant.name)
	
	name1 = cli.canonical_words[5]
	name2 = cli.canonical_words[3]
	
	node1 = MonitoredElement.objects.get(name=name1,tenant=tenant)
	node2 = MonitoredElement.objects.get(name=name2,tenant=tenant)
	
	tag_in_node1 = [ et.tag for et in ElementTag.objects.filter(elementid=node1.uuid).select_related('tag') ]
	tag_in_node2 = [ et.tag for et in ElementTag.objects.filter(elementid=node2.uuid).select_related('tag') ]
	
	if sys.stdin.isatty():
	
		for t in tag_in_node1:
			if t not in tag_in_node2:
				
				resp = raw_input ('Do you want to add tag '+t.path+' to '+node2.name+' ? (Y/N)')
				
				if resp == 'Y':
					tag_element(t, node2)
		
		for t in tag_in_node2:
			if t not in tag_in_node1:
				
				resp = raw_input ('Do you want to remove tag '+t.path+' to '+node2.name+' ? (Y/N)')
				
				if resp == 'Y':
					untag_element(t, node2)
	else:
		print 'This is an interactive command, you can not use it in non-interactive tty'
	
	return
	
def cmd_conf_tag_node_like(cli):
	
	tenant_name  = cli.env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant.name)
	
	name1 = cli.canonical_words[5]
	name2 = cli.canonical_words[3]
	
	node1 = MonitoredElement.objects.get(name=name1,tenant=tenant)
	node2 = MonitoredElement.objects.get(name=name2,tenant=tenant)
	
	tag_in_node1 = [ et.tag for et in ElementTag.objects.filter(elementid=node1.uuid).select_related('tag') ]
	tag_in_node2 = [ et.tag for et in ElementTag.objects.filter(elementid=node2.uuid).select_related('tag') ]
	
	for t in tag_in_node1:
		if t not in tag_in_node2:
			tag_element(t, node2)
			
	return

def cmd_conf_tag_node_like_with_confirm(cli):
	
	tenant_name  = cli.env['CONFIG_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant.name)
		
	name1 = cli.canonical_words[5]
	name2 = cli.canonical_words[3]
	
	node1 = MonitoredElement.objects.get(name=name1,tenant=tenant)
	node2 = MonitoredElement.objects.get(name=name2,tenant=tenant)
	
	tag_in_node1 = [ et.tag for et in ElementTag.objects.filter(elementid=node1.uuid).select_related('tag') ]
	tag_in_node2 = [ et.tag for et in ElementTag.objects.filter(elementid=node2.uuid).select_related('tag') ]
	
	if sys.stdin.isatty():
		
		for t in tag_in_node1:
			
			if t not in tag_in_node2:
				
				resp = raw_input ('Do you want to add tag '+t.path+' to '+node2.name+' ? (Y/N)')
				
				if resp == 'Y':
					tag_element(t, node2)
	else:
		print 'This is an interactive command, you can not use it in non-interactive tty'
			
	return
	
menu_configure_tag = [
	T_Keyword('copy', help='Tag configuration in nodes', pipe=None, subtree=[
		T_Keyword('tags', help='Tag configuration in nodes', pipe=None, subtree=[
			T_Keyword('node', help='Tag configuration in nodes', pipe=None, subtree=[
				T_MonitoredNodecompleter(help="Node configuration", subtree=[
				
					T_Keyword('like', help="Node configuration", subtree=[
						T_MonitoredNodecompleter(help="Node configuration", callback=cmd_conf_tag_node_like, subtree=[
							T_Keyword('ask-confirm', help="Node configuration", callback=cmd_conf_tag_node_like_with_confirm)
						])
					]),
					T_Keyword('exactly-as', help="Node configuration", subtree=[
						T_MonitoredNodecompleter(help="Node configuration", callback=cmd_conf_tag_node_as, subtree=[
							T_Keyword('ask-confirm', help="Node configuration", callback=cmd_conf_tag_node_as_with_confirm)
						])
					]),
				]), 
			]),
		]),
	])
]

def show_node_diff(cli):
	
	tenant_name  = cli.env['CURRENT_TENANT']
	tenant = tenant_get(tenant_name)
	db_set_current_tenant_schema(tenant.name)
	
	name1 = cli.canonical_words[3]
	name2 = cli.canonical_words[4]
	
	node1 = MonitoredElement.objects.get(name=name1, tenant=tenant)
	node2 = MonitoredElement.objects.get(name=name2, tenant=tenant)
	
	tag_in_node1 = [ et.tag for et in ElementTag.objects.filter(elementid=node1.uuid).select_related('tag') ]
	tag_in_node2 = [ et.tag for et in ElementTag.objects.filter(elementid=node2.uuid).select_related('tag') ]
	
	tag_in_node1.sort(key=lambda x: x.path)
	tag_in_node2.sort(key=lambda x: x.path)
	
	header = [[
		[_('TAG'), 50],
		[_('PATH'), 200]
	]]
	
	table1 = header
	print 'tag node '+name1+' and not '+name2+':\n'
	for t in tag_in_node1:
		if t not in tag_in_node2:
			table1 = table1 + [[
				t.name,
				t.path
			]]

	
	output_table(cli, table1)
	
	table2 = header		
	print 'tag node '+name2+' and not '+name1+':\n'
	for t in tag_in_node2:
		if t not in tag_in_node1:
			table2 = table2 + [[
				t.name,
				t.path
			]]
	
	output_table(cli, table2)
	
	table3 = header		
	print 'tag node '+name1+' and '+name2+':\n'
	for t in tag_in_node1:
		if t in tag_in_node2:
			table3 = table3 + [[
				t.name,
				t.path
			]]
	
	output_table(cli, table3)
			
	return
	
menu_tag_diff = [
	T_Keyword('diff', help='Difference between nodes in tag configuration', pipe=None, subtree=[
		T_Keyword('tag', help='Difference between nodes in tag configuration', pipe=None, subtree=[
			T_MonitoredNodecompleter(help="Node 1", pipe=None, subtree=[
				T_MonitoredNodecompleter(help="Node 2", pipe=None, callback=show_node_diff)
			])
		])
	])
]
	
	
	



