# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import re

from django.utils.translation import ugettext as _

from sanet_common.sanetdefs.constants import *
from sanet_common.timeutils import timestamp_to_isostr

from core.api.base import tenant_get

from cli.clilib.ioslike import T_Keyword
from cli.tokens import *
from cli.textformat import *
from cli.prompts import cli_get_registered_hooks
from cli.prompts.defs import *
from cli.prompts.common import output_table

from cli.prompts.show.datagroups import _show_datagroups_list
from cli.prompts.show.datagroups import _show_conditions_list
from cli.prompts.show.datagroups import _show_timegraphs_list
from cli.prompts.show.datagroups import _show_datasources_list

from tags.models import *
from tags.cli.tokens import *

from tags.server.tagsd.api import get_tags_state_safe, get_trees_state_safe

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def show_conf_tags(cli, tenant, level=0):

	user = cli.env['CURRENT_USER']

	l = level

	cli.pij(l, cli.CLI_COMMENT('#')      )   
	cli.pij(l, cli.CLI_COMMENT('# TAGS') )
	cli.pij(l, cli.CLI_COMMENT('#')      )
	
	#trees =  TagTree.objects.filter(tenant=tenant)
	
	trees = TagTree.get_visible_by_user(tenant, user)
	
	for tree in trees:
		show_conf_tagtree(cli, tree, level=level)
		
	return

def cmd_show_conf_tagtrees(cli):

	user = cli.env['CURRENT_USER']

	# show conf tagtree [<name>]

	tenant_name = cli.env['CURRENT_TENANT']
	tenant = tenant_get(tenant_name)

	db_set_current_tenant_schema(tenant.name)

	if len(cli.canonical_words) > 3:
		name = cli.canonical_words[3]
		trees = TagTree.objects.filter(name=name)
	else:
		#trees = TagTree.objects.all()
		trees = TagTree.get_visible_by_user(tenant, user)

	for t in trees:
		 show_conf_tagtree(cli, t)

def show_conf_tagtree(cli, tree, level=0):
	l = level
	cli.piqj(l, cli.CLI_KEYWORD('tagtree'), tree.name)
	l+=1

	if tree.owner:
		cli.piqj(l, cli.CLI_KEYWORD('owner'   ), tree.owner.username)

	for auth in TagTreeGroupPermission.objects.filter(tenant=tree.tenant, dirtree=tree):
		cli.piqj(l, cli.CLI_KEYWORD('group'), auth.group.name, cli.CLI_KEYWORD('read-only' if auth.auth_type == DIRENTRY_AUTH_READONLY else 'read-write' ) )

	if tree.uniques : cli.piqj(l, cli.CLI_KEYWORD('uniques' ), CLI_BOOL(tree.uniques))
	if tree.allnodes: cli.piqj(l, cli.CLI_KEYWORD('allnodes'), CLI_BOOL(tree.allnodes))
	if tree.nostate : cli.piqj(l, cli.CLI_KEYWORD('nostate' ), CLI_BOOL(tree.nostate))
	if tree.nomap   : cli.piqj(l, cli.CLI_KEYWORD('nomap'   ), CLI_BOOL(tree.nomap))

	for tag in Tag.objects.filter(tenant=tree.tenant, tree=tree, parent=None):
		show_conf_tag(cli, tag, level=l)

	l-=1 
	cli.piqj(l, cli.CLI_KEYWORD('exit') )

def show_conf_tag(cli, tag, level=0):

	l = level
	cli.piqj(l, cli.CLI_KEYWORD('tag'), tag.name)
	l+=1

	if tag.descr:
		cli.piqj(l, cli.CLI_KEYWORD('description') , tag.descr)


	hooks = cli_get_registered_hooks('show:tag:configuration')
	for h in hooks:
		h(cli, tag, level=l)

	# Show meta
	for r in tag.metadata.order_by('name').all():
		cli.piqj(l, cli.CLI_KEYWORD('meta'), r.name, r.value)

	# Show insert rules
	for r in TagEntryInsertRule.objects.filter(tenant=tag.tenant, tagentry=tag).order_by('rule').all():
		cli.piqj(l, cli.CLI_KEYWORD('insert-rule'), r.rule)

	# Show subtags
	for t in Tag.objects.filter(tenant=tag.tenant, parent=tag).order_by('name').all():
		show_conf_tag(cli, t, level=l)

	show_tagged_elements(cli, tag, level=l)

	show_explosive_elements(cli, tag, level=l)

	# Show configuration from plugins/sub-apps
	hooks = cli_get_registered_hooks('show:tag:configuration_post')
	for h in hooks:
		h(cli, tag, level=l)

	l-=1 
	cli.piqj(l, cli.CLI_KEYWORD('exit'))


def show_tagged_elements(cli, tag, level=0):

	l = level

	tagged_elements = ElementTag.objects.filter(tenant=tag.tenant, tag=tag)

	tagged_elements_data_by_id = dict([ (r['elementid'], r) for r in tagged_elements.values('elementid', 'elementtype', 'directly_tagged') ])

	#
	# NODI taggati direttamente
	# 

	extras = {
		'tables': ['tagentry','tags_elementtag'],
		'where' : [
			'tagentry.uuid = %s', 
			'tagentry.uuid = tags_elementtag.tag_id', 
			'tags_elementtag.elementid = node.monitoredelement_ptr_id', 
			'tags_elementtag.directly_tagged = true'
		],
		'params': [tag.uuid]
	}
	nodes = MonitoredNode.objects.extra(**extras).order_by('name')

	for node in nodes:

		cli.piqj(l, cli.CLI_KEYWORD('node'), node.name)

		#
		# Datagroup taggati con condition non taggate
		#
		datagroups = _get_datagroups_to_show(cli, tagged_elements_data_by_id, node)

		_show_datagroups(cli, datagroups, level=l+1)

		#
		# Interfacce taggate contenenti datagroup con condition non taggate
		#
		for cmd, kind in [ ('interface', ELEMENT_KIND_IFACE), ('storage', ELEMENT_KIND_STORAGE), ('service', ELEMENT_KIND_SERVICE), ('device', ELEMENT_KIND_DEVICE), ]:

			elements = node.get_sub_elements(kind=kind)

			l += 1
			for i in elements:
				tagdata = tagged_elements_data_by_id.get(i.uuid, None)
				if tagdata is not None:

					if tagdata['directly_tagged'] == False:

						datagroups = _get_datagroups_to_show(cli, tagged_elements_data_by_id, i)
						if not datagroups: continue

						cli.piqj(l, cli.CLI_KEYWORD(cmd), i.name)

						_show_datagroups(cli, datagroups, level=l+1)

						cli.piqj(l, cli.CLI_KEYWORD('exit'))
				else:
						cli.piqj(l, cli.CLI_KEYWORD('no'), cli.CLI_KEYWORD(cmd), i.name)
			l -= 1

		cli.piqj(l, cli.CLI_KEYWORD('exit'))

	# dobbiamo mostrare tutti gli elementi di monitoraggio che non sono nodi taggati direttamente
	TAGGED_ELEMENTS = [
		('interface', MonitoredIface  , 'iface'  ),
		('storage'  , MonitoredStorage, 'storage'),
		('service'  , MonitoredService, 'service'),
		('device'   , MonitoredDevice , 'device' ),
	]

	#
	# IFACE: solo quelle taggate direttamente (e non ricorsivamente dal nodo
	#
	for cli_cmd, class_model, table_name in TAGGED_ELEMENTS:

		extras = {
			'tables': ['tagentry','tags_elementtag'],
			'where' : [
				'tagentry.uuid = %s', 
				'tagentry.uuid = tags_elementtag.tag_id', 
				'tags_elementtag.elementid = %s.monitoredelement_ptr_id' % (table_name), 
				'tags_elementtag.directly_tagged = true'
			],
			'params': [tag.uuid]
		}
		elements = class_model.objects.extra(**extras).order_by('name')

		for element in elements:

			cli.piqj(l, cli.CLI_KEYWORD(cli_cmd), element.get_node().name, element.name)

			datagroups_to_show = _get_datagroups_to_show(cli, tagged_elements_data_by_id, element)
			if datagroups_to_show: 
				_show_datagroups(cli, datagroups_to_show , level=l+1)

			cli.piqj(l, cli.CLI_KEYWORD('exit'))

	#
	# DATAGROUPS: solo quelle taggate direttamente (e non ricorsivamente dal nodo
	#

	"""
	extras = {
		'tables': ['tagentry','tags_elementtag'],
		'where' : ['tagentry.uuid = %s', 'tagentry.uuid = tags_elementtag.tag_id', 'tags_elementtag.elementid = datagroup.uuid', 'tags_elementtag.directly_tagged = true'],
		'params': [tag.uuid]
	}
	single_datagroups = DataGroup.objects.extra(**extras).order_by('name')

	for dg in single_datagroups:
		
		cli.piqj(l, 'datagroup', dg.path)
		
		close = False
		l += 1
		conditions = dg.conditions.all()
		for c in conditions:
			#if c.uuid not in tagged_ids:
			if c.uuid not in tagged_elements_data_by_id:
				cli.piqj(l, 'no', 'condition', c.name)
				close = True
		l -= 1
		
		if not close:
			continue
			
		cli.piqj(l, 'exit')
	"""

	#
	# CONDITIONS: solo quelle taggate direttamente (e non ricorsivamente dal nodo
	#

	extras = {
		'tables': ['tagentry','tags_elementtag'],
		'where' : [
			'tagentry.uuid = %s', 
			'tagentry.uuid = tags_elementtag.tag_id', 
			'tags_elementtag.elementid = condition.uuid', 
			'tags_elementtag.directly_tagged = true'
		],
		'params': [tag.uuid]
	}
	conditions = Condition.objects.extra(**extras).order_by('path').values_list('path')
	for c in conditions:
		cli.piqj(l, cli.CLI_KEYWORD('condition'), c[0])


def _get_datagroups_to_show(cli, tagged_elements_data_by_id, element):

	datagroup_to_expand = {}

	for dg in element.datagroups.all():

		# Tagged?
		tagdata = tagged_elements_data_by_id.get(dg.uuid, None)
		if tagdata is None:
			continue

		if tagdata['directly_tagged'] == True:
			continue

		for c in dg.conditions.all():

			tagdata = tagged_elements_data_by_id.get(c.uuid, None)

			if tagdata is None:
				dg_conf = datagroup_to_expand.get( dg.uuid  ,  datagroup_to_expand.setdefault( dg.uuid, {'dg':dg, 'conditions':[] } ) )

				dg_conf['conditions'].append(c)

	return datagroup_to_expand


def _show_datagroups(cli, datagroup_to_expand, level=0):
	l = level

	for dg_conf in datagroup_to_expand.values():

		if not dg_conf['conditions']:
			continue

		cli.piqj(l, cli.CLI_KEYWORD('datagroup'), dg_conf['dg'].name)
		l += 1
		for c in dg_conf['conditions']:
			cli.piqj(l, cli.CLI_KEYWORD('no'), cli.CLI_KEYWORD('condition'), c.name)
		l -= 1
		cli.piqj(l, cli.CLI_KEYWORD('exit'))


def show_explosive_elements(cli, tag, level=0):

	l = level

	tagged_elements = ElementTag.objects.filter(tenant=tag.tenant, tag=tag, is_explosive=True).all()

	for et in tagged_elements:

		model = TAG_ELEMENTTYPE_TO_MODEL[et.elementtype]

		element = model.objects.get(uuid=et.elementid)

		if et.elementtype == 'condition':
			args = [ cli.CLI_KEYWORD('explosive'), cli.CLI_KEYWORD('condition'), element.path]
		else:
			args = [ cli.CLI_KEYWORD('explosive'), cli.CLI_KEYWORD(et.elementtype), ]

			if element.kind == ELEMENT_KIND_NODE:
				args += [ element.name ]
			else:
				args += [ element.parent.name, element.name ]

		if et.explosive_group:
			args += [ cli.CLI_KEYWORD('group'), str(et.explosive_group) ]

		cli.piqj(l, *args)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def _tag_status(cli, s):
	# L'output colorato va usato solo se e' abilitato e l'output e' in formato normale.
	# Il csv non deve contenere carattereri speciali
	if cli.env.get('COLORS', False) == True and cli.env.get('OUTPUT_FORMAT','normal') == 'normal':
		return (COLORED_STATUS_TEXT, s     , CSTATE_TO_CODENAME(s)     )
	else:
		return CSTATE_TO_CODENAME(s)


def cmd_show_tagtree(cli):

	user = cli.env['CURRENT_USER']

	# show tagtree [ name  <name> ]
	# show tagtree [ visible-by <user> ]

	current_tenant = cli.env.get('CURRENT_TENANT', None)

	cols = cli.env.get('COLS',None)
	tenant = tenant_get(current_tenant)

	db_set_current_tenant_schema(tenant.name)

	if len(cli.canonical_words) <= 2:

		ret = TagTree.get_visible_trees_by_user(tenant, user)

		_tagtree_table(cli, tenant, ret)

	elif cli.canonical_words[2] == 'visible-by':

		if user:
			if not user.is_superuser:
				cli.error("Command not allowed for normal users")
				return

		username = cli.canonical_words[3]

		user = User.objects.get(username=username)

		ret = TagTree.get_visible_trees_by_user(tenant, user)

		_tagtree_table(cli, tenant, ret)

	elif cli.canonical_words[2] == 'name':

		name = cli.canonical_words[3]

		e = TagTree.objects.get(tenant=tenant, name=name)

		cli.prnt(
			generic_details(
				e,
			)
		)

		r = []
		r.append([
			[_('GROUP')        ,  9],
			[_('AUTH')         ,  3],
		])
		for gp in e.get_group_permissions().all():
			r.append([
				gp.group.name,
				gp.str_auth_type()
			])
		output_table(cli, r)

	return



def _tagtree_table(cli, tenant, tagtrees):

		tagtrees = tagtrees.order_by('-nostate', 'name')

		#
		# GET STATE
		#
		states_by_uuid = get_trees_state_safe(tenant.uuid, [ e.uuid for e in tagtrees if e.nostate == False] )

		#
		# OUTPUT
		#
		r = []
		r.append([
			[_('TAG')        ,  9],
			[_('ALL')        ,  3],
			[_('UNIQUE')     ,  5],
			[_('NOSTATE')    ,  5],
			[_('NOMAP')      ,  5],
			[_('OWNER')      , 10],
			[_('DESCRIPTION'), 60],
			[_('STATUS')     , 30],
			[_('STIME')       , 20],
		])

		for e in tagtrees:

			# Is sanetserver working?
			if states_by_uuid is not None:
				data = states_by_uuid.get(e.uuid, None)
				if data is not None:
					status_str = _tag_status(cli, data['visible_status'])
					stime_str  = timestamp_to_isostr(data['_ts'])
				else:
					status_str = '-'
					stime_str  = '-'
			else:
				status_str = '*ERROR*'
				stime_str  = '-'

			r.append([
				e.name,
				'*' if e.allnodes else '',
				'*' if e.uniques  else '',
				'*' if e.nostate  else '',
				'*' if e.nomap  else '',
				e.owner,
				e.descr,
				status_str,
				stime_str,
			])

		output_table(cli, r)

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def cmd_show_tags(cli):
	
	user = cli.env['CURRENT_USER']

	# show tags [ tree <name> ]

	current_tenant = cli.env.get('CURRENT_TENANT', None)

	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	tree_name = None
	if len(cli.canonical_words) > 2:
		if cli.canonical_words[2] == 'tree':
			tree_name = cli.canonical_words[3]

	#
	# Load tags to show
	#
	visible_trees = TagTree.get_visible_trees_by_user(tenant, user)


	tags = Tag.objects.select_related('tree', 'tree__owner')

	tags = tags.filter(tree__in=visible_trees)

	if tree_name:
		tags = tags.filter(tree__name=tree_name)

	tags = tags.order_by('-tree__nostate', 'path')

	#
	# Load states
	#
	states_by_uuid = get_tags_state_safe( tenant.uuid, [t.uuid for t in tags])

	#
	# Print 
	#
	r = []

	r.append([
		[_('UUID')        , 32],
		[_('TREE')        , 20],
		[_('NAME')        , 20],
		[_('ICON')        , 10],
		[_('OWNER')       , 20],
		[_('PATH')        , 20],
		[_('STATUS')      ,  5],
		[_('STIME')       , 20],
	])

	for t in tags:

		# Is sanetserver working?
		if states_by_uuid is not None:
			data = states_by_uuid[t.uuid]
			if data is not None:
				status_str = _tag_status(cli, data['visible_status'])
				stime_str  = timestamp_to_isostr(data['_ts'])
			else:
				status_str = '-'
				stime_str  = '-'
		else:
			status_str = '*ERROR*'
			stime_str  = '-'

		r.append([
			t.uuid,
			t.tree.name,
			t.name,
			t.icon,
			t.tree.owner.username if t.tree.owner is not None else '',
			t.path,

			status_str,
			stime_str,
		])

	output_table(cli, r)


def cmd_show_tag_info(cli):

	cols = cli.env['COLS']

	# show tag <tree> <path>

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)

	tree_name = cli.canonical_words[3]
	path      = cli.canonical_words[4]

	tree = TagTree.objects.get(tenant=tenant, name=tree_name)

	path = TAG_PATH_DIVIDER.join([tree.path, path])

	tag = tree.tags.get(path=path)

	metas = tag.metadata.order_by('name').all()
	if metas:
		cli.prnt("")
		r = []
		r.append([
			[_('META'), 20],
			[_('VALUE')  , 60],
		])
		for m in metas:
			r.append([
				m.name,
				m.value.replace('\n',"\\n"),
			])
		cli.prnt(text_table(r, cols))

	subtags = Tag.objects.filter(tenant=tag.tenant, parent=tag).order_by('name').all()
	if subtags:
		cli.prnt("")
		r = []
		r.append([
			[_('SUB TAG'), 20],
			[_('DESCR')  , 60],
		])
		for t in subtags:
			r.append([
				t.name,
				t.description
			])
		cli.prnt(text_table(r, cols))


	for etype in TAG_ELEMENTTYPE_TO_MODEL.keys():

		r = []
		r.append([
			[_('TYPE')    , 20],
			[_('NAME')    , 20],
			[_('DIRECT')  ,  6],
			[_('E')       , 10],
			[_('GROUP ID'), 20],
		])
		elements = ElementTag.objects.filter(tenant=tag.tenant, tag=tag, elementtype=etype).all()
		if not elements:
			continue

		for te in elements:
			r.append([
				te.elementtype, #TAG_ELEMENTTYPE_TO_CLI[te.elementtype], 
				te.element.name,
				'*' if te.directly_tagged else '',
				'E' if te.is_explosive else '',
				str(te.explosive_group) if te.explosive_group is not None else '',
			])

		cli.prnt(text_table(r, cols))


def cmd_show_tag_by_node(cli):

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	cols = cli.env.get('COLS',None)

	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	name = cli.canonical_words[3]

	kind = ELEMENT_KIND_NODE
	element = MonitoredElement.objects.get(tenant=tenant, kind=kind, name=name)

	et = ElementTag.objects.filter(elementid=element.uuid).order_by('tag__name')

	r = []
	r.append([
		[_('NAME'), 20],
		[_('PATH'), 20],
	])

	for e in et:
		r.append( [ e.tag.name, e.tag.path ] )

	output_table(cli, r)


def cmd_show_tag_by_iface(cli):
	_cmd_show_tag_by_element(cli, ELEMENT_KIND_IFACE)

def cmd_show_tag_by_storage(cli):
	_cmd_show_tag_by_element(cli, ELEMENT_KIND_STORAGE)

def cmd_show_tag_by_service(cli):
	_cmd_show_tag_by_element(cli, ELEMENT_KIND_SERVICE)

def cmd_show_tag_by_device(cli):
	_cmd_show_tag_by_element(cli, ELEMENT_KIND_DEVICE)

def _cmd_show_tag_by_element(cli, kind):

	# show tag iface <node> <iface>

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	cols = cli.env.get('COLS',None)

	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	node_name = cli.canonical_words[3]
	name = cli.canonical_words[4]

	node = MonitoredNode.objects.get(tenant=tenant, name=node_name)

	element = MonitoredElement.objects.get(tenant=tenant, kind=kind, name=name, parent=node)

	et = ElementTag.objects.filter(elementid=element.uuid).order_by('tag__name')

	r = []
	r.append([
		[_('NAME'), 20],
		[_('PATH'), 20],
	])

	for e in et:
		r.append( [ e.tag.name, e.tag.path ] )

	output_table(cli, r)

def cmd_show_tag_counters(cli):

	current_tenant = cli.env.get('CURRENT_TENANT', None)

	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	tag_path = cli.canonical_words[3]+':/'+cli.canonical_words[4]

	tag = Tag.objects.get(path=tag_path)

	conters = tag.get_tag_element_counter()

	for c in conters:
		print c[1]+" : "+str(c[0])

	return

_menu_show_tag = [

	T_Keyword('tagtree', help=_('Show defined tag trees'), callback=cmd_show_tagtree, subtree=[

		T_Keyword('name', help=_('by name'), subtree=[
			T_TagTreeCompleter(help=_('Show tag tree informations'), callback=cmd_show_tagtree)

		]),
		T_Keyword('visible-by', help=_('Visible by user'), subtree=[
			T_Usercompleter(help=_('Visible by user'), callback=cmd_show_tagtree)
		]),

	]),

	T_Keyword('tag', help=_('Show tag informatinos'), callback=cmd_show_tags, subtree=[
	
		T_Keyword('info', help=_("Show tag's informations"), subtree=[
			T_TagTreeCompleter(help=_("Show tag's informations"), subtree=[
				T_InLineTagPathCompleter(help=_("Show tag's informations"), callback=cmd_show_tag_info, subtree=[
					T_Keyword('summary', help=_("Show tag's element type counters"), callback=cmd_show_tag_counters)
				]),
			])
		]),

		T_Keyword('tree', help=_("Show tags' by tree"), subtree=[
			T_TagTreeCompleter(help=_("Show tag's informations"), callback=cmd_show_tags)
		]),

		T_Keyword('node', help=_("Show tags assigned to node"), subtree=[
			T_MonitoredNodecompleter(help=_("Show tags assigned to node"), callback=cmd_show_tag_by_node )
			
		]),

		T_Keyword('interface', help=_("Show tags assigned to interface"), subtree=[
			T_MonitoredNodecompleter(help=_("Show tags assigned to interface"), subtree=[
				T_InlineIfaceCompleter(help=_("Show tags assigned to interface"), callback=cmd_show_tag_by_iface )
			])
		]),

		T_Keyword('storage', help=_("Show tags assigned to storage"), subtree=[
			T_MonitoredNodecompleter(help=_("Show tags assigned to storage"), subtree=[
				T_InlineStorageCompleter(help=_("Show tags assigned to storage"), callback=cmd_show_tag_by_storage )
			])
		]),

		T_Keyword('service', help=_("Show tags assigned to service"), subtree=[
			T_MonitoredNodecompleter(help=_("Show tags assigned to service"), subtree=[
				T_InlineServiceCompleter(help=_("Show tags assigned to service"), callback=cmd_show_tag_by_service )
			])
		]),

		T_Keyword('device', help=_("Show tags assigned to device"), subtree=[
			T_MonitoredNodecompleter(help=_("Show tags assigned to service"), subtree=[
				T_InlineDeviceCompleter(help=_("Show tags assigned to device"), callback=cmd_show_tag_by_device )
			])
		]),

	]),
]


def cmd_show_tag_logs(cli):

	from cli.prompts.show.logs import _show_last_resource_logs

	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	cols = cli.env['COLS']

	# show tag <tree> <path>

	tree_name = cli.canonical_words[3]
	path      = cli.canonical_words[4]

	path = tree_name + ':/' + path

	tree = TagTree.objects.get(tenant=tenant, name=tree_name)
	tag = Tag.objects.get(tenant=tenant, tree=tree, path=path)

	_show_last_resource_logs(cli, tag, None)


_menu_show_log_tag = [
	T_Keyword('tag', help=_('Show tag informations'), subtree=[
		T_TagTreeCompleter(help=_("Show tag's informations"), subtree=[
			T_InLineTagPathCompleter(help=_("Show tag's informations"), callback=cmd_show_tag_logs )
		])
	]),
]

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def _get_command_params(cli):

	# show node tagtree <tree> <tag>

	current_tenant = cli.env.get('CURRENT_TENANT', None)

	tenant = tenant_get(current_tenant)
	db_set_current_tenant_schema(tenant.name)

	cols = cli.env['COLS']

	# show tag <tree> <path>
	
	current_tenant = cli.env.get('CURRENT_TENANT', None)
	tenant = tenant_get(current_tenant)

	tree_name = cli.canonical_words[3]
	path = None
	if len(cli.canonical_words) > 4:
		path      = cli.canonical_words[4]
		path = tree_name +':/' + path

	recursive = not (len(cli.canonical_words) > 5 and cli.canonical_words[5] == 'no-recursive')

	tree = TagTree.objects.get(tenant=tenant, name=tree_name)
	if path is not None:
		tag = Tag.objects.get(tenant=tenant, tree=tree, path=path)
	else:
		tag = Tag.objects.get(tenant=tenant, tree=tree, parent=None)

	return tag, recursive


def show_node_tag(cli):
	## show node tagtree <tree> <tag>

	tag, recursive = _get_command_params(cli)

	nodes = tag.get_all_nodes(recursive=recursive)

	#
	# Load states
	#

	tree = tag.tree
	tenant = tag.tenant
	
	#
	# FIX: un nodo potrebbe essere taggato in punti diversi del tree ed avere un tagging di copertura diverso.
	#      Quesot codice restituisce un solo stato del nodo e non e' chiaro a quale elementtag fara' riferimento.
	#
	elementtag_ids       = ElementTag.objects.filter(elementtype=TAG_ELEMENTTYPE_NODE, tree=tree).values_list('uuid', 'elementid')
	elementtag_id_by_nodeid = dict([ (e[1], e[0]) for e in elementtag_ids])
	states_by_uuid = get_tags_state_safe( tenant.uuid, [e[0] for e in elementtag_ids])


	r = []
	r.append([
		[_('NAME') , 20],
		[_('STATE'), 10],
		[_('TIME'),  20],
	])

	for n in nodes:

		# Is sanetserver working?
		if states_by_uuid is not None:

			et_id = elementtag_id_by_nodeid[n.uuid]

			data = states_by_uuid.get(et_id, None)
			if data is not None:
				status_str = _tag_status(cli, data['visible_status'])
				stime_str  = timestamp_to_isostr(data['_ts'])
			else:
				status_str = '-'
				stime_str  = '-'
		else:
			status_str = '*ERROR*'
			stime_str  = '-'

		r.append( [ n.name, status_str, stime_str ] )

	output_table(cli, r)

def show_datagroup_tag(cli):

	tag, recursive = _get_command_params(cli)

	datagroups = tag.get_all_datagroups()

	_show_datagroups_list(cli, datagroups)

	return

def show_datasource_tag(cli):

	tag, recursive = _get_command_params(cli)

	nodes = tag.get_all_nodes(recursive=recursive)

	datasources = tag.get_all_datasources()

	_show_datasources_list(cli, datasources)

	return

def show_condition_tag(cli):

	tag, recursive = _get_command_params(cli)

	nodes = tag.get_all_nodes(recursive=recursive)

	conditions = tag.get_all_conditions()

	_show_conditions_list(cli, conditions)

	return

def show_timegraph_tag(cli):

	tag, recursive = _get_command_params(cli)

	timegraphs = tag.get_all_timegraphs()

	_show_timegraphs_list(cli, timegraphs)

	return

def _cmd_show_element_by_tag(cli):

	# show  <element type>  tagtree <tree> <path>

	if cli.canonical_words[1] == 'node'      : return show_node_tag(cli)
	if cli.canonical_words[1] == 'datagroup' : return show_datagroup_tag(cli)
	if cli.canonical_words[1] == 'datasource': return show_datasourcee_tag(cli)
	if cli.canonical_words[1] == 'condition' : return show_condition_tag(cli)
	if cli.canonical_words[1] == 'timegraph' : return show_timegraph_tag(cli)

	return

_menu_show_element_by_tag = [
	T_Keyword('tagtree', help=_(""), subtree=[
		T_TagTreeCompleter(help=_(""), callback=_cmd_show_element_by_tag, subtree=[
			T_InLineTagPathCompleter(help=_(""), callback=_cmd_show_element_by_tag, subtree=[
				T_Keyword('no-recursive', help=_(""), callback=_cmd_show_element_by_tag,)
			])
		])
	]),
]

_menu_show_node_tag       = _menu_show_element_by_tag
_menu_show_datagroup_tag  = _menu_show_element_by_tag
_menu_show_datasource_tag = _menu_show_element_by_tag
_menu_show_condition_tag  = _menu_show_element_by_tag
_menu_show_timegraph_tag  = _menu_show_element_by_tag

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

_menu_show_conf_tagtree = [
	T_Keyword('tagtree' , help=_('show tag tree configuration'), callback=cmd_show_conf_tagtrees, subtree=[
		T_TagTreeCompleter(help=_('show tag tree configuration'), callback=cmd_show_conf_tagtrees, ),
	]),
]

def show_configuration_this_tagtree(cli):
	tag_tree = cli.env.get('CONFIG_TREE', None)
	if tag_tree:
		#n = node_get(node_name, tenant_name=tenant_name)
		show_conf_tagtree(cli, tag_tree)
		return True
	return False

def tag_show_node_tag_meta(cli, n):

	tagged = ElementTag.objects.filter(elementid=n.uuid)
	tags = [ et.tag for et in tagged ]

	metas = TagMetaData.objects.filter(tag__in=tags).select_related('tag').all()

	r = []
	r.append([
		[_('TAG PATH'), 20],
		[_('META'), 20],
		[_('VALUE'), 20],
	])

	for mt in metas:
		r.append( [ mt.tag.path, mt.name, mt.value.replace("\n", "\\n") ] )
	output_table(cli, r)	

