# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from django import db
#from django.db import connection

from core.models import Tenant

from core.api.base import *

from tags.models import *

from cli.clilib.ioslike import *

from cli.tokens import T_DataGroupCompleter, T_ConditionCompleter

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#


class T_TagTreeCompleter(T_Anyword):

	klass        = None   

	def __init__(self, *args, **kwargs):
		super(T_TagTreeCompleter, self).__init__(*args, **kwargs)
		self.token_repr = '<tree>'

	def get_query_set(self, cli):
		user = cli.env['CURRENT_USER']

		current_tenant_name = cli.env['CURRENT_TENANT']
		db_set_current_tenant_schema(current_tenant_name)
		tenant = Tenant.objects.get(name=current_tenant_name)

		trees = TagTree.get_visible_by_user(tenant, user)

		return trees

	def matches(self, cli, word, ):
		trees = self.get_query_set(cli)

		return trees.filter(**{'name__startswith': word}).count() > 0

	def completion(self, cli, text):
		try:
			trees = self.get_query_set(cli)

			res = trees.filter( **{'name__startswith' :text} ).all()

			rv = []
			for n in res:
				rv.append(HelpingString( n.name, n.name))
			return rv
		except Exception, e:
			raise e

	def canonical(self, cli, word):
		#try:
		#	trees = self.get_query_set(cli)
		#
		#	res = trees.get( **{'name': word} )
		#	return res.name
		#except TagTree.DoesNotExist, e:
		#	return None
		#
		#except TagTree.MultipleObjectsReturned, e:
		#	try:
		#		trees = self.get_query_set(cli)
		#		r = trees.get(**{'name': word}).name
		#	except Exception, e:
		#		return "**ERROR**"
		#
		#except Exception, e:
		#	raise e
		try:
			trees = self.get_query_set(cli)
			names = [t[0] for t in trees.filter( **{'name__startswith' :word} ).values_list('name').all() ]
			if len(names) == 1:
				return names[0]
			elif len(names) > 1:
				for name in names:
					if name == word:
						return name
			return super(T_TagTreeCompleter, self).canonical(cli, word)
		except Exception, e:
			raise e

class T_SystemTagTreeCompleter(T_TagTreeCompleter):

	def __init__(self, *args, **kwargs):
		super(T_SystemTagTreeCompleter, self).__init__(*args, **kwargs)
		self.token_repr = '<system tree>'

	def get_query_set(self, cli):
		return TagTree.objects.filter(owner__isnull=True)


class T_InlineUserTagTreeCompleter(T_TagTreeCompleter):
	
	def __init__(self, *args, **kwargs):
		super(T_InlineUserTagTreeCompleter, self).__init__(*args, **kwargs)
		self.token_repr = '<user tree>'
		
	def get_query_set(self, cli):
		from django.contrib.auth.models import User
		user = User.objects.get(username=cli.canonical_words[-1])
		return TagTree.objects.filter(owner=user)



from tags.models import TAG_PATH_DIVIDER

class T_TagCompleter(T_Anyword):
	
	klass        = None   
	
	def __init__(self, *args, **kwargs):

		super(T_TagCompleter, self).__init__(*args, **kwargs)

		self.token_repr = '<tag>'

	def get_tags(self, cli):
		current_tenant_name = cli.env['CURRENT_TENANT']
		current_tree        = cli.env['CURRENT_TREE']
		current_tag         = cli.env['CURRENT_TAG']
		db_set_current_tenant_schema(current_tenant_name)
		tags = Tag.objects.filter(tree=current_tree, parent=current_tag)
		return tags

	def matches(self, cli, word, ):
		
		cli.debug("%s MATCH %s %s %s" % (self.__class__.__name__, word, repr(word), type(word), ) ) 

		tags = self.get_tags(cli)
		
		return tags.filter(**{'name__startswith': word}).count() > 0

	def completion(self, cli, text):

		try:
			tags = self.get_tags(cli)			
			res = tags.filter( **{'name__startswith' :text} ).all()
			rv = []
			for n in res:
				rv.append(HelpingString( n.name, n.name))
			return rv
		except Exception, e:
			raise e
			
	def canonical(self, cli, word):

		try:
			tags = self.get_tags(cli)			
			res = tags.get( **{'name': word} )			
			return res.name
			
		except Tag.DoesNotExist, e:
			return None
		except Tag.MultipleObjectsReturned, e:
			try:
				return tags.get(**{'name': word}).name
			except:
				return "**ERROR**"
		except Exception, e:
			raise e

class T_InLineTagPathCompleter(T_Anyword):
	
	klass        = None   
	
	def __init__(self, *args, **kwargs):

		super(T_InLineTagPathCompleter, self).__init__(*args, **kwargs)

		self.token_repr = '<tag path>'

	def get_tags(self, cli):
		tenant = tenant_get(cli.env['CURRENT_TENANT'])
		db_set_current_tenant_schema(tenant.name)
		tree = TagTree.objects.get(tenant=tenant, name=cli.canonical_words[-1])
		tags = tree.tags
		return tree, tags

	def matches(self, cli, word, ):
		
		cli.debug("%s MATCH %s %s %s" % (self.__class__.__name__, word, repr(word), type(word), ) ) 

		tree, tags = self.get_tags(cli)
		
		prefix = TAG_PATH_DIVIDER.join([tree.path, ''])
			
		return tags.filter(**{'path__startswith': prefix + word}).count() > 0

	def completion(self, cli, text):

		try:
			tree, tags = self.get_tags(cli)
			
			prefix = TAG_PATH_DIVIDER.join([tree.path, ''])

			res = tags.filter( **{'path__startswith' : prefix + text} ).all()
			rv = []
			for n in res:
				rv.append(HelpingString( n.path[len(prefix):], n.description))
			return rv
		except Exception, e:
			print e
			raise e
			
	def canonical(self, cli, word):

		tree, tags = self.get_tags(cli)			

		prefix = TAG_PATH_DIVIDER.join([tree.path, ''])

		try:
			res = tags.get( **{'path__startswith': prefix + word} )
			return res.path[len(prefix):]
			
		except Tag.DoesNotExist, e:
			return None
		except Tag.MultipleObjectsReturned, e:
			try:
				return tags.get(**{'path': prefix + word}).path[len(prefix):]
			except:
				return "**ERROR**"
		except Exception, e:
			print e
			raise e




class T_TaggedDataGroupCompleter(T_DataGroupCompleter):

	def get_filters(self, cli, word):

		filters = super(T_TaggedDataGroupCompleter, self).get_filters(cli, word)

		tag = cli.env['CURRENT_TAG']
		tagged_ids = [et[0] for et in ElementTag.objects.filter(tag=tag).values_list('elementid')]
		filters['uuid__in'] = tagged_ids

		return filters	


class T_TaggedConditionCompleter(T_ConditionCompleter):

	def get_filters(self, cli, word):

		filters = super(T_TaggedConditionCompleter, self).get_filters(cli, word)

		tag = cli.env['CURRENT_TAG']
		tagged_ids = [et[0] for et in ElementTag.objects.filter(tag=tag).values_list('elementid')]
		filters['uuid__in'] = tagged_ids

		return filters	






class T_TagMetaDataCompleter(T_Anyword):
	
	def __init__(self, *args, **kwargs):

		super(T_TagMetaDataCompleter, self).__init__(*args, **kwargs)

		self.token_repr = '<tag meta>'

	def get_metas(self, cli, include_tag=False):
		current_tenant_name = cli.env['CURRENT_TENANT']
		current_tree        = cli.env['CURRENT_TREE']
		current_tag         = cli.env['CURRENT_TAG']
		db_set_current_tenant_schema(current_tenant_name)
		metas = TagMetaData.objects.distinct('name')

		if include_tag:
			metas = metas.filter(tag=current_tag)

		metas = metas.order_by('name')
		return metas

	def matches(self, cli, word, ):
		metas = self.get_metas(cli)
		return metas.filter(**{'name__startswith': word}).count() > 0

	def completion(self, cli, text):
		try:
			metas = self.get_metas(cli)
			res = metas.filter( **{'name__startswith' :text} ).all()
			rv = []
			for n in res:
				rv.append(HelpingString( n.name, n.name))
			return rv
		except Exception, e:
			raise e
			
	def canonical(self, cli, word):

		try:

			res = self.get_metas(cli, include_tag=True)
			res = res.get( **{'name': word} )			

			return res.name
			
		except TagMetaData.DoesNotExist, e:
			return None
		except TagMetaData.MultipleObjectsReturned, e:
			try:
				res = self.get_metas(cli, include_tag=True)
				
				return res.get(**{'name': word}).name
			except:
				return "**ERROR**"
		except Exception, e:
			raise e

