# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.
"""
debug_tag_state
========================================================================================

Questo comando serve per analizzare l'albero di calcolo degli stati di un TagTree e/o uno specifico 
Tag di quel Tree.

Utilizzo:

::
	
	sanet-manage debug_tag_state  <nome tree>  [ <path tag completo ]

Esempio:

::
	
	sanet-manage debug_tag_state  tree1
	sanet-manage debug_tag_state  tree1  "tree1:/Global/Sedi"
	sanet-manage debug_tag_state  tree1  "tree1:/Global/Sedi/Pontecchio Marconi"

"""

import os
import json
import time
import cPickle

from optparse import make_option

from core.api.base import *
from core.models import TenantSetting

from tags.models import TagTree

from tags.api.state2 import *

from sanet_common.termutils import *

from core.management.commands import TenantBasedCommand

from tags.server.tagsd.api import get_tree_structure

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(TenantBasedCommand):
    
	help = "** INTERNAL COMMAND. DO NOT USE **"

	sanet_internal = True

	#option_list = BaseCommand.option_list + (
	#	make_option('--tenant'          , action='store'     , dest='tenant'    , default=None , help=''),
	#	make_option('--debug'           , action='store_true', dest='debug'     , default=False, help=''),
	#	                                
	#	make_option('--nodetails'       , action='store_true', dest='nodetails' , default=False, help='Non carica le nomi/path degli elementi taggati (faster)'),
	#	
	#	make_option('--load-from-tagsd' , action='store_true', dest='load_from_tagsd' , default=False, help='XXXX'),
	#)

	def add_arguments(self, parser):
		
		super(Command, self).add_arguments(parser)
				
		parser.add_argument('--debug'           , action='store_true', dest='debug'     , default=False, help=''),
		                                
		parser.add_argument('--nodetails'       , action='store_true', dest='nodetails' , default=False, help='Non carica le nomi/path degli elementi taggati (faster)'),
		
		parser.add_argument('--load-from-tagsd' , action='store_true', dest='load_from_tagsd' , default=False, help='XXXX'),


		#
		# Positional arguments
		#
		parser.add_argument('tree_id', default='')
		parser.add_argument('tag_id', nargs='?', default='')


	#def handle(self, tree_id, tag_id=None, **options):
	def handle(self, **options):

		tree_id = options['tree_id']
		tag_id  = options['tag_id']

		#tenant_name = options['tenant']
		#if tenant_name == None:
		#	tenant  = tenant_get_primary()
		#else:
		#	tenant  = tenant_get(tenant_name)
		#
		#if tenant == None:
		#	raise Exception("No primary tenant found. You must specify a tenant name")		
		#
		#db_set_current_tenant_schema(tenant.name)

		tenant = self.get_tenant(options)		
		#
		# Carichiamo tagtree (e tag) da parametri
		#
		tree = TagTree.objects.get(Q(uuid=tree_id)|Q(name=tree_id))

		if tag_id:
			tag = Tag.objects.get(Q(path=tag_id)|Q(uuid=tag_id))
		else:
			tag = None


		#
		# Carichiamo i dati per il tagging
		#

		# Dati sulle condition
		conditions_data_dict = load_conditions_for_tagging()

		
		if not options['load_from_tagsd']:
		
			# Livelli di priorita'
			critical_priority  = tenant.get_setting(TenantSetting.SETTING_CRITICAL_LEVEL)
			minimum_priority   = tenant.get_setting(TenantSetting.SETTING_MINIMUM_LEVEL)
			print "Critical:", critical_priority
			print "Minimum :", minimum_priority 


			if tree.root is None:
				print ("Tree %s does not contain root tag" % (tree.name))
				return

			#
			# Caricamento dati + creazione struttura
			#
			tags_by_parent, tagged_by_tag, tagged_by_parent = load_tree_tagging_data(tree)

			statetree_dict = {}

			stateroot = create_tag_statetree_structure(statetree_dict, tags_by_parent, tagged_by_tag, tagged_by_parent, tree.root, debug=options['debug'])

			#
			# Calcolo degli stati dal tree
			#
			stateroot.calculate_state(conditions_data_dict, critical_priority=critical_priority, minimum_priority=minimum_priority)

			if tag:
				stateroot = statetree_dict[ tag.uuid ]


		else:
			#
			# La struttura ad albero (a partire dal tag indicato) la carichiamo dal server tagsd (cosi' vediamo cosa ha in pancia)
			#
			stateroot = get_tree_structure(tenant.uuid, tag.uuid if tag else tree.root.uuid)

		if not options['nodetails']:

			# Per caricare i nomi ci serve avere il mapping UUID -> nome/path
			monitored_element_by_uuid = dict([ (v[0], v[1]) for v in MonitoredElement.objects.values_list('uuid', 'name')      ])
			dg_by_uuid                = dict([ (v[0], v[1]) for v in DataGroup.objects.values_list('uuid', 'path')      ])
			tags_by_uuid              = dict([ (v[0], v[1]) for v in Tag.objects.filter(tree=tree).values_list('uuid', 'name') ])

			def _visit_with_details(l, e):
				s = str(e)
				o = monitored_element_by_uuid.get(e.r_id, None)
				if not o:
					o = tags_by_uuid.get(e.r_id, None) 
					if not o:
						o = dg_by_uuid.get(e.r_id, None)
						if not o:
							o = conditions_data_dict.get(e.r_id, None)
							if o:
								o = o[4]
							else:
								o = "*NOT LOADED (is it deleted condition?)*"

				s = "%s %20s %s %s" % (s, e.__class__.__name__, '    ' * l, o)
				if e.visible_status == CSTATE_UP:
					s = term_color(TERM_COLOR_LIGHT_GREEN, s)
				elif e.visible_status == CSTATE_DN:
					s = term_color(TERM_COLOR_LIGHT_RED, s)
				print s

			_visit = _visit_with_details

		else:

			def _visit_no_details(l, e):
				s = str(e)
				s = "%s %20s %s %s" % (s, e.__class__.__name__, '    ' * l, e.r_id)
				if e.visible_status == CSTATE_UP:
					s = term_color(TERM_COLOR_LIGHT_GREEN, s)
				elif e.visible_status == CSTATE_DN:
					s = term_color(TERM_COLOR_LIGHT_RED, s)
				print s

			_visit = _visit_no_details

		stateroot.visit(visit_fun=_visit)
