# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.
"""
fix_parental_relationships
========================================================================================

Questo comando serve per analizzare l'albero dei tagtree e fixare eventuali "parentele" sbagliate nei datagroup.

Utilizzo:

	::
		
		sanet-manage fix_parental_relationships  <nome tree> [--tenant <nome tenant>]

Esempio:

	::
		
		sanet-manage fix_parental_relationships  geo

.. warning:: This is an internal Sanet command. It is not recommended to use this command.

"""

import os
import json
import time
import cPickle

from optparse import make_option

#from django.core.management.base import BaseCommand
from django.db import connection

from core.api.base import *
#from core.models import TenantSetting

from core.management.commands import SanetInternalCommand, TenantBasedCommand

from tags.models import TagTree, ElementTag

from tags.api.state2 import *

from sanet_common.termutils import *


#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

def dictfetchall(cursor):
	"Returns all rows from a cursor as a dict"
	desc = cursor.description
	return [
		dict(zip([col[0] for col in desc], row))
		for row in cursor.fetchall()
	]


class Command(SanetInternalCommand, TenantBasedCommand):

	help = "** INTERNAL COMMAND. DO NOT USE **"

	sanet_internal = True

	#option_list = BaseCommand.option_list + (
	#	make_option('--tenant'    , action='store'     , dest='tenant'    , default=None , help=''),
	#	make_option('--debug'     , action='store_true', dest='debug'     , default=False, help=''),
	#	make_option('--nodetails' , action='store_true', dest='nodetails' , default=False, help='Non carica le nomi/path degli elementi taggati (faster)'),
	#)

	def add_arguments(self, parser):
		super(Command, self).add_arguments(parser)

		parser.add_argument('--debug'     , action='store_true', dest='debug'     , default=False, help=''),
		parser.add_argument('--nodetails' , action='store_true', dest='nodetails' , default=False, help='Non carica le nomi/path degli elementi taggati (faster)'),

		parser.add_argument('tree_id', nargs='?', default=None)
		parser.add_argument('tag_id' , nargs='?', default=None)


	def handle(self, **options):
		
		tree_id = options['tree_id']
		tag_id  = options['tag_id']
		
		t1 = time.time()

		#tenant_name = options['tenant']
		#if tenant_name == None:
		#	tenant  = tenant_get_primary()
		#else:
		#	tenant  = tenant_get(tenant_name)
		#
		#if tenant == None:
		#	raise Exception("No primary tenant found. You must specify a tenant name")
		#
		#db_set_current_tenant_schema(tenant.name)

		tenant = self.get_tenant(options)

		if tenant == None:
			raise Exception("No primary tenant found. You must specify a tenant name")

		cursor = connection.cursor()

		#
		# Carichiamo tagtree (e tag) da parametri
		#
		tree = TagTree.objects.get(Q(uuid=tree_id) | Q(name=tree_id))

		if tag_id:
			tag = Tag.objects.get(Q(path=tag_id) | Q(uuid=tag_id))
		else:
			tag = None

		deleted = 0
		counter = 0
		orphaned_counter = 0
		# prento tutti i datagroup taggati nel tree e nel tag indicati
		for tag in Tag.objects.filter(tree=tree):
			query = """
				SELECT te.uuid AS te_uuid, te.elementid AS te_dg_uuid, te.tagged_parent AS te_parent_uuid, dg.path AS dg_path, me.uuid AS me_uuid, me.kind AS me_kind
				FROM tags_elementtag te JOIN datagroup dg ON (te.elementid = dg.uuid) JOIN monitoredelement me ON (dg.element_id = me.uuid)
				WHERE elementtype='datagroup' AND tree_id='%(tagtree)s' AND tag_id='%(tag)s' ;
			""" % {
					'tagtree': tree.uuid,
					'tag': tag.uuid
				  }
			cursor.execute(query)
			et_datagroups = dictfetchall(cursor)
			tot = len(et_datagroups)

			for i, d in enumerate(et_datagroups):
				sys.stdout.write('Elaborating dg %s of %s in tag %s.\r' % (i+1, tot, tag.name))
				sys.stdout.flush()
				try:
					t_parent = ElementTag.objects.get(uuid=d['te_parent_uuid'])
				except ElementTag.DoesNotExist:
					#print d['te_parent_uuid']
					te = ElementTag.objects.get(uuid=d['te_uuid'])
					try:
						real_parent = ElementTag.objects.get(elementid=d['me_uuid'], tree=tree.uuid, tag=tag)
					except ElementTag.DoesNotExist:
						te.delete()
						deleted += 1
					else:
						te.tagged_parent = real_parent
						te.save()
						#print d['dg_path']
						counter += 1
					orphaned_counter += 1
				else:
					if d['me_uuid'] != t_parent.elementid:
						# decommentare per fixare
						te = ElementTag.objects.get(uuid=d['te_uuid'])
						try:
							real_parent = ElementTag.objects.get(elementid=d['me_uuid'], tree=tree.uuid, tag=tag)
						except ElementTag.DoesNotExist:
							te.delete()
							deleted += 1
						else:
							te.tagged_parent = real_parent
							te.save()
							#print d['dg_path']
							counter += 1
			print ''

		print "%s records tagged wrong %s seconds. Orphans %s. Deleted %s" % (counter, time.time()-t1, orphaned_counter, deleted)

