"""
get_missing_dgtag_number
========================================================================================

Questo comando serve per analizzare l'albero dei tagtree e fixare eventuali "parentele" sbagliate nei datagroup.

Utilizzo:

	::
		
		sanet-manage get_missing_dgtag_number [--tenant <nome tenant>]

Esempio:

	::
		
		sanet-manage get_missing_dgtag_number

.. warning:: This is an internal Sanet command. It is not recommended to use this command.

"""

import time

from django.db import connection

from core.api.base import db_set_current_tenant_schema, tenant_get, tenant_get_primary
from core.management.commands import SanetInternalCommand, TenantBasedCommand

from tags.models import TagTree


query_dg = """
			SELECT COUNT(distinct dg.uuid)
			FROM datagroup dg JOIN monitoredelement me ON (dg.element_id = me.uuid)
			JOIN tags_elementtag te ON (te.elementid = me.uuid)
			WHERE te.tree_id = '%s' 
			AND not exists (SELECT 1
							FROM datagroup dg JOIN tags_elementtag tedg ON (dg.uuid = tedg.elementid)
							WHERE tedg.tagged_parent = te.uuid)
"""

query = """
			SELECT distinct teem.tag_id AS te_tagid, dg.path AS dg_path, me.uuid AS me_uuid, me.kind AS me_kind, me.name AS me_name
			FROM monitoredelement me JOIN tags_elementtag teem ON (me.uuid = teem.elementid) JOIN datagroup dg ON (dg.element_id = me.uuid)
			WHERE not exists (
				SELECT 1
				FROM tags_elementtag te
				WHERE te.elementid = dg.uuid
			)
		"""

class Command(SanetInternalCommand, TenantBasedCommand):

	help = "** INTERNAL COMMAND. DO NOT USE **"

	sanet_internal = True

	#option_list = BaseCommand.option_list + (
	#	make_option('--tenant'    , action='store'     , dest='tenant'    , default=None , help=''),
	#	make_option('--debug'     , action='store_true', dest='debug'     , default=False, help=''),
	#	make_option('--nodetails' , action='store_true', dest='nodetails' , default=False, help='Non carica le nomi/path degli elementi taggati (faster)'),
	#)

	def add_arguments(self, parser):
		super(Command, self).add_argument(parser)

		parser.add_argument('--debug'     , action='store_true', dest='debug'     , default=False, help=''),
		parser.add_argument('--nodetails' , action='store_true', dest='nodetails' , default=False, help='Non carica le nomi/path degli elementi taggati (faster)'),

	def handle(self, **options):#, tag_id=None, **options):
		t1 = time.time()

		#tenant_name = options['tenant']
		#if tenant_name == None:
		#	tenant  = tenant_get_primary()
		#else:
		#	tenant  = tenant_get(tenant_name)
		#
		#if tenant == None:
		#	raise Exception("No primary tenant found. You must specify a tenant name")
		#
		#db_set_current_tenant_schema(tenant.name)
		
		tenant = self.get_tenant(options)
		
		cursor = connection.cursor()

		trees = TagTree.objects.all()

		for tree in trees:
			cursor.execute(query_dg % tree.uuid)
			n = cursor.fetchone()[0]
			print "%s datagroups non taggati nel tree %s, tempo di fix stimato circa %s secondi" % (n, tree.name, n/166)
