# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

get_untagged_nodes
=====================================================================================

Uso:

	::
		
		sanet-manage get_untagged_nodes [ --tenant <name> ] [ --trees <nomi tree> ]


Restituisce l'elenco dei nodi non presenti in nessuno dei tagtree esistenti (o elencati dal parametro *trees*) per il tenant *tenant*.

Esempio: Elenco di tutti i nodi non taggati dentro i tree geo e rancid

	::
		
		sanet-manage get_untagged_nodes --tenant site  --trees geo,rancid



"""

from optparse import make_option

#from django.core.management.base import BaseCommand

from django.db import connection

from core.management.commands import TenantBasedCommand

from tags.models import TagTree

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#


class Command(TenantBasedCommand):
    
	help = "Get untagged nodes."

	#option_list = TenantBasedCommand.option_list + (
	#	make_option('--trees' , action='store', dest='trees' , default=None, help=''),
	#)

	def add_arguments(self, parser):

		super(Command, self).add_arguments(parser)

		parser.add_argument('--trees' , action='store', dest='trees' , default=None, help='')


	def handle(self, *args, **options):

		#tenant_name = options['tenant']
		#if tenant_name == None:
		#	tenant  = tenant_get_primary()
		#else:
		#	tenant  = tenant_get(tenant_name)
		#if tenant == None:
		#	raise Exception("No primary tenant found. You must specify a tenant name")
		tenant = self.get_tenant(options)

		tenant.db_set_current_schema()
		
		cursor = connection.cursor()
		
		query = """
SELECT n.name
  FROM monitoredelement n 
 WHERE 
       n.kind = 1
   AND
       n.uuid not in (
                      SELECT et.elementid
                        FROM tags_elementtag et inner join tagtree tree ON (et.tree_id = tree.uuid)
                       WHERE 
                             et.elementtype = 'node'
                         AND
                             tree.name in (
                                        %s
                                       )
                     )
ORDER BY n.name
; 
"""		
		if options['trees']:
			params = options['trees'].split(',')
		else:
			params = [ t.name for t in TagTree.objects.all() ]
		
		params = ','.join([ "'%s'" % p for p in params]) 
		
		query = query % (params)
		
		cursor.execute(query )
		
		data = sorted([ r[0] for r in cursor.fetchall()])
		
		for name in data:
			print name
