# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os
import json
import time

from optparse import make_option

from django.core.management.base import BaseCommand

from core.models import *
from core.api.base import *
from core.api.base import tenant_get
from core.api.base import tenant_get_primary

from tags.models import *


from tags.models import _tag_raw_entity, _untag_raw_entity
from tags.api import tag_element, untag_element

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#


def visit_tag(tag, level=0):
	print ' ' * level, tag
	r = [ (level, tag) ]
	for ct in tag.child_tags.all():
		r += visit_tag(ct, level=level+1)
	return r

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

SELECTED = 'on'
UNSELECTED = ''

class Command(BaseCommand):
    
	help = "** INTERNAL COMMAND. DO NOT USE **"
	
	sanet_internal = True

	def add_arguments(self, parser):
		super(Command, self).add_arguments(parser)
		
		parser.add_argument('--test'  , action='store_true', dest='test'  , default=False, help=''),
		parser.add_argument('--tenant', action='store'     , dest='tenant', default=None, help=''),

		#
		# Positional arguments
		#
		parser.add_argument('element_type', default='')
		parser.add_argument('element_path', default='')

	#def handle(self, element_type, element_path, *args, **options):
	def handle(self, *args, **options):
		
		element_type = options['element_type']
		element_path = options['element_path']

		tenant_name = options['tenant']
		if tenant_name == None:
			tenant  = tenant_get_primary()
		else:
			tenant  = tenant_get(tenant_name)
		if tenant == None:
			raise Exception("No primary tenant found. You must specify a tenant name")


		db_set_current_tenant_schema(tenant.name)
		
		
		if options['test']:
			return

		# Costruiamo l'albero di tutti i tree per il menu' da visualizzare
		ret = []
		for tree in TagTree.objects.all().order_by('name'):
			try:
				root = tree.tags.get(parent=None)
				ret += visit_tag(root)
			except Tag.DoesNotExist, e:
				pass

		all_tags_by_uuid = dict([ (tag.uuid, tag) for level, tag in ret ])


		if element_type in ['node', 'interface', 'storace', 'service', 'device']:

			# Troviamo l'elemento da taggare
			data = element_path.split(':')
			if len(data) == 1:
				element = MonitoredElement.objects.get(name=data[0])
				element = element.get_specific_element()
			else:
				element = MonitoredElement.objects.get(name=data[1], parent__name=data[0])
				element = element.get_specific_element()
		elif element_type in ['datagroup']:
			element   = DataGroup.objects.get(path=element_path)
		elif element_type in ['condition']:
			element   = Condition.active_objects.get(path=element_path)
		else:
			raise Exception("type not valid: %s" % (element_path))

		# Troviamo tutti i tag in cui e' presente il nodo e l'elementag corrispondente
		elementid   = element.uuid
		active_tags = dict([(et.tag.uuid, et) for et in ElementTag.objects.filter(elementid=elementid).select_related('tag').all()])

		#
		# Creiamo il menu di selezione con tutte i tag in cui l'elemento e' taggato
		#
		menu_entries = []
		for level, tag in ret:

			descr    = tag.path.encode('ascii', errors='replace')
			selected = SELECTED if active_tags.has_key(tag.uuid) else UNSELECTED

			menu_entries += [ tag.uuid, descr, selected ]

		selected_menu_entries_ids = launch_select_menu('Select tags', menu_entries)
		if selected_menu_entries_ids is None: # Cancel from user
			return

		#
		# Aggiorniamo il tagging sul nodo in base ai tag selezionati
		#
		update_tagging( element, active_tags, selected_menu_entries_ids )

		return


def update_tagging( element, active_tags, selected_tag_uuids ):
	
		node = element
	
		#
		# Remove unselected tags
		#
		unselected_tags = set(active_tags.keys()) - set(selected_tag_uuids)

		if unselected_tags:
			print "Removing unselected tags"
			for tag_uuid in unselected_tags:
				
				tag = active_tags[tag_uuid].tag
				print "\t", tag
		
				untag_element( tag, element)

		#
		# Select node elements to tag
		#
		for uuid in selected_tag_uuids:
			
			tag = Tag.objects.get(uuid=uuid)
			
			elements_tagged = tag.tagged.all()

			# Calcoliamo le entries del menu con indicazione di cosa e' gia' taggato
			entities_by_menu_uuid, menu_entries = produce_monitoredelement_menu_entries(tag, elements_tagged, element)
			
			# Mostriamo il menu
			menu_enties_list = []
			for me in menu_entries:
				menu_enties_list += me
			selected_menu_entries_uuids = launch_select_menu('Tag entities with: [%s]' % (tag.path), menu_enties_list)
			
			if selected_menu_entries_uuids is None:
				continue
			
			for m_id in selected_menu_entries_uuids:
				print entities_by_menu_uuid[m_id][0]
			
			# Calculate selected/unselecte entries
			menu_entries_ids           = set([ m_key for m_key, descr, selected in  menu_entries ])
			
			selected_menu_entries_ids  = set(selected_menu_entries_uuids)
			unselected_menu_enries_ids = menu_entries_ids - selected_menu_entries_ids
			
			
			if selected_menu_entries_ids:
				
				print "Tagging:"
				for m_id in selected_menu_entries_ids:
					
					entity, parent_menu_entry_id = entities_by_menu_uuid[m_id]
					
					if parent_menu_entry_id in selected_menu_entries_ids:
						directly_tagged = False
					else:
						directly_tagged = True

					print "\t", "directly=%s" % (directly_tagged), entity, parent_menu_entry_id
					
					_tag_raw_entity(tag, entity, directly_tagged=directly_tagged, recursive=False)
					
			
			if unselected_menu_enries_ids:
				
				print "Untagging"
				for m_id in unselected_menu_enries_ids:

					entity, parent_menu_entry_id = entities_by_menu_uuid[m_id]
					
					print "\t", entity, parent_menu_entry_id

					_untag_raw_entity(tag, entity, False)
					
			
			#tag_element(tag, node)
		

def launch_select_menu(title, elements):
	
		import subprocess

		try:

			args = [ "dialog", "--stderr", "--separate-output", "--checklist", title, "0", "0", "0", ] + elements
			p = subprocess.Popen(args, stderr=subprocess.PIPE, close_fds=True)
			stdout, stderr = p.communicate()
		
			if p.returncode != 0: # cancel
				return None
		
			data = stderr.strip()
			if not data:
				return []
		
			selected_uuids = data.split('\n')
			
			return selected_uuids

		except OSError, e:
			raise Exception("Comando 'dialog' non trova")
		
		



def MENU_KEY(e):
	return "%s_%s" % (MODEL_TO_TAG_ELEMENTTYPE[e.__class__], e.uuid)

def produce_monitoredelement_menu_entries(tag, elements_tagged, me, parent_menu_entry_id=None, level=0):

	elements_tagged_by_id = dict([(et.elementid, et) for et in elements_tagged])

	l = level

	objects = {}
	entries = []

	SPACE = '    '

	key = MENU_KEY(me)
	objects[key] = (me, parent_menu_entry_id) 
	selected = 'on' if elements_tagged_by_id.has_key(me.uuid) else ''
	
	entries.append( [ key, (SPACE * l) + "[%s]" % me.name, selected ]  )

	parent_menu_entry_id = key

	if isinstance(me, MonitoredElement):
		l += 1
		for dg in me.datagroups.all():

			key = MENU_KEY(dg)
			objects[key] = (dg, parent_menu_entry_id)
			selected = 'on' if elements_tagged_by_id.has_key(dg.uuid) else ''
			
			entries.append( [ key, "%s %s (%s)" % ((SPACE * l), dg.name, str(dg.deleted)), selected ]  )

			parent_menu_entry_id2 = key

			l += 1
			for c in dg.conditions.order_by('path'):
				key = MENU_KEY(c)
				objects[key] = (c, parent_menu_entry_id2)
				selected = 'on' if elements_tagged_by_id.has_key(c.uuid) else ''
				
				entries.append( [ key, "%s %s (%s)" % ((SPACE * l) , c.name, c.deleted) , selected ]  )
			l -= 1
			
		for sube in me.get_sub_elements().order_by('kind', 'name'):
			sube = sube.get_specific_element()

			key = MENU_KEY(sube)
			objects[key] = (sube, parent_menu_entry_id)
			objects2, entries2 = produce_monitoredelement_menu_entries(tag, elements_tagged, sube,parent_menu_entry_id, level=l)
			
			objects.update(objects2)
			entries += entries2

		l -= 1
		
	elif isinstance(me, DataGroup):
			dg = me
			
			key = MENU_KEY(dg)
			#objects[key] = (dg, parent_menu_entry_id)
			#selected = 'on' if elements_tagged_by_id.has_key(dg.uuid) else ''
			
			#entries.append( [ key, (SPACE * l) + dg.name, selected ]  )

			parent_menu_entry_id2 = key

			l += 1
			for c in dg.conditions.order_by('path'):
				key = MENU_KEY(c)
				objects[key] = (c, parent_menu_entry_id2)
				selected = 'on' if elements_tagged_by_id.has_key(c.uuid) else ''
				
				entries.append( [ key, "%s %s %s" % ((SPACE * l), c.name, c.delete), selected ]  )
			l -= 1
			
	return objects, entries




