"""
tag_find_missing_dgs
========================================================================================

Questo comando serve per taggare i datagroup non taggati insieme ai propri elementi.

Utilizzo:

	::
		
		sanet-manage tag_find_missing_dgs <tagtree id or name> [--tenant <nome tenant>]

Esempio:

	::
		
		sanet-manage tag_find_missing_dgs <tagtree id or name>

.. warning:: This is an internal Sanet command. It is not recommended to use this command.

"""

import os
import json
import time
import cPickle
import sys

from optparse import make_option

#from django.core.management.base import BaseCommand
from django.db import connection

from core.api.base import *
#from core.models import DataGroup, MonitoredElement

from core.management.commands import SanetInternalCommand, TenantBasedCommand

from tags.models import TagTree, ElementTag
#from tags.models import TAG_ELEMENTTYPE_DATAGROUP
#from tags.models import TAG_ELEMENTTYPE_CONDITION

from tags.api.state2 import *

from sanet_common.termutils import *


#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

def dictfetchall(cursor):
	"Returns all rows from a cursor as a dict"
	desc = cursor.description
	return [
		dict(zip([col[0] for col in desc], row))
		for row in cursor.fetchall()
	]


query_elem = """
			SELECT me.*
			FROM monitoredelement me JOIN tags_elementtag et ON (me.uuid = et.elementid)
			WHERE te.tree_id = '%s';
		"""

query_et = """
			SELECT et.*
			FROM monitoredelement me JOIN tags_elementtag et ON (me.uuid = et.elementid)
			WHERE te.tree_id = '%s';
		"""

# fetch missing dgs
query_dg = """
			SELECT dg.uuid as dg_uuid, 
					dg.element_id as dg_elem, 
					me.uuid as me_uuid, 
					te.uuid as te_uuid, 
					te.elementid as te_elem, 
					te.tag_id as te_tag, 
					te.tree_id as te_tree,
					te.is_explosive as te_explosive,
					te.explosive_group as te_exp_group
			FROM datagroup dg JOIN monitoredelement me ON (dg.element_id = me.uuid)
			JOIN tags_elementtag te ON (te.elementid = me.uuid)
			WHERE te.tree_id = '%s' 
			AND not exists (SELECT 1
							FROM datagroup dg JOIN tags_elementtag tedg ON (dg.uuid = tedg.elementid)
							WHERE tedg.tagged_parent = te.uuid)
"""


class Command(SanetInternalCommand, TenantBasedCommand):

	help = "** INTERNAL COMMAND. DO NOT USE **"

	sanet_internal = True

	def add_arguments(self, parser):
		super(Command, self).add_arguments(parser)
		
		parser.add_argument('--debug'     , action='store_true', dest='debug'     , default=False, help=''),
		parser.add_argument('--nodetails' , action='store_true', dest='nodetails' , default=False, help='Non carica le nomi/path degli elementi taggati (faster)'),

		parser.add_argument('tree_id', nargs='?', default=None)

	def handle(self, **options):

		tree_id = options['tree_id']

		t1 = time.time()

		#tenant_name = options['tenant']
		#if tenant_name == None:
		#	tenant  = tenant_get_primary()
		#else:
		#	tenant  = tenant_get(tenant_name)
		#
		#if tenant == None:
		#	raise Exception("No primary tenant found. You must specify a tenant name")
		#
		#db_set_current_tenant_schema(tenant.name)
		
		tenant = self.get_tenant(options)
		
		cursor = connection.cursor()

		#
		# Carichiamo tagtree (e tag) da parametri
		#
		tree = TagTree.objects.get(Q(uuid=tree_id) | Q(name=tree_id))

		# if tree.uniques is not True:
		# 	print "Funziona solo con i tagtree unique"
		# 	return

		tags = Tag.objects.filter(tree=tree)

		unique_params = {'unique_sentinel': ElementTag.TAGGED_UNIQUE}

		cursor.execute(query_dg % tree.uuid)
		dgs = dictfetchall(cursor)
		tot = len(dgs)
		integrity_counter = 0

		bulk_chunck = []

		for i, dg in enumerate(dgs):
			sys.stdout.write('Elaborating dg %s of %s.\r' % (i+1, tot))
			sys.stdout.flush()
			dg_et = ElementTag(
				tenant=tenant,
				elementid=dg['dg_uuid'],
				elementtype='datagroup',
				tagged_parent=dg['te_uuid'],
				tree_id=dg['te_tree'],
				tag_id=dg['te_tag'],
				directly_tagged=False,
				is_explosive=dg['te_explosive'],
				explosive_group=dg['te_exp_group'],
				**unique_params
			)
			bulk_chunck.append(dg_et)

			conditions = Condition.objects.filter(datagroup_id=dg['dg_uuid']).values_list('uuid', flat=True)
			c = ElementTag.objects.filter(elementid__in=conditions, tree_id=dg['te_tree'], tag_id=dg['te_tag']).update(tagged_parent=dg_et.uuid)

			if c == 0:
				for u in conditions:
					cet = ElementTag(
						tenant=tenant,
						elementid=u,
						elementtype='condition',
						tagged_parent=dg_et.uuid,
						tree_id=dg['te_tree'],
						tag_id=dg['te_tag'],
						directly_tagged=False,
						is_explosive=dg['te_explosive'],
						explosive_group=dg['te_exp_group'],
						**unique_params
					)
					bulk_chunck.append(cet)


		ElementTag.objects.bulk_create(bulk_chunck)

		print "Fixed %s dgs in %s seconds with %s integrity errors solved" % (len(dgs), time.time()-t1, integrity_counter)
