# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os
import json
import time

from optparse import make_option

from django.core.management.base import BaseCommand

from core.api.base import db_set_current_tenant_schema, tenant_get_primary, tenant_get

from tags.models import TagTree
from tags.api.state2 import _load_data_for_tagged_parent_fix, _update_tree_tagged_parent

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(BaseCommand):
    
	help = "** INTERNAL COMMAND. DO NOT USE **"

	sanet_internal = True

	def add_arguments(self, parser):
		parser.add_argument('--tenant', action='store'     , dest='tenant', default=None , help=''),
		parser.add_argument('--force' , action='store_true', dest='force' , default=False, help=''),

		parser.add_argument('tree_name', nargs='?', default=None)

	def handle(self, **options):
		
		tree_name = options['tree_name']

		tenant_name = options['tenant']
		if tenant_name == None:
			tenant  = tenant_get_primary()
		else:
			tenant  = tenant_get(tenant_name)
		if tenant == None:
			raise Exception("No primary tenant found. You must specify a tenant name")				

		db_set_current_tenant_schema(tenant.name)

		# Load tree to update
		trees = TagTree.objects
		if tree_name:
			trees = trees.filter(name=tree_name)
		trees = trees.order_by('name').all()

		# Load data for update
		print "Loading..."
		elements_by_parent, datagroups_by_parent, conditions_by_parent = _load_data_for_tagged_parent_fix()

		for tree in trees:

			if tree.nostate == False:
				if not options['force']:
					print "Tree", tree.name, "skipped because is not a service tag (nostate not true)."
					continue

			t1 = time.time()
			updated_count = _update_tree_tagged_parent( elements_by_parent, datagroups_by_parent, conditions_by_parent, tree)

			t2 = time.time()
			print "\t", "Updated:", updated_count, "Elapsed:", t2 - t1

