# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""
tags_cleanup
==============================================================================

Remove broken references to deleted elements (nodes, interface, conditions, ecc..)

Usage:

	::
		
		sanet-manage tags_cleanup

.. warning:: This is an internal Sanet command. It is not recommended to use this command.

"""
import os
import json
import time

from optparse import make_option

#from django.core.management.base import BaseCommand

from django.db import connection

from core.api.base import tenant_select_all

from core.management.commands import SanetInternalCommand, SanetBaseCommand

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(SanetInternalCommand, SanetBaseCommand):
    
	help = "Remove broken references to deleted elements (nodes, interface, conditions, ecc..)"

	sanet_internal = True


	def handle(self, *args, **options):

		schemas = ['public',] + [ t.name for t in tenant_select_all() ]

		for name in schemas:

			db_cursor = connection.cursor()
		
			print "Updating schema %s" % (name)
			db_cursor.execute("SET search_path TO %s;" % (name))


			query = """
delete from logicmapentity where elementtag_id in (
   select et.uuid from tags_elementtag et left join monitoredelement e on (et.elementid = e.uuid) where et.elementtype in ('node', 'interface', 'storage', 'service', 'device') and e.uuid is null
   union
   select et.uuid from tags_elementtag et left join datagroup        e on (et.elementid = e.uuid) where et.elementtype in ('datagroup'                                        ) and e.uuid is null
   union
   select et.uuid from tags_elementtag et left join condition        e on (et.elementid = e.uuid) where et.elementtype in ('condition'                                        ) and e.uuid is null
   union
   select et.uuid from tags_elementtag et left join datasource       e on (et.elementid = e.uuid) where et.elementtype in ('datasource'                                       ) and e.uuid is null
)
;
"""

			db_cursor.execute(query)
			print "Deleted %s broken references" % (db_cursor.rowcount)

			query = """
delete from tags_elementtag where uuid in (
   select et.uuid from tags_elementtag et left join monitoredelement e on (et.elementid = e.uuid) where et.elementtype in ('node', 'interface', 'storage', 'service', 'device') and e.uuid is null
   union
   select et.uuid from tags_elementtag et left join datagroup        e on (et.elementid = e.uuid) where et.elementtype in ('datagroup'                                        ) and e.uuid is null
   union
   select et.uuid from tags_elementtag et left join condition        e on (et.elementid = e.uuid) where et.elementtype in ('condition'                                        ) and e.uuid is null
   union
   select et.uuid from tags_elementtag et left join datasource       e on (et.elementid = e.uuid) where et.elementtype in ('datasource'                                       ) and e.uuid is null
)
;
"""
			
			db_cursor.execute(query);
			print "Deleted %s broken references" % (db_cursor.rowcount)

			db_cursor.close()
			connection.close()


