# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

if __name__ == "__main__":

	import os
	import sys
	import logging
	
	import argparse

	#
	# Be sure django will find modules with the right path
	#
	sanet_install_dir = sys.argv[1]
	if not sanet_install_dir in sys.path:
		sys.path.insert(0, sanet_install_dir)
	os.environ.setdefault("DJANGO_SETTINGS_MODULE", "conf.settings")
	
	# Added for django 1.11
	import django
	django.setup()

	from django.db import connection

	# These imports depend on django
	from django.conf import settings
	from conf import get_configuration

	from tags.server.tagsd.tagsdserver import init_logger
	from tags.server.tagsd.tagsdserver import TagServer


	conf = get_configuration()

	#
	# Parse args
	#
	parser = argparse.ArgumentParser(prog='tagsd', usage='%(prog)s <SANET INSTALL DIR> [options]')
	parser.add_argument('-l'       , dest='loglevel', nargs=1, type=int , default=[conf.TAGSD_LOG_LEVEL], help='debug level')
	parser.add_argument('--trees'  , dest='trees'   , nargs=1,            default=[None]                , help='debug level')
	options = parser.parse_args(sys.argv[2:])

	#
	# INIT
	#

	# Set log 
	log_level = options.loglevel[0] or  conf.TAGSD_LOG_LEVEL
	log_file  = os.sep.join( [conf.LOG_DIR, 'tagsd_log'])

	logger = init_logger(log_level, log_file)

	#
	# START
	#

	if sanet_install_dir != conf.SANET_INSTALL_DIR:
		msg = "Install dir %s is different from configuration install dir %s" % (sanet_install_dir, conf.SANET_INSTALL_DIR)
		print msg 
		logger.critical(msg)

	# Close Djagno DB connection. Main program does not need it.
	connection.close()
	
	debug_tagtree_names = options.trees[0]

	s = TagServer(logger, debug_tagtree_names=debug_tagtree_names)
	s.start()

	# never returns
