# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import json

from django.contrib.auth.models import Group
from django.http import HttpResponse, HttpResponseRedirect

#from django.db import transaction

from sanet_common.sanetdefs.constants import DIRENTRY_AUTH_READWRITE, DIRENTRY_AUTH_READONLY

from core.db_utils import transaction_commit_on_success

from core.api.base import db_set_current_tenant_schema

from tags.models import Tag, TagTree, TagTreeGroupPermission, ElementTag
from tags.models import TAG_ELEMENTTYPE_TO_MODEL
from tags.models import TAG_ELEMENTTYPE_CONDITION
from tags.models import TAG_ELEMENTTYPE_NODE, TAG_ELEMENTTYPE_IFACE, TAG_ELEMENTTYPE_STORAGE, TAG_ELEMENTTYPE_SERVICE, TAG_ELEMENTTYPE_DEVICE

from tags.api import tag_create_root, tag_create, tag_remove
from tags.api import tag_element, untag_element, tag_condition, untag_condition

from tags.server.tagsd.api import signal_state_refresh


from webui.decorators import tenant_dependent

from webui.api.response import HttpJsonErrorResponse, HttpJsonSuccessResponse
from webui.api.response import ajax_response

from webui.decorators import ajax_capability_required

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

@tenant_dependent
@ajax_capability_required("tags.tag.create")
def add_tag(request):

	try:
		tenant = request.tenant
		user   = request.user
		
		db_set_current_tenant_schema(tenant.name)

		tag_data = request.GET_OR_POST().get('tag')
		tag_data = json.loads(tag_data)

		name = tag_data['name']

		parent_id = tag_data['parent']
		
		visibility = int(tag_data['visibility'])

		if parent_id == "/":
			tag = tag_create_root(request.tenant, name, owner=user)
			
			if visibility != -1:
				
				tree = tag.tree
				
				if visibility == 1:
					perm = DIRENTRY_AUTH_READONLY
				elif visibility == 2:
					perm = DIRENTRY_AUTH_READWRITE
				else:
					raise Exception("Invalid visibility value: %s" % (visibility))
			
				for g in user.groups.all():

					ttgp, created = TagTreeGroupPermission.objects.get_or_create(tenant=tenant, dirtree=tree, group=g)
					ttgp.auth_type = perm
					ttgp.save()
			
		else:
			parent_tag = Tag.objects.get(tenant=request.tenant, uuid=parent_id)
			tag = tag_create(parent_tag.tree, name, parent_tag)
			

		return HttpJsonSuccessResponse("Added")
		
	except Exception, e:
		return HttpJsonErrorResponse(str(e))
    
#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

@tenant_dependent
@ajax_capability_required("tags.tag.delete")
def remove_tag(request):
	
	try:
		tenant = request.tenant
		db_set_current_tenant_schema(tenant.name)

		tagid = request.GET_OR_POST().get('id')

		tag = Tag.objects.get(uuid=tagid)
	    
		tag_remove(tag, autodelete_tree=True)
		
		return HttpJsonSuccessResponse('deleted')
		
	except Exception, e:

		return HttpJsonErrorResponse('Error: %s' % (e))
		
		 
	#return HttpResponseRedirect(reverse('webui.views.index' , kwargs={'rt': rt,'uuid': uuid}))

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

@tenant_dependent
@ajax_capability_required("tags.tag.tag_element")
def do_tag_element(request):
	
	try:
		tenant = request.tenant

		eid   = request.GET_OR_POST()['e']
		etype = request.GET_OR_POST()['k']
		
		tagid = request.GET_OR_POST()['t']
		
		tag = Tag.objects.get(uuid=tagid)
		
		model = TAG_ELEMENTTYPE_TO_MODEL[etype]
		entity = model.objects.get(uuid=eid)
		
		#
		# FIX: non usare questa funzione
		#
		if etype == 'condition':
			tag_condition(tag, entity)
		elif etype in ['node', 'interface', 'storage', 'device', 'service']:
			tag_element(tag, entity)
		else:
			raise Exception("Invalid type")

		#return HttpResponse("OK")
		return HttpJsonSuccessResponse("OK")
		
	except Exception, e:
		return HttpJsonErrorResponse(str(e))

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#
@tenant_dependent
def do_untag_elementid (request):

	tenant = request.tenant
    
	element_uuid = request.GET_OR_POST()['uuid']
	tag_uuid       = request.GET_OR_POST()['tag_uuid']
    
	db_set_current_tenant_schema(tenant.name)
    
	try:
		et = ElementTag.objects.get(tag__uuid=tag_uuid, elementid=element_uuid)
        
		element = et.element

		#untag_element( et.tag, element)
		if et.elementtype == TAG_ELEMENTTYPE_CONDITION:
			untag_condition(et.tag, element)
			return HttpJsonSuccessResponse('deleted')

		elif et.elementtype in [ TAG_ELEMENTTYPE_NODE, TAG_ELEMENTTYPE_IFACE, TAG_ELEMENTTYPE_STORAGE, TAG_ELEMENTTYPE_SERVICE, TAG_ELEMENTTYPE_DEVICE ]:
			untag_element(et.tag, element)
			return HttpJsonSuccessResponse('deleted')
		else:
			return HttpJsonErrorResponse('Element type not valid: %s' % (et.elementtype))
		

		return HttpJsonSuccessResponse('deleted')
	
	except ElementTag.DoesNotExist, e:

		return HttpJsonErrorResponse('Error: %s' % (e))
        
@tenant_dependent
def do_untag_element (request):

	tenant = request.tenant

	et_uuid = request.GET_OR_POST()['uuid']

	db_set_current_tenant_schema(tenant.name)

	try:
		#
		# FIX: TROVARE UN MODO MIGLIORE PER STAGGARE IN BASE All'ElementTag.uuid
		#
		et = ElementTag.objects.get(uuid=et_uuid)

		element = et.element
		
		if et.elementtype == TAG_ELEMENTTYPE_CONDITION:
			untag_condition(et.tag, element)
			return HttpJsonSuccessResponse('deleted')

		elif et.elementtype in [ TAG_ELEMENTTYPE_NODE, TAG_ELEMENTTYPE_IFACE, TAG_ELEMENTTYPE_STORAGE, TAG_ELEMENTTYPE_SERVICE, TAG_ELEMENTTYPE_DEVICE ]:
			untag_element(et.tag, element)
			return HttpJsonSuccessResponse('deleted')
		else:
			return HttpJsonErrorResponse('Element type not valid: %s' % (et.elementtype))


	except ElementTag.DoesNotExist, e:

		return HttpJsonErrorResponse('Error: %s' % (e))

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

@tenant_dependent
def rename_tag(request):

	tag = request.GET_OR_POST().get('t')
	name = request.GET_OR_POST().get('n')

	r = do_rename_tag(tag, name, request.tenant)

	if r:
		return HttpResponse('OK')

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#
    
@tenant_dependent
def move_tag(request):
	
	tag = request.GET_OR_POST().get('t')
	new_parent = request.GET_OR_POST().get('p')

	r = do_rename_tag(tag, new_parent, request.tenant)

	if r:
		return HttpResponse('OK')
    
#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

#@tenant_dependent
#@transaction_commit_on_success
#def create_map_from_tag(request):
#
#	tenant = request.tenant
#	db_set_current_tenant_schema(tenant.name)
#
#	tag = Tag.objects.get(uuid=tagid)
#
#
#	tagid = request.GET_OR_POST().get('tagid')
#	
#	return HttpResponse("OK")

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

@tenant_dependent
@transaction_commit_on_success
def set_group_perm(request):

	tenant = request.tenant

	group_id = int(request.GET_OR_POST()['group_id'])
	tree_id  =     request.GET_OR_POST()['tree_id']
	perm     = int(request.GET_OR_POST()['perm'])

	g    = Group.objects.get(id=group_id)
	tree = TagTree.objects.get(uuid=tree_id)

	if perm == -1:

		try:
			p = TagTreeGroupPermission.objects.get(tenant=tenant, group=g, dirtree=tree)
			p.delete()
		except TagTreeGroupPermission, e:
			pass
	else:
		p, created = TagTreeGroupPermission.objects.get_or_create(tenant=tenant, group=g, dirtree=tree)
		p.auth_type = perm
		p.save()

	return HttpResponse("ok")


#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

@ajax_response
@tenant_dependent
def reload_state(request):
	"""
	Questa view serve per noti
	"""
	tenant = request.tenant

	elementtype = request.GET_OR_POST()['elementtype']
	elementid   = request.GET_OR_POST()['elementid']

	#print __file__, "RELOAD SIGNAL", request.user, elementtype, elementid

	db_set_current_tenant_schema(tenant.name)

	try:
		tree_ids = []

		if elementtype == 'tag':
			tag = Tag.objects.get(uuid=elementid)
			uuid_to_signal = [tag.tree_id]
		else:
			uuid_to_signal = list(set([et[0] for et in ElementTag.objects.filter(tenant=tenant).filter(elementtype=elementtype, elementid=elementid).values_list('tree_id')]))

		r = signal_state_refresh(tenant.uuid, uuid_to_signal, force=True)

		if not r:
			raise Exception("Unable to force reload.")

		msg = "Recalculating state"
		return msg

	except Tag.DoesNotExist, e:
		raise Exception("Unable to reload state for element %s:%s" % (elementtype, elementid))



