import os
import sys
import time

import math

from optparse import make_option

# Local libraries (Sanet+Django related)
from django.utils.translation import ugettext as _

from sanet_common.sanetdefs.constants import *

from core.management.commands import TenantBasedCommand

from core.models import Event, Condition

from sanet_common.termutils import term_color, TERM_COLOR_CYAN
from sanet_common.timeutils import timestamp_to_isostr

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#


from cli.prompts.defs import COLORED_STATUS_TEXT

class Command(TenantBasedCommand):

	DATE_FORMAT = "%Y-%m-%d"
	DATE_FORMAT2 = "%Y-%m-%d %H:%M:%S"

	help = _("Execute an existing datagroup")

	def add_arguments(self, parser):

		super(Command, self).add_arguments(parser)

		parser.add_argument('--start'             , action='store', dest='start'                 , default=False  , help=''),
		parser.add_argument('--end'               , action='store', dest='end'                   , default=None   , help=''),
		parser.add_argument('--flap_penalty'      , action='store', dest='flap_penalty'          , default=1000   , help=''),
		parser.add_argument('--suppress_limit'    , action='store', dest='suppress_limit'        , default=2000   , help=''),
		parser.add_argument('--resume_limit'      , action='store', dest='resume_limit'          , default=1000   , help=''),
		parser.add_argument('--flap_half_life'    , action='store', dest='flap_half_life'        , default=900    , help=''),
		parser.add_argument('--max_penalty'       , action='store', dest='max_penalty'           , default=None   , help=''),
		parser.add_argument('--image'             , action='store', dest='image'                 , default=None   , help=''),
		parser.add_argument('-W'                  , action='store', dest='width'                 , default=1024   , help=''),
		parser.add_argument('-H'                  , action='store', dest='height'                , default=300    , help=''),

		#
		# Positional arguments
		#
		parser.add_argument('condition_path', nargs='?', default=None)


	#def handle(self, condition_path, **options):
	def handle(self, *args, **options):
		
		condition_path = options['condition_path']
		
		tenant = self.get_tenant(options)

		if options['end']:
			ts_end   = self.convert_date( options['end'  ] )
		else:
			ts_end   = int(time.time())
		
		if options['start']:
			ts_start = self.convert_date( options['start'] )
		else:
			ts_start = int(ts_end - 60*60*24)
		
		MAX_PENALTY = options['max_penalty']
		if MAX_PENALTY is not None:
			MAX_PENALTY = int(MAX_PENALTY)
		else:
			MAX_PENALTY = int( options['suppress_limit'] )


		#
		# Find condition's log
		#
		try:
			c = Condition.objects.get(path=condition_path)
			logs = c.get_logs()
		except Condition.DoesNotExist, e:
			# Old condition, but logs still exist
			logs = Event.objects.filter(condition_path=condition_path)


		logs = logs.filter(timestamp__gte=ts_start, timestamp__lte=ts_end)
		logs = logs.order_by('timestamp')
		
		if logs.count() == 0:
			print "No logs"
			return
		
		#
		# Fake object
		#
		c = Condition(path=condition_path)

		c.lastdone       = None
		c.penalty        = 0
		c.is_flapping    = False

		c.flap_penalty   = int(options['flap_penalty'   ])
		c.flap_half_life = int(options['flap_half_life' ])
		c.suppress_limit = int(options['suppress_limit' ])
		c.resume_limit   = int(options['resume_limit'   ])
		
		#max_penalty = 0
		#
		# SCAN
		#
		
		periods = []
		penalty_line = []
		first = True

		print "%-20s" % "TS"   ,
		print "%-5s"  % "LAST" ,
		print "%-5s"  % "CUR"  ,
		print "%10s"   % "SEC"  ,
		print "%6s"   % "PEN"  ,
		print "%6s"   % "LIMIT",
		print "FLAPPING"

		for log in logs.all():
			
			now = log.timestamp
			
			

			
			if first:
				first = False
				
				penalty_line.append( (ts_start, c.penalty, 0, c.is_flapping, log.status) )
				
				#periods.append( (ts_start, now, log.statuslast, c.is_flapping) )

				c.lastdone = ts_start
			



			periods.append( (c.lastdone, now, log.statuslast, c.is_flapping) )

			elapsed = now - c.lastdone

			self.update_penalty(now, c)

			
			penalty_line.append( (now, c.penalty, 0, c.is_flapping, log.status) )

			
			
			if is_failing_transition(log.statuslast, log.status) or is_down_transition( log.statuslast, log.status):
				self.apply_penalty(now, c, MAX_PENALTY)
				
			elif is_up_transition( log.statuslast, log.status):
				self.apply_penalty(now, c, MAX_PENALTY)

			

			penalty_line.append( (now, c.penalty, 1, c.is_flapping, log.status) )

			
			#max_penalty = max(c.penalty, max_penalty)


			print COLORED_STATUS_TEXT( log.status     , "%-20s" % (timestamp_to_isostr(log.timestamp))  ) ,
			print COLORED_STATUS_TEXT( log.statuslast , "%-5s" % CSTATE_TO_CODENAME(log.statuslast) ) ,
			print COLORED_STATUS_TEXT( log.status     , "%-5s" % CSTATE_TO_CODENAME(log.status)     ) ,
			print                                       "%10s" % (elapsed),   
			print                                       "%6s" % (c.penalty),   
			print                                       "%6s" % (c.resume_limit),   
			print term_color( TERM_COLOR_CYAN , 'F' if c.is_flapping else '' )
			
			c.lastdone = now
			

			
		periods.append( (c.lastdone, ts_end, log.status, c.is_flapping) )

		self.update_penalty(ts_end, c)
		penalty_line.append( (ts_end, c.penalty, 0, c.is_flapping, log.status) )
			

		if options['image']:
			
			MAX_PENALTY = MAX_PENALTY * 1.2
			
			
			IMAGE_WIDTH  = int(options['width'])
			IMAGE_HEIGHT = int(options['height'])
			
			W_FACTOR = float(IMAGE_WIDTH) / float(ts_end - ts_start)
			H_FACTOR = float(IMAGE_HEIGHT) / float(MAX_PENALTY)

			#width  = int(ts_end - ts_start)
			#height = int(c.suppress_limit * 1.5)
			#
			#
			#width  = int( width / FACTOR )
			#height = int( width / FACTOR )
			
			file_name = options['image']
			
			from PIL import Image
			from PIL import ImageDraw

			im = Image.new('RGB', (IMAGE_WIDTH,IMAGE_HEIGHT+2), (255,255,255))
			dr = ImageDraw.Draw(im)

			
			#
			#
			#
			ts = ts_start
			for ts_s, ts_e, status, is_flapping in periods:

				if status == CSTATE_UP  : color = '#80ff80'
				elif status == CSTATE_FA: color = '#ffff80'
				elif status == CSTATE_DN: color = '#ff8080'
				else: color = 'gray'

				x =  (ts_s - ts_start) * W_FACTOR
				x2 = (ts_e - ts_start) * W_FACTOR
			
				dr.rectangle( ( x, 0 , x2, IMAGE_HEIGHT), fill=color )
				
				#if is_flapping:
				#	dr.rectangle( ( x, IMAGE_HEIGHT/2-30 , x2, IMAGE_HEIGHT/2+30), fill='cyan' )

			

			#
			#
			#
			y = c.suppress_limit 
			dr.line(
				[ (0,float(IMAGE_HEIGHT) - y * H_FACTOR),  (IMAGE_WIDTH, float(IMAGE_HEIGHT) - y * H_FACTOR) ],
				fill="red",
				width=4,
			)				
                        #
			y = c.resume_limit 
			dr.line(
				[ (0,float(IMAGE_HEIGHT) - y * H_FACTOR),  (IMAGE_WIDTH, float(IMAGE_HEIGHT) - y * H_FACTOR) ],
				fill="#22aa22",
				width=4,
			)				



			#
			# LINEA PENALTY
			#
			points = []
			for ts,v, size, is_flapping, status in penalty_line:
				x = float(ts-ts_start) 
				y = v
				p = (x * W_FACTOR, float(IMAGE_HEIGHT) - y * H_FACTOR )
				points.append( p )
				
				if not is_flapping and status in [CSTATE_UP, CSTATE_DN]:
					size = 4 * size
				
				dr.ellipse((p[0]-size, p[1]-size, p[0]+size, p[1]+size), fill = 'blue', outline ='blue')

			dr.line(
				points, 
				fill="blue",
				width=1,
			)	
		
			#
			# OUTLINE
			#
			#dr.line(
			#	[
			#		(0, 0),
			#		(IMAGE_WIDTH-1, 0),
			#		(IMAGE_WIDTH-1, IMAGE_HEIGHT-1),
			#		(0, IMAGE_HEIGHT-1),
			#		(0, 0),
			#	]
			#	, 
			#	fill=0,
			#	width=2,
			#)
			
			del dr

			# write to stdout
			im.save(file_name, "PNG")



	def update_penalty(self, now, c): 
	
		old = c.penalty
		
		new = int(old * math.exp(-0.6931 * (float(now) - c.lastdone) /  c.flap_half_life  ))
		
		c.penalty = new

		if c.is_flapping and new < c.resume_limit:
			c.is_flapping = 0

	
	def apply_penalty(self, now, c, MAX_PENALTY):
		
		if c.flap_penalty == 0:
			return (0, 0, "no applicable penalty")
	
		is_flapping = c.is_flapping
		olddamp     = is_flapping
		old         = c.penalty
	
		new     = old + c.flap_penalty
	
		new     = min( MAX_PENALTY, new )
	
		if is_flapping == 0 and (new >= c.suppress_limit):
			c.is_flapping = 1
	
		c.penalty  = new;



	def convert_date(self, date):
		try:
			return int(time.mktime(time.strptime(date, self.DATE_FORMAT)))
		except:
			return int(time.mktime(time.strptime(date, self.DATE_FORMAT2)))
