
from webui.api.types import WEB_ELEMENT_TYPE_TO_MODEL

class BaseDataSet(object):

	def __init__(self, rid):

		# id della risorsa della pagina (uuid o altro identificativo univoco)
		self.eid      = None
		# stringa con il tipo della pagina (tenant, site, userpage, configuration, ecc..)
		self.etype    = None
		

		self.etype, self.eid = rid.split('_')
		
	def init_global_settings(self, settings_dict):
		"""
		Inizializza l'oggetto
		"""
		self.__settings = settings_dict
	
	def get_settings(self):
		return self.__settings
		
	def get_global_cache(self):
		return self.__settings['__cache_manager__']
		
	def get_local_cache(self):
		return self.__settings['__dataset_cache__']

	def get_data(self, options):
		"""
		Tutti i datasource devono implementare questo metodo
		"""

		raise Exception('Abstract Class')


	def fetch_entity(self, options):
		"""
		Metodo di comodo per caricare dal DB la risorsa interessata
		e salvarla nella cache. In questo modo piu' DataSet possono
		usarla senza ricaricare piu' volte roba dal DB
		"""

		entity_cache = options['__dataset_cache__']
		
		entity = entity_cache.get(self.eid, None)
		if entity is None:
		
			model = WEB_ELEMENT_TYPE_TO_MODEL[self.etype]
			entity = model.objects.get(uuid=self.eid)
			
			entity_cache[self.eid] = entity
		
		return entity
