# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import random

from sanet_common.sanetdefs.constants import *

from core.models import PriorityLabel

from webui.api.Data import BaseDataSet

from webui.api.utils import render_template

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class DataSet(BaseDataSet):

	def __init__(self, id):

		super(DataSet, self).__init__(id)

	def get_data(self, options): 
	
		request = options['request']

		user   = options.get('user')
		tenant = options.get('tenant')
		
		entity = self.fetch_entity(options)
		
		# Il widget "alarmscount" non ha uno stato quindi per sapere se l'utente sta filtando
		# su un priority_level guardiamo nella sessione. (Vedi alarmslist.py)
		priority_level = request.session['priority_level'] if (	'priority_level' in request.session) else None
		
		data = create_alarmscount_data(options, user, tenant, entity, priority_level=priority_level)
		
		return {
			'dataset': data
		}

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

import time

def create_alarmscount_data(options, user, tenant, element, priority_level=None):


	critical_priority = options['critical_priority']
	minimum_priority  = options['minimum_priority']
	
	critical_level = PriorityLabel.get_label(critical_priority)
	minimum_level  = PriorityLabel.get_label(minimum_priority)

	levels = get_entity_down_conditions_by_priority_level(user, element, minimum_priority)

	data = {
		'structure': {
			'template': render_template( 'html/parts/alarms_count_table.html', { 
			
						'id'            : int(time.time()),
			
						'critical_level': critical_level if critical_level else "" ,
						'minimum_level' : minimum_level  if minimum_level  else "" ,

						'levels'        : levels, 
						'width'         : 100.0 / len(levels),
						'priority_level': priority_level,
			} )
		}
	}

	return data


def get_entity_down_conditions_by_priority_level(user, entity, minimum_priority):
	
	valid_states = [CSTATE_DN]

	selected_conditions = entity.get_all_conditions_by_user(user).filter(status__in=valid_states).all()


	#
	# Sommiamo i livelli di priorita'
	#

	counters, vector = CREATE_PRIORITY_COUNTERS_DATA()
	counters[None] = 0
	for c in selected_conditions:
		level_id = vector[c.priority]
		counters[level_id] += 1

	levels = []
	for label in PriorityLabel.objects.order_by('-max_level'):
		levels.append({
			'name'     : label.label, 
			
			'min_level': label.min_level,
			'max_level': label.max_level,
			
			'count'    : counters[label.id],
			
			'visible'  : (label.min_level >= minimum_priority)
		})

	levels.append({
		'name'   : '', 
		'count'  : counters[None],
		'visible': PRIORITY_NONE >= minimum_priority,
	})
	
	return levels

#
# Struttura di comodo che associa ad ogni priorita' il proprio range, corrispondente
#
def CREATE_PRIORITY_COUNTERS_DATA():

	counters = {}

	v = [ None ] * (PRIORITY_MAX+1)
	for label in PriorityLabel.objects.order_by('min_level'):

		counters[label.id] = 0

		for i in xrange(label.min_level , label.max_level + 1):
			v[i] = label.id
	return counters, v
