# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import time

from sanet_common.sanetdefs.constants import PRIORITY_NONE

from django.conf import settings
from core.compatibility import reverse

from sanet_common.sanetdefs.constants import CSTATE_DN

from core.models import PriorityLabel, UserCustomSetting, Tenant
from core.models import MonitoredElement
from core.api.cache import init_cache_manager_from_conf

from notes.models import UserNote

from webui.models import UserTagFilters

from webui.api import *

from webui.api import DEFAULTS_CUSTOM_SETTINGS

from webui.api.Data import BaseDataSet

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class DataSet(BaseDataSet):

	def __init__(self, id):
		
		super(DataSet, self).__init__(id)

	def get_data(self, options): 

		user   = options.get('user')
		tenant = options.get('tenant')
		
		request = options.get('request')

		element = self.fetch_entity(options)

		return _alarmslist_get_data(request, user, tenant, element, options)


def _alarmslist_get_data(request, user, tenant, element, options):
	
		selected_mode  = options['params'].get('selected_mode' , None)
		short_view     = options['params'].get('short_view'    , None)
		show_all       = options['params'].get('show_all'      , False)
		priority_level = options['params'].get('priority_level', None)


		# Il widget "alarmscount" e' un "customdiv" e non supporta uno stato.
		# Per sapere quale priority_level e' stato selezionato come filtro salviamo nella
		# sessione questa informazione.
		if ('priority_level' not in request.session) or (request.session['priority_level'] != priority_level):
			request.session['priority_level'] = priority_level


		# Priorita' minima impostata da configurazione utente
		minimum_priority = options['minimum_priority']

		# Configurazione utente
		show_last_notable_change = UserCustomSetting.get_user_setting(user, 'page.alarms_list.show_last_notable_change')

		if show_last_notable_change is None:
			show_last_notable_change = DEFAULTS_CUSTOM_SETTINGS.get('page.alarms_list.show_last_notable_change', None)

		data = create_alarmslist_data(user, tenant, element, minimum_priority=minimum_priority, selected_mode=selected_mode, short_view=short_view, show_all=show_all, show_last_notable_change=show_last_notable_change, priority_level_name=priority_level)

		return {
			'dataset': data
		} 

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#


def create_alarmslist_data(user, tenant, resource, minimum_priority=None, selected_mode=None, short_view=None, show_all=False, show_last_notable_change=True, priority_level_name=None):

	#db_set_current_tenant_schema(tenant.name)
	tenant.db_set_current_schema()

	# Se e' stato specificato il nome di un priority_level, dobbiamo filtarre tutto su quello.
	# NOTA: potrebbe esseres stato cancellato, quindi prevediamolo.
	priority_level = None
	if priority_level_name is not None:
		
		if priority_level_name == '':
			priority_level = PriorityLabel(min_level=PRIORITY_NONE, max_level=PRIORITY_NONE)
		else:
			try:
				priority_level = PriorityLabel.objects.get(label=priority_level_name)
			except PriorityLabel.DoesNotExist, e:
				pass
	
	#
	# Vediamo quali tree l'utente sta filtrando
	#
	if isinstance(resource, Tenant):
		excluded_trees = None
		tag_filters = UserTagFilters.objects.filter(user=user)
		if tag_filters:
			excluded_trees = [ f.tagtree for f in tag_filters.all() ]
		can_filter_trees = True
	else:
		excluded_trees = []
		can_filter_trees = False

	#
	# Troviamo le condition (maga
	#
	valid_states = [ CSTATE_DN, ]

	if isinstance(resource, MonitoredElement):
		resource = resource.get_specific_element()

	#
	# Selezioniamo le condition. 
	# NOTA: Solo per il tenant e' possibile "escludere" (filtarre) in base agli UserTagtreeFilter
	#
	if isinstance(resource, Tenant):
		selected_conditions = resource.get_all_conditions_by_user(user, excluded_tagtrees=excluded_trees)
	else:
		selected_conditions = resource.get_all_conditions_by_user(user)
	
	selected_conditions = selected_conditions.select_related('element', 'element__parent').filter(status__in=valid_states)

	if priority_level is not None:
		selected_conditions = selected_conditions.filter(priority__range=(priority_level.min_level, priority_level.max_level))
	else:
		if not show_all:
			if minimum_priority is not None:
				selected_conditions = selected_conditions.exclude(priority__lt=minimum_priority)

	#
	# Seleziona note
	#


	# Allarmi raggruppati per piu' recenti/piu' vecchi di 24 ore
	MODE_LAST24HOUR   = 'last24'
	# Raggruppati per elemento in ordine alfabetico
	MODE_ELEMENT      = 'element'
	# Raggruppati per priorita' decrescente
	MODE_PRIORITY     = 'priority'
	# Raggruppati per priorita' decrescente
	MODE_ELEMENT_KIND = 'elementkind'


	VALID_MODES = [
		{ 'type' : MODE_LAST24HOUR  , 'icon' : 'clock-o'           , 'descr': 'Show last recent alarms' },
		{ 'type' : MODE_ELEMENT     , 'icon' : 'puzzle-piece'      , 'descr': 'Sort by node'          },
		{ 'type' : MODE_PRIORITY    , 'icon' : 'tags'              , 'descr': 'Sort by priority'      },
		{ 'type' : MODE_ELEMENT_KIND, 'icon' : 'exclamation-circle', 'descr': 'Sort by element type'   },
	]

	DEFAULT_SHORT_VIEW_LIMIT = settings.ALARMS_TABLE_SHORT_VIEW_LIMIT

	ONE_SECOND = 1
	ONE_MINUTE = 60 * ONE_SECOND
	ONE_HOUR   = 60 * ONE_MINUTE
	ONE_DAY    = 24 * ONE_HOUR

	#
	# FIX: questo codice va spostato nel Data
	#
	#valid_conditions = self.get_alarm_conditions()

	if selected_mode is None: selected_mode = MODE_LAST24HOUR
	if short_view    is None: short_view    = True

	short_view_limit = DEFAULT_SHORT_VIEW_LIMIT

	tot_selected_conditions = 0

	if selected_mode == MODE_LAST24HOUR:

		selected_conditions = selected_conditions.select_related('element')
		#selected_conditions = selected_conditions.order_by('-statuslastchange')
		selected_conditions = selected_conditions.order_by('-statuslastnotablechange')

		tot_selected_conditions = len(selected_conditions)
		
		if short_view:
			selected_conditions = selected_conditions[0:short_view_limit]

		# Troviamo tutte le note associate alle condition selezionate
		notes_by_condition = group_by_condition(selected_conditions)

		H24_TIME_LIMIT  = ONE_HOUR * 24
		WEEK_TIME_LIMIT = ONE_HOUR * 24 * 7
			
		now = time.time()

		recent_conditions = []
		week_conditions   = []
		older_conditions  = []
		for c in selected_conditions:

			# FIX: ma se qualcuno configura le sue personalizzazioni per non mostrare l'ultima variazione significativa, ma solo l'ultima variazione
			#      questa cosa non va bene.

			#if   ((now - c.statuslastchange <= H24_TIME_LIMIT )): recent_conditions.append(c)
			#elif ((now - c.statuslastchange <= WEEK_TIME_LIMIT)): week_conditions.append(c)
			#else:                                                 older_conditions.append(c)


			if   ((now - c.statuslastnotablechange <= H24_TIME_LIMIT )): recent_conditions.append(c)
			elif ((now - c.statuslastnotablechange <= WEEK_TIME_LIMIT)): week_conditions.append(c)
			else:                                                        older_conditions.append(c)


		blocks = []

		if recent_conditions:
			blocks.append({
				'title': 'Last 24h',
				'rows' : format_condition_rows(recent_conditions, notes_by_condition, show_last_notable_change=show_last_notable_change)
			})

		if week_conditions:
			blocks.append({
				'title': 'Last 7 days',
				'rows' : format_condition_rows(week_conditions, notes_by_condition, show_last_notable_change=show_last_notable_change)
			})

		if older_conditions:
			blocks.append({
				'title': 'Older',
				'rows' : format_condition_rows(older_conditions, notes_by_condition, show_last_notable_change=show_last_notable_change)
			})

	elif selected_mode == MODE_ELEMENT:
		
		selected_conditions = selected_conditions.select_related('element', 'element__parent')

		# IMPORTANTE: il sort di default (collate) di postgres gestisce male i ";" nel path! Serve un workaround per far funzionare il sort su path.
		#selected_conditions =  selected_conditions.extra(select={'path2':"replace(condition.path,';','1')"}, order_by=['path2','statuslastchange'])
		#selected_conditions =  selected_conditions.order_by('-priority','-statuslastchange')
		selected_conditions =  selected_conditions.order_by('-priority','-statuslastnotablechange')
		

		tot_selected_conditions = len(selected_conditions)
		
		if short_view:
			selected_conditions = selected_conditions[0:short_view_limit]

		notes_by_condition = group_by_condition(selected_conditions)


		blocks = []
		current_block = None
		current_element = None
		
		for c in selected_conditions:
			
			
			node_element = c.element.parent if c.element.parent is not None else c.element
			
			if node_element != current_element:
				
				if current_block is not None:
					current_block['rows'] = format_condition_rows(current_block['rows'], notes_by_condition, show_last_notable_change=show_last_notable_change)
					blocks.append(current_block)
									
				current_element = node_element
				current_block = {
					'title': c.element.path,
					'rows': []
				}
			
			current_block['rows'].append(c)
			
		if current_block is not None:
			current_block['rows'] = format_condition_rows(current_block['rows'], notes_by_condition, show_last_notable_change=show_last_notable_change)
			blocks.append(current_block)

	elif selected_mode == MODE_PRIORITY:
		
		#selected_conditions = selected_conditions.order_by('-priority', '-statuslastchange')
		selected_conditions = selected_conditions.order_by('-priority', '-statuslastnotablechange')
		
		tot_selected_conditions = len(selected_conditions)
		
		if short_view:
			selected_conditions = selected_conditions[0:short_view_limit]

		notes_by_condition = group_by_condition(selected_conditions)

		blocks = []

		label_cache = {}

		current_block = None
		current_priority = None
		
		for c in selected_conditions:
			
			label = label_cache.get(c.priority, None)
			if label is None:
				label = PriorityLabel.get_label(c.priority)
				if label:
					label = "Priority: %s" % (label)
				else:
					label = "No priority"
	
			if label != current_priority:
				
				if current_block is not None:
					current_block['rows'] = format_condition_rows(current_block['rows'], notes_by_condition, show_last_notable_change=show_last_notable_change)
					blocks.append(current_block)

				current_priority = label

				current_block = {
					'title': label,
					'rows' : []
				}
			
			current_block['rows'].append(c)
			
		if current_block is not None:
			current_block['rows'] = format_condition_rows(current_block['rows'], notes_by_condition, show_last_notable_change=show_last_notable_change)
			blocks.append(current_block)

	elif selected_mode == MODE_ELEMENT_KIND:

		from sanet_common.sanetdefs.constants import ELEMENT_KIND_NODE, ELEMENT_KIND_IFACE, ELEMENT_KIND_STORAGE, ELEMENT_KIND_SERVICE, ELEMENT_KIND_DEVICE

		LABELS = {
			ELEMENT_KIND_NODE    : 'Nodes'     ,
			ELEMENT_KIND_IFACE   : 'Interfaces',
			ELEMENT_KIND_STORAGE : 'Storages'  ,
			ELEMENT_KIND_SERVICE : 'Services'  ,
			ELEMENT_KIND_DEVICE  : 'Devices'   ,
		}

		#selected_conditions = selected_conditions.select_related('element').order_by('element__kind', '-priority', '-statuslastchange')
		selected_conditions = selected_conditions.select_related('element').order_by('element__kind', '-priority', '-statuslastnotablechange')

		tot_selected_conditions = len(selected_conditions)
		if short_view:
			selected_conditions = selected_conditions[0:short_view_limit]
	
	
		notes_by_condition = group_by_condition(selected_conditions)
	
		blocks = []
		current_block = None
		current_kind = None
		
		for c in selected_conditions:
			
			if c.element.kind != current_kind:
				
				if current_block is not None:
					current_block['rows'] = format_condition_rows(current_block['rows'], notes_by_condition, show_last_notable_change=show_last_notable_change)
					blocks.append(current_block)

				current_kind = c.element.kind
									
				current_block = {
					'title': "%s" % (LABELS[current_kind]),
					'rows' : []
				}

			
			current_block['rows'].append(c)
			
		if current_block is not None:
			current_block['rows'] = format_condition_rows(current_block['rows'], notes_by_condition, show_last_notable_change=show_last_notable_change)
			blocks.append(current_block)	
	
	#
	# Dati totali
	#

	# Filtri attivi (ordinate per nome del root tag)

	filters = []
	if can_filter_trees:
		tag_filters = [ e[1] for e in sorted([ (tf.tagtree.root.name if tf.tagtree.root is not None else None, tf) for tf in tag_filters.all() ]) ]
		for f in tag_filters:
			tagtree = f.tagtree
			if tagtree.root is None: continue # Tree senza root (inutile)

			name    = tagtree.root.name
			owner   = tagtree.owner.username if tagtree.owner is not None else None
			entry = {'uuid': tagtree.uuid, 'tree': tagtree.name, 'owner': owner, 'path': tagtree.root.name }
			filters.append(entry)

	#
	# Gruppo finale
	#
	data = {
		'resource_url'         : reverse('base_page'),
		'details_url'          : reverse('webui.views.objects.conditions.get_details'),

		# Url per la gestione dei filtri. Lato client, se queste url non sono valorizzate i filtri non sono modificabili.
		'get_tag_filters_url'  : reverse('alarms_list_get_filter_tags')   if can_filter_trees else None,
		'add_tag_filter_url'   : reverse('alarms_list_add_tag_filter')    if can_filter_trees else None,
		'remove_tag_filter_url': reverse('alarms_list_remove_tag_filter') if can_filter_trees else None,
		
		'check_condition_url'  : reverse('signal_check_condition_now'),
		
		'enable_filters'   : can_filter_trees,
		'tag_filters'      : filters,
		'tag_filters_count': len(filters),

		'modes': VALID_MODES,
		'data' : {
			'type'  : selected_mode,
			'blocks': blocks
		},

		'show_all'        : show_all,
		'priority_level'  : priority_level_name,
		
		'short_view_limit': short_view_limit,
		'num_alarms'      : len(selected_conditions),
		'tot_alarms'      : tot_selected_conditions                # numero totale di allarmi			
	}

	return data


# TODO: il nome e' fuorviante. E' da cambiare: sono raggruppati per uuid dell'elemento e non della condition. ATTENZIONE: chiamata anche su nocview
''' TODO: il nome e' fuorviante. E' da cambiare: sono raggruppati per uuid dell'elemento e non della condition. ATTENZIONE: chiamata anche su nocview'''
def group_by_condition(selected_conditions):
	
	uuids = [c.uuid for c in selected_conditions] + [c.element_id for c in selected_conditions] + [c.element.parent_id for c in selected_conditions]
	
	notes = UserNote.objects.filter(object_pk__in=uuids)
	notes_by_resource_id = {}
	for n in notes:
		l = notes_by_resource_id.get( n.object_pk, notes_by_resource_id.setdefault( n.object_pk, [] ))
		l.append(n)
		
	return notes_by_resource_id
	

def format_condition_rows(conditions, notes_by_condition, show_last_notable_change=True):
	
	# get cache manager
	cache_manager = init_cache_manager_from_conf()
	
	labels = {}
	
	rows = []
	for c in conditions:
		
		e = c.element.get_specific_element()
		
		notes = notes_by_condition.get(c.uuid, []) + notes_by_condition.get(c.element_id, []) + notes_by_condition.get(c.element.parent_id, [])
		
		p_label = labels.get( c.priority, labels.setdefault(c.priority, PriorityLabel.get_label(c.priority)))

		# get extended tile for condition
		c.load_fields_from_cache(cache_manager)
		title = c.expanded_title(cache_manager=cache_manager)

		node = e.get_node()
		rows.append(
			(c.uuid,
			 title,
			 #c.statuslastchange,
			 c.statuslastnotablechange if show_last_notable_change else c.statuslastchange,
			 
			 node.uuid,
			 node.get_icon_url(),
			 node.name,
			 
			 e.uuid,
			 e.get_icon_url(),
			 e.name,
			 
			 len(notes) > 0,
			 
			 e.resource_type,

			 c.get_state(),
			 p_label,

			)
		)
	return rows
