# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import time

from django.contrib.auth.models import Group

from core.models import Tenant
from core.api.base import tenant_get_primary

from webui.models import *

from webui.api import *

from webui.api.Data import BaseDataSet

from webui.api.utils import render_template

from webui.capabilities import CAPABILITIES_LIST

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class DataSet(BaseDataSet):

	def __init__(self, id):

		super(DataSet, self).__init__(id)

	def get_data(self, options): 

		selected_tenant_id = options.get('params',{}).get('selected_tenant', None)

		tenants = Tenant.objects.all()
		groups = Group.objects.all().order_by('name')

		capabilities = [ { 'code': c, 'descr': d, 'groups': [] } for c,d in CAPABILITIES_LIST ]

		if selected_tenant_id:
			tenant = Tenant.objects.get(uuid=selected_tenant_id)
		else:
			tenant = tenant_get_primary()
			selected_tenant_id = tenant.uuid

		for c in capabilities:

			for g in groups:

				active_capabilities = g.get_capabilities(tenant)

				if c['code'] in active_capabilities:
					c['groups'].append({'id':g.id, 'associated':True })
				else:
					c['groups'].append({'id':g.id, 'associated':False})

		html_text = render_template( 'html/parts/configuration_capabilities.html', { 
			'groups'            : groups, 
			'capabilities'      : capabilities, 
			'tenants'           : tenants,
			'selected_tenant_id': selected_tenant_id
		})

		data = { 
			'structure': {
				'template': html_text
			}
		}

		return {
			'dataset': data
		} 
