# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from core.compatibility import reverse
from django.core.exceptions import ObjectDoesNotExist


from sanet_common.sanetdefs.constants import ELEMENT_KIND_NODE   
from sanet_common.sanetdefs.constants import ELEMENT_KIND_IFACE  
from sanet_common.sanetdefs.constants import ELEMENT_KIND_STORAGE
from sanet_common.sanetdefs.constants import ELEMENT_KIND_SERVICE
from sanet_common.sanetdefs.constants import ELEMENT_KIND_DEVICE 
from sanet_common.sanetdefs.constants import CLUSTER_ROLE_STR

from conf.default_settings import XFORMS_ALIASES

from core.models import Cluster
from core.api.base import icon_select_all
from core.api.base import tenant_get_icon_url


from tags.models import *

from webui.api import *
from webui.api.Data import BaseDataSet
from webui.api.types import *

#-------------------------------------------------------------------------------#
#                   Recupera custom group, field e value                        #
#-------------------------------------------------------------------------------#

def get_custom_groups(tenant, element):

	from customfield import *
	from customfield.models import CustomGroup, CustomField, CustomFieldValue

	
	custom_groups = CustomGroup.objects.filter(tenant=tenant)
	custom_list = []
	
	type_to_html = {
		'str'  : 'text',
		'num'  : 'text',
		'date' : 'text_date'
	}
	
	for group in custom_groups:
		
		custom_dict           = {}
		custom_dict['name']   = group.name
		custom_dict['id']     = group.uuid
		custom_dict['fields'] = []
		
		fields = CustomField.objects.filter(custom_group=group)
		
		for field in fields:
			
			field_dict = {}
			
			try:
				value = CustomFieldValue.objects.get(custom_field=field, tenant=tenant, element=element)
				value = value.value
			except ObjectDoesNotExist:
				value = ''
			
			field_dict['group']       = group.uuid
			field_dict['long_name']   = field.long_name
			field_dict['name']        = field.name
			field_dict['id']          = field.uuid
			field_dict['value']       = value
			field_dict['field_type']  = type_to_html[ field.field_type ]
			field_dict['description'] = field.description
			
			custom_dict['fields'].append(field_dict)
		
		custom_list.append(custom_dict)
		
	return custom_list

#-------------------------------------------------------------------------------#
#                Prendo i valori dei field del monitoredElement                 #
#-------------------------------------------------------------------------------#

class DataSet(BaseDataSet):

	def __init__(self, id):

		super(DataSet, self).__init__(id)

	def get_data(self, options): 
		
		#user   = options['user']
		element = self.fetch_entity(options)
		tenant = options.get('tenant')

		icon_names = icon_select_all(tenant.name)
		
		icons = []
		for name in icon_names:
			icons.append({
				'name': name,
				'url' : tenant_get_icon_url(tenant.uuid, name + '.png')
			})
			
		clusters = {'null': 'None'}
		for cluster in Cluster.objects.all():
			clusters[cluster.uuid] = cluster.name
			
		xforms = []
		
		element_fields = [
			'icon',                 
			'description',          
			'priority',             
			'mtime'
		]
		
		elementdata = {}
		autocomplete = {}
		
		if element.kind is ELEMENT_KIND_NODE:
			element_fields.extend([
				'dft_timeout',          
				                       
				'use_ipv4',             
				'dns_name',             
				'dns6_name',            
				'ip4_mgt',              
				'ip6_mgt',              
				                       
				'snmp_version',         
				'snmp_port',            
				'snmp_community',       
				'snmp_v3_authdata',     
				                       
				'cluster_role',         
				'cluster_distinguisher',
				'cluster_xform',        
				'cluster_instance',     
				
				'is_router',
				'is_switch'
			])
			
			for nome_campo in element_fields:
				elementdata[nome_campo] = {'name': nome_campo, 'value': getattr(element, nome_campo)}
			
			if element.cluster == None:
				elementdata['cluster'] = ''
			
			else:
				elementdata['cluster']      = {'name': 'cluster', 'value': element.cluster.uuid}

			autocomplete = {
				'icon'                  : {'name': 'icon'        , 'values': icons            },
				'snmp_version'          : {'name': 'snmp_version', 'values': [1, 2, 3]        },
				'cluster'               : {'name': 'cluster'     , 'values': clusters         },
				'cluster_role'          : {'name': 'cluster_role', 'values': CLUSTER_ROLE_STR },
			}

		elif element.kind is ELEMENT_KIND_IFACE:
			element_fields.extend([
				'distinguisher',
				'xform',
				'ifindex',      
				'backbone'
			])
			
			for nome_campo in element_fields:
				elementdata[nome_campo] = {'name': nome_campo, 'value': getattr(element, nome_campo)}
			
			aliases = XFORMS_ALIASES['interfaces']
			for k in aliases:
				xforms.append(k)
			
			autocomplete = {
				'icon'                  : {'name': 'icon', 'values': icons},
				'xform'                 : {'name': 'xform', 'value': xforms}
			}
			
		elif element.kind is ELEMENT_KIND_STORAGE:
			element_fields.extend([
				'distinguisher',
				'xform',     
				'stindex'
			])
			
			for nome_campo in element_fields:
				elementdata[nome_campo] = {'name': nome_campo, 'value': getattr(element, nome_campo)}
			
			aliases = XFORMS_ALIASES['storages']
			for k in aliases:
				xforms.append(k)
				
			autocomplete = {
				'icon'                  : {'name': 'icon', 'values': icons},
				'xform'                 : {'name': 'xform', 'value': xforms}
			}
			
		elif element.kind is ELEMENT_KIND_SERVICE:
			element_fields.extend([
				'distinguisher',
				'xform',  
				'swrunindex'
			])
			
			for nome_campo in element_fields:
				elementdata[nome_campo] = {'name': nome_campo, 'value': getattr(element, nome_campo)}
			
			aliases = XFORMS_ALIASES['services']
			for k in aliases:
				xforms.append(k)
				
			autocomplete = {
				'icon'                  : {'name': 'name', 'values': icons},
				'xform'                 : {'name': 'icon', 'value': xforms}
			}
			
		elif element.kind is ELEMENT_KIND_DEVICE:
			element_fields.extend([
				'distinguisher',
				'xform',     
				'devindex'
			])
			
			for nome_campo in element_fields:
				elementdata[nome_campo] = {'name': nome_campo, 'value': getattr(element, nome_campo)}
			
			aliases = XFORMS_ALIASES['devices']
			for k in aliases:
				xforms.append(k)
				
			autocomplete = {
				'icon'                  : {'name': 'name', 'values': icons},
				'xform'                 : {'name': 'xform', 'value': xforms}
			}
			
		def_values = {}
		for k, v in element.get_fields_default_value_dict().items():
			def_values[k] = {'name': k, 'value': v}
		
		
		#custom_dict = get_custom_groups(tenant, element=element)

		data = {
			'elemtype'      : ELEMENT_KIND_STR[element.kind],
			'data'          : elementdata,
			'autocomplete'  : autocomplete,
			'default_values': def_values,
			
			'submit_url'    : reverse('webui.views.objects.configure.monitoredelement_config_fields', kwargs={'uuid': element.uuid})
			
			
			#'custom_dict'   : custom_dict,
		}

		return {
			'dataset': data
		} 
