# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from core.compatibility import reverse

from conf import  get_broker_connection_string

from core.api.base import tenant_get_by_id

from webui.api import *

from webui.api.Data import BaseDataSet

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class DataSet(BaseDataSet):

	def __init__(self, id):
		self.id = id

	def get_data(self, options):
		return self.get_entables_monitoring_stats(self.id, options.get('filter_field'), options.get('filter_criteria'))


	def get_entables_monitoring_stats(self, tenant_uuid, filter_field = None, filter_criteria=None):

		tenant = tenant_get_by_id(tenant_uuid)


		# get entables information running
		from entables.management.commands.entables_ctl import entables_ctl
		broker_address = get_broker_connection_string()

		info = None
		status = 'DN'
		try:
			cmd = 'info'
			args = []

			broker_address = get_broker_connection_string()
			info           = entables_ctl(broker_address, cmd, list(args))
			if info is not None:
				status = 'UP'

		except Exception, e:
			info = None
			status = 'DN'




		sort_column = 1
		headers = [ 'Service', 'Running', ]

		row_classes = [status]
		
		rows = []
		rows.append(
			['entables daemon', '*' if status == 'UP' else '' ]
		)
		
		
		if status == 'UP':
			ref = [{
				'type'   :'wloader',
				'url'    : reverse('webui.views.objects.agents.get_stats_entables'),
				'wtype'  : 'customdiv',
				'wparams': {
					'template': '<pre>{{info}}</pre>'
				},
			}]
		else:
			ref = []
		
        
		return {
			'dataset':{
				'url': reverse('webui.views.objects.monitored.get_data')+'?d=%s.%s' % ('default', 'entables_monitoring_stats'),

				"data" : {
					"headers": headers,
					'values' : rows,
					'classes': row_classes,
					"ref"    : [ ref ],
				},
				"ctx": {
					"results": len(rows),
					"offset" : 0,
					"paged"  : len(rows)
				},
				"req" : {
					"filter"  : {
					"enabled" : False,
					"field"   : filter_field,
					"criteria": filter_criteria
				},
				"sort": {
					"enabled": True,
					"field"  : sort_column, 
					"dir"    : True
					}
				}
			}
		}
		
		
		
		
        
