

from django.db.models import Q
from core.compatibility import reverse

from sanet_common.sanetdefs.constants import CSTATE_TO_CODENAME, _CSTATE_FROM_CODENAME, CSTATE_FROM_CODENAME, ELEMENT_KIND_STR, ELEMENT_KIND_VAL

from core.models import Condition

from tags.models import *

from webui.api.Data import BaseDataSet

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class DataSet(BaseDataSet):

	def __init__(self, id):
		super(DataSet, self).__init__(id)
		self.id = id
		self.part = "event_log"

	def get_data(self, options):
		request = options['request']

		user   = options.get('user')
		tenant = options.get('tenant')
		
		#tenant.db_set_current_schema()

		#condition = Condition.objects.get(uuid=self.id)
		condition = self.fetch_entity(options)

		kinds = {
			'Condition': 1,
			'Syslog'   : 2,
			'Trap SNMP': 3,
		}
		
		filters = {
			'T':{
				'field': 'kind',
				'op'   : lambda x : kinds[x] if x in kinds else None,
				'rel'  : ''
			},
			'element' : {
				'field': 'element_path',
				'op'   : lambda x: x,
				'rel'  : '__contains'
			},
			'kind' : {
				'field': 'element_path',
				'op'   : lambda x: ELEMENT_KIND_VAL[x],
				'rel'  : ''
			},
			'condition':{
				'field': 'condition_path',
				'op'   : lambda x: x,
				'rel'  : '__contains'
			},
			'message':{
				'field': 'msg',
				'op'   : lambda x: x,
				'rel'  : '__contains'
			},
			'PS':{
				'field': 'statuslast',
				'op'   : lambda x:_CSTATE_FROM_CODENAME[x],
				'rel'  : ''
			},
			'S':{
				'field': 'status',
				'op'   : lambda x:_CSTATE_FROM_CODENAME[x],
				'rel'  : ''
			},
			'uuid':{
				'field':'uuid',
				'op'   : lambda x: x,
				'rel'  : '__contains'
			}
		}
		
		if options.get('remote'):
			fields = ['kind', 'timestamp','statuslast','msg', 'element_path', 'condition_path', 'priority','uuid', 'status']
			headers = ['T', 'time', 'condition', 'PS','S','message',]#, 'uuid']
			data_op = lambda x: [
				{
					'type':'template',
					'template':'eventtypes',
					'value':{'kind':icons[x['kind']], 'kind_str':label[x['kind']]}
				},
				{
					'type':'datetime', 'value':x['timestamp']
				},
				x['condition__name'],
				CSTATE_TO_CODENAME(x['statuslast']),
				CSTATE_TO_CODENAME(x['status']),
				x['msg'],
			]
		else:
			fields = [
				'kind', 
				'timestamp',
				'statuslast',
				'msg', 
				'element_path', 
				'condition_path', 
				'condition_uuid',
				'priority','uuid', 
				'status'
			]
			headers = [
				'T', 
				'time', 
				'element', 
				'kind', 
				'condition', 
				'PS',
				'S',
				'message',
			]#, 'uuid']
			
			data_op = lambda x: [
				{
					'type'    :'template',
					'template':'eventtypes',
					'value'   : {'kind':icons[x['kind']], 'kind_str':label[x['kind']]}
				},
				{
					'type':'datetime', 
					'value':x['timestamp']
				},
				{
					'label': x['element_path'], 
					'value':reverse('resource_page', kwargs={'rt':ELEMENT_KIND_STR[x['element__kind']], 'uuid':x['element__uuid']}),
					'type' :'url'
				} if x['element__uuid'] is not None else "",
				ELEMENT_KIND_STR[x['element__kind']], 
				x['condition__name'],
				CSTATE_TO_CODENAME(x['statuslast']),
				CSTATE_TO_CODENAME(x['status']),
				x['msg'],
			] if x.has_key('element__kind') else []

		do_filter = Q()
		ff = {}

		if options.get('filter_field') in filters.keys():
			ff[filters[options.get('filter_field')]['field']+filters[options.get('filter_field')]['rel']] = filters[options.get('filter_field')]['op'](options.get('filter_criteria'))
			do_filter = do_filter | Q(**ff)

		if self.id == "default":
			filter = {}
			header = []
			op = lambda x: x
		else:
			filter = {}
			header = []
			op = lambda x: x
		
		if len(self.id.split("_")) == 2:
			# check 
			self.id = self.id.split("_")[1]
		
		#if self.id != 'default':
		#	cs = list(Condition.objects.filter(uuid=self.id).values_list('path', flat=True))
		#    
		#	try:
		#		total_events =  Tag.objects.get(uuid=self.id).get_logs().count() # FIXME: check count optimization
		#		events = list(Tag.objects.get(uuid=self.id).get_logs().values(*fields)[options.get('offset'):options.get('offset')+options.get('pagesize')])
		#	except:
		#		total_events = Event.objects.filter(condition_path__in=cs).filter(**ff).count() # FIXME: check count optimization
		#		events = list(Event.objects.filter(condition_path__in=cs).filter(**ff).values(*fields)[options.get('offset'):options.get('offset')+options.get('pagesize')])
		#else:
		#	cs = list(Condition.objects.all().values_list('path', flat=True))
		#
		#	total_events =  Event.objects.filter(condition_path__in=cs).filter(**ff).count() # FIXME: check count optimization
		#	events = list(Event.objects.filter(condition_path__in=cs).filter(**ff).values(*fields)[options.get('offset'):options.get('offset')+options.get('pagesize')])
		    
		
		events = condition.get_logs() 
		print events.count()
		events = events.filter(**ff)
		events = events.values(*fields)
		
		events = events[options.get('offset'):options.get('offset')+options.get('pagesize')]
		
		print len(events)
		total_events = events.count()
		
		    
		for e in events:
			try:
				c = Condition.objects.get(Q(path = e['condition_path'])|Q(uuid=e['condition_uuid']))
			except Condition.DoesNotExist, e:
				continue
				
			e['condition__uuid'] = c.uuid
			e['condition__name'] = c.name
			e['element_path'] = c.element.calculate_path()
			e['element__uuid'] = c.element.uuid
			e['element__kind'] = c.element.kind
			e['element__name'] = c.element.name
			e['element__parent'] = c.element.parent is not None
			
			if c.element.parent is not None:
				e['element__parent__name'] = c.element.parent.name
				e['element__parent__uuid'] = c.element.parent.uuid
				e['element__parent__kind'] = c.element.parent.kind

		icons = {
			1: 'info-sign',
			2: 'reorder',
			3: 'download-alt',
		}

		label = {
			1: 'Condition',
			2: 'Syslog',
			3: 'Trap SNMP',
		}

		dataset = {
			'url': reverse('webui.views.objects.monitored.get_data') + '?d=' + json.dumps({self.id+'.'+self.part:{}}),

			"data" : {
				"headers": headers,
				"values" : [data_op(x) for x in events],  
				'classes': [CSTATE_TO_CODENAME(x['status']) for x in events],
				'ref'    : [[{
					'foldicon' :'icon-info icon-fixed-width', 
					'foldtitle':'Details',
					'wheight'  :'250px',
					'type'     :'wloader',
					'url'      : reverse('webui.views.objects.logs.get_details') + "?uuid="+x['uuid'],
					'wtype'    :'cutomdiv',
				}] for  x in events],
			},
			"ctx": {
				"results": total_events, 
				"offset" : 0,
				"paged"  : len(events),
			},
			"req" : {
				"filter": {
					"enabled" : True,
					"field"   : options.get('filter_field'),
					"criteria": options.get('filter_criteria')
				},
				"sort": {
					"enabled": True,
					"field"  : 'status',
					"dir"    : False
				}
			}
		}

		return {'dataset':dataset }

