# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import time

from core.compatibility import reverse

from sanet_common.sanetdefs.constants import *
from sanet_common.timeutils import timestamp_to_isostr

from tags.models import *

from webui.api import *
from webui.api.Data import BaseDataSet
from webui.api.types import *

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

class DataSet(BaseDataSet):

	def __init__(self, id):

		super(DataSet, self).__init__(id)

	def get_data(self, options): 

		user   = options.get('user')

		element = self.fetch_entity(options)

		data = _get_logs(user, element, options.get('params', {}))

		return {
			'dataset': data
		} 

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

DEFAULT_TIMESTAMP_START = 0
DEFAULT_TIMESTAMP_END   = 2147000000 #Jan 2038-01-13 13:53:20 2038'

DEFAULT_DATE_START=''
DEFAULT_DATE_END  =''
DEFAULT_HOUR_START='00:00:00'
DEFAULT_HOUR_END  ='23:59:59'

DEFAULT_TRANSITIONS = {

	'UP': {
		'FA': 1,
		'DN': 1,
	},

	'FA': {
		'UP': 1,
		'DN': 1,
	},

	'DN': {
		'UP': 1,
	},
}

LOG_TYPES = [
	EVENT_CONDITION,
	EVENT_SYSLOG   ,
	EVENT_SNMPTRAP ,
	EVENT_CUSTOM,
]

def LogOptions(**settings):

	d = {
			'show_options': False,

			'sort_type'   : 1,
			'sort_field'  : "timestamp",

			'page'        : 1,
			'pagesize'    : 25,

			'priority'    : None,
			'freetext'    : None,

			'transitions' : None, #DEFAULT_TRANSITIONS,
			'show_transitions_table': False,

			'date_start'  : None,
			'date_end'    : None,
			'hour_start'  : None,
			'hour_end'    : None,
			
			'selected_kinds': {
				EVENT_CONDITION : True,
				EVENT_SYSLOG    : True,
				EVENT_SNMPTRAP  : True,
				EVENT_CUSTOM    : True,
			}
	}
	d.update(settings)

	return d


def parse_date_hour(d, h):

	d = (d if d else DEFAULT_DATE_END)
	h = (h if h else DEFAULT_HOUR_START)
	d_str =  d+' '+h

	for f in ["%Y-%m-%d %H:%M:%S", "%Y-%m-%d %H:%M", "%Y-%m-%d %H",]:
		try:
			v = time.mktime(time.strptime(d_str, f))
			return v
		except Exception, e:
			pass

	return None




def _get_logs(user, element, options, return_query_set=False, no_pagesize=False):
	
	
	#
	# Read options
	#
	show_options = options.get('show_options', False)
	sort_type    = int(options.get('sort_type' , 1))
	sort_field   = options.get('sort_field', 'timestamp') 
	page         = max(1, options.get('page'      , 1))
	pagesize     = options.get('pagesize'  , 25) 
	priority     = options.get('priority', None)
	freetext     = options.get('freetext', None)
	transitions  = options.get('transitions', {})
	
	date_start   = options.get('date_start', '')
	date_end     = options.get('date_end'  , '')
	
	hour_start   = options.get('hour_start', '')
	hour_end     = options.get('hour_end'  , '')
	
	selected_kinds = options.get('selected_kinds', {})
	
	#
	# DATES
	#
	timestamp_start = None
	timestamp_end   = None

	timestamp_start = parse_date_hour(date_start, hour_start)
	if timestamp_start is None:
		date_start = ''
		hour_start = ''
		timestamp_start = DEFAULT_TIMESTAMP_START

	timestamp_end = parse_date_hour(date_end, hour_end)
	if timestamp_end is None:
		date_end = ''
		hour_end = ''
		timestamp_end = DEFAULT_TIMESTAMP_END

	#
	# Compose query
	#
	selected_logs = element.get_logs_by_user(user)

	filter_params = {}
	filter_params['timestamp__gt'] = timestamp_start
	filter_params['timestamp__lt'] = timestamp_end
	
	selected_logs = selected_logs.filter(**filter_params)
	
	
	excluded_kinds = []
	for lt in LOG_TYPES:
		s = selected_kinds.get(str(lt), selected_kinds.setdefault(str(lt), True))
		if (s == False):
			excluded_kinds.append(lt)
	
			
	if excluded_kinds:
		selected_logs = selected_logs.exclude(kind__in=excluded_kinds)
	

	if priority:
		selected_logs = selected_logs.filter(priority=int(priority))
	
	if freetext:
		selected_logs = selected_logs.filter(condition_path__contains=freetext)
	
	if transitions is not None and transitions != 'all':
			transitions_Q = Q()
			for t_from, to_transitions in transitions.items():
				for  t_to, val in to_transitions.items():
					if val == 0:
						continue

					statuslast = CSTATE_FROM_CODENAME(t_from)
					status     = CSTATE_FROM_CODENAME(t_to)

					transitions_Q = transitions_Q | (Q(statuslast=statuslast) & Q(status=status))

			selected_logs = selected_logs.filter(transitions_Q)


	# Order by
	sort_options = ['%s%s' % ( '-' if sort_type == 1 else '', sort_field)]
	selected_logs = selected_logs.order_by(*sort_options)
	
	# page/position
	if not no_pagesize:
		offset = (page - 1) * pagesize
		limit  = offset     + pagesize
		selected_logs = selected_logs[offset:limit]
	
	if return_query_set:
		return selected_logs
	
	#
	# Produce data
	#
	
	logs = []
	for l in selected_logs:

		d = {
				'id'     : l.uuid,
				'k'      : l.kind,
				'ts'     : timestamp_to_isostr(l.timestamp),
				'element': l.element_path,
				'cpath'  : l.condition_path or l.trapoid or "custom event",
				'prev'   : CSTATE_TO_CODENAME(l.statuslast),
				'cur'    : CSTATE_TO_CODENAME(l.status),
 				'p'      : l.priority,
				'info'   : l.msg,
		}

		logs.append(d)
		
		

	options.update({
			'show_options': show_options,
			
			'sort_type'   : sort_type,
			'sort_field'  : sort_field,
			
			'page'        : page,
			'pagesize'    : pagesize,
			
			'priority'    : priority,
			'freetext'    : freetext,
			
			'transitions' : transitions,

			'date_start'  : date_start,
			'date_end'    : date_end,
			
			'selected_kinds': selected_kinds,
	})

	data = {
		'etype'       : element.resource_type,
		'uuid'        : element.resource_id,
		
		'element_path': element.path,
		'icon_url'    : element.get_icon_url(),
		
		'logs'        : logs,
		'options'     : LogOptions(**options),
		'url'         : reverse('webui.views.objects.logs.get_details'),
		'url_entables': reverse('webui.views.objects.logs.get_entables_details'),
		'url_path'    : reverse('webui.views.objects.monitored.page_by_path'),
		'exporturl'   : reverse('webui.views.objects.logs.export_csv')
	}

	return data
