# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from core.compatibility import reverse

from sanet_common.timeutils import timestamp_to_isostr
from sanet_common.sanetdefs.constants import CSTATE_TO_CODENAME

from core.models     import TimeGraph, Condition
from core.api.cache  import init_cache_manager_from_conf

from webui.api.Data  import BaseDataSet

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class DataSet(BaseDataSet):
	"""
	Usato dalls troubleshooting view per mostrare i dettagli degli elementi elencati (condition, timegraph, ecc.)
	"""

	def __init__(self, id):
		super(DataSet, self).__init__(id)

	def get_data(self, options):

		info = {}

		element = self.fetch_entity(options)

		if isinstance(element, TimeGraph):
			
			dg = element.datagroup

			info['values'] = [
				('Element'            , str(element.element.path) ),
				('Title'              , element.title ),
				('Path'              , element.path ),
				('Datagroup'          , element.datagroup.path),
				('Datagroup template' , element.datagroup.cdatagroup.get_tdatagroup().name ),
				('Internal Id'        , element.uuid),
			]
			
			real_series     = list(element.get_real_series().all())
			computed_series = list(element.get_computed_series().all())
			
			datasources_by_name = dict([ (ds.name, ds) for ds in dg.datasources.filter(name__in=[ s.variable_value for s in real_series ]) ])
	
			
			for s in real_series + computed_series:

				label = 'Serie %s' % (s.name)
				
				if s.variable_value:
					ds = datasources_by_name[s.variable_value]
					url = reverse('resource_page', kwargs={'rt': ds.resource_type, 'uuid': ds.resource_id } )
					
					text = "datasource: %s <a href='%s'> %s </a>" % (ds.get_outcome(), url, s.name,  ) 
				else:
					text = "computed: %s" % (s.computed_value_expr)
				
				info['values'].append( (label, text) )

		elif isinstance(element, Condition):

			cache_manager = init_cache_manager_from_conf()

			element.load_fields_from_cache(cache_manager)


			explanation = element.get_full_verbstatus()
			
			explanation = explanation.replace("\n", "<br />")

			info['values'] = [
				("Explanation"        , explanation   ),
				
				#('Status'             , CSTATE_TO_CODENAME(element.status) ),
				#('Tries'              , "%s / %s" % (element.tries, element.max_tries )),
				('MinPeriod'          , "%s " % (element.datagroup.minperiod) ),
				('Verbose Status'     , "<tt>" + str(element.verbstatus) + "</tt>" ),

				('Last done'          , timestamp_to_isostr(element.lastdone)         if element.lastdone           is not None else "n/a"  ),
				('Last check time'    , "%.5f" % (element.lastchecktotaltime)         if element.lastchecktotaltime is not None else "n/a"  ),
				('Last change'        , timestamp_to_isostr(element.statuslastchange) if element.statuslastchange != 0          else "n/a"  ),
				('Last Status'        , CSTATE_TO_CODENAME(element.statuslast) ),
				#(''                   , ''),
				('Expr'               , "<tt>" + element.expr + "</tt>"),
				#('Short tries'        , element.datagroup.shorttries),
				#(''                   , ''),
				('Primary'            , str(element.primary) ),
				#('Priority'           , element.priority_label),
				('UN Fallback'        , CSTATE_TO_CODENAME(element.uncheckable_fallback) if element.uncheckable_fallback is not None else "-"),
				('Status action'      , element.statuschange_action),
				#(''                   , ''),
				('Title'              , element.expanded_title(cache_manager) ),
				('Internal Id'        , element.uuid),
				('Path'               , element.path ),
				('Element'            , str(element.element.path) ),
				('Agent'              , str(element.agent.name)),
				('Datagroup'          , element.datagroup.path),
				('Datagroup template' , element.datagroup.cdatagroup.get_tdatagroup().name ),
				#(''                   , ''),
				("Down email"         , element.email        ),
				("Down subject"       , element.msg_downsubj ),
				("Down body"          , element.msg_downbody ),
				("Up email"           , element.upemail      ),
				("Up subject"         , element.msg_upsubj   ),
				("Up body"            , element.msg_upbody   ),
			]
		else:
			info['values'] = {}

		return {'dataset':info }
