# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import time

from core.models import Tenant, Agent

from core.api.cache import init_cache_manager_from_conf

from webui.api import *

from webui.api.Data import BaseDataSet     
   
from webui.api.utils import render_template

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class DataSet(BaseDataSet):

	def __init__(self, id):	
		super(DataSet, self).__init__(id)

	def get_data(self, options):

		cache_manager = init_cache_manager_from_conf()

		all_agents = []

		tenants = Tenant.objects.all().order_by('name')

		for tenant in tenants:

			tenant.db_set_current_schema()

			agents = Agent.objects.filter(tenant=tenant).order_by('name')

			for a in agents:
				a.load_fields_from_cache(cache_manager)
				all_agents.append(a)

		html = render_template( 'html/parts/stats_agents.html', { 
			'TIME'  : time.time(),
			'agents': all_agents,
			'tenant': tenant,
		})

		data = { 
			'structure': {
				'template': html
			}
		}

		return {
			'dataset': data
		}
