# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import time

from django.conf import settings

from core.compatibility import reverse

from sanet_common.timeutils import timestamp_to_isostr

from core.api.sysstats import sanetd_get_current_state

from webui.api.Data import BaseDataSet

from webui.api.utils import render_template

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class DataSet(BaseDataSet):

	def __init__(self, id):	
		super(DataSet, self).__init__(id)

	def get_data(self, options):

		tenant = options.get('tenant')

		instances = []

		for instance_id in settings.SANETD_INSTANCES:
			#
			# Current status
			#
			status = sanetd_get_current_state(instance_id=instance_id)

			if status:
				status['checktime'] = timestamp_to_isostr(status['checktime'])
			else:
				status = {
					'checktime': None,
					'running'  : False,
					'info'     : "Unable to get status",
				}

			graphs = [
				reverse('sanetd_stat_chart', kwargs={ 'instanceid': instance_id, 'chart': 'eventread'   }), 
				reverse('sanetd_stat_chart', kwargs={ 'instanceid': instance_id, 'chart': 'queue'       }), 

				reverse('sanetd_stat_chart', kwargs={ 'instanceid': instance_id, 'chart': 'processrate' }), 

				reverse('sanetd_stat_chart', kwargs={ 'instanceid': instance_id, 'chart': 'processtimes'}), 
			]

			instances.append({ 
				'name'     : instance_id,
				'status'   : status,
				'graphs'   : graphs,
			})

		#
		# RENDER
		#
		html = render_template( 'html/parts/stats_sanetd.html', { 
			'TIME'     : time.time(), 
			'instances': instances 
		})

		data = { 
			'structure': {
				'template': html
			}
		}

		return {
			'dataset': data
		}
