# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings

from sanet_common.sanetdefs.constants import *

from core.models import Cluster
from core.api.cache import init_cache_manager_from_conf

from tags.models import TAG_FILTER

from webui.models import *

from webui.api.widgets import CustomDivWidget 
from webui.api.widgets import widget_resource_section_title

from webui.api.Pages import BasePage
#from webui.api.Pages import PageTab
#from webui.api.Pages import VerticalLayout

from webui.api.utils import render_template

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class Page(BasePage):

	_element_class = Cluster

	TAB_GENERAL         = 'general'

	LAYOUT_INFO         = 'L-info' 

	def __init__(self, rid, tenant, global_settings):

		super(Page, self).__init__(rid, tenant, global_settings)

		user = global_settings.get('user')

		# Carichiamo l'oggetto giusto per questa pagina
		self.element  = self._element_class.objects.get(uuid=self.rid)
		self.resource = self.element

	def get_title(self):
		return self.resource.name

	def get_widgets(self):
		
		user = self.global_settings['user']

		cluster = self.element


		widgets = []


		title = widget_resource_section_title(cluster, "Cluster", self.TAB_GENERAL, self.LAYOUT_INFO)
		widgets.append(title)

		cache_manager = init_cache_manager_from_conf()

		nodes = MonitoredNode.objects.filter(cluster=cluster)
		tot_cluster_nodes = nodes.count()

		nodes = TAG_FILTER(cluster.tenant, user, nodes, 'monitoredelement')

		for n in nodes:
			n.load_fields_from_cache(cache_manager)

		template_data = {
			'nodes'            : nodes,
			'tot_nodes'        : len(nodes),
			'tot_cluster_nodes': tot_cluster_nodes,
		}

		html = render_template( 'html/parts/cluster_nodes.html', template_data )

		w = CustomDivWidget(

			id        = '%s_cluster_nodes' % (cluster.uuid),

			tab       = self.TAB_GENERAL,
			container = self.LAYOUT_INFO,

			rdata     = {
				'structure': {
					'template': html,
				}
			}
		)
		widgets.append(w.to_dict())

		return widgets

