# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from sanet_common.timeutils import timestamp_to_isostr
from sanet_common.sanetdefs.constants import CSTATE_TO_CODENAME

from core.api.cache import init_cache_manager_from_conf

from core.models import Condition

from webui.models import *

from webui.api import format_html 

from webui.api.Pages import PageTab, VerticalLayout, HorizontalLayout, BasePage, LogPage

from webui.api.utils import render_template

from webui.api.widgets import FrameWidget, CustomDivWidget, ConditionGraphWidget, NotesWidget

from tags.models import ElementTag

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#


class Page(BasePage, LogPage):

	_element_class = Condition

	TAB_GENERAL         = 'general'
	TAB_LOG             = 'log'

	LAYOUT_INFO         = 'L-info' 
	LAYOUT_GRAPH        = 'L-graph'
	LAYOUT_LOG          = 'L-log'

	def __init__(self, rid, tenant, global_settings):

		super(Page, self).__init__(rid, tenant, global_settings)

		# Carichiamo l'oggetto giusto per questa pagina
		self.element = self._element_class.objects.get(uuid=self.rid)

		self.resource = self.element

	def get_title(self):
		return self.resource.title
	
	def get_tabs(self):

		self.tabs = []

		self.tabs.extend([
			PageTab(**{
				'name'      : self.TAB_GENERAL, # name for tab
				'title'     : 'Global',         # title human readable
				'icontype'  : 'fonticon',       # choose icon: fonticon, image
				'iconvalue' : 'icon-list-alt',  # icon url, run for icon and image
				'active_tab': True,             # active tab information
			})
		])
		self.tabs.extend( LogPage.get_tab_def(self)        )

		return self.tabs

	def get_page_layout(self):

		tabs_layout = {}

		tabs_layout[self.TAB_GENERAL ] = VerticalLayout('L1', [
							VerticalLayout(self.LAYOUT_INFO ) ,
							VerticalLayout(self.LAYOUT_GRAPH) ,
							HorizontalLayout(self.LAYOUT_LOG)
		])

		return tabs_layout

	#-------------------------------------------------------------------------------#

	def get_log_tab_layout(self):
		return self.TAB_GENERAL, self.LAYOUT_LOG

	def render(self):

		tabs    = self.get_tabs()
		layout  = self.get_page_layout()
		
		widgets = self.get_widgets()

		page_struct =  {
			'name'   : self.pagetype,   
			'layout' : layout,
			'widgets': widgets,
			'tabs'   : tabs,
		}

		return page_struct


	def get_widgets(self):

		widgets = []

		widgets += self.get_details_widget()

		widgets += self.get_graph()

		widgets += self.get_log_widgets()
		
		return widgets

	def get_graph(self):

		widgets = []

		c = self.element

		w = ConditionGraphWidget(c, tab=self.TAB_GENERAL, container=self.LAYOUT_GRAPH).to_dict()
		widgets.append(w)

		return widgets

	def get_details_widget(self):

		condition = self.element

		cache_manager = init_cache_manager_from_conf()
		condition.load_fields_from_cache(cache_manager)

		widgets = []

		element = condition









		#
		#
		#
		details = []
		details.append({ 'name': 'Title'              , 'value':  element.expanded_title(cache_manager)                                                                 })
		details.append({ 'name': 'Element'            , 'value':  element.element                                                                                       })
		details.append({ 'name': 'Datagroup'          , 'value':  element.datagroup                                                                                     })
		details.append({ 'name': 'Primary'            , 'value':  str(element.primary)                                                                                  })
		details.append({ 'name': 'Priority'           , 'value':  element.priority_label                                                                                })
		details.append({ 'name': 'Tries'              , 'value':  "%s" % (element.verbose_tries )                                                                       })
		details.append({ 'name': 'MinPeriod'          , 'value':  "%s" % (element.datagroup.minperiod)                                                                  })
		details.append({ 'name': 'Verbose Status'     , 'value':  str(element.verbstatus)                                                                               })


		
		

		icon = condition.get_icon_url_with_state(CSTATE_TO_CODENAME(condition.status))

		c = {
			'icon_url'         : icon, 
			'element'          : condition,
			'state'            : CSTATE_TO_CODENAME(condition.status),
			'details'          : details,
		}		

		html = render_template( 'html/parts/condition_page_details.html', c )
		
		
		widget_info = CustomDivWidget(
			tab       = self.TAB_GENERAL,
			container = self.LAYOUT_INFO,

			rdata = {
				'structure': {
					'template': format_html( html  )
				}
			}
		).to_dict()
		
		#widgets.append(w)
		
		
		
		
		
		
		
		
		

		#
		# MORE
		#

		details = []
		details.append({ 'name': 'Expr'               , 'value':  element.expr                                                                                          })
		details.append({ 'name': 'Last done'          , 'value':  timestamp_to_isostr(element.lastdone)         if element.lastdone  is not None else ""                })
		details.append({ 'name': 'Last change'        , 'value':  timestamp_to_isostr(element.statuslastchange) if element.statuslastchange != 0 else ""                })
		details.append({ 'name': 'Last Status'        , 'value':  CSTATE_TO_CODENAME(element.statuslast)                                                                })
		details.append({ 'name': 'UN Fallback'        , 'value':  CSTATE_TO_CODENAME(element.uncheckable_fallback) if element.uncheckable_fallback is not None else ""  })
		details.append({ 'name': 'Status action'      , 'value':  element.statuschange_action                                                                           })
		details.append({ 'name': 'Datagroup template' , 'value':  element.datagroup.cdatagroup.get_tdatagroup().name                                                    })
		details.append({ 'name': "Down email"         , 'value':  element.email                                                                                         })
		details.append({ 'name': "Down subject"       , 'value':  element.msg_downsubj                                                                                  })
		details.append({ 'name': "Down body"          , 'value':  element.msg_downbody                                                                                  })
		details.append({ 'name': "Up email"           , 'value':  element.upemail                                                                                       })
		details.append({ 'name': "Up subject"         , 'value':  element.msg_upsubj                                                                                    })
		details.append({ 'name': "Up body"            , 'value':  element.msg_upbody                                                                                    })
		details.append({ 'name': 'Internal Id'        , 'value':  element.uuid                                                                                          })
		details.append({ 'name': 'Path'               , 'value':  element.path                                                                                          })

		icon = condition.get_icon_url_with_state(CSTATE_TO_CODENAME(condition.status))

		html = render_template( 'html/parts/advanced_details.html', {
			'details'           : details,
		} )

		widget_info_more = CustomDivWidget(
			tab       = self.TAB_GENERAL,
			container = self.LAYOUT_INFO,
			rdata = {
				'structure': {
					'template': format_html( html  )
				}
			}
		).to_dict()

		widget_tags = self.create_tags_widget(condition)

		widget_notes = self.create_notes_widget(condition)

		widget = FrameWidget(
			id = 'a',
			
			tab       = self.TAB_GENERAL,
			container = self.LAYOUT_INFO,
			
			rdata = {
				"struct": [
					{
						"type"     : "widget",
						"contents" : widget_info
					},
					{
						"type"     : "wframesub",
						"hideable" : True,
						"visible"  : False,
						"buttonpos": 0,
						"contents" :
						[
							{
								"type"    : "widget",
								"contents": widget_info_more
							},
							{
								"type"    : "widget",
								"contents": widget_notes
							},
							{
								"type"    : "widget",
								"contents": widget_tags
							}
						]
					}
				]
			},
		)
		widgets.append(widget.to_dict())

		return widgets

	def create_notes_widget(self, n):

		widget_notes = NotesWidget(
			id = "%s_%sA%s" % (n.resource_type, n.resource_id, 'notelist'),

			tab       = self.TAB_GENERAL,
			container = self.LAYOUT_INFO,

			rdata = {},

			remote = "%s_%s.%s" % (n.resource_type, n.resource_id, 'notelist')
		)

		return widget_notes.to_dict()


	def create_tags_widget(self, n):

		#
		# QUESTA VA NEL CODICE DELLA REMOTE PER GESTIRE L'AGGIORNAMENTO.
		#

		etype = 'condition'
		eid   = n.uuid

		ets = ElementTag.objects.filter(elementtype=etype, elementid=eid).select_related('tag').all()

		tags = []
		for et in ets:

			path = []
			for path_tag in et.tag.get_tags_path():

				path.append({
					'uuid' : et.uuid,
					'tagid': path_tag.uuid, 
					'name' : path_tag.name,
				})

			entry = {
				'uuid': et.uuid,
				'path': path,
			}

			tags.append(entry)

		rdata = {
			'tags': tags,
		}

		widget_test = {
			'id'       : 0,
			'wtype'    : ["taglist"],

			'tab'      : ' '.join([self.TAB_GENERAL,]),
			'container': self.LAYOUT_INFO,

			'rdata'    : rdata,
		}
		return widget_test
