# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import time

from django.conf import settings

from django.http import HttpResponse
from django.template import RequestContext, loader, Context

from django.contrib.auth.models import User, Group

from sanet_common.sanetdefs.constants import ELEMENT_KIND_NODE, ELEMENT_KIND_IFACE, ELEMENT_KIND_STORAGE, ELEMENT_KIND_SERVICE, ELEMENT_KIND_DEVICE

from core.models import Tenant, TenantGroup
from core.models import PriorityLabel
from core.models import Agent
from core.models import TDataGroup
from core.models import TElement

from core.api.base import icon_select_all
from core.api.base import tenant_get_media_url
from core.api.base import tenant_get_icon_url

from tags.models import TagTree, TagTreeGroupPermission, Tag

from webui.models import *

from webui.api import format_html

from webui.api.constans import USER_TENANT_NODES_DATA_FILTERS

from webui.api.widgets import WIDGET_CONFIG
from webui.api.widgets import CustomDivWidget, DividerWidget, create_label_widget

from webui.api.Pages import BasePage
from webui.api.Pages import PageTab
from webui.api.Pages import VerticalLayout

from webui.api.utils import render_template

from webui.views.objects.tenants import render_tenant_config_details

USER_TENANT_NODES_DATA_FILTERS = 'tenant_nodes_data.filter_settings'

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class Page(BasePage):
	
	TAB_GENERAL        = 'general'
	LAYOUT_GENERAL     = 'L-general'
	                  
	TAB_AGENTS         = 'agents'
	LAYOUT_AGENTS      = 'L-agents'
                      
	TAB_NODES          = 'nodes'
	LAYOUT_NODES       = 'L-nodes'
                      
	TAB_TEMPLATES      = 'templates'
	LAYOUT_TEMPLATES   = 'L-templates'
                      
	TAB_TAGS           = 'tags'
	LAYOUT_TAGS        = 'L-tags'
                      
	TAB_ENTABLES       = 'entables'
	LAYOUT_ENTABLES    = 'L-entables'
                      
	TAB_UTILS          = 'utils'
	LAYOUT_UTILS       = 'L-utils'

	def __init__(self, rid, tenant, global_settings):

		super(Page, self).__init__(rid, tenant, global_settings)

		self.has_nav_bar       = False             # Show navigation bar on the left side

		self.pagetype = 'configtenant'
		
		self.resource = tenant
		
	def get_title(self):
		return self.resource.long_name or self.resource.name

	def userCanAccess(self, request):
		error = super(Page, self).userCanAccess(request)
		if error is not None:
			return error
		
		if 'core.tenant.edit' not in self.user_capabilities:
				context = {
					'required_capabilites': [ 'core.tenant.edit', ]
				}
				#c = RequestContext(request, context)  django 1.11
				t = loader.get_template( 'html/user_not_allowed.html' )
				html_text = t.render(context, request)	 #   django 1.11
				return HttpResponse(html_text)
				
		return None
		
	
	def get_tabs(self):
		
		user = self.global_settings['user']
		
		tabs = []
		
		tabs += super(Page, self).get_tabs()
		
		# define tabs element for page
		tabs +=  [
			#PageTab(**{
			#	'name'      : self.TAB_GENERAL, 
			#	'title'     : 'Global settings',         
			#	'icontype'  : 'fonticon',       
			#	'iconvalue' : 'icon-gear',      
			#	'active_tab': True,             
			#}),

			PageTab(**{
				'name'      : self.TAB_AGENTS,  
				'title'     : 'Agents',         
				'icontype'  : 'fonticon',       
				'iconvalue' : 'icon-sitemap',   
				'active_tab': False,
				'order'     : 100,
			}),
			
			PageTab(**{
				'name'      : self.TAB_NODES,   
				'title'     : 'Nodes',          
				'icontype'  : 'fonticon',       
				'iconvalue' : 'icon-laptop',    
				'active_tab': False,            
				'order'     : 200,
			}),

			PageTab(**{
				'name'      : self.TAB_TAGS,    
				'title'     : 'Tagging',       
				'icontype'  : 'fonticon',      
				'iconvalue' : 'icon-folder-open',   
				'active_tab': False,            
				'order'     : 300,
			}),

			#PageTab(**{
			#	'name'      : self.TAB_TEMPLATES, 
			#	'title'     : 'Templates',        
			#	'icontype'  : 'fonticon',         
			#	'iconvalue' : 'icon-copy',        
			#	'active_tab': False,              
			#}),

			PageTab(**{
				'name'      : self.TAB_ENTABLES,  
				'title'     : 'Entables',         
				'icontype'  : 'fonticon',         
				'iconvalue' : 'icon-envelope',    
				'active_tab': False,              
				'order'     : 400,
			}),
		] 

		tabs += [
			PageTab(**{
				'name'      : self.TAB_UTILS,  # name for tab
				'title'     : 'Tests',  # title human readable
				'icontype'  : 'fonticon', # choose icon: fonticon, image
				'iconvalue' : 'icon-wrench', # icon url, run for icon and image
				'enabled'   : user.is_superuser,
				'order'     : 500,
			})		
		]

		return tabs

	def get_page_layout(self):
		
		#tabs_layout = {}
		
		tabs_layout = super(Page, self).get_page_layout()
		
		tabs_layout[self.TAB_GENERAL]        = VerticalLayout(self.LAYOUT_GENERAL)
		
		tabs_layout[self.TAB_AGENTS]         = VerticalLayout(self.LAYOUT_AGENTS)
		tabs_layout[self.TAB_NODES]          = VerticalLayout(self.LAYOUT_NODES)
		tabs_layout[self.TAB_TAGS]           = VerticalLayout(self.LAYOUT_TAGS)
		#tabs_layout[self.TAB_TEMPLATES]      = VerticalLayout(self.LAYOUT_TEMPLATES)
		tabs_layout[self.TAB_ENTABLES]       = VerticalLayout(self.LAYOUT_ENTABLES)

		#tabs_layout.update( AutoConf.get_layouts(self) )
		#tabs_layout.update( WebScenarios.get_layouts(self) )
		
		tabs_layout[self.TAB_UTILS]          = VerticalLayout(self.LAYOUT_UTILS)
		#tabs_layout[self.TAB_WEBSCENARIO]    = VerticalLayout(self.LAYOUT_WEBSCENARIO)
		
		#ddays added
		#tabs_layout[self.TAB_DDAYS]          = VerticalLayout(self.LAYOUT_DDAYS)
		
		return tabs_layout


	def get_widgets(self):
		
		widgets = []
		
		widgets += super(Page, self).get_widgets()
		
		widgets += self.create_general_section_widgets()
		widgets += self.create_agents_section_widgets()
		widgets += self.create_nodes_section_widgets()
		widgets += self.create_tags_section_widgets()
		#widgets += self.create_template_section_widgets()
		widgets += self.create_entables_section_widgets()

		#widgets += self.create_autoconf_section_widgets()
		
		#widgets += self.get_web_scenario_widgets()
		
		# Utils tabs
		widgets += self.get_text_expr_widgets()
		
		#DDAYS_ADDED
		#widgets += self.get_ddays_widgets()

		return widgets
		
	def create_tags_section_widgets(self):
		
		user = self.global_settings['user']
		
		self.user_capabilities

		widgets = []
		
		tenant = self.resource
		
		groups = Group.objects.all().order_by('name')
		trees  = TagTree.objects.all().order_by('name')
		
		visible_trees = TagTree.get_visible_trees_by_user(tenant, user)
		
		for t in trees:
			
			can_edit = False
			if user.is_superuser:
				can_edit = True
			else:
				if t.owner == user:
					can_edit = True
				#else:
				#	if 'tags.tagtree.edit' in self.user_capabilities:
				#		if t in visible_trees:
				#			can_edit = True
				#		else:
				#			can_edit = False
				#	else:
				#		can_edit = False
					
			t.can_edit = can_edit
			
			perms_by_group_id = dict([ (gp.group.id, gp) for gp in TagTreeGroupPermission.objects.filter(dirtree=t) ])
			
			perms = []
			for g in groups:
				if g.id in perms_by_group_id:
					perms.append((g.id, perms_by_group_id[g.id].auth_type))
				else:
					perms.append((g.id, None))
					
			t.perms = perms
			
		html = render_template( 'html/parts/configtenant_tagging.html', { 
			'trees': trees,
			'groups': groups,
		})

		w = CustomDivWidget(
			id    = "wgroup_tree_perm",
			
			tab       = self.TAB_TAGS,
			container = self.LAYOUT_TAGS,  
			
			rdata = {
				'structure': { 
					'template': html
				}
			}
		)
		widgets.append( w.to_dict() )
		
		return widgets

	def create_general_section_widgets(self):
		
		widgets = []
		
		tenant = self.resource

		#
		# Priority levels
		#
		html = render_template( 'html/parts/configtenant_prioritylevels.html', { 
			'tenant': self.tenant, 
			'levels': PriorityLabel.objects.order_by('min_level').all(),
		})
		
		w = CustomDivWidget(
		
			id        = "wprioritylevels",
			
			tab       = self.TAB_GENERAL,
			container = self.LAYOUT_GENERAL,  
			
			rdata = {
				'structure': { 
					'template': format_html( html)
				}
			}
		)
		widgets.append( w.to_dict() )

		widgets.append( DividerWidget(tab=self.TAB_GENERAL, container=self.LAYOUT_GENERAL).to_dict() )

		w = create_label_widget(
			"Priorities settings",
			tab       = self.TAB_GENERAL,
			container = self.LAYOUT_GENERAL,  
		)
		widgets.append( w )

		#
		# Critical priority form
		#
		html = render_tenant_config_details(tenant)
		
		w = CustomDivWidget(
		
			tab       = self.TAB_GENERAL,
			container = self.LAYOUT_GENERAL,  
			
			rdata     = {
				'structure': { 
					'template': format_html( '<div id="update_settings_container" style="margin-left: 30px">' + html  + '</div>')
				}
			}
		)
		widgets.append( w.to_dict() )

		widgets.append( DividerWidget(tab=self.TAB_GENERAL, container=self.LAYOUT_GENERAL).to_dict() )

		#
		# Tenant icon
		#
		icon_url = tenant_get_media_url(self.tenant.uuid, 'site.png')
		
		html = render_template( 'html/parts/configtenant_upload_icon.html', { 
			'tenant'  : self.tenant, 
			'icon_url': icon_url,
			'ts':time.time()
		})
		
		w = CustomDivWidget(
			id    = "wiconconf",
			tab       = self.TAB_GENERAL,
			container = self.LAYOUT_GENERAL,  
			rdata = {
				'structure': { 
					'template': format_html( html )
				}
			}
		)
		widgets.append( w.to_dict() )

		widgets.append( DividerWidget(tab=self.TAB_GENERAL, container=self.LAYOUT_GENERAL).to_dict() )

		#
		# Icons
		#
		icon_names = icon_select_all(self.tenant.name)
		
		icons = []
		for name in icon_names:
			icons.append({
				'name': name,
				'url' : tenant_get_icon_url(self.tenant.uuid, name + '.png')
			})
		
		html = render_template( 'html/parts/configtenant_icons.html', { 
			'tenant':self.tenant,
			'icons':icons 
		})
		
		w = CustomDivWidget(
			id    = "wiconsconf",
			tab       = self.TAB_GENERAL,
			container = self.LAYOUT_GENERAL,  
			rdata = {
				'structure': { 
					'template': format_html( html )
				}
			}
		)
		widgets.append( w.to_dict() )

		return widgets

	def create_agents_section_widgets(self):
		widgets = []

		w = create_label_widget(
			"Agents",
			tab       = self.TAB_AGENTS,
			container = self.LAYOUT_AGENTS,  
		)
		widgets.append( w )
		
		#
		# Agents list
		#
		tenant = self.tenant
		agents = Agent.objects.filter(tenant=tenant)
		
		html = render_template( 'html/parts/configtenant_agents.html', { 
			'agents': agents,
			'tenant': tenant,
		})
		
		w = CustomDivWidget(
			id        = "wagents",
			
			tab       = self.TAB_AGENTS,
			container = self.LAYOUT_AGENTS,  
			
			rdata = {
				'structure': { 
					'template': format_html(html)
				}
			}
		)
		widgets.append( w.to_dict() )

		return widgets

	def create_nodes_section_widgets(self):

		tenant  = self.global_settings['tenant']
		user   = self.global_settings['user']
		capabilities = user.get_capabilities(tenant)

		widgets = []
		
		html = render_template( 'html/parts/configtenant_shownodes.html', {
			'USER_CAPABILITIES': capabilities
		})
		
		rdata = {
			'url'       : {
				            'data_request' : reverse('tenant_nodes_data'),          # url richiesta dati sui nodi
				            'export_csv'   : reverse('tenant_nodes_export_csv'),    # url esportazione in csv
				            'conf_save'    : reverse('tenant_save_configuration'),  # url salvataggio configurazione tabella
				            'table_data'   : reverse('tenant_table_data')           # url richiesta dei dati per l'inizializzazione della tabella
			              },
			'structure' : {
				            'template': format_html(html)
			              }
		}

		w = WIDGET_CONFIG({
			'id'                 : self.rid + "-shownodes",
			'wtype'              : ["shownodes"],
			                     
			'tab'                : self.TAB_NODES,
			'container'          : self.LAYOUT_NODES,
			                     
			'height'             : 'min',
			'rdata'              : rdata,
			                     
			'rtimer'             : 0,
			'autorefresh_button' : True,
		})
		
		widgets.append(w)

		return widgets


	def create_template_section_widgets(self):
		widgets = []

		w = create_label_widget(
			"Templates",
			tab       = self.TAB_TEMPLATES,
			container = self.LAYOUT_TEMPLATES,  
		)
		widgets.append( w )


		#
		# Datagroup templates
		#
		tenant = self.tenant
		tdgs = TDataGroup.objects.filter(tenant=tenant).order_by('name').all()
		
		html = render_template( 'html/parts/configtenant_tdatagroups.html', { 'tdgs': tdgs })
		
		w = CustomDivWidget(

			id        = "wdatagroups",

			tab       = self.TAB_TEMPLATES,
			container = self.LAYOUT_TEMPLATES,  

			rdata = {
				'structure': { 
					'template': format_html(html)
				}
			}
		)
		widgets.append( w.to_dict() )

		#
		# Element Templates
		#
		tenant = self.tenant
		
		labels = {
			ELEMENT_KIND_NODE   : 'Node', 
			ELEMENT_KIND_IFACE  : 'Interface', 
			ELEMENT_KIND_STORAGE: 'Storage', 
			ELEMENT_KIND_SERVICE: 'Service', 
			ELEMENT_KIND_DEVICE : 'Device',
		}
		
		for kind in [ELEMENT_KIND_NODE, ELEMENT_KIND_IFACE, ELEMENT_KIND_STORAGE, ELEMENT_KIND_SERVICE, ELEMENT_KIND_DEVICE]:
		
			w = create_label_widget(
				"%s templates" % ( labels[kind] ),
				tab       = self.TAB_TEMPLATES,
				container = self.LAYOUT_TEMPLATES,  
			)
			widgets.append( w )
		
			tes = TElement.objects.filter(tenant=tenant, kind=kind).order_by('name').all()
			html = render_template( 'html/parts/configtenant_telements.html', { 'tes': tes })
			w = CustomDivWidget(
				id        = "wtelement",
				tab       = self.TAB_TEMPLATES,
				container = self.LAYOUT_TEMPLATES,  
				rdata = {
					'structure': { 
						'template': format_html(html)
					}
				}
			)
			widgets.append( w.to_dict() )

		return widgets


	def create_entables_section_widgets(self):
		widgets = []

		#
		#
		#
		w = create_label_widget(
			"Entables",
			tab       = self.TAB_ENTABLES,
			container = self.LAYOUT_ENTABLES,  
		)
		widgets.append( w )

		#
		#
		#
		w = CustomDivWidget(
			id         = "entables",

			tab        = self.TAB_ENTABLES,
			container  = self.LAYOUT_ENTABLES,  

			remote     = '%s_%s.entables_config' % (self.resource.resource_type, self.resource.resource_id),
			datasource = '%s_%s.entables_config' % (self.resource.resource_type, self.resource.resource_id),

			rdata = {
				'structure': { 
					'template': "loading..."
				}
			},
		)
		widgets.append( w.to_dict() )
		
		return widgets

	##ddays added
	#def get_ddays_widgets(self):
	#	
	#	widgets = []
	#	
	#	ddays_for_tenant = Dday.get_ddays_for_tenant(self.tenant)
	#				
	#	html = render_template( 'html/parts/dday_table.html', { 'ddays': ddays_for_tenant })
	#	
	#	w = CustomDivWidget(
	#		id    = "wgroup_tree_perm",
	#		
	#		tab       = self.TAB_DDAYS,
	#		container = self.LAYOUT_DDAYS,  
	#		
	#		rdata = {
	#			'structure': { 
	#				'template': format_html(html)
	#			}
	#		}
	#	)
	#	
	#	widgets.append( w.to_dict() )
	#	
	#	return widgets

	def get_tenants_widgets(self):
		
		widgets = []
		
		tenants = Tenant.objects.all()
		
		html = render_template( 'html/parts/configuration_tenants.html', { 'tenants': tenants })
		
		w = CustomDivWidget(
			id        = "wtenants",

			tab       = self.TAB_TENANTS,
			container = self.LAYOUT_TENANTS,  

			rdata     = {
				'structure': { 
					'template': format_html(html)
				}
			}
		)
		widgets.append( w.to_dict() )
		
		return widgets

	def get_usersgroups_widgets(self):
		
		widgets = []
		
		#
		#
		#
		users = User.objects.all().order_by('username')

		html = render_template( 'html/parts/configuration_users.html', { 'users': users })

		w = CustomDivWidget(
			id        = "wusers",
			
			tab       = self.TAB_USERSGROUPS,
			container = self.LAYOUT_USERSGROUPS,  
			
			rdata     = {
				'structure': { 
					'template': format_html(html)
				}
			}
		)
		widgets.append( w.to_dict() )
		
		
		#
		#
		#
		groups = Group.objects.all().order_by('name')

		html = render_template( 'html/parts/configuration_groups.html', { 'groups': groups })

		w = CustomDivWidget(
			id        = "wgroups",
			
			tab       = self.TAB_USERSGROUPS,
			container = self.LAYOUT_USERSGROUPS,  
			
			rdata     = {
				'structure': { 
					'template': format_html(html)
				}
			}
		)
		widgets.append( w.to_dict() )
		


		#
		# MARIX USER, GROUPS
		#
		groups = Group.objects.all().order_by('name')
		
		users = User.objects.all().order_by('username')
		for u in users:
			
			user_groups = set([ g for g in u.groups.all() ])
			
			u.groups_row = []
			for g in groups:
				if g in user_groups:
					u.groups_row.append(g)
				else:
					u.groups_row.append(None)
		

		html = render_template( 'html/parts/configuration_usersgroups.html', { 'groups': groups, 'users': users })

		w = CustomDivWidget(
			id        = "wusersgroups",
			
			tab       = self.TAB_USERSGROUPS,
			container = self.LAYOUT_USERSGROUPS,  
			
			rdata     = {
				'structure': { 
					'template': format_html(html)
				}
			}
		)
		widgets.append( w.to_dict() )
		
		
		
		#
		# MARIX TENANT, GROUPS
		#
		groups = Group.objects.all().order_by('name')
		
		tenants = Tenant.objects.all().order_by('name')
		for t in tenants:
			
			tenant_groups = set([ g.group for g in TenantGroup.objects.filter(tenant=t).all() ])
			
			t.groups_row = []
			for g in groups:
				if g in tenant_groups:
					t.groups_row.append(g)
				else:
					t.groups_row.append(None)
		html = render_template( 'html/parts/configuration_tenantgroups.html', { 'groups': groups, 'tenants': tenants })

		w = CustomDivWidget(
			id        = "wtenantgroups",
			
			tab       = self.TAB_USERSGROUPS,
			container = self.LAYOUT_USERSGROUPS,  
			
			rdata     = {
				'structure': { 
					'template': format_html(html)
				}
			}
		)
		widgets.append( w.to_dict() )		
		
		return widgets
		
	def get_library_widgets(self):
		
		widgets = []

		w = CustomDivWidget(
			id        = "wlibrary",
			
			tab       = self.TAB_LIBRARY,
			container = self.LAYOUT_LIBRARY,  
			
			rdata     = {
				'structure': { 
					'template': "LIBRARY"
				}
			}
		)
		widgets.append( w.to_dict() )
		
		return widgets
		
		
	def get_text_expr_widgets(self):
		#n = self.element
		
		widgets = []

		elements = MonitoredElement.objects.select_related('parent').order_by('parent__name', 'kind', 'name').all()
		parameters = '\n'.join([
			 '$node=localhost'
			,'$community=public'
			,'$snmpversion=1'
			,'$snmpport=161'
		])
		expr       = '1.3.6.1.2.1.1.1.0@'
				
		c = {
			'elements'  : elements,
			'parameters': parameters,
			'expr'      : expr,
			
		}
		html = render_template('html/parts/test_expr.html', c )

		widget_logo = CustomDivWidget(
		
			tab       = self.TAB_UTILS,  
			container = self.LAYOUT_UTILS ,
			
			rdata = {
				'structure': { 'template': html }
			}
		)
		widgets.append( widget_logo.to_dict() )
		
		return widgets	
		
		
	#def get_web_scenario_widgets(self):
	#	widgets = []
        #
	#	widgets += [ create_label_widget('Web Scenario', tab=self.TAB_WEBSCENARIO, container=self.LAYOUT_WEBSCENARIO) ]
	#	
	#	w = WIDGET_CONFIG({
	#		'id'        : self.rid + "-webscenario",
        #
	#		'tab'       : self.TAB_WEBSCENARIO,
	#		'container' : self.LAYOUT_WEBSCENARIO,
	#		
	#		'wtype'     : ["webscenarios"],
	#		
	#		'remote'    : '%s_%s.webscenarios' % (self.resource.resource_type, self.resource.resource_id),
	#		'rtimer'    : settings.WEBUI_REFRESH_TIMERS['webscenario_list'],
	#		
	#		'autorefresh_button': True,
	#	})
	#	
	#	widgets += [ w ]
	#	
	#	return widgets

	#def render(self):
	#
	#	tabs   = self.get_tabs()
	#	
	#	layout = self.get_page_layout()
	#	
	#	widgets = self.get_widgets()
	#
	#	return {
	#		'name'   : self.pagetype,   
	#
	#		'tabs'   : tabs,
	#		'layout' : layout,
	#
	#		'widgets': widgets,
	#	}


