# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from django.contrib.auth.models import User, Group

from core.models import Tenant

from webui.api.Pages import BasePage

from webui.api import format_html

from webui.api.Pages import BasePage
from webui.api.Pages import PageTab
from webui.api.Pages import VerticalLayout

from webui.api.widgets import CustomDivWidget, create_label_widget

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class Page(BasePage):

	PAGE_HTML_TEMPLATE = 'html/index_configuration.html'

	
	TAB_TENANTS        = 'tenants'
	LAYOUT_TENANTS     = 'L-tenants'
	
	TAB_USERSGROUPS    = 'usersgroups'
	TAB_USERSGROUPS    = 'general'
	LAYOUT_USERSGROUPS = 'L-usersgroups'
	
	TAB_LIBRARY        = 'library'
	LAYOUT_LIBRARY     = 'L-library'
	
	
	

	def __init__(self, rid, tenant, global_settings):

		super(Page, self).__init__(rid, tenant, global_settings)

		self.pagetype = 'configuration'
		
		self.admin_required = True
		
		self.has_nav_bar = False
		
		self.has_search  = False
		
		# define tabs element for page
		self.tabs = [
			
			#{
			#	'name'      : self.TAB_TENANTS,
			#	'title'     : 'Tenants',       
			#	'icontype'  : 'fonticon',      
			#	'iconvalue' : 'icon-home',     
			#	'active_tab': False,           
			#},
			
			PageTab(**{
				'name'      : self.TAB_USERSGROUPS,
				'title'     : 'Users & Groups', 
				'icontype'  : 'fonticon',       
				'iconvalue' : 'icon-group',     
				'active_tab': True,             
			}),

			#{
			#	'name'      : self.TAB_LIBRARY,
			#	'title'     : 'Library',       
			#	'icontype'  : 'fonticon',      
			#	'iconvalue' : 'icon-copy',     
			#	'active_tab': False,           
			#}
		]

	def get_title(self):
		return "Global configuration"

	def get_tabs(self):
		return self.tabs

	def get_page_layout(self):
		
		tabs_layout = {}
		
		#tabs_layout[self.TAB_TENANTS]     = VerticalLayout(self.LAYOUT_TENANTS)
		                                  
		tabs_layout[self.TAB_USERSGROUPS] = VerticalLayout(self.LAYOUT_USERSGROUPS)
		                                  
		#tabs_layout[self.TAB_LIBRARY]     = VerticalLayout(self.LAYOUT_LIBRARY)
		
		return tabs_layout


	def getAdditionalRenderSettings(self, request):
		return {
			"show_tab_title": True
		}

	def get_widgets(self):
		
		widgets = []
		
		widgets += self.get_tenants_widgets()
		
		widgets += self.get_usersgroups_widgets()
		
		#widgets += self.get_library_widgets()
		
		return widgets
		
		
	def get_tenants_widgets(self):

		tab       = self.TAB_TENANTS,
		container = self.LAYOUT_TENANTS,  

		widgets = []
		
		widgets.append( create_label_widget("Tenants", tab=tab, container=container) )
		
		w = CustomDivWidget(
			id        = "wtenants",
			
			tab       = tab, 
			container = container,
			
			remote    = "configuration_1.%s" % ('configuration_tenants'),
			dataset   = "configuration_1.%s" % ('configuration_tenants'),
			
			rdata = {
				'structure': { 
					'template': "loading...",
				}
			},
		)
		widgets.append( w.to_dict() )
		
		return widgets

	def get_usersgroups_widgets(self):

		tab       = self.TAB_USERSGROUPS,
		container = self.LAYOUT_USERSGROUPS,  

		widgets = []

		#
		#
		#
		widgets.append( create_label_widget("Users", tab=tab, container=container) )
		
		w = CustomDivWidget(
			id        = "wusers",
			
			tab       = tab, 
			container = container,  
			
			remote    = "configuration_1.%s" % ('configuration_users'),
			dataset   = "configuration_1.%s" % ('configuration_users'),

			rdata     = {
				'structure': { 
					'template': "loading...",
				}
			},
		)
		widgets.append( w.to_dict() )

		#
		#
		#
		widgets.append( create_label_widget("Groups", tab=tab, container=container) )

		w = CustomDivWidget(
			id        = "wgroups",
			
			tab       = tab, 
			container = container,  
			
			remote    = "configuration_1.%s" % ('configuration_groups'),
			dataset   = "configuration_1.%s" % ('configuration_groups'),
			
			rdata     = {
				'structure': { 
					'template': "loading...",
				}
			},
		)
		widgets.append( w.to_dict() )
		

		#
		# MARIX USER, GROUPS
		#
		widgets.append( create_label_widget("Users / Groups", tab=tab, container=container) )

		w = CustomDivWidget(
			id        = "wusersgroups",
			
			tab       = tab, 
			container = container,  
			
			remote    = "configuration_1.%s" % ('configuration_usersgroups'),
			dataset   = "configuration_1.%s" % ('configuration_usersgroups'),
			
			rdata     = {
				'structure': { 
					'template': "loading...", #format_html(html)
				}
			},
		)
		widgets.append( w.to_dict() )
		
		
		
		#
		# MARIX TENANT, GROUPS
		#
		widgets.append( create_label_widget("Tenants / Groups", tab=tab, container=container) )

		w = CustomDivWidget(
			id        = "wtenantsgroups",
			
			tab       = tab, 
			container = container,  

			remote    = "configuration_1.%s" % ('configuration_tenantsgroups'),
			dataset   = "configuration_1.%s" % ('configuration_tenantsgroups'),
			
			rdata     = {
				'structure': { 
					'template': "loading...",
				}
			},
		)
		widgets.append( w.to_dict() )
		
		
		#
		# Capabilities
		#
		widgets.append( create_label_widget("Groups capabilites", tab=tab, container=container) )

		w = CustomDivWidget(
			id        = "wcapabilities",

			tab       = tab, 
			container = container,  

			remote    = "configuration_1.%s" % ('configuration_capabilities'),
			dataset   = "configuration_1.%s" % ('configuration_capabilities'),

			rdata     = {
				'structure': { 
					'template': "loading...", #format_html(html)
				}
			},
		)
		widgets.append( w.to_dict() )
		
		return widgets

	def get_library_widgets(self):
		
		widgets = []

		w = CustomDivWidget(
			id        = "wlibrary",
			
			tab       = self.TAB_LIBRARY,
			container = self.LAYOUT_LIBRARY,  
			
			rdata = {
				'structure': { 
					'template': "LIBRARY"
				}
			}
		)
		widgets.append( w.to_dict() )				
		
		return widgets

	def render(self):

		layout = self.get_page_layout()
		
		widgets = self.get_widgets()

		return {
			'name'   : self.pagetype,   
			'layout' : layout,
			'widgets': widgets,
			'tabs'   : self.tabs,
		}
