# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from sanet_common.sanetdefs.constants import *

from core.models import DataGroup 

from core.api.cache import init_cache_manager_from_conf

from webui.models import *

from webui.api import format_html 

from webui.api.widgets import CustomDivWidget 
from webui.api.widgets import DividerWidget
from webui.api.widgets import create_label_widget

from webui.api.widgets import DataSourceGraphWidget, ConditionGraphWidget, TimeGraph3Widget

from webui.api.Pages import PageTab
from webui.api.Pages import VerticalLayout, HorizontalLayout

from webui.api.Pages import BasePage
from webui.api.Pages import LogPage

from webui.api.utils import render_template

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

from core.models import Condition

class Page(BasePage, LogPage):

	_element_class = DataGroup

	TAB_GENERAL         = 'general'
	LAYOUT_INFO         = 'L-info' 
	
	LAYOUT_DATASOURCES   = 'L-datasources'
	LAYOUT_CONDITIONS    = 'L-conditions'
	LAYOUT_TIMEGRAPHS    = 'L-timegraph'

	
	TAB_LOG          = 'log'
	LAYOUT_LOG       = 'L-log'
	

	def __init__(self, rid, tenant, global_settings):
		
		super(Page, self).__init__(rid, tenant, global_settings)

		self.pagetype = 'datagroup'
		
		user = global_settings.get('user')
		
		# Carichiamo l'oggetto giusto per questa pagina
		self.element = self._element_class.objects.get(uuid=self.rid)	
		self.resource = self.element
	
	def get_title(self):
		return self.resource.title

	
	def get_tabs(self):
		
		tabs = [
			PageTab(**{
				'name'      : self.TAB_GENERAL,       # name for tab
				'title'     : 'Info',       # title human readable
				'icontype'  : 'fonticon',      # choose icon: fonticon, image
				'iconvalue' : 'icon-list-alt', # icon url, run for icon and image
				'active_tab': True,            # active tab information
			})
		]		
		
		
		tabs.extend( LogPage.get_tab_def(self) )

		return tabs



	def get_layouts(self):

		tabs_layout = {}

		tabs_layout[self.TAB_GENERAL ] = VerticalLayout('L1', [
							HorizontalLayout(self.LAYOUT_INFO),
							HorizontalLayout(self.LAYOUT_DATASOURCES),
							HorizontalLayout(self.LAYOUT_CONDITIONS),
							HorizontalLayout(self.LAYOUT_TIMEGRAPHS),
		])		

		tabs_layout.update( LogPage.get_layouts(self) )


		return tabs_layout

	#-------------------------------------------------------------------------------#

	def get_log_tab_layout(self):
		return self.TAB_LOG, self.LAYOUT_LOG

	#def get_log_tab_layout(self):
	#	return self.TAB_GENERAL, self.LAYOUT_INFO

	#-------------------------------------------------------------------------------#

	def render(self):

		tabs    = self.get_tabs()
		
		layout  = self.get_layouts()
		
		widgets = self.get_widgets()

		page_struct =  {
			'name'   : self.pagetype,   
			'layout' : layout,
			'widgets': widgets,
			'tabs'   : tabs,
		}

		return page_struct


	def get_widgets(self):

		widgets = []

		datagroup = self.element
		
		cache_manager = init_cache_manager_from_conf()
		
	
		datagroup.load_fields_from_cache(cache_manager)
	

		#
		# DETAILS
		#

		state = datagroup.get_state()
		icon = datagroup.get_icon_url_with_state(state)

		details = []

		details.append({ 'name': 'Title'         , 'value': datagroup.title          })
		details.append({ 'name': 'Min period'    , 'value': datagroup.minperiod      })
		details.append({ 'name': 'Lastdone'      , 'value': datagroup.lastdone       })
		details.append({ 'name': 'shorttries'    , 'value': datagroup.shorttries     })
		details.append({ 'name': 'timeout'       , 'value': datagroup.timeout        })

		details.append({ 'name': 'Element'       , 'value': datagroup.element        })

		details.append({ 'name': 'Classification', 'value': datagroup.classification })
		details.append({ 'name': 'Passive match' , 'value': datagroup.passive_match  })

		c = {
			'icon_url'         : icon,
			'element'          : datagroup,
			'state'            : state,
			'details'          : details,
		}

		html = render_template( 'html/parts/datagroup_page_details.html', c )

		w = CustomDivWidget(
			tab       = self.TAB_GENERAL,
			container = self.LAYOUT_INFO,
			rdata = {
				'structure': {
					'template': format_html( html  )
				}
			}
		).to_dict()
		
		widgets.append(w)

		#
		# Objects
		#

		#
		# Datasources
		#
		
		w = create_label_widget("Datasources", self.TAB_GENERAL, self.LAYOUT_DATASOURCES)
		widgets.append(w)

		datasources = datagroup.datasources.filter(save_value=True).order_by('title').all()
		
		for ds in datasources:
			widgets.append(DividerWidget(tab=self.TAB_GENERAL, container=self.LAYOUT_DATASOURCES).to_dict() )

			w = DataSourceGraphWidget(ds, tab=self.TAB_GENERAL, container=self.LAYOUT_DATASOURCES).to_dict()
			widgets.append(w)

		#
		# COnditions
		#
		conditions  = datagroup.conditions.order_by('title').all()

		w = create_label_widget("Conditions", self.TAB_GENERAL, self.LAYOUT_CONDITIONS)
		widgets.append(w)

		for c in conditions:
			widgets.append(DividerWidget(tab=self.TAB_GENERAL, container=self.LAYOUT_CONDITIONS).to_dict() )

			w = ConditionGraphWidget(c, tab=self.TAB_GENERAL, container=self.LAYOUT_CONDITIONS).to_dict()
			widgets.append(w)

		#
		# Timegraphs
		#
		timegraphs  = datagroup.timegraphs.order_by('title').all()

		w = create_label_widget("Timegraphs", self.TAB_GENERAL, self.LAYOUT_TIMEGRAPHS)
		widgets.append(w)
		
		for t in timegraphs:
			widgets.append(DividerWidget(tab=self.TAB_GENERAL, container=self.LAYOUT_TIMEGRAPHS).to_dict() )

			w = TimeGraph3Widget(t, tab=self.TAB_GENERAL, container=self.LAYOUT_TIMEGRAPHS).to_dict()
			widgets.append(w)		
		
		widgets += self.get_log_widgets()
		

		return widgets


