# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from core.api.cache import init_cache_manager_from_conf
from core.models import DataSource

from webui.models import *

from webui.api import format_html 

from webui.api.Pages import PageTab
from webui.api.Pages import VerticalLayout, HorizontalLayout

from webui.api.Pages import BasePage

from webui.api.utils import render_template

from webui.api.widgets import CustomDivWidget 
from webui.api.widgets import DataSourceGraphWidget
from webui.api.widgets import create_label_widget

from sanet_common.timeutils import timestamp_to_isostr

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#


class Page(BasePage):

	_element_class = DataSource

	TAB_GENERAL         = 'general'
	
	LAYOUT_INFO         = 'L-info' 
	LAYOUT_GRAPH        = 'L-graph'
	LAYOUT_LOGS         = 'L-logs'

	def __init__(self, rid, tenant, global_settings):
		
		super(Page, self).__init__(rid, tenant, global_settings)
		
		user = global_settings.get('user')
		
		# Carichiamo l'oggetto giusto per questa pagina
		self.element = self._element_class.objects.get(uuid=self.rid)
		
		self.resource = self.element
		
		self.tabs = [
			PageTab(**{
				'name'      : self.TAB_GENERAL,       # name for tab
				'title'     : 'Global',       # title human readable
				'icontype'  : 'fonticon',      # choose icon: fonticon, image
				'iconvalue' : 'icon-list-alt', # icon url, run for icon and image
				'active_tab': True,            # active tab information
			})
		]		

	def get_title(self):
		return self.resource.title

	def get_page_layout(self):

		tabs_layout = {}

		tabs_layout[self.TAB_GENERAL ] = VerticalLayout("L1", [
			HorizontalLayout(self.LAYOUT_INFO),
			HorizontalLayout(self.LAYOUT_GRAPH),
		])

		return tabs_layout

	#-------------------------------------------------------------------------------#


	def render(self):

		tabs    = self.get_tabs()
		layout  = self.get_page_layout()
		
		widgets = self.get_widgets()

		page_struct =  {
			'name'   : self.pagetype,   
			'layout' : layout,
			'widgets': widgets,
			'tabs'   : tabs,
		}

		return page_struct


	def get_widgets(self):

		widgets = []

		widgets += self.get_details_widget()

		widgets += self.get_graph()

		return widgets

	def get_graph(self):

		widgets = []

		c = self.element

		w = create_label_widget("Values", self.TAB_GENERAL, self.LAYOUT_GRAPH)
		widgets.append(w)

		w = DataSourceGraphWidget(c, tab=self.TAB_GENERAL, container=self.LAYOUT_GRAPH).to_dict()
		widgets.append(w)

		return widgets

	def get_details_widget(self):

		datasource = self.element

		cache_manager = init_cache_manager_from_conf()
		datasource.load_fields_from_cache(cache_manager)

		widgets = []

		element = datasource
		
		#
		#
		#
		details = []
		
		details.append({ 'name': 'Title'         , 'value': element.title          })
		
		details.append({ 'name': 'Element'       , 'value': element.element        })
		details.append({ 'name': 'Data Group'    , 'value': element.datagroup      })
		details.append({})
		details.append({ 'name': 'Saved on DB'   , 'value': element.save_value     })
		details.append({ 'name': 'Type'          , 'value': "GAUGE" if element.gauge else "COUNTER" })
		details.append({ 'name': 'Min Value'     , 'value': element.min_val        })
		details.append({ 'name': 'Max Value'     , 'value': element.max_val        })
		details.append({ 'name': 'Expression'    , 'value': element.expr           })
		details.append({})
		details.append({ 'name': 'Last done'     , 'value': timestamp_to_isostr(element.lastdone) })
		details.append({ 'name': 'Min period'    , 'value': element.datagroup.minperiod })
		details.append({})
		details.append({ 'name': 'Status'        , 'value': element.get_outcome()  })
		details.append({ 'name': 'Current value' , 'value': element.lastvalue      })
		details.append({ 'name': 'Verbose status', 'value': element.verbstatus     })
		details.append({})
		details.append({ 'name': 'Order'         , 'value': element.order          })
		details.append({ 'name': 'Cascade'       , 'value': element.cascade        })
		details.append({ 'name': 'Classification', 'value': element.classification })


		state = datasource.get_outcome()

		icon_url = datasource.get_icon_url_with_state(state)

		c = {
			'icon_url'         : icon_url,
			'element'          : datasource,
			'state'            : 'OK',
			'details'          : details,
		}		

		html = render_template( 'html/parts/datasource_page_details.html', c )
		
		
		w = CustomDivWidget(
			tab       = self.TAB_GENERAL,
			container = self.LAYOUT_INFO,

			rdata = {
				'structure': {
					'template': format_html( html  )
				}
			}
		).to_dict()
		
		widgets.append(w)
		
		return widgets
		



