# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from core.models import MonitoredDevice

from webui.api.Pages import LogPage

from webui.api.Pages.element import ElementPage

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class Page(ElementPage, LogPage):

	TAB_LOG          = 'log'
	LAYOUT_LOG       = 'L-log'

	_element_class = MonitoredDevice

	def __init__(self, rid, tenant, global_settings):

		super(Page, self).__init__(rid, tenant, global_settings)

		self.pagetype = 'device'

	def get_page_layout(self):
		layout = {}
		layout.update( super(Page, self).get_page_layout() )
		layout.update( LogPage.get_layouts(self) )
		return layout

	def get_widgets(self):
		widgets = []
		widgets += super(Page, self).get_widgets()
		widgets += self.get_log_widgets()
		return widgets

	def get_info_details(self, n):
		details = []
		details.append({ 'name': 'Priority'   , 'value': n.priority_level_descr })
		details.append({ 'name': 'Description', 'value': n.description })
		return details

	def get_advanced_details(self, n):

		templates_str = ','.join([t.name for t in n.get_used_templates()])

		details = []
		details.append({ 'name': 'Templates'           , 'value': templates_str   })
		details.append({ 'name': 'SANET Distinguisher' , 'value': n.distinguisher })
		details.append({ 'name': 'SANET XForm'         , 'value': n.xform         })
		details.append({ 'name': 'SANET ifIndex'       , 'value': n.devindex      })

		#
		# OTHER DETAILS
		#
		details += self.get_advanced_details_from_extensions(n)

		return details
