# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings
from django.utils.encoding import smart_unicode

from core.models import MonitoredNode

from webui.models import *

from webui.api import *

from webui.api.widgets import WIDGET_CONFIG
from webui.api.widgets import CustomDivWidget, create_label_widget

from webui.api.widgets import widget_resource_section_title

from webui.api.Pages import PageTab
from webui.api.Pages import VerticalLayout

from webui.api.Pages.element import ElementPage

from webui.api.Pages import TroubleshootPage

from webui.api.Pages import LogPage

from webui.api.utils import render_template

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class Page(ElementPage, TroubleshootPage, LogPage):


	TAB_LOG          = 'log'
	LAYOUT_LOG       = 'L-log'

	_element_class = MonitoredNode

	def __init__(self, rid, tenant, global_settings):

		super(Page, self).__init__(rid, tenant, global_settings)

		self.pagetype = 'node'

		# adjacency tab only for node

	def get_tabs(self):

		tabs = super(ElementPage, self).get_tabs()

		tab_enabled = 'core.element.conf' in self.user_capabilities
		
		tabs += [
		]

		tabs.extend( LogPage.get_tab_def(self) )

		tabs.extend([PageTab(**{
				'name'      : self.TAB_CONFIG,
				'title'     : 'Configuration',
				'icontype'  : 'fonticon',
				'iconvalue' : 'icon-cog',
				'active_tab': False,     
				'enabled'   : tab_enabled,
				'order'     : 20000,
			}),
		])

		return tabs

	def get_page_layout(self):

		layout = {}

		layout.update( super(Page, self).get_page_layout() )


		layout.update( LogPage.get_layouts(self) )

		return layout

	def get_additional_jsfile_urls(self):
		urls = super(Page, self).get_additional_jsfile_urls()
		return urls

	def get_widgets(self):

		widgets = []

		widgets += super(Page, self).get_widgets()

		# Widget specifici del nodo




		widgets += self.get_log_widgets()

		return widgets

	def get_info_details(self, n):

		details = super(Page, self).get_info_details(n)

		#
		# Be sure to fetch some informations (SNMP Descr) from the cache
		#
		n.set_cache_manager( self.cache_manager )

		#
		# Details
		#
		details.append({ 'name': 'Description'            , 'value': n.description })

		e_name, e_ip = n.effective_name_and_address
		details.append({ 
			'name' : 'Effective Hostname/IP', 
			'value': e_ip if (e_name == e_ip) else ("%s / %s" % (e_name, e_ip if e_ip else '*unknown*'))
		})

		details.append({ 'name': 'Priority'               , 'value': n.priority_level_descr })
		details.append({ 'name': 'SNMP Desc'              , 'value': n.get_sysdescr()})

		return details

	def get_advanced_details(self, n):

		user   = self.request_user
		tenant = self.tenant

		# Normal user can view "sensibile data" only with the capability. #545
		if user is not None:
			if 'gui.view_sensible_data' not in user.get_capabilities(tenant):
				n.hide_sensible_data()

		details = []

		#
		# LINKS
		#
		adjacent_list = []
		for i, link, l_i, l_n in n.get_links_path():
			v = [ i, l_n, l_i ]
			adjacent_list.append( v )

		#
		# IPS
		#
		all_ips = n.ipv4_addresses

		#
		# Templates
		#
		templates_str = ','.join([t.name for t in n.get_used_templates()])

		#
		# OUTPUT
		#
		sysname =  smart_unicode(n.get_sysname(), errors='replace')

		details.append({ 'name': 'Agent'         , 'value': n.agent })
		details.append({ 'name': 'Adjacency'     , 'value': adjacent_list, 'type': 'adjlist' })

		details.append({ 'name': 'Cluster'              , 'value': n.cluster })
		details.append({ 'name': 'Cluster role'         , 'value': n.cluster_role_descr })
		details.append({ 'name': 'Cluster distinguisher', 'value': n.cluster_distinguisher })

		details.append({ 'name': 'Sanet DNS IPs' , 'value': (','.join(all_ips) if (all_ips is not None) else None) })

		details.append({ 'name': 'Templates'     , 'value': templates_str })

		details.append({ 'name': 'SNMP'          , 'value': 'Version: %s, Community: %s' % (n.snmp_version, n.snmp_community if n.snmp_version != 3 else '*HIDDEN DATA*') })
		details.append({ 'name': 'SNMP Name'     , 'value': sysname })
		details.append({ 'name': 'SNMP Location' , 'value': n.get_syslocation() })

		#
		# OTHER DETAILS
		#
		details += self.get_advanced_details_from_extensions(n)

		return details

