# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from core.models import MonitoredStorage

from webui.api.Pages import LogPage
from webui.api.Pages.element import ElementPage

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class Page(ElementPage, LogPage):
	
	TAB_LOG          = 'log'
	LAYOUT_LOG       = 'L-log'

	_element_class = MonitoredStorage

	def __init__(self, rid, tenant, global_settings):

		super(Page, self).__init__(rid, tenant, global_settings)

		self.pagetype = 'storage'

	def get_page_layout(self):
		layout = {}
		layout.update( super(Page, self).get_page_layout() )
		layout.update( LogPage.get_layouts(self) )
		return layout

	def get_widgets(self):
		widgets = []
		widgets += super(Page, self).get_widgets()
		widgets += self.get_log_widgets()
		return widgets

	def get_info_details(self, n):
		details = []
		details.append({ 'name': 'Priority'             , 'value': n.priority_level_descr })
		details.append({ 'name': 'Description', 'value': n.description })
		return details

	def get_advanced_details(self, n):

		templates_str = ','.join([t.name for t in n.get_used_templates()])

		details = []

		details += self.get_occupation_details(n)

		details.append({ 'name': 'Templates'            , 'value': templates_str })

		details.append({ 'name': 'SANET Distinguisher'  , 'value': n.distinguisher })
		details.append({ 'name': 'SANET XForm'          , 'value': n.xform })
		details.append({ 'name': 'SANET ifIndex'        , 'value': n.stindex })

		details += self.get_snmp_info(n)

		#
		# OTHER DETAILS
		#
		details += self.get_advanced_details_from_extensions(n)

		return details

	def get_occupation_details(self, n):

		# Estraiamo i rate di traffico IN/OUT dal rate salvato in cache (calcolato dall'agente)
		# NOTA 1: potremmo andare a leggere i file RRD, ma e' piu' lento.
		# NOTA 2: i valori letti dalla cache non sono uguali a quelli restituiti dai file RRD
		STORAGE_INSTALLED = 'system.storage.installed'
		STORAGE_USED      = 'system.storage.used'

		rate_by_classification = {}
		for ds in n.monitored_datasources.filter(classification__in=[STORAGE_INSTALLED, STORAGE_USED]).all():
			ds.load_fields_from_cache(self.cache_manager)
			rate_by_classification[ ds.classification ] = (ds.lastvalue, ds.get_scaled_last_value())

		#
		# Format visible entry
		#
		installed = rate_by_classification.get(STORAGE_INSTALLED, None)
		used      = rate_by_classification.get(STORAGE_USED, None)
		if installed and used and installed[0] is not None and used[0] is not None:
			value = "%.2f%% - %sB / %sB" % (float(used[0])/float(installed[0]) * 100, used[1], installed[1])
		else:
			value = '*UNKNOWN*'

		details = []
		details.append({ 'name': 'Occupation' , 'value': value, 'hint': "The traffic rate shown here is only estimated" })

		return details

	def get_snmp_info(self, i):

		from ipmon.discovery.protocols.snmp import SnmpProtocolHandler
		from ipmon.discovery.helpers.snmp import SnmpHelper

		details = []

		node = i.get_node()

		try:
			snmp = SnmpProtocolHandler(node.name)
			snmp.set_supported_version(node.snmp_version, node.snmp_community)
			helper = SnmpHelper(snmp)
			entry_data = helper.get_single_storage_entry( str(i.stindex) )
			entry_data = dict([ ("SNMP %s" % (k), v) for k,v in entry_data.items() ])
			for k, v in sorted(entry_data.items()):
				details.append({ 'name': k              , 'value': v })

		except Exception, ex:
			details.append({'name': 'SNMP Info', 'value': 'ERROR:%s' % (ex)})

		return details

