# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import time

from webui.api import *

from webui.api.Pages import BasePage

from webui.api.Pages import PageTab
from webui.api.Pages import VerticalLayout

from webui.api.widgets import CustomDivWidget, create_label_widget


#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class Page(BasePage):

	PAGE_HTML_TEMPLATE = 'html/index_systemstats.html'

	TAB_GENERAL = 'general'
	LAYOUT_GENERAL = 'general'
    
	def __init__(self, rid, tenant, global_settings):

		super(Page, self).__init__(rid, tenant, global_settings)

		self.pagetype = 'systemstats'
				
		self.has_nav_bar = False
		self.has_search  = False
		
	def get_title(self):
		return "System Status"
		

	def get_tabs(self):

		return [
			PageTab(**{
				'name'      : self.TAB_GENERAL,       # name for tab
				'title'     : 'Global status',       # title human readable
				'icontype'  : 'fonticon',      # choose icon: fonticon, image
				'iconvalue' : 'icon-home', # icon url, run for icon and image
				'active_tab': True,            # active tab information
			}),
		]

	def get_page_layout(self):
		
		layout = {
			self.LAYOUT_GENERAL : 
				VerticalLayout('L0',  items=[
					VerticalLayout('L1', items=[]),
				]),
				
		}
		
		return layout
		
	def get_widgets(self):
		
		widgets = []

		#
		# Sanetd
		#
		widgets += self.create_sanetd_section_widgets()


		w = create_label_widget( "&nbsp;", tab="general", container="L1", htmltag='h3')
		widgets.append( w )

		#
		# Entables
		#
		widgets += self.create_entables_section_widgets()


		w = create_label_widget( "&nbsp;", tab="general", container="L1", htmltag='h3')
		widgets.append( w )

		#
		# Agents
		#
		widgets += self.create_agents_section_widgets()



		
		return widgets

	def create_sanetd_section_widgets(self):
		widgets = []

		w = create_label_widget( "Central Server's status", tab="general", container="L1", )
		widgets.append( w )


		w = CustomDivWidget(
			id        = "sanetd",
			
			tab       = "general",
			container = "L1",  
			
			rdata = {
				'structure': { 
					'template': "loading..."
				}
			},			
			remote     = '%s_%s.stats_sanetd' % ('systemstats', '1'),
		)
		widgets.append( w.to_dict() )
		

		return widgets

	def create_agents_section_widgets(self):
		widgets = []

		w = create_label_widget( "Agents' status", tab="general", container="L1", )
		widgets.append( w )
		
		w = CustomDivWidget(
			id        = "agents",
			
			tab       = "general",
			container = "L1",  
			
			rdata = {
				'structure': { 
					'template': "loading..."
				}
			},
			
			remote     = '%s_%s.stats_agents' % ('systemstats', '1'),
		)
		widgets.append( w.to_dict() )
		
		return widgets


	def create_entables_section_widgets(self):
		widgets = []

		w = create_label_widget( "Entables's status", tab="general", container="L1", )
		widgets.append( w )


		w = CustomDivWidget(
			id        = "entables",
			
			tab       = "general",
			container = "L1",  
			
			rdata = {
				'structure': { 
					'template': "loading..."
				}
			},			
			remote     = '%s_%s.stats_entables' % ('systemstats', '1'),
		)
		widgets.append( w.to_dict() )		
		

		return widgets




	def get_widgets2(self):
		
		widgets = []
		
		title = CustomDivWidget(
			id        = "title",
			tab       = 'general',
			container = 'L1',  
			rdata = {
				'structure': { 'template': format_html( '''<h1>System stats</h2>''') }
			}
		)
		
		widgets.append(title.to_dict())

		valid_tabs = ['general']
		container = 'L1'

		
		widgets.extend( [
				create_label_widget('System informations', 'general', container),

				create_label_widget('&nbsp;', 'general', container),

				{
					'id'     : str(self.tenant.uuid) + "tenant_monitoring_stats_id",
					'wtype'  : ["ajaxtable"],
					'wparams': { 
						'autosize':False,
						'hidefilters':True,
					},
					'tab'    : valid_tabs,
					'container': container,
					'wconf'  : None,
					'width'  : 'max',
					#'height' : 'min',
					'setup'  : None,

					'remote' : str(self.tenant.uuid) + '.tenant_monitoring_stats',
					'dataset': str(self.tenant.uuid) + '.tenant_monitoring_stats_dataset'
				},

				create_label_widget('&nbsp;', 'general', container),

				create_label_widget('Notify system', 'general', container),

				{
					'id'     : str(self.tenant.uuid) + "entables_monitoring_stats_id",
					'wtype'  : ["ajaxtable"],
					'wparams': { 
						'autosize':False,
						'hidefilters':True,
					},
					'tab'    : valid_tabs,
					'container': container,
					'wconf'  : None,
					'width'  : 'max',
					#'height' : 'min',
					'setup'  : None,

					'remote' : str(self.tenant.uuid) + '.entables_monitoring_stats',
					'dataset': str(self.tenant.uuid) + '.entables_monitoring_stats_dataset'
				}   
			]
		)
		

		return widgets




