# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os
import time

from sanet_common.sanetdefs.constants import CSTATE_TO_CODENAME, CSTATE_UU, CSTATE_DN

from django.conf import settings

from webui.api import format_html

from tags.models import Tag
from tags.server.tagsd.api import get_tags_state_safe

from webui.api.constans import ICON_WARNING
from webui.api.constans import ICON_NOSTATE
from webui.api.constans import ICON_OLD

from webui.api.widgets import CustomDivWidget
from webui.api.widgets import FrameWidget
from webui.api.widgets import widget_resource_section_title

from webui.api.utils import render_template

from webui.api.Pages import BasePage
from webui.api.Pages import PageTab
from webui.api.Pages import VerticalLayout, HorizontalLayout
from webui.api.Pages import AlarmsCountPage
from webui.api.Pages import TroubleshootPage
from webui.api.Pages import LogPage

#
# Load data for a first quick view 
#
from webui.api.Data.badgeview import create_badgeview_object
from webui.api.Data.badgeview import create_element_sections


#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

DEBUG_TIME=0

class Page(BasePage, AlarmsCountPage, TroubleshootPage, LogPage):

	LAYOUT_INFO    = 'L-info' 
	LAYOUT_ALARMS  = 'L-alarms'
	LAYOUT_MONITOR = 'L1-monitoringarea'
	LAYOUT_TROUBLE = 'L-troubleshootarea'

	TAB_LOG    = 'log'
	LAYOUT_LOG = 'L-log'

	def __init__(self, rid, tenant, global_settings):

		super(Page, self).__init__(rid, tenant, global_settings)

		self.pagetype = 'tag'

		user = global_settings.get('user')

		self.tag = Tag.objects.get(tenant=tenant, uuid=rid)

		self.element  = self.tag
		self.resource = self.tag

	def get_title(self):
		return self.resource.name

	def get_tabs(self):
		
		tabs = super(Page, self).get_tabs()


		tabs.extend( LogPage.get_tab_def(self) )

		return tabs

	def initRender(self, request):

		super(Page, self).initRender(request)

		self.layout_trouble    = self.user_settings.get('page.tag.troubleshoot_view', 0) == 1
		self.layout_monitor    = not self.layout_trouble

	def get_page_layout(self):

		layout = {}

		layout.update( super(Page, self).get_page_layout() )

		layout[self.TAB_GENERAL] = VerticalLayout('L1', [
						HorizontalLayout('L1-1', [
								VerticalLayout(self.LAYOUT_INFO  , min_height=170),  # dettagli
								VerticalLayout(self.LAYOUT_ALARMS, min_height=170)   # allarmi
						]),
						HorizontalLayout(self.LAYOUT_MONITOR, hidden=not self.layout_monitor),     
						HorizontalLayout(self.LAYOUT_TROUBLE, hidden=not self.layout_trouble),
					])

		layout.update( LogPage.get_layouts(self) )

		return layout


	def get_additional_jsfile_urls(self):
		urls = super(Page, self).get_additional_jsfile_urls()
		return urls

	#-------------------------------------------------------------------------------#

	# Metodi di  AlarmsCountPage

	def get_alarms_tab_container(self):
		return self.TAB_GENERAL, self.LAYOUT_ALARMS

	#-------------------------------------------------------------------------------#

	def get_troubleshoot_tab_container(self):
		return self.TAB_GENERAL, self.LAYOUT_TROUBLE, self.LAYOUT_MONITOR

	#-------------------------------------------------------------------------------#

	def render(self):

		if DEBUG_TIME: print "%s.render()" % (self.pagetype)

		user = self.global_settings['user']

		#
		# Load subtags / tagged elements
		#

		tree             = self.element.tree

		subtags          = self.element.child_tags.all()

		tagged, children = self.element.get_elementtags_with_elements(only_directly_tagged=True)

		#
		# Caricamento stato del tag e dei sotto tag da far vedere nella pagina.
		#

		# Dobbiamo chiedere al server gli stati degli UUID del 
		#     * tag 
		#     * sottotag visibili
		#     * elementi taggati
		ids_for_state  = [ self.element.uuid ]
		ids_for_state += [ t.uuid  for t  in subtags ]
		ids_for_state += [ et.uuid for et in tagged  ] 

		state_by_uuid = get_tags_state_safe(self.element.tenant.uuid, ids_for_state)
		if state_by_uuid:
			# Vediamo se gli stati sono "aggiornati" 
			for uuid in ids_for_state:
				state_data = state_by_uuid.get(uuid,None)
				if state_data:
					if tree.get_state_time() > state_data['_ts']:
						state_data['isold'] = 1

		t1 = time.time()

		widgets = []
		
		widgets = super(Page, self).get_widgets()

		#
		# Carichiamo lo stato globale del tag 
		#
		tags = [ self.element, ]

		t2 = time.time()
		if DEBUG_TIME: print "\t", "caricato stato di %s in %s" % (self.element, t2-t1)

		#
		# DETTAGLI E DETTAGLI AVANZATI
		#
		
		
		widget_info = self.create_global_info_widget(state_by_uuid)
		widget_more = self.create_advanced_info_widget()

		widget_expander = FrameWidget(
			tab = self.TAB_GENERAL,
			container = self.LAYOUT_INFO,
			rdata = {
				"struct": [
					{
						"type": "widget",
						"contents": widget_info
					},
					{
						"type": "wframesub",
						"hideable": True,
						"visible": False,
						"buttonpos": 0,
						"contents":
						[
							{
								"type": "widget",
								"contents": widget_more
							},
						]
					}
				]
			},
		)
		w = widget_expander.to_dict()
		widgets.append(w)

		t2b = time.time()
		#
		# TABELLA PRIORITA' E DETTAGLIO ALLARMI
		#
		if DEBUG_TIME: print "\t", "dettagli e dettagli avanzati... fatto", t2b-t2

		widgets += self.create_alarms_section_widgets()

		t3 = time.time()

		if DEBUG_TIME: print "\t", "sezione allarmi... fatto in %.2f secondi" % (t3-t2)

		#
		# MONITORING AREA
		#

		#
		# PULSANTE
		w = CustomDivWidget(
			id        = "details",
			tab       = self.TAB_GENERAL,
			container = self.LAYOUT_MONITOR,

			rdata = {
				'structure': {
					'template':  format_html('''
						<div class="button" data-show="L-troubleshootarea" data-hide="self">
							Show Troubleshooting View
						</div>
					''')
				}
			}
		)
		widget = w.to_dict()
		widgets.append(widget)
		
		
		# ELEMENTI ESPANDIBILI
		widgets += self.tags_grid_widget(self.element, 'general', self.LAYOUT_MONITOR, subtags, tagged, children, state_by_uuid, user=user)
			
			
		t11 = time.time()

		if DEBUG_TIME: print "\t", "elementi espandibili... fatto in %.2f secondi" % (t11-t3)

		widgets += self.create_troubleshoot_area_widget()
		t12 = time.time()
		if DEBUG_TIME: print "\t", "troubleshooting area... fatto in %.2f secondi" % (t12-t11)

		widgets += self.get_log_widgets()

		return {
			'name'   : self.pagetype,   

			'layout' : self.get_page_layout(),
			'tabs'   : self.get_tabs(),

			'widgets': widgets,
		}


	def tags_grid_widget(self, element, tab, container, subtags, tagged, children, state_by_uuid, user=None):

		sections = []

		#
		# Render Subtags badges
		#

		objects = []
		for subtag in list(subtags):
			o = create_badgeview_object(state_by_uuid, subtag, user=user)
			objects.append(o)
			
		sections.append({'name': 'Tags', 'objects':objects})
		
		#
		# Render Tagged elements badges
		#

		elementtag_by_elementid = dict([ (et.elementid, et) for et in tagged])

		objects_by_type = {}
		for c in list(children):
			
			elementtag = elementtag_by_elementid.get( c.uuid, None)
			c.cache_id = elementtag.uuid

			
			objects_list = objects_by_type.get(c.resource_type,  objects_by_type.setdefault(c.resource_type, []))

			o = create_badgeview_object(state_by_uuid, c, user=user)
			objects_list.append(o) 

		#
		# Create sections data
		#
		sections += create_element_sections(objects_by_type)

		data = {
			'etype': element.resource_type,
			'uuid' : element.resource_id,

			'sections': sections,
			'can_untag' : False
		}

		#
		# Create widget
		#
		
		widgets = []
		
		rt  = element.resource_type
		rid = element.uuid

		w = {
			'id'       : 'tagsgrid-%s' % (self.resource.uuid),
			'wtype'    : ["badgeview"],

			'tab'      : ' '.join([tab,]),
			'container': container,

			'width'    : 'max',

			'rdata'    : data,

			'remote'   :  "%s_%s.%s" % (rt, rid,  'badgeview'),
			'dataset'  :  "%s_%s.%s" % (rt, rid,  'badgeview'),
			'rtimer'   :  120,
		}
		widgets.append(w)
		
		return widgets


	def create_global_info_widget(self, state_by_uuid):

		tag  = self.element
		tree = tag.tree

		#
		# Stato del tag
		#
		status_code  = CSTATE_TO_CODENAME(CSTATE_UU)
		icon_overlay = None
		icon_url     = None

		if not tree.nostate:
			
			if state_by_uuid is not None:

				cached_state = state_by_uuid.get(tag.uuid, None)
				
				if cached_state is not None:

					status                = cached_state['visible_status']
					alarms_count          = cached_state['alarms']
					critical_alarms_count = cached_state['critical']
					exploded              = cached_state['exploded']

					status_code           = CSTATE_TO_CODENAME(status)

					if critical_alarms_count > 0 or exploded:
						status_code      = CSTATE_TO_CODENAME(CSTATE_DN)

						icon_overlay = ICON_WARNING

					if cached_state.has_key('isold') and settings.WEBUI_STATE_SHOW_OLD_ICON:
						icon_overlay = ICON_OLD

				else:
					icon_overlay = ICON_OLD
			else:
				icon_overlay = ICON_NOSTATE
		else:
			icon_overlay = ICON_NOSTATE


		icon_url = tag.get_icon_url_with_state(status_code)

		c = {
			'element'      : tag,

			'path'         : tag.ancestors_tags_list(),

			'element_state': status_code,
			'icon_url'     : icon_url,
			'icon_overlay' : icon_overlay,
		}

		html = render_template( 'html/parts/tag_details.html', c )

		w = CustomDivWidget(
			id     = 'info',
			tab    = 'general',
			rdata  = {
				"structure": {
					'template': format_html( html )
				}
			}
		)

		return w.to_dict()


	def create_advanced_info_widget(self):

		html = render_template( 'html/parts/tag_details_more.html', {
			'element'      : self.element,
			'metadata'     : self.tag.metadata.order_by('name').all(),
			'groups_perm'  : self.element.tree.get_group_permissions() if self.element.parent is None else None,
		} )

		w = CustomDivWidget(
			id = "tenant_%s_more_info" % (self.tenant.uuid),
			tab       = self.TAB_GENERAL,
			container = self.LAYOUT_INFO,
			
			rdata = {
				'structure': {
					'template': format_html( html )
				}
			}
		)
		return w.to_dict()


	#-------------------------------------------------------------------------------#

	def get_log_tab_layout(self):
		return self.TAB_LOG, self.LAYOUT_LOG

