# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import types

from django.conf import settings

from django.contrib.auth.models  import User

from core.models import UserCustomSetting

from webui.api import DEFAULTS_CUSTOM_SETTINGS

from webui.api import format_html

from webui.api.widgets import CustomDivWidget

from webui.api.utils import render_template

from webui.api.Pages import BasePage

from webui.api.Pages import VerticalLayout

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class Page(BasePage):
	
	TAB_GENERAL    = 'general'
	
	LAYOUT_GENERAL = 'main'
	
	PAGE_HTML_TEMPLATE = 'html/index_userpage.html'

	def __init__(self, rid, tenant, global_settings):

		super(Page, self).__init__(rid, tenant, global_settings)

		self.pagetype = 'userpage'

		self.has_nav_bar = False

		self.has_search = False

		self.resource = User.objects.get(id=int(self.rid))

	def get_page_layout(self):

		tabs_layout = super(Page, self).get_page_layout()
		
		
		
		tabs_layout[ self.TAB_GENERAL ] = VerticalLayout('L0',  items=[
			VerticalLayout(self.LAYOUT_GENERAL, items=[]),
		])

		return tabs_layout


	def get_tabs(self):

		tabs = super(Page, self).get_tabs()
		
		return tabs


	def get_title(self):
		return "Userpage"

	def get_additional_menus(self):
		return []

	def get_widgets(self):

		user = self.resource

		widgets = super(Page, self).get_widgets()
		
		#title = CustomDivWidget(
		#	id        = "title",
		#	tab       = self.TAB_GENERAL,
		#	container = self.LAYOUT_GENERAL,  
		#	rdata = {
		#		'structure': { 'template': format_html( '''<h1>User Page</h2>''') }
		#	}
		#)
		#
		#widgets.append(title.to_dict())
		
		#
		# AREA DATI
		#
		
		#
		# Load user settings
		#
		custom_settings = {}
		for us in user.customsettings.filter(tenant=None).all():
			val = us.get_value()
			if isinstance(val,  types.BooleanType):
				val = 1 if val else 0
			custom_settings[us.key] = val

		# Set defaults if not configured
		for k, v in DEFAULTS_CUSTOM_SETTINGS.items():
			if not custom_settings.has_key(k):
				custom_settings[k] = v

		# Save current settings (if different from defauls)
		UserCustomSetting.save_user_settings_dict(user, custom_settings, tenant=None)

		#
		# Fix
		#
		for k, v in custom_settings.items():
			k   = k.replace('.','_')
			custom_settings[k] = v

		html = render_template( 'html/parts/user_data.html', { 'user' : user , 'user_settings': custom_settings } )

		w = CustomDivWidget(
			id        = "user_data",
			
			tab       = self.TAB_GENERAL,
			container = self.LAYOUT_GENERAL,  
			
			rdata     = {
				'structure': { 
					'template': format_html( html )
				}
			}
		)
		widgets.append( w.to_dict() )

		return widgets



