# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from core.models import TenantSetting, MonitoredElement

from core.statecolor import get_state_color

from webui.api import *

from webui.api.Parts import BasePart

from webui.api.widgets import widget_list_of_datagroup

from webui.api.Pages import VerticalLayout, PAGELAYOUT_STYLE_TRANSPARENT

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class Part(BasePart):

	def __init__(self, rid, tenant):
		super(Part, self).__init__(rid, tenant)
		
		self.element = MonitoredElement.objects.get(uuid = self.rid)

	def render(self):
		
		#
		# Load data
		#
		tenant = self.element.tenant
		
		critical_priority = tenant.get_setting(TenantSetting.SETTING_CRITICAL_LEVEL)
		minimum_priority  = tenant.get_setting(TenantSetting.SETTING_MINIMUM_LEVEL)
		
		loaded_conditions = self.element.get_all_conditions()

		status, critical_alarms_count, alarms_count, dn_count = self.element.get_state_and_conditions(critical_priority=critical_priority, minimum_priority=minimum_priority, loaded_conditions=loaded_conditions)
		
		#
		# Produrre l'output con i nodi taggati
		#
		
		color = get_state_color(status)
		
		layout_name = "P_"+self.rid

		layout = {
			'general': 
				VerticalLayout(layout_name, css_class=PAGELAYOUT_STYLE_TRANSPARENT+ ' boxcontainer', style_params = [['border-color', "#"+color]]),
		}

		widgets = []
		
		#
		# FIX: widget_list_of_datagroup() fa troppi accessi al DB
		#
		widgets += widget_list_of_datagroup(self.rid, layout_name, 'general', critical_priority=critical_priority, minimum_priority=minimum_priority, loaded_conditions=loaded_conditions)

		return {
			'widgets': widgets, 
			'layout': layout
		}

