# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os
import re
import os.path

from django.conf import settings
from core.compatibility import reverse

from core.api.base import tenant_get_data_dir

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

EXTERNAL_MENUS_DIR_NAME = 'menus'

EXTERNAL_SCRIPT_TIMEOUT = 10

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def get_menus_dir_path(tenant, rt):
	data_dir = tenant_get_data_dir(tenant.uuid)	
	path = os.sep.join([data_dir, EXTERNAL_MENUS_DIR_NAME, rt])
	return path

def load_resource_menu_entries(tenant, resource_type):
	
	menu_entries = []

	try:
		menu_dir = get_menus_dir_path(tenant, resource_type)
			
		files = os.listdir( menu_dir )
		for filename in files:
			
			if filename > '':
				if filename[0] == '.':
					continue
			
			file_path = os.sep.join([menu_dir, filename])
			
			f = open(file_path)
			f.readline() # skill script header
			meta_info_line = f.readline()
			if not meta_info_line:
				continue

			m = re.match(r"\#SANET_META\:([^,]*),([^,]*)(?:,(.*))*", meta_info_line)
			if m == None:
				continue

			label = m.group(1).strip()
			descr = m.group(2).strip()
			icon  = m.group(3)
			icon  = icon.strip() if icon != None else icon
		
			menu_entry = { 
				 'script_name'  : filename
				,'label' : label
				,'descr' : descr
				,'icon'  : icon
			}
			
			
			menu_entries.append( menu_entry )
			
	except Exception, e:
		pass

	menu_entries = sorted( menu_entries, lambda x,y: -1 if (x['label'] < y['label']) else (1 if (x['label'] > y['label']) else 0)   )

	return menu_entries


def get_external_menu_entries(tenant, resource_type, resource_id):

	

	entries = []
	
	external_entries = load_resource_menu_entries(tenant, resource_type)
	
	for e in external_entries:
		
		icontype = 'image'
		iconvalue = ''
		
		if e['icon']:
			icon_path = os.sep.join( [settings.STATIC_WEB_ROOT, 'images', 'icons',  e['icon']] )
			if os.path.exists(icon_path):
				icontype  = 'image'
				url_path  = '/'.join( ['images', 'icons',  e['icon'] ] )
				iconvalue = reverse('static.file.serve', kwargs={'path': url_path } )
			else:
				icontype  = 'font'
				iconvalue = e['icon']
			
		url    = reverse('launch_external_script', kwargs={'rt': resource_type, 'uuid': resource_id, 'scriptname': str(e['script_name'])})

		menu_entry = {
			  'id'       : e['script_name']
			, 'descr'    : e['label'] + ' ' + e['descr']
			, 'icontype' : icontype
			, 'iconvalue': iconvalue
			, 'url'      : url
		}

		entries.append(menu_entry)
		
	return entries




