# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.
"""

Quando l'interfaccia web deve scambiarsi dati via ajax con i widget, il formato e' il seguente:

	{
		'status': 'success' | 'error'
		'info'  : '....',
		'data'  : ......
	}
	
Le view che rispondono a richieste ajax dovranno sempre restituire oggetti creati con 
HttpJsonSuccessResponse o HttpJsonSuccessResponse.

	def mia_view_ajax(....):
		...
		...
		return HttpJsonSuccessResponse('ok', [1,2,3 4,5])


"""

import json
import traceback

from django.http import HttpResponse

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

RESPONSE_SUCCESS = 'success'
RESPONSE_ERROR   = 'error'

def __json_response( status, text_info='', data=None):
	return {
		'status': status,
		'info'  : text_info,
		'data'  : data,
	}

def HttpJsonSuccessResponse(text_info='', data=None):
	return HttpResponse(json.dumps(__json_response(RESPONSE_SUCCESS, text_info=text_info, data=data)))

def HttpJsonErrorResponse(text_info='', data=None):
	return HttpResponse(json.dumps(__json_response(RESPONSE_ERROR, text_info=text_info, data=data)))

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def ajax_response(view_fun):
	"""
	Questa decoratore converte in HttpJsonSuccessResponse il valore restitutito dalla VIEW.
	
	Se la view solleva una eccezione, la funzione restituisce HttpJsonErrorResponse.
	"""

	def _ajax_response(request, *args, **kwargs):


		try:
			data = view_fun(request, *args, **kwargs)

			return HttpJsonSuccessResponse(data=data)

		except Exception, e:
			msg = str(e) + traceback.format_exc()

			#print __file__, "ajax_response", view_fun, args, kwargs, msg

			return HttpJsonErrorResponse(text_info=str(e), data=msg)

	return _ajax_response
