# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from core.models import Tenant
from core.models import Agent
from core.models import Cluster
from core.models import MonitoredNode
from core.models import MonitoredIface
from core.models import MonitoredStorage
from core.models import MonitoredService
from core.models import MonitoredDevice
from core.models import DataGroup
from core.models import DataSource
from core.models import Condition
from core.models import TimeGraph

from tags.models import Tag


"""
L'interfaccia web identifica le diverse entita' da visualizzare/gestire 
con un nome (in minuscolo) che non combiacia col nome della classe
del modello django. 

Per standardizzare la gestione dei nomi si usa questi due dizionari
per il mapping diretto e inverso da "tipo entita" a "modello django"
"""

WEB_ELEMENT_TYPE_TO_MODEL = {

	# CORE
	'tenant'    : Tenant,
	'agent'     : Agent,
	'cluster'   : Cluster,
	'node'      : MonitoredNode,
	'interface' : MonitoredIface,
	'storage'   : MonitoredStorage,
	'service'   : MonitoredService,
	'device'    : MonitoredDevice,
	
	'datagroup' : DataGroup,
	'datasource': DataSource, 
	'condition' : Condition,
	'timegraph' : TimeGraph,

	'tag'       : Tag,
	
	#'webscenario': WebScenario
}

# Reverse
MODEL_TO_WEB_ELEMENT_TYPE = dict( (v,k) for k,v in WEB_ELEMENT_TYPE_TO_MODEL.items())


def register_web_element_model( web_element_type, element_model):

	global WEB_ELEMENT_TYPE_TO_MODEL
	global MODEL_TO_WEB_ELEMENT_TYPE

	# Add
	WEB_ELEMENT_TYPE_TO_MODEL[ web_element_type ] = element_model

	# Update Reverse
	MODEL_TO_WEB_ELEMENT_TYPE = dict( (v,k) for k,v in WEB_ELEMENT_TYPE_TO_MODEL.items())



