# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import uuid
import operator

from sanet_common.sanetdefs.constants import *

from core.models import *
from core.api.cache import init_cache_manager_from_conf

# import state colo function
from core.statecolor import get_state_color
from core.statecolor import get_tag_state_color

from tags.models import *

from webui.api import format_html
from webui.api import format_html_template
from webui.api.colors import  lighten_rgb

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

#DIVIDER = '''<div><br /></div>'''

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

def WIDGET_CONFIG(custom_settings_dict):

	d = {
			'id'       : None,
			'wtype'    : None,

			'container': None, # position in layout page

			'tab'      : None, # XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX QUESTO PARAMETRO NON DOVREBBE PIU' SERVIRE

			# funziona SOLO max oppure numeri 1-12
			'width'    : 'max',

			# funziona SOLO min oppure solo con numero 1-15
			'height'   : 'min',

			# Conviene i dati che servono al widget. Il formato di questo campo e' fisso per tutti i nuovi widget.
			# Questo campo nei widget e' conosciuto come "wdata", ma l'interfaccia web, per evitare 
			# "sovrapposizioni" in certi momenti, usa un nome diverso.
			'rdata'    : { },

			'wconf'    : None,  # A che serve?

			#'remote'   : None, # remote address for dat
			#'rtimer'   : None,
			#'setup'    : None,
			#'dataset'  : None

			# Identificativo del "datasource" associato al widget.
			# Se rtimer e' <> 0  il "datasource" viene scaricato (in qualche modo) e il widget aggiornato periodicamente.
			'remote'    : None,

			# intervallo di refresh in secondi. 0 = no refresh.
			'rtimer'    : 0,     

			# Il widget ha un suo pulsante di start/stop autorefresh
			'autorefresh_button': False,

			# Durante il refresh automatico fa un lock del widget mostrando uno spinner visibile dall'utente
			'showspinner'       : True,

			# ?????????????????????????????????????????
			'dataset'           : None,
	}
	d.update(custom_settings_dict)
	return d

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class BaseWidget(object):

	wtype = None

	def __init__(self, **kwargs):

		self.fields = WIDGET_CONFIG({
			'id'       : str(uuid.uuid4()),
			'wtype'    : [ self.wtype ],
		})
		self.fields.update(kwargs)

	def to_dict(self):
		return self.fields

#-------------------------------------------------------------------------------#

class FrameWidget( BaseWidget ):

	wtype = "widgetframe"

	def __init__(self, **kwargs):
		super(FrameWidget, self).__init__(**kwargs)

#-------------------------------------------------------------------------------#

class QGridWidget( BaseWidget ):

	wtype = "qgrid"

	def __init__(self, **kwargs):
		super(QGridWidget, self).__init__(**kwargs)

		self.fields['width' ] = 4
		self.fields['height'] = 1

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class WCoreBaseWidget( BaseWidget ):

	wlib = "wcore"

#-------------------------------------------------------------------------------#

class CustomDivWidget( WCoreBaseWidget ):

	wtype = "customdiv"
	wlib  = "wcore"

	def __init__(self, **kwargs):
		super(CustomDivWidget, self).__init__(**kwargs)

#-------------------------------------------------------------------------------#

class ConditionGraphWidget(WCoreBaseWidget):

	wtype = "timegraph3"

	def __init__(self, c, **kwargs):
		super(ConditionGraphWidget, self).__init__(**kwargs)

		self.fields["wlib" ] = "lwp"
		self.fields["remote"] = ConditionGraphWidget.get_remote(c.uuid)

	@staticmethod
	def get_remote(c_uuid):
		return "condition_%s.timegraph3condition" % (c_uuid)

#-------------------------------------------------------------------------------#

class DataSourceGraphWidget(WCoreBaseWidget):

	wtype = "timegraph3"

	def __init__(self, ds, **kwargs):
		super(DataSourceGraphWidget, self).__init__(**kwargs)

		self.fields["wlib"  ] = "lwp"
		self.fields["remote"] = DataSourceGraphWidget.get_remote(ds.uuid)

	@staticmethod
	def get_remote(ds_uuid):
		return "datasource_%s.timegraph3datasource" % (ds_uuid)

#-------------------------------------------------------------------------------#

class TimeGraph3Widget(WCoreBaseWidget):

	wtype = "timegraph3"

	def __init__(self, tg, **kwargs):
		super(TimeGraph3Widget, self).__init__(**kwargs)

		self.fields["wlib"  ] = "lwp"
		self.fields["remote"] = TimeGraph3Widget.get_remote(tg.uuid)

	@staticmethod
	def get_remote(ts_uuid):
		return "timegraph_%s.timegraph3" % (ts_uuid)

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class WebScenarioWidget(WCoreBaseWidget):

	wtype = "timegraph3"

	def __init__(self, step, **kwargs):
		super(WebScenarioWidget, self).__init__(**kwargs)

		self.fields["wlib"  ] = "lwp"
		self.fields["remote"] = WebScenarioStepWidget.get_remote(step.uuid)

	@staticmethod
	def get_remote(ws_uuid):
		return "webscenario_%s.timegraph3ws" % (ws_uuid)

class WebScenarioStepWidget(WCoreBaseWidget):

	wtype = "timegraph3"

	def __init__(self, step, **kwargs):
		super(WebScenarioStepWidget, self).__init__(**kwargs)

		self.fields["wlib"  ] = "lwp"
		self.fields["remote"] = WebScenarioStepWidget.get_remote(step.uuid)

	@staticmethod
	def get_remote(step_uuid):
		return "webscenariostep_%s.timegraph3step" % (step_uuid)

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class NotesWidget( BaseWidget ):

	wtype = "notelist"

	def __init__(self, **kwargs):
		super(NotesWidget, self).__init__(**kwargs)

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class AlarmsListWidget( BaseWidget ):

	wtype = "alarmslist"

	def __init__(self, **kwargs):
		super(AlarmsListWidget, self).__init__(**kwargs)

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class DividerWidget( CustomDivWidget ):

	wtype = "customdiv"

	def __init__(self, **kwargs):
		super(DividerWidget, self).__init__(**kwargs)

		self.fields['rdata'] = {
				'structure': {
					'template':'''<div class="divider"></div>'''
				}
		}
		self.fields['height'] = 'min'

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

def create_label_widget(label, tab, container, htmltag='h2'):

	w = CustomDivWidget(
		id    = "tags_label_%s" % str(uuid.uuid4()),

		tab       = tab,
		container = container,  

		rdata = {
			'structure': { 'template': '''
				<div class="divider">
					<%(htmltag)s>
						<span class="name">%(label)s</span>
					</%(htmltag)s>
				</div>
				''' % { 
					'label'  : label, 
					'htmltag': htmltag
				}
		#		%(divider)s
		#			'divider': DIVIDER , 
			}
		},
		role = 'section-divider',
	)
	return w.to_dict()

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

AREAGETTER_TYPE = {
	'tenant'    : 'subtags',
	'tag'       : 'subtags',
	'node'      : 'datagroups',
	'interface' : 'datagroups',
	'service'   : 'datagroups',
	'storage'   : 'datagroups',
	'device'    : 'datagroups',
}

# Questa funzione e' generica e potrebbe essere usata anche nella pagina del tag
def create_element_badge_widget(element, alarms_count, critical_alarms_count, icon_url, icon_overlay_url, status_code, pagetype, tab, container, refreshtime=0, cache_manager=None, parent_element=None):

	rt  = element.resource_type
	rid = element.uuid

	element_page_url = element.page_url

	if status_code == "NONE":
		overlay_add_class = "badge_overlay_icon_small"
	else:
		overlay_add_class = ""
    
	details = {
		"url_page"  : element_page_url,
		"url_delete": None,
	}
	

	dataset = { 
		"id"         : rid,
		"name"       : element.name,
		'description': element.description,
		"state"      : {
			"alarms"           : alarms_count,
			"img"              : icon_url,
			"overlay"          : icon_overlay_url,
			"overlay_add_class": overlay_add_class,
		},
		'details'  : details,
		'measures' : None,
		'clickable': True
	}

	#
	# STATUS
	#
	if isinstance(element, Tag) or isinstance(element, MonitoredNode):
		bar_color  = get_tag_state_color(status_code)
	else:
		bar_color  = get_state_color(status_code)


	# get bar color for element
	bg_color   = lighten_rgb(bar_color)

	# get measure for data
	measures = {
		#"type":  "BAR",
		#"data":  75 ,
		#"type":  "PAIR",
		#"data":  [ 10, 30 ],
	}
	# attenzione non mettere il valore se vuoto , problemi col widget
	if measures:
		dataset['measures'] = measures


	#clickable = True
	#if pagetype in ('tenant','tag') and not isinstance(element, Tag):
	#	# click or not click this is the trouble ...
	#	clickable = False

	# delete element
	if pagetype in ('tenant','tag') and parent_element:
			if not isinstance(element, Tag):
				details["url_delete"] = "javascript:untag_elementid(\'%s\', \'%s\');" % (rid, parent_element.uuid)
			else:
				details["url_delete"] = "javascript:delete_tag(\'%s\');" % (rid)

	#
	# change element badget title and description for different elementype and page 
	#
	if isinstance(element, Condition) or isinstance(element, DataGroup):
		element.load_fields_from_cache(cache_manager)
		#title       = " : ".join([item for item in element.element.path.split(":")])

		title        = element.path.split(";",1)[0]

		description = element.expanded_title(cache_manager=cache_manager)

	elif isinstance(element, MonitoredElement):
		title       = element.name
		description = element.description
	else:
		title       = element.anme
		description = ''


	# update dataset name
	dataset['name']        = title
	dataset['description'] = description

	widgets = []

	badge_widget_id = rid+"--pager"

	w = {
		'id'                     : badge_widget_id,
		'wtype'                  : ["elementbadge"],

		'tab'                    : tab,
		'container'              : container,            # Uhmmmm

		'width'                  : 4,                   # Uhmmmm
		'height'                 : 2,                   # Uhmmmm

		'rdata'                  : dataset,

		'droppable'              :' '.join(['node', 'interface', 'service', 'storage', 'condition', 'timegraph', 'datagroup']),

		# tutti queeti parametri sono necessari SOLO se vuoi che l'elementbadge deve essere cliccabile e aprire l'area sotto
		'sid'                    : rid,
		'toggle_group'           : rid+"_tg",
		'activation_toggle_class': ['statuswidget', 'statusbar'],
		'areagetter'             : rid,
		'areagetter_type'        : AREAGETTER_TYPE.get(rt, ''),

		'state'                  : status_code,
	}
	widgets.append(w)



	# FIXME: inseriti colori a caso per visualizzare: da mettere colori effettivi

	# elementi necessari per colorare gli elementi che si aprono
	# colore del bordo dell'area aperta
	# bar_color  = get_tag_state_color(status)
	# bar_color  = '000000'
	# colore di sfondo dell'area aperta
	bg_color   = 'eeeeee'

	rows = []

	rows.append({   
			"label": "",
			"content": [
				{
					"content": {
						
						"type"   : "wloader",
						"wid"    : rid + '--divider',
						"wtype"  : "customdiv",
						"wlib"   : "wcore",
						
						"attrs"  : {},
						"classes": [],
						"wdata" : {
							'structure': {
								'template' :''' 
									<style>
										#_%(uuid)s { 
											border-color    : #%(bar_color)s; 
											background-color: #%(bg_color)s;
										}
									</style>
									<div class="statuswidget divider">
										<h2 style="background-color: #%(bg_color)s; border-color: #%(bar_color)s;">
											<span class="i">
												<img src="%(icon_url)s" style="max-width:25px;max-height:25px;padding-left: 5px;" >
											</span>
											<span class="name">%(name)s</span>
											<a target="_blank" href="%(element_url)s"><i style="font-size: large; color:black;margin-left:10px;" class="icon-share" ></i></a>
										</h2>
									</div>
									''' % {
										'uuid'       : rid,
										'name'       : title,
										'element_url': element_page_url,
										'icon_url'   : icon_url,
										'bar_color'  : bar_color,
										'bg_color'   : bg_color,
									},
								'active'   :'.statuswidget',
							},
						},
					},
				},
			],
			"style": {}
	})

	widget_table = {
		"id"                 : rid + "--table-divider",
		'wtype'	             : ["tablegrid"],

		'tab'                : tab,
		'container'          : container,
                                     
		'width'              : 'max',
		'height'             : 'max',

		'depends_on'         : badge_widget_id,
		'sid'                : rid,

		'areagetter_before'  : rid,
		'toggle_group'       : rid + "_tg",

		'droppable'          :' '.join(['node', 'interface', 'service', 'storage', 'condition', 'timegraph', 'datagroup']),

		'rdata': {
			"cols": [
				{
					"label": "",
					"weight": 0.7,
					"width": None,
					"style": {}
				},
			],
			"rows"      : rows,
			"tablestyle": "datacontainer",
			"styles"    : { 
				"border-color": "#"+bar_color
			}
		}
	}
	widgets.append(widget_table)


	w = {
		'id'               : rid + "--end-divider",
		'wtype'            : ["customdiv"],

		'tab'              : tab,
		'container'        : container, 

		'width'            : "max",
		'height'           : "min",

		'setup'            : None,  # FIX: XXXXXXXXXXXXXXXX??????????????????????????

		'areagetter_after' : rid,
		'sid'              : rid,

		'depends_on'       : rid + "--pager",
		'toggle_group'     : rid + "_tg",
		'hide_tools'       : True,

		'rdata'            : {
			'structure'    : { 
				'template': '''<div class="statuswidget divider"></div>''',
			},
		},
	}
	widgets.append(w)

	widget_div = DividerWidget(
		id        = "divider_%s_end" % rid,

		sid       = rid,

		tab       = tab,  
		container = container,

		width     = 'min',
	)
	widgets.append(widget_div.to_dict())

	return widgets

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

def widget_list_of_datagroup(element_uuid, container, tab, critical_priority=PRIORITY_NONE, minimum_priority=PRIORITY_NONE, loaded_conditions=None, cache_manager=None):

	widgets = []

	if cache_manager is None:
		cache_manager = init_cache_manager_from_conf()

	datagroups_list = DataGroup.active_objects.filter(element__uuid = element_uuid).all()

	# Load & sort by expanded title
	for dg in datagroups_list:
		dg.load_fields_from_cache(cache_manager)
		dg.expanded_title_str = dg.expanded_title(cache_manager=cache_manager)

	datagroups_list = sorted(datagroups_list, key=operator.attrgetter('expanded_title_str'))

	# Create datagroup widget 
	pos = 0
	for dg in datagroups_list:

		#
		# Calculate state
		#
		if loaded_conditions is not None:
			conditions = [ c for c in loaded_conditions if (c.datagroup_id == dg.uuid and c.deleted==False) ] 
		else:
			conditions = dg.conditions.filter(deleted=False).order_by('title').all()

		dg.has_primary = True if (conditions and max([ c.primary for c in conditions ])) else False

		state, critical_dn_count, minimum_dn_count, dn_count, tot_ignored = dg.get_state_and_conditions_with_ignored(critical_priority=critical_priority, minimum_priority=minimum_priority, loaded_conditions=conditions)

		if critical_dn_count > 0:
			state = CSTATE_TO_CODENAME(CSTATE_DN)

		dg.has_ignored_conditions = (dn_count > minimum_dn_count) or (tot_ignored > 0)

		
		#
		# Widget
		#
		dg.position = pos
		dg.state = state

		DG_WIDGET_ID = dg.uuid + '_dg_area'

		w = CustomDivWidget(
			id        = DG_WIDGET_ID,
			sid       = dg.uuid,

			container = container,
			tab       = tab,

			rdata = {
				'structure': {
					'template':  format_html_template("""

						<table style="width:100%; " class='{% if dg.position > 0 %}datagroup-row-odd{% endif %}'>
							<tr>
								<td style="width: 30px;" valign="top">
									<span class="state-box bg-{{dg.state}} expandable">{{dg.state}}</span>
								</td>

								<td style="width:100%;">

									<h3 style="margin: 0px; display:inline-block;align: left;">
										<span class="name expandable" title="{{dg.name}}">{{dg.expanded_title_str}}</span>
									</h3>
									
										{% if dg.has_primary %}
										<div title="This datagroup contains primary conditions"
										     style="display: inline-block; width: 16px; height:16px; border-radius: 10px; background-color: blue; border: 1px solid blue; color:white; font-weight:bold; text-align:center; text-valign:middle">
											P
										</div>
										{% endif %}
										
										{% if dg.has_ignored_conditions %}
											<div title="This datagroup contains ignored conditions"
											     style="display: inline-block; width: 16px; height:16px; border-radius: 10px; background-color: yellow; border: 1px solid yellow; color: black; font-weight:bold; text-align:center; text-valign:middle">
												<li class="icon-exclamation"></li>
											</div>
										{% endif %}

								</td>
								<td style="width: 20px;">
									<span style="align:left;" class="expandable"><i class="gui-button gui-gradient icon-chevron-down"></i></span>
								</td>
							</tr>
						</table>

						""", {
							'dg': dg
						}),
					'active':'.expandable',
				}
			},
		)
		widget = w.to_dict()
		widgets.append(widget)

		pos += 1


		rows = []
		for c in conditions:

			status_color = '#'+get_state_color(CSTATE_TO_CODENAME(c.status))

			# La prima colonna contiene info sul grafico a fianco
			spacer = "<div style='width: 50px'>"
			if c.priority < minimum_priority and not c.primary:
				spacer += '<div class="icon_ignored" title="This datagroup contains ignored down conditions"><li class="icon-exclamation"></li></div>'
			if c.primary:
				spacer += '<div class="icon_primary" title="This is the primary condition">P</div>'
			spacer += "</div>"      

			rows.append({   
				"label"  : None,
				"style"  : None,
				"content":
				[
					{
						"style"  : None,
						"content": spacer, 
					},
					{
						"style"  : None,
						"content": {
							"type"   : "wloader",
							"wid"    : c.uuid + '_condition_chart',
							
							"wtype"  : ConditionGraphWidget.wtype, 
							
							"wlib"   : TimeGraph3Widget.wlib,
							"remote" : ConditionGraphWidget.get_remote(c.uuid),
							
							"rtimer" : settings.WEBUI_REFRESH_TIMERS['datagroup_timegraph'],
							
							"attrs"  : {
								"style" : """
									border: 5px solid %(status_color)s ; 
									border-radius: 10px; -webkit-border-radius: 10px; -moz-border-radius: 10px; 
									padding: 5px;
								""" % { "status_color": status_color }
							},
							"classes": [ # NOTA/FIX: Questa gestione delle classi per specificare i dataset e gestire il refresh e' verametne una schiefezza. NOn si capisce niente.
								#"datadset_%s_dash_condgraph" % c.uuid
								"datadset_%s_timegraph3condition" % c.uuid
							]
						},
					}
				],
			})

		timegraphs = dg.timegraphs.filter(deleted=False).all()
		for t in timegraphs:
			
			spacer = "<div style='width: 50px'>"
			spacer += "</div>"      
			
			rows.append({   
				"label"  : None,
				"style"  : None,
				"content": 
				[
					{
						"style"  : None,
						"content": spacer,
					},
					{
						"style"  : None,
						"content": {
							"type"   : "wloader",

							"wid"    : t.uuid  + '_timegraph_chart',
							
							"wlib"   : TimeGraph3Widget.wlib,
							"wtype"  : TimeGraph3Widget.wtype,              

							"remote" : TimeGraph3Widget.get_remote(t.uuid), 
							
							"rtimer" : settings.WEBUI_REFRESH_TIMERS['datagroup_timegraph'],

							"attrs"  : {
								"style": "border: 1px solid black; border-radius: 10px; -webkit-border-radius: 10px; -moz-border-radius: 10px; padding:5px;"
							}
							,
							"classes": [  # NOTA/FIX: Questa gestione delle classi per specificare i dataset e gestire il refresh e' verametne una schiefezza. NOn si capisce niente.
								
								#"datadset_%s_dash_timegraph" % t.uuid,
								
								"datadset %s_timegraph3" % (t.uuid)
							]
						},
					}
				],
			})

		widget_table = WIDGET_CONFIG({
				"id"        : dg.uuid + "_widgettable",
				'wtype'     : ["tablegrid"],

				'tab'       : tab,
				'container' : container,

				"sid"       : dg.uuid,
				
				'depends_on': DG_WIDGET_ID,   # gestione della dipendenza

				'rdata': {
					"cols": [
						{
							"weight": 0,
							"label" : None,
							"width" : None,
							"style" : None,
						},
						{
							"weight": 1,
							"label" : None,
							"width" : None,
							"style" : None,
						},
					],
					"rows": rows,
				}
		})
		widgets.append(widget_table)

	w = DividerWidget(
		id        = "divider_datagroups",
		tab       = tab,
		container = container,  
	)
	widget = w.to_dict()
	widgets.append(widget)

	return widgets

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

def widget_resource_section_title(resource, title, tab, container):
	
	w = CustomDivWidget(
		tab       = tab,
		container = container,  
		rdata = {
			'structure': { 
				'template': format_html_template(
					"""
					<div>
						<span style='font-size: 1.5em; font-weight:bold' >{{title}}:</span> 
						{% if resource.get_icon_url %}
							<img class='icon16x16' src='{{ resource.get_icon_url }}' />
						{% endif %} 
						{{ resource.name }} 
					<div>
					"""
					,
					{
						'title'   : title,
						'resource': resource,
					}
				)
			}
		},
	)
	return  w.to_dict()
