
import os
import sys

import pkgutil
import gettext


class BaseWidget(object):

	def __init__(self, widget_module_name):
		
		self.name = None
		self.widget_module_name = widget_module_name
		
		self._lang = None

	def get_name(self):
		return self.name

	def get_module(self):
		return self.widget_module_name


	def load_template_string(self, name):
	
		file_name = os.sep.join( ['templates', name ] )

		template_data = pkgutil.get_data(self.get_module(), file_name)
		
		return template_data

	#
	# i18n files
	#
		
	def get_base_local_path(self):

		package = self.get_module()
		
		# Find locale directory inside python package
		local_path = os.path.dirname(sys.modules[package].__file__)
		local_path = os.sep.join( [local_path, 'locale'] )
		
		return local_path
		
	def init_locale(self):
		
		if self._lang:
			return 

		domain_name = self.get_name()
		
		local_path  = self.get_base_local_path()
		
		languages   = ['it']

		try:
			t = gettext.translation(domain_name, local_path, languages=languages)
		except IOError, e:
			t = gettext.NullTranslations()
			
		self._lang = t
		
	def get_translator(self):
		
		if not self._lang:
			self.init_locale()
		
		return self._lang.gettext
		
		#if self._lang:
		#	return self._lang.gettext
		#else:
		#	return lambda x: x


	def render_content(self):
		raise Exception("Not implemented")

