# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

clean_temp_web_data
====================================================================================

Lancio: 

::
	
	sanet-manage clean_temp_web_data

Rimuove tutti i file temporanei prodotti dall'interfaccia web durante la gestione delle richieste.


Le directory che vengono controllate sono:

::
	
	{{VAR_DIR}}/ <tenant uuid> / media / timegraph_images / *

"""

import os
import shutil

from core.models import Tenant
from core.api.base import tenant_get_media_file_path

from core.management.commands import SanetInternalCommand

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class Command(SanetInternalCommand):

	help = "Remove temporafy files produced"

	#option_list = SanetInternalCommand.option_list

	def handle(self, *args, **options):

		self.clean_global()
		self.clean_tenants()

		return

	def clean_global(self):

		# TODO: cancellare file temporanei globali in {{VAR_DIR}}

		return

	def clean_tenants(self):

		for tenant in Tenant.objects.all():
			self.clean_tenant(tenant)

	def clean_tenant(self, tenant):

		self.clean_timegraph_images(tenant)

		# TODO: aggiungere altre cose per il tenant

		return

	def clean_timegraph_images(self, tenant):

		# {{VAR_DIR}}/ <uuid> / media / timegraph_images / *

		dest_dir   = tenant_get_media_file_path(tenant.uuid, 'timegraph_images') 

		count = 0

		for root, dirs, files in os.walk(dest_dir):
			for f in files:
				os.unlink(os.path.join(root, f))
				count += 1
			for d in dirs:
				shutil.rmtree(os.path.join(root, d))
				count += 1

		if count:
			print "\t", "Cleanded %s (deleted %s items)" % (dest_dir, count)


