# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import logging

from core.compatibility import reverse
from django.db import models
from django.dispatch import receiver
from django.contrib.auth.models import User, Group

from django.dispatch import Signal

from sanet_common.sanetdefs.constants import ELEMENT_KIND_STR

from core.models import sanet_init_subapp
from core.models import sanet_check_tenant
from core.models import sanet_cleanup_tenant
from core.models import sanet_post_check
from core.models import TenantDependentModel
from core.models import Tenant, Agent, Cluster, MonitoredElement, MonitoredNode, MonitoredIface, MonitoredStorage, MonitoredService, MonitoredDevice
from core.models import DataGroup, DataSource, Condition, TimeGraph

from tags.models import Tag, TagTree

from webui.api.types import MODEL_TO_WEB_ELEMENT_TYPE

#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#
# WEBUI MODELS                                                                #
#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#

class UserTagFilters(TenantDependentModel):

	user           = models.ForeignKey(User)
	tagtree        = models.ForeignKey(TagTree)

	class Meta:
		unique_together = (
			("tenant", "user", "tagtree"),
		)
		db_table = "usertagfilters"

#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#
# Extend core models                                                          #
#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#

#
# Per non legare il core con la webui, aggiungiamo dinamicamente metodi agli oggetti del core.
#

@property
def __resource_type_property(self):
	return MODEL_TO_WEB_ELEMENT_TYPE[ self.__class__ ]

@property
def __m_resource_type_property(self):
	return ELEMENT_KIND_STR[self.kind]

@property
def __resource_id(self):
	return self.uuid

@property
def __get_page_url(self):
	return reverse('resource_page'        , kwargs={'rt': self.resource_type, 'uuid': self.uuid}  )

@property
def __get_global_page_url(self):
	return reverse('global_resource_page' , kwargs={'tenantid': self.tenant_id, 'rt': self.resource_type, 'uuid': self.uuid}  )


Tenant.add_to_class('resource_type', __resource_type_property)
Tenant.add_to_class('resource_id'  , __resource_id)
Tenant.add_to_class('page_url'     , __get_page_url)
Tenant.add_to_class('global_page_url'     , __get_global_page_url)

Agent.add_to_class('resource_type', __resource_type_property)
Agent.add_to_class('resource_id'  , __resource_id)

Cluster.add_to_class('resource_type', __resource_type_property)
Cluster.add_to_class('resource_id'  , __resource_id)

MonitoredElement.add_to_class('resource_type', __m_resource_type_property)
MonitoredElement.add_to_class('resource_id'  , __resource_id)
MonitoredElement.add_to_class('page_url'     , __get_page_url)
MonitoredElement.add_to_class('global_page_url', __get_global_page_url)

MonitoredNode.add_to_class('resource_type', __resource_type_property)
MonitoredNode.add_to_class('resource_id'  , __resource_id)
MonitoredNode.add_to_class('page_url'     , __get_page_url)
MonitoredNode.add_to_class('global_page_url', __get_global_page_url)

MonitoredIface.add_to_class('resource_type', __resource_type_property)
MonitoredIface.add_to_class('resource_id'  , __resource_id)
MonitoredIface.add_to_class('page_url'     , __get_page_url)
MonitoredIface.add_to_class('global_page_url', __get_global_page_url)

MonitoredStorage.add_to_class('resource_type', __resource_type_property)
MonitoredStorage.add_to_class('resource_id'  , __resource_id)
MonitoredStorage.add_to_class('page_url'     , __get_page_url)
MonitoredStorage.add_to_class('global_page_url', __get_global_page_url)

MonitoredService.add_to_class('resource_type', __resource_type_property)
MonitoredService.add_to_class('resource_id'  , __resource_id)
MonitoredService.add_to_class('page_url'     , __get_page_url)
MonitoredService.add_to_class('global_page_url', __get_global_page_url)

MonitoredDevice.add_to_class('resource_type', __resource_type_property)
MonitoredDevice.add_to_class('resource_id'  , __resource_id)
MonitoredDevice.add_to_class('page_url'     , __get_page_url)
MonitoredDevice.add_to_class('global_page_url', __get_global_page_url)

DataGroup.add_to_class('resource_type', __resource_type_property)
DataGroup.add_to_class('resource_id'  , __resource_id)
DataGroup.add_to_class('page_url'     , __get_page_url)
DataGroup.add_to_class('global_page_url', __get_global_page_url)

DataSource.add_to_class('resource_type', __resource_type_property)
DataSource.add_to_class('resource_id'  , __resource_id)
DataSource.add_to_class('page_url'     , __get_page_url)
DataSource.add_to_class('global_page_url', __get_global_page_url)

Condition.add_to_class('resource_type', __resource_type_property)
Condition.add_to_class('resource_id'  , __resource_id)
Condition.add_to_class('page_url'     , __get_page_url)
Condition.add_to_class('global_page_url', __get_global_page_url)

TimeGraph.add_to_class('resource_type', __resource_type_property)
TimeGraph.add_to_class('resource_id'  , __resource_id)
TimeGraph.add_to_class('page_url'     , __get_page_url)
TimeGraph.add_to_class('global_page_url', __get_global_page_url)

Tag.add_to_class('resource_type', __resource_type_property)
Tag.add_to_class('resource_id'  , __resource_id)
Tag.add_to_class('page_url'     , __get_page_url)
Tag.add_to_class('global_page_url', __get_global_page_url)

#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#
# WEBUI SIGNALS                                                               #
#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#
#-----------------------------------------------------------------------------#

#
# Questo segnale viene lanciato per chiedere ad ogni sub-app di 
# definire nuovi tipi gestiti dall'interfaccia web.
#
webui_load_extension_web_types = Signal(providing_args=['web_element_type', 'element_model'])

#-----------------------------------------------------------------------------#

class WebuiPageExtension(object):
	"""
	Oggetto che implementa i metodi richiamati dall'interfaccia web
	per estendere/modificare l'aspetto di una pagina dell'interfaccia WEB.
	
	L'interfaccia web, in fase di caricamento di una pagina invoca il segnale
	
		webui_load_page_extension
	
	Tutte le subapp che lo desiderano possono restituire o meno un oggetto
	per estendere la pagina che l'interfaccia web sta generando.
	"""

	"""
	TAB     = 'dummy'
	LAYOUT  = 'L-dummy'
	"""

	def __init__(self, page):
		self.page = page

	def get_extension_tabs(self):
		"""
		Restituisce l'elenco dei TAB aggiuntivi per la pagina
		che l'interafccia web sta generando.
		"""

		"""
		Esempio
			
			tabs = [
				PageTab(**{
					'name'      : self.TAB,  # name for tab
					'title'     : 'DUMMY',         # title human readable
					'icontype'  : 'fonticon',         # choose icon: fonticon, image
					'iconvalue' : 'icon-stethoscope',        # icon url, run for icon and image
					'active_tab': False,              # active tab information
					'enabled'   : True,
					'order'     : 100,
				})
			]
			return tabs
		"""

		return []

	def get_extension_additional_jsfile_urls(self):
		"""
		Restituisceu una lista di stringhe con URL di file javascript
		da caricare all'interno della pagina.
		"""
		
		"""
		Esempio:
			page_js_file = 'page/dummy.js'
			url = reverse('static.file.serve' , kwargs={'path': page_js_file})
			return [ url, ]
		"""
		return []

	def get_extension_layouts(self):
		"""
		Restituisce un dizionario con l'elenco dei LAYOUT da gestire
		nella pagina.
		"""
		
		"""
		Esempio:
			conf =  {}
			conf[ self.TAB ] = VerticalLayout(self.LAYOUT )
			return conf
		"""
		return []

	#-----------------------------------------------------------------------------#

	def get_info_widgets(self):
		"""
		Restituisce i widget aggiuntivi specifici per la plugin da gestire nella sezione "informativa"
		(pagina del tenant e gli altri elementi di monitoraggio)
		"""
		return []

	#-----------------------------------------------------------------------------#

	def get_element_advanced(self, element):
		"""
		Restituisce un elenco di "proprieta" da visualizzare nei "Details" della sezione "advanced info".
		"""
		
		"""
		Esempio:
			details = []
			details.append({ 'name': 'Dummy Data'      , 'value': '12345'  })
			return details
		"""
		return []

	def get_advanced_info_widgets(self):
		"""
		Restituisce i widget da visualizzare nella sezione "advanced info"
		degli elementi (tenant, nodi, interfacce, ecc.)
		"""
		return []

	#-----------------------------------------------------------------------------#

	def get_extension_widgets(self):
		"""
		Esempio:
			widgets = []
			
			w = widget_resource_section_title(None, 'DUMMY WIDGET', tab=self.TAB, container=self.LAYOUT)
			widgets.append(w)
			return widgets
		"""
		return []

	#-----------------------------------------------------------------------------#

	def get_extension_additional_header_section_html(self):
		"""
		Restituisce codice HTML da aggiugere all'header della pagina (per pulsanti addizionali, ecc.)
		"""
		return None

	#-----------------------------------------------------------------------------#

	def get_extension_tenant_menu_entries(self):
		"""
		Restituisce entry da aggiungere al menu' del tenant nell'header della pagina
		La tupla e' del tipo:

			{
				'url'  : '<url>',
				'label': _("Scheduled activities"),
				'icon' : '<html code per mostrare una icona>',
			}

		"""

		return []

#-----------------------------------------------------------------------------#

#
# Questo segnale viene lanciato per chiedere ad ogni sub-app di 
# restituire oggetti WebuiPageExtension da usare per 
# estendere l'interfaccia WEB.
#
webui_load_page_extension  = Signal(providing_args=['page', 'resource_type', 'resource_id'])

#-----------------------------------------------------------------------------#

#
# Questo segnale viene lanciato per ottenere l'elenco di url 
# da inserire come pulsanti nel "badge" di un elemento.
#
#     app_results = [
#         ...
#         {
#             'url'  : 'http://.....'
#             'icon' : '<icon code name>'
#         }
#         ...
#     }
webui_elementbadge_urls  = Signal(providing_args=['element'])

#-----------------------------------------------------------------------------#

#
# Questo segnale viene lanciato per chiedere alle sub-app di 
# effettuare una ricerca per IP/MASK quando via web viene effettuata 
# una ricerca (o auto-completamento).
#
webui_search_node_name_by_ip  = Signal(providing_args=['partial_ip', 'mask'])

#-----------------------------------------------------------------------------#

#
# Questo segnale viene inviato dall'interfaccia web per sapere se nel periodo di 
# interesse ci sono "informazioni" utili da visualizzare. 
#
#     app_results = {
#         ...
#         '<name>': {
#             'subject'  : '<text>'
#             'color'    : '<#rrggbb>'
#             'ts_start' : '<timestamp>'
#             'ts_end'   : '<timestamp>'
#         }
#         ...
#     }
#
#
webui_calculate_highlight_times_data = Signal(providing_args=['tenant', 'ts_start', 'ts_end'])

def calculate_highlight_times_data(start, end, tenant):

	# Signal every app for highlight times
	apps_results = webui_calculate_highlight_times_data.send(None, tenant=tenant, ts_start=start, ts_end=end)

	# Merge all
	result = {}
	for app_handler, results in apps_results:
		result.update(results)

	if result:
		return result
	else:
		return None


#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
# SANET SIGNALS                                                                #
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#

@receiver(sanet_init_subapp)
def _sanet_init_subapp_webui(sender, **kwargs):

	logger = logging.getLogger('sanet.global')

	logger.info( "[webui] Init" )

@receiver(sanet_check_tenant)
def _sanet_check_tenant_webui(sender, **kwargs):

	logger = logging.getLogger('sanet.global')

	t = kwargs['instance']
	logger.info( "[webui] Check tenant %s" % (t) )

@receiver(sanet_cleanup_tenant)
def _sanet_cleanup_tenant_webui(sender, **kwargs):

	logger = logging.getLogger('sanet.global')

	t = kwargs['instance']
	logger.info( "[webui] Cleanup tenant %s" % (t) )

@receiver(sanet_post_check)
def _sanet_post_check_webui(sender, **kwargs):

	logger = logging.getLogger('sanet.global')

	logger.info( "[webui] Post check" )

