# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from django.dispatch import receiver
from django.contrib.auth.signals import user_logged_in, user_logged_out

from core.models import get_user_default_tenant

from core.api.base import tenant_get_primary

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

SESSION_TENANTID_KEY     = 'tenantid'

def set_session_tenant_keys(session, tenant):

	session[SESSION_TENANTID_KEY] = tenant.uuid

	# Force session save
	session.modified = True  

def reset_session_tenant_keys(session):

	if session.has_key(SESSION_TENANTID_KEY): del session[SESSION_TENANTID_KEY]

	# Force session save
	session.modified = True

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

@receiver(user_logged_in)
def set_tenant_logged_in_user(sender, **kwargs):
	"""
	Al termine della fase di login, salviamo nella sessione le informazioni
	sul tenant di default per l'utente.
	"""

	user    = kwargs['user']
	request = kwargs['request']

	tenant = get_user_default_tenant(user)

	if tenant is None:
		if user.is_superuser:
			tenant = tenant_get_primary()

	if tenant is not None:
		set_session_tenant_keys(request.session, tenant)

	return

#------------------------------------------------------------------------------#

@receiver(user_logged_out)
def set_tenant_logged_out_user(sender, **kwargs):

	request = kwargs['request']

	reset_session_tenant_keys(request.session)

	return

