# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings

from django.conf.urls import include, url

import django.contrib.auth.views
import django.views.i18n

from core.compatibility import reverse_lazy
 
import webui.views
import webui.views.debug
import webui.views.debug.datasources_temp
import webui.views.auth
import webui.views.users
import webui.views.external
import webui.views.redirects

#For Django 1.11
from webui.views.icons import get_colored_icon3

from webui.views import index as webui_index
from webui.views import timegraph_data
from webui.views import switch_user_form

  
#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

# This is the list of (python) packages that contain a 'locale' directory 
# for javascript code
# 	webui/locale/<language>/LC_MESSAGES/djangojs.po
webui_i18n_js_settings = {
	'domain'  : 'djangojs',
	'packages': ('webui',),
}

reverse_lazy('django.contrib.auth.views.login')

urlpatterns = [ #patterns('',


	# DEBUG DBILLI: NON USARE O SPE
	#
	url(r'^debug/env$'							, webui.views.debug.environment                       , name='webui.views.debug.environment' ),
	url(r'^debug/test$'							, webui.views.debug.test                              , name='webui.views.debug.test' ),
	url(r'^debug/datasources$'						, webui.views.debug.datasources_temp.list_datasources , name='webui.views.debug.datasources_temp.list_datasources' ),
	url(r'^debug/datasources/rrdgraph/(?P<dsuuid>.+)$'			, webui.views.debug.datasources_temp.rrdgraph         , name='webui.views.debug.datasources_temp.rrdgraph' ),
	url(r'^debug/datasources/fetch_data/(?P<dsuuid>.+)$'			, webui.views.debug.datasources_temp.fetch_data       , name='webui.views.debug.datasources_temp.fetch_data' ),
	
	url(r'^debug/set_conditions_status$'					, webui.views.debug.set_conditions_status             , name='webui.views.debug.set_conditions_status'),
	
	#
	# VIEW
	#
	url(r'^system_status$'							, webui.views.system_status                           , name='system_status'),

	# LOGIN/LOGOUT
	url(r'^accounts/login$'							, django.contrib.auth.views.login          , {'template_name': 'html/login.html' }         , name='login_page'  ),
	url(r'^accounts/logout$'						, django.contrib.auth.views.logout         , {'next_page': settings.LOGIN_URL }            , name='logout_view' ),

	#
	# ACCOUNT
	#
	url(r'^accounts/change_password$'					, webui.views.change_password                        , name='change_password'),
	url(r'^accounts/change_settings$'					, webui.views.change_settings                        , name='change_settings'),

	url(r'^accounts/switch_tenant$'						, webui.views.switch_tenant                          , name='webui.views.switch_tenant'    ),
	url(r'^accounts/switch_user$'                                           , webui.views.switch_user                            , name='webui.views.switch_user'      ),
	url(r'^accounts/switch_user_form$'                                      , webui.views.switch_user_form                       , name='webui.views.switch_user_form' ),

	url(r'^accounts/alarms_list/alarms_list_get_filter_tags'		, webui.views.users.alarms_list_get_filter_tags      , name='alarms_list_get_filter_tags'   ),
	url(r'^accounts/alarms_list/add_tag_filter$'				, webui.views.users.alarms_list_add_tag_filter       , name='alarms_list_add_tag_filter'    ),
	url(r'^accounts/alarms_list/remove_tag_filter$'				, webui.views.users.alarms_list_remove_tag_filter    , name='alarms_list_remove_tag_filter' ),


	# FIX: rimosso da dbilli: dash non viene micca usato da nessuno
	# DASH
	#
	# url(r'^dash/add$'       						, webui.views.dash.add_dash),
	# url(r'^dash/clear$'       						, webui.views.dash.clear_dash),
	# url(r'^dash/rm$'       							, webui.views.dash.rm_dash),
	# url(r'^dash/get_all$'      						, webui.views.dash.get_all),
	# url(r'^dash/addwidget$'       						, webui.views.dash.add_widget),

	#
	# PAGES
	#

	url(r'^tenantpage/(?P<tenantid>\w+)/(?P<rt>\w+)/(?P<uuid>\w+)$'		, webui.views.global_index                           , name='global_resource_page'),

	url(r'^page$'                                                           , webui_index                                        , name='base_page'),
	url(r'^page/(?P<rt>\w+)/(?P<uuid>\w+)$'                                 , webui_index                                        , name='resource_page'),
	
	
	#url(r'^page/(?P<rt>\w+)/(?P<uuid>\w+)/page_data$' 			, webui.views.page_data),  test di dbilli
	#url(r'^page/(?P<rt>\w+)/(?P<uuid>\w+)/primary$'			, webui.views.primary_timegraph),
	#url(r'^page/(?P<rt>\w+)/(?P<uuid>\w+)/nprimary$'			, webui.views.new_primary_timegraph),


	#
	# TAGS
	#
	url(r'^tags/'								, include('tags.urls')),

	#
	# TIMEGRAPHS
	#
	#url(r'^data/tg/(?P<uuid>\w+)$'						, webui.views.timegraph_data                        , name='webui.views.timegraph_data'),
	#For Django 1.11
	url(r'^data/tg/(?P<uuid>\w+)$'                                          , timegraph_data                                    , name='timegraph_data '),

	# 
	# SEARCH
	#
	url(r'^quick_search_autocomplete$'					, webui.views.quick_search_autocomplete             , name="quick_search_autocomplete"),
	url(r'^search_melement$'						, webui.views.search_melement_pagination            , name="webui.views.search_melement_pagination"),
	url(r'^search_conditions'						, webui.views.search_condition_pagination           , name="webui.views.search_condition_pagination"),

	#
	# EXTERNAL COMMANDS
	#
	url(r'^external/(?P<rt>\w+)/(?P<uuid>\w+)/(?P<scriptname>.+)$'		, webui.views.external.launch_external_script       , name='launch_external_script'),

	#
	# VARIA
	#
	url(r'^objects/'							, include('webui.views.objects.urls')),

	url(r'^part$'								, webui.views.op_get_part                           , name='webui.views.op_get_part'),

	url(r'^redirect/timegraph2node/(?P<uuid>.+)$'				, webui.views.redirects.timegraph_to_node           , name='webui.views.redirects.timegraph_to_node'), 
	url(r'^redirect/taguuidtoelement/(?P<uuid>\w+)$'			, webui.views.redirects.taguuid_to_element          , name='webui.views.redirects.taguuid_to_element'),
	url(r'^redirect/timegraph2conditions/(?P<uuid>.+)$'			, webui.views.redirects.timegraph_to_conditions     , name='webui.views.redirects.timegraph_to_conditions'), 

	url(r'^convert$'							, webui.views.svgutils.render_img                   , name='webui.views.svgutils.render_img'),

	url(r'^test_expr$'							, webui.views.test_expr                             , name='test_expr'),

	#
	# ICONS
	#
	#url(r'^icons/(?P<tenantid>\w+)/(?P<icon>.+)/(?P<color>.*)$'		, webui.views.icons.get_colored_icon3               , name='webui.views.icons.get_colored_icon3'),
	#url(r'^icons/(?P<tenantid>\w+)/(?P<icon>.+)/$'				, webui.views.icons.get_colored_icon3               , name='webui.views.icons.get_colored_icon3'),
	#For Django 1.11
	url(r'^icons/(?P<tenantid>\w+)/(?P<icon>.+)/(?P<color>.*)$'		, get_colored_icon3                                 , name='webui.views.icons.get_colored_icon3'),
	url(r'^icons/(?P<tenantid>\w+)/(?P<icon>.+)/$'				, get_colored_icon3                                 , name='webui.views.icons.get_colored_icon3'),
	
	#
	# DJANGO JAVASCRIPT
	#

	# javascript locales used by rest javascript code
	# Use "named" url because the view is used also in other subapps
	url(r'^i18n_js$'                                                        , django.views.i18n.javascript_catalog              , webui_i18n_js_settings, name='webui_js_catalog'),
] #)




