# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os
import time
import json
import ast
import logging
import datetime
from StringIO import StringIO
from operator import attrgetter

from django.conf import settings

from django.db.models import Q
from core.compatibility import reverse
from django.core.exceptions import ObjectDoesNotExist
from django.core.paginator import Paginator

from django.http import Http404
from django.http import HttpResponse, HttpResponseRedirect

from django.shortcuts import render_to_response

from django.contrib.auth import login
from django.contrib.auth.models import User
from django.contrib.auth.decorators import login_required

from django.template import Context, Template
from django.utils.translation import ugettext as _


import sanet_common
from sanet_common.sanetdefs.constants import ELEMENT_KIND_NODE, CSTATE_TO_CODENAME, CSTATE_FROM_CODENAME
from sanet_common.netutils import valid_partial_ip

from sanet_poller.utils.shell import load_symbol_from_line

import conf
from conf import get_configuration

conf = get_configuration()

from conf.settings import TIMEGRAPH_FORCE_AREA_TO_LINE

from core.models import Tenant, TenantSetting, TimeGraph
from core.models import UserCustomSetting
from core.models import Tenant, Agent
from core.models import ExpandedTitleModel
from core.management.commands.exec_expr import text_expression

from core.api.base import db_set_current_tenant_schema
from core.api.cache import init_cache_manager_from_conf
from core.api.datasource import DataSourceStoreException
from core.api.datasource import init_datasource_store_from_conf
from core.api.timegraph import calculate_timegraph_series_data

from core.api.sysstats import sanetd_get_current_state
from core.api.sysstats import entables_get_current_state
from core.api.sysstats import sanet_subsystem_get_current_state

from cli.clilib.lex import lex_join

from tags.models import TAG_FILTER

# Add methods and properityes to core models
from webui.models import *  
from webui.models import webui_search_node_name_by_ip
from webui.session import set_session_tenant_keys

from webui.api import get_page_instance
from webui.api import get_part
from webui.api.response import HttpJsonSuccessResponse, HttpJsonErrorResponse
from webui.api.utils import render_template

from webui.decorators import tenant_dependent

from webui.views.objects.conditions import get_states_palette
from webui.views.objects.datasources import get_new_datasource_dataset
from webui.views.objects.datasources import get_new_datasource_dataset_with_data

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

@login_required
@tenant_dependent
def index(request, rt=None, uuid=None):

	user   = request.user
	tenant = request.tenant

	# Default case, move to the tenant page
	if rt is None and uuid is None:
		url = tenant.page_url
		return HttpResponseRedirect( url )  

	#
	# Parametri per il rendering della pagina
	#

	# FIX: questi parametri hanno senso per alcune pagine.
	#      Fare refactory di questa view in sub-view per gestire i parametri solo
	#      per le "sezioni" che li usano.
	t = time.time()
	ts_to = request.GET_OR_POST().get('to')
	if ts_to is None:
		ts_to = int(t)
	else: 
		ts_to = int(ts_to)

	ts_from = request.GET_OR_POST().get('from')
	if ts_from is None:
		ts_from = int(t)-60*60*8
	else:
		ts_from = int(ts_from)

	critical_priority = tenant.get_setting(TenantSetting.SETTING_CRITICAL_LEVEL)
	minimum_priority  = tenant.get_setting(TenantSetting.SETTING_MINIMUM_LEVEL)

	cache_manager = init_cache_manager_from_conf() # FIX: get from main view

	page_settings = {

		'TIME'                       : str(time.time()),

		'DEBUG_MODE'                 : settings.DEBUG_JS, 
		
		'cache_manager'              : cache_manager,

		'prefix'                     : reverse('core.views.index'),

		'user'                       : user,

		'request'                    : request,

		'bar'                        : False,

		'tenant'                     : tenant,
		'tenant_time'                : time.time(),

		'rt'                         : rt,
		'rid'                        : uuid,

		'critical_priority'          : critical_priority,
		'minimum_priority'           : minimum_priority,

		'ts_from'                    : ts_from,
		'ts_to'                      : ts_to,

		'system_time_hh_mm'          : time.strftime("%H:%M"),
	}

	#
	# Istanzio la pagina
	#
	if settings.DEBUG:
		page = get_page_instance(rt, uuid, tenant, page_settings)
		return page.renderResponse(request, page_settings)
	else:
		try:
			page = get_page_instance(rt, uuid, tenant, page_settings)
			return page.renderResponse(request, page_settings)
		except ObjectDoesNotExist, e:
			raise Http404

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

@login_required
def global_index(request, tenantid, rt, uuid):
	"""
	Url globale per accedere ad una risorsa specifica di un tenant
	"""
	user = request.user

	#
	# L'utente loggato e' posizionato sul tenant di default.
	# Dobbiamo spostare la sessione sul tenant specifico e fare un redirect
	#

	# Vediamo se l'utente puo' accedere al tenant
	tenant = Tenant.objects.get(uuid=tenantid)
	
	if not user.get_profile().can_view_tenant(tenant):
		return HttpResponse("User '%s' cannot see tenant %s" % (user.username, tenant.name))

	# Ok, spostiamoci sul tenant
	set_session_tenant_keys(request.session, tenant)

	# Andiamo alla pagina
	local_url = reverse('resource_page', kwargs={'rt':rt, 'uuid':uuid})
	return HttpResponseRedirect(local_url)

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

@login_required
@tenant_dependent
def op_get_part(request):

	rt = request.GET_OR_POST().get('rt')
	rid = request.GET_OR_POST().get('uuid')

	data = get_part(rt, rid, request.tenant)

	return HttpResponse(json.dumps(data))	

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

@login_required
def system_status(request):

	now = time.time()

	system_status = {
		'hh_mm': time.strftime("%H:%M"),
	}

	if settings.SANETD_WATCHDOG_CONF['enabled'] == True:

		MAX_DELAY = settings.SANETD_WATCHDOG_CONF['valid_interval']

		info = []

		#
		# Stato di Sanetd 
		#
		for instance_id in settings.SANETD_INSTANCES:
			sanetd_state   = sanetd_get_current_state(instance_id=instance_id)
			if sanetd_state is not None:
				if now - sanetd_state['checktime'] <= MAX_DELAY:
					sanetd_state = sanetd_state['running']
					if sanetd_state is not True:
						info.append( 'SERVER: not running' )
				else:
					sanetd_state = False
					info.append( 'SERVER: last check too old' )
			else:
				sanetd_state = False
				info.append( 'SERVER: not running' )

		#
		# Stato di Entables
		#
		entables_state = entables_get_current_state()
		if entables_state is not None:
			if now - entables_state['checktime'] <= MAX_DELAY:
				entables_state = entables_state['running']
				if entables_state is not True:
					info.append( 'ENTABLES: not running' )
			else:
				entables_state = False
				info.append( 'ENTABLES: Last check too old' )
		else:
			entables_state = False
			info.append( 'ENTABLES: not running' )

		#
		# Stato degli Agenti
		#
		agents_state = True
		for t in Tenant.objects.all():

			t.db_set_current_schema()

			for a in Agent.objects.filter(tenant=t).all():

				if not a.is_enabled:
					continue

				s = a.get_current_state()

				if s['checktime'] is not None :
					if now - s['checktime'] <= MAX_DELAY:
						if s['running'] is not True:
							agents_state = False
							info.append('AGENT: "%s:%s" not running' % (t.name, a.name))
					else:
						agents_state = False 
						info.append('AGENT: "%s:%s" last check too old' % (t.name, a.name))
				else:
					agents_state = False 
					info.append('AGENT: "%s:%s" last check too old' % (t.name, a.name))

		#
		# Stato di Tagsd
		#
		dsstore_state = sanet_subsystem_get_current_state('datasourcestore')
		if dsstore_state is not None:
			if now - dsstore_state['checktime'] <= MAX_DELAY:
				dsstore_state = dsstore_state['running']
				if dsstore_state is not True:
					info.append( 'DSSTORE: not running' )
			else:
				dsstore_state = False
				info.append( 'DSSTORE: Last check too old' )
		else:
			dsstore_state = False
			info.append( 'DSSTORE: not running' )

		#
		# Stato di Tagsd
		#
		tagsd_state = sanet_subsystem_get_current_state('tagsd')
		if tagsd_state is not None:
			if now - tagsd_state['checktime'] <= MAX_DELAY:
				tagsd_state = tagsd_state['running']
				if tagsd_state is not True:
					info.append( 'TAGSD: not running' )
			else:
				tagsd_state = False
				info.append( 'TAGSD: Last check too old' )
		else:
			tagsd_state = False
			info.append( 'TAGSD: not running' )

		system_status['services'] = {

			# Component critici
			'sanetd'  : sanetd_state,
			'entables': entables_state,
			'agents'  : agents_state,

			# Componenti "raccomandati"
			'tagsd'   : tagsd_state,

			# datasource store
			'datasourcestore': dsstore_state,

			'info'    : '\n'.join(info),
		}

	data = json.dumps(system_status)

	return HttpResponse(data)

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

@login_required
def switch_user(request):

	if not request.user.is_superuser:
		raise Exception("Switch not allowed for this user")

	u = User.objects.get(username=request.GET_OR_POST()['username'])
	# Django 1.11 - u.backend = 'django.contrib.auth.backends.ModelBackend'

	login( request, u)

	return HttpResponseRedirect ( reverse("base_page") )

@login_required
def switch_user_form(request):

	current_user = request.user

	vname = "webui.views.switch_user"
	kwargs = {}
	url = reverse(vname, args=[], kwargs=kwargs)

	if request.user.is_superuser:
		users = User.objects.filter(is_active=True).exclude(id=current_user.id)
		users = users.order_by('username')
	else:
		users = []

	context = {
		'users'       : users,
		'form_action' : url
	}
	return render_to_response('html/user_switch.html', context)

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

@login_required
def switch_tenant(request):

	new_tenant = None

	if request.GET_OR_POST().has_key('tenantid')     : new_tenant = Tenant.objects.get(uuid=request.GET_OR_POST()['tenantid'])
	elif request.GET_OR_POST().has_key('tenantname') : new_tenant = Tenant.objects.get(name=request.GET_OR_POST()['tenantname'])

	if new_tenant is None:
		raise Exception("Unable to find requested tenant: %s" % (request.GET_OR_POST().get('tenantid', request.GET_OR_POST().get('tenantname',''))))

	user = request.user

	if not user.get_profile().can_view_tenant(new_tenant):
		return HttpResponse("User '%s' cannot see tenant %s" % (user.username, new_tenant.name))

	set_session_tenant_keys(request.session, new_tenant)

	return HttpResponseRedirect(reverse('base_page'))

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

@login_required
def change_password(request):

	try:
		user          = request.user

		password      = request.GET_OR_POST().get('password')
		new_password  = request.GET_OR_POST().get('new_password')
		new_password2 = request.GET_OR_POST().get('new_password2')

		if not new_password:
			raise Exception( _("Invalid password!"))

		if not (new_password == new_password2):
			raise Exception( _("New passwords do not match!"))

		if not validate_password(user, new_password):
			raise Exception( _("Try a stronger password") )

		if not user.check_password(password):
			raise Exception( _("Invalid password!"))

		user.set_password(new_password)
		user.save()

		return HttpResponse("OK")
	except Exception, e:
		return HttpResponse(str(e))

def validate_password(user, p):

	if  not p: return False

	if len(p) < 7: return False

	if user.username == p: return False

	# TODO: add tests
	
	return True

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

def _checked(val):
	if val == "on":
		return 1
	return 0

@login_required
def change_settings(request):

	try:
		user          = request.user

		#
		# FIX: MUST use django forms to validate this input
		#
		
		d = {
			'page.show_explore_bars'                   : _checked(request.GET_OR_POST().get('page.show_explore_bars'        , None)),

			'page.tabs.show_title'                     : _checked(request.GET_OR_POST().get('page.tabs.show_title'          , None)),

			'page.tenant.troubleshoot_view'            : _checked(request.GET_OR_POST().get('page.tenant.troubleshoot_view' , None)),
			'page.tenant.details_expanded'             : _checked(request.GET_OR_POST().get('page.tenant.details_expanded'  , None)),

			'page.element.troubleshoot_view'           : _checked(request.GET_OR_POST().get('page.element.troubleshoot_view', None)),
			'page.element.details_expanded'            : _checked(request.GET_OR_POST().get('page.element.details_expanded' , None)),

			'page.alarms_list.show_last_notable_change': _checked(request.GET_OR_POST().get('page.alarms_list.show_last_notable_change'     , None)),

			'page.tag.troubleshoot_view'               : _checked(request.GET_OR_POST().get('page.tag.troubleshoot_view'    , None)),
			'page.tag.details_expanded'                : _checked(request.GET_OR_POST().get('page.tag.details_expanded'     , None)),

			'page.badge_sort_type'                     : request.GET_OR_POST().get('page.badge_sort_type'                   , None),
		}

		UserCustomSetting.save_user_settings_dict(user, d, tenant=None)

		return HttpJsonSuccessResponse('Updated')

	except Exception, e:
		return HttpJsonErrorResponse(str(e))

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

@login_required
@tenant_dependent
def timegraph_data(request, uuid):

	return _timegraph_data(request, uuid)

# risluzione massima in un giorno
MAX_RESOLUTION = 1440
# aggregazione per 30 minuti, se guardo piu' di un giorno sto' valutando un anadamento. Non serve avere una risoluzione massima.
AGGREGATED_STEP = 30

def _timegraph_data(request, uuid):

	tenant = request.tenant

	db_set_current_tenant_schema(tenant.name)

	add_zero = request.GET_OR_POST().get('zero')
	g_type   = request.GET_OR_POST().get('type', None)
	ts_start = request.GET_OR_POST().get('from', None)
	ts_end   = request.GET_OR_POST().get('to', None)

	if ts_start == None: ts_start = time.time() - 60*60*8
	ts_start = int(ts_start)

	if ts_end == None: ts_end = time.time()
	ts_end = int(ts_end)

	try:
		tglist = TimeGraph.objects.get(uuid = uuid)

	except TimeGraph.DoesNotExist, e:
		tglist = TimeGraph.get_by_path(uuid) #uuid could be a path

	data = render_new_timegraph([tglist], uuid, tenant, add_zero, g_type, ts_start, ts_end)

	# aggregate data

	series = data['series']
	new_series = []
	aggregated_step=AGGREGATED_STEP
	for s in series:
		values = s['values']
		if len(values) > MAX_RESOLUTION:
			new_values = computed_values(values,aggregated_step=aggregated_step)
			step = s['timescale']['step']
			s['timescale']['step'] = step * aggregated_step
		else:
			new_values = values

		s['values'] = new_values

		new_series.append(s)

	data['series'] = new_series

	return HttpResponse(json.dumps(data))

# funzione per aggregare i dati dei timegraph
def computed_values(values, aggregated_step=10):
    
	run = len(values)/aggregated_step
     
	overead = len(values)-run*aggregated_step
    
	new_values = []
    
	for i in range(0,run):
		tmp = values[i*aggregated_step:(i+1)*aggregated_step]
		tmpval = max(tmp)
		new_values.append(tmpval)

	return new_values

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

def render_new_timegraph(tglist, uuid, tenant, add_zero, g_type, ts_start, ts_end, is_primary=False):

	datasource_store = init_datasource_store_from_conf()

	cache_manager    = init_cache_manager_from_conf()

	ret_series = []
	title = ''
	p_uuid = ''

	for tg in tglist:

		dg = tg.datagroup

		unit = tg.ylegend

		title = ""

		p_uuid = tg.uuid

		if not is_primary and tg.element.parent is not None :
			title = title + tg.element.parent.name + " - "

		title = title + tg.element.name + " - " + tg.name 

		series = calculate_timegraph_series(datasource_store, tg, ts_start, ts_end, g_type=g_type, cache_manager=cache_manager)

		ret_series += series

	presel_end = time.time()

	ret = {
		'title':title,
		'flags': {
			'timeline': not is_primary, 				# //display timeline or not, default true
			'controls': True, 				# // display conrtol buttons or not, default true
			'locked'  : False, 				#// if the user can make selections and change range, default false
			'dataview': not is_primary,
			'legend'  : True,
		},
		'url'   : reverse('webui.views.timegraph_data',kwargs={'uuid':p_uuid} ), 					#// if missing the map goes static, with both uuid and url it's dynamic
		'series': ret_series,
		'presel': {
			'from': presel_end-60*60,
			'to'  : presel_end
		}, 					#// key of the intervals, defaults to 1h
		'scale': {            				#// mandatory as structure
			'min' : None,     			#// mandatory but can be null
			'max' : None,     			#// mandatory but can be null
			'log' : False,     			#//if scale is logaritmic, handled via Flotr2, defaults to false
			'unit': unit,					#// can be  missing/null
		},
		#'zorder':     [ number ] 			#// could be implied by series (i.e. missing)
	}

	return ret


def calculate_timegraph_series(datasource_store, tg, ts_start, ts_end, g_type=None, include_errors=False, cache_manager=None):

	ret_series = []

	datasources_by_name, series_by_name = calculate_timegraph_series_data(datasource_store, tg, ts_start, ts_end, g_type=g_type, include_errors=include_errors, cache_manager=cache_manager)

	#
	# Creiamo il json per la struttura dati finale
	#
	
	# Le leggende possono contenere simboli da espandere ($node, $iface, {rttmax@}, ecc.) . Vediamo di caricarli
	title_symbols = tg.element.get_specific_element().get_title_symbols_dict()
	
	# Tra i simboli espandibili ci sono anche i valori dei datasource (es: {foo@})
	for ds in datasources_by_name.values():
		ds.load_fields_from_cache(cache_manager)
		title_symbols['{%s@}' % (ds.name) ] = ds.lastvalue

	for s in sorted( series_by_name.values() , key=attrgetter('name') ):

		# FIX: le serie "invisible" non sono gestite bene lato client dal widget.
		#      Per il momento non le inviamo nemmeno, ma potrebbero esserci dei problemi di
		#      "scala" se non visualizziamo dei dati che in realta' hanno contribuito
		#      a creare il grafico cosi' come lo vediamo.
		if s.style == 'invisible':
			continue

		ds = datasources_by_name.get(s.variable_value) 

		title_ds = ds.path if hasattr(ds, 'path') else s.name

		ds_data = s.ds_data

		ts_start = ds_data['scalex'][0]
		ts_end   = ds_data['scalex'][1]
		step     = ds_data['scalex'][2]

		# Expand legend with symbols ($node, {..@}, ecc..)
		title_symbols2 = {} 
		title_symbols2.update(title_symbols)
		if ds:
			title_symbols2.update(ds.get_title_symbols_dict())

		legend = ExpandedTitleModel.subst_title(s.legend, title_symbols2)
		
		serie_data = get_new_datasource_dataset_with_data(
			title_ds,
			ts_start, 
			ts_end, 
			step, 
			ds_data['dataseries'],
			style   = g_type if (g_type is not None and s.style != 'tick') else s.style,
			color   = s.color, 
			ylegend = tg.ylegend, 
			legend  = legend,
		)
		
		serie_data['order'] = s.render_order

		if include_errors:
			if ds_data.has_key('errors'):
				serie_data['errors'] = ds_data['errors']

		ret_series.append(serie_data)

	return ret_series

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

def GUI_SETTINGS(settings):
	d = {
		'main-graph-height': 90,
	
		'show-title'       : True,
		'show-yaxis'       : True,
		'show-controls'    : True,
		'show-timeline'    : False,
		'show-perc-yaxis'  : False,
		'show-legend'      : True,
		'expanded'         : False,
		'virtual-max-value': None
	}
	d.update(settings)
	return d

def RENDER_STATE():
	s = {
		#'_' : None,
		'ts_start'          : None,
		'ts_end'            : None,
		
		#'step'              : 60,
		#'selection_locked'  : False, 
		#'selection_ts_start': None,
		#'selection_ts_end'  : None,
	}
	return s

def TIMEGRAPH_DATA(title='', ylegend='', series=[], render_state={}, gui_settings={}, highlight_times=None):
	
	ret = {
		'title'       : title,
		'ylegend'     : ylegend,
		'series'      : series,
		'render_state': render_state,
		'svg_url'     : reverse('webui.views.svgutils.render_img'),
		'gui-settings': GUI_SETTINGS(gui_settings),
	}
	
	if highlight_times is not None:
		ret['highlight_times'] = highlight_times
		
	return ret

def TIMEGRAPH_SERIE_DATA(title='', label='', values=[], format='line', color='#000000', ts_start=0, ts_end=0, ts_step=0, order=0):

	ret = {
		'format'    : format, 

		'order'     : order,

		'title'     : title,
		'label'     : label, 

		'values'    : values, 

		'color'     : color, 

		'timescale': {        			#// all mandatory
			'from': ts_start, 		#// Python timestamp
			'to'  : ts_end, 		#// Python timestamp
			'step': ts_step,  		#// Python timestamp
		},		
        }
	return ret



def render_new_timegraph3(tg, ts_start, ts_end, render_state):

	datasource_store = init_datasource_store_from_conf()
	cache_manager    = init_cache_manager_from_conf()

	now = int( time.time() )

	#
	# Lo scopo e' scaricare solo i dati in piu' rispetto a quelli gia' visualizzati dal widget
	#
	if render_state is None:
		render_state = RENDER_STATE()

	time_span = 60 * 60 * 8

	# Se non ho specificato un range, calcolo il range in base al render_state che mi hanno dato
	if ts_start == None and ts_end == None:

		# Ho specificato entrambi gli estremi
		if render_state['ts_end'] is not None and render_state['ts_start'] is not None:
			time_span = render_state['ts_end'] - render_state['ts_start']
		elif render_state['ts_start'] is not None and render_state['ts_start'] < 0:
			time_span = render_state['ts_start'] * -1

		data_ts_end   = now
		data_ts_start = data_ts_end - time_span

		render_state['ts_start'] = - time_span
		render_state['ts_end'  ] = None

	else:
		ts_start = int(ts_start)
		ts_end = int(ts_end)
		if ts_start < 0:
			data_ts_end   = now
			data_ts_start = data_ts_end + ts_start
		else:
			data_ts_start = ts_start
			data_ts_end   = ts_end

		render_state['ts_start'] = ts_start
		render_state['ts_end'  ] = ts_end
	

	g_type = 'line' if settings.TIMEGRAPH_FORCE_AREA_TO_LINE else None
	if 'g_type' in render_state:
		g_type = render_state['g_type']
	
	series = calculate_timegraph_series(datasource_store, tg, data_ts_start, data_ts_end, g_type=g_type, cache_manager=cache_manager)

	highlight_times_data = calculate_highlight_times_data(data_ts_start, data_ts_end, tg.tenant)
	
	ret = TIMEGRAPH_DATA(**{
	
		'title' : tg.expanded_title(cache_manager=cache_manager),
		
		'ylegend': str(tg.ylegend),

		'series': series,

		'render_state': render_state,
		
		#'svg_url': reverse('webui.views.svgutils.render_img'),
		
		'gui_settings': GUI_SETTINGS({
			'show-title'   : True,
			'show-controls': True,
			'show-timeline': True,
			'show-perc-yaxis': False ,
			
			'expanded': False,
			'virtual-max-value': None
		}),
		
		'highlight_times': highlight_times_data,
	})

	return ret


def render_new_timegraph3_datasource(ds, ts_start, ts_end, render_state):

	datasource_store = init_datasource_store_from_conf()
	cache_manager    = init_cache_manager_from_conf()

	now = int( time.time() )

	#
	# Lo scopo e' scaricare solo i dati in piu' rispetto a quelli gia' visualizzati dal widget
	#
	if render_state is None:
		render_state = RENDER_STATE()

	time_span = 60 * 60 * 8

	# Se non ho specificato un range, calcolo il range in base al render_state che mi hanno dato
	if ts_start == None and ts_end == None:

		# Ho specificato entrambi gli estremi
		if render_state['ts_end'] is not None and render_state['ts_start'] is not None:
			time_span = render_state['ts_end'] - render_state['ts_start']
		elif render_state['ts_start'] is not None and render_state['ts_start'] < 0:
			time_span = render_state['ts_start'] * -1

		data_ts_end   = now
		data_ts_start = data_ts_end - time_span

		render_state['ts_start'] = - time_span
		render_state['ts_end'  ] = None

	else:
		data_ts_start = ts_start
		data_ts_end   = ts_end

		render_state['ts_start'] = ts_start
		render_state['ts_end'  ] = ts_end

	try:
		data = get_new_datasource_dataset(ds.tenant, ds, ts_start=data_ts_start, ts_end=data_ts_end, datasource_store=datasource_store, color='#0000ff')

		title = ds.name
		label = ds.name

		tg_series = [ data ]

	except DataSourceStoreException, e:

		title = "%s (NO DATA)" % (ds.name)
		label = title

		values = [ None for i in xrange( int((float(data_ts_end) - float(data_ts_start))/60) ) ]

		empty_serie = TIMEGRAPH_SERIE_DATA(title=title, label=label, values=values, format='line', color='#0000ff', ts_start=data_ts_start, ts_end=data_ts_end, ts_step=60, order=0)

		tg_series = [ empty_serie ]


	highlight_times_data = calculate_highlight_times_data(data_ts_start, data_ts_end, ds.tenant)
	
	ret = TIMEGRAPH_DATA(**{
		'title'       : title,
		'ylegend'     : '',
		'series'      : tg_series, 
		'render_state': render_state,
		'gui_settings': GUI_SETTINGS({ 
			'show-title'   : True,
			'show-controls': True, 
			'show-timeline': True, 
			'show-yaxis'   : True,
		}),

		'highlight_times': highlight_times_data,
	})

	return ret



def render_new_timegraph3_condition(c, ts_start, ts_end, render_state):

	cache_manager    = init_cache_manager_from_conf()

	now = int( time.time() )

	#
	# Lo scopo e' scaricare solo i dati in piu' rispetto a quelli gia' visualizzati dal widget
	#
	if render_state is None:
		render_state = RENDER_STATE()

	time_span = 60 * 60 * 8

	# Se non ho specificato un range, calcolo il range in base al render_state che mi hanno dato
	if ts_start == None and ts_end == None:

		# Ho specificato entrambi gli estremi
		if render_state['ts_end'] is not None and render_state['ts_start'] is not None:
			time_span = render_state['ts_end'] - render_state['ts_start']
		elif render_state['ts_start'] is not None and render_state['ts_start'] < 0:
			time_span = render_state['ts_start'] * -1

		data_ts_end   = now
		data_ts_start = data_ts_end - time_span

		render_state['ts_start'] = - time_span
		render_state['ts_end'  ] = None

	else:
		data_ts_start = ts_start
		data_ts_end   = ts_end

		render_state['ts_start'] = ts_start
		render_state['ts_end'  ] = ts_end
	
	
	changes_sequence = c.logchanges_sequence(data_ts_start, data_ts_end)
	
	# states_mapping serve per visualizzare alcuni stati come altri stati. Es. mostrare i FA come UP o gli UN come UP (o DN).
	# states_mapping e' un dizionario con chiave lo stato originario e come valore lo stato come lo si vuole mostrare.
	if 'states_mapping' in render_state:
		
		tmp_changes_sequence = []
		states_mapping = render_state['states_mapping']
		
		for item in changes_sequence:
			
			if CSTATE_TO_CODENAME(item[3]) in states_mapping:
				state_to_change = CSTATE_TO_CODENAME(item[3])
				tmp_changes_sequence.append((item[0], item[1], item[2], CSTATE_FROM_CODENAME(states_mapping[state_to_change])))
				
			else:
				tmp_changes_sequence.append(item)
		changes_sequence = tmp_changes_sequence

	color_palette = get_states_palette(changes_sequence)

	serie_data = TIMEGRAPH_SERIE_DATA(title=c.title, label=c.name, values=changes_sequence, format='changesequence', color=color_palette, ts_start=data_ts_start, ts_end=data_ts_end, ts_step=0, order=0)

	highlight_times_data = calculate_highlight_times_data(data_ts_start, data_ts_end, c.tenant)

	ret = TIMEGRAPH_DATA(**{
		'title'       : c.expanded_title(cache_manager=cache_manager),
		'ylegend'     : '',
		'series'      : [ serie_data], 
		'render_state': render_state,
		'gui_settings': GUI_SETTINGS({ 
			'main-graph-height': 30,
			'show-yaxis'       : False,
			'show-title'       : True,
			'show-controls'    : True, 
			'show-timeline'    : True, 
			'show-legend'      : False,
		}),
		'highlight_times': highlight_times_data,
	})

	return ret

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

def render_new_timegraph3_webscenario(webscenario, ts_start, ts_end, render_state):

	now = int( time.time() )

	#
	# Lo scopo e' scaricare solo i dati in piu' rispetto a quelli gia' visualizzati dal widget
	#
	if render_state is None:
		render_state = RENDER_STATE()

	time_span = 60 * 60 * 8

	# Se non ho specificato un range, calcolo il range in base al render_state che mi hanno dato
	if ts_start == None and ts_end == None:

		# Ho specificato entrambi gli estremi
		if render_state['ts_end'] is not None and render_state['ts_start'] is not None:
			time_span = render_state['ts_end'] - render_state['ts_start']
		elif render_state['ts_start'] is not None and render_state['ts_start'] < 0:
			time_span = render_state['ts_start'] * -1

		data_ts_end   = now
		data_ts_start = data_ts_end - time_span

		render_state['ts_start'] = - time_span
		render_state['ts_end'  ] = None

	else:
		data_ts_start = ts_start
		data_ts_end   = ts_end

		render_state['ts_start'] = ts_start
		render_state['ts_end'  ] = ts_end


	ws_data = webscenario.fetch_data(ts_start = data_ts_start, ts_end = data_ts_end )

	#
	# Mostriamo tutte le serie pero' "stacked" una sopra l'altra tutte con colori diversi. 
	#
	# Dobbiamo "massaggiare" i dati.
	#
	series = []
	
	steps_values = None
	
	steps = webscenario.step_set.order_by('order')
	num_steps = len(steps)
	for step in steps:
		step_data = step.fetch_data(ts_start = data_ts_start, ts_end = data_ts_end )
		
		if steps_values is None:
			steps_values = step_data['dataseries']
		else:
			for i in xrange(len(step_data['dataseries'])):
				v = step_data['dataseries'][i]
				if steps_values[i] is None:
					steps_values[i] = v
				else:
					steps_values[i] += (v if v != None else 0)
				
		steps_values2 = [ v for v in steps_values ]
		serie_def = get_new_datasource_dataset_with_data(
			str(step.order),
			ws_data['scalex'][0], # start
			ws_data['scalex'][1], # end
			ws_data['scalex'][2], # step
			steps_values2,
			legend=step.description,
			style='area',
		)
		series.append(serie_def)

	series.reverse()

	#
	# Serie globale
	#
	# IMPORTANTE: per problemi di "media" il tempo totale associato al webscenario
	#             potrebbe essere minore della somma dei tempi degli step. Aggiustiamo il valore.
	#
	d = []
	for i in xrange(len(ws_data['dataseries'])):
		v1 = ws_data['dataseries'][i]
		v2 = steps_values[i]
		if v1 and v2:
			d.append(max(v1,v2))
		else:
			d.append(v1)
	
	ws_serie_def = get_new_datasource_dataset_with_data(
		'Total Execution time',
		ws_data['scalex'][0], # start
		ws_data['scalex'][1], # end
		ws_data['scalex'][2], # step
		d,
		color='#000000',
		legend='TOTALE',
	)

	series.append(ws_serie_def)

	#series.reverse()

	"""
	steps_ticks_fatal    = [ (1 if v == num_steps else 0) for v in steps_values ]
	steps_ticks_critical = [ (1 if v > 0          else 0) for v in steps_values ]

	serie_fatal = get_new_datasource_dataset_with_data(
		'Fatal',
		data['scalex'][0], # start
		data['scalex'][1], # end
		data['scalex'][2], # step
		steps_ticks_fatal,
		style='tick',
		color='red',
		legend='Fatal',
	)

	serie_critical = get_new_datasource_dataset_with_data(
		'Fail',
		data['scalex'][0], # start
		data['scalex'][1], # end
		data['scalex'][2], # step
		steps_ticks_critical,
		style='tick',
		color='yellow',
		legend='Fail',
	)
	"""

	highlight_times_data = calculate_highlight_times_data(data_ts_start, data_ts_end, webscenario.tenant)
	
	ret = TIMEGRAPH_DATA(**{
		'title'       : "%s : %s" % (webscenario.name, webscenario.description),
		'ylegend'     : 's',
		'series'      : series, 
		'render_state': render_state,
		'gui_settings': GUI_SETTINGS({ 
			'show-yaxis'       : True,
			'show-title'       : True,
			'show-controls'    : True, 
			'show-timeline'    : True, 
			'show-legend'      : False,
		}),
		
		'highlight_times': highlight_times_data,
	})

	return ret



def render_new_timegraph3_step(webscenario_step, ts_start, ts_end, render_state):

	now = int( time.time() )

	#
	# Lo scopo e' scaricare solo i dati in piu' rispetto a quelli gia' visualizzati dal widget
	#
	if render_state is None:
		render_state = RENDER_STATE()

	time_span = 60 * 60 * 8

	# Se non ho specificato un range, calcolo il range in base al render_state che mi hanno dato
	if ts_start == None and ts_end == None:

		# Ho specificato entrambi gli estremi
		if render_state['ts_end'] is not None and render_state['ts_start'] is not None:
			time_span = render_state['ts_end'] - render_state['ts_start']
		elif render_state['ts_start'] is not None and render_state['ts_start'] < 0:
			time_span = render_state['ts_start'] * -1

		data_ts_end   = now
		data_ts_start = data_ts_end - time_span

		render_state['ts_start'] = - time_span
		render_state['ts_end'  ] = None

	else:
		data_ts_start = ts_start
		data_ts_end   = ts_end

		render_state['ts_start'] = ts_start
		render_state['ts_end'  ] = ts_end

	data = webscenario_step.fetch_data(ts_start = data_ts_start, ts_end = data_ts_end )

	serie_data = get_new_datasource_dataset_with_data(
		webscenario_step.order,
		data['scalex'][0], # start
		data['scalex'][1], # end
		data['scalex'][2], # step
		data['dataseries'],
	)

	#serie_timeout = get_new_datasource_dataset_with_data(
	#	webscenario_step.order,
	#	data['scalex'][0], # start
	#	data['scalex'][1], # end
	#	data['scalex'][2], # step
	#	[ (1 if v is None else 0) for v in data['dataseries'] ],
	#	style='tick',
	#	color='red',
	#	legend='Timeout',
	#)

	highlight_times_data = calculate_highlight_times_data(data_ts_start, data_ts_end, webscenario_step.tenant)
	
	ret = TIMEGRAPH_DATA(**{
		'title'       : "%s : %s" % (webscenario_step.order, webscenario_step.description),
		'ylegend'     : 's',
		#'series'      : [ serie_timeout, serie_data], 
		'series'      : [ serie_data ], 
		'render_state': render_state,
		'gui_settings': GUI_SETTINGS({ 
			'show-yaxis'       : True,
			'show-title'       : True,
			'show-controls'    : True, 
			'show-timeline'    : True, 
			'show-legend'      : False,
		}),
		
		'highlight_times': highlight_times_data,
	})

	return ret



#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

def search_names_by_ip(tenant, term):
	"""
	Search monitored nodes or discovered nodes with ip matching.

	@param term: string with IPv4 or IPv4 mask. Format 'x.x.x.x/x'
	@return: names list
	"""

	# Fix string "192.168." -> "192.168"
	partial_ip = term.rstrip('.')  

	if not valid_partial_ip(partial_ip):
		return []

	partial_ip = partial_ip.encode('ascii')

	mask = partial_ip + "/%s" % (len(partial_ip.split('.'))*8)

	names = []

	#
	# Dati di monitoraggio
	#
	names += list(MonitoredNode.objects.filter(Q(name__contains=partial_ip) | Q(ip4_mgt__contains=partial_ip) | Q(effective_ip__contains=partial_ip)).values_list('name', flat=True))

	# Ricerca tramite networkmask
	conds = ' or '.join([ 
		"ip4_mgt::INET      <<= inet %s ",
		"effective_ip::INET <<= inet %s ",
	])
	params = [
		mask,
		mask,
	]
	names += list(MonitoredNode.objects.all().extra(where=[conds], params=params).values_list('name', flat=True))

	#
	# Search IP in extensions
	#
	signal_ret_data = webui_search_node_name_by_ip.send(None, tenant=tenant, partial_ip=partial_ip, mask=mask)
	for handler, ret_names in signal_ret_data:
		names.extend(ret_names)

	#
	# Remove duplicates
	#
	names = list(set(names))

	return names


@tenant_dependent
def search_melement_pagination(request):

	user   = request.user
	tenant = request.tenant

	search_key = request.GET['search_key']
	page = request.GET['page']

	elements = MonitoredElement.objects
	elements = TAG_FILTER(tenant, user, elements, "monitoredelement")

	filters = Q() 
	filters = filters | Q(name__icontains=search_key) 
	filters = filters | Q(parent__name__icontains=search_key) 
	filters = filters | Q(description__icontains=search_key)

	names = search_names_by_ip(tenant, search_key)
	if names:
		filters = filters | Q(name__in=names)

	elements = list(elements.filter(filters).all().order_by('kind', 'parent', 'name').select_related('parent'))
	p_elem = Paginator(elements, 50)

	c = {
		'elements': p_elem.page(page)
	}

	template = """
	{% comment %}{% load url from future %}{% endcomment %}
	{% for e in elements %}
			<tr  class='gui-tr'>
				<td  class='gui-td' >

					<img class="icon16x16" src="{{ e.get_icon_url }}" >
					{% if e.parent %}

						{% with e.parent as e %}
						<a href="{% url 'resource_page' rt=e.resource_type uuid=e.resource_id %}" >
							{{ e.name }}
						</a>
						{% endwith %}
						:
					{% endif %}

					<a href="{% url 'resource_page' rt=e.resource_type uuid=e.resource_id %}" >
						{{ e.name }}
					</a>
				</td>
				<td  class='gui-td' >{{ e.description }} </td>
			</tr>
	{% endfor %}
	"""

	template = Template(template)
	#c = Context(c)
	html = template.render(c)

	data = {
		'html': html,
		'next_page': p_elem.page(page).next_page_number(),
		'fetched': len(p_elem.page(page).object_list)
	}

	return HttpResponse(json.dumps(data))


#@tenant_dependent
def search_condition_pagination(request):
	from django.core.paginator import Paginator
	from django.template import Context, Template

	user = request.user
	tenant = request.tenant

	search_key = request.GET['search_key']
	page = request.GET['page']

	filters = Q() | Q(name__icontains=search_key) | Q(parent__name__icontains=search_key) | Q(
		description__icontains=search_key)

	names = search_names_by_ip(tenant, search_key)
	if names:
		filters = filters | Q(name__in=names)

	conditions = Condition.objects
	conditions = TAG_FILTER(tenant, user, conditions, "condition")

	count = conditions.filter(Q(path__icontains=search_key)).count()
	conditions = list(conditions.filter(Q(path__icontains=search_key)))
	c_elem = Paginator(conditions, 50)

	c = {
		'conditions': c_elem.page(page)
	}

	template = """
	{% load url from future %}
	{% for e in conditions %}
			<tr  class='gui-tr'>
				<td  class='gui-td' >
					<img class="icon16x16" src="{{ e.get_icon_url }}" >

					<a href="{% url 'resource_page' rt=e.resource_type uuid=e.resource_id %}" >
						{{ e.path }}
					</a>
				</td>
				<td  class='gui-td' >{{ e.description }} </td>
			</tr>
	{% endfor %}
	"""

	template = Template(template)
	c = Context(c)
	html = template.render(c)

	data = {
		'html': html,
		'next_page': c_elem.page(page).next_page_number(),
		'fetched': len(c_elem.page(page).object_list)
	}

	return HttpResponse(json.dumps(data))

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

@tenant_dependent
def quick_search_autocomplete(request):

	user   = request.user

	tenant = request.tenant
	db_set_current_tenant_schema(tenant.name)

	MAX = 5

	def _cut(s):
		if len(s) > 30:
			return s[0:30]+"..."
		return s


	term = request.GET_OR_POST()['term']
	term = term.strip()

	data = []
	
	#
	# Filtraggi
	#

	# query set base (filtrata per visibilita dell'utente)
	elements = MonitoredElement.objects.filter(tenant=tenant)
	elements = TAG_FILTER(tenant, user, elements, 'monitoredelement')

	filters = Q()

	# Se la parola chiave contiene un ":" forse l'utente ha scritto: <nodo>:<sotto elemento>
	if term.find(":") > -1:
		node_term, sub_term = term.split(":", 1)

		node_filter = Q(parent__name__icontains=node_term) | Q(parent__description__icontains=node_term)
		sub_filter  = Q(name__icontains=sub_term)          | Q(description__icontains=sub_term )
		filters = filters | Q(node_filter & sub_filter)
	else:
		filters = filters | Q(name__icontains=term) | Q(description__icontains=term)

	# Se l'utente ha inserito un indirizzo IP cerchiamo ci capire se c'e' qualcosa in sanet
	# che e' visto/monitorato con quell'IP.
	names = search_names_by_ip(tenant, term)
	if names:
		filters = filters | Q(name__in=names)

	elements = elements.filter(filters)


	#
	# Elaborazione risultati
	#

	tot_results = elements.count()

	elements = elements.order_by('kind', 'parent', 'name')

	elements = elements[0:MAX]
	if len(elements) == 1:

		e = elements[0]

		descr = _cut(e.description)

		data.append({
			'icon' : e.get_icon_url(),
			'label': e.path,
			'descr': descr or '',
			'url'  : reverse('resource_page', kwargs={ 'rt': e.resource_type, 'uuid': e.uuid } )
		})

		if e.kind == ELEMENT_KIND_NODE and term in e.name:
			subelements = MonitoredElement.objects.filter(tenant=tenant, parent=e)
			subelements = TAG_FILTER(tenant, user, subelements, 'monitoredelement')

			for sube in subelements:
				descr = sube.description
				data.append({
					'icon' : sube.get_icon_url(),
					'label': sube.path,
					'descr': _cut(descr) or '',
					'url'  : reverse('resource_page', kwargs={ 'rt': sube.resource_type, 'uuid': sube.uuid } )
				})
	else:
		for e in elements:
			descr = _cut(e.description)

			data.append({
				'icon' : e.get_icon_url(),
				'label': e.path,
				'descr': descr or '',
				'url'  : reverse('resource_page', kwargs={ 'rt': e.resource_type, 'uuid': e.uuid } )
			})

	data = sorted(data, key=lambda k: k['label'])

	#
	# Aggiungiamo una entry 
	#
	if tot_results > MAX:
		data.append({
			'more_results' : 1
		})

	data = json.dumps(data)

	return HttpResponse(data)

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

@tenant_dependent
def test_expr(request):

	tenant = request.tenant
	db_set_current_tenant_schema(tenant.name)

	element       = None
	parameters    = ""
	expr          = ""
	value         = None
	infos         = None
	output_string = None

	if request.POST:

		log_level  = int(request.POST['log_level'])
		element    = request.POST['element']
		parameters = request.POST['parameters']
		expr       = request.POST['expr']
		
		symbols = {}
		for line in parameters.split('\n'):
			load_symbol_from_line(symbols, line)

		logger = None
		log_hdlr = None
		try:

			output_stream   = StringIO()
			
			format_str = '%(asctime)s %(levelname)s> %(message)s'
			logger = logging.getLogger('test_expr')
			logger.setLevel( log_level )
			log_hdlr = logging.StreamHandler(output_stream)
			log_hdlr.setFormatter( logging.Formatter(format_str) )
			log_hdlr.setLevel( log_level )
			logger.addHandler(log_hdlr)
			
			value, infos = text_expression(logger, tenant, element, expr, symbols)
			output_string = output_stream.getvalue()
		except Exception, e:
			output_string = output_stream.getvalue() + '\n' + str(e)
		finally:
			if logger:
				logger.removeHandler(log_hdlr)

		if not output_stream.closed:
			output_stream.close()
	else:
		parameters = '\n'.join([
			 '$node=localhost'
			,'$community=public'
			,'$snmpversion=1'
			,'$snmpport=161'
		])
		expr       = '1.3.6.1.2.1.1.1.@'

	if output_string is not None:
		
		output_string = output_string.split('\n')

	elements = MonitoredElement.objects.select_related('parent').order_by('parent__name', 'kind', 'name').all()
	
	c = { 
		'element'   : element , 
		'elements'  : elements,
		'parameters': parameters,
		'expr'      : expr,
		'cli_expr'  : lex_join(['expr', expr]),
		'value'     : value,
		'infos'     : infos,
		'output'    : output_string, #output_string.split('\n') if output_string is not None else None,
	}
	html = render_template('html/parts/test_expr.html', c )

	return HttpResponse(html)
