# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import urlparse

from django import forms

from django.conf import settings
from django.http import HttpResponseRedirect
from django.template.response import TemplateResponse
from django.views.decorators.debug import sensitive_post_parameters
from django.views.decorators.cache import never_cache
from django.views.decorators.csrf import csrf_protect
from django.utils.translation import ugettext as _

#from django.contrib.sites.models import get_current_site  removed django 1.11

# Avoid shadowing the login() and logout() views below.
from django.contrib.auth import REDIRECT_FIELD_NAME, login as auth_login

from django.contrib.auth.forms import AuthenticationForm

from conf.settings import AUTH_WEB_LOGIN_USERNAME_CASE_SENSITIVE

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class RememberMeAuthenticationForm(AuthenticationForm):

	remember_me = forms.BooleanField(label=_("Remember me"), required=False)

	def __init__(self, request=None, *args, **kwargs):
		super(RememberMeAuthenticationForm, self).__init__(*args, **kwargs)



@sensitive_post_parameters()
@csrf_protect
@never_cache
def remember_me_login(request, 
		  template_name='registration/login.html',
		  redirect_field_name=REDIRECT_FIELD_NAME,
		  authentication_form=RememberMeAuthenticationForm,
		  current_app=None, 
		  extra_context=None
):
	"""
	Displays the login form and handles the login action.
	"""
	redirect_to = request.GET_OR_POST().get(redirect_field_name, '')

	if request.method == "POST":
		rec_form = {}
		
		for k, v in request.POST.items():
			rec_form[k] = v
		
		if not AUTH_WEB_LOGIN_USERNAME_CASE_SENSITIVE:
			rec_form['username'] = rec_form['username'].lower()
		
		form = authentication_form(data=rec_form)
		if form.is_valid():
			# >>>>>>>>>>>
			# REMEMBER ME
			if not form.cleaned_data [ 'remember_me' ]:
				 request.session.set_expiry ( 0 )        # user's session cookie will expire when the user's Web browser is closed.
			# <<<<<<<<<<<

			netloc = urlparse.urlparse(redirect_to)[1]

			# Use default setting if redirect_to is empty
			if not redirect_to:
				redirect_to = settings.LOGIN_REDIRECT_URL

			# Heavier security check -- don't allow redirection to a different
			# host.
			elif netloc and netloc != request.get_host():
				redirect_to = settings.LOGIN_REDIRECT_URL
			
			# Okay, security checks complete. Log the user in.
			auth_login(request, form.get_user())

			if request.session.test_cookie_worked():
				request.session.delete_test_cookie()

			return HttpResponseRedirect(redirect_to)
	else:
		form = authentication_form(request)

	request.session.set_test_cookie()

	#current_site = get_current_site(request) removed django 1.11

	context = {
		'form': form,
		redirect_field_name: redirect_to,
		#'site': current_site,  removed django 1.11
		#'site_name': current_site.name, removed django 1.11
	}
	if extra_context is not None:
		context.update(extra_context)
		
	return TemplateResponse(request, template_name, context) #, current_app=current_app)  removed django 1.11



