import os
import json
import time
from django.template import RequestContext, loader
from django.http import HttpResponse

from webui.decorators import tenant_dependent

from core.models import DataSource
from core.api.datasource import init_datasource_store_from_conf

from core.api.base import tenant_get_media_file_path
from core.api.base import tenant_get_media_url_path


from conf import get_configuration


from webui.decorators import tenant_dependent


@tenant_dependent
def list_datasources(request):
	
	tenant = request.tenant
	
	datasources = DataSource.objects.order_by('datagroup__element__name', 'name').all()

	context = {
		'datasources': datasources,
	}

	#c = RequestContext(request, context)
	t = loader.get_template( 'debug/datasources.html' )
	html_text = t.render(context, request)	
	
	return HttpResponse(html_text)


@tenant_dependent
def rrdgraph(request, dsuuid):
	
	from core.api.datasource import init_datasource_store_from_conf
	import rrdtool
	
	tenant = request.tenant

	start = str(request.GET_OR_POST().get('s', '-480m'))
	end   = str(request.GET_OR_POST().get('e', 'now'))
	width = int(request.GET_OR_POST().get('w', 800))

	print start
	print end

	
	ds = DataSource.objects.get(uuid=dsuuid)
	
	conf = get_configuration()

	DEBUG = [ (s, getattr(conf, s)) for s in sorted(dir(conf)) ]

	#DEBUG += [ (s,os.environ.get(s,None)) for s in sorted(os.environ.keys()) ]
	#raise Exception(DEBUG)




	datasource_store = init_datasource_store_from_conf()
	
	rrd_dir_name, rrd_file_path = datasource_store._get_datasource_rrd_filename(tenant.name, ds.uuid)
	
	media_file_name = ds.uuid + ".png"
	
	media_file_path = tenant_get_media_file_path(tenant.uuid, media_file_name)
	media_file_url  = tenant_get_media_url_path(tenant.uuid, media_file_name)
	
	args = [
		 media_file_path
		,'--imgformat', 'PNG'
		,"DEF:data=%s:data:AVERAGE" % (rrd_file_path)
		,"LINE:data#0000FF:\"%s\"" % (ds.name)
		,"-s" ,start
		,"-e" ,end
		,"-w" , str(width),
	]
	args = [ str(s) for s in args ]
	
	rrdtool.graph( *args )
		
	
	context = {
		'STATIC_ROOT'    : conf.STATIC_ROOT,
		'STATIC_WEB_ROOT': conf.STATIC_WEB_ROOT,
		'STATIC_WEB_URL_PREFIX' : conf.STATIC_WEB_URL_PREFIX,

		'MEDIA_ROOT'          : conf.MEDIA_ROOT,
		'MEDIA_WEB_ROOT'      : conf.MEDIA_WEB_ROOT,
		'MEDIA_WEB_URL_PREFIX': conf.MEDIA_WEB_URL_PREFIX,

		'rrd_dir_name'   : rrd_dir_name,
		'rrd_file_path'  : rrd_file_path,

		'media_file_name': media_file_name,
		'media_file_path':media_file_path,
		'media_file_url' :media_file_url,

		'start': start,
		'end'  : end,
		
	}

	#c = RequestContext(request, context)
	t = loader.get_template( 'debug/ds.html' )
	html_text = t.render(context, request)	
	
	return HttpResponse(html_text)

	
	
	
	
@tenant_dependent
def fetch_data(request, dsuuid):
	
	ds_id = dsuuid
	
	#ds_id     = request.GET_OR_POST()['ds']
	
	ts_start = request.GET_OR_POST().get('s', None)
	if ts_start == None:
		ts_start = time.time() - 60*60
	ts_start = int(ts_start)
	
	
	ts_end   = request.GET_OR_POST().get('e', None)
	if ts_end == None:
		ts_end = time.time()
	ts_end = int(ts_end)

	tenant = request.tenant
	
	ds = DataSource.objects.get(uuid=ds_id)
	
	conf = get_configuration()

	datasource_store = init_datasource_store_from_conf()
	#datasource_store = init_datasource_store(conf.DATASOURCE_STORAGE_HANDLER, conf.DATASOURCE_STORAGE_PARAMS)
	
	data = datasource_store.fetch_values(tenant.uuid, ds.uuid, ts_start=ts_start, ts_end=ts_end)



	data_worked = []
	data_worked.append(data['scalex'])
	data_worked.append([ds.name,])
	dataseries = []
	for datapoint in data['dataseries']:
		dataseries.append([datapoint,])
	data_worked.append(dataseries)

	data_text = json.dumps(data_worked)

	
	r = HttpResponse(data_text)
	#r['content-type'] = 'text/json'
	return r

