# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import sys, traceback

from django import http
from django.template import Context, RequestContext, loader, TemplateDoesNotExist
from django.views.decorators.csrf import requires_csrf_token


from django.conf import settings
#from django.views.defaults import bad_request       # 400
# 403
from django.views.defaults import permission_denied 
# 404
from django.views.defaults import page_not_found    
# 500
from django.views.defaults import server_error      


#@requires_csrf_token
#def custom_403_bad_request(request, template_name='html/custom_403.html'):
#	
#	if settings.DEBUG:
#		return bad_request(request)
#	
#	try:
#		template = loader.get_template(template_name)
#	except TemplateDoesNotExist:
#		return http.HttpResponseForbidden('<h1>403 Forbidden</h1>')
#		
#	return http.HttpResponseForbidden(template.render(RequestContext(request)))


# This can be called when CsrfViewMiddleware.process_view has not run,
# therefore need @requires_csrf_token in case the template needs
# {% csrf_token %}.
@requires_csrf_token
def custom_403_permission_denied(request, template_name='html/custom_403.html'):
	"""
	Permission denied (403) handler.

	Templates: `403.html`
	  Context: None

	If the template does not exist, an Http403 response containing the text
	"403 Forbidden" (as per RFC 2616) will be returned.
	"""
	
	if settings.DEBUG:
		return permission_denied(request)
	
	try:
		template = loader.get_template(template_name)
	except TemplateDoesNotExist, e:
		return http.HttpResponseForbidden('<h1>403 Forbidden</h1>')
		
	return http.HttpResponseForbidden(template.render(request))


# This can be called when CsrfViewMiddleware.process_view has not run,
# therefore need @requires_csrf_token in case the template needs
# {% csrf_token %}.
@requires_csrf_token
def custom_404_page_not_found(request, template_name='html/custom_404.html'):
	"""
	Default 404 handler.
	Templates: `404.html`
	  Context:
	request_path
	    The path of the requested URL (e.g., '/app/pages/bad_page/')
	"""
	if settings.DEBUG:
		return page_not_found(request)
	
	t = loader.get_template(template_name) # You need to create a 404.html template.
	
	context = {'request_path': request.path}
	
	return http.HttpResponseNotFound(t.render(context, request))


#@requires_csrf_token
def custom_500_server_error(request, template_name='html/custom_500.html'):
	"""
	500 error handler.

	Templates: `500.html`
	Context: None
	"""
	if settings.DEBUG:
		return server_error(request)
		
	t = loader.get_template(template_name) # You need to create a 500.html template.
	exception_type, value, tb = sys.exc_info()

	return http.HttpResponseServerError(t.render(Context({
		'exception_value': value,
		'exception_type': '\n'.join(traceback.format_tb(tb)),
	})))

