# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os
import subprocess
import select
import re
import types

from django.http import HttpResponse

from core.api.base import db_set_current_tenant_schema
from core.api.base import tenant_get_data_dir

from webui.models import *

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#

def get_exec_environment_from_resource(resource):
	
	prefix = 'SANET'
	
	ret = {}
	
	ret = os.environ
	
	ret['SANET_RESOURCE_TYPE'] = resource.resource_type
	
	ret['SANET_RESOURCE_%s_ID' % (resource.resource_type.upper()) ] = str(resource.resource_id)
	
	for key in resource.__dict__:
		var_name  = str( '_'.join([prefix, 'RESOURCE', key.upper()]) )
		var_value = getattr(resource, key)
		
		if isinstance(var_value, types.StringType) or isinstance(var_value, types.UnicodeType):
			var_value = var_value.encode('ascii','replace')
			
		# FIX: PossibleState will be converted to interger value. We should not use its __unicode/str__() method.
		#elif isinstance(var_value, PossibleState):
		#	var_value = var_value.name
			
		ret[ var_name ] = str(var_value)

	#if resource.resource_type in ['datagroup', 'condition']:
	#	
	#	node  = resource.node if resource.node != None else resource.iface.node
	#	n = str( '_'.join([prefix, 'RESOURCE', 'NODE']) )
	#	ret[ n ] = str(node.name)
	#	n = str( '_'.join([prefix, 'RESOURCE', 'NODE', 'ID']) )
	#	ret[ n ] = str(node.id)
	#	
	#	if resource.iface != None:
	#		n = str( '_'.join([prefix, 'RESOURCE', 'IFACE']) )
	#		ret[ n ] = str(resource.iface.name)
	#		n = str( '_'.join([prefix, 'RESOURCE', 'IFACE','ID']) )
	#		ret[ n ] = str(resource.iface.id)

	return ret


def launch_external_script(request, rt, uuid, scriptname):
	
	from webui.api.types import WEB_ELEMENT_TYPE_TO_MODEL
	from webui.api.extra_menu import get_menus_dir_path, EXTERNAL_SCRIPT_TIMEOUT
	
	"""
	Launch an external program (script) and returns its output.
	
	The first parameter of the script is the Resource::name
	"""
	
	
	tenant = request.tenant
	db_set_current_tenant_schema(tenant.name)
	
	model = WEB_ELEMENT_TYPE_TO_MODEL[rt]
	resource = model.objects.get(uuid=str(uuid))
	
	#
	# PATH
	#
	
	
	#data_dir = tenant_get_data_dir(tenant.uuid)
	#path = os.sep.join([data_dir, EXTERNAL_MENUS_DIR_NAME])
	path = get_menus_dir_path(tenant, rt)

	extern_script = os.sep.join( [path, scriptname] )

	script_args   = [ resource.name ]

	command = [ extern_script ] + script_args

	#
	# ENV
	#
	environment = get_exec_environment_from_resource(resource)

	timeout = EXTERNAL_SCRIPT_TIMEOUT
	output = ''

	#
	# Launch command
	#
	# FIX: use TIMEOUT !
	try: 
		retcode = None
		process = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, shell=False, env=environment)

		rd, wr, er = select.select([process.stdout], 
		                           [], 
		                           [process.stderr],
		                           timeout)
		if rd or er:
			process.wait()

		if process.returncode == None:
			process = None
			ourput = "%s: Timeout" %(command)
		elif process.returncode != 0:
			error = process.stderr.read()
			output = "error return code %s: %s" % (process.returncode, error) 
		else:
			output = process.stdout.read()

	except OSError, es:
		output = "exec error %s:%s" % (command, es)
	except ValueError, es:
		output = "popen error %s: %s: %s" % (command, es)

	return HttpResponse(output)







