# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os

from PIL import Image

from django.conf import settings

from django.http import *

from core.compatibility import FileWrapper

from core.models import Tenant
from core.api.base import tenant_get_icons_dir, tenant_get_media_dir

#----------------------------------------------------------------------------------#
# BASE FUNCTIONS                                                                   #
#----------------------------------------------------------------------------------#

def _get_colored_icon(tenant_id, icon_name, color_str):

	if color_str == '':
		color_str = None

	# Se non troviamo l'icona indicata, forso e' una icona di default
	try:
		base_dir = tenant_get_icons_dir(tenant_id)
		file_name = os.path.join(base_dir, icon_name + ".png")
		mtime = os.stat(file_name).st_mtime
	except OSError, er:
		base_dir = os.path.join(settings.SANET_INSTALL_DIR , 'static','webui','images','resources')
		file_name = os.path.join(base_dir, icon_name + ".png")
		mtime = os.stat(file_name).st_mtime

	if color_str != None:

		color     = (int(color_str[0:2],16), int(color_str[2:4],16), int(color_str[4:6],16), 255)

		icon_name2 = icon_name + "_%s_%s" % (color_str, mtime)

		temp_dir = tenant_get_media_dir(tenant_id)

		file_name2 = os.path.join(temp_dir, icon_name2 + ".png")

		if not os.path.exists(file_name2):
			_png_file_replace_color(file_name, file_name2, color)
	else:
		file_name2 = file_name

	f = open(file_name2, "r")
	wrapper = FileWrapper(f)
	response = HttpResponse(wrapper, content_type='image/png')
	response['Content-Length'] = os.path.getsize(file_name2)

	response['Access-Control-Allow-Origin'] = '*'	# Questo serve perche' applicazioni web esterne (dashboard) devono scaricare le icone senza problemi di Cross-site HTTP requests (CORS)
	response['Cache-Control'] = 'max-age=%s' % settings.ICON_MAX_AGE

	return response

def _png_file_replace_color(in_file_name, out_file_name, color):

	img = Image.open(in_file_name)
	img = img.convert("RGBA")

	datas = img.getdata()

	new_data = []
	for rgba in datas:

		if rgba[3] != 0: #skip transparent color
			# Convertiamo il pixel al colore desiderato tenendo conto solo della sua  della luminosita'
			cmax = max(rgba[0],rgba[1],rgba[2])
			#if cmax != 255:
			cmax = cmax / 255.0
			rgba = ( int(color[0] *cmax),  int(color[1] *cmax), int(color[2] *cmax), color[3])

		new_data.append(rgba)

	img.putdata(new_data)
	img.save(out_file_name, "PNG")

	return True

#----------------------------------------------------------------------------------#
# VIEWS                                                                            #
#----------------------------------------------------------------------------------#

#@tenant_dependent
#def get_colored_icon(request):
#
#	icon_name = request.REQUEST['icon']
#	color     = request.REQUEST.get('color', '')
#
#	return _get_colored_icon(request, icon_name, color)

#----------------------------------------------------------------------------------#

#@tenant_dependent
#def get_colored_icon2(request, icon, color=None):
#	return _get_colored_icon(request, icon, color)

#----------------------------------------------------------------------------------#

def get_colored_icon3(request, tenantid, icon, color=None):

	#request.tenant = Tenant.objects.get(uuid=tenantid)

	return _get_colored_icon(tenantid, icon, color)
