# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os
import re
import json
import time

from core.compatibility import reverse
from django.http import HttpResponse

from django.shortcuts import render
from django import forms

from sanet_common.sanetdefs.constants import VALID_AGENT_NAME

from sanet_poller.agentctl import agent_ctl

from conf import  get_broker_connection_string
from conf import get_configuration

from core.compatibility import ErrorList

from core.models import Tenant, Agent
from core.api.base import agent_create_r
from core.api.base import tenant_get_media_dir
from core.api.base import db_set_current_tenant_schema
from core.api.base import agent_remove_safe
from core.api.cache import init_cache_manager_from_conf

from core.api.sysstats import sanetd_create_rrd_graph_file_elements
from core.api.sysstats import agent_create_rrd_graph_file_elements
from core.api.sysstats import AGENT_STAT_DS_ELEMENTS
from core.api.sysstats import AGENT_STAT_DS_MONITOR
from core.api.sysstats import AGENT_STAT_DS_QUEUES
from core.api.sysstats import AGENT_STAT_DS_RATES
from core.api.sysstats import AGENT_STATS_DS_MEMORY
from core.api.sysstats import AGENT_COLORS

from core.api.sysstats import SANETD_STAT_DS_ELEMENTS
from core.api.sysstats import SANETD_COLORS


from core.api.sysstats import sysstats_create_rrd_graph_file


from webui.api.utils import render_template
from webui.decorators import tenant_dependent, capability_required
from webui.decorators import user_is_admin

from entables.management.commands.entables_ctl import entables_ctl

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

@capability_required('gui.view.system_status')
@tenant_dependent
def get_stats(request, tenantid, uuid):

	tenant = Tenant.objects.get(uuid=tenantid)

	tenant.db_set_current_schema()

	cache_manager = init_cache_manager_from_conf()

	agent = Agent.objects.get(tenant=tenant, uuid=uuid)

	agent.load_fields_from_cache(cache_manager)

	try:
		cmd = 'info'
		args = []
		broker_address = get_broker_connection_string()
		info_text      = agent_ctl(broker_address, agent.uuid, cmd, list(args))
	except Exception, e:
		info_text = 'ERRORE: ' + str(e)

	# Graphs
	graphs = [
		reverse('agent_stat_chart', kwargs={ 'agent_uuid':agent.uuid, 'chart': 'elements'      }),
		reverse('agent_stat_chart', kwargs={ 'agent_uuid':agent.uuid, 'chart': 'monitor'       }),
		reverse('agent_stat_chart', kwargs={ 'agent_uuid':agent.uuid, 'chart': 'memory'        }),
		reverse('agent_stat_chart', kwargs={ 'agent_uuid':agent.uuid, 'chart': 'queues'        }),
		reverse('agent_stat_chart', kwargs={ 'agent_uuid':agent.uuid, 'chart': 'rates'         }),
		reverse('agent_stat_chart', kwargs={ 'agent_uuid':agent.uuid, 'chart': 'processtimes'  }),
	]

	#
	# Render
	#
	html = render_template( 'html/parts/stats_agent.html', {
		'TIME'  : time.time(),
		'agent' : agent,
		'info'  : info_text,
		'graphs': graphs,
	})

	return HttpResponse(html)

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

def ts_or_timespec(s):

	if s is None:
		return None

	FACTOR = {
		'M' : 60 * 60 * 24 * 30,
		'w' : 60 * 60 * 24 * 7,
		'd' : 60 * 60 * 24,
		'h' : 60 * 60,
		'm' : 1,
	}

	try:
		return int(s)
	except Exception, e:
		pass

	try:
		(n,f) = re.match('-([0-9]+)(\w)', s).groups()

		return  -1 * int(n) * FACTOR[f]
	except Exception, e:
		raise Exception("Unable to parse time spec: %s. %s" % (s, e))


@tenant_dependent
def stat_chart(request, agent_uuid, chart):

	ts_start = request.GET_OR_POST().get('s', None)
	ts_end   = request.GET_OR_POST().get('e', None)

	ts_start = ts_or_timespec(ts_start)
	ts_end   = ts_or_timespec(ts_end)

	tenant = request.tenant

	agent = Agent.objects.get(tenant=tenant, uuid=agent_uuid)

	media_dir = tenant_get_media_dir(tenant.uuid)

	if get_configuration().SANET_STATS_VERSION == 1:
		if chart == 'elements': file_name, w, h = agent_create_rrd_graph_file_elements(agent, media_dir, 'agent_%s_elements.png' % (agent.uuid), AGENT_STAT_DS_ELEMENTS, AGENT_COLORS, ltype='AREA', stacked=True, title="Elementi"   , ts_start=ts_start, ts_end=ts_end)
		if chart == 'monitor' : file_name, w, h = agent_create_rrd_graph_file_elements(agent, media_dir, 'agent_%s_monitor.png'  % (agent.uuid), AGENT_STAT_DS_MONITOR , AGENT_COLORS, ltype='AREA', stacked=True, title="Datagrops"  , ts_start=ts_start, ts_end=ts_end) 
		if chart == 'queues'  : file_name, w, h = agent_create_rrd_graph_file_elements(agent, media_dir, 'agent_%s_queues.png'   % (agent.uuid), AGENT_STAT_DS_QUEUES  , AGENT_COLORS                            , title="Code"       , ts_start=ts_start, ts_end=ts_end) 
		if chart == 'rates'   : file_name, w, h = agent_create_rrd_graph_file_elements(agent, media_dir, 'agent_%s_rates.png'    % (agent.uuid), AGENT_STAT_DS_RATES   , AGENT_COLORS                            , title="Rate"       , ts_start=ts_start, ts_end=ts_end) 
		if chart == 'memory'  : file_name, w, h = agent_create_rrd_graph_file_elements(agent, media_dir, 'agent_%s_memory.png'   % (agent.uuid), AGENT_STATS_DS_MEMORY , AGENT_COLORS                            , title="Memoria"    , ts_start=ts_start, ts_end=ts_end) 
	else:
		ltype            = 'LINE'
		stacked          = False
		selected_ds_type = 'gauge'
		colors = AGENT_COLORS

		if chart == 'elements': 
			selected_stats   = AGENT_STAT_DS_ELEMENTS
			ltype   = 'AREA'
			stacked = True
			title   = "Elementi v2"  

		if chart == 'monitor' : 
			selected_stats   = AGENT_STAT_DS_MONITOR 
			ltype   = 'AREA'
			stacked = True
			title   = "Datagroups v2" 

		if chart == 'queues'  : 
			selected_stats   = AGENT_STAT_DS_QUEUES  
			title   = "Code v2"      

		if chart == 'rates'   : 
			selected_stats   = AGENT_STAT_DS_RATES   
			title   = "Rate v2"      

		if chart == 'memory'  : 
			selected_stats   = AGENT_STATS_DS_MEMORY 
			title   = "Memoria v2"   

		if chart == 'processtimes'  : 
			title   = "Process time v2"   
			selected_stats   = [
				'threadpool.process.avg',
				'threadpool.process.max',
				'threadpool.process.min',
			]
			
			colors = dict([
				('avg', '0000ff'),
				('max', 'ff0000'),
				('min', '00ff00'),
			])
			
		selected_stats = [ 'agent.%s.%s' % (agent.name, sid) for sid in selected_stats ]

		chart_file_name = 'agent_%s_%s.png'   % (agent.uuid, chart)
		
		file_name, w, h = sysstats_create_rrd_graph_file(
			media_dir, 
			chart_file_name, 
			selected_stats, 
			colors, 
			ltype            = ltype, 
			stacked          = stacked, 
			title            = title, 
			ts_start         = ts_start, 
			ts_end           = ts_end, 
			tenant_id        = agent.tenant_id,
			selected_ds_type = selected_ds_type
		)


	with open(file_name, "rb") as f:
		return HttpResponse(f.read(), mimetype="image/png")

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

@user_is_admin
def get_stats_entables(request):
	

	tenant = request.tenant

	try:
		cmd = 'info'
		args = []

		broker_address = get_broker_connection_string()
		info           = entables_ctl(broker_address, cmd, list(args))

	except Exception, e:

		info = 'ERRORE: ' + str(e)

	ret = {
		'values': { 'info': info }
	}

	return HttpResponse(json.dumps(ret))

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

@tenant_dependent
def sanetd_stat_chart(request, chart):

	ts_start = request.GET_OR_POST().get('s', None)
	ts_end   = request.GET_OR_POST().get('e', None)

	ts_start = ts_or_timespec(ts_start)
	ts_end   = ts_or_timespec(ts_end)

	media_dir = os.path.join(get_configuration().MEDIA_WEB_ROOT, '_stats')

	if chart == 'queue'   : 
		file_name, w, h = sanetd_create_rrd_graph_file_elements(media_dir, 'sanetd_queue.png', SANETD_STAT_DS_ELEMENTS, SANETD_COLORS, ltype='LINE', stacked=False, title="Data Queue"   , ts_start=ts_start, ts_end=ts_end)

	with open(file_name, "rb") as f:
		response = HttpResponse(f.read(), mimetype="image/png")
		response['Content-Length'] = os.path.getsize(file_name)			
		response['Cache-Control'] = 'no-cache'
		response['Pragma']        = 'no-cache'
		return response

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class AgentForm(forms.Form):

	name        = forms.CharField(required=True)
	description = forms.CharField(required=False)
	#is_enabled  = forms.BooleanField()
	#is_primary  = forms.BooleanField()
	#is_remote   = forms.BooleanField()
	threads                 = forms.IntegerField( min_value=1, max_value=30, widget=forms.Select(choices=[(n,n) for n in xrange(1,1+30)]))
	config_update_minperiod = forms.IntegerField( forms.Select(choices=[((n+1)*60,(n+1)*60) for n in xrange(2,5)]) )

	def clean(self):

		if not self.errors:
			cleaned_data = self.cleaned_data
			
			name      = cleaned_data.get("name")
			if not name or not VALID_AGENT_NAME.match(name):
				self._errors["name"] = ErrorList(["Invalid name"])

			# Always return the full collection of cleaned data.
			return cleaned_data

		return super(AgentForm, self).clean()

def add_new(request, uuid):

	tenant = Tenant.objects.get(uuid=uuid)

	if request.method == 'POST': 
		form = AgentForm(request.POST)
		if form.is_valid():

			name       = form.cleaned_data['name']
			descr      = form.cleaned_data['description']
			#is_enabled = bool(form.cleaned_data['is_enabled'])
			#is_primary = bool(form.cleaned_data['is_primary'])
			#is_remote  = bool(form.cleaned_data['is_remote'])
			threads    = int(form.cleaned_data['threads'])
			config_update_minperiod = int(form.cleaned_data['config_update_minperiod'])

			a = agent_create_r(tenant, name, primary=False)

			a.description = descr
			#a.is_enabled = False
			#a.is_remote  = False
			a.threads = threads
			a.config_update_minperiod = config_update_minperiod			
			a.save()

			return HttpResponse("OK")
	else:
		form = AgentForm() # An unbound form

	return render(request, 'html/parts/configtenant_agent.html', {
		'form'  : form  ,
		'action_url': reverse('tenant_add_agent', kwargs={'uuid': tenant.uuid}),
		'tenant': tenant,
	})

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

class EditAgentForm(forms.ModelForm):
	class Meta:
		model = Agent
		fields = [
			'name',
			'description',
			#'is_enabled',
			'threads', 
			'config_update_minperiod',
		]
		widgets = {
			'name'                    : forms.TextInput(),
			'description'             : forms.TextInput(),
			'threads'                 : forms.Select(choices=[(n,n) for n in xrange(1,1+30)]),
			'config_update_minperiod' : forms.Select(choices=[((n+1)*60,(n+1)*60) for n in xrange(2,5)]),
		}

@user_is_admin
def edit(request, uuid, auuid):

	tenant = Tenant.objects.get(uuid=uuid)
	db_set_current_tenant_schema(tenant.name)

	agent  = Agent.objects.get(tenant=tenant, uuid=auuid)

	if request.method == 'POST': 
		form = EditAgentForm(request.POST, instance=agent)
		if form.is_valid():

			#name       = form.cleaned_data['name']
			#descr      = form.cleaned_data['description']
			#is_enabled = bool(form.cleaned_data['is_enabled'])
			#is_primary = bool(form.cleaned_data['is_primary'])
			#is_remote  = bool(form.cleaned_data['is_remote'])
			#threads    = int(form.cleaned_data['threads'])

			#a = agent_create_r(tenant, name, primary=False)

			#a.description = descr
			#a.is_enabled = False
			#a.is_remote  = False
			#a.save()

			return HttpResponse("OK")
	else:
		form = EditAgentForm(instance=agent) # An unbound form

	return render(request, 'html/parts/configtenant_agent.html', {
		'form'  : form  ,
		'action_url': reverse('tenant_edit_agent', kwargs={'uuid':tenant.uuid, 'auuid':agent.uuid}),
		'tenant': tenant,
	})

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#


@user_is_admin
def remove(request, uuid, auuid):

	tenant = Tenant.objects.get(uuid=uuid)

	db_set_current_tenant_schema(tenant.name)

	agent  = Agent.objects.get(tenant=tenant, uuid=auuid)

	agent_remove_safe(agent, tenant)

	return HttpResponse("OK")
