# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os
import re
import time

import json

from django.http import HttpResponse

from conf import get_configuration

from core.models import DataGroup,DataSource, Condition
from core.api.base import db_set_current_tenant_schema

from webui.decorators import tenant_dependent


#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

@tenant_dependent
def get_used(request):
	
	return _get_user(request)

	
def _get_user(request):

	db_set_current_tenant_schema(request.tenant.name)

	DataGroup.objects.only('classification').distinct('classification')

	c1 = set( e[0] for e in DataGroup.active_objects.distinct('classification').values_list('classification'))
	
	c2 = set( e[0] for e in DataSource.active_objects.distinct('classification').values_list('classification'))
	c3 = set( e[0] for e in Condition.active_objects.distinct('classification').values_list('classification'))

	classifications = sorted([ c for c in c1.union(c2).union(c3) ] )

	return HttpResponse( json.dumps(classifications) )

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

def get_all(request):


	config = get_configuration()
	file_name = os.path.join(config.BASE_DIR, 'data', 'classifications')


	r = []


	filter = request.GET_OR_POST().get('filter', '.*')

	filter = re.compile(filter)


	#afile_name = '/usr/local/labsdashboard/collect_static/classification.json'

	f = open(file_name, 'r')

	line = f.readline()
	while line:

		line = line.strip()

		if filter.search(line):
			r.append(line)

		line = f.readline()

	f.close()


	return HttpResponse( json.dumps( sorted(r) ) )
