# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import time
import json

from django.http import HttpResponse

from core.api.base import db_set_current_tenant_schema

from core.models import Cluster

from tags.models import *

from webui.decorators import tenant_dependent

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

@tenant_dependent
def select_all(request):
	return _select_all(request)

def _select_all(request):

	user = request.user

	tenant = request.tenant
	db_set_current_tenant_schema(tenant.name)

	clusters = Cluster.get_visible_by_user(tenant, user)
	clusters = clusters.order_by('name')

	r = []
	for c in clusters.all():
		r.append({
			'data': {
				'stype': 'cluster',
				'sid': c.uuid,
			},

			'title'   : c.name, 
			'icon'    : None, 

			'folder': True,
			"lazy"  : True,
		})

	return HttpResponse(json.dumps(r))


@tenant_dependent
def sub_elements(request):

	user   = request.user

	tenant = request.tenant
	db_set_current_tenant_schema(tenant.name)

	sid    = request.GET_OR_POST().get('sid')

	cluster = Cluster.objects.get(tenant=tenant, uuid=sid)


	clusters = cluster.get_nodes_visible_by_user(user)
	clusters = clusters.order_by('cluster_role', 'name' ).all()

	r = []
	for n in clusters:
		r.append({
			'data': {
				'stype': 'node',
				'sid'  : n.uuid,
				'srole': n.cluster_role,
			},

			'title'   : n.name, 
			'icon'    : n.get_icon_url(), 

			'folder': False,
			"lazy"  : False,
		})

	return HttpResponse(json.dumps(r))
