# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.
import time
import json

from sanet_common.sanetdefs.constants import EVENT_CONDITION

from django.http import HttpResponse
from core.compatibility import reverse
from django.template import RequestContext, loader

from core.models import Condition, Event
from core.api.base import db_set_current_tenant_schema

from core.statecolor import get_state_color


from core.api.cache import init_cache_manager_from_conf

from webui.decorators import tenant_dependent

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

def get_start_end_params(request):
	
	ts_start = request.GET_OR_POST().get('from', None)
	if ts_start == None:
		ts_start = time.time() - 60*60*8
	ts_start = int(ts_start)
	
	ts_end   = request.GET_OR_POST().get('to', None)
	if ts_end == None:
		ts_end = time.time()
	ts_end = int(ts_end)

	return ts_start, ts_end

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

@tenant_dependent
def get_details(request):
	
	return _get_conditions_details(request)
	
def _get_conditions_details(request):
	
	tenant = request.tenant
	
	db_set_current_tenant_schema(tenant.name)
	
	uuid = request.GET_OR_POST()['uuid']
	
	
	
	condition = Condition.active_objects.get(uuid=uuid)
	
	
	cache_manager = init_cache_manager_from_conf()
	condition.load_fields_from_cache(cache_manager)
	
	context = {
		'condition': condition
	}
	
	#c = RequestContext(request, context)
	t = loader.get_template( 'html/parts/condition_details.html' )
	html_text = t.render(context, request)	
	
	return HttpResponse( html_text )
	

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

from sanet_common.sanetdefs.constants import  CSTATE_TO_CODENAME

@tenant_dependent
def get_changes_raw(request, uuid):

	ts_start, ts_end = get_start_end_params(request)

	tenant = request.tenant

	c = Condition.active_objects.get(uuid=uuid)

	#sequence = logchanges_sequence(c, ts_start, ts_end)
	sequence = c.logchanges_sequence(c, ts_start, ts_end)

	return HttpResponse( json.dumps( sequence ) )


@tenant_dependent
def get_changes(request, uuid):

        ts_start, ts_end = get_start_end_params(request)

        tenant = request.tenant

        c = Condition.active_objects.get(uuid=uuid)

        #sequence = logchanges_sequence(c, ts_start, ts_end)
        sequence = c.logchanges_sequence(c, ts_start, ts_end)

        r = convert_to_widget_format(sequence)

        return HttpResponse( json.dumps( r ) )

@tenant_dependent
def new_get_changes(request, uuid):

        ts_start, ts_end = get_start_end_params(request)

        tenant = request.tenant

        c = Condition.active_objects.get(uuid=uuid)

        #sequence = logchanges_sequence(c, ts_start, ts_end)
        sequence = c.logchanges_sequence(c, ts_start, ts_end)

        r = convert_to_new_widget_format(c, sequence, uuid)

        return HttpResponse( json.dumps( r ) )



state_dict = {
			'UP':'#00ff00',
			'DN':'#ff0000',
			'UU':'#dddddd',
			'UD':'#dddddd',
			'DU':'#dddddd',
			'DD':'#dddddd',
			'UN':'#dddddd',
			'FA':'#ffff00',
			'IN':'#ffffff',
}

def _new_get_changes(uuid, tenant, ts_start, ts_end):

	c = Condition.active_objects.get(uuid=uuid)

	#sequence = logchanges_sequence(c, ts_start, ts_end)
	sequence = c.logchanges_sequence(c, ts_start, ts_end)

	r = convert_to_new_widget_format(c,sequence, uuid)
	return r

def get_states_palette(sequence):
	states = list(set( [ status for (s,e,d,status) in sequence ] ))
	return dict([ (s,  state_dict[CSTATE_TO_CODENAME(s)]) for s in states ])

def convert_to_widget_format(sequence):

	# (s,e, d, status)   
	#
	#  [
	#     (label1,label2,....),
	#     (s, e,  label_pos, data)
	#  ]
	states = sorted( dict(  [ (status,'') for (s,e,d,status) in sequence ] ).keys() )

	states_pos = {}
	for i in xrange(len(states)):
		states_pos[ states[i] ] = i

	r = []
	r.append( [CSTATE_TO_CODENAME(state) for pos,state in  sorted([ (pos,state) for state,pos in states_pos.items() ])]    )
	r.append([])
	
	for e in sequence:
		r[1].append( (e[0], e[1], states_pos[ e[3] ], [ CSTATE_TO_CODENAME(e[3]), e[2], seconds_to_descr(e[2])   ]) )

	ret = {
			'values': r,
			'renderopts': {
				'wconf':{
						'timeline': False
				},
				'series': [
					{
					'color':state_dict[CSTATE_TO_CODENAME(x)]
					}  for a,x in  sorted([ (pos,state) for state,pos in states_pos.items() ])
				]
			}
		}
	return ret

def convert_to_new_widget_format(c, sequence, uuid):
	
	
	# (s,e, d, status)   
	#
	#  [
	#     (label1,label2,....),
	#     (s, e,  label_pos, data)
	#  ]
	states = sorted( dict(  [ (status,'') for (s,e,d,status) in sequence ] ).keys() )

	states_pos = {}
	for i in xrange(len(states)):
		states_pos[ states[i] ] = i

	r = []
	#r.append( [CSTATE_TO_CODENAME(state) for pos,state in  sorted([ (pos,state) for state,pos in states_pos.items() ])]    )
	#r.append()
	
	for e in sequence:
		
		#r[1].append( (e[0], e[1], states_pos[ e[3] ], [ CSTATE_TO_CODENAME(e[3]), e[2], seconds_to_descr(e[2])   ]) )
		r.append( {
	 		'from'   : e[0], #// Python timestamp
			'to'     : e[1], #// Python timestamp
			'value'  : states_pos[ e[3] ], #// index of state, can be null/undefined
			'details': [ CSTATE_TO_CODENAME(e[3]), e[2], seconds_to_descr(e[2])   ] #// textcell item with details, can be null/undefined
        })

	presel_end = time.time()
	presel_end = int(presel_end)
	ret = {
		'title': c.path,
		'flags': {
		        'timeline':   True, #display timeline or not, default true
		        'controls':    True, # display conrtol buttons or not, default true
		        'locked':     False, #// if the user can make selections and change range, default false
		},
		'presel': {
			'from': presel_end-60*60,
			'to'  : presel_end
		},
		'url'     : reverse('webui.views.objects.conditions.new_get_changes', kwargs={'uuid':uuid}), # if missing the map goes static, with both uuid and url it's dynamic
		'sequence': r,

		#'states': [{'label':CSTATE_TO_CODENAME(x), 'color':state_dict[CSTATE_TO_CODENAME(x)]} for a,x in  sorted([ (pos,state) for state,pos in states_pos.items()])],
		'states': [{'label':CSTATE_TO_CODENAME(x), 'color':'#'+get_state_color( CSTATE_TO_CODENAME(x)  )} for a,x in  sorted([ (pos,state) for state,pos in states_pos.items()])],

		#'presel': None, # key of the intervals, defaults to 1h
	}

	return ret

def seconds_to_descr(s):

	r = []

	days = s / (60*60*24)
	if days:
		r.append( '%s days' % (days) )

	s = s % (60*60*24)

	hours = s / (60*60)
	if hours:
		r.append('%s hours' % (hours))

	s = s % (60*60)

	minutes = s / 60
	if minutes:
		r.append('%s minutes' % (minutes))

	s = s % 60

	r.append( '%s s' % (s) )

	return ','.join(r)




"""
def logchanges_sequence(c, start, end):

	states_sequence = []

	#events = Event.objects.filter(
	#	kind           = EVENT_CONDITION,
	#	condition_path = c.path
	#)
	
	log_changes = Event.objects.filter(kind=EVENT_CONDITION, condition_path=c.path).filter(timestamp__gte=start, timestamp__lte=end).order_by('timestamp').all()
	
	if len(log_changes) == 0:
		
		changes = Event.objects.filter(kind=EVENT_CONDITION, condition_path=c.path).filter(timestamp__lt=start).order_by('-timestamp')[0:1]

		if len(changes) > 0:

			#  *->DN  |               |
			# --+-----|---------------|----------->
			#         |      DN       |
			#       start            end
			change   = changes[0]
			status   = change.status
			duration = end - start
		else:
			# NO CHANGES IN THE INTERVAL. FINDING FIRST AFTER END"

			changes = Event.objects.filter(kind=EVENT_CONDITION, condition_path=c.path).filter(timestamp__gte=end).order_by('timestamp')[0:1]
			if len(changes) > 0:
				#         |               | DN->*
				# --------|---------------|----+------>
				#         |      DN       |
				#       start            end
				
				change = changes[0]
				status = change.statuslast
				duration = end - start
			else:
				# Unable to find an entry inside the log. Assume 
				# the current status is valid.
				#status = self.status_id
				status = c.status
				duration = end - start
		
		e = ( int(start), int(end), int(duration), int(status) )
		states_sequence.append(e)
	else:
		#  DN   |    UP         DN          UP          DN          UP    |  
		# ------|----+----------+-----------+-----------+-----------+-----|----------->
		#       | DN |    UP    |     DN    |     UP    |     DN    | UP  |
		#       |                                                         |
		#     start                                                      end
		
		current_status   = None
		last_change_time = start
		
		first = True
		for c in log_changes:
			if current_status == None:
				current_status = c.statuslast
				
			duration = c.timestamp - last_change_time
			e = ( int(last_change_time), int(c.timestamp), int(duration), int(current_status))
			states_sequence.append( e )
			
			current_status = c.status
			last_change_time = c.timestamp
			
		duration = end - last_change_time
		e = (int(last_change_time), int(end), int(duration), int(current_status))
		states_sequence.append( e )
		
	if len(states_sequence) > 0:

		states_sequence2 = []
		current = states_sequence.pop(0)
		while len(states_sequence) > 0:
			
			i = states_sequence.pop(0)
			
			if current[3] == i[3]:
				if (current[1] == i[0]) or (current[1] == (i[0]-1)):
					current = (int(current[0]), int(i[1]), int(i[1] - current[0]), int(i[3]))
					continue
			states_sequence2.append(current)
			current = i
		states_sequence2.append(current)
		states_sequence = states_sequence2

	return states_sequence
"""

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

from webui.api.response import ajax_response

@tenant_dependent
@ajax_response
def signal_check_now(request):
	"""
	Questa view viene usata dal widget "alarmslist" per chiedere "via web" di effettuare il check di una condition
	immediatamente.
	
	Per fare questo bisogna contattare l'agente associato.
	"""

	tenant = request.tenant
	db_set_current_tenant_schema(tenant.name)

	condition_id = request.GET_OR_POST()['uuid']

	c = Condition.objects.get(tenant=tenant, uuid=condition_id)

	try:
		#
		# FIX: Per velocita' di implementazione usiamo il comando "agentinfo":
		#
		#    client ----> WEB (view) -----call django command----> agentinfo ---- IPC API ----------------> AGENTE
		#
		# Bisogna utilizzare direttamente le API per comunicare con l'agente.
		#  
		#    client ----> WEB (view) -------------------------------------------- IPC API ----------------> AGENTE
		#
		
		from django.core.management import call_command

		path = c.datagroup.path

		args = [
			'check_now',
			path,
		]

		call_command('agentinfo', *args, agent=c.agent.name, interactive=False, verbosity=True, show_traceback=True)
		
		# Il decoratore ajax_response gestisce la risposta HTTP/JSON. Qui dobbiamo restituire solo dati grezzi python.
		return "Command sent"

	except Exception, e:
		raise Exception("Error while sending command to agent: %s:%s" % (type(e), str(e)))
		
		
		
	
