# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import time
import json
import socket
import urllib

from django.core.exceptions import ValidationError
from django.core import validators
from django.http import HttpResponse

from django.forms import ModelForm

from sanet_common.sanetdefs.constants import *

from sanet_poller.lib.evaluation import compile_str_expr

from conf.default_settings import XFORMS_ALIASES

from core.models import MonitoredElement, MonitoredNode, MonitoredDevice, MonitoredIface, MonitoredService, MonitoredStorage

from core.api.base import db_set_current_tenant_schema



from webui.decorators import tenant_dependent

#-------------------------------------------------------------------------------#
#       Form di validazione personalizzati per tipo di monitoredElement         #
#-------------------------------------------------------------------------------#

class MonitoredElementForm(ModelForm):

	def __init__(self, *args, **kwargs):

		super(MonitoredElementForm, self).__init__(*args, **kwargs)

		#self.fields['icon'].required           = False

		# For these fields a default value will be set
		#self.fields['priority'].required       = False
		#self.fields['description'].required    = False
		#self.fields['cluster'].required        = False
		#self.fields['cluster_role'].required   = False

	#	print self.fields

		for elem in self.fields.values():
			elem.required = False

		self.fields['cluster_role'].validators = [clusterRoleValidator]

	def clean(self):

		cleaned_data = self.cleaned_data

		#print '\n', cleaned_data, '\n'

		#for field in self.Meta.null_fields:
		#	if cleaned_data[field] == '':
		#		cleaned_data[field] = None

		return cleaned_data

	class Meta:
		model = MonitoredElement

		null_fields = [
			'icon',
		]

		fields = [
			'icon',       
			'description',
			'priority'
		]

		exclude = [
			'name',
			'uuid',
			'kind',
			'parent',
			'is_abstract',
			'agent',
			'tenant',   
			'mtime',
			'ctime',
		]

class MonitoredNodeForm(MonitoredElementForm):

	def __init__(self, *args, **kwargs):

		super(MonitoredNodeForm, self).__init__(*args, **kwargs)

		self.fields['dns_name'].initial          = None

		for elem in self.fields.values():
			elem.required = False

		#self.fields['dns_name'].required         = False
		#self.fields['dns6_name'].required        = False
		#self.fields['ip4_mgt'].required          = False
		#self.fields['ip6_mgt'].required          = False
		#self.fields['timezone'].required         = False
		#self.fields['snmp_v3_authdata'].required = False
		#self.fields['snmp_port'].required        = False
		#self.fields['dft_timeout'].required      = False
		#
		#self.fields['owner'].required            = False
		#self.fields['scheduling_group'].required = False
		#self.fields['telement_generator'].required   = False

		self.fields['cluster_xform'].validators  = [XformValidator(self.instance)]

	class Meta(MonitoredElementForm.Meta):
		model = MonitoredNode

		null_fields = MonitoredElementForm.Meta.null_fields + [
			'dns_name',
			'dns6_name',

			'ip4_mgt',
			'ip6_mgt',

			'snmp_v3_authdata',

			'cluster',              
			'cluster_role',         
			'cluster_distinguisher',
			'cluster_xform',        
			'cluster_instance'
		]

		fields = MonitoredElementForm.Meta.fields.append([
			'dft_timeout',          

			'use_ipv4',             
			'dns_name',             
			'dns6_name',            
			'ip4_mgt',              
			'ip6_mgt',              

			'snmp_version',         
			'snmp_port',            
			'snmp_community',       
			'snmp_v3_authdata',     

			'cluster',              
			'cluster_role',         
			'cluster_distinguisher',
			'cluster_xform',        
			'cluster_instance',     

			'is_router',
			'is_switch'
		])


class MonitoredServiceForm(MonitoredElementForm):

	def __init__(self, *args, **kwargs):

		super(MonitoredServiceForm, self).__init__(*args, **kwargs)

		#self.fields['distinguisher'].required = False
		#self.fields['xform'].required = False
		#self.fields['swrunindex'].required = False

		for elem in self.fields.values():
			elem.required = False

		self.fields['xform'].validators = [XformValidator(self.instance)]

	class Meta(MonitoredElementForm.Meta):
		model = MonitoredService

		null_fields = MonitoredElementForm.Meta.null_fields + [
			'distinguisher',
			'xform',
			'swrunindex'
		]

		fields = MonitoredElementForm.Meta.fields.append([
			'distinguisher',
			'xform',
			'swrunindex'
		])

class MonitoredStorageForm(MonitoredElementForm):

	def __init__(self, *args, **kwargs):

		super(MonitoredStorageForm, self).__init__(*args, **kwargs)

		#self.fields['distinguisher'].required = False
		#self.fields['xform'].required = False
		#self.fields['stindex'].required = False

		for elem in self.fields.values():
			elem.required = False

		self.fields['xform'].validators = [XformValidator(self.instance)]

	class Meta(MonitoredElementForm.Meta):
		model = MonitoredStorage

		null_fields = MonitoredElementForm.Meta.null_fields + [
			'distinguisher',
			'xform',
			'stindex'
		]

		fields = MonitoredElementForm.Meta.fields.append([
			'distinguisher',
			'xform',
			'stindex'
		])

class MonitoredDeviceForm(MonitoredElementForm):

	def __init__(self, *args, **kwargs):

		super(MonitoredDeviceForm, self).__init__(*args, **kwargs)

		#self.fields['distinguisher'].required = False
		#self.fields['xform'].required = False
		#self.fields['devindex'].required = False

		for elem in self.fields.values():
			elem.required = False

		self.fields['xform'].validators = [XformValidator(self.instance)]

	class Meta(MonitoredElementForm.Meta):
		model = MonitoredDevice

		null_fields = MonitoredElementForm.Meta.null_fields + [
			'distinguisher',
			'xform',
			'devindex'
		]

		fields = MonitoredElementForm.Meta.fields.append([
			'distinguisher',
			'xform',
			'devindex'
		])

class MonitoredIfaceForm(MonitoredElementForm):

	def __init__(self, *args, **kwargs):

		super(MonitoredIfaceForm, self).__init__(*args, **kwargs)

		#self.fields['distinguisher'].required = False
		#self.fields['xform'].required = False
		#self.fields['ifindex'].required = False

		for elem in self.fields.values():
			elem.required = False

		self.fields['xform'].validators = [XformValidator(self.instance)]

	class Meta(MonitoredElementForm.Meta):
		model = MonitoredIface

		null_fields = MonitoredElementForm.Meta.null_fields + [
			'distinguisher',
			'xform',
			'ifindex',
		]

		fields = MonitoredElementForm.Meta.fields.append([
			'distinguisher',
			'xform',
			'ifindex',
			'backbone'
		])


#-------------------------------------------------------------------------------#
#                 Validator per il cluster role                                 #
#-------------------------------------------------------------------------------#

def clusterRoleValidator(value):
	
	if value != 1 and value != 0:
		raise ValidationError("Incorrect Cluster Role value")

#-------------------------------------------------------------------------------#
#                        Validator per le xform                                 #
#-------------------------------------------------------------------------------#

class XformValidator(object):

	def __init__(self, element=None):
		
		if element is not None:
			self.element = element
	
	def __call__(self, value):
		
		
		try:
			compile_str_expr(None, value)
		
		except Exception, e:
			
			elem = ''
			if self.element.kind in ELEMENT_KIND_STR:
				elem = ELEMENT_KIND_STR[self.element.kind] + 's'
			
				if elem in XFORMS_ALIASES and value in XFORMS_ALIASES[elem]:
					value = XFORMS_ALIASES[ELEMENT_KIND_STR[self.element.kind]+'s'][value][0]
				else:
					raise ValidationError("This is not a valid alias or expression")
			
			else:
				raise ValidationError("Invalid xform")

#-------------------------------------------------------------------------------#
#                      Validator per i campi generici                           #
#-------------------------------------------------------------------------------#

def validate_data(form, risposta):

	if form.is_valid():
		mtime = time.time()
		
		#
		# FIX: DO NOT UPDATE FIELDS IN THIS WAY. Use concurrent_update()
		#
		
		form.instance.mtime = int(mtime)
		form.save()
	else:
		risposta['status'] = 'error'
		risposta['error_msg'] = 'Wrong field inputs'
		risposta['wrong_fields'] = form.errors


@tenant_dependent
def monitoredelement_config_fields(request, uuid):

	tenant = request.tenant
	db_set_current_tenant_schema(tenant.name)

	element = MonitoredElement.objects.get(uuid=uuid)
	element = element.get_specific_element()

	default_values = element.get_fields_default_value_dict()

	data = json.loads(request.POST['d'])

	risposta                 = {}
	risposta['status']       = 'success'
	risposta['wrong_fields'] = {}

	if int(data['mtime']) != int(element.mtime):
		risposta['status']    = 'error'
		risposta['mtime']     = data['mtime']
		risposta['error_msg'] = 'This element has been modified since it was loaded'

	else:
		data['mtime'] = time.time()

		if element.kind is ELEMENT_KIND_NODE:
			if data['cluster'] == 'null':
				del data['cluster']
			
			if data['cluster_xform']:
				data['cluster_xform'] = urllib.unquote(data['cluster_xform']).decode('utf-8')
			
			form = MonitoredNodeForm(data, instance=element, initial=default_values)
			validate_data(form, risposta)
	
		elif element.kind is ELEMENT_KIND_IFACE:
			
			if data['xform']:
				data['xform'] = urllib.unquote(data['xform']).decode('utf-8')
			
			form = MonitoredIfaceForm(data, instance=element, initial=default_values)
			validate_data(form, risposta)
	
		elif element.kind is ELEMENT_KIND_STORAGE:
			
			if data['xform']:
				data['xform'] = urllib.unquote(data['xform']).decode('utf-8')
			
			form = MonitoredStorageForm(data, instance=element, initial=default_values)
			validate_data(form, risposta)
	
		elif element.kind is ELEMENT_KIND_SERVICE:

			if data['xform']:
				data['xform'] = urllib.unquote(data['xform']).decode('utf-8')

			form = MonitoredServiceForm(data, instance=element, initial=default_values)
			validate_data(form, risposta)

		elif element.kind is ELEMENT_KIND_DEVICE:

			if data['xform']:
				data['xform'] = urllib.unquote(data['xform']).decode('utf-8')
			
			form = MonitoredDeviceForm(data, instance=element, initial=default_values)
			validate_data(form, risposta)

		risposta['mtime'] = element.mtime

	risposta = json.dumps(risposta)

	return HttpResponse(risposta)
