# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import time
import json

from django.conf import settings
from core.compatibility import reverse
from django.http import HttpResponse


from webui.decorators import tenant_dependent

from core.models import DataSource

from core.api.datasource import init_datasource_store_from_conf
from core.api.datasource import DataSourceStoreException

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

def get_start_end_params(request):
	ts_start = request.GET_OR_POST().get('s', None)
	if ts_start == None:
		ts_start = time.time() - 60*60*8
	ts_start = int(ts_start)
	
	ts_end   = request.GET_OR_POST().get('e', None)
	if ts_end == None:
		ts_end = time.time()
	ts_end = int(ts_end)

	return ts_start, ts_end


"""
OBSOLETO???

def get_new_start_end_params(request):
	ts_start = request.REQUEST.get('from', None)
	if ts_start == None:
		ts_start = time.time() - 60*60*8
	ts_start = int(ts_start)
	
	ts_end   = request.REQUEST.get('to', None)
	if ts_end == None:
		ts_end = time.time()
	ts_end = int(ts_end)

	return ts_start, ts_end
"""

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

@tenant_dependent
def fetch_data(request,uuid):

	tenant = request.tenant

	ts_start, ts_end = get_start_end_params(request)
	
	ds = DataSource.objects.get(uuid=uuid)

	data = get_datasource_dataset(tenant, ds, ts_start, ts_end)

	data_text = json.dumps(data)

	r = HttpResponse(data_text)
	return r


"""
OBSOLETO???

@tenant_dependent
def new_fetch_data(request,uuid):
	ds_id = uuid
	
	ts_start, ts_end = get_new_start_end_params(request)

	tenant = request.tenant
	
	ds = DataSource.objects.get(uuid=ds_id)
	
	try:
		data = get_new_datasource_dataset(tenant, ds, ts_start, ts_end)
	except DataSourceStoreException, e:
		data = []


	ret = {
	    'flags': {
	        'timeline': True, 				# //display timeline or not, default true
	        'controls': True, 				# // display conrtol buttons or not, default true
	        'locked': False, 				#// if the user can make selections and change range, default false
	        'dataview':True,
	    },
	    'url': "/sanet3/web/objects/datasources/%s/nfetch_data" % uuid, 					#// if missing the map goes static, with both uuid and url it's dynamic
	    'series': [data],
	    #'presel': string, 					#// key of the intervals, defaults to 1h
	    'scale': {            				#// mandatory as structure
	        'min':    None,     			#// mandatory but can be null
	        'max':    None,     			#// mandatory but can be null
	        'log':    False,     			#//if scale is logaritmic, handled via Flotr2, defaults to false
	        'unit':   None,					#// can be  missing/null
	    },
	    #'zorder':     [ number ] 			#// could be implied by series (i.e. missing)
	}
	#r['content-type'] = 'text/json'
	return HttpResponse(json.dumps(ret))
"""

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

NAMED_COLORS = {
	'yellow': 'ffff00',
	'red'   : 'ff0000',
	'blue'  : '0000ff',
	'green' : '00ff00',
}


def get_datasource_dataset(tenant, ds, ts_start=None, ts_end=None, datasource_store=None, style="line", color="0000ff", legend="", ylegend=""):

	if not datasource_store:
		datasource_store = init_datasource_store_from_conf()

	try:
		data = datasource_store.fetch_values(tenant.uuid, ds.uuid, ts_start=ts_start, ts_end=ts_end, resolution=60, min_value=ds.min_val, max_value=ds.max_val)
	except DataSourceStoreException, e:
		raise e

	ret = {}

	data_worked = []
	data_worked.append(data['scalex'])
	if legend != "":
		data_worked.append([legend])
	else:
		data_worked.append([ds.name,])
	dataseries = []
	for datapoint in data['dataseries']:
		dataseries.append(datapoint,)
		
	data_worked.append(dataseries)

	ret['values'] = data_worked
	ret["renderopts"] = {
		"series": [{
				"visNorm": -1 if style== "tick" else 1,
				'dataType':style,
				'label':legend,
				'unit':ylegend
			}
		],
		"zlist": [0]
	}
	
	if color == "0000ff":
		color = ""
	
	if color != "":
		if color.startswith('#'):
			color = color.split('#')[1]		 
		if color in NAMED_COLORS:
			color = NAMED_COLORS[color]
		color = "#"+color
		ret['renderopts']['series'][0]['color'] = color
	else:
		color = None
	
	return ret

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

def get_new_datasource_dataset(tenant, ds, ts_start=None, ts_end=None, datasource_store=None, style="line", color="", legend="", ylegend=""):
	
	if not datasource_store:
		datasource_store = init_datasource_store_from_conf()

	if ds.datagroup.minperiod == 0:
		step = 60  # fixme: review minperiod=0
	else:
		step = ds.datagroup.minperiod
        
	try:
		data = datasource_store.fetch_values(tenant.uuid, ds.uuid, ts_start=ts_start, ts_end=ts_end, resolution=step, min_value=ds.min_val, max_value=ds.max_val)
	except DataSourceStoreException, e:
		raise e

	scalex = data['scalex']
	e_ts_start = scalex[0]
	e_ts_end   = scalex[1]
	e_step     = scalex[2]

	return get_new_datasource_dataset_with_data(
		ds.path, 
		ts_start = e_ts_start, 
		ts_end   = e_ts_end, 
		ts_step  = e_step,
		values   = data['dataseries'],
		style    = style, 
		color    = color, 
		legend   = ds.name, 
		ylegend  = ylegend,
	)


def get_new_datasource_dataset_with_data(title, ts_start, ts_end, ts_step, values, style="line", color="", legend="", ylegend=""):

	if color == "0000ff":  #?????????????????????????? PERCHE'???
		color = ""

	if color != "":
		if color.startswith('#'):
			color = color.split('#')[1]
		if color in NAMED_COLORS:
			color = NAMED_COLORS[color]
		color = "#"+color
	else:
		color = None

	ret = {
		'title': title,
		'label': legend, 			#// mandatory
		'timescale': {        			#// all mandatory
			'from': ts_start, 		#// Python timestamp
			'to'  : ts_end, 		#// Python timestamp
			'step': ts_step,  		#// Python timestamp
		},
		'values'    : values, 			#//mandatory
		'format'    : style, 			#string(line | area | bars | tick | bartick), //defaults to bars|bartick
		'tick'      : style == "tick",		#// default false, determines if the series is normalised to tick
		'scaleauth' : True, 			#// default true, ignored if tick is true, if the series contributes to scaling when applicable (min/max == null)
		'threshold' : 0, 			#// markers only?? could be used for callout TOO, default is 0	
		'threshside': True, 			#//defaults to true (greater than) if missing
		'color'     : color, 			#// defaults to lwp standard colors
        }

	return ret

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

import json
import base64
from django.template import RequestContext, loader, Context
from core.models import TimeGraph
from webui.api.widgets import DataSourceGraphWidget, TimeGraph3Widget

@tenant_dependent
def graph_page(request, uuid):

	user   = request.user
	tenant = request.tenant

	#
	# In base al datasource vediamo se visualizzare solo il datasource o il
	# primo TimeGraph presente nello stesso DataGroup.
	#
	ds = DataSource.objects.get(uuid=uuid)

	tg = TimeGraph.objects.filter(datagroup=ds.datagroup).order_by('primary','name')[:1]
	if len(tg) > 0:
		tg = tg[0]
	else:
		tg = None

	#
	# Only 1 Widget
	#

	widgets = []

	if tg:
		w = TimeGraph3Widget     (tg, tab='general', container='default', height='max').to_dict()
	else:
		w = DataSourceGraphWidget(ds, tab='general', container='default', height='max').to_dict()

	widgets.append(w)

	page_structure =  {
		'widgets': widgets,
	}

	#
	# SETUP PAGE
	#
	page_structure_encoded   = base64.b64encode(json.dumps(page_structure))

	page_settings = {
		'DEBUG_MODE'                 : settings.DEBUG_JS, 
		'prefix'                     : reverse('core.views.index'),

		'page_structure'             : page_structure,
		'page_structure_encoded'     : page_structure_encoded,
	}

	#
	# RENDER PAGE
	#
	PAGE_HTML_TEMPLATE = 'html/pages/datasource_graph_page.html'

	c = RequestContext(request, page_settings)
	t = loader.get_template(PAGE_HTML_TEMPLATE )
	html_text = t.render(c)	

	return HttpResponse(html_text)
