# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import json


from django.http import HttpResponse
from django import forms
from django.forms import Form
from django.contrib.auth.models import User, Group

from core.compatibility import reverse

from core.models import Tenant
from core.models import Capability

from core.api.base import group_create
from core.api.base import _user_add_or_remove_to_group, tenant_toggle_group, group_delete

from webui.decorators import tenant_dependent

from webui.api.utils import render_template
from webui.api.response import HttpJsonSuccessResponse

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

@tenant_dependent
def delete(request):

	group_id = int(request.GET_OR_POST()['groupid'])
	group = Group.objects.get(id=group_id)

	group_delete(group.name)

	return HttpResponse("ok")

@tenant_dependent
def toggle_user_association(request):

	user_id  = int(request.GET_OR_POST()['userid'] )
	group_id = int(request.GET_OR_POST()['groupid'])

	user  = User.objects.get(id=user_id)
	group = Group.objects.get(id=group_id)

	added = _user_add_or_remove_to_group(user, group)

	#return HttpResponse(str(added))
	return HttpJsonSuccessResponse(data={"added":added})

@tenant_dependent
def toggle_tenant_association(request):

	tenant_id  = request.GET_OR_POST()['tenantid']
	group_id   = int(request.GET_OR_POST()['groupid'])

	t = Tenant.objects.get(uuid=tenant_id)
	g = Group.objects.get (id=group_id)

	added = tenant_toggle_group(t, g)

	#return HttpResponse(added)
	return HttpJsonSuccessResponse(data={"added":added})

@tenant_dependent
def toggle_capability(request):

	capability_id  = request.GET_OR_POST()['capabilityid']
	group_id       = int(request.GET_OR_POST()['groupid'])
	tenant_id      = request.GET_OR_POST()['tenantid']

	t = Tenant.objects.get(uuid=tenant_id)
	g = Group.objects.get (id=group_id)

	c, created = Capability.objects.get_or_create(tenant=t, group=g, code=capability_id)

	if created:
		added = "true"
	else:
		c.delete()
		added = "false"

	#for c in Capability.objects.filter(tenant=t, group=g).order_by('code').all():
	#	print c.code
	
	return HttpJsonSuccessResponse(data={"added":added})

	#return HttpResponse(added)

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class GroupForm(Form):
	name     = forms.CharField   (label='Name'        , required=True)

def create(request):
	
	if request.method == 'POST':
		form = GroupForm(request.POST) # A form bound to the POST data
		if form.is_valid(): 
			#user = form.save(commit=False)
			#user.save()
			
			name = form.cleaned_data['name']
			
			group = group_create(name)
			
			return HttpResponse('OK')
	else:
		form = GroupForm(initial={}) 


	html = render_template('forms/discovery_generic_field_form.html', { 
		'action': reverse('group_create'),
		'form': form, 
		'button_label': 'Confirm',
	})

	return HttpResponse(html)
