# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os
import re
import time
import datetime
import csv
import json
import pprint

from django.http import HttpResponse

from sanet_common.uuids import is_uuid
from sanet_common.sanetdefs.constants import CSTATE_TO_CODENAME

from core.compatibility import FileWrapper

from core.models import Event, Alarm, MonitoredElement
from core.api.base import db_set_current_tenant_schema, tenant_get_media_file_path

from tags.models import Tag

from webui.decorators import tenant_dependent
from webui.api.types import WEB_ELEMENT_TYPE_TO_MODEL

from webui.api.Data.logs import _get_logs

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#


@tenant_dependent
def get_details(request):
	
	user = request.user
	
	tenant = request.tenant
	db_set_current_tenant_schema(tenant.name)
	
	uuid = request.GET_OR_POST()['uuid']
	
	l = Event.objects.get(tenant=tenant, uuid=uuid)
	
	# Normal user can view "sensibile data" only with the capability. #545
	if user is not None:
		if 'gui.view_sensible_data' not in user.get_capabilities(tenant):
				l.hide_sensible_data()	
	
	response_data = {
		'msg': l.msg
	}
	
	return HttpResponse(json.dumps(response_data))


@tenant_dependent
def get_entables_details(request):
	
	
	from entables.models import AlarmArchive as ArchivedAlarm
	from entables.models import Alarm        as PendingAlarm
	
	user = request.user
	
	tenant = request.tenant
	tenant.db_set_current_schema()
	
	uuid = request.GET_OR_POST()['uuid']
	
	l = Event.objects.get(tenant=tenant, uuid=uuid)
	
	if user is not None:
		if 'gui.view_sensible_data' not in user.get_capabilities(tenant):
			# FIX: ugly, but works
			l.hide_sensible_data()
	
	try:
		sanet_a = Alarm.objects.get(event=l)
	except Alarm.DoesNotExist, e:
		sanet_a = None
		

	entables_a = None
	if sanet_a is not None:
		try:
			entables_a = ArchivedAlarm.objects.get(uuid=sanet_a.uuid)
		except ArchivedAlarm.DoesNotExist, e:
			
			try:
				entables_a = PendingAlarm.objects.get(uuid=sanet_a.uuid)
			except PendingAlarm.DoesNotExist, e2:
				entables_a = None
			
	if entables_a is not None:

		original_alarm = entables_a.get_data()
		
		if user is not None:
			if 'gui.view_sensible_data' not in user.get_capabilities(tenant):
				# FIX: ugly, but works
				data_str = json.dumps(original_alarm)
				data_str = Event.hide_sensible_data_in_msg(data_str)
				original_alarm = json.loads(data_str)

		
		stages         = entables_a.get_stages()
		
		msg = ''
		msg += 'STAGES\n'
		msg += '======\n'
		msg += '\n'
		msg += '\n'.join([ str(s) for s in stages ] ) + '\n'
		msg += '\n'
		msg += 'DATA\n'
		msg += '====\n'
		msg += '\n'
		msg += pprint.pformat( original_alarm ) + '\n'
	
	else:
		msg = "*** No notification action recordered for this event ***"

	data = {
		'msg': msg
	}
	
	return HttpResponse(json.dumps(data))
		

#------------------------------------------------------------------------------#
#                                                                              #
#------------------------------------------------------------------------------#


def _timestamp_to_isostr(timestamp):
	try:
		return datetime.datetime.fromtimestamp(timestamp).strftime('%Y-%m-%d %H:%M:%S')
	except TypeError:
		return ''
		
@tenant_dependent
def export_csv(request):
	#return _export_csv(request)
	
#def _export_csv(request, entity=None):
	
	user    = request.user

	uuid    = request.GET_OR_POST()['uuid']
	etype   = request.GET_OR_POST().get('etype', None)
	#options = request.GET_OR_POST().get('options', "{}")
	
	
	#
	# Load element
	#
	tenant = request.tenant
	db_set_current_tenant_schema(tenant.name)



	#
	# FIX: refactory di questo codice perche' fa schifo fare questi "if" e questi catch fatti cosi'
	#

	#element = MonitoredElement.objects.get(uuid=uuid)
	#element = element.get_specific_element()

	if etype is not None:
		model = WEB_ELEMENT_TYPE_TO_MODEL[etype]
		if is_uuid(uuid):
			element = model.objects.get(uuid=uuid)
		else:
			element = model.objects.get(name=uuid)
	else:
		try:
			entity = Tag.objects.get(uuid = uuid)
		except Tag.DoesNotExist, e:
			try:
				entity = MonitoredElement.objects.get(uuid=uuid)
			except MonitoredElement.DoesNotExist, e:
				if uuid.find(":") != -1: #found
					node, sub = uuid.split(":")
					entity = MonitoredElement.objects.get(parent__name = node, name = sub)
				else:
					try:
						entity = MonitoredElement.objects.get(name = uuid)
					except Exception, e:
						print e
						raise e
			entity = entity.get_specific_element()

		element = entity
		
	return _export_csv(request, element)

def _export_csv(request, element):
	
	user    = request.user
	tenant = request.tenant
	db_set_current_tenant_schema(tenant.name)

	options = request.GET_OR_POST().get('options', "{}")
	options = json.loads(options)
	
	#
	# Extract data
	#
	logs = _get_logs(user, element, options, return_query_set=True, no_pagesize=True)
	logs = logs.values_list('tenant__name', 'timestamp', 'element_path', 'condition_path', 'status', 'statuslast', 'priority', 'msg')

	logs2 = []
	for l in logs:
		l = list(l)
		l[1] = _timestamp_to_isostr(l[1])
		l[4] = CSTATE_TO_CODENAME(l[4])
		l[5] = CSTATE_TO_CODENAME(l[5])
		logs2.append(l)
		
	logs = logs2

	#
	# Create file
	#
	t = int(time.time())
	
	
	path_valid = element.path.replace(':', '-').replace('/','-')
	
	filename = 'log-%s-%s_%s.csv' % (path_valid, _timestamp_to_isostr(t).replace(" ","_"), t)   #'%s.csv' % (int(time.time()))
	file_path = tenant_get_media_file_path(tenant.uuid, filename)

	f = open(file_path, 'wb')
	spamWriter = csv.writer( f , delimiter=',' , quotechar='"' , quoting=csv.QUOTE_MINIMAL )
	
	for l in logs:
		spamWriter.writerow( l )

	f.close()
	
	#
	# Send
	#
	
	f = open(file_path, "r+")
	wrapper = FileWrapper(f)	
	response = HttpResponse(wrapper, content_type='text/csv')
	response['Content-Disposition'] = 'attachment; filename=%s' % (filename)
	response['Content-Length'] = os.path.getsize(file_path)			
	response['Cache-Control'] = 'no-cache'
	response['Pragma']        = 'no-cache'
	
	return response
