# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import json
import operator
import csv

from django.conf import settings

from django import db
from django.db import connection
from django.http import HttpResponse

from core.compatibility import reverse

from core.models import MonitoredNode, UserCustomSetting 
from core.api.base import db_set_current_tenant_schema

from tags.models import Tag

from webui.decorators import tenant_dependent

#
# FIX: poiche' queste view servono per interagire con il componene 
#


#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

# Questa e' l'identificativo usato per memorizzare negli UserCustomSettings
# i parametri di filtraggio.
USER_TENANT_NODES_DATA_FILTERS = 'tenant_nodes_data.filter_settings'

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

COL_MODEL_DICT = {
	'Name'              : 'monitoredelement.name', 
	'Agent'             : 'agent.name AS agent', 
	'Cluster'           : 'cluster.name AS cluster', 
	'Priority'          : 'monitoredelement.priority', 
	'Description'       : 'monitoredelement.description', 
	'IPv4 address'      : 'node.ip4_mgt', 
	'IPv6 address'      : 'node.ip6_mgt', 
	'DNS name (A)'      : 'node.dns_name', 
	'DNS v6 name (AAAA)': 'node.dns6_name',
	'SNMP v'            : 'node.snmp_version',
	'SNMP Community'    : 'node.snmp_community',
	'SNMP p'            : 'node.snmp_port',
	'Switch'            : 'node.is_switch',
	'Router'            : 'node.is_router',
	'Template'          : 'template.name AS template',         
	'url'               : 'monitoredelement.uuid',
}

#COL_ORD_DICT = COL_MODEL_DICT

# FIX: QUESTO DIZIONARIO SEMBRA NON SERIVRE. BASTA COL_MODEL_DICT.
#
COL_ORD_DICT = {
	'Name'              : 'monitoredelement.name', 
	'Agent'             : 'agent', 
	'Cluster'           : 'cluster', 
	'Priority'          : 'monitoredelement.priority', 
	'Description'       : 'monitoredelement.description', 
	'IPv4 address'      : 'node.ip4_mgt', 
	'IPv6 address'      : 'node.ip6_mgt', 
	'DNS name (A)'      : 'node.dns_name', 
	'DNS v6 name (AAAA)': 'node.dns6_name',
	'SNMP v'            : 'node.snmp_version',
	'SNMP Community'    : 'node.snmp_community',
	'SNMP p'            : 'node.snmp_port',
	'Switch'            : 'node.is_switch',
	'Router'            : 'node.is_router',
	'Template'          : 'template',                           
	'url'               : 'monitoredelement.uuid',
}

COL_INVERSE_DICT = {
	'name'          : 'Name',
	'agent'         : 'Agent',
	'cluster'       : 'Cluster',           
	'priority'      : 'Priority',          
	'description'   : 'Description',       
	'ip4_mgt'       : 'IPv4 address',      
	'ip6_mgt'       : 'IPv6 address',      
	'dns_name'      : 'DNS name (A)',      
	'dns6_name'     : 'DNS v6 name (AAAA)',
	'snmp_version'  : 'SNMP v',            
	'snmp_community': 'SNMP Community',    
	'snmp_port'     : 'SNMP p',            
	'is_switch'     : 'Switch',            
	'is_router'     : 'Router',            
	'template'      : 'Template',                               
	'latitude'      : 'Latitude',          
	'longitude'     : 'Longitude',         
	'uuid'          : 'url'               
}


#
# Pezzi di query
#
BASE_QUERY = '''
 FROM node FULL OUTER JOIN monitoredelement ON (node.monitoredelement_ptr_id = monitoredelement.uuid)
'''

TAG_QUERY          = '''          LEFT JOIN tags_elementtag    ON (tags_elementtag.elementid         = monitoredelement.uuid) LEFT JOIN tagentry               ON (tagentry.uuid = tags_elementtag.tag_id)
'''


TEMPLATE_QUERY     = '''          LEFT JOIN (
                     SELECT melement_id, string_agg(template_name, ',') as name
                       FROM (
                             SELECT et.melement_id, te.name as template_name
                               FROM c_element_template et INNER JOIN c_telement te ON (et.telement_id    = te.id)
                              ORDER BY te.name                            
                            ) t
                      GROUP BY melement_id
                    ) template ON (monitoredelement.uuid = template.melement_id)
'''

AGENT_QUERY        = '''          LEFT JOIN agent              ON (monitoredelement.agent_id         = agent.uuid)
'''

CLUSTER_QUERY      = '''          LEFT JOIN cluster            ON (monitoredelement.cluster_instance = cluster.uuid)
'''


LIMIT_OFFSET = '''
LIMIT %s OFFSET %s
'''

ORDER_BY_ASC = '''
ORDER BY %s ASC
'''

ORDER_BY_DESC = '''
ORDER BY %s DESC
'''

WHERE_SELECTOR = '''
WHERE monitoredelement.kind = 1 
'''

WHERE_END_SELECTOR = '''
      )
'''

WHERE_C_FIELD      = ''' "c_value_%(group)s_%(field)s".value LIKE '{0}' '''


WHERE_DICT = {
	'monitoredelement.name'       : "monitoredelement.name                   LIKE '{0}' ",
	'monitoredelement.priority'   : "CAST(monitoredelement.priority AS text) LIKE '{0}' ",
	'monitoredelement.description': "monitoredelement.description            LIKE '{0}' ",
	'template.name AS template'   : "template.name                           LIKE '{0}' ", 
	'agent.name AS agent'         : "agent.name                              LIKE '{0}' ",
	'cluster.name AS cluster'     : "cluster.name                            LIKE '{0}' ",
	'node.ip4_mgt'                : "CAST(node.ip4_mgt AS text)              LIKE '{0}' ",
	'node.ip6_mgt'                : "CAST(node.ip6_mgt AS text)              LIKE '{0}' ",
	'node.dns_name'               : "CAST(node.dns_name AS text)             LIKE '{0}' ",
	'node.dns6_name'              : "CAST(node.dns6_name AS text)            LIKE '{0}' ",
	'node.snmp_version'           : "CAST(node.snmp_version AS text)         LIKE '{0}' ",
	'node.snmp_port'              : "CAST(node.snmp_port AS text)            LIKE '{0}' ",
	'node.snmp_community'         : "node.snmp_community                     LIKE '{0}' ",
}

FIELD_DICT = {
	'agent.name AS agent'      : AGENT_QUERY,
	'template.name AS template': TEMPLATE_QUERY,  
	'cluster.name AS cluster'  : CLUSTER_QUERY,
}





MODE_COUNT  = 1
MODE_SELECT = 2

def _create_query(tenant, search_value, columns, hidden, custom_fields, tag, mode=MODE_SELECT, order=None, limit=None, offset=None):

	geo_flag        = False
	where_list      = []
	where_tag_value = ()
	order_by        = ''
	select          = []
	final_query     = ''
	
	final_query = BASE_QUERY
	
	where_s     = WHERE_SELECTOR
	
	#
	# Costruisco la query
	#
	if tag:
		final_query        = final_query + TAG_QUERY

		where_s            = where_s + """
		AND (
			tagentry.path = %s  -- tag itself
			or 
			tagentry.path like %s -- subtags
		)
"""
		where_tag_value = where_tag_value + (tag, tag+'/%%',)

	for column in columns:

		if column not in hidden:

			if column in FIELD_DICT:
				
				if not added:
					final_query = final_query + FIELD_DICT[column]

			if column in WHERE_DICT:
				where_list.append(WHERE_DICT[column])

			if mode == MODE_SELECT: 
				select.append(column)

	
	
	if search_value:
		where_s += """
  AND (
"""
    
		where_s += "       " + "\n    OR ".join(where_list)
		where_s = where_s.format('%%' + search_value + '%%')
	
		where_s += """
        )
"""

	QUERY = ''
	
	#
	# SELECT
	#
	selected_fields = select
	
	if mode == MODE_COUNT:
		select = "SELECT DISTINCT monitoredelement.uuid AS uuid"
	else:
		select = "SELECT DISTINCT\n                "
		select += "\n                , ".join(selected_fields)

	QUERY += select

	#
	# FROM
	#
	QUERY += final_query 
	
	#
	# WHERE
	#
	QUERY += where_s
	
	if mode == MODE_COUNT:
	
		QUERY = "SELECT COUNT(counter.uuid) FROM (\n"+QUERY+") AS counter" 
	else:
		#
		# ORDER BY
		#
		if order[1] == 'asc':
			order_by = ORDER_BY_ASC % order[0]
		
		elif order[1] == 'desc':
			order_by = ORDER_BY_DESC % order[0]
		
		QUERY += order_by
		
		
		#
		# LIMIT
		#
		if not limit is None and not offset is None:
			QUERY += LIMIT_OFFSET % (limit, offset)

	
	#
	# Eseguo la query
	#

	return QUERY, where_tag_value

#-------------------------------------------------------------------------------#
#       Funzione che conta il numero di elementi che matchano la ricerca        #
#-------------------------------------------------------------------------------#

def query_count(tenant, search_value, columns, hidden, custom_fields, tag):

	cursor          = connection.cursor()
	db_set_current_tenant_schema(tenant.name, db_cursor=cursor)

	query_string, query_params = _create_query(tenant, search_value, columns, hidden, custom_fields, tag, mode=MODE_COUNT)
	
	cursor.execute(query_string, query_params)

	return cursor.fetchall()[0][0]

#-------------------------------------------------------------------------------#
#       Funzione che prepara ed esegue la query solo sui dati richiesti         #
#-------------------------------------------------------------------------------#

def perform_query(tenant, search_value, order, columns, hidden, custom_fields, tag, requestor=None, limit=None, offset=None):

	cursor          = connection.cursor()
	tenant.db_set_current_schema(db_cursor=cursor)

	query_string, query_params = _create_query(tenant, search_value, columns, hidden, custom_fields, tag, mode=MODE_SELECT, order=order, offset=offset, limit=limit)

	cursor.execute(query_string, query_params)
	
	def dictfetchall(cursor):
		"Returns all rows from a cursor as a dict"
		desc = cursor.description
		return [
			dict(zip([col[0] for col in desc], row))
			for row in cursor.fetchall()
		]
	
	#
	# Se la richiesta viene da datatables restituisco un dizionario
	# se viene dalla view di esportazione in csv restituisco una tupla
	#
	if requestor == "table":
		res = dictfetchall(cursor)
	elif requestor == "csv":
		res = cursor.fetchall()
	
	return res

#-------------------------------------------------------------------------------#
#                    Funzione di salvataggio configurazione                     #
#-------------------------------------------------------------------------------#

def save_table_configuration(user, tenant, data):
	
	settings = {}

	current_configuration, created = UserCustomSetting.objects.get_or_create(tenant=tenant, user=user, key=USER_TENANT_NODES_DATA_FILTERS)
	if not created:
		settings = current_configuration.get_value()

	for k, v in data.items():
		if not k in settings or v != settings[k]:
			settings[k] = v

	current_configuration.set_value(settings)
	current_configuration.save()

#-------------------------------------------------------------------------------#
#                 View di salvataggio configurazione tabella                    #
#-------------------------------------------------------------------------------#

@tenant_dependent
def save_configuration(request):
	#
	# FIX: spiegare nel dettaglio a cosa serve questa view.
	#

	tenant = request.tenant
	db_set_current_tenant_schema(tenant.name)

	user = request.user

	
	order                        = json.loads(request.POST['col_order'])

	#
	# FIX: Spiegare il significato di questi parametri e come devono essere gestiti
	#
	table_conf                   = {}
	table_conf['columns']        = json.loads(request.POST['columns'])
	table_conf['hidden_columns'] = json.loads(request.POST['hidden'])
	table_conf['length']         = json.loads(request.POST['length'])
	table_conf['cell_order']     = json.loads(request.POST['cell_order'])
	
	order.append('url')
	
	table_conf['columns'].sort(key=lambda x: order.index(x['data']))
	
	save_table_configuration(user, tenant, table_conf)
	
	return HttpResponse()

#-------------------------------------------------------------------------------#
#              View di richiesta dati per costruire la tabella                  #
#-------------------------------------------------------------------------------#

@tenant_dependent
def get_table_data(request):
	#
	# FIX: spiegare nel dettaglio a cosa serve questa view.
	#
	
	tenant       = request.tenant
	user         = request.user
	
	default_base_fields = {
	
		#
		# FIX: Scrivere qui cosa a cosa servono ogniuno di questi campi 'group', 'name', 'data', 'sName', 'mData'
		#
	
		'Name'               : {'group':'Base fields', 'name':'Name'              , 'data':'Name'              , 'sName':'Name'              , 'mData': 'Name'              },
		'Agent'              : {'group':'Base fields', 'name':'Agent'             , 'data':'Agent'             , 'sName':'Agent'             , 'mData': 'Agent'             },
		'Cluster'            : {'group':'Base fields', 'name':'Cluster'           , 'data':'Cluster'           , 'sName':'Cluster'           , 'mData': 'Cluster'           },
		'Default Priority'   : {'group':'Base fields', 'name':'Priority'          , 'data':'Priority'          , 'sName':'Priority'          , 'mData': 'Priority'          },
		'Description'        : {'group':'Base fields', 'name':'Description'       , 'data':'Description'       , 'sName':'Description'       , 'mData': 'Description'       , 'className': 'wrapper', 'sClass': 'wrapper'},
		'IPv4 address'       : {'group':'Base fields', 'name':'IPv4 address'      , 'data':'IPv4 address'      , 'sName':'IPv4 address'      , 'mData': 'IPv4 address'      },
		'IPv6 address'       : {'group':'Base fields', 'name':'IPv6 address'      , 'data':'IPv6 address'      , 'sName':'IPv6 address'      , 'mData': 'IPv6 address'      },
		'DNS name (A)'       : {'group':'Base fields', 'name':'DNS name (A)'      , 'data':'DNS name (A)'      , 'sName':'DNS name (A)'      , 'mData': 'DNS name (A)'      },
		'DNS v6 name (AAAA)' : {'group':'Base fields', 'name':'DNS v6 name (AAAA)', 'data':'DNS v6 name (AAAA)', 'sName':'DNS v6 name (AAAA)', 'mData': 'DNS v6 name (AAAA)'},
		'SNMP version'       : {'group':'Base fields', 'name':'SNMP v'            , 'data':'SNMP v'            , 'sName':'SNMP v'            , 'mData': 'SNMP v'            },
		'SNMP Community'     : {'group':'Base fields', 'name':'SNMP Community'    , 'data':'SNMP Community'    , 'sName':'SNMP Community'    , 'mData': 'SNMP Community'    },
		'SNMP UDP port'      : {'group':'Base fields', 'name':'SNMP p'            , 'data':'SNMP p'            , 'sName':'SNMP p'            , 'mData': 'SNMP p'            },
		'Switch capabilities': {'group':'Base fields', 'name':'Switch'            , 'data':'Switch'            , 'sName':'Switch'            , 'mData': 'Switch'            },
		'Router capabilities': {'group':'Base fields', 'name':'Router'            , 'data':'Router'            , 'sName':'Router'            , 'mData': 'Router'            },
		'Template'           : {'group':'Base fields', 'name':'Template'          , 'data':'Template'          , 'mData':'Template'          , 'sName': 'Template'          },

		'url'                : {'group':'Base fields', 'name':'url'               , 'data':'url'               , 'mData':'url'               , 'sName': 'url'               }
	}
	
	
	default_fields_list = [ default_base_fields[x] for x in default_base_fields ]
	
	#
	# TAG
	#
	tag_list = Tag.objects.order_by('path').all()
	
	tmp = []
	
	for tag in tag_list:
		tmp.append(tag.path)
	
	tag_list = tmp
	
	#
	# Cerco una configurazione salvata sul database, se non esiste
	# ne creo una di default e la salvo
	#
	current_configuration, created = UserCustomSetting.objects.get_or_create(tenant=tenant, user=user, key=USER_TENANT_NODES_DATA_FILTERS)
	created = True
	if created:
		table_columns = []
		
		for field in MonitoredNode.get_fields_verbose_names(MonitoredNode):
			if field in default_base_fields:
				table_columns.append(default_base_fields[field])
		
		#
		# Aggiungo i campi non presenti nel modello dei nodi
		#
		table_columns.append(default_base_fields['Template'])
		table_columns.append(default_base_fields['url'])
		
		table_conf                   = {}
		table_conf['columns']        = table_columns
		table_conf['hidden_columns'] = []
		table_conf['length']         = 10
		table_conf['cell_order']     = [[0, 'asc']]
		
		current_configuration.set_value(table_conf)
		current_configuration.save()
	else:
		table_conf = current_configuration.get_value()
	
	data = {
	
		# Informazioni sulle colonne da visualizzare e altri parametri utili per inizializzare l'oggetto datatable.
		'table_conf': table_conf,
		
		# Elenco dei tag da visualizzare nel form di ricerca
		'tag_list'  : tag_list
	}
	
	return HttpResponse(json.dumps(data))

#-------------------------------------------------------------------------------#
#                   View di richiesta dati per datatables                       #
#-------------------------------------------------------------------------------#

@tenant_dependent
def tenant_nodes_data(request):
	#
	# FIX: spiegare nel dettaglio a cosa serve questa view.
	#
	
	tenant = request.tenant
	user = request.user
	
	hid = []
	
	t_request      = json.loads(request.POST['d'])
	hidden_columns = json.loads(request.POST['hidden'])
	custom_fields  = json.loads(request.POST['custom_fields'])
	tag            = json.loads(request.POST['tag'])
	
	last_search    = json.loads(request.POST['last_search'])

	#
	# Preparo i dati da passare a perform_query()
	#
	if tag == 'None':
		tag = None
	
	if last_search['tag'] == 'None':
		last_search['tag'] = None
	
	for column in hidden_columns:
		
		if column in COL_MODEL_DICT:
			hid.append(COL_MODEL_DICT[column])
	
	for column in custom_fields:
		
		if '/'.join( [ column['group'], column['title'] ] ) in hidden_columns:
			hid.append( '"c_value_'+column['group']+'_'+column['field_name']+'".value AS "c_value_'+column['group']+'_'+column['field_name']+'"' )
	
	
	table_conf                   = {}
	table_conf['columns']        = json.loads(request.POST['col_conf'])
	table_conf['hidden_columns'] = hidden_columns
	table_conf['length']         = t_request['length']
	
	if len(hidden_columns) == len(table_conf['columns'])-1:
		return HttpResponse()
	
	columns = []
	s_columns = []
	
	ord_col = t_request['order'][0]['column']
	
	
	for column in table_conf['columns']:
		
		if column['name'] in COL_MODEL_DICT:
			columns.append(COL_MODEL_DICT[column['name']])
		
		else:
			columns.append('"c_value_'+column['group']+'_'+column['field_name']+'".value AS "c_value_'+column['group']+'_'+column['field_name']+'"')
	
	total = MonitoredNode.objects.count()
	
	#print "ORDER1", t_request['order']
	
	if ord_col['name'] in COL_MODEL_DICT:
		order = (COL_ORD_DICT[ ord_col['name'] ], t_request['order'][0]['dir'])
	
	else:
		order = ('"c_value_'+ord_col['group']+'_'+ord_col['field_name']+'"', t_request['order'][0]['dir'])

	#print "ORDER2", order
	
	nodes_data = perform_query(tenant, t_request['search']['value'], order, columns, hid, custom_fields, tag, "table", limit=t_request['length'], offset=t_request['start'])

	print nodes_data
		
	
	if last_search['search'] != t_request['search'] or last_search['tag'] != tag:
		filtered = query_count(tenant, t_request['search']['value'], columns, hid, custom_fields, tag)
	else:
		filtered = json.loads(request.POST['last_filtered'])
	
	
	
	data = []
	
	#
	# Costruisco il dizionario da mandare a datatables
	#
	for node_row_dict in nodes_data:
		
		row = {}
		
		for k, v in COL_INVERSE_DICT.items():
			
			if k == 'uuid' and k in node_row_dict:
				row[v] = reverse('resource_page', kwargs={'rt': 'node', 'uuid': node_row_dict[k] })
			
			elif k in node_row_dict:
				row[v] = node_row_dict[k]
			
			else:
				row[v] = ''
			
		for col in custom_fields:
			
			if 'c_value_'+col['group']+'_'+col['field_name'] in node_row_dict:
				row[col['name']] = node_row_dict['c_value_'+col['group']+'_'+col['field_name']]
			
			else:
				row[col['name']] = ''
		
		data.append(row)
	
	
	#
	# Struttura dati di DataTable da mandare esattamente cosi' (nessuna "modifica" lato client)
	#
	r_data = {
		'draw'           : t_request['draw'],
		'recordsTotal'   : total,
		'recordsFiltered': filtered,
		'data'           : data,
	}

	return HttpResponse(json.dumps(r_data))

#-------------------------------------------------------------------------------#
#                   View di esportazione della tabella in csv                   #
#-------------------------------------------------------------------------------#

def _csv_struct_encode(struct):
	'''
	Codifica i caratteri estratti in ASCII per passarli a un oggetto csv.writer
	'''
	return [[ s.encode('ascii', errors='backslashreplace', ) if type(s) == unicode else s for s in elem ] for elem in struct ]



@tenant_dependent
def export_csv(request):
	
	tenant    = request.tenant
	
	t_request = json.loads(request.POST['d'])
	
	columns       = []
	cell_order    = t_request['cell_order']
	col_order     = t_request['order'][0]['column']
	hidden_columns= t_request['hidden']
	custom_fields = t_request['custom_fields']
	limit         = None
	offset        = None
	tag           = t_request['tag']
	header        = ()
	
	
	if 'length' in t_request:
		limit = t_request['length']
		
	if 'start' in t_request:
		offset = t_request['start']
	
	if tag == 'None':
		tag = None
	
	#
	# Preparo i dati da passare a perform_query()
	#
	
	t_request['columns'].sort(key=lambda x: cell_order.index(x['data']))
	
	for column in t_request['columns']:
		if column['name'] in COL_MODEL_DICT:
			columns.append(COL_MODEL_DICT[column['name']])
			header += (column['name'],)
	
		else:
			c_column = custom_fields[ next( index for (index, d) in enumerate(custom_fields) if d["name"] == column['name'] ) ]
			columns.append('"c_value_'+c_column['group']+'_'+c_column['field_name']+'".value AS "c_value_'+c_column['group']+'_'+c_column['field_name']+'"')
			header += (c_column['title'],)
	
	hid = []
	
	for column in hidden_columns:
		
		if column in COL_MODEL_DICT:
			hid.append(COL_MODEL_DICT[column])
	
	
	for column in custom_fields:
		
		if '/'.join( [ column['group'], column['title'] ] ) in hidden_columns:
			hid.append('"c_value_'+column['group']+'_'+column['field_name']+'".value AS "c_value_'+column['group']+'_'+column['field_name']+'"')
	
	if col_order['name'] in COL_MODEL_DICT:
		order = (COL_ORD_DICT[ col_order['name'] ], t_request['order'][0]['dir'])
	
	else:
		order = ('c_value_'+col_order['group']+'_'+col_order['field_name'], t_request['order'][0]['dir'])
	
	#
	# Genera CSV
	#
	res = perform_query(tenant, t_request['search']['value'], order, columns, hid, custom_fields, tag, "csv", limit=limit, offset=offset)
	
	res = _csv_struct_encode(res)
	res.insert(0, header)
	
	response = HttpResponse(content_type='text/csv')
	response['Content-Disposition'] = 'attachment; filename="exported_table.csv"'
	
	cw = csv.writer(response)
	cw.writerows(res)
	
	return response


