# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import os

from django.http import HttpResponse

from conf import get_configuration

from core.api.sysstats import SANETD_STAT_DS_ELEMENTS, SANETD_COLORS, sanetd_create_rrd_graph_file_elements
from core.api.sysstats import sysstats_create_rrd_graph_file

from webui.views.objects.agents import ts_or_timespec

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

def sanetd_stat_chart(request, instanceid, chart):

	ts_start = request.GET_OR_POST().get('s', None)
	ts_end   = request.GET_OR_POST().get('e', None)

	ts_start = ts_or_timespec(ts_start)
	ts_end   = ts_or_timespec(ts_end)

	media_dir = os.path.join(get_configuration().MEDIA_WEB_ROOT, '_stats')

	if get_configuration().SANET_STATS_VERSION == 1:
		if chart == 'queue': 
			file_name, w, h = sanetd_create_rrd_graph_file_elements(media_dir, 'sanetd_queue.png', SANETD_STAT_DS_ELEMENTS, SANETD_COLORS, ltype='LINE', stacked=False, title="Data Queue"   , ts_start=ts_start, ts_end=ts_end)
		else:
			raise Exception("Invalid chart type '%s'" % (chart))
	else:

		if chart == 'queue': 
			selected_stats =[]

			for sid in SANETD_STAT_DS_ELEMENTS + ['processqueuesize'] :
				stat_id = "sanetd.%s.%s" % (instanceid, sid)
				selected_stats.append(stat_id)

			colors = SANETD_COLORS

			file_name, w, h = sysstats_create_rrd_graph_file(
				media_dir, 
				'sanetd_queue.png', 
				selected_stats, 
				colors, 
				ltype    = 'LINE', 
				stacked  = False, 
				title    = "Queues", 
				ts_start = ts_start, 
				ts_end   = ts_end
			)

		elif chart == 'eventread': 

				stat_id = "sanetd.%s.%s" % (instanceid, 'queueread')

				selected_stats =[
					stat_id
				]

				output_filename = 'sanetd_%s.png' % (stat_id)

				colors = {} 

				file_name, w, h = sysstats_create_rrd_graph_file(
					media_dir, 
					output_filename, 
					selected_stats, 
					colors, 
					title            = "Received data (event/s)", 
					ts_start         = ts_start, 
					ts_end           = ts_end, 
					selected_ds_type = 'rate'
				)

		elif chart == 'processrate': 

				stat_id = "sanetd.%s.%s" % (instanceid, 'processqueuetot')

				selected_stats =[
					stat_id
				]

				output_filename = 'sanetd_%s.png' % (stat_id)

				colors = {} 

				legend = {}
				legend[stat_id] = "Processed"

				file_name, w, h = sysstats_create_rrd_graph_file(
					media_dir, 
					output_filename, 
					selected_stats, 
					colors, 
					ltype            = 'LINE', 
					stacked          = False, 
					title            = "Processing rate (chunck/s)", 
					ts_start         = ts_start, 
					ts_end           = ts_end, 
					selected_ds_type = 'rate',
					legend           = legend
				)

		elif chart == 'processtimes': 

				colors = {
					'min': '00ff00',
					'max': 'ff0000',
					'avg': '0000ff',
				} 

				selected_stats = [ "sanetd.%s.process.%s" % (instanceid, sid) for sid in colors.keys() ]

				output_filename = 'sanetd_%s.png' % (chart)

				file_name, w, h = sysstats_create_rrd_graph_file(
					media_dir, 
					output_filename, 
					selected_stats, 
					colors, 
					ltype            = 'LINE', 
					stacked          = False, 
					title            = "Execution times", 
					ts_start         = ts_start, 
					ts_end           = ts_end, 
					#selected_ds_type = 'rate',
					legend           = colors
				)

		else:
			raise Exception("Invalid chart type '%s'" % (chart))

	with open(file_name, "rb") as f:
		response = HttpResponse(f.read(), mimetype="image/png")
		response['Content-Length'] = os.path.getsize(file_name)
		response['Cache-Control'] = 'no-cache'
		response['Pragma']        = 'no-cache'
		return response

