# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import json

from django.http import HttpResponse

from core.api.base import db_set_current_tenant_schema
from core.models import MonitoredElement, Condition

from tags.models import TAG_FILTER

from webui.decorators import tenant_dependent

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

@tenant_dependent
def select_all_by_type(request, rtype):

	user   = request.user
	tenant = request.tenant

	db_set_current_tenant_schema(tenant.name)
		
	if rtype == 'e':
		elements = MonitoredElement.objects.select_related('parent').order_by('parent__name', 'name')
		
		elements = TAG_FILTER(tenant, user, elements, 'monitoredelement')
		
		elements = elements.all() 
		
		elements = sorted(elements, key=lambda x: x.path )  # FIX: non e' un sort molto veloce perche' ".path" e' un attributo "calcolato". Bisognerebbe ottimizzare questi sorting con un campo di comodo nel DB

	elif rtype == 'c':
		elements = Condition.active_objects

		elements = TAG_FILTER(tenant, user, elements, 'condition')

		elements = elements.all().order_by('path')
		
	r = [ {'uuid':e.uuid, 'name':e.path} for e in elements ]
		
	return HttpResponse( json.dumps(r) )
