# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import time

import json

from django.http import HttpResponse

from sanet_common.sanetdefs.constants import *

from core.models import DataGroup, DataSource, DataGroupParameter
from core.models import MonitoredNode
from core.models import DataSource

from core.api.base import db_set_current_tenant_schema
from core.api.base import db_set_current_tenant_schema

from core.api.cache      import init_cache_manager_from_conf

from webui.decorators import tenant_dependent

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

@tenant_dependent
def get_stats(request):

	return _get_stats(request)


def _get_stats(request):
        
        db_set_current_tenant_schema(request.tenant.name)
	nodes = request.GET_OR_POST()['nodes']
	
	cache_manager = init_cache_manager_from_conf()
	
	nodes = nodes.split(',')
	
	
	CPU_CLASS       = 'system.cpu'
	CPU_USED_CLASS  = 'system.cpu.load'
	
	DISK_CLASS      = 'system.storage'
	DISK_USED_CLASS = 'system.storage.used'
	DISK_TOT_CLASS  = 'system.storage.installed'

	RAM_CLASS       = 'system.ram'
	RAM_USED_CLASS  = 'system.ram.used'
	RAM_TOT_CLASS   = 'system.ram.installed'
	
	"""
	
	dg_classiciations = [CPU_CLASS, RAM_CLASS, DISK_CLASS]
	classiciations = [CPU_CLASS, DISK_USED_CLASS, DISK_TOT_CLASS, RAM_USED_CLASS, RAM_TOT_CLASS]
	
	datasources = DataSource.objects.filter(datagroup__element__name__in=nodes).filter(classification__in=classiciations).all()

	ds_by_node = {}
	p_by_element = {}
	
	for ds in datasources:
		
		node_id = ds.datagroup.element.parent.uuid if ds.datagroup.element.parent != None else ds.datagroup.element.uuid
		
		d = ds_by_node.get(node_id, { 
			'cpu' : {
			},
			'ram' : {
			},
			'disks': {
			},
		})
		
		if ds.classification == CPU_USED_CLASS:
			d['cpu'][CPU_CLASS] = ds
			
		elif ds.classification in [RAM_USED_CLASS, RAM_TOT_CLASS]:
			d['ram'][CPU_CLASS] = ds
			
		if ds.classification in [DISK_USED_CLASS, DISK_TOT_CLASS]:
			disk_id = ds.datagroup.element.uuid
			
			d2 = d['disks'].get(disk_id, {})
			ds[ds.classification] = ds
			d['disks'][disk_id] = d2
			
	thresholds  = DataGroupParameter.objects.filter(datagroup__element__name__in=nodes).filter(datagroup__classification__in=dg_classiciations).all()         
	for t in thresholds:
		p_by_element[ "%s_%s" % (ds.datagroup.element.uuid, ds.datagroup.classification) ] = t.get_value()
	"""	
	
	ret = []
	for node in MonitoredNode.objects.filter(name__in=nodes).all():
	
		t1 = time.time()
		
		ram_threshold  = None
		cpu_threshold  = None

		"""
		
		cpu_used      = ds_by_node.get(node.uuid,{}).get(CPU_USED_CLASS   , None)
		cpu_threshold = p_by_element.get( "%s_%s" % (node.uuid, CPU_CLASS) , None)
		
		ram_used      = ds_by_node.get(node.uuid,{}).get(RAM_USED_CLASS, None)
		ram_tot       = ds_by_node.get(node.uuid,{}).get(RAM_TOT_CLASS , None)
		ram_threshold = p_by_element.get( "%s_%s" % (node.uuid, RAM_CLASS), None)

		"""
		
		cpu_used = DataSource.objects.filter(datagroup__element=node).filter(classification=CPU_USED_CLASS)[0:1]
		cpu_used = cpu_used[0] if cpu_used else None
		
		try:
			if cpu_used:
				cpu_threshold = cpu_used.datagroup.datagroupparameter_set.get(name='threshold').value
		except DataGroupParameter.DoesNotExist, e:
			pass
		
		ram_used = DataSource.objects.filter(datagroup__element=node).filter(classification=RAM_USED_CLASS)[0:1]
		ram_used = ram_used[0] if ram_used else None

		ram_tot  = DataSource.objects.filter(datagroup__element=node).filter(classification=RAM_TOT_CLASS)[0:1]
		ram_tot  = ram_tot[0] if ram_tot else None

		try:
			if ram_used:
				ram_threshold = ram_used.datagroup.datagroupparameter_set.get(name='threshold').value
		except DataGroupParameter.DoesNotExist, e:
			pass


		
		if cpu_used : cpu_used.load_fields_from_cache(cache_manager)

		if ram_used : ram_used.load_fields_from_cache(cache_manager)
		if ram_tot  : ram_tot.load_fields_from_cache(cache_manager)
		

		d = {
			'name': node.name,
			'uuid': node.uuid, 
			'cpu' : {
				 'used'     : cpu_used.lastvalue  if cpu_used else None
				,'threshold': cpu_threshold,
			},
			'ram' : {
				 'used'     : ram_used.lastvalue  if ram_used else None
				,'tot'      : ram_tot.lastvalue   if ram_used else None
				,'threshold': ram_threshold,
			},
			'disks': [] ,
			
		}







		for s in node.get_sub_elements(ELEMENT_KIND_STORAGE):
			
			t1_s = time.time()

			disk_threshold = None

			"""
			data = ds_by_node.get(node.uuid, {}).get('disks',{}).get(s.uuid,{})
			disk_used = data.get(DISK_USED_CLASS, None)
			disk_tot  = data.get(DISK_TOT_CLASS , None)

			"""
			
			disk_used = DataSource.objects.filter(datagroup__element=s).filter(classification=DISK_USED_CLASS)[0:1]
			disk_used = disk_used[0] if disk_used else None
			
			disk_tot  = DataSource.objects.filter(datagroup__element=s).filter(classification=DISK_TOT_CLASS)[0:1]
			disk_tot  = disk_tot[0] if disk_tot else None

			try:
				if disk_tot:
					disk_threshold  = disk_tot.datagroup.datagroupparameter_set.get(name='threshold ').value
			except DataGroupParameter.DoesNotExist, e:
				pass



			if disk_used: disk_used.load_fields_from_cache(cache_manager)
			if disk_tot : disk_tot.load_fields_from_cache(cache_manager)
			
			
			t2_s = time.time()
		

			d['disks'].append({
				'uuid'     : s.uuid,
				'name'     : s.name,
				'used'     : disk_used.lastvalue if disk_used else None,
				'tot'      : disk_tot.lastvalue if disk_tot  else None,
				'threshold': disk_threshold,
				'_t': "%.4f" % (t2_s - t1_s), 
			})

		t2 = time.time()
		d['_t'] = "%.4f" % (t2 - t1), 
		ret.append(d)

	return HttpResponse( json.dumps( ret ) )



