# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

import time
import datetime
import pprint
import urllib
import csv
import StringIO

try:
	import json
except: 
	from django.utils import simplejson as json

from django.http import HttpResponse
from django.http import HttpResponseBadRequest

from sanet_common.sanetdefs.constants import *

from core.models import MonitoredStorage, DataGroup, DataSource, Condition, DataGroupParameter
from core.api.base import db_set_current_tenant_schema
from core.api.cache      import init_cache_manager_from_conf

from core.api.datasource import init_datasource_store_from_conf
from core.api.datasource import DataSourceStoreException

from webui.decorators import tenant_dependent

from tags.models import Tag

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

def timestamp_to_isostr(timestamp):
	try:
		return datetime.datetime.fromtimestamp(timestamp).strftime('%Y-%m-%d %H:%M:%S')
	except TypeError:
		return ''

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

@tenant_dependent
def get_trends(request):
	
	return _get_trends(request)


def get_last_value_storage(request):

	return _get_last_value_storage(request)


def _get_last_value_storage(request):
	
	"""
		Funzione di estrazione dei dati sugli storage per i report. Prende i parametri in GET o in POST dalla richiesta.
		
		:param tagid: tag contenente gli storage
		:type tagid: id or path
		:param ts_end: timestamp finale della finestra temporale
		:param ts_start: timestamp iniziale della finestra temporale
		:rtype: csv
	"""
	
	tenant = request.tenant
	db_set_current_tenant_schema(tenant.name)


	dg_classification = 'system.storage.occupation'
	dg_param          = 'usage_threshold'
	ds_used_class     = 'system.storage.used'
	ds_tot_class      = 'system.storage.installed'
	c_class           = 'system.storage.usage'
	

	tag_path_or_uuid      = request.GET_OR_POST().get('tagid', None)
	if tag_path_or_uuid != None:
		tag_path_or_uuid      = urllib.unquote(tag_path_or_uuid)
	else:
		return HttpResponseBadRequest("Id o path del tag (paramentro tagid) non trovato, controllare il proprio datasource")

	ts_end   = request.GET_OR_POST().get('ts_end', None)
	if ts_end == None:
		return HttpResponseBadRequest("Parametro ts_end non trovato, controllare il proprio datasource")
	else:
		ts_end = int(ts_end)
	
	ts_start = request.GET_OR_POST().get('ts_start', None)
	if ts_start == None:
		return HttpResponseBadRequest("Paramentro ts_start non trovato, controllare il proprio datasource")
	else:
		ts_start = int(ts_start)

	ret = []
	
	datasource_storage = init_datasource_store_from_conf()
	cache_manager      = init_cache_manager_from_conf()
	
	if tag_path_or_uuid:
		try:
			tag = Tag.objects.get(uuid=tag_path_or_uuid)
		except Tag.DoesNotExist, e:
			try:
				tag = Tag.objects.get(path=tag_path_or_uuid)
			except Tag.DoesNotExist, e:
				return HttpResponseBadRequest("Tag non trovato, controllare il parametro tagid nella configurazione del proprio datasource")
		
		all_storages = tag.get_all_storages()
		
	all_datagroups = DataGroup.active_objects.filter(tenant=tenant, element__in=all_storages, classification=dg_classification).all()
	
	
	datagroups_by_s = {}
	for d in all_datagroups:
		l = datagroups_by_s.setdefault(d.element_id,[])
		l.append(d)

	all_used_ds = DataSource.active_objects.filter(tenant=tenant, datagroup__in=all_datagroups, classification=ds_used_class)
	used_by_dg  = dict( [ (ds.datagroup_id, ds) for ds in all_used_ds ] )

	all_tot_ds  = DataSource.active_objects.filter(tenant=tenant, datagroup__in=all_datagroups, classification=ds_tot_class)
	tot_by_dg   = dict( [ (ds.datagroup_id, ds) for ds in all_tot_ds  ] )

	all_c       = Condition.active_objects.filter(tenant=tenant, datagroup__in=all_datagroups, classification=c_class)
	c_by_dg     = dict( [ (c.datagroup_id, c)   for c  in all_c       ] )

	all_p       = DataGroupParameter.objects.filter(tenant=tenant, datagroup__in=all_datagroups, name=dg_param)
	p_by_dg     = dict( [ (p.datagroup_id, p)   for p in all_p ] )

	for storage in all_storages:

		node = storage.get_node()
		
		print node.name, storage.name

		datagroups  = datagroups_by_s.get( storage.uuid , [] ) 

		for dg in datagroups:

			used_value = None
			tot_value  = None

			param   = p_by_dg.get( dg.uuid , None)
			ds_used = used_by_dg.get( dg.uuid, None)
			ds_tot  = tot_by_dg.get(dg.uuid, None)
			c       = c_by_dg.get(dg.uuid, None)
			
			
			if not (param != None and ds_used != None and ds_tot != None and c != None):
				continue


			# only few data for dashboard visaulization
			if c.status not in (CSTATE_UP, CSTATE_FA, CSTATE_DN):
				continue


			threshold = float(param.get_value())
			

			try:
				print tenant.uuid, ds_used.uuid, ts_start, ts_end
				used_serie = datasource_storage.fetch_values(tenant.uuid, ds_used.uuid, ts_start=ts_start, ts_end=ts_end)
				used_values = used_serie['dataseries']
				
				tot_serie = datasource_storage.fetch_values(tenant.uuid, ds_tot.uuid, ts_start=ts_start, ts_end=ts_end)
				tot_values = tot_serie['dataseries']

				for i in xrange(len(used_values)):
					if used_values[-i] is not None:
						used_value = used_values[-i]
				for i in xrange(len(tot_values)):
					if tot_values[-i] is not None:
						tot_value  = tot_values[-i]

				if used_value == None and tot_value == None:
					continue

			except DataSourceStoreException, e:
				continue

			#            0          1             2           3          4
			ret.append( [node.name, storage.name, used_value, threshold, tot_value] )

	output = StringIO.StringIO()
	
	if len(ret) > 0:

		writer = csv.writer(output)
		writer.writerows(ret)

	return HttpResponse( output.getvalue() )

def _get_trends(request):
	
	tenant = request.tenant
	db_set_current_tenant_schema(tenant.name)

	
	delta    = float(request.GET_OR_POST().get('delta', 10))

	order_by = request.GET_OR_POST().get('order_by', 'status_used_perc')
	limit    = request.GET_OR_POST().get('limit', 10)

	tag_uuid      = request.GET_OR_POST().get('tagid', None)
	if tag_uuid != None:
		tag_uuid      = urllib.unquote(tag_uuid)

	resolution = request.GET_OR_POST().get('resolution', None)


	#
	# Range temporale
	#
	h        = request.GET_OR_POST().get('h', None)
	m        = request.GET_OR_POST().get('m', None)
	if h != None:
		ts_end   = time.time()
		ts_start = ts_end - (60*60*int(h))
	elif m != None:
		ts_end   = time.time()
		ts_start = ts_end - (60*int(m))
	else:
		ts_start = request.GET_OR_POST().get('s', None)
		ts_end   = request.GET_OR_POST().get('e', None)

	if ts_end is None:
		ts_end   = time.time()
	ts_end = int(ts_end)
	
	if ts_start is None:
		ts_start = ts_end - 60*60*24
	ts_start = int(ts_start)
	
	ret = self._calc_trends(ts_start, ts_end, tag_uuid, tenant, delta, resolution)
	
	if len(ret) > 0:

		# order by
		if   order_by == 'status_used_perc'   :   ret = sorted(ret, key=lambda ret: (ret[1], -ret[2]))
		elif order_by == 'free'               :   ret = sorted(ret, key=lambda ret: -ret[4])
		elif order_by == 'status_free_space'  :   ret = sorted(ret, key=lambda ret: (ret[1], ret[4]))
		elif order_by == 'node_name'          :   ret = sorted(ret, key=lambda ret: ret[0])
		elif order_by == 'status_used_space'  :   ret = sorted(ret, key=lambda ret: (ret[1], -ret[3]))

		ret = ret[0:int(limit)]

		t4 = time.time()

		ret[0][5]['_tottime'] = t4 - t0
	
	if request.REQUEST.get('fullformat', None):
		return HttpResponse( pprint.pformat( ret ) )

	return HttpResponse( json.dumps( ret ) )
	

def _calc_trends(ts_start, ts_end, tag_uuid, tenant, delta = 10, resolution = None):
	
	dg_classification = 'system.storage.occupation'
	dg_param          = 'usage_threshold'
	ds_used_class     = 'system.storage.used'
	ds_tot_class      = 'system.storage.installed'
	c_class           = 'system.storage.usage'
	
	#print ("Searching for: ",tag_uuid)

	t0 = time.time()
	
	ret = []
	
	datasource_storage = init_datasource_store_from_conf()
	cache_manager      = init_cache_manager_from_conf()
	
	if tag_uuid:
		try:
			tag = Tag.objects.get(uuid=tag_uuid)
		except Tag.DoesNotExist, e:
			tag = Tag.objects.get(path=tag_uuid)

		all_storages = tag.get_all_storages()
	else:
		all_storages = MonitoredStorage.objects.filter(tenant=tenant).select_related('element', 'element__parent').all()

	# XXXXXXXXXXXXXX HACK SPEEDUP XXXXXXXXXXXXX
	all_datagroups = DataGroup.active_objects.filter(tenant=tenant, element__in=all_storages, classification=dg_classification).all()

	datagroups_by_s = {}
	for d in all_datagroups:
		l = datagroups_by_s.get( d.element_id, datagroups_by_s.setdefault(d.element_id,[])  )
		l.append(d)

	all_used_ds = DataSource.active_objects.filter(tenant=tenant, datagroup__in=all_datagroups, classification=ds_used_class)
	used_by_dg  = dict( [ (ds.datagroup_id, ds) for ds in all_used_ds ] )

	all_tot_ds  = DataSource.active_objects.filter(tenant=tenant, datagroup__in=all_datagroups, classification=ds_tot_class)
	tot_by_dg   = dict( [ (ds.datagroup_id, ds) for ds in all_tot_ds  ] )

	all_c       = Condition.active_objects.filter(tenant=tenant, datagroup__in=all_datagroups, classification=c_class)
	c_by_dg     = dict( [ (c.datagroup_id, c)   for c  in all_c       ] )

	all_p       = DataGroupParameter.objects.filter(tenant=tenant, datagroup__in=all_datagroups, name=dg_param)
	p_by_dg     = dict( [ (p.datagroup_id, p)   for p in all_p ] )
	# XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

	
	for storage in all_storages:

		node = storage.get_node()

		#datagroups = DataGroup.objects.filter(tenant=tenant, element=storage, classification=dg_classification).all()

		# XXXXXXXXXXX HACK SPEEDUP XXXXXXXXXXXXXXX
		datagroups  = datagroups_by_s.get( storage.uuid , [] ) 
		# XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

		for dg in datagroups:

			t1 = time.time()

			#param   = dg.datagroupparameter_set.get(tenant=tenant, datagroup=dg, name=dg_param)
			#ds_used = dg.datasources.get(tenant=tenant, classification=ds_used_class)
			#ds_tot  = dg.datasources.get(tenant=tenant, classification=ds_tot_class)
			#c       = dg.conditions.get (tenant=tenant, classification=c_class)

			# XXXXXXXXXXX HACK SPEEDUP XXXXXXXXXXXXXX
			param   = p_by_dg.get( dg.uuid , None)
			ds_used = used_by_dg.get( dg.uuid, None)
			ds_tot  = tot_by_dg.get(dg.uuid, None)
			c       = c_by_dg.get(dg.uuid, None)

			if not (param != None and ds_used != None and ds_tot != None and c != None):
				continue
			# XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


			# only few data for dashboard visaulization
			if c.status not in (CSTATE_UP, CSTATE_FA, CSTATE_DN):
				continue


			threshold = float(param.get_value())
			
			d = {
				'node_name'   : node.name,
				'node_uuid'   : node.uuid,
				'storage'     : storage.name,
				'storage_uuid': storage.uuid,
				'title'       : c.expanded_title(cache_manager),
				'threshold'   : threshold,
				'condition'   : { 'status': c.status },
				'used'        : [],
				'tot'         : [],
			}

			try:
				used_serie = datasource_storage.fetch_values(tenant.uuid, ds_used.uuid, ts_start=ts_start, ts_end=ts_end, resolution=resolution)			
				used_values = used_serie['dataseries']
				
				tot_serie = datasource_storage.fetch_values(tenant.uuid, ds_tot.uuid, ts_start=ts_start, ts_end=ts_end, resolution=resolution)
				tot_values = tot_serie['dataseries']

				d['used_id'] = ds_used.uuid
				d['used']    = used_values
				d['tot' ]    = tot_values

				trend_values, M,B = calculate_serie_trend_values(used_serie)
				d['trend'] = trend_values
                                #d['trend_factor'] = "%.6f" % (M * 100)
				d['trend_factor'] = M

				d['trend_next_alarm_ts']   = ''
				d['trend_full_ts']         = ''

				d['trend_params'] = (M, B)
				
				d['last_used'] = None
				d['last_tot']  = None
				for i in xrange(len(used_values)):
					if used_values[-i] is not None:
						d['last_used'] = used_values[-i]
				for i in xrange(len(tot_values)):
					if tot_values[-i] is not None:
						d['last_tot']  = tot_values[-i]

				if d['last_used'] and d['last_tot']:
					if ((threshold /  100.0) - (d['last_used'] / d['last_tot'])) <= (delta/100.0):
						d['flag'] = 1


					if ( (d['last_tot'] * (threshold/100.0)) <= d['last_used']):
						d['trend_next_alarm_ts'] = 'passed'

					try:
						today = time.time()
						if M > 0:
							ts = int( ((d['last_tot'] * (threshold/100.0)) - B) / M  )
							if (today < ts):
								d['trend_next_alarm_ts']  =  timestamp_to_isostr(ts)
							


							ts = int( (d['last_tot'] - B) / M  )
							if (today < ts):
								d['trend_full_ts']        =  timestamp_to_isostr(ts)
					except Exception, e:
						pass


			except DataSourceStoreException, e:
				d['error'] = str(e)
				continue
                    	try:
				d['used_perc'] = round(d['last_used']*100/d['last_tot'],2)

			except (TypeError, ZeroDivisionError, KeyError), e:
				# error in last used data
				d['used_perc'] = 0
			try:
				d['free'] = d['last_tot']-d['last_used']
			except (TypeError, KeyError):
				d['free'] = 0

			t2 = time.time()

			d['_t'] = (t2-t1)

			#            0               1                         2               3               4          5
			ret.append( (d['node_name'], d['condition']['status'], d['used_perc'], d['last_used'], d['free'], d) )

	return ret

import math


def calculate_serie_trend_values(ds):
	data = ds['dataseries']
	ts   = ds['scalex'][0] # (ts start, ts end, step)
	#ts   = 0
	step = ds['scalex'][2] # (ts start, ts end, step)

	values = []
	ts1 = ts

	for v in data:
		if v is not None:
			dd = (ts1, v)
			values.append( dd  )   # x, y
		ts1 += step
		
	if len(values) != 0:
		try:
			M,B = calculate_trend_line_params(values)
		except ZeroDivisionError:
			M,B = 0.0,0.0
	else:
		M,B = 0.0,0.0
	
	values = []
		
	return values, M, B

def calculate_trend_line_params( values ):
	
	# Equazione per il trend
	# http://classroom.synonym.com/calculate-trendline-2709.html
	
	# y = M x + B
	
	n = float(len(values))

	"""
	x_  =  sum([x for x,y in values]) / n
	y_  =  sum([y for x,y in values]) / n

	
	var_x  = sum([ (x - x_)**2 for x,y in values  ]) / n
	cov_xy = sum([ ((x - x_)*(y - y_)) for x,y in values ]) / n

	a = cov_xy / var_x
	b = y_ - x_ #(* values[0][0])

	M = a
	B = b

	return M, B
	XXXXXXXXXXXXXXXXXXXXXXXXX
	"""

	"""
	x_values_sum = float(sum([x for x,y in values])) / n
	y_values_sum = float(sum([y for x,y in values])) / n
	
	# M = (a-b) / (c-d)
	a = sum([(x*y) for x,y in values])
	b = (x_values_sum * y_values_sum ) * n
	
	c = sum([(x**2) for x,y in values]) 
	d = (x_values_sum ** 2) * n
	
	M = (a - b) / (c - d)
	
	# B = (e-f)
	e = y_values_sum
	f = x_values_sum * M

	# Y = M * x + B
	#B = (e - f) / n
	B = e - f

	return M, B
	"""

	
	a = b = c = d = e = f = x_sum = y_sum = 0.0

	for x,y in values:
		x = float( x / (10.0 ** 9))
		y = float( y / (10.0 ** 9))

		x_sum += x
		y_sum += y
		a     += (x * y)
		c     += (x**2)
		
	a = float(a * n)
	b = float(x_sum * y_sum)
	c = float(c * n)
	d = (x_sum ** 2)
	
	M = (a - b) / (c - d)

	e = y_sum 
	f = x_sum * M
	
	B = (e - f) / n

	return M, B*(10**9)







