# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from django.conf.urls import url

import webui.views.objects.users
import webui.views.objects.groups
import webui.views.objects.tenants
import webui.views.objects.agents
import webui.views.objects.monitored
import webui.views.objects.datasources
import webui.views.objects.conditions
import webui.views.objects.monitored
import webui.views.objects.servers
import webui.views.objects.storages
import webui.views.objects.classifications
import webui.views.objects.selectors
import webui.views.objects.logs
import webui.views.objects.clusters
import webui.views.objects.dirutils
import webui.views.objects.timegraphs
import webui.views.objects.configure
import webui.views.objects.nodes

import webui.views.objects.sanetd
import webui.views.svgutils

import webui.views.icons

#----------------------------------------------------------------------------------#

urlpatterns = [ #patterns('webui.views.objects',

	# Groups
	url(r'^user/delete$'								, webui.views.objects.users.delete			, name='user_delete'),
	url(r'^user/create$'								, webui.views.objects.users.create			, name='user_create'),
	url(r'^user/update/(?P<id>.+)$$'						, webui.views.objects.users.update			, name='user_update'),

	url(r'^group/create$'								, webui.views.objects.groups.create			, name='group_create'),
	url(r'^group/delete$'								, webui.views.objects.groups.delete			, name='group_delete'),

	url(r'^group/toggle_user_association$'						, webui.views.objects.groups.toggle_user_association	, name='group_toggle_user_association'),
	url(r'^group/toggle_tenant_association$'					, webui.views.objects.groups.toggle_tenant_association	, name='group_toggle_tenant_association'),
	url(r'^group/toggle_capability$'						, webui.views.objects.groups.toggle_capability		, name='group_toggle_capability'),


	# Tenant
	url(r'^tenant/(?P<uuid>\w+)/update_settings$'					, webui.views.objects.tenants.update_settings		, name='tenant_update_settings'),
	url(r'^tenant/(?P<uuid>\w+)/update_icon$'					, webui.views.objects.tenants.update_icon		, name='tenant_update_icon'),

	url(r'^tenant/(?P<uuid>\w+)/add_icon$'						, webui.views.objects.tenants.add_icon			, name='tenant_add_icon'),
	url(r'^tenant/(?P<uuid>\w+)/remove_icon/(?P<icon_name>.+)$'			, webui.views.objects.tenants.remove_icon		, name='tenant_remove_icon'),

	url(r'^tenant/(?P<uuid>\w+)/tenant_remove_priority_level/(?P<label>.+)$'	, webui.views.objects.tenants.remove_priority_level	, name='tenant_remove_priority_level'),


	url(r'^tenant/(?P<uuid>\w+)/add_priority_level/'				, webui.views.objects.tenants.add_priority_level	, name='tenant_add_priority_level'),


	url(r'^tenant/(?P<uuid>\w+)/add_agent$'						, webui.views.objects.agents.add_new			, name='tenant_add_agent'),
	url(r'^tenant/(?P<uuid>\w+)/remove_agent/(?P<auuid>\w+)$'			, webui.views.objects.agents.remove			, name='tenant_remove_agent'),
	url(r'^tenant/(?P<uuid>\w+)/edit_agent/(?P<auuid>\w+)$'				, webui.views.objects.agents.edit			, name='tenant_edit_agent'),

	url(r'^agents/(?P<tenantid>\w+)/(?P<uuid>\w+)/get_stats$'			, webui.views.objects.agents.get_stats			, name='webui.views.objects.agents.get_stats'),

	url(r'^agents/(?P<agent_uuid>\w+)/stat_chart/(?P<chart>\w+)$'			, webui.views.objects.agents.stat_chart         	, name='agent_stat_chart'),

	url(r'^agents/get_stats_entables$'						, webui.views.objects.agents.get_stats_entables		, name='webui.views.objects.agents.get_stats_entables'),

	#url(r'^system/status/stat_chart/(?P<chart>\w+)$'				, webui.views.objects.sanetd.sanetd_stat_chart		, name='sanetd_stat_chart'),
	url(r'^system/status/stat_chart/(?P<instanceid>\w+)/(?P<chart>\w+)$'		, webui.views.objects.sanetd.sanetd_stat_chart 		, name='sanetd_stat_chart'),

	url(r'^elementpage_by_path/$'							, webui.views.objects.monitored.page_by_path		, name='webui.views.objects.monitored.page_by_path'),


	#
	# Datasources
	#
	url(r'^datasources/(?P<uuid>\w+)/fetch_data$',					webui.views.objects.datasources.fetch_data                   , name='webui.views.objects.datasources.fetch_data'      ),
	#url(r'^datasources/(?P<uuid>\w+)/nfetch_data$',					webui.views.objects.datasources.new_fetch_data               , name='webui.views.objects.datasources.new_fetch_data'  ),      USATO DA CHI? OBSOLETO?
	url(r'^datasources/(?P<uuid>\w+)/graph_page$',		                        webui.views.objects.datasources.graph_page                   , name='webui.views.objects.datasources.graph_page'      ),

	#
	# Conditions
	#
	url(r'^conditions/get_details/$',						webui.views.objects.conditions.get_details                   , name='webui.views.objects.conditions.get_details' ),  #used by alarmslist
	url(r'^conditions/(?P<uuid>\w+)/get_changes$'    , 				webui.views.objects.conditions.get_changes                   , name='webui.views.objects.conditions.get_changes' ),
	url(r'^conditions/(?P<uuid>\w+)/new_get_changes$', 				webui.views.objects.conditions.new_get_changes               , name='webui.views.objects.conditions.new_get_changes' ),
	url(r'^conditions/(?P<uuid>\w+)/get_changes_raw$', 				webui.views.objects.conditions.get_changes_raw               , name='webui.views.objects.conditions.get_changes_raw' ),
	url(r'^conditions/signal_check_now$',						webui.views.objects.conditions.signal_check_now              , name='signal_check_condition_now' ),

	#
	# Nodes
	#
	url(r'^nodes/csv_list$', 							webui.views.objects.monitored.export_list_csv                , name='nodes_export_list_csv'),

	#
	# Others
	#
	url(r'^servers/get_stats$', 							webui.views.objects.servers.get_stats                        , name='webui.views.objects.servers.get_stats'),
	url(r'^storages/get_trends$', 							webui.views.objects.storages.get_trends                      , name='webui.views.objects.storages.get_trends'),

	url(r'^classifications/get_used$', 						webui.views.objects.classifications.get_used                 , name='webui.views.objects.classifications.get_used'),
	url(r'^classifications/get_all$', 						webui.views.objects.classifications.get_all                  , name='webui.views.objects.classifications.get_all'),
	url(r'^selectors/all/(?P<rtype>\w+)$', 						webui.views.objects.selectors.select_all_by_type             , name='webui.views.objects.selectors.select_all_by_type'),

	url(r'^log/details$',								webui.views.objects.logs.get_details                         , name='webui.views.objects.logs.get_details'),
	url(r'^log/entables_details$',							webui.views.objects.logs.get_entables_details                , name='webui.views.objects.logs.get_entables_details'),
	url(r'^log/export_csv$',							webui.views.objects.logs.export_csv                          , name='webui.views.objects.logs.export_csv'),

	url(r'^cluster/select_all$',							webui.views.objects.clusters.select_all                      , name='webui.views.objects.clusters.select_all'),
	url(r'^cluster/sub_elements$',							webui.views.objects.clusters.sub_elements                    , name='webui.views.objects.clusters.sub_elements'),

	url(r'^dir$', 									webui.views.objects.dirutils.sub_elements                    , name='webui.views.objects.dirutils.sub_elements'),
	url(r'^tags$', 									webui.views.objects.dirutils.sub_tags                        , name='webui.views.objects.dirutils.sub_tags'),
	url(r'^tag/addmap$', 								webui.views.objects.dirutils.addmap                          , name='webui.views.objects.dirutils.addmap'),

	url(r'^get_data$', 								webui.views.objects.monitored.get_data                       , name='webui.views.objects.monitored.get_data'),

	url(r'^timegraph/(?P<uuid>\w+)/image',                  			webui.views.objects.timegraphs.timegraph_image               , name='webui.views.objects.timegraphs.timegraph_image'),

	# dashboard url			
	url(r'^get_tree_list_order$',							webui.views.objects.monitored.get_tree_list_order            , name='webui.views.objects.monitored.get_tree_list_order'),

	url(r'^config_element_fields/(?P<uuid>.+)$',					webui.views.objects.configure.monitoredelement_config_fields , name='webui.views.objects.configure.monitoredelement_config_fields'),

	#url per gestire l'albero dei tag			
	url(r'^rename_tag_or_element$', 						webui.views.objects.dirutils.rename_tag_or_element           , name='webui.views.objects.dirutils.rename_tag_or_element'),
	url(r'^delete_tag$', 								webui.views.objects.dirutils.delete_tag                      , name='webui.views.objects.dirutils.delete_tag'),
	url(r'^untag_tree_element$', 							webui.views.objects.dirutils.untag_tree_element              , name='webui.views.objects.dirutils.untag_tree_element'),

	#url(r'^all_data$', 								'monitored.get_all_data'),
	#url(r'^get_tree_list$',							'monitored.get_tree_list'),  


	# Url per gestire la sezione per la ricerca ed esportazione dei nodi
	url(r'^nodes/data$',								webui.views.objects.nodes.tenant_nodes_data                  , name='tenant_nodes_data'),
	url(r'^nodes/export_csv$',							webui.views.objects.nodes.export_csv                         , name='tenant_nodes_export_csv'),
	url(r'^nodes/conf_save$',							webui.views.objects.nodes.save_configuration                 , name='tenant_save_configuration'),
	url(r'^nodes/table_data$',							webui.views.objects.nodes.get_table_data                     , name='tenant_table_data'),
] #)

