# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

from django import forms
from django.forms import Form, ModelForm
from django.core.validators import RegexValidator
from django.http import HttpResponse
from django.contrib.auth.models import User

from sanet_common.sanetdefs.constants import VALID_USER_NAME

from core.compatibility import reverse

from core.api.base import user_create, user_update
from core.api.base import user_delete

from webui.api import *
from webui.api.utils import render_template

from webui.decorators import user_is_admin

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

@user_is_admin
def delete(request):

	user_id  = int(request.GET_OR_POST()['userid'] )
	user  = User.objects.get(id=user_id)

	user_delete(user.username)

	return HttpResponse("ok")

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class UserForm(Form):
	username     = forms.CharField   (label='Username'        , required=True, validators=[ RegexValidator(VALID_USER_NAME) ])
	first_name   = forms.CharField   (label='First name'      , required=False)
	last_name    = forms.CharField   (label='Last name'       , required=False)
	email        = forms.CharField   (label='Email'           , required=False)
	password     = forms.CharField   (label='Password'        , required=True)
	password2    = forms.CharField   (label='Confirm password', required=True)

	is_superuser = forms.BooleanField(label='Superuser'       , required=False, initial=False)
	is_active    = forms.BooleanField(label='Enabled'         , required=False, initial=True)

@user_is_admin
def create(request):

	if request.method == 'POST':
		form = UserForm(request.POST) # A form bound to the POST data
		if form.is_valid(): 

			username = form.cleaned_data['username']

			fields = dict([ (fname, form.cleaned_data[fname]) for fname in ['first_name','last_name','email','password', 'is_superuser', 'is_active' ] ] )

			u = user_create(username, fields=fields)

			return HttpResponse('OK')
	else:
		form = UserForm(initial={}) 

	html = render_template('html/generic_field_form.html', { 
		'action': reverse('user_create'),
		'form': form, 
		'button_label': 'Confirm',
	})

	return HttpResponse(html)

#----------------------------------------------------------------------#
#                                                                      #
#----------------------------------------------------------------------#

class UpdateUserForm(ModelForm):
	class Meta:
		model = User

		fields = [
			'first_name',
			'last_name',
			'email',
			'is_superuser',
			'is_active',
		]

@user_is_admin
def update(request, id):

	user = User.objects.get(id=id)

	if request.method == 'POST':
		form = UpdateUserForm(request.POST, instance=user) # A form bound to the POST data
		if form.is_valid(): 

			for fname in ['first_name','last_name', 'email', 'is_superuser', 'is_active' ]:
				v = form.cleaned_data[fname]
				if fname == 'password':
					user.set_password(v)
				else:
					setattr(user, fname, v)

			user_update(user)

			return HttpResponse('OK')
	else:
		form = UpdateUserForm(instance=user) 

	html = render_template('html/generic_field_form.html', { 
		'action': reverse('user_update', kwargs={'id':user.id}),
		'form': form, 
		'button_label': 'Confirm',
	})

	return HttpResponse(html)

