# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

#from django.db import transaction

from django.contrib.auth.decorators import login_required

from core.db_utils import transaction_commit_on_success

from core.api.base import db_set_current_tenant_schema

from tags.models import TagTree, Tag

from webui.models import UserTagFilters

from webui.api.response import HttpJsonSuccessResponse

#-------------------------------------------------------------------------------#
#                                                                               #
#-------------------------------------------------------------------------------#

@login_required
def alarms_list_get_filter_tags(request):

	user = request.user
	tenant = request.tenant

	db_set_current_tenant_schema(tenant.name)

	filters_by_treeid = dict([ (f.tagtree_id, f) for f in UserTagFilters.objects.filter(tenant=tenant, user=user).all() ])

	trees = TagTree.get_visible_trees_by_user(tenant, user)

	data = []
	for t in trees:
		r = t.root
		if r is None:
			continue

		if filters_by_treeid.has_key(t.uuid):
			continue

		name  = r.name
		owner = t.owner.username if t.owner is not None else None

		data.append( { 'uuid': t.uuid, 'tree': t.name, 'path': name , 'owner': owner, 'filtered':  False} )

	return HttpJsonSuccessResponse('Filters', data=data)

@login_required
@transaction_commit_on_success
def alarms_list_add_tag_filter(request):

	user = request.user
	user.username
	tenant = request.tenant

	db_set_current_tenant_schema(tenant.name)
	uuid = request.GET_OR_POST()['uuid'];

	data = []
	for uuid in uuid.split("|"):
		
		tagtree = TagTree.objects.get(uuid=uuid)

		f, created = UserTagFilters.objects.get_or_create(tenant=tenant, user=user, tagtree=tagtree)

		name = tagtree.root.name
		owner = tagtree.owner.username if tagtree.owner is not None else None
		data.append( { 'uuid': tagtree.uuid, 'tree': tagtree.name, 'path': tagtree.root.path , 'owner': owner, 'created': created } )

	return HttpJsonSuccessResponse("Added", data=data)

@login_required
@transaction_commit_on_success
def alarms_list_remove_tag_filter(request):

	user = request.user
	tenant = request.tenant

	db_set_current_tenant_schema(tenant.name)

	uuid = request.GET_OR_POST()['uuid'];

	tagtree = TagTree.objects.get(uuid=uuid)

	f = UserTagFilters.objects.get(user=user, tagtree=tagtree)
	f.delete()
	
	return HttpJsonSuccessResponse("Removed", data={ 'uuid': uuid } )


