# Copyright (C) 2018 Laboratori Guglielmo Marconi S.p.A. <http://www.labs.it>
#
# This file is part of SANET
# SANET is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, version 3 of the License
#
# SANET is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with SANET. If not, see <http://www.gnu.org/licenses/>.

"""

################################################################################
Database
################################################################################

Introduzione
================================================================================

Database e Django
--------------------

TODO

Principi di progettazione
================================================================================

Core
--------------------------------------------------------------------

TODO

Gestione delle chiavi primarie e degli ID
--------------------------------------------------------------------

TODO

.. _sanet.core.models.schemas:

Gestione degli schemi nel DB: tenant e schemi
-----------------------------------------------

TODO

Gestione degli aggiornamenti 
================================================================================

TODO


Schema ER
================================================================================

.. image:: ../_static/ER/sanet3_schema_ER.png
	:width: 800











Element Template
============================================================================================

Gli TElement sono i template per gli elementi di monitoraggio.

::
  
                        +----------+        0,N                    0,N     +------------------+
                        | TElement |------------<monitoredelement>---------| MonitoredElement |
                        +----------+                                       +------------------+
                             |
       +---------+-----------+-----------+-----------+
       |         |           |           |           |
   +------+  +-------+  +---------+ +---------+ +--------+
   | Node |  | Iface |  | Storage | | Service | | Device |
   +------+  +-------+  +---------+ +---------+ +--------+
     |
     +- is_switch
     +- is_router
     +- priority


Gestione sotto-elementi nei template.
-------------------------------------

Vincoli applicativi:

    1) I sotto-elementi possono essere definiti solo nei node-template.
    2) Non possono esistere due sotto-elementi dello stesso tipo con lo stesso nome.

Gli TElement possono essere di due tipi:

::
    
    1) Normali
    
        CONFIGURAZIONE                                               NAME
        
        node-template server-base                                    server-base
            ...
        exit
        
        interface-template iface-base                                iface-base
            ... 
            non posso specificare xform e/o distinguisher 
            ...
        exit

    2) Subtemplate

        CONFIGURAZIONE                                               NAME
        
        node-template server-base                                    server-base
            
            interface eth0 [ distinguisher ... [ xform ... ] ]       server-base:eth0
                template iface-base 
                template iface-stp                                       
                template iface-...
            exit
            
        exit


Schema ER Completo
-------------------

::
    
                    +----------+                 0,N                  0,N            +------------------+
                    | TElement |--------------------[elementtemplate]----------------| MonitoredElement |
                    +----------+                            |                        +------------------+
                         |                                  |                                 |
           +-------------+-------------+                    |                                 |
           |                           |                    |                                 |
    +------------+               +------------+             |                                 | 0,N
    |   Normal   |               | SubElement |----------------------------------------------< >
    +------------+               +------------+             |                             0,1
          | |                         | |                   |
          | |      1,1        0,N     | |                   |
          | +---------<parent>--------+ |                   |
          |                             |                   |
          +------------<use>------------+                   |
                   0,N   |  0,N                             |
                         +----------------------------------+
                         
"""

"""
TODO:
- fix unique  nelle tabelle tenant-dependent (l'unicita' e' data' anche sulla colonna tenant)
- tune db_index
- tune default_value
- add verbose_name
"""
import os
import time
import json
import sys
import datetime

from string import Template as StringTemplate

from sanet_common.sanetdefs.constants    import *
from sanet_common.sanetdefs.constants    import AUTH_READONLY, AUTH_READWRITE
from sanet_common.uuids                  import MAX_UUID_LEN, create_global_uuid
from sanet_common.netutils               import resolve_ipv4
from sanet_common.exprparser.utils       import parseString
from sanet_common.exprparser.ExprFactory import ParserException
from sanet_common.formatutils            import number_to_str
from sanet_common.sanetdefs.ipc          import AGENT_NETINFOHELPER

from sanet_libsnmp                       import snmp_get, snmp_walk, SnmpWrapperException

from sanet_poller.lib.snmp.base          import parse_snmpv3_auth_options

# FIX: this method should be "public"? (remove "_")
from sanet_poller.lib.snmp.base          import _get_snmpv3_auth_options_from_string  

from sanet_poller.agentctl               import agent_sendreceive
from sanet_poller.helpers.netinfo        import NetInfoAgentHelper # removed from OSE

from django import db
from django.db import connection
from django.db import models
#from django.db import transaction

#from django.db.models import GenericIPAddressField as  IPAddressField
from core.compatibility import IPAddressField

from django.db.models.query import QuerySet
from django.db.models import Q
from django.db.models.signals import post_save
from django.db.models.signals import post_delete, post_save
from django.db.models.signals import pre_delete
from core.compatibility import reverse
from django.core.validators import MinValueValidator
from django.dispatch import Signal
from django.dispatch import receiver

from django.contrib.auth.models import User, Group

from conf import settings
from conf import get_configuration
from conf import get_broker_connection_string

from core.db_utils import transaction_commit_on_success

from core.exceptions import ApplicationException, InvalidConfigurationException
from core.exceptions import ConcurrentUpdateException

from core.statecolor import get_state_color
from core.statecolor import get_outcome_color

from core.cachemodels import AbstractCachedFieldModel

from core.api.cache import init_cache_manager_from_conf
from core.api.datasource import init_datasource_store_from_conf
from core.api.timegraph import validate_timegraphserie_expr


#-----------------------------------------------------------------------------#
# DEBUG                                                                       #
#-----------------------------------------------------------------------------#

DEBUG = get_configuration().DEBUG_CONFIG or os.environ.get('DEBUG_CONFIG', False)

def _DEBUG(*args):
	if DEBUG: print "\t" * args[0], ' '.join([str(a) for a in args[1:]])


#-----------------------------------------------------------------------------#
# CONSTANTS                                                                   #
#-----------------------------------------------------------------------------#

DEFAULT_SNMP_VERSION = settings.DEFAULT_SNMP_VERSION

_DEFAULT_MONITOREDELEMENT_ICONS = {
	ELEMENT_KIND_NODE   : 'node',
	ELEMENT_KIND_IFACE  : 'iface',
	ELEMENT_KIND_STORAGE: 'storage',
	ELEMENT_KIND_SERVICE: 'service',
	ELEMENT_KIND_DEVICE : 'device',
}

#-----------------------------------------------------------------------------#
# UTILITIES                                                                   #
#-----------------------------------------------------------------------------#

def _dictfetchall(cursor):
	"""
	Restituisce il risultato di un cursore sotto forma di dizionario
	utilizzando le informazioni della "SELECT" per dare nomi ai campi
	"""
	desc = cursor.description
	r = []
	for row in cursor.fetchall():
		r.append(  dict(zip([col[0] for col in desc], row))  )
	return r


import socket
def ASSERT_VALID_IPv4(ipstr):
	try:
		socket.inet_aton(ipstr)
	except Exception, e:
		#raise ValueError("Invalid IP string %s" % (ipstr))
		raise InvalidConfigurationException("Invalid IP string %s" % (ipstr))

#-----------------------------------------------------------------------------#
# CUSTOM SIGNALS                                                              #
#-----------------------------------------------------------------------------#

#
# Questo segnale viene lanciato dopo che il core ha effettuato aggiornamenti
# dello schema.
#
sanet_post_update_schema  = Signal(providing_args=['schema_version'])

#
# Questo segnale viene lanciato dal comando "sanetdata" per inizializzare
# tutte le subapp.
#
sanet_init_subapp    = Signal()

#
# Questo segnale viene lanciato dal comando "sanetdata" per fare in modo
# che ogni subapp controllo la consistena dei dati per un singolo tenant.
#
sanet_check_tenant   = Signal(providing_args=['instance'])

#
# Questo segnale viene lanciato dal comando "sanetdata" per fare in modo
# che ogni subapp cancelli dati spuri/obsoleti per ogni tenant.
#
sanet_cleanup_tenant = Signal(providing_args=['instance'])

#
# Questo segnale viene lanciato dal comando "sanetdata" al termine
# della procedura di update dell'applicazione nel caso qualche subapp
# abbia bisogno di fare controlli finali.
#
sanet_post_check     = Signal()


#
# Questo segnale viene lanciato dal comando "sanetdata" al termine
# della procedura di update dell'applicazione nel caso qualche subapp
# abbia bisogno di fare controlli finali.
#
sanet_tenant_create        = Signal(providing_args=['instance'])

sanet_tenant_create_vardir = Signal(providing_args=['tenant_id'])


#
# Questo evento viene sollegato quando viene eseguito l'UPDATE di un modello
# senza utilizzare il metodo .save() di django.
#
sanet_model_update = Signal(providing_args=["instance", "updated_fields"])


#
# Segnale di alto livello triggerato quando un utente effettua una 
# azione su una entita':
#
#   datagroup-template
#
#   node-template
#   interface-template
#   storage-template
#   service-template
#   device-template
#
#   node
#   interface
#   storage
#   service
#   device
#
#   tagtree
#   tag
#
#   note
#
sanet_config_user_action = Signal(providing_args=["user", "tenant", "entity_type", "entity_id", "msg"])

#
# Segnale lanciato da un modello per verificare se l'<user> ha il permesso <auth_type>
# sull'oggetto <instance>.
#
# Chi si registra a questo segnale deve restituire una tupla:
#
# 	(<priorita>, <result>)
#
#sanet_check_permission = Signal(providing_args=["instance", "user", "auth_type"])


#
# Segnale lanciato quando vengono processati dei "additional_updates" da Sanetd, ovvero
# dati addizionali che non riguardano il "core" di Sanet.
#
# Le "sub-app" possono registrarsi per intercettare questo segnale e vedere se
# tra i "additional_updates" ci sono dati che devono processare:
#
# I dati contentuti in "additional_updates" sono tuple:
#
#	( <dtype>, <ddata> )
#
# I parametri del segnale sono:
#
#	log                  Oggetto "logger"
#	tenant               Tenant associato al segnale
#	agentid              UUID dell'agente che ha inviato i dati
#	additional_updates   Additional updates
#
sanetd_process_event_additional_updates = Signal(providing_args=["log", "tenant", "agentid", "additional_updates"])

#
# Segnale lanciato dopo che tutti i dati di update di un datagroup sono stati processati
# ed e' necessario "informare" i diversi sotto-sistemi di sanet che una condition
# ha subito delle variazioni di stato.
#
sanetd_process_changed_conditions = Signal(providing_args=["log", "tenant", "agentid", "changed_conditions"])

#
# Segnale lanciato quando devono essere espanse le wildcard {{}} nei messaggi UP e DOWN.
# Le diverse subapp possono registrarsi ed aggiugnere wildcard in base alle informazioni
# contenute nel segnale
#
# 	log               Oggetto "logger"
#	tenant            Tenant associato al segnale
#	condition         Condition legata all'allarme
#	node              Nodo associato all'allarme
#	element           Elemento direttamente associato alla condition
#	template_symbols  Dizionario contenente tutte le wildcard registrate (da aggiornare)
#
sanetd_alarm_msg_create_template_symbols = Signal(providing_args=["log", "tenant", "node", "element", "condition", "template_symbols"])

#
# Segnale inviato dal demone statusd quando riceve via IPC (rete) dei comandi
# da agenti remoti che devono essere gestiti da sub-app.
#
statusd_handle_request = Signal(providing_args=["server", "command", "params"])


#-----------------------------------------------------------------------------#
# ABSTRACT CLASSES                                                            #
#-----------------------------------------------------------------------------#

class CTimeModel(models.Model):
	"""
	Classe astratta per gestire tutti i modelli che hanno bisogno di memorizzare
	un timestamp di creazione.
	"""
	ctime = models.IntegerField(null=False , default=time.time, db_index=True)

	class Meta:
		abstract = True

#-----------------------------------------------------------------------------#

class MTimeModel(models.Model):
	"""
	Classe astratta per gestire tutti i modelli che hanno bisogno di memorizzare
	un timestamp di modifica.
	
	Il campo mtime codifica (dovrebbe codificare) il tempo in UTC.
	
	Questa classe specifica il metodo di utility concurrent_save() per poter effettuare
	un aggiornamento concorrente usando il campo "mtime" come campo sentinella.
	
	Tutti gli oggetti dei modelli che possono subire aggiornamenti concorrenti, dovrebbero
	estendere questa classe.
	"""
	mtime    = models.IntegerField(null=False, default=time.time, db_index=True)

	def save(self, *args, **kwargs):
		#
		# BUG: c'e' uno strano problema di perdita di precisione quando carichiamo/salviamo i dati
		#      dal DB. Questo crea problemi in fase di aggiornamenti usando il campo 'mtime'
		#      come "sentinella".
		#      Per il momento arrotondiamo in fase di salvataggio. 
		#      
		#
		self.mtime = int(self.mtime)

		return super(MTimeModel, self).save(*args, **kwargs)

	def concurrent_save(self, fields_name=None, new_sentinel_value=None):
		"""
		Effettua un aggiornamento sul DB garantendo che non ci siano problemi di concorrenza.
		
		Per garantire aggiornamenti concorrenti, il campo "mtime" viene usato come campo sentinella.
		Se il valore del campo "mtime" non corrisponde al valore nel DB l'aggiornamento fallice e
		viene sollevata una eccezione di tipo ConcurrentUpdateException.
		
		Se "new_sentinel_value" e' specificato, il valore di mtime viene aggiornato a "new_sentinel_value"
		durante l'update su DB. Se "new_sentinel_value" e' None o 0, il valore di mtime viene aggiornato
		con il current time.time().
		
		Se la lista "fields_name" non e' vuota, vengono salvati tutti i campi specificati nella lista.
		Se la lista "fields_name" e' vuota o None, vengono salvati tutti i campi dell'oggetto.
		
		"""

		if new_sentinel_value is None: new_sentinel_value = time.time()

		# Questo e' il valore che ci aspettiamo nel DB (il valore sentinella)
		current_sentinel = self.mtime

		# Questo e' il nuovo valore che deve essere salvato nel DB
		self.mtime = new_sentinel_value

		return self._save_fields_with_sentinel(fields_name=fields_name, sentinel_field='mtime', sentinel_value=current_sentinel)

	def _save_fields_with_sentinel(self, fields_name=None, sentinel_field=None, sentinel_value=None):
		"""
		Metodo a basso livello per aggiornare l'oggetto.
		"""

		klass   = self.__class__

		pk_name = self._meta.pk.name
		pk_val  = getattr(self, pk_name)

		# Salviamo tutti i campi se non specificato diversamente
		# (il campo corrispondente alla chiave primaria non lo consideriamo perche' e' inutile)
		if not fields_name:
			fields_name = [ f.name for f in klass._meta.fields if f.name != pk_name ]

		# Il campo sentinella va sempre salvato
		if sentinel_field:
			if sentinel_field not in fields_name:
				# IMPORTANTE: Potrebbe arrivare in input una tupla e non una lista
				fields_name = list(fields_name)
				fields_name.append(sentinel_field)

		# Aggiorniamo tutti i campi dell'oggetto in memoria
		fields_update = {}
		for fname in fields_name:
			fields_update[fname] = getattr(self, fname)

		# Creiamo la query di update.
		#
		#   UPDATE  <object> SET  <field> = <field value> , ..... WHERE  objects.pk = <pk_val> AND  <sentinel_field> = <sentinel_value>;
		#
		query = klass.objects.filter(pk=pk_val)
		if sentinel_value is not None:
			sentinel_filter = {}
			sentinel_filter[sentinel_field] = sentinel_value
			query = query.filter(**sentinel_filter)

		# Aggiorniamo
		updated = query.update(**fields_update)
		if updated != 1:
			# L'aggiornamento non ha funzionato:
			#  - l'oggetto nel DB e' stato rimosso
			#  - l'oggetto sul DB e' stato aggiornato il suo valore sentinella e' cambiato
			#  - abbiamo modificato piu' di un oggetto. QUESTO E' VIRTUALMENTE IMPOSSIBILE.
			raise ConcurrentUpdateException("Concurrent update on %s(pkval=%s) [update fields=%s] [sentinel=%s]" % (klass.__name__, pk_val, ','.join(fields_update.keys()), (sentinel_field, sentinel_value)))

		if updated:
			#
			# Nofifichiamo che e' stato effettuato l'update del modello
			#

			sanet_model_update.send(sender=self.__class__, instance=self, updated_fields=fields_name)

		return updated

	class Meta:
		abstract = True

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class UUIDKeyModel(models.Model):
	"""
	Tutte le entita' del modello che hanno bisogno di gestire una chiave
	primaria UUID ereditano da questa classe astratta
	"""
	uuid = models.CharField(primary_key=True, max_length=MAX_UUID_LEN, default=create_global_uuid, db_index=True)

	class Meta:
		abstract = True

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class ModelFieldsUtils(object):
	"""
	Classe astratta con metodi di comodo per ottenere dinamicamente
	informazioni sui campi dei modelli, i loro valori di default, ecc.
	"""

	# I metodi di questa classe ignorano i seguenti campi nei modelli.
	SKIP_SPECIAL_FIELDS = ['uuid','mtime','ctime']

	# Campi non "resettabili" con operazioni di "refresh" (stati/timestamp)
	# Ogni classe che eredita da questa dovrebbe definire i suoi particolari "uneditable fields"
	UNEDITABLE_FIELDS = []  

	#
	# Metodi statici
	#
	@staticmethod
	def get_model_fields_default_value_dict(model_class):
		d = {}
		for f in model_class._meta.fields:
			if f.name in ModelFieldsUtils.SKIP_SPECIAL_FIELDS: continue
			if f.has_default():
				d[f.name] = f.default
		return d

	@staticmethod
	def get_model_blank_value_fields(model_class):
		l = []
		for f in model_class._meta.fields:
			if f.name in ModelFieldsUtils.SKIP_SPECIAL_FIELDS: continue
			if f.blank:
				l.append(f.name)
		return l

	#
	# Metodi di oggetto
	#
	def get_blank_fields(self):
		"""
		Returns the fields which allow empty string as value
		"""
		return ModelFieldsUtils.get_model_blank_value_fields(self.__class__)

	def get_fields_default_value_dict(self):
		"""
		Returns the default for every field of the model which extends this class.
		"""
		return ModelFieldsUtils.get_model_fields_default_value_dict(self.__class__)

	def reset_fields_defaults(self, l=0, only_fields_names=None):
		"""
		Imposta al valore di default tutti i campi dell'oggetto. 
		
		Se la lista di strighe "only_fields_names" non e' vuota o nulla, vengono resettati solo
		i campi specificati nella lista.
		
		I campi definiti dalla costante di classe UNEDITABLE_FIELDS non vengono mai resettati.
		"""
		d = self.get_fields_default_value_dict()

		_DEBUG(l, "Fields' default: %s" % (d))

		for f_name, val in d.items():

			if f_name in self.UNEDITABLE_FIELDS:
				continue

			if only_fields_names and f_name not in only_fields_names:
				continue

			old = getattr(self, f_name)
			if old != val:
				setattr(self, f_name, val)
				_DEBUG(l+1, "Reset %s to: %s" % (f_name, val))

	#
	# FIX: questo metodo e' pressoche' inutile. E' stato creato e usato nell'interfaccia web ( webui/objects/nodes.py )
	#      ma all'interno di una logica applicativa che va rivista. E' probabile che questo metodo andara' a morire.
	#
	@staticmethod
	def get_fields_verbose_names(model_class):
		field_names = []
		for field in model_class._meta.fields:
			field_names.append(field.verbose_name)
		return field_names

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class ExpandedTitleModel(models.Model):
	"""
	Metodi condivisi per gestire l'expanded title
	"""

	def expire_title(self, cache_manager=None):
		cache_manager = cache_manager or self.get_cache_manager()
		key = cache_manager.create_key( 'cache', 'expanded_title', self.tenant_id, self.uuid, )
		cache_manager.delete(key)

	def expanded_title(self, cache_manager=None):

		cache_manager = cache_manager or self.get_cache_manager()
		if cache_manager:
			key = cache_manager.create_key( 'cache', 'expanded_title', self.tenant_id, self.uuid, )
			data = cache_manager.get(key)
			if data != None:
				mtime, title = data
				if mtime == self.mtime:
					return title

			title = self.update_expanded_title(cache_manager=cache_manager, cache_key=key)
		else:
			title = self.title

		return title

	def update_expanded_title(self, cache_manager=None, cache_key=None):

		cache_manager = cache_manager or self.get_cache_manager()
		cache_key     = cache_key     or cache_manager.create_key( 'cache', 'expanded_title', self.tenant_id, self.uuid, )

		title = self._subst_title(self.title)

		if cache_manager:
			data = (self.mtime, title)
			cache_manager.set(cache_key, data)

		return title

	def _subst_title(self, title_str):
		symbols = self.element.get_specific_element().get_title_symbols_dict()
		return ExpandedTitleModel.subst_title(title_str, symbols)

	@staticmethod
	def subst_title(title_str, symbols):
		
		formated = dict([ (s, number_to_str(v)) for s,v in symbols.items() ])

		t = StringTemplate(title_str).safe_substitute(**formated)
		for s,v in formated.items():
			if s.startswith('{'):
				t = t.replace(s, str(v))
		return str(t)

	class Meta:
		abstract = True

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class OwnerUserModel(models.Model):
	"""
	Tutte le entita' del modello che hanno bisogno della gestione dell'utente OWNER
	ereditano da questa classe astratta.
	"""
	owner = models.ForeignKey(User, null=True, default=None, db_index=True, on_delete=models.SET_NULL)


	OWNER_TABLES = [
		'scheduling_group',
		'cluster',
		'monitoredelement',
		'datagroup',
		'datasource',
		'condition',
		'timegraph',
		'timegraphserie',
		'c_telement',
		'c_tdatagroup',
	]

	class Meta:
		abstract = True


@receiver(pre_delete, sender=User)
def _delete_owner(sender, **kwargs):
	"""
	Quando viene cancellato un utente, il field "owner" di alcune tabelle dovrebbe essere azzerato,
	ma la gestione in "schemi" impedisce alle API di django di modificare le tabelle in tutti
	gli schemi associati ai tenant.
	
	Dobbiamo gestire la cosa manualmente prima di cancellare effettivamente l'utente.
	"""

	cursor = connection.cursor()

	uid = kwargs['instance'].id

	for tenant in Tenant.objects.all():

		Tenant.db_set_current_tenant_schema(tenant.name, db_cursor=cursor)

		for table in OwnerUserModel.OWNER_TABLES:
			query = 'UPDATE %s set owner_id = null WHERE owner_id = %s' % (table, uid)
			#print query
			cursor.execute(query)


class PermissionModel(models.Model):
	"""
	La gestione delle autorizzazioni su alcune entita' del modello e' dinamica.
	
	Per supportare la verifica delle autorizzazioni l'oggetto del modello deve implementare il metodo _verify_permission() 
	
	Questo metodo puo' essere definito anche da sub_app esterne al "core" e aggiunto dinamicamente attraverso
	il moetodo add_to_class().
	
	"""

	def verify_permission(self, user, auth_type):
		if hasattr(self, '_verify_permission'):
			return self._verify_permission(user, auth_type)
		else:
			return True

	def can_write(self, user):
		return self.verify_permission(user, AUTH_READWRITE)

	def can_read(self, user):
		return self.verify_permission(user, AUTH_READONLY)

	def assert_permission(self, user, auth_type):
		v = self.verify_permission(user, auth_type)
		if not v:
			raise ApplicationException("User '%s' has no permission %s to modify element '%s'." % (user, auth_type, self))
		return True

	def assert_permission_read(self, user):
		return self.assert_permission(user, AUTH_READONLY)

	def assert_permission_readwrite(self, user):
		return self.assert_permission(user, AUTH_READWRITE)

	class Meta:
		abstract = True

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class CachedUtil(object):
	"""
	Classe astratta che definisce metodi per accedere al cache_manager usato da Sanet.
	"""
	def set_cache_manager(self, manager):
		self.cache_manager = manager

	def get_cache_manager(self):
		if hasattr(self, 'cache_manager'):
			return self.cache_manager
		else:
			return None

#-----------------------------------------------------------------------------#
#  INTERNAL CONFIGURATION                                                     #
#-----------------------------------------------------------------------------#

class DBSetting(models.Model):
	"""
	Tabella con i parametri di configurazione salvati sul database.
	
	Teoricamente tutti i parametri applicativi di sanet dovrebbero stare nel DB
	e non nel file di configrauzione di DJANGO (default: "conf/settings.py").
	
	I parametri sul DB sono salvati tramite la tupla
	
		<nome>  <valore complesso>
	
	Nel DB il <valore complesso> viene codificato in formato JSON.
	"""

	SANET_SCHEMA_VERSION = 'version'

	name  = models.TextField(primary_key=True, null=False, blank=False, db_index=True , )

	value = models.TextField(                  null=False, blank=False, db_index=False, ) # json format

	def get_value(self):
		try:
			return json.loads(self.value)
		except ValueError, e:
			# FIX: this catch is for retro-compatibility. Remove as soon as possible
			return self.value

	def set_value(self, v):
		self.value = json.dumps(v)

	# Utility methods

	@staticmethod
	def get_version():
		return int(DBSetting.objects.get(name=DBSetting.SANET_SCHEMA_VERSION).get_value())

	@staticmethod
	def set_version(v):
		setting = DBSetting.objects.get(name=DBSetting.SANET_SCHEMA_VERSION)
		setting.set_value(v)
		setting.save()

	class Meta:
		db_table = "dbsetting"

#-----------------------------------------------------------------------------#
#  GRUPPI                                                                     #
#-----------------------------------------------------------------------------#

class SANETGroupProfile(MTimeModel):
	"""
	Informazioni di Sanet associate ad ogni gruppo". 
	
	Viene creata una istanza per ogni gruppo che viene creato applicativamente (vedi trigger *_on_group_save_create_profile*).
	"""

	group = models.ForeignKey(Group, unique=True)

	description = models.TextField(null=False, blank=True, default='', db_index=False)

	class Meta:
		db_table = "sanetgroupprofile"


def _on_group_save_create_profile(sender, **kwargs):

	group   = kwargs['instance']
	created = kwargs['created']

	if not created:
		return

	gp = SANETGroupProfile(group=group)
	gp.save()

post_save.connect(_on_group_save_create_profile, sender=Group)

#-----------------------------------------------------------------------------#
#  USER                                                                       #
#-----------------------------------------------------------------------------#

class SANETUserProfile(MTimeModel):
	"""
	Informazioni di Sanet associate ad ogni utente". 
	
	Viene creata una istanza per ogni utente che viene creato applicativamente (vedi trigger *_on_user_save_create_profile*).
	"""

	# This is the only required field
	user           = models.ForeignKey(User, unique=True)

	def get_visible_tenants(self):
		return Tenant.get_visible_by_user(self.user)

	def can_view_tenant(self, tenant):

		if self.user.is_superuser:
			return True

		user = self.user
		return TenantGroup.objects.filter(group__in=user.groups.all(), tenant=tenant).count() > 0

	def get_setting(self, key, tenant=None):

		try:
			cs = self.user.customsettings.get(tenant=tenant, key=key)
			return cs.get_value()
		except UserCustomSetting.DoesNotExist, e:
			return None

	class Meta:
		db_table = "sanetuserprofile"

#-----------------------------------------------------------------------------#

def _on_user_save_create_profile(sender, **kwargs):

	user    = kwargs['instance']
	created = kwargs['created']

	if not created:
		return

	up = SANETUserProfile(user=user)
	up.save()

post_save.connect(_on_user_save_create_profile, sender=User)


#
# COMPATIBILITY: Django >=1.8 removed the get_profile() method from User class.
#
def _user_get_profile(self):
	#
	# Fix: salvare in una cache questo valore.
	#
	return SANETUserProfile.objects.get(user__id__exact=self.id)

User.add_to_class('get_profile', _user_get_profile)


#-----------------------------------------------------------------------------#
#  TENANT                                                                     #
#-----------------------------------------------------------------------------#

TENANT_DEFAULT_MIN_PRIORITY = 0
TENANT_DEFAULT_MAX_PRIORITY = 10

class Tenant(UUIDKeyModel, MTimeModel):
	
	# uuid
	# mtime
	
	name        = models.TextField(null=False, blank=False, unique=True)
	long_name   = models.TextField(blank=False)

	timezone    = models.TextField(null=False, blank=False, default=DEFAULT_TIMEZONE)  

	# Default tenant
	is_primary  = models.BooleanField(null=False, default=False)

	# linked groups
	groups      = models.ManyToManyField(Group, through='TenantGroup') 

	@property
	def tenant(self):
		return self

	@property
	def path(self):
		return self.name

	@property
	def visible_name(self):
		return self.long_name or self.name

	def __str__(self):
		return "<Tenant:%s:%s>" % (self.uuid, self.name)

	def get_logs(self):
		#
		# FIX: skip "deleted=True" conditions
		#
		return Event.objects.filter(tenant=self)

	def get_all_elements(self, kind=None):
		q = MonitoredElement.objects
		if kind is not None:
			q = q.filter(kind=kind)
		return q

	def get_all_datagroups(self):
		return DataGroup.active_objects.filter(tenant=self)

	def get_all_datasources(self):
		return DataSource.active_objects.filter(tenant=self)

	def get_all_conditions(self):
		return Condition.active_objects.filter(tenant=self)

	def get_all_timegraphs(self):
		return TimeGraph.active_objects.filter(tenant=self)

	def get_all_timegraphseries(self):
		return TimeGraphSerie.active_objects.filter(tenant=self)

	def get_counters(self):
		c = connection.cursor()
		c.execute('select kind, count(*) from monitoredelement group by kind')
		counters = c.fetchall()
		return dict(counters)

	def get_setting(self, name):
		return TenantSetting.get_setting(self, name)

	def get_icon_url(self):
		icon_name = 'tenant'
		icon_url = reverse('webui.views.icons.get_colored_icon3', kwargs = {'icon': icon_name, 'tenantid': self.uuid})
		return icon_url

	@staticmethod
	def get_visible_by_user(user):

		tenants = Tenant.objects
		
		if user is not None and not user.is_superuser:
			uuids = [ tg[0] for tg in TenantGroup.objects.filter(group__in=user.groups.all()).values_list('tenant_id') ]
			tenants = tenants.filter(uuid__in=uuids)

		return tenants

	@staticmethod
	def tenant_name_to_schema(tenant_name):
		#
		# FIX: convert tenant_name to a valid postgres schema name
		#
		pg_schema_name = tenant_name

		return pg_schema_name

	@staticmethod
	def db_set_current_tenant_schema(tenant_name, include_public_schema=True, db_cursor=None):

		schema_name = Tenant.tenant_name_to_schema(tenant_name)

		if db_cursor == None:
			db_cursor = connection.cursor()

		if include_public_schema:
			q = 'SET search_path TO  %s,public;' % (schema_name)
		else:
			q = 'SET search_path TO %s;' % (schema_name)

		db_cursor.execute(q)

	def db_set_current_schema(self, include_public_schema=True, db_cursor=None):
		return Tenant.db_set_current_tenant_schema(self.name, include_public_schema=include_public_schema, db_cursor=db_cursor)

	@staticmethod
	def get_db_current_schema(db_cursor=None):

		if db_cursor == None:
			db_cursor = connection.cursor()

		db_cursor.execute('show search_path')
		data = db_cursor.fetchall()
		return ','.join(data[0])

	class Meta:
		db_table = "tenant"

class TenantDependentModel(models.Model):
	"""
	Tutte le entita' che devono essere gestire i dati dei tenant in maniera
	disgiunta devono ereditare da questa tabella.
	"""

	tenant = models.ForeignKey(Tenant, db_index=True,  related_name="%(app_label)s_%(class)s_related" )

	class Meta:
		abstract = True

class TenantRelatedModel(models.Model):
	"""
	Tutte le entita' che devono avere un normale riferimento al tenant
	ereditano da questa tabella.
	"""
	
	tenant = models.ForeignKey(Tenant, null=True, db_index=True, related_name="%(app_label)s_%(class)s_related" )
	
	class Meta:
		abstract = True

class TenantSetting(TenantRelatedModel):
	"""
	Ogni tenant ha dei parametri di default che devono essere modificabili via web e quindi
	devono essere memorizzati su database.

	Il tipo di ogni parametro non e' noto a priori. Salviamo i dati in json sul DB
	cosi' viene salvato anche il tipo.
	"""

	SETTING_CRITICAL_LEVEL      = 'critical_level'

	SETTING_MINIMUM_LEVEL       = 'minimum_level'
	
	SETTINGS = [
		SETTING_CRITICAL_LEVEL,
		SETTING_MINIMUM_LEVEL,
	]
	
	DEFAULTS = {
		SETTING_CRITICAL_LEVEL: PRIORITY_MAX + 1,
		SETTING_MINIMUM_LEVEL : PRIORITY_NONE,
	}

	# tenant
	name        = models.TextField(null=False, blank=False)
	value       = models.TextField(null=True , blank=True, default=None)

	@staticmethod
	def get_setting(tenant, name):
		try:
			return json.loads(TenantSetting.objects.get(tenant=tenant, name=name).value)
		except TenantSetting.DoesNotExist, e:
			#return None
			return TenantSetting.DEFAULTS.get(name, None)
			

	@staticmethod
	def save_setting(tenant, name, value):
		s, created = TenantSetting.objects.get_or_create(tenant=tenant, name=name)
		s.value = json.dumps(value)
		s.save()

	@staticmethod
	def get_all_settings(tenant):

		r = dict([(setting_name, None) for setting_name in TenantSetting.SETTINGS])

		settings = TenantSetting.objects.filter(tenant=tenant).all()
		for s in settings:
			try:
				r[s.name] = json.loads(s.value)
			except ValueError, e:
				pass
		return r

	class Meta:
		db_table = "tenantsetting"

		unique_together = ("tenant", "name")

#-----------------------------------------------------------------------------#
#  GROUP                                                                      #
#-----------------------------------------------------------------------------#

# Tabella relazione N:N  tra Tenant e Gruppo
class TenantGroup(MTimeModel):

	group  = models.ForeignKey(Group)
	tenant = models.ForeignKey(Tenant)

	# Default priority level for the group
	default_min_priority = models.IntegerField(default=PRIORITY_NONE, db_index=True, )

	class Meta:
		unique_together = ("group", "tenant")
		db_table = "tenant_group"


#-----------------------------------------------------------------------------#
#  Dati utenti per tenant                                                     #
#-----------------------------------------------------------------------------#

class TenantUserSettings(MTimeModel):

	user   = models.ForeignKey(User)
	tenant = models.ForeignKey(Tenant)

	# Default priority level for the group
	default_min_priority = models.IntegerField(default=PRIORITY_NONE, db_index=True, )

	is_default = models.BooleanField(null=False, default=False)

	@property
	def is_configured(self):
	
		if self.default_min_priority != PRIORITY_NONE:
			return True

		if self.is_default != False:
			return True

		return False

	class Meta:
		unique_together = ("user", "tenant")
		db_table = "tenant_user"

def get_user_default_tenant(user):
	"""
	Restituisce il tenant di default dell'utente.
	
	Controlla in ordine:
		1) Se esiste un tenant configurato di default, restituisce quello.
		2) Trova tutti i tenant visibili dai gruppi a cui appartiene l'utente. 
		     Se tra questi c'e' il tenant primario, restituisce quello.
		     Se non c'e' nessun primario, restituisce il primo tenant in ordine alfabetico.
		3) Se l'utente e' admin, comunque visualizza il tenant primary.
	"""

	# E' stato configurato un tenant di default?
	try:
		return TenantUserSettings.objects.get(user=user, is_default=True).tenant
	except TenantUserSettings.DoesNotExist, e:
		pass

	# Dei tenant visibili prendiamo (1) il primaorio o (2) il primo in ordine alfabetico
	visible_tenants = TenantGroup.objects.filter(group__in=user.groups.all()).order_by('-tenant__is_primary', 'tenant__name').all()
	if len(visible_tenants) > 0:
		return visible_tenants[0].tenant

	# Fallback: i superuser, in assenza di configurazioni, vedono sicuramente il tenant primary
	if user.is_superuser:
		return Tenant.objects.get(is_primary=True)

	return None

class UserCustomSetting(models.Model):
	
	BLANK_VALUE = ''

	user   = models.ForeignKey(User, related_name="customsettings")
	tenant = models.ForeignKey(Tenant, null=True)

	key    = models.TextField(null=False, blank=False)
	value  = models.TextField(null=False, blank=False)

	def set_value(self, obj_value):
		self.value = json.dumps(obj_value)

	def get_value(self):
		try:
			return json.loads(self.value)
		except Exception, e:
			return None

	@staticmethod
	def get_user_settings(user, tenant=None):

		d = {}
		for u in UserCustomSetting.objects.filter(user=user, tenant=None):
			d[u.key] = u.get_value()

		if tenant:
			for u in UserCustomSetting.objects.filter(user=user, tenant=tenant):
				d[u.key] = u.get_value()

		return d

	@staticmethod
	def get_user_setting(user, key, tenant=None):

		try:
			return UserCustomSetting.objects.get(user=user, key=key, tenant=tenant).get_value()
		except UserCustomSetting.DoesNotExist, e:
			return None

	@staticmethod
	def save_user_settings_dict(user, settings, tenant=None):

		for k, v in settings.items():
			s, created = UserCustomSetting.objects.get_or_create(user=user, tenant=tenant, key=k)
			s.set_value( v )
			s.save()

	class Meta:
		unique_together = ("user", "tenant", "key")
		db_table = "usercustomsetting"

#-----------------------------------------------------------------------------#
#  Tenant Configuration                                                       #
#-----------------------------------------------------------------------------#

class PriorityLabel(TenantDependentModel, MTimeModel):

	PRIORITY_LEVEL_MAX_NUM = 8

	# tenant    =

	min_level   = models.IntegerField(null=False, db_index=True)
	max_level   = models.IntegerField(null=False, db_index=True)
	label       = models.TextField(null=False, unique=True)
	description = models.TextField(null=False)

	def __str__(self):
		return "<PriorityLabel:%s:%s:%s>" % (self.label, self.min_level, self.max_level)

	def in_range(self, value):
		return (self.min_level <= value) and (value <= self.max_level)

	#
	# FIX: BRUTTISSIMO. Abbiamo "sovraccaricato" le classi del modello per
	#      gestire cose che sono specifiche dell'interfaccia grafica.
	#      Queste funzionalita' sono webui-dependente e dovrebbero stare
	#      nell'interfaccia grafica
	#
	@staticmethod
	def can_add_new():

		if PriorityLabel.objects.count() >= PriorityLabel.PRIORITY_LEVEL_MAX_NUM:
			return False

		return True

	@staticmethod
	def get_label(p):
		try:
			return PriorityLabel.objects.get(min_level__lte=p, max_level__gte=p).label
		except PriorityLabel.DoesNotExist, e:
			return None

	def get_state_and_conditions(self, critical_priority=None, clist_uuid=None):

		critical_priority = critical_priority if critical_priority != None else PRIORITY_NONE

		filters = {}
		filters['datagroup__suspended'] = False
		filters['priority__gte'] = self.min_level
		filters['priority__lte'] = self.max_level
		filters['status'] = CSTATE_DN

		if clist_uuid is not None:
			filters['uuid__in'] = clist_uuid

		conditions = Condition.active_objects.filter(**filters)
		countDN = 0
		for c in conditions:
			if c.priority >= critical_priority:
				countDN += 1

		if countDN:
			return (CSTATE_TO_CODENAME(CSTATE_DN), countDN)
		elif len(conditions) > 0:
			return (CSTATE_TO_CODENAME(CSTATE_DN), countDN)
		else:
			return (CSTATE_TO_CODENAME(CSTATE_UD), countDN)

	def get_conditions(self):
		filters = {}
		filters['priority__gte'] = self.min_level
		filters['priority__lte'] = self.max_level
		return Condition.active_objects.filter(**filters).all()

	def get_icon_url(self):
		icon_name = self.get_icon_name_or_default()
		icon_url = reverse('webui.views.icons.get_colored_icon3', kwargs = {'icon': icon_name, 'tenantid': self.tenant_id})
		return icon_url

	def get_icon_url_with_state(self, status_code):
		return self.get_icon_url() + get_state_color(status_code)

	def get_icon_name_or_default(self):
		return 'priority'

	def validate(self):

		if self.min_level > self.max_level:
			raise Exception("Min level (%s) >= Max level (%s)" % (self.min_level, self.max_level))

		if not ((PRIORITY_MIN <= self.min_level) and (self.min_level <= PRIORITY_MAX)):
			raise Exception("Invalid min level: %s. It must be between (%s, %s)" % (self.min_level, PRIORITY_MIN, PRIORITY_MAX))

		if not ((PRIORITY_MIN <= self.max_level) and (self.max_level <= PRIORITY_MAX)):
			raise Exception("Invalid max level: %s. It must be between (%s, %s)" % (self.max_level, PRIORITY_MIN, PRIORITY_MAX))

		if PriorityLabel.objects.exclude(label=self.label).filter(Q(min_level__range=(self.min_level, self.max_level))|Q(max_level__range=(self.min_level, self.max_level))).count() > 0:
			raise Exception("Overlapping level")

		return True

	class Meta:
		unique_together = ("label","tenant")
		db_table = "prioritylabel"

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#


class Agent(TenantDependentModel, UUIDKeyModel, MTimeModel, AbstractCachedFieldModel):

	# tenant
	# uuid
	# mtime  

	name        = models.TextField(null=False, db_index=True)

	description = models.TextField(null=False, blank=True, default='')

	is_enabled  = models.BooleanField(null=False, default=True)

	# All nodes will be associated to this agent
	is_primary  = models.BooleanField(null=False, default=False)

	is_remote   = models.BooleanField(null=False, default=False)

	threads     = models.PositiveSmallIntegerField(null=False, default=AGENT_DEFAULT_NUM_THREADS)

	config_update_minperiod = models.PositiveSmallIntegerField(null=False, default=180)

	# Parametri di configurazione di scheduling
	scheduling_min_gap      = models.PositiveSmallIntegerField(null=True , default=None, blank=True , validators=[MinValueValidator(1)])
	scheduling_max_parallel = models.PositiveSmallIntegerField(null=True , default=None, blank=True , validators=[MinValueValidator(1)])
	scheduling_max_deadline = models.PositiveSmallIntegerField(null=True ,               blank=True , validators=[MinValueValidator(1)])  # NOTA: min-value = 1 e' una pura formalita'. In realta' nel 99.9% dei casi dovrebbe essere configurato con un valore almeno di 60 secondi.

	cached_fields = [
		'checktime', # last check timestamp
		'running',   # is running?
		'info',      # textual informations
	]

	def cache_key(self):
		return "%s" % (self.uuid)

	def monitored_nodes_count(self):
		return self.monitoredelement_set.filter(kind=ELEMENT_KIND_NODE ).count()
	
	def monitored_elements_counters(self):
		from django.db.models import Count
		return self.monitoredelement_set.all().values('kind').annotate(total=Count('kind')).order_by('total')

	def __str__(self):
		return "<AGENT:%s:%s>" % (self.uuid, self.name)

	def set_current_state(self, checktime, running, info):
		cache_manager = init_cache_manager_from_conf()
		self.checktime  = checktime
		self.running    = running
		self.info       = info
		self.save_fields_in_cache(cache_manager)

	@property
	def is_running(self):
		if self.running: return True
		else:
			return False

	def get_current_state(self, cache_manager=None):

		if cache_manager is None:
			cache_manager = init_cache_manager_from_conf()

		self.load_fields_from_cache(cache_manager)

		return {
			'checktime': self.checktime,
			'running'  : self.running,
			'info'     : self.info,
		}

	def assert_permission_readwrite(self, user):

		if user and not user.is_superuser:
			raise ApplicationException("User '%s' not allowed to modify agent '%s'" % (user.username, self.name))

		return True

	class Meta:
		unique_together = (
			"tenant", 
			"name"
		)
		db_table = "agent"

#-----------------------------------------------------------------------------#
#  Scheduler classes                                                          #
#-----------------------------------------------------------------------------#

"""

SchedulingGroups
=======================

Gli scheduling group sono gestiti attraverso il modello SchedulingGroup.

Quando uno SchedulingGroup viene definito esplicitamente gli altri oggetti del modello (
Cluster, MonitoredElement, DataGroup) referenziano lo SchedulingGroup in maniera "normale".

Quando un Cluster o MonitoredElement vengono configurati con parametri di scheduling "direttamente", 
viene sempre creato uno SchedulingGroup "implicito" (is_real=False) e vengono valorizzi i suoi attributi.


 esplicita                         +----------+   +----------+
           < >---------------------|   real   |   | implicit |
            |                      +----+-----+   +-----+----+
            |                           |               |
            |                           +-------+-------+
            |                                   |
            |               0,1        +-----------------+         0,1
            |            < >-----------| SchedulingGroup |------------< >
            |          0,N|            +-----------------+             |0,N
            |             |                 .                          |
            |    +-----------------+        .                 +-----------------+
            |    |     Element     |--------.--< >------------|     Cluster     |
            |    +-----------------+        .                 +-----------------+
            |             |                 .                         '
            |             | 1,1             .                         '
            |            < >                .                         '
            |             | 0,N             .                         '
            |             |                 .                         '
            |    +-----------------+.......< >                        ' 
            +----|    Datagroup    |.................................< >
                 +-----------------+           implicite

Dati per gli agenti: campo scheduling_group_id
------------------------------------------------

Il server centrale manda agli agenti due tipi di informazioni:

  * Gli SchedulingGroup configurati (espliciti o impliciti) nel DB
  
  * Per ogni Cluster, MonitoredElement e DataGroup invia un campo "scheduling_group_id"
    che contiene sempre l'UUID dello schedulinggroup a cui quell'entita' appartiene
    (relazione diretta o indiretta).
    
       a) diretta   (foreignkey)
       
       b) indiretta (DataGroup -> [subelement -> ] Node -> Ccluster -> SchedulingGroup
    
    In questo modo l'agente sa immediatamente ogni entita' a quale scheduling-group
    e' associata (direttamente o indirettamente).

"""

class SchedulingGroup(TenantDependentModel, UUIDKeyModel, ModelFieldsUtils, OwnerUserModel):
	"""
	
	"""
	
	# tenant     =
	# uuid       =
	
	# owner      =
	
	# Campo di comodo per filtrare velocemente i schedulinggroup espliciti da quelli creati in base ad elementi di monitoraggio
	is_real      = models.BooleanField(null=False, default=True)

	name         = models.TextField(null=False, blank=False, unique=True)
	description  = models.TextField(null=False, blank=True, default='')

	# None = non configurato. Nessun gap
	# 0    = Nessun gap
	# N    = valore intero
	min_gap      = models.PositiveSmallIntegerField(null=True , default=None, blank=True , validators=[MinValueValidator(1)])
	
	# None = non configurato (parallelismo massimo)
	# 0    = non valido
	# N    = mass
	max_parallel = models.PositiveSmallIntegerField(null=True , default=None, blank=True , validators=[MinValueValidator(1)])
	
	# None = Non configurato.
	# 0    = non valido
	max_deadline = models.PositiveSmallIntegerField(null=True ,               blank=True , validators=[MinValueValidator(1)])  # NOTA: min-value = 1 e' una pura formalita'. In realta' nel 99.9% dei casi dovrebbe essere configurato con un valore almeno di 60 secondi.


	def validate(self):
		if self.min_gap is None and self.max_parallel is None:
			raise InvalidConfigurationException("Min Gap or Max Parallel must be configured.")

		if self.min_gap      == 0: raise InvalidConfigurationException("Min gap can not be 0")
		if self.max_parallel == 0: raise InvalidConfigurationException("Max Parallel can not be 0")
		if self.max_deadline == 0: raise InvalidConfigurationException("Max Deadline can not be 0")

	def check_and_delete(self, force=False):

		if not force:
			if DataGroup       .objects.filter(tenant=self.tenant, scheduling_group=self).count() > 0 or \
			   MonitoredElement.objects.filter(tenant=self.tenant, scheduling_group=self).count() > 0: 
			   raise InvalidConfigurationException("Unable to delete '%s'. It's used inside other configurations" % (self.name))

		# I singoli datagroup di un elemento potrebbero essere configurati per essere associati ad uno scheduling-group.
		# Togliamo il riferimento "debole" in configurazione per ogni CDataGroup
		#
		#    SchedulingGroup . . . . . . . . . . . . . . . < >
		#           |                                       .   
		#          < >                                      .
		#           |                                       .
		#       DataGroup---< >---CDataGroup--< >--CDataGroupFieldValue
		#
		configured_fields = CDataGroupFieldValue.objects.filter(tenant=self.tenant).filter(tfield__name='scheduling_group', value=self.name)
		configured_fields.delete()

		# Questa delete dovrebbe cancellare lo schedulinggroupe ed azzerare tutte le foreignkey.
		self.delete()
		
		return


	@transaction_commit_on_success
	def set_owner(self, owner, action_user=None):

		if action_user:
			if not action_user.is_superuser:
				raise ApplicationException("User '%s' is not superuser." % (action_user.username))

		self.owner = owner
		self.save()


	@transaction_commit_on_success
	def unset_owner(self, action_user=None):

		if action_user:
			if not action_user.is_superuser:
				raise ApplicationException("User '%s' is not superuser." % (action_user.username))

		self.owner = None
		self.save()


	@staticmethod
	def load_for_agent(tenant):
		"""
		Il server manda agli agenti solo gli SchedulingGroup configurati nel DB.
		
		Tutti gli SchedulingGroup "impliciti" che l'agente ha bisogno di creare
		in piu' (a causa di parametrizzazioni globali dell'agente) vengono 
		gestiti dall'agente a runtime.
		"""
		return SchedulingGroup.objects.filter(tenant=tenant).all().values(
			'uuid',
			'name',
			'min_gap',
			'max_deadline',
			'max_parallel',
		)



	def get_group_permissions(self):
		return SchedulingGroupPermission.objects.filter(tenant=self.tenant, schedulinggroup=self)

	def assert_permission_readwrite(self, user):

		if user.is_superuser: 
			return True

		if self.owner == user:
			return True

		if self.get_group_permissions().filter(group__in=user.groups.all(), auth_type=AUTH_READWRITE).count() > 0:
			return True

		raise ApplicationException("User '%s' has not READ-WRITE permission on schedulingroup '%s'" % (user.username, self))

	def add_group_permission(self, group, auth_type, action_user):

		if action_user:
			#action_user.assert_capability(self.tenant, 'core.template.element.acl.edit')
			self.assert_permission_readwrite(action_user)
			action_user.assert_in_group(group.name)

		tsgp, created = SchedulingGroupPermission.objects.get_or_create(tenant=self.tenant, schedulinggroup=self, group=group)
		tsgp.auth_type = auth_type
		tsgp.save()

	def remove_group_permission(self, group, action_user):

		if action_user:
			#action_user.assert_capability(self.tenant, 'core.template.element.acl.edit')
			self.assert_permission_readwrite(action_user)
			action_user.assert_in_group(group.name)

		tsgp = SchedulingGroupPermission.objects.filter(tenant=self.tenant, schedulinggroup=self, group=group)

		tsgp.delete()

	@staticmethod
	def get_visible_by_user(tenant, user):

		query = SchedulingGroup.objects

		query = query.filter(tenant=tenant)

		if user and not user.is_superuser:
			groups = user.groups.all()
			#print __file__, groups
			ids = [ r[0] for r in SchedulingGroupPermission.objects.filter(tenant=tenant, group__in=groups).values_list('schedulinggroup_id')]
			query = query.filter(Q(owner=user)|Q(uuid__in=ids))

		return query



	class Meta:
		db_table = "scheduling_group"
		unique_together = (
			"tenant",
			"name"
		)

class AbstractSchedulingModel(models.Model): #AbstractSchedulerBaseModel):
	"""
	Abstract class used in MonitoredElement, Cluster and Datagroup
	to implement scheduling parameters and group on db.
	"""

	scheduling_group = models.ForeignKey('SchedulingGroup', null=True, on_delete=models.SET_NULL)

	class Meta:
		abstract = True

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class Cluster(TenantDependentModel, UUIDKeyModel, MTimeModel, OwnerUserModel, AbstractSchedulingModel): #, AbstractSchedulerBaseModel):

	# tenant    =
	# uuid      =
	# mtime     =
	
	# owner     =
	
	# scheduling_group =
	
	name        = models.TextField(null=False, blank=False)
	description = models.TextField(null=False, blank=True, default='')

	def __str__(self):
		return "<Cluster:%s>" % (self.name)

	def get_dict(self):
		return {
			'uuid'       : self.uuid,
			'mtime'      : self.mtime,
			'name'       : self.name,
			'description': self.description,
		}

	@transaction_commit_on_success
	def set_owner(self, owner, action_user=None):

		if action_user:
			if not action_user.is_superuser:
				raise ApplicationException("User '%s' is not superuser." % (action_user.username))

		self.owner = owner
		self.save()

	@transaction_commit_on_success
	def unset_owner(self, action_user=None):

		if action_user:
			if not action_user.is_superuser:
				raise ApplicationException("User '%s' is not superuser." % (action_user.username))

		self.owner = None
		self.save()


	@staticmethod
	def load_for_agent(tenant):
		elements = Cluster.objects.filter(tenant=tenant)

		# Nei dati che mandiamo all'agente serve che tutti gli oggetti che hanno uno scehduling group associato
		# abbiamo il campo "scheduling_group_id".
		#
		# Il queryset esporterebbe "scheduling_group". Per esportare il campo con il nome giusto 
		# dobbiamo fare un giochino di "labelling"
		
		elements = elements.extra(select={'scheduling_group_id':'scheduling_group_id'})

		elements = elements.values(
			'uuid',
			'mtime',
			'name',
			'description',

			'scheduling_group_id',
		)

		return elements.all()

	def get_all_nodes(self):
		return self.monitoredelement_set.filter(kind = ELEMENT_KIND_NODE)


	@property
	def implicit_scheduling_group_name(self):
		"""
		Metodo che restituisce il nome dello SchedulingGroup creato implicitamente per un cluster.
		
		IMPOTANTE: usiamo l'UUID e non il nome perche' in caso di "rename" del cluster dovremmo aggiornare
		           anche il nome dello SchedulingGroup.
		"""
		return "cluster:%s" % (self.uuid)


	def get_group_permissions(self):
		return ClusterGroupPermission.objects.filter(tenant=self.tenant, cluster=self)

	def assert_permission_readwrite(self, user):

		if user.is_superuser: 
			return True

		if self.owner == user:
			return True

		if self.get_group_permissions().filter(group__in=user.groups.all(), auth_type=AUTH_READWRITE).count() > 0:
			return True

		raise ApplicationException("User '%s' has not READ-WRITE permission on cluster '%s'" % (user.username, self))

	def add_group_permission(self, group, auth_type, action_user):

		if action_user:
			#action_user.assert_capability(self.tenant, 'core.template.element.acl.edit')
			self.assert_permission_readwrite(action_user)
			action_user.assert_in_group(group.name)

		cgp, created = ClusterGroupPermission.objects.get_or_create(tenant=self.tenant, cluster=self, group=group)
		cgp.auth_type = auth_type
		cgp.save()

	def remove_group_permission(self, group, action_user):

		if action_user:
			#action_user.assert_capability(self.tenant, 'core.template.element.acl.edit')
			self.assert_permission_readwrite(action_user)
			action_user.assert_in_group(group.name)

		cgp = ClusterGroupPermission.objects.filter(tenant=self.tenant, cluster=self, group=group)
		cgp.delete()

	@staticmethod
	def get_visible_by_user(tenant, user):

		query = Cluster.objects

		query = query.filter(tenant=tenant)

		if user and not user.is_superuser:
			groups = user.groups.all()
			#print __file__, groups
			ids = [ r[0] for r in ClusterGroupPermission.objects.filter(tenant=tenant, group__in=groups).values_list('cluster_id')]
			query = query.filter(Q(owner=user)|Q(uuid__in=ids))

		return query

	class Meta:
		unique_together = (
			"tenant", 
			"name"
		)
		db_table = "cluster"


@receiver(post_delete, sender=Cluster)
def _cluster_post_delete_schedgroup(sender, **kwargs):
	"""
	Gli SchedulingGroup "impliciti" (is_real=False) vengono creati automaticamente
	e devono essere cancellati automaticamente quando un elemento viene cancellato.
	"""

	cluster = kwargs['instance']

	name = cluster.implicit_scheduling_group_name

	try:
		SchedulingGroup.objects.get(tenant=cluster.tenant, is_real=False, name=name).delete()
	except SchedulingGroup.DoesNotExist, e:
		pass

	return



#-----------------------------------------------------------------------------#
# Element Templates                                                           #
#                                                                             #
# 	node-template                                                         #
# 	interface-template                                                    #
# 	storage-template                                                      #
# 	service-template                                                      #
# 	device-template                                                       #
#                                                                             #
#-----------------------------------------------------------------------------#

"""
                             +------------+
                             |  TElement  |
                             +------------+
                                   |
            +----------------------+-----------------------+
            |                                              |
     +------------+         1,1        0,N          +------------+
     |   Normal   |------------<parent>-------------| SubElement |
     +------------+                                 +------------+
            |                                              |
            | 1,1                                          | 1,1
           < >---------<telementappliedtemplate>----------< >
                    0,N                         0,N               
"""

class TElement(TenantRelatedModel, MTimeModel, OwnerUserModel):

	# tenant    =
	# owner     =
	# mtime     =

	kind        = models.PositiveIntegerField(null=False, default=ELEMENT_KIND_NODE)

	# Il nome di un template e' codificato in due modi:
	#  - <nome>          -> per gli TElement normali
	#  - <parent>:<nome> -> per i TElement che rappresentano sub-element-template
	name        = models.TextField(null=True, blank=False, default=None)

	description = models.TextField(null=False, blank=False, default='')

	parent      = models.ForeignKey('self', null=True, on_delete=models.CASCADE)

	#
	# Campi NORMAL (di fatto solo node-template)
	#

	# Questi campi hanno senso solo per i TElement per i nodi
	is_switch   = models.BooleanField(null=False, default=False)
	is_router   = models.BooleanField(null=False, default=False)
	priority    = models.IntegerField(null=False, default=PRIORITY_NONE, db_index=True)

	#
	# Campi SubElement (di fatto iface-template/storage-template/service-template/device-template.
	#

	# Per semplificare, questi campi sono gli stessi per interfacce, storage, service, device.
	# Il campo "instance" serve per parametrizzare il corrispondente campo ifindex, strindex, swrunindex, devindex.
	distinguisher = models.TextField(null=True, blank=False, default=None)
	xform         = models.TextField(null=True, blank=False, default=None)
	instance      = models.TextField(null=True, blank=False, default=None)







	def get_reference_count(self):
		
		if self.parent_id is None:
			return self.elementtemplate_set.count()
		else:
			return MonitoredElement.objects.filter(telement_generator=self).count()

	def get_element_name_to_apply(self):
		assert self.parent_id is not None
		return self.name.split(':')[1]

	def get_name(self):
		if self.parent is None:
			return self.name
		else:
			return self.get_element_name_to_apply()

	def get_sub_elements(self):
		return self.telement_set.all()

	def get_applied_templates(self):
		if self.parent is None:
			raise Exception("BUG: non doveva succedere. get_applied_templates() su '%s' che non e' un sub-element template" % (self.name))
		
		#return [ e.telementapplied for e in self.telements_applied.order_by('applyorder') ]
		
		return self.telements_applied.order_by('applyorder')
		
		
	def get_configured_element_fields_dict(self):
		"""
		Metodo di comodo per avere l'elenco di tutti i campi valorizzati in questo template.
		"""
		return {
			'is_switch': self.is_switch,
			'is_router': self.is_router,
			
			'distinguisher': self.distinguisher,
			'xform'        : self.xform,
			'instance'     : self.instance,
		}

	def __str__(self):
		return "<TElement:%s:%s>"%(self.kind, self.name)

	@property
	def template_kind_str(self):
		return ELEMENT_KIND_STR[self.kind]



	def add_group_permission(self, group, auth_type, action_user):

		if action_user:
			action_user.assert_capability(self.tenant, 'core.template.element.acl.edit')
			self.assert_permission_readwrite(action_user)
			action_user.assert_in_group(group.name)

		tep, created = TElementPermission.objects.get_or_create(tenant=self.tenant, telement=self, group=group)
		tep.auth_type = auth_type
		tep.save()

	def remove_group_permission(self, group, action_user):

		if action_user:
			action_user.assert_capability(self.tenant, 'core.template.element.acl.edit')
			self.assert_permission_readwrite(action_user)
			action_user.assert_in_group(group.name)

		tep = TElementPermission.objects.filter(tenant=self.tenant, telement=self, group=group)

		tep.delete()

	def get_group_permissions(self):
		return TElementPermission.objects.filter(tenant=self.tenant, telement=self)


	def assert_permission_readwrite(self, user):

		if user.is_superuser: 
			return True

		if self.owner == user:
			return True

		if TElementPermission.objects.filter(tenant=self.tenant, telement=self, group__in=user.groups.all(),  auth_type=AUTH_READWRITE).count() > 0:
			return True

		raise ApplicationException("User '%s' has not READ-WRITE permission on template '%s'" % (user.username, self))


	@staticmethod
	def get_visible_by_user(tenant, user):

		query = TElement.objects

		query = query.filter(tenant=tenant)

		if user and not user.is_superuser:
			groups = user.groups.all()
			ids = [ r[0] for r in TElementPermission.objects.filter(tenant=tenant, group__in=groups).values_list('telement_id')]
			query = query.filter(Q(owner=user)|Q(id__in=ids))

		return query




	class Meta:
		db_table = "c_telement"
		unique_together = (
			#
			# FIX: sarebbe piu' corretto definire
			#         ("tenant", "parent", "kind", "name")
			#      ma poiche' il "name" contiene codificato il nome del "parent"
			#      per il momento non serve modificare quesot vincolo di integrita'.
			#
			
			("tenant",          "kind","name"),
			#("tenant","parent", "kind","name"),   # FIX: definizione piu' corretta da gestire con sanetdata per le installazioni correnti.
		)

class TElementAppliedTemplate(TenantDependentModel):
	
	# tenant

	telement        = models.ForeignKey(TElement, null=False, related_name='telements_applied')

	telementapplied = models.ForeignKey(TElement, null=False)
	applyorder      = models.PositiveIntegerField(null=False, default=0)

	class Meta:
		db_table = "c_telementappliedtemplate"
		unique_together = (
			("tenant", "telement", "telementapplied")
		)

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

def _get_state_from_condition(element_uuid, conditions, minimum_priority=PRIORITY_NONE):
	"""
	Algoritmo generale per il calcolo dello stato di un elemento/datagroup.
	"""
	
	#print __file__, "_get_state_from_condition", 1
	#print "\t", element_uuid
	#print "\t", len(conditions)
	#print "\t", minimum_priority
	#print __file__, "_get_state_from_condition", 2

	# Se l'elemento non ha neanche una condition allora non sappiamo nulla di lui
	# IMPORTANTE: diverso e' il caso di condition scartate perche' sotto soglia minima
	if len(conditions) == 0:
		return CSTATE_TO_CODENAME(CSTATE_DEFAULT), 0

	#
	# CALCOLO CONTATORI
	#
	is_down        = False
	up             = 0
	fa             = 0
	down           = 0
	uncheckable    = 0
	uncheckable_up = 0
	uncheckable_dn = 0
	suspended      = 0

	tot       = len(conditions)
	tot_valid = 0
	#atot_valid = len([c for c in conditions if c.priority >= minimum_priority])
	
	for c in conditions:

		if c.priority < minimum_priority:
			continue

		tot_valid += 1

		# Il primario di questo elemento e' down? -> DOWN
		#if c.primary and c.element_id == self.uuid:
		if c.primary:
			if c.element_id == element_uuid:
				#tot += 1
				if c.status in [CSTATE_DN,]:
					is_down = True
		else:
			if c.priority < minimum_priority:
				continue

		if   c.status == CSTATE_UP             : up          += 1
		elif c.status == CSTATE_FA             : fa          += 1
		elif c.status == CSTATE_DN             : down        += 1
		elif c.status == CSTATE_IN             : suspended   += 1
		elif c.status in [CSTATE_UU, CSTATE_DU, CSTATE_TU]:
			uncheckable    += 1
			uncheckable_up += 1
		elif c.status in [CSTATE_UD, CSTATE_DD, CSTATE_TD]:
			uncheckable    += 1
			uncheckable_dn += 1

	if 0:
		print "\t", "tot"        , len(conditions)
		print "\t", "tot valid"  , tot_valid
		print "\t", "up"         , up
		print "\t", "fa"         , fa
		print "\t", "down"       , down
		print "\t", "UN up"      , uncheckable_up
		print "\t", "UN dn"      , uncheckable_dn
		print "\t", "suspended"  , suspended

	#
	# CALCOLO STATO
	#

	if is_down:
		return CSTATE_TO_CODENAME(CSTATE_DN), down


	if tot_valid == 0:
		return CSTATE_TO_CODENAME(CSTATE_UP), down

	#print 1
	if down:
		if up > 0:
			return CSTATE_TO_CODENAME(CSTATE_FA), down
		elif fa > 0:
			return CSTATE_TO_CODENAME(CSTATE_FA), down
		else:
			#print "YYYYYYYYYYY"
			return CSTATE_TO_CODENAME(CSTATE_DN), down

	#print 2

	if fa:
		return CSTATE_TO_CODENAME(CSTATE_FA), down

	#print 3

	if up:
		return CSTATE_TO_CODENAME(CSTATE_UP), down

	#print 4

	#print tot, tot_valid, suspended
	#if tot_valid > 0 and suspended == tot_valid:
	if suspended == tot_valid:
		if tot == tot_valid:
			return CSTATE_TO_CODENAME(CSTATE_IN), down
		else:
			return CSTATE_TO_CODENAME(CSTATE_UP), down

	#print 5

	# Se tutte le condition sono uncheckable l'elemento e' uncheckable
	if uncheckable_dn:
		return CSTATE_TO_CODENAME(CSTATE_UD), down

	#print 6

	if uncheckable_up:
		return CSTATE_TO_CODENAME(CSTATE_UU), down

	#print 7

	# Non dovremmo mai arrivare a questo punto
	return "UP", down

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class MonitoredElement(TenantDependentModel, UUIDKeyModel, CTimeModel, MTimeModel, CachedUtil, AbstractCachedFieldModel, ModelFieldsUtils, OwnerUserModel, PermissionModel, AbstractSchedulingModel):

	# tenant              =
	# uuid                =
	# mtime               = 

	# owner               = 
	
	# scheduling_group    =

	kind                  = models.PositiveIntegerField(null=False, default=ELEMENT_KIND_NODE)

	name                  = models.TextField(null=False, blank=False, verbose_name='Name')
	description           = models.TextField(null=False, blank=True, default='', verbose_name="Description")

	agent                 = models.ForeignKey(Agent, null=True, default=None, verbose_name="Agent")#, related_name='monitoring_agent')

	is_abstract           = models.BooleanField(null=False, default=False)
	parent                = models.ForeignKey('MonitoredElement', null=True, default=None)

	cluster               = models.ForeignKey(Cluster, null=True, default=None, verbose_name="Cluster")
	cluster_role          = models.PositiveIntegerField(null=True, default=CLUSTER_ROLE_MASTER  , verbose_name="Cluster role")
	cluster_distinguisher = models.TextField(null=True, blank=True, db_index=False, default=None, verbose_name="Cluster distinguisher")
	cluster_xform         = models.TextField(null=True, blank=True, db_index=False, default=None, verbose_name="Cluster Xform")
	cluster_instance      = models.TextField(null=True, blank=True, db_index=False, default=None, verbose_name="Cluster instance") # da cachare

	# Se specificato, livello di priority (normalizzato) degli allarmi associatia questo elemento.
	priority              = models.IntegerField(null=False, default=PRIORITY_NONE, db_index=True, verbose_name="Default Priority")

	icon                  = models.TextField(null=True, blank=True, db_index=False, default=None, verbose_name="Icon name")

	notify_address        = models.TextField(null=True, blank=True, default=None, verbose_name="Email used for system anomialies.")

	# Gli elementi di monitoraggio generati da sub-elementi di un node-template
	# vengono automaticamente assegnati al loro "sub-TElement". In questo modo,
	# quando un sub-TElement viene cancellato, automaticamente viene rimosso
	# l'elemento che rappresenta l'instanza di quel sub-TElement.
	# Teoricamente questo campo avrebbe senso solo per interfacce, storage, service, device,
	# ma lo mettiamo qua perche' e' piu' pratico gestire le ricerche se questo campo
	# e' nella tabella comune e non nelle specializzazioni.
	telement_generator    = models.ForeignKey(TElement, null=True, on_delete=models.CASCADE)


	@staticmethod
	def kind_to_class(kind):
		
		d = {
			ELEMENT_KIND_IFACE  : MonitoredIface,
			ELEMENT_KIND_STORAGE: MonitoredStorage,
			ELEMENT_KIND_SERVICE: MonitoredService,
			ELEMENT_KIND_DEVICE : MonitoredDevice,
		}
		return d[kind]


	CLUSTER_FIELDS = [
		  'cluster'              
		, 'cluster_role'         
		, 'cluster_distinguisher'
		, 'cluster_xform'        
		, 'cluster_instance'     
	]

	cached_fields = [
		'cluster_element_id',    # Master element id
		'cluster_instance_time'  # timestamp last cluster association recalculation
	]

	def cache_key(self):
		return "%s" % (self.uuid)

	def __str__(self):
		return "<Element:%s:%s>" % (self.kind, self.name)

	@property
	def kind_str(self):
		return ELEMENT_KIND_STR[self.kind]


	def get_dict(self):
		return {
			'uuid'                 : self.uuid,
			'tenant_id'            : self.tenant_id,
			'mtime'                : self.mtime,
			'name'                 : self.name,
			'kind'                 : self.kind,
			'description'          : self.description,
			'is_abstract'          : self.is_abstract,
			'parent_id'            : self.parent_id,
			'cluster_id'           : self.cluster_id,
			'cluster_role'         : self.cluster_role,
			'cluster_distinguisher': self.cluster_distinguisher,
			'cluster_xform'        : self.cluster_xform,
			'cluster_instance'     : self.cluster_instance,

			'cluster_element_id'   : self.cluster_element_id,
			'cluster_instance_time': self.cluster_instance_time,
		}


	@property
	def priority_level_descr(self):
		c = self.get_primary_condition()
		if c is not None:
			if c.priority is not PRIORITY_NONE:
				return '%s (%s)' % (PriorityLabel.get_label(c.priority),c.priority)
			else:
				return None
		return ''

	@property
	def cluster_role_descr(self):
		if self.cluster_role:
			return CLUSTER_ROLE_STR[self.cluster_role]
		else:
			return "n/a"

	@property
	def cluster_element(self):

		if not hasattr(self, 'cluster_element_id'):
			return None

		if self.cluster_element_id is None:
			return None

		try:
			return MonitoredElement.objects.get(tenant=self.tenant, uuid=self.cluster_element_id)
		except MonitoredElement.DoesNotExist, e:
			return None

	def validate_fields(self):

		if not( self.priority == PRIORITY_NONE or (PRIORITY_MIN <= self.priority and self.priority <= PRIORITY_MAX)):
			raise Exception("Invalid priority value: %s" % (self.priority))

		#
		# FIX: other fields?
		#
		return True

	def validate_and_save(self):
		self.validate_fields()
		self.save()

	def get_node(self):
		if self.kind == ELEMENT_KIND_NODE:
			return self.get_specific_element()
		else:
			return self.parent.get_specific_element()

	def get_iface(self):
		if self.kind == ELEMENT_KIND_IFACE:
			return self.get_specific_element()
		return None
		

	def create_sub_element(self, name, kind):
		"""
		Crea MonitoredElement di tipo "kind" con nome "name".
		
		Se l'elemento esiste gia' non effettua alcuna operazione.
		
		Restituisce un oggetto MonitoredElemento che rappresenta l'elemento.
		"""

		kind_to_class = {
			ELEMENT_KIND_IFACE  : MonitoredIface,
			ELEMENT_KIND_STORAGE: MonitoredStorage,
			ELEMENT_KIND_SERVICE: MonitoredService,
			ELEMENT_KIND_DEVICE : MonitoredDevice,
		}

		klass = kind_to_class[kind]

		sub, created = klass.objects.get_or_create(
			tenant = self.tenant, 
			name   = name, 
			parent = self,
			kind   = kind,
		)
		sub.agent = self.agent
		sub.owner = self.owner
		sub.save()

		return sub


	@property
	def ifaces(self):
		return MonitoredIface.objects.filter(parent=self).select_related()

	@property
	def storages(self):
		return MonitoredStorage.objects.filter(parent=self).select_related()

	@property
	def timegraphs(self):
		return TimeGraph.objects.filter(Q(element=self))


	def get_all_sub_elements(self):
		return self.monitoredelement_set.all()

	def get_sub_elements(self, kind=None):
		r = self.monitoredelement_set
		if kind != None:
			r = r.filter(kind=kind)
		return r

	def get_all_datagroups(self):
		return DataGroup.active_objects.filter(Q(element__in=self.monitoredelement_set.all())|Q(element=self)).order_by('path')

	def get_all_datasources(self):
		return DataSource.active_objects.filter(Q(element__in=self.monitoredelement_set.all())|Q(element=self)).order_by('path')

	def get_all_conditions(self):
		return Condition.active_objects.filter(Q(element__in=self.monitoredelement_set.all())|Q(element=self)).order_by('path')

	def get_all_timegraphs(self):
		return TimeGraph.active_objects.filter(Q(element__in=self.monitoredelement_set.all())|Q(element=self)).order_by('element__name')

	def get_all_timegraphseries(self):
		return TimeGraphSerie.active_objects.filter(timegraph__in=self.get_all_timegraphs())


	def get_primary_condition(self):
		conditions = self.monitored_conditions.filter(deleted=False).filter(primary=True).select_related('element', 'element__parent', 'datagroup').order_by('path')
		if conditions:
			return conditions[0]
		return None

	def get_primary_timegraph(self):
		try:
			return self.timegraphs.get(primary=True)
		except TimeGraph.DoesNotExist, e:
			return None
		except TimeGraph.MultipleObjectsReturned, e:
			return None

	def _get_fields(self):
		d = {}
		for f in self._meta.fields:
			d[f.name] = f
		return d

	def convert_field_str_value(self, f_name, f_str_value):

		# IMPORTANT: this is a super hack. We should define our type-conversion functions
		#return f.to_python(f_str_value)

		f = self._get_fields()[f_name]

		if f.get_internal_type() == 'BooleanField':
			if f_str_value.lower() in ['0','false']:
				return False
			else:
				return True

		return f.get_prep_value(f_str_value)

	def not_default_fields(self, exclude_fields=None):

		# Elenco dei campi dell'oggetto valorizzati con valori diversi da quelli di default nel modello
		r = []
		for field in self._meta.fields:

			if isinstance(field, models.ForeignKey): continue

			if exclude_fields and field.name in exclude_fields: continue

			if field.has_default():
				v = getattr(self, field.name)
				v2 = field.get_default()
				if v != v2:
					r.append(field.name)
			else:
				r.append(field.name)
		return r

	def get_specific_element(self):

		if   self.kind == ELEMENT_KIND_NODE   : e = self.monitorednode
		elif self.kind == ELEMENT_KIND_IFACE  : e = self.monitorediface
		elif self.kind == ELEMENT_KIND_STORAGE: e = self.monitoredstorage
		elif self.kind == ELEMENT_KIND_SERVICE: e = self.monitoredservice
		elif self.kind == ELEMENT_KIND_DEVICE : e = self.monitoreddevice
		else: raise Exception("INVALID KIND: %s" % (self.kind))

		return e

	#
	# Path
	#
	@property
	def path(self):
		return self.calculate_path()
	
	def calculate_path(self):
		# <element> [ : <element> ] * ; [ <element template>  ] ; <datagroup name> 
		ppath = self.parent.calculate_path() if self.parent else None			
		return self.calculate_path_with_parent(ppath)

	def calculate_path_with_parent(self, parent_path):
		if parent_path is not None:
			return parent_path + DATAGROUP_PATH_SUBSEP + self.name
		else:
			return self.name

	#
	# Priority
	#

	def refresh_elements_priority(self, min_priority=None, mtime=None):

		# Check defaults
		if mtime is None: mtime = self.mtime

		#
		self.refresh_datagroups_priority(min_priority=min_priority, mtime=mtime)

		for e in self.get_sub_elements().all():
			e_priority = e.priority if e.priority != PRIORITY_NONE else min_priority

			e.refresh_elements_priority(min_priority=e_priority, mtime=mtime)

		# IMPORTANTE: con questa chiamata possiamo annullare qualunque transazione "concorrente"
		#             sull'elemento corrente o sotto-elemento.
		#             Questo dovrebbe garantire che aggiornamenti ricorsivi
		#             siano "atomici".
		self.concurrent_save(['mtime'])

	def refresh_datagroups_priority(self, min_priority=None, mtime=None):

		# Check defaults
		if min_priority is None: min_priority = self.get_effective_priority()
		if mtime        is None: mtime = self.mtime

		# Refresh datagroups
		for dg in self.datagroups.all():
			dg.refresh_priority(min_priority=min_priority, mtime=mtime)

	def get_effective_priority(self):
		p = self.priority
		if p == PRIORITY_NONE:
			if self.parent is not None:
				p = self.parent.priority
		return p

	#
	# State
	#
	def get_state(self, critical_priority=PRIORITY_NONE, minimum_priority=PRIORITY_NONE):

		state, critical_dn_count, minimum_dn_count, dn_count = self.get_state_and_conditions(critical_priority=critical_priority, minimum_priority=minimum_priority)

		return state


	def get_state_and_conditions(self, critical_priority=PRIORITY_NONE, minimum_priority=PRIORITY_NONE, loaded_conditions=None):

		if loaded_conditions is not None:
			conditions = loaded_conditions
		else:
			conditions = Condition.active_objects.filter(Q(element__in=self.monitoredelement_set.all())|Q(element=self)).only("primary", "status").all()

		state, critical_dn_count, minimum_dn_count, dn_count =  self._get_state_and_conditions(conditions, critical_priority=critical_priority, minimum_priority=minimum_priority)

		# Stato ROSSO (DN) se c'e' almeno una condition "critical" DOWN
		if (critical_dn_count > 0):
			state = CSTATE_TO_CODENAME(CSTATE_DN)

		return state, critical_dn_count, minimum_dn_count, dn_count

	def _get_state_and_conditions(self, conditions, critical_priority=PRIORITY_NONE, minimum_priority=PRIORITY_NONE):

		critical_dn_count = 0
		minimum_dn_count  = 0
		all_dn_count      = 0

		for c in conditions:
			critical_dn_count += 1 if (c.status == CSTATE_DN and c.priority != PRIORITY_NONE and (c.priority >= critical_priority            ))  else 0
			#minimum_dn_count  += 1 if (c.status == CSTATE_DN and c.priority != PRIORITY_NONE and (c.priority >= minimum_priority or c.primary))  else 0
			minimum_dn_count  += 1 if (c.status == CSTATE_DN and (c.priority >= minimum_priority or c.primary))  else 0
			all_dn_count      += 1 if (c.status == CSTATE_DN)                                                    else 0

		#state = self.get_state_from_conditions(conditions, minimum_priority=minimum_priority)
		
		state, dn_count2 = _get_state_from_condition(self.uuid, conditions, minimum_priority=minimum_priority)

		return (state, critical_dn_count, minimum_dn_count, all_dn_count)

	#def get_state_from_conditions(self, conditions, minimum_priority=PRIORITY_NONE):
	#	code, dn_count = _get_state_from_condition(self.uuid, conditions, minimum_priority=minimum_priority)
	#	return code

	def get_icon_url(self):
		icon_name = self.get_icon_name_or_default()
		icon_url = reverse('webui.views.icons.get_colored_icon3', kwargs = {'icon': icon_name, 'tenantid': self.tenant_id})
		return icon_url

	def get_icon_url_with_state(self, status_code):
		return self.get_icon_url() + get_state_color(status_code)

	def get_icon_name_or_default(self):
		if self.icon: return self.icon
		return _DEFAULT_MONITOREDELEMENT_ICONS[self.kind]

	def get_logs(self):
		#
		# FIX: skip logs related to "deleted=True" conditions
		#      skip logs related to old conditions (now deleted)
		#
		path = self.path

		# I log della risorsa hanno come element_path:
		# - il path dell'elemento                     <node name>
		# - il path di tutti i sotto elementi         <node name>:<sub>

		filters = Q(element_path=path) | Q(element_path__startswith=path+':')

		# - I log potrebbero essere associati a vecchi path, 
		#    quindi cerchiamo i log anche in base all'UUID degli elementi/conditions
		
		e_uuids = [self.uuid] + list(self.get_all_sub_elements().values_list('uuid', flat=True).all())
		filters |= Q(element_uuid__in=e_uuids)

		c_uuids = self.get_all_conditions().values_list('uuid', flat=True)
		filters |= Q(condition_uuid__in=c_uuids)

		return Event.objects.filter(filters)

	@transaction_commit_on_success
	def set_owner(self, owner, action_user=None):

		#print __file__, "set_owner", self, owner, action_user

		if action_user:
			if not action_user.is_superuser:
				raise ApplicationException("User '%s' is not superuser." % (action_user.username))


		self.owner = owner
		self.save()

		self.get_all_sub_elements()   .update(owner=self.owner)
		self.get_all_datagroups()     .update(owner=self.owner)
		self.get_all_conditions()     .update(owner=self.owner)
		self.get_all_datasources()    .update(owner=self.owner)
		self.get_all_timegraphs()     .update(owner=self.owner)
		self.get_all_timegraphseries().update(owner=self.owner)

	@transaction_commit_on_success
	def unset_owner(self, action_user=None):

		#print __file__, "set_owner", self, owner, action_user

		if action_user:
			if not action_user.is_superuser:
				raise Exception("User '%s' is not superuser." % (action_user.username))

		self.owner = None
		self.save()

		self.get_all_sub_elements()   .update(owner=self.owner)
		self.get_all_datagroups()     .update(owner=self.owner)
		self.get_all_conditions()     .update(owner=self.owner)
		self.get_all_datasources()    .update(owner=self.owner)
		self.get_all_timegraphs()     .update(owner=self.owner)
		self.get_all_timegraphseries().update(owner=self.owner)

	@property
	def implicit_scheduling_group_name(self):
		"""
		Metodo per creare il nome dello SchedulingGroup implicito associato all'elemento di monitoraggio.
		
		IMPORTANTE: usiamo l'uuid e non il "name" per evitare problemi in caso di rename dell'elemento.
		"""
		return "%s:%s" % (ELEMENT_KIND_STR[self.kind], self.uuid)


	def hide_sensible_data(self):
		pass

	class Meta:
		unique_together = (
			"tenant", 
			"kind",
			"name",
			"parent",
		)
		db_table = "monitoredelement"
		ordering= ['name']



@receiver(post_delete, sender=MonitoredElement)
def _monitoredelement_post_delete_schedgroup(sender, **kwargs):
	"""
	Gli SchedulingGroup "impliciti" (is_real=False) vengono creati automaticamente
	e devono essere cancellati automaticamente quando un elemento viene cancellato.
	"""

	me = kwargs['instance']

	#name = '%s:%s' % (me.kind_str, me.path)
	name = me.implicit_scheduling_group_name

	try:
		SchedulingGroup.objects.get(tenant=me.tenant, is_real=False, name=name).delete()
	except SchedulingGroup.DoesNotExist, e:
		pass

	return



#-----------------------------------------------------------------------------#

class ElementTemplate(TenantDependentModel):

	melement                = models.ForeignKey(MonitoredElement, null=False)

	telement                = models.ForeignKey(TElement        , null=False)
	applyorder              = models.PositiveIntegerField(null=False, default=0)

	# Questo campo e' valorizzato solo quando questo "ElementTemplate" e' stato generato
	# per gestire la relazione con un template specificato di un TSubElement.
	telementappliedtemplate = models.ForeignKey(TElementAppliedTemplate, null=True, on_delete=models.CASCADE)

	class Meta:
		db_table = "c_element_template"	

@classmethod
def filter_by_template(cls, template_name):

	objects = cls.objects

	objects = objects.extra(
		tables = ['c_element_template', 'c_telement'],
		where  = [
			"monitoredelement.uuid          = c_element_template.melement_id",
			"c_element_template.telement_id = c_telement.id",
			"c_telement.name                = %s",
		],
		params=[ template_name]
	)

	return objects

MonitoredElement.add_to_class('filter_by_template', filter_by_template)

def _get_used_templates(self):
	return [ et.telement for et in ElementTemplate.objects.filter(tenant=self.tenant, melement=self) ]

MonitoredElement.add_to_class('get_used_templates', _get_used_templates)

#-----------------------------------------------------------------------------#
# Specializzazioni                                                            #
#-----------------------------------------------------------------------------#


class IPAddressFieldNullable(IPAddressField) :
	def get_db_prep_save(self, value, connection):
		return value or None

class MonitoredNode(MonitoredElement):

	use_ipv4         = models.BooleanField(null=False, default=True, verbose_name="IPv4")

	dns_name         = models.TextField(null=True, blank=True, default=None, verbose_name="DNS name (A)")
	dns6_name        = models.TextField(null=True, blank=True, default=None, verbose_name="DNS v6 name (AAAA)")

	ip4_mgt          = IPAddressFieldNullable(null=True, blank=True, default=None, verbose_name="IPv4 address")
	ip6_mgt          = models.TextField     (null=True, blank=True, default=None, verbose_name="IPv6 address")

	timezone         = models.TextField(null=True, default=None, verbose_name="Timezone") 

	snmp_version     = models.PositiveSmallIntegerField(default=DEFAULT_SNMP_VERSION, db_index=False, verbose_name="SNMP version")
	snmp_port        = models.PositiveSmallIntegerField(null=False, default=161, db_index=False, verbose_name="SNMP UDP port")
	snmp_community   = models.TextField(null=False, blank=True, default='public', verbose_name="SNMP Community")
	snmp_v3_authdata = models.TextField(null=True , blank=True, default=None)

	dft_timeout      = models.PositiveSmallIntegerField(null=False, default=1, db_index=False)

	agent_runningon  = models.BooleanField(null=False, default=False)

	is_switch        = models.BooleanField(null=False, default=False, verbose_name="Switch capabilities")
	is_router        = models.BooleanField(null=False, default=False, verbose_name="Router capabilities")

	# Campo di comodo per memorizzare l'indirizzo IP aggiornato effettivamente usato da sanet per il monitoraggio
	effective_ip     = models.TextField     (null=True, blank=True, default=None, verbose_name="Effective IP")

	cached_fields = MonitoredElement.cached_fields + [
	]

	def __str__(self):
		return "<Node:%s>" % (self.name)

	def get_icon_name_or_default(self):
		if   self.icon     : return self.icon
		elif self.is_router: return 'router'
		elif self.is_switch: return 'switch'
		return _DEFAULT_MONITOREDELEMENT_ICONS[self.kind]

	def validate_fields(self):

		super(MonitoredNode, self).validate_fields()

		if self.ip4_mgt is not None:
			ASSERT_VALID_IPv4(self.ip4_mgt)

		# SNMP communty may contain
		if self.snmp_version == 3:

			if self.snmp_v3_authdata:
				# This will raise exception on errors
				parse_snmpv3_auth_options(self.snmp_v3_authdata)

			# COMPATIBILITY MODE 
			if self.snmp_community:
				# This will raise exception on errors
				parse_snmpv3_auth_options(self.snmp_community)

		return True

	def get_title_symbols_dict(self, prefix=''):
		d = {
			'%snode'        % (prefix) : self.name,
			'%ssnmpversion' % (prefix) : self.snmp_version     ,
			'%scommunity'   % (prefix) : self.snmp_community   ,
		}
		return d

	def get_dict(self):
		d = {}
		d.update( {
			'uuid'                 : self.uuid,
			'tenant_id'            : self.tenant_id,
			'agent_id'             : self.agent_id,
			'name'                 : self.name,
			'kind'                 : self.kind,
			'description'          : self.description,
			'is_abstract'          : self.is_abstract,
			'parent_id'            : self.parent_id,
			'cluster_id'           : self.cluster_id,
			'cluster_role'         : self.cluster_role,
			'cluster_distinguisher': self.cluster_distinguisher,
			'cluster_xform'        : self.cluster_xform,
			'cluster_instance'     : self.cluster_instance,

			'dns_name'         : self.dns_name         ,
			'dns6_name'        : self.dns6_name        ,
			'ip4_mgt'          : self.ip4_mgt          ,
			'ip6_mgt'          : self.ip6_mgt          ,
			'use_ipv4'         : self.use_ipv4         ,
			'timezone'         : self.timezone         ,
			'snmp_version'     : self.snmp_version     ,
			'snmp_community'   : self.snmp_community   ,
			'snmp_v3_authdata' : self.snmp_v3_authdata ,
			'snmp_port'        : self.snmp_port        ,
			'dft_timeout'      : self.dft_timeout      ,
			
			'effective_ip'     : self.effective_ip,
			
			'cluster_instance_time': self.cluster_instance_time, # from cache
			'cluster_element_id'   : self.cluster_element_id   , # from cache
		})
		return d

	@staticmethod
	def load_for_agent(tenant, agent, cache_manager=None):

		cursor = connection.cursor()
		result = cursor.execute("""
		    --
		    -- Tutti i nodi associati all'agente 
		    --
		    SELECT
		           mn.uuid,
		           mn.tenant_id,
		           mn.name,
		           mn.kind,
		           mn.is_abstract,
		           mn.parent_id,
		           mn.cluster_id,
		           mn.cluster_role,
		           mn.cluster_distinguisher,
		           mn.cluster_xform,
		           mn.cluster_instance,
		           --mn.scheduling_group_id,
		           --mn.scheduling_min_gap,
		           --mn.scheduling_max_deadline,
		           --mn.scheduling_max_parallel,
		           n.dns_name,
		           n.dns6_name,
		           n.ip4_mgt,
		           n.ip6_mgt,
		           n.use_ipv4,
		           n.timezone,
		           n.snmp_version,
		           n.snmp_community,
		           n.snmp_v3_authdata,
		           n.snmp_port,
		           n.dft_timeout,
		           n.effective_ip,
		           
		           COALESCE(mn.scheduling_group_id, c.scheduling_group_id) as scheduling_group_id
		           
		      FROM node              n INNER JOIN monitoredelement mn   ON (n.monitoredelement_ptr_id = mn.uuid)
		                               LEFT  JOIN cluster          c    ON (mn.cluster_id             = c.uuid)
		     WHERE mn.agent_id = %s
		     
		     UNION 
		    --
		    -- Tutti i nodi adiacenti che hanno un'interfaccia interessata da un link
		    -- che coinvolge interfacce monitorate dall'agente.
		    --
		    SELECT
		           mn.uuid,
		           mn.tenant_id,
		           mn.name,
		           mn.kind,
		           mn.is_abstract,
		           mn.parent_id,
		           mn.cluster_id,
		           mn.cluster_role,
		           mn.cluster_distinguisher,
		           mn.cluster_xform,
		           mn.cluster_instance,
		           --mn.scheduling_group_id,
		           --mn.scheduling_min_gap,
		           --mn.scheduling_max_deadline,
		           --mn.scheduling_max_parallel,
		           n.dns_name,
		           n.dns6_name,
		           n.ip4_mgt,
		           n.ip6_mgt,
		           n.use_ipv4,
		           n.timezone,
		           n.snmp_version,
		           n.snmp_community,
		           n.snmp_v3_authdata,
		           n.snmp_port,
		           n.dft_timeout,
		           n.effective_ip,
		
		           COALESCE(mn.scheduling_group_id, c.scheduling_group_id) as scheduling_group_id
		
		      FROM node              n INNER JOIN monitoredelement mn   ON (n.monitoredelement_ptr_id = mn.uuid     )
		                               INNER JOIN monitoredelement mi   ON (mn.uuid                   = mi.parent_id)
		                               INNER JOIN link_element     le   ON (mi.uuid                   = le.iface_id )
		                               LEFT  JOIN cluster          c    ON (mn.cluster_id             = c.uuid)
		     WHERE le.link_id IN (
		                          SELECT DISTINCT l.uuid
		                            FROM link l INNER JOIN link_element     le ON (l.uuid      = le.link_id)
		                                        INNER JOIN monitoredelement i  ON (le.iface_id = i.uuid)
		                           WHERE i.agent_id = %s 
		                         )
		""", [agent.uuid, agent.uuid])

		elements = _dictfetchall(cursor)

		if cache_manager:
			for d in elements:
				d.update( MonitoredNode.raw_load_fields_from_cache(cache_manager,  d['uuid'], MonitoredNode.cached_fields ) )

		return elements

	def get_title_symbols_dict(self, prefix=''):
		d = {
			'%snode' % (prefix)        : self.name,
			'%ssnmpversion' % (prefix) : self.snmp_version     ,
			'%scommunity' % (prefix)   : self.snmp_community   ,
		}
		return d

	@property
	def fake_property(self):
		return "ciao"


	def _name_or_ip_for_snmp_request(self):
		if self.use_ipv4:
			if self.ip4_mgt:
				return self.ip4_mgt
			else:
				return self.dns_name  or self.name
		else:
 			if self.ip6_mgt:
				return self.ip6_mgt
			else:
				return self.dns6_name or self.name

	#
	# SNMP Utils
	#

	def snmp_walk(self, oid, **kw):
		return self._snmp_walk(oid, **kw)
	
	
	#
	# LOW LEVEL SNMP 
	#
	def _snmp_params(self):

		# Global params
		params = {
			'Version'  : self.snmp_version,
			'port'     : self.snmp_port,
			'timeout'  : 1,
			#'timeout'  : self.dft_timeout,
		}
		
		if self.snmp_version == 3:
			params.update( self._snmp_v3_params() )
		else:
			params['community'] = self.snmp_community

		return params

	def _snmp_v3_params(self):
		if self.snmp_version != 3:
			return {}

		try:
			comm_or_auth = self.snmp_v3_authdata or self.snmp_community  # For compatibility
			return _get_snmpv3_auth_options_from_string(comm_or_auth)
		except Exception, e:
			print __file__, "ASSERTION FAILED: INVALID VALUE FOR SNMPv3 auth data. Node:", self.name
			return {}

	def _snmp_get(self, oid, **kw):

		params = self._snmp_params()
		for k in kw.keys(): params[k] = kw[k]

		ip_or_name = self._name_or_ip_for_snmp_request()

		if not self.agent.is_remote:
			try:
				return snmp_get(oid, ip_or_name, **params)
			except SnmpWrapperException, e:
				pass
		else:
			try:
				params['oid'] = oid
				params['host'] = ip_or_name
				return self._remote_request(NetInfoAgentHelper.REQUEST_NETINFO_SNMP_GET, params)
			except Exception, e:
				pass

		return None

	def _snmp_walk(self, oid, **kw):

		params = self._snmp_params()
		for k in kw.keys(): params[k] = kw[k]

		ip_or_name = self._name_or_ip_for_snmp_request()

		if self.agent is None or not self.agent.is_remote:
			try:
				return snmp_walk(oid, ip_or_name, **params)
			except SnmpWrapperException, e:
				pass
		else:
			try:
				params['oid'] = oid
				params['host'] = ip_or_name
				return self._remote_request(NetInfoAgentHelper.REQUEST_NETINFO_SNMP_WALK, params)
			except Exception, e:
				pass

		return None

	def _remote_request(self, cmd, params_dict, timeout=3):
		broker  = get_broker_connection_string()
		agent   = self.agent

		data = agent_sendreceive(broker, agent.uuid, AGENT_NETINFOHELPER, cmd, [params_dict,], timeout=timeout)

		return data

	#
	# SNMP UTILS
	#
	def get_sysdescr(self, cache_manager=None):
		return self.__get_snmp_cached('1.3.6.1.2.1.1.1.0', cache_manager, permanent=True)

	def get_sysname(self, cache_manager=None):
		return self.__get_snmp_cached('1.3.6.1.2.1.1.5.0', cache_manager)

	def get_syslocation(self, cache_manager=None):
		return self.__get_snmp_cached('1.3.6.1.2.1.1.6.0', cache_manager, permanent=True)

	def __get_snmp_cached(self, oid, cache_manager=None, permanent=False, live_time=60):

		cache_manager = cache_manager or self.get_cache_manager()
		if cache_manager:
			key  = cache_manager.create_key( 'snmp_cache', self.tenant_id, 'node', self.uuid, oid)

			if permanent:
				keyp = cache_manager.create_key( 'snmp_cache', self.tenant_id, 'node', self.uuid, oid,'permanent')

			rv = cache_manager.get(key)

			if rv is None and permanent:
				rv = cache_manager.get(keyp)

			if rv != None:
				return rv

		rv = self._snmp_get(oid)

		if cache_manager:
			if permanent:
				keyp = cache_manager.create_key( 'snmp_cache', self.tenant_id, 'node', self.uuid, oid,'permanent')

				if rv is None:
					# ***This is a very useful field for the NOC.***
					# The cache is expired but we can not retrieve the data
					# because the node is probably down.
					# Just use the cached value.
					return cache_manager.get(keyp)
				else:
					cache_manager.set(keyp, rv)

			cache_manager.set(key, rv, live_time)

		return rv




	#
	# Name resolution utilities
	#
	@property
	def ipv4_addresses(self, cache_manager=None):
		return self.get_ipv4_addresses(cache_manager=cache_manager)

	def get_ipv4_addresses(self, cache_manager=None, forced_name=None):

		cache_manager = cache_manager or self.get_cache_manager()
		if cache_manager:
			key = cache_manager.create_key( 'snmp_cache', self.tenant_id, 'node', self.uuid, 'ipv4_addresses')

			rv = cache_manager.get(key)
			if rv != None:
				return rv

		if not self.agent.is_remote:
			try:
				rv = resolve_ipv4(forced_name or self.name)
			except Exception, e:
				rv = None

			if rv == None:
				return None
		else:
			try:
				params = {
					'hostname': forced_name or self.name 
				}
				rv = self._remote_request(NetInfoAgentHelper.REQUEST_NETINFO_RESOLVE_IPv4, params)
			except Exception, e:
				rv = None

		if cache_manager:
			cache_manager.set(key, rv, 60)

		return rv

	@property
	def ipv4_addresses_spaced(self):
		ips = self.ipv4_addresses
		if ips:
			return ' '.join(ips)
		return ''

	@property
	def effective_name_and_address(self):
		effective_name = self.name
		effective_ip   = self.effective_ip

		#if self.use_ipv4:
		#	if self.ip4_mgt:
		#		effective_ip = self.ip4_mgt
		#	else:
		#		effective_name = self.dns_name  or self.name
		#		ips = self.get_ipv4_addresses(forced_name=effective_name)
		#		if ips:
		#			effective_ip = ips[0]
		#else:
		#	if self.ip6_mgt:
		#		effective_ip = self.ip6_mgt
		#	else:
		#		effective_name = self.dns6_name or self.name

		return effective_name, effective_ip

	@property
	def main_ip(self):
		ips = self.ipv4_addresses
		if ips:
			return ips[0]
		return None

	#
	# Sub element utils
	#
	def get_ifaces(self):
		return self.get_sub_elements(kind=ELEMENT_KIND_IFACE)

	def get_storages(self):
		return self.get_sub_elements(kind=ELEMENT_KIND_STORAGE)

	def get_links_path(self):
		"""
		FIX: QUESTO METODO E' MALEDETTAMENTE INEFFICIENTE
		"""
		r = []
		
		#ifaces = MonitoredIface.objects.filter(parent=self)
		#ifaces = ifaces.all()
		#
		#lelements = LinkElement.objects.filter(iface__in=ifaces)
		#
		#links = dict([ (l.uuid, l) for l in Link.objects.filter(uuid__in=[le.link_id for le in lelements]) ])
		#
		#linked_elements = LinkElement.objects.filter(link__in=links).exclude(id__in=[le.link_id for le in lelements])
		#
		#linked_ifaces   = dict([ (i.uuid, i) for i in MonitoredIface.objects.filter(uuid__in=[ l.iface_id for l in linked_elements ]) ])
		#
		#linked_nodes    = dict([ (n.uuid, n) for n in MonitoredNode.objects.filter(uuid__in=[ i.parent_id for i in linked_ifaces ])   ])
                #
		##for i in self.get_ifaces():
		##	i = i.get_specific_element()
                #
		#for i in ifaces:
		#	for (link, linked_i) in i.get_linked_ifaces():
		#		#r.append(  (i, link, linked_i, linked_i.get_node() ) )
		#		
		#		l  = links.get()
		#		li = linked_iface.get()
		#		ln = linked_nodes.get()
		#		
		#		r.append(  (i, l, li, ln ) )

		QUERY = """
			select i.uuid, li.link_id, i2.uuid, n2.uuid
			  from monitoredelement n inner join monitoredelement i   on (n.uuid       = i.parent_id)
						  inner join link_element     li  on (i.uuid       = li.iface_id)
						  inner join link_element     li2 on (li.link_id   = li2.link_id  and li.iface_id <> li2.iface_id)
						  inner join monitoredelement i2  on (li2.iface_id = i2.uuid)
						  inner join monitoredelement n2  on (i2.parent_id = n2.uuid)
			 where n.uuid = %s
		"""
		cursor = connection.cursor()
		cursor.execute(QUERY,[self.uuid])
		
		data = cursor.fetchall()
		
		ifaces_by_uuid = dict( [ (e.uuid, e) for e in MonitoredIface.objects.filter(uuid__in=[ row[0] for row in data ] + [ row[2] for row in data ]) ] )
		links_by_uuid  = dict( [ (e.uuid, e) for e in Link          .objects.filter(uuid__in=[ row[1] for row in data ]) ] )
		nodes_by_uuid  = dict( [ (e.uuid, e) for e in MonitoredNode .objects.filter(uuid__in=[ row[3] for row in data ]) ] )
		
		for row in data:
			i  = ifaces_by_uuid[ row[0] ]
			l  = links_by_uuid [ row[1] ]
			li = ifaces_by_uuid[ row[2] ]
			ln = nodes_by_uuid [ row[3] ]
			
			r.append( (i, l, li, ln) )

		return r 

	def get_linked_nodes(self):
		#  node1 -> iface1 -> link_iface1 <-link-> link_iface2 -> iface2 -> node2
		nodes = MonitoredElement.objects.raw("""
			select n2.*
			  from monitoredelement n inner join monitoredelement i   on (n.uuid       = i.parent_id)
						  inner join link_element     li  on (i.uuid       = li.iface_id)
						  inner join link_element     li2 on (li.link_id   = li2.link_id  and li.iface_id <> li2.iface_id)
						  inner join monitoredelement i2  on (li2.iface_id = i2.uuid)
						  inner join monitoredelement n2  on (i2.parent_id = n2.uuid)
			 where n.uuid = %s
			 order by n2.name
		""", [self.uuid])

		return list(nodes)

	def get_infos(self):
		# monitored element info
		infos = []
		# label, value or list of value, type
		infos.append(['Description'         , self.description          , 'text'])
		infos.append(['IPv4 addresses (dns)', self.ipv4_addresses_spaced, 'text'])
		infos.append(['sysdescr (snmp)'     , self.get_sysdescr()       , 'text'])
		infos.append(['sysname (snmp)'      , self.get_sysname()        , 'text'])
		infos.append(['syslocation (snmp)'  , self.get_syslocation()    , 'text'])
		return infos

	@staticmethod
	def get_from_sourceip(sourceip):
		try:
			sanet_node = MonitoredNode.objects.get(ip4_mgt=sourceip)
			return sanet_node
		except MonitoredNode.DoesNotExist, e:
			return None

	def hide_sensible_data(self):
		"""
		Ticket #545
		"""
		super(MonitoredNode, self).hide_sensible_data()
		self.snmp_community   = settings.HIDDEN_DATA
		self.snmp_v3_authdata = settings.HIDDEN_DATA


	#
	# For wildcards {{ node.<field> }} in messages
	#
	def sysname(self):
		# The sanetd server should have used the set_cache_manager()
		cache_manager = self.get_cache_manager()

		return self.get_sysname(cache_manager)

	def syslocation(self):
		# The sanetd server should have used the set_cache_manager()
		cache_manager = self.get_cache_manager()

		return self.get_syslocation(cache_manager)

	def syscontact(self):
		# The sanetd server should have used the set_cache_manager()
		cache_manager = self.get_cache_manager()

		return self.__get_snmp_cached('1.3.6.1.2.1.1.4.0', cache_manager)

	def sysuptime(self):
		value = self._snmp_get('1.3.6.1.2.1.1.3.0') # IMPORTANT: No cache for sysuptime!!

		seconds = int(value) / 100                     # The time (in hundredths of a second) since the network management portion of the system was last re-initialized.

		s = str(datetime.timedelta(seconds = seconds)) # Converti in format "<n> days, <h>:<m>:<s>"

		return s



	class Meta:
		db_table = "node"
		ordering= ['name']

#-----------------------------------------------------------------------------#

class MonitoredIface(MonitoredElement):

	distinguisher = models.TextField(null=True, db_index=True)
	xform         = models.TextField(null=True, db_index=False)
	ifindex       = models.TextField(null=True, default=None, db_index=True)

	#
	# Campi informativi
	#

	# Uplink/backbone flag.
	backbone      = models.BooleanField(null=False, default=False, db_index=True)

	# Interface speed in bit/s.
	speed         = models.BigIntegerField(null=True, default=None, db_index=True)

	cached_fields = MonitoredElement.cached_fields + [
		'ifindex_time',
	]

	def __str__(self):
		return "<Iface:%s>" % (self.name)

	def get_scaled_speed(self):
		return DataSource.calculate_scaled_value(self.speed)

	def get_link(self):
		l = self.links.all()
		if l:
			return l[0].link
		return None

	def get_title_symbols_dict(self, prefix='', exclude_linked_elements=False):

		r = {}

		node = self.get_node()
		dn = node.get_title_symbols_dict(prefix=prefix)
		r.update(dn)

		di = {
			'%siface'    % (prefix): str(self.name),
			'%sinstance' % (prefix): str(self.distinguisher),
			'%sifindex'  % (prefix): str(self.ifindex),
		}
		r.update(di)

		if not exclude_linked_elements:
			linked_iface = self.get_linked_iface()
			if linked_iface:
				ldi = linked_iface.get_title_symbols_dict(prefix='linked_', exclude_linked_elements=True)
				r.update(ldi)
		return r

	def get_linked_iface(self):

		# Troviamo tutti i link L2 dove compare questa interfaccia
		# e cerchiamo l'interfaccia corrispondente sull'altro lato.
		# FIX: inefficiente, implementare ricerca piu' veloce
		linked_elements = self.links.filter(link__ltype=LINK_TYPE_L2).all()
		for le  in linked_elements:
			link = le.link
			linked_element = le.link.ifaces.exclude(iface=self).all()[0]

			# FIX: workaround per schivare i link "rotti". Quando avremo fixato questo problema toglieremo anche questo hack
			linked_element = le.link.ifaces.exclude(iface=self).all()[0:1]
			if len(linked_element) == 1:
				return linked_element[0].iface

		return None

	def get_dict(self):
		d = {}
		d.update( {
			'uuid'                 : self.uuid,
			'name'                 : self.name,
			'kind'                 : self.kind,
			'description'          : self.description,
			'is_abstract'          : self.is_abstract,
			'parent_id'            : self.parent_id,
			'cluster_id'           : self.cluster_id,
			'cluster_role'         : self.cluster_role,
			'cluster_distinguisher': self.cluster_distinguisher,
			'cluster_xform'        : self.cluster_xform,
			'cluster_instance'     : self.cluster_distinguisher,

			'distinguisher': self.distinguisher,
			'xform'        : self.xform,
			'ifindex'      : self.ifindex,
			'backbone'     : self.backbone,
			'speed'        : self.speed,
		})
		return d

	@staticmethod
	def load_for_agent(tenant, agent, cache_manager=None):

		cursor = connection.cursor()

		result = cursor.execute("""
		    --
		    -- Tutte le interfacce monitorate dall'agente
		    --
		    SELECT 
		           mi.uuid, 
		           mi.name, 
		           mi.kind, 
		           mi.description, 
		           mi.is_abstract, 
		           mi.parent_id, 
		           mi.cluster_id, 
		           mi.cluster_role, 
		           mi.cluster_distinguisher, 
		           mi.cluster_xform, 
		           mi.cluster_instance,
		           --mi.scheduling_group_id,
		           --mi.scheduling_min_gap,
		           --mi.scheduling_max_deadline,
		           --mi.scheduling_max_parallel,
		           i.distinguisher,
		           i.xform, 
		           i.ifindex,
		           i.backbone,
		           i.speed,
		           COALESCE(mi.scheduling_group_id, n.scheduling_group_id, c.scheduling_group_id) as scheduling_group_id
		      FROM iface              i INNER JOIN monitoredelement mi   ON (i.monitoredelement_ptr_id = mi.uuid)
		                                INNER JOIN monitoredelement n    ON (mi.parent_id              = n.uuid)
		                                LEFT  JOIN cluster          c    ON (n.cluster_id              = c.uuid)
		     WHERE mi.agent_id = %s
		     
		     UNION 
		     
		    --
		    -- Tutte le interfacce adiacenti collegate a interfacce monitrate dall'agente
		    --
		    SELECT
		           mi.uuid, 
		           mi.name, 
		           mi.kind, 
		           mi.description, 
		           mi.is_abstract, 
		           mi.parent_id, 
		           mi.cluster_id, 
		           mi.cluster_role, 
		           mi.cluster_distinguisher, 
		           mi.cluster_xform, 
		           mi.cluster_instance,
		           --mi.scheduling_group_id,
		           --mi.scheduling_min_gap,
		           --mi.scheduling_max_deadline,
		           --mi.scheduling_max_parallel,
		           i.distinguisher, 
		           i.xform, 
		           i.ifindex,
		           i.backbone,
		           i.speed,
		           COALESCE(mi.scheduling_group_id, n.scheduling_group_id, c.scheduling_group_id) as scheduling_group_id
		      FROM iface              i INNER JOIN monitoredelement mi   ON (i.monitoredelement_ptr_id = mi.uuid    )
		                                INNER JOIN link_element     le   ON (mi.uuid                   = le.iface_id)                         
		                                INNER JOIN monitoredelement n    ON (mi.parent_id              = n.uuid)
		                                LEFT  JOIN cluster          c    ON (n.cluster_id              = c.uuid)
		     WHERE le.link_id IN (
		                          SELECT DISTINCT l.uuid
		                            FROM link l INNER JOIN link_element     le ON (l.uuid      = le.link_id)
		                                        INNER JOIN monitoredelement i  ON (le.iface_id = i.uuid    )
		                           WHERE i.agent_id = %s 
		                         )
		""", [agent.uuid, agent.uuid])

		data = _dictfetchall(cursor)

		return data

	def get_linked_ifaces(self):
		r = []
		linked_elements = self.links.select_related('link').all()
		for le  in linked_elements:
			link = le.link
			linked_element = le.link.ifaces.exclude(iface=self).all()[0]
			if linked_element.iface == None:
				raise Exception("BUG: linked element without iface ")

			r.append( (link, linked_element.iface) )

		return r

	def get_infos(self):
		# monitored element info
		infos = []

		# label, value or list of value, type
		infos.append(['description', self.description,    'text'])
		return infos

	@property
	def description_snmp(self):
		return self.get_verbose_snmp_description()

	def get_verbose_snmp_description(self, cache_manager=None):

		if not self.ifindex:
			return None

		cache_manager = cache_manager or self.get_cache_manager()
		if cache_manager:
			key  = cache_manager.create_key( 'snmp_cache', self.tenant_id, 'iface', self.uuid, 'snmp_description')
			keyp = cache_manager.create_key( 'snmp_cache', self.tenant_id, 'iface', self.uuid, 'snmp_description','permanent')

			rv = cache_manager.get(key)
			if rv != None:
				return rv

		rv = self._get_verbose_snmp_description()

		if cache_manager:
			if rv == None:
				# ***This is a very useful field for the NOC.***
				# The cache is expired but we can not retrieve the data
				# because the node is probably down.
				# Just use the cached value.
				return cache_manager.get(keyp)
			else:
				cache_manager.set(key, rv, 60)
				cache_manager.set(keyp, rv)
		return rv

	def _get_verbose_snmp_description(self):

		rv = ''

		node = self.get_node()

		OIDS = [
			'1.3.6.1.2.1.2.2.1.2.%s',
			'1.3.6.1.2.1.31.1.1.1.1.%s',
			'1.3.6.1.2.1.31.1.1.1.18.%s',
			'1.3.6.1.4.1.9.2.2.1.1.28.%s',
		]

		descriptions=[]
		for oidt in OIDS:
			d = node._snmp_get(oidt % self.ifindex)
			if d and not d in descriptions:
				descriptions.append(d)
		if descriptions:
			rv = ' '.join(descriptions)
		return rv

	class Meta:
		db_table = "iface"
		ordering= ['name']

#-----------------------------------------------------------------------------#

class MonitoredStorage(MonitoredElement):

	distinguisher = models.TextField(null=True, blank=True, db_index=True)
	xform         = models.TextField(null=True, blank=True, db_index=False)
	#stindex       = models.PositiveIntegerField(null=True, blank=True, default=None, db_index=True) # hrStorageIndex) 
	stindex       = models.TextField(null=True, blank=True, default=None, db_index=True) # hrStorageIndex) 

	cached_fields = MonitoredElement.cached_fields + [
		'stindex_time',
	]

	def __str__(self):
		return "<Storage:%s>" % (self.name)

	def get_dict(self):
		d = {}
		d.update( {
			'uuid'                 : self.uuid,
			'name'                 : self.name,
			'kind'                 : self.kind,
			'description'          : self.description,
			'is_abstract'          : self.is_abstract,
			'parent_id'            : self.parent_id,
			'cluster_id'           : self.cluster_id,
			'cluster_role'         : self.cluster_role,
			'cluster_distinguisher': self.cluster_distinguisher,
			'cluster_xform'        : self.cluster_xform,
			'cluster_instance'     : self.cluster_distinguisher,

			'distinguisher': self.distinguisher,
			'xform'        : self.xform,
			'stindex'      : self.stindex,
		})
		return d

	@staticmethod
	def load_for_agent(tenant, agent, cache_manager=None):

		#elements = MonitoredStorage.objects.filter(tenant=tenant, agent=agent)
		#elements = elements.values(
		#	 'uuid'                 
		#	,'name'                 
		#	,'kind'                 
		#	,'description'          
		#	,'is_abstract'          
		#	,'parent_id'            
		#	,'cluster_id'           
		#	,'cluster_role'         
		#	,'cluster_distinguisher'
		#	,'cluster_xform'        
		#	,'cluster_instance'     
		#	
		#	,'distinguisher'
		#	,'xform'
		#	,'stindex'
                #
		#	,'scheduling_group_id'
		#	#,'scheduling_min_gap'
		#	#,'scheduling_max_deadline'
		#	#,'scheduling_max_parallel'
		#)
		
		
		cursor = connection.cursor()

		result = cursor.execute("""
		    SELECT 
		           ms.uuid, 
		           ms.name, 
		           ms.kind, 
		           ms.description, 
		           ms.is_abstract, 
		           ms.parent_id, 
		           ms.cluster_id, 
		           ms.cluster_role, 
		           ms.cluster_distinguisher, 
		           ms.cluster_xform, 
		           ms.cluster_instance,

		           s.distinguisher,
		           s.xform, 
		           s.stindex,

		           COALESCE(ms.scheduling_group_id, n.scheduling_group_id, c.scheduling_group_id) as scheduling_group_id
		      FROM storage            s INNER JOIN monitoredelement ms   ON (s.monitoredelement_ptr_id = ms.uuid)
		                                INNER JOIN monitoredelement n    ON (ms.parent_id              = n.uuid)
		                                LEFT  JOIN cluster          c    ON (n.cluster_id              = c.uuid)
		     WHERE ms.agent_id = %s
		""", [agent.uuid])

		elements = _dictfetchall(cursor)

		return elements

	def get_state(self):
		return super(MonitoredStorage, self).get_state()

	def get_infos(self):
		# monitored element info
		infos = []
		# label, value or list of value, type
		infos.append(['description',  self.description,    'text'])
		infos.append(['distinguisher', self.distinguisher, 'text'])
		infos.append(['xform',         self.xform,         'text'])
		return infos

	def get_title_symbols_dict(self, prefix=''):

		r = {}

		node = self.get_node()
		r.update(node.get_title_symbols_dict(prefix=prefix))

		r.update( {
			'%sstorage'       % (prefix): str(self.name),
			'%sdistinguisher' % (prefix): str(self.distinguisher),
			'%sstindex'       % (prefix): str(self.stindex),
		})

		return r

	class Meta:
		db_table = "storage"
		ordering = ['name']

#-----------------------------------------------------------------------------#

class MonitoredService(MonitoredElement):

	distinguisher = models.TextField(null=True, blank=True, default=None, db_index=True)
	xform         = models.TextField(null=True, blank=True, default=None, db_index=False)
	#swrunindex    = models.PositiveIntegerField(null=True, blank=True, default=None, db_index=True) # hrSWRunIndex
	swrunindex    = models.TextField(null=True, blank=True, default=None, db_index=True) # hrSWRunIndex

	cached_fields = MonitoredElement.cached_fields + [
		'swrunindex_time',
	]

	def __str__(self):
		return "<Service:%s>" % (self.name)

	def get_dict(self):
		d = {}
		d.update( {
			'uuid'                 : self.uuid,
			'name'                 : self.name,
			'kind'                 : self.kind,
			'description'          : self.description,
			'is_abstract'          : self.is_abstract,
			'parent_id'            : self.parent_id,
			'cluster_id'           : self.cluster_id,
			'cluster_role'         : self.cluster_role,
			'cluster_distinguisher': self.cluster_distinguisher,
			'cluster_xform'        : self.cluster_xform,
			'cluster_instance'     : self.cluster_distinguisher,

			'distinguisher'        : self.distinguisher,
			'xform'                : self.xform,
			'swrunindex'           : self.swrunindex,
		})
		return d
		
	@staticmethod
	def load_for_agent(tenant, agent, cache_manager=None):
		
		#elements = MonitoredService.objects.filter(tenant=tenant, agent=agent).values(
		#	 'uuid'                 
		#	,'name'                 
		#	,'kind'                 
		#	,'description'          
		#	,'is_abstract'          
		#	,'parent_id'            
		#	,'cluster_id'           
		#	,'cluster_role'         
		#	,'cluster_distinguisher'
		#	,'cluster_xform'        
		#	,'cluster_instance'     
		#	
		#	,'distinguisher'
		#	,'xform'
		#	,'swrunindex'
                #
		#	,'scheduling_group_id'
		#	#,'scheduling_min_gap'
		#	#,'scheduling_max_deadline'
		#	#,'scheduling_max_parallel'
		#)
                #
		##for e in elements:
		##	e.update( Monitored.raw_load_fields_from_cache(cache_manager,  d['uuid'], MonitoredNode.cached_fields ) )


		cursor = connection.cursor()

		result = cursor.execute("""
		    SELECT 
		           ms.uuid, 
		           ms.name, 
		           ms.kind, 
		           ms.description, 
		           ms.is_abstract, 
		           ms.parent_id, 
		           ms.cluster_id, 
		           ms.cluster_role, 
		           ms.cluster_distinguisher, 
		           ms.cluster_xform, 
		           ms.cluster_instance,
		
		           s.distinguisher,
		           s.xform, 
		           s.swrunindex,
		
		           COALESCE(ms.scheduling_group_id, n.scheduling_group_id, c.scheduling_group_id) as scheduling_group_id
		      FROM service            s INNER JOIN monitoredelement ms   ON (s.monitoredelement_ptr_id = ms.uuid)
		                                INNER JOIN monitoredelement n    ON (ms.parent_id              = n.uuid)
		                                LEFT  JOIN cluster          c    ON (n.cluster_id              = c.uuid)
		     WHERE ms.agent_id = %s
		""", [agent.uuid])

		elements = _dictfetchall(cursor)

		return elements


	def get_title_symbols_dict(self, prefix=''):

		r = {}

		node = self.get_node()
		r.update(node.get_title_symbols_dict(prefix=prefix))

		r.update( {
			'%sservice'       % (prefix): str(self.name),
			'%sdistinguisher' % (prefix): str(self.distinguisher),
			'%sswrunindex'       % (prefix): str(self.swrunindex),
		})

		return r
		
	def get_infos(self):
		# monitored element info
		infos = []
		# label, value or list of value, type
		infos.append(['description',  self.description,    'text'])
		infos.append(['distinguisher', self.distinguisher, 'text'])
		infos.append(['xform',         self.xform,         'text'])
		return infos

	class Meta:
		db_table = "service"

#-----------------------------------------------------------------------------#

class MonitoredDevice(MonitoredElement):

	distinguisher = models.TextField(null=True, default=None, db_index=True)
	xform         = models.TextField(null=True, default=None, db_index=False)
	devindex      = models.TextField(null=True, default=None, db_index=True)

	cached_fields = MonitoredElement.cached_fields + [
		'devindex_time',
	]

	def __str__(self):
		return "<Device:%s>" % (self.name)

	def get_state(self):
		return super(MonitoredDevice, self).get_state()

	def get_title_symbols_dict(self, prefix=''):

		r = {}
		
		node = self.get_node()
		r.update(node.get_title_symbols_dict(prefix=prefix))

		r.update( {
			'%sdevice'         % (prefix): str(self.name),
			'%sdistinguisher'  % (prefix): str(self.distinguisher),
			'%sdevindex'       % (prefix): str(self.devindex),
		})
		return r

	def get_dict(self):
		d = {}
		d.update( {
			'uuid'                 : self.uuid,
			'name'                 : self.name,
			'kind'                 : self.kind,
			'description'          : self.description,
			'is_abstract'          : self.is_abstract,
			'parent_id'            : self.parent_id,
			'cluster_id'           : self.cluster_id,
			'cluster_role'         : self.cluster_role,
			'cluster_distinguisher': self.cluster_distinguisher,
			'cluster_xform'        : self.cluster_xform,
			'cluster_instance'     : self.cluster_distinguisher,

			'distinguisher'        : self.distinguisher,
			'xform'                : self.xform,
			'devindex'             : self.devindex,
		})
		return d

	@staticmethod
	def load_for_agent(tenant, agent, cache_manager=None):
		
		#elements = MonitoredDevice.objects.filter(tenant=tenant, agent=agent).values(
		#	 'uuid'                 
		#	,'name'                 
		#	,'kind'                 
		#	,'description'          
		#	,'is_abstract'          
		#	,'parent_id'            
		#	,'cluster_id'           
		#	,'cluster_role'         
		#	,'cluster_distinguisher'
		#	,'cluster_xform'        
		#	,'cluster_instance'     
		#	
		#	,'distinguisher'
		#	,'xform'
		#	,'devindex'
		#
		#	,'scheduling_group_id'
		#	#,'scheduling_min_gap'
		#	#,'scheduling_max_deadline'
		#	#,'scheduling_max_parallel'
		#)
		#
		##for e in elements:
		##	e.update( Monitored.raw_load_fields_from_cache(cache_manager,  d['uuid'], MonitoredNode.cached_fields ) )

		cursor = connection.cursor()

		result = cursor.execute("""
		    SELECT 
		           md.uuid, 
		           md.name, 
		           md.kind, 
		           md.description, 
		           md.is_abstract, 
		           md.parent_id, 
		           md.cluster_id, 
		           md.cluster_role, 
		           md.cluster_distinguisher, 
		           md.cluster_xform, 
		           md.cluster_instance,
		
		           d.distinguisher,
		           d.xform, 
		           d.devindex,
		
		           COALESCE(md.scheduling_group_id, n.scheduling_group_id, c.scheduling_group_id) as scheduling_group_id
		      FROM device            d INNER JOIN monitoredelement md   ON (d.monitoredelement_ptr_id = md.uuid)
		                               INNER JOIN monitoredelement n    ON (md.parent_id              = n.uuid)
		                               LEFT  JOIN cluster          c    ON (n.cluster_id              = c.uuid)
		     WHERE md.agent_id = %s
		""", [agent.uuid])
		
		elements = _dictfetchall(cursor)
		
		return elements



	def get_infos(self):
		# monitored element info
		infos = []
		# label, value or list of value, type
		infos.append(['description',  self.description,    'text'])
		infos.append(['distinguisher', self.distinguisher, 'text'])
		infos.append(['xform',         self.xform,         'text'])
		return infos

	class Meta:
		db_table = "device"
		ordering= ['name']

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

#
# Dizionario di comodo per istanziare velocemente il giusto queryset in base al kind di un elemento.
#
ELEMENT_MODEL_BY_KIND = {
	ELEMENT_KIND_NODE   : MonitoredNode,
	ELEMENT_KIND_IFACE  : MonitoredIface,
	ELEMENT_KIND_STORAGE: MonitoredStorage,
	ELEMENT_KIND_SERVICE: MonitoredService,
	ELEMENT_KIND_DEVICE : MonitoredDevice,
}

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class Link(TenantDependentModel, UUIDKeyModel, MTimeModel):
	# tenant     =
	# uuid       =
	# mtime      =
	name         = models.TextField(null=False, blank=False, unique=True)
	description  = models.TextField(null=False, blank=True, default='')
	ltype        = models.PositiveIntegerField(null=False, default=LINK_TYPE_L2)

	def __str__(self):
		return "<%s>" % (self.name)

	@staticmethod
	def default_link_name(t, n1, i1, n2, i2):
		"""
		Poiche' nei link l'ordine delle interfacce non ha senso, per
		creare un nome "stabile" nel tempo creiamo dei nomi 
		indicando per primo il nodo quello che viene prima alfabeticamente.
		"""
		if n1.name <= n2.name:
			return Link.compose_link_name(t, n1, i1, n2, i2)
		else:
			return Link.compose_link_name(t, n2, i2, n1, i1)

	@staticmethod
	def compose_link_name(t, n1, i1, n2, i2):
		"""
		Il nome di un link e' nella forma:
		
			<tipo>-<node>:<iface>-<node>:<iface>
		"""
		return '%s-%s:%s-%s:%s' % (t, n1.name, i1.name, n2.name, i2.name)


	def get_dict(self):
		d = {
			'uuid'       : self.uuid,
			'mtime'      : self.mtime,
			'name'       : self.description,
			'ltype'      : self.ltype,
			'description': self.description,
		}
		ifaces = []
		if hasattr(self, 'iface1'): ifaces.append(self.iface1.uuid)
		if hasattr(self, 'iface2'): ifaces.append(self.iface2.uuid)
		d['ifaces'] = ifaces
		return d

	class Meta:
		unique_together = (
			"tenant", 
			"name",
		)
		db_table = "link"

class LinkElement(TenantDependentModel, MTimeModel):

	# tenant     =
	# mtime      =
	iface = models.ForeignKey(MonitoredIface, related_name="links")
	link  = models.ForeignKey(Link, related_name="ifaces")

	class Meta:
		unique_together = ("link", "iface")
		db_table = "link_element"

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class Parameter(models.Model):

	name    = models.TextField                (null=False, blank=False, db_index=True)
	vtype   = models.PositiveSmallIntegerField(null=False, blank=False, default=PARAM_TYPE_STR)
	value   = models.TextField                (null=False, blank=True , default='', db_index=False)

	def __str__(self):
		return "<%s:%s:%s>" % (self.name, self.vtype, self.value)

	def get_value(self):
		return  convert_to_param_type(self.vtype, self.value)

	class Meta:
		abstract = True

#-----------------------------------------------------------------------------#
# DATAGROUPS                                                                  #
#-----------------------------------------------------------------------------#

class ActiveDataGroupFilterQuerySet(QuerySet):
	def active(self):
		return self.exclude(deleted=True)

class ActiveDataGroupFilterManager(models.Manager):
	def get_query_set(self):
		return ActiveDataGroupFilterQuerySet(self.model).active()

class ActiveFilterQuerySet(QuerySet):
	def active(self):
		return self.exclude(deleted=True).filter(datagroup__deleted=False)

class ActiveFilterManager(models.Manager):
	def get_query_set(self):
		return ActiveFilterQuerySet(self.model).active()


DEP_NOT_CONFIGURED = None

DEP_CONFIGURED_NONE = 'none'

SUSPEND_REASON_NONE  = ''
SUSPEND_RESUME_NEVER = 0

class DataGroup(TenantDependentModel, UUIDKeyModel, MTimeModel, AbstractCachedFieldModel, ModelFieldsUtils, ExpandedTitleModel, CachedUtil, OwnerUserModel, PermissionModel):

	PASSIVE_MATCH_REGEX_PREFIX = 'regex:'

	# tenant
	# uuid
	# mtime

	element             = models.ForeignKey(MonitoredElement, null=False, related_name="datagroups")

	path                = models.TextField(null=False, blank=False, db_index=True)
	name                = models.TextField(null=False, blank=False, db_index=True)

	title               = models.TextField(null=False, blank=True, default='', db_index=True)

	#
	# Editable fields
	#
	dependson           = models.ForeignKey('Condition', null=True, blank=True, default=None, db_index=True, related_name="depending_datagroups", on_delete=models.SET_NULL)

	minperiod           = models.PositiveIntegerField(null=False, blank=False, default=60)
	shorttries          = models.PositiveSmallIntegerField(default=3, null=False, blank=False)
	timeout             = models.FloatField(null=False, blank=False, default=3.0)

	deleted             = models.BooleanField(default=False, db_index=True) # N.B.: if deleted, then suspended must be true, and target won't show up in listings

	# Fields for suspension management:
	suspended           = models.BooleanField        (null=False, default=False               , db_index=True) # if True, operator has suspended the target
	suspend_reason      = models.TextField           (null=False, default=SUSPEND_REASON_NONE , db_index=False)
	suspend_time        = models.PositiveIntegerField(null=True , default=None                , db_index=True)
	suspend_resume_time = models.PositiveIntegerField(null=True , default=None                , db_index=True) 

	classification      = models.TextField(null=False, blank=True, default='', db_index=True)

	# Stringa (pattern) per identificare quale datagroup deve essere essere eseguito quando
	# viene eseguito 
	#   - un controllo passivo
	#   - una trap SNMP
	#
	# Puo' contenere:
	#   - stringa di testo normale 
	#           -> ricerca tramite match esattto
	#   - una stringa con sintassi  "regexpr:....." 
	#          -> ricerca tramite match della regular expression indicata.
	# 
	passive_match       = models.TextField(null=False, blank=True, default='', db_index=True)

	# FIX: BAD! tightly coupled to configuration management. Move to one-to-one table?
	cdatagroup          = models.ForeignKey('CDataGroup', null=False)

	# shortcut fields
	agent               = models.ForeignKey(Agent, null=False, related_name='monitored_datagroups')

	objects             = models.Manager()
	active_objects      = ActiveDataGroupFilterManager()

	scheduling_group    = models.ForeignKey('SchedulingGroup', null=True, default=None, on_delete=models.SET_NULL)


	UNEDITABLE_FIELDS = [
		'owner',
	]

	cached_fields = [
		 'lastdone'          ,  # timestamp last check

		 'lastchecktotaltime',  # Last total execution time
	]

	def __init__(self, *args, **kwargs):
		super(DataGroup, self).__init__(*args, **kwargs)

		# Fields from cache
		self.lastdone           = None
		self.lastchecktotaltime = None


	def __str__(self):
		return "<DataGroup:%s:%s>" % (self.uuid, self.path)

	def is_passive(self):
		return (self.minperiod == 0)

	def get_icon_url_with_state(self, status_code):
		return self.get_icon_url() + get_state_color(status_code)

	def get_icon_url(self):
		icon_name = self.get_icon_name_or_default()
		icon_url = reverse('webui.views.icons.get_colored_icon3', kwargs = {'icon': icon_name, 'tenantid': self.tenant_id})
		return icon_url

	def get_icon_name_or_default(self):
		return 'datagroup'


	def save(self, *args, **kwargs):
		#try:
		#	# FIX: QUesta gestione non mi piace per niente! Tanto vale aggiungere un campo di comodo nel DB
		#	# invece di usare una cache.
		#	from core.api.cache import init_cache_manager_from_conf
		#	cache_manager = self.get_cache_manager() or init_cache_manager_from_conf()
		#	self.update_expanded_title(cache_manager=cache_manager)
		#except Exception, e:
		#	print "Unable to update expanded title for %s:" % (self), e
		#	pass

		super(DataGroup, self).save(*args, **kwargs)

	#
	# Expanded title
	#
	def _subst_title(self, title_str):
		symbols = self.element.get_specific_element().get_title_symbols_dict()
		pd = self.get_title_symbols_dict()
		symbols.update( pd )
		return ExpandedTitleModel.subst_title(title_str, symbols)

	def get_title_symbols_dict(self):

		r = {
			'dgname': self.name
		}

		for p in self.datagroupparameter_set.all():
			try:
				name = '%s' % (p.name)
				v    = p.get_value()
			except Exception, e:
				v    = ''
			r[name] = v
		return r

	@property
	def anchored_node(self):
		# We don't know if this datagroup belongs to a node or to an other monitored element
		return self.element.get_node()

	def cache_key(self):
		return "%s" % (self.uuid)

	@staticmethod
	def cache_key_from_uuid(uuid):
		return "%s" % (uuid)

	def get_dict(self):
		return {
			'uuid'               : self.uuid,
			'mtime'              : self.mtime,
			
			'element'            : self.element_id,
			
			'path'               : self.path,
			'name'               : self.name,
			'title'              : self.title,
			
			'dependson_id'       : self.dependson_id,
			'minperiod'          : self.minperiod,
			'shorttries'         : self.shorttries,
			'timeout'            : self.timeout,
			
			'deleted'            : self.deleted,
			
			'suspended'          : self.suspended,
			'suspend_reason'     : self.suspend_reason,
			'suspend_time'       : self.suspend_time,
			'suspend_resume_time': self.suspend_resume_time,
			
			#
			# Cached fields
			#
			'lastdone'           : self.lastdone,
			'lastchecktotaltime' : self.lastchecktotaltime,

			#
			# Special fields used for datagroup check
			#
			'datasources'        : [], # ds.uuid for ds in self.datasources.all() ],
			'conditions'         : [], # c.uuid  for c  in self.conditions.all()  ],
		}

	@staticmethod
	def load_for_agent(tenant, agent, cache_manager=None):

		#
		# Con questa query calcolo velocemente gli scheduling-group di tutti gli elementi di monitoraggio
		# tenendo conto del livello di configurazione dell'elemento. 
		#   - nodo?        -> suo gruoppo o quello del suo cluster
		#   - interfaccia? -> suo gruppo o quello del cluster del suo nodo padre
		#

		cursor = connection.cursor()
		tenant.db_set_current_schema(db_cursor=cursor)

		query = """
SELECT
       me.uuid,
       COALESCE(me.scheduling_group_id, parent.scheduling_group_id, c_me.scheduling_group_id, c_parent.scheduling_group_id)
  FROM monitoredelement AS me  LEFT JOIN monitoredelement AS parent   ON (me.parent_id      = parent.uuid  )
                               LEFT JOIN cluster          AS c_me     ON (me.cluster_id     = c_me.uuid    )
                               LEFT JOIN cluster          AS c_parent ON (parent.cluster_id = c_parent.uuid)
 WHERE me.tenant_id = %s
;
		"""

		cursor.execute(query, [tenant.uuid])
		
		data = cursor.fetchall()
		
		scheduling_group_by_element_id = dict(data)

		
		#
		# Estrazione dei datagroup
		#
		datagroups = DataGroup.objects.filter(tenant=tenant, agent=agent).exclude(deleted=True)

		datagroups = datagroups.values(
			'uuid',
			'mtime',
			'element',
			'path',
			'name',
			'title',
			'dependson_id',
			'minperiod',
			'shorttries',
			'timeout',
			'deleted',
			'suspended',
			'suspend_reason',
			'suspend_time',
			'suspend_resume_time',
			'scheduling_group_id',
		)
		
		for d in datagroups:

			if d['scheduling_group_id'] is None:
				d['scheduling_group_id'] = scheduling_group_by_element_id.get(d['element'], None)

			#
			# Cached fields
			#
			d['lastdone'          ] = None
			d['lastchecktotaltime'] = None
			
			if cache_manager is not None:
				d.update( DataGroup.raw_load_fields_from_cache(cache_manager,  d['uuid'], DataGroup.cached_fields ) )

			#
			# Special fields. See also self.get_dict()
			#
			d['datasources'] = []
			d['conditions' ] = []


		return datagroups


	def get_dict_with_ds_and_c(self):
		d = self.get_dict()
		d.update({
			'datasources'        : [ ds.uuid for ds in self.datasources.filter(deleted=False).all() ],
			'conditions'         : [ c.uuid  for c  in self.conditions .filter(deleted=False).all()  ],
		})
		return d

	@property
	def anchored_node(self):
		# We don't know if this datagroup belongs to a node or to an other monitored element
		return self.element.get_node()

	@property
	def anchored_iface(self):
		# We don't know if this datagroup belongs to a node or to an other monitored element
		return self.element.get_iface()


	def get_state(self, critical_priority=PRIORITY_NONE, minimum_priority=PRIORITY_NONE):
		state, critical_dn_count, minimum_dn_count, tot_dn_count =  self.get_state_and_conditions(minimum_priority=minimum_priority, critical_priority=critical_priority)
		return state

	def get_state_and_conditions(self, critical_priority=PRIORITY_NONE, minimum_priority=PRIORITY_NONE, loaded_conditions=None):

		state, critical_dn_count, minimum_dn_count, tot_dn_count, tot_ignored = self.get_state_and_conditions_with_ignored(critical_priority=critical_priority, minimum_priority=minimum_priority, loaded_conditions=loaded_conditions)

		return state, critical_dn_count, minimum_dn_count, tot_dn_count

	def get_state_and_conditions_with_ignored(self, critical_priority=PRIORITY_NONE, minimum_priority=PRIORITY_NONE, loaded_conditions=None):

		status = CSTATE_UP

		if self.suspended:
			return CSTATE_TO_CODENAME(CSTATE_IN), 0, 0, 0, 0

		if loaded_conditions is not None:
			conditions = loaded_conditions
		else:
			conditions = self.conditions.filter(deleted=False).all()

		if len(conditions) == 0:
			# FIX: IL CALCOLO DELLO STATO DI UN DATAGROUP NON TENEVA CONTO DEI DATASOURCE!!! QUESTA COSA E' UNA MERDA E VA FATTA UN'ANALISI APPRODONDITA
			return self.get_state_from_datasources(), 0, 0, 0, 0

		critical_dn_count = 0
		minimum_dn_count  = 0
		tot_dn_count      = 0
		tot_ignored       = 0
		for c in conditions:
			critical_dn_count += 1 if (c.status == CSTATE_DN and c.priority >= critical_priority)                else 0
			minimum_dn_count  += 1 if (c.status == CSTATE_DN and (c.priority >= minimum_priority or c.primary))  else 0
			tot_dn_count      += 1 if (c.status == CSTATE_DN)                                                    else 0 
			tot_ignored       += 1 if (c.priority < minimum_priority and c.primary == False)                     else 0

		#state = self.get_state_from_conditions(conditions, minimum_priority=minimum_priority)

		state, dn_count2 = _get_state_from_condition(self.element_id, conditions, minimum_priority=minimum_priority)

		return state, critical_dn_count, minimum_dn_count, tot_dn_count, tot_ignored

	#def get_state_from_conditions(self, conditions, minimum_priority=PRIORITY_NONE):
        #
	#	code, dn_count = _get_state_from_condition(self.element_id, conditions, minimum_priority=minimum_priority)
        #
	#	return code



	def get_state_from_datasources(self):
		# FIX FIX FIX:  DA FARE ANALISI. Vedi get_state()

		status = CSTATE_UP

		datasources = self.datasources.all()
		if len(datasources) == 0:
			return CSTATE_TO_CODENAME(CSTATE_UU)

		counters = { 'up': 0, 'dn':0, 'un':0 }
		for ds in datasources:
			if ds.status in [  DATASOURCE_STATUS_OK , DATASOURCE_STATUS_STRANGE_VAL ]:
				counters['up'] += 1
			elif ds.status in [ DATASOURCE_STATUS_ERROR ]:
				counters['dn'] += 1
			else:
				counters['un'] += 1

		if len(datasources) == counters['dn']:
			status = CSTATE_DN
		elif len(datasources) == counters['un']:
			status = CSTATE_UU

		return CSTATE_TO_CODENAME(status)


	#
	# META
	#
	@staticmethod
	def get_fields():
		return [ (f.name, f.verbose_name) for f in DataGroup._meta.fields ] 

	@staticmethod
	def get_fields_dict():
		return dict([ (f.name, f) for f in DataGroup._meta.fields ] )

	@staticmethod
	def validate_field_value(field_name, value):
		f = DataGroup.get_fields_dict()[field_name]
		value = f.get_prep_value(value)
		
		
		if field_name == 'passive_match':
			# 1. string
			# 2. regex:<regex string>
			if value.startswith(DataGroup.PASSIVE_MATCH_REGEX_PREFIX):
				try:
					v = value.replace(DataGroup.PASSIVE_MATCH_REGEX_PREFIX,'')
					m = re.compile(v)
				except Exception, e:
					raise InvalidConfigurationException("Invalid passive match string: '%s' is not a regular expression" % (value))
		
		return value

	#
	# Utils
	#
	def get_configured_datagroup(self):
		return self.cdatagroup
	
	def calculate_path(self):
		cdg = self.get_configured_datagroup()
		return cdg.calculate_path()


	#
	# Update methods
	#
	def update_from_config(self, mtime=None, only_fields_names=None):
		"""
		Aggiorna il datagroup in base alla configurazione corrente sul database.
		
		Parametri:
			mtime                 facoltativo      orario di modifica facoltativo
			only_fields_names     facoltativo      lista di nomi per specificare solo quali field aggiornare. Aggiorna tutti se la lista e' None.
		"""
		if mtime is None: mtime = time.time()

		self._update_from_config(mtime=mtime, only_fields_names=only_fields_names)

		self.concurrent_save(new_sentinel_value=mtime)

		try:
			#from core.api.cache import init_cache_manager_from_conf
			cache_manager = self.get_cache_manager() or init_cache_manager_from_conf()
			self.expire_title(cache_manager=cache_manager)
		except Exception, e:
			print >> sys.stderr, "Generic error while updating datagroup: %s -> %s:%s " % (self, type(e), str(e))
			pass

	def update_path(self):
		
		p = self.calculate_path()
		
		if self.path != p:
			self.path = p
			return True
			
		return False
		

	def _update_from_config(self, mtime, only_fields_names=None):

		l=0
		_DEBUG(l, "Datagroup update from config")
		l=l+1
		
		updated = False
		
		# Fixiamo il path
		new_path = self.calculate_path()
		if self.path != new_path:
			self.path = new_path
			updated = True

		_DEBUG(l, self.path, ',', new_path)

		cdg = self.get_configured_datagroup()

		updated = self._update_fields_from_config(cdg, mtime=mtime, only_fields_names=only_fields_names, l=l) or updated
		
		if updated:
			_DEBUG(l, "XXXXXXXXXXXXXXXXXXXXXXXXXXXX Saving datagroup before updating sub elements")
			self.save()

		self._update_parameters_from_config(cdg, mtime=mtime, l=l)
		
		self._update_datasource_from_config(self.element, cdg, mtime=mtime, l=l)
		
		self._update_condition_from_config(cdg, mtime=mtime, l=l)
		
		self._update_timegraph_from_config(cdg, mtime=mtime, l=l)

		_DEBUG(l, "Need save: ", updated)
		if updated:
			self.concurrent_save(['mtime'], new_sentinel_value=mtime)

		#self._update_depending_elements(mtime=mtime,l=l)

		return


	#def _update_depending_elements(self, mtime=None, l=0):
	#	
	#	for c in self.conditions.all():
	#		_DEBUG(l, "Updating depending condition ", updated)
	#		c.update_depending_datagroup(mtime=mtime, l=l)

	DEFERRED_FIELDS = [
		'dependson',
		'scheduling_group'
	]

	#
	# Fields
	#

	def _update_fields_from_config(self, cdg, mtime=None, only_fields_names=None, l=0):
		l=l+1

		_DEBUG(l, "Resetting fields' default")

		self.reset_fields_defaults(l=l, only_fields_names=only_fields_names)

		_DEBUG(l, "Getting configured fields")

		fields = cdg.get_configured_fields(l=l)
		
		# Se dobbiamo aggiornare solo alcuni campi, li filtriamo
		if only_fields_names is not None:
			fields = [ f for f in fields if f.name in only_fields_names ]

		fields, special_fields = self._filter_special_fields(fields)

		updated = self._update_fields_from_list(fields, l=l)

		_DEBUG(l, "Updating configured fields")

		updated = self._update_special_fields(special_fields, l=l) or updated

		return updated

	def _filter_special_fields(self, fields):
		n = []
		s = []

		for f in fields:
			if f.name in self.DEFERRED_FIELDS:
				s.append(f)
			else:
				n.append(f)
		return n, s
	
	def _update_fields_from_list(self, fields, l=0):
		update = False
		for f in fields:
			update = self.update_single_field(f, l=l) or update
		return update
				
	def update_single_field(self, f, l=0):

		if f.name == 'dependson':
			return self._update_dependson(f.value, l=l)

		elif f.name == 'scheduling_group':
			return self._update_scheduling_group(f.value, l=l)

		else:
			old = getattr(self, f.name)
			new = f.get_value()
			if old != new:
				_DEBUG(l+1, f.name, old, new)
				setattr(self, f.name, new)
				return True
		return False
		
	def update_single_field_and_update(self, f, mtime=None, l=0):
		if self.update_single_field(f, l=l):
			self.concurrent_save([f.name], new_sentinel_value=mtime)
			return True
		else:
			return False
	
	#
	# Special fields
	#

	def _update_special_fields(self, special_fields, l=0):

		updated = False

		_DEBUG(l, "Updating special fields")

		# Dependson
		dep_path = None
		dependson_config = self._get_field_from_list_by_name(special_fields, 'dependson')
		if dependson_config:
			dep_path = dependson_config.value
		updated = self._update_dependson(dep_path, l=l+1) or updated

		# Scheduling group
		sg_name = None
		sg_config = self._get_field_from_list_by_name(special_fields, 'scheduling_group')
		if sg_config:
			sg_name = sg_config.value
		updated = self._update_scheduling_group(sg_name, l=l+1) or updated



		# Scheduling group
		sg_name = None
		sg_config = self._get_field_from_list_by_name(special_fields, 'scheduling_group')
		if sg_config:
			sg_name = sg_config.value
		updated = self._update_scheduling_group(sg_name, l=l+1) or updated


		return updated

	def _update_scheduling_group(self, sg_name, l=0):

			# If is None we are "unsetting" the value
			if sg_name is not None:
				sg = SchedulingGroup.objects.get(tenant=self.tenant, name=sg_name)
			else:
				sg = None

			if self.scheduling_group != sg:
				self.scheduling_group = sg
				return True

			return False

	def _update_dependson(self, dep_path, l=0):

		_DEBUG(l , "updating dependson config (dep_path=%r)" % (dep_path))
		l=l+1

		c = None

		if dep_path == DEP_NOT_CONFIGURED:
			_DEBUG(l, "dependson = Not configured. Finding primary")

			# Non possiamo puntare ad una condition di questo stesso datagroup.
			c = self._find_primary_condition(exclude_datagroups=[self])

		elif dep_path.lower() == DEP_CONFIGURED_NONE:
			_DEBUG(l, "dependson = None")
			c = None

		else:
			# Find path
			_DEBUG(l, "dependson = %s." % (dep_path))
			try:
				c = Condition.objects.get(path=dep_path)
			except Condition.DoesNotExist, e:
				_DEBUG(l+1, "Unable to find condition %s. Set to null" % (dep_path))
				c = None

		_DEBUG(l, "updating dependson field with: %s, old: %s" % (c, self.dependson))
		if self.dependson != c:
			self.dependson = c
			_DEBUG(l+1, "Updated! ")
			return True

		return False


	def _find_primary_condition(self, exclude_conditions=[], exclude_datagroups=[]):

		c = None

		# Vediamo se l'elemento di questo datagroup ha gia' una primary condition
		c = self.element.get_primary_condition()

		# Se non ce l'ha o non ci va bene, vediamo se l'elemento padre ha una
		# primary condition (es: interfaccia -> nodo)
		if c is None or (c in exclude_conditions) or (c.datagroup in exclude_datagroups):
			c = None
			if self.element.parent: #FIX: implement a recursive search
				c = self.element.parent.get_primary_condition()

		return c




	def _get_field_from_list_by_name(self, l, name):
		for f in l:
			if f.name == name:
				return f
		return None
		

	#
	# Parameters
	#
	def update_single_parameter_and_save(self, pv, mtime=None, l=0):
		
		# Vogliamo "unsettare il parametro
		if pv.value is not None:

			try:
				p = self.datagroupparameter_set.get(name=pv.name)

				_DEBUG(l+1, "updating ", pv.name, pv.vtype, pv.value)

			except DataGroupParameter.DoesNotExist, e:

				_DEBUG(l+1, "creating ", pv.name, pv.vtype, pv.value)
				
				p = DataGroupParameter(
					tenant = self.tenant,
					mtime  = time.time(),
					datagroup = self,
					name = pv.name,
					vtype = pv.vtype,
				)
				p.save()
				
			v = convert_to_param_type(pv.vtype, pv.value)
			
			p.value = pv.value
			
			p.concurrent_save(['value'], new_sentinel_value=mtime)
			
			self.concurrent_save(['mtime'], new_sentinel_value=mtime)
			
			
		else:
			try:
				p = self.datagroupparameter_set.get(name=pv.name)
				_DEBUG(l+1, "deleting ", pv.name)
				p.delete()
			except DataGroupParameter.DoesNotExist, e:
				pass
		return
		
		
		
	def _update_parameters_from_config(self, cdg, mtime=0, l=0):

		_DEBUG(l, "Updating parameters")
		l = l + 1
		
		configured_params = cdg.get_configured_params( l=l )

		_DEBUG(l, "Configured params:")
		for p in configured_params:
			_DEBUG(l+1, p)

		_DEBUG(l, "Current params:")
		current_params = [ p for p in self.datagroupparameter_set.all() ]
		for p in current_params:
			_DEBUG(l+1, p)
			_DEBUG(l+1, "Removing old parameter", p )
			p.delete()

		_DEBUG(l, "Creating current params")
		for cp in configured_params:

			_DEBUG(l+1, "creating ", cp.name, cp.vtype, cp.value)
			p = DataGroupParameter(
				tenant = self.tenant,
				mtime  = mtime,
				datagroup = self,
				name = cp.name,
				vtype = cp.vtype,
				value = cp.value    
			)
			p.save()
		
	def _update_datasource_from_config(self, element, cdg, mtime=0, l=0):
		_DEBUG(l, self, "Updating datasources")
		l=l+1

		_DEBUG(l, "Reading configuration")
		
		datasources_dict = cdg.get_configured_datasources(l=l+1)
		
		_DEBUG(l, "Delating old datasources")
		for old_ds in DataSource.objects.filter(tenant=self.tenant, element=self.element, datagroup = self).all():			
			if not datasources_dict.has_key(old_ds.name):
				_DEBUG(l+1, "Removing datasource", old_ds.name)
				old_ds.delete()

		_DEBUG(l, "Creating datasources")
		for ds_name, fields_dict in datasources_dict.items():
			
			try:
				ds = DataSource.objects.get(tenant=self.tenant, element=self.element, name = ds_name, datagroup = self)
				
				#
				# FIX: reset fields to defaults values.
				#
				
				_DEBUG(l+1, "Alread exists", ds_name)
				
			except DataSource.DoesNotExist, e:
			
				_DEBUG(l+1, "CREATING", ds_name)
				ds = DataSource(
					tenant     = self.tenant,
					element    = self.element,
					agent      = self.agent,
					name       = ds_name,
					datagroup  = self,
					
					owner      = self.owner,
				)
				ds.save()
			
			ds.update_path()
			ds.reset_fields_defaults()
			
			updated_fields = ds.update_fields_from_dict( fields_dict , l=l+1)
			
			if updated_fields:
				ds.adjust_fields()

			ds.concurrent_save(new_sentinel_value=mtime)  # FIX: update only updated fields

		_DEBUG(l, "Done")
		
		pass
		

	def adjust_fields(self):
		
		try:
			# FIX: QUesta gestione non mi piace per niente! Tanto vale aggiungere un campo di comodo nel DB
			# invece di usare una cache.
			#from core.api.cache import init_cache_manager_from_conf
			cache_manager = self.get_cache_manager() or init_cache_manager_from_conf()
			self.expire_title(cache_manager=cache_manager)
		except Exception, e:
			print "Unable to update expanded title for %s:" % (self), e
			pass
		
		
	def _update_condition_from_config(self, cdg, mtime=0, l=0):
		
		_DEBUG(l, self, "Updating conditions")
		l=l+1

		_DEBUG(l, "Reading configuration")
		conditions_dict = cdg.get_configured_conditions(l=l+1)
		
		_DEBUG(l, "Delating old conditions")
		for old_c in Condition.objects.filter(tenant=self.tenant, element=self.element, datagroup = self).all():
			if not conditions_dict.has_key(old_c.name):
				_DEBUG(l+1, "Removing condition", old_c.name)
				old_c.delete()		
		
		updated_conditions = []
		
		
		e_priority = self.element.get_effective_priority()
		
		_DEBUG(l, "Creating conditions")
		for c_name, fields_dict in conditions_dict.items():
			
			try:
				c = Condition.objects.get(tenant=self.tenant, element=self.element, name = c_name, datagroup = self)
				_DEBUG(l+1, "Alread exists", c_name)
			except Condition.DoesNotExist, e:
				_DEBUG(l+1, "CREATING", c_name, "with", str(fields_dict))
				c = Condition(
					tenant     = self.tenant,
					element    = self.element,
					agent      = self.agent,
					name       = c_name,
					datagroup  = self,

					owner      = self.owner,
				)
				c.save()
				created = True
			
			c.update_path()
			c.concurrent_save(new_sentinel_value=mtime)  # FIX: update only updated fields			

			c.reset_fields_defaults(l=l+1)
			
			#if e_priority != PRIORITY_NONE:
			#	c.priority = e_priority
			#	updated = True
			
			updated = c.update_fields_from_dict( fields_dict , l=l+1)
			
			c.adjust_fields()

			#if e_priority != PRIORITY_NONE and e_priority != c.priority:
			#	c.priority = e_priority
			#	updated = True

			#c.update_fields_from_dict( fields_dict , l=l+1)
			#c.adjust_fields()
			#c.fix_dependson_field()

			if updated:
				c.concurrent_save(new_sentinel_value=mtime)  # FIX: update only updated fields


		#_DEBUG(l, "Fixing all dependson references")
		#for c in updated_conditions:
		#	_DEBUG(l+1, "Checking %s" % ( c.path) )
		#	config_dep = None
		#	config = conditions_dict.get(c_name,None)
		#	if config:
		#		config_dep = config.get('dependson',None)
		#	if config_dep == None:
		#		if c.fix_dependson_field(None, l=l+2):
		#			c.mtime = mtime
		#			c.save()

		_DEBUG(l, "Done")

		return

	def _update_timegraph_from_config(self, cdg, mtime=0, l=0):
		
		_DEBUG(l, self, "Updating timegraphs")
		l=l+1

		_DEBUG(l, "Reading configuration")
		
		timegraphs_dict = cdg.get_configured_timegraphs(l=l+1)
		
		_DEBUG(l, "Delating old timegraphs")
		for old_tg in TimeGraph.objects.filter(tenant=self.tenant, element=self.element, datagroup = self).all():			
			if not timegraphs_dict.has_key(old_tg.name):
				_DEBUG(l+1, "Removing condition", old_tg.name)
				old_tg.delete()
		
		for tg_name, timegraph_dict in timegraphs_dict.items():

			_DEBUG(l+1, "CREATING", tg_name)
			
			tg = None
			created = False
			try:
				#tg = TimeGraph.objects.get(tenant=self.tenant, element=self.element, name=tg_name, user=None, datagroup=self)
				tg = TimeGraph.objects.get(tenant=self.tenant, element=self.element, name=tg_name, datagroup=self)
			except TimeGraph.DoesNotExist, e:
				tg = None

			except TimeGraph.MultipleObjectsReturned, e:

				tg = None

				#
				# workaround: database is dirty, but we can
				#
				#broken_tgs =  TimeGraph.objects.filter(tenant=self.tenant, element=self.element, name=tg_name, user=None, datagroup=self).all()
				broken_tgs =  TimeGraph.objects.filter(tenant=self.tenant, element=self.element, name=tg_name, datagroup=self).all()
				for btg in broken_tgs:
					btg.delete()



			if tg == None:
				#tg, created = TimeGraph.objects.get_or_create(tenant=self.tenant, element=self.element, name=tg_name, user=None, datagroup=self)
				tg, created = TimeGraph.objects.get_or_create(tenant=self.tenant, element=self.element, name=tg_name, datagroup=self)
				tg.owner = self.owner
				tg.save()
			
			if created:
				tg.mtime = mtime
			
			fields_dict = timegraph_dict['fields']
			

			tg.reset_fields_defaults(l=l+1)
			tg.update_fields_from_dict( fields_dict , l=l+1)

			tg.mtime = mtime
			tg.owner = self.owner

			tg.adjust_fields()

			tg.save()
			
			series = timegraph_dict['series']

			for tgs in tg.series.all():
				if tgs.name not in series.keys():
					_DEBUG(l+1, "Removing serie", tgs.name)
					tgs.delete()

			for tgs_name, fields_dict in series.items():

				_DEBUG(l+2, "Creating serie", tgs_name)
				tgs, created = TimeGraphSerie.objects.get_or_create(
					tenant    = self.tenant,
					timegraph = tg,
					name      = tgs_name,
				)

				tgs.save()

				if created:
					tgs.mtime = mtime #time.time()

				tgs.reset_fields_defaults(l=l+1)
				tgs.update_fields_from_dict( fields_dict , l=l+1)

				tgs.owner = self.owner

				tgs.mtime = mtime
				#tgs.adjust_fields()
				tgs.save()
				

		_DEBUG(l, "Done")
		pass
		
	#
	# Priority
	#
	def refresh_priority(self, min_priority=None, mtime=None, l=0):
		
		_DEBUG(l, "Refreshing priority: min %s" % (min_priority))
		
		# Check defaults
		if mtime        is None: mtime = time.time()
		if min_priority is None: min_priority = self.element.get_element_effective_priority()

		# Search all configured conditions
		configured_conditions = self.cdatagroup.get_configured_conditions()
		for c in self.conditions.all():
			
			configured_priority = DEFAULT_PRIORITY

			_DEBUG(l+1, "Condition %s (priority %s)" % ( c.path, c.priority) )
			
			config = configured_conditions.get(c.name,None)
			if config is None:
				continue

			configured_field = config.get('priority', None)
			if configured_field is not None:
				configured_priority = configured_field.get_value()

			_DEBUG(l+2, "Configured priority: %s" % (configured_priority) )

			#min_priority = max(min_priority, configured_priority)
			min_priority = configured_priority if (configured_priority != PRIORITY_NONE) else min_priority
			
			_DEBUG(l+2, "set priority to %s" % (min_priority) )
			c.refresh_priority(min_priority=min_priority, mtime=mtime)
			
		self.concurrent_save(new_sentinel_value=mtime)



	def suspend(self, suspend_time=None, resume_time=0, reason=None, force=False):

		if self.suspended and not force:
			raise Exception("Can not suspend a suspended datagroup.")

		if suspend_time == None:
			suspend_time = int(time.time())

		# Suspend reason can not be "empty"
		if reason == None or reason == SUSPEND_REASON_NONE:
			reason = 'No reason specified (Suspended at %s)' % (suspend_time)

		if resume_time is None:
			resume_time = SUSPEND_RESUME_NEVER

		if resume_time is not SUSPEND_RESUME_NEVER: 
			if resume_time <= suspend_time:
				raise Exception("Resume time (%s) must be after suspend time (%s)" % (resume_time, suspend_time))

		#
		# Update datagroup
		#
		self.suspended           = True
		self.suspend_reason      = reason
		self.suspend_time        = suspend_time
		self.suspend_resume_time = resume_time
		self.save()

		fields = {
			TField.datagroup_field('suspended')          : 'true',
			TField.datagroup_field('suspend_reason')     : reason,
			TField.datagroup_field('suspend_resume_time'): str(resume_time),
			TField.datagroup_field('suspend_time')       : str(suspend_time),
		}
		for f,v in fields.items():
			cdgfv, created = CDataGroupFieldValue.objects.get_or_create(tenant=self.tenant, cdatagroup=self.cdatagroup, tfield=f)
			if created or cdgfv.value != v:
				cdgfv.value = v
				cdgfv.mtime = suspend_time
				cdgfv.save()

		#
		# Update datasource
		#
		for ds in self.datasources.all():
			#ds.status = DATASOURCE_STATUS_IN
			#ds.lastdone = suspend_time

			# Do not change the mtime if the suspend time is older than the last time we modified the datasource!
			if ds.mtime < suspend_time:
				ds.mtime = suspend_time
				ds.save()

		#
		# Update conditions
		#
		for c in self.conditions.all():

			status_changed = False

			if c.status != CSTATE_IN:
				c.statuslast       = c.status
				c.status           = CSTATE_IN

				status_changed = True

			c.statuslastchange = suspend_time
			c.verbstatus       = reason

			# Do not change the mtime if the suspend time is older than the last time we modified the condition!
			if c.mtime < suspend_time:
				c.mtime = suspend_time

			c.update_notable_state(suspend_time)
			c.save()

			if status_changed:

				# Create LOG
				e = Event(
					tenant           = self.tenant,
					mtime            = time.time(),
					kind             = EVENT_CONDITION,
					timestamp        = time.time(),
					msg              = reason,
					element_path     = self.element.path,
					condition_path   = c.path,

					element_uuid     = self.element.uuid,
					condition_uuid   = c.uuid,

					status           = c.status,
					statuslast       = c.statuslast,
					statuslastchange = c.statuslastchange,
					priority         = c.priority,
				)
				e.save()

		return

	def resume(self, resume_time=None, reason=None, force=False):

		if not self.suspended and not force:
			raise Exception("Can not resume a active datagroup.")

		if resume_time == None:
			resume_time = time.time()

		if reason == None:
			reason = 'Resumed at %s' % (suspend_time)

		#
		# Update datagroup
		#
		self.suspended           = False
		self.suspend_reason      = ''
		self.suspend_time        = None
		self.suspend_resume_time = None
		self.mtime               = resume_time
		self.save()

		self.remove_configured_suspend_fields()

		#
		# Update datasources
		#
		for ds in self.datasources.all():
			ds.mtime = resume_time
			ds.save()

		#
		# Update conditions
		#
		for c in self.conditions.all():
			
			status_changed = False
			
			if c.status == CSTATE_IN:
				last               = c.status
				c.status           = c.statuslast
				c.statuslast       = last
				
				status_changed = True

			c.statuslastchange = resume_time
			c.verbstatus       = reason

			if c.mtime < resume_time:
				c.mtime            = resume_time
			
			c.update_notable_state(resume_time)
				
			c.save()
			
			if status_changed:

				# Create LOG
				e = Event(
					tenant           = self.tenant,
					mtime            = time.time(),
					kind             = EVENT_CONDITION,
					timestamp        = resume_time,
					msg              = reason,
					element_path     = self.element.path,
					condition_path   = c.path,
					
					element_uuid     = self.element.uuid,
					condition_uuid   = c.uuid,
					
					status           = c.status,
					statuslast       = c.statuslast,
					statuslastchange = c.statuslastchange,
					priority         = c.priority,
				)
				e.save()

		return

	def remove_configured_suspend_fields(self):
		fields = [
			TField.datagroup_field('suspended')          ,
			TField.datagroup_field('suspend_reason')     ,
			TField.datagroup_field('suspend_time')       ,
			TField.datagroup_field('suspend_resume_time'),
		]
		for f in CDataGroupFieldValue.objects.filter(cdatagroup=self.cdatagroup, tfield__in=fields):
			f.delete()


	def get_logs(self):

		# All logs related to conditions with same path prefix
		filters = Q(condition_path__startswith=self.path) # FIX: skip logs related to "deleted=True" conditions.
		
		# all logs belonging to condition, but saved with an old "path"
		c_uuids = self.conditions.filter(deleted=False).values_list('uuid', flat=True).all()
		filters |= Q(condition_uuid__in=c_uuids)
		
		return Event.objects.filter(filters)

	def get_safe_params_dict(self):
		"""
		Returns a dictionary of defined params, but ignores
		invalid parameters.
		"""
		d = {}
		for dgp in self.datagroupparameter_set.all():
			try:
				d[dgp.name] = dgp.get_value() 
			except Exception, e:
				print "EXCEPTION get_safe_params_dict()", e
				pass
		return d
		
	#
	# Rename methods
	#
	def rename_parameter(self, param_name, new_name):
		p = self.datagroupparameter_set.get(name=param_name)
		p.name = new_name
		p.save()   # FIX: use concurrent update
	
	def rename_datasource(self, old_name, new_name):
		ds = self.datasources.get(name=old_name)
		ds.name = new_name
		ds.update_path()
		ds.save() # FIX: use concurrent update

	def rename_condition(self, old_name, new_name):
		c = self.conditions.get(name=old_name)
		c.name = new_name
		c.update_path()
		c.save() # FIX: use concurrent update
	
	def rename_timegraph(self, old_name, new_name):
		tg = self.timegraphs.get(name=old_name)
		tg.name = new_name
		tg.save() # FIX: use concurrent update
	
	def rename_timegraphserie(self, tg_name, old_name, new_name):
		tg = self.timegraphs.get(name=tg_name)

		tgs = tg.series.get(name=old_name)
		tgs.name = new_name
		tgs.save() # FIX: use concurrent update
		
		
	"""
	def force_status(self, logger, new_status, change_time=None, selected_condition_names=None, reason=None, cache_manager=None):
		
		if cache_manager is None:
			cache_manager = init_cache_manager_from_conf()
		
		if change_time is None:
			change_time = time.time()
			
		if reason is None:
			reason = 'Datagroup to status %s at %s' % (CSTATE_TO_CODENAME(new_status), change_time)
		#
		# Update datagroup
		#
		self.lastdone = change_time
		self.save_fields_in_cache(cache_manager)
		
		#
		# Update conditions
		#
		conditions = self.conditions
		if selected_condition_names is not None:
			conditions = conditions.filter(name__in=selected_condition_names)

		conditions = conditions.all()
		for c in conditions:
			logger.info("Condition %s", c.path)
			
			last               = c.status
			
			if last == new_status:
				logger.info("not changed")
				continue
			
			c.lastdone         = change_time
			c.status           = new_status
			c.statuslast       = last
			c.statuslastchange = change_time
			c.verbstatus       = reason
			
			c.update_notable_state(change_time)

			c.mtime            = change_time
			
			#
			# Update cache
			#
			c.save_fields_in_cache(cache_manager)
			
			#
			# Update DB
			#
			c.save()

			e = Event(
				tenant           = self.tenant,
				mtime            = change_time,
				kind             = EVENT_CONDITION,
				timestamp        = change_time,
				msg              = reason,
				element_path     = self.element.path,
				condition_path   = c.path,
				status           = c.status,
				statuslast       = c.statuslast,
				statuslastchange = c.statuslastchange,
				priority         = c.priority,
			)
			e.save()
			logger.info("\tupdated")
	"""


	class Meta:
		unique_together = (
			"tenant", 
			"path",
		)
		db_table = "datagroup"

#-----------------------------------------------------------------------------#

class DataGroupParameter(TenantDependentModel, MTimeModel, Parameter):

	# tenant
	# mtime

	# name
	# vtype
	# value

	datagroup = models.ForeignKey(DataGroup)

	def __str__(self):
		return "<%s|%s,%s,%s>" % (self.datagroup.path, self.name, self.vtype, self.value)
		
	def get_dict(self):
		return {
			'mtime': self.mtime,
			'name' : self.name,
			'vtype': self.vtype,
			'value': self.value,
			'datagroup_id': self.datagroup_id,
		}

	@staticmethod
	def load_for_agent(tenant, agent):

		ret = DataGroupParameter.objects.filter(tenant=tenant, datagroup__agent=agent).values(
			'mtime',
			'name',
			'vtype',
			'value',
			'datagroup_id',
		).all()

		return ret


	class Meta:
		db_table = "datagroup_parameter"

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

from sanet_common.formatutils import calc_num_prefix

class DataSource(TenantDependentModel, UUIDKeyModel, MTimeModel, AbstractCachedFieldModel, ModelFieldsUtils, ExpandedTitleModel, CachedUtil, OwnerUserModel):

	# tenant
	# uuid
	# mtime

	# shortcut fields
	element        = models.ForeignKey(MonitoredElement, null=False, related_name='monitored_datasources')
	agent          = models.ForeignKey(Agent           , null=False, related_name='monitored_datasources')	
	datagroup      = models.ForeignKey(DataGroup, related_name="datasources")

	path           = models.TextField(null=False, blank=False, db_index=True)
	name           = models.TextField(null=False, blank=False, db_index=True)

	title          = models.TextField(null=False, blank=True, default='', db_index=True)

	# Editable fields
	expr           = models.TextField(null=False, blank=True, default='', db_index=False)
	status         = models.SmallIntegerField(null=False, default=DATASOURCE_STATUS_OK, db_index=True)

	order          = models.PositiveSmallIntegerField(null=False, default=0) # evaluation order
	cascade        = models.BooleanField(null=False, default=False)           # Stop evaluation of all the following datasource on error or uncheckable value of this datasource

	save_value     = models.BooleanField(null=False, default=True)           # Save value on Database

	gauge          = models.BooleanField(null=False, default=True)           # FALSE=counter , TRUE=gauge

	# FIX: use il 'bits' flag?
	min_val        = models.FloatField(null=True, default=None)
	max_val        = models.FloatField(null=True, default=None)

	storage_spec   = models.TextField(null=True, default=None)               # OLD: series      

	classification = models.TextField(null=False, blank=True, default='', db_index=True)

	deleted        = models.BooleanField(default=False, db_index=True) # N.B.: if deleted, then suspended must be true, and target won't show up in listings


	objects        = models.Manager()
	active_objects = ActiveFilterManager()


	UNEDITABLE_FIELDS = [
		'owner',

		'status'
	]

	cached_fields = [
		 'lastdone'           # timestamp last check
		,'verbstatus'         

		,'lastvalue'          # last value received from network
		,'lastrate'           # last rate  calculated by agents for COUNTERS datasources 

		,'lastchecktotaltime' # Last total execution time
	]

	def __init__(self, *args, **kwargs):
		super(DataSource, self).__init__(*args, **kwargs)

		# Cached fields
		self.lastdone           = 0
		self.verbstatus         = None
		
		self.lastvalue          = None
		self.lastrate           = None

		self.lastchecktotaltime = None

	def __str__(self):
		return "<DataSource:%s:%s>" % (self.uuid, self.path)


	def get_last_value(self):
		if self.gauge:
			v = self.lastvalue
		else:
			v = self.lastrate
		return v

	def get_scaled_last_value(self):

                try:
			v = self.get_last_value()
			v = DataSource.calculate_scaled_value(v)
		except TypeError, e:
			# NOTE: last value is not a number? For a datasource is strange but not impossibile.
			v = None

		if v is None:
			return "NONE"

		return v

	@staticmethod
	def calculate_scaled_value(v):
		try:
			(v, base, e, p) = calc_num_prefix(v)
			if int(v) < v:
				return "%.3f%s" % (v, p)
			else:
				return "%.0f%s" % (v, p)
		except ValueError, e:
			return None


	def get_icon_name_or_default(self):
		return 'datasource'

	def get_icon_url(self):
		icon_name = self.get_icon_name_or_default()
		icon_url = reverse('webui.views.icons.get_colored_icon3', kwargs = {'icon': icon_name, 'tenantid': self.tenant_id})
		return icon_url

	def get_icon_url_with_state(self, outcome_code):
		return self.get_icon_url() + get_outcome_color(outcome_code)

	def save(self, *args, **kwargs):
		#try:
		#	# FIX: QUesta gestione non mi piace per niente! Tanto vale aggiungere un campo di comodo nel DB
		#	# invece di usare una cache.
		#	from core.api.cache import init_cache_manager_from_conf
		#	cache_manager = self.get_cache_manager() or init_cache_manager_from_conf()
		#	self.update_expanded_title(cache_manager=cache_manager)
		#except Exception, e:
		#	print "Unable to update expanded title for %s:" % (self), e
		#	pass
			
		super(DataSource, self).save(*args, **kwargs)

	def adjust_fields(self):

		try:
			ds.check_and_update_storage_spec()
		except Exception, e:
			print __file__, str(e)
			pass

		try:
			# FIX: QUesta gestione non mi piace per niente! Tanto vale aggiungere un campo di comodo nel DB
			# invece di usare una cache.
			#from core.api.cache import init_cache_manager_from_conf
			cache_manager = self.get_cache_manager() or init_cache_manager_from_conf()
			self.expire_title(cache_manager=cache_manager)
		except Exception, e:
			print "Unable to update expanded title for %s:" % (self), e
			pass

	#
	# Expanded title
	#
	def _subst_title(self, title_str):
		symbols = self.element.get_specific_element().get_title_symbols_dict()
		pd = self.get_title_symbols_dict()
		symbols.update( pd )
		#s = Template(title_str).safe_substitute(**symbols)
		#return s
		return ExpandedTitleModel.subst_title(title_str, symbols)

	def get_title_symbols_dict(self):
		return self.datagroup.get_title_symbols_dict()

	#
	# Others
	#
	def get_outcome(self):
		return DSSTATE_TO_CODENAME(self.status)

	def cache_key(self):
		return "%s" % (self.uuid)

	def get_dict(self):
		return {
			'uuid'           : self.uuid,
			'mtime'          : self.mtime, 
			'datagroup_id'   : self.datagroup_id,
			'path'           : self.path,
			'name'           : self.name,
			#'title'          : self.title,
			'expr'           : self.expr,
			'status'         : self.status,
			'order'          : self.order,
			'cascade'        : self.cascade,
			'save_value'     : self.save_value,
			'gauge'          : self.gauge,
			'min_val'        : self.min_val,
			'max_val'        : self.max_val,

			#
			# Cached fields
			#
			'lastdone'           : self.lastdone           if self.lastdone != None else 0,
			#'verbstatus'         : self.verbstatus        
			#'lastvalue'          : self.lastvalue         
			#'lastrate'           : self.lastrate          
			'lastchecktotaltime' : self.lastchecktotaltime,
		}

	@staticmethod
	def load_for_agent(tenant, agent, cache_manager):

		ret = DataSource.active_objects.filter(tenant=tenant, agent=agent).values(
			'uuid'         
			,'mtime'       
			,'datagroup_id'
			,'path'        
			,'name'        
			,'expr'        
			,'status'      
			,'order'       
			,'cascade'     
			,'save_value'  
			,'gauge'       
			,'min_val'     
			,'max_val'    
		)

		for d in ret:
			#
			# Cached fields
			#
			d['lastdone'          ] = None
			#d['verbstatus'        ] = None
			#d['lastvalue'         ] = None
			#d['lastrate'          ] = None
			d['lastchecktotaltime'] = None

			d.update( DataSource.raw_load_fields_from_cache(cache_manager,  d['uuid'], DataSource.cached_fields ) )

		return ret

	def get_rrd_values(self, ts_start, ts_end):
		from core.api.datasource import init_datasource_store_from_conf

		datasource_store = init_datasource_store_from_conf()

		step = self.datagroup.minperiod

		data = datasource_store.fetch_values(self.tenant_id,
		                                     self.uuid,
		                                     ts_start=ts_start,
		                                     ts_end=ts_end,
		                                     resolution=step,
		                                     min_value=self.min_val,
		                                     max_value=self.max_val)
		return data

	def calculate_path(self):
		path = self.datagroup.calculate_path()
		path = path + DATAGROUP_PATH_SEP + self.name
		return path

	def update_path(self):
		self.path = self.calculate_path()

	def update_fields_from_dict(self, fields_dict, l=0):
		updated = False
		_DEBUG(l, self, "update_fields_from_dict()")
		for name, f in fields_dict.items():
			update = self.update_single_field(f, l=l) or updated
		return updated

	def update_single_field(self, f, l=0):
		
		old = getattr(self, f.name)
		new = f.get_value()
		if old != new:
			_DEBUG(l+1, f.name, old, new)
			setattr(self, f.name, new)
			return True
			
		return False

	def update_single_field_and_save(self, f, mtime=None, l=0):
		if self.update_single_field(f, l=l):
			
			self.concurrent_save([f.name], new_sentinel_value=mtime)
			
			self.check_storage_spec_changes()
			
			return True
		return False

	def check_storage_spec_changes(self):
		"""
		Allinea i parametri di configurazione 'storage_perc' del datasource con
		la configurazione attualmente impostata a livello di storage (file RRD)
		"""
		storage_object = init_datasource_store_from_conf()
		return storage_object.check_and_update_storage_spec(self.tenant_id, self.uuid, self.datagroup.minperiod, self.gauge, self.storage_spec)

	#
	# META
	#
	@staticmethod
	def get_fields():
		return [ (f.name, f.verbose_name) for f in DataSource._meta.fields ] 	

	@staticmethod
	def get_fields_dict():
		return dict([ (f.name, f) for f in DataSource._meta.fields ] )

	@staticmethod
	def validate_field_value(field_name, value):

		f = DataSource.get_fields_dict()[field_name]
		value = f.get_prep_value(value)	

		if field_name == 'expr':
			try:
				parseString(value)
			except ParserException, e:
				#raise ValueError("Invalid expression: %s" % (e))
				raise InvalidConfigurationException("Invalid expression: %s" % (e))

		if field_name == 'storage_spec':
			
			ds_store = init_datasource_store_from_conf()
			try:
				ds_store.validate_storage_spec_options(value)
			except Exception, e:
				raise InvalidConfigurationException(str(e))
				
		return value

	class Meta:
		unique_together = (
			"tenant", 
			"path",
		)
		db_table = "datasource"
		

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

DEFAULT_EMAIL = 'netwarning'
EMAIL_NONE_VALUE = ''

TEMPLATE_NONE_VALUE = 'none'

class Condition(TenantDependentModel, UUIDKeyModel, MTimeModel, AbstractCachedFieldModel, ModelFieldsUtils, ExpandedTitleModel, CachedUtil, OwnerUserModel):

	# tenant
	# uuid
	# mtime

	# shortcut fields
	element                 = models.ForeignKey(MonitoredElement, null=False, related_name='monitored_conditions')
	agent                   = models.ForeignKey(Agent           , null=False, related_name='monitored_conditions')
	datagroup               = models.ForeignKey(DataGroup, related_name="conditions")

	path                    = models.TextField(null=False, db_index=True)
	name                    = models.TextField(null=False, db_index=True)

	title                   = models.TextField(null=False, blank=True, default='', db_index=True)

	primary                 = models.BooleanField(null=False, default=False)

	dependson               = models.ForeignKey('Condition', null=True, default=None, db_index=True, on_delete=models.SET_NULL)

	order                   = models.PositiveSmallIntegerField(null=False, default=0, db_index=True)  # evaluation order
	cascade                 = models.BooleanField(null=False, default=False, db_index=True)           # Stop evaluation of all the following conditions on false or uncheckable value

	expr                    = models.TextField(null=False, blank=True, default='', db_index=False)

	max_tries               = models.PositiveSmallIntegerField(null=False, default=3)

	uncheckable_fallback    = models.SmallIntegerField(null=True, default=None, db_index=False)
	statuschange_action     = models.TextField(null=True, db_index=False)

	priority                = models.IntegerField(null=False, default=PRIORITY_NONE, db_index=True)

	classification          = models.TextField(null=False, blank=True, default='', db_index=True)

	# See if it is time to review the semantics of email
	email                   = models.EmailField(null=False, max_length=1000, default=DEFAULT_EMAIL, blank=True, db_index=False)
	upemail                 = models.EmailField(null=True , max_length=1000, default=EMAIL_NONE_VALUE, blank=True, db_index=False)
	msg_downsubj            = models.TextField(default='', blank=True, null=True, db_index=False)
	msg_downbody            = models.TextField(default='', blank=True, null=True, db_index=False)
	msg_upsubj              = models.TextField(default='', blank=True, null=True, db_index=False)
	msg_upbody              = models.TextField(default='', blank=True, null=True, db_index=False)

	#lastdone               = models.PositiveSmallIntegerField(null=False, default=0, db_index=True)
	#tries                  = models.PositiveSmallIntegerField(null=False, default=0, db_index=True)

	#penalty                = models.PositiveSmallIntegerField(default=0, db_index=False)
	#dampened               = models.BooleanField(default=0, db_index=True)
	status                  = models.SmallIntegerField(null=False, default=CSTATE_DEFAULT, db_index=True)
	statuslast              = models.SmallIntegerField(null=False, default=CSTATE_DEFAULT, db_index=True)

	# Timestamp dell'ultimo cambiamento di stato
	statuslastchange        = models.PositiveIntegerField(default=0, db_index=True)

	#statuslastnotable      = models.SmallIntegerField(null=False, default=CSTATE_DEFAULT, db_index=True)
	statuslastnotablechange = models.PositiveIntegerField(default=0, db_index=True)

	deleted                 = models.BooleanField(default=False, db_index=True) # N.B.: if deleted, then suspended must be true, and target won't show up in listings


	objects        = models.Manager()
	active_objects = ActiveFilterManager()


	UNEDITABLE_FIELDS = [
		'status',
		'statuslast',
		'statuslastchange',
		#'statuslastnotable',
		'statuslastnotablechange',
		'owner',
	]

	cached_fields = [
		'lastdone',
		'tries',

		'verbstatus', 

		#,'penalty' 
		#,'dampened'

		# Current alarm expanded message ( msg_downbody or msg_upbody )
		'verbstatus_msgbody', 


		'lastchecktotaltime',
	]

	def __init__(self, *args, **kwargs):
		super(Condition, self).__init__(*args, **kwargs)
		
		# Cached fields
		self.tries              = 0
		self.lastdone           = None
		self.verbstatus_msgbody = None
		self.lastchecktotaltime = None

	@property
	def priority_label(self):

		try:
			return PriorityLabel.objects.get(min_level__lte=self.priority, max_level__gte=self.priority).label
		except PriorityLabel.DoesNotExist, e:
			return "*not configured*"
    
	def __str__(self):
		return "<Condition:%s:%s>" % (self.uuid, self.path)	

	def get_icon_url(self):
		icon_name = self.get_icon_name_or_default()
		icon_url = reverse('webui.views.icons.get_colored_icon3', kwargs = {'icon': icon_name, 'tenantid': self.tenant_id})
		return icon_url
		
	def get_icon_name_or_default(self):
		return 'condition'

	@property
	def description(self):
		return self.title

	def update_notable_state(self, ctime):
		"""
		Aggiorna il campo "statuslastnotablechange" con il valore "ctime" se
		lo stato attuale e precedente della condition rappresentano una variazione "significativa"
		"""

		if is_notable_transition(self.statuslast, self.status):
			self.statuslastnotablechange = ctime
			return True

		return False

	def save(self, *args, **kwargs):
		#try:
		#	# FIX: QUesta gestione non mi piace per niente! Tanto vale aggiungere un campo di comodo nel DB
		#	# invece di usare una cache.
		#	from core.api.cache import init_cache_manager_from_conf
		#	cache_manager = self.get_cache_manager() or init_cache_manager_from_conf()
		#	self.update_expanded_title(cache_manager=cache_manager)
		#except Exception, e:
		#	print "Unable to update expanded title for %s:" % (self), e
		#	pass
			
		super(Condition, self).save(*args, **kwargs)
		

	#
	# Expanded title
	#
	def _subst_title(self, title_str):
		symbols = self.element.get_specific_element().get_title_symbols_dict()
		pd = self.get_title_symbols_dict()
		symbols.update( pd )
		return ExpandedTitleModel.subst_title(title_str, symbols)

	def get_title_symbols_dict(self):
		return self.datagroup.get_title_symbols_dict()

	@property
	def verbose_tries(self):
		return "%s/%s" % ( self.tries if self.tries else 0 , self.max_tries )

	#
	# State
	#
	def get_state(self, critical_priority=PRIORITY_NONE, minimum_priority=PRIORITY_NONE):

		#
		# FIX: handle critical_priority, minimum_priority
		#

		status = CSTATE_TO_CODENAME(self.status)

		return status

	def get_statelast(self):
		return CSTATE_TO_CODENAME(self.statuslast)

	def get_state_and_conditions(self, critical_priority=None):
		critical_priority = critical_priority if critical_priority != None else PRIORITY_NONE
		s = CSTATE_TO_CODENAME(self.status)
		n = 1 if (self.status == CSTATE_DN and self.priority >= critical_priority) else 0
		return (s, n)

	def get_cstate_at(self, timestamp):
		
		logs = self.get_logs().filter(timestamp__gt=int(timestamp)).order_by('timestamp')[0:1]
		if len(logs) > 0:
			return logs[0].statuslast
			
		return self.status

	#
	# Icon
	#
	def get_icon_url(self):
		icon_name = 'condition'
		icon_url = reverse('webui.views.icons.get_colored_icon3', kwargs = {'icon': icon_name, 'tenantid': self.tenant_id})
		return icon_url

	def get_icon_url_with_state(self, status_code):
		return self.get_icon_url() + get_state_color(status_code)
	#
	# Cache
	#
	def cache_key(self):
		return "%s" % (self.uuid)

	def get_dict(self):
		return {
			'uuid'                 : self.uuid,
			'mtime'                : self.mtime, 

			'datagroup_id'         : self.datagroup_id,
			'element_id'           : self.element_id,
			'path'                 : self.path,
			'name'                 : self.name,
			'title'                : self.title,
			'primary'              : self.primary,
			'classification'       : self.classification,
			'dependson_id'         : self.dependson_id,
			'order'                : self.order,
			'cascade'              : self.cascade,
			'expr'                 : self.expr,
			'max_tries'            : self.max_tries,
			'priority'             : self.priority,
			'uncheckable_fallback' : self.uncheckable_fallback,
			'statuschange_action'  : self.statuschange_action,

			'tries'                : self.tries    if self.tries    != None else 0,
			'lastdone'             : self.lastdone if self.lastdone != None else 0,

			'status'               : self.status,          
			'statuslast'           : self.statuslast,
			'statuslastchange'     : self.statuslastchange,
			#'statuslastnotable'       : self.statuslastnotable,
			#'statuslastnotablechange' : self.statuslastnotablechange,
			
			'lastchecktotaltime'   : self.lastchecktotaltime,
		}
		
	def get_full_verbstatus(self):
		
		return self.verbstatus_msgbody or self.verbstatus or ''

	@staticmethod
	def load_for_agent(tenant, agent, cache_manager):

		ret = Condition.active_objects.filter(tenant=tenant, agent=agent).values(
			'uuid',
			'mtime',
			'datagroup_id',
			'element_id',
			'path'        ,
			'name'        ,
			'title'       ,
			'primary'     ,
			'classification',
			'dependson_id',
			'order',
			'cascade',
			'expr'        ,
			'max_tries'   ,
			'priority'    ,
			'uncheckable_fallback',
			'statuschange_action',
			#'tries',
			#'lastdone',
			'status',
			'statuslast',
			'statuslastchange',
			#'statuslastnotable',
			#'statuslastnotablechange',
		)

		for d in ret:
			d['tries'             ] = None
			d['lastdone'          ] = None
			d['lastchecktotaltime'] = None
			
			d.update( Condition.raw_load_fields_from_cache(cache_manager,  d['uuid'], Condition.cached_fields ) )
			
			d['tries'             ] = d['tries'             ] if d['tries'             ] is not None else 0
			d['lastdone'          ] = d['lastdone'          ] if d['lastdone'          ] is not None else 0
			d['lastchecktotaltime'] = d['lastchecktotaltime'] if d['lastchecktotaltime'] is not None else 0

		return ret


	#
	# Configuration
	#
	def calculate_path(self):
		path = self.datagroup.calculate_path()
		path = path + DATAGROUP_PATH_SEP + self.name
		return path

	def update_path(self):
		self.path = self.calculate_path()

	def update_fields_from_dict(self, fields_dict, l=0):

		_DEBUG(l, self, "update_fields_from_dict()")

		#SPECIAL_FIELDS = [ 'dependson', ]
		#
		# Update normal fields
		#
		changed = False
		for name, f in fields_dict.items():
			#if name in SPECIAL_FIELDS:
			#	continue
			
			if self.update_single_field(f, l=l):
				changed = True
		#
		# Update special fields
		#
		#dep_path = None
		#f = fields_dict.get('dependson', None)
		#if f != None:
		#	dep_path = f.get_value()
		
		# DISABILITATO PER ORA	
		#if self.fix_dependson_field(dep_path, l=l+1):
		#	changed = True

		return changed

	def update_single_field(self, f, l=0):
		
		_DEBUG(l, f.name, f.get_value() )
		#
		# DISABILITATO PER ORA
		#
		#if f.name == 'dependson':
		#	path = f.get_value()
		#	return self.fix_dependson_field(path, l=l+1)
		
		if f.name == 'dependson':
			return False
		
		old = getattr(self, f.name)
		new = f.get_value()


		#if f.name == 'dependson':
		#	if new == DEP_NOT_CONFIGURED:
		#		new = None
		#	else:
		#		return False
		
		
		if old != new:
			_DEBUG(l+1, f.name, old, new)
			setattr(self, f.name, new)

			return True
			
		return False

	def update_single_field_and_save(self, f, mtime=None, l=0):

		if self.update_single_field(f, l=l):

			# FIX: Invece di "aggiustare" tutti i campi, dovremmo controllare solo il campo
			#      coinvolto da questo update. Pero' "aggiustare" solo un preciso 
			#      campo potrebbe non andare bene.
			#self.adjust_fields()
			
			#self.mtime = time.time()
			# FIX: UPDATE
			#self.save()
			#time.sleep(3)
			self.concurrent_save([f.name], new_sentinel_value=mtime)
			return True

		return False
	
	#
	# DISABILITATO PER ORA
	#
	#def fix_dependson_field(self, path, l=0):
	#	_DEBUG(l, "fix_dependson_field(%s)" % (path,))
	#
	#	c = None
	#	
	#	if path == None:
	#
	#		_DEBUG(l+1, "Searching primary condition inside %s" % (self.datagroup.path))
	#		
	#		local_primary_conditions = self.datagroup.conditions.filter(primary=True).order_by('name')
	#		if local_primary_conditions:
	#			c = local_primary_conditions[0]
	#			if c == self:
	#				c = None
	#
	#		if c == None:
	#			if self.datagroup.dependson != None:
	#				_DEBUG(l+1, "Datagroup depends on: %s" % (self.datagroup.dependson))
	#				c = self.datagroup.dependson
	#
	#		if c == None:
	#			_DEBUG(l+1, "Searching primary condition for datagroup %s" % (self.datagroup.path))
	#			c = self.datagroup._find_primary_condition(exclude_conditions=[self])
	#
	#		if c != None:
	#			if c == self:
	#				c = None
	#
	#		_DEBUG(l+2, "Calculated condition: %s" % (c))
	#
	#	elif path.lower() == TEMPLATE_NONE_VALUE:
	#		_DEBUG(l+1, "dependson set to None")
	#		c = None
	#	else: 
	#		try:
	#			_DEBUG(l+1, "configured dependson: %s. Searching..." % (path))
	#			c = Condition.objects.get(path=path)
	#		except Condition.DoesNotExist, e:
	#			_DEBUG(l+1, "dependson %s not found. Skip!" % (path))
	#			return False
	#
	#	_DEBUG(l+1, "new value for dependson: %s, old: %s" % (c, self.dependson))
	#
	#	if c != self.dependson:
	#		self.dependson = c
	#		return True
	#
	#	return False

	def adjust_fields(self):
		# FIX: quali altri campi campi?
		#self.fix_priority()

		try:
			# FIX: QUesta gestione non mi piace per niente! Tanto vale aggiungere un campo di comodo nel DB
			# invece di usare una cache.
			#from core.api.cache import init_cache_manager_from_conf
			cache_manager = self.get_cache_manager() or init_cache_manager_from_conf()
			self.expire_title(cache_manager=cache_manager)
		except Exception, e:
			print >> sys.stderr, "Unable to update expanded title for %s: %s %s" % (self, type(e), str(e))
			pass

	#def fix_priority(self):
	#	
	#	raise Excpetion("CONTROLLARE COSA C'E' NELLA CONFIGURAZIONE")
	#	
	#	e_priority = self.element.get_effective_priority()
	#	self.priority = max(self.priority, e_priority)

	#
	# Priority
	#
	def refresh_priority(self, min_priority=None, mtime=None):

		# check default
		if mtime is None: mtime = time.time()

		self.priority = min_priority

		self.concurrent_save(['priority'], new_sentinel_value=mtime)
		

	#
	# META
	#
	@staticmethod
	def get_fields():
		return [ (f.name, f.verbose_name) for f in Condition._meta.fields ] 	

	@staticmethod
	def get_fields_dict():
		return dict([ (f.name, f) for f in Condition._meta.fields ] )

	@staticmethod
	def validate_field_value(field_name, value):

		f = Condition.get_fields_dict()[field_name]
		v = f.get_prep_value(value)

		# FIX: Check if we can do with with a method from the model.
		if field_name == 'priority':
			if v < PRIORITY_MIN or PRIORITY_MAX < v:
				raise InvalidConfigurationException("Invalid priority value: %s (min=%s, max=%s)" % (v, PRIORITY_MIN, PRIORITY_MAX))

		elif field_name == 'expr':
			try:
				parseString(value)
			except ParserException, e:
				raise InvalidConfigurationException("Invalid expression: %s\nExpression: %s" % (e, value))

		elif field_name in ['msg_downsubj', 'msg_downbody', 'msg_upsubj','msg_upbody']:
			try:
				render_msg_template(value, {})
			except Exception, e:
				raise InvalidConfigurationException("Text string invalid: %s" % (str(e)))

		return v

	def update_depending_datagroup(self, mtime=None, l=0):

		_DEBUG(l, "Updating depending datagroups/condition on %s" % (self.path))
		
		path   = self.path
		tenant = self.tenant
		
		configured_dependson_fields = CDataGroupFieldValue.objects.filter(tenant=tenant, tfield__name='dependson', value=path)
		for cf in configured_dependson_fields:
			
			dg = cf.cdatagroup.get_datagroup()
			
			_DEBUG(l+1, "Checking datagroup %s" % (dg.path))
			
			need_update = False
			if dg.dependson != None:
				if dg.dependson.path != path:
					need_update = True
			else:
				need_update = True
				
			if not need_update:
				continue
			
			_DEBUG(l+2, "Updated")
			dg.dependson = self
			#dg.mtime = time.time()
			#dg.save()
			dg.concurrent_save(new_sentinel_value=mtime)
			

	#
	# LOG
	#
	def get_logs(self):
		return Event.objects.filter(Q(condition_path=self.path)|Q(condition_uuid=self.uuid))

	def logchanges_interval_sequence(self, start, end, timespec):
		# import da spostare
		from sanet_common.timeutils import get_working_timestamps_in_range

		states_sequence = []
		timestamps      = get_working_timestamps_in_range(start, end, timespec)

		for couple in timestamps:
			#print "start timestamp", couple[0]
			#print "end timestamp", couple[1]
			states_sequence += self.logchanges_sequence(couple[0], couple[1])

		return states_sequence

	# ATTENZIONE! interval DEVE essere specificato se end == None
	def logchanges_sequence(self, start, end=None, interval=None):

		#search_query = Event.objects.filter(condition_path=self.path)
		search_query = self.get_logs()

		states_sequence = []

		#
		# Ottimizzazione: statuslastchange indica che l'ultima variazione (e quindi LOG)
		#                 e' avvenuta prima di start, quindi tutto il periodo
		#                 tra "start" e "end" lo stato e' uguale allo stato corrente.
		#
		#      UP   DN   UP   FA             start               end       NOW
		#    --+----+----+----+--------------|-------------------|---------|------->
		#        UP   DN   UP       FA               FA               FA 
		#  
		#
		if start >= self.statuslastchange:
			end      = end if end is not None else time.time()
			duration = end - start
			status   = self.status
			e        = ( start, end, duration, status )

			states_sequence.append(e)

			return states_sequence

		log_changes = []

		if end is None:

			log_changes = search_query.filter(timestamp__gte=start).order_by('timestamp').all()

			# Caso ottimizzato.
			# Se "end" non e' specificato, e non ho trovato variazioni, allora posso "ragionevolemente" dedurre
			# che in tutto l'invervallo temporale lo stato della condition sia quello attuale.
			if len(log_changes) == 0:

				#        |                .
				# -------|--------------->.
				#        |      *         .
				#      start              UP
				end2     = time.time()

				duration = interval
				start    = end2 - interval
				status   = self.status
				e        = ( start, end2, duration, status )
				states_sequence.append(e)
		
		else:
			log_changes = search_query.filter( timestamp__gte=start, timestamp__lt=end).order_by('timestamp').all()

			if len(log_changes) == 0:
				from django.db.models import Max

				if search_query.filter( timestamp__lt=start).exists():

					last_ts = search_query.filter(timestamp__lt=start).aggregate(Max('timestamp'))['timestamp__max']

					changes = search_query.filter(timestamp=last_ts)[0:1]
					#  *->DN  |               |
					# --+-----|---------------|----------->
					#         |      DN       |
					#       start            end
					change   = changes[0]
					status   = change.status
					duration = end - start
				else: # no element found
					# NO CHANGES IN THE INTERVAL. FINDING FIRST AFTER END"
					changes = search_query.filter( timestamp__gte=end).order_by('timestamp')[0:1]
					if len(changes) > 0:
						#         |               | DN->*
						# --------|---------------|----+------>
						#         |      DN       |
						#       start            end
						change = changes[0]
						status = change.statuslast
						duration = end - start
					else:
						# Unable to find an entry inside the log. Assume
						# the current status (or statuslast) is valid.
						if self.statuslastchange > end:
							status = self.statuslast
						else:
							status = self.status
						duration = end - start

				e = ( int(start), int(end), int(duration), int(status) )
				states_sequence.append(e)

		# Se ho trovato delle variazioni, devo creare le tuple degli intervalli
		if len(log_changes) > 0:
			
			#  DN   |    UP         DN          UP          DN          UP    |
			# ------|----+----------+-----------+-----------+-----------+-----|----------->
			#       | DN |    UP    |     DN    |     UP    |     DN    | UP  |
			#       |                                                         |
			#     start                                                      end

			current_status   = None
			last_change_time = start

			for c in log_changes:
				if current_status == None:
					current_status = c.statuslast

				duration = c.timestamp - last_change_time

				e = ( int(last_change_time), int(c.timestamp), int(duration), int(current_status))
				states_sequence.append( e )

				current_status = c.status
				last_change_time = c.timestamp

			duration = 0
			if end is None:
				end = time.time()

			duration = end - last_change_time

			e = (int(last_change_time), int(end), int(duration), int(current_status))
			states_sequence.append( e )

		#
		# Compattiamo le tuple se ci sono delle tuple adiacenti con gli stessi stati
		#
		if len(states_sequence) > 0:

			states_sequence2 = []
			current = states_sequence.pop(0)
			while len(states_sequence) > 0:

				# elemento
				i = states_sequence.pop(0)

				# Se la tupla "current" e la tupla "i" hanno lo stesso stato e sono adiacenti temporalmente senza "buchi" in mezzo
				# allora creo una unica tupla fondendo "current" e "i"
				if current[3] == i[3]:
					if (current[1] == i[0]) or (current[1] == (i[0]-1)):
						current = (int(current[0]), int(i[1]), int(i[1] - current[0]), int(i[3]))
						continue
				states_sequence2.append(current)
				current = i
			states_sequence2.append(current)
			states_sequence = states_sequence2

		return states_sequence

	@property
	def node(self):
		return self.element.get_node()

	class Meta:
		unique_together = (
			"tenant", 
			"path",
		)
		db_table = "condition"

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

from django.template      import Template as DjangoTemplate, Context
from django.template.base import TemplateSyntaxError
from django.template.base import Variable
from django.template.base import VariableDoesNotExist

#
# I template di django espandono con stringa vuota i simboli non riconosciuti.
#
# Per fare questo e' necessario cambiare il comportamento del metodo:
#
#     Variable._resolve_lookup()
#
# Non possiamo "estendere la classe Variable di Django, quindi per 
# "sovrascrivere" questo metodo dobbiamo fare un "replace" del metodo 
# il metodo new_resolve_lookup().
#
# Questa "sostituzione rischai di generare riferimenti ricorsivi tra:
#
#    Variable._resolve_lookup --> new_resolve_lookup --+
#             ^                                        |
#             +----------------------------------------+
#

_ORIGINAL_DJANGO_VARIABLE_RESOLVE_LOOKUP = Variable._resolve_lookup

def _new_variable_resolve_lookup(self, *args, **kwargs):
	try:
		o = _ORIGINAL_DJANGO_VARIABLE_RESOLVE_LOOKUP(self, *args, **kwargs)
		if o is not None:
			return o
		else:
			return "*UNDEFINED*"
	except VariableDoesNotExist, e:
		return "*UNDEFINED*"
	except IndexError, e:
		return "*UNDEFINED*"
	except Exception, e:
		#print __file__, type(e), args, kwargs
		return "*ERROR (%s)*" % (str(e))

def render_msg_template(msg, template_symbols):

	#
	# NOTE: avoid prossible tracebacks if the template uses {{ params.items }} inside a {% for %}
	#    
	#
	if not template_symbols.has_key('params'):
		template_symbols['params'] = {}

	try:
		# Assicuriamoci che le variabili ventano espanse in maniera
		# "smart"
		Variable._resolve_lookup = _new_variable_resolve_lookup
		
		msg = _fix_wildcard_names_before_render(msg)
		
		# Per poter espandere correttamente tutto (anche i templatetags) dobbiamo gestire
		# il messaggio come un template quindi dobbiamo usare la sintasssi django
		# per importare tutto quello che serve.
		template_string = ''
		template_string += "{% load sanetd_notify %}"
		template_string += "{% autoescape off %}" 
		template_string	+= msg 
		template_string	+= "{% endautoescape %}"
		
		t = DjangoTemplate(template_string)
		c = Context(template_symbols)
		msg = t.render(c)
		
	except TemplateSyntaxError, e:
		raise Exception("Invalid syntax: %s" % (str(e)) )
		
	return msg

def _fix_wildcard_names_before_render(msg):
	"""
	Trova e sostituisce in "msg" tutte le occorrenze di {{ <name>.<name>.<name>....}} dove <name> contiene '-' e 
	sostituisce '-' con '_'.
	"""
	
	PATTERN = r'''{{[\s\w\.\-|]+}}'''  # FIX: Tutto cio che compare tra {{ }}. 
	
	wildcards = re.findall(PATTERN, msg)
	for w in wildcards:
		w_fixed = w
		w_fixed = w_fixed.replace('-', '_')
		msg = msg.replace(w,w_fixed)
	
	return msg

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class TimeGraph(TenantDependentModel, MTimeModel, UUIDKeyModel, ModelFieldsUtils, ExpandedTitleModel, CachedUtil, OwnerUserModel):
	
	# tenant
	# mtime
	# uuid

	datagroup    = models.ForeignKey(DataGroup, null=True, related_name="timegraphs")
	element      = models.ForeignKey(MonitoredElement, null=True)
	
	# Who created this element. User == NULL
	#user         = models.ForeignKey(User, null=True)

	name         = models.TextField(null=False, blank=False, db_index=True)	
	title        = models.TextField(null=False, blank=False, default='', db_index=True)

	# The main graph for the monitored element
	primary      = models.BooleanField(null=False, default=False)

	stacked      = models.BooleanField(null=False, default=True)

	ylegend      = models.TextField(null=False, default='', blank=True, db_index=False)

	deleted      = models.BooleanField(null=False, default=False)

	classification = models.TextField(null=False, blank=True, default='', db_index=True)


	objects        = models.Manager()
	active_objects = ActiveFilterManager()



	def __str__(self):
		return "<TimeGraph:%s:%s>" % (self.uuid, self.name)
		
	@property
	def path(self):
		return self.datagroup.path + ';' + self.name
		
	@staticmethod
	def get_by_path(path):
		dg_path, tg_name = path.rsplit(";", 1)
		return TimeGraph.objects.get(datagroup__path = dg_path, name = tg_name)

	def save(self, *args, **kwargs):
		#try:
		#	# FIX: QUesta gestione non mi piace per niente! Tanto vale aggiungere un campo di comodo nel DB
		#	# invece di usare una cache.
		#	from core.api.cache import init_cache_manager_from_conf
		#	cache_manager = self.get_cache_manager() or init_cache_manager_from_conf()
		#	self.update_expanded_title(cache_manager=cache_manager)
		#except Exception, e:
		#	print "Unable to update expanded title for %s:" % (self), e
		#	pass
			
		super(TimeGraph, self).save(*args, **kwargs)

	def adjust_fields(self):
		
		try:
			# FIX: QUesta gestione non mi piace per niente! Tanto vale aggiungere un campo di comodo nel DB
			# invece di usare una cache.
			#from core.api.cache import init_cache_manager_from_conf
			cache_manager = self.get_cache_manager() or init_cache_manager_from_conf()
			self.expire_title(cache_manager=cache_manager)
		except Exception, e:
			print >> sys.stderr, "Unable to update expanded title for %s: %s %s" % (self, type(e), str(e))
			pass


	#
	# Expanded title
	#
	def _subst_title(self, title_str):
		#print __file__, "_subst_title", title_str
		symbols = self.element.get_specific_element().get_title_symbols_dict()
		pd = self.get_title_symbols_dict()
		symbols.update( pd )
		#s = Template(title_str).safe_substitute(**symbols)
		#return s
		return ExpandedTitleModel.subst_title(title_str, symbols)

	def get_title_symbols_dict(self):		
		return self.datagroup.get_title_symbols_dict()
		
		
	
	@staticmethod
	def get_fields_dict():
		return dict([ (f.name, f) for f in TimeGraph._meta.fields ] )

	@staticmethod
	def validate_field_value(field_name, value):
		f = TimeGraph.get_fields_dict()[field_name]
		return f.get_prep_value(value)


	def update_fields_from_dict(self, fields_dict, l=0):
		
		_DEBUG(l, self, "update_fields_from_dict()")
		
		_DEBUG(l, "%s" % (fields_dict))
		
		updated = False
		for name, f in fields_dict.items():
			updated = self.update_single_field(f, l=l) or updated
			
		return updated

	def update_single_field(self, f, l=0):
		
		old = getattr(self, f.name)
		new = f.get_value()
		if old != new:
			_DEBUG(l+1, f.name, old, new)
			setattr(self, f.name, new)
			return True
			
		return False

	def update_single_field_and_save(self, f, mtime=None, l=0):
		if self.update_single_field(f, l=l):
			#self.mtime = time.time()
			# FIX: UPDATE
			#self.save()
			self.concurrent_save([f.name], new_sentinel_value=mtime)
			return True
		return False

	def get_real_series(self):
		return self.series.filter(deleted=False).filter(variable_value__isnull=False)

	def get_computed_series(self):
		return self.series.filter(deleted=False).filter(computed_value_expr__isnull=False)
		
	def get_state_from_datasources(self, datasources):
		
		state = CSTATE_UP
		for ds in datasources:
			if ds.status != DATASOURCE_STATUS_OK:
				state = CSTATE_FA
		return CSTATE_TO_CODENAME(state)

	class Meta:
		db_table = "timegraph"



class ActiveSerieFilterQuerySet(QuerySet):
	def active(self):
		return self.exclude(deleted=True).filter(timegraph__datagroup__deleted=False)

class ActiveSerieFilterManager(models.Manager):
	def get_query_set(self):
		return ActiveSerieFilterQuerySet(self.model).active()




class TimeGraphSerie(TenantDependentModel, MTimeModel, UUIDKeyModel, ModelFieldsUtils, OwnerUserModel):
	# tenant
	# mtime
	# uuid

	timegraph           = models.ForeignKey(TimeGraph, related_name="series")

	name                = models.TextField(null=False, blank=False, db_index=True)

	legend              = models.TextField(null=False, blank=True, default='', db_index=False)
	color               = models.TextField(null=False, default=DEFAULT_TIMEGRAPH_SERIE_COLOR , db_index=False)
	style               = models.TextField(null=False, blank=False, default=TIMEGRAPH_SERIE_STYLE_AREA, db_index=False)
	
	render_order        = models.PositiveSmallIntegerField(null=False, default=0)

	# 
	variable_value      = models.TextField(null=True, blank=True, default=None, db_index=False)
	computed_value_expr = models.TextField(null=True, blank=True, default=None, db_index=False)
	
	deleted      = models.BooleanField(null=False, default=False)


	objects        = models.Manager()
	active_objects = ActiveSerieFilterManager()


	def __str__(self):
		return "<TimeGraphSerie:%s:%s>" % (self.uuid, self.name)

	@staticmethod
	def get_fields_dict():
		return dict([ (f.name, f) for f in TimeGraphSerie._meta.fields ] )

	@staticmethod
	def validate_field_value(field_name, value):
		f = TimeGraphSerie.get_fields_dict()[field_name]
		value = f.get_prep_value(value)

		if field_name == 'computed_value_expr':
			validate_timegraphserie_expr(value)

		return value

	def update_fields_from_dict(self, fields_dict, l=0):

		_DEBUG(l, self, "update_fields_from_dict()")

		for name, f in fields_dict.items():
			self.update_single_field(f, l=l)

	def update_single_field(self, f, l=0):

		f_name = f.name
		if f_name == 'var_value':
			f_name = 'variable_value'

		old = getattr(self, f_name)
		new = f.get_value()
		if old != new:
			_DEBUG(l+1, "UPDATED", f_name, old, type(old), new, type(new))
			setattr(self, f_name, new)
			return True

		return False

	def update_single_field_and_save(self, f, mtime=None, l=0):
		if self.update_single_field(f, l=l):
			#self.mtime = time.time()
			# FIX: UPDATE
			#self.save()
			self.concurrent_save([f.name], new_sentinel_value=mtime)
			return True
		return False

	#
	# Expanded title
	#
	def _subst_title(self, title_str):
		#print __file__, "_subst_title", title_str
		symbols = self.element.get_specific_element().get_title_symbols_dict()
		pd = self.get_title_symbols_dict()
		symbols.update( pd )
		#s = Template(title_str).safe_substitute(**symbols)
		#return s
		return ExpandedTitleModel.subst_title(title_str, symbols)

	def get_title_symbols_dict(self):
		return self.datagroup.get_title_symbols_dict()


	class Meta:
		db_table = "timegraphserie"

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class Event(TenantDependentModel, MTimeModel, UUIDKeyModel):

	# tenant   =
	# mtime    = 
	# uuid     =

	kind             = models.SmallIntegerField(null=False, blank=False, db_index=True, )

	timestamp        = models.PositiveIntegerField(default=0, db_index=True)
	msg              = models.TextField(null=False, blank=True, db_index=False)
	element_path     = models.CharField(max_length=256, db_index=True)
	condition_path   = models.CharField(max_length=256, db_index=True)

	element_uuid     = models.CharField(max_length=MAX_UUID_LEN, null=True, db_index=True)
	condition_uuid   = models.CharField(max_length=MAX_UUID_LEN, null=True, db_index=True)

	# stato corrente
	status           = models.SmallIntegerField(null=True, db_index=True)

	# stato precedente
	statuslast       = models.SmallIntegerField(null=True, db_index=True)

	# timestamp dell'ultima variazione
	statuslastchange = models.PositiveIntegerField(blank=True, null=True, db_index=True)
	lastdone         = models.PositiveIntegerField(blank=True, null=True, db_index=True)
	penalty          = models.PositiveSmallIntegerField(blank=True, null=True, db_index=False)
	dampened         = models.NullBooleanField(blank=True, null=True, db_index=True)

	priority         = models.IntegerField(null=True, db_index=True)

	#verbstatus      -> msg

	# SysLog fields ( http://www.faqs.org/rfcs/rfc3164.html - 4.4.1 )
	sourceip         = models.CharField(max_length=64, blank=True, null=True, db_index=True)
	facility         = models.PositiveIntegerField(blank=True, null=True, db_index=True)
	severity         = models.PositiveIntegerField(blank=True, null=True, db_index=True)
	#text            -> msg

	# SNMP Trap fields
	#sysuptime       -> 'timestamp'    models.PositiveIntegerField(default=0, db_index=True, verbose_name=_('sysup time') ) 
	#sourceip        -> 'sourceip'   
	trapoid          = models.TextField(null=True, blank=True, db_index=True)
	varbindlist      = models.TextField(null=True, blank=True, db_index=False)

	# Conditions fields
	#timestamp       -> timestamp      models.PositiveIntegerField(default=0, db_index=True, verbose_name=_("timestamp")
	datetime         = models.CharField(max_length=20, null=True, blank=True, db_index=True) # AAAAMMGGHHMMSSD+HHSS
	timeticks        = models.PositiveIntegerField(default=0, null=True, blank=True, db_index=True)
	sysuptime        = models.PositiveIntegerField(default=0, null=True, blank=True, db_index=True)
	#text            -> msg            models.TextField(null=True, blank=True, db_index=True, verbose_name=_('oid'))

	class Meta:
		ordering = ('-timestamp',)
		db_table = "event"

	@property
	def condition(self):
		try:
			p = self.condition_path
			return Condition.objects.get(path = self.condition_path)
		except Condition.DoesNotExist, e:
			return None

	@staticmethod
	def log_custom_event(tenant, ts, source_ip, element_path, element_uuid, msg):
		event = Event()
		event.tenant       = tenant
		event.kind         = EVENT_CUSTOM
		event.timestamp    = int(ts)
		event.sourceip     = source_ip
		event.element_path = element_path
		event.element_uuid = element_uuid
		event.msg          = msg
		event.save()
		return event

	@staticmethod
	def hide_sensible_data_in_msg(msg):
		#SNMPV3_PATTERN='comm=(username:[^\s]*,sec_level:[^\s]*,auth_proto:[^\s]*,auth_pass:[^\s]*'
		SNMPV3_PATTERN='comm=[^\s]*' 
		return re.sub(SNMPV3_PATTERN, settings.HIDDEN_DATA, msg)

	def hide_sensible_data(self):
		self.msg = Event.hide_sensible_data_in_msg(self.msg)

	@property
	def element(self):

		try:
			parts = self.element_path.split(':')
			q = {}
			if len(parts) > 1:
				q["name"]=parts[1]
				q["parent__name"]=parts[0]
			else:
				q["name"]=parts[0]
				q["parent__name__isnull"]=True

			return MonitoredElement.objects.get(**q)

		except MonitoredElement.DoesNotExist, e:
			return None

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#


class Alarm(TenantDependentModel, MTimeModel, UUIDKeyModel):
	
	# tenant = 
	# mtime  = 
	# uuid   = 

	event = models.ForeignKey(Event)
	
	ack_needed = models.BooleanField(blank=True, null=False, default=False, db_index=True)
	ack        = models.ForeignKey('Ack', null=True)
	
	flow_information = models.TextField(null=True, default=None, db_index=False)

	def set_flow_informations(self, flow):
		self.flow_informations = json.dumps(flow)

	def get_flow_informations(self):
		if self.flow_information == None:
			return []
		return json.loads( self.flow_information )

	def append_flow_information(self, dict_data):
		flow = self.get_flow_informations()
		flow.append(dict_data)
		self.flow_information = json.dumps(flow)

	class Meta:
		db_table = "alarm"

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

class AcknowledgementType(TenantDependentModel, MTimeModel, UUIDKeyModel):

	name  = models.CharField(max_length=255, null=False, blank=False, db_index=True , unique=True)

	descr = models.TextField(null=False, blank=True , db_index=False, default='')

	class Meta:
		db_table = "acknowledgementtype"


class Ack(TenantDependentModel, MTimeModel, UUIDKeyModel):

	acktype   = models.ForeignKey(AcknowledgementType, null=False, db_index=True, db_column="acktype_id")
	timestamp = models.PositiveIntegerField(default=0, null=False, db_index=True)

	timestamplastchange = models.PositiveIntegerField(default=0, null=False, db_index=True)

	reason    = models.TextField(blank=False, null=False, db_index=False)
	user_name = models.TextField(blank=False, null=False, db_index=True)

	class Meta:
		db_table = "acknowledgementlog"	

#-----------------------------------------------------------------------------#
# CONFIGURATION                                                               #
#-----------------------------------------------------------------------------#

"""
Semplified view of tables and what you see in CLI:


             CLI                        WORKING                     CONFIGURATION
                                        TABLES                         TABLES                               CLI
                                                                                                                                 
                                 |                  |                                          |                                                                            
       datagroup-template dg     |                  |   TDG                                    |                               
          datasource ds          |                  |    . \                                   |                               
          exit                   |                  |    .  \                                  |                               
       exit                      |                  |    .   \                                 |                               
                                 |                  |    .    TDS                        TE    |                               
                                 |                  |    .                               .     |                               
                                 |                  |    .                               .     |                               
                                 |                  |    .                               .     |                               
       datagroup-template dg     |                  |    TDG...........................TEDG    |    node-template te           
          parent dg library      |                  |    . \                          /  .     |         datagroup dg          
          datasource ds          |                  |    .  \                        /   .     |               datasource ds   
          exit                   |                  |    .   \                      /    .     |                  title "a"    
       exit                      |                  |    .    TDS.............TEDSFV     .     |               exit            
                                 |                  |    .                      "a"      .     |         exit                  
                                 |                  |    .                               .     |    exit                       
                                 |                  |    .                               .     |                               
                                 |                  |    .                               .     |                               
       node x                    |                  |    .                               .     |                               
           template te           |                  |    .                               .     |                               
                                 |            DG....|....CDG                             .     |                               
           datagroup dg          |           /      |       \                            .     |                               
               datasource ds     |          /       |        \                           .     |                               
                   title "b"     |         /        |         \                          .     |                               
               exit              |        /         |          TDSFV                     .     |                               
           exit                  |      ME          |           "b"                      .     |                               
                                 |   "x"  \         |                                    .     |                               
                                 |         \        |                                    .     |                               
                                 |          \       |                                    .     |                               
                                 |           \      |                                    .     |                               
           template-datagroup dg |            DG....|....CDG..............................     |                               
               datasource ds     |                  |       \                                  |                               
                   title "c"     |                  |        \                                 |                               
               exit              |                  |         \                                |                               
           exit                  |                  |          TDSFV                           |                               
                                 |                  |           "c"                            |                               
        exit                     |                  |                                          |                               
                                 |                  |                                          |                                                                            
"""



class TField(models.Model):

	FIELD_DATAGROUP  = 1

	FIELD_DATASOURCE = 2

	FIELD_CONDITION  = 3

	FIELD_TGRAPH     = 4
	FIELD_TSERIE     = 5

	fclass = models.SmallIntegerField(null=False, blank=False, db_index=True)
	vtype  = models.TextField        (null=False, blank=False)
	name   = models.TextField        (null=False, blank=False, db_index=True)

	@staticmethod
	def datagroup_field(name):
		return TField.objects.get(fclass=TField.FIELD_DATAGROUP, name=name)

	def convert(self, str_value):

		if str_value is None:
			return None

		try:
			v = convert_to_field_type(self.vtype, str_value)
			return v
		except Exception, e:
			raise Exception("Unable to convert value '%s' for field '%s': %s" % (str_value, self.name, e) )

	@staticmethod
	def get_store_repr_value(value):
		return get_store_repr_value(value)


	def validate_value(self, str_value):
		# TFields string value -> runtime value
		value = self.convert(str_value)

		# Check if the "runtime value" is compatible with the model field
		if   self.fclass == TField.FIELD_DATAGROUP : return DataGroup.validate_field_value     (self.name, value)
		elif self.fclass == TField.FIELD_DATASOURCE: return DataSource.validate_field_value    (self.name, value)
		elif self.fclass == TField.FIELD_CONDITION : return Condition.validate_field_value     (self.name, value)
		elif self.fclass == TField.FIELD_TGRAPH    : return TimeGraph.validate_field_value     (self.name, value)
		elif self.fclass == TField.FIELD_TSERIE    : return TimeGraphSerie.validate_field_value(self.name, value)
		else: raise Exception("validation not implemetned for %s:%s" % (self.fclass, self.name))

	class Meta:
		db_table = "c_tfield"
		unique_together = (
			"fclass", 
			"name",
		)

#-----------------------------------------------------------------------------#

class TConfiguredFieldValue(models.Model):
	"""
	Modello comune a tutte le entita' che definisco un campo di configurazione.
	"""
	tfield = models.ForeignKey(TField, null=False)
	value  = models.TextField(null=True, blank=True, default=None)

	def __str__(self):
		s = "%s..." % ((self.value[0:20]) if (self.value is not None and len(self.value) > 20) else self.value)
		return "<%s:%s:%s>" % (self.__class__.__name__, self.tfield.name, s)

	@property
	def name(self):
		return self.tfield.name
		
	def get_value(self):
		return self.tfield.convert(self.value)

	class Meta:
		abstract = True

class TDataGroup(TenantRelatedModel, MTimeModel, UUIDKeyModel, OwnerUserModel):

	# tenant    =
	# mtime     =
	# uuid      =
	# owner     =

	source      = models.SmallIntegerField(null=False, blank=False, db_index=True, default=SOURCE_LIBRARY)
	name        = models.TextField        (null=False, blank=False, db_index=True)
	parent      = models.ForeignKey       ('self', null=True, blank=True, db_index=True, default=None)
	
	description = models.TextField(null=False, default='')
	
	def __str__(self):
		if self.tenant_id:
			return "<%s,%s,%s>" % (self.name, SOURCE_NAME(self.source), self.tenant)
		else:
			return "<%s,%s>" % (self.name, SOURCE_NAME(self.source))

	@staticmethod
	def get_visible_by_user(user, tenant=None):

		query = TDataGroup.objects

		if tenant is None:
			query = query.filter(source=SOURCE_LIBRARY)
		else:
			if user and not user.is_superuser:

				groups = user.groups.all()
				uuids = [ r[0] for r in TDataGroupPermission.objects.filter(tenant=tenant, group__in=groups).values_list('tdatagroup_id')]

				query = query.filter(Q(owner=user) | Q(source=SOURCE_LIBRARY)| Q(source=SOURCE_LOCAL, tenant=tenant, uuid__in=uuids))
			else:
				query = query.filter( Q(source=SOURCE_LOCAL, tenant=tenant) | Q(source=SOURCE_LIBRARY))

		return query

	def assert_permission_readwrite(self, user):

		if user.is_superuser: 
			return True

		if self.owner == user:
			return True

		if TDataGroupPermission.objects.filter(tenant=self.tenant, tdatagroup=self, group__in=user.groups.all(),  auth_type=AUTH_READWRITE).count() > 0:
			return True

		raise ApplicationException("User '%s' has not READ-WRITE permission on template '%s'" % (user.username, self))



	def rename_parameter(self, param_name, new_name):
		try:
			p = self.tdatagroupparametervalue_set.get(name=param_name)
			p.name = new_name
			p.save()
		except TDataGroupParameterValue.DoesNotExist, e:
			# FIX: in certi casi e' obblicagorio che questo elemento esista.
			#      prevedere due casi e forzare l'errore con un flag?
			pass

	def rename_datasource(self, old_name, new_name):
		try:
			tds = self.tdatasource_set.get(name=old_name)
			tds.name = new_name
			tds.save()
		except TDataSource.DoesNotExist, e:
			# FIX: in certi casi e' obblicagorio che questo elemento esista.
			#      prevedere due casi e forzare l'errore con un flag?
			pass

	def rename_condition(self, old_name, new_name):
		try:
			tc = self.tcondition_set.get(name=old_name)
			tc.name = new_name
			tc.save()
		except TCondition.DoesNotExist, e:
			# FIX: in certi casi e' obblicagorio che questo elemento esista.
			#      prevedere due casi e forzare l'errore con un flag?
			pass

	def rename_timegraph(self, old_name, new_name):
		try:
			ttg = self.ttimegraph_set.get(name=old_name)
			ttg.name = new_name
			ttg.save()
		except TTimeGraph.DoesNotExist, e:
			# FIX: in certi casi e' obblicagorio che questo elemento esista.
			#      prevedere due casi e forzare l'errore con un flag?
			pass

	def rename_timegraphserie(self, tg_name, old_name, new_name):
		try:
			ttg = self.ttimegraph_set.get(name=tg_name)
			ttgs = ttg.ttimegraphserie_set.get(name=old_name)
			ttgs.name = new_name
			ttgs.save()
		except TTimeGraphSerie.DoesNotExist, e:
			# FIX: in certi casi e' obblicagorio che questo elemento esista.
			#      prevedere due casi e forzare l'errore con un flag?
			pass





	def duplicate(self):
		return TDataGroup(
			source=SOURCE_LIBRARY,
			parent=self,
			name=self.name,
			description=self.description,
		)
		
	def used_count(self):
		c = self.additional_datagroups.count()
		for te in self.tedatagroup_set.all():
			c += int(te.telement.elementtemplate_set.count())
		return c

	def indirect_used_count(self):
		c = 0
		for child_tdg in self.tdatagroup_set.all():
			c += child_tdg.used_count()
		return c

	def get_ancestors_tree(self, include_self=True):
		
		tree = []
		
		if include_self:
			tree.append(self)
			
		current = self.parent
		while current != None:
			tree.append(current)
			current = current.parent
			
		tree.reverse()
		
		return tree

	def get_first_ancestor(self):
		return self.get_ancestors_tree()[0]		


	def get_childrens(self):
		return self.tdatagroup_set.all()
		
	def get_recursive_children(self):
		r = []
		for c in self.get_childrens():
			r.append(c)
			r += c.get_recursive_children()
			
		return r


	def get_defined_datasources(self):
		tdgs = self.get_ancestors_tree()
		return TDataSource.objects.filter(tdatagroup__in=tdgs)

	def get_defined_conditions(self):
		tdgs = self.get_ancestors_tree()
		return TCondition.objects.filter(tdatagroup__in=tdgs)

	def get_defined_timegraphs(self):
		tdgs = self.get_ancestors_tree()
		return TTimeGraph.objects.filter(tdatagroup__in=tdgs)

	def get_defined_timegraph_series(self, timegraph_name):
		tdgs = self.get_ancestors_tree()
		
		ttgs = TTimeGraph.objects.filter(tdatagroup__in=tdgs, name=timegraph_name)
		return TTimeGraphSerie.objects.filter(ttimegraph__in=ttgs)

	def get_local_defined_datasources(self):
		return self.tdatasource_set
		
	def get_local_defined_conditions(self):
		return self.tcondition_set

	def get_local_defined_timegraphs(self):
		return self.ttimegraph_set

	def get_last_defined_datasource(self, name):

		current = self
		while current != None:
			try:
				return current.tdatasource_set.get(name=name)
			except TDataSource.DoesNotExist, e:
				pass
			current = current.parent

		raise TDataSource.DoesNotExist("%s non defined in any template" % (name))

	def get_last_defined_condition(self, name):

		current = self

		while current != None:
			try:
				return current.tcondition_set.get(name=name)
			except TCondition.DoesNotExist, e:
				pass
			current = current.parent

		raise TCondition.DoesNotExist("%s non defined in any template" % (name))

	def get_last_defined_timegraph(self, name):
		current = self
		while current != None:
			try:
				return current.ttimegraph_set.get(name=name)
			except TTimeGraph.DoesNotExist, e:
				pass
			current = current.parent
		raise TTimeGraph.DoesNotExist("%s non defined in any template" % (name))

	def get_last_defined_timegraphserie(self, tg_name, name):
		current = self
		while current != None:
			try:
				#tg = current.ttimegraph_set.get(name=tg_name)
				#return tg.ttimegraphserie_set.get(name=name)
				return TTimeGraphSerie.objects.get(ttimegraph__tdatagroup=current, ttimegraph__name=tg_name, name=name)
			except TTimeGraph.DoesNotExist, e:
				pass
			except TTimeGraphSerie.DoesNotExist, e:
				pass
			current = current.parent
		raise TTimeGraphSerie.DoesNotExist("%s:%s non defined in any template" % (tg_name, name))

	#
	# Local configured fields
	#
	def get_local_configured_field_value(self, tfield):
		try:
			self.tdatagroupfieldvalue_set.get(tenant=self.tenant, tfield=tfield)
		except TDataGroupFieldValue.DoesNotExist, e:
			return None
			
	def get_local_configured_field_dict(self):
		d = {}
		field_values = self.tdatagroupfieldvalue_set.filter(tenant=self.tenant).select_related("tfield")
		for f in field_values:
			d[ f.tfield.name ] = f.value
		return d	

	#
	# Local configured parameters
	#
	def get_local_configured_parameter_value(self, name):
		try:
			return self.tdatagroupparametervalue_set.get(name=name)
		except TDataGroupParameterValue.DoesNotExist, e:
			return None

	def get_local_configured_parameters(self):
		return self.tdatagroupparametervalue_set.filter(tenant=self.tenant).all()
			
		
	#
	# Local configured datasources
	#
	def get_local_configured_datasource_field_value(self, ds_name, tfield):
		try:
			tds = self.tdatasource_set.get(tenant=self.tenant, name=ds_name)
			return tds.tdatasourcefieldvalue_set.get(tenant=self.tenant, tfield=tfield)
		except TDataSourceFieldValue.DoesNotExist, e:
			return None
		except TDataSource.DoesNotExist, e:
			return None
			
	#
	# Local configured conditions
	#
	def get_local_configured_condition_field_value(self, c_name, tfield):
		try:
			tds = self.tcondition_set.get(tenant=self.tenant, name=c_name)
			return tds.tconditionfieldvalue_set.get(tenant=self.tenant, tfield=tfield)
		except TCondition.DoesNotExist, e:
			return None
		except TConditionFieldValue.DoesNotExist, e:
			return None

	# 
	# Local configured timegraphs
	#
	def get_local_configured_timegraph_field_value(self, tg_name, tfield):
		try:
			ttg = self.ttimegraph_set.get(tenant=self.tenant, name=tg_name)
			return ttg.ttimegraphfieldvalue_set.get(tenant=self.tenant, tfield=tfield)
		except TTimeGraph.DoesNotExist, e:
			return None
		except TTimeGraphFieldValue.DoesNotExist, e:
			return None
			
	def get_local_configured_timegraphserie_field_value(self, tg_name, serie_name, tfield):
		try:
			ttg = self.ttimegraph_set.get(tenant=self.tenant, name=tg_name)
			ttgs = ttg.ttimegraphserie_set.get(tenant=self.tenant, name=serie_name)
			return ttgs.ttimegraphseriefieldvalue_set.get(tenant=self.tenant, tfield=tfield)
		except TTimeGraph.DoesNotExist, e:
			return None
		except TTimeGraphSerie.DoesNotExist, e:
			return None
		except TTimeGraphSerieFieldValue.DoesNotExist, e:
			return None
			
			
			
			

	def get_local_timegraphs_config_dict(self, l=0):
		
		l=l+1
		
		tg_config_by_name = {}
		for ttg in self.ttimegraph_set.filter(tenant=self.tenant).all():
			_DEBUG(l, ttg.name)
			d = {
				'fields': ttg.get_configured_fields_dict(l=l),
				'series': ttg.get_configured_series_dict(l=l),
			}
			tg_config_by_name[ ttg.name ] = d

		return tg_config_by_name

	def get_configured_param(self, param_name, only_anchestors=False):
		config = self.get_configured_params_dict(l=0, filter_names=[param_name,], only_anchestors=only_anchestors) #FIX: 
		if config:
			return config[param_name]
		return None

	def get_configured_params(self, filter_names=None, only_anchestors=False, l=0):
		return self.get_configured_params_dict(filter_names=filter_names, only_anchestors=only_anchestors, l=l).values()
		
	def get_configured_params_dict(self, filter_names=None, only_anchestors=False, l=0):
		
		_DEBUG(l, self, "get_configured_params")
		l=l+1
		
		configured_params = {}

		if only_anchestors:

			current = self.parent
		else:
			current = self
			
		while current != None:
			_DEBUG(l, "selecting configured fields for", current)
			params = current.tdatagroupparametervalue_set.all()
			for f in params: 
			
				if filter_names: 
					if not (f.name in filter_names):
						continue
			
				if not configured_params.has_key(f.name):
					configured_params[f.name] = f
					_DEBUG(l+1, f)

			current = current.parent

		return configured_params

	#
	# Configured fields
	#
	def get_configured_field(self, name):
		
		config = self.get_configured_fields_dict(selected_fields=[name,])
		return config[name]


	def get_configured_field_or_default(self, name):

		config = self.get_configured_fields_dict(selected_fields=[name,])
		if config.has_key(name):
			return config[name]
		else:
			memory_value = ModelFieldsUtils.get_model_fields_default_value_dict( DataGroup )[name]
			tf       = TField.objects.get(fclass=TField.FIELD_DATAGROUP, name=name)
			if memory_value is not None:
				tf_value = TField.get_store_repr_value(memory_value)
			else:
				tf_value = None
			v = TDataGroupFieldValue(tfield=tf, value=tf_value)

			return v


	def get_configured_fields(self, selected_fields=None, l=0):
		
		configured_fields_dict = self.get_configured_fields_dict(selected_fields=selected_fields, l=l)
		
		return configured_fields_dict.values()
		

	def get_configured_fields_or_default(self, selected_fields=None, l=0):
		
		configured_fields_dict = self.get_configured_fields_dict(selected_fields=selected_fields, l=l)
		
		tfields = TField.objects.filter(fclass=TField.FIELD_DATAGROUP)
		if selected_fields: 
			tfields = tfields.filter(name__in=selected_fields)
		
		for tf in tfields:

			#print tf.name

			if configured_fields_dict.has_key(tf.name):
				continue
			
			memory_value = ModelFieldsUtils.get_model_fields_default_value_dict( DataGroup )[tf.name]
			tf       = TField.objects.get(fclass=TField.FIELD_DATAGROUP, name=tf.name)
			if memory_value is not None:
				tf_value = TField.get_store_repr_value(memory_value)
			else:
				tf_value = None
			v = TDataGroupFieldValue(tfield=tf, value=tf_value)
			
			configured_fields_dict[tf.name] = v
		
		return configured_fields_dict.values()
			
			
		

	def get_configured_fields_dict(self, selected_fields=None, l=0):

		configured_fields = {}

		current = self
		while current != None:
			_DEBUG(l, "selecting configured fields for", current)
			fields = current.tdatagroupfieldvalue_set.all()
			for f in fields: 
			
				if selected_fields:
					if not (f.name in selected_fields):
						continue
			
				if not configured_fields.has_key(f.name):
					configured_fields[f.name] = f
					_DEBUG(l+1, f.name)

			current = current.parent

		return configured_fields



	#
	# Configured Datasources
	#
	def get_configured_datasource_field_value_or_default(self, ds_name, f_name):

		config = self.get_configured_datasources(selected_datasource=[ds_name,], selected_field=[f_name,])

		ds_config = config[ds_name]
		if ds_config.has_key(f_name):
			return ds_config[f_name]

		else:
			memory_value = ModelFieldsUtils.get_model_fields_default_value_dict( DataSource )[f_name]

			tf      = TField.objects.get(fclass=TField.FIELD_DATASOURCE, name=f_name)
			if memory_value is not None:
				tf_value = TField.get_store_repr_value(memory_value)
			else:
				tf_value = None
			v = TDataSourceFieldValue(tfield=tf, value=tf_value)
			return v

	def get_configured_datasources(self, selected_datasource=None, selected_field=None, l=0):

		_DEBUG(l, self, "get_configured_datasources")
		l=l+1

		tree = self.get_ancestors_tree()

		_DEBUG(l, "TREE")
		for t in tree: _DEBUG(l+1, t)

		effective_datasources = {}
		for tdg in tree:

			_DEBUG(l, tdg)

			datasources_dict = tdg._get_datasources_fields_dict(selected_datasource=selected_datasource, selected_field=selected_field,    l=l+1)

			for name, ds_fields_dict in datasources_dict.items():

				_DEBUG(l+1, name)

				effective_fields = effective_datasources.get(name,None)

				if effective_fields == None:
					_DEBUG(l+2, "not configured fields")
				else:
					for fname, d  in effective_fields.items():
						_DEBUG(l+2, fname, d)

				if effective_fields == None:
					effective_fields = {}
					effective_datasources[name] = effective_fields
					
				effective_fields.update(ds_fields_dict)

		return effective_datasources
	
	def _get_datasources_fields_dict(self, selected_datasource=None, selected_field=None, l=0):
		
		_DEBUG(l, self, "Reading local datasources")
		l=l+1
		
		ds_fields_by_name = {}
		
		tdatasources = self.tdatasource_set.filter(tenant=self.tenant)
		if selected_datasource:
			tdatasources = tdatasources.filter(name__in=selected_datasource)

		for tds in tdatasources.all():
			
			_DEBUG(l, "Reading fields for ds ", tds.name)
			
			fields = tds.get_configured_fields(selected_field=selected_field, l=l)
			for f in fields:
				_DEBUG(l+1, f)
				
			d = dict( [ (f.name, f) for f in fields ] )
			ds_fields_by_name[tds.name] = d

		return ds_fields_by_name



	#
	# Configured Conditions
	#
	def get_configured_condition_field_value_or_default(self, c_name, f_name):
		
		config = self.get_configured_conditions(selected_conditions=[c_name,], selected_fields=[f_name,])
		
		c_config = config[c_name]
		if c_config.has_key(f_name):
			return c_config[f_name]
		else:
			memory_value = ModelFieldsUtils.get_model_fields_default_value_dict( Condition )[f_name]

			tf = TField.objects.get(fclass=TField.FIELD_CONDITION, name=f_name)
			if memory_value is not None:
				tf_value = TField.get_store_repr_value(memory_value)
			else:
				tf_value = None
			
			v = TConditionFieldValue(tfield=tf, value=tf_value)
			return v


	def get_configured_conditions(self, selected_conditions=None, selected_fields=None, l=0):

		_DEBUG(l, self, "get_configured_conditions")
		l=l+1

		tree = self.get_ancestors_tree()

		_DEBUG(l, "TREE")
		for t in tree: _DEBUG(l+1, t)

		effective_conditions = {}
		for tdg in tree:

			_DEBUG(l, tdg)

			conditions_dict = tdg.get_conditions_fields_dict(selected_conditions=selected_conditions, selected_fields=selected_fields, l=l+1)

			for name, c_fields_dict in conditions_dict.items():

				_DEBUG(l+1, name)

				effective_fields = effective_conditions.get(name,None)
				
				if effective_fields == None:
					_DEBUG(l+2, "not configured fields")
				else:
					for fname, d  in effective_fields.items():
						_DEBUG(l+2, fname, d)

				if effective_fields == None:
					effective_fields = {}
					effective_conditions[name] = effective_fields
					
				effective_fields.update(c_fields_dict)

		return effective_conditions

	def get_conditions_fields_dict(self, selected_conditions=None, selected_fields=None, l=0):
		
		_DEBUG(l, self, "Reading local conditions")
		l=l+1
		
		c_fields_by_name = {}
		
		tconditions = self.tcondition_set.filter(tenant=self.tenant)
		if selected_conditions:
			tconditions = tconditions.filter(name__in=selected_conditions)
		
		
		for tc in tconditions.all():
			
			_DEBUG(l, "Reading fields for condition ", tc.name)
			
			fields = tc.get_configured_fields(selected_fields=selected_fields, l=l)
			
			for f in fields:
				_DEBUG(l+1, f)
				
			d = dict( [ (f.name, f) for f in fields ] )
			c_fields_by_name[tc.name] = d

		return c_fields_by_name


	#
	# Configured Timegraphs
	#

	def get_configured_timegraph_field_value_or_default(self, tg_name, field_name):

		try:
			v = self.get_configured_timegraph_field(tg_name, field_name)
			return v
		except KeyError, e:
			pass

		memory_value = ModelFieldsUtils.get_model_fields_default_value_dict( TimeGraph )[field_name]

		tf       = TField.objects.get(fclass=TField.FIELD_TGRAPH, name=field_name)
		if memory_value is not None:
			tf_value = TField.get_store_repr_value(memory_value)
		else:
			tf_value = None

		v = TTimeGraphFieldValue(tfield=tf, value=tf_value)

		return v

	def get_configured_timegraphserie_field_value_or_default(self, tg_name, serie_name, field_name):

		try:
			v = self.get_configured_timegraphserie_field(tg_name, serie_name, field_name)
			return v
		except KeyError, e:
			pass

		memory_value = ModelFieldsUtils.get_model_fields_default_value_dict( TimeGraphSerie )[field_name]

		tf       = TField.objects.get(fclass=TField.FIELD_TSERIE, name=field_name)

		if memory_value is not None:
			tf_value = TField.get_store_repr_value(memory_value)
		else:
			tf_value = None

		v = TTimeGraphSerieFieldValue(tfield=tf, value=tf_value)

		return v



	def get_configured_timegraph_field(self, tg_name, fname, l=0):

		config = self.get_configured_timegraphs(l=l)
		return config[tg_name]['fields'][fname]

	def get_configured_timegraphserie_field(self, tg_name, serie_name, fname, l=0):

		config = self.get_configured_timegraphs(l=l)
		return config[tg_name]['series'][serie_name][fname]


	def get_configured_timegraphs_with_default(self, l=0):

		config = self.get_configured_timegraphs(l=l)

		for tg_name, tg_config in config.items():

			for tf in TField.objects.filter(fclass=TField.FIELD_TGRAPH).all():

				if tf.name not in tg_config['fields']:
					memory_value = ModelFieldsUtils.get_model_fields_default_value_dict( TimeGraph )[tf.name]
					if memory_value is not None:
						tf_value = TField.get_store_repr_value(memory_value)
					else:
						tf_value = None
					v = TTimeGraphFieldValue(tfield=tf, value=tf_value)

					tg_config['fields'][tf.name] = v

			for serie_name, serie_fields in tg_config['series'].items():

				for tf in TField.objects.filter(fclass=TField.FIELD_TSERIE).all():
					if tf.name not in serie_fields:
						memory_value = ModelFieldsUtils.get_model_fields_default_value_dict( TimeGraphSerie )[tf.name]
						if memory_value is not None:
							tf_value = TField.get_store_repr_value(memory_value)
						else:
							tf_value = None
						v = TTimeGraphSerieFieldValue(tfield=tf, value=tf_value)
						serie_fields[tf.name] = v
		return config


	def get_configured_timegraphs(self, l=0):
		_DEBUG(l, self, "get_configured_timegraphs")
		l=l+1

		tree = self.get_ancestors_tree()

		_DEBUG(l, "TREE")
		for t in tree: _DEBUG(l+1, t)

		effective_timegraphs = {}
		for tdg in tree:

			l=l+1
			_DEBUG(l, tdg)

			timegraphs_dict = tdg.get_local_timegraphs_config_dict(l=l+1)

			_DEBUG(l, "Updating effective timegraphs")
			for tg_name, tg_config_dict in timegraphs_dict.items():

				_DEBUG(l+1, tg_name)

				effective_config = effective_timegraphs.get(tg_name,None)

				if effective_config == None:
					effective_config = { 'fields': {} , 'series': {} }
					effective_timegraphs[tg_name] = effective_config

				effective_config['fields'].update(tg_config_dict['fields'])

				for serie_name, serie_fields in tg_config_dict['series'].items():
					_DEBUG(l+2, "updating serie", serie_name)
					e_serie_config = effective_config['series'].get(serie_name, None)
					if e_serie_config == None:
						e_serie_config = {}
						effective_config['series'][serie_name] = e_serie_config

					e_serie_config.update(serie_fields)

		return effective_timegraphs


	def set_owner(self, user):
		self.owner = user
		self.save()

	def unset_owner(self):
		self.owner = None
		self.save()

	def add_group_permission(self, group, auth_type):
		tep, created = TDataGroupPermission.objects.get_or_create(tenant=self.tenant, tdatagroup=self, group=group, auth_type=auth_type)
		tep.save()

	def remove_group_permission(self, group):
		tep = TDataGroupPermission.objects.filter(tenant=self.tenant, tdatagroup=self, group=group)
		tep.delete()

	def get_group_permissions(self):
		return TDataGroupPermission.objects.filter(tenant=self.tenant, tdatagroup=self).order_by('group')



	class Meta:
		db_table = "c_tdatagroup"
		unique_together = (
			"tenant",
			"source",
			"name",
		)


class TDataGroupFieldValue(TenantRelatedModel, MTimeModel, UUIDKeyModel, TConfiguredFieldValue):

	# tenant    =
	# mtime     =
	# uuid      =
	tdatagroup        = models.ForeignKey(TDataGroup, null=False)

	def duplicate(self, tdg):
		return TDataGroupFieldValue(tdatagroup=tdg, tfield=self.tfield, value=self.value)
	
	class Meta:
		db_table = "c_tdatagroupfieldvalue"
		unique_together = (
			"tenant", 
			"tdatagroup",
			"tfield"
		)

class TDataGroupParameterValue(TenantRelatedModel, MTimeModel, UUIDKeyModel, Parameter):
	
	# tenant    =
	# mtime     =
	# uuid      =
	tdatagroup  = models.ForeignKey(TDataGroup, null=False)
	
	class Meta:
		db_table  = "c_tdatagroupparametervalue"
		unique_together = (
			"tenant",
			"tdatagroup", 
			"name", 
		)

#-----------------------------------------------------------------------------#

class TDataSource(TenantRelatedModel, MTimeModel, UUIDKeyModel):

	tdatagroup        = models.ForeignKey(TDataGroup, null=False, blank=False, db_index=True)
	name              = models.TextField (null=False, blank=False, db_index=True)

	def duplicate(self, tdg):
		return TDataSource(tdatagroup=tdg, name=self.name)

	def get_configured_fields(self, selected_field=None, l=0):

		fields = self.tdatasourcefieldvalue_set.filter(tenant=self.tenant).select_related('tfield')
		if selected_field:
			fields = fields.filter(tfield__name__in=selected_field)

		return fields.all()
		
	def get_ancestors(self, include_self=True):
		anchestors = self.tdatagroup.get_ancestors_tree(include_self=include_self)
		r = TDataSource.objects.filter(tdatagroup__in=anchestors, name=self.name).all()
		return r

	def get_ancestors_tree(self, include_self=True):
		anchestors = self.tdatagroup.get_ancestors_tree(include_self=include_self)
		r = []
		for tdg in anchestors:
			tds = tdg.tdatasource_set.filter(name=self.name).all()[0:]
			if len(tds) == 0:
				continue
			if len(tds) > 1:
				raise Exception("2 datasources with same name")
			r.append( tds[0] )
		return r

	class Meta:
		db_table = "c_tdatasource"
		unique_together = (
			"tenant", 
			"tdatagroup",
			"name",
		)

class TDataSourceFieldValue(TenantRelatedModel, MTimeModel, UUIDKeyModel, TConfiguredFieldValue):

	tdatasource        = models.ForeignKey(TDataSource, null=False)

	class Meta:
		db_table = "c_tdatasourcefieldvalue"
		unique_together = (
			"tenant", 
			"tdatasource",
			"tfield",
		)

#-----------------------------------------------------------------------------#

class TCondition(TenantRelatedModel, MTimeModel, UUIDKeyModel):

	tdatagroup         = models.ForeignKey(TDataGroup, null=False, blank=False, db_index=True)

	name               = models.TextField (null=False, blank=False, db_index=True)
	
	def duplicate(self, tdg):
		return TCondition(tdatagroup=tdg, name=self.name)

	def get_configured_fields(self, selected_fields=None, l=0):
		
		fields = self.tconditionfieldvalue_set.filter(tenant=self.tenant).select_related('tfield')
		
		if selected_fields:
			fields = fields.filter(tfield__name__in=selected_fields)
			
		return fields.all()

	def get_ancestors(self, include_self=True):
		anchestors = self.tdatagroup.get_ancestors_tree(include_self=include_self)
		r = TCondition.objects.filter(tdatagroup__in=anchestors, name=self.name).all()
		return r

	def get_ancestors_tree(self, include_self=True):
		anchestors = self.tdatagroup.get_ancestors_tree(include_self=include_self)
		r = []
		for tdg in anchestors:
			tc = tdg.tcondition_set.filter(name=self.name).all()[0:]
			if len(tc) == 0:
				continue
			if len(tc) > 1:
				raise Exception("2 TCondition with same name")
			r.append( tc[0] )
		return r

	class Meta:
		db_table = "c_tcondition"
		unique_together = (
			"tenant", 
			"tdatagroup",
			"name",
		)

class TConditionFieldValue(TenantRelatedModel, MTimeModel, UUIDKeyModel, TConfiguredFieldValue):

	tcondition         = models.ForeignKey(TCondition, null=False)

	class Meta:
		db_table = "c_tconditionfieldvalue"
		unique_together = (
			"tenant", 
			"tcondition",
			"tfield",
		)

class TTimeGraph(TenantRelatedModel, MTimeModel, UUIDKeyModel):

	tdatagroup         = models.ForeignKey(TDataGroup, null=False, blank=False, db_index=True)
	name               = models.TextField (null=False, blank=False, db_index=True)
	
	def get_configured_fields_dict(self, l=0):
		return dict( [ (f.tfield.name, f) for f in self.ttimegraphfieldvalue_set.filter(tenant=self.tenant).all() ] )
		
	def get_configured_series_dict(self, l=0):
		d = {}
		for ttgs in self.ttimegraphserie_set.filter(tenant=self.tenant).all():
			d[ ttgs.name ] = ttgs.get_configured_fields_dict()
		return d


	def get_ancestors_tree(self, include_self=True):
		anchestors = self.tdatagroup.get_ancestors_tree(include_self=include_self)
		r = []
		for tdg in anchestors:
			ttg = tdg.ttimegraph_set.filter(name=self.name).all()[0:]
			if len(ttg) == 0:
				continue
			if len(ttg) > 1:
				raise Exception("2 datasources with same name")
			r.append( ttg[0] )
		return r


	class Meta:	
		db_table = "c_ttimegraph"
		unique_together = (
			"tenant",
			"tdatagroup", 
			"name",
		)

class TTimeGraphFieldValue(TenantRelatedModel, MTimeModel, UUIDKeyModel, TConfiguredFieldValue):

	ttimegraph         = models.ForeignKey(TTimeGraph, null=False)

	class Meta:	
		db_table = "c_ttimegraphfieldvalue"
		unique_together = (
			"tenant",
			"ttimegraph", 
			"tfield",
		)

class TTimeGraphSerie(TenantRelatedModel, MTimeModel, UUIDKeyModel):
	
	ttimegraph = models.ForeignKey(TTimeGraph)
	name      = models.TextField(null=False, blank=False, db_index=True)
	
	def get_configured_fields_dict(self, l=0):
		return dict( [ (f.tfield.name, f) for f in self.ttimegraphseriefieldvalue_set.filter(tenant=self.tenant).all() ] )

	def get_ancestors_tree(self, include_self=True):

		ttg_anchestors = self.ttimegraph.get_ancestors_tree(include_self=include_self)
		
		r = []
		for ttgs in ttg_anchestors:
			ttgs = ttgs.ttimegraphserie_set.filter(name=self.name).all()[0:]
			if len(ttgs) == 0:
				continue
			if len(ttgs) > 1:
				raise Exception("2 datasources with same name")
			r.append( ttgs[0] )
		return r


	class Meta:	
		db_table = "c_ttimegraphserie"
		unique_together = (
			"tenant",
			"ttimegraph",
			"name", 
		)

class TTimeGraphSerieFieldValue(TenantRelatedModel, MTimeModel, UUIDKeyModel, TConfiguredFieldValue):
	
	ttimegraphserie    = models.ForeignKey(TTimeGraphSerie, null=False)

	class Meta:
		db_table = "c_ttimegraphseriefieldvalue"
		unique_together = (
			"tenant",
			"ttimegraphserie",
			"tfield", 
		)

#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#


class TEDataGroup(TenantRelatedModel):
	# tenant   =
	telement   = models.ForeignKey(TElement  , null=False)
	tdatagroup = models.ForeignKey(TDataGroup, null=False)

	name       = models.TextField(null=True, blank=False, default=None)

	def __str__(self):
		return "<TEDataGroup:%s:%s:%s>" %(self.name,self.telement,self.tdatagroup)

	def rename_parameter(self, param_name, new_name):
		try:
			p = self.tedatagroupparametervalue_set.get(name=param_name)
			p.name = new_name
			p.save()
		except TEDataGroupParameterValue.DoesNotExist, e:
			pass

	def get_local_configured_field_value(self, tfield):
		
		try:
			return self.tedatagroupfieldvalue_set.get(tenant=self.tenant, tfield=tfield)
		except TEDataGroupFieldValue.DoesNotExist, e:
			return None

	def get_local_configured_parameter_value(self, name):
		try:
			return self.tedatagroupparametervalue_set.get(tenant=self.tenant, name=name)
		except TEDataGroupParameterValue.DoesNotExist, e:
			return None

	# FIX: rename get_locaL_defined_*  + get_local_configured  -> get_local_overridden_
	def get_local_defined_datasources_fields_dict(self):
		dsfields_by_ds = {}
		all_fields = self.tedatasourcefieldvalue_set.filter(tenant=self.tenant).all()
		for f in all_fields:
			ds_name = f.tdatasource.name
			d = dsfields_by_ds.get(ds_name, {})
			d[ f.tfield.name ] = f
			dsfields_by_ds[ ds_name ] = d
		return dsfields_by_ds
		
	def get_local_defined_conditions_fields_dict(self):
		cfields_by_c = {}
		all_fields = self.teconditionfieldvalue_set.filter(tenant=self.tenant).all()
		for f in all_fields:
			c_name = f.tcondition.name
			d = cfields_by_c.get(c_name, {})
			d[ f.tfield.name ] = f
			cfields_by_c[ c_name ] = d
		return cfields_by_c
		
	def get_local_defined_timegraphs_fields_dict(self):
		
		tgs = {}
		
		all_defs = self.tetimegraphfieldvalue_set.filter(tenant=self.tenant).all()
		for f in all_defs:
			tg_name = f.ttimegraph.name
			tg = tgs.get(tg_name, { 'fields':{}, 'series':{} } )
			tg['fields'][f.tfield.name ] = f			
			tgs[tg_name] = tg

		all_defs = self.tetimegraphseriefieldvalue_set.filter(tenant=self.tenant).all()
		for f in all_defs:
			tg_name  = f.ttimegraphserie.ttimegraph.name
			s_name = f.ttimegraphserie.name

			tg = tgs.get(tg_name, { 'fields':{}, 'series':{} } )

			serie = tg['series'].get(s_name, {})
			serie[f.tfield.name] = f
			tg['series'][s_name] = serie

			tgs[tg_name] = tg

		return tgs			
		
	def get_local_configured_datasource_field_value(self, ds_name, tfield):
		
		try:
			cdsfv = self.tedatasourcefieldvalue_set.get(tenant=self.tenant, tdatasource__name=ds_name, tfield=tfield)
			return cdsfv
		except TEDataSourceFieldValue.DoesNotExist, e:
			return None

	def get_local_configured_condition_field_value(self, c_name, tfield):
		
		try:
			tecfv = self.teconditionfieldvalue_set.get(tenant=self.tenant, tcondition__name=c_name, tfield=tfield)
			return tecfv
		except TEConditionFieldValue.DoesNotExist, e:
			return None

	def get_local_configured_timegraph_field_value(self, tg_name, tfield):
		
		try:
			tecfv = self.tetimegraphfieldvalue_set.get(tenant=self.tenant, ttimegraph__name=tg_name, tfield=tfield)
			return tecfv
		except TETimeGraphFieldValue.DoesNotExist, e:
			return None

	def get_local_configured_timegraphserie_field_value(self, tg_name, serie_name, tfield):
		
		try:
			tecfv = self.tetimegraphseriefieldvalue_set.get(tenant=self.tenant, ttimegraphserie__name=serie_name, tfield=tfield)
			return tecfv
		except TETimeGraphSerieFieldValue.DoesNotExist, e:
			return None

	def get_configured_fields(self, selected_fields=None, l=0):

		_DEBUG(l, self, "get_configured_params(%s)" % (selected_fields))
		l=l+1
		
		configured_fields = {}

		_DEBUG(l, self, "tdatagroup tree fields...")

		configured_fields = dict([(f.name, f) for f in self.tdatagroup.get_configured_fields_or_default(selected_fields=selected_fields, l=l)])  
		
		for f in configured_fields:
			_DEBUG(l, self, "configured_field %s" % (f))
		
		_DEBUG(l, self, "overwritten fields...")
		fields = self.tedatagroupfieldvalue_set.filter(tenant=self.tenant).all()
		for f in fields:
			
			if selected_fields:
				if f.name not in selected_fields:
					continue
			
			configured_fields[f.name] = f
			_DEBUG(l+1, f)
		
		return configured_fields.values()

	def get_configured_param(self, param_name):

		config = self.get_configured_params_dict(l=0, filter_names=[param_name,]) #FIX: 
		if config:
			return config[param_name]
		return None

	def get_configured_params(self, l=0, filter_names=None):
		
		return self.get_configured_params_dict(l=l, filter_names=filter_names).values()

	def get_configured_params_dict(self, l=0, filter_names=None):

		_DEBUG(l, self, "get_configured_params()")
		l=l+1
		
		_DEBUG(l, self, "tdatagroup tree params...")
		configured_params = self.tdatagroup.get_configured_params_dict(l=l)
		
		_DEBUG(l, self, "overwritten params")
		params = self.tedatagroupparametervalue_set.filter(tenant=self.tenant).all()
		for p in params:
			
			if filter_names:
				if not p.name in filter_names:
					continue
			
			_DEBUG(l+1, p)

			configured_params[p.name] = p

		return configured_params
		
	
	def get_configured_datasource_field(self, ds_name, field_name):
		# FIX: filtrare la configurazione solo del datasource che ci interessa
		#effective_datasources_by_name = self.tdatagroup.get_configured_datasources(l=0)      XXXXXXXXXXXXXXXXXXXXX
		#return effective_datasources_by_name[ds_name][field_name]
		return self.tdatagroup.get_configured_datasource_field_value_or_default(ds_name, field_name)
	
	def get_configured_datasources(self, l=0):
		
		_DEBUG(l, self, "get_configured_datasources")
		l=l+1

		effective_datasources_by_name = self.tdatagroup.get_configured_datasources(l=l)
		
		for ds_name, ds_fields_dict in effective_datasources_by_name.items():
			
			_DEBUG(l+1, ds_name)

			local_ds_fields = self.tedatasourcefieldvalue_set.filter(tenant=self.tenant).filter(tdatasource__name=ds_name).all()
			
			for f in local_ds_fields:
				_DEBUG(l+2, f.name, f)
				ds_fields_dict[f.name] = f

		return effective_datasources_by_name


	def get_configured_condition_field(self, c_name, field_name):
		# FIX: filtrare la configurazione solo del datasource che ci interessa
		#effective_conditions_by_name = self.tdatagroup.get_configured_conditions(l=0)
		#return effective_conditions_by_name[c_name][field_name]
		return self.tdatagroup.get_configured_condition_field_value_or_default(c_name, field_name)

	def get_configured_conditions(self, l=0):
		
		_DEBUG(l, self, "get_configured_conditions")
		l=l+1
		
		effective_conditions_by_name = self.tdatagroup.get_configured_conditions(l=l)
		
		for c_name, c_fields_dict in effective_conditions_by_name.items():
			
			_DEBUG(l+1, c_name)

			local_c_fields = self.teconditionfieldvalue_set.filter(tenant=self.tenant).filter(tcondition__name=c_name).all()
			
			for f in local_c_fields:
				_DEBUG(l+2, f.name, f)
				c_fields_dict[f.name] = f

		return effective_conditions_by_name

	def get_configured_timegraph_field(self, tg_name, fname):
		return self.get_configured_timegraphs()[tg_name]['fields'][fname]

	def get_configured_timegraphserie_field(self, tg_name, serie_name, f_name):
		conf = self.get_configured_timegraphs()[tg_name]['series']
		return conf[serie_name][f_name]

	def get_configured_timegraphs(self, l=0):

		_DEBUG(l, self, "get_configured_timegraphs")
		l=l+1

		#effective_timegraphs_by_name = self.tdatagroup.get_configured_timegraphs(l=l)
		effective_timegraphs_by_name = self.tdatagroup.get_configured_timegraphs_with_default(l=l)
		
		_DEBUG(l, self, "loading local configuration")
		local_timegraphs_by_name     = self._get_local_timegraphs_dict(l=l+1)
		
		_DEBUG(l, self, "updating from local configuration")
		l=l+1
		for local_tg_name, local_tg_config in local_timegraphs_by_name.items():
			
			_DEBUG(l, local_tg_name)
			
			effective_config = effective_timegraphs_by_name[local_tg_name]
			
			# Aggiorna campi
			effective_config['fields'].update(local_tg_config['fields'])
			
			# Aggiorna serie
			l=l+1
			for e_serie_name, e_serie_config in effective_config['series'].items():
				
				l_serie_config = local_tg_config['series'].get(e_serie_name, None)
				if l_serie_config == None:
					continue

				_DEBUG(l, e_serie_name)
				
				e_serie_config.update(l_serie_config)
		
		return effective_timegraphs_by_name
		

	def _get_local_timegraphs_dict(self, l=0):

		tgconfig_by_name = {}

		_DEBUG(l, "loading local timegraph params")
		
		all_fields = self.tetimegraphfieldvalue_set.filter(tenant=self.tenant).all()
		for f in all_fields:
			
			tg_name = f.ttimegraph.name
			
			config_dict = tgconfig_by_name.get(tg_name, None)
			if config_dict == None:
				config_dict = { 'fields': {}, 'series': {} }
				tgconfig_by_name[ tg_name ] = config_dict

			config_dict['fields'][ f.tfield.name ] = f

		all_series_fields = self.tetimegraphseriefieldvalue_set.filter(tenant=self.tenant).all()
		for f in all_series_fields:
			
			tg_name       = f.ttimegraphserie.ttimegraph.name
			tg_serie_name = f.ttimegraphserie.name
			
			

			config_dict = tgconfig_by_name.get(tg_name, None)
			if config_dict == None:
				config_dict = { 'fields': {}, 'series': {} }
				tgconfig_by_name[ tg_name ] = config_dict
				
			serie_config_dict = config_dict['series'].get(tg_serie_name, None)
			if serie_config_dict == None:
				serie_config_dict = {}
				config_dict['series'][tg_serie_name] = serie_config_dict

			serie_config_dict[ f.tfield.name ] = f

		return tgconfig_by_name

	class Meta:
		db_table = "c_tedatagroup"
		unique_together = (
			"tenant",
			"telement",
			"tdatagroup", 
			"name",
		)

class TEDataGroupFieldValue(TenantRelatedModel, MTimeModel, UUIDKeyModel, TConfiguredFieldValue):
	tedatagroup       = models.ForeignKey(TEDataGroup, null=False)

	class Meta:
		db_table = "c_tedatagroupfieldvalue"
		unique_together = (
			"tenant",
			"tedatagroup", 
			"tfield", 
		)

class TEDataGroupParameterValue(TenantRelatedModel, MTimeModel, UUIDKeyModel, Parameter):
	tedatagroup       = models.ForeignKey(TEDataGroup, null=False)

	class Meta:
		db_table = "c_tedatagroupparametervalue"
		unique_together = (
			"tenant",
			"tedatagroup", 
			"name", 
		)	

class TEDataSourceFieldValue(TenantRelatedModel, MTimeModel, UUIDKeyModel, TConfiguredFieldValue):

	tedatagroup       = models.ForeignKey(TEDataGroup, null=False)
	tdatasource       = models.ForeignKey(TDataSource, null=False)
	
	class Meta:
		db_table = "c_tedatasourcefieldvalue"
		unique_together = (
			"tenant",
			"tedatagroup", 
			"tdatasource", 
			"tfield", 
		)

class TEConditionFieldValue(TenantRelatedModel, MTimeModel, UUIDKeyModel, TConfiguredFieldValue):

	tedatagroup       = models.ForeignKey(TEDataGroup, null=False)	
	tcondition        = models.ForeignKey(TCondition, null=False)

	class Meta:
		db_table = "c_teconditionfieldvalue"
		unique_together = (
			"tenant",
			"tedatagroup", 
			"tcondition", 
			"tfield", 
		)
	
class TETimeGraphFieldValue(TenantRelatedModel, MTimeModel, UUIDKeyModel, TConfiguredFieldValue):
	
	tedatagroup       = models.ForeignKey(TEDataGroup, null=False)	
	ttimegraph        = models.ForeignKey(TTimeGraph, null=False)

	class Meta:
		db_table = "c_tetimegraphfieldvalue"
		unique_together = (
			"tenant",
			"tedatagroup", 
			"ttimegraph", 
			"tfield", 
		)

class TETimeGraphSerieFieldValue(TenantRelatedModel, MTimeModel, UUIDKeyModel, TConfiguredFieldValue):

	tedatagroup       = models.ForeignKey(TEDataGroup, null=False)	
	ttimegraphserie   = models.ForeignKey(TTimeGraphSerie, null=False)

	class Meta:
		db_table = "c_tetimegraphseriefieldvalue"
		unique_together = (
			"tenant",
			"tedatagroup", 
			"ttimegraphserie", 
			"tfield", 
		)

#-----------------------------------------------------------------------------#
# Configured datagroups for each element                                      #
#-----------------------------------------------------------------------------#

class CDataGroup(TenantDependentModel, MTimeModel):

	element     = models.ForeignKey(MonitoredElement, null=False)

	tdatagroup  = models.ForeignKey(TDataGroup , null=True, related_name="additional_datagroups") #\_____ XXX COSI' GESTIAMO LA CONFIGURAZIONE SIA COME OPZIONALE SIA COME category-datagroup
	tedatagroup = models.ForeignKey(TEDataGroup, null=True, related_name="telement_datagroups"  ) #/

	name        = models.TextField(null=False, blank=False, default=None)

	def __str__(self):
		return "<CDataGroup,%s,%s,%s,%s>" % (self.id, self.name, self.tdatagroup, self.tedatagroup)
		
	@property
	def is_direct(self):
		return (self.tdatagroup_id != None)
		
	def rename_parameter(self, param_name, new_name):
		try:
			p = self.cdatagroupparametervalue_set.get(name=param_name)
			p.name = new_name
			p.save()
		except CDataGroupParameterValue.DoesNotExist, e:
			pass
		

	def get_datagroup(self):
		return self.datagroup_set.filter(tenant=self.tenant).all()[0]

	def get_tdatagroup(self):		
		if self.tdatagroup != None:
			return self.tdatagroup
		else:
			return self.tedatagroup.tdatagroup

	def find_tdatasource(self, ds_name):
		tdatagroup = self.get_tdatagroup()
		return tdatagroup.get_last_defined_datasource(ds_name)

	def find_tcondition(self, c_name):
		tdatagroup = self.get_tdatagroup()
		return tdatagroup.get_last_defined_condition(c_name)

	def calculate_path(self):
		# <element> [ : <element> ] * ; [ <element template>  ] ; <datagroup name> 
		path = ''
		path += self.element.calculate_path()
		path += DATAGROUP_PATH_SEP
		if self.tedatagroup != None: path += self.tedatagroup.telement.name
		path += DATAGROUP_PATH_SEP 
		path += self.name

		return path
		
	#
	# Fields
	#
		
	def get_configured_field(self, name):
		return self.get_configured_fields(selected_fields=[name,])[0]
		
		
	def get_configured_field_or_default(self, name):
		
		config = self.get_configured_fields_dict(selected_fields=[name,])
		if config.has_key(name):
			return config[name]
		else:
			memory_value = ModelFieldsUtils.get_model_fields_default_value_dict( DataGroup )[name]

			tf = TField.objects.get(fclass=TField.FIELD_DATAGROUP, name=name)
			if memory_value is not None:
				tf_value = TField.get_store_repr_value(memory_value)
			else:
				tf_value = None
			v = TDataGroupFieldValue(tfield=tf, value=tf_value)
			return v

	def get_configured_fields(self, selected_fields=None, l=0):
		fields = self.get_configured_fields_dict(selected_fields=selected_fields, l=l)
		return fields.values()
		
	def get_configured_fields_dict(self, selected_fields=None, l=0):
		_DEBUG(l, self, "get_configured_fields")
		l = l + 1
		
		effective_fields = {}
		
		_DEBUG(l, "Loading fields from template...")
		# Get configured fields from template "tree"
		if self.tedatagroup != None:
			_DEBUG(l+1, "Using fields from tedatagroup")
			fields = self.tedatagroup.get_configured_fields(selected_fields=selected_fields, l=l+1)
		else:
			_DEBUG(l+1, "Using fields from tdatagroup")
			fields = self.tdatagroup.get_configured_fields(selected_fields=selected_fields, l=l+1)

		_DEBUG(l, self, "Effective fields...")
		for f in fields: 
			_DEBUG(l+1, f)
			effective_fields[f.name] = f

		# Get local configuration
		_DEBUG(l, "Loading overridend fields...")
		fields = self.cdatagroupfieldvalue_set.all() #.filter(tenant=self.tenant).all()
		for f in fields: 
			if selected_fields is not None:
				if not (f.name in selected_fields):
					continue

			_DEBUG(l+1, f)
			effective_fields[f.name] = f
			
		return effective_fields

	def get_local_configured_field_value(self, tfield):

		try:
			return self.cdatagroupfieldvalue_set.get(tenant=self.tenant, tfield=tfield)
		except CDataGroupFieldValue.DoesNotExist, e:
			return None

	
	#
	# Params
	#
	
	def get_configured_param(self, param_name):

		config = self.get_configured_params_dict(l=0, filter_names=[param_name,]) #FIX: 
		return config.get(param_name, None)
	
	def get_configured_params(self, filter_names=None, l=0):

		return self.get_configured_params_dict(filter_names=filter_names, l=l).values()
		
	def get_configured_params_dict(self, filter_names=None, l=0):
		
		_DEBUG(l, self, "get_configured_params()")
		l=l+1

		effective_params = {}

		_DEBUG(l, "Loading fields from template...")
		if self.tdatagroup != None:
			_DEBUG(l+1, "Using tdatagroup params")
			params = self.tdatagroup.get_configured_params(l=l+1) # FIX: use filter_names
		else:
			_DEBUG(l+1, "Using tedatagroup params")
			params = self.tedatagroup.get_configured_params(l=l+1) # FIX: use filter_names
		
		_DEBUG(l, self, "Effective params...")
		for p in params:
			_DEBUG(l+1, p)
			effective_params[p.name] = p

		_DEBUG(l, "Using configured params")
		params = self.cdatagroupparametervalue_set.filter(tenant=self.tenant).all()
		for p in params:
			_DEBUG(l+1, p)
			effective_params[p.name] = p
		
		return effective_params

	def get_local_configured_parameter_value(self, name):
		try:
			return self.cdatagroupparametervalue_set.filter(tenant=self.tenant).get(tenant=self.tenant, name=name)
		except CDataGroupParameterValue.DoesNotExist, e:
			return None
			
	#
	# Datasources
	#

	def get_configured_datasource_field(self, ds_name, field_name):
	
		config = self.get_configured_datasources()  #FIX: estrare solo il valore dell'unico campo che ci interessa
		return config[ds_name][field_name]
		
	def get_configured_datasource_field_or_default(self, ds_name, field_name):
		
		config = self.get_configured_datasources()  #FIX: estrare solo il valore dell'unico campo che ci interessa
		if config.has_key(ds_name):
			if config[ds_name].has_key(field_name):
				return config[ds_name][field_name]

		memory_value = ModelFieldsUtils.get_model_fields_default_value_dict( DataSource )[field_name]
		
		tf       = TField.objects.get(fclass=TField.FIELD_DATASOURCE, name=field_name)
		if memory_value is not None:
			tf_value = TField.get_store_repr_value(memory_value)
		else:
			tf_value = None
		v = TDataGroupFieldValue(tfield=tf, value=tf_value)
		
		return v

	def get_configured_datasources(self, l=0):
		
		effective_ds = {}
		
		_DEBUG(l, "get_configured_datasources()")
		l=l+1

		_DEBUG(l, "Loading from template...")
		if self.tdatagroup != None:
			_DEBUG(l+1, "Using tdatagroup ds")
			dss = self.tdatagroup.get_configured_datasources(l=l+1)
		else:
			_DEBUG(l+1, "Using tedatagroup ds")
			dss = self.tedatagroup.get_configured_datasources(l=l+1)
			
		for name, ds in dss.items():
			_DEBUG(l+1, ds)
			effective_ds[name] = ds
		
		
		_DEBUG(l, "Using configured datasources")
		ds_dict = self._get_local_datasources_dict(l=l)
		
		for name, ds_fields_dict in ds_dict.items():
			
			_DEBUG(l+1, name)
			for fname, f in ds_fields_dict.items():
				_DEBUG(l+1, fname, f)
			
			effective_ds[name].update(ds_fields_dict)

		return effective_ds

	def get_local_defined_datasources_fields_dict(self):
		dsfields_by_ds = {}
		all_fields = self.cdatasourcefieldvalue_set.filter(tenant=self.tenant).all()
		for f in all_fields:
			ds_name = f.tdatasource.name
			d = dsfields_by_ds.get(ds_name, {})
			d[ f.tfield.name ] = f
			dsfields_by_ds[ ds_name ] = d
		return dsfields_by_ds

	def _get_local_datasources_dict(self, l=0):

		dsfields_by_ds = {}

		_DEBUG(l, "loading local datasource params")
		
		all_fields = self.cdatasourcefieldvalue_set.filter(tenant=self.tenant).all()
		
		for f in all_fields:
			
			ds_name = f.tdatasource.name
			
			fields_dict = dsfields_by_ds.get(ds_name, None)
			if fields_dict == None:
				fields_dict = {}
				dsfields_by_ds[ ds_name ] = fields_dict

			fields_dict[ f.tfield.name ] = f
		
		return dsfields_by_ds

	def get_local_configured_datasource_field_value(self, ds_name, tfield):
		try:
			return self.cdatasourcefieldvalue_set.get(tenant=self.tenant, tdatasource__name=ds_name, tfield=tfield)
		except CDataSourceFieldValue.DoesNotExist, e:
			return None

	#
	# Conditions
	#
	def get_configured_condition_field(self, c_name, field_name):
		config = self.get_configured_conditions()  #FIX: estrare solo il valore dell'unico campo che ci interessa
		
		return config[c_name][field_name]

	def get_configured_condition_field_or_default(self, c_name, field_name):
		
		config = self.get_configured_conditions()  #FIX: estrare solo il valore dell'unico campo che ci interessa
		if config.has_key(c_name):
			if config[c_name].has_key(field_name):
				return config[c_name][field_name]

		memory_value = None
		tf       = TField.objects.get(fclass=TField.FIELD_CONDITION, name=field_name)

		# Alcuni campi vengono sovrascritti a livello di elemento
		# quindi se il campo non e' configurato a livello di datagroup
		# vediamo l'elemento.
		if field_name == 'priority':
			e_priority = self.element.get_effective_priority()
			if e_priority != PRIORITY_NONE:
				memory_value = e_priority

		
		_DEBUG(0, "USING DEFAULT")
		if memory_value is None:
			memory_value = ModelFieldsUtils.get_model_fields_default_value_dict( Condition )[field_name]
			
		if memory_value is not None:
			tf_value = TField.get_store_repr_value(memory_value)
		else:
			tf_value = None

		v = TConditionFieldValue(tfield=tf, value=tf_value)
		
		return v

	def get_configured_conditions(self, l=0):
		
		effective_c = {}
		
		_DEBUG(l, "get_configured_conditions")
		l=l+1

		if self.tdatagroup != None:
			_DEBUG(l, "Using tdatagroup ds")
			cs = self.tdatagroup.get_configured_conditions(l=l+1)
		else:
			_DEBUG(l, "Using tedatagroup ds")
			cs = self.tedatagroup.get_configured_conditions(l=l+1)
			
		for name, c in cs.items():
			_DEBUG(l+1, "Condition field ---------------------------------------------------------", name, c)
			effective_c[name] = c
			
			# Ci sono alcuni campi la cui configurazione viene annullata da configurazioni sull'elemento
			# - priority
			
			e_priority = self.element.get_effective_priority()
			if e_priority != PRIORITY_NONE:
				tf       = TField.objects.get(fclass=TField.FIELD_CONDITION, name='priority')
				tf_value = TField.get_store_repr_value(e_priority)
				tcfv = TConditionFieldValue(tfield=tf, value=tf_value)
				c['priority'] = tcfv
		
		_DEBUG(l, "Using local conditions")
		
		c_dict = self._get_local_conditions_dict(l=l)
		
		for name, c_fields_dict in c_dict.items():
			
			_DEBUG(l+1, name)
			for fname, f in c_fields_dict.items():
				_DEBUG(l+1, "local ---------------------------------------------------------", fname, f)
			
			effective_c[name].update(c_fields_dict)
			
		return effective_c

	def _get_local_conditions_dict(self, l=0):

		cfields_by_c = {}

		_DEBUG(l, "loading local datasource params")
		
		all_fields = self.cconditionfieldvalue_set.filter(tenant=self.tenant).all()
		
		for f in all_fields:
			
			c_name = f.tcondition.name
			
			fields_dict = cfields_by_c.get(c_name, None)
			if fields_dict == None:
				fields_dict = {}
				cfields_by_c[ c_name ] = fields_dict

			fields_dict[ f.tfield.name ] = f
		
		return cfields_by_c
		
	def get_local_defined_conditions_fields_dict(self):
		cfields_by_c = {}
		all_fields = self.cconditionfieldvalue_set.filter(tenant=self.tenant).all()
		for f in all_fields:
			c_name = f.tcondition.name
			d = cfields_by_c.get(c_name, {})
			d[ f.tfield.name ] = f
			cfields_by_c[ c_name ] = d
		return cfields_by_c
		
		
	def get_local_configured_condition_field_value(self, c_name, tfield):
		try:
			return self.cconditionfieldvalue_set.filter(tenant=self.tenant).get(tenant=self.tenant, tcondition__name=c_name, tfield=tfield)
		except CConditionFieldValue.DoesNotExist, e:
			return None

	#
	# Timegraphs
	#

	def get_local_configured_timegraphserie_field_value(self, tg_name, s_name, tfield):
	
		config = self.get_local_defined_timegraphs_fields_dict()
		
		config = config.get(tg_name, None)
		if not config:
			return None

		config = config['series'].get(s_name,None)
		if not config:
			return None
		
		return config.get(tfield.name, None)

	def get_local_defined_timegraphs_fields_dict(self):
		
		tgs = {}
		
		all_defs = self.ctimegraphfieldvalue_set.filter(tenant=self.tenant).all()
		for f in all_defs:
			
			tg_name = f.ttimegraph.name
			tg = tgs.get(tg_name, { 'fields':{}, 'series':{} } )
			tg['fields'][f.tfield.name ] = f
			tgs[tg_name] = tg
			
		all_defs = self.ctimegraphseriefieldvalue_set.filter(tenant=self.tenant).all()
		for f in all_defs:
			tg_name  = f.ttimegraphserie.ttimegraph.name
			s_name = f.ttimegraphserie.name

			tg = tgs.get(tg_name, { 'fields':{}, 'series':{} } )

			serie = tg['series'].get(s_name, {})
			serie[f.tfield.name] = f
			tg['series'][s_name] = serie

			tgs[tg_name] = tg

		return tgs

	def get_local_configured_timegraph_field_value(self, tg_name, tfield):
		try:
			return self.ctimegraphfieldvalue_set.get(tenant=self.tenant, ttimegraph__name=tg_name, tfield=tfield)
		except CTimeGraphFieldValue.DoesNotExist, e:
			return None

	def get_configured_timegraph_field(self, tg_name, fname):
		config = self.get_configured_timegraphs()[tg_name]['fields']
		return config[fname]

	def get_configured_timegraphserie_field(self, tg_name, s_name, fname):
		config = self.get_configured_timegraphs()[tg_name]['series'][s_name]
		return config[fname]

	def get_configured_timegraphs(self, l=0):

		effective_tgs = {}
		
		_DEBUG(l, "get_configured_timegraphs")
		l=l+1

		if self.tdatagroup != None:
			_DEBUG(l, "Using tdatagroup ds")
			tgs = self.tdatagroup.get_configured_timegraphs(l=l+1)
		else:
			_DEBUG(l, "Using tedatagroup ds")
			tgs = self.tedatagroup.get_configured_timegraphs(l=l+1)
			
		for tg_name, tg_config in tgs.items():
			_DEBUG(l+1,  tg_config)
			effective_tgs[tg_name] =  tg_config
			
		_DEBUG(l, "Using local timegraphs")
		
		tg_dict = self._get_local_timegraphs_dict(l=l)
		
		for tg_name, tg_config in tg_dict.items():
			
			_DEBUG(l+1, tg_name)
			
			for fname, f in tg_config['fields'].items():
				_DEBUG(l+1, fname, f)
			
			# updating fields
			effective_tgs[tg_name]['fields'].update(tg_config['fields'])
			
			# updating series
			for tg_serie_name, tg_serie_fields in tg_config['series'].items():
				effective_tgs[tg_name]['series'][tg_serie_name].update(tg_serie_fields)

		return effective_tgs
		

	def _get_local_timegraphs_dict(self, l=0):

		tgconfig_by_name = {}

		_DEBUG(l, "loading local timegraph params")
		
		all_fields = self.ctimegraphfieldvalue_set.filter(tenant=self.tenant).all()
		for f in all_fields:
			
			tg_name = f.ttimegraph.name
			
			config_dict = tgconfig_by_name.get(tg_name, None)
			if config_dict == None:
				config_dict = { 'fields': {}, 'series': {} }
				tgconfig_by_name[ tg_name ] = config_dict

			config_dict['fields'][ f.tfield.name ] = f

		all_series_fields = self.ctimegraphseriefieldvalue_set.filter(tenant=self.tenant).all()
		for f in all_series_fields:
			
			tg_name       = f.ttimegraphserie.ttimegraph.name
			tg_serie_name = f.ttimegraphserie.name

			config_dict = tgconfig_by_name.get(tg_name, None)
			if config_dict == None:
				config_dict = { 'fields': {}, 'series': {} }
				tgconfig_by_name[ tg_name ] = config_dict
				
			serie_config_dict = config_dict['series'].get(tg_serie_name, None)
			if serie_config_dict == None:
				serie_config_dict = {}
				config_dict['series'][tg_serie_name] = serie_config_dict

			serie_config_dict[ f.tfield.name ] = f

		return tgconfig_by_name

	class Meta:
		db_table = "c_cdatagroup"
		unique_together = (
			"tenant",
			"tdatagroup",
			"tedatagroup",
			"element",
			"name",
		)

class CDataGroupFieldValue(TenantDependentModel, MTimeModel, UUIDKeyModel, TConfiguredFieldValue):

	cdatagroup  = models.ForeignKey(CDataGroup, null=False)

	class Meta:	
		db_table = "c_cdatagroupfieldvalue"
		unique_together = (
			"tenant",
			"cdatagroup",
			"tfield",
		)

class CDataGroupParameterValue(TenantDependentModel, MTimeModel, UUIDKeyModel, Parameter):

	cdatagroup  = models.ForeignKey(CDataGroup, null=False)
	# name
	# vtype
	# value
	
	class Meta:	
		db_table = "c_cdatagroupparametervalue"
		unique_together = (
			"tenant",
			"cdatagroup",
			"name",
		)

class CDataSourceFieldValue(TenantDependentModel, MTimeModel, TConfiguredFieldValue):

	cdatagroup  = models.ForeignKey(CDataGroup, null=False)
	tdatasource = models.ForeignKey(TDataSource, null=False)

	class Meta:	
		db_table = "c_cdatasourcefieldvalue"
		unique_together = (
			"tenant",
			"cdatagroup",
			"tdatasource",
			"tfield",
		)

class CConditionFieldValue(TenantDependentModel, MTimeModel, TConfiguredFieldValue):

	cdatagroup  = models.ForeignKey(CDataGroup, null=False)
	tcondition  = models.ForeignKey(TCondition, null=False)

	class Meta:
		db_table = "c_cconditionfieldvalue"
		unique_together = (
			"cdatagroup",
			"tcondition",
			"tfield",
		)

class CTimeGraphFieldValue(TenantDependentModel, MTimeModel, TConfiguredFieldValue):
	
	cdatagroup  = models.ForeignKey(CDataGroup, null=False)
	ttimegraph  = models.ForeignKey(TTimeGraph, null=False)

	class Meta:
		db_table = "c_ctimegraphfieldvalue"
		unique_together = (
			"tenant",
			"cdatagroup",
			"ttimegraph",
			"tfield",
		)

class CTimeGraphSerieFieldValue(TenantDependentModel, MTimeModel, TConfiguredFieldValue):

	cdatagroup       = models.ForeignKey(CDataGroup, null=False)
	ttimegraphserie  = models.ForeignKey(TTimeGraphSerie, null=False)
	
	class Meta:
		db_table = "c_ctimegraphseriefieldvalue"
		unique_together = (
			"tenant",
			"cdatagroup",
			"ttimegraphserie",
			"tfield",
		)


#-----------------------------------------------------------------------------#
#                                                                             #
#-----------------------------------------------------------------------------#

from core.capabilities import CAPABILITIES


class Capability(TenantRelatedModel):
	# id     =
	# tenant =
	group    = models.ForeignKey(Group, null=False, db_index=True)
	code     = models.TextField(null=False, db_index=False)

	class Meta:
		unique_together = (
			("tenant", "group", "code"),
		)
		db_table = "capability"


class CapabilityException(ApplicationException):
	pass

#-----------------------------------------------------------------------------#

def __group_get_capabilities(self, tenant):
	return set(self.capability_set.filter(tenant=tenant).values_list('code', flat=True))

Group.add_to_class('get_capabilities', __group_get_capabilities)

#-----------------------------------------------------------------------------#

def __user_get_capabilities(self, tenant, use_cached=True):

	# Per motivi di performance le capability le salviamo nell'oggetto user.
	# cosi' piu' accessi a questo metodo usano gia' caricati
	
	#if use_cached:
	#	if hasattr(self, '_capabilities'):
	#		return self._capabilities

	if self.is_superuser:
		#self._capabilities = set(CAPABILITIES.keys())
		return set(CAPABILITIES.keys())
	else:	
		capabilities = set()
		for g in self.groups.all():
			s = g.get_capabilities(tenant)

			capabilities = capabilities | s
			
		return capabilities

		#self._capabilities = capabilities
	
	#return self._capabilities

def __user_assert_capability(self, tenant, capability):

	#print __file__, "__user_assert_capability", self, tenant, capability

	if self.is_superuser:
		return True

	#
	# FIX: inefficient
	#
	capabilities = self.get_capabilities(tenant)

	if capability not in capabilities:
		raise CapabilityException("User '%s' on tenant '%s' not allowed to capability '%s'. Enabled capabilities: %s." % (self, tenant.name, capability, '\n'.join(capabilities)))
	
	return True

User.add_to_class('get_capabilities'  , __user_get_capabilities)

User.add_to_class('assert_capability' , __user_assert_capability)

def __assert_in_group(self, group_name):

	if self.is_superuser:
		return True
		
	#print __file__, "assert_in_group", self, group_name
	
	if not User.objects.filter(pk=self.id, groups__name=group_name).exists():
		raise Exception("User '%s' does not belong to group '%s'." % (self.username, group_name))

User.add_to_class('assert_in_group'  , __assert_in_group)

#-----------------------------------------------------------------------------#
# Permissions                                                                 #
#-----------------------------------------------------------------------------#

class TDataGroupPermission(TenantRelatedModel, models.Model):

	tdatagroup = models.ForeignKey(TDataGroup)
	group      = models.ForeignKey(Group)
	auth_type  = models.PositiveSmallIntegerField(null=False, default=AUTH_READONLY) 

	class Meta:
		db_table = "tdatagroup_group_permission"

class TElementPermission(TenantRelatedModel, models.Model):

	telement  = models.ForeignKey(TElement)
	group     = models.ForeignKey(Group)
	auth_type = models.PositiveSmallIntegerField(null=False, default=AUTH_READONLY) 

	class Meta:
		db_table = "telement_group_permission"

class SchedulingGroupPermission(TenantDependentModel, models.Model):

	schedulinggroup  = models.ForeignKey(SchedulingGroup)
	group            = models.ForeignKey(Group)
	auth_type        = models.PositiveSmallIntegerField(null=False, default=AUTH_READONLY) 

	class Meta:
		db_table = "schedulinggroup_group_permission"

class ClusterGroupPermission(TenantDependentModel, models.Model):

	cluster    = models.ForeignKey(Cluster)
	group      = models.ForeignKey(Group)
	auth_type  = models.PositiveSmallIntegerField(null=False, default=AUTH_READONLY) 

	class Meta:
		db_table = "cluster_group_permission"

